// Defined functions code for the WASP_LGRTC model, described in A computationally efficient model for probabilistic local warming projections constrained by history matching and pattern scaling

// Manuscript for Geoscientific Model Development Discussions


// Philip Goodwin1,*, Martin Leduc2, Antti-Ilari Partanen3, H. Damon Matthews4 and Alex Rogers5


//1School of Ocean and Earth Science, University of Southampton, Southampton, SO14 3ZH, UK
//2Ouranos, Montreal, Canada
//3Climate System Research, Finnish Meteorological Institute, Helsinki, Finland
//4Department of Geography, Planning and Environment, Concordia University, Montreal, Canada
//5Department of Computer Science, University of Oxford, Oxford, UK


// * Corresponding author, email: p.a.goodwin@soton.ac.uk



//The Warming, Acidification and Sea-level Projector with Local to Global Temperautre Change: WASP/LGRT (September 2019)

//WASP is an 8-box representation of the Earth system for efficient very-large ensemble simulations.
//Coded by Philip Goodwin, This version 16-September-2019


//Citations for use of code:

//This combined WASP/LGRTC model version:
//Goodwin, P., M. Leduc, A.-I. Partanen, H.D. Matthews and A. Rogers (2019, submitted), A computationally efficient model for probabilistic local warming projections constrained by history matching and pattern scaling, submitted to Geoscientific Model Development Discussions.


//Previous version of the WASP model:
//Goodwin, P., V. M. Roussenov, A. Katavouta, G. L. Foster, E. J. Rohling and R. G. Williams (2018), Pathways to 1.5 and 2 °C warming based on observational and geological constraints, Nature Geoscience, doi:10.1038/s41561-017-0054-8.

//WASP Model description:

//Goodwin, P., A. Katavouta,, V. M. Roussenov, G. L. Foster, E. J. Rohling and R. G. Williams (2018), Pathways to 1.5 and 2 °C warming based on observational and geological constraints, Nature Geoscience

//Goodwin, P. (2016) How historic simulation-observation discrepancy affects future warming projections in a very large model ensemble, Climate Dynamics, CLDY-D-15-00368R2, doi: 10.1007/s00382-015-2960-z.

//Utilising the input distribution for climate sensitivity, S=1/lambda, from geological data requires citation of:
//Rohling, E. J. et al - Palaeosens Project Members (2012), Making sense of Palaeoclimate sensitivity, Nature 491, 683-691, doi:10.1038/nature11574.

//The theory behind how WASP calculated atmospheric CO2 and surface temperature anomaly is given here:
//Goodwin, P., R.G. Williams and A. Ridgwell (2015), Sensitivity of climate to cumulative carbon emissions due to compensation of ocean heat and carbon uptake, Nature Geoscience, Vol. 8, p29-34. doi:10.1038/ngeo2304.

//If you use the sea level component of WASP (commented out in the code), also cite:
//Goodwin, P., I. D. Haigh, E. J. Rohling, and A. Slangen (2017), A new approach to projecting 21st century sea-level changes and extremes, Earth’s Future, 5, doi:10.1002/2016EF000508.




//Code compiled on a Mac Darwin environment, operating system OS X version 10.11.6, using command line compilation on the Terminal.
//To compile: place the files WASP_LGRTC_ESM_main.cpp and WASP_LGRTC_ESM_functions.cpp together in a directory. Add a folder called 'RESULTS' for the results files to be written into.
//Using compiler g++ code can be compiled with command line: g++ -O3 -o filename WASP_LGRTC_ESM_main.cpp

//This code has *not* been checked across a range of C++ compilers.
//Also, the sequence of pseudo-random numbers may be compiler and platform dependent



//This header file containes the future scenarios (RCP) and palaeo-distributions for climate sensitivity

#include <cmath>
#include <iostream>
#include <fstream>
#include <random>


using namespace std;


//Define global variables//////////////
const int tmax=30000;

double I_em[tmax];           //Cumulative carbon emissions (PgC)
double CO2[tmax];            //atmospheric CO2 mixing ratio
double RnonCO2[tmax];
double R_WMGHG_nonCO2[tmax];        //Radiative forcing from non-CO2 WMGHG agents
double R_volcanic[tmax];            //Radiative forcing from volanic activity
double R_aerosol[tmax];             //Radiative foricng from aerosols excluding volanism
double R_solar[tmax];               //Radiative forcing from solar activity
double AOD[tmax];                   //Aerosol optical depth at 550nm

//Random number functions////////////////////

//This function applies the Box-Muller Method - this routine generates two uniform random numbers and calculates only one of the random-normal numbers

double getRandomNormal(double mu, double sigma)
{
    double pi = -acos(-1.0);
    
    double U, V;
    
    double Random_normal;
    
    U = double(rand())/double(RAND_MAX);
    V = double(rand())/double(RAND_MAX);
    
    Random_normal =  ( ( sqrt(-2.0*log(U))*cos(2*pi*V) ) * sigma ) + mu;
    
    return Random_normal;
}

double getRandomLinear(double Min, double Max)
{
    double value;
    
    value = Min + ( double(rand())/double(RAND_MAX) ) * (Max - Min);
    
    return value;
}

double getRandomLinear2(double Min, double Max, int seed)
{
    double value;
    
    std::minstd_rand0 generator (seed);
    
    value = Min + ( double(generator())/(double(generator.max())-double(generator.min() ) +1.0  ) ) * (Max - Min);
    
    //cout << generator() << '\t' << generator.max() << '\t' << generator.min() << '\t' << value << endl;
    
    return value;
}



//The terrestrial system functions
double getdCvdt(double pCO2, double pCO2init, double DeltaTemp, double Cv, double Cv0, double NPP0, double gamma, double dNPPdT)
{
    
    double dCvdt;
    
    //Alternatively could do with half saturation coefficients
    //double kc=29.0; //in ppm
    //double km=145.0; //in ppm
    
    
    
    
    
    double f1, f2;
    
    f1 = 1.0 + (dNPPdT/NPP0)*DeltaTemp;
    
    //f2 = ( (pCO2 - kc)/(km + pCO2 - kc) ) / ( (pCO2init - kc)/(km + pCO2init - kc) );
    
    f2 = 1.0 + gamma*log(pCO2/pCO2init);
    
    dCvdt = NPP0*f1*f2 - NPP0*(Cv/Cv0);
    
    return dCvdt;
    
    
}

double getdCsdt(double Cv, double Cv0, double Cs, double Cs0, double NPP0, double DeltaTemp, double dtaudT)
{
    double dCsdt;
    double f3;
    
    
    f3 = (Cs0/NPP0) / ( (Cs0/NPP0) + (dtaudT*DeltaTemp) );
    
    dCsdt = ( NPP0 * (Cv/Cv0) ) - ( NPP0 * (Cs/Cs0) * f3 );
    
    return dCsdt;
    
    
}

//The scenarios and palaeo input distributions

//This sets lambda from the log-normal palaeosens distribution (Rohling et al, Nature)
double get_lambda_palaeosens1(double random)
{
    double lambda_palaeosens = 0.0;
    double S=0;
    
    if(random >= 	0.000000E+00	) S=	0.01	;
    if(random >= 	1.701000E-14	) S=	0.02	;
    if(random >= 	3.298100E-13	) S=	0.03	;
    if(random >= 	9.384100E-13	) S=	0.04	;
    if(random >= 	5.841410E-12	) S=	0.05	;
    if(random >= 	1.503941E-11	) S=	0.06	;
    if(random >= 	9.417941E-11	) S=	0.07	;
    if(random >= 	5.539794E-10	) S=	0.08	;
    if(random >= 	2.557979E-09	) S=	0.09	;
    if(random >= 	9.586979E-09	) S=	0.1	;
    if(random >= 	3.040698E-08	) S=	0.11	;
    if(random >= 	0.0000000843	) S=	0.12	;
    if(random >= 	0.0000002094	) S=	0.13	;
    if(random >= 	0.0000004748	) S=	0.14	;
    if(random >= 	0.0000009969	) S=	0.15	;
    if(random >= 	0.0000019606	) S=	0.16	;
    if(random >= 	0.0000036456	) S=	0.17	;
    if(random >= 	0.0000064566	) S=	0.18	;
    if(random >= 	0.0000109606	) S=	0.19	;
    if(random >= 	0.0000179286	) S=	0.2	;
    if(random >= 	0.0000283786	) S=	0.21	;
    if(random >= 	0.0000436586	) S=	0.22	;
    if(random >= 	0.0000654586	) S=	0.23	;
    if(random >= 	0.0000959486	) S=	0.24	;
    if(random >= 	0.0001378286	) S=	0.25	;
    if(random >= 	0.0001944486	) S=	0.26	;
    if(random >= 	0.0002699186	) S=	0.27	;
    if(random >= 	0.0003692286	) S=	0.28	;
    if(random >= 	0.0004983286	) S=	0.29	;
    if(random >= 	0.0006644286	) S=	0.3	;
    if(random >= 	0.0008759286	) S=	0.31	;
    if(random >= 	0.0011425286	) S=	0.32	;
    if(random >= 	0.0014754286	) S=	0.33	;
    if(random >= 	0.0018873286	) S=	0.34	;
    if(random >= 	0.0023924286	) S=	0.35	;
    if(random >= 	0.0030063286	) S=	0.36	;
    if(random >= 	0.0037461286	) S=	0.37	;
    if(random >= 	0.0046302286	) S=	0.38	;
    if(random >= 	0.0056782286	) S=	0.39	;
    if(random >= 	0.0069112286	) S=	0.4	;
    if(random >= 	0.0083502286	) S=	0.41	;
    if(random >= 	0.0100182286	) S=	0.42	;
    if(random >= 	0.0119382286	) S=	0.43	;
    if(random >= 	0.0141352286	) S=	0.44	;
    if(random >= 	0.0166332286	) S=	0.45	;
    if(random >= 	0.0194582286	) S=	0.46	;
    if(random >= 	0.0226362286	) S=	0.47	;
    if(random >= 	0.0261932286	) S=	0.48	;
    if(random >= 	0.0301552286	) S=	0.49	;
    if(random >= 	0.0345482286	) S=	0.5	;
    if(random >= 	0.0393962286	) S=	0.51	;
    if(random >= 	0.0447212286	) S=	0.52	;
    if(random >= 	0.0505412286	) S=	0.53	;
    if(random >= 	0.0568742286	) S=	0.54	;
    if(random >= 	0.0637322286	) S=	0.55	;
    if(random >= 	0.0711232286	) S=	0.56	;
    if(random >= 	0.0790532286	) S=	0.57	;
    if(random >= 	0.0875222286	) S=	0.58	;
    if(random >= 	0.0965292286	) S=	0.59	;
    if(random >= 	0.1060682286	) S=	0.6	;
    if(random >= 	0.1160682286	) S=	0.61	;
    if(random >= 	0.1270682286	) S=	0.62	;
    if(random >= 	0.1380682286	) S=	0.63	;
    if(random >= 	0.1500682286	) S=	0.64	;
    if(random >= 	0.1620682286	) S=	0.65	;
    if(random >= 	0.1750682286	) S=	0.66	;
    if(random >= 	0.1880682286	) S=	0.67	;
    if(random >= 	0.2010682286	) S=	0.68	;
    if(random >= 	0.2150682286	) S=	0.69	;
    if(random >= 	0.2290682286	) S=	0.7	;
    if(random >= 	0.2440682286	) S=	0.71	;
    if(random >= 	0.2590682286	) S=	0.72	;
    if(random >= 	0.2740682286	) S=	0.73	;
    if(random >= 	0.2900682286	) S=	0.74	;
    if(random >= 	0.3060682286	) S=	0.75	;
    if(random >= 	0.3220682286	) S=	0.76	;
    if(random >= 	0.3380682286	) S=	0.77	;
    if(random >= 	0.3540682286	) S=	0.78	;
    if(random >= 	0.3700682286	) S=	0.79	;
    if(random >= 	0.3860682286	) S=	0.8	;
    if(random >= 	0.4020682286	) S=	0.81	;
    if(random >= 	0.4180682286	) S=	0.82	;
    if(random >= 	0.4340682286	) S=	0.83	;
    if(random >= 	0.4490682286	) S=	0.84	;
    if(random >= 	0.4640682286	) S=	0.85	;
    if(random >= 	0.4790682286	) S=	0.86	;
    if(random >= 	0.4940682286	) S=	0.87	;
    if(random >= 	0.5080682286	) S=	0.88	;
    if(random >= 	0.5220682286	) S=	0.89	;
    if(random >= 	0.5360682286	) S=	0.9	;
    if(random >= 	0.5490682286	) S=	0.91	;
    if(random >= 	0.5620682286	) S=	0.92	;
    if(random >= 	0.5750682286	) S=	0.93	;
    if(random >= 	0.5880682286	) S=	0.94	;
    if(random >= 	0.6000682286	) S=	0.95	;
    if(random >= 	0.6120682286	) S=	0.96	;
    if(random >= 	0.6240682286	) S=	0.97	;
    if(random >= 	0.6350682286	) S=	0.98	;
    if(random >= 	0.6460682286	) S=	0.99	;
    if(random >= 	0.6570682286	) S=	1	;
    if(random >= 	0.6670682286	) S=	1.01	;
    if(random >= 	0.6770682286	) S=	1.02	;
    if(random >= 	0.6868072286	) S=	1.03	;
    if(random >= 	0.6962522286	) S=	1.04	;
    if(random >= 	0.7054092286	) S=	1.05	;
    if(random >= 	0.7142832286	) S=	1.06	;
    if(random >= 	0.7228812286	) S=	1.07	;
    if(random >= 	0.7312092286	) S=	1.08	;
    if(random >= 	0.7392752286	) S=	1.09	;
    if(random >= 	0.7470852286	) S=	1.1	;
    if(random >= 	0.7546462286	) S=	1.11	;
    if(random >= 	0.7619652286	) S=	1.12	;
    if(random >= 	0.7690502286	) S=	1.13	;
    if(random >= 	0.7759082286	) S=	1.14	;
    if(random >= 	0.7825452286	) S=	1.15	;
    if(random >= 	0.7889692286	) S=	1.16	;
    if(random >= 	0.7951872286	) S=	1.17	;
    if(random >= 	0.8012052286	) S=	1.18	;
    if(random >= 	0.8070292286	) S=	1.19	;
    if(random >= 	0.8126672286	) S=	1.2	;
    if(random >= 	0.8181242286	) S=	1.21	;
    if(random >= 	0.8234062286	) S=	1.22	;
    if(random >= 	0.8285202286	) S=	1.23	;
    if(random >= 	0.8334702286	) S=	1.24	;
    if(random >= 	0.8382632286	) S=	1.25	;
    if(random >= 	0.8429032286	) S=	1.26	;
    if(random >= 	0.8473962286	) S=	1.27	;
    if(random >= 	0.8517472286	) S=	1.28	;
    if(random >= 	0.8559612286	) S=	1.29	;
    if(random >= 	0.8600422286	) S=	1.3	;
    if(random >= 	0.8639952286	) S=	1.31	;
    if(random >= 	0.8678242286	) S=	1.32	;
    if(random >= 	0.8715332286	) S=	1.33	;
    if(random >= 	0.8751262286	) S=	1.34	;
    if(random >= 	0.8786072286	) S=	1.35	;
    if(random >= 	0.8819802286	) S=	1.36	;
    if(random >= 	0.8852492286	) S=	1.37	;
    if(random >= 	0.8884172286	) S=	1.38	;
    if(random >= 	0.8914872286	) S=	1.39	;
    if(random >= 	0.8944632286	) S=	1.4	;
    if(random >= 	0.8973482286	) S=	1.41	;
    if(random >= 	0.9001452286	) S=	1.42	;
    if(random >= 	0.9028572286	) S=	1.43	;
    if(random >= 	0.9054862286	) S=	1.44	;
    if(random >= 	0.9080362286	) S=	1.45	;
    if(random >= 	0.9105092286	) S=	1.46	;
    if(random >= 	0.9129082286	) S=	1.47	;
    if(random >= 	0.9152352286	) S=	1.48	;
    if(random >= 	0.9174922286	) S=	1.49	;
    if(random >= 	0.9196822286	) S=	1.5	;
    if(random >= 	0.9218082286	) S=	1.51	;
    if(random >= 	0.9238712286	) S=	1.52	;
    if(random >= 	0.9258732286	) S=	1.53	;
    if(random >= 	0.9278172286	) S=	1.54	;
    if(random >= 	0.9297042286	) S=	1.55	;
    if(random >= 	0.9315362286	) S=	1.56	;
    if(random >= 	0.9333152286	) S=	1.57	;
    if(random >= 	0.9350432286	) S=	1.58	;
    if(random >= 	0.9367222286	) S=	1.59	;
    if(random >= 	0.9383532286	) S=	1.6	;
    if(random >= 	0.9399372286	) S=	1.61	;
    if(random >= 	0.9414772286	) S=	1.62	;
    if(random >= 	0.9429732286	) S=	1.63	;
    if(random >= 	0.9444272286	) S=	1.64	;
    if(random >= 	0.9458412286	) S=	1.65	;
    if(random >= 	0.9472162286	) S=	1.66	;
    if(random >= 	0.9485532286	) S=	1.67	;
    if(random >= 	0.9498532286	) S=	1.68	;
    if(random >= 	0.9511172286	) S=	1.69	;
    if(random >= 	0.9523472286	) S=	1.7	;
    if(random >= 	0.9535442286	) S=	1.71	;
    if(random >= 	0.9547082286	) S=	1.72	;
    if(random >= 	0.9558412286	) S=	1.73	;
    if(random >= 	0.9569442286	) S=	1.74	;
    if(random >= 	0.9580172286	) S=	1.75	;
    if(random >= 	0.9590622286	) S=	1.76	;
    if(random >= 	0.9600792286	) S=	1.77	;
    if(random >= 	0.9610699286	) S=	1.78	;
    if(random >= 	0.9620346286	) S=	1.79	;
    if(random >= 	0.9629743286	) S=	1.8	;
    if(random >= 	0.9638897286	) S=	1.81	;
    if(random >= 	0.9647815286	) S=	1.82	;
    if(random >= 	0.9656504286	) S=	1.83	;
    if(random >= 	0.9664972286	) S=	1.84	;
    if(random >= 	0.9673225286	) S=	1.85	;
    if(random >= 	0.9681269286	) S=	1.86	;
    if(random >= 	0.9689110286	) S=	1.87	;
    if(random >= 	0.9696754286	) S=	1.88	;
    if(random >= 	0.9704208286	) S=	1.89	;
    if(random >= 	0.9711475286	) S=	1.9	;
    if(random >= 	0.9718563286	) S=	1.91	;
    if(random >= 	0.9725476286	) S=	1.92	;
    if(random >= 	0.9732219286	) S=	1.93	;
    if(random >= 	0.9738797286	) S=	1.94	;
    if(random >= 	0.9745214286	) S=	1.95	;
    if(random >= 	0.9751476286	) S=	1.96	;
    if(random >= 	0.9757586286	) S=	1.97	;
    if(random >= 	0.9763548286	) S=	1.98	;
    if(random >= 	0.9769367286	) S=	1.99	;
    if(random >= 	0.9775047286	) S=	2	;
    if(random >= 	0.9780591286	) S=	2.01	;
    if(random >= 	0.9786004286	) S=	2.02	;
    if(random >= 	0.9791288286	) S=	2.03	;
    if(random >= 	0.9796448286	) S=	2.04	;
    if(random >= 	0.9801487286	) S=	2.05	;
    if(random >= 	0.9806407286	) S=	2.06	;
    if(random >= 	0.9811212286	) S=	2.07	;
    if(random >= 	0.9815905286	) S=	2.08	;
    if(random >= 	0.9820489286	) S=	2.09	;
    if(random >= 	0.9824967286	) S=	2.1	;
    if(random >= 	0.9829341286	) S=	2.11	;
    if(random >= 	0.9833614286	) S=	2.12	;
    if(random >= 	0.9837789286	) S=	2.13	;
    if(random >= 	0.9841867286	) S=	2.14	;
    if(random >= 	0.9845852286	) S=	2.15	;
    if(random >= 	0.9849747286	) S=	2.16	;
    if(random >= 	0.9853553286	) S=	2.17	;
    if(random >= 	0.9857273286	) S=	2.18	;
    if(random >= 	0.9860909286	) S=	2.19	;
    if(random >= 	0.9864463286	) S=	2.2	;
    if(random >= 	0.9867936286	) S=	2.21	;
    if(random >= 	0.9871331286	) S=	2.22	;
    if(random >= 	0.9874650286	) S=	2.23	;
    if(random >= 	0.9877895286	) S=	2.24	;
    if(random >= 	0.9881067286	) S=	2.25	;
    if(random >= 	0.9884168286	) S=	2.26	;
    if(random >= 	0.9887200286	) S=	2.27	;
    if(random >= 	0.9890165286	) S=	2.28	;
    if(random >= 	0.9893064286	) S=	2.29	;
    if(random >= 	0.9895899286	) S=	2.3	;
    if(random >= 	0.9898671286	) S=	2.31	;
    if(random >= 	0.9901382286	) S=	2.32	;
    if(random >= 	0.9904033286	) S=	2.33	;
    if(random >= 	0.9906626286	) S=	2.34	;
    if(random >= 	0.9909162286	) S=	2.35	;
    if(random >= 	0.9911643286	) S=	2.36	;
    if(random >= 	0.9914070286	) S=	2.37	;
    if(random >= 	0.9916444286	) S=	2.38	;
    if(random >= 	0.9918766286	) S=	2.39	;
    if(random >= 	0.9921037286	) S=	2.4	;
    if(random >= 	0.9923259286	) S=	2.41	;
    if(random >= 	0.9925432286	) S=	2.42	;
    if(random >= 	0.9927558286	) S=	2.43	;
    if(random >= 	0.9929638286	) S=	2.44	;
    if(random >= 	0.9931673286	) S=	2.45	;
    if(random >= 	0.9933664286	) S=	2.46	;
    if(random >= 	0.9935612286	) S=	2.47	;
    if(random >= 	0.9937518286	) S=	2.48	;
    if(random >= 	0.9939383286	) S=	2.49	;
    if(random >= 	0.9941208286	) S=	2.5	;
    if(random >= 	0.9942994286	) S=	2.51	;
    if(random >= 	0.9944741286	) S=	2.52	;
    if(random >= 	0.9946451286	) S=	2.53	;
    if(random >= 	0.9948125286	) S=	2.54	;
    if(random >= 	0.9949763286	) S=	2.55	;
    if(random >= 	0.9951366286	) S=	2.56	;
    if(random >= 	0.9952934286	) S=	2.57	;
    if(random >= 	0.9954469286	) S=	2.58	;
    if(random >= 	0.9955971286	) S=	2.59	;
    if(random >= 	0.9957442286	) S=	2.6	;
    if(random >= 	0.9958881286	) S=	2.61	;
    if(random >= 	0.9960290286	) S=	2.62	;
    if(random >= 	0.9961669286	) S=	2.63	;
    if(random >= 	0.9963019286	) S=	2.64	;
    if(random >= 	0.9964340286	) S=	2.65	;
    if(random >= 	0.9965633286	) S=	2.66	;
    if(random >= 	0.9966899286	) S=	2.67	;
    if(random >= 	0.9968138286	) S=	2.68	;
    if(random >= 	0.9969351286	) S=	2.69	;
    if(random >= 	0.9970538286	) S=	2.7	;
    if(random >= 	0.9971700286	) S=	2.71	;
    if(random >= 	0.9972838286	) S=	2.72	;
    if(random >= 	0.9973952286	) S=	2.73	;
    if(random >= 	0.9975043286	) S=	2.74	;
    if(random >= 	0.9976111286	) S=	2.75	;
    if(random >= 	0.9977156286	) S=	2.76	;
    if(random >= 	0.9978179286	) S=	2.77	;
    if(random >= 	0.9979181286	) S=	2.78	;
    if(random >= 	0.9980162086	) S=	2.79	;
    if(random >= 	0.9981122186	) S=	2.8	;
    if(random >= 	0.9982062586	) S=	2.81	;
    if(random >= 	0.9982983286	) S=	2.82	;
    if(random >= 	0.9983884786	) S=	2.83	;
    if(random >= 	0.9984767486	) S=	2.84	;
    if(random >= 	0.9985631786	) S=	2.85	;
    if(random >= 	0.9986478086	) S=	2.86	;
    if(random >= 	0.9987306786	) S=	2.87	;
    if(random >= 	0.9988118286	) S=	2.88	;
    if(random >= 	0.9988912986	) S=	2.89	;
    if(random >= 	0.9989691086	) S=	2.9	;
    if(random >= 	0.9990453086	) S=	2.91	;
    if(random >= 	0.9991199186	) S=	2.92	;
    if(random >= 	0.9991929886	) S=	2.93	;
    if(random >= 	0.9992645486	) S=	2.94	;
    if(random >= 	0.9993346286	) S=	2.95	;
    if(random >= 	0.9994032586	) S=	2.96	;
    if(random >= 	0.9994704686	) S=	2.97	;
    if(random >= 	0.9995362886	) S=	2.98	;
    if(random >= 	0.9996007486	) S=	2.99	;
    if(random >= 	0.9996638886	) S=	3	;
    if(random >= 	0.9997257186	) S=	3.01	;
    if(random >= 	0.9997862686	) S=	3.02	;
    if(random >= 	0.9998455786	) S=	3.03	;
    if(random >= 	0.9999036686	) S=	3.04	;
    if(random >= 	0.9999605586	) S=	3.05	;
    if(random >= 	1.0000162886	) S=	3.06	;
    if(random >= 	1.0000708686	) S=	3.07	;
    if(random >= 	1.0001243286	) S=	3.08	;
    if(random >= 	1.0001766986	) S=	3.09	;
    if(random >= 	1.0002279986	) S=	3.1	;
    if(random >= 	1.0002782386	) S=	3.11	;
    if(random >= 	1.0003274486	) S=	3.12	;
    if(random >= 	1.0003756586	) S=	3.13	;
    if(random >= 	1.0004228786	) S=	3.14	;
    if(random >= 	1.0004691286	) S=	3.15	;
    if(random >= 	1.0005144386	) S=	3.16	;
    if(random >= 	1.0005588286	) S=	3.17	;
    if(random >= 	1.0006023086	) S=	3.18	;
    if(random >= 	1.0006448986	) S=	3.19	;
    if(random >= 	1.0006866286	) S=	3.2	;
    if(random >= 	1.0007275086	) S=	3.21	;
    if(random >= 	1.0007675486	) S=	3.22	;
    if(random >= 	1.0008067886	) S=	3.23	;
    if(random >= 	1.0008452286	) S=	3.24	;
    if(random >= 	1.0008828786	) S=	3.25	;
    if(random >= 	1.0009197686	) S=	3.26	;
    if(random >= 	1.0009559086	) S=	3.27	;
    if(random >= 	1.0009913186	) S=	3.28	;
    if(random >= 	1.0010260186	) S=	3.29	;
    if(random >= 	1.0010600086	) S=	3.3	;
    if(random >= 	1.0010933086	) S=	3.31	;
    if(random >= 	1.0011259386	) S=	3.32	;
    if(random >= 	1.0011579086	) S=	3.33	;
    if(random >= 	1.0011892286	) S=	3.34	;
    if(random >= 	1.0012199286	) S=	3.35	;
    if(random >= 	1.0012499986	) S=	3.36	;
    if(random >= 	1.0012794586	) S=	3.37	;
    if(random >= 	1.0013083286	) S=	3.38	;
    if(random >= 	1.0013366186	) S=	3.39	;
    if(random >= 	1.0013643386	) S=	3.4	;
    if(random >= 	1.0013915086	) S=	3.41	;
    if(random >= 	1.0014181286	) S=	3.42	;
    if(random >= 	1.0014442086	) S=	3.43	;
    if(random >= 	1.0014697686	) S=	3.44	;
    if(random >= 	1.0014948186	) S=	3.45	;
    if(random >= 	1.0015193586	) S=	3.46	;
    if(random >= 	1.0015434086	) S=	3.47	;
    if(random >= 	1.0015669786	) S=	3.48	;
    if(random >= 	1.0015900786	) S=	3.49	;
    if(random >= 	1.0016127186	) S=	3.5	;
    if(random >= 	1.0016349086	) S=	3.51	;
    if(random >= 	1.0016566486	) S=	3.52	;
    if(random >= 	1.0016779586	) S=	3.53	;
    if(random >= 	1.0016988486	) S=	3.54	;
    if(random >= 	1.0017193186	) S=	3.55	;
    if(random >= 	1.0017393786	) S=	3.56	;
    if(random >= 	1.0017590386	) S=	3.57	;
    if(random >= 	1.0017783086	) S=	3.58	;
    if(random >= 	1.0017971986	) S=	3.59	;
    if(random >= 	1.0018157086	) S=	3.6	;
    if(random >= 	1.0018338486	) S=	3.61	;
    if(random >= 	1.0018516286	) S=	3.62	;
    if(random >= 	1.0018690586	) S=	3.63	;
    if(random >= 	1.0018861486	) S=	3.64	;
    if(random >= 	1.0019028986	) S=	3.65	;
    if(random >= 	1.0019193186	) S=	3.66	;
    if(random >= 	1.0019354086	) S=	3.67	;
    if(random >= 	1.0019511786	) S=	3.68	;
    if(random >= 	1.0019666486	) S=	3.69	;
    if(random >= 	1.0019818086	) S=	3.7	;
    if(random >= 	1.0019966686	) S=	3.71	;
    if(random >= 	1.0020112386	) S=	3.72	;
    if(random >= 	1.0020255186	) S=	3.73	;
    if(random >= 	1.0020395186	) S=	3.74	;
    if(random >= 	1.0020532386	) S=	3.75	;
    if(random >= 	1.0020666986	) S=	3.76	;
    if(random >= 	1.0020798886	) S=	3.77	;
    if(random >= 	1.0020928186	) S=	3.78	;
    if(random >= 	1.0021054986	) S=	3.79	;
    if(random >= 	1.0021179286	) S=	3.8	;
    if(random >= 	1.0021301186	) S=	3.81	;
    if(random >= 	1.0021420686	) S=	3.82	;
    if(random >= 	1.0021537786	) S=	3.83	;
    if(random >= 	1.0021652686	) S=	3.84	;
    if(random >= 	1.0021765286	) S=	3.85	;
    if(random >= 	1.0021875686	) S=	3.86	;
    if(random >= 	1.0021983886	) S=	3.87	;
    if(random >= 	1.0022089986	) S=	3.88	;
    if(random >= 	1.0022194086	) S=	3.89	;
    if(random >= 	1.0022296086	) S=	3.9	;
    if(random >= 	1.0022396086	) S=	3.91	;
    if(random >= 	1.0022494136	) S=	3.92	;
    if(random >= 	1.0022590316	) S=	3.93	;
    if(random >= 	1.0022684636	) S=	3.94	;
    if(random >= 	1.0022777106	) S=	3.95	;
    if(random >= 	1.0022867796	) S=	3.96	;
    if(random >= 	1.0022956716	) S=	3.97	;
    if(random >= 	1.0023043876	) S=	3.98	;
    if(random >= 	1.0023129356	) S=	3.99	;
    if(random >= 	1.0023213226	) S=	4.0	;
    if(random >= 	1.0023295426	) S=	4.01	;
    if(random >= 	1.0023376076	) S=	4.02	;
    if(random >= 	1.0023455156	) S=	4.03	;
    if(random >= 	1.0023532706	) S=	4.04	;
    if(random >= 	1.0023608766	) S=	4.05	;
    if(random >= 	1.0023683356	) S=	4.06	;
    if(random >= 	1.0023756506	) S=	4.07	;
    if(random >= 	1.0023828256	) S=	4.08	;
    if(random >= 	1.0023898626	) S=	4.09	;
    if(random >= 	1.0023967646	) S=	4.1	;
    if(random >= 	1.0024035326	) S=	4.11	;
    if(random >= 	1.0024101716	) S=	4.12	;
    if(random >= 	1.0024166826	) S=	4.13	;
    if(random >= 	1.0024230696	) S=	4.14	;
    if(random >= 	1.0024293336	) S=	4.15	;
    if(random >= 	1.0024354776	) S=	4.16	;
    if(random >= 	1.0024415046	) S=	4.17	;
    if(random >= 	1.0024474166	) S=	4.18	;
    if(random >= 	1.0024532146	) S=	4.19	;
    if(random >= 	1.0024589016	) S=	4.2	;
    if(random >= 	1.0024644806	) S=	4.21	;
    if(random >= 	1.0024699536	) S=	4.22	;
    if(random >= 	1.0024753226	) S=	4.23	;
    if(random >= 	1.0024805886	) S=	4.24	;
    if(random >= 	1.0024857546	) S=	4.25	;
    if(random >= 	1.0024908226	) S=	4.26	;
    if(random >= 	1.0024957936	) S=	4.27	;
    if(random >= 	1.0025006706	) S=	4.28	;
    if(random >= 	1.0025054546	) S=	4.29	;
    if(random >= 	1.0025101476	) S=	4.3	;
    if(random >= 	1.0025147516	) S=	4.31	;
    if(random >= 	1.0025192686	) S=	4.32	;
    if(random >= 	1.0025236996	) S=	4.33	;
    if(random >= 	1.0025280476	) S=	4.34	;
    if(random >= 	1.0025323126	) S=	4.35	;
    if(random >= 	1.0025364976	) S=	4.36	;
    if(random >= 	1.0025406036	) S=	4.37	;
    if(random >= 	1.0025446316	) S=	4.38	;
    if(random >= 	1.0025485836	) S=	4.39	;
    if(random >= 	1.0025524616	) S=	4.4	;
    if(random >= 	1.0025562666	) S=	4.41	;
    if(random >= 	1.0025600006	) S=	4.42	;
    if(random >= 	1.0025636636	) S=	4.43	;
    if(random >= 	1.0025672586	) S=	4.44	;
    if(random >= 	1.0025707856	) S=	4.45	;
    if(random >= 	1.0025742456	) S=	4.46	;
    if(random >= 	1.0025776406	) S=	4.47	;
    if(random >= 	1.0025809726	) S=	4.48	;
    if(random >= 	1.0025842426	) S=	4.49	;
    if(random >= 	1.0025874516	) S=	4.5	;
    if(random >= 	1.0025906006	) S=	4.51	;
    if(random >= 	1.0025936906	) S=	4.52	;
    if(random >= 	1.0025967226	) S=	4.53	;
    if(random >= 	1.0025996986	) S=	4.54	;
    if(random >= 	1.0026026186	) S=	4.55	;
    if(random >= 	1.0026054846	) S=	4.56	;
    if(random >= 	1.0026082966	) S=	4.57	;
    if(random >= 	1.0026110566	) S=	4.58	;
    if(random >= 	1.0026137646	) S=	4.59	;
    if(random >= 	1.0026164226	) S=	4.6	;
    if(random >= 	1.0026190306	) S=	4.61	;
    if(random >= 	1.0026215906	) S=	4.62	;
    if(random >= 	1.0026241036	) S=	4.63	;
    if(random >= 	1.0026265696	) S=	4.64	;
    if(random >= 	1.0026289896	) S=	4.65	;
    if(random >= 	1.0026313656	) S=	4.66	;
    if(random >= 	1.0026336976	) S=	4.67	;
    if(random >= 	1.0026359866	) S=	4.68	;
    if(random >= 	1.0026382336	) S=	4.69	;
    if(random >= 	1.0026404386	) S=	4.7	;
    if(random >= 	1.0026426026	) S=	4.71	;
    if(random >= 	1.0026447266	) S=	4.72	;
    if(random >= 	1.0026468116	) S=	4.73	;
    if(random >= 	1.0026488586	) S=	4.74	;
    if(random >= 	1.0026508676	) S=	4.75	;
    if(random >= 	1.0026528396	) S=	4.76	;
    if(random >= 	1.0026547756	) S=	4.77	;
    if(random >= 	1.0026566756	) S=	4.78	;
    if(random >= 	1.0026585416	) S=	4.79	;
    if(random >= 	1.0026603736	) S=	4.8	;
    if(random >= 	1.0026621716	) S=	4.81	;
    if(random >= 	1.0026639366	) S=	4.82	;
    if(random >= 	1.0026656696	) S=	4.83	;
    if(random >= 	1.0026673706	) S=	4.84	;
    if(random >= 	1.0026690406	) S=	4.85	;
    if(random >= 	1.0026706806	) S=	4.86	;
    if(random >= 	1.0026722896	) S=	4.87	;
    if(random >= 	1.0026738696	) S=	4.88	;
    if(random >= 	1.0026754206	) S=	4.89	;
    if(random >= 	1.0026769436	) S=	4.9	;
    if(random >= 	1.0026784386	) S=	4.91	;
    if(random >= 	1.0026799066	) S=	4.92	;
    if(random >= 	1.0026813486	) S=	4.93	;
    if(random >= 	1.0026827646	) S=	4.94	;
    if(random >= 	1.0026841546	) S=	4.95	;
    if(random >= 	1.0026855186	) S=	4.96	;
    if(random >= 	1.0026868586	) S=	4.97	;
    if(random >= 	1.0026881736	) S=	4.98	;
    if(random >= 	1.0026894646	) S=	4.99	;
    if(random >= 	1.0026907336	) S=	5	;
    if(random >= 	1.0026919796	) S=	5.01	;
    if(random >= 	1.0026932026	) S=	5.02	;
    if(random >= 	1.0026944036	) S=	5.03	;
    if(random >= 	1.0026955826	) S=	5.04	;
    if(random >= 	1.0026967406	) S=	5.05	;
    if(random >= 	1.0026978776	) S=	5.06	;
    if(random >= 	1.0026989946	) S=	5.07	;
    if(random >= 	1.0027000906	) S=	5.08	;
    if(random >= 	1.0027011676	) S=	5.09	;
    if(random >= 	1.0027022256	) S=	5.1	;
    if(random >= 	1.0027032646	) S=	5.11	;
    if(random >= 	1.0027042846	) S=	5.12	;
    if(random >= 	1.0027052866	) S=	5.13	;
    if(random >= 	1.0027062705	) S=	5.14	;
    if(random >= 	1.0027072364	) S=	5.15	;
    if(random >= 	1.0027081850	) S=	5.16	;
    if(random >= 	1.0027091170	) S=	5.17	;
    if(random >= 	1.0027100325	) S=	5.18	;
    if(random >= 	1.0027109316	) S=	5.19	;
    if(random >= 	1.0027118143	) S=	5.2	;
    if(random >= 	1.0027126813	) S=	5.21	;
    if(random >= 	1.0027135333	) S=	5.22	;
    if(random >= 	1.0027143697	) S=	5.23	;
    if(random >= 	1.0027151912	) S=	5.24	;
    if(random >= 	1.0027159986	) S=	5.25	;
    if(random >= 	1.0027167912	) S=	5.26	;
    if(random >= 	1.0027175697	) S=	5.27	;
    if(random >= 	1.0027183346	) S=	5.28	;
    if(random >= 	1.0027190859	) S=	5.29	;
    if(random >= 	1.0027198239	) S=	5.3	;
    if(random >= 	1.0027205489	) S=	5.31	;
    if(random >= 	1.0027212611	) S=	5.32	;
    if(random >= 	1.0027219607	) S=	5.33	;
    if(random >= 	1.0027226479	) S=	5.34	;
    if(random >= 	1.0027233230	) S=	5.35	;
    if(random >= 	1.0027239862	) S=	5.36	;
    if(random >= 	1.0027246377	) S=	5.37	;
    if(random >= 	1.0027252777	) S=	5.38	;
    if(random >= 	1.0027259065	) S=	5.39	;
    if(random >= 	1.0027265242	) S=	5.4	;
    if(random >= 	1.0027271310	) S=	5.41	;
    if(random >= 	1.0027277272	) S=	5.42	;
    if(random >= 	1.0027283129	) S=	5.43	;
    if(random >= 	1.0027288884	) S=	5.44	;
    if(random >= 	1.0027294538	) S=	5.45	;
    if(random >= 	1.0027300093	) S=	5.46	;
    if(random >= 	1.0027305550	) S=	5.47	;
    if(random >= 	1.0027310912	) S=	5.48	;
    if(random >= 	1.0027316180	) S=	5.49	;
    if(random >= 	1.0027321356	) S=	5.5	;
    if(random >= 	1.0027326442	) S=	5.51	;
    if(random >= 	1.0027331439	) S=	5.52	;
    if(random >= 	1.0027336349	) S=	5.53	;
    if(random >= 	1.0027341173	) S=	5.54	;
    if(random >= 	1.0027345914	) S=	5.55	;
    if(random >= 	1.0027350572	) S=	5.56	;
    if(random >= 	1.0027355149	) S=	5.57	;
    if(random >= 	1.0027359646	) S=	5.58	;
    if(random >= 	1.0027364065	) S=	5.59	;
    if(random >= 	1.0027368408	) S=	5.6	;
    if(random >= 	1.0027372675	) S=	5.61	;
    if(random >= 	1.0027376868	) S=	5.62	;
    if(random >= 	1.0027380989	) S=	5.63	;
    if(random >= 	1.0027385038	) S=	5.64	;
    if(random >= 	1.0027389018	) S=	5.65	;
    if(random >= 	1.0027392929	) S=	5.66	;
    if(random >= 	1.0027396772	) S=	5.67	;
    if(random >= 	1.0027400549	) S=	5.68	;
    if(random >= 	1.0027404261	) S=	5.69	;
    if(random >= 	1.0027407909	) S=	5.7	;
    if(random >= 	1.0027411494	) S=	5.71	;
    if(random >= 	1.0027415017	) S=	5.72	;
    if(random >= 	1.0027418480	) S=	5.73	;
    if(random >= 	1.0027421883	) S=	5.74	;
    if(random >= 	1.0027425228	) S=	5.75	;
    if(random >= 	1.0027428516	) S=	5.76	;
    if(random >= 	1.0027431747	) S=	5.77	;
    if(random >= 	1.0027434923	) S=	5.78	;
    if(random >= 	1.0027438044	) S=	5.79	;
    if(random >= 	1.0027441112	) S=	5.8	;
    if(random >= 	1.0027444127	) S=	5.81	;
    if(random >= 	1.0027447091	) S=	5.82	;
    if(random >= 	1.0027450005	) S=	5.83	;
    if(random >= 	1.0027452869	) S=	5.84	;
    if(random >= 	1.0027455683	) S=	5.85	;
    if(random >= 	1.0027458450	) S=	5.86	;
    if(random >= 	1.0027461170	) S=	5.87	;
    if(random >= 	1.0027463843	) S=	5.88	;
    if(random >= 	1.0027466471	) S=	5.89	;
    if(random >= 	1.0027469054	) S=	5.9	;
    if(random >= 	1.0027471593	) S=	5.91	;
    if(random >= 	1.0027474089	) S=	5.92	;
    if(random >= 	1.0027476543	) S=	5.93	;
    if(random >= 	1.0027478955	) S=	5.94	;
    if(random >= 	1.0027481326	) S=	5.95	;
    if(random >= 	1.0027483658	) S=	5.96	;
    if(random >= 	1.0027485950	) S=	5.97	;
    if(random >= 	1.0027488203	) S=	5.98	;
    if(random >= 	1.0027490418	) S=	5.99	;
    if(random >= 	1.0027492596	) S=	6.0	;
    if(random >= 	1.0027494737	) S=	6.01	;
    if(random >= 	1.0027496842	) S=	6.02	;
    if(random >= 	1.0027498912	) S=	6.03	;
    if(random >= 	1.0027500947	) S=	6.04	;
    if(random >= 	1.0027502947	) S=	6.05	;
    if(random >= 	1.0027504913	) S=	6.06	;
    if(random >= 	1.0027506847	) S=	6.07	;
    if(random >= 	1.0027508748	) S=	6.08	;
    if(random >= 	1.0027510617	) S=	6.09	;
    if(random >= 	1.0027512455	) S=	6.1	;
    if(random >= 	1.0027514262	) S=	6.11	;
    if(random >= 	1.0027516039	) S=	6.12	;
    if(random >= 	1.0027517786	) S=	6.13	;
    if(random >= 	1.0027519504	) S=	6.14	;
    if(random >= 	1.0027521193	) S=	6.15	;
    if(random >= 	1.0027522854	) S=	6.16	;
    if(random >= 	1.0027524487	) S=	6.17	;
    if(random >= 	1.0027526093	) S=	6.18	;
    if(random >= 	1.0027527672	) S=	6.19	;
    if(random >= 	1.0027529225	) S=	6.2	;
    if(random >= 	1.0027530752	) S=	6.21	;
    if(random >= 	1.0027532254	) S=	6.22	;
    if(random >= 	1.0027533731	) S=	6.23	;
    if(random >= 	1.0027535183	) S=	6.24	;
    if(random >= 	1.0027536611	) S=	6.25	;
    if(random >= 	1.0027538015	) S=	6.26	;
    if(random >= 	1.0027539396	) S=	6.27	;
    if(random >= 	1.0027540754	) S=	6.28	;
    if(random >= 	1.0027542090	) S=	6.29	;
    if(random >= 	1.0027543404	) S=	6.3	;
    if(random >= 	1.0027544696	) S=	6.31	;
    if(random >= 	1.0027545966	) S=	6.32	;
    if(random >= 	1.0027547216	) S=	6.33	;
    if(random >= 	1.0027548445	) S=	6.34	;
    if(random >= 	1.0027549654	) S=	6.35	;
    if(random >= 	1.0027550843	) S=	6.36	;
    if(random >= 	1.0027552012	) S=	6.37	;
    if(random >= 	1.0027553162	) S=	6.38	;
    if(random >= 	1.0027554293	) S=	6.39	;
    if(random >= 	1.0027555405	) S=	6.4	;
    if(random >= 	1.0027556499	) S=	6.41	;
    if(random >= 	1.0027557575	) S=	6.42	;
    if(random >= 	1.0027558633	) S=	6.43	;
    if(random >= 	1.0027559674	) S=	6.44	;
    if(random >= 	1.0027560698	) S=	6.45	;
    if(random >= 	1.0027561706	) S=	6.46	;
    if(random >= 	1.0027562697	) S=	6.47	;
    if(random >= 	1.0027563672	) S=	6.48	;
    if(random >= 	1.0027564631	) S=	6.49	;
    if(random >= 	1.0027565574	) S=	6.5	;
    if(random >= 	1.0027566502	) S=	6.51	;
    if(random >= 	1.0027567415	) S=	6.52	;
    if(random >= 	1.0027568313	) S=	6.53	;
    if(random >= 	1.0027569196	) S=	6.54	;
    if(random >= 	1.0027570065	) S=	6.55	;
    if(random >= 	1.0027570920	) S=	6.56	;
    if(random >= 	1.0027571761	) S=	6.57	;
    if(random >= 	1.0027572588	) S=	6.58	;
    if(random >= 	1.0027573402	) S=	6.59	;
    if(random >= 	1.0027574202	) S=	6.6	;
    if(random >= 	1.0027574990	) S=	6.61	;
    if(random >= 	1.0027575764	) S=	6.62	;
    if(random >= 	1.0027576527	) S=	6.63	;
    if(random >= 	1.0027577276	) S=	6.64	;
    if(random >= 	1.0027578014	) S=	6.65	;
    if(random >= 	1.0027578740	) S=	6.66	;
    if(random >= 	1.0027579454	) S=	6.67	;
    if(random >= 	1.0027580156	) S=	6.68	;
    if(random >= 	1.0027580847	) S=	6.69	;
    if(random >= 	1.0027581527	) S=	6.7	;
    if(random >= 	1.0027582197	) S=	6.71	;
    if(random >= 	1.0027582855	) S=	6.72	;
    if(random >= 	1.0027583503	) S=	6.73	;
    if(random >= 	1.0027584140	) S=	6.74	;
    if(random >= 	1.0027584767	) S=	6.75	;
    if(random >= 	1.0027585385	) S=	6.76	;
    if(random >= 	1.0027585992	) S=	6.77	;
    if(random >= 	1.0027586589	) S=	6.78	;
    if(random >= 	1.0027587177	) S=	6.79	;
    if(random >= 	1.0027587756	) S=	6.8	;
    if(random >= 	1.0027588325	) S=	6.81	;
    if(random >= 	1.0027588885	) S=	6.82	;
    if(random >= 	1.0027589436	) S=	6.83	;
    if(random >= 	1.0027589979	) S=	6.84	;
    if(random >= 	1.0027590512	) S=	6.85	;
    if(random >= 	1.0027591037	) S=	6.86	;
    if(random >= 	1.0027591554	) S=	6.87	;
    if(random >= 	1.0027592063	) S=	6.88	;
    if(random >= 	1.0027592563	) S=	6.89	;
    if(random >= 	1.0027593056	) S=	6.9	;
    if(random >= 	1.0027593541	) S=	6.91	;
    if(random >= 	1.0027594018	) S=	6.92	;
    if(random >= 	1.0027594487	) S=	6.93	;
    if(random >= 	1.0027594949	) S=	6.94	;
    if(random >= 	1.0027595404	) S=	6.95	;
    if(random >= 	1.0027595851	) S=	6.96	;
    if(random >= 	1.0027596292	) S=	6.97	;
    if(random >= 	1.0027596725	) S=	6.98	;
    if(random >= 	1.0027597152	) S=	6.99	;
    if(random >= 	1.0027597571	) S=	7.0	;
    if(random >= 	1.0027597985	) S=	7.01	;
    if(random >= 	1.0027598391	) S=	7.02	;
    if(random >= 	1.0027598792	) S=	7.03	;
    if(random >= 	1.0027599186	) S=	7.04	;
    if(random >= 	1.0027599573	) S=	7.05	;
    if(random >= 	1.0027599955	) S=	7.06	;
    if(random >= 	1.0027600331	) S=	7.07	;
    if(random >= 	1.0027600700	) S=	7.08	;
    if(random >= 	1.0027601064	) S=	7.09	;
    if(random >= 	1.0027601423	) S=	7.1	;
    if(random >= 	1.0027601775	) S=	7.11	;
    if(random >= 	1.0027602122	) S=	7.12	;
    if(random >= 	1.0027602464	) S=	7.13	;
    if(random >= 	1.0027602800	) S=	7.14	;
    if(random >= 	1.0027603131	) S=	7.15	;
    if(random >= 	1.0027603457	) S=	7.16	;
    if(random >= 	1.0027603778	) S=	7.17	;
    if(random >= 	1.0027604093	) S=	7.18	;
    if(random >= 	1.0027604404	) S=	7.19	;
    if(random >= 	1.0027604710	) S=	7.2	;
    if(random >= 	1.0027605011	) S=	7.21	;
    if(random >= 	1.0027605308	) S=	7.22	;
    if(random >= 	1.0027605599	) S=	7.23	;
    if(random >= 	1.0027605887	) S=	7.24	;
    if(random >= 	1.0027606170	) S=	7.25	;
    if(random >= 	1.0027606448	) S=	7.26	;
    if(random >= 	1.0027606722	) S=	7.27	;
    if(random >= 	1.0027606992	) S=	7.28	;
    if(random >= 	1.0027607258	) S=	7.29	;
    if(random >= 	1.0027607519	) S=	7.3	;
    if(random >= 	1.0027607777	) S=	7.31	;
    if(random >= 	1.0027608030	) S=	7.32	;
    if(random >= 	1.0027608280	) S=	7.33	;
    if(random >= 	1.0027608526	) S=	7.34	;
    if(random >= 	1.0027608768	) S=	7.35	;
    if(random >= 	1.0027609006	) S=	7.36	;
    if(random >= 	1.0027609240	) S=	7.37	;
    if(random >= 	1.0027609471	) S=	7.38	;
    if(random >= 	1.0027609698	) S=	7.39	;
    if(random >= 	1.0027609922	) S=	7.4	;
    if(random >= 	1.0027610143	) S=	7.41	;
    if(random >= 	1.0027610360	) S=	7.42	;
    if(random >= 	1.0027610573	) S=	7.43	;
    if(random >= 	1.0027610784	) S=	7.44	;
    if(random >= 	1.0027610991	) S=	7.45	;
    if(random >= 	1.0027611195	) S=	7.46	;
    if(random >= 	1.0027611396	) S=	7.47	;
    if(random >= 	1.0027611594	) S=	7.48	;
    if(random >= 	1.0027611788	) S=	7.49	;
    if(random >= 	1.0027611980	) S=	7.5	;
    if(random >= 	1.0027612169	) S=	7.51	;
    if(random >= 	1.0027612355	) S=	7.52	;
    if(random >= 	1.0027612538	) S=	7.53	;
    if(random >= 	1.0027612718	) S=	7.54	;
    if(random >= 	1.0027612896	) S=	7.55	;
    if(random >= 	1.0027613070	) S=	7.56	;
    if(random >= 	1.0027613243	) S=	7.57	;
    if(random >= 	1.0027613412	) S=	7.58	;
    if(random >= 	1.0027613579	) S=	7.59	;
    if(random >= 	1.0027613744	) S=	7.6	;
    if(random >= 	1.0027613905	) S=	7.61	;
    if(random >= 	1.0027614065	) S=	7.62	;
    if(random >= 	1.0027614222	) S=	7.63	;
    if(random >= 	1.0027614377	) S=	7.64	;
    if(random >= 	1.0027614529	) S=	7.65	;
    if(random >= 	1.0027614679	) S=	7.66	;
    if(random >= 	1.0027614827	) S=	7.67	;
    if(random >= 	1.0027614972	) S=	7.68	;
    if(random >= 	1.0027615116	) S=	7.69	;
    if(random >= 	1.0027615257	) S=	7.7	;
    if(random >= 	1.0027615396	) S=	7.71	;
    if(random >= 	1.0027615533	) S=	7.72	;
    if(random >= 	1.0027615667	) S=	7.73	;
    if(random >= 	1.0027615800	) S=	7.74	;
    if(random >= 	1.0027615931	) S=	7.75	;
    if(random >= 	1.0027616060	) S=	7.76	;
    if(random >= 	1.0027616187	) S=	7.77	;
    if(random >= 	1.0027616312	) S=	7.78	;
    if(random >= 	1.0027616435	) S=	7.79	;
    if(random >= 	1.0027616556	) S=	7.8	;
    if(random >= 	1.0027616675	) S=	7.81	;
    if(random >= 	1.0027616793	) S=	7.82	;
    if(random >= 	1.0027616909	) S=	7.83	;
    if(random >= 	1.0027617023	) S=	7.84	;
    if(random >= 	1.0027617135	) S=	7.85	;
    if(random >= 	1.0027617246	) S=	7.86	;
    if(random >= 	1.0027617355	) S=	7.87	;
    if(random >= 	1.0027617463	) S=	7.88	;
    if(random >= 	1.0027617568	) S=	7.89	;
    if(random >= 	1.0027617673	) S=	7.9	;
    if(random >= 	1.0027617775	) S=	7.91	;
    if(random >= 	1.0027617877	) S=	7.92	;
    if(random >= 	1.0027617976	) S=	7.93	;
    if(random >= 	1.0027618075	) S=	7.94	;
    if(random >= 	1.0027618171	) S=	7.95	;
    if(random >= 	1.0027618267	) S=	7.96	;
    if(random >= 	1.0027618360	) S=	7.97	;
    if(random >= 	1.0027618453	) S=	7.98	;
    if(random >= 	1.0027618544	) S=	7.99	;
    if(random >= 	1.0027618634	) S=	8.0	;
    if(random >= 	1.0027618722	) S=	8.01	;
    if(random >= 	1.0027618809	) S=	8.02	;
    if(random >= 	1.0027618895	) S=	8.03	;
    if(random >= 	1.0027618979	) S=	8.04	;
    if(random >= 	1.0027619063	) S=	8.05	;
    if(random >= 	1.0027619145	) S=	8.06	;
    if(random >= 	1.0027619226	) S=	8.07	;
    if(random >= 	1.0027619305	) S=	8.08	;
    if(random >= 	1.0027619384	) S=	8.09	;
    if(random >= 	1.0027619461	) S=	8.1	;
    if(random >= 	1.0027619537	) S=	8.11	;
    if(random >= 	1.0027619612	) S=	8.12	;
    if(random >= 	1.0027619686	) S=	8.13	;
    if(random >= 	1.0027619759	) S=	8.14	;
    if(random >= 	1.0027619831	) S=	8.15	;
    if(random >= 	1.0027619901	) S=	8.16	;
    if(random >= 	1.0027619971	) S=	8.17	;
    if(random >= 	1.0027620040	) S=	8.18	;
    if(random >= 	1.0027620107	) S=	8.19	;
    if(random >= 	1.0027620174	) S=	8.2	;
    if(random >= 	1.0027620240	) S=	8.21	;
    if(random >= 	1.0027620305	) S=	8.22	;
    if(random >= 	1.0027620368	) S=	8.23	;
    if(random >= 	1.0027620431	) S=	8.24	;
    if(random >= 	1.0027620493	) S=	8.25	;
    if(random >= 	1.0027620554	) S=	8.26	;
    if(random >= 	1.0027620614	) S=	8.27	;
    if(random >= 	1.0027620673	) S=	8.28	;
    if(random >= 	1.0027620732	) S=	8.29	;
    if(random >= 	1.0027620789	) S=	8.3	;
    if(random >= 	1.0027620846	) S=	8.31	;
    if(random >= 	1.0027620902	) S=	8.32	;
    if(random >= 	1.0027620957	) S=	8.33	;
    if(random >= 	1.0027621011	) S=	8.34	;
    if(random >= 	1.0027621065	) S=	8.35	;
    if(random >= 	1.0027621117	) S=	8.36	;
    if(random >= 	1.0027621169	) S=	8.37	;
    if(random >= 	1.0027621220	) S=	8.38	;
    if(random >= 	1.0027621271	) S=	8.39	;
    if(random >= 	1.0027621321	) S=	8.4	;
    if(random >= 	1.0027621369	) S=	8.41	;
    if(random >= 	1.0027621418	) S=	8.42	;
    if(random >= 	1.0027621465	) S=	8.43	;
    if(random >= 	1.0027621512	) S=	8.44	;
    if(random >= 	1.0027621558	) S=	8.45	;
    if(random >= 	1.0027621604	) S=	8.46	;
    if(random >= 	1.0027621649	) S=	8.47	;
    if(random >= 	1.0027621693	) S=	8.48	;
    if(random >= 	1.0027621737	) S=	8.49	;
    if(random >= 	1.0027621780	) S=	8.5	;
    if(random >= 	1.0027621822	) S=	8.51	;
    if(random >= 	1.0027621864	) S=	8.52	;
    if(random >= 	1.0027621905	) S=	8.53	;
    if(random >= 	1.0027621945	) S=	8.54	;
    if(random >= 	1.0027621985	) S=	8.55	;
    if(random >= 	1.0027622025	) S=	8.56	;
    if(random >= 	1.0027622063	) S=	8.57	;
    if(random >= 	1.0027622102	) S=	8.58	;
    if(random >= 	1.0027622139	) S=	8.59	;
    if(random >= 	1.0027622177	) S=	8.6	;
    if(random >= 	1.0027622213	) S=	8.61	;
    if(random >= 	1.0027622249	) S=	8.62	;
    if(random >= 	1.0027622285	) S=	8.63	;
    if(random >= 	1.0027622320	) S=	8.64	;
    if(random >= 	1.0027622355	) S=	8.65	;
    if(random >= 	1.0027622389	) S=	8.66	;
    if(random >= 	1.0027622422	) S=	8.67	;
    if(random >= 	1.0027622455	) S=	8.68	;
    if(random >= 	1.0027622488	) S=	8.69	;
    if(random >= 	1.0027622520	) S=	8.7	;
    if(random >= 	1.0027622552	) S=	8.71	;
    if(random >= 	1.0027622583	) S=	8.72	;
    if(random >= 	1.0027622614	) S=	8.73	;
    if(random >= 	1.0027622645	) S=	8.74	;
    if(random >= 	1.0027622675	) S=	8.75	;
    if(random >= 	1.0027622704	) S=	8.76	;
    if(random >= 	1.0027622733	) S=	8.77	;
    if(random >= 	1.0027622762	) S=	8.78	;
    if(random >= 	1.0027622790	) S=	8.79	;
    if(random >= 	1.0027622818	) S=	8.8	;
    if(random >= 	1.0027622846	) S=	8.81	;
    if(random >= 	1.0027622873	) S=	8.82	;
    if(random >= 	1.0027622899	) S=	8.83	;
    if(random >= 	1.0027622926	) S=	8.84	;
    if(random >= 	1.0027622952	) S=	8.85	;
    if(random >= 	1.0027622977	) S=	8.86	;
    if(random >= 	1.0027623003	) S=	8.87	;
    if(random >= 	1.0027623028	) S=	8.88	;
    if(random >= 	1.0027623052	) S=	8.89	;
    if(random >= 	1.0027623076	) S=	8.9	;
    if(random >= 	1.0027623100	) S=	8.91	;
    if(random >= 	1.0027623124	) S=	8.92	;
    if(random >= 	1.0027623147	) S=	8.93	;
    if(random >= 	1.0027623170	) S=	8.94	;
    if(random >= 	1.0027623192	) S=	8.95	;
    if(random >= 	1.0027623215	) S=	8.96	;
    if(random >= 	1.0027623237	) S=	8.97	;
    if(random >= 	1.0027623259	) S=	8.98	;
    if(random >= 	1.0027623280	) S=	8.99	;
    if(random >= 	1.0027623302	) S=	9.0	;
    if(random >= 	1.0027623323	) S=	9.01	;
    
    //cout << random << '\t' << S << endl;
    
    lambda_palaeosens = 1.0/S;
    
    return lambda_palaeosens;
    
}

//This sets lambda from the linear-error palaeosens distribution (Rohling et al, Nature)
double get_lambda_palaeosens2(double random)
{
    double lambda_palaeosens = 0.0;
    double S=0;
    
    
    if(random >= 	0.00000000000000E+00	) S=	-2.2	;
    if(random >= 	1.14800000000000E-15	) S=	-2.19	;
    if(random >= 	2.51600000000000E-15	) S=	-2.18	;
    if(random >= 	4.14500000000000E-15	) S=	-2.17	;
    if(random >= 	6.08300000000000E-15	) S=	-2.16	;
    if(random >= 	8.38800000000000E-15	) S=	-2.15	;
    if(random >= 	1.11280000000000E-14	) S=	-2.14	;
    if(random >= 	1.43820000000000E-14	) S=	-2.13	;
    if(random >= 	1.82450000000000E-14	) S=	-2.12	;
    if(random >= 	2.28290000000000E-14	) S=	-2.11	;
    if(random >= 	2.82650000000000E-14	) S=	-2.1	;
    if(random >= 	3.47080000000000E-14	) S=	-2.09	;
    if(random >= 	4.23380000000000E-14	) S=	-2.08	;
    if(random >= 	5.13710000000000E-14	) S=	-2.07	;
    if(random >= 	6.20610000000000E-14	) S=	-2.06	;
    if(random >= 	7.47010000000000E-14	) S=	-2.05	;
    if(random >= 	8.96410000000000E-14	) S=	-2.04	;
    if(random >= 	1.07291000000000E-13	) S=	-2.03	;
    if(random >= 	1.28121000000000E-13	) S=	-2.02	;
    if(random >= 	1.52701000000000E-13	) S=	-2.01	;
    if(random >= 	1.81691000000000E-13	) S=	-2	;
    if(random >= 	2.15851000000000E-13	) S=	-1.99	;
    if(random >= 	2.56091000000000E-13	) S=	-1.98	;
    if(random >= 	3.03461000000000E-13	) S=	-1.97	;
    if(random >= 	3.59191000000000E-13	) S=	-1.96	;
    if(random >= 	4.24731000000000E-13	) S=	-1.95	;
    if(random >= 	5.01721000000000E-13	) S=	-1.94	;
    if(random >= 	5.92191000000000E-13	) S=	-1.93	;
    if(random >= 	6.98391000000000E-13	) S=	-1.92	;
    if(random >= 	8.22991000000000E-13	) S=	-1.91	;
    if(random >= 	9.69091000000000E-13	) S=	-1.9	;
    if(random >= 	1.14029100000000E-12	) S=	-1.89	;
    if(random >= 	1.34089100000000E-12	) S=	-1.88	;
    if(random >= 	1.57569100000000E-12	) S=	-1.87	;
    if(random >= 	1.85039100000000E-12	) S=	-1.86	;
    if(random >= 	2.17159100000000E-12	) S=	-1.85	;
    if(random >= 	2.54699100000000E-12	) S=	-1.84	;
    if(random >= 	2.98549100000000E-12	) S=	-1.83	;
    if(random >= 	3.49739100000000E-12	) S=	-1.82	;
    if(random >= 	4.09459100000000E-12	) S=	-1.81	;
    if(random >= 	4.79109100000000E-12	) S=	-1.8	;
    if(random >= 	5.60269100000000E-12	) S=	-1.79	;
    if(random >= 	6.54779100000000E-12	) S=	-1.78	;
    if(random >= 	7.64779100000000E-12	) S=	-1.77	;
    if(random >= 	8.92879100000000E-12	) S=	-1.76	;
    if(random >= 	1.04177910000000E-11	) S=	-1.75	;
    if(random >= 	1.21487910000000E-11	) S=	-1.74	;
    if(random >= 	1.41587910000000E-11	) S=	-1.73	;
    if(random >= 	1.64927910000000E-11	) S=	-1.72	;
    if(random >= 	1.92007910000000E-11	) S=	-1.71	;
    if(random >= 	2.23407910000000E-11	) S=	-1.7	;
    if(random >= 	2.59807910000000E-11	) S=	-1.69	;
    if(random >= 	3.01967910000000E-11	) S=	-1.68	;
    if(random >= 	3.50777910000000E-11	) S=	-1.67	;
    if(random >= 	4.07257910000000E-11	) S=	-1.66	;
    if(random >= 	4.72567910000000E-11	) S=	-1.65	;
    if(random >= 	5.48037910000000E-11	) S=	-1.64	;
    if(random >= 	6.35247910000000E-11	) S=	-1.63	;
    if(random >= 	7.35947910000000E-11	) S=	-1.62	;
    if(random >= 	8.52147910000000E-11	) S=	-1.61	;
    if(random >= 	9.86147910000000E-11	) S=	-1.6	;
    if(random >= 	1.14054791000000E-10	) S=	-1.59	;
    if(random >= 	1.31844791000000E-10	) S=	-1.58	;
    if(random >= 	1.52324791000000E-10	) S=	-1.57	;
    if(random >= 	1.75894791000000E-10	) S=	-1.56	;
    if(random >= 	2.03014791000000E-10	) S=	-1.55	;
    if(random >= 	2.34184791000000E-10	) S=	-1.54	;
    if(random >= 	2.69994791000000E-10	) S=	-1.53	;
    if(random >= 	3.11114791000000E-10	) S=	-1.52	;
    if(random >= 	3.58304791000000E-10	) S=	-1.51	;
    if(random >= 	4.12434791000000E-10	) S=	-1.5	;
    if(random >= 	4.74494791000000E-10	) S=	-1.49	;
    if(random >= 	5.45624791000000E-10	) S=	-1.48	;
    if(random >= 	6.27054791000000E-10	) S=	-1.47	;
    if(random >= 	7.20274791000000E-10	) S=	-1.46	;
    if(random >= 	8.26874791000000E-10	) S=	-1.45	;
    if(random >= 	9.48774791000000E-10	) S=	-1.44	;
    if(random >= 	1.08807479100000E-09	) S=	-1.43	;
    if(random >= 	1.24717479100000E-09	) S=	-1.42	;
    if(random >= 	1.42877479100000E-09	) S=	-1.41	;
    if(random >= 	1.63597479100000E-09	) S=	-1.4	;
    if(random >= 	1.87227479100000E-09	) S=	-1.39	;
    if(random >= 	2.14157479100000E-09	) S=	-1.38	;
    if(random >= 	2.44827479100000E-09	) S=	-1.37	;
    if(random >= 	2.79747479100000E-09	) S=	-1.36	;
    if(random >= 	3.19487479100000E-09	) S=	-1.35	;
    if(random >= 	3.64687479100000E-09	) S=	-1.34	;
    if(random >= 	4.16067479100000E-09	) S=	-1.33	;
    if(random >= 	4.74437479100000E-09	) S=	-1.32	;
    if(random >= 	5.40717479100000E-09	) S=	-1.31	;
    if(random >= 	6.15977479100000E-09	) S=	-1.3	;
    if(random >= 	7.01277479100000E-09	) S=	-1.29	;
    if(random >= 	7.98027479100000E-09	) S=	-1.28	;
    if(random >= 	9.07627479100000E-09	) S=	-1.27	;
    if(random >= 	1.03182747910000E-08	) S=	-1.26	;
    if(random >= 	1.17232747910000E-08	) S=	-1.25	;
    if(random >= 	1.33132747910000E-08	) S=	-1.24	;
    if(random >= 	1.51112747910000E-08	) S=	-1.23	;
    if(random >= 	1.71442747910000E-08	) S=	-1.22	;
    if(random >= 	1.94402747910000E-08	) S=	-1.21	;
    if(random >= 	2.20332747910000E-08	) S=	-1.2	;
    if(random >= 	2.49592747910000E-08	) S=	-1.19	;
    if(random >= 	2.82592747910000E-08	) S=	-1.18	;
    if(random >= 	3.19802747910000E-08	) S=	-1.17	;
    if(random >= 	3.61732747910000E-08	) S=	-1.16	;
    if(random >= 	4.08942747910000E-08	) S=	-1.15	;
    if(random >= 	4.62092747910000E-08	) S=	-1.14	;
    if(random >= 	5.21892747910000E-08	) S=	-1.13	;
    if(random >= 	5.89132747910000E-08	) S=	-1.12	;
    if(random >= 	6.64692747910000E-08	) S=	-1.11	;
    if(random >= 	7.49582747910000E-08	) S=	-1.1	;
    if(random >= 	8.44932747910000E-08	) S=	-1.09	;
    if(random >= 	9.51932747910000E-08	) S=	-1.08	;
    if(random >= 	1.07203274791000E-07	) S=	-1.07	;
    if(random >= 	1.20663274791000E-07	) S=	-1.06	;
    if(random >= 	1.35743274791000E-07	) S=	-1.05	;
    if(random >= 	1.52633274791000E-07	) S=	-1.04	;
    if(random >= 	1.71543274791000E-07	) S=	-1.03	;
    if(random >= 	1.92713274791000E-07	) S=	-1.02	;
    if(random >= 	2.16383274791000E-07	) S=	-1.01	;
    if(random >= 	2.42843274791000E-07	) S=	-1	;
    if(random >= 	2.72413274791000E-07	) S=	-0.99	;
    if(random >= 	3.05433274791000E-07	) S=	-0.98	;
    if(random >= 	3.42293274791000E-07	) S=	-0.97	;
    if(random >= 	3.83423274791000E-07	) S=	-0.96	;
    if(random >= 	4.29293274791000E-07	) S=	-0.95	;
    if(random >= 	4.80423274791000E-07	) S=	-0.94	;
    if(random >= 	5.37403274791000E-07	) S=	-0.93	;
    if(random >= 	6.00853274791000E-07	) S=	-0.92	;
    if(random >= 	6.71473274791000E-07	) S=	-0.91	;
    if(random >= 	7.50043274791000E-07	) S=	-0.9	;
    if(random >= 	8.37423274791000E-07	) S=	-0.89	;
    if(random >= 	9.34543274791000E-07	) S=	-0.88	;
    if(random >= 	1.04244327479100E-06	) S=	-0.87	;
    if(random >= 	1.16234327479100E-06	) S=	-0.86	;
    if(random >= 	1.29534327479100E-06	) S=	-0.85	;
    if(random >= 	1.44294327479100E-06	) S=	-0.84	;
    if(random >= 	1.60664327479100E-06	) S=	-0.83	;
    if(random >= 	1.78804327479100E-06	) S=	-0.82	;
    if(random >= 	1.98904327479100E-06	) S=	-0.81	;
    if(random >= 	2.21164327479100E-06	) S=	-0.8	;
    if(random >= 	2.45794327479100E-06	) S=	-0.79	;
    if(random >= 	2.73054327479100E-06	) S=	-0.78	;
    if(random >= 	3.03194327479100E-06	) S=	-0.77	;
    if(random >= 	3.36514327479100E-06	) S=	-0.76	;
    if(random >= 	3.73334327479100E-06	) S=	-0.75	;
    if(random >= 	4.13994327479100E-06	) S=	-0.74	;
    if(random >= 	4.58874327479100E-06	) S=	-0.73	;
    if(random >= 	5.08394327479100E-06	) S=	-0.72	;
    if(random >= 	5.63014327479100E-06	) S=	-0.71	;
    if(random >= 	6.23224327479100E-06	) S=	-0.7	;
    if(random >= 	6.89574327479100E-06	) S=	-0.69	;
    if(random >= 	7.62664327479100E-06	) S=	-0.68	;
    if(random >= 	8.43124327479100E-06	) S=	-0.67	;
    if(random >= 	9.31684327479100E-06	) S=	-0.66	;
    if(random >= 	1.02909432747910E-05	) S=	-0.65	;
    if(random >= 	1.13619432747910E-05	) S=	-0.64	;
    if(random >= 	1.25389432747910E-05	) S=	-0.63	;
    if(random >= 	1.38319432747910E-05	) S=	-0.62	;
    if(random >= 	1.52519432747910E-05	) S=	-0.61	;
    if(random >= 	1.68109432747910E-05	) S=	-0.6	;
    if(random >= 	1.85219432747910E-05	) S=	-0.59	;
    if(random >= 	2.03979432747910E-05	) S=	-0.58	;
    if(random >= 	2.24549432747910E-05	) S=	-0.57	;
    if(random >= 	2.47089432747910E-05	) S=	-0.56	;
    if(random >= 	2.71779432747910E-05	) S=	-0.55	;
    if(random >= 	2.98829432747910E-05	) S=	-0.54	;
    if(random >= 	3.28429432747910E-05	) S=	-0.53	;
    if(random >= 	3.60819432747910E-05	) S=	-0.52	;
    if(random >= 	3.96249432747910E-05	) S=	-0.51	;
    if(random >= 	4.34989432747910E-05	) S=	-0.5	;
    if(random >= 	4.77329432747910E-05	) S=	-0.49	;
    if(random >= 	5.23599432747910E-05	) S=	-0.48	;
    if(random >= 	5.74129432747910E-05	) S=	-0.47	;
    if(random >= 	6.29299432747910E-05	) S=	-0.46	;
    if(random >= 	6.89519432747910E-05	) S=	-0.45	;
    if(random >= 	7.55229432747910E-05	) S=	-0.44	;
    if(random >= 	8.26909432747910E-05	) S=	-0.43	;
    if(random >= 	9.05069432747910E-05	) S=	-0.42	;
    if(random >= 	9.90269432747910E-05	) S=	-0.41	;
    if(random >= 	1.08310943274791E-04	) S=	-0.4	;
    if(random >= 	1.18430943274791E-04	) S=	-0.39	;
    if(random >= 	1.29450943274791E-04	) S=	-0.38	;
    if(random >= 	1.41450943274791E-04	) S=	-0.37	;
    if(random >= 	1.54510943274791E-04	) S=	-0.36	;
    if(random >= 	1.68720943274791E-04	) S=	-0.35	;
    if(random >= 	1.84180943274791E-04	) S=	-0.34	;
    if(random >= 	2.01000943274791E-04	) S=	-0.33	;
    if(random >= 	2.19290943274791E-04	) S=	-0.32	;
    if(random >= 	2.39180943274791E-04	) S=	-0.31	;
    if(random >= 	2.60790943274791E-04	) S=	-0.3	;
    if(random >= 	2.84280943274791E-04	) S=	-0.29	;
    if(random >= 	3.09800943274791E-04	) S=	-0.28	;
    if(random >= 	3.37530943274791E-04	) S=	-0.27	;
    if(random >= 	3.67650943274791E-04	) S=	-0.26	;
    if(random >= 	4.00350943274791E-04	) S=	-0.25	;
    if(random >= 	4.35860943274791E-04	) S=	-0.24	;
    if(random >= 	4.74400943274791E-04	) S=	-0.23	;
    if(random >= 	5.16240943274791E-04	) S=	-0.22	;
    if(random >= 	5.61650943274791E-04	) S=	-0.21	;
    if(random >= 	6.10920943274791E-04	) S=	-0.2	;
    if(random >= 	6.64360943274791E-04	) S=	-0.19	;
    if(random >= 	7.22340943274791E-04	) S=	-0.18	;
    if(random >= 	7.85220943274791E-04	) S=	-0.17	;
    if(random >= 	8.53410943274791E-04	) S=	-0.16	;
    if(random >= 	9.27340943274791E-04	) S=	-0.15	;
    if(random >= 	1.00748094327479E-03	) S=	-0.14	;
    if(random >= 	1.09435094327479E-03	) S=	-0.13	;
    if(random >= 	1.18849094327479E-03	) S=	-0.12	;
    if(random >= 	1.29049094327479E-03	) S=	-0.11	;
    if(random >= 	1.40099094327479E-03	) S=	-0.1	;
    if(random >= 	1.52069094327479E-03	) S=	-0.09	;
    if(random >= 	1.65029094327479E-03	) S=	-0.08	;
    if(random >= 	1.79059094327479E-03	) S=	-0.07	;
    if(random >= 	1.94249094327479E-03	) S=	-0.06	;
    if(random >= 	2.10689094327479E-03	) S=	-0.05	;
    if(random >= 	2.28479094327479E-03	) S=	-0.04	;
    if(random >= 	2.47719094327479E-03	) S=	-0.03	;
    if(random >= 	2.68529094327479E-03	) S=	-0.02	;
    if(random >= 	2.91019094327479E-03	) S=	-0.01	;
    if(random >= 	3.15329094327479E-03	) S=	0	;
    if(random >= 	3.41599094327479E-03	) S=	0.01	;
    if(random >= 	3.69969094327479E-03	) S=	0.02	;
    if(random >= 	4.00599094327479E-03	) S=	0.03	;
    if(random >= 	4.33659094327479E-03	) S=	0.04	;
    if(random >= 	4.69319094327479E-03	) S=	0.05	;
    if(random >= 	5.07779094327479E-03	) S=	0.06	;
    if(random >= 	5.49239094327479E-03	) S=	0.07	;
    if(random >= 	5.93909094327479E-03	) S=	0.08	;
    if(random >= 	6.42009094327479E-03	) S=	0.09	;
    if(random >= 	6.93779094327479E-03	) S=	0.1	;
    if(random >= 	7.49479094327479E-03	) S=	0.11	;
    if(random >= 	8.09379094327479E-03	) S=	0.12	;
    if(random >= 	8.73739094327479E-03	) S=	0.13	;
    if(random >= 	9.42869094327479E-03	) S=	0.14	;
    if(random >= 	1.01706909432748E-02	) S=	0.15	;
    if(random >= 	1.09666909432748E-02	) S=	0.16	;
    if(random >= 	1.18199909432748E-02	) S=	0.17	;
    if(random >= 	1.27342909432748E-02	) S=	0.18	;
    if(random >= 	1.37131909432748E-02	) S=	0.19	;
    if(random >= 	1.47601909432748E-02	) S=	0.2	;
    if(random >= 	1.58801909432748E-02	) S=	0.21	;
    if(random >= 	1.70771909432748E-02	) S=	0.22	;
    if(random >= 	1.83551909432748E-02	) S=	0.23	;
    if(random >= 	1.97191909432748E-02	) S=	0.24	;
    if(random >= 	2.11731909432748E-02	) S=	0.25	;
    if(random >= 	2.27221909432748E-02	) S=	0.26	;
    if(random >= 	2.43721909432748E-02	) S=	0.27	;
    if(random >= 	2.61281909432748E-02	) S=	0.28	;
    if(random >= 	2.79951909432748E-02	) S=	0.29	;
    if(random >= 	2.99801909432748E-02	) S=	0.3	;
    if(random >= 	3.20881909432748E-02	) S=	0.31	;
    if(random >= 	3.43251909432748E-02	) S=	0.32	;
    if(random >= 	3.66981909432748E-02	) S=	0.33	;
    if(random >= 	3.92141909432748E-02	) S=	0.34	;
    if(random >= 	4.18801909432748E-02	) S=	0.35	;
    if(random >= 	4.47031909432748E-02	) S=	0.36	;
    if(random >= 	4.76901909432748E-02	) S=	0.37	;
    if(random >= 	5.08491909432748E-02	) S=	0.38	;
    if(random >= 	5.41881909432748E-02	) S=	0.39	;
    if(random >= 	5.77151909432748E-02	) S=	0.4	;
    if(random >= 	6.14381909432748E-02	) S=	0.41	;
    if(random >= 	6.53661909432748E-02	) S=	0.42	;
    if(random >= 	6.95071909432748E-02	) S=	0.43	;
    if(random >= 	7.38691909432748E-02	) S=	0.44	;
    if(random >= 	7.84611909432748E-02	) S=	0.45	;
    if(random >= 	8.32921909432748E-02	) S=	0.46	;
    if(random >= 	8.83701909432748E-02	) S=	0.47	;
    if(random >= 	9.37031909432748E-02	) S=	0.48	;
    if(random >= 	9.92981909432748E-02	) S=	0.49	;
    if(random >= 	1.05164190943275E-01	) S=	0.5	;
    if(random >= 	1.11307190943275E-01	) S=	0.51	;
    if(random >= 	1.17734190943275E-01	) S=	0.52	;
    if(random >= 	1.24451190943275E-01	) S=	0.53	;
    if(random >= 	1.31463190943275E-01	) S=	0.54	;
    if(random >= 	1.38776190943275E-01	) S=	0.55	;
    if(random >= 	1.46393190943275E-01	) S=	0.56	;
    if(random >= 	1.54317190943275E-01	) S=	0.57	;
    if(random >= 	1.62552190943275E-01	) S=	0.58	;
    if(random >= 	1.71100190943275E-01	) S=	0.59	;
    if(random >= 	1.79961190943275E-01	) S=	0.6	;
    if(random >= 	1.89137190943275E-01	) S=	0.61	;
    if(random >= 	1.98627190943275E-01	) S=	0.62	;
    if(random >= 	2.08433190943275E-01	) S=	0.63	;
    if(random >= 	2.18433190943275E-01	) S=	0.64	;
    if(random >= 	2.28433190943275E-01	) S=	0.65	;
    if(random >= 	2.39433190943275E-01	) S=	0.66	;
    if(random >= 	2.50433190943275E-01	) S=	0.67	;
    if(random >= 	2.61433190943275E-01	) S=	0.68	;
    if(random >= 	2.73433190943275E-01	) S=	0.69	;
    if(random >= 	2.85433190943275E-01	) S=	0.7	;
    if(random >= 	2.97433190943275E-01	) S=	0.71	;
    if(random >= 	3.09433190943275E-01	) S=	0.72	;
    if(random >= 	3.21433190943275E-01	) S=	0.73	;
    if(random >= 	3.34433190943275E-01	) S=	0.74	;
    if(random >= 	3.47433190943275E-01	) S=	0.75	;
    if(random >= 	3.60433190943275E-01	) S=	0.76	;
    if(random >= 	3.73433190943275E-01	) S=	0.77	;
    if(random >= 	3.86433190943275E-01	) S=	0.78	;
    if(random >= 	3.99433190943275E-01	) S=	0.79	;
    if(random >= 	4.12433190943275E-01	) S=	0.8	;
    if(random >= 	4.25433190943275E-01	) S=	0.81	;
    if(random >= 	4.38433190943275E-01	) S=	0.82	;
    if(random >= 	4.51433190943275E-01	) S=	0.83	;
    if(random >= 	4.64433190943275E-01	) S=	0.84	;
    if(random >= 	4.77433190943275E-01	) S=	0.85	;
    if(random >= 	4.90433190943275E-01	) S=	0.86	;
    if(random >= 	5.03433190943275E-01	) S=	0.87	;
    if(random >= 	5.16433190943275E-01	) S=	0.88	;
    if(random >= 	5.29433190943275E-01	) S=	0.89	;
    if(random >= 	5.42433190943275E-01	) S=	0.9	;
    if(random >= 	5.54433190943275E-01	) S=	0.91	;
    if(random >= 	5.66433190943275E-01	) S=	0.92	;
    if(random >= 	5.78433190943275E-01	) S=	0.93	;
    if(random >= 	5.90433190943275E-01	) S=	0.94	;
    if(random >= 	6.02433190943275E-01	) S=	0.95	;
    if(random >= 	6.14433190943275E-01	) S=	0.96	;
    if(random >= 	6.26433190943275E-01	) S=	0.97	;
    if(random >= 	6.37433190943275E-01	) S=	0.98	;
    if(random >= 	6.48433190943275E-01	) S=	0.99	;
    if(random >= 	6.59433190943275E-01	) S=	1	;
    if(random >= 	6.70433190943275E-01	) S=	1.01	;
    if(random >= 	6.80433190943275E-01	) S=	1.02	;
    if(random >= 	6.90290190943275E-01	) S=	1.03	;
    if(random >= 	6.99917190943275E-01	) S=	1.04	;
    if(random >= 	7.09314190943275E-01	) S=	1.05	;
    if(random >= 	7.18482190943275E-01	) S=	1.06	;
    if(random >= 	7.27422190943275E-01	) S=	1.07	;
    if(random >= 	7.36136190943275E-01	) S=	1.08	;
    if(random >= 	7.44626190943275E-01	) S=	1.09	;
    if(random >= 	7.52894190943275E-01	) S=	1.1	;
    if(random >= 	7.60943190943275E-01	) S=	1.11	;
    if(random >= 	7.68775190943275E-01	) S=	1.12	;
    if(random >= 	7.76394190943275E-01	) S=	1.13	;
    if(random >= 	7.83804190943275E-01	) S=	1.14	;
    if(random >= 	7.91007190943275E-01	) S=	1.15	;
    if(random >= 	7.98008190943275E-01	) S=	1.16	;
    if(random >= 	8.04811190943275E-01	) S=	1.17	;
    if(random >= 	8.11420190943275E-01	) S=	1.18	;
    if(random >= 	8.17838190943275E-01	) S=	1.19	;
    if(random >= 	8.24070190943275E-01	) S=	1.2	;
    if(random >= 	8.30119190943275E-01	) S=	1.21	;
    if(random >= 	8.35990190943275E-01	) S=	1.22	;
    if(random >= 	8.41686190943275E-01	) S=	1.23	;
    if(random >= 	8.47211190943275E-01	) S=	1.24	;
    if(random >= 	8.52570190943275E-01	) S=	1.25	;
    if(random >= 	8.57766190943275E-01	) S=	1.26	;
    if(random >= 	8.62802190943275E-01	) S=	1.27	;
    if(random >= 	8.67683190943275E-01	) S=	1.28	;
    if(random >= 	8.72412190943275E-01	) S=	1.29	;
    if(random >= 	8.76993190943275E-01	) S=	1.3	;
    if(random >= 	8.81429190943275E-01	) S=	1.31	;
    if(random >= 	8.85723190943275E-01	) S=	1.32	;
    if(random >= 	8.89879190943275E-01	) S=	1.33	;
    if(random >= 	8.93901190943275E-01	) S=	1.34	;
    if(random >= 	8.97792190943275E-01	) S=	1.35	;
    if(random >= 	9.01555190943275E-01	) S=	1.36	;
    if(random >= 	9.05194190943275E-01	) S=	1.37	;
    if(random >= 	9.08711190943275E-01	) S=	1.38	;
    if(random >= 	9.12110190943275E-01	) S=	1.39	;
    if(random >= 	9.15394190943275E-01	) S=	1.4	;
    if(random >= 	9.18567190943275E-01	) S=	1.41	;
    if(random >= 	9.21632190943275E-01	) S=	1.42	;
    if(random >= 	9.24592190943275E-01	) S=	1.43	;
    if(random >= 	9.27449190943275E-01	) S=	1.44	;
    if(random >= 	9.30207190943275E-01	) S=	1.45	;
    if(random >= 	9.32869190943275E-01	) S=	1.46	;
    if(random >= 	9.35438190943275E-01	) S=	1.47	;
    if(random >= 	9.37916190943275E-01	) S=	1.48	;
    if(random >= 	9.40307190943275E-01	) S=	1.49	;
    if(random >= 	9.42613190943275E-01	) S=	1.5	;
    if(random >= 	9.44837190943275E-01	) S=	1.51	;
    if(random >= 	9.46981190943275E-01	) S=	1.52	;
    if(random >= 	9.49048190943275E-01	) S=	1.53	;
    if(random >= 	9.51041190943275E-01	) S=	1.54	;
    if(random >= 	9.52962190943275E-01	) S=	1.55	;
    if(random >= 	9.54813190943275E-01	) S=	1.56	;
    if(random >= 	9.56597190943275E-01	) S=	1.57	;
    if(random >= 	9.58316190943275E-01	) S=	1.58	;
    if(random >= 	9.59972190943275E-01	) S=	1.59	;
    if(random >= 	9.61567190943275E-01	) S=	1.6	;
    if(random >= 	9.63104190943275E-01	) S=	1.61	;
    if(random >= 	9.64584190943275E-01	) S=	1.62	;
    if(random >= 	9.66010190943275E-01	) S=	1.63	;
    if(random >= 	9.67383190943275E-01	) S=	1.64	;
    if(random >= 	9.68705190943275E-01	) S=	1.65	;
    if(random >= 	9.69978190943275E-01	) S=	1.66	;
    if(random >= 	9.71203190943275E-01	) S=	1.67	;
    if(random >= 	9.72382190943275E-01	) S=	1.68	;
    if(random >= 	9.73517190943275E-01	) S=	1.69	;
    if(random >= 	9.74609190943275E-01	) S=	1.7	;
    if(random >= 	9.75660190943275E-01	) S=	1.71	;
    if(random >= 	9.76671190943275E-01	) S=	1.72	;
    if(random >= 	9.77643790943275E-01	) S=	1.73	;
    if(random >= 	9.78579190943276E-01	) S=	1.74	;
    if(random >= 	9.79478790943276E-01	) S=	1.75	;
    if(random >= 	9.80343690943275E-01	) S=	1.76	;
    if(random >= 	9.81175290943275E-01	) S=	1.77	;
    if(random >= 	9.81974590943275E-01	) S=	1.78	;
    if(random >= 	9.82742690943275E-01	) S=	1.79	;
    if(random >= 	9.83480790943275E-01	) S=	1.8	;
    if(random >= 	9.84189790943275E-01	) S=	1.81	;
    if(random >= 	9.84870790943275E-01	) S=	1.82	;
    if(random >= 	9.85524590943275E-01	) S=	1.83	;
    if(random >= 	9.86152290943276E-01	) S=	1.84	;
    if(random >= 	9.86754790943275E-01	) S=	1.85	;
    if(random >= 	9.87332890943275E-01	) S=	1.86	;
    if(random >= 	9.87887490943275E-01	) S=	1.87	;
    if(random >= 	9.88419290943275E-01	) S=	1.88	;
    if(random >= 	9.88929290943275E-01	) S=	1.89	;
    if(random >= 	9.89418090943275E-01	) S=	1.9	;
    if(random >= 	9.89886490943275E-01	) S=	1.91	;
    if(random >= 	9.90335190943275E-01	) S=	1.92	;
    if(random >= 	9.90764890943275E-01	) S=	1.93	;
    if(random >= 	9.91176190943275E-01	) S=	1.94	;
    if(random >= 	9.91569790943275E-01	) S=	1.95	;
    if(random >= 	9.91946390943275E-01	) S=	1.96	;
    if(random >= 	9.92306590943275E-01	) S=	1.97	;
    if(random >= 	9.92650890943275E-01	) S=	1.98	;
    if(random >= 	9.92979990943275E-01	) S=	1.99	;
    if(random >= 	9.93294390943275E-01	) S=	2	;
    if(random >= 	9.93594690943275E-01	) S=	2.01	;
    if(random >= 	9.93881290943275E-01	) S=	2.02	;
    if(random >= 	9.94154790943275E-01	) S=	2.03	;
    if(random >= 	9.94415690943275E-01	) S=	2.04	;
    if(random >= 	9.94664390943275E-01	) S=	2.05	;
    if(random >= 	9.94901490943275E-01	) S=	2.06	;
    if(random >= 	9.95127290943275E-01	) S=	2.07	;
    if(random >= 	9.95342390943275E-01	) S=	2.08	;
    if(random >= 	9.95547090943275E-01	) S=	2.09	;
    if(random >= 	9.95741890943275E-01	) S=	2.1	;
    if(random >= 	9.95927190943276E-01	) S=	2.11	;
    if(random >= 	9.96103290943276E-01	) S=	2.12	;
    if(random >= 	9.96270590943276E-01	) S=	2.13	;
    if(random >= 	9.96429490943275E-01	) S=	2.14	;
    if(random >= 	9.96580290943275E-01	) S=	2.15	;
    if(random >= 	9.96723390943275E-01	) S=	2.16	;
    if(random >= 	9.96859190943275E-01	) S=	2.17	;
    if(random >= 	9.96987890943275E-01	) S=	2.18	;
    if(random >= 	9.97109890943275E-01	) S=	2.19	;
    if(random >= 	9.97225390943275E-01	) S=	2.2	;
    if(random >= 	9.97334690943275E-01	) S=	2.21	;
    if(random >= 	9.97438090943275E-01	) S=	2.22	;
    if(random >= 	9.97535900943275E-01	) S=	2.23	;
    if(random >= 	9.97628380943275E-01	) S=	2.24	;
    if(random >= 	9.97715750943275E-01	) S=	2.25	;
    if(random >= 	9.97798230943275E-01	) S=	2.26	;
    if(random >= 	9.97876060943275E-01	) S=	2.27	;
    if(random >= 	9.97949480943275E-01	) S=	2.28	;
    if(random >= 	9.98018690943275E-01	) S=	2.29	;
    if(random >= 	9.98083910943275E-01	) S=	2.3	;
    if(random >= 	9.98145330943275E-01	) S=	2.31	;
    if(random >= 	9.98203150943275E-01	) S=	2.32	;
    if(random >= 	9.98257550943275E-01	) S=	2.33	;
    if(random >= 	9.98308700943275E-01	) S=	2.34	;
    if(random >= 	9.98356770943275E-01	) S=	2.35	;
    if(random >= 	9.98401930943275E-01	) S=	2.36	;
    if(random >= 	9.98444320943275E-01	) S=	2.37	;
    if(random >= 	9.98484100943275E-01	) S=	2.38	;
    if(random >= 	9.98521400943275E-01	) S=	2.39	;
    if(random >= 	9.98556360943275E-01	) S=	2.4	;
    if(random >= 	9.98589110943275E-01	) S=	2.41	;
    if(random >= 	9.98619770943275E-01	) S=	2.42	;
    if(random >= 	9.98648460943275E-01	) S=	2.43	;
    if(random >= 	9.98675290943275E-01	) S=	2.44	;
    if(random >= 	9.98700370943275E-01	) S=	2.45	;
    if(random >= 	9.98723800943275E-01	) S=	2.46	;
    if(random >= 	9.98745680943275E-01	) S=	2.47	;
    if(random >= 	9.98766090943275E-01	) S=	2.48	;
    if(random >= 	9.98785130943275E-01	) S=	2.49	;
    if(random >= 	9.98802870943275E-01	) S=	2.5	;
    if(random >= 	9.98819400943275E-01	) S=	2.51	;
    if(random >= 	9.98834790943275E-01	) S=	2.52	;
    if(random >= 	9.98849110943275E-01	) S=	2.53	;
    if(random >= 	9.98862430943275E-01	) S=	2.54	;
    if(random >= 	9.98874810943275E-01	) S=	2.55	;
    if(random >= 	9.98886310943275E-01	) S=	2.56	;
    if(random >= 	9.98896980943275E-01	) S=	2.57	;
    if(random >= 	9.98906882943275E-01	) S=	2.58	;
    if(random >= 	9.98916066943275E-01	) S=	2.59	;
    if(random >= 	9.98924577943275E-01	) S=	2.6	;
    if(random >= 	9.98932461943275E-01	) S=	2.61	;
    if(random >= 	9.98939757943275E-01	) S=	2.62	;
    if(random >= 	9.98946509943275E-01	) S=	2.63	;
    if(random >= 	9.98952752943275E-01	) S=	2.64	;
    if(random >= 	9.98958521943275E-01	) S=	2.65	;
    if(random >= 	9.98963849943275E-01	) S=	2.66	;
    if(random >= 	9.98968766943275E-01	) S=	2.67	;
    if(random >= 	9.98973302943275E-01	) S=	2.68	;
    if(random >= 	9.98977484943275E-01	) S=	2.69	;
    if(random >= 	9.98981337943275E-01	) S=	2.7	;
    if(random >= 	9.98984885943276E-01	) S=	2.71	;
    if(random >= 	9.98988150943276E-01	) S=	2.72	;
    if(random >= 	9.98991153943276E-01	) S=	2.73	;
    if(random >= 	9.98993914943275E-01	) S=	2.74	;
    if(random >= 	9.98996450943275E-01	) S=	2.75	;
    if(random >= 	9.98998779943275E-01	) S=	2.76	;
    if(random >= 	9.99000916943275E-01	) S=	2.77	;
    if(random >= 	9.99002875943275E-01	) S=	2.78	;
    if(random >= 	9.99004671943275E-01	) S=	2.79	;
    if(random >= 	9.99006316943275E-01	) S=	2.8	;
    if(random >= 	9.99007821943275E-01	) S=	2.81	;
    if(random >= 	9.99009198943275E-01	) S=	2.82	;
    if(random >= 	9.99010457943275E-01	) S=	2.83	;
    if(random >= 	9.99011608943275E-01	) S=	2.84	;
    if(random >= 	9.99012659943275E-01	) S=	2.85	;
    if(random >= 	9.99013618643276E-01	) S=	2.86	;
    if(random >= 	9.99014493543276E-01	) S=	2.87	;
    if(random >= 	9.99015291043276E-01	) S=	2.88	;
    if(random >= 	9.99016017443276E-01	) S=	2.89	;
    if(random >= 	9.99016678743276E-01	) S=	2.9	;
    if(random >= 	9.99017280443276E-01	) S=	2.91	;
    if(random >= 	9.99017827543276E-01	) S=	2.92	;
    if(random >= 	9.99018324743276E-01	) S=	2.93	;
    if(random >= 	9.99018776343276E-01	) S=	2.94	;
    if(random >= 	9.99019186243276E-01	) S=	2.95	;
    if(random >= 	9.99019558143276E-01	) S=	2.96	;
    if(random >= 	9.99019895343276E-01	) S=	2.97	;
    if(random >= 	9.99020200843276E-01	) S=	2.98	;
    if(random >= 	9.99020477443276E-01	) S=	2.99	;
    if(random >= 	9.99020727743276E-01	) S=	3	;
    if(random >= 	9.99020954143276E-01	) S=	3.01	;
    if(random >= 	9.99021158743276E-01	) S=	3.02	;
    if(random >= 	9.99021343543276E-01	) S=	3.03	;
    if(random >= 	9.99021510443276E-01	) S=	3.04	;
    if(random >= 	9.99021660943276E-01	) S=	3.05	;
    if(random >= 	9.99021796743276E-01	) S=	3.06	;
    if(random >= 	9.99021919043276E-01	) S=	3.07	;
    if(random >= 	9.99022029243276E-01	) S=	3.08	;
    if(random >= 	9.99022128353276E-01	) S=	3.09	;
    if(random >= 	9.99022217543276E-01	) S=	3.1	;
    if(random >= 	9.99022297693276E-01	) S=	3.11	;
    if(random >= 	9.99022369693276E-01	) S=	3.12	;
    if(random >= 	9.99022434343276E-01	) S=	3.13	;
    if(random >= 	9.99022492353276E-01	) S=	3.14	;
    if(random >= 	9.99022544373276E-01	) S=	3.15	;
    if(random >= 	9.99022590993276E-01	) S=	3.16	;
    if(random >= 	9.99022632743276E-01	) S=	3.17	;
    if(random >= 	9.99022670113276E-01	) S=	3.18	;
    if(random >= 	9.99022703543276E-01	) S=	3.19	;
    if(random >= 	9.99022733423276E-01	) S=	3.2	;
    if(random >= 	9.99022760123276E-01	) S=	3.21	;
    if(random >= 	9.99022783963276E-01	) S=	3.22	;
    if(random >= 	9.99022805233276E-01	) S=	3.23	;
    if(random >= 	9.99022824203276E-01	) S=	3.24	;
    if(random >= 	9.99022841113276E-01	) S=	3.25	;
    if(random >= 	9.99022856173276E-01	) S=	3.26	;
    if(random >= 	9.99022869573276E-01	) S=	3.27	;
    if(random >= 	9.99022881493276E-01	) S=	3.28	;
    if(random >= 	9.99022892093276E-01	) S=	3.29	;
    if(random >= 	9.99022901510276E-01	) S=	3.3	;
    if(random >= 	9.99022909875276E-01	) S=	3.31	;
    if(random >= 	9.99022917295275E-01	) S=	3.32	;
    if(random >= 	9.99022923877275E-01	) S=	3.33	;
    if(random >= 	9.99022929710275E-01	) S=	3.34	;
    if(random >= 	9.99022934877275E-01	) S=	3.35	;
    if(random >= 	9.99022939451275E-01	) S=	3.36	;
    if(random >= 	9.99022943498275E-01	) S=	3.37	;
    if(random >= 	9.99022947075275E-01	) S=	3.38	;
    if(random >= 	9.99022950236275E-01	) S=	3.39	;
    if(random >= 	9.99022953027275E-01	) S=	3.4	;
    if(random >= 	9.99022955490275E-01	) S=	3.41	;
    if(random >= 	9.99022957662275E-01	) S=	3.42	;
    if(random >= 	9.99022959577275E-01	) S=	3.43	;
    if(random >= 	9.99022961263275E-01	) S=	3.44	;
    if(random >= 	9.99022962748276E-01	) S=	3.45	;
    if(random >= 	9.99022964054275E-01	) S=	3.46	;
    if(random >= 	9.99022965203275E-01	) S=	3.47	;
    if(random >= 	9.99022966212275E-01	) S=	3.48	;
    if(random >= 	9.99022967098375E-01	) S=	3.49	;
    if(random >= 	9.99022967876176E-01	) S=	3.5	;
    if(random >= 	9.99022968558176E-01	) S=	3.51	;
    if(random >= 	9.99022969155876E-01	) S=	3.52	;
    if(random >= 	9.99022969679376E-01	) S=	3.53	;
    if(random >= 	9.99022970137675E-01	) S=	3.54	;
    if(random >= 	9.99022970538676E-01	) S=	3.55	;
    if(random >= 	9.99022970889275E-01	) S=	3.56	;
    if(random >= 	9.99022971195575E-01	) S=	3.57	;
    if(random >= 	9.99022971463076E-01	) S=	3.58	;
    if(random >= 	9.99022971696576E-01	) S=	3.59	;
    if(random >= 	9.99022971900176E-01	) S=	3.6	;
    if(random >= 	9.99022972077676E-01	) S=	3.61	;
    if(random >= 	9.99022972232276E-01	) S=	3.62	;
    if(random >= 	9.99022972366876E-01	) S=	3.63	;
    if(random >= 	9.99022972483976E-01	) S=	3.64	;
    if(random >= 	9.99022972585776E-01	) S=	3.65	;
    if(random >= 	9.99022972674226E-01	) S=	3.66	;
    if(random >= 	9.99022972751036E-01	) S=	3.67	;
    if(random >= 	9.99022972817666E-01	) S=	3.68	;
    if(random >= 	9.99022972875476E-01	) S=	3.69	;
    if(random >= 	9.99022972925576E-01	) S=	3.7	;
    if(random >= 	9.99022972968976E-01	) S=	3.71	;
    if(random >= 	9.99022973006546E-01	) S=	3.72	;
    if(random >= 	9.99022973039046E-01	) S=	3.73	;
    if(random >= 	9.99022973067145E-01	) S=	3.74	;
    if(random >= 	9.99022973091426E-01	) S=	3.75	;
    if(random >= 	9.99022973112396E-01	) S=	3.76	;
    if(random >= 	9.99022973130485E-01	) S=	3.77	;
    if(random >= 	9.99022973146095E-01	) S=	3.78	;
    if(random >= 	9.99022973159545E-01	) S=	3.79	;
    if(random >= 	9.99022973171135E-01	) S=	3.8	;
    if(random >= 	9.99022973181110E-01	) S=	3.81	;
    if(random >= 	9.99022973189689E-01	) S=	3.82	;
    if(random >= 	9.99022973197068E-01	) S=	3.83	;
    if(random >= 	9.99022973203408E-01	) S=	3.84	;
    if(random >= 	9.99022973208852E-01	) S=	3.85	;
    if(random >= 	9.99022973213525E-01	) S=	3.86	;
    if(random >= 	9.99022973217532E-01	) S=	3.87	;
    if(random >= 	9.99022973220967E-01	) S=	3.88	;
    if(random >= 	9.99022973223909E-01	) S=	3.89	;
    if(random >= 	9.99022973226428E-01	) S=	3.9	;
    if(random >= 	9.99022973228583E-01	) S=	3.91	;
    if(random >= 	9.99022973230425E-01	) S=	3.92	;
    if(random >= 	9.99022973231999E-01	) S=	3.93	;
    if(random >= 	9.99022973233343E-01	) S=	3.94	;
    if(random >= 	9.99022973234490E-01	) S=	3.95	;
    if(random >= 	9.99022973235468E-01	) S=	3.96	;
    if(random >= 	9.99022973236301E-01	) S=	3.97	;
    if(random >= 	9.99022973237011E-01	) S=	3.98	;
    if(random >= 	9.99022973237615E-01	) S=	3.99	;
    if(random >= 	9.99022973238129E-01	) S=	4	;
    if(random >= 	9.99022973238567E-01	) S=	4.01	;
    if(random >= 	9.99022973238938E-01	) S=	4.02	;
    if(random >= 	9.99022973239254E-01	) S=	4.03	;
    if(random >= 	9.99022973239521E-01	) S=	4.04	;
    if(random >= 	9.99022973239749E-01	) S=	4.05	;
    if(random >= 	9.99022973239941E-01	) S=	4.06	;
    if(random >= 	9.99022973240104E-01	) S=	4.07	;
    if(random >= 	9.99022973240242E-01	) S=	4.08	;
    if(random >= 	9.99022973240359E-01	) S=	4.09	;
    if(random >= 	9.99022973240457E-01	) S=	4.1	;
    if(random >= 	9.99022973240541E-01	) S=	4.11	;
    if(random >= 	9.99022973240611E-01	) S=	4.12	;
    if(random >= 	9.99022973240670E-01	) S=	4.13	;
    if(random >= 	9.99022973240721E-01	) S=	4.14	;
    if(random >= 	9.99022973240763E-01	) S=	4.15	;
    if(random >= 	9.99022973240798E-01	) S=	4.16	;
    if(random >= 	9.99022973240828E-01	) S=	4.17	;
    if(random >= 	9.99022973240853E-01	) S=	4.18	;
    if(random >= 	9.99022973240875E-01	) S=	4.19	;
    if(random >= 	9.99022973240892E-01	) S=	4.2	;
    if(random >= 	9.99022973240907E-01	) S=	4.21	;
    if(random >= 	9.99022973240920E-01	) S=	4.22	;
    if(random >= 	9.99022973240930E-01	) S=	4.23	;
    if(random >= 	9.99022973240939E-01	) S=	4.24	;
    if(random >= 	9.99022973240947E-01	) S=	4.25	;
    if(random >= 	9.99022973240953E-01	) S=	4.26	;
    if(random >= 	9.99022973240958E-01	) S=	4.27	;
    if(random >= 	9.99022973240962E-01	) S=	4.28	;
    if(random >= 	9.99022973240966E-01	) S=	4.29	;
    if(random >= 	9.99022973240969E-01	) S=	4.3	;
    if(random >= 	9.99022973240972E-01	) S=	4.31	;
    if(random >= 	9.99022973240974E-01	) S=	4.32	;
    if(random >= 	9.99022973240975E-01	) S=	4.33	;
    if(random >= 	9.99022973240977E-01	) S=	4.34	;
    if(random >= 	9.99022973240978E-01	) S=	4.35	;
    if(random >= 	9.99022973240979E-01	) S=	4.36	;
    
    //cout << random << '\t' << S << endl;
    
    lambda_palaeosens = 1.0/S;
    
    return lambda_palaeosens;
    
}


void convertI_em(int t_steps_year) //converts emission scenarios when more than 1 time step per year
{
    //First set a dummy array equal to I_em;
    double I_em_dummy[tmax];
    for (int i = 0; i<tmax; i++)
    {
        I_em_dummy[i] = I_em[i];
    }
    //Now re-set original Iem array by linearising between years
    
    int years=0;
    int count = 0;
    double annual_em;
    for(int i=0; i<tmax; i++)
    {
        annual_em = I_em_dummy[years+1]-I_em_dummy[years]; //set annual emission rate
        
        I_em[i] = I_em_dummy[years] + double(count)*annual_em/double(t_steps_year);
        
        count++;
        
        if(count == (t_steps_year))
        {
            count=0;
            years++;
        }
        
        
    }
    
    
}

void convertCO2(int t_steps_year) //converts emission scenarios when more than 1 time step per year
{
    //First set a dummy array equal to CO2;
    double CO2_dummy[tmax];
    for (int i = 0; i<tmax; i++)
    {
        CO2_dummy[i] = CO2[i];
    }
    //Now re-set original CO2 array by linearising between years
    
    int years=0;
    int count = 0;
    double annual_dCO2;
    for(int i=0; i<tmax; i++)
    {
        annual_dCO2 = CO2_dummy[years+1]-CO2_dummy[years]; //set annual emission rate
        
        CO2[i] = CO2_dummy[years] + double(count)*annual_dCO2/double(t_steps_year);
        
        count++;
        
        if(count == (t_steps_year))
        {
            count=0;
            years++;
        }
        
        
    }
    
    
}

void convertRnonCO2(int t_steps_year) //converts RnonCO2 when mpre than 1 time step per year
{
    double R_dummy[tmax];
    for (int i = 0; i<tmax; i++)
    {
        R_dummy[i] = RnonCO2[i];
    }
    //Now re-set original CO2 array by linearising between years
    
    int years=0;
    int count = 0;
    double annual_dRnonCO2;
    for(int i=0; i<tmax; i++)
    {
        annual_dRnonCO2 = R_dummy[years+1]-R_dummy[years]; //set annual emission rate
        
        RnonCO2[i] = R_dummy[years] + double(count)*annual_dRnonCO2/double(t_steps_year);
        
        count++;
        
        if(count == (t_steps_year))
        {
            count=0;
            years++;
        }
        
        
    }
    
}

void convertAOD(int t_steps_month) //converts RnonCO2 when mpre than 1 time step per year
{
    double R_dummy[tmax];
    for (int i = 0; i<tmax; i++)
    {
        R_dummy[i] = AOD[i];
    }
    //Now re-set original array by linearising between years
    
    int years=0;
    int count = 0;
    double annual_dRnonCO2;
    for(int i=0; i<tmax; i++)
    {
        annual_dRnonCO2 = R_dummy[years+1]-R_dummy[years];
        
        AOD[i] = R_dummy[years] + double(count)*annual_dRnonCO2/double(t_steps_month);
        
        count++;
        
        if(count == (t_steps_month))
        {
            count=0;
            years++;
        }
        
        
    }
    
}





double getRateINDCs(double yearEm)//Fawcett et al (2015) analysis of CO2 emissions from INDC
{
    double EmRate2030= 10.67 + 0.96; //Fawcett et al (2015) + constant 2014 land use change contribution which is average 2010-2014 from Carbon Budget 2015.
    
    double EmRate2014= 9.80+1.09;
    
    double RateEm = 0.0; //is in PgC yr-1
    
    if(yearEm < 2014.0)
    {
        yearEm = 2014.0;
    }
    
    if(yearEm <= 2031.0) //Linear increase to 2030
    {
        RateEm =  EmRate2014 + (EmRate2030-EmRate2014)* ( (yearEm - 2014.0)/(2031.0-2014.0) );
    }
    if(yearEm > 2031.0) //Constant after 2030.
    {
        RateEm = EmRate2030;
    }
    
    return RateEm;
    
}


//Function returns the annual emission rate from The Carbon Budget 2015, LeQuere et al (2015).
double getRateCarbonBudget(double yearEm)
{
    double RateEm=0.0; //is in PgC per year
    
    if(yearEm<2006.0)
    {
        RateEm = 8.08+1.02;
    }
    if(yearEm>=2006.0 && yearEm <2007.0)
    {
        RateEm = 6.36+1.09;
    }
    if(yearEm>=2007.0 && yearEm <2008.0)
    {
        RateEm = 8.53+0.97;
    }
    if(yearEm>=2008.0 && yearEm <2009.0)
    {
        RateEm = 8.74+0.67;
    }
    if(yearEm>=2009.0 && yearEm <2010.0)
    {
        RateEm = 8.70+0.76;
    }
    if(yearEm>=2010.0 && yearEm <2011.0)
    {
        RateEm = 9.14+0.88;
    }
    if(yearEm>=2011.0 && yearEm <2012.0)
    {
        RateEm = 9.45+0.91;
    }
    if(yearEm>=2012.0 && yearEm <2013.0)
    {
        RateEm = 9.58+0.97;
    }
    if(yearEm>=2013.0 && yearEm <2014.0)
    {
        RateEm = 9.74+0.92;
    }
    if(yearEm>=2014.0)
    {
        RateEm = 9.80+1.09;
    }
    return RateEm;
}

void setCO2RCP3PD(double coeff)
{
    CO2[	0	]=	278.05158	/coeff;
    CO2[	1	]=	278.10615	/coeff;
    CO2[	2	]=	278.22039	/coeff;
    CO2[	3	]=	278.34305	/coeff;
    CO2[	4	]=	278.47058	/coeff;
    CO2[	5	]=	278.60047	/coeff;
    CO2[	6	]=	278.73275	/coeff;
    CO2[	7	]=	278.86881	/coeff;
    CO2[	8	]=	279.00907	/coeff;
    CO2[	9	]=	279.15318	/coeff;
    CO2[	10	]=	279.30180	/coeff;
    CO2[	11	]=	279.45680	/coeff;
    CO2[	12	]=	279.61808	/coeff;
    CO2[	13	]=	279.78192	/coeff;
    CO2[	14	]=	279.94320	/coeff;
    CO2[	15	]=	280.09740	/coeff;
    CO2[	16	]=	280.24280	/coeff;
    CO2[	17	]=	280.38168	/coeff;
    CO2[	18	]=	280.51832	/coeff;
    CO2[	19	]=	280.65720	/coeff;
    CO2[	20	]=	280.80260	/coeff;
    CO2[	21	]=	280.95680	/coeff;
    CO2[	22	]=	281.11808	/coeff;
    CO2[	23	]=	281.28192	/coeff;
    CO2[	24	]=	281.44320	/coeff;
    CO2[	25	]=	281.59820	/coeff;
    CO2[	26	]=	281.74682	/coeff;
    CO2[	27	]=	281.89093	/coeff;
    CO2[	28	]=	282.03119	/coeff;
    CO2[	29	]=	282.16725	/coeff;
    CO2[	30	]=	282.29901	/coeff;
    CO2[	31	]=	282.42680	/coeff;
    CO2[	32	]=	282.55093	/coeff;
    CO2[	33	]=	282.67123	/coeff;
    CO2[	34	]=	282.78730	/coeff;
    CO2[	35	]=	282.89901	/coeff;
    CO2[	36	]=	283.00677	/coeff;
    CO2[	37	]=	283.11093	/coeff;
    CO2[	38	]=	283.21129	/coeff;
    CO2[	39	]=	283.30737	/coeff;
    CO2[	40	]=	283.39963	/coeff;
    CO2[	41	]=	283.48978	/coeff;
    CO2[	42	]=	283.57796	/coeff;
    CO2[	43	]=	283.66116	/coeff;
    CO2[	44	]=	283.73511	/coeff;
    CO2[	45	]=	283.79676	/coeff;
    CO2[	46	]=	283.84667	/coeff;
    CO2[	47	]=	283.88853	/coeff;
    CO2[	48	]=	283.92613	/coeff;
    CO2[	49	]=	283.96267	/coeff;
    CO2[	50	]=	284.00107	/coeff;
    CO2[	51	]=	284.04267	/coeff;
    CO2[	52	]=	284.08613	/coeff;
    CO2[	53	]=	284.12853	/coeff;
    CO2[	54	]=	284.16667	/coeff;
    CO2[	55	]=	284.19820	/coeff;
    CO2[	56	]=	284.22333	/coeff;
    CO2[	57	]=	284.24427	/coeff;
    CO2[	58	]=	284.26307	/coeff;
    CO2[	59	]=	284.28133	/coeff;
    CO2[	60	]=	284.30027	/coeff;
    CO2[	61	]=	284.32000	/coeff;
    CO2[	62	]=	284.34000	/coeff;
    CO2[	63	]=	284.36000	/coeff;
    CO2[	64	]=	284.38000	/coeff;
    CO2[	65	]=	284.40000	/coeff;
    CO2[	66	]=	284.38500	/coeff;
    CO2[	67	]=	284.28000	/coeff;
    CO2[	68	]=	284.12500	/coeff;
    CO2[	69	]=	283.97500	/coeff;
    CO2[	70	]=	283.82500	/coeff;
    CO2[	71	]=	283.67500	/coeff;
    CO2[	72	]=	283.52500	/coeff;
    CO2[	73	]=	283.42500	/coeff;
    CO2[	74	]=	283.40000	/coeff;
    CO2[	75	]=	283.40000	/coeff;
    CO2[	76	]=	283.42500	/coeff;
    CO2[	77	]=	283.50000	/coeff;
    CO2[	78	]=	283.60000	/coeff;
    CO2[	79	]=	283.72500	/coeff;
    CO2[	80	]=	283.90000	/coeff;
    CO2[	81	]=	284.07500	/coeff;
    CO2[	82	]=	284.22500	/coeff;
    CO2[	83	]=	284.40000	/coeff;
    CO2[	84	]=	284.57500	/coeff;
    CO2[	85	]=	284.72500	/coeff;
    CO2[	86	]=	284.87500	/coeff;
    CO2[	87	]=	285.00000	/coeff;
    CO2[	88	]=	285.12500	/coeff;
    CO2[	89	]=	285.27500	/coeff;
    CO2[	90	]=	285.42500	/coeff;
    CO2[	91	]=	285.57500	/coeff;
    CO2[	92	]=	285.72500	/coeff;
    CO2[	93	]=	285.90000	/coeff;
    CO2[	94	]=	286.07500	/coeff;
    CO2[	95	]=	286.22500	/coeff;
    CO2[	96	]=	286.37500	/coeff;
    CO2[	97	]=	286.50000	/coeff;
    CO2[	98	]=	286.62500	/coeff;
    CO2[	99	]=	286.77500	/coeff;
    CO2[	100	]=	286.90000	/coeff;
    CO2[	101	]=	287.00000	/coeff;
    CO2[	102	]=	287.10000	/coeff;
    CO2[	103	]=	287.22500	/coeff;
    CO2[	104	]=	287.37500	/coeff;
    CO2[	105	]=	287.52500	/coeff;
    CO2[	106	]=	287.70000	/coeff;
    CO2[	107	]=	287.90000	/coeff;
    CO2[	108	]=	288.12500	/coeff;
    CO2[	109	]=	288.40000	/coeff;
    CO2[	110	]=	288.70000	/coeff;
    CO2[	111	]=	289.02500	/coeff;
    CO2[	112	]=	289.40000	/coeff;
    CO2[	113	]=	289.80000	/coeff;
    CO2[	114	]=	290.22500	/coeff;
    CO2[	115	]=	290.70000	/coeff;
    CO2[	116	]=	291.20000	/coeff;
    CO2[	117	]=	291.67500	/coeff;
    CO2[	118	]=	292.12500	/coeff;
    CO2[	119	]=	292.57500	/coeff;
    CO2[	120	]=	292.97500	/coeff;
    CO2[	121	]=	293.30000	/coeff;
    CO2[	122	]=	293.57500	/coeff;
    CO2[	123	]=	293.80000	/coeff;
    CO2[	124	]=	294.00000	/coeff;
    CO2[	125	]=	294.17500	/coeff;
    CO2[	126	]=	294.32500	/coeff;
    CO2[	127	]=	294.47500	/coeff;
    CO2[	128	]=	294.60000	/coeff;
    CO2[	129	]=	294.70000	/coeff;
    CO2[	130	]=	294.80000	/coeff;
    CO2[	131	]=	294.90000	/coeff;
    CO2[	132	]=	295.02500	/coeff;
    CO2[	133	]=	295.22500	/coeff;
    CO2[	134	]=	295.50000	/coeff;
    CO2[	135	]=	295.80000	/coeff;
    CO2[	136	]=	296.12500	/coeff;
    CO2[	137	]=	296.47500	/coeff;
    CO2[	138	]=	296.82500	/coeff;
    CO2[	139	]=	297.20000	/coeff;
    CO2[	140	]=	297.62500	/coeff;
    CO2[	141	]=	298.07500	/coeff;
    CO2[	142	]=	298.50000	/coeff;
    CO2[	143	]=	298.90000	/coeff;
    CO2[	144	]=	299.30000	/coeff;
    CO2[	145	]=	299.70000	/coeff;
    CO2[	146	]=	300.07500	/coeff;
    CO2[	147	]=	300.42500	/coeff;
    CO2[	148	]=	300.77500	/coeff;
    CO2[	149	]=	301.10000	/coeff;
    CO2[	150	]=	301.40000	/coeff;
    CO2[	151	]=	301.72500	/coeff;
    CO2[	152	]=	302.07500	/coeff;
    CO2[	153	]=	302.40000	/coeff;
    CO2[	154	]=	302.70000	/coeff;
    CO2[	155	]=	303.02500	/coeff;
    CO2[	156	]=	303.40000	/coeff;
    CO2[	157	]=	303.77500	/coeff;
    CO2[	158	]=	304.12500	/coeff;
    CO2[	159	]=	304.52500	/coeff;
    CO2[	160	]=	304.97500	/coeff;
    CO2[	161	]=	305.40000	/coeff;
    CO2[	162	]=	305.82500	/coeff;
    CO2[	163	]=	306.30000	/coeff;
    CO2[	164	]=	306.77500	/coeff;
    CO2[	165	]=	307.22500	/coeff;
    CO2[	166	]=	307.70000	/coeff;
    CO2[	167	]=	308.17500	/coeff;
    CO2[	168	]=	308.60000	/coeff;
    CO2[	169	]=	309.00000	/coeff;
    CO2[	170	]=	309.40000	/coeff;
    CO2[	171	]=	309.75000	/coeff;
    CO2[	172	]=	310.00000	/coeff;
    CO2[	173	]=	310.17500	/coeff;
    CO2[	174	]=	310.30000	/coeff;
    CO2[	175	]=	310.37500	/coeff;
    CO2[	176	]=	310.37500	/coeff;
    CO2[	177	]=	310.30000	/coeff;
    CO2[	178	]=	310.20000	/coeff;
    CO2[	179	]=	310.12500	/coeff;
    CO2[	180	]=	310.10000	/coeff;
    CO2[	181	]=	310.12500	/coeff;
    CO2[	182	]=	310.20000	/coeff;
    CO2[	183	]=	310.32500	/coeff;
    CO2[	184	]=	310.50000	/coeff;
    CO2[	185	]=	310.75000	/coeff;
    CO2[	186	]=	311.10000	/coeff;
    CO2[	187	]=	311.50000	/coeff;
    CO2[	188	]=	311.92500	/coeff;
    CO2[	189	]=	312.42500	/coeff;
    CO2[	190	]=	313.00000	/coeff;
    CO2[	191	]=	313.60000	/coeff;
    CO2[	192	]=	314.22500	/coeff;
    CO2[	193	]=	314.84750	/coeff;
    CO2[	194	]=	315.50000	/coeff;
    CO2[	195	]=	316.27250	/coeff;
    CO2[	196	]=	317.07500	/coeff;
    CO2[	197	]=	317.79500	/coeff;
    CO2[	198	]=	318.39750	/coeff;
    CO2[	199	]=	318.92500	/coeff;
    CO2[	200	]=	319.64750	/coeff;
    CO2[	201	]=	320.64750	/coeff;
    CO2[	202	]=	321.60500	/coeff;
    CO2[	203	]=	322.63500	/coeff;
    CO2[	204	]=	323.90250	/coeff;
    CO2[	205	]=	324.98500	/coeff;
    CO2[	206	]=	325.85500	/coeff;
    CO2[	207	]=	327.14000	/coeff;
    CO2[	208	]=	328.67750	/coeff;
    CO2[	209	]=	329.74250	/coeff;
    CO2[	210	]=	330.58500	/coeff;
    CO2[	211	]=	331.74750	/coeff;
    CO2[	212	]=	333.27250	/coeff;
    CO2[	213	]=	334.84750	/coeff;
    CO2[	214	]=	336.52500	/coeff;
    CO2[	215	]=	338.36000	/coeff;
    CO2[	216	]=	339.72750	/coeff;
    CO2[	217	]=	340.79250	/coeff;
    CO2[	218	]=	342.19750	/coeff;
    CO2[	219	]=	343.78250	/coeff;
    CO2[	220	]=	345.28250	/coeff;
    CO2[	221	]=	346.79750	/coeff;
    CO2[	222	]=	348.64500	/coeff;
    CO2[	223	]=	350.73750	/coeff;
    CO2[	224	]=	352.48750	/coeff;
    CO2[	225	]=	353.85500	/coeff;
    CO2[	226	]=	355.01750	/coeff;
    CO2[	227	]=	355.88500	/coeff;
    CO2[	228	]=	356.77750	/coeff;
    CO2[	229	]=	358.12750	/coeff;
    CO2[	230	]=	359.83750	/coeff;
    CO2[	231	]=	361.46250	/coeff;
    CO2[	232	]=	363.15500	/coeff;
    CO2[	233	]=	365.32250	/coeff;
    CO2[	234	]=	367.34750	/coeff;
    CO2[	235	]=	368.86500	/coeff;
    CO2[	236	]=	370.46750	/coeff;
    CO2[	237	]=	372.52250	/coeff;
    CO2[	238	]=	374.76000	/coeff;
    CO2[	239	]=	376.81250	/coeff;
    CO2[	240	]=	378.81250	/coeff;
    CO2[	241	]=	380.82750	/coeff;
    CO2[	242	]=	382.77750	/coeff;
    CO2[	243	]=	384.80000	/coeff;
    CO2[	244	]=	387.00054	/coeff;
    CO2[	245	]=	389.28521	/coeff;
    CO2[	246	]=	391.56252	/coeff;
    CO2[	247	]=	393.84273	/coeff;
    CO2[	248	]=	396.11734	/coeff;
    CO2[	249	]=	398.39568	/coeff;
    CO2[	250	]=	400.68068	/coeff;
    CO2[	251	]=	402.96830	/coeff;
    CO2[	252	]=	405.25182	/coeff;
    CO2[	253	]=	407.52882	/coeff;
    CO2[	254	]=	409.80029	/coeff;
    CO2[	255	]=	412.06783	/coeff;
    CO2[	256	]=	414.32565	/coeff;
    CO2[	257	]=	416.51662	/coeff;
    CO2[	258	]=	418.60322	/coeff;
    CO2[	259	]=	420.60132	/coeff;
    CO2[	260	]=	422.51575	/coeff;
    CO2[	261	]=	424.34901	/coeff;
    CO2[	262	]=	426.09675	/coeff;
    CO2[	263	]=	427.75232	/coeff;
    CO2[	264	]=	429.31381	/coeff;
    CO2[	265	]=	430.78315	/coeff;
    CO2[	266	]=	432.16344	/coeff;
    CO2[	267	]=	433.43618	/coeff;
    CO2[	268	]=	434.59273	/coeff;
    CO2[	269	]=	435.65302	/coeff;
    CO2[	270	]=	436.62804	/coeff;
    CO2[	271	]=	437.52158	/coeff;
    CO2[	272	]=	438.33433	/coeff;
    CO2[	273	]=	439.06015	/coeff;
    CO2[	274	]=	439.69061	/coeff;
    CO2[	275	]=	440.22240	/coeff;
    CO2[	276	]=	440.65666	/coeff;
    CO2[	277	]=	441.02476	/coeff;
    CO2[	278	]=	441.34651	/coeff;
    CO2[	279	]=	441.62100	/coeff;
    CO2[	280	]=	441.86440	/coeff;
    CO2[	281	]=	442.08482	/coeff;
    CO2[	282	]=	442.28340	/coeff;
    CO2[	283	]=	442.45830	/coeff;
    CO2[	284	]=	442.60091	/coeff;
    CO2[	285	]=	442.70046	/coeff;
    CO2[	286	]=	442.75184	/coeff;
    CO2[	287	]=	442.76100	/coeff;
    CO2[	288	]=	442.73417	/coeff;
    CO2[	289	]=	442.66263	/coeff;
    CO2[	290	]=	442.54767	/coeff;
    CO2[	291	]=	442.40642	/coeff;
    CO2[	292	]=	442.24833	/coeff;
    CO2[	293	]=	442.07535	/coeff;
    CO2[	294	]=	441.88625	/coeff;
    CO2[	295	]=	441.67274	/coeff;
    CO2[	296	]=	441.42415	/coeff;
    CO2[	297	]=	441.13450	/coeff;
    CO2[	298	]=	440.80284	/coeff;
    CO2[	299	]=	440.43041	/coeff;
    CO2[	300	]=	440.01021	/coeff;
    CO2[	301	]=	439.54473	/coeff;
    CO2[	302	]=	439.05222	/coeff;
    CO2[	303	]=	438.54291	/coeff;
    CO2[	304	]=	438.01932	/coeff;
    CO2[	305	]=	437.48062	/coeff;
    CO2[	306	]=	436.91878	/coeff;
    CO2[	307	]=	436.34251	/coeff;
    CO2[	308	]=	435.76436	/coeff;
    CO2[	309	]=	435.18189	/coeff;
    CO2[	310	]=	434.59500	/coeff;
    CO2[	311	]=	433.99542	/coeff;
    CO2[	312	]=	433.38456	/coeff;
    CO2[	313	]=	432.77961	/coeff;
    CO2[	314	]=	432.18978	/coeff;
    CO2[	315	]=	431.61659	/coeff;
    CO2[	316	]=	431.05847	/coeff;
    CO2[	317	]=	430.51026	/coeff;
    CO2[	318	]=	429.96413	/coeff;
    CO2[	319	]=	429.41411	/coeff;
    CO2[	320	]=	428.85906	/coeff;
    CO2[	321	]=	428.29910	/coeff;
    CO2[	322	]=	427.72652	/coeff;
    CO2[	323	]=	427.14308	/coeff;
    CO2[	324	]=	426.56607	/coeff;
    CO2[	325	]=	426.00472	/coeff;
    CO2[	326	]=	425.46065	/coeff;
    CO2[	327	]=	424.93745	/coeff;
    CO2[	328	]=	424.43129	/coeff;
    CO2[	329	]=	423.93083	/coeff;
    CO2[	330	]=	423.43058	/coeff;
    CO2[	331	]=	422.92945	/coeff;
    CO2[	332	]=	422.42761	/coeff;
    CO2[	333	]=	421.91753	/coeff;
    CO2[	334	]=	421.40111	/coeff;
    CO2[	335	]=	420.89546	/coeff;
    CO2[	336	]=	420.40998	/coeff;
    CO2[	337	]=	419.94532	/coeff;
    CO2[	338	]=	419.49831	/coeff;
    CO2[	339	]=	419.05981	/coeff;
    CO2[	340	]=	418.61902	/coeff;
    CO2[	341	]=	418.17082	/coeff;
    CO2[	342	]=	417.71454	/coeff;
    CO2[	343	]=	417.25070	/coeff;
    CO2[	344	]=	416.77219	/coeff;
    CO2[	345	]=	416.28127	/coeff;
    CO2[	346	]=	415.79518	/coeff;
    CO2[	347	]=	415.32319	/coeff;
    CO2[	348	]=	414.86696	/coeff;
    CO2[	349	]=	414.42496	/coeff;
    CO2[	350	]=	413.98893	/coeff;
    CO2[	351	]=	413.54867	/coeff;
    CO2[	352	]=	413.09942	/coeff;
    CO2[	353	]=	412.64076	/coeff;
    CO2[	354	]=	412.17338	/coeff;
    CO2[	355	]=	411.69045	/coeff;
    CO2[	356	]=	411.19435	/coeff;
    CO2[	357	]=	410.70228	/coeff;
    CO2[	358	]=	410.22339	/coeff;
    CO2[	359	]=	409.75930	/coeff;
    CO2[	360	]=	409.30847	/coeff;
    CO2[	361	]=	408.86270	/coeff;
    CO2[	362	]=	408.41656	/coeff;
    CO2[	363	]=	407.96956	/coeff;
    CO2[	364	]=	407.52073	/coeff;
    CO2[	365	]=	407.07051	/coeff;
    CO2[	366	]=	406.61189	/coeff;
    CO2[	367	]=	406.14707	/coeff;
    CO2[	368	]=	405.69286	/coeff;
    CO2[	369	]=	405.25809	/coeff;
    CO2[	370	]=	404.84413	/coeff;
    CO2[	371	]=	404.44922	/coeff;
    CO2[	372	]=	404.06507	/coeff;
    CO2[	373	]=	403.68161	/coeff;
    CO2[	374	]=	403.29415	/coeff;
    CO2[	375	]=	402.90226	/coeff;
    CO2[	376	]=	402.50662	/coeff;
    CO2[	377	]=	402.10056	/coeff;
    CO2[	378	]=	401.68653	/coeff;
    CO2[	379	]=	401.28131	/coeff;
    CO2[	380	]=	400.89375	/coeff;
    CO2[	381	]=	400.52523	/coeff;
    CO2[	382	]=	400.17404	/coeff;
    CO2[	383	]=	399.83204	/coeff;
    CO2[	384	]=	399.48930	/coeff;
    CO2[	385	]=	399.14131	/coeff;
    CO2[	386	]=	398.78772	/coeff;
    CO2[	387	]=	398.42927	/coeff;
    CO2[	388	]=	398.05950	/coeff;
    CO2[	389	]=	397.68091	/coeff;
    CO2[	390	]=	397.31021	/coeff;
    CO2[	391	]=	396.95616	/coeff;
    CO2[	392	]=	396.62009	/coeff;
    CO2[	393	]=	396.30030	/coeff;
    CO2[	394	]=	395.98871	/coeff;
    CO2[	395	]=	395.67554	/coeff;
    CO2[	396	]=	395.35634	/coeff;
    CO2[	397	]=	395.03085	/coeff;
    CO2[	398	]=	394.69985	/coeff;
    CO2[	399	]=	394.35700	/coeff;
    CO2[	400	]=	394.00489	/coeff;
    CO2[	401	]=	393.66010	/coeff;
    CO2[	402	]=	393.33128	/coeff;
    CO2[	403	]=	393.01974	/coeff;
    CO2[	404	]=	392.72374	/coeff;
    CO2[	405	]=	392.43525	/coeff;
    CO2[	406	]=	392.14460	/coeff;
    CO2[	407	]=	391.84744	/coeff;
    CO2[	408	]=	391.54354	/coeff;
    CO2[	409	]=	391.23370	/coeff;
    CO2[	410	]=	390.91172	/coeff;
    CO2[	411	]=	390.58024	/coeff;
    CO2[	412	]=	390.25571	/coeff;
    CO2[	413	]=	389.94668	/coeff;
    CO2[	414	]=	389.65441	/coeff;
    CO2[	415	]=	389.37713	/coeff;
    CO2[	416	]=	389.10687	/coeff;
    CO2[	417	]=	388.83404	/coeff;
    CO2[	418	]=	388.55437	/coeff;
    CO2[	419	]=	388.26765	/coeff;
    CO2[	420	]=	387.97473	/coeff;
    CO2[	421	]=	387.66950	/coeff;
    CO2[	422	]=	387.35465	/coeff;
    CO2[	423	]=	387.04651	/coeff;
    CO2[	424	]=	386.75353	/coeff;
    CO2[	425	]=	386.47692	/coeff;
    CO2[	426	]=	386.21487	/coeff;
    CO2[	427	]=	385.95946	/coeff;
    CO2[	428	]=	385.70118	/coeff;
    CO2[	429	]=	385.43582	/coeff;
    CO2[	430	]=	385.16322	/coeff;
    CO2[	431	]=	384.88424	/coeff;
    CO2[	432	]=	384.59286	/coeff;
    CO2[	433	]=	384.29183	/coeff;
    CO2[	434	]=	383.99735	/coeff;
    CO2[	435	]=	383.71778	/coeff;
    CO2[	436	]=	383.45425	/coeff;
    CO2[	437	]=	383.20495	/coeff;
    CO2[	438	]=	382.96197	/coeff;
    CO2[	439	]=	382.71591	/coeff;
    CO2[	440	]=	382.46260	/coeff;
    CO2[	441	]=	382.20191	/coeff;
    CO2[	442	]=	381.93471	/coeff;
    CO2[	443	]=	381.65510	/coeff;
    CO2[	444	]=	381.36585	/coeff;
    CO2[	445	]=	381.08304	/coeff;
    CO2[	446	]=	380.81494	/coeff;
    CO2[	447	]=	380.56263	/coeff;
    CO2[	448	]=	380.32426	/coeff;
    CO2[	449	]=	380.09195	/coeff;
    CO2[	450	]=	379.85638	/coeff;
    CO2[	451	]=	379.61345	/coeff;
    CO2[	452	]=	379.36304	/coeff;
    CO2[	453	]=	379.10604	/coeff;
    CO2[	454	]=	378.83665	/coeff;
    CO2[	455	]=	378.55765	/coeff;
    CO2[	456	]=	378.28503	/coeff;
    CO2[	457	]=	378.02696	/coeff;
    CO2[	458	]=	377.78445	/coeff;
    CO2[	459	]=	377.55563	/coeff;
    CO2[	460	]=	377.33266	/coeff;
    CO2[	461	]=	377.10629	/coeff;
    CO2[	462	]=	376.87246	/coeff;
    CO2[	463	]=	376.63109	/coeff;
    CO2[	464	]=	376.38308	/coeff;
    CO2[	465	]=	376.12271	/coeff;
    CO2[	466	]=	375.85281	/coeff;
    CO2[	467	]=	375.58923	/coeff;
    CO2[	468	]=	375.34006	/coeff;
    CO2[	469	]=	375.10628	/coeff;
    CO2[	470	]=	374.88595	/coeff;
    CO2[	471	]=	374.67128	/coeff;
    CO2[	472	]=	374.45310	/coeff;
    CO2[	473	]=	374.22739	/coeff;
    CO2[	474	]=	373.99410	/coeff;
    CO2[	475	]=	373.75412	/coeff;
    CO2[	476	]=	373.50184	/coeff;
    CO2[	477	]=	373.24011	/coeff;
    CO2[	478	]=	372.98467	/coeff;
    CO2[	479	]=	372.74353	/coeff;
    CO2[	480	]=	372.51759	/coeff;
    CO2[	481	]=	372.30492	/coeff;
    CO2[	482	]=	372.09773	/coeff;
    CO2[	483	]=	371.88692	/coeff;
    CO2[	484	]=	371.66854	/coeff;
    CO2[	485	]=	371.44254	/coeff;
    CO2[	486	]=	371.20984	/coeff;
    CO2[	487	]=	370.96489	/coeff;
    CO2[	488	]=	370.71059	/coeff;
    CO2[	489	]=	370.46256	/coeff;
    CO2[	490	]=	370.22872	/coeff;
    CO2[	491	]=	370.00993	/coeff;
    CO2[	492	]=	369.80422	/coeff;
    CO2[	493	]=	369.60385	/coeff;
    CO2[	494	]=	369.39976	/coeff;
    CO2[	495	]=	369.18806	/coeff;
    CO2[	496	]=	368.96871	/coeff;
    CO2[	497	]=	368.74266	/coeff;
    CO2[	498	]=	368.50442	/coeff;
    CO2[	499	]=	368.25693	/coeff;
    CO2[	500	]=	368.01570	/coeff;
    CO2[	501	]=	367.78857	/coeff;
    CO2[	502	]=	367.57635	/coeff;
    CO2[	503	]=	367.37703	/coeff;
    CO2[	504	]=	367.18290	/coeff;
    CO2[	505	]=	366.98498	/coeff;
    CO2[	506	]=	366.77941	/coeff;
    CO2[	507	]=	366.56619	/coeff;
    CO2[	508	]=	366.34625	/coeff;
    CO2[	509	]=	366.11421	/coeff;
    CO2[	510	]=	365.87300	/coeff;
    CO2[	511	]=	365.63806	/coeff;
    CO2[	512	]=	365.41713	/coeff;
    CO2[	513	]=	365.21098	/coeff;
    CO2[	514	]=	365.01757	/coeff;
    CO2[	515	]=	364.82921	/coeff;
    CO2[	516	]=	364.63699	/coeff;
    CO2[	517	]=	364.43710	/coeff;
    CO2[	518	]=	364.22954	/coeff;
    CO2[	519	]=	364.01526	/coeff;
    CO2[	520	]=	363.78896	/coeff;
    CO2[	521	]=	363.55360	/coeff;
    CO2[	522	]=	363.32451	/coeff;
    CO2[	523	]=	363.10935	/coeff;
    CO2[	524	]=	362.90884	/coeff;
    CO2[	525	]=	362.72091	/coeff;
    CO2[	526	]=	362.53791	/coeff;
    CO2[	527	]=	362.35099	/coeff;
    CO2[	528	]=	362.15637	/coeff;
    CO2[	529	]=	361.95408	/coeff;
    CO2[	530	]=	361.74508	/coeff;
    CO2[	531	]=	361.52413	/coeff;
    CO2[	532	]=	361.29423	/coeff;
    CO2[	533	]=	361.07060	/coeff;
    CO2[	534	]=	360.86083	/coeff;
    CO2[	535	]=	360.66558	/coeff;
    CO2[	536	]=	360.48278	/coeff;
    CO2[	537	]=	360.30479	/coeff;
    CO2[	538	]=	360.12281	/coeff;
    CO2[	539	]=	359.93311	/coeff;
    CO2[	540	]=	359.73574	/coeff;
    CO2[	541	]=	359.53167	/coeff;
    CO2[	542	]=	359.31573	/coeff;
    CO2[	543	]=	359.09095	/coeff;
    CO2[	544	]=	358.87244	/coeff;
    CO2[	545	]=	358.66772	/coeff;
    CO2[	546	]=	358.47742	/coeff;
    CO2[	547	]=	358.29942	/coeff;
    CO2[	548	]=	358.12611	/coeff;
    CO2[	549	]=	357.94876	/coeff;
    CO2[	550	]=	357.76367	/coeff;
    CO2[	551	]=	357.57092	/coeff;
    CO2[	552	]=	357.37148	/coeff;
    CO2[	553	]=	357.16025	/coeff;
    CO2[	554	]=	356.94027	/coeff;
    CO2[	555	]=	356.72659	/coeff;
    CO2[	556	]=	356.52662	/coeff;
    CO2[	557	]=	356.34097	/coeff;
    CO2[	558	]=	356.16748	/coeff;
    CO2[	559	]=	355.99857	/coeff;
    CO2[	560	]=	355.82557	/coeff;
    CO2[	561	]=	355.64482	/coeff;
    CO2[	562	]=	355.45642	/coeff;
    CO2[	563	]=	355.26133	/coeff;
    CO2[	564	]=	355.05453	/coeff;
    CO2[	565	]=	354.83910	/coeff;
    CO2[	566	]=	354.62997	/coeff;
    CO2[	567	]=	354.43449	/coeff;
    CO2[	568	]=	354.25322	/coeff;
    CO2[	569	]=	354.08399	/coeff;
    CO2[	570	]=	353.91923	/coeff;
    CO2[	571	]=	353.75033	/coeff;
    CO2[	572	]=	353.57367	/coeff;
    CO2[	573	]=	353.38936	/coeff;
    CO2[	574	]=	353.19838	/coeff;
    CO2[	575	]=	352.99578	/coeff;
    CO2[	576	]=	352.78464	/coeff;
    CO2[	577	]=	352.57981	/coeff;
    CO2[	578	]=	352.38859	/coeff;
    CO2[	579	]=	352.21147	/coeff;
    CO2[	580	]=	352.04626	/coeff;
    CO2[	581	]=	351.88542	/coeff;
    CO2[	582	]=	351.72039	/coeff;
    CO2[	583	]=	351.54760	/coeff;
    CO2[	584	]=	351.36717	/coeff;
    CO2[	585	]=	351.18007	/coeff;
    CO2[	586	]=	350.98144	/coeff;
    CO2[	587	]=	350.77437	/coeff;
    CO2[	588	]=	350.57364	/coeff;
    CO2[	589	]=	350.38645	/coeff;
    CO2[	590	]=	350.21326	/coeff;
    CO2[	591	]=	350.05186	/coeff;
    CO2[	592	]=	349.89473	/coeff;
    CO2[	593	]=	349.73338	/coeff;
    CO2[	594	]=	349.56425	/coeff;
    CO2[	595	]=	349.38748	/coeff;
    CO2[	596	]=	349.20408	/coeff;
    CO2[	597	]=	349.00921	/coeff;
    CO2[	598	]=	348.80602	/coeff;
    CO2[	599	]=	348.60917	/coeff;
    CO2[	600	]=	348.42581	/coeff;
    CO2[	601	]=	348.25637	/coeff;
    CO2[	602	]=	348.09859	/coeff;
    CO2[	603	]=	347.94498	/coeff;
    CO2[	604	]=	347.78711	/coeff;
    CO2[	605	]=	347.62145	/coeff;
    CO2[	606	]=	347.44817	/coeff;
    CO2[	607	]=	347.26827	/coeff;
    CO2[	608	]=	347.07699	/coeff;
    CO2[	609	]=	346.87749	/coeff;
    CO2[	610	]=	346.68435	/coeff;
    CO2[	611	]=	346.50464	/coeff;
    CO2[	612	]=	346.33875	/coeff;
    CO2[	613	]=	346.18441	/coeff;
    CO2[	614	]=	346.03416	/coeff;
    CO2[	615	]=	345.87960	/coeff;
    CO2[	616	]=	345.71725	/coeff;
    CO2[	617	]=	345.54729	/coeff;
    CO2[	618	]=	345.37072	/coeff;
    CO2[	619	]=	345.18286	/coeff;
    CO2[	620	]=	344.98687	/coeff;
    CO2[	621	]=	344.79727	/coeff;
    CO2[	622	]=	344.62105	/coeff;
    CO2[	623	]=	344.45856	/coeff;
    CO2[	624	]=	344.30750	/coeff;
    CO2[	625	]=	344.16044	/coeff;
    CO2[	626	]=	344.00903	/coeff;
    CO2[	627	]=	343.84983	/coeff;
    CO2[	628	]=	343.68304	/coeff;
    CO2[	629	]=	343.50965	/coeff;
    CO2[	630	]=	343.32505	/coeff;
    CO2[	631	]=	343.13243	/coeff;
    CO2[	632	]=	342.94620	/coeff;
    CO2[	633	]=	342.77331	/coeff;
    CO2[	634	]=	342.61407	/coeff;
    CO2[	635	]=	342.46614	/coeff;
    CO2[	636	]=	342.32213	/coeff;
    CO2[	637	]=	342.17373	/coeff;
    CO2[	638	]=	342.01754	/coeff;
    CO2[	639	]=	341.85376	/coeff;
    CO2[	640	]=	341.68342	/coeff;
    CO2[	641	]=	341.50194	/coeff;
    CO2[	642	]=	341.31253	/coeff;
    CO2[	643	]=	341.12955	/coeff;
    CO2[	644	]=	340.95985	/coeff;
    CO2[	645	]=	340.80371	/coeff;
    CO2[	646	]=	340.65878	/coeff;
    CO2[	647	]=	340.51768	/coeff;
    CO2[	648	]=	340.37215	/coeff;
    CO2[	649	]=	340.21884	/coeff;
    CO2[	650	]=	340.05795	/coeff;
    CO2[	651	]=	339.89052	/coeff;
    CO2[	652	]=	339.71203	/coeff;
    CO2[	653	]=	339.52571	/coeff;
    CO2[	654	]=	339.34583	/coeff;
    CO2[	655	]=	339.17919	/coeff;
    CO2[	656	]=	339.02602	/coeff;
    CO2[	657	]=	338.88397	/coeff;
    CO2[	658	]=	338.74565	/coeff;
    CO2[	659	]=	338.60288	/coeff;
    CO2[	660	]=	338.45232	/coeff;
    CO2[	661	]=	338.29420	/coeff;
    CO2[	662	]=	338.12955	/coeff;
    CO2[	663	]=	337.95392	/coeff;
    CO2[	664	]=	337.77057	/coeff;
    CO2[	665	]=	337.59367	/coeff;
    CO2[	666	]=	337.42997	/coeff;
    CO2[	667	]=	337.27966	/coeff;
    CO2[	668	]=	337.14036	/coeff;
    CO2[	669	]=	337.00472	/coeff;
    CO2[	670	]=	336.86458	/coeff;
    CO2[	671	]=	336.71666	/coeff;
    CO2[	672	]=	336.56120	/coeff;
    CO2[	673	]=	336.39922	/coeff;
    CO2[	674	]=	336.22635	/coeff;
    CO2[	675	]=	336.04585	/coeff;
    CO2[	676	]=	335.87182	/coeff;
    CO2[	677	]=	335.71094	/coeff;
    CO2[	678	]=	335.56338	/coeff;
    CO2[	679	]=	335.42672	/coeff;
    CO2[	680	]=	335.29364	/coeff;
    CO2[	681	]=	335.15604	/coeff;
    CO2[	682	]=	335.01065	/coeff;
    CO2[	683	]=	334.85773	/coeff;
    CO2[	684	]=	334.69833	/coeff;
    CO2[	685	]=	334.52811	/coeff;
    CO2[	686	]=	334.35035	/coeff;
    CO2[	687	]=	334.17908	/coeff;
    CO2[	688	]=	334.02092	/coeff;
    CO2[	689	]=	333.87600	/coeff;
    CO2[	690	]=	333.74189	/coeff;
    CO2[	691	]=	333.61127	/coeff;
    CO2[	692	]=	333.47610	/coeff;
    CO2[	693	]=	333.33315	/coeff;
    CO2[	694	]=	333.18269	/coeff;
    CO2[	695	]=	333.02576	/coeff;
    CO2[	696	]=	332.85808	/coeff;
    CO2[	697	]=	332.68296	/coeff;
    CO2[	698	]=	332.51436	/coeff;
    CO2[	699	]=	332.35883	/coeff;
    CO2[	700	]=	332.21645	/coeff;
    CO2[	701	]=	332.08479	/coeff;
    CO2[	702	]=	331.95654	/coeff;
    CO2[	703	]=	331.82371	/coeff;
    CO2[	704	]=	331.68311	/coeff;
    CO2[	705	]=	331.53500	/coeff;
    CO2[	706	]=	331.38045	/coeff;
    CO2[	707	]=	331.21523	/coeff;
    CO2[	708	]=	331.04267	/coeff;
    CO2[	709	]=	330.87664	/coeff;
    CO2[	710	]=	330.72364	/coeff;
    CO2[	711	]=	330.58372	/coeff;
    CO2[	712	]=	330.45441	/coeff;
    CO2[	713	]=	330.32845	/coeff;
    CO2[	714	]=	330.19788	/coeff;
    CO2[	715	]=	330.05953	/coeff;
    CO2[	716	]=	329.91370	/coeff;
    CO2[	717	]=	329.76144	/coeff;
    CO2[	718	]=	329.59860	/coeff;
    CO2[	719	]=	329.42850	/coeff;
    CO2[	720	]=	329.26495	/coeff;
    CO2[	721	]=	329.11439	/coeff;
    CO2[	722	]=	328.97684	/coeff;
    CO2[	723	]=	328.84981	/coeff;
    CO2[	724	]=	328.72606	/coeff;
    CO2[	725	]=	328.59766	/coeff;
    CO2[	726	]=	328.46149	/coeff;
    CO2[	727	]=	328.31785	/coeff;
    CO2[	728	]=	328.16781	/coeff;
    CO2[	729	]=	328.00726	/coeff;
    CO2[	730	]=	327.83954	/coeff;
    CO2[	731	]=	327.67840	/coeff;
    CO2[	732	]=	327.53021	/coeff;
    CO2[	733	]=	327.39495	/coeff;
    CO2[	734	]=	327.27012	/coeff;
    CO2[	735	]=	327.20980	/coeff;
    
    for(int i = 736; i<tmax; i++)
    {
        CO2[i] = CO2[i-1];
    }
    
}
void setCO2RCP45(double coeff)
{
    CO2[	0	]=	278.05158	/coeff;
    CO2[	1	]=	278.10615	/coeff;
    CO2[	2	]=	278.22039	/coeff;
    CO2[	3	]=	278.34305	/coeff;
    CO2[	4	]=	278.47058	/coeff;
    CO2[	5	]=	278.60047	/coeff;
    CO2[	6	]=	278.73275	/coeff;
    CO2[	7	]=	278.86881	/coeff;
    CO2[	8	]=	279.00907	/coeff;
    CO2[	9	]=	279.15318	/coeff;
    CO2[	10	]=	279.30180	/coeff;
    CO2[	11	]=	279.45680	/coeff;
    CO2[	12	]=	279.61808	/coeff;
    CO2[	13	]=	279.78192	/coeff;
    CO2[	14	]=	279.94320	/coeff;
    CO2[	15	]=	280.09740	/coeff;
    CO2[	16	]=	280.24280	/coeff;
    CO2[	17	]=	280.38168	/coeff;
    CO2[	18	]=	280.51832	/coeff;
    CO2[	19	]=	280.65720	/coeff;
    CO2[	20	]=	280.80260	/coeff;
    CO2[	21	]=	280.95680	/coeff;
    CO2[	22	]=	281.11808	/coeff;
    CO2[	23	]=	281.28192	/coeff;
    CO2[	24	]=	281.44320	/coeff;
    CO2[	25	]=	281.59820	/coeff;
    CO2[	26	]=	281.74682	/coeff;
    CO2[	27	]=	281.89093	/coeff;
    CO2[	28	]=	282.03119	/coeff;
    CO2[	29	]=	282.16725	/coeff;
    CO2[	30	]=	282.29901	/coeff;
    CO2[	31	]=	282.42680	/coeff;
    CO2[	32	]=	282.55093	/coeff;
    CO2[	33	]=	282.67123	/coeff;
    CO2[	34	]=	282.78730	/coeff;
    CO2[	35	]=	282.89901	/coeff;
    CO2[	36	]=	283.00677	/coeff;
    CO2[	37	]=	283.11093	/coeff;
    CO2[	38	]=	283.21129	/coeff;
    CO2[	39	]=	283.30737	/coeff;
    CO2[	40	]=	283.39963	/coeff;
    CO2[	41	]=	283.48978	/coeff;
    CO2[	42	]=	283.57796	/coeff;
    CO2[	43	]=	283.66116	/coeff;
    CO2[	44	]=	283.73511	/coeff;
    CO2[	45	]=	283.79676	/coeff;
    CO2[	46	]=	283.84667	/coeff;
    CO2[	47	]=	283.88853	/coeff;
    CO2[	48	]=	283.92613	/coeff;
    CO2[	49	]=	283.96267	/coeff;
    CO2[	50	]=	284.00107	/coeff;
    CO2[	51	]=	284.04267	/coeff;
    CO2[	52	]=	284.08613	/coeff;
    CO2[	53	]=	284.12853	/coeff;
    CO2[	54	]=	284.16667	/coeff;
    CO2[	55	]=	284.19820	/coeff;
    CO2[	56	]=	284.22333	/coeff;
    CO2[	57	]=	284.24427	/coeff;
    CO2[	58	]=	284.26307	/coeff;
    CO2[	59	]=	284.28133	/coeff;
    CO2[	60	]=	284.30027	/coeff;
    CO2[	61	]=	284.32000	/coeff;
    CO2[	62	]=	284.34000	/coeff;
    CO2[	63	]=	284.36000	/coeff;
    CO2[	64	]=	284.38000	/coeff;
    CO2[	65	]=	284.40000	/coeff;
    CO2[	66	]=	284.38500	/coeff;
    CO2[	67	]=	284.28000	/coeff;
    CO2[	68	]=	284.12500	/coeff;
    CO2[	69	]=	283.97500	/coeff;
    CO2[	70	]=	283.82500	/coeff;
    CO2[	71	]=	283.67500	/coeff;
    CO2[	72	]=	283.52500	/coeff;
    CO2[	73	]=	283.42500	/coeff;
    CO2[	74	]=	283.40000	/coeff;
    CO2[	75	]=	283.40000	/coeff;
    CO2[	76	]=	283.42500	/coeff;
    CO2[	77	]=	283.50000	/coeff;
    CO2[	78	]=	283.60000	/coeff;
    CO2[	79	]=	283.72500	/coeff;
    CO2[	80	]=	283.90000	/coeff;
    CO2[	81	]=	284.07500	/coeff;
    CO2[	82	]=	284.22500	/coeff;
    CO2[	83	]=	284.40000	/coeff;
    CO2[	84	]=	284.57500	/coeff;
    CO2[	85	]=	284.72500	/coeff;
    CO2[	86	]=	284.87500	/coeff;
    CO2[	87	]=	285.00000	/coeff;
    CO2[	88	]=	285.12500	/coeff;
    CO2[	89	]=	285.27500	/coeff;
    CO2[	90	]=	285.42500	/coeff;
    CO2[	91	]=	285.57500	/coeff;
    CO2[	92	]=	285.72500	/coeff;
    CO2[	93	]=	285.90000	/coeff;
    CO2[	94	]=	286.07500	/coeff;
    CO2[	95	]=	286.22500	/coeff;
    CO2[	96	]=	286.37500	/coeff;
    CO2[	97	]=	286.50000	/coeff;
    CO2[	98	]=	286.62500	/coeff;
    CO2[	99	]=	286.77500	/coeff;
    CO2[	100	]=	286.90000	/coeff;
    CO2[	101	]=	287.00000	/coeff;
    CO2[	102	]=	287.10000	/coeff;
    CO2[	103	]=	287.22500	/coeff;
    CO2[	104	]=	287.37500	/coeff;
    CO2[	105	]=	287.52500	/coeff;
    CO2[	106	]=	287.70000	/coeff;
    CO2[	107	]=	287.90000	/coeff;
    CO2[	108	]=	288.12500	/coeff;
    CO2[	109	]=	288.40000	/coeff;
    CO2[	110	]=	288.70000	/coeff;
    CO2[	111	]=	289.02500	/coeff;
    CO2[	112	]=	289.40000	/coeff;
    CO2[	113	]=	289.80000	/coeff;
    CO2[	114	]=	290.22500	/coeff;
    CO2[	115	]=	290.70000	/coeff;
    CO2[	116	]=	291.20000	/coeff;
    CO2[	117	]=	291.67500	/coeff;
    CO2[	118	]=	292.12500	/coeff;
    CO2[	119	]=	292.57500	/coeff;
    CO2[	120	]=	292.97500	/coeff;
    CO2[	121	]=	293.30000	/coeff;
    CO2[	122	]=	293.57500	/coeff;
    CO2[	123	]=	293.80000	/coeff;
    CO2[	124	]=	294.00000	/coeff;
    CO2[	125	]=	294.17500	/coeff;
    CO2[	126	]=	294.32500	/coeff;
    CO2[	127	]=	294.47500	/coeff;
    CO2[	128	]=	294.60000	/coeff;
    CO2[	129	]=	294.70000	/coeff;
    CO2[	130	]=	294.80000	/coeff;
    CO2[	131	]=	294.90000	/coeff;
    CO2[	132	]=	295.02500	/coeff;
    CO2[	133	]=	295.22500	/coeff;
    CO2[	134	]=	295.50000	/coeff;
    CO2[	135	]=	295.80000	/coeff;
    CO2[	136	]=	296.12500	/coeff;
    CO2[	137	]=	296.47500	/coeff;
    CO2[	138	]=	296.82500	/coeff;
    CO2[	139	]=	297.20000	/coeff;
    CO2[	140	]=	297.62500	/coeff;
    CO2[	141	]=	298.07500	/coeff;
    CO2[	142	]=	298.50000	/coeff;
    CO2[	143	]=	298.90000	/coeff;
    CO2[	144	]=	299.30000	/coeff;
    CO2[	145	]=	299.70000	/coeff;
    CO2[	146	]=	300.07500	/coeff;
    CO2[	147	]=	300.42500	/coeff;
    CO2[	148	]=	300.77500	/coeff;
    CO2[	149	]=	301.10000	/coeff;
    CO2[	150	]=	301.40000	/coeff;
    CO2[	151	]=	301.72500	/coeff;
    CO2[	152	]=	302.07500	/coeff;
    CO2[	153	]=	302.40000	/coeff;
    CO2[	154	]=	302.70000	/coeff;
    CO2[	155	]=	303.02500	/coeff;
    CO2[	156	]=	303.40000	/coeff;
    CO2[	157	]=	303.77500	/coeff;
    CO2[	158	]=	304.12500	/coeff;
    CO2[	159	]=	304.52500	/coeff;
    CO2[	160	]=	304.97500	/coeff;
    CO2[	161	]=	305.40000	/coeff;
    CO2[	162	]=	305.82500	/coeff;
    CO2[	163	]=	306.30000	/coeff;
    CO2[	164	]=	306.77500	/coeff;
    CO2[	165	]=	307.22500	/coeff;
    CO2[	166	]=	307.70000	/coeff;
    CO2[	167	]=	308.17500	/coeff;
    CO2[	168	]=	308.60000	/coeff;
    CO2[	169	]=	309.00000	/coeff;
    CO2[	170	]=	309.40000	/coeff;
    CO2[	171	]=	309.75000	/coeff;
    CO2[	172	]=	310.00000	/coeff;
    CO2[	173	]=	310.17500	/coeff;
    CO2[	174	]=	310.30000	/coeff;
    CO2[	175	]=	310.37500	/coeff;
    CO2[	176	]=	310.37500	/coeff;
    CO2[	177	]=	310.30000	/coeff;
    CO2[	178	]=	310.20000	/coeff;
    CO2[	179	]=	310.12500	/coeff;
    CO2[	180	]=	310.10000	/coeff;
    CO2[	181	]=	310.12500	/coeff;
    CO2[	182	]=	310.20000	/coeff;
    CO2[	183	]=	310.32500	/coeff;
    CO2[	184	]=	310.50000	/coeff;
    CO2[	185	]=	310.75000	/coeff;
    CO2[	186	]=	311.10000	/coeff;
    CO2[	187	]=	311.50000	/coeff;
    CO2[	188	]=	311.92500	/coeff;
    CO2[	189	]=	312.42500	/coeff;
    CO2[	190	]=	313.00000	/coeff;
    CO2[	191	]=	313.60000	/coeff;
    CO2[	192	]=	314.22500	/coeff;
    CO2[	193	]=	314.84750	/coeff;
    CO2[	194	]=	315.50000	/coeff;
    CO2[	195	]=	316.27250	/coeff;
    CO2[	196	]=	317.07500	/coeff;
    CO2[	197	]=	317.79500	/coeff;
    CO2[	198	]=	318.39750	/coeff;
    CO2[	199	]=	318.92500	/coeff;
    CO2[	200	]=	319.64750	/coeff;
    CO2[	201	]=	320.64750	/coeff;
    CO2[	202	]=	321.60500	/coeff;
    CO2[	203	]=	322.63500	/coeff;
    CO2[	204	]=	323.90250	/coeff;
    CO2[	205	]=	324.98500	/coeff;
    CO2[	206	]=	325.85500	/coeff;
    CO2[	207	]=	327.14000	/coeff;
    CO2[	208	]=	328.67750	/coeff;
    CO2[	209	]=	329.74250	/coeff;
    CO2[	210	]=	330.58500	/coeff;
    CO2[	211	]=	331.74750	/coeff;
    CO2[	212	]=	333.27250	/coeff;
    CO2[	213	]=	334.84750	/coeff;
    CO2[	214	]=	336.52500	/coeff;
    CO2[	215	]=	338.36000	/coeff;
    CO2[	216	]=	339.72750	/coeff;
    CO2[	217	]=	340.79250	/coeff;
    CO2[	218	]=	342.19750	/coeff;
    CO2[	219	]=	343.78250	/coeff;
    CO2[	220	]=	345.28250	/coeff;
    CO2[	221	]=	346.79750	/coeff;
    CO2[	222	]=	348.64500	/coeff;
    CO2[	223	]=	350.73750	/coeff;
    CO2[	224	]=	352.48750	/coeff;
    CO2[	225	]=	353.85500	/coeff;
    CO2[	226	]=	355.01750	/coeff;
    CO2[	227	]=	355.88500	/coeff;
    CO2[	228	]=	356.77750	/coeff;
    CO2[	229	]=	358.12750	/coeff;
    CO2[	230	]=	359.83750	/coeff;
    CO2[	231	]=	361.46250	/coeff;
    CO2[	232	]=	363.15500	/coeff;
    CO2[	233	]=	365.32250	/coeff;
    CO2[	234	]=	367.34750	/coeff;
    CO2[	235	]=	368.86500	/coeff;
    CO2[	236	]=	370.46750	/coeff;
    CO2[	237	]=	372.52250	/coeff;
    CO2[	238	]=	374.76000	/coeff;
    CO2[	239	]=	376.81250	/coeff;
    CO2[	240	]=	378.81250	/coeff;
    CO2[	241	]=	380.82750	/coeff;
    CO2[	242	]=	382.77750	/coeff;
    CO2[	243	]=	384.80000	/coeff;
    CO2[	244	]=	386.95160	/coeff;
    CO2[	245	]=	389.12785	/coeff;
    CO2[	246	]=	391.27357	/coeff;
    CO2[	247	]=	393.42110	/coeff;
    CO2[	248	]=	395.58283	/coeff;
    CO2[	249	]=	397.76408	/coeff;
    CO2[	250	]=	399.96631	/coeff;
    CO2[	251	]=	402.18432	/coeff;
    CO2[	252	]=	404.41077	/coeff;
    CO2[	253	]=	406.64292	/coeff;
    CO2[	254	]=	408.88170	/coeff;
    CO2[	255	]=	411.12868	/coeff;
    CO2[	256	]=	413.37804	/coeff;
    CO2[	257	]=	415.63944	/coeff;
    CO2[	258	]=	417.93551	/coeff;
    CO2[	259	]=	420.27395	/coeff;
    CO2[	260	]=	422.65593	/coeff;
    CO2[	261	]=	425.07983	/coeff;
    CO2[	262	]=	427.53791	/coeff;
    CO2[	263	]=	430.02060	/coeff;
    CO2[	264	]=	432.52340	/coeff;
    CO2[	265	]=	435.04594	/coeff;
    CO2[	266	]=	437.58886	/coeff;
    CO2[	267	]=	440.13137	/coeff;
    CO2[	268	]=	442.66419	/coeff;
    CO2[	269	]=	445.20699	/coeff;
    CO2[	270	]=	447.76978	/coeff;
    CO2[	271	]=	450.35539	/coeff;
    CO2[	272	]=	452.96337	/coeff;
    CO2[	273	]=	455.58649	/coeff;
    CO2[	274	]=	458.21520	/coeff;
    CO2[	275	]=	460.84499	/coeff;
    CO2[	276	]=	463.47549	/coeff;
    CO2[	277	]=	466.09336	/coeff;
    CO2[	278	]=	468.67807	/coeff;
    CO2[	279	]=	471.23389	/coeff;
    CO2[	280	]=	473.78031	/coeff;
    CO2[	281	]=	476.32819	/coeff;
    CO2[	282	]=	478.88085	/coeff;
    CO2[	283	]=	481.43826	/coeff;
    CO2[	284	]=	483.99308	/coeff;
    CO2[	285	]=	486.53532	/coeff;
    CO2[	286	]=	489.06035	/coeff;
    CO2[	287	]=	491.53558	/coeff;
    CO2[	288	]=	493.93186	/coeff;
    CO2[	289	]=	496.24365	/coeff;
    CO2[	290	]=	498.47436	/coeff;
    CO2[	291	]=	500.64502	/coeff;
    CO2[	292	]=	502.76788	/coeff;
    CO2[	293	]=	504.84729	/coeff;
    CO2[	294	]=	506.88408	/coeff;
    CO2[	295	]=	508.87135	/coeff;
    CO2[	296	]=	510.79905	/coeff;
    CO2[	297	]=	512.64717	/coeff;
    CO2[	298	]=	514.40151	/coeff;
    CO2[	299	]=	516.06461	/coeff;
    CO2[	300	]=	517.62854	/coeff;
    CO2[	301	]=	519.09606	/coeff;
    CO2[	302	]=	520.48828	/coeff;
    CO2[	303	]=	521.81772	/coeff;
    CO2[	304	]=	523.08871	/coeff;
    CO2[	305	]=	524.30217	/coeff;
    CO2[	306	]=	525.45089	/coeff;
    CO2[	307	]=	526.50901	/coeff;
    CO2[	308	]=	527.45707	/coeff;
    CO2[	309	]=	528.29593	/coeff;
    CO2[	310	]=	529.02718	/coeff;
    CO2[	311	]=	529.64253	/coeff;
    CO2[	312	]=	530.14419	/coeff;
    CO2[	313	]=	530.55341	/coeff;
    CO2[	314	]=	530.88313	/coeff;
    CO2[	315	]=	531.13797	/coeff;
    CO2[	316	]=	531.31935	/coeff;
    CO2[	317	]=	531.48991	/coeff;
    CO2[	318	]=	531.70213	/coeff;
    CO2[	319	]=	531.94208	/coeff;
    CO2[	320	]=	532.20472	/coeff;
    CO2[	321	]=	532.48672	/coeff;
    CO2[	322	]=	532.77553	/coeff;
    CO2[	323	]=	533.06978	/coeff;
    CO2[	324	]=	533.38792	/coeff;
    CO2[	325	]=	533.74072	/coeff;
    CO2[	326	]=	534.13086	/coeff;
    CO2[	327	]=	534.55754	/coeff;
    CO2[	328	]=	535.01142	/coeff;
    CO2[	329	]=	535.47962	/coeff;
    CO2[	330	]=	535.95487	/coeff;
    CO2[	331	]=	536.43510	/coeff;
    CO2[	332	]=	536.91986	/coeff;
    CO2[	333	]=	537.39855	/coeff;
    CO2[	334	]=	537.87136	/coeff;
    CO2[	335	]=	538.35830	/coeff;
    CO2[	336	]=	538.87150	/coeff;
    CO2[	337	]=	539.38819	/coeff;
    CO2[	338	]=	539.88381	/coeff;
    CO2[	339	]=	540.35213	/coeff;
    CO2[	340	]=	540.78203	/coeff;
    CO2[	341	]=	541.16804	/coeff;
    CO2[	342	]=	541.50962	/coeff;
    CO2[	343	]=	541.80769	/coeff;
    CO2[	344	]=	542.05261	/coeff;
    CO2[	345	]=	542.24555	/coeff;
    CO2[	346	]=	542.40767	/coeff;
    CO2[	347	]=	542.55941	/coeff;
    CO2[	348	]=	542.71221	/coeff;
    CO2[	349	]=	542.86612	/coeff;
    CO2[	350	]=	543.01262	/coeff;
    CO2[	351	]=	543.13926	/coeff;
    CO2[	352	]=	543.23925	/coeff;
    CO2[	353	]=	543.31105	/coeff;
    CO2[	354	]=	543.35475	/coeff;
    CO2[	355	]=	543.35993	/coeff;
    CO2[	356	]=	543.32712	/coeff;
    CO2[	357	]=	543.28815	/coeff;
    CO2[	358	]=	543.26597	/coeff;
    CO2[	359	]=	543.26382	/coeff;
    CO2[	360	]=	543.28162	/coeff;
    CO2[	361	]=	543.31031	/coeff;
    CO2[	362	]=	543.33700	/coeff;
    CO2[	363	]=	543.35452	/coeff;
    CO2[	364	]=	543.36102	/coeff;
    CO2[	365	]=	543.35628	/coeff;
    CO2[	366	]=	543.32954	/coeff;
    CO2[	367	]=	543.28277	/coeff;
    CO2[	368	]=	543.23814	/coeff;
    CO2[	369	]=	543.20814	/coeff;
    CO2[	370	]=	543.19691	/coeff;
    CO2[	371	]=	543.20476	/coeff;
    CO2[	372	]=	543.22301	/coeff;
    CO2[	373	]=	543.23901	/coeff;
    CO2[	374	]=	543.24580	/coeff;
    CO2[	375	]=	543.24169	/coeff;
    CO2[	376	]=	543.22663	/coeff;
    CO2[	377	]=	543.18977	/coeff;
    CO2[	378	]=	543.13128	/coeff;
    CO2[	379	]=	543.07193	/coeff;
    CO2[	380	]=	543.02451	/coeff;
    CO2[	381	]=	542.99340	/coeff;
    CO2[	382	]=	542.97911	/coeff;
    CO2[	383	]=	542.97309	/coeff;
    CO2[	384	]=	542.96279	/coeff;
    CO2[	385	]=	542.95532	/coeff;
    CO2[	386	]=	542.95532	/coeff;
    CO2[	387	]=	542.95532	/coeff;
    CO2[	388	]=	542.95532	/coeff;
    CO2[	389	]=	542.95532	/coeff;
    CO2[	390	]=	542.95532	/coeff;
    CO2[	391	]=	542.95532	/coeff;
    CO2[	392	]=	542.95532	/coeff;
    CO2[	393	]=	542.95532	/coeff;
    CO2[	394	]=	542.95532	/coeff;
    CO2[	395	]=	542.95532	/coeff;
    CO2[	396	]=	542.95532	/coeff;
    CO2[	397	]=	542.95532	/coeff;
    CO2[	398	]=	542.95532	/coeff;
    CO2[	399	]=	542.95532	/coeff;
    CO2[	400	]=	542.95532	/coeff;
    CO2[	401	]=	542.95532	/coeff;
    CO2[	402	]=	542.95532	/coeff;
    CO2[	403	]=	542.95532	/coeff;
    CO2[	404	]=	542.95532	/coeff;
    CO2[	405	]=	542.95532	/coeff;
    CO2[	406	]=	542.95532	/coeff;
    CO2[	407	]=	542.95532	/coeff;
    CO2[	408	]=	542.95532	/coeff;
    CO2[	409	]=	542.95532	/coeff;
    CO2[	410	]=	542.95532	/coeff;
    CO2[	411	]=	542.95532	/coeff;
    CO2[	412	]=	542.95532	/coeff;
    CO2[	413	]=	542.95532	/coeff;
    CO2[	414	]=	542.95532	/coeff;
    CO2[	415	]=	542.95532	/coeff;
    CO2[	416	]=	542.95532	/coeff;
    CO2[	417	]=	542.95532	/coeff;
    CO2[	418	]=	542.95532	/coeff;
    CO2[	419	]=	542.95532	/coeff;
    CO2[	420	]=	542.95532	/coeff;
    CO2[	421	]=	542.95532	/coeff;
    CO2[	422	]=	542.95532	/coeff;
    CO2[	423	]=	542.95532	/coeff;
    CO2[	424	]=	542.95532	/coeff;
    CO2[	425	]=	542.95532	/coeff;
    CO2[	426	]=	542.95532	/coeff;
    CO2[	427	]=	542.95532	/coeff;
    CO2[	428	]=	542.95532	/coeff;
    CO2[	429	]=	542.95532	/coeff;
    CO2[	430	]=	542.95532	/coeff;
    CO2[	431	]=	542.95532	/coeff;
    CO2[	432	]=	542.95532	/coeff;
    CO2[	433	]=	542.95532	/coeff;
    CO2[	434	]=	542.95532	/coeff;
    CO2[	435	]=	542.95532	/coeff;
    CO2[	436	]=	542.95532	/coeff;
    CO2[	437	]=	542.95532	/coeff;
    CO2[	438	]=	542.95532	/coeff;
    CO2[	439	]=	542.95532	/coeff;
    CO2[	440	]=	542.95532	/coeff;
    CO2[	441	]=	542.95532	/coeff;
    CO2[	442	]=	542.95532	/coeff;
    CO2[	443	]=	542.95532	/coeff;
    CO2[	444	]=	542.95532	/coeff;
    CO2[	445	]=	542.95532	/coeff;
    CO2[	446	]=	542.95532	/coeff;
    CO2[	447	]=	542.95532	/coeff;
    CO2[	448	]=	542.95532	/coeff;
    CO2[	449	]=	542.95532	/coeff;
    CO2[	450	]=	542.95532	/coeff;
    CO2[	451	]=	542.95532	/coeff;
    CO2[	452	]=	542.95532	/coeff;
    CO2[	453	]=	542.95532	/coeff;
    CO2[	454	]=	542.95532	/coeff;
    CO2[	455	]=	542.95532	/coeff;
    CO2[	456	]=	542.95532	/coeff;
    CO2[	457	]=	542.95532	/coeff;
    CO2[	458	]=	542.95532	/coeff;
    CO2[	459	]=	542.95532	/coeff;
    CO2[	460	]=	542.95532	/coeff;
    CO2[	461	]=	542.95532	/coeff;
    CO2[	462	]=	542.95532	/coeff;
    CO2[	463	]=	542.95532	/coeff;
    CO2[	464	]=	542.95532	/coeff;
    CO2[	465	]=	542.95532	/coeff;
    CO2[	466	]=	542.95532	/coeff;
    CO2[	467	]=	542.95532	/coeff;
    CO2[	468	]=	542.95532	/coeff;
    CO2[	469	]=	542.95532	/coeff;
    CO2[	470	]=	542.95532	/coeff;
    CO2[	471	]=	542.95532	/coeff;
    CO2[	472	]=	542.95532	/coeff;
    CO2[	473	]=	542.95532	/coeff;
    CO2[	474	]=	542.95532	/coeff;
    CO2[	475	]=	542.95532	/coeff;
    CO2[	476	]=	542.95532	/coeff;
    CO2[	477	]=	542.95532	/coeff;
    CO2[	478	]=	542.95532	/coeff;
    CO2[	479	]=	542.95532	/coeff;
    CO2[	480	]=	542.95532	/coeff;
    CO2[	481	]=	542.95532	/coeff;
    CO2[	482	]=	542.95532	/coeff;
    CO2[	483	]=	542.95532	/coeff;
    CO2[	484	]=	542.95532	/coeff;
    CO2[	485	]=	542.95532	/coeff;
    CO2[	486	]=	542.95532	/coeff;
    CO2[	487	]=	542.95532	/coeff;
    CO2[	488	]=	542.95532	/coeff;
    CO2[	489	]=	542.95532	/coeff;
    CO2[	490	]=	542.95532	/coeff;
    CO2[	491	]=	542.95532	/coeff;
    CO2[	492	]=	542.95532	/coeff;
    CO2[	493	]=	542.95532	/coeff;
    CO2[	494	]=	542.95532	/coeff;
    CO2[	495	]=	542.95532	/coeff;
    CO2[	496	]=	542.95532	/coeff;
    CO2[	497	]=	542.95532	/coeff;
    CO2[	498	]=	542.95532	/coeff;
    CO2[	499	]=	542.95532	/coeff;
    CO2[	500	]=	542.95532	/coeff;
    CO2[	501	]=	542.95532	/coeff;
    CO2[	502	]=	542.95532	/coeff;
    CO2[	503	]=	542.95532	/coeff;
    CO2[	504	]=	542.95532	/coeff;
    CO2[	505	]=	542.95532	/coeff;
    CO2[	506	]=	542.95532	/coeff;
    CO2[	507	]=	542.95532	/coeff;
    CO2[	508	]=	542.95532	/coeff;
    CO2[	509	]=	542.95532	/coeff;
    CO2[	510	]=	542.95532	/coeff;
    CO2[	511	]=	542.95532	/coeff;
    CO2[	512	]=	542.95532	/coeff;
    CO2[	513	]=	542.95532	/coeff;
    CO2[	514	]=	542.95532	/coeff;
    CO2[	515	]=	542.95532	/coeff;
    CO2[	516	]=	542.95532	/coeff;
    CO2[	517	]=	542.95532	/coeff;
    CO2[	518	]=	542.95532	/coeff;
    CO2[	519	]=	542.95532	/coeff;
    CO2[	520	]=	542.95532	/coeff;
    CO2[	521	]=	542.95532	/coeff;
    CO2[	522	]=	542.95532	/coeff;
    CO2[	523	]=	542.95532	/coeff;
    CO2[	524	]=	542.95532	/coeff;
    CO2[	525	]=	542.95532	/coeff;
    CO2[	526	]=	542.95532	/coeff;
    CO2[	527	]=	542.95532	/coeff;
    CO2[	528	]=	542.95532	/coeff;
    CO2[	529	]=	542.95532	/coeff;
    CO2[	530	]=	542.95532	/coeff;
    CO2[	531	]=	542.95532	/coeff;
    CO2[	532	]=	542.95532	/coeff;
    CO2[	533	]=	542.95532	/coeff;
    CO2[	534	]=	542.95532	/coeff;
    CO2[	535	]=	542.95532	/coeff;
    CO2[	536	]=	542.95532	/coeff;
    CO2[	537	]=	542.95532	/coeff;
    CO2[	538	]=	542.95532	/coeff;
    CO2[	539	]=	542.95532	/coeff;
    CO2[	540	]=	542.95532	/coeff;
    CO2[	541	]=	542.95532	/coeff;
    CO2[	542	]=	542.95532	/coeff;
    CO2[	543	]=	542.95532	/coeff;
    CO2[	544	]=	542.95532	/coeff;
    CO2[	545	]=	542.95532	/coeff;
    CO2[	546	]=	542.95532	/coeff;
    CO2[	547	]=	542.95532	/coeff;
    CO2[	548	]=	542.95532	/coeff;
    CO2[	549	]=	542.95532	/coeff;
    CO2[	550	]=	542.95532	/coeff;
    CO2[	551	]=	542.95532	/coeff;
    CO2[	552	]=	542.95532	/coeff;
    CO2[	553	]=	542.95532	/coeff;
    CO2[	554	]=	542.95532	/coeff;
    CO2[	555	]=	542.95532	/coeff;
    CO2[	556	]=	542.95532	/coeff;
    CO2[	557	]=	542.95532	/coeff;
    CO2[	558	]=	542.95532	/coeff;
    CO2[	559	]=	542.95532	/coeff;
    CO2[	560	]=	542.95532	/coeff;
    CO2[	561	]=	542.95532	/coeff;
    CO2[	562	]=	542.95532	/coeff;
    CO2[	563	]=	542.95532	/coeff;
    CO2[	564	]=	542.95532	/coeff;
    CO2[	565	]=	542.95532	/coeff;
    CO2[	566	]=	542.95532	/coeff;
    CO2[	567	]=	542.95532	/coeff;
    CO2[	568	]=	542.95532	/coeff;
    CO2[	569	]=	542.95532	/coeff;
    CO2[	570	]=	542.95532	/coeff;
    CO2[	571	]=	542.95532	/coeff;
    CO2[	572	]=	542.95532	/coeff;
    CO2[	573	]=	542.95532	/coeff;
    CO2[	574	]=	542.95532	/coeff;
    CO2[	575	]=	542.95532	/coeff;
    CO2[	576	]=	542.95532	/coeff;
    CO2[	577	]=	542.95532	/coeff;
    CO2[	578	]=	542.95532	/coeff;
    CO2[	579	]=	542.95532	/coeff;
    CO2[	580	]=	542.95532	/coeff;
    CO2[	581	]=	542.95532	/coeff;
    CO2[	582	]=	542.95532	/coeff;
    CO2[	583	]=	542.95532	/coeff;
    CO2[	584	]=	542.95532	/coeff;
    CO2[	585	]=	542.95532	/coeff;
    CO2[	586	]=	542.95532	/coeff;
    CO2[	587	]=	542.95532	/coeff;
    CO2[	588	]=	542.95532	/coeff;
    CO2[	589	]=	542.95532	/coeff;
    CO2[	590	]=	542.95532	/coeff;
    CO2[	591	]=	542.95532	/coeff;
    CO2[	592	]=	542.95532	/coeff;
    CO2[	593	]=	542.95532	/coeff;
    CO2[	594	]=	542.95532	/coeff;
    CO2[	595	]=	542.95532	/coeff;
    CO2[	596	]=	542.95532	/coeff;
    CO2[	597	]=	542.95532	/coeff;
    CO2[	598	]=	542.95532	/coeff;
    CO2[	599	]=	542.95532	/coeff;
    CO2[	600	]=	542.95532	/coeff;
    CO2[	601	]=	542.95532	/coeff;
    CO2[	602	]=	542.95532	/coeff;
    CO2[	603	]=	542.95532	/coeff;
    CO2[	604	]=	542.95532	/coeff;
    CO2[	605	]=	542.95532	/coeff;
    CO2[	606	]=	542.95532	/coeff;
    CO2[	607	]=	542.95532	/coeff;
    CO2[	608	]=	542.95532	/coeff;
    CO2[	609	]=	542.95532	/coeff;
    CO2[	610	]=	542.95532	/coeff;
    CO2[	611	]=	542.95532	/coeff;
    CO2[	612	]=	542.95532	/coeff;
    CO2[	613	]=	542.95532	/coeff;
    CO2[	614	]=	542.95532	/coeff;
    CO2[	615	]=	542.95532	/coeff;
    CO2[	616	]=	542.95532	/coeff;
    CO2[	617	]=	542.95532	/coeff;
    CO2[	618	]=	542.95532	/coeff;
    CO2[	619	]=	542.95532	/coeff;
    CO2[	620	]=	542.95532	/coeff;
    CO2[	621	]=	542.95532	/coeff;
    CO2[	622	]=	542.95532	/coeff;
    CO2[	623	]=	542.95532	/coeff;
    CO2[	624	]=	542.95532	/coeff;
    CO2[	625	]=	542.95532	/coeff;
    CO2[	626	]=	542.95532	/coeff;
    CO2[	627	]=	542.95532	/coeff;
    CO2[	628	]=	542.95532	/coeff;
    CO2[	629	]=	542.95532	/coeff;
    CO2[	630	]=	542.95532	/coeff;
    CO2[	631	]=	542.95532	/coeff;
    CO2[	632	]=	542.95532	/coeff;
    CO2[	633	]=	542.95532	/coeff;
    CO2[	634	]=	542.95532	/coeff;
    CO2[	635	]=	542.95532	/coeff;
    CO2[	636	]=	542.95532	/coeff;
    CO2[	637	]=	542.95532	/coeff;
    CO2[	638	]=	542.95532	/coeff;
    CO2[	639	]=	542.95532	/coeff;
    CO2[	640	]=	542.95532	/coeff;
    CO2[	641	]=	542.95532	/coeff;
    CO2[	642	]=	542.95532	/coeff;
    CO2[	643	]=	542.95532	/coeff;
    CO2[	644	]=	542.95532	/coeff;
    CO2[	645	]=	542.95532	/coeff;
    CO2[	646	]=	542.95532	/coeff;
    CO2[	647	]=	542.95532	/coeff;
    CO2[	648	]=	542.95532	/coeff;
    CO2[	649	]=	542.95532	/coeff;
    CO2[	650	]=	542.95532	/coeff;
    CO2[	651	]=	542.95532	/coeff;
    CO2[	652	]=	542.95532	/coeff;
    CO2[	653	]=	542.95532	/coeff;
    CO2[	654	]=	542.95532	/coeff;
    CO2[	655	]=	542.95532	/coeff;
    CO2[	656	]=	542.95532	/coeff;
    CO2[	657	]=	542.95532	/coeff;
    CO2[	658	]=	542.95532	/coeff;
    CO2[	659	]=	542.95532	/coeff;
    CO2[	660	]=	542.95532	/coeff;
    CO2[	661	]=	542.95532	/coeff;
    CO2[	662	]=	542.95532	/coeff;
    CO2[	663	]=	542.95532	/coeff;
    CO2[	664	]=	542.95532	/coeff;
    CO2[	665	]=	542.95532	/coeff;
    CO2[	666	]=	542.95532	/coeff;
    CO2[	667	]=	542.95532	/coeff;
    CO2[	668	]=	542.95532	/coeff;
    CO2[	669	]=	542.95532	/coeff;
    CO2[	670	]=	542.95532	/coeff;
    CO2[	671	]=	542.95532	/coeff;
    CO2[	672	]=	542.95532	/coeff;
    CO2[	673	]=	542.95532	/coeff;
    CO2[	674	]=	542.95532	/coeff;
    CO2[	675	]=	542.95532	/coeff;
    CO2[	676	]=	542.95532	/coeff;
    CO2[	677	]=	542.95532	/coeff;
    CO2[	678	]=	542.95532	/coeff;
    CO2[	679	]=	542.95532	/coeff;
    CO2[	680	]=	542.95532	/coeff;
    CO2[	681	]=	542.95532	/coeff;
    CO2[	682	]=	542.95532	/coeff;
    CO2[	683	]=	542.95532	/coeff;
    CO2[	684	]=	542.95532	/coeff;
    CO2[	685	]=	542.95532	/coeff;
    CO2[	686	]=	542.95532	/coeff;
    CO2[	687	]=	542.95532	/coeff;
    CO2[	688	]=	542.95532	/coeff;
    CO2[	689	]=	542.95532	/coeff;
    CO2[	690	]=	542.95532	/coeff;
    CO2[	691	]=	542.95532	/coeff;
    CO2[	692	]=	542.95532	/coeff;
    CO2[	693	]=	542.95532	/coeff;
    CO2[	694	]=	542.95532	/coeff;
    CO2[	695	]=	542.95532	/coeff;
    CO2[	696	]=	542.95532	/coeff;
    CO2[	697	]=	542.95532	/coeff;
    CO2[	698	]=	542.95532	/coeff;
    CO2[	699	]=	542.95532	/coeff;
    CO2[	700	]=	542.95532	/coeff;
    CO2[	701	]=	542.95532	/coeff;
    CO2[	702	]=	542.95532	/coeff;
    CO2[	703	]=	542.95532	/coeff;
    CO2[	704	]=	542.95532	/coeff;
    CO2[	705	]=	542.95532	/coeff;
    CO2[	706	]=	542.95532	/coeff;
    CO2[	707	]=	542.95532	/coeff;
    CO2[	708	]=	542.95532	/coeff;
    CO2[	709	]=	542.95532	/coeff;
    CO2[	710	]=	542.95532	/coeff;
    CO2[	711	]=	542.95532	/coeff;
    CO2[	712	]=	542.95532	/coeff;
    CO2[	713	]=	542.95532	/coeff;
    CO2[	714	]=	542.95532	/coeff;
    CO2[	715	]=	542.95532	/coeff;
    CO2[	716	]=	542.95532	/coeff;
    CO2[	717	]=	542.95532	/coeff;
    CO2[	718	]=	542.95532	/coeff;
    CO2[	719	]=	542.95532	/coeff;
    CO2[	720	]=	542.95532	/coeff;
    CO2[	721	]=	542.95532	/coeff;
    CO2[	722	]=	542.95532	/coeff;
    CO2[	723	]=	542.95532	/coeff;
    CO2[	724	]=	542.95532	/coeff;
    CO2[	725	]=	542.95532	/coeff;
    CO2[	726	]=	542.95532	/coeff;
    CO2[	727	]=	542.95532	/coeff;
    CO2[	728	]=	542.95532	/coeff;
    CO2[	729	]=	542.95532	/coeff;
    CO2[	730	]=	542.95532	/coeff;
    CO2[	731	]=	542.95532	/coeff;
    CO2[	732	]=	542.95532	/coeff;
    CO2[	733	]=	542.95532	/coeff;
    CO2[	734	]=	542.95532	/coeff;
    CO2[	735	]=	542.95532	/coeff;
    
    
    for(int i = 736; i<tmax; i++)
    {
        CO2[i] = CO2[i-1];
    }
}

void setCO2RCP6(double coeff)
{
    CO2[	0	]=	278.05158	/coeff;
    CO2[	1	]=	278.10615	/coeff;
    CO2[	2	]=	278.22039	/coeff;
    CO2[	3	]=	278.34305	/coeff;
    CO2[	4	]=	278.47058	/coeff;
    CO2[	5	]=	278.60047	/coeff;
    CO2[	6	]=	278.73275	/coeff;
    CO2[	7	]=	278.86881	/coeff;
    CO2[	8	]=	279.00907	/coeff;
    CO2[	9	]=	279.15318	/coeff;
    CO2[	10	]=	279.30180	/coeff;
    CO2[	11	]=	279.45680	/coeff;
    CO2[	12	]=	279.61808	/coeff;
    CO2[	13	]=	279.78192	/coeff;
    CO2[	14	]=	279.94320	/coeff;
    CO2[	15	]=	280.09740	/coeff;
    CO2[	16	]=	280.24280	/coeff;
    CO2[	17	]=	280.38168	/coeff;
    CO2[	18	]=	280.51832	/coeff;
    CO2[	19	]=	280.65720	/coeff;
    CO2[	20	]=	280.80260	/coeff;
    CO2[	21	]=	280.95680	/coeff;
    CO2[	22	]=	281.11808	/coeff;
    CO2[	23	]=	281.28192	/coeff;
    CO2[	24	]=	281.44320	/coeff;
    CO2[	25	]=	281.59820	/coeff;
    CO2[	26	]=	281.74682	/coeff;
    CO2[	27	]=	281.89093	/coeff;
    CO2[	28	]=	282.03119	/coeff;
    CO2[	29	]=	282.16725	/coeff;
    CO2[	30	]=	282.29901	/coeff;
    CO2[	31	]=	282.42680	/coeff;
    CO2[	32	]=	282.55093	/coeff;
    CO2[	33	]=	282.67123	/coeff;
    CO2[	34	]=	282.78730	/coeff;
    CO2[	35	]=	282.89901	/coeff;
    CO2[	36	]=	283.00677	/coeff;
    CO2[	37	]=	283.11093	/coeff;
    CO2[	38	]=	283.21129	/coeff;
    CO2[	39	]=	283.30737	/coeff;
    CO2[	40	]=	283.39963	/coeff;
    CO2[	41	]=	283.48978	/coeff;
    CO2[	42	]=	283.57796	/coeff;
    CO2[	43	]=	283.66116	/coeff;
    CO2[	44	]=	283.73511	/coeff;
    CO2[	45	]=	283.79676	/coeff;
    CO2[	46	]=	283.84667	/coeff;
    CO2[	47	]=	283.88853	/coeff;
    CO2[	48	]=	283.92613	/coeff;
    CO2[	49	]=	283.96267	/coeff;
    CO2[	50	]=	284.00107	/coeff;
    CO2[	51	]=	284.04267	/coeff;
    CO2[	52	]=	284.08613	/coeff;
    CO2[	53	]=	284.12853	/coeff;
    CO2[	54	]=	284.16667	/coeff;
    CO2[	55	]=	284.19820	/coeff;
    CO2[	56	]=	284.22333	/coeff;
    CO2[	57	]=	284.24427	/coeff;
    CO2[	58	]=	284.26307	/coeff;
    CO2[	59	]=	284.28133	/coeff;
    CO2[	60	]=	284.30027	/coeff;
    CO2[	61	]=	284.32000	/coeff;
    CO2[	62	]=	284.34000	/coeff;
    CO2[	63	]=	284.36000	/coeff;
    CO2[	64	]=	284.38000	/coeff;
    CO2[	65	]=	284.40000	/coeff;
    CO2[	66	]=	284.38500	/coeff;
    CO2[	67	]=	284.28000	/coeff;
    CO2[	68	]=	284.12500	/coeff;
    CO2[	69	]=	283.97500	/coeff;
    CO2[	70	]=	283.82500	/coeff;
    CO2[	71	]=	283.67500	/coeff;
    CO2[	72	]=	283.52500	/coeff;
    CO2[	73	]=	283.42500	/coeff;
    CO2[	74	]=	283.40000	/coeff;
    CO2[	75	]=	283.40000	/coeff;
    CO2[	76	]=	283.42500	/coeff;
    CO2[	77	]=	283.50000	/coeff;
    CO2[	78	]=	283.60000	/coeff;
    CO2[	79	]=	283.72500	/coeff;
    CO2[	80	]=	283.90000	/coeff;
    CO2[	81	]=	284.07500	/coeff;
    CO2[	82	]=	284.22500	/coeff;
    CO2[	83	]=	284.40000	/coeff;
    CO2[	84	]=	284.57500	/coeff;
    CO2[	85	]=	284.72500	/coeff;
    CO2[	86	]=	284.87500	/coeff;
    CO2[	87	]=	285.00000	/coeff;
    CO2[	88	]=	285.12500	/coeff;
    CO2[	89	]=	285.27500	/coeff;
    CO2[	90	]=	285.42500	/coeff;
    CO2[	91	]=	285.57500	/coeff;
    CO2[	92	]=	285.72500	/coeff;
    CO2[	93	]=	285.90000	/coeff;
    CO2[	94	]=	286.07500	/coeff;
    CO2[	95	]=	286.22500	/coeff;
    CO2[	96	]=	286.37500	/coeff;
    CO2[	97	]=	286.50000	/coeff;
    CO2[	98	]=	286.62500	/coeff;
    CO2[	99	]=	286.77500	/coeff;
    CO2[	100	]=	286.90000	/coeff;
    CO2[	101	]=	287.00000	/coeff;
    CO2[	102	]=	287.10000	/coeff;
    CO2[	103	]=	287.22500	/coeff;
    CO2[	104	]=	287.37500	/coeff;
    CO2[	105	]=	287.52500	/coeff;
    CO2[	106	]=	287.70000	/coeff;
    CO2[	107	]=	287.90000	/coeff;
    CO2[	108	]=	288.12500	/coeff;
    CO2[	109	]=	288.40000	/coeff;
    CO2[	110	]=	288.70000	/coeff;
    CO2[	111	]=	289.02500	/coeff;
    CO2[	112	]=	289.40000	/coeff;
    CO2[	113	]=	289.80000	/coeff;
    CO2[	114	]=	290.22500	/coeff;
    CO2[	115	]=	290.70000	/coeff;
    CO2[	116	]=	291.20000	/coeff;
    CO2[	117	]=	291.67500	/coeff;
    CO2[	118	]=	292.12500	/coeff;
    CO2[	119	]=	292.57500	/coeff;
    CO2[	120	]=	292.97500	/coeff;
    CO2[	121	]=	293.30000	/coeff;
    CO2[	122	]=	293.57500	/coeff;
    CO2[	123	]=	293.80000	/coeff;
    CO2[	124	]=	294.00000	/coeff;
    CO2[	125	]=	294.17500	/coeff;
    CO2[	126	]=	294.32500	/coeff;
    CO2[	127	]=	294.47500	/coeff;
    CO2[	128	]=	294.60000	/coeff;
    CO2[	129	]=	294.70000	/coeff;
    CO2[	130	]=	294.80000	/coeff;
    CO2[	131	]=	294.90000	/coeff;
    CO2[	132	]=	295.02500	/coeff;
    CO2[	133	]=	295.22500	/coeff;
    CO2[	134	]=	295.50000	/coeff;
    CO2[	135	]=	295.80000	/coeff;
    CO2[	136	]=	296.12500	/coeff;
    CO2[	137	]=	296.47500	/coeff;
    CO2[	138	]=	296.82500	/coeff;
    CO2[	139	]=	297.20000	/coeff;
    CO2[	140	]=	297.62500	/coeff;
    CO2[	141	]=	298.07500	/coeff;
    CO2[	142	]=	298.50000	/coeff;
    CO2[	143	]=	298.90000	/coeff;
    CO2[	144	]=	299.30000	/coeff;
    CO2[	145	]=	299.70000	/coeff;
    CO2[	146	]=	300.07500	/coeff;
    CO2[	147	]=	300.42500	/coeff;
    CO2[	148	]=	300.77500	/coeff;
    CO2[	149	]=	301.10000	/coeff;
    CO2[	150	]=	301.40000	/coeff;
    CO2[	151	]=	301.72500	/coeff;
    CO2[	152	]=	302.07500	/coeff;
    CO2[	153	]=	302.40000	/coeff;
    CO2[	154	]=	302.70000	/coeff;
    CO2[	155	]=	303.02500	/coeff;
    CO2[	156	]=	303.40000	/coeff;
    CO2[	157	]=	303.77500	/coeff;
    CO2[	158	]=	304.12500	/coeff;
    CO2[	159	]=	304.52500	/coeff;
    CO2[	160	]=	304.97500	/coeff;
    CO2[	161	]=	305.40000	/coeff;
    CO2[	162	]=	305.82500	/coeff;
    CO2[	163	]=	306.30000	/coeff;
    CO2[	164	]=	306.77500	/coeff;
    CO2[	165	]=	307.22500	/coeff;
    CO2[	166	]=	307.70000	/coeff;
    CO2[	167	]=	308.17500	/coeff;
    CO2[	168	]=	308.60000	/coeff;
    CO2[	169	]=	309.00000	/coeff;
    CO2[	170	]=	309.40000	/coeff;
    CO2[	171	]=	309.75000	/coeff;
    CO2[	172	]=	310.00000	/coeff;
    CO2[	173	]=	310.17500	/coeff;
    CO2[	174	]=	310.30000	/coeff;
    CO2[	175	]=	310.37500	/coeff;
    CO2[	176	]=	310.37500	/coeff;
    CO2[	177	]=	310.30000	/coeff;
    CO2[	178	]=	310.20000	/coeff;
    CO2[	179	]=	310.12500	/coeff;
    CO2[	180	]=	310.10000	/coeff;
    CO2[	181	]=	310.12500	/coeff;
    CO2[	182	]=	310.20000	/coeff;
    CO2[	183	]=	310.32500	/coeff;
    CO2[	184	]=	310.50000	/coeff;
    CO2[	185	]=	310.75000	/coeff;
    CO2[	186	]=	311.10000	/coeff;
    CO2[	187	]=	311.50000	/coeff;
    CO2[	188	]=	311.92500	/coeff;
    CO2[	189	]=	312.42500	/coeff;
    CO2[	190	]=	313.00000	/coeff;
    CO2[	191	]=	313.60000	/coeff;
    CO2[	192	]=	314.22500	/coeff;
    CO2[	193	]=	314.84750	/coeff;
    CO2[	194	]=	315.50000	/coeff;
    CO2[	195	]=	316.27250	/coeff;
    CO2[	196	]=	317.07500	/coeff;
    CO2[	197	]=	317.79500	/coeff;
    CO2[	198	]=	318.39750	/coeff;
    CO2[	199	]=	318.92500	/coeff;
    CO2[	200	]=	319.64750	/coeff;
    CO2[	201	]=	320.64750	/coeff;
    CO2[	202	]=	321.60500	/coeff;
    CO2[	203	]=	322.63500	/coeff;
    CO2[	204	]=	323.90250	/coeff;
    CO2[	205	]=	324.98500	/coeff;
    CO2[	206	]=	325.85500	/coeff;
    CO2[	207	]=	327.14000	/coeff;
    CO2[	208	]=	328.67750	/coeff;
    CO2[	209	]=	329.74250	/coeff;
    CO2[	210	]=	330.58500	/coeff;
    CO2[	211	]=	331.74750	/coeff;
    CO2[	212	]=	333.27250	/coeff;
    CO2[	213	]=	334.84750	/coeff;
    CO2[	214	]=	336.52500	/coeff;
    CO2[	215	]=	338.36000	/coeff;
    CO2[	216	]=	339.72750	/coeff;
    CO2[	217	]=	340.79250	/coeff;
    CO2[	218	]=	342.19750	/coeff;
    CO2[	219	]=	343.78250	/coeff;
    CO2[	220	]=	345.28250	/coeff;
    CO2[	221	]=	346.79750	/coeff;
    CO2[	222	]=	348.64500	/coeff;
    CO2[	223	]=	350.73750	/coeff;
    CO2[	224	]=	352.48750	/coeff;
    CO2[	225	]=	353.85500	/coeff;
    CO2[	226	]=	355.01750	/coeff;
    CO2[	227	]=	355.88500	/coeff;
    CO2[	228	]=	356.77750	/coeff;
    CO2[	229	]=	358.12750	/coeff;
    CO2[	230	]=	359.83750	/coeff;
    CO2[	231	]=	361.46250	/coeff;
    CO2[	232	]=	363.15500	/coeff;
    CO2[	233	]=	365.32250	/coeff;
    CO2[	234	]=	367.34750	/coeff;
    CO2[	235	]=	368.86500	/coeff;
    CO2[	236	]=	370.46750	/coeff;
    CO2[	237	]=	372.52250	/coeff;
    CO2[	238	]=	374.76000	/coeff;
    CO2[	239	]=	376.81250	/coeff;
    CO2[	240	]=	378.81250	/coeff;
    CO2[	241	]=	380.82750	/coeff;
    CO2[	242	]=	382.77750	/coeff;
    CO2[	243	]=	384.80000	/coeff;
    CO2[	244	]=	386.93455	/coeff;
    CO2[	245	]=	389.07150	/coeff;
    CO2[	246	]=	391.16650	/coeff;
    CO2[	247	]=	393.24066	/coeff;
    CO2[	248	]=	395.29786	/coeff;
    CO2[	249	]=	397.34576	/coeff;
    CO2[	250	]=	399.38717	/coeff;
    CO2[	251	]=	401.41789	/coeff;
    CO2[	252	]=	403.43127	/coeff;
    CO2[	253	]=	405.42513	/coeff;
    CO2[	254	]=	407.40083	/coeff;
    CO2[	255	]=	409.36026	/coeff;
    CO2[	256	]=	411.29764	/coeff;
    CO2[	257	]=	413.21903	/coeff;
    CO2[	258	]=	415.14445	/coeff;
    CO2[	259	]=	417.08292	/coeff;
    CO2[	260	]=	419.03635	/coeff;
    CO2[	261	]=	421.00380	/coeff;
    CO2[	262	]=	422.97812	/coeff;
    CO2[	263	]=	424.95028	/coeff;
    CO2[	264	]=	426.91631	/coeff;
    CO2[	265	]=	428.87629	/coeff;
    CO2[	266	]=	430.83200	/coeff;
    CO2[	267	]=	432.80746	/coeff;
    CO2[	268	]=	434.83148	/coeff;
    CO2[	269	]=	436.91619	/coeff;
    CO2[	270	]=	439.06785	/coeff;
    CO2[	271	]=	441.28581	/coeff;
    CO2[	272	]=	443.56720	/coeff;
    CO2[	273	]=	445.90288	/coeff;
    CO2[	274	]=	448.28176	/coeff;
    CO2[	275	]=	450.69811	/coeff;
    CO2[	276	]=	453.15021	/coeff;
    CO2[	277	]=	455.64509	/coeff;
    CO2[	278	]=	458.18181	/coeff;
    CO2[	279	]=	460.76247	/coeff;
    CO2[	280	]=	463.40530	/coeff;
    CO2[	281	]=	466.11968	/coeff;
    CO2[	282	]=	468.90757	/coeff;
    CO2[	283	]=	471.76784	/coeff;
    CO2[	284	]=	474.69236	/coeff;
    CO2[	285	]=	477.67043	/coeff;
    CO2[	286	]=	480.69694	/coeff;
    CO2[	287	]=	483.77696	/coeff;
    CO2[	288	]=	486.91560	/coeff;
    CO2[	289	]=	490.10250	/coeff;
    CO2[	290	]=	493.33845	/coeff;
    CO2[	291	]=	496.64173	/coeff;
    CO2[	292	]=	500.02229	/coeff;
    CO2[	293	]=	503.48287	/coeff;
    CO2[	294	]=	507.02296	/coeff;
    CO2[	295	]=	510.63443	/coeff;
    CO2[	296	]=	514.30531	/coeff;
    CO2[	297	]=	518.02671	/coeff;
    CO2[	298	]=	521.79702	/coeff;
    CO2[	299	]=	525.61935	/coeff;
    CO2[	300	]=	529.48604	/coeff;
    CO2[	301	]=	533.39980	/coeff;
    CO2[	302	]=	537.38147	/coeff;
    CO2[	303	]=	541.44308	/coeff;
    CO2[	304	]=	545.58888	/coeff;
    CO2[	305	]=	549.81989	/coeff;
    CO2[	306	]=	554.12904	/coeff;
    CO2[	307	]=	558.48622	/coeff;
    CO2[	308	]=	562.86724	/coeff;
    CO2[	309	]=	567.27205	/coeff;
    CO2[	310	]=	571.70141	/coeff;
    CO2[	311	]=	576.14574	/coeff;
    CO2[	312	]=	580.60640	/coeff;
    CO2[	313	]=	585.10462	/coeff;
    CO2[	314	]=	589.65317	/coeff;
    CO2[	315	]=	594.25683	/coeff;
    CO2[	316	]=	598.91790	/coeff;
    CO2[	317	]=	603.53811	/coeff;
    CO2[	318	]=	608.01988	/coeff;
    CO2[	319	]=	612.36372	/coeff;
    CO2[	320	]=	616.57173	/coeff;
    CO2[	321	]=	620.64773	/coeff;
    CO2[	322	]=	624.58304	/coeff;
    CO2[	323	]=	628.38085	/coeff;
    CO2[	324	]=	632.06458	/coeff;
    CO2[	325	]=	635.64876	/coeff;
    CO2[	326	]=	639.14086	/coeff;
    CO2[	327	]=	642.59729	/coeff;
    CO2[	328	]=	646.06094	/coeff;
    CO2[	329	]=	649.51513	/coeff;
    CO2[	330	]=	652.95074	/coeff;
    CO2[	331	]=	656.36419	/coeff;
    CO2[	332	]=	659.75421	/coeff;
    CO2[	333	]=	663.10742	/coeff;
    CO2[	334	]=	666.42313	/coeff;
    CO2[	335	]=	669.72317	/coeff;
    CO2[	336	]=	673.02173	/coeff;
    CO2[	337	]=	676.29128	/coeff;
    CO2[	338	]=	679.50127	/coeff;
    CO2[	339	]=	682.64541	/coeff;
    CO2[	340	]=	685.71141	/coeff;
    CO2[	341	]=	688.69282	/coeff;
    CO2[	342	]=	691.58868	/coeff;
    CO2[	343	]=	694.39983	/coeff;
    CO2[	344	]=	697.11406	/coeff;
    CO2[	345	]=	699.73157	/coeff;
    CO2[	346	]=	702.27614	/coeff;
    CO2[	347	]=	704.76390	/coeff;
    CO2[	348	]=	707.20265	/coeff;
    CO2[	349	]=	709.59530	/coeff;
    CO2[	350	]=	711.93394	/coeff;
    CO2[	351	]=	714.20507	/coeff;
    CO2[	352	]=	716.40081	/coeff;
    CO2[	353	]=	718.51915	/coeff;
    CO2[	354	]=	720.56005	/coeff;
    CO2[	355	]=	722.51007	/coeff;
    CO2[	356	]=	724.36814	/coeff;
    CO2[	357	]=	726.15805	/coeff;
    CO2[	358	]=	727.89640	/coeff;
    CO2[	359	]=	729.59084	/coeff;
    CO2[	360	]=	731.24405	/coeff;
    CO2[	361	]=	732.84773	/coeff;
    CO2[	362	]=	734.38928	/coeff;
    CO2[	363	]=	735.86182	/coeff;
    CO2[	364	]=	737.26278	/coeff;
    CO2[	365	]=	738.59171	/coeff;
    CO2[	366	]=	739.83434	/coeff;
    CO2[	367	]=	740.98904	/coeff;
    CO2[	368	]=	742.07980	/coeff;
    CO2[	369	]=	743.12350	/coeff;
    CO2[	370	]=	744.12799	/coeff;
    CO2[	371	]=	745.09601	/coeff;
    CO2[	372	]=	746.01899	/coeff;
    CO2[	373	]=	746.88224	/coeff;
    CO2[	374	]=	747.67691	/coeff;
    CO2[	375	]=	748.40031	/coeff;
    CO2[	376	]=	749.05184	/coeff;
    CO2[	377	]=	749.61690	/coeff;
    CO2[	378	]=	750.09334	/coeff;
    CO2[	379	]=	750.50572	/coeff;
    CO2[	380	]=	750.87141	/coeff;
    CO2[	381	]=	751.19813	/coeff;
    CO2[	382	]=	751.48890	/coeff;
    CO2[	383	]=	751.73512	/coeff;
    CO2[	384	]=	751.92185	/coeff;
    CO2[	385	]=	751.99877	/coeff;
    CO2[	386	]=	751.99877	/coeff;
    CO2[	387	]=	751.99877	/coeff;
    CO2[	388	]=	751.99877	/coeff;
    CO2[	389	]=	751.99877	/coeff;
    CO2[	390	]=	751.99877	/coeff;
    CO2[	391	]=	751.99877	/coeff;
    CO2[	392	]=	751.99877	/coeff;
    CO2[	393	]=	751.99877	/coeff;
    CO2[	394	]=	751.99877	/coeff;
    CO2[	395	]=	751.99877	/coeff;
    CO2[	396	]=	751.99877	/coeff;
    CO2[	397	]=	751.99877	/coeff;
    CO2[	398	]=	751.99877	/coeff;
    CO2[	399	]=	751.99877	/coeff;
    CO2[	400	]=	751.99877	/coeff;
    CO2[	401	]=	751.99877	/coeff;
    CO2[	402	]=	751.99877	/coeff;
    CO2[	403	]=	751.99877	/coeff;
    CO2[	404	]=	751.99877	/coeff;
    CO2[	405	]=	751.99877	/coeff;
    CO2[	406	]=	751.99877	/coeff;
    CO2[	407	]=	751.99877	/coeff;
    CO2[	408	]=	751.99877	/coeff;
    CO2[	409	]=	751.99877	/coeff;
    CO2[	410	]=	751.99877	/coeff;
    CO2[	411	]=	751.99877	/coeff;
    CO2[	412	]=	751.99877	/coeff;
    CO2[	413	]=	751.99877	/coeff;
    CO2[	414	]=	751.99877	/coeff;
    CO2[	415	]=	751.99877	/coeff;
    CO2[	416	]=	751.99877	/coeff;
    CO2[	417	]=	751.99877	/coeff;
    CO2[	418	]=	751.99877	/coeff;
    CO2[	419	]=	751.99877	/coeff;
    CO2[	420	]=	751.99877	/coeff;
    CO2[	421	]=	751.99877	/coeff;
    CO2[	422	]=	751.99877	/coeff;
    CO2[	423	]=	751.99877	/coeff;
    CO2[	424	]=	751.99877	/coeff;
    CO2[	425	]=	751.99877	/coeff;
    CO2[	426	]=	751.99877	/coeff;
    CO2[	427	]=	751.99877	/coeff;
    CO2[	428	]=	751.99877	/coeff;
    CO2[	429	]=	751.99877	/coeff;
    CO2[	430	]=	751.99877	/coeff;
    CO2[	431	]=	751.99877	/coeff;
    CO2[	432	]=	751.99877	/coeff;
    CO2[	433	]=	751.99877	/coeff;
    CO2[	434	]=	751.99877	/coeff;
    CO2[	435	]=	751.99877	/coeff;
    CO2[	436	]=	751.99877	/coeff;
    CO2[	437	]=	751.99877	/coeff;
    CO2[	438	]=	751.99877	/coeff;
    CO2[	439	]=	751.99877	/coeff;
    CO2[	440	]=	751.99877	/coeff;
    CO2[	441	]=	751.99877	/coeff;
    CO2[	442	]=	751.99877	/coeff;
    CO2[	443	]=	751.99877	/coeff;
    CO2[	444	]=	751.99877	/coeff;
    CO2[	445	]=	751.99877	/coeff;
    CO2[	446	]=	751.99877	/coeff;
    CO2[	447	]=	751.99877	/coeff;
    CO2[	448	]=	751.99877	/coeff;
    CO2[	449	]=	751.99877	/coeff;
    CO2[	450	]=	751.99877	/coeff;
    CO2[	451	]=	751.99877	/coeff;
    CO2[	452	]=	751.99877	/coeff;
    CO2[	453	]=	751.99877	/coeff;
    CO2[	454	]=	751.99877	/coeff;
    CO2[	455	]=	751.99877	/coeff;
    CO2[	456	]=	751.99877	/coeff;
    CO2[	457	]=	751.99877	/coeff;
    CO2[	458	]=	751.99877	/coeff;
    CO2[	459	]=	751.99877	/coeff;
    CO2[	460	]=	751.99877	/coeff;
    CO2[	461	]=	751.99877	/coeff;
    CO2[	462	]=	751.99877	/coeff;
    CO2[	463	]=	751.99877	/coeff;
    CO2[	464	]=	751.99877	/coeff;
    CO2[	465	]=	751.99877	/coeff;
    CO2[	466	]=	751.99877	/coeff;
    CO2[	467	]=	751.99877	/coeff;
    CO2[	468	]=	751.99877	/coeff;
    CO2[	469	]=	751.99877	/coeff;
    CO2[	470	]=	751.99877	/coeff;
    CO2[	471	]=	751.99877	/coeff;
    CO2[	472	]=	751.99877	/coeff;
    CO2[	473	]=	751.99877	/coeff;
    CO2[	474	]=	751.99877	/coeff;
    CO2[	475	]=	751.99877	/coeff;
    CO2[	476	]=	751.99877	/coeff;
    CO2[	477	]=	751.99877	/coeff;
    CO2[	478	]=	751.99877	/coeff;
    CO2[	479	]=	751.99877	/coeff;
    CO2[	480	]=	751.99877	/coeff;
    CO2[	481	]=	751.99877	/coeff;
    CO2[	482	]=	751.99877	/coeff;
    CO2[	483	]=	751.99877	/coeff;
    CO2[	484	]=	751.99877	/coeff;
    CO2[	485	]=	751.99877	/coeff;
    CO2[	486	]=	751.99877	/coeff;
    CO2[	487	]=	751.99877	/coeff;
    CO2[	488	]=	751.99877	/coeff;
    CO2[	489	]=	751.99877	/coeff;
    CO2[	490	]=	751.99877	/coeff;
    CO2[	491	]=	751.99877	/coeff;
    CO2[	492	]=	751.99877	/coeff;
    CO2[	493	]=	751.99877	/coeff;
    CO2[	494	]=	751.99877	/coeff;
    CO2[	495	]=	751.99877	/coeff;
    CO2[	496	]=	751.99877	/coeff;
    CO2[	497	]=	751.99877	/coeff;
    CO2[	498	]=	751.99877	/coeff;
    CO2[	499	]=	751.99877	/coeff;
    CO2[	500	]=	751.99877	/coeff;
    CO2[	501	]=	751.99877	/coeff;
    CO2[	502	]=	751.99877	/coeff;
    CO2[	503	]=	751.99877	/coeff;
    CO2[	504	]=	751.99877	/coeff;
    CO2[	505	]=	751.99877	/coeff;
    CO2[	506	]=	751.99877	/coeff;
    CO2[	507	]=	751.99877	/coeff;
    CO2[	508	]=	751.99877	/coeff;
    CO2[	509	]=	751.99877	/coeff;
    CO2[	510	]=	751.99877	/coeff;
    CO2[	511	]=	751.99877	/coeff;
    CO2[	512	]=	751.99877	/coeff;
    CO2[	513	]=	751.99877	/coeff;
    CO2[	514	]=	751.99877	/coeff;
    CO2[	515	]=	751.99877	/coeff;
    CO2[	516	]=	751.99877	/coeff;
    CO2[	517	]=	751.99877	/coeff;
    CO2[	518	]=	751.99877	/coeff;
    CO2[	519	]=	751.99877	/coeff;
    CO2[	520	]=	751.99877	/coeff;
    CO2[	521	]=	751.99877	/coeff;
    CO2[	522	]=	751.99877	/coeff;
    CO2[	523	]=	751.99877	/coeff;
    CO2[	524	]=	751.99877	/coeff;
    CO2[	525	]=	751.99877	/coeff;
    CO2[	526	]=	751.99877	/coeff;
    CO2[	527	]=	751.99877	/coeff;
    CO2[	528	]=	751.99877	/coeff;
    CO2[	529	]=	751.99877	/coeff;
    CO2[	530	]=	751.99877	/coeff;
    CO2[	531	]=	751.99877	/coeff;
    CO2[	532	]=	751.99877	/coeff;
    CO2[	533	]=	751.99877	/coeff;
    CO2[	534	]=	751.99877	/coeff;
    CO2[	535	]=	751.99877	/coeff;
    CO2[	536	]=	751.99877	/coeff;
    CO2[	537	]=	751.99877	/coeff;
    CO2[	538	]=	751.99877	/coeff;
    CO2[	539	]=	751.99877	/coeff;
    CO2[	540	]=	751.99877	/coeff;
    CO2[	541	]=	751.99877	/coeff;
    CO2[	542	]=	751.99877	/coeff;
    CO2[	543	]=	751.99877	/coeff;
    CO2[	544	]=	751.99877	/coeff;
    CO2[	545	]=	751.99877	/coeff;
    CO2[	546	]=	751.99877	/coeff;
    CO2[	547	]=	751.99877	/coeff;
    CO2[	548	]=	751.99877	/coeff;
    CO2[	549	]=	751.99877	/coeff;
    CO2[	550	]=	751.99877	/coeff;
    CO2[	551	]=	751.99877	/coeff;
    CO2[	552	]=	751.99877	/coeff;
    CO2[	553	]=	751.99877	/coeff;
    CO2[	554	]=	751.99877	/coeff;
    CO2[	555	]=	751.99877	/coeff;
    CO2[	556	]=	751.99877	/coeff;
    CO2[	557	]=	751.99877	/coeff;
    CO2[	558	]=	751.99877	/coeff;
    CO2[	559	]=	751.99877	/coeff;
    CO2[	560	]=	751.99877	/coeff;
    CO2[	561	]=	751.99877	/coeff;
    CO2[	562	]=	751.99877	/coeff;
    CO2[	563	]=	751.99877	/coeff;
    CO2[	564	]=	751.99877	/coeff;
    CO2[	565	]=	751.99877	/coeff;
    CO2[	566	]=	751.99877	/coeff;
    CO2[	567	]=	751.99877	/coeff;
    CO2[	568	]=	751.99877	/coeff;
    CO2[	569	]=	751.99877	/coeff;
    CO2[	570	]=	751.99877	/coeff;
    CO2[	571	]=	751.99877	/coeff;
    CO2[	572	]=	751.99877	/coeff;
    CO2[	573	]=	751.99877	/coeff;
    CO2[	574	]=	751.99877	/coeff;
    CO2[	575	]=	751.99877	/coeff;
    CO2[	576	]=	751.99877	/coeff;
    CO2[	577	]=	751.99877	/coeff;
    CO2[	578	]=	751.99877	/coeff;
    CO2[	579	]=	751.99877	/coeff;
    CO2[	580	]=	751.99877	/coeff;
    CO2[	581	]=	751.99877	/coeff;
    CO2[	582	]=	751.99877	/coeff;
    CO2[	583	]=	751.99877	/coeff;
    CO2[	584	]=	751.99877	/coeff;
    CO2[	585	]=	751.99877	/coeff;
    CO2[	586	]=	751.99877	/coeff;
    CO2[	587	]=	751.99877	/coeff;
    CO2[	588	]=	751.99877	/coeff;
    CO2[	589	]=	751.99877	/coeff;
    CO2[	590	]=	751.99877	/coeff;
    CO2[	591	]=	751.99877	/coeff;
    CO2[	592	]=	751.99877	/coeff;
    CO2[	593	]=	751.99877	/coeff;
    CO2[	594	]=	751.99877	/coeff;
    CO2[	595	]=	751.99877	/coeff;
    CO2[	596	]=	751.99877	/coeff;
    CO2[	597	]=	751.99877	/coeff;
    CO2[	598	]=	751.99877	/coeff;
    CO2[	599	]=	751.99877	/coeff;
    CO2[	600	]=	751.99877	/coeff;
    CO2[	601	]=	751.99877	/coeff;
    CO2[	602	]=	751.99877	/coeff;
    CO2[	603	]=	751.99877	/coeff;
    CO2[	604	]=	751.99877	/coeff;
    CO2[	605	]=	751.99877	/coeff;
    CO2[	606	]=	751.99877	/coeff;
    CO2[	607	]=	751.99877	/coeff;
    CO2[	608	]=	751.99877	/coeff;
    CO2[	609	]=	751.99877	/coeff;
    CO2[	610	]=	751.99877	/coeff;
    CO2[	611	]=	751.99877	/coeff;
    CO2[	612	]=	751.99877	/coeff;
    CO2[	613	]=	751.99877	/coeff;
    CO2[	614	]=	751.99877	/coeff;
    CO2[	615	]=	751.99877	/coeff;
    CO2[	616	]=	751.99877	/coeff;
    CO2[	617	]=	751.99877	/coeff;
    CO2[	618	]=	751.99877	/coeff;
    CO2[	619	]=	751.99877	/coeff;
    CO2[	620	]=	751.99877	/coeff;
    CO2[	621	]=	751.99877	/coeff;
    CO2[	622	]=	751.99877	/coeff;
    CO2[	623	]=	751.99877	/coeff;
    CO2[	624	]=	751.99877	/coeff;
    CO2[	625	]=	751.99877	/coeff;
    CO2[	626	]=	751.99877	/coeff;
    CO2[	627	]=	751.99877	/coeff;
    CO2[	628	]=	751.99877	/coeff;
    CO2[	629	]=	751.99877	/coeff;
    CO2[	630	]=	751.99877	/coeff;
    CO2[	631	]=	751.99877	/coeff;
    CO2[	632	]=	751.99877	/coeff;
    CO2[	633	]=	751.99877	/coeff;
    CO2[	634	]=	751.99877	/coeff;
    CO2[	635	]=	751.99877	/coeff;
    CO2[	636	]=	751.99877	/coeff;
    CO2[	637	]=	751.99877	/coeff;
    CO2[	638	]=	751.99877	/coeff;
    CO2[	639	]=	751.99877	/coeff;
    CO2[	640	]=	751.99877	/coeff;
    CO2[	641	]=	751.99877	/coeff;
    CO2[	642	]=	751.99877	/coeff;
    CO2[	643	]=	751.99877	/coeff;
    CO2[	644	]=	751.99877	/coeff;
    CO2[	645	]=	751.99877	/coeff;
    CO2[	646	]=	751.99877	/coeff;
    CO2[	647	]=	751.99877	/coeff;
    CO2[	648	]=	751.99877	/coeff;
    CO2[	649	]=	751.99877	/coeff;
    CO2[	650	]=	751.99877	/coeff;
    CO2[	651	]=	751.99877	/coeff;
    CO2[	652	]=	751.99877	/coeff;
    CO2[	653	]=	751.99877	/coeff;
    CO2[	654	]=	751.99877	/coeff;
    CO2[	655	]=	751.99877	/coeff;
    CO2[	656	]=	751.99877	/coeff;
    CO2[	657	]=	751.99877	/coeff;
    CO2[	658	]=	751.99877	/coeff;
    CO2[	659	]=	751.99877	/coeff;
    CO2[	660	]=	751.99877	/coeff;
    CO2[	661	]=	751.99877	/coeff;
    CO2[	662	]=	751.99877	/coeff;
    CO2[	663	]=	751.99877	/coeff;
    CO2[	664	]=	751.99877	/coeff;
    CO2[	665	]=	751.99877	/coeff;
    CO2[	666	]=	751.99877	/coeff;
    CO2[	667	]=	751.99877	/coeff;
    CO2[	668	]=	751.99877	/coeff;
    CO2[	669	]=	751.99877	/coeff;
    CO2[	670	]=	751.99877	/coeff;
    CO2[	671	]=	751.99877	/coeff;
    CO2[	672	]=	751.99877	/coeff;
    CO2[	673	]=	751.99877	/coeff;
    CO2[	674	]=	751.99877	/coeff;
    CO2[	675	]=	751.99877	/coeff;
    CO2[	676	]=	751.99877	/coeff;
    CO2[	677	]=	751.99877	/coeff;
    CO2[	678	]=	751.99877	/coeff;
    CO2[	679	]=	751.99877	/coeff;
    CO2[	680	]=	751.99877	/coeff;
    CO2[	681	]=	751.99877	/coeff;
    CO2[	682	]=	751.99877	/coeff;
    CO2[	683	]=	751.99877	/coeff;
    CO2[	684	]=	751.99877	/coeff;
    CO2[	685	]=	751.99877	/coeff;
    CO2[	686	]=	751.99877	/coeff;
    CO2[	687	]=	751.99877	/coeff;
    CO2[	688	]=	751.99877	/coeff;
    CO2[	689	]=	751.99877	/coeff;
    CO2[	690	]=	751.99877	/coeff;
    CO2[	691	]=	751.99877	/coeff;
    CO2[	692	]=	751.99877	/coeff;
    CO2[	693	]=	751.99877	/coeff;
    CO2[	694	]=	751.99877	/coeff;
    CO2[	695	]=	751.99877	/coeff;
    CO2[	696	]=	751.99877	/coeff;
    CO2[	697	]=	751.99877	/coeff;
    CO2[	698	]=	751.99877	/coeff;
    CO2[	699	]=	751.99877	/coeff;
    CO2[	700	]=	751.99877	/coeff;
    CO2[	701	]=	751.99877	/coeff;
    CO2[	702	]=	751.99877	/coeff;
    CO2[	703	]=	751.99877	/coeff;
    CO2[	704	]=	751.99877	/coeff;
    CO2[	705	]=	751.99877	/coeff;
    CO2[	706	]=	751.99877	/coeff;
    CO2[	707	]=	751.99877	/coeff;
    CO2[	708	]=	751.99877	/coeff;
    CO2[	709	]=	751.99877	/coeff;
    CO2[	710	]=	751.99877	/coeff;
    CO2[	711	]=	751.99877	/coeff;
    CO2[	712	]=	751.99877	/coeff;
    CO2[	713	]=	751.99877	/coeff;
    CO2[	714	]=	751.99877	/coeff;
    CO2[	715	]=	751.99877	/coeff;
    CO2[	716	]=	751.99877	/coeff;
    CO2[	717	]=	751.99877	/coeff;
    CO2[	718	]=	751.99877	/coeff;
    CO2[	719	]=	751.99877	/coeff;
    CO2[	720	]=	751.99877	/coeff;
    CO2[	721	]=	751.99877	/coeff;
    CO2[	722	]=	751.99877	/coeff;
    CO2[	723	]=	751.99877	/coeff;
    CO2[	724	]=	751.99877	/coeff;
    CO2[	725	]=	751.99877	/coeff;
    CO2[	726	]=	751.99877	/coeff;
    CO2[	727	]=	751.99877	/coeff;
    CO2[	728	]=	751.99877	/coeff;
    CO2[	729	]=	751.99877	/coeff;
    CO2[	730	]=	751.99877	/coeff;
    CO2[	731	]=	751.99877	/coeff;
    CO2[	732	]=	751.99877	/coeff;
    CO2[	733	]=	751.99877	/coeff;
    CO2[	734	]=	751.99877	/coeff;
    CO2[	735	]=	751.99877	/coeff;
    
    for(int i = 736; i<tmax; i++)
    {
        CO2[i] = CO2[i-1];
    }
    
}

void setCO2RCP85(double coeff)
{
    CO2[	0	]=	278.05158	/coeff;
    CO2[	1	]=	278.10615	/coeff;
    CO2[	2	]=	278.22039	/coeff;
    CO2[	3	]=	278.34305	/coeff;
    CO2[	4	]=	278.47058	/coeff;
    CO2[	5	]=	278.60047	/coeff;
    CO2[	6	]=	278.73275	/coeff;
    CO2[	7	]=	278.86881	/coeff;
    CO2[	8	]=	279.00907	/coeff;
    CO2[	9	]=	279.15318	/coeff;
    CO2[	10	]=	279.30180	/coeff;
    CO2[	11	]=	279.45680	/coeff;
    CO2[	12	]=	279.61808	/coeff;
    CO2[	13	]=	279.78192	/coeff;
    CO2[	14	]=	279.94320	/coeff;
    CO2[	15	]=	280.09740	/coeff;
    CO2[	16	]=	280.24280	/coeff;
    CO2[	17	]=	280.38168	/coeff;
    CO2[	18	]=	280.51832	/coeff;
    CO2[	19	]=	280.65720	/coeff;
    CO2[	20	]=	280.80260	/coeff;
    CO2[	21	]=	280.95680	/coeff;
    CO2[	22	]=	281.11808	/coeff;
    CO2[	23	]=	281.28192	/coeff;
    CO2[	24	]=	281.44320	/coeff;
    CO2[	25	]=	281.59820	/coeff;
    CO2[	26	]=	281.74682	/coeff;
    CO2[	27	]=	281.89093	/coeff;
    CO2[	28	]=	282.03119	/coeff;
    CO2[	29	]=	282.16725	/coeff;
    CO2[	30	]=	282.29901	/coeff;
    CO2[	31	]=	282.42680	/coeff;
    CO2[	32	]=	282.55093	/coeff;
    CO2[	33	]=	282.67123	/coeff;
    CO2[	34	]=	282.78730	/coeff;
    CO2[	35	]=	282.89901	/coeff;
    CO2[	36	]=	283.00677	/coeff;
    CO2[	37	]=	283.11093	/coeff;
    CO2[	38	]=	283.21129	/coeff;
    CO2[	39	]=	283.30737	/coeff;
    CO2[	40	]=	283.39963	/coeff;
    CO2[	41	]=	283.48978	/coeff;
    CO2[	42	]=	283.57796	/coeff;
    CO2[	43	]=	283.66116	/coeff;
    CO2[	44	]=	283.73511	/coeff;
    CO2[	45	]=	283.79676	/coeff;
    CO2[	46	]=	283.84667	/coeff;
    CO2[	47	]=	283.88853	/coeff;
    CO2[	48	]=	283.92613	/coeff;
    CO2[	49	]=	283.96267	/coeff;
    CO2[	50	]=	284.00107	/coeff;
    CO2[	51	]=	284.04267	/coeff;
    CO2[	52	]=	284.08613	/coeff;
    CO2[	53	]=	284.12853	/coeff;
    CO2[	54	]=	284.16667	/coeff;
    CO2[	55	]=	284.19820	/coeff;
    CO2[	56	]=	284.22333	/coeff;
    CO2[	57	]=	284.24427	/coeff;
    CO2[	58	]=	284.26307	/coeff;
    CO2[	59	]=	284.28133	/coeff;
    CO2[	60	]=	284.30027	/coeff;
    CO2[	61	]=	284.32000	/coeff;
    CO2[	62	]=	284.34000	/coeff;
    CO2[	63	]=	284.36000	/coeff;
    CO2[	64	]=	284.38000	/coeff;
    CO2[	65	]=	284.40000	/coeff;
    CO2[	66	]=	284.38500	/coeff;
    CO2[	67	]=	284.28000	/coeff;
    CO2[	68	]=	284.12500	/coeff;
    CO2[	69	]=	283.97500	/coeff;
    CO2[	70	]=	283.82500	/coeff;
    CO2[	71	]=	283.67500	/coeff;
    CO2[	72	]=	283.52500	/coeff;
    CO2[	73	]=	283.42500	/coeff;
    CO2[	74	]=	283.40000	/coeff;
    CO2[	75	]=	283.40000	/coeff;
    CO2[	76	]=	283.42500	/coeff;
    CO2[	77	]=	283.50000	/coeff;
    CO2[	78	]=	283.60000	/coeff;
    CO2[	79	]=	283.72500	/coeff;
    CO2[	80	]=	283.90000	/coeff;
    CO2[	81	]=	284.07500	/coeff;
    CO2[	82	]=	284.22500	/coeff;
    CO2[	83	]=	284.40000	/coeff;
    CO2[	84	]=	284.57500	/coeff;
    CO2[	85	]=	284.72500	/coeff;
    CO2[	86	]=	284.87500	/coeff;
    CO2[	87	]=	285.00000	/coeff;
    CO2[	88	]=	285.12500	/coeff;
    CO2[	89	]=	285.27500	/coeff;
    CO2[	90	]=	285.42500	/coeff;
    CO2[	91	]=	285.57500	/coeff;
    CO2[	92	]=	285.72500	/coeff;
    CO2[	93	]=	285.90000	/coeff;
    CO2[	94	]=	286.07500	/coeff;
    CO2[	95	]=	286.22500	/coeff;
    CO2[	96	]=	286.37500	/coeff;
    CO2[	97	]=	286.50000	/coeff;
    CO2[	98	]=	286.62500	/coeff;
    CO2[	99	]=	286.77500	/coeff;
    CO2[	100	]=	286.90000	/coeff;
    CO2[	101	]=	287.00000	/coeff;
    CO2[	102	]=	287.10000	/coeff;
    CO2[	103	]=	287.22500	/coeff;
    CO2[	104	]=	287.37500	/coeff;
    CO2[	105	]=	287.52500	/coeff;
    CO2[	106	]=	287.70000	/coeff;
    CO2[	107	]=	287.90000	/coeff;
    CO2[	108	]=	288.12500	/coeff;
    CO2[	109	]=	288.40000	/coeff;
    CO2[	110	]=	288.70000	/coeff;
    CO2[	111	]=	289.02500	/coeff;
    CO2[	112	]=	289.40000	/coeff;
    CO2[	113	]=	289.80000	/coeff;
    CO2[	114	]=	290.22500	/coeff;
    CO2[	115	]=	290.70000	/coeff;
    CO2[	116	]=	291.20000	/coeff;
    CO2[	117	]=	291.67500	/coeff;
    CO2[	118	]=	292.12500	/coeff;
    CO2[	119	]=	292.57500	/coeff;
    CO2[	120	]=	292.97500	/coeff;
    CO2[	121	]=	293.30000	/coeff;
    CO2[	122	]=	293.57500	/coeff;
    CO2[	123	]=	293.80000	/coeff;
    CO2[	124	]=	294.00000	/coeff;
    CO2[	125	]=	294.17500	/coeff;
    CO2[	126	]=	294.32500	/coeff;
    CO2[	127	]=	294.47500	/coeff;
    CO2[	128	]=	294.60000	/coeff;
    CO2[	129	]=	294.70000	/coeff;
    CO2[	130	]=	294.80000	/coeff;
    CO2[	131	]=	294.90000	/coeff;
    CO2[	132	]=	295.02500	/coeff;
    CO2[	133	]=	295.22500	/coeff;
    CO2[	134	]=	295.50000	/coeff;
    CO2[	135	]=	295.80000	/coeff;
    CO2[	136	]=	296.12500	/coeff;
    CO2[	137	]=	296.47500	/coeff;
    CO2[	138	]=	296.82500	/coeff;
    CO2[	139	]=	297.20000	/coeff;
    CO2[	140	]=	297.62500	/coeff;
    CO2[	141	]=	298.07500	/coeff;
    CO2[	142	]=	298.50000	/coeff;
    CO2[	143	]=	298.90000	/coeff;
    CO2[	144	]=	299.30000	/coeff;
    CO2[	145	]=	299.70000	/coeff;
    CO2[	146	]=	300.07500	/coeff;
    CO2[	147	]=	300.42500	/coeff;
    CO2[	148	]=	300.77500	/coeff;
    CO2[	149	]=	301.10000	/coeff;
    CO2[	150	]=	301.40000	/coeff;
    CO2[	151	]=	301.72500	/coeff;
    CO2[	152	]=	302.07500	/coeff;
    CO2[	153	]=	302.40000	/coeff;
    CO2[	154	]=	302.70000	/coeff;
    CO2[	155	]=	303.02500	/coeff;
    CO2[	156	]=	303.40000	/coeff;
    CO2[	157	]=	303.77500	/coeff;
    CO2[	158	]=	304.12500	/coeff;
    CO2[	159	]=	304.52500	/coeff;
    CO2[	160	]=	304.97500	/coeff;
    CO2[	161	]=	305.40000	/coeff;
    CO2[	162	]=	305.82500	/coeff;
    CO2[	163	]=	306.30000	/coeff;
    CO2[	164	]=	306.77500	/coeff;
    CO2[	165	]=	307.22500	/coeff;
    CO2[	166	]=	307.70000	/coeff;
    CO2[	167	]=	308.17500	/coeff;
    CO2[	168	]=	308.60000	/coeff;
    CO2[	169	]=	309.00000	/coeff;
    CO2[	170	]=	309.40000	/coeff;
    CO2[	171	]=	309.75000	/coeff;
    CO2[	172	]=	310.00000	/coeff;
    CO2[	173	]=	310.17500	/coeff;
    CO2[	174	]=	310.30000	/coeff;
    CO2[	175	]=	310.37500	/coeff;
    CO2[	176	]=	310.37500	/coeff;
    CO2[	177	]=	310.30000	/coeff;
    CO2[	178	]=	310.20000	/coeff;
    CO2[	179	]=	310.12500	/coeff;
    CO2[	180	]=	310.10000	/coeff;
    CO2[	181	]=	310.12500	/coeff;
    CO2[	182	]=	310.20000	/coeff;
    CO2[	183	]=	310.32500	/coeff;
    CO2[	184	]=	310.50000	/coeff;
    CO2[	185	]=	310.75000	/coeff;
    CO2[	186	]=	311.10000	/coeff;
    CO2[	187	]=	311.50000	/coeff;
    CO2[	188	]=	311.92500	/coeff;
    CO2[	189	]=	312.42500	/coeff;
    CO2[	190	]=	313.00000	/coeff;
    CO2[	191	]=	313.60000	/coeff;
    CO2[	192	]=	314.22500	/coeff;
    CO2[	193	]=	314.84750	/coeff;
    CO2[	194	]=	315.50000	/coeff;
    CO2[	195	]=	316.27250	/coeff;
    CO2[	196	]=	317.07500	/coeff;
    CO2[	197	]=	317.79500	/coeff;
    CO2[	198	]=	318.39750	/coeff;
    CO2[	199	]=	318.92500	/coeff;
    CO2[	200	]=	319.64750	/coeff;
    CO2[	201	]=	320.64750	/coeff;
    CO2[	202	]=	321.60500	/coeff;
    CO2[	203	]=	322.63500	/coeff;
    CO2[	204	]=	323.90250	/coeff;
    CO2[	205	]=	324.98500	/coeff;
    CO2[	206	]=	325.85500	/coeff;
    CO2[	207	]=	327.14000	/coeff;
    CO2[	208	]=	328.67750	/coeff;
    CO2[	209	]=	329.74250	/coeff;
    CO2[	210	]=	330.58500	/coeff;
    CO2[	211	]=	331.74750	/coeff;
    CO2[	212	]=	333.27250	/coeff;
    CO2[	213	]=	334.84750	/coeff;
    CO2[	214	]=	336.52500	/coeff;
    CO2[	215	]=	338.36000	/coeff;
    CO2[	216	]=	339.72750	/coeff;
    CO2[	217	]=	340.79250	/coeff;
    CO2[	218	]=	342.19750	/coeff;
    CO2[	219	]=	343.78250	/coeff;
    CO2[	220	]=	345.28250	/coeff;
    CO2[	221	]=	346.79750	/coeff;
    CO2[	222	]=	348.64500	/coeff;
    CO2[	223	]=	350.73750	/coeff;
    CO2[	224	]=	352.48750	/coeff;
    CO2[	225	]=	353.85500	/coeff;
    CO2[	226	]=	355.01750	/coeff;
    CO2[	227	]=	355.88500	/coeff;
    CO2[	228	]=	356.77750	/coeff;
    CO2[	229	]=	358.12750	/coeff;
    CO2[	230	]=	359.83750	/coeff;
    CO2[	231	]=	361.46250	/coeff;
    CO2[	232	]=	363.15500	/coeff;
    CO2[	233	]=	365.32250	/coeff;
    CO2[	234	]=	367.34750	/coeff;
    CO2[	235	]=	368.86500	/coeff;
    CO2[	236	]=	370.46750	/coeff;
    CO2[	237	]=	372.52250	/coeff;
    CO2[	238	]=	374.76000	/coeff;
    CO2[	239	]=	376.81250	/coeff;
    CO2[	240	]=	378.81250	/coeff;
    CO2[	241	]=	380.82750	/coeff;
    CO2[	242	]=	382.77750	/coeff;
    CO2[	243	]=	384.80000	/coeff;
    CO2[	244	]=	387.01226	/coeff;
    CO2[	245	]=	389.32416	/coeff;
    CO2[	246	]=	391.63801	/coeff;
    CO2[	247	]=	394.00866	/coeff;
    CO2[	248	]=	396.46384	/coeff;
    CO2[	249	]=	399.00402	/coeff;
    CO2[	250	]=	401.62793	/coeff;
    CO2[	251	]=	404.32819	/coeff;
    CO2[	252	]=	407.09588	/coeff;
    CO2[	253	]=	409.92701	/coeff;
    CO2[	254	]=	412.82151	/coeff;
    CO2[	255	]=	415.78022	/coeff;
    CO2[	256	]=	418.79629	/coeff;
    CO2[	257	]=	421.86439	/coeff;
    CO2[	258	]=	424.99469	/coeff;
    CO2[	259	]=	428.19734	/coeff;
    CO2[	260	]=	431.47473	/coeff;
    CO2[	261	]=	434.82619	/coeff;
    CO2[	262	]=	438.24456	/coeff;
    CO2[	263	]=	441.72080	/coeff;
    CO2[	264	]=	445.25085	/coeff;
    CO2[	265	]=	448.83485	/coeff;
    CO2[	266	]=	452.47359	/coeff;
    CO2[	267	]=	456.17700	/coeff;
    CO2[	268	]=	459.96398	/coeff;
    CO2[	269	]=	463.85181	/coeff;
    CO2[	270	]=	467.85003	/coeff;
    CO2[	271	]=	471.96047	/coeff;
    CO2[	272	]=	476.18237	/coeff;
    CO2[	273	]=	480.50799	/coeff;
    CO2[	274	]=	484.92724	/coeff;
    CO2[	275	]=	489.43545	/coeff;
    CO2[	276	]=	494.03235	/coeff;
    CO2[	277	]=	498.72970	/coeff;
    CO2[	278	]=	503.52959	/coeff;
    CO2[	279	]=	508.43266	/coeff;
    CO2[	280	]=	513.45614	/coeff;
    CO2[	281	]=	518.61062	/coeff;
    CO2[	282	]=	523.90006	/coeff;
    CO2[	283	]=	529.32418	/coeff;
    CO2[	284	]=	534.87520	/coeff;
    CO2[	285	]=	540.54279	/coeff;
    CO2[	286	]=	546.32201	/coeff;
    CO2[	287	]=	552.21189	/coeff;
    CO2[	288	]=	558.21220	/coeff;
    CO2[	289	]=	564.31311	/coeff;
    CO2[	290	]=	570.51669	/coeff;
    CO2[	291	]=	576.84343	/coeff;
    CO2[	292	]=	583.30471	/coeff;
    CO2[	293	]=	589.90539	/coeff;
    CO2[	294	]=	596.64656	/coeff;
    CO2[	295	]=	603.52045	/coeff;
    CO2[	296	]=	610.51650	/coeff;
    CO2[	297	]=	617.60526	/coeff;
    CO2[	298	]=	624.76367	/coeff;
    CO2[	299	]=	631.99471	/coeff;
    CO2[	300	]=	639.29052	/coeff;
    CO2[	301	]=	646.65274	/coeff;
    CO2[	302	]=	654.09843	/coeff;
    CO2[	303	]=	661.64491	/coeff;
    CO2[	304	]=	669.30474	/coeff;
    CO2[	305	]=	677.07762	/coeff;
    CO2[	306	]=	684.95429	/coeff;
    CO2[	307	]=	692.90196	/coeff;
    CO2[	308	]=	700.89416	/coeff;
    CO2[	309	]=	708.93159	/coeff;
    CO2[	310	]=	717.01548	/coeff;
    CO2[	311	]=	725.13597	/coeff;
    CO2[	312	]=	733.30667	/coeff;
    CO2[	313	]=	741.52368	/coeff;
    CO2[	314	]=	749.80466	/coeff;
    CO2[	315	]=	758.18230	/coeff;
    CO2[	316	]=	766.64451	/coeff;
    CO2[	317	]=	775.17446	/coeff;
    CO2[	318	]=	783.75141	/coeff;
    CO2[	319	]=	792.36578	/coeff;
    CO2[	320	]=	801.01880	/coeff;
    CO2[	321	]=	809.71464	/coeff;
    CO2[	322	]=	818.42214	/coeff;
    CO2[	323	]=	827.15719	/coeff;
    CO2[	324	]=	835.95594	/coeff;
    CO2[	325	]=	844.80471	/coeff;
    CO2[	326	]=	853.72536	/coeff;
    CO2[	327	]=	862.72597	/coeff;
    CO2[	328	]=	871.77680	/coeff;
    CO2[	329	]=	880.86435	/coeff;
    CO2[	330	]=	889.98162	/coeff;
    CO2[	331	]=	899.12407	/coeff;
    CO2[	332	]=	908.28871	/coeff;
    CO2[	333	]=	917.47137	/coeff;
    CO2[	334	]=	926.66527	/coeff;
    CO2[	335	]=	935.87437	/coeff;
    CO2[	336	]=	945.13213	/coeff;
    CO2[	337	]=	954.46620	/coeff;
    CO2[	338	]=	963.83906	/coeff;
    CO2[	339	]=	973.24080	/coeff;
    CO2[	340	]=	982.68037	/coeff;
    CO2[	341	]=	992.14288	/coeff;
    CO2[	342	]=	1001.63110	/coeff;
    CO2[	343	]=	1011.11910	/coeff;
    CO2[	344	]=	1020.60850	/coeff;
    CO2[	345	]=	1030.10040	/coeff;
    CO2[	346	]=	1039.58920	/coeff;
    CO2[	347	]=	1049.12330	/coeff;
    CO2[	348	]=	1058.70020	/coeff;
    CO2[	349	]=	1068.32160	/coeff;
    CO2[	350	]=	1077.99950	/coeff;
    CO2[	351	]=	1087.69990	/coeff;
    CO2[	352	]=	1097.43030	/coeff;
    CO2[	353	]=	1107.17650	/coeff;
    CO2[	354	]=	1116.91220	/coeff;
    CO2[	355	]=	1126.65920	/coeff;
    CO2[	356	]=	1136.40150	/coeff;
    CO2[	357	]=	1146.13440	/coeff;
    CO2[	358	]=	1155.90640	/coeff;
    CO2[	359	]=	1165.74010	/coeff;
    CO2[	360	]=	1175.61760	/coeff;
    CO2[	361	]=	1185.52950	/coeff;
    CO2[	362	]=	1195.48330	/coeff;
    CO2[	363	]=	1205.47720	/coeff;
    CO2[	364	]=	1215.46610	/coeff;
    CO2[	365	]=	1225.45300	/coeff;
    CO2[	366	]=	1235.44930	/coeff;
    CO2[	367	]=	1245.41900	/coeff;
    CO2[	368	]=	1255.39700	/coeff;
    CO2[	369	]=	1265.42110	/coeff;
    CO2[	370	]=	1275.48430	/coeff;
    CO2[	371	]=	1285.59430	/coeff;
    CO2[	372	]=	1295.76320	/coeff;
    CO2[	373	]=	1305.96250	/coeff;
    CO2[	374	]=	1316.17080	/coeff;
    CO2[	375	]=	1326.39360	/coeff;
    CO2[	376	]=	1336.62830	/coeff;
    CO2[	377	]=	1346.85940	/coeff;
    CO2[	378	]=	1357.07270	/coeff;
    CO2[	379	]=	1367.27970	/coeff;
    CO2[	380	]=	1377.50970	/coeff;
    CO2[	381	]=	1387.79300	/coeff;
    CO2[	382	]=	1398.13760	/coeff;
    CO2[	383	]=	1408.52260	/coeff;
    CO2[	384	]=	1418.94670	/coeff;
    CO2[	385	]=	1429.39690	/coeff;
    CO2[	386	]=	1439.83540	/coeff;
    CO2[	387	]=	1450.21110	/coeff;
    CO2[	388	]=	1460.47930	/coeff;
    CO2[	389	]=	1470.59150	/coeff;
    CO2[	390	]=	1480.55640	/coeff;
    CO2[	391	]=	1490.45520	/coeff;
    CO2[	392	]=	1500.29940	/coeff;
    CO2[	393	]=	1510.05730	/coeff;
    CO2[	394	]=	1519.73320	/coeff;
    CO2[	395	]=	1529.32760	/coeff;
    CO2[	396	]=	1538.82740	/coeff;
    CO2[	397	]=	1548.22140	/coeff;
    CO2[	398	]=	1557.50250	/coeff;
    CO2[	399	]=	1566.68380	/coeff;
    CO2[	400	]=	1575.70930	/coeff;
    CO2[	401	]=	1584.57920	/coeff;
    CO2[	402	]=	1593.38900	/coeff;
    CO2[	403	]=	1602.14440	/coeff;
    CO2[	404	]=	1610.82320	/coeff;
    CO2[	405	]=	1619.41890	/coeff;
    CO2[	406	]=	1627.92830	/coeff;
    CO2[	407	]=	1636.34230	/coeff;
    CO2[	408	]=	1644.65440	/coeff;
    CO2[	409	]=	1652.86210	/coeff;
    CO2[	410	]=	1660.94720	/coeff;
    CO2[	411	]=	1668.87140	/coeff;
    CO2[	412	]=	1676.64910	/coeff;
    CO2[	413	]=	1684.34790	/coeff;
    CO2[	414	]=	1691.98550	/coeff;
    CO2[	415	]=	1699.55430	/coeff;
    CO2[	416	]=	1707.05420	/coeff;
    CO2[	417	]=	1714.46710	/coeff;
    CO2[	418	]=	1721.78570	/coeff;
    CO2[	419	]=	1728.99860	/coeff;
    CO2[	420	]=	1736.07300	/coeff;
    CO2[	421	]=	1743.02040	/coeff;
    CO2[	422	]=	1749.82720	/coeff;
    CO2[	423	]=	1756.48450	/coeff;
    CO2[	424	]=	1763.04690	/coeff;
    CO2[	425	]=	1769.54200	/coeff;
    CO2[	426	]=	1775.97200	/coeff;
    CO2[	427	]=	1782.32810	/coeff;
    CO2[	428	]=	1788.59850	/coeff;
    CO2[	429	]=	1794.76050	/coeff;
    CO2[	430	]=	1800.79990	/coeff;
    CO2[	431	]=	1806.73340	/coeff;
    CO2[	432	]=	1812.52010	/coeff;
    CO2[	433	]=	1818.14230	/coeff;
    CO2[	434	]=	1823.64980	/coeff;
    CO2[	435	]=	1829.05560	/coeff;
    CO2[	436	]=	1834.37330	/coeff;
    CO2[	437	]=	1839.61220	/coeff;
    CO2[	438	]=	1844.78120	/coeff;
    CO2[	439	]=	1849.86820	/coeff;
    CO2[	440	]=	1854.84900	/coeff;
    CO2[	441	]=	1859.71060	/coeff;
    CO2[	442	]=	1864.44690	/coeff;
    CO2[	443	]=	1869.03620	/coeff;
    CO2[	444	]=	1873.46720	/coeff;
    CO2[	445	]=	1877.78400	/coeff;
    CO2[	446	]=	1881.99470	/coeff;
    CO2[	447	]=	1886.10970	/coeff;
    CO2[	448	]=	1890.15540	/coeff;
    CO2[	449	]=	1894.13130	/coeff;
    CO2[	450	]=	1898.01230	/coeff;
    CO2[	451	]=	1901.77660	/coeff;
    CO2[	452	]=	1905.42350	/coeff;
    CO2[	453	]=	1908.96030	/coeff;
    CO2[	454	]=	1912.34450	/coeff;
    CO2[	455	]=	1915.54650	/coeff;
    CO2[	456	]=	1918.62170	/coeff;
    CO2[	457	]=	1921.61260	/coeff;
    CO2[	458	]=	1924.52270	/coeff;
    CO2[	459	]=	1927.35200	/coeff;
    CO2[	460	]=	1930.10000	/coeff;
    CO2[	461	]=	1932.75130	/coeff;
    CO2[	462	]=	1935.29260	/coeff;
    CO2[	463	]=	1937.71270	/coeff;
    CO2[	464	]=	1940.01030	/coeff;
    CO2[	465	]=	1942.15830	/coeff;
    CO2[	466	]=	1944.15720	/coeff;
    CO2[	467	]=	1946.02780	/coeff;
    CO2[	468	]=	1947.77620	/coeff;
    CO2[	469	]=	1949.43920	/coeff;
    CO2[	470	]=	1951.01210	/coeff;
    CO2[	471	]=	1952.51380	/coeff;
    CO2[	472	]=	1953.94330	/coeff;
    CO2[	473	]=	1955.27180	/coeff;
    CO2[	474	]=	1956.46040	/coeff;
    CO2[	475	]=	1957.49290	/coeff;
    CO2[	476	]=	1958.42800	/coeff;
    CO2[	477	]=	1959.18970	/coeff;
    CO2[	478	]=	1959.77070	/coeff;
    CO2[	479	]=	1960.28470	/coeff;
    CO2[	480	]=	1960.72880	/coeff;
    CO2[	481	]=	1961.06740	/coeff;
    CO2[	482	]=	1961.31940	/coeff;
    CO2[	483	]=	1961.49570	/coeff;
    CO2[	484	]=	1961.56830	/coeff;
    CO2[	485	]=	1961.57740	/coeff;
    CO2[	486	]=	1961.57740	/coeff;
    CO2[	487	]=	1961.57740	/coeff;
    CO2[	488	]=	1961.57740	/coeff;
    CO2[	489	]=	1961.57740	/coeff;
    CO2[	490	]=	1961.57740	/coeff;
    CO2[	491	]=	1961.57740	/coeff;
    CO2[	492	]=	1961.57740	/coeff;
    CO2[	493	]=	1961.57740	/coeff;
    CO2[	494	]=	1961.57740	/coeff;
    CO2[	495	]=	1961.57740	/coeff;
    CO2[	496	]=	1961.57740	/coeff;
    CO2[	497	]=	1961.57740	/coeff;
    CO2[	498	]=	1961.57740	/coeff;
    CO2[	499	]=	1961.57740	/coeff;
    CO2[	500	]=	1961.57740	/coeff;
    CO2[	501	]=	1961.57740	/coeff;
    CO2[	502	]=	1961.57740	/coeff;
    CO2[	503	]=	1961.57740	/coeff;
    CO2[	504	]=	1961.57740	/coeff;
    CO2[	505	]=	1961.57740	/coeff;
    CO2[	506	]=	1961.57740	/coeff;
    CO2[	507	]=	1961.57740	/coeff;
    CO2[	508	]=	1961.57740	/coeff;
    CO2[	509	]=	1961.57740	/coeff;
    CO2[	510	]=	1961.57740	/coeff;
    CO2[	511	]=	1961.57740	/coeff;
    CO2[	512	]=	1961.57740	/coeff;
    CO2[	513	]=	1961.57740	/coeff;
    CO2[	514	]=	1961.57740	/coeff;
    CO2[	515	]=	1961.57740	/coeff;
    CO2[	516	]=	1961.57740	/coeff;
    CO2[	517	]=	1961.57740	/coeff;
    CO2[	518	]=	1961.57740	/coeff;
    CO2[	519	]=	1961.57740	/coeff;
    CO2[	520	]=	1961.57740	/coeff;
    CO2[	521	]=	1961.57740	/coeff;
    CO2[	522	]=	1961.57740	/coeff;
    CO2[	523	]=	1961.57740	/coeff;
    CO2[	524	]=	1961.57740	/coeff;
    CO2[	525	]=	1961.57740	/coeff;
    CO2[	526	]=	1961.57740	/coeff;
    CO2[	527	]=	1961.57740	/coeff;
    CO2[	528	]=	1961.57740	/coeff;
    CO2[	529	]=	1961.57740	/coeff;
    CO2[	530	]=	1961.57740	/coeff;
    CO2[	531	]=	1961.57740	/coeff;
    CO2[	532	]=	1961.57740	/coeff;
    CO2[	533	]=	1961.57740	/coeff;
    CO2[	534	]=	1961.57740	/coeff;
    CO2[	535	]=	1961.57740	/coeff;
    CO2[	536	]=	1961.57740	/coeff;
    CO2[	537	]=	1961.57740	/coeff;
    CO2[	538	]=	1961.57740	/coeff;
    CO2[	539	]=	1961.57740	/coeff;
    CO2[	540	]=	1961.57740	/coeff;
    CO2[	541	]=	1961.57740	/coeff;
    CO2[	542	]=	1961.57740	/coeff;
    CO2[	543	]=	1961.57740	/coeff;
    CO2[	544	]=	1961.57740	/coeff;
    CO2[	545	]=	1961.57740	/coeff;
    CO2[	546	]=	1961.57740	/coeff;
    CO2[	547	]=	1961.57740	/coeff;
    CO2[	548	]=	1961.57740	/coeff;
    CO2[	549	]=	1961.57740	/coeff;
    CO2[	550	]=	1961.57740	/coeff;
    CO2[	551	]=	1961.57740	/coeff;
    CO2[	552	]=	1961.57740	/coeff;
    CO2[	553	]=	1961.57740	/coeff;
    CO2[	554	]=	1961.57740	/coeff;
    CO2[	555	]=	1961.57740	/coeff;
    CO2[	556	]=	1961.57740	/coeff;
    CO2[	557	]=	1961.57740	/coeff;
    CO2[	558	]=	1961.57740	/coeff;
    CO2[	559	]=	1961.57740	/coeff;
    CO2[	560	]=	1961.57740	/coeff;
    CO2[	561	]=	1961.57740	/coeff;
    CO2[	562	]=	1961.57740	/coeff;
    CO2[	563	]=	1961.57740	/coeff;
    CO2[	564	]=	1961.57740	/coeff;
    CO2[	565	]=	1961.57740	/coeff;
    CO2[	566	]=	1961.57740	/coeff;
    CO2[	567	]=	1961.57740	/coeff;
    CO2[	568	]=	1961.57740	/coeff;
    CO2[	569	]=	1961.57740	/coeff;
    CO2[	570	]=	1961.57740	/coeff;
    CO2[	571	]=	1961.57740	/coeff;
    CO2[	572	]=	1961.57740	/coeff;
    CO2[	573	]=	1961.57740	/coeff;
    CO2[	574	]=	1961.57740	/coeff;
    CO2[	575	]=	1961.57740	/coeff;
    CO2[	576	]=	1961.57740	/coeff;
    CO2[	577	]=	1961.57740	/coeff;
    CO2[	578	]=	1961.57740	/coeff;
    CO2[	579	]=	1961.57740	/coeff;
    CO2[	580	]=	1961.57740	/coeff;
    CO2[	581	]=	1961.57740	/coeff;
    CO2[	582	]=	1961.57740	/coeff;
    CO2[	583	]=	1961.57740	/coeff;
    CO2[	584	]=	1961.57740	/coeff;
    CO2[	585	]=	1961.57740	/coeff;
    CO2[	586	]=	1961.57740	/coeff;
    CO2[	587	]=	1961.57740	/coeff;
    CO2[	588	]=	1961.57740	/coeff;
    CO2[	589	]=	1961.57740	/coeff;
    CO2[	590	]=	1961.57740	/coeff;
    CO2[	591	]=	1961.57740	/coeff;
    CO2[	592	]=	1961.57740	/coeff;
    CO2[	593	]=	1961.57740	/coeff;
    CO2[	594	]=	1961.57740	/coeff;
    CO2[	595	]=	1961.57740	/coeff;
    CO2[	596	]=	1961.57740	/coeff;
    CO2[	597	]=	1961.57740	/coeff;
    CO2[	598	]=	1961.57740	/coeff;
    CO2[	599	]=	1961.57740	/coeff;
    CO2[	600	]=	1961.57740	/coeff;
    CO2[	601	]=	1961.57740	/coeff;
    CO2[	602	]=	1961.57740	/coeff;
    CO2[	603	]=	1961.57740	/coeff;
    CO2[	604	]=	1961.57740	/coeff;
    CO2[	605	]=	1961.57740	/coeff;
    CO2[	606	]=	1961.57740	/coeff;
    CO2[	607	]=	1961.57740	/coeff;
    CO2[	608	]=	1961.57740	/coeff;
    CO2[	609	]=	1961.57740	/coeff;
    CO2[	610	]=	1961.57740	/coeff;
    CO2[	611	]=	1961.57740	/coeff;
    CO2[	612	]=	1961.57740	/coeff;
    CO2[	613	]=	1961.57740	/coeff;
    CO2[	614	]=	1961.57740	/coeff;
    CO2[	615	]=	1961.57740	/coeff;
    CO2[	616	]=	1961.57740	/coeff;
    CO2[	617	]=	1961.57740	/coeff;
    CO2[	618	]=	1961.57740	/coeff;
    CO2[	619	]=	1961.57740	/coeff;
    CO2[	620	]=	1961.57740	/coeff;
    CO2[	621	]=	1961.57740	/coeff;
    CO2[	622	]=	1961.57740	/coeff;
    CO2[	623	]=	1961.57740	/coeff;
    CO2[	624	]=	1961.57740	/coeff;
    CO2[	625	]=	1961.57740	/coeff;
    CO2[	626	]=	1961.57740	/coeff;
    CO2[	627	]=	1961.57740	/coeff;
    CO2[	628	]=	1961.57740	/coeff;
    CO2[	629	]=	1961.57740	/coeff;
    CO2[	630	]=	1961.57740	/coeff;
    CO2[	631	]=	1961.57740	/coeff;
    CO2[	632	]=	1961.57740	/coeff;
    CO2[	633	]=	1961.57740	/coeff;
    CO2[	634	]=	1961.57740	/coeff;
    CO2[	635	]=	1961.57740	/coeff;
    CO2[	636	]=	1961.57740	/coeff;
    CO2[	637	]=	1961.57740	/coeff;
    CO2[	638	]=	1961.57740	/coeff;
    CO2[	639	]=	1961.57740	/coeff;
    CO2[	640	]=	1961.57740	/coeff;
    CO2[	641	]=	1961.57740	/coeff;
    CO2[	642	]=	1961.57740	/coeff;
    CO2[	643	]=	1961.57740	/coeff;
    CO2[	644	]=	1961.57740	/coeff;
    CO2[	645	]=	1961.57740	/coeff;
    CO2[	646	]=	1961.57740	/coeff;
    CO2[	647	]=	1961.57740	/coeff;
    CO2[	648	]=	1961.57740	/coeff;
    CO2[	649	]=	1961.57740	/coeff;
    CO2[	650	]=	1961.57740	/coeff;
    CO2[	651	]=	1961.57740	/coeff;
    CO2[	652	]=	1961.57740	/coeff;
    CO2[	653	]=	1961.57740	/coeff;
    CO2[	654	]=	1961.57740	/coeff;
    CO2[	655	]=	1961.57740	/coeff;
    CO2[	656	]=	1961.57740	/coeff;
    CO2[	657	]=	1961.57740	/coeff;
    CO2[	658	]=	1961.57740	/coeff;
    CO2[	659	]=	1961.57740	/coeff;
    CO2[	660	]=	1961.57740	/coeff;
    CO2[	661	]=	1961.57740	/coeff;
    CO2[	662	]=	1961.57740	/coeff;
    CO2[	663	]=	1961.57740	/coeff;
    CO2[	664	]=	1961.57740	/coeff;
    CO2[	665	]=	1961.57740	/coeff;
    CO2[	666	]=	1961.57740	/coeff;
    CO2[	667	]=	1961.57740	/coeff;
    CO2[	668	]=	1961.57740	/coeff;
    CO2[	669	]=	1961.57740	/coeff;
    CO2[	670	]=	1961.57740	/coeff;
    CO2[	671	]=	1961.57740	/coeff;
    CO2[	672	]=	1961.57740	/coeff;
    CO2[	673	]=	1961.57740	/coeff;
    CO2[	674	]=	1961.57740	/coeff;
    CO2[	675	]=	1961.57740	/coeff;
    CO2[	676	]=	1961.57740	/coeff;
    CO2[	677	]=	1961.57740	/coeff;
    CO2[	678	]=	1961.57740	/coeff;
    CO2[	679	]=	1961.57740	/coeff;
    CO2[	680	]=	1961.57740	/coeff;
    CO2[	681	]=	1961.57740	/coeff;
    CO2[	682	]=	1961.57740	/coeff;
    CO2[	683	]=	1961.57740	/coeff;
    CO2[	684	]=	1961.57740	/coeff;
    CO2[	685	]=	1961.57740	/coeff;
    CO2[	686	]=	1961.57740	/coeff;
    CO2[	687	]=	1961.57740	/coeff;
    CO2[	688	]=	1961.57740	/coeff;
    CO2[	689	]=	1961.57740	/coeff;
    CO2[	690	]=	1961.57740	/coeff;
    CO2[	691	]=	1961.57740	/coeff;
    CO2[	692	]=	1961.57740	/coeff;
    CO2[	693	]=	1961.57740	/coeff;
    CO2[	694	]=	1961.57740	/coeff;
    CO2[	695	]=	1961.57740	/coeff;
    CO2[	696	]=	1961.57740	/coeff;
    CO2[	697	]=	1961.57740	/coeff;
    CO2[	698	]=	1961.57740	/coeff;
    CO2[	699	]=	1961.57740	/coeff;
    CO2[	700	]=	1961.57740	/coeff;
    CO2[	701	]=	1961.57740	/coeff;
    CO2[	702	]=	1961.57740	/coeff;
    CO2[	703	]=	1961.57740	/coeff;
    CO2[	704	]=	1961.57740	/coeff;
    CO2[	705	]=	1961.57740	/coeff;
    CO2[	706	]=	1961.57740	/coeff;
    CO2[	707	]=	1961.57740	/coeff;
    CO2[	708	]=	1961.57740	/coeff;
    CO2[	709	]=	1961.57740	/coeff;
    CO2[	710	]=	1961.57740	/coeff;
    CO2[	711	]=	1961.57740	/coeff;
    CO2[	712	]=	1961.57740	/coeff;
    CO2[	713	]=	1961.57740	/coeff;
    CO2[	714	]=	1961.57740	/coeff;
    CO2[	715	]=	1961.57740	/coeff;
    CO2[	716	]=	1961.57740	/coeff;
    CO2[	717	]=	1961.57740	/coeff;
    CO2[	718	]=	1961.57740	/coeff;
    CO2[	719	]=	1961.57740	/coeff;
    CO2[	720	]=	1961.57740	/coeff;
    CO2[	721	]=	1961.57740	/coeff;
    CO2[	722	]=	1961.57740	/coeff;
    CO2[	723	]=	1961.57740	/coeff;
    CO2[	724	]=	1961.57740	/coeff;
    CO2[	725	]=	1961.57740	/coeff;
    CO2[	726	]=	1961.57740	/coeff;
    CO2[	727	]=	1961.57740	/coeff;
    CO2[	728	]=	1961.57740	/coeff;
    CO2[	729	]=	1961.57740	/coeff;
    CO2[	730	]=	1961.57740	/coeff;
    CO2[	731	]=	1961.57740	/coeff;
    CO2[	732	]=	1961.57740	/coeff;
    CO2[	733	]=	1961.57740	/coeff;
    CO2[	734	]=	1961.57740	/coeff;
    CO2[	735	]=	1961.57740	/coeff;
    
    for(int i = 736; i<tmax; i++)
    {
        CO2[i] = CO2[i-1];
    }
    
}


void setIemB1()
{
    
    I_em[	0	]=	0	;
    I_em[	1	]=	0.05964	;
    I_em[	2	]=	0.1398	;
    I_em[	3	]=	0.22596	;
    I_em[	4	]=	0.31704	;
    I_em[	5	]=	0.4272	;
    I_em[	6	]=	0.54372	;
    I_em[	7	]=	0.65484	;
    I_em[	8	]=	0.79512	;
    I_em[	9	]=	0.93084	;
    I_em[	10	]=	1.0836	;
    I_em[	11	]=	1.24404	;
    I_em[	12	]=	1.41072	;
    I_em[	13	]=	1.596	;
    I_em[	14	]=	1.78764	;
    I_em[	15	]=	1.98612	;
    I_em[	16	]=	2.20344	;
    I_em[	17	]=	2.42736	;
    I_em[	18	]=	2.65788	;
    I_em[	19	]=	2.9076	;
    I_em[	20	]=	3.16428	;
    I_em[	21	]=	3.42744	;
    I_em[	22	]=	3.71016	;
    I_em[	23	]=	3.99996	;
    I_em[	24	]=	4.3086	;
    I_em[	25	]=	4.62492	;
    I_em[	26	]=	4.9482	;
    I_em[	27	]=	5.2788	;
    I_em[	28	]=	5.6286	;
    I_em[	29	]=	5.99808	;
    I_em[	30	]=	6.37368	;
    I_em[	31	]=	6.75768	;
    I_em[	32	]=	7.1604	;
    I_em[	33	]=	7.57068	;
    I_em[	34	]=	7.99836	;
    I_em[	35	]=	8.43192	;
    I_em[	36	]=	8.86992	;
    I_em[	37	]=	9.32244	;
    I_em[	38	]=	9.79008	;
    I_em[	39	]=	10.24692	;
    I_em[	40	]=	10.72944	;
    I_em[	41	]=	11.19828	;
    I_em[	42	]=	11.6814	;
    I_em[	43	]=	12.16056	;
    I_em[	44	]=	12.63504	;
    I_em[	45	]=	13.12128	;
    I_em[	46	]=	13.60092	;
    I_em[	47	]=	14.08596	;
    I_em[	48	]=	14.57832	;
    I_em[	49	]=	15.06552	;
    I_em[	50	]=	15.53028	;
    I_em[	51	]=	15.987	;
    I_em[	52	]=	16.44636	;
    I_em[	53	]=	16.88448	;
    I_em[	54	]=	17.31324	;
    I_em[	55	]=	17.74812	;
    I_em[	56	]=	18.1638	;
    I_em[	57	]=	18.5868	;
    I_em[	58	]=	18.993	;
    I_em[	59	]=	19.39536	;
    I_em[	60	]=	19.79844	;
    I_em[	61	]=	20.18088	;
    I_em[	62	]=	20.5818	;
    I_em[	63	]=	20.97864	;
    I_em[	64	]=	21.39036	;
    I_em[	65	]=	21.7956	;
    I_em[	66	]=	22.19124	;
    I_em[	67	]=	22.59084	;
    I_em[	68	]=	23.01876	;
    I_em[	69	]=	23.4558	;
    I_em[	70	]=	23.89236	;
    I_em[	71	]=	24.34584	;
    I_em[	72	]=	24.81348	;
    I_em[	73	]=	25.28928	;
    I_em[	74	]=	25.79352	;
    I_em[	75	]=	26.29644	;
    I_em[	76	]=	26.82348	;
    I_em[	77	]=	27.36588	;
    I_em[	78	]=	27.91392	;
    I_em[	79	]=	28.45692	;
    I_em[	80	]=	29.01024	;
    I_em[	81	]=	29.56068	;
    I_em[	82	]=	30.10248	;
    I_em[	83	]=	30.64116	;
    I_em[	84	]=	31.1826	;
    I_em[	85	]=	31.70844	;
    I_em[	86	]=	32.22432	;
    I_em[	87	]=	32.74824	;
    I_em[	88	]=	33.261	;
    I_em[	89	]=	33.7584	;
    I_em[	90	]=	34.24728	;
    I_em[	91	]=	34.7394	;
    I_em[	92	]=	35.2296	;
    I_em[	93	]=	35.70468	;
    I_em[	94	]=	36.18876	;
    I_em[	95	]=	36.67728	;
    I_em[	96	]=	37.15356	;
    I_em[	97	]=	37.64484	;
    I_em[	98	]=	38.14512	;
    I_em[	99	]=	38.65104	;
    I_em[	100	]=	39.16596	;
    I_em[	101	]=	39.69048	;
    I_em[	102	]=	40.25304	;
    I_em[	103	]=	40.81452	;
    I_em[	104	]=	41.38848	;
    I_em[	105	]=	41.98668	;
    I_em[	106	]=	42.59844	;
    I_em[	107	]=	43.22364	;
    I_em[	108	]=	43.87644	;
    I_em[	109	]=	44.5446	;
    I_em[	110	]=	45.24132	;
    I_em[	111	]=	45.96576	;
    I_em[	112	]=	46.70664	;
    I_em[	113	]=	47.47488	;
    I_em[	114	]=	48.26052	;
    I_em[	115	]=	49.07304	;
    I_em[	116	]=	49.89756	;
    I_em[	117	]=	50.74668	;
    I_em[	118	]=	51.6174	;
    I_em[	119	]=	52.52496	;
    I_em[	120	]=	53.44236	;
    I_em[	121	]=	54.37872	;
    I_em[	122	]=	55.31028	;
    I_em[	123	]=	56.27124	;
    I_em[	124	]=	57.23604	;
    I_em[	125	]=	58.21404	;
    I_em[	126	]=	59.19108	;
    I_em[	127	]=	60.16548	;
    I_em[	128	]=	61.13448	;
    I_em[	129	]=	62.09316	;
    I_em[	130	]=	63.05544	;
    I_em[	131	]=	64.00908	;
    I_em[	132	]=	64.97076	;
    I_em[	133	]=	65.9184	;
    I_em[	134	]=	66.8808	;
    I_em[	135	]=	67.85916	;
    I_em[	136	]=	68.84688	;
    I_em[	137	]=	69.84612	;
    I_em[	138	]=	70.8606	;
    I_em[	139	]=	71.90436	;
    I_em[	140	]=	72.978	;
    I_em[	141	]=	74.06772	;
    I_em[	142	]=	75.1842	;
    I_em[	143	]=	76.31172	;
    I_em[	144	]=	77.44872	;
    I_em[	145	]=	78.61656	;
    I_em[	146	]=	79.77912	;
    I_em[	147	]=	80.96988	;
    I_em[	148	]=	82.16496	;
    I_em[	149	]=	83.37708	;
    I_em[	150	]=	84.59028	;
    I_em[	151	]=	85.81836	;
    I_em[	152	]=	87.0486	;
    I_em[	153	]=	88.29108	;
    I_em[	154	]=	89.53416	;
    I_em[	155	]=	90.77748	;
    I_em[	156	]=	92.04228	;
    I_em[	157	]=	93.29652	;
    I_em[	158	]=	94.55892	;
    I_em[	159	]=	95.81508	;
    I_em[	160	]=	97.07772	;
    I_em[	161	]=	98.33064	;
    I_em[	162	]=	99.5736	;
    I_em[	163	]=	100.80156	;
    I_em[	164	]=	102.04488	;
    I_em[	165	]=	103.25256	;
    I_em[	166	]=	104.46156	;
    I_em[	167	]=	105.66264	;
    I_em[	168	]=	106.86984	;
    I_em[	169	]=	108.05976	;
    I_em[	170	]=	109.26132	;
    I_em[	171	]=	110.46276	;
    I_em[	172	]=	111.66828	;
    I_em[	173	]=	112.8816	;
    I_em[	174	]=	114.10488	;
    I_em[	175	]=	115.3536	;
    I_em[	176	]=	116.60928	;
    I_em[	177	]=	117.89436	;
    I_em[	178	]=	119.21808	;
    I_em[	179	]=	120.5664	;
    I_em[	180	]=	121.95792	;
    I_em[	181	]=	123.37824	;
    I_em[	182	]=	124.85712	;
    I_em[	183	]=	126.36912	;
    I_em[	184	]=	127.93056	;
    I_em[	185	]=	129.5412	;
    I_em[	186	]=	131.21652	;
    I_em[	187	]=	132.93384	;
    I_em[	188	]=	134.721	;
    I_em[	189	]=	136.56708	;
    I_em[	190	]=	138.48744	;
    I_em[	191	]=	140.4726	;
    I_em[	192	]=	142.53708	;
    I_em[	193	]=	144.6876	;
    I_em[	194	]=	146.97636	;
    I_em[	195	]=	149.69388	;
    I_em[	196	]=	152.28828	;
    I_em[	197	]=	154.93656	;
    I_em[	198	]=	157.24776	;
    I_em[	199	]=	159.33492	;
    I_em[	200	]=	161.59884	;
    I_em[	201	]=	164.87568	;
    I_em[	202	]=	167.87496	;
    I_em[	203	]=	171.05784	;
    I_em[	204	]=	175.10496	;
    I_em[	205	]=	178.72056	;
    I_em[	206	]=	181.67772	;
    I_em[	207	]=	185.3754	;
    I_em[	208	]=	190.51704	;
    I_em[	209	]=	194.08548	;
    I_em[	210	]=	197.26212	;
    I_em[	211	]=	200.862	;
    I_em[	212	]=	205.59096	;
    I_em[	213	]=	210.60324	;
    I_em[	214	]=	215.30448	;
    I_em[	215	]=	220.44912	;
    I_em[	216	]=	225.07044	;
    I_em[	217	]=	229.32912	;
    I_em[	218	]=	234.34128	;
    I_em[	219	]=	239.66664	;
    I_em[	220	]=	244.60728	;
    I_em[	221	]=	249.29916	;
    I_em[	222	]=	254.93004	;
    I_em[	223	]=	261.65832	;
    I_em[	224	]=	267.27372	;
    I_em[	225	]=	272.10456	;
    I_em[	226	]=	276.95856	;
    I_em[	227	]=	280.80648	;
    I_em[	228	]=	284.45292	;
    I_em[	229	]=	289.51176	;
    I_em[	230	]=	294.01644	;
    I_em[	231	]=	299.29944	;
    I_em[	232	]=	304.23168	;
    I_em[	233	]=	311.2686	;
    I_em[	234	]=	318.06756	;
    I_em[	235	]=	323.31228	;
    I_em[	236	]=	328.76628	;
    I_em[	237	]=	335.19948	;
    I_em[	238	]=	342.56232	;
    I_em[	239	]=	349.15008	;
    I_em[	240	]=	355.75632	;
    I_em[	241	]=	362.64336	;
    I_em[	242	]=	369.14064	;
    I_em[	243	]=	375.83124	;
    I_em[	244	]=	381.7788	;
    I_em[	245	]=	389.15496	;
    I_em[	246	]=	398.57916	;
    I_em[	247	]=	408.13344	;
    I_em[	248	]=	417.82572	;
    I_em[	249	]=	427.64328	;
    I_em[	250	]=	437.60232	;
    I_em[	251	]=	447.69672	;
    I_em[	252	]=	457.9332	;
    I_em[	253	]=	468.29316	;
    I_em[	254	]=	478.79112	;
    I_em[	255	]=	489.40452	;
    I_em[	256	]=	500.09304	;
    I_em[	257	]=	510.81924	;
    I_em[	258	]=	521.60208	;
    I_em[	259	]=	532.41972	;
    I_em[	260	]=	543.2898	;
    I_em[	261	]=	554.21028	;
    I_em[	262	]=	565.18656	;
    I_em[	263	]=	576.19752	;
    I_em[	264	]=	587.26176	;
    I_em[	265	]=	598.3752	;
    I_em[	266	]=	609.54888	;
    I_em[	267	]=	620.77332	;
    I_em[	268	]=	632.07012	;
    I_em[	269	]=	643.43124	;
    I_em[	270	]=	654.84444	;
    I_em[	271	]=	666.32304	;
    I_em[	272	]=	677.86716	;
    I_em[	273	]=	689.4642	;
    I_em[	274	]=	701.12724	;
    I_em[	275	]=	712.83156	;
    I_em[	276	]=	724.50948	;
    I_em[	277	]=	736.14636	;
    I_em[	278	]=	747.73176	;
    I_em[	279	]=	759.28188	;
    I_em[	280	]=	770.79816	;
    I_em[	281	]=	782.25936	;
    I_em[	282	]=	793.67844	;
    I_em[	283	]=	805.05696	;
    I_em[	284	]=	816.3954	;
    I_em[	285	]=	827.66088	;
    I_em[	286	]=	838.79736	;
    I_em[	287	]=	849.78036	;
    I_em[	288	]=	860.613	;
    I_em[	289	]=	871.28376	;
    I_em[	290	]=	881.79552	;
    I_em[	291	]=	892.15896	;
    I_em[	292	]=	902.36472	;
    I_em[	293	]=	912.41316	;
    I_em[	294	]=	922.31664	;
    I_em[	295	]=	932.05716	;
    I_em[	296	]=	941.64252	;
    I_em[	297	]=	951.07536	;
    I_em[	298	]=	960.34992	;
    I_em[	299	]=	969.46152	;
    I_em[	300	]=	978.42072	;
    I_em[	301	]=	987.22632	;
    I_em[	302	]=	995.87712	;
    I_em[	303	]=	1004.36832	;
    I_em[	304	]=	1012.69512	;
    I_em[	305	]=	1020.88032	;
    I_em[	306	]=	1028.90952	;
    I_em[	307	]=	1036.78632	;
    I_em[	308	]=	1044.51792	;
    I_em[	309	]=	1052.10672	;
    I_em[	310	]=	1059.54192	;
    I_em[	311	]=	1066.83192	;
    I_em[	312	]=	1073.98392	;
    I_em[	313	]=	1080.97272	;
    I_em[	314	]=	1087.82832	;
    I_em[	315	]=	1094.52552	;
    I_em[	316	]=	1101.08592	;
    I_em[	317	]=	1107.51432	;
    I_em[	318	]=	1113.79752	;
    I_em[	319	]=	1119.95112	;
    I_em[	320	]=	1125.95712	;
    I_em[	321	]=	1131.83232	;
    I_em[	322	]=	1137.56352	;
    I_em[	323	]=	1143.16032	;
    I_em[	324	]=	1148.62512	;
    I_em[	325	]=	1153.95672	;
    I_em[	326	]=	1159.16952	;
    I_em[	327	]=	1164.26352	;
    I_em[	328	]=	1169.26152	;
    I_em[	329	]=	1174.13952	;
    I_em[	330	]=	1178.92032	;
    I_em[	331	]=	1183.58232	;
    I_em[	332	]=	1188.13872	;
    I_em[	333	]=	1192.58952	;
    I_em[	334	]=	1196.93352	;
    I_em[	335	]=	1201.16352	;
    I_em[	336	]=	1205.31912	;
    I_em[	337	]=	1209.37992	;
    I_em[	338	]=	1213.35792	;
    I_em[	339	]=	1217.24592	;
    I_em[	340	]=	1221.05352	;
    I_em[	341	]=	1224.77472	;
    I_em[	342	]=	1228.42152	;
    I_em[	343	]=	1231.96872	;
    I_em[	344	]=	1235.44512	;
    I_em[	345	]=	1238.81952	;
    I_em[	346	]=	1242.12432	;
    I_em[	347	]=	1245.34272	;
    I_em[	348	]=	1248.47352	;
    I_em[	349	]=	1251.52032	;
    I_em[	350	]=	1254.48072	;
    I_em[	351	]=	1257.35592	;
    I_em[	352	]=	1260.14112	;
    I_em[	353	]=	1262.85432	;
    I_em[	354	]=	1265.48112	;
    I_em[	355	]=	1268.01792	;
    I_em[	356	]=	1270.47072	;
    I_em[	357	]=	1272.83712	;
    I_em[	358	]=	1275.13032	;
    I_em[	359	]=	1277.32032	;
    I_em[	360	]=	1279.43832	;
    I_em[	361	]=	1281.46992	;
    I_em[	362	]=	1283.42352	;
    I_em[	363	]=	1285.28232	;
    I_em[	364	]=	1287.06192	;
    I_em[	365	]=	1288.74792	;
    I_em[	366	]=	1290.36072	;
    I_em[	367	]=	1291.87752	;
    I_em[	368	]=	1293.32112	;
    I_em[	369	]=	1294.67832	;
    I_em[	370	]=	1295.94552	;
    I_em[	371	]=	1297.12752	;
    I_em[	372	]=	1298.22432	;
    I_em[	373	]=	1299.24552	;
    I_em[	374	]=	1300.16712	;
    I_em[	375	]=	1301.01912	;
    I_em[	376	]=	1301.77752	;
    I_em[	377	]=	1302.46032	;
    I_em[	378	]=	1303.04352	;
    I_em[	379	]=	1303.55232	;
    I_em[	380	]=	1303.98672	;
    I_em[	381	]=	1304.31792	;
    I_em[	382	]=	1304.57712	;
    I_em[	383	]=	1304.73912	;
    I_em[	384	]=	1304.82792	;
    I_em[	385	]=	1304.84232	;
    
    for(int i = 386; i<tmax; i++)
    {
        I_em[i] = I_em[i-1];
    }
}

void setIemA1T()//A1T
{
    I_em[	0	]=	0	;
    I_em[	1	]=	0.05964	;
    I_em[	2	]=	0.1398	;
    I_em[	3	]=	0.22596	;
    I_em[	4	]=	0.31704	;
    I_em[	5	]=	0.4272	;
    I_em[	6	]=	0.54372	;
    I_em[	7	]=	0.65484	;
    I_em[	8	]=	0.79512	;
    I_em[	9	]=	0.93084	;
    I_em[	10	]=	1.0836	;
    I_em[	11	]=	1.24404	;
    I_em[	12	]=	1.41072	;
    I_em[	13	]=	1.596	;
    I_em[	14	]=	1.78764	;
    I_em[	15	]=	1.98612	;
    I_em[	16	]=	2.20344	;
    I_em[	17	]=	2.42736	;
    I_em[	18	]=	2.65788	;
    I_em[	19	]=	2.9076	;
    I_em[	20	]=	3.16428	;
    I_em[	21	]=	3.42744	;
    I_em[	22	]=	3.71016	;
    I_em[	23	]=	3.99996	;
    I_em[	24	]=	4.3086	;
    I_em[	25	]=	4.62492	;
    I_em[	26	]=	4.9482	;
    I_em[	27	]=	5.2788	;
    I_em[	28	]=	5.6286	;
    I_em[	29	]=	5.99808	;
    I_em[	30	]=	6.37368	;
    I_em[	31	]=	6.75768	;
    I_em[	32	]=	7.1604	;
    I_em[	33	]=	7.57068	;
    I_em[	34	]=	7.99836	;
    I_em[	35	]=	8.43192	;
    I_em[	36	]=	8.86992	;
    I_em[	37	]=	9.32244	;
    I_em[	38	]=	9.79008	;
    I_em[	39	]=	10.24692	;
    I_em[	40	]=	10.72944	;
    I_em[	41	]=	11.19828	;
    I_em[	42	]=	11.6814	;
    I_em[	43	]=	12.16056	;
    I_em[	44	]=	12.63504	;
    I_em[	45	]=	13.12128	;
    I_em[	46	]=	13.60092	;
    I_em[	47	]=	14.08596	;
    I_em[	48	]=	14.57832	;
    I_em[	49	]=	15.06552	;
    I_em[	50	]=	15.53028	;
    I_em[	51	]=	15.987	;
    I_em[	52	]=	16.44636	;
    I_em[	53	]=	16.88448	;
    I_em[	54	]=	17.31324	;
    I_em[	55	]=	17.74812	;
    I_em[	56	]=	18.1638	;
    I_em[	57	]=	18.5868	;
    I_em[	58	]=	18.993	;
    I_em[	59	]=	19.39536	;
    I_em[	60	]=	19.79844	;
    I_em[	61	]=	20.18088	;
    I_em[	62	]=	20.5818	;
    I_em[	63	]=	20.97864	;
    I_em[	64	]=	21.39036	;
    I_em[	65	]=	21.7956	;
    I_em[	66	]=	22.19124	;
    I_em[	67	]=	22.59084	;
    I_em[	68	]=	23.01876	;
    I_em[	69	]=	23.4558	;
    I_em[	70	]=	23.89236	;
    I_em[	71	]=	24.34584	;
    I_em[	72	]=	24.81348	;
    I_em[	73	]=	25.28928	;
    I_em[	74	]=	25.79352	;
    I_em[	75	]=	26.29644	;
    I_em[	76	]=	26.82348	;
    I_em[	77	]=	27.36588	;
    I_em[	78	]=	27.91392	;
    I_em[	79	]=	28.45692	;
    I_em[	80	]=	29.01024	;
    I_em[	81	]=	29.56068	;
    I_em[	82	]=	30.10248	;
    I_em[	83	]=	30.64116	;
    I_em[	84	]=	31.1826	;
    I_em[	85	]=	31.70844	;
    I_em[	86	]=	32.22432	;
    I_em[	87	]=	32.74824	;
    I_em[	88	]=	33.261	;
    I_em[	89	]=	33.7584	;
    I_em[	90	]=	34.24728	;
    I_em[	91	]=	34.7394	;
    I_em[	92	]=	35.2296	;
    I_em[	93	]=	35.70468	;
    I_em[	94	]=	36.18876	;
    I_em[	95	]=	36.67728	;
    I_em[	96	]=	37.15356	;
    I_em[	97	]=	37.64484	;
    I_em[	98	]=	38.14512	;
    I_em[	99	]=	38.65104	;
    I_em[	100	]=	39.16596	;
    I_em[	101	]=	39.69048	;
    I_em[	102	]=	40.25304	;
    I_em[	103	]=	40.81452	;
    I_em[	104	]=	41.38848	;
    I_em[	105	]=	41.98668	;
    I_em[	106	]=	42.59844	;
    I_em[	107	]=	43.22364	;
    I_em[	108	]=	43.87644	;
    I_em[	109	]=	44.5446	;
    I_em[	110	]=	45.24132	;
    I_em[	111	]=	45.96576	;
    I_em[	112	]=	46.70664	;
    I_em[	113	]=	47.47488	;
    I_em[	114	]=	48.26052	;
    I_em[	115	]=	49.07304	;
    I_em[	116	]=	49.89756	;
    I_em[	117	]=	50.74668	;
    I_em[	118	]=	51.6174	;
    I_em[	119	]=	52.52496	;
    I_em[	120	]=	53.44236	;
    I_em[	121	]=	54.37872	;
    I_em[	122	]=	55.31028	;
    I_em[	123	]=	56.27124	;
    I_em[	124	]=	57.23604	;
    I_em[	125	]=	58.21404	;
    I_em[	126	]=	59.19108	;
    I_em[	127	]=	60.16548	;
    I_em[	128	]=	61.13448	;
    I_em[	129	]=	62.09316	;
    I_em[	130	]=	63.05544	;
    I_em[	131	]=	64.00908	;
    I_em[	132	]=	64.97076	;
    I_em[	133	]=	65.9184	;
    I_em[	134	]=	66.8808	;
    I_em[	135	]=	67.85916	;
    I_em[	136	]=	68.84688	;
    I_em[	137	]=	69.84612	;
    I_em[	138	]=	70.8606	;
    I_em[	139	]=	71.90436	;
    I_em[	140	]=	72.978	;
    I_em[	141	]=	74.06772	;
    I_em[	142	]=	75.1842	;
    I_em[	143	]=	76.31172	;
    I_em[	144	]=	77.44872	;
    I_em[	145	]=	78.61656	;
    I_em[	146	]=	79.77912	;
    I_em[	147	]=	80.96988	;
    I_em[	148	]=	82.16496	;
    I_em[	149	]=	83.37708	;
    I_em[	150	]=	84.59028	;
    I_em[	151	]=	85.81836	;
    I_em[	152	]=	87.0486	;
    I_em[	153	]=	88.29108	;
    I_em[	154	]=	89.53416	;
    I_em[	155	]=	90.77748	;
    I_em[	156	]=	92.04228	;
    I_em[	157	]=	93.29652	;
    I_em[	158	]=	94.55892	;
    I_em[	159	]=	95.81508	;
    I_em[	160	]=	97.07772	;
    I_em[	161	]=	98.33064	;
    I_em[	162	]=	99.5736	;
    I_em[	163	]=	100.80156	;
    I_em[	164	]=	102.04488	;
    I_em[	165	]=	103.25256	;
    I_em[	166	]=	104.46156	;
    I_em[	167	]=	105.66264	;
    I_em[	168	]=	106.86984	;
    I_em[	169	]=	108.05976	;
    I_em[	170	]=	109.26132	;
    I_em[	171	]=	110.46276	;
    I_em[	172	]=	111.66828	;
    I_em[	173	]=	112.8816	;
    I_em[	174	]=	114.10488	;
    I_em[	175	]=	115.3536	;
    I_em[	176	]=	116.60928	;
    I_em[	177	]=	117.89436	;
    I_em[	178	]=	119.21808	;
    I_em[	179	]=	120.5664	;
    I_em[	180	]=	121.95792	;
    I_em[	181	]=	123.37824	;
    I_em[	182	]=	124.85712	;
    I_em[	183	]=	126.36912	;
    I_em[	184	]=	127.93056	;
    I_em[	185	]=	129.5412	;
    I_em[	186	]=	131.21652	;
    I_em[	187	]=	132.93384	;
    I_em[	188	]=	134.721	;
    I_em[	189	]=	136.56708	;
    I_em[	190	]=	138.48744	;
    I_em[	191	]=	140.4726	;
    I_em[	192	]=	142.53708	;
    I_em[	193	]=	144.6876	;
    I_em[	194	]=	146.97636	;
    I_em[	195	]=	149.69388	;
    I_em[	196	]=	152.28828	;
    I_em[	197	]=	154.93656	;
    I_em[	198	]=	157.24776	;
    I_em[	199	]=	159.33492	;
    I_em[	200	]=	161.59884	;
    I_em[	201	]=	164.87568	;
    I_em[	202	]=	167.87496	;
    I_em[	203	]=	171.05784	;
    I_em[	204	]=	175.10496	;
    I_em[	205	]=	178.72056	;
    I_em[	206	]=	181.67772	;
    I_em[	207	]=	185.3754	;
    I_em[	208	]=	190.51704	;
    I_em[	209	]=	194.08548	;
    I_em[	210	]=	197.26212	;
    I_em[	211	]=	200.862	;
    I_em[	212	]=	205.59096	;
    I_em[	213	]=	210.60324	;
    I_em[	214	]=	215.30448	;
    I_em[	215	]=	220.44912	;
    I_em[	216	]=	225.07044	;
    I_em[	217	]=	229.32912	;
    I_em[	218	]=	234.34128	;
    I_em[	219	]=	239.66664	;
    I_em[	220	]=	244.60728	;
    I_em[	221	]=	249.29916	;
    I_em[	222	]=	254.93004	;
    I_em[	223	]=	261.65832	;
    I_em[	224	]=	267.27372	;
    I_em[	225	]=	272.10456	;
    I_em[	226	]=	276.95856	;
    I_em[	227	]=	280.80648	;
    I_em[	228	]=	284.45292	;
    I_em[	229	]=	289.51176	;
    I_em[	230	]=	294.01644	;
    I_em[	231	]=	299.29944	;
    I_em[	232	]=	304.23168	;
    I_em[	233	]=	311.2686	;
    I_em[	234	]=	318.06756	;
    I_em[	235	]=	323.31228	;
    I_em[	236	]=	328.76628	;
    I_em[	237	]=	335.19948	;
    I_em[	238	]=	342.56232	;
    I_em[	239	]=	349.15008	;
    I_em[	240	]=	355.75632	;
    I_em[	241	]=	362.64336	;
    I_em[	242	]=	369.14064	;
    I_em[	243	]=	375.83124	;
    I_em[	244	]=	381.7788	;
    I_em[	245	]=	389.19348	;
    I_em[	246	]=	398.66364	;
    I_em[	247	]=	408.2292	;
    I_em[	248	]=	417.86556	;
    I_em[	249	]=	427.60812	;
    I_em[	250	]=	437.427	;
    I_em[	251	]=	447.33804	;
    I_em[	252	]=	457.32936	;
    I_em[	253	]=	467.41524	;
    I_em[	254	]=	477.59052	;
    I_em[	255	]=	487.87452	;
    I_em[	256	]=	498.34212	;
    I_em[	257	]=	509.02896	;
    I_em[	258	]=	519.9354	;
    I_em[	259	]=	531.04944	;
    I_em[	260	]=	542.37048	;
    I_em[	261	]=	553.90836	;
    I_em[	262	]=	565.65504	;
    I_em[	263	]=	577.61028	;
    I_em[	264	]=	589.78032	;
    I_em[	265	]=	602.1402	;
    I_em[	266	]=	614.55252	;
    I_em[	267	]=	626.9904	;
    I_em[	268	]=	639.45696	;
    I_em[	269	]=	651.94716	;
    I_em[	270	]=	664.47012	;
    I_em[	271	]=	677.02176	;
    I_em[	272	]=	689.59392	;
    I_em[	273	]=	702.18384	;
    I_em[	274	]=	714.81816	;
    I_em[	275	]=	727.45236	;
    I_em[	276	]=	740.07	;
    I_em[	277	]=	752.65056	;
    I_em[	278	]=	765.18396	;
    I_em[	279	]=	777.67584	;
    I_em[	280	]=	790.14108	;
    I_em[	281	]=	802.55484	;
    I_em[	282	]=	814.938	;
    I_em[	283	]=	827.28372	;
    I_em[	284	]=	839.58168	;
    I_em[	285	]=	851.84532	;
    I_em[	286	]=	864.01968	;
    I_em[	287	]=	876.10512	;
    I_em[	288	]=	888.11088	;
    I_em[	289	]=	900.01284	;
    I_em[	290	]=	911.8392	;
    I_em[	291	]=	923.57748	;
    I_em[	292	]=	935.22624	;
    I_em[	293	]=	946.77672	;
    I_em[	294	]=	958.25352	;
    I_em[	295	]=	969.62232	;
    I_em[	296	]=	980.85672	;
    I_em[	297	]=	991.93632	;
    I_em[	298	]=	1002.86832	;
    I_em[	299	]=	1013.65032	;
    I_em[	300	]=	1024.27632	;
    I_em[	301	]=	1034.75232	;
    I_em[	302	]=	1045.08912	;
    I_em[	303	]=	1055.26512	;
    I_em[	304	]=	1065.28632	;
    I_em[	305	]=	1075.16712	;
    I_em[	306	]=	1084.85832	;
    I_em[	307	]=	1094.35392	;
    I_em[	308	]=	1103.67912	;
    I_em[	309	]=	1112.81832	;
    I_em[	310	]=	1121.76192	;
    I_em[	311	]=	1130.52432	;
    I_em[	312	]=	1139.09832	;
    I_em[	313	]=	1147.49472	;
    I_em[	314	]=	1155.70392	;
    I_em[	315	]=	1163.72712	;
    I_em[	316	]=	1171.56912	;
    I_em[	317	]=	1179.24672	;
    I_em[	318	]=	1186.73112	;
    I_em[	319	]=	1194.04992	;
    I_em[	320	]=	1201.18512	;
    I_em[	321	]=	1208.15832	;
    I_em[	322	]=	1214.93952	;
    I_em[	323	]=	1221.54552	;
    I_em[	324	]=	1227.98472	;
    I_em[	325	]=	1234.24272	;
    I_em[	326	]=	1240.30392	;
    I_em[	327	]=	1246.16832	;
    I_em[	328	]=	1251.85152	;
    I_em[	329	]=	1257.33072	;
    I_em[	330	]=	1262.61672	;
    I_em[	331	]=	1267.71072	;
    I_em[	332	]=	1272.60912	;
    I_em[	333	]=	1277.31672	;
    I_em[	334	]=	1281.82992	;
    I_em[	335	]=	1286.16912	;
    I_em[	336	]=	1290.40032	;
    I_em[	337	]=	1294.54392	;
    I_em[	338	]=	1298.60352	;
    I_em[	339	]=	1302.57432	;
    I_em[	340	]=	1306.45992	;
    I_em[	341	]=	1310.24952	;
    I_em[	342	]=	1313.96592	;
    I_em[	343	]=	1317.59712	;
    I_em[	344	]=	1321.13232	;
    I_em[	345	]=	1324.59192	;
    I_em[	346	]=	1327.96032	;
    I_em[	347	]=	1331.23392	;
    I_em[	348	]=	1334.43192	;
    I_em[	349	]=	1337.54352	;
    I_em[	350	]=	1340.56512	;
    I_em[	351	]=	1343.49312	;
    I_em[	352	]=	1346.34192	;
    I_em[	353	]=	1349.10792	;
    I_em[	354	]=	1351.78872	;
    I_em[	355	]=	1354.36992	;
    I_em[	356	]=	1356.87792	;
    I_em[	357	]=	1359.28872	;
    I_em[	358	]=	1361.62512	;
    I_em[	359	]=	1363.87392	;
    I_em[	360	]=	1366.02192	;
    I_em[	361	]=	1368.09312	;
    I_em[	362	]=	1370.08032	;
    I_em[	363	]=	1371.98232	;
    I_em[	364	]=	1373.79792	;
    I_em[	365	]=	1375.52592	;
    I_em[	366	]=	1377.16272	;
    I_em[	367	]=	1378.71312	;
    I_em[	368	]=	1380.18672	;
    I_em[	369	]=	1381.57272	;
    I_em[	370	]=	1382.85912	;
    I_em[	371	]=	1384.07112	;
    I_em[	372	]=	1385.19672	;
    I_em[	373	]=	1386.22392	;
    I_em[	374	]=	1387.18512	;
    I_em[	375	]=	1388.04912	;
    I_em[	376	]=	1388.82072	;
    I_em[	377	]=	1389.51192	;
    I_em[	378	]=	1390.11672	;
    I_em[	379	]=	1390.63152	;
    I_em[	380	]=	1391.06112	;
    I_em[	381	]=	1391.40672	;
    I_em[	382	]=	1391.66592	;
    I_em[	383	]=	1391.83872	;
    I_em[	384	]=	1391.92272	;
    I_em[	385	]=	1391.93592	;
    for(int i = 386; i<tmax; i++)
    {
        I_em[i] = I_em[i-1];
    }
}

void setIemA1AIM() //A1AIM
{
    I_em[	0	]=	0	;
    I_em[	1	]=	0.05964	;
    I_em[	2	]=	0.1398	;
    I_em[	3	]=	0.22596	;
    I_em[	4	]=	0.31704	;
    I_em[	5	]=	0.4272	;
    I_em[	6	]=	0.54372	;
    I_em[	7	]=	0.65484	;
    I_em[	8	]=	0.79512	;
    I_em[	9	]=	0.93084	;
    I_em[	10	]=	1.0836	;
    I_em[	11	]=	1.24404	;
    I_em[	12	]=	1.41072	;
    I_em[	13	]=	1.596	;
    I_em[	14	]=	1.78764	;
    I_em[	15	]=	1.98612	;
    I_em[	16	]=	2.20344	;
    I_em[	17	]=	2.42736	;
    I_em[	18	]=	2.65788	;
    I_em[	19	]=	2.9076	;
    I_em[	20	]=	3.16428	;
    I_em[	21	]=	3.42744	;
    I_em[	22	]=	3.71016	;
    I_em[	23	]=	3.99996	;
    I_em[	24	]=	4.3086	;
    I_em[	25	]=	4.62492	;
    I_em[	26	]=	4.9482	;
    I_em[	27	]=	5.2788	;
    I_em[	28	]=	5.6286	;
    I_em[	29	]=	5.99808	;
    I_em[	30	]=	6.37368	;
    I_em[	31	]=	6.75768	;
    I_em[	32	]=	7.1604	;
    I_em[	33	]=	7.57068	;
    I_em[	34	]=	7.99836	;
    I_em[	35	]=	8.43192	;
    I_em[	36	]=	8.86992	;
    I_em[	37	]=	9.32244	;
    I_em[	38	]=	9.79008	;
    I_em[	39	]=	10.24692	;
    I_em[	40	]=	10.72944	;
    I_em[	41	]=	11.19828	;
    I_em[	42	]=	11.6814	;
    I_em[	43	]=	12.16056	;
    I_em[	44	]=	12.63504	;
    I_em[	45	]=	13.12128	;
    I_em[	46	]=	13.60092	;
    I_em[	47	]=	14.08596	;
    I_em[	48	]=	14.57832	;
    I_em[	49	]=	15.06552	;
    I_em[	50	]=	15.53028	;
    I_em[	51	]=	15.987	;
    I_em[	52	]=	16.44636	;
    I_em[	53	]=	16.88448	;
    I_em[	54	]=	17.31324	;
    I_em[	55	]=	17.74812	;
    I_em[	56	]=	18.1638	;
    I_em[	57	]=	18.5868	;
    I_em[	58	]=	18.993	;
    I_em[	59	]=	19.39536	;
    I_em[	60	]=	19.79844	;
    I_em[	61	]=	20.18088	;
    I_em[	62	]=	20.5818	;
    I_em[	63	]=	20.97864	;
    I_em[	64	]=	21.39036	;
    I_em[	65	]=	21.7956	;
    I_em[	66	]=	22.19124	;
    I_em[	67	]=	22.59084	;
    I_em[	68	]=	23.01876	;
    I_em[	69	]=	23.4558	;
    I_em[	70	]=	23.89236	;
    I_em[	71	]=	24.34584	;
    I_em[	72	]=	24.81348	;
    I_em[	73	]=	25.28928	;
    I_em[	74	]=	25.79352	;
    I_em[	75	]=	26.29644	;
    I_em[	76	]=	26.82348	;
    I_em[	77	]=	27.36588	;
    I_em[	78	]=	27.91392	;
    I_em[	79	]=	28.45692	;
    I_em[	80	]=	29.01024	;
    I_em[	81	]=	29.56068	;
    I_em[	82	]=	30.10248	;
    I_em[	83	]=	30.64116	;
    I_em[	84	]=	31.1826	;
    I_em[	85	]=	31.70844	;
    I_em[	86	]=	32.22432	;
    I_em[	87	]=	32.74824	;
    I_em[	88	]=	33.261	;
    I_em[	89	]=	33.7584	;
    I_em[	90	]=	34.24728	;
    I_em[	91	]=	34.7394	;
    I_em[	92	]=	35.2296	;
    I_em[	93	]=	35.70468	;
    I_em[	94	]=	36.18876	;
    I_em[	95	]=	36.67728	;
    I_em[	96	]=	37.15356	;
    I_em[	97	]=	37.64484	;
    I_em[	98	]=	38.14512	;
    I_em[	99	]=	38.65104	;
    I_em[	100	]=	39.16596	;
    I_em[	101	]=	39.69048	;
    I_em[	102	]=	40.25304	;
    I_em[	103	]=	40.81452	;
    I_em[	104	]=	41.38848	;
    I_em[	105	]=	41.98668	;
    I_em[	106	]=	42.59844	;
    I_em[	107	]=	43.22364	;
    I_em[	108	]=	43.87644	;
    I_em[	109	]=	44.5446	;
    I_em[	110	]=	45.24132	;
    I_em[	111	]=	45.96576	;
    I_em[	112	]=	46.70664	;
    I_em[	113	]=	47.47488	;
    I_em[	114	]=	48.26052	;
    I_em[	115	]=	49.07304	;
    I_em[	116	]=	49.89756	;
    I_em[	117	]=	50.74668	;
    I_em[	118	]=	51.6174	;
    I_em[	119	]=	52.52496	;
    I_em[	120	]=	53.44236	;
    I_em[	121	]=	54.37872	;
    I_em[	122	]=	55.31028	;
    I_em[	123	]=	56.27124	;
    I_em[	124	]=	57.23604	;
    I_em[	125	]=	58.21404	;
    I_em[	126	]=	59.19108	;
    I_em[	127	]=	60.16548	;
    I_em[	128	]=	61.13448	;
    I_em[	129	]=	62.09316	;
    I_em[	130	]=	63.05544	;
    I_em[	131	]=	64.00908	;
    I_em[	132	]=	64.97076	;
    I_em[	133	]=	65.9184	;
    I_em[	134	]=	66.8808	;
    I_em[	135	]=	67.85916	;
    I_em[	136	]=	68.84688	;
    I_em[	137	]=	69.84612	;
    I_em[	138	]=	70.8606	;
    I_em[	139	]=	71.90436	;
    I_em[	140	]=	72.978	;
    I_em[	141	]=	74.06772	;
    I_em[	142	]=	75.1842	;
    I_em[	143	]=	76.31172	;
    I_em[	144	]=	77.44872	;
    I_em[	145	]=	78.61656	;
    I_em[	146	]=	79.77912	;
    I_em[	147	]=	80.96988	;
    I_em[	148	]=	82.16496	;
    I_em[	149	]=	83.37708	;
    I_em[	150	]=	84.59028	;
    I_em[	151	]=	85.81836	;
    I_em[	152	]=	87.0486	;
    I_em[	153	]=	88.29108	;
    I_em[	154	]=	89.53416	;
    I_em[	155	]=	90.77748	;
    I_em[	156	]=	92.04228	;
    I_em[	157	]=	93.29652	;
    I_em[	158	]=	94.55892	;
    I_em[	159	]=	95.81508	;
    I_em[	160	]=	97.07772	;
    I_em[	161	]=	98.33064	;
    I_em[	162	]=	99.5736	;
    I_em[	163	]=	100.80156	;
    I_em[	164	]=	102.04488	;
    I_em[	165	]=	103.25256	;
    I_em[	166	]=	104.46156	;
    I_em[	167	]=	105.66264	;
    I_em[	168	]=	106.86984	;
    I_em[	169	]=	108.05976	;
    I_em[	170	]=	109.26132	;
    I_em[	171	]=	110.46276	;
    I_em[	172	]=	111.66828	;
    I_em[	173	]=	112.8816	;
    I_em[	174	]=	114.10488	;
    I_em[	175	]=	115.3536	;
    I_em[	176	]=	116.60928	;
    I_em[	177	]=	117.89436	;
    I_em[	178	]=	119.21808	;
    I_em[	179	]=	120.5664	;
    I_em[	180	]=	121.95792	;
    I_em[	181	]=	123.37824	;
    I_em[	182	]=	124.85712	;
    I_em[	183	]=	126.36912	;
    I_em[	184	]=	127.93056	;
    I_em[	185	]=	129.5412	;
    I_em[	186	]=	131.21652	;
    I_em[	187	]=	132.93384	;
    I_em[	188	]=	134.721	;
    I_em[	189	]=	136.56708	;
    I_em[	190	]=	138.48744	;
    I_em[	191	]=	140.4726	;
    I_em[	192	]=	142.53708	;
    I_em[	193	]=	144.6876	;
    I_em[	194	]=	146.97636	;
    I_em[	195	]=	149.69388	;
    I_em[	196	]=	152.28828	;
    I_em[	197	]=	154.93656	;
    I_em[	198	]=	157.24776	;
    I_em[	199	]=	159.33492	;
    I_em[	200	]=	161.59884	;
    I_em[	201	]=	164.87568	;
    I_em[	202	]=	167.87496	;
    I_em[	203	]=	171.05784	;
    I_em[	204	]=	175.10496	;
    I_em[	205	]=	178.72056	;
    I_em[	206	]=	181.67772	;
    I_em[	207	]=	185.3754	;
    I_em[	208	]=	190.51704	;
    I_em[	209	]=	194.08548	;
    I_em[	210	]=	197.26212	;
    I_em[	211	]=	200.862	;
    I_em[	212	]=	205.59096	;
    I_em[	213	]=	210.60324	;
    I_em[	214	]=	215.30448	;
    I_em[	215	]=	220.44912	;
    I_em[	216	]=	225.07044	;
    I_em[	217	]=	229.32912	;
    I_em[	218	]=	234.34128	;
    I_em[	219	]=	239.66664	;
    I_em[	220	]=	244.60728	;
    I_em[	221	]=	249.29916	;
    I_em[	222	]=	254.93004	;
    I_em[	223	]=	261.65832	;
    I_em[	224	]=	267.27372	;
    I_em[	225	]=	272.10456	;
    I_em[	226	]=	276.95856	;
    I_em[	227	]=	280.80648	;
    I_em[	228	]=	284.45292	;
    I_em[	229	]=	289.51176	;
    I_em[	230	]=	294.01644	;
    I_em[	231	]=	299.29944	;
    I_em[	232	]=	304.23168	;
    I_em[	233	]=	311.2686	;
    I_em[	234	]=	318.06756	;
    I_em[	235	]=	323.31228	;
    I_em[	236	]=	328.76628	;
    I_em[	237	]=	335.19948	;
    I_em[	238	]=	342.56232	;
    I_em[	239	]=	349.15008	;
    I_em[	240	]=	355.75632	;
    I_em[	241	]=	362.64336	;
    I_em[	242	]=	369.14064	;
    I_em[	243	]=	375.83124	;
    I_em[	244	]=	381.7788	;
    I_em[	245	]=	389.94564	;
    I_em[	246	]=	400.99968	;
    I_em[	247	]=	412.23576	;
    I_em[	248	]=	423.64464	;
    I_em[	249	]=	435.23136	;
    I_em[	250	]=	446.99688	;
    I_em[	251	]=	458.93532	;
    I_em[	252	]=	471.05028	;
    I_em[	253	]=	483.34632	;
    I_em[	254	]=	495.81564	;
    I_em[	255	]=	508.46208	;
    I_em[	256	]=	521.29356	;
    I_em[	257	]=	534.30516	;
    I_em[	258	]=	547.49688	;
    I_em[	259	]=	560.88312	;
    I_em[	260	]=	574.44324	;
    I_em[	261	]=	588.20304	;
    I_em[	262	]=	602.12796	;
    I_em[	263	]=	616.25412	;
    I_em[	264	]=	630.552	;
    I_em[	265	]=	645.02652	;
    I_em[	266	]=	659.604	;
    I_em[	267	]=	674.268	;
    I_em[	268	]=	689.01648	;
    I_em[	269	]=	703.85472	;
    I_em[	270	]=	718.77372	;
    I_em[	271	]=	733.785	;
    I_em[	272	]=	748.8786	;
    I_em[	273	]=	764.05452	;
    I_em[	274	]=	779.331	;
    I_em[	275	]=	794.68524	;
    I_em[	276	]=	810.14304	;
    I_em[	277	]=	825.7038	;
    I_em[	278	]=	841.37148	;
    I_em[	279	]=	857.13468	;
    I_em[	280	]=	873.00744	;
    I_em[	281	]=	888.9888	;
    I_em[	282	]=	905.0688	;
    I_em[	283	]=	921.25176	;
    I_em[	284	]=	937.54272	;
    I_em[	285	]=	953.89752	;
    I_em[	286	]=	970.24992	;
    I_em[	287	]=	986.57352	;
    I_em[	288	]=	1002.84192	;
    I_em[	289	]=	1019.07432	;
    I_em[	290	]=	1035.28392	;
    I_em[	291	]=	1051.43472	;
    I_em[	292	]=	1067.56512	;
    I_em[	293	]=	1083.64272	;
    I_em[	294	]=	1099.69632	;
    I_em[	295	]=	1115.69592	;
    I_em[	296	]=	1131.68472	;
    I_em[	297	]=	1147.63032	;
    I_em[	298	]=	1163.55912	;
    I_em[	299	]=	1179.45672	;
    I_em[	300	]=	1195.32792	;
    I_em[	301	]=	1211.16552	;
    I_em[	302	]=	1226.98872	;
    I_em[	303	]=	1242.77232	;
    I_em[	304	]=	1258.52592	;
    I_em[	305	]=	1274.25432	;
    I_em[	306	]=	1289.93592	;
    I_em[	307	]=	1305.55152	;
    I_em[	308	]=	1321.12272	;
    I_em[	309	]=	1336.64592	;
    I_em[	310	]=	1352.10552	;
    I_em[	311	]=	1367.50872	;
    I_em[	312	]=	1382.85792	;
    I_em[	313	]=	1398.15072	;
    I_em[	314	]=	1413.39792	;
    I_em[	315	]=	1428.58032	;
    I_em[	316	]=	1443.67032	;
    I_em[	317	]=	1458.67992	;
    I_em[	318	]=	1473.60072	;
    I_em[	319	]=	1488.44472	;
    I_em[	320	]=	1503.18312	;
    I_em[	321	]=	1517.84112	;
    I_em[	322	]=	1532.40672	;
    I_em[	323	]=	1546.89552	;
    I_em[	324	]=	1561.28472	;
    I_em[	325	]=	1575.58872	;
    I_em[	326	]=	1589.80872	;
    I_em[	327	]=	1603.94832	;
    I_em[	328	]=	1618.01592	;
    I_em[	329	]=	1631.98392	;
    I_em[	330	]=	1645.87872	;
    I_em[	331	]=	1659.70272	;
    I_em[	332	]=	1673.43552	;
    I_em[	333	]=	1687.09152	;
    I_em[	334	]=	1700.66472	;
    I_em[	335	]=	1714.11792	;
    I_em[	336	]=	1727.34672	;
    I_em[	337	]=	1740.29952	;
    I_em[	338	]=	1752.97152	;
    I_em[	339	]=	1765.38912	;
    I_em[	340	]=	1777.52832	;
    I_em[	341	]=	1789.40472	;
    I_em[	342	]=	1801.00632	;
    I_em[	343	]=	1812.33192	;
    I_em[	344	]=	1823.39592	;
    I_em[	345	]=	1834.18872	;
    I_em[	346	]=	1844.71392	;
    I_em[	347	]=	1854.96792	;
    I_em[	348	]=	1864.94952	;
    I_em[	349	]=	1874.66112	;
    I_em[	350	]=	1884.10392	;
    I_em[	351	]=	1893.28872	;
    I_em[	352	]=	1902.18192	;
    I_em[	353	]=	1910.82192	;
    I_em[	354	]=	1919.18592	;
    I_em[	355	]=	1927.27512	;
    I_em[	356	]=	1935.09912	;
    I_em[	357	]=	1942.65672	;
    I_em[	358	]=	1949.94312	;
    I_em[	359	]=	1956.96432	;
    I_em[	360	]=	1963.70592	;
    I_em[	361	]=	1970.17872	;
    I_em[	362	]=	1976.38632	;
    I_em[	363	]=	1982.32752	;
    I_em[	364	]=	1987.99752	;
    I_em[	365	]=	1993.39272	;
    I_em[	366	]=	1998.52272	;
    I_em[	367	]=	2003.37192	;
    I_em[	368	]=	2007.96552	;
    I_em[	369	]=	2012.27952	;
    I_em[	370	]=	2016.32952	;
    I_em[	371	]=	2020.10472	;
    I_em[	372	]=	2023.61592	;
    I_em[	373	]=	2026.84992	;
    I_em[	374	]=	2029.81752	;
    I_em[	375	]=	2032.51872	;
    I_em[	376	]=	2034.95472	;
    I_em[	377	]=	2037.10512	;
    I_em[	378	]=	2038.99512	;
    I_em[	379	]=	2040.61152	;
    I_em[	380	]=	2041.96032	;
    I_em[	381	]=	2043.04512	;
    I_em[	382	]=	2043.85392	;
    I_em[	383	]=	2044.39032	;
    I_em[	384	]=	2044.66272	;
    I_em[	385	]=	2044.71552	;
    for(int i = 386; i<tmax; i++)
    {
        I_em[i] = I_em[i-1];
    }
    
    
}

void setIemB2() //B2 MESSAGE
{
    I_em[	0	]=	0	;
    I_em[	1	]=	0.05964	;
    I_em[	2	]=	0.1398	;
    I_em[	3	]=	0.22596	;
    I_em[	4	]=	0.31704	;
    I_em[	5	]=	0.4272	;
    I_em[	6	]=	0.54372	;
    I_em[	7	]=	0.65484	;
    I_em[	8	]=	0.79512	;
    I_em[	9	]=	0.93084	;
    I_em[	10	]=	1.0836	;
    I_em[	11	]=	1.24404	;
    I_em[	12	]=	1.41072	;
    I_em[	13	]=	1.596	;
    I_em[	14	]=	1.78764	;
    I_em[	15	]=	1.98612	;
    I_em[	16	]=	2.20344	;
    I_em[	17	]=	2.42736	;
    I_em[	18	]=	2.65788	;
    I_em[	19	]=	2.9076	;
    I_em[	20	]=	3.16428	;
    I_em[	21	]=	3.42744	;
    I_em[	22	]=	3.71016	;
    I_em[	23	]=	3.99996	;
    I_em[	24	]=	4.3086	;
    I_em[	25	]=	4.62492	;
    I_em[	26	]=	4.9482	;
    I_em[	27	]=	5.2788	;
    I_em[	28	]=	5.6286	;
    I_em[	29	]=	5.99808	;
    I_em[	30	]=	6.37368	;
    I_em[	31	]=	6.75768	;
    I_em[	32	]=	7.1604	;
    I_em[	33	]=	7.57068	;
    I_em[	34	]=	7.99836	;
    I_em[	35	]=	8.43192	;
    I_em[	36	]=	8.86992	;
    I_em[	37	]=	9.32244	;
    I_em[	38	]=	9.79008	;
    I_em[	39	]=	10.24692	;
    I_em[	40	]=	10.72944	;
    I_em[	41	]=	11.19828	;
    I_em[	42	]=	11.6814	;
    I_em[	43	]=	12.16056	;
    I_em[	44	]=	12.63504	;
    I_em[	45	]=	13.12128	;
    I_em[	46	]=	13.60092	;
    I_em[	47	]=	14.08596	;
    I_em[	48	]=	14.57832	;
    I_em[	49	]=	15.06552	;
    I_em[	50	]=	15.53028	;
    I_em[	51	]=	15.987	;
    I_em[	52	]=	16.44636	;
    I_em[	53	]=	16.88448	;
    I_em[	54	]=	17.31324	;
    I_em[	55	]=	17.74812	;
    I_em[	56	]=	18.1638	;
    I_em[	57	]=	18.5868	;
    I_em[	58	]=	18.993	;
    I_em[	59	]=	19.39536	;
    I_em[	60	]=	19.79844	;
    I_em[	61	]=	20.18088	;
    I_em[	62	]=	20.5818	;
    I_em[	63	]=	20.97864	;
    I_em[	64	]=	21.39036	;
    I_em[	65	]=	21.7956	;
    I_em[	66	]=	22.19124	;
    I_em[	67	]=	22.59084	;
    I_em[	68	]=	23.01876	;
    I_em[	69	]=	23.4558	;
    I_em[	70	]=	23.89236	;
    I_em[	71	]=	24.34584	;
    I_em[	72	]=	24.81348	;
    I_em[	73	]=	25.28928	;
    I_em[	74	]=	25.79352	;
    I_em[	75	]=	26.29644	;
    I_em[	76	]=	26.82348	;
    I_em[	77	]=	27.36588	;
    I_em[	78	]=	27.91392	;
    I_em[	79	]=	28.45692	;
    I_em[	80	]=	29.01024	;
    I_em[	81	]=	29.56068	;
    I_em[	82	]=	30.10248	;
    I_em[	83	]=	30.64116	;
    I_em[	84	]=	31.1826	;
    I_em[	85	]=	31.70844	;
    I_em[	86	]=	32.22432	;
    I_em[	87	]=	32.74824	;
    I_em[	88	]=	33.261	;
    I_em[	89	]=	33.7584	;
    I_em[	90	]=	34.24728	;
    I_em[	91	]=	34.7394	;
    I_em[	92	]=	35.2296	;
    I_em[	93	]=	35.70468	;
    I_em[	94	]=	36.18876	;
    I_em[	95	]=	36.67728	;
    I_em[	96	]=	37.15356	;
    I_em[	97	]=	37.64484	;
    I_em[	98	]=	38.14512	;
    I_em[	99	]=	38.65104	;
    I_em[	100	]=	39.16596	;
    I_em[	101	]=	39.69048	;
    I_em[	102	]=	40.25304	;
    I_em[	103	]=	40.81452	;
    I_em[	104	]=	41.38848	;
    I_em[	105	]=	41.98668	;
    I_em[	106	]=	42.59844	;
    I_em[	107	]=	43.22364	;
    I_em[	108	]=	43.87644	;
    I_em[	109	]=	44.5446	;
    I_em[	110	]=	45.24132	;
    I_em[	111	]=	45.96576	;
    I_em[	112	]=	46.70664	;
    I_em[	113	]=	47.47488	;
    I_em[	114	]=	48.26052	;
    I_em[	115	]=	49.07304	;
    I_em[	116	]=	49.89756	;
    I_em[	117	]=	50.74668	;
    I_em[	118	]=	51.6174	;
    I_em[	119	]=	52.52496	;
    I_em[	120	]=	53.44236	;
    I_em[	121	]=	54.37872	;
    I_em[	122	]=	55.31028	;
    I_em[	123	]=	56.27124	;
    I_em[	124	]=	57.23604	;
    I_em[	125	]=	58.21404	;
    I_em[	126	]=	59.19108	;
    I_em[	127	]=	60.16548	;
    I_em[	128	]=	61.13448	;
    I_em[	129	]=	62.09316	;
    I_em[	130	]=	63.05544	;
    I_em[	131	]=	64.00908	;
    I_em[	132	]=	64.97076	;
    I_em[	133	]=	65.9184	;
    I_em[	134	]=	66.8808	;
    I_em[	135	]=	67.85916	;
    I_em[	136	]=	68.84688	;
    I_em[	137	]=	69.84612	;
    I_em[	138	]=	70.8606	;
    I_em[	139	]=	71.90436	;
    I_em[	140	]=	72.978	;
    I_em[	141	]=	74.06772	;
    I_em[	142	]=	75.1842	;
    I_em[	143	]=	76.31172	;
    I_em[	144	]=	77.44872	;
    I_em[	145	]=	78.61656	;
    I_em[	146	]=	79.77912	;
    I_em[	147	]=	80.96988	;
    I_em[	148	]=	82.16496	;
    I_em[	149	]=	83.37708	;
    I_em[	150	]=	84.59028	;
    I_em[	151	]=	85.81836	;
    I_em[	152	]=	87.0486	;
    I_em[	153	]=	88.29108	;
    I_em[	154	]=	89.53416	;
    I_em[	155	]=	90.77748	;
    I_em[	156	]=	92.04228	;
    I_em[	157	]=	93.29652	;
    I_em[	158	]=	94.55892	;
    I_em[	159	]=	95.81508	;
    I_em[	160	]=	97.07772	;
    I_em[	161	]=	98.33064	;
    I_em[	162	]=	99.5736	;
    I_em[	163	]=	100.80156	;
    I_em[	164	]=	102.04488	;
    I_em[	165	]=	103.25256	;
    I_em[	166	]=	104.46156	;
    I_em[	167	]=	105.66264	;
    I_em[	168	]=	106.86984	;
    I_em[	169	]=	108.05976	;
    I_em[	170	]=	109.26132	;
    I_em[	171	]=	110.46276	;
    I_em[	172	]=	111.66828	;
    I_em[	173	]=	112.8816	;
    I_em[	174	]=	114.10488	;
    I_em[	175	]=	115.3536	;
    I_em[	176	]=	116.60928	;
    I_em[	177	]=	117.89436	;
    I_em[	178	]=	119.21808	;
    I_em[	179	]=	120.5664	;
    I_em[	180	]=	121.95792	;
    I_em[	181	]=	123.37824	;
    I_em[	182	]=	124.85712	;
    I_em[	183	]=	126.36912	;
    I_em[	184	]=	127.93056	;
    I_em[	185	]=	129.5412	;
    I_em[	186	]=	131.21652	;
    I_em[	187	]=	132.93384	;
    I_em[	188	]=	134.721	;
    I_em[	189	]=	136.56708	;
    I_em[	190	]=	138.48744	;
    I_em[	191	]=	140.4726	;
    I_em[	192	]=	142.53708	;
    I_em[	193	]=	144.6876	;
    I_em[	194	]=	146.97636	;
    I_em[	195	]=	149.69388	;
    I_em[	196	]=	152.28828	;
    I_em[	197	]=	154.93656	;
    I_em[	198	]=	157.24776	;
    I_em[	199	]=	159.33492	;
    I_em[	200	]=	161.59884	;
    I_em[	201	]=	164.87568	;
    I_em[	202	]=	167.87496	;
    I_em[	203	]=	171.05784	;
    I_em[	204	]=	175.10496	;
    I_em[	205	]=	178.72056	;
    I_em[	206	]=	181.67772	;
    I_em[	207	]=	185.3754	;
    I_em[	208	]=	190.51704	;
    I_em[	209	]=	194.08548	;
    I_em[	210	]=	197.26212	;
    I_em[	211	]=	200.862	;
    I_em[	212	]=	205.59096	;
    I_em[	213	]=	210.60324	;
    I_em[	214	]=	215.30448	;
    I_em[	215	]=	220.44912	;
    I_em[	216	]=	225.07044	;
    I_em[	217	]=	229.32912	;
    I_em[	218	]=	234.34128	;
    I_em[	219	]=	239.66664	;
    I_em[	220	]=	244.60728	;
    I_em[	221	]=	249.29916	;
    I_em[	222	]=	254.93004	;
    I_em[	223	]=	261.65832	;
    I_em[	224	]=	267.27372	;
    I_em[	225	]=	272.10456	;
    I_em[	226	]=	276.95856	;
    I_em[	227	]=	280.80648	;
    I_em[	228	]=	284.45292	;
    I_em[	229	]=	289.51176	;
    I_em[	230	]=	294.01644	;
    I_em[	231	]=	299.29944	;
    I_em[	232	]=	304.23168	;
    I_em[	233	]=	311.2686	;
    I_em[	234	]=	318.06756	;
    I_em[	235	]=	323.31228	;
    I_em[	236	]=	328.76628	;
    I_em[	237	]=	335.19948	;
    I_em[	238	]=	342.56232	;
    I_em[	239	]=	349.15008	;
    I_em[	240	]=	355.75632	;
    I_em[	241	]=	362.64336	;
    I_em[	242	]=	369.14064	;
    I_em[	243	]=	375.83124	;
    I_em[	244	]=	381.7788	;
    I_em[	245	]=	388.88856	;
    I_em[	246	]=	397.70748	;
    I_em[	247	]=	406.55148	;
    I_em[	248	]=	415.4142	;
    I_em[	249	]=	424.30428	;
    I_em[	250	]=	433.22916	;
    I_em[	251	]=	442.18092	;
    I_em[	252	]=	451.15464	;
    I_em[	253	]=	460.15356	;
    I_em[	254	]=	469.18248	;
    I_em[	255	]=	478.25532	;
    I_em[	256	]=	487.38624	;
    I_em[	257	]=	496.61724	;
    I_em[	258	]=	505.9266	;
    I_em[	259	]=	515.31396	;
    I_em[	260	]=	524.79396	;
    I_em[	261	]=	534.35172	;
    I_em[	262	]=	544.0014	;
    I_em[	263	]=	553.73136	;
    I_em[	264	]=	563.55732	;
    I_em[	265	]=	573.45264	;
    I_em[	266	]=	583.43568	;
    I_em[	267	]=	593.49228	;
    I_em[	268	]=	603.62976	;
    I_em[	269	]=	613.85616	;
    I_em[	270	]=	624.15	;
    I_em[	271	]=	634.52736	;
    I_em[	272	]=	644.9814	;
    I_em[	273	]=	655.52688	;
    I_em[	274	]=	666.1434	;
    I_em[	275	]=	676.82292	;
    I_em[	276	]=	687.55068	;
    I_em[	277	]=	698.3142	;
    I_em[	278	]=	709.10616	;
    I_em[	279	]=	719.92332	;
    I_em[	280	]=	730.77288	;
    I_em[	281	]=	741.6624	;
    I_em[	282	]=	752.57256	;
    I_em[	283	]=	763.52904	;
    I_em[	284	]=	774.50232	;
    I_em[	285	]=	785.51724	;
    I_em[	286	]=	796.5774	;
    I_em[	287	]=	807.68928	;
    I_em[	288	]=	818.85132	;
    I_em[	289	]=	830.0508	;
    I_em[	290	]=	841.3158	;
    I_em[	291	]=	852.60852	;
    I_em[	292	]=	863.964	;
    I_em[	293	]=	875.3724	;
    I_em[	294	]=	886.821	;
    I_em[	295	]=	898.31604	;
    I_em[	296	]=	909.83208	;
    I_em[	297	]=	921.34572	;
    I_em[	298	]=	932.88972	;
    I_em[	299	]=	944.42988	;
    I_em[	300	]=	955.99284	;
    I_em[	301	]=	967.56312	;
    I_em[	302	]=	979.15632	;
    I_em[	303	]=	990.74472	;
    I_em[	304	]=	1002.35832	;
    I_em[	305	]=	1013.98272	;
    I_em[	306	]=	1025.65992	;
    I_em[	307	]=	1037.38872	;
    I_em[	308	]=	1049.17392	;
    I_em[	309	]=	1061.00472	;
    I_em[	310	]=	1072.89672	;
    I_em[	311	]=	1084.83672	;
    I_em[	312	]=	1096.82952	;
    I_em[	313	]=	1108.88472	;
    I_em[	314	]=	1120.98912	;
    I_em[	315	]=	1133.14512	;
    I_em[	316	]=	1145.37432	;
    I_em[	317	]=	1157.65872	;
    I_em[	318	]=	1170.00192	;
    I_em[	319	]=	1182.42672	;
    I_em[	320	]=	1194.89832	;
    I_em[	321	]=	1207.44432	;
    I_em[	322	]=	1220.04552	;
    I_em[	323	]=	1232.71872	;
    I_em[	324	]=	1245.44832	;
    I_em[	325	]=	1258.24992	;
    I_em[	326	]=	1271.09952	;
    I_em[	327	]=	1283.99952	;
    I_em[	328	]=	1296.95952	;
    I_em[	329	]=	1309.96752	;
    I_em[	330	]=	1323.02232	;
    I_em[	331	]=	1336.13832	;
    I_em[	332	]=	1349.31312	;
    I_em[	333	]=	1362.52512	;
    I_em[	334	]=	1375.80072	;
    I_em[	335	]=	1389.07512	;
    I_em[	336	]=	1402.14072	;
    I_em[	337	]=	1414.92792	;
    I_em[	338	]=	1427.45592	;
    I_em[	339	]=	1439.71512	;
    I_em[	340	]=	1451.70552	;
    I_em[	341	]=	1463.43432	;
    I_em[	342	]=	1474.90152	;
    I_em[	343	]=	1486.09392	;
    I_em[	344	]=	1497.01632	;
    I_em[	345	]=	1507.67592	;
    I_em[	346	]=	1518.07872	;
    I_em[	347	]=	1528.20432	;
    I_em[	348	]=	1538.05872	;
    I_em[	349	]=	1547.66232	;
    I_em[	350	]=	1556.98992	;
    I_em[	351	]=	1566.04512	;
    I_em[	352	]=	1574.84352	;
    I_em[	353	]=	1583.37552	;
    I_em[	354	]=	1591.63632	;
    I_em[	355	]=	1599.62952	;
    I_em[	356	]=	1607.35752	;
    I_em[	357	]=	1614.81912	;
    I_em[	358	]=	1622.02392	;
    I_em[	359	]=	1628.95032	;
    I_em[	360	]=	1635.61632	;
    I_em[	361	]=	1642.00512	;
    I_em[	362	]=	1648.14192	;
    I_em[	363	]=	1654.00752	;
    I_em[	364	]=	1659.59952	;
    I_em[	365	]=	1664.94072	;
    I_em[	366	]=	1669.99872	;
    I_em[	367	]=	1674.79392	;
    I_em[	368	]=	1679.33232	;
    I_em[	369	]=	1683.59592	;
    I_em[	370	]=	1687.59672	;
    I_em[	371	]=	1691.32512	;
    I_em[	372	]=	1694.78592	;
    I_em[	373	]=	1697.98872	;
    I_em[	374	]=	1700.91912	;
    I_em[	375	]=	1703.59152	;
    I_em[	376	]=	1705.98312	;
    I_em[	377	]=	1708.11432	;
    I_em[	378	]=	1709.98392	;
    I_em[	379	]=	1711.58472	;
    I_em[	380	]=	1712.91552	;
    I_em[	381	]=	1713.98112	;
    I_em[	382	]=	1714.78872	;
    I_em[	383	]=	1715.31672	;
    I_em[	384	]=	1715.58912	;
    I_em[	385	]=	1715.62512	;
    for(int i = 386; i<tmax; i++)
    {
        I_em[i] = I_em[i-1];
    }
}

void setIemA2() //A2 ASF
{
    
    I_em[	0	]=	0	;
    I_em[	1	]=	0.05964	;
    I_em[	2	]=	0.1398	;
    I_em[	3	]=	0.22596	;
    I_em[	4	]=	0.31704	;
    I_em[	5	]=	0.4272	;
    I_em[	6	]=	0.54372	;
    I_em[	7	]=	0.65484	;
    I_em[	8	]=	0.79512	;
    I_em[	9	]=	0.93084	;
    I_em[	10	]=	1.0836	;
    I_em[	11	]=	1.24404	;
    I_em[	12	]=	1.41072	;
    I_em[	13	]=	1.596	;
    I_em[	14	]=	1.78764	;
    I_em[	15	]=	1.98612	;
    I_em[	16	]=	2.20344	;
    I_em[	17	]=	2.42736	;
    I_em[	18	]=	2.65788	;
    I_em[	19	]=	2.9076	;
    I_em[	20	]=	3.16428	;
    I_em[	21	]=	3.42744	;
    I_em[	22	]=	3.71016	;
    I_em[	23	]=	3.99996	;
    I_em[	24	]=	4.3086	;
    I_em[	25	]=	4.62492	;
    I_em[	26	]=	4.9482	;
    I_em[	27	]=	5.2788	;
    I_em[	28	]=	5.6286	;
    I_em[	29	]=	5.99808	;
    I_em[	30	]=	6.37368	;
    I_em[	31	]=	6.75768	;
    I_em[	32	]=	7.1604	;
    I_em[	33	]=	7.57068	;
    I_em[	34	]=	7.99836	;
    I_em[	35	]=	8.43192	;
    I_em[	36	]=	8.86992	;
    I_em[	37	]=	9.32244	;
    I_em[	38	]=	9.79008	;
    I_em[	39	]=	10.24692	;
    I_em[	40	]=	10.72944	;
    I_em[	41	]=	11.19828	;
    I_em[	42	]=	11.6814	;
    I_em[	43	]=	12.16056	;
    I_em[	44	]=	12.63504	;
    I_em[	45	]=	13.12128	;
    I_em[	46	]=	13.60092	;
    I_em[	47	]=	14.08596	;
    I_em[	48	]=	14.57832	;
    I_em[	49	]=	15.06552	;
    I_em[	50	]=	15.53028	;
    I_em[	51	]=	15.987	;
    I_em[	52	]=	16.44636	;
    I_em[	53	]=	16.88448	;
    I_em[	54	]=	17.31324	;
    I_em[	55	]=	17.74812	;
    I_em[	56	]=	18.1638	;
    I_em[	57	]=	18.5868	;
    I_em[	58	]=	18.993	;
    I_em[	59	]=	19.39536	;
    I_em[	60	]=	19.79844	;
    I_em[	61	]=	20.18088	;
    I_em[	62	]=	20.5818	;
    I_em[	63	]=	20.97864	;
    I_em[	64	]=	21.39036	;
    I_em[	65	]=	21.7956	;
    I_em[	66	]=	22.19124	;
    I_em[	67	]=	22.59084	;
    I_em[	68	]=	23.01876	;
    I_em[	69	]=	23.4558	;
    I_em[	70	]=	23.89236	;
    I_em[	71	]=	24.34584	;
    I_em[	72	]=	24.81348	;
    I_em[	73	]=	25.28928	;
    I_em[	74	]=	25.79352	;
    I_em[	75	]=	26.29644	;
    I_em[	76	]=	26.82348	;
    I_em[	77	]=	27.36588	;
    I_em[	78	]=	27.91392	;
    I_em[	79	]=	28.45692	;
    I_em[	80	]=	29.01024	;
    I_em[	81	]=	29.56068	;
    I_em[	82	]=	30.10248	;
    I_em[	83	]=	30.64116	;
    I_em[	84	]=	31.1826	;
    I_em[	85	]=	31.70844	;
    I_em[	86	]=	32.22432	;
    I_em[	87	]=	32.74824	;
    I_em[	88	]=	33.261	;
    I_em[	89	]=	33.7584	;
    I_em[	90	]=	34.24728	;
    I_em[	91	]=	34.7394	;
    I_em[	92	]=	35.2296	;
    I_em[	93	]=	35.70468	;
    I_em[	94	]=	36.18876	;
    I_em[	95	]=	36.67728	;
    I_em[	96	]=	37.15356	;
    I_em[	97	]=	37.64484	;
    I_em[	98	]=	38.14512	;
    I_em[	99	]=	38.65104	;
    I_em[	100	]=	39.16596	;
    I_em[	101	]=	39.69048	;
    I_em[	102	]=	40.25304	;
    I_em[	103	]=	40.81452	;
    I_em[	104	]=	41.38848	;
    I_em[	105	]=	41.98668	;
    I_em[	106	]=	42.59844	;
    I_em[	107	]=	43.22364	;
    I_em[	108	]=	43.87644	;
    I_em[	109	]=	44.5446	;
    I_em[	110	]=	45.24132	;
    I_em[	111	]=	45.96576	;
    I_em[	112	]=	46.70664	;
    I_em[	113	]=	47.47488	;
    I_em[	114	]=	48.26052	;
    I_em[	115	]=	49.07304	;
    I_em[	116	]=	49.89756	;
    I_em[	117	]=	50.74668	;
    I_em[	118	]=	51.6174	;
    I_em[	119	]=	52.52496	;
    I_em[	120	]=	53.44236	;
    I_em[	121	]=	54.37872	;
    I_em[	122	]=	55.31028	;
    I_em[	123	]=	56.27124	;
    I_em[	124	]=	57.23604	;
    I_em[	125	]=	58.21404	;
    I_em[	126	]=	59.19108	;
    I_em[	127	]=	60.16548	;
    I_em[	128	]=	61.13448	;
    I_em[	129	]=	62.09316	;
    I_em[	130	]=	63.05544	;
    I_em[	131	]=	64.00908	;
    I_em[	132	]=	64.97076	;
    I_em[	133	]=	65.9184	;
    I_em[	134	]=	66.8808	;
    I_em[	135	]=	67.85916	;
    I_em[	136	]=	68.84688	;
    I_em[	137	]=	69.84612	;
    I_em[	138	]=	70.8606	;
    I_em[	139	]=	71.90436	;
    I_em[	140	]=	72.978	;
    I_em[	141	]=	74.06772	;
    I_em[	142	]=	75.1842	;
    I_em[	143	]=	76.31172	;
    I_em[	144	]=	77.44872	;
    I_em[	145	]=	78.61656	;
    I_em[	146	]=	79.77912	;
    I_em[	147	]=	80.96988	;
    I_em[	148	]=	82.16496	;
    I_em[	149	]=	83.37708	;
    I_em[	150	]=	84.59028	;
    I_em[	151	]=	85.81836	;
    I_em[	152	]=	87.0486	;
    I_em[	153	]=	88.29108	;
    I_em[	154	]=	89.53416	;
    I_em[	155	]=	90.77748	;
    I_em[	156	]=	92.04228	;
    I_em[	157	]=	93.29652	;
    I_em[	158	]=	94.55892	;
    I_em[	159	]=	95.81508	;
    I_em[	160	]=	97.07772	;
    I_em[	161	]=	98.33064	;
    I_em[	162	]=	99.5736	;
    I_em[	163	]=	100.80156	;
    I_em[	164	]=	102.04488	;
    I_em[	165	]=	103.25256	;
    I_em[	166	]=	104.46156	;
    I_em[	167	]=	105.66264	;
    I_em[	168	]=	106.86984	;
    I_em[	169	]=	108.05976	;
    I_em[	170	]=	109.26132	;
    I_em[	171	]=	110.46276	;
    I_em[	172	]=	111.66828	;
    I_em[	173	]=	112.8816	;
    I_em[	174	]=	114.10488	;
    I_em[	175	]=	115.3536	;
    I_em[	176	]=	116.60928	;
    I_em[	177	]=	117.89436	;
    I_em[	178	]=	119.21808	;
    I_em[	179	]=	120.5664	;
    I_em[	180	]=	121.95792	;
    I_em[	181	]=	123.37824	;
    I_em[	182	]=	124.85712	;
    I_em[	183	]=	126.36912	;
    I_em[	184	]=	127.93056	;
    I_em[	185	]=	129.5412	;
    I_em[	186	]=	131.21652	;
    I_em[	187	]=	132.93384	;
    I_em[	188	]=	134.721	;
    I_em[	189	]=	136.56708	;
    I_em[	190	]=	138.48744	;
    I_em[	191	]=	140.4726	;
    I_em[	192	]=	142.53708	;
    I_em[	193	]=	144.6876	;
    I_em[	194	]=	146.97636	;
    I_em[	195	]=	149.69388	;
    I_em[	196	]=	152.28828	;
    I_em[	197	]=	154.93656	;
    I_em[	198	]=	157.24776	;
    I_em[	199	]=	159.33492	;
    I_em[	200	]=	161.59884	;
    I_em[	201	]=	164.87568	;
    I_em[	202	]=	167.87496	;
    I_em[	203	]=	171.05784	;
    I_em[	204	]=	175.10496	;
    I_em[	205	]=	178.72056	;
    I_em[	206	]=	181.67772	;
    I_em[	207	]=	185.3754	;
    I_em[	208	]=	190.51704	;
    I_em[	209	]=	194.08548	;
    I_em[	210	]=	197.26212	;
    I_em[	211	]=	200.862	;
    I_em[	212	]=	205.59096	;
    I_em[	213	]=	210.60324	;
    I_em[	214	]=	215.30448	;
    I_em[	215	]=	220.44912	;
    I_em[	216	]=	225.07044	;
    I_em[	217	]=	229.32912	;
    I_em[	218	]=	234.34128	;
    I_em[	219	]=	239.66664	;
    I_em[	220	]=	244.60728	;
    I_em[	221	]=	249.29916	;
    I_em[	222	]=	254.93004	;
    I_em[	223	]=	261.65832	;
    I_em[	224	]=	267.27372	;
    I_em[	225	]=	272.10456	;
    I_em[	226	]=	276.95856	;
    I_em[	227	]=	280.80648	;
    I_em[	228	]=	284.45292	;
    I_em[	229	]=	289.51176	;
    I_em[	230	]=	294.01644	;
    I_em[	231	]=	299.29944	;
    I_em[	232	]=	304.23168	;
    I_em[	233	]=	311.2686	;
    I_em[	234	]=	318.06756	;
    I_em[	235	]=	323.31228	;
    I_em[	236	]=	328.76628	;
    I_em[	237	]=	335.19948	;
    I_em[	238	]=	342.56232	;
    I_em[	239	]=	349.15008	;
    I_em[	240	]=	355.75632	;
    I_em[	241	]=	362.64336	;
    I_em[	242	]=	369.14064	;
    I_em[	243	]=	375.83124	;
    I_em[	244	]=	381.7788	;
    I_em[	245	]=	389.31768	;
    I_em[	246	]=	399.17064	;
    I_em[	247	]=	409.2888	;
    I_em[	248	]=	419.68056	;
    I_em[	249	]=	430.3302	;
    I_em[	250	]=	441.255	;
    I_em[	251	]=	452.43564	;
    I_em[	252	]=	463.89204	;
    I_em[	253	]=	475.62444	;
    I_em[	254	]=	487.61604	;
    I_em[	255	]=	499.872	;
    I_em[	256	]=	512.37336	;
    I_em[	257	]=	525.12888	;
    I_em[	258	]=	538.12944	;
    I_em[	259	]=	551.37744	;
    I_em[	260	]=	564.87432	;
    I_em[	261	]=	578.62272	;
    I_em[	262	]=	592.6098	;
    I_em[	263	]=	606.85128	;
    I_em[	264	]=	621.33576	;
    I_em[	265	]=	636.04776	;
    I_em[	266	]=	650.91336	;
    I_em[	267	]=	665.91276	;
    I_em[	268	]=	681.04884	;
    I_em[	269	]=	696.32148	;
    I_em[	270	]=	711.7266	;
    I_em[	271	]=	727.25856	;
    I_em[	272	]=	742.9392	;
    I_em[	273	]=	758.75292	;
    I_em[	274	]=	774.6924	;
    I_em[	275	]=	790.7712	;
    I_em[	276	]=	806.991	;
    I_em[	277	]=	823.33884	;
    I_em[	278	]=	839.82984	;
    I_em[	279	]=	856.4526	;
    I_em[	280	]=	873.20544	;
    I_em[	281	]=	890.09496	;
    I_em[	282	]=	907.12932	;
    I_em[	283	]=	924.28992	;
    I_em[	284	]=	941.57832	;
    I_em[	285	]=	959.02272	;
    I_em[	286	]=	976.62312	;
    I_em[	287	]=	994.39992	;
    I_em[	288	]=	1012.36152	;
    I_em[	289	]=	1030.47672	;
    I_em[	290	]=	1048.78032	;
    I_em[	291	]=	1067.25072	;
    I_em[	292	]=	1085.88672	;
    I_em[	293	]=	1104.70032	;
    I_em[	294	]=	1123.69032	;
    I_em[	295	]=	1142.86152	;
    I_em[	296	]=	1162.18512	;
    I_em[	297	]=	1181.70192	;
    I_em[	298	]=	1201.37712	;
    I_em[	299	]=	1221.23472	;
    I_em[	300	]=	1241.25792	;
    I_em[	301	]=	1261.45512	;
    I_em[	302	]=	1281.83352	;
    I_em[	303	]=	1302.37392	;
    I_em[	304	]=	1323.09312	;
    I_em[	305	]=	1343.99712	;
    I_em[	306	]=	1365.11832	;
    I_em[	307	]=	1386.47232	;
    I_em[	308	]=	1408.06872	;
    I_em[	309	]=	1429.88712	;
    I_em[	310	]=	1451.94552	;
    I_em[	311	]=	1474.22952	;
    I_em[	312	]=	1496.75712	;
    I_em[	313	]=	1519.51872	;
    I_em[	314	]=	1542.49992	;
    I_em[	315	]=	1565.73552	;
    I_em[	316	]=	1589.24472	;
    I_em[	317	]=	1613.05272	;
    I_em[	318	]=	1637.15592	;
    I_em[	319	]=	1661.56032	;
    I_em[	320	]=	1686.25152	;
    I_em[	321	]=	1711.23192	;
    I_em[	322	]=	1736.51472	;
    I_em[	323	]=	1762.08552	;
    I_em[	324	]=	1787.94672	;
    I_em[	325	]=	1814.10072	;
    I_em[	326	]=	1840.55712	;
    I_em[	327	]=	1867.30392	;
    I_em[	328	]=	1894.34832	;
    I_em[	329	]=	1921.68072	;
    I_em[	330	]=	1949.29992	;
    I_em[	331	]=	1977.22032	;
    I_em[	332	]=	2005.43592	;
    I_em[	333	]=	2033.93472	;
    I_em[	334	]=	2062.74192	;
    I_em[	335	]=	2091.68592	;
    I_em[	336	]=	2120.19552	;
    I_em[	337	]=	2148.12312	;
    I_em[	338	]=	2175.46992	;
    I_em[	339	]=	2202.23352	;
    I_em[	340	]=	2228.42352	;
    I_em[	341	]=	2254.02792	;
    I_em[	342	]=	2279.04792	;
    I_em[	343	]=	2303.48712	;
    I_em[	344	]=	2327.33472	;
    I_em[	345	]=	2350.61712	;
    I_em[	346	]=	2373.30912	;
    I_em[	347	]=	2395.41912	;
    I_em[	348	]=	2416.94352	;
    I_em[	349	]=	2437.89192	;
    I_em[	350	]=	2458.25472	;
    I_em[	351	]=	2478.04152	;
    I_em[	352	]=	2497.24032	;
    I_em[	353	]=	2515.85472	;
    I_em[	354	]=	2533.89792	;
    I_em[	355	]=	2551.35552	;
    I_em[	356	]=	2568.22272	;
    I_em[	357	]=	2584.52112	;
    I_em[	358	]=	2600.23152	;
    I_em[	359	]=	2615.35992	;
    I_em[	360	]=	2629.90392	;
    I_em[	361	]=	2643.87432	;
    I_em[	362	]=	2657.25432	;
    I_em[	363	]=	2670.06072	;
    I_em[	364	]=	2682.27072	;
    I_em[	365	]=	2693.90952	;
    I_em[	366	]=	2704.96392	;
    I_em[	367	]=	2715.44472	;
    I_em[	368	]=	2725.33632	;
    I_em[	369	]=	2734.64472	;
    I_em[	370	]=	2743.36992	;
    I_em[	371	]=	2751.51552	;
    I_em[	372	]=	2759.07912	;
    I_em[	373	]=	2766.06072	;
    I_em[	374	]=	2772.47112	;
    I_em[	375	]=	2778.28752	;
    I_em[	376	]=	2783.52312	;
    I_em[	377	]=	2788.17552	;
    I_em[	378	]=	2792.24952	;
    I_em[	379	]=	2795.74032	;
    I_em[	380	]=	2798.65272	;
    I_em[	381	]=	2800.98312	;
    I_em[	382	]=	2802.73392	;
    I_em[	383	]=	2803.89192	;
    I_em[	384	]=	2804.48352	;
    I_em[	385	]=	2804.58072	;
    for(int i = 386; i<tmax; i++)
    {
        I_em[i] = I_em[i-1];
    }
}

void setIemA1G() //A1G MINICAM
{
    
    I_em[	0	]=	0	;
    I_em[	1	]=	0.05964	;
    I_em[	2	]=	0.1398	;
    I_em[	3	]=	0.22596	;
    I_em[	4	]=	0.31704	;
    I_em[	5	]=	0.4272	;
    I_em[	6	]=	0.54372	;
    I_em[	7	]=	0.65484	;
    I_em[	8	]=	0.79512	;
    I_em[	9	]=	0.93084	;
    I_em[	10	]=	1.0836	;
    I_em[	11	]=	1.24404	;
    I_em[	12	]=	1.41072	;
    I_em[	13	]=	1.596	;
    I_em[	14	]=	1.78764	;
    I_em[	15	]=	1.98612	;
    I_em[	16	]=	2.20344	;
    I_em[	17	]=	2.42736	;
    I_em[	18	]=	2.65788	;
    I_em[	19	]=	2.9076	;
    I_em[	20	]=	3.16428	;
    I_em[	21	]=	3.42744	;
    I_em[	22	]=	3.71016	;
    I_em[	23	]=	3.99996	;
    I_em[	24	]=	4.3086	;
    I_em[	25	]=	4.62492	;
    I_em[	26	]=	4.9482	;
    I_em[	27	]=	5.2788	;
    I_em[	28	]=	5.6286	;
    I_em[	29	]=	5.99808	;
    I_em[	30	]=	6.37368	;
    I_em[	31	]=	6.75768	;
    I_em[	32	]=	7.1604	;
    I_em[	33	]=	7.57068	;
    I_em[	34	]=	7.99836	;
    I_em[	35	]=	8.43192	;
    I_em[	36	]=	8.86992	;
    I_em[	37	]=	9.32244	;
    I_em[	38	]=	9.79008	;
    I_em[	39	]=	10.24692	;
    I_em[	40	]=	10.72944	;
    I_em[	41	]=	11.19828	;
    I_em[	42	]=	11.6814	;
    I_em[	43	]=	12.16056	;
    I_em[	44	]=	12.63504	;
    I_em[	45	]=	13.12128	;
    I_em[	46	]=	13.60092	;
    I_em[	47	]=	14.08596	;
    I_em[	48	]=	14.57832	;
    I_em[	49	]=	15.06552	;
    I_em[	50	]=	15.53028	;
    I_em[	51	]=	15.987	;
    I_em[	52	]=	16.44636	;
    I_em[	53	]=	16.88448	;
    I_em[	54	]=	17.31324	;
    I_em[	55	]=	17.74812	;
    I_em[	56	]=	18.1638	;
    I_em[	57	]=	18.5868	;
    I_em[	58	]=	18.993	;
    I_em[	59	]=	19.39536	;
    I_em[	60	]=	19.79844	;
    I_em[	61	]=	20.18088	;
    I_em[	62	]=	20.5818	;
    I_em[	63	]=	20.97864	;
    I_em[	64	]=	21.39036	;
    I_em[	65	]=	21.7956	;
    I_em[	66	]=	22.19124	;
    I_em[	67	]=	22.59084	;
    I_em[	68	]=	23.01876	;
    I_em[	69	]=	23.4558	;
    I_em[	70	]=	23.89236	;
    I_em[	71	]=	24.34584	;
    I_em[	72	]=	24.81348	;
    I_em[	73	]=	25.28928	;
    I_em[	74	]=	25.79352	;
    I_em[	75	]=	26.29644	;
    I_em[	76	]=	26.82348	;
    I_em[	77	]=	27.36588	;
    I_em[	78	]=	27.91392	;
    I_em[	79	]=	28.45692	;
    I_em[	80	]=	29.01024	;
    I_em[	81	]=	29.56068	;
    I_em[	82	]=	30.10248	;
    I_em[	83	]=	30.64116	;
    I_em[	84	]=	31.1826	;
    I_em[	85	]=	31.70844	;
    I_em[	86	]=	32.22432	;
    I_em[	87	]=	32.74824	;
    I_em[	88	]=	33.261	;
    I_em[	89	]=	33.7584	;
    I_em[	90	]=	34.24728	;
    I_em[	91	]=	34.7394	;
    I_em[	92	]=	35.2296	;
    I_em[	93	]=	35.70468	;
    I_em[	94	]=	36.18876	;
    I_em[	95	]=	36.67728	;
    I_em[	96	]=	37.15356	;
    I_em[	97	]=	37.64484	;
    I_em[	98	]=	38.14512	;
    I_em[	99	]=	38.65104	;
    I_em[	100	]=	39.16596	;
    I_em[	101	]=	39.69048	;
    I_em[	102	]=	40.25304	;
    I_em[	103	]=	40.81452	;
    I_em[	104	]=	41.38848	;
    I_em[	105	]=	41.98668	;
    I_em[	106	]=	42.59844	;
    I_em[	107	]=	43.22364	;
    I_em[	108	]=	43.87644	;
    I_em[	109	]=	44.5446	;
    I_em[	110	]=	45.24132	;
    I_em[	111	]=	45.96576	;
    I_em[	112	]=	46.70664	;
    I_em[	113	]=	47.47488	;
    I_em[	114	]=	48.26052	;
    I_em[	115	]=	49.07304	;
    I_em[	116	]=	49.89756	;
    I_em[	117	]=	50.74668	;
    I_em[	118	]=	51.6174	;
    I_em[	119	]=	52.52496	;
    I_em[	120	]=	53.44236	;
    I_em[	121	]=	54.37872	;
    I_em[	122	]=	55.31028	;
    I_em[	123	]=	56.27124	;
    I_em[	124	]=	57.23604	;
    I_em[	125	]=	58.21404	;
    I_em[	126	]=	59.19108	;
    I_em[	127	]=	60.16548	;
    I_em[	128	]=	61.13448	;
    I_em[	129	]=	62.09316	;
    I_em[	130	]=	63.05544	;
    I_em[	131	]=	64.00908	;
    I_em[	132	]=	64.97076	;
    I_em[	133	]=	65.9184	;
    I_em[	134	]=	66.8808	;
    I_em[	135	]=	67.85916	;
    I_em[	136	]=	68.84688	;
    I_em[	137	]=	69.84612	;
    I_em[	138	]=	70.8606	;
    I_em[	139	]=	71.90436	;
    I_em[	140	]=	72.978	;
    I_em[	141	]=	74.06772	;
    I_em[	142	]=	75.1842	;
    I_em[	143	]=	76.31172	;
    I_em[	144	]=	77.44872	;
    I_em[	145	]=	78.61656	;
    I_em[	146	]=	79.77912	;
    I_em[	147	]=	80.96988	;
    I_em[	148	]=	82.16496	;
    I_em[	149	]=	83.37708	;
    I_em[	150	]=	84.59028	;
    I_em[	151	]=	85.81836	;
    I_em[	152	]=	87.0486	;
    I_em[	153	]=	88.29108	;
    I_em[	154	]=	89.53416	;
    I_em[	155	]=	90.77748	;
    I_em[	156	]=	92.04228	;
    I_em[	157	]=	93.29652	;
    I_em[	158	]=	94.55892	;
    I_em[	159	]=	95.81508	;
    I_em[	160	]=	97.07772	;
    I_em[	161	]=	98.33064	;
    I_em[	162	]=	99.5736	;
    I_em[	163	]=	100.80156	;
    I_em[	164	]=	102.04488	;
    I_em[	165	]=	103.25256	;
    I_em[	166	]=	104.46156	;
    I_em[	167	]=	105.66264	;
    I_em[	168	]=	106.86984	;
    I_em[	169	]=	108.05976	;
    I_em[	170	]=	109.26132	;
    I_em[	171	]=	110.46276	;
    I_em[	172	]=	111.66828	;
    I_em[	173	]=	112.8816	;
    I_em[	174	]=	114.10488	;
    I_em[	175	]=	115.3536	;
    I_em[	176	]=	116.60928	;
    I_em[	177	]=	117.89436	;
    I_em[	178	]=	119.21808	;
    I_em[	179	]=	120.5664	;
    I_em[	180	]=	121.95792	;
    I_em[	181	]=	123.37824	;
    I_em[	182	]=	124.85712	;
    I_em[	183	]=	126.36912	;
    I_em[	184	]=	127.93056	;
    I_em[	185	]=	129.5412	;
    I_em[	186	]=	131.21652	;
    I_em[	187	]=	132.93384	;
    I_em[	188	]=	134.721	;
    I_em[	189	]=	136.56708	;
    I_em[	190	]=	138.48744	;
    I_em[	191	]=	140.4726	;
    I_em[	192	]=	142.53708	;
    I_em[	193	]=	144.6876	;
    I_em[	194	]=	146.97636	;
    I_em[	195	]=	149.69388	;
    I_em[	196	]=	152.28828	;
    I_em[	197	]=	154.93656	;
    I_em[	198	]=	157.24776	;
    I_em[	199	]=	159.33492	;
    I_em[	200	]=	161.59884	;
    I_em[	201	]=	164.87568	;
    I_em[	202	]=	167.87496	;
    I_em[	203	]=	171.05784	;
    I_em[	204	]=	175.10496	;
    I_em[	205	]=	178.72056	;
    I_em[	206	]=	181.67772	;
    I_em[	207	]=	185.3754	;
    I_em[	208	]=	190.51704	;
    I_em[	209	]=	194.08548	;
    I_em[	210	]=	197.26212	;
    I_em[	211	]=	200.862	;
    I_em[	212	]=	205.59096	;
    I_em[	213	]=	210.60324	;
    I_em[	214	]=	215.30448	;
    I_em[	215	]=	220.44912	;
    I_em[	216	]=	225.07044	;
    I_em[	217	]=	229.32912	;
    I_em[	218	]=	234.34128	;
    I_em[	219	]=	239.66664	;
    I_em[	220	]=	244.60728	;
    I_em[	221	]=	249.29916	;
    I_em[	222	]=	254.93004	;
    I_em[	223	]=	261.65832	;
    I_em[	224	]=	267.27372	;
    I_em[	225	]=	272.10456	;
    I_em[	226	]=	276.95856	;
    I_em[	227	]=	280.80648	;
    I_em[	228	]=	284.45292	;
    I_em[	229	]=	289.51176	;
    I_em[	230	]=	294.01644	;
    I_em[	231	]=	299.29944	;
    I_em[	232	]=	304.23168	;
    I_em[	233	]=	311.2686	;
    I_em[	234	]=	318.06756	;
    I_em[	235	]=	323.31228	;
    I_em[	236	]=	328.76628	;
    I_em[	237	]=	335.19948	;
    I_em[	238	]=	342.56232	;
    I_em[	239	]=	349.15008	;
    I_em[	240	]=	355.75632	;
    I_em[	241	]=	362.64336	;
    I_em[	242	]=	369.14064	;
    I_em[	243	]=	375.83124	;
    I_em[	244	]=	381.7788	;
    I_em[	245	]=	389.39724	;
    I_em[	246	]=	399.43932	;
    I_em[	247	]=	409.7826	;
    I_em[	248	]=	420.42456	;
    I_em[	249	]=	431.36148	;
    I_em[	250	]=	442.59828	;
    I_em[	251	]=	454.14	;
    I_em[	252	]=	465.97032	;
    I_em[	253	]=	478.11744	;
    I_em[	254	]=	490.55496	;
    I_em[	255	]=	503.298	;
    I_em[	256	]=	516.38604	;
    I_em[	257	]=	529.81344	;
    I_em[	258	]=	543.59832	;
    I_em[	259	]=	557.71332	;
    I_em[	260	]=	572.18352	;
    I_em[	261	]=	586.98876	;
    I_em[	262	]=	602.14788	;
    I_em[	263	]=	617.65476	;
    I_em[	264	]=	633.49884	;
    I_em[	265	]=	649.70052	;
    I_em[	266	]=	666.27768	;
    I_em[	267	]=	683.22828	;
    I_em[	268	]=	700.55184	;
    I_em[	269	]=	718.26432	;
    I_em[	270	]=	736.34664	;
    I_em[	271	]=	754.80708	;
    I_em[	272	]=	773.65956	;
    I_em[	273	]=	792.8766	;
    I_em[	274	]=	812.47464	;
    I_em[	275	]=	832.45788	;
    I_em[	276	]=	852.82752	;
    I_em[	277	]=	873.59508	;
    I_em[	278	]=	894.75132	;
    I_em[	279	]=	916.29672	;
    I_em[	280	]=	938.23752	;
    I_em[	281	]=	960.58272	;
    I_em[	282	]=	983.30112	;
    I_em[	283	]=	1006.42752	;
    I_em[	284	]=	1029.94032	;
    I_em[	285	]=	1053.80832	;
    I_em[	286	]=	1077.89352	;
    I_em[	287	]=	1102.15992	;
    I_em[	288	]=	1126.61112	;
    I_em[	289	]=	1151.22672	;
    I_em[	290	]=	1176.03192	;
    I_em[	291	]=	1201.00872	;
    I_em[	292	]=	1226.17032	;
    I_em[	293	]=	1251.50832	;
    I_em[	294	]=	1277.03352	;
    I_em[	295	]=	1302.72072	;
    I_em[	296	]=	1328.58312	;
    I_em[	297	]=	1354.59672	;
    I_em[	298	]=	1380.77592	;
    I_em[	299	]=	1407.10272	;
    I_em[	300	]=	1433.59392	;
    I_em[	301	]=	1460.24952	;
    I_em[	302	]=	1487.05152	;
    I_em[	303	]=	1514.02272	;
    I_em[	304	]=	1541.15472	;
    I_em[	305	]=	1568.44152	;
    I_em[	306	]=	1595.87232	;
    I_em[	307	]=	1623.44352	;
    I_em[	308	]=	1651.15152	;
    I_em[	309	]=	1678.99992	;
    I_em[	310	]=	1706.98872	;
    I_em[	311	]=	1735.10952	;
    I_em[	312	]=	1763.37552	;
    I_em[	313	]=	1791.78912	;
    I_em[	314	]=	1820.34072	;
    I_em[	315	]=	1848.99672	;
    I_em[	316	]=	1877.65872	;
    I_em[	317	]=	1906.29192	;
    I_em[	318	]=	1934.91072	;
    I_em[	319	]=	1963.49712	;
    I_em[	320	]=	1992.05232	;
    I_em[	321	]=	2020.58352	;
    I_em[	322	]=	2049.08952	;
    I_em[	323	]=	2077.57632	;
    I_em[	324	]=	2106.03552	;
    I_em[	325	]=	2134.46832	;
    I_em[	326	]=	2162.87952	;
    I_em[	327	]=	2191.26672	;
    I_em[	328	]=	2219.65032	;
    I_em[	329	]=	2247.99672	;
    I_em[	330	]=	2276.33352	;
    I_em[	331	]=	2304.66312	;
    I_em[	332	]=	2332.95672	;
    I_em[	333	]=	2361.24192	;
    I_em[	334	]=	2389.50552	;
    I_em[	335	]=	2417.65872	;
    I_em[	336	]=	2445.32952	;
    I_em[	337	]=	2472.44832	;
    I_em[	338	]=	2498.98992	;
    I_em[	339	]=	2524.97352	;
    I_em[	340	]=	2550.39192	;
    I_em[	341	]=	2575.24632	;
    I_em[	342	]=	2599.53912	;
    I_em[	343	]=	2623.25712	;
    I_em[	344	]=	2646.42432	;
    I_em[	345	]=	2669.02152	;
    I_em[	346	]=	2691.04512	;
    I_em[	347	]=	2712.50592	;
    I_em[	348	]=	2733.40872	;
    I_em[	349	]=	2753.74032	;
    I_em[	350	]=	2773.51272	;
    I_em[	351	]=	2792.71632	;
    I_em[	352	]=	2811.36192	;
    I_em[	353	]=	2829.43392	;
    I_em[	354	]=	2846.94552	;
    I_em[	355	]=	2863.88952	;
    I_em[	356	]=	2880.27432	;
    I_em[	357	]=	2896.08912	;
    I_em[	358	]=	2911.34712	;
    I_em[	359	]=	2926.03632	;
    I_em[	360	]=	2940.15672	;
    I_em[	361	]=	2953.70712	;
    I_em[	362	]=	2966.70072	;
    I_em[	363	]=	2979.13512	;
    I_em[	364	]=	2990.99112	;
    I_em[	365	]=	3002.28792	;
    I_em[	366	]=	3013.02552	;
    I_em[	367	]=	3023.19072	;
    I_em[	368	]=	3032.78712	;
    I_em[	369	]=	3041.82672	;
    I_em[	370	]=	3050.30952	;
    I_em[	371	]=	3058.21032	;
    I_em[	372	]=	3065.55552	;
    I_em[	373	]=	3072.34272	;
    I_em[	374	]=	3078.54792	;
    I_em[	375	]=	3084.20712	;
    I_em[	376	]=	3089.28312	;
    I_em[	377	]=	3093.80472	;
    I_em[	378	]=	3097.75632	;
    I_em[	379	]=	3101.15112	;
    I_em[	380	]=	3103.97832	;
    I_em[	381	]=	3106.23672	;
    I_em[	382	]=	3107.93952	;
    I_em[	383	]=	3109.06272	;
    I_em[	384	]=	3109.62912	;
    I_em[	385	]=	3109.72272	;
    for(int i = 386; i<tmax; i++)
    {
        I_em[i] = I_em[i-1];
    }
}
    
//Now Non_CO2 radiative forcing
void setRnonCO2_RCP3PD(void)
{
    RnonCO2[	0	]=	-0.019988515	;
    RnonCO2[	1	]=	-0.005145555	;
    RnonCO2[	2	]=	0.009090068	;
    RnonCO2[	3	]=	0.007768629	;
    RnonCO2[	4	]=	0.006256789	;
    RnonCO2[	5	]=	0.004769658	;
    RnonCO2[	6	]=	0.002906336	;
    RnonCO2[	7	]=	0.001080178	;
    RnonCO2[	8	]=	-0.000291468	;
    RnonCO2[	9	]=	-0.001612412	;
    RnonCO2[	10	]=	-0.00288359	;
    RnonCO2[	11	]=	-0.00410611	;
    RnonCO2[	12	]=	-0.005282233	;
    RnonCO2[	13	]=	-0.006413549	;
    RnonCO2[	14	]=	-0.007501746	;
    RnonCO2[	15	]=	-0.008551056	;
    RnonCO2[	16	]=	-0.010019335	;
    RnonCO2[	17	]=	-0.011459991	;
    RnonCO2[	18	]=	-0.012417579	;
    RnonCO2[	19	]=	-0.013333977	;
    RnonCO2[	20	]=	-0.014201345	;
    RnonCO2[	21	]=	-0.015012487	;
    RnonCO2[	22	]=	-0.015760238	;
    RnonCO2[	23	]=	-0.016437067	;
    RnonCO2[	24	]=	-0.017035543	;
    RnonCO2[	25	]=	-0.017549544	;
    RnonCO2[	26	]=	-0.018434793	;
    RnonCO2[	27	]=	-0.019238993	;
    RnonCO2[	28	]=	-0.019510794	;
    RnonCO2[	29	]=	-0.019709367	;
    RnonCO2[	30	]=	-0.019838439	;
    RnonCO2[	31	]=	-0.019901316	;
    RnonCO2[	32	]=	-0.020355353	;
    RnonCO2[	33	]=	-0.020749651	;
    RnonCO2[	34	]=	-0.020634193	;
    RnonCO2[	35	]=	-0.02091346	;
    RnonCO2[	36	]=	-0.021125633	;
    RnonCO2[	37	]=	-0.02171132	;
    RnonCO2[	38	]=	-0.021774359	;
    RnonCO2[	39	]=	-0.020873209	;
    RnonCO2[	40	]=	-0.02036553	;
    RnonCO2[	41	]=	-0.020256367	;
    RnonCO2[	42	]=	-0.020105185	;
    RnonCO2[	43	]=	-0.01947287	;
    RnonCO2[	44	]=	-0.018814746	;
    RnonCO2[	45	]=	-0.018134946	;
    RnonCO2[	46	]=	-0.017872094	;
    RnonCO2[	47	]=	-0.017576381	;
    RnonCO2[	48	]=	-0.016804485	;
    RnonCO2[	49	]=	-0.016001407	;
    RnonCO2[	50	]=	-0.015606441	;
    RnonCO2[	51	]=	-0.015619264	;
    RnonCO2[	52	]=	-0.015600061	;
    RnonCO2[	53	]=	-0.015116575	;
    RnonCO2[	54	]=	-0.014172313	;
    RnonCO2[	55	]=	-0.013205379	;
    RnonCO2[	56	]=	-0.012216093	;
    RnonCO2[	57	]=	-0.011635129	;
    RnonCO2[	58	]=	-0.011459514	;
    RnonCO2[	59	]=	-0.010829842	;
    RnonCO2[	60	]=	-0.010175791	;
    RnonCO2[	61	]=	-0.009499636	;
    RnonCO2[	62	]=	-0.00879934	;
    RnonCO2[	63	]=	-0.008077943	;
    RnonCO2[	64	]=	-0.006912545	;
    RnonCO2[	65	]=	-0.008242223	;
    RnonCO2[	66	]=	-0.009135821	;
    RnonCO2[	67	]=	-0.007501052	;
    RnonCO2[	68	]=	-0.006687418	;
    RnonCO2[	69	]=	-0.005858015	;
    RnonCO2[	70	]=	-0.005005979	;
    RnonCO2[	71	]=	-0.005767925	;
    RnonCO2[	72	]=	-0.006086639	;
    RnonCO2[	73	]=	-0.005137938	;
    RnonCO2[	74	]=	-0.004558881	;
    RnonCO2[	75	]=	-0.004336125	;
    RnonCO2[	76	]=	-0.004045198	;
    RnonCO2[	77	]=	-0.003665109	;
    RnonCO2[	78	]=	-0.003220213	;
    RnonCO2[	79	]=	-0.002760518	;
    RnonCO2[	80	]=	-0.003527376	;
    RnonCO2[	81	]=	-0.003576651	;
    RnonCO2[	82	]=	-0.003275283	;
    RnonCO2[	83	]=	-0.003386771	;
    RnonCO2[	84	]=	-0.003510917	;
    RnonCO2[	85	]=	-0.004646356	;
    RnonCO2[	86	]=	-0.003691576	;
    RnonCO2[	87	]=	-0.009505158	;
    RnonCO2[	88	]=	-0.016364432	;
    RnonCO2[	89	]=	-0.017996784	;
    RnonCO2[	90	]=	-0.019670058	;
    RnonCO2[	91	]=	-0.020430126	;
    RnonCO2[	92	]=	-0.021643321	;
    RnonCO2[	93	]=	-0.02142206	;
    RnonCO2[	94	]=	-0.022552345	;
    RnonCO2[	95	]=	-0.031588211	;
    RnonCO2[	96	]=	-0.046252985	;
    RnonCO2[	97	]=	-0.042365448	;
    RnonCO2[	98	]=	-0.033665717	;
    RnonCO2[	99	]=	-0.034555638	;
    RnonCO2[	100	]=	-0.037298509	;
    RnonCO2[	101	]=	-0.046083195	;
    RnonCO2[	102	]=	-0.05558168	;
    RnonCO2[	103	]=	-0.057464266	;
    RnonCO2[	104	]=	-0.053606449	;
    RnonCO2[	105	]=	-0.05168834	;
    RnonCO2[	106	]=	-0.061304945	;
    RnonCO2[	107	]=	-0.086608149	;
    RnonCO2[	108	]=	-0.108195137	;
    RnonCO2[	109	]=	-0.112259625	;
    RnonCO2[	110	]=	-0.111643119	;
    RnonCO2[	111	]=	-0.108991576	;
    RnonCO2[	112	]=	-0.10203489	;
    RnonCO2[	113	]=	-0.098533227	;
    RnonCO2[	114	]=	-0.101949678	;
    RnonCO2[	115	]=	-0.114562053	;
    RnonCO2[	116	]=	-0.123024156	;
    RnonCO2[	117	]=	-0.119809652	;
    RnonCO2[	118	]=	-0.118941351	;
    RnonCO2[	119	]=	-0.120181779	;
    RnonCO2[	120	]=	-0.117316831	;
    RnonCO2[	121	]=	-0.115023243	;
    RnonCO2[	122	]=	-0.116983521	;
    RnonCO2[	123	]=	-0.125997633	;
    RnonCO2[	124	]=	-0.129568227	;
    RnonCO2[	125	]=	-0.131248987	;
    RnonCO2[	126	]=	-0.137316696	;
    RnonCO2[	127	]=	-0.141273792	;
    RnonCO2[	128	]=	-0.144231375	;
    RnonCO2[	129	]=	-0.147973341	;
    RnonCO2[	130	]=	-0.153688224	;
    RnonCO2[	131	]=	-0.15652816	;
    RnonCO2[	132	]=	-0.157109013	;
    RnonCO2[	133	]=	-0.158858513	;
    RnonCO2[	134	]=	-0.162186704	;
    RnonCO2[	135	]=	-0.165318228	;
    RnonCO2[	136	]=	-0.171733193	;
    RnonCO2[	137	]=	-0.178096644	;
    RnonCO2[	138	]=	-0.183870407	;
    RnonCO2[	139	]=	-0.189730028	;
    RnonCO2[	140	]=	-0.194221589	;
    RnonCO2[	141	]=	-0.201576481	;
    RnonCO2[	142	]=	-0.208859035	;
    RnonCO2[	143	]=	-0.207375441	;
    RnonCO2[	144	]=	-0.202571711	;
    RnonCO2[	145	]=	-0.204126848	;
    RnonCO2[	146	]=	-0.201876709	;
    RnonCO2[	147	]=	-0.195147841	;
    RnonCO2[	148	]=	-0.191449584	;
    RnonCO2[	149	]=	-0.183770215	;
    RnonCO2[	150	]=	-0.173476305	;
    RnonCO2[	151	]=	-0.170909799	;
    RnonCO2[	152	]=	-0.171515561	;
    RnonCO2[	153	]=	-0.169079801	;
    RnonCO2[	154	]=	-0.159414516	;
    RnonCO2[	155	]=	-0.155013584	;
    RnonCO2[	156	]=	-0.151089869	;
    RnonCO2[	157	]=	-0.144432651	;
    RnonCO2[	158	]=	-0.152359798	;
    RnonCO2[	159	]=	-0.158148636	;
    RnonCO2[	160	]=	-0.155277608	;
    RnonCO2[	161	]=	-0.152617947	;
    RnonCO2[	162	]=	-0.154120923	;
    RnonCO2[	163	]=	-0.156019482	;
    RnonCO2[	164	]=	-0.155886394	;
    RnonCO2[	165	]=	-0.152021034	;
    RnonCO2[	166	]=	-0.141834688	;
    RnonCO2[	167	]=	-0.13108946	;
    RnonCO2[	168	]=	-0.125408882	;
    RnonCO2[	169	]=	-0.125515632	;
    RnonCO2[	170	]=	-0.125194614	;
    RnonCO2[	171	]=	-0.126120729	;
    RnonCO2[	172	]=	-0.12532345	;
    RnonCO2[	173	]=	-0.115303554	;
    RnonCO2[	174	]=	-0.10787249	;
    RnonCO2[	175	]=	-0.109257271	;
    RnonCO2[	176	]=	-0.113724728	;
    RnonCO2[	177	]=	-0.117421621	;
    RnonCO2[	178	]=	-0.11813163	;
    RnonCO2[	179	]=	-0.119629474	;
    RnonCO2[	180	]=	-0.119789192	;
    RnonCO2[	181	]=	-0.124270593	;
    RnonCO2[	182	]=	-0.135220142	;
    RnonCO2[	183	]=	-0.145282294	;
    RnonCO2[	184	]=	-0.1490379	;
    RnonCO2[	185	]=	-0.154219559	;
    RnonCO2[	186	]=	-0.178760049	;
    RnonCO2[	187	]=	-0.203310188	;
    RnonCO2[	188	]=	-0.212758852	;
    RnonCO2[	189	]=	-0.221219612	;
    RnonCO2[	190	]=	-0.230341307	;
    RnonCO2[	191	]=	-0.239337924	;
    RnonCO2[	192	]=	-0.247524974	;
    RnonCO2[	193	]=	-0.250847198	;
    RnonCO2[	194	]=	-0.245200212	;
    RnonCO2[	195	]=	-0.237476895	;
    RnonCO2[	196	]=	-0.24791739	;
    RnonCO2[	197	]=	-0.261719365	;
    RnonCO2[	198	]=	-0.263179809	;
    RnonCO2[	199	]=	-0.263298027	;
    RnonCO2[	200	]=	-0.261139159	;
    RnonCO2[	201	]=	-0.257781723	;
    RnonCO2[	202	]=	-0.255724839	;
    RnonCO2[	203	]=	-0.25169763	;
    RnonCO2[	204	]=	-0.241647721	;
    RnonCO2[	205	]=	-0.221452127	;
    RnonCO2[	206	]=	-0.196790013	;
    RnonCO2[	207	]=	-0.182093765	;
    RnonCO2[	208	]=	-0.174745229	;
    RnonCO2[	209	]=	-0.16714548	;
    RnonCO2[	210	]=	-0.159384584	;
    RnonCO2[	211	]=	-0.156373877	;
    RnonCO2[	212	]=	-0.153042015	;
    RnonCO2[	213	]=	-0.139817455	;
    RnonCO2[	214	]=	-0.125210902	;
    RnonCO2[	215	]=	-0.119848303	;
    RnonCO2[	216	]=	-0.119430668	;
    RnonCO2[	217	]=	-0.123211365	;
    RnonCO2[	218	]=	-0.125809708	;
    RnonCO2[	219	]=	-0.120193155	;
    RnonCO2[	220	]=	-0.110911249	;
    RnonCO2[	221	]=	-0.102402229	;
    RnonCO2[	222	]=	-0.096562536	;
    RnonCO2[	223	]=	-0.086930587	;
    RnonCO2[	224	]=	-0.072710932	;
    RnonCO2[	225	]=	-0.05997191	;
    RnonCO2[	226	]=	-0.058006544	;
    RnonCO2[	227	]=	-0.065604043	;
    RnonCO2[	228	]=	-0.067949961	;
    RnonCO2[	229	]=	-0.065994527	;
    RnonCO2[	230	]=	-0.058283284	;
    RnonCO2[	231	]=	-0.04756956	;
    RnonCO2[	232	]=	-0.049208339	;
    RnonCO2[	233	]=	-0.056699684	;
    RnonCO2[	234	]=	-0.059531967	;
    RnonCO2[	235	]=	-0.065638041	;
    RnonCO2[	236	]=	-0.069976074	;
    RnonCO2[	237	]=	-0.065660736	;
    RnonCO2[	238	]=	-0.060876733	;
    RnonCO2[	239	]=	-0.057486962	;
    RnonCO2[	240	]=	-0.052586013	;
    RnonCO2[	241	]=	-0.042941813	;
    RnonCO2[	242	]=	-0.030744718	;
    RnonCO2[	243	]=	-0.017809043	;
    RnonCO2[	244	]=	-0.004130627	;
    RnonCO2[	245	]=	0.011524255	;
    RnonCO2[	246	]=	0.030489302	;
    RnonCO2[	247	]=	0.05014081	;
    RnonCO2[	248	]=	0.067538237	;
    RnonCO2[	249	]=	0.082834842	;
    RnonCO2[	250	]=	0.096507674	;
    RnonCO2[	251	]=	0.10872806	;
    RnonCO2[	252	]=	0.119627461	;
    RnonCO2[	253	]=	0.129321759	;
    RnonCO2[	254	]=	0.138477337	;
    RnonCO2[	255	]=	0.145950472	;
    RnonCO2[	256	]=	0.150358206	;
    RnonCO2[	257	]=	0.152908292	;
    RnonCO2[	258	]=	0.154993814	;
    RnonCO2[	259	]=	0.157280133	;
    RnonCO2[	260	]=	0.159876589	;
    RnonCO2[	261	]=	0.162771652	;
    RnonCO2[	262	]=	0.165935075	;
    RnonCO2[	263	]=	0.169331817	;
    RnonCO2[	264	]=	0.172938007	;
    RnonCO2[	265	]=	0.175565587	;
    RnonCO2[	266	]=	0.176019044	;
    RnonCO2[	267	]=	0.176093924	;
    RnonCO2[	268	]=	0.176987578	;
    RnonCO2[	269	]=	0.178093057	;
    RnonCO2[	270	]=	0.179425591	;
    RnonCO2[	271	]=	0.180988877	;
    RnonCO2[	272	]=	0.182779529	;
    RnonCO2[	273	]=	0.184783219	;
    RnonCO2[	274	]=	0.186973518	;
    RnonCO2[	275	]=	0.18815089	;
    RnonCO2[	276	]=	0.187102261	;
    RnonCO2[	277	]=	0.184735629	;
    RnonCO2[	278	]=	0.181929696	;
    RnonCO2[	279	]=	0.178661362	;
    RnonCO2[	280	]=	0.174994888	;
    RnonCO2[	281	]=	0.170975598	;
    RnonCO2[	282	]=	0.166654116	;
    RnonCO2[	283	]=	0.16212844	;
    RnonCO2[	284	]=	0.157462204	;
    RnonCO2[	285	]=	0.151799657	;
    RnonCO2[	286	]=	0.144259517	;
    RnonCO2[	287	]=	0.13582153	;
    RnonCO2[	288	]=	0.127542922	;
    RnonCO2[	289	]=	0.119424904	;
    RnonCO2[	290	]=	0.111420416	;
    RnonCO2[	291	]=	0.10353929	;
    RnonCO2[	292	]=	0.095784605	;
    RnonCO2[	293	]=	0.088157588	;
    RnonCO2[	294	]=	0.080662353	;
    RnonCO2[	295	]=	0.074117031	;
    RnonCO2[	296	]=	0.069346782	;
    RnonCO2[	297	]=	0.065440076	;
    RnonCO2[	298	]=	0.061757366	;
    RnonCO2[	299	]=	0.058553736	;
    RnonCO2[	300	]=	0.055746195	;
    RnonCO2[	301	]=	0.053274915	;
    RnonCO2[	302	]=	0.051135541	;
    RnonCO2[	303	]=	0.049319228	;
    RnonCO2[	304	]=	0.047816059	;
    RnonCO2[	305	]=	0.047030318	;
    RnonCO2[	306	]=	0.047344319	;
    RnonCO2[	307	]=	0.048299122	;
    RnonCO2[	308	]=	0.04943385	;
    RnonCO2[	309	]=	0.050715671	;
    RnonCO2[	310	]=	0.0521283	;
    RnonCO2[	311	]=	0.053621844	;
    RnonCO2[	312	]=	0.05516537	;
    RnonCO2[	313	]=	0.056779921	;
    RnonCO2[	314	]=	0.058478875	;
    RnonCO2[	315	]=	0.059888399	;
    RnonCO2[	316	]=	0.060629189	;
    RnonCO2[	317	]=	0.061309456	;
    RnonCO2[	318	]=	0.062260672	;
    RnonCO2[	319	]=	0.06320141	;
    RnonCO2[	320	]=	0.064129242	;
    RnonCO2[	321	]=	0.065041548	;
    RnonCO2[	322	]=	0.065901312	;
    RnonCO2[	323	]=	0.066688978	;
    RnonCO2[	324	]=	0.067434448	;
    RnonCO2[	325	]=	0.068235885	;
    RnonCO2[	326	]=	0.069192689	;
    RnonCO2[	327	]=	0.070176058	;
    RnonCO2[	328	]=	0.071111938	;
    RnonCO2[	329	]=	0.072068475	;
    RnonCO2[	330	]=	0.073036663	;
    RnonCO2[	331	]=	0.074013231	;
    RnonCO2[	332	]=	0.074994935	;
    RnonCO2[	333	]=	0.075944888	;
    RnonCO2[	334	]=	0.076842806	;
    RnonCO2[	335	]=	0.076570091	;
    RnonCO2[	336	]=	0.073996001	;
    RnonCO2[	337	]=	0.070865472	;
    RnonCO2[	338	]=	0.068420938	;
    RnonCO2[	339	]=	0.066145907	;
    RnonCO2[	340	]=	0.064015257	;
    RnonCO2[	341	]=	0.062010361	;
    RnonCO2[	342	]=	0.060118713	;
    RnonCO2[	343	]=	0.058329335	;
    RnonCO2[	344	]=	0.056598323	;
    RnonCO2[	345	]=	0.054899378	;
    RnonCO2[	346	]=	0.053256032	;
    RnonCO2[	347	]=	0.051684028	;
    RnonCO2[	348	]=	0.050196156	;
    RnonCO2[	349	]=	0.048807185	;
    RnonCO2[	350	]=	0.047512835	;
    RnonCO2[	351	]=	0.046296455	;
    RnonCO2[	352	]=	0.045146887	;
    RnonCO2[	353	]=	0.044057934	;
    RnonCO2[	354	]=	0.043024796	;
    RnonCO2[	355	]=	0.042008614	;
    RnonCO2[	356	]=	0.040988061	;
    RnonCO2[	357	]=	0.039990186	;
    RnonCO2[	358	]=	0.039034443	;
    RnonCO2[	359	]=	0.038136373	;
    RnonCO2[	360	]=	0.037307082	;
    RnonCO2[	361	]=	0.036533216	;
    RnonCO2[	362	]=	0.035794144	;
    RnonCO2[	363	]=	0.035080656	;
    RnonCO2[	364	]=	0.034388489	;
    RnonCO2[	365	]=	0.033714148	;
    RnonCO2[	366	]=	0.033020341	;
    RnonCO2[	367	]=	0.032287006	;
    RnonCO2[	368	]=	0.031542283	;
    RnonCO2[	369	]=	0.030806257	;
    RnonCO2[	370	]=	0.030094998	;
    RnonCO2[	371	]=	0.029426589	;
    RnonCO2[	372	]=	0.028800011	;
    RnonCO2[	373	]=	0.028201386	;
    RnonCO2[	374	]=	0.027622153	;
    RnonCO2[	375	]=	0.027058686	;
    RnonCO2[	376	]=	0.026507668	;
    RnonCO2[	377	]=	0.025932742	;
    RnonCO2[	378	]=	0.025314145	;
    RnonCO2[	379	]=	0.024680521	;
    RnonCO2[	380	]=	0.02405179	;
    RnonCO2[	381	]=	0.023444463	;
    RnonCO2[	382	]=	0.022876762	;
    RnonCO2[	383	]=	0.022347732	;
    RnonCO2[	384	]=	0.02184406	;
    RnonCO2[	385	]=	0.021356977	;
    RnonCO2[	386	]=	0.020883323	;
    RnonCO2[	387	]=	0.020420138	;
    RnonCO2[	388	]=	0.019930802	;
    RnonCO2[	389	]=	0.01939644	;
    RnonCO2[	390	]=	0.0188452	;
    RnonCO2[	391	]=	0.018296944	;
    RnonCO2[	392	]=	0.017768763	;
    RnonCO2[	393	]=	0.017278489	;
    RnonCO2[	394	]=	0.01682528	;
    RnonCO2[	395	]=	0.016395648	;
    RnonCO2[	396	]=	0.015981599	;
    RnonCO2[	397	]=	0.015579655	;
    RnonCO2[	398	]=	0.01518669	;
    RnonCO2[	399	]=	0.014766694	;
    RnonCO2[	400	]=	0.014300633	;
    RnonCO2[	401	]=	0.013816767	;
    RnonCO2[	402	]=	0.013335132	;
    RnonCO2[	403	]=	0.012872182	;
    RnonCO2[	404	]=	0.012446022	;
    RnonCO2[	405	]=	0.012055953	;
    RnonCO2[	406	]=	0.011688624	;
    RnonCO2[	407	]=	0.011335763	;
    RnonCO2[	408	]=	0.010993987	;
    RnonCO2[	409	]=	0.010660417	;
    RnonCO2[	410	]=	0.010299162	;
    RnonCO2[	411	]=	0.00989118	;
    RnonCO2[	412	]=	0.009464652	;
    RnonCO2[	413	]=	0.009039623	;
    RnonCO2[	414	]=	0.008632569	;
    RnonCO2[	415	]=	0.008261461	;
    RnonCO2[	416	]=	0.007925489	;
    RnonCO2[	417	]=	0.007611611	;
    RnonCO2[	418	]=	0.007311291	;
    RnonCO2[	419	]=	0.007021657	;
    RnonCO2[	420	]=	0.006739261	;
    RnonCO2[	421	]=	0.006428648	;
    RnonCO2[	422	]=	0.00607091	;
    RnonCO2[	423	]=	0.005694103	;
    RnonCO2[	424	]=	0.005318119	;
    RnonCO2[	425	]=	0.004959447	;
    RnonCO2[	426	]=	0.004636049	;
    RnonCO2[	427	]=	0.004347021	;
    RnonCO2[	428	]=	0.004079385	;
    RnonCO2[	429	]=	0.00382504	;
    RnonCO2[	430	]=	0.003580394	;
    RnonCO2[	431	]=	0.003342669	;
    RnonCO2[	432	]=	0.003076327	;
    RnonCO2[	433	]=	0.00276219	;
    RnonCO2[	434	]=	0.002428561	;
    RnonCO2[	435	]=	0.002095293	;
    RnonCO2[	436	]=	0.001778879	;
    RnonCO2[	437	]=	0.001496989	;
    RnonCO2[	438	]=	0.001249057	;
    RnonCO2[	439	]=	0.00102177	;
    RnonCO2[	440	]=	0.000807344	;
    RnonCO2[	441	]=	0.000602153	;
    RnonCO2[	442	]=	0.000403545	;
    RnonCO2[	443	]=	0.000175641	;
    RnonCO2[	444	]=	-0.000100305	;
    RnonCO2[	445	]=	-0.000396193	;
    RnonCO2[	446	]=	-0.00069209	;
    RnonCO2[	447	]=	-0.000971786	;
    RnonCO2[	448	]=	-0.001217351	;
    RnonCO2[	449	]=	-0.001429421	;
    RnonCO2[	450	]=	-0.001621204	;
    RnonCO2[	451	]=	-0.00180072	;
    RnonCO2[	452	]=	-0.001971341	;
    RnonCO2[	453	]=	-0.002136017	;
    RnonCO2[	454	]=	-0.002330385	;
    RnonCO2[	455	]=	-0.00257275	;
    RnonCO2[	456	]=	-0.002835529	;
    RnonCO2[	457	]=	-0.003098864	;
    RnonCO2[	458	]=	-0.003346138	;
    RnonCO2[	459	]=	-0.003559862	;
    RnonCO2[	460	]=	-0.003740454	;
    RnonCO2[	461	]=	-0.003901281	;
    RnonCO2[	462	]=	-0.00405017	;
    RnonCO2[	463	]=	-0.00419066	;
    RnonCO2[	464	]=	-0.004325436	;
    RnonCO2[	465	]=	-0.00449015	;
    RnonCO2[	466	]=	-0.00470337	;
    RnonCO2[	467	]=	-0.004937067	;
    RnonCO2[	468	]=	-0.005171618	;
    RnonCO2[	469	]=	-0.005390849	;
    RnonCO2[	470	]=	-0.005576447	;
    RnonCO2[	471	]=	-0.005729591	;
    RnonCO2[	472	]=	-0.005863231	;
    RnonCO2[	473	]=	-0.005985416	;
    RnonCO2[	474	]=	-0.006099425	;
    RnonCO2[	475	]=	-0.006207957	;
    RnonCO2[	476	]=	-0.006346669	;
    RnonCO2[	477	]=	-0.006534243	;
    RnonCO2[	478	]=	-0.006742512	;
    RnonCO2[	479	]=	-0.006951957	;
    RnonCO2[	480	]=	-0.007146001	;
    RnonCO2[	481	]=	-0.007307361	;
    RnonCO2[	482	]=	-0.007436262	;
    RnonCO2[	483	]=	-0.007545934	;
    RnonCO2[	484	]=	-0.007644501	;
    RnonCO2[	485	]=	-0.007735122	;
    RnonCO2[	486	]=	-0.007820801	;
    RnonCO2[	487	]=	-0.007936748	;
    RnonCO2[	488	]=	-0.008101617	;
    RnonCO2[	489	]=	-0.008287551	;
    RnonCO2[	490	]=	-0.008474862	;
    RnonCO2[	491	]=	-0.008647147	;
    RnonCO2[	492	]=	-0.00878687	;
    RnonCO2[	493	]=	-0.008894579	;
    RnonCO2[	494	]=	-0.008983349	;
    RnonCO2[	495	]=	-0.009061228	;
    RnonCO2[	496	]=	-0.009131254	;
    RnonCO2[	497	]=	-0.009196735	;
    RnonCO2[	498	]=	-0.009292563	;
    RnonCO2[	499	]=	-0.009437653	;
    RnonCO2[	500	]=	-0.009603896	;
    RnonCO2[	501	]=	-0.009771736	;
    RnonCO2[	502	]=	-0.009924803	;
    RnonCO2[	503	]=	-0.010045599	;
    RnonCO2[	504	]=	-0.010134561	;
    RnonCO2[	505	]=	-0.010204736	;
    RnonCO2[	506	]=	-0.010264242	;
    RnonCO2[	507	]=	-0.010316427	;
    RnonCO2[	508	]=	-0.010364035	;
    RnonCO2[	509	]=	-0.010442352	;
    RnonCO2[	510	]=	-0.010569824	;
    RnonCO2[	511	]=	-0.010718688	;
    RnonCO2[	512	]=	-0.010869385	;
    RnonCO2[	513	]=	-0.011005583	;
    RnonCO2[	514	]=	-0.011109531	;
    RnonCO2[	515	]=	-0.011182135	;
    RnonCO2[	516	]=	-0.011236114	;
    RnonCO2[	517	]=	-0.01127965	;
    RnonCO2[	518	]=	-0.011315818	;
    RnonCO2[	519	]=	-0.011347524	;
    RnonCO2[	520	]=	-0.011410299	;
    RnonCO2[	521	]=	-0.011522401	;
    RnonCO2[	522	]=	-0.011655994	;
    RnonCO2[	523	]=	-0.01179167	;
    RnonCO2[	524	]=	-0.011912844	;
    RnonCO2[	525	]=	-0.012002101	;
    RnonCO2[	526	]=	-0.01206008	;
    RnonCO2[	527	]=	-0.012099752	;
    RnonCO2[	528	]=	-0.012129063	;
    RnonCO2[	529	]=	-0.012151257	;
    RnonCO2[	530	]=	-0.012169254	;
    RnonCO2[	531	]=	-0.01221823	;
    RnonCO2[	532	]=	-0.012316698	;
    RnonCO2[	533	]=	-0.012436908	;
    RnonCO2[	534	]=	-0.012559166	;
    RnonCO2[	535	]=	-0.01266708	;
    RnonCO2[	536	]=	-0.012743282	;
    RnonCO2[	537	]=	-0.012788581	;
    RnonCO2[	538	]=	-0.012815601	;
    RnonCO2[	539	]=	-0.012832346	;
    RnonCO2[	540	]=	-0.012842079	;
    RnonCO2[	541	]=	-0.012847885	;
    RnonCO2[	542	]=	-0.01288472	;
    RnonCO2[	543	]=	-0.012971203	;
    RnonCO2[	544	]=	-0.013079385	;
    RnonCO2[	545	]=	-0.013189886	;
    RnonCO2[	546	]=	-0.013286215	;
    RnonCO2[	547	]=	-0.013351053	;
    RnonCO2[	548	]=	-0.013384779	;
    RnonCO2[	549	]=	-0.013400708	;
    RnonCO2[	550	]=	-0.013406301	;
    RnonCO2[	551	]=	-0.013405137	;
    RnonCO2[	552	]=	-0.013400171	;
    RnonCO2[	553	]=	-0.013426426	;
    RnonCO2[	554	]=	-0.013502179	;
    RnonCO2[	555	]=	-0.013599889	;
    RnonCO2[	556	]=	-0.013699748	;
    RnonCO2[	557	]=	-0.013785919	;
    RnonCO2[	558	]=	-0.013840533	;
    RnonCO2[	559	]=	-0.013864285	;
    RnonCO2[	560	]=	-0.01387013	;
    RnonCO2[	561	]=	-0.013866028	;
    RnonCO2[	562	]=	-0.01385543	;
    RnonCO2[	563	]=	-0.013840702	;
    RnonCO2[	564	]=	-0.013857385	;
    RnonCO2[	565	]=	-0.013923857	;
    RnonCO2[	566	]=	-0.014012244	;
    RnonCO2[	567	]=	-0.014102918	;
    RnonCO2[	568	]=	-0.014179798	;
    RnonCO2[	569	]=	-0.014225518	;
    RnonCO2[	570	]=	-0.014240334	;
    RnonCO2[	571	]=	-0.014237589	;
    RnonCO2[	572	]=	-0.014224837	;
    RnonCO2[	573	]=	-0.014205399	;
    RnonCO2[	574	]=	-0.014182411	;
    RnonCO2[	575	]=	-0.014190714	;
    RnonCO2[	576	]=	-0.014248788	;
    RnonCO2[	577	]=	-0.014328737	;
    RnonCO2[	578	]=	-0.014411267	;
    RnonCO2[	579	]=	-0.014480209	;
    RnonCO2[	580	]=	-0.014517947	;
    RnonCO2[	581	]=	-0.014524897	;
    RnonCO2[	582	]=	-0.014514334	;
    RnonCO2[	583	]=	-0.014493857	;
    RnonCO2[	584	]=	-0.014467111	;
    RnonCO2[	585	]=	-0.014436334	;
    RnonCO2[	586	]=	-0.014437332	;
    RnonCO2[	587	]=	-0.014487922	;
    RnonCO2[	588	]=	-0.014560651	;
    RnonCO2[	589	]=	-0.014636114	;
    RnonCO2[	590	]=	-0.014697742	;
    RnonCO2[	591	]=	-0.014728282	;
    RnonCO2[	592	]=	-0.014728466	;
    RnonCO2[	593	]=	-0.014711034	;
    RnonCO2[	594	]=	-0.014683784	;
    RnonCO2[	595	]=	-0.014650073	;
    RnonCO2[	596	]=	-0.01461292	;
    RnonCO2[	597	]=	-0.014607102	;
    RnonCO2[	598	]=	-0.014651305	;
    RnonCO2[	599	]=	-0.014717451	;
    RnonCO2[	600	]=	-0.014786334	;
    RnonCO2[	601	]=	-0.014841909	;
    RnonCO2[	602	]=	-0.014866216	;
    RnonCO2[	603	]=	-0.01485999	;
    RnonCO2[	604	]=	-0.014836665	;
    RnonCO2[	605	]=	-0.014803311	;
    RnonCO2[	606	]=	-0.01476361	;
    RnonCO2[	607	]=	-0.014720443	;
    RnonCO2[	608	]=	-0.014708782	;
    RnonCO2[	609	]=	-0.014747257	;
    RnonCO2[	610	]=	-0.014807788	;
    RnonCO2[	611	]=	-0.014870907	;
    RnonCO2[	612	]=	-0.014920638	;
    RnonCO2[	613	]=	-0.014939388	;
    RnonCO2[	614	]=	-0.014927899	;
    RnonCO2[	615	]=	-0.014898904	;
    RnonCO2[	616	]=	-0.014860286	;
    RnonCO2[	617	]=	-0.014815439	;
    RnonCO2[	618	]=	-0.014766948	;
    RnonCO2[	619	]=	-0.014750129	;
    RnonCO2[	620	]=	-0.014783246	;
    RnonCO2[	621	]=	-0.014838687	;
    RnonCO2[	622	]=	-0.01489688	;
    RnonCO2[	623	]=	-0.014941608	;
    RnonCO2[	624	]=	-0.01495534	;
    RnonCO2[	625	]=	-0.01493898	;
    RnonCO2[	626	]=	-0.014905173	;
    RnonCO2[	627	]=	-0.014861843	;
    RnonCO2[	628	]=	-0.014812246	;
    RnonCO2[	629	]=	-0.014759136	;
    RnonCO2[	630	]=	-0.014737706	;
    RnonCO2[	631	]=	-0.014766316	;
    RnonCO2[	632	]=	-0.014817052	;
    RnonCO2[	633	]=	-0.014870707	;
    RnonCO2[	634	]=	-0.014910982	;
    RnonCO2[	635	]=	-0.014920408	;
    RnonCO2[	636	]=	-0.014899737	;
    RnonCO2[	637	]=	-0.01486168	;
    RnonCO2[	638	]=	-0.014814047	;
    RnonCO2[	639	]=	-0.014760264	;
    RnonCO2[	640	]=	-0.014703101	;
    RnonCO2[	641	]=	-0.014677622	;
    RnonCO2[	642	]=	-0.014701968	;
    RnonCO2[	643	]=	-0.014748709	;
    RnonCO2[	644	]=	-0.014798386	;
    RnonCO2[	645	]=	-0.014834769	;
    RnonCO2[	646	]=	-0.014840302	;
    RnonCO2[	647	]=	-0.014815738	;
    RnonCO2[	648	]=	-0.014773851	;
    RnonCO2[	649	]=	-0.014722489	;
    RnonCO2[	650	]=	-0.014664941	;
    RnonCO2[	651	]=	-0.014604148	;
    RnonCO2[	652	]=	-0.014575037	;
    RnonCO2[	653	]=	-0.014595844	;
    RnonCO2[	654	]=	-0.014639003	;
    RnonCO2[	655	]=	-0.014684956	;
    RnonCO2[	656	]=	-0.014717709	;
    RnonCO2[	657	]=	-0.01471993	;
    RnonCO2[	658	]=	-0.014691899	;
    RnonCO2[	659	]=	-0.01464661	;
    RnonCO2[	660	]=	-0.014591952	;
    RnonCO2[	661	]=	-0.014531227	;
    RnonCO2[	662	]=	-0.014466919	;
    RnonCO2[	663	]=	-0.014434447	;
    RnonCO2[	664	]=	-0.014452139	;
    RnonCO2[	665	]=	-0.01449198	;
    RnonCO2[	666	]=	-0.01453495	;
    RnonCO2[	667	]=	-0.014564499	;
    RnonCO2[	668	]=	-0.014563524	;
    RnonCO2[	669	]=	-0.014532622	;
    RnonCO2[	670	]=	-0.014484372	;
    RnonCO2[	671	]=	-0.014426698	;
    RnonCO2[	672	]=	-0.014363081	;
    RnonCO2[	673	]=	-0.014295855	;
    RnonCO2[	674	]=	-0.014260617	;
    RnonCO2[	675	]=	-0.014275311	;
    RnonCO2[	676	]=	-0.014312424	;
    RnonCO2[	677	]=	-0.014352369	;
    RnonCO2[	678	]=	-0.014379311	;
    RnonCO2[	679	]=	-0.014375581	;
    RnonCO2[	680	]=	-0.014341777	;
    RnonCO2[	681	]=	-0.014290853	;
    RnonCO2[	682	]=	-0.014230613	;
    RnonCO2[	683	]=	-0.014164232	;
    RnonCO2[	684	]=	-0.014094539	;
    RnonCO2[	685	]=	-0.014056662	;
    RnonCO2[	686	]=	-0.014068798	;
    RnonCO2[	687	]=	-0.014103305	;
    RnonCO2[	688	]=	-0.014140826	;
    RnonCO2[	689	]=	-0.014165127	;
    RnonCO2[	690	]=	-0.014159093	;
    RnonCO2[	691	]=	-0.014122839	;
    RnonCO2[	692	]=	-0.014069537	;
    RnonCO2[	693	]=	-0.014007026	;
    RnonCO2[	694	]=	-0.013938337	;
    RnonCO2[	695	]=	-0.013866314	;
    RnonCO2[	696	]=	-0.013826092	;
    RnonCO2[	697	]=	-0.013835798	;
    RnonCO2[	698	]=	-0.013868149	;
    RnonCO2[	699	]=	-0.013903374	;
    RnonCO2[	700	]=	-0.013925486	;
    RnonCO2[	701	]=	-0.013917285	;
    RnonCO2[	702	]=	-0.013878879	;
    RnonCO2[	703	]=	-0.013823501	;
    RnonCO2[	704	]=	-0.013758861	;
    RnonCO2[	705	]=	-0.013688005	;
    RnonCO2[	706	]=	-0.013613954	;
    RnonCO2[	707	]=	-0.013571686	;
    RnonCO2[	708	]=	-0.01357958	;
    RnonCO2[	709	]=	-0.013609746	;
    RnonCO2[	710	]=	-0.013643134	;
    RnonCO2[	711	]=	-0.013663356	;
    RnonCO2[	712	]=	-0.013652965	;
    RnonCO2[	713	]=	-0.013612713	;
    RnonCO2[	714	]=	-0.013555565	;
    RnonCO2[	715	]=	-0.013488941	;
    RnonCO2[	716	]=	-0.013416226	;
    RnonCO2[	717	]=	-0.013340293	;
    RnonCO2[	718	]=	-0.013296285	;
    RnonCO2[	719	]=	-0.013302182	;
    RnonCO2[	720	]=	-0.013330624	;
    RnonCO2[	721	]=	-0.01336199	;
    RnonCO2[	722	]=	-0.013380463	;
    RnonCO2[	723	]=	-0.013368513	;
    RnonCO2[	724	]=	-0.013326726	;
    RnonCO2[	725	]=	-0.013267632	;
    RnonCO2[	726	]=	-0.013199498	;
    RnonCO2[	727	]=	-0.013125074	;
    RnonCO2[	728	]=	-0.013047572	;
    RnonCO2[	729	]=	-0.013001972	;
    RnonCO2[	730	]=	-0.013006178	;
    RnonCO2[	731	]=	-0.01303304	;
    RnonCO2[	732	]=	-0.013063014	;
    RnonCO2[	733	]=	-0.01307972	;
    RnonCO2[	734	]=	-0.013066198	;
    RnonCO2[	735	]=	-0.013050088	;
    
    
    for(int i = 736; i<tmax; i++)
    {
        RnonCO2[i] = RnonCO2[i-1];
    }
    
}



void setRnonCO2_RCP45(void)
{
    RnonCO2[	0	]=	-0.019988515	;
    RnonCO2[	1	]=	-0.005145555	;
    RnonCO2[	2	]=	0.009090068	;
    RnonCO2[	3	]=	0.007768629	;
    RnonCO2[	4	]=	0.006256789	;
    RnonCO2[	5	]=	0.004769658	;
    RnonCO2[	6	]=	0.002906336	;
    RnonCO2[	7	]=	0.001080178	;
    RnonCO2[	8	]=	-0.000291468	;
    RnonCO2[	9	]=	-0.001612412	;
    RnonCO2[	10	]=	-0.00288359	;
    RnonCO2[	11	]=	-0.00410611	;
    RnonCO2[	12	]=	-0.005282233	;
    RnonCO2[	13	]=	-0.006413549	;
    RnonCO2[	14	]=	-0.007501746	;
    RnonCO2[	15	]=	-0.008551056	;
    RnonCO2[	16	]=	-0.010019335	;
    RnonCO2[	17	]=	-0.011459991	;
    RnonCO2[	18	]=	-0.012417579	;
    RnonCO2[	19	]=	-0.013333977	;
    RnonCO2[	20	]=	-0.014201345	;
    RnonCO2[	21	]=	-0.015012487	;
    RnonCO2[	22	]=	-0.015760238	;
    RnonCO2[	23	]=	-0.016437067	;
    RnonCO2[	24	]=	-0.017035543	;
    RnonCO2[	25	]=	-0.017549544	;
    RnonCO2[	26	]=	-0.018434793	;
    RnonCO2[	27	]=	-0.019238993	;
    RnonCO2[	28	]=	-0.019510794	;
    RnonCO2[	29	]=	-0.019709367	;
    RnonCO2[	30	]=	-0.019838439	;
    RnonCO2[	31	]=	-0.019901316	;
    RnonCO2[	32	]=	-0.020355353	;
    RnonCO2[	33	]=	-0.020749651	;
    RnonCO2[	34	]=	-0.020634193	;
    RnonCO2[	35	]=	-0.02091346	;
    RnonCO2[	36	]=	-0.021125633	;
    RnonCO2[	37	]=	-0.02171132	;
    RnonCO2[	38	]=	-0.021774359	;
    RnonCO2[	39	]=	-0.020873209	;
    RnonCO2[	40	]=	-0.02036553	;
    RnonCO2[	41	]=	-0.020256367	;
    RnonCO2[	42	]=	-0.020105185	;
    RnonCO2[	43	]=	-0.01947287	;
    RnonCO2[	44	]=	-0.018814746	;
    RnonCO2[	45	]=	-0.018134946	;
    RnonCO2[	46	]=	-0.017872094	;
    RnonCO2[	47	]=	-0.017576381	;
    RnonCO2[	48	]=	-0.016804485	;
    RnonCO2[	49	]=	-0.016001407	;
    RnonCO2[	50	]=	-0.015606441	;
    RnonCO2[	51	]=	-0.015619264	;
    RnonCO2[	52	]=	-0.015600061	;
    RnonCO2[	53	]=	-0.015116575	;
    RnonCO2[	54	]=	-0.014172313	;
    RnonCO2[	55	]=	-0.013205379	;
    RnonCO2[	56	]=	-0.012216093	;
    RnonCO2[	57	]=	-0.011635129	;
    RnonCO2[	58	]=	-0.011459514	;
    RnonCO2[	59	]=	-0.010829842	;
    RnonCO2[	60	]=	-0.010175791	;
    RnonCO2[	61	]=	-0.009499636	;
    RnonCO2[	62	]=	-0.00879934	;
    RnonCO2[	63	]=	-0.008077943	;
    RnonCO2[	64	]=	-0.006912545	;
    RnonCO2[	65	]=	-0.008242223	;
    RnonCO2[	66	]=	-0.009135821	;
    RnonCO2[	67	]=	-0.007501052	;
    RnonCO2[	68	]=	-0.006687418	;
    RnonCO2[	69	]=	-0.005858015	;
    RnonCO2[	70	]=	-0.005005979	;
    RnonCO2[	71	]=	-0.005767925	;
    RnonCO2[	72	]=	-0.006086639	;
    RnonCO2[	73	]=	-0.005137938	;
    RnonCO2[	74	]=	-0.004558881	;
    RnonCO2[	75	]=	-0.004336125	;
    RnonCO2[	76	]=	-0.004045198	;
    RnonCO2[	77	]=	-0.003665109	;
    RnonCO2[	78	]=	-0.003220213	;
    RnonCO2[	79	]=	-0.002760518	;
    RnonCO2[	80	]=	-0.003527376	;
    RnonCO2[	81	]=	-0.003576651	;
    RnonCO2[	82	]=	-0.003275283	;
    RnonCO2[	83	]=	-0.003386771	;
    RnonCO2[	84	]=	-0.003510917	;
    RnonCO2[	85	]=	-0.004646356	;
    RnonCO2[	86	]=	-0.003691576	;
    RnonCO2[	87	]=	-0.009505158	;
    RnonCO2[	88	]=	-0.016364432	;
    RnonCO2[	89	]=	-0.017996784	;
    RnonCO2[	90	]=	-0.019670058	;
    RnonCO2[	91	]=	-0.020430126	;
    RnonCO2[	92	]=	-0.021643321	;
    RnonCO2[	93	]=	-0.02142206	;
    RnonCO2[	94	]=	-0.022552345	;
    RnonCO2[	95	]=	-0.031588211	;
    RnonCO2[	96	]=	-0.046252985	;
    RnonCO2[	97	]=	-0.042365448	;
    RnonCO2[	98	]=	-0.033665717	;
    RnonCO2[	99	]=	-0.034555638	;
    RnonCO2[	100	]=	-0.037298509	;
    RnonCO2[	101	]=	-0.046083195	;
    RnonCO2[	102	]=	-0.05558168	;
    RnonCO2[	103	]=	-0.057464266	;
    RnonCO2[	104	]=	-0.053606449	;
    RnonCO2[	105	]=	-0.05168834	;
    RnonCO2[	106	]=	-0.061304945	;
    RnonCO2[	107	]=	-0.086608149	;
    RnonCO2[	108	]=	-0.108195137	;
    RnonCO2[	109	]=	-0.112259625	;
    RnonCO2[	110	]=	-0.111643119	;
    RnonCO2[	111	]=	-0.108991576	;
    RnonCO2[	112	]=	-0.10203489	;
    RnonCO2[	113	]=	-0.098533227	;
    RnonCO2[	114	]=	-0.101949678	;
    RnonCO2[	115	]=	-0.114562053	;
    RnonCO2[	116	]=	-0.123024156	;
    RnonCO2[	117	]=	-0.119809652	;
    RnonCO2[	118	]=	-0.118941351	;
    RnonCO2[	119	]=	-0.120181779	;
    RnonCO2[	120	]=	-0.117316831	;
    RnonCO2[	121	]=	-0.115023243	;
    RnonCO2[	122	]=	-0.116983521	;
    RnonCO2[	123	]=	-0.125997633	;
    RnonCO2[	124	]=	-0.129568227	;
    RnonCO2[	125	]=	-0.131248987	;
    RnonCO2[	126	]=	-0.137316696	;
    RnonCO2[	127	]=	-0.141273792	;
    RnonCO2[	128	]=	-0.144231375	;
    RnonCO2[	129	]=	-0.147973341	;
    RnonCO2[	130	]=	-0.153688224	;
    RnonCO2[	131	]=	-0.15652816	;
    RnonCO2[	132	]=	-0.157109013	;
    RnonCO2[	133	]=	-0.158858513	;
    RnonCO2[	134	]=	-0.162186704	;
    RnonCO2[	135	]=	-0.165318228	;
    RnonCO2[	136	]=	-0.171733193	;
    RnonCO2[	137	]=	-0.178096644	;
    RnonCO2[	138	]=	-0.183870407	;
    RnonCO2[	139	]=	-0.189730028	;
    RnonCO2[	140	]=	-0.194221589	;
    RnonCO2[	141	]=	-0.201576481	;
    RnonCO2[	142	]=	-0.208859035	;
    RnonCO2[	143	]=	-0.207375441	;
    RnonCO2[	144	]=	-0.202571711	;
    RnonCO2[	145	]=	-0.204126848	;
    RnonCO2[	146	]=	-0.201876709	;
    RnonCO2[	147	]=	-0.195147841	;
    RnonCO2[	148	]=	-0.191449584	;
    RnonCO2[	149	]=	-0.183770215	;
    RnonCO2[	150	]=	-0.173476305	;
    RnonCO2[	151	]=	-0.170909799	;
    RnonCO2[	152	]=	-0.171515561	;
    RnonCO2[	153	]=	-0.169079801	;
    RnonCO2[	154	]=	-0.159414516	;
    RnonCO2[	155	]=	-0.155013584	;
    RnonCO2[	156	]=	-0.151089869	;
    RnonCO2[	157	]=	-0.144432651	;
    RnonCO2[	158	]=	-0.152359798	;
    RnonCO2[	159	]=	-0.158148636	;
    RnonCO2[	160	]=	-0.155277608	;
    RnonCO2[	161	]=	-0.152617947	;
    RnonCO2[	162	]=	-0.154120923	;
    RnonCO2[	163	]=	-0.156019482	;
    RnonCO2[	164	]=	-0.155886394	;
    RnonCO2[	165	]=	-0.152021034	;
    RnonCO2[	166	]=	-0.141834688	;
    RnonCO2[	167	]=	-0.13108946	;
    RnonCO2[	168	]=	-0.125408882	;
    RnonCO2[	169	]=	-0.125515632	;
    RnonCO2[	170	]=	-0.125194614	;
    RnonCO2[	171	]=	-0.126120729	;
    RnonCO2[	172	]=	-0.12532345	;
    RnonCO2[	173	]=	-0.115303554	;
    RnonCO2[	174	]=	-0.10787249	;
    RnonCO2[	175	]=	-0.109257271	;
    RnonCO2[	176	]=	-0.113724728	;
    RnonCO2[	177	]=	-0.117421621	;
    RnonCO2[	178	]=	-0.11813163	;
    RnonCO2[	179	]=	-0.119629474	;
    RnonCO2[	180	]=	-0.119789192	;
    RnonCO2[	181	]=	-0.124270593	;
    RnonCO2[	182	]=	-0.135220142	;
    RnonCO2[	183	]=	-0.145282294	;
    RnonCO2[	184	]=	-0.1490379	;
    RnonCO2[	185	]=	-0.154219559	;
    RnonCO2[	186	]=	-0.178760049	;
    RnonCO2[	187	]=	-0.203310188	;
    RnonCO2[	188	]=	-0.212758852	;
    RnonCO2[	189	]=	-0.221219612	;
    RnonCO2[	190	]=	-0.230341307	;
    RnonCO2[	191	]=	-0.239337924	;
    RnonCO2[	192	]=	-0.247524974	;
    RnonCO2[	193	]=	-0.250847198	;
    RnonCO2[	194	]=	-0.245200212	;
    RnonCO2[	195	]=	-0.237476895	;
    RnonCO2[	196	]=	-0.24791739	;
    RnonCO2[	197	]=	-0.261719365	;
    RnonCO2[	198	]=	-0.263179809	;
    RnonCO2[	199	]=	-0.263298027	;
    RnonCO2[	200	]=	-0.261139159	;
    RnonCO2[	201	]=	-0.257781723	;
    RnonCO2[	202	]=	-0.255724839	;
    RnonCO2[	203	]=	-0.25169763	;
    RnonCO2[	204	]=	-0.241647721	;
    RnonCO2[	205	]=	-0.221452127	;
    RnonCO2[	206	]=	-0.196790013	;
    RnonCO2[	207	]=	-0.182093765	;
    RnonCO2[	208	]=	-0.174745229	;
    RnonCO2[	209	]=	-0.16714548	;
    RnonCO2[	210	]=	-0.159384584	;
    RnonCO2[	211	]=	-0.156373877	;
    RnonCO2[	212	]=	-0.153042015	;
    RnonCO2[	213	]=	-0.139817455	;
    RnonCO2[	214	]=	-0.125210902	;
    RnonCO2[	215	]=	-0.119848303	;
    RnonCO2[	216	]=	-0.119430668	;
    RnonCO2[	217	]=	-0.123211365	;
    RnonCO2[	218	]=	-0.125809708	;
    RnonCO2[	219	]=	-0.120193155	;
    RnonCO2[	220	]=	-0.110911249	;
    RnonCO2[	221	]=	-0.102402229	;
    RnonCO2[	222	]=	-0.096562536	;
    RnonCO2[	223	]=	-0.086930587	;
    RnonCO2[	224	]=	-0.072710932	;
    RnonCO2[	225	]=	-0.05997191	;
    RnonCO2[	226	]=	-0.058006544	;
    RnonCO2[	227	]=	-0.065604043	;
    RnonCO2[	228	]=	-0.067949961	;
    RnonCO2[	229	]=	-0.065994527	;
    RnonCO2[	230	]=	-0.058283284	;
    RnonCO2[	231	]=	-0.04756956	;
    RnonCO2[	232	]=	-0.049208339	;
    RnonCO2[	233	]=	-0.056699684	;
    RnonCO2[	234	]=	-0.059531967	;
    RnonCO2[	235	]=	-0.065638041	;
    RnonCO2[	236	]=	-0.069976074	;
    RnonCO2[	237	]=	-0.065660736	;
    RnonCO2[	238	]=	-0.060876733	;
    RnonCO2[	239	]=	-0.057486962	;
    RnonCO2[	240	]=	-0.052009693	;
    RnonCO2[	241	]=	-0.040632843	;
    RnonCO2[	242	]=	-0.026916588	;
    RnonCO2[	243	]=	-0.013452099	;
    RnonCO2[	244	]=	0.000389465	;
    RnonCO2[	245	]=	0.014221018	;
    RnonCO2[	246	]=	0.027735641	;
    RnonCO2[	247	]=	0.041750056	;
    RnonCO2[	248	]=	0.056057191	;
    RnonCO2[	249	]=	0.07006948	;
    RnonCO2[	250	]=	0.084104898	;
    RnonCO2[	251	]=	0.098192736	;
    RnonCO2[	252	]=	0.112337742	;
    RnonCO2[	253	]=	0.12653946	;
    RnonCO2[	254	]=	0.141365316	;
    RnonCO2[	255	]=	0.155687536	;
    RnonCO2[	256	]=	0.168102785	;
    RnonCO2[	257	]=	0.179884397	;
    RnonCO2[	258	]=	0.191686884	;
    RnonCO2[	259	]=	0.203223672	;
    RnonCO2[	260	]=	0.214540005	;
    RnonCO2[	261	]=	0.22566407	;
    RnonCO2[	262	]=	0.236614088	;
    RnonCO2[	263	]=	0.247389908	;
    RnonCO2[	264	]=	0.257994987	;
    RnonCO2[	265	]=	0.268753245	;
    RnonCO2[	266	]=	0.279985254	;
    RnonCO2[	267	]=	0.290918066	;
    RnonCO2[	268	]=	0.301097825	;
    RnonCO2[	269	]=	0.310839024	;
    RnonCO2[	270	]=	0.32015312	;
    RnonCO2[	271	]=	0.329049844	;
    RnonCO2[	272	]=	0.337537645	;
    RnonCO2[	273	]=	0.345614846	;
    RnonCO2[	274	]=	0.353268441	;
    RnonCO2[	275	]=	0.360818709	;
    RnonCO2[	276	]=	0.368598018	;
    RnonCO2[	277	]=	0.375950812	;
    RnonCO2[	278	]=	0.382536458	;
    RnonCO2[	279	]=	0.388726864	;
    RnonCO2[	280	]=	0.394624748	;
    RnonCO2[	281	]=	0.400312866	;
    RnonCO2[	282	]=	0.405855738	;
    RnonCO2[	283	]=	0.41130301	;
    RnonCO2[	284	]=	0.4166845	;
    RnonCO2[	285	]=	0.421077518	;
    RnonCO2[	286	]=	0.423561654	;
    RnonCO2[	287	]=	0.425023251	;
    RnonCO2[	288	]=	0.426380104	;
    RnonCO2[	289	]=	0.42764123	;
    RnonCO2[	290	]=	0.428800693	;
    RnonCO2[	291	]=	0.42991107	;
    RnonCO2[	292	]=	0.43101409	;
    RnonCO2[	293	]=	0.43214539	;
    RnonCO2[	294	]=	0.433340809	;
    RnonCO2[	295	]=	0.434382931	;
    RnonCO2[	296	]=	0.435036038	;
    RnonCO2[	297	]=	0.43550197	;
    RnonCO2[	298	]=	0.435985817	;
    RnonCO2[	299	]=	0.43648989	;
    RnonCO2[	300	]=	0.436977344	;
    RnonCO2[	301	]=	0.437429856	;
    RnonCO2[	302	]=	0.437887378	;
    RnonCO2[	303	]=	0.43838134	;
    RnonCO2[	304	]=	0.438938956	;
    RnonCO2[	305	]=	0.439353059	;
    RnonCO2[	306	]=	0.439393229	;
    RnonCO2[	307	]=	0.439255715	;
    RnonCO2[	308	]=	0.439141872	;
    RnonCO2[	309	]=	0.439052028	;
    RnonCO2[	310	]=	0.438990524	;
    RnonCO2[	311	]=	0.43892044	;
    RnonCO2[	312	]=	0.438822803	;
    RnonCO2[	313	]=	0.438746251	;
    RnonCO2[	314	]=	0.438789983	;
    RnonCO2[	315	]=	0.437769672	;
    RnonCO2[	316	]=	0.434413586	;
    RnonCO2[	317	]=	0.430671228	;
    RnonCO2[	318	]=	0.427880723	;
    RnonCO2[	319	]=	0.425391182	;
    RnonCO2[	320	]=	0.423169243	;
    RnonCO2[	321	]=	0.421185991	;
    RnonCO2[	322	]=	0.419376105	;
    RnonCO2[	323	]=	0.417695746	;
    RnonCO2[	324	]=	0.416158862	;
    RnonCO2[	325	]=	0.414767954	;
    RnonCO2[	326	]=	0.41352279	;
    RnonCO2[	327	]=	0.412434383	;
    RnonCO2[	328	]=	0.411494314	;
    RnonCO2[	329	]=	0.410676404	;
    RnonCO2[	330	]=	0.409961491	;
    RnonCO2[	331	]=	0.409337126	;
    RnonCO2[	332	]=	0.408792275	;
    RnonCO2[	333	]=	0.40827871	;
    RnonCO2[	334	]=	0.407767906	;
    RnonCO2[	335	]=	0.406977545	;
    RnonCO2[	336	]=	0.405618095	;
    RnonCO2[	337	]=	0.40423418	;
    RnonCO2[	338	]=	0.403194974	;
    RnonCO2[	339	]=	0.402313242	;
    RnonCO2[	340	]=	0.401556185	;
    RnonCO2[	341	]=	0.400899141	;
    RnonCO2[	342	]=	0.400325016	;
    RnonCO2[	343	]=	0.399818956	;
    RnonCO2[	344	]=	0.399330522	;
    RnonCO2[	345	]=	0.398828962	;
    RnonCO2[	346	]=	0.39833881	;
    RnonCO2[	347	]=	0.397877845	;
    RnonCO2[	348	]=	0.397459365	;
    RnonCO2[	349	]=	0.397099141	;
    RnonCO2[	350	]=	0.39679216	;
    RnonCO2[	351	]=	0.39651924	;
    RnonCO2[	352	]=	0.396267272	;
    RnonCO2[	353	]=	0.396029339	;
    RnonCO2[	354	]=	0.395799758	;
    RnonCO2[	355	]=	0.39553551	;
    RnonCO2[	356	]=	0.395212477	;
    RnonCO2[	357	]=	0.394860407	;
    RnonCO2[	358	]=	0.394501907	;
    RnonCO2[	359	]=	0.394154959	;
    RnonCO2[	360	]=	0.393841161	;
    RnonCO2[	361	]=	0.393558458	;
    RnonCO2[	362	]=	0.393287414	;
    RnonCO2[	363	]=	0.393017745	;
    RnonCO2[	364	]=	0.392744506	;
    RnonCO2[	365	]=	0.392464174	;
    RnonCO2[	366	]=	0.392136086	;
    RnonCO2[	367	]=	0.391737949	;
    RnonCO2[	368	]=	0.391301246	;
    RnonCO2[	369	]=	0.390849523	;
    RnonCO2[	370	]=	0.390401642	;
    RnonCO2[	371	]=	0.389977762	;
    RnonCO2[	372	]=	0.389576885	;
    RnonCO2[	373	]=	0.389183696	;
    RnonCO2[	374	]=	0.388788342	;
    RnonCO2[	375	]=	0.388386838	;
    RnonCO2[	376	]=	0.387975714	;
    RnonCO2[	377	]=	0.387515138	;
    RnonCO2[	378	]=	0.386983291	;
    RnonCO2[	379	]=	0.386411815	;
    RnonCO2[	380	]=	0.385824643	;
    RnonCO2[	381	]=	0.385240624	;
    RnonCO2[	382	]=	0.384680004	;
    RnonCO2[	383	]=	0.384142172	;
    RnonCO2[	384	]=	0.38361196	;
    RnonCO2[	385	]=	0.383347124	;
    RnonCO2[	386	]=	0.383347124	;
    RnonCO2[	387	]=	0.383347124	;
    RnonCO2[	388	]=	0.383347124	;
    RnonCO2[	389	]=	0.383347124	;
    RnonCO2[	390	]=	0.383347124	;
    RnonCO2[	391	]=	0.383347124	;
    RnonCO2[	392	]=	0.383347124	;
    RnonCO2[	393	]=	0.383347124	;
    RnonCO2[	394	]=	0.383347124	;
    RnonCO2[	395	]=	0.383347124	;
    RnonCO2[	396	]=	0.383347124	;
    RnonCO2[	397	]=	0.383347124	;
    RnonCO2[	398	]=	0.383347124	;
    RnonCO2[	399	]=	0.383347124	;
    RnonCO2[	400	]=	0.383347124	;
    RnonCO2[	401	]=	0.383347124	;
    RnonCO2[	402	]=	0.383347124	;
    RnonCO2[	403	]=	0.383347124	;
    RnonCO2[	404	]=	0.383347124	;
    RnonCO2[	405	]=	0.383347124	;
    RnonCO2[	406	]=	0.383347124	;
    RnonCO2[	407	]=	0.383347124	;
    RnonCO2[	408	]=	0.383347124	;
    RnonCO2[	409	]=	0.383347124	;
    RnonCO2[	410	]=	0.383347124	;
    RnonCO2[	411	]=	0.383347124	;
    RnonCO2[	412	]=	0.383347124	;
    RnonCO2[	413	]=	0.383347124	;
    RnonCO2[	414	]=	0.383347124	;
    RnonCO2[	415	]=	0.383347124	;
    RnonCO2[	416	]=	0.383347124	;
    RnonCO2[	417	]=	0.383347124	;
    RnonCO2[	418	]=	0.383347124	;
    RnonCO2[	419	]=	0.383347124	;
    RnonCO2[	420	]=	0.383347124	;
    RnonCO2[	421	]=	0.383347124	;
    RnonCO2[	422	]=	0.383347124	;
    RnonCO2[	423	]=	0.383347124	;
    RnonCO2[	424	]=	0.383347124	;
    RnonCO2[	425	]=	0.383347124	;
    RnonCO2[	426	]=	0.383347124	;
    RnonCO2[	427	]=	0.383347124	;
    RnonCO2[	428	]=	0.383347124	;
    RnonCO2[	429	]=	0.383347124	;
    RnonCO2[	430	]=	0.383347124	;
    RnonCO2[	431	]=	0.383347124	;
    RnonCO2[	432	]=	0.383347124	;
    RnonCO2[	433	]=	0.383347124	;
    RnonCO2[	434	]=	0.383347124	;
    RnonCO2[	435	]=	0.383347124	;
    RnonCO2[	436	]=	0.383347124	;
    RnonCO2[	437	]=	0.383347124	;
    RnonCO2[	438	]=	0.383347124	;
    RnonCO2[	439	]=	0.383347124	;
    RnonCO2[	440	]=	0.383347124	;
    RnonCO2[	441	]=	0.383347124	;
    RnonCO2[	442	]=	0.383347124	;
    RnonCO2[	443	]=	0.383347124	;
    RnonCO2[	444	]=	0.383347124	;
    RnonCO2[	445	]=	0.383347124	;
    RnonCO2[	446	]=	0.383347124	;
    RnonCO2[	447	]=	0.383347124	;
    RnonCO2[	448	]=	0.383347124	;
    RnonCO2[	449	]=	0.383347124	;
    RnonCO2[	450	]=	0.383347124	;
    RnonCO2[	451	]=	0.383347124	;
    RnonCO2[	452	]=	0.383347124	;
    RnonCO2[	453	]=	0.383347124	;
    RnonCO2[	454	]=	0.383347124	;
    RnonCO2[	455	]=	0.383347124	;
    RnonCO2[	456	]=	0.383347124	;
    RnonCO2[	457	]=	0.383347124	;
    RnonCO2[	458	]=	0.383347124	;
    RnonCO2[	459	]=	0.383347124	;
    RnonCO2[	460	]=	0.383347124	;
    RnonCO2[	461	]=	0.383347124	;
    RnonCO2[	462	]=	0.383347124	;
    RnonCO2[	463	]=	0.383347124	;
    RnonCO2[	464	]=	0.383347124	;
    RnonCO2[	465	]=	0.383347124	;
    RnonCO2[	466	]=	0.383347124	;
    RnonCO2[	467	]=	0.383347124	;
    RnonCO2[	468	]=	0.383347124	;
    RnonCO2[	469	]=	0.383347124	;
    RnonCO2[	470	]=	0.383347124	;
    RnonCO2[	471	]=	0.383347124	;
    RnonCO2[	472	]=	0.383347124	;
    RnonCO2[	473	]=	0.383347124	;
    RnonCO2[	474	]=	0.383347124	;
    RnonCO2[	475	]=	0.383347124	;
    RnonCO2[	476	]=	0.383347124	;
    RnonCO2[	477	]=	0.383347124	;
    RnonCO2[	478	]=	0.383347124	;
    RnonCO2[	479	]=	0.383347124	;
    RnonCO2[	480	]=	0.383347124	;
    RnonCO2[	481	]=	0.383347124	;
    RnonCO2[	482	]=	0.383347124	;
    RnonCO2[	483	]=	0.383347124	;
    RnonCO2[	484	]=	0.383347124	;
    RnonCO2[	485	]=	0.383347124	;
    RnonCO2[	486	]=	0.383347124	;
    RnonCO2[	487	]=	0.383347124	;
    RnonCO2[	488	]=	0.383347124	;
    RnonCO2[	489	]=	0.383347124	;
    RnonCO2[	490	]=	0.383347124	;
    RnonCO2[	491	]=	0.383347124	;
    RnonCO2[	492	]=	0.383347124	;
    RnonCO2[	493	]=	0.383347124	;
    RnonCO2[	494	]=	0.383347124	;
    RnonCO2[	495	]=	0.383347124	;
    RnonCO2[	496	]=	0.383347124	;
    RnonCO2[	497	]=	0.383347124	;
    RnonCO2[	498	]=	0.383347124	;
    RnonCO2[	499	]=	0.383347124	;
    RnonCO2[	500	]=	0.383347124	;
    RnonCO2[	501	]=	0.383347124	;
    RnonCO2[	502	]=	0.383347124	;
    RnonCO2[	503	]=	0.383347124	;
    RnonCO2[	504	]=	0.383347124	;
    RnonCO2[	505	]=	0.383347124	;
    RnonCO2[	506	]=	0.383347124	;
    RnonCO2[	507	]=	0.383347124	;
    RnonCO2[	508	]=	0.383347124	;
    RnonCO2[	509	]=	0.383347124	;
    RnonCO2[	510	]=	0.383347124	;
    RnonCO2[	511	]=	0.383347124	;
    RnonCO2[	512	]=	0.383347124	;
    RnonCO2[	513	]=	0.383347124	;
    RnonCO2[	514	]=	0.383347124	;
    RnonCO2[	515	]=	0.383347124	;
    RnonCO2[	516	]=	0.383347124	;
    RnonCO2[	517	]=	0.383347124	;
    RnonCO2[	518	]=	0.383347124	;
    RnonCO2[	519	]=	0.383347124	;
    RnonCO2[	520	]=	0.383347124	;
    RnonCO2[	521	]=	0.383347124	;
    RnonCO2[	522	]=	0.383347124	;
    RnonCO2[	523	]=	0.383347124	;
    RnonCO2[	524	]=	0.383347124	;
    RnonCO2[	525	]=	0.383347124	;
    RnonCO2[	526	]=	0.383347124	;
    RnonCO2[	527	]=	0.383347124	;
    RnonCO2[	528	]=	0.383347124	;
    RnonCO2[	529	]=	0.383347124	;
    RnonCO2[	530	]=	0.383347124	;
    RnonCO2[	531	]=	0.383347124	;
    RnonCO2[	532	]=	0.383347124	;
    RnonCO2[	533	]=	0.383347124	;
    RnonCO2[	534	]=	0.383347124	;
    RnonCO2[	535	]=	0.383347124	;
    RnonCO2[	536	]=	0.383347124	;
    RnonCO2[	537	]=	0.383347124	;
    RnonCO2[	538	]=	0.383347124	;
    RnonCO2[	539	]=	0.383347124	;
    RnonCO2[	540	]=	0.383347124	;
    RnonCO2[	541	]=	0.383347124	;
    RnonCO2[	542	]=	0.383347124	;
    RnonCO2[	543	]=	0.383347124	;
    RnonCO2[	544	]=	0.383347124	;
    RnonCO2[	545	]=	0.383347124	;
    RnonCO2[	546	]=	0.383347124	;
    RnonCO2[	547	]=	0.383347124	;
    RnonCO2[	548	]=	0.383347124	;
    RnonCO2[	549	]=	0.383347124	;
    RnonCO2[	550	]=	0.383347124	;
    RnonCO2[	551	]=	0.383347124	;
    RnonCO2[	552	]=	0.383347124	;
    RnonCO2[	553	]=	0.383347124	;
    RnonCO2[	554	]=	0.383347124	;
    RnonCO2[	555	]=	0.383347124	;
    RnonCO2[	556	]=	0.383347124	;
    RnonCO2[	557	]=	0.383347124	;
    RnonCO2[	558	]=	0.383347124	;
    RnonCO2[	559	]=	0.383347124	;
    RnonCO2[	560	]=	0.383347124	;
    RnonCO2[	561	]=	0.383347124	;
    RnonCO2[	562	]=	0.383347124	;
    RnonCO2[	563	]=	0.383347124	;
    RnonCO2[	564	]=	0.383347124	;
    RnonCO2[	565	]=	0.383347124	;
    RnonCO2[	566	]=	0.383347124	;
    RnonCO2[	567	]=	0.383347124	;
    RnonCO2[	568	]=	0.383347124	;
    RnonCO2[	569	]=	0.383347124	;
    RnonCO2[	570	]=	0.383347124	;
    RnonCO2[	571	]=	0.383347124	;
    RnonCO2[	572	]=	0.383347124	;
    RnonCO2[	573	]=	0.383347124	;
    RnonCO2[	574	]=	0.383347124	;
    RnonCO2[	575	]=	0.383347124	;
    RnonCO2[	576	]=	0.383347124	;
    RnonCO2[	577	]=	0.383347124	;
    RnonCO2[	578	]=	0.383347124	;
    RnonCO2[	579	]=	0.383347124	;
    RnonCO2[	580	]=	0.383347124	;
    RnonCO2[	581	]=	0.383347124	;
    RnonCO2[	582	]=	0.383347124	;
    RnonCO2[	583	]=	0.383347124	;
    RnonCO2[	584	]=	0.383347124	;
    RnonCO2[	585	]=	0.383347124	;
    RnonCO2[	586	]=	0.383347124	;
    RnonCO2[	587	]=	0.383347124	;
    RnonCO2[	588	]=	0.383347124	;
    RnonCO2[	589	]=	0.383347124	;
    RnonCO2[	590	]=	0.383347124	;
    RnonCO2[	591	]=	0.383347124	;
    RnonCO2[	592	]=	0.383347124	;
    RnonCO2[	593	]=	0.383347124	;
    RnonCO2[	594	]=	0.383347124	;
    RnonCO2[	595	]=	0.383347124	;
    RnonCO2[	596	]=	0.383347124	;
    RnonCO2[	597	]=	0.383347124	;
    RnonCO2[	598	]=	0.383347124	;
    RnonCO2[	599	]=	0.383347124	;
    RnonCO2[	600	]=	0.383347124	;
    RnonCO2[	601	]=	0.383347124	;
    RnonCO2[	602	]=	0.383347124	;
    RnonCO2[	603	]=	0.383347124	;
    RnonCO2[	604	]=	0.383347124	;
    RnonCO2[	605	]=	0.383347124	;
    RnonCO2[	606	]=	0.383347124	;
    RnonCO2[	607	]=	0.383347124	;
    RnonCO2[	608	]=	0.383347124	;
    RnonCO2[	609	]=	0.383347124	;
    RnonCO2[	610	]=	0.383347124	;
    RnonCO2[	611	]=	0.383347124	;
    RnonCO2[	612	]=	0.383347124	;
    RnonCO2[	613	]=	0.383347124	;
    RnonCO2[	614	]=	0.383347124	;
    RnonCO2[	615	]=	0.383347124	;
    RnonCO2[	616	]=	0.383347124	;
    RnonCO2[	617	]=	0.383347124	;
    RnonCO2[	618	]=	0.383347124	;
    RnonCO2[	619	]=	0.383347124	;
    RnonCO2[	620	]=	0.383347124	;
    RnonCO2[	621	]=	0.383347124	;
    RnonCO2[	622	]=	0.383347124	;
    RnonCO2[	623	]=	0.383347124	;
    RnonCO2[	624	]=	0.383347124	;
    RnonCO2[	625	]=	0.383347124	;
    RnonCO2[	626	]=	0.383347124	;
    RnonCO2[	627	]=	0.383347124	;
    RnonCO2[	628	]=	0.383347124	;
    RnonCO2[	629	]=	0.383347124	;
    RnonCO2[	630	]=	0.383347124	;
    RnonCO2[	631	]=	0.383347124	;
    RnonCO2[	632	]=	0.383347124	;
    RnonCO2[	633	]=	0.383347124	;
    RnonCO2[	634	]=	0.383347124	;
    RnonCO2[	635	]=	0.383347124	;
    RnonCO2[	636	]=	0.383347124	;
    RnonCO2[	637	]=	0.383347124	;
    RnonCO2[	638	]=	0.383347124	;
    RnonCO2[	639	]=	0.383347124	;
    RnonCO2[	640	]=	0.383347124	;
    RnonCO2[	641	]=	0.383347124	;
    RnonCO2[	642	]=	0.383347124	;
    RnonCO2[	643	]=	0.383347124	;
    RnonCO2[	644	]=	0.383347124	;
    RnonCO2[	645	]=	0.383347124	;
    RnonCO2[	646	]=	0.383347124	;
    RnonCO2[	647	]=	0.383347124	;
    RnonCO2[	648	]=	0.383347124	;
    RnonCO2[	649	]=	0.383347124	;
    RnonCO2[	650	]=	0.383347124	;
    RnonCO2[	651	]=	0.383347124	;
    RnonCO2[	652	]=	0.383347124	;
    RnonCO2[	653	]=	0.383347124	;
    RnonCO2[	654	]=	0.383347124	;
    RnonCO2[	655	]=	0.383347124	;
    RnonCO2[	656	]=	0.383347124	;
    RnonCO2[	657	]=	0.383347124	;
    RnonCO2[	658	]=	0.383347124	;
    RnonCO2[	659	]=	0.383347124	;
    RnonCO2[	660	]=	0.383347124	;
    RnonCO2[	661	]=	0.383347124	;
    RnonCO2[	662	]=	0.383347124	;
    RnonCO2[	663	]=	0.383347124	;
    RnonCO2[	664	]=	0.383347124	;
    RnonCO2[	665	]=	0.383347124	;
    RnonCO2[	666	]=	0.383347124	;
    RnonCO2[	667	]=	0.383347124	;
    RnonCO2[	668	]=	0.383347124	;
    RnonCO2[	669	]=	0.383347124	;
    RnonCO2[	670	]=	0.383347124	;
    RnonCO2[	671	]=	0.383347124	;
    RnonCO2[	672	]=	0.383347124	;
    RnonCO2[	673	]=	0.383347124	;
    RnonCO2[	674	]=	0.383347124	;
    RnonCO2[	675	]=	0.383347124	;
    RnonCO2[	676	]=	0.383347124	;
    RnonCO2[	677	]=	0.383347124	;
    RnonCO2[	678	]=	0.383347124	;
    RnonCO2[	679	]=	0.383347124	;
    RnonCO2[	680	]=	0.383347124	;
    RnonCO2[	681	]=	0.383347124	;
    RnonCO2[	682	]=	0.383347124	;
    RnonCO2[	683	]=	0.383347124	;
    RnonCO2[	684	]=	0.383347124	;
    RnonCO2[	685	]=	0.383347124	;
    RnonCO2[	686	]=	0.383347124	;
    RnonCO2[	687	]=	0.383347124	;
    RnonCO2[	688	]=	0.383347124	;
    RnonCO2[	689	]=	0.383347124	;
    RnonCO2[	690	]=	0.383347124	;
    RnonCO2[	691	]=	0.383347124	;
    RnonCO2[	692	]=	0.383347124	;
    RnonCO2[	693	]=	0.383347124	;
    RnonCO2[	694	]=	0.383347124	;
    RnonCO2[	695	]=	0.383347124	;
    RnonCO2[	696	]=	0.383347124	;
    RnonCO2[	697	]=	0.383347124	;
    RnonCO2[	698	]=	0.383347124	;
    RnonCO2[	699	]=	0.383347124	;
    RnonCO2[	700	]=	0.383347124	;
    RnonCO2[	701	]=	0.383347124	;
    RnonCO2[	702	]=	0.383347124	;
    RnonCO2[	703	]=	0.383347124	;
    RnonCO2[	704	]=	0.383347124	;
    RnonCO2[	705	]=	0.383347124	;
    RnonCO2[	706	]=	0.383347124	;
    RnonCO2[	707	]=	0.383347124	;
    RnonCO2[	708	]=	0.383347124	;
    RnonCO2[	709	]=	0.383347124	;
    RnonCO2[	710	]=	0.383347124	;
    RnonCO2[	711	]=	0.383347124	;
    RnonCO2[	712	]=	0.383347124	;
    RnonCO2[	713	]=	0.383347124	;
    RnonCO2[	714	]=	0.383347124	;
    RnonCO2[	715	]=	0.383347124	;
    RnonCO2[	716	]=	0.383347124	;
    RnonCO2[	717	]=	0.383347124	;
    RnonCO2[	718	]=	0.383347124	;
    RnonCO2[	719	]=	0.383347124	;
    RnonCO2[	720	]=	0.383347124	;
    RnonCO2[	721	]=	0.383347124	;
    RnonCO2[	722	]=	0.383347124	;
    RnonCO2[	723	]=	0.383347124	;
    RnonCO2[	724	]=	0.383347124	;
    RnonCO2[	725	]=	0.383347124	;
    RnonCO2[	726	]=	0.383347124	;
    RnonCO2[	727	]=	0.383347124	;
    RnonCO2[	728	]=	0.383347124	;
    RnonCO2[	729	]=	0.383347124	;
    RnonCO2[	730	]=	0.383347124	;
    RnonCO2[	731	]=	0.383347124	;
    RnonCO2[	732	]=	0.383347124	;
    RnonCO2[	733	]=	0.383347124	;
    RnonCO2[	734	]=	0.383347124	;
    RnonCO2[	735	]=	0.383347124	;
    
    for(int i = 736; i<tmax; i++)
    {
        RnonCO2[i] = RnonCO2[i-1];
    }
}

void setRnonCO2_RCP6(void)
{
    RnonCO2[	0	]=	-0.019988515	;
    RnonCO2[	1	]=	-0.005145555	;
    RnonCO2[	2	]=	0.009090068	;
    RnonCO2[	3	]=	0.007768629	;
    RnonCO2[	4	]=	0.006256789	;
    RnonCO2[	5	]=	0.004769658	;
    RnonCO2[	6	]=	0.002906336	;
    RnonCO2[	7	]=	0.001080178	;
    RnonCO2[	8	]=	-0.000291468	;
    RnonCO2[	9	]=	-0.001612412	;
    RnonCO2[	10	]=	-0.00288359	;
    RnonCO2[	11	]=	-0.00410611	;
    RnonCO2[	12	]=	-0.005282233	;
    RnonCO2[	13	]=	-0.006413549	;
    RnonCO2[	14	]=	-0.007501746	;
    RnonCO2[	15	]=	-0.008551056	;
    RnonCO2[	16	]=	-0.010019335	;
    RnonCO2[	17	]=	-0.011459991	;
    RnonCO2[	18	]=	-0.012417579	;
    RnonCO2[	19	]=	-0.013333977	;
    RnonCO2[	20	]=	-0.014201345	;
    RnonCO2[	21	]=	-0.015012487	;
    RnonCO2[	22	]=	-0.015760238	;
    RnonCO2[	23	]=	-0.016437067	;
    RnonCO2[	24	]=	-0.017035543	;
    RnonCO2[	25	]=	-0.017549544	;
    RnonCO2[	26	]=	-0.018434793	;
    RnonCO2[	27	]=	-0.019238993	;
    RnonCO2[	28	]=	-0.019510794	;
    RnonCO2[	29	]=	-0.019709367	;
    RnonCO2[	30	]=	-0.019838439	;
    RnonCO2[	31	]=	-0.019901316	;
    RnonCO2[	32	]=	-0.020355353	;
    RnonCO2[	33	]=	-0.020749651	;
    RnonCO2[	34	]=	-0.020634193	;
    RnonCO2[	35	]=	-0.02091346	;
    RnonCO2[	36	]=	-0.021125633	;
    RnonCO2[	37	]=	-0.02171132	;
    RnonCO2[	38	]=	-0.021774359	;
    RnonCO2[	39	]=	-0.020873209	;
    RnonCO2[	40	]=	-0.02036553	;
    RnonCO2[	41	]=	-0.020256367	;
    RnonCO2[	42	]=	-0.020105185	;
    RnonCO2[	43	]=	-0.01947287	;
    RnonCO2[	44	]=	-0.018814746	;
    RnonCO2[	45	]=	-0.018134946	;
    RnonCO2[	46	]=	-0.017872094	;
    RnonCO2[	47	]=	-0.017576381	;
    RnonCO2[	48	]=	-0.016804485	;
    RnonCO2[	49	]=	-0.016001407	;
    RnonCO2[	50	]=	-0.015606441	;
    RnonCO2[	51	]=	-0.015619264	;
    RnonCO2[	52	]=	-0.015600061	;
    RnonCO2[	53	]=	-0.015116575	;
    RnonCO2[	54	]=	-0.014172313	;
    RnonCO2[	55	]=	-0.013205379	;
    RnonCO2[	56	]=	-0.012216093	;
    RnonCO2[	57	]=	-0.011635129	;
    RnonCO2[	58	]=	-0.011459514	;
    RnonCO2[	59	]=	-0.010829842	;
    RnonCO2[	60	]=	-0.010175791	;
    RnonCO2[	61	]=	-0.009499636	;
    RnonCO2[	62	]=	-0.00879934	;
    RnonCO2[	63	]=	-0.008077943	;
    RnonCO2[	64	]=	-0.006912545	;
    RnonCO2[	65	]=	-0.008242223	;
    RnonCO2[	66	]=	-0.009135821	;
    RnonCO2[	67	]=	-0.007501052	;
    RnonCO2[	68	]=	-0.006687418	;
    RnonCO2[	69	]=	-0.005858015	;
    RnonCO2[	70	]=	-0.005005979	;
    RnonCO2[	71	]=	-0.005767925	;
    RnonCO2[	72	]=	-0.006086639	;
    RnonCO2[	73	]=	-0.005137938	;
    RnonCO2[	74	]=	-0.004558881	;
    RnonCO2[	75	]=	-0.004336125	;
    RnonCO2[	76	]=	-0.004045198	;
    RnonCO2[	77	]=	-0.003665109	;
    RnonCO2[	78	]=	-0.003220213	;
    RnonCO2[	79	]=	-0.002760518	;
    RnonCO2[	80	]=	-0.003527376	;
    RnonCO2[	81	]=	-0.003576651	;
    RnonCO2[	82	]=	-0.003275283	;
    RnonCO2[	83	]=	-0.003386771	;
    RnonCO2[	84	]=	-0.003510917	;
    RnonCO2[	85	]=	-0.004646356	;
    RnonCO2[	86	]=	-0.003691576	;
    RnonCO2[	87	]=	-0.009505158	;
    RnonCO2[	88	]=	-0.016364432	;
    RnonCO2[	89	]=	-0.017996784	;
    RnonCO2[	90	]=	-0.019670058	;
    RnonCO2[	91	]=	-0.020430126	;
    RnonCO2[	92	]=	-0.021643321	;
    RnonCO2[	93	]=	-0.02142206	;
    RnonCO2[	94	]=	-0.022552345	;
    RnonCO2[	95	]=	-0.031588211	;
    RnonCO2[	96	]=	-0.046252985	;
    RnonCO2[	97	]=	-0.042365448	;
    RnonCO2[	98	]=	-0.033665717	;
    RnonCO2[	99	]=	-0.034555638	;
    RnonCO2[	100	]=	-0.037298509	;
    RnonCO2[	101	]=	-0.046083195	;
    RnonCO2[	102	]=	-0.05558168	;
    RnonCO2[	103	]=	-0.057464266	;
    RnonCO2[	104	]=	-0.053606449	;
    RnonCO2[	105	]=	-0.05168834	;
    RnonCO2[	106	]=	-0.061304945	;
    RnonCO2[	107	]=	-0.086608149	;
    RnonCO2[	108	]=	-0.108195137	;
    RnonCO2[	109	]=	-0.112259625	;
    RnonCO2[	110	]=	-0.111643119	;
    RnonCO2[	111	]=	-0.108991576	;
    RnonCO2[	112	]=	-0.10203489	;
    RnonCO2[	113	]=	-0.098533227	;
    RnonCO2[	114	]=	-0.101949678	;
    RnonCO2[	115	]=	-0.114562053	;
    RnonCO2[	116	]=	-0.123024156	;
    RnonCO2[	117	]=	-0.119809652	;
    RnonCO2[	118	]=	-0.118941351	;
    RnonCO2[	119	]=	-0.120181779	;
    RnonCO2[	120	]=	-0.117316831	;
    RnonCO2[	121	]=	-0.115023243	;
    RnonCO2[	122	]=	-0.116983521	;
    RnonCO2[	123	]=	-0.125997633	;
    RnonCO2[	124	]=	-0.129568227	;
    RnonCO2[	125	]=	-0.131248987	;
    RnonCO2[	126	]=	-0.137316696	;
    RnonCO2[	127	]=	-0.141273792	;
    RnonCO2[	128	]=	-0.144231375	;
    RnonCO2[	129	]=	-0.147973341	;
    RnonCO2[	130	]=	-0.153688224	;
    RnonCO2[	131	]=	-0.15652816	;
    RnonCO2[	132	]=	-0.157109013	;
    RnonCO2[	133	]=	-0.158858513	;
    RnonCO2[	134	]=	-0.162186704	;
    RnonCO2[	135	]=	-0.165318228	;
    RnonCO2[	136	]=	-0.171733193	;
    RnonCO2[	137	]=	-0.178096644	;
    RnonCO2[	138	]=	-0.183870407	;
    RnonCO2[	139	]=	-0.189730028	;
    RnonCO2[	140	]=	-0.194221589	;
    RnonCO2[	141	]=	-0.201576481	;
    RnonCO2[	142	]=	-0.208859035	;
    RnonCO2[	143	]=	-0.207375441	;
    RnonCO2[	144	]=	-0.202571711	;
    RnonCO2[	145	]=	-0.204126848	;
    RnonCO2[	146	]=	-0.201876709	;
    RnonCO2[	147	]=	-0.195147841	;
    RnonCO2[	148	]=	-0.191449584	;
    RnonCO2[	149	]=	-0.183770215	;
    RnonCO2[	150	]=	-0.173476305	;
    RnonCO2[	151	]=	-0.170909799	;
    RnonCO2[	152	]=	-0.171515561	;
    RnonCO2[	153	]=	-0.169079801	;
    RnonCO2[	154	]=	-0.159414516	;
    RnonCO2[	155	]=	-0.155013584	;
    RnonCO2[	156	]=	-0.151089869	;
    RnonCO2[	157	]=	-0.144432651	;
    RnonCO2[	158	]=	-0.152359798	;
    RnonCO2[	159	]=	-0.158148636	;
    RnonCO2[	160	]=	-0.155277608	;
    RnonCO2[	161	]=	-0.152617947	;
    RnonCO2[	162	]=	-0.154120923	;
    RnonCO2[	163	]=	-0.156019482	;
    RnonCO2[	164	]=	-0.155886394	;
    RnonCO2[	165	]=	-0.152021034	;
    RnonCO2[	166	]=	-0.141834688	;
    RnonCO2[	167	]=	-0.13108946	;
    RnonCO2[	168	]=	-0.125408882	;
    RnonCO2[	169	]=	-0.125515632	;
    RnonCO2[	170	]=	-0.125194614	;
    RnonCO2[	171	]=	-0.126120729	;
    RnonCO2[	172	]=	-0.12532345	;
    RnonCO2[	173	]=	-0.115303554	;
    RnonCO2[	174	]=	-0.10787249	;
    RnonCO2[	175	]=	-0.109257271	;
    RnonCO2[	176	]=	-0.113724728	;
    RnonCO2[	177	]=	-0.117421621	;
    RnonCO2[	178	]=	-0.11813163	;
    RnonCO2[	179	]=	-0.119629474	;
    RnonCO2[	180	]=	-0.119789192	;
    RnonCO2[	181	]=	-0.124270593	;
    RnonCO2[	182	]=	-0.135220142	;
    RnonCO2[	183	]=	-0.145282294	;
    RnonCO2[	184	]=	-0.1490379	;
    RnonCO2[	185	]=	-0.154219559	;
    RnonCO2[	186	]=	-0.178760049	;
    RnonCO2[	187	]=	-0.203310188	;
    RnonCO2[	188	]=	-0.212758852	;
    RnonCO2[	189	]=	-0.221219612	;
    RnonCO2[	190	]=	-0.230341307	;
    RnonCO2[	191	]=	-0.239337924	;
    RnonCO2[	192	]=	-0.247524974	;
    RnonCO2[	193	]=	-0.250847198	;
    RnonCO2[	194	]=	-0.245200212	;
    RnonCO2[	195	]=	-0.237476895	;
    RnonCO2[	196	]=	-0.24791739	;
    RnonCO2[	197	]=	-0.261719365	;
    RnonCO2[	198	]=	-0.263179809	;
    RnonCO2[	199	]=	-0.263298027	;
    RnonCO2[	200	]=	-0.261139159	;
    RnonCO2[	201	]=	-0.257781723	;
    RnonCO2[	202	]=	-0.255724839	;
    RnonCO2[	203	]=	-0.25169763	;
    RnonCO2[	204	]=	-0.241647721	;
    RnonCO2[	205	]=	-0.221452127	;
    RnonCO2[	206	]=	-0.196790013	;
    RnonCO2[	207	]=	-0.182093765	;
    RnonCO2[	208	]=	-0.174745229	;
    RnonCO2[	209	]=	-0.16714548	;
    RnonCO2[	210	]=	-0.159384584	;
    RnonCO2[	211	]=	-0.156373877	;
    RnonCO2[	212	]=	-0.153042015	;
    RnonCO2[	213	]=	-0.139817455	;
    RnonCO2[	214	]=	-0.125210902	;
    RnonCO2[	215	]=	-0.119848303	;
    RnonCO2[	216	]=	-0.119430668	;
    RnonCO2[	217	]=	-0.123211365	;
    RnonCO2[	218	]=	-0.125809708	;
    RnonCO2[	219	]=	-0.120193155	;
    RnonCO2[	220	]=	-0.110911249	;
    RnonCO2[	221	]=	-0.102402229	;
    RnonCO2[	222	]=	-0.096562536	;
    RnonCO2[	223	]=	-0.086930587	;
    RnonCO2[	224	]=	-0.072710932	;
    RnonCO2[	225	]=	-0.05997191	;
    RnonCO2[	226	]=	-0.058006544	;
    RnonCO2[	227	]=	-0.065604043	;
    RnonCO2[	228	]=	-0.067949961	;
    RnonCO2[	229	]=	-0.065994527	;
    RnonCO2[	230	]=	-0.058283284	;
    RnonCO2[	231	]=	-0.04756956	;
    RnonCO2[	232	]=	-0.049208339	;
    RnonCO2[	233	]=	-0.056699684	;
    RnonCO2[	234	]=	-0.059531967	;
    RnonCO2[	235	]=	-0.065638041	;
    RnonCO2[	236	]=	-0.069976074	;
    RnonCO2[	237	]=	-0.065660736	;
    RnonCO2[	238	]=	-0.060876733	;
    RnonCO2[	239	]=	-0.057486962	;
    RnonCO2[	240	]=	-0.053923583	;
    RnonCO2[	241	]=	-0.048265245	;
    RnonCO2[	242	]=	-0.041908889	;
    RnonCO2[	243	]=	-0.035470985	;
    RnonCO2[	244	]=	-0.028488885	;
    RnonCO2[	245	]=	-0.020170078	;
    RnonCO2[	246	]=	-0.009654576	;
    RnonCO2[	247	]=	0.002229379	;
    RnonCO2[	248	]=	0.013767156	;
    RnonCO2[	249	]=	0.024503705	;
    RnonCO2[	250	]=	0.034790944	;
    RnonCO2[	251	]=	0.044692559	;
    RnonCO2[	252	]=	0.054247902	;
    RnonCO2[	253	]=	0.063487271	;
    RnonCO2[	254	]=	0.07300731	;
    RnonCO2[	255	]=	0.083115143	;
    RnonCO2[	256	]=	0.093858679	;
    RnonCO2[	257	]=	0.105274945	;
    RnonCO2[	258	]=	0.117015494	;
    RnonCO2[	259	]=	0.129042558	;
    RnonCO2[	260	]=	0.141370327	;
    RnonCO2[	261	]=	0.153994973	;
    RnonCO2[	262	]=	0.166903607	;
    RnonCO2[	263	]=	0.18006939	;
    RnonCO2[	264	]=	0.193471412	;
    RnonCO2[	265	]=	0.204195251	;
    RnonCO2[	266	]=	0.209318794	;
    RnonCO2[	267	]=	0.211921957	;
    RnonCO2[	268	]=	0.214765729	;
    RnonCO2[	269	]=	0.217500548	;
    RnonCO2[	270	]=	0.220111388	;
    RnonCO2[	271	]=	0.222584471	;
    RnonCO2[	272	]=	0.224905928	;
    RnonCO2[	273	]=	0.227053889	;
    RnonCO2[	274	]=	0.228995888	;
    RnonCO2[	275	]=	0.232519566	;
    RnonCO2[	276	]=	0.239403838	;
    RnonCO2[	277	]=	0.247369643	;
    RnonCO2[	278	]=	0.254573	;
    RnonCO2[	279	]=	0.26146722	;
    RnonCO2[	280	]=	0.268143306	;
    RnonCO2[	281	]=	0.274673454	;
    RnonCO2[	282	]=	0.281112631	;
    RnonCO2[	283	]=	0.28750532	;
    RnonCO2[	284	]=	0.293860225	;
    RnonCO2[	285	]=	0.299059232	;
    RnonCO2[	286	]=	0.302026911	;
    RnonCO2[	287	]=	0.303902576	;
    RnonCO2[	288	]=	0.305810546	;
    RnonCO2[	289	]=	0.307728275	;
    RnonCO2[	290	]=	0.309636186	;
    RnonCO2[	291	]=	0.311574085	;
    RnonCO2[	292	]=	0.313572228	;
    RnonCO2[	293	]=	0.315655383	;
    RnonCO2[	294	]=	0.317849754	;
    RnonCO2[	295	]=	0.323336003	;
    RnonCO2[	296	]=	0.335279015	;
    RnonCO2[	297	]=	0.349977553	;
    RnonCO2[	298	]=	0.364212377	;
    RnonCO2[	299	]=	0.378446117	;
    RnonCO2[	300	]=	0.392644356	;
    RnonCO2[	301	]=	0.406791296	;
    RnonCO2[	302	]=	0.420931456	;
    RnonCO2[	303	]=	0.435100184	;
    RnonCO2[	304	]=	0.449329044	;
    RnonCO2[	305	]=	0.462134841	;
    RnonCO2[	306	]=	0.472000098	;
    RnonCO2[	307	]=	0.480345009	;
    RnonCO2[	308	]=	0.488498229	;
    RnonCO2[	309	]=	0.496365261	;
    RnonCO2[	310	]=	0.503969592	;
    RnonCO2[	311	]=	0.511290562	;
    RnonCO2[	312	]=	0.518324478	;
    RnonCO2[	313	]=	0.525126448	;
    RnonCO2[	314	]=	0.531742174	;
    RnonCO2[	315	]=	0.536514421	;
    RnonCO2[	316	]=	0.537793824	;
    RnonCO2[	317	]=	0.537189424	;
    RnonCO2[	318	]=	0.535963926	;
    RnonCO2[	319	]=	0.533830098	;
    RnonCO2[	320	]=	0.530854212	;
    RnonCO2[	321	]=	0.527096934	;
    RnonCO2[	322	]=	0.522571459	;
    RnonCO2[	323	]=	0.517304785	;
    RnonCO2[	324	]=	0.511378144	;
    RnonCO2[	325	]=	0.504950814	;
    RnonCO2[	326	]=	0.498173588	;
    RnonCO2[	327	]=	0.491427323	;
    RnonCO2[	328	]=	0.485029515	;
    RnonCO2[	329	]=	0.478935116	;
    RnonCO2[	330	]=	0.473115467	;
    RnonCO2[	331	]=	0.467548525	;
    RnonCO2[	332	]=	0.462213949	;
    RnonCO2[	333	]=	0.457139058	;
    RnonCO2[	334	]=	0.452401937	;
    RnonCO2[	335	]=	0.447294793	;
    RnonCO2[	336	]=	0.441058515	;
    RnonCO2[	337	]=	0.435046091	;
    RnonCO2[	338	]=	0.430165827	;
    RnonCO2[	339	]=	0.425920409	;
    RnonCO2[	340	]=	0.422225695	;
    RnonCO2[	341	]=	0.41901013	;
    RnonCO2[	342	]=	0.416215121	;
    RnonCO2[	343	]=	0.413788317	;
    RnonCO2[	344	]=	0.411645604	;
    RnonCO2[	345	]=	0.409724998	;
    RnonCO2[	346	]=	0.408023702	;
    RnonCO2[	347	]=	0.406535893	;
    RnonCO2[	348	]=	0.405253984	;
    RnonCO2[	349	]=	0.404174485	;
    RnonCO2[	350	]=	0.403275605	;
    RnonCO2[	351	]=	0.402522488	;
    RnonCO2[	352	]=	0.401887996	;
    RnonCO2[	353	]=	0.401352761	;
    RnonCO2[	354	]=	0.400899942	;
    RnonCO2[	355	]=	0.400477424	;
    RnonCO2[	356	]=	0.40005214	;
    RnonCO2[	357	]=	0.399645998	;
    RnonCO2[	358	]=	0.399275485	;
    RnonCO2[	359	]=	0.398953056	;
    RnonCO2[	360	]=	0.398690295	;
    RnonCO2[	361	]=	0.3984758	;
    RnonCO2[	362	]=	0.398286722	;
    RnonCO2[	363	]=	0.398108358	;
    RnonCO2[	364	]=	0.397932702	;
    RnonCO2[	365	]=	0.397752792	;
    RnonCO2[	366	]=	0.397526109	;
    RnonCO2[	367	]=	0.397227301	;
    RnonCO2[	368	]=	0.396885187	;
    RnonCO2[	369	]=	0.396522534	;
    RnonCO2[	370	]=	0.396156959	;
    RnonCO2[	371	]=	0.39580735	;
    RnonCO2[	372	]=	0.395472249	;
    RnonCO2[	373	]=	0.395134885	;
    RnonCO2[	374	]=	0.394784137	;
    RnonCO2[	375	]=	0.394414873	;
    RnonCO2[	376	]=	0.394023161	;
    RnonCO2[	377	]=	0.393568791	;
    RnonCO2[	378	]=	0.393028761	;
    RnonCO2[	379	]=	0.392433693	;
    RnonCO2[	380	]=	0.391807878	;
    RnonCO2[	381	]=	0.391170327	;
    RnonCO2[	382	]=	0.390541212	;
    RnonCO2[	383	]=	0.389919841	;
    RnonCO2[	384	]=	0.38929067	;
    RnonCO2[	385	]=	0.388972503	;
    RnonCO2[	386	]=	0.388972503	;
    RnonCO2[	387	]=	0.388972503	;
    RnonCO2[	388	]=	0.388972503	;
    RnonCO2[	389	]=	0.388972503	;
    RnonCO2[	390	]=	0.388972503	;
    RnonCO2[	391	]=	0.388972503	;
    RnonCO2[	392	]=	0.388972503	;
    RnonCO2[	393	]=	0.388972503	;
    RnonCO2[	394	]=	0.388972503	;
    RnonCO2[	395	]=	0.388972503	;
    RnonCO2[	396	]=	0.388972503	;
    RnonCO2[	397	]=	0.388972503	;
    RnonCO2[	398	]=	0.388972503	;
    RnonCO2[	399	]=	0.388972503	;
    RnonCO2[	400	]=	0.388972503	;
    RnonCO2[	401	]=	0.388972503	;
    RnonCO2[	402	]=	0.388972503	;
    RnonCO2[	403	]=	0.388972503	;
    RnonCO2[	404	]=	0.388972503	;
    RnonCO2[	405	]=	0.388972503	;
    RnonCO2[	406	]=	0.388972503	;
    RnonCO2[	407	]=	0.388972503	;
    RnonCO2[	408	]=	0.388972503	;
    RnonCO2[	409	]=	0.388972503	;
    RnonCO2[	410	]=	0.388972503	;
    RnonCO2[	411	]=	0.388972503	;
    RnonCO2[	412	]=	0.388972503	;
    RnonCO2[	413	]=	0.388972503	;
    RnonCO2[	414	]=	0.388972503	;
    RnonCO2[	415	]=	0.388972503	;
    RnonCO2[	416	]=	0.388972503	;
    RnonCO2[	417	]=	0.388972503	;
    RnonCO2[	418	]=	0.388972503	;
    RnonCO2[	419	]=	0.388972503	;
    RnonCO2[	420	]=	0.388972503	;
    RnonCO2[	421	]=	0.388972503	;
    RnonCO2[	422	]=	0.388972503	;
    RnonCO2[	423	]=	0.388972503	;
    RnonCO2[	424	]=	0.388972503	;
    RnonCO2[	425	]=	0.388972503	;
    RnonCO2[	426	]=	0.388972503	;
    RnonCO2[	427	]=	0.388972503	;
    RnonCO2[	428	]=	0.388972503	;
    RnonCO2[	429	]=	0.388972503	;
    RnonCO2[	430	]=	0.388972503	;
    RnonCO2[	431	]=	0.388972503	;
    RnonCO2[	432	]=	0.388972503	;
    RnonCO2[	433	]=	0.388972503	;
    RnonCO2[	434	]=	0.388972503	;
    RnonCO2[	435	]=	0.388972503	;
    RnonCO2[	436	]=	0.388972503	;
    RnonCO2[	437	]=	0.388972503	;
    RnonCO2[	438	]=	0.388972503	;
    RnonCO2[	439	]=	0.388972503	;
    RnonCO2[	440	]=	0.388972503	;
    RnonCO2[	441	]=	0.388972503	;
    RnonCO2[	442	]=	0.388972503	;
    RnonCO2[	443	]=	0.388972503	;
    RnonCO2[	444	]=	0.388972503	;
    RnonCO2[	445	]=	0.388972503	;
    RnonCO2[	446	]=	0.388972503	;
    RnonCO2[	447	]=	0.388972503	;
    RnonCO2[	448	]=	0.388972503	;
    RnonCO2[	449	]=	0.388972503	;
    RnonCO2[	450	]=	0.388972503	;
    RnonCO2[	451	]=	0.388972503	;
    RnonCO2[	452	]=	0.388972503	;
    RnonCO2[	453	]=	0.388972503	;
    RnonCO2[	454	]=	0.388972503	;
    RnonCO2[	455	]=	0.388972503	;
    RnonCO2[	456	]=	0.388972503	;
    RnonCO2[	457	]=	0.388972503	;
    RnonCO2[	458	]=	0.388972503	;
    RnonCO2[	459	]=	0.388972503	;
    RnonCO2[	460	]=	0.388972503	;
    RnonCO2[	461	]=	0.388972503	;
    RnonCO2[	462	]=	0.388972503	;
    RnonCO2[	463	]=	0.388972503	;
    RnonCO2[	464	]=	0.388972503	;
    RnonCO2[	465	]=	0.388972503	;
    RnonCO2[	466	]=	0.388972503	;
    RnonCO2[	467	]=	0.388972503	;
    RnonCO2[	468	]=	0.388972503	;
    RnonCO2[	469	]=	0.388972503	;
    RnonCO2[	470	]=	0.388972503	;
    RnonCO2[	471	]=	0.388972503	;
    RnonCO2[	472	]=	0.388972503	;
    RnonCO2[	473	]=	0.388972503	;
    RnonCO2[	474	]=	0.388972503	;
    RnonCO2[	475	]=	0.388972503	;
    RnonCO2[	476	]=	0.388972503	;
    RnonCO2[	477	]=	0.388972503	;
    RnonCO2[	478	]=	0.388972503	;
    RnonCO2[	479	]=	0.388972503	;
    RnonCO2[	480	]=	0.388972503	;
    RnonCO2[	481	]=	0.388972503	;
    RnonCO2[	482	]=	0.388972503	;
    RnonCO2[	483	]=	0.388972503	;
    RnonCO2[	484	]=	0.388972503	;
    RnonCO2[	485	]=	0.388972503	;
    RnonCO2[	486	]=	0.388972503	;
    RnonCO2[	487	]=	0.388972503	;
    RnonCO2[	488	]=	0.388972503	;
    RnonCO2[	489	]=	0.388972503	;
    RnonCO2[	490	]=	0.388972503	;
    RnonCO2[	491	]=	0.388972503	;
    RnonCO2[	492	]=	0.388972503	;
    RnonCO2[	493	]=	0.388972503	;
    RnonCO2[	494	]=	0.388972503	;
    RnonCO2[	495	]=	0.388972503	;
    RnonCO2[	496	]=	0.388972503	;
    RnonCO2[	497	]=	0.388972503	;
    RnonCO2[	498	]=	0.388972503	;
    RnonCO2[	499	]=	0.388972503	;
    RnonCO2[	500	]=	0.388972503	;
    RnonCO2[	501	]=	0.388972503	;
    RnonCO2[	502	]=	0.388972503	;
    RnonCO2[	503	]=	0.388972503	;
    RnonCO2[	504	]=	0.388972503	;
    RnonCO2[	505	]=	0.388972503	;
    RnonCO2[	506	]=	0.388972503	;
    RnonCO2[	507	]=	0.388972503	;
    RnonCO2[	508	]=	0.388972503	;
    RnonCO2[	509	]=	0.388972503	;
    RnonCO2[	510	]=	0.388972503	;
    RnonCO2[	511	]=	0.388972503	;
    RnonCO2[	512	]=	0.388972503	;
    RnonCO2[	513	]=	0.388972503	;
    RnonCO2[	514	]=	0.388972503	;
    RnonCO2[	515	]=	0.388972503	;
    RnonCO2[	516	]=	0.388972503	;
    RnonCO2[	517	]=	0.388972503	;
    RnonCO2[	518	]=	0.388972503	;
    RnonCO2[	519	]=	0.388972503	;
    RnonCO2[	520	]=	0.388972503	;
    RnonCO2[	521	]=	0.388972503	;
    RnonCO2[	522	]=	0.388972503	;
    RnonCO2[	523	]=	0.388972503	;
    RnonCO2[	524	]=	0.388972503	;
    RnonCO2[	525	]=	0.388972503	;
    RnonCO2[	526	]=	0.388972503	;
    RnonCO2[	527	]=	0.388972503	;
    RnonCO2[	528	]=	0.388972503	;
    RnonCO2[	529	]=	0.388972503	;
    RnonCO2[	530	]=	0.388972503	;
    RnonCO2[	531	]=	0.388972503	;
    RnonCO2[	532	]=	0.388972503	;
    RnonCO2[	533	]=	0.388972503	;
    RnonCO2[	534	]=	0.388972503	;
    RnonCO2[	535	]=	0.388972503	;
    RnonCO2[	536	]=	0.388972503	;
    RnonCO2[	537	]=	0.388972503	;
    RnonCO2[	538	]=	0.388972503	;
    RnonCO2[	539	]=	0.388972503	;
    RnonCO2[	540	]=	0.388972503	;
    RnonCO2[	541	]=	0.388972503	;
    RnonCO2[	542	]=	0.388972503	;
    RnonCO2[	543	]=	0.388972503	;
    RnonCO2[	544	]=	0.388972503	;
    RnonCO2[	545	]=	0.388972503	;
    RnonCO2[	546	]=	0.388972503	;
    RnonCO2[	547	]=	0.388972503	;
    RnonCO2[	548	]=	0.388972503	;
    RnonCO2[	549	]=	0.388972503	;
    RnonCO2[	550	]=	0.388972503	;
    RnonCO2[	551	]=	0.388972503	;
    RnonCO2[	552	]=	0.388972503	;
    RnonCO2[	553	]=	0.388972503	;
    RnonCO2[	554	]=	0.388972503	;
    RnonCO2[	555	]=	0.388972503	;
    RnonCO2[	556	]=	0.388972503	;
    RnonCO2[	557	]=	0.388972503	;
    RnonCO2[	558	]=	0.388972503	;
    RnonCO2[	559	]=	0.388972503	;
    RnonCO2[	560	]=	0.388972503	;
    RnonCO2[	561	]=	0.388972503	;
    RnonCO2[	562	]=	0.388972503	;
    RnonCO2[	563	]=	0.388972503	;
    RnonCO2[	564	]=	0.388972503	;
    RnonCO2[	565	]=	0.388972503	;
    RnonCO2[	566	]=	0.388972503	;
    RnonCO2[	567	]=	0.388972503	;
    RnonCO2[	568	]=	0.388972503	;
    RnonCO2[	569	]=	0.388972503	;
    RnonCO2[	570	]=	0.388972503	;
    RnonCO2[	571	]=	0.388972503	;
    RnonCO2[	572	]=	0.388972503	;
    RnonCO2[	573	]=	0.388972503	;
    RnonCO2[	574	]=	0.388972503	;
    RnonCO2[	575	]=	0.388972503	;
    RnonCO2[	576	]=	0.388972503	;
    RnonCO2[	577	]=	0.388972503	;
    RnonCO2[	578	]=	0.388972503	;
    RnonCO2[	579	]=	0.388972503	;
    RnonCO2[	580	]=	0.388972503	;
    RnonCO2[	581	]=	0.388972503	;
    RnonCO2[	582	]=	0.388972503	;
    RnonCO2[	583	]=	0.388972503	;
    RnonCO2[	584	]=	0.388972503	;
    RnonCO2[	585	]=	0.388972503	;
    RnonCO2[	586	]=	0.388972503	;
    RnonCO2[	587	]=	0.388972503	;
    RnonCO2[	588	]=	0.388972503	;
    RnonCO2[	589	]=	0.388972503	;
    RnonCO2[	590	]=	0.388972503	;
    RnonCO2[	591	]=	0.388972503	;
    RnonCO2[	592	]=	0.388972503	;
    RnonCO2[	593	]=	0.388972503	;
    RnonCO2[	594	]=	0.388972503	;
    RnonCO2[	595	]=	0.388972503	;
    RnonCO2[	596	]=	0.388972503	;
    RnonCO2[	597	]=	0.388972503	;
    RnonCO2[	598	]=	0.388972503	;
    RnonCO2[	599	]=	0.388972503	;
    RnonCO2[	600	]=	0.388972503	;
    RnonCO2[	601	]=	0.388972503	;
    RnonCO2[	602	]=	0.388972503	;
    RnonCO2[	603	]=	0.388972503	;
    RnonCO2[	604	]=	0.388972503	;
    RnonCO2[	605	]=	0.388972503	;
    RnonCO2[	606	]=	0.388972503	;
    RnonCO2[	607	]=	0.388972503	;
    RnonCO2[	608	]=	0.388972503	;
    RnonCO2[	609	]=	0.388972503	;
    RnonCO2[	610	]=	0.388972503	;
    RnonCO2[	611	]=	0.388972503	;
    RnonCO2[	612	]=	0.388972503	;
    RnonCO2[	613	]=	0.388972503	;
    RnonCO2[	614	]=	0.388972503	;
    RnonCO2[	615	]=	0.388972503	;
    RnonCO2[	616	]=	0.388972503	;
    RnonCO2[	617	]=	0.388972503	;
    RnonCO2[	618	]=	0.388972503	;
    RnonCO2[	619	]=	0.388972503	;
    RnonCO2[	620	]=	0.388972503	;
    RnonCO2[	621	]=	0.388972503	;
    RnonCO2[	622	]=	0.388972503	;
    RnonCO2[	623	]=	0.388972503	;
    RnonCO2[	624	]=	0.388972503	;
    RnonCO2[	625	]=	0.388972503	;
    RnonCO2[	626	]=	0.388972503	;
    RnonCO2[	627	]=	0.388972503	;
    RnonCO2[	628	]=	0.388972503	;
    RnonCO2[	629	]=	0.388972503	;
    RnonCO2[	630	]=	0.388972503	;
    RnonCO2[	631	]=	0.388972503	;
    RnonCO2[	632	]=	0.388972503	;
    RnonCO2[	633	]=	0.388972503	;
    RnonCO2[	634	]=	0.388972503	;
    RnonCO2[	635	]=	0.388972503	;
    RnonCO2[	636	]=	0.388972503	;
    RnonCO2[	637	]=	0.388972503	;
    RnonCO2[	638	]=	0.388972503	;
    RnonCO2[	639	]=	0.388972503	;
    RnonCO2[	640	]=	0.388972503	;
    RnonCO2[	641	]=	0.388972503	;
    RnonCO2[	642	]=	0.388972503	;
    RnonCO2[	643	]=	0.388972503	;
    RnonCO2[	644	]=	0.388972503	;
    RnonCO2[	645	]=	0.388972503	;
    RnonCO2[	646	]=	0.388972503	;
    RnonCO2[	647	]=	0.388972503	;
    RnonCO2[	648	]=	0.388972503	;
    RnonCO2[	649	]=	0.388972503	;
    RnonCO2[	650	]=	0.388972503	;
    RnonCO2[	651	]=	0.388972503	;
    RnonCO2[	652	]=	0.388972503	;
    RnonCO2[	653	]=	0.388972503	;
    RnonCO2[	654	]=	0.388972503	;
    RnonCO2[	655	]=	0.388972503	;
    RnonCO2[	656	]=	0.388972503	;
    RnonCO2[	657	]=	0.388972503	;
    RnonCO2[	658	]=	0.388972503	;
    RnonCO2[	659	]=	0.388972503	;
    RnonCO2[	660	]=	0.388972503	;
    RnonCO2[	661	]=	0.388972503	;
    RnonCO2[	662	]=	0.388972503	;
    RnonCO2[	663	]=	0.388972503	;
    RnonCO2[	664	]=	0.388972503	;
    RnonCO2[	665	]=	0.388972503	;
    RnonCO2[	666	]=	0.388972503	;
    RnonCO2[	667	]=	0.388972503	;
    RnonCO2[	668	]=	0.388972503	;
    RnonCO2[	669	]=	0.388972503	;
    RnonCO2[	670	]=	0.388972503	;
    RnonCO2[	671	]=	0.388972503	;
    RnonCO2[	672	]=	0.388972503	;
    RnonCO2[	673	]=	0.388972503	;
    RnonCO2[	674	]=	0.388972503	;
    RnonCO2[	675	]=	0.388972503	;
    RnonCO2[	676	]=	0.388972503	;
    RnonCO2[	677	]=	0.388972503	;
    RnonCO2[	678	]=	0.388972503	;
    RnonCO2[	679	]=	0.388972503	;
    RnonCO2[	680	]=	0.388972503	;
    RnonCO2[	681	]=	0.388972503	;
    RnonCO2[	682	]=	0.388972503	;
    RnonCO2[	683	]=	0.388972503	;
    RnonCO2[	684	]=	0.388972503	;
    RnonCO2[	685	]=	0.388972503	;
    RnonCO2[	686	]=	0.388972503	;
    RnonCO2[	687	]=	0.388972503	;
    RnonCO2[	688	]=	0.388972503	;
    RnonCO2[	689	]=	0.388972503	;
    RnonCO2[	690	]=	0.388972503	;
    RnonCO2[	691	]=	0.388972503	;
    RnonCO2[	692	]=	0.388972503	;
    RnonCO2[	693	]=	0.388972503	;
    RnonCO2[	694	]=	0.388972503	;
    RnonCO2[	695	]=	0.388972503	;
    RnonCO2[	696	]=	0.388972503	;
    RnonCO2[	697	]=	0.388972503	;
    RnonCO2[	698	]=	0.388972503	;
    RnonCO2[	699	]=	0.388972503	;
    RnonCO2[	700	]=	0.388972503	;
    RnonCO2[	701	]=	0.388972503	;
    RnonCO2[	702	]=	0.388972503	;
    RnonCO2[	703	]=	0.388972503	;
    RnonCO2[	704	]=	0.388972503	;
    RnonCO2[	705	]=	0.388972503	;
    RnonCO2[	706	]=	0.388972503	;
    RnonCO2[	707	]=	0.388972503	;
    RnonCO2[	708	]=	0.388972503	;
    RnonCO2[	709	]=	0.388972503	;
    RnonCO2[	710	]=	0.388972503	;
    RnonCO2[	711	]=	0.388972503	;
    RnonCO2[	712	]=	0.388972503	;
    RnonCO2[	713	]=	0.388972503	;
    RnonCO2[	714	]=	0.388972503	;
    RnonCO2[	715	]=	0.388972503	;
    RnonCO2[	716	]=	0.388972503	;
    RnonCO2[	717	]=	0.388972503	;
    RnonCO2[	718	]=	0.388972503	;
    RnonCO2[	719	]=	0.388972503	;
    RnonCO2[	720	]=	0.388972503	;
    RnonCO2[	721	]=	0.388972503	;
    RnonCO2[	722	]=	0.388972503	;
    RnonCO2[	723	]=	0.388972503	;
    RnonCO2[	724	]=	0.388972503	;
    RnonCO2[	725	]=	0.388972503	;
    RnonCO2[	726	]=	0.388972503	;
    RnonCO2[	727	]=	0.388972503	;
    RnonCO2[	728	]=	0.388972503	;
    RnonCO2[	729	]=	0.388972503	;
    RnonCO2[	730	]=	0.388972503	;
    RnonCO2[	731	]=	0.388972503	;
    RnonCO2[	732	]=	0.388972503	;
    RnonCO2[	733	]=	0.388972503	;
    RnonCO2[	734	]=	0.388972503	;
    RnonCO2[	735	]=	0.388972503	;
    
    for(int i = 736; i<tmax; i++)
    {
        RnonCO2[i] = RnonCO2[i-1];
    }
    
}

void setRnonCO2_RCP85(void)
{
    RnonCO2[	0	]=	-0.019988515	;
    RnonCO2[	1	]=	-0.005145555	;
    RnonCO2[	2	]=	0.009090068	;
    RnonCO2[	3	]=	0.007768629	;
    RnonCO2[	4	]=	0.006256789	;
    RnonCO2[	5	]=	0.004769658	;
    RnonCO2[	6	]=	0.002906336	;
    RnonCO2[	7	]=	0.001080178	;
    RnonCO2[	8	]=	-0.000291468	;
    RnonCO2[	9	]=	-0.001612412	;
    RnonCO2[	10	]=	-0.00288359	;
    RnonCO2[	11	]=	-0.00410611	;
    RnonCO2[	12	]=	-0.005282233	;
    RnonCO2[	13	]=	-0.006413549	;
    RnonCO2[	14	]=	-0.007501746	;
    RnonCO2[	15	]=	-0.008551056	;
    RnonCO2[	16	]=	-0.010019335	;
    RnonCO2[	17	]=	-0.011459991	;
    RnonCO2[	18	]=	-0.012417579	;
    RnonCO2[	19	]=	-0.013333977	;
    RnonCO2[	20	]=	-0.014201345	;
    RnonCO2[	21	]=	-0.015012487	;
    RnonCO2[	22	]=	-0.015760238	;
    RnonCO2[	23	]=	-0.016437067	;
    RnonCO2[	24	]=	-0.017035543	;
    RnonCO2[	25	]=	-0.017549544	;
    RnonCO2[	26	]=	-0.018434793	;
    RnonCO2[	27	]=	-0.019238993	;
    RnonCO2[	28	]=	-0.019510794	;
    RnonCO2[	29	]=	-0.019709367	;
    RnonCO2[	30	]=	-0.019838439	;
    RnonCO2[	31	]=	-0.019901316	;
    RnonCO2[	32	]=	-0.020355353	;
    RnonCO2[	33	]=	-0.020749651	;
    RnonCO2[	34	]=	-0.020634193	;
    RnonCO2[	35	]=	-0.02091346	;
    RnonCO2[	36	]=	-0.021125633	;
    RnonCO2[	37	]=	-0.02171132	;
    RnonCO2[	38	]=	-0.021774359	;
    RnonCO2[	39	]=	-0.020873209	;
    RnonCO2[	40	]=	-0.02036553	;
    RnonCO2[	41	]=	-0.020256367	;
    RnonCO2[	42	]=	-0.020105185	;
    RnonCO2[	43	]=	-0.01947287	;
    RnonCO2[	44	]=	-0.018814746	;
    RnonCO2[	45	]=	-0.018134946	;
    RnonCO2[	46	]=	-0.017872094	;
    RnonCO2[	47	]=	-0.017576381	;
    RnonCO2[	48	]=	-0.016804485	;
    RnonCO2[	49	]=	-0.016001407	;
    RnonCO2[	50	]=	-0.015606441	;
    RnonCO2[	51	]=	-0.015619264	;
    RnonCO2[	52	]=	-0.015600061	;
    RnonCO2[	53	]=	-0.015116575	;
    RnonCO2[	54	]=	-0.014172313	;
    RnonCO2[	55	]=	-0.013205379	;
    RnonCO2[	56	]=	-0.012216093	;
    RnonCO2[	57	]=	-0.011635129	;
    RnonCO2[	58	]=	-0.011459514	;
    RnonCO2[	59	]=	-0.010829842	;
    RnonCO2[	60	]=	-0.010175791	;
    RnonCO2[	61	]=	-0.009499636	;
    RnonCO2[	62	]=	-0.00879934	;
    RnonCO2[	63	]=	-0.008077943	;
    RnonCO2[	64	]=	-0.006912545	;
    RnonCO2[	65	]=	-0.008242223	;
    RnonCO2[	66	]=	-0.009135821	;
    RnonCO2[	67	]=	-0.007501052	;
    RnonCO2[	68	]=	-0.006687418	;
    RnonCO2[	69	]=	-0.005858015	;
    RnonCO2[	70	]=	-0.005005979	;
    RnonCO2[	71	]=	-0.005767925	;
    RnonCO2[	72	]=	-0.006086639	;
    RnonCO2[	73	]=	-0.005137938	;
    RnonCO2[	74	]=	-0.004558881	;
    RnonCO2[	75	]=	-0.004336125	;
    RnonCO2[	76	]=	-0.004045198	;
    RnonCO2[	77	]=	-0.003665109	;
    RnonCO2[	78	]=	-0.003220213	;
    RnonCO2[	79	]=	-0.002760518	;
    RnonCO2[	80	]=	-0.003527376	;
    RnonCO2[	81	]=	-0.003576651	;
    RnonCO2[	82	]=	-0.003275283	;
    RnonCO2[	83	]=	-0.003386771	;
    RnonCO2[	84	]=	-0.003510917	;
    RnonCO2[	85	]=	-0.004646356	;
    RnonCO2[	86	]=	-0.003691576	;
    RnonCO2[	87	]=	-0.009505158	;
    RnonCO2[	88	]=	-0.016364432	;
    RnonCO2[	89	]=	-0.017996784	;
    RnonCO2[	90	]=	-0.019670058	;
    RnonCO2[	91	]=	-0.020430126	;
    RnonCO2[	92	]=	-0.021643321	;
    RnonCO2[	93	]=	-0.02142206	;
    RnonCO2[	94	]=	-0.022552345	;
    RnonCO2[	95	]=	-0.031588211	;
    RnonCO2[	96	]=	-0.046252985	;
    RnonCO2[	97	]=	-0.042365448	;
    RnonCO2[	98	]=	-0.033665717	;
    RnonCO2[	99	]=	-0.034555638	;
    RnonCO2[	100	]=	-0.037298509	;
    RnonCO2[	101	]=	-0.046083195	;
    RnonCO2[	102	]=	-0.05558168	;
    RnonCO2[	103	]=	-0.057464266	;
    RnonCO2[	104	]=	-0.053606449	;
    RnonCO2[	105	]=	-0.05168834	;
    RnonCO2[	106	]=	-0.061304945	;
    RnonCO2[	107	]=	-0.086608149	;
    RnonCO2[	108	]=	-0.108195137	;
    RnonCO2[	109	]=	-0.112259625	;
    RnonCO2[	110	]=	-0.111643119	;
    RnonCO2[	111	]=	-0.108991576	;
    RnonCO2[	112	]=	-0.10203489	;
    RnonCO2[	113	]=	-0.098533227	;
    RnonCO2[	114	]=	-0.101949678	;
    RnonCO2[	115	]=	-0.114562053	;
    RnonCO2[	116	]=	-0.123024156	;
    RnonCO2[	117	]=	-0.119809652	;
    RnonCO2[	118	]=	-0.118941351	;
    RnonCO2[	119	]=	-0.120181779	;
    RnonCO2[	120	]=	-0.117316831	;
    RnonCO2[	121	]=	-0.115023243	;
    RnonCO2[	122	]=	-0.116983521	;
    RnonCO2[	123	]=	-0.125997633	;
    RnonCO2[	124	]=	-0.129568227	;
    RnonCO2[	125	]=	-0.131248987	;
    RnonCO2[	126	]=	-0.137316696	;
    RnonCO2[	127	]=	-0.141273792	;
    RnonCO2[	128	]=	-0.144231375	;
    RnonCO2[	129	]=	-0.147973341	;
    RnonCO2[	130	]=	-0.153688224	;
    RnonCO2[	131	]=	-0.15652816	;
    RnonCO2[	132	]=	-0.157109013	;
    RnonCO2[	133	]=	-0.158858513	;
    RnonCO2[	134	]=	-0.162186704	;
    RnonCO2[	135	]=	-0.165318228	;
    RnonCO2[	136	]=	-0.171733193	;
    RnonCO2[	137	]=	-0.178096644	;
    RnonCO2[	138	]=	-0.183870407	;
    RnonCO2[	139	]=	-0.189730028	;
    RnonCO2[	140	]=	-0.194221589	;
    RnonCO2[	141	]=	-0.201576481	;
    RnonCO2[	142	]=	-0.208859035	;
    RnonCO2[	143	]=	-0.207375441	;
    RnonCO2[	144	]=	-0.202571711	;
    RnonCO2[	145	]=	-0.204126848	;
    RnonCO2[	146	]=	-0.201876709	;
    RnonCO2[	147	]=	-0.195147841	;
    RnonCO2[	148	]=	-0.191449584	;
    RnonCO2[	149	]=	-0.183770215	;
    RnonCO2[	150	]=	-0.173476305	;
    RnonCO2[	151	]=	-0.170909799	;
    RnonCO2[	152	]=	-0.171515561	;
    RnonCO2[	153	]=	-0.169079801	;
    RnonCO2[	154	]=	-0.159414516	;
    RnonCO2[	155	]=	-0.155013584	;
    RnonCO2[	156	]=	-0.151089869	;
    RnonCO2[	157	]=	-0.144432651	;
    RnonCO2[	158	]=	-0.152359798	;
    RnonCO2[	159	]=	-0.158148636	;
    RnonCO2[	160	]=	-0.155277608	;
    RnonCO2[	161	]=	-0.152617947	;
    RnonCO2[	162	]=	-0.154120923	;
    RnonCO2[	163	]=	-0.156019482	;
    RnonCO2[	164	]=	-0.155886394	;
    RnonCO2[	165	]=	-0.152021034	;
    RnonCO2[	166	]=	-0.141834688	;
    RnonCO2[	167	]=	-0.13108946	;
    RnonCO2[	168	]=	-0.125408882	;
    RnonCO2[	169	]=	-0.125515632	;
    RnonCO2[	170	]=	-0.125194614	;
    RnonCO2[	171	]=	-0.126120729	;
    RnonCO2[	172	]=	-0.12532345	;
    RnonCO2[	173	]=	-0.115303554	;
    RnonCO2[	174	]=	-0.10787249	;
    RnonCO2[	175	]=	-0.109257271	;
    RnonCO2[	176	]=	-0.113724728	;
    RnonCO2[	177	]=	-0.117421621	;
    RnonCO2[	178	]=	-0.11813163	;
    RnonCO2[	179	]=	-0.119629474	;
    RnonCO2[	180	]=	-0.119789192	;
    RnonCO2[	181	]=	-0.124270593	;
    RnonCO2[	182	]=	-0.135220142	;
    RnonCO2[	183	]=	-0.145282294	;
    RnonCO2[	184	]=	-0.1490379	;
    RnonCO2[	185	]=	-0.154219559	;
    RnonCO2[	186	]=	-0.178760049	;
    RnonCO2[	187	]=	-0.203310188	;
    RnonCO2[	188	]=	-0.212758852	;
    RnonCO2[	189	]=	-0.221219612	;
    RnonCO2[	190	]=	-0.230341307	;
    RnonCO2[	191	]=	-0.239337924	;
    RnonCO2[	192	]=	-0.247524974	;
    RnonCO2[	193	]=	-0.250847198	;
    RnonCO2[	194	]=	-0.245200212	;
    RnonCO2[	195	]=	-0.237476895	;
    RnonCO2[	196	]=	-0.24791739	;
    RnonCO2[	197	]=	-0.261719365	;
    RnonCO2[	198	]=	-0.263179809	;
    RnonCO2[	199	]=	-0.263298027	;
    RnonCO2[	200	]=	-0.261139159	;
    RnonCO2[	201	]=	-0.257781723	;
    RnonCO2[	202	]=	-0.255724839	;
    RnonCO2[	203	]=	-0.25169763	;
    RnonCO2[	204	]=	-0.241647721	;
    RnonCO2[	205	]=	-0.221452127	;
    RnonCO2[	206	]=	-0.196790013	;
    RnonCO2[	207	]=	-0.182093765	;
    RnonCO2[	208	]=	-0.174745229	;
    RnonCO2[	209	]=	-0.16714548	;
    RnonCO2[	210	]=	-0.159384584	;
    RnonCO2[	211	]=	-0.156373877	;
    RnonCO2[	212	]=	-0.153042015	;
    RnonCO2[	213	]=	-0.139817455	;
    RnonCO2[	214	]=	-0.125210902	;
    RnonCO2[	215	]=	-0.119848303	;
    RnonCO2[	216	]=	-0.119430668	;
    RnonCO2[	217	]=	-0.123211365	;
    RnonCO2[	218	]=	-0.125809708	;
    RnonCO2[	219	]=	-0.120193155	;
    RnonCO2[	220	]=	-0.110911249	;
    RnonCO2[	221	]=	-0.102402229	;
    RnonCO2[	222	]=	-0.096562536	;
    RnonCO2[	223	]=	-0.086930587	;
    RnonCO2[	224	]=	-0.072710932	;
    RnonCO2[	225	]=	-0.05997191	;
    RnonCO2[	226	]=	-0.058006544	;
    RnonCO2[	227	]=	-0.065604043	;
    RnonCO2[	228	]=	-0.067949961	;
    RnonCO2[	229	]=	-0.065994527	;
    RnonCO2[	230	]=	-0.058283284	;
    RnonCO2[	231	]=	-0.04756956	;
    RnonCO2[	232	]=	-0.049208339	;
    RnonCO2[	233	]=	-0.056699684	;
    RnonCO2[	234	]=	-0.059531967	;
    RnonCO2[	235	]=	-0.065638041	;
    RnonCO2[	236	]=	-0.069976074	;
    RnonCO2[	237	]=	-0.065660736	;
    RnonCO2[	238	]=	-0.060876733	;
    RnonCO2[	239	]=	-0.057486962	;
    RnonCO2[	240	]=	-0.051265033	;
    RnonCO2[	241	]=	-0.037670076	;
    RnonCO2[	242	]=	-0.019893764	;
    RnonCO2[	243	]=	-0.000823001	;
    RnonCO2[	244	]=	0.019502902	;
    RnonCO2[	245	]=	0.037441814	;
    RnonCO2[	246	]=	0.049417144	;
    RnonCO2[	247	]=	0.059768113	;
    RnonCO2[	248	]=	0.071444345	;
    RnonCO2[	249	]=	0.083442351	;
    RnonCO2[	250	]=	0.096022256	;
    RnonCO2[	251	]=	0.10916019	;
    RnonCO2[	252	]=	0.122811854	;
    RnonCO2[	253	]=	0.13693036	;
    RnonCO2[	254	]=	0.15204114	;
    RnonCO2[	255	]=	0.168591967	;
    RnonCO2[	256	]=	0.186766657	;
    RnonCO2[	257	]=	0.205173036	;
    RnonCO2[	258	]=	0.222866088	;
    RnonCO2[	259	]=	0.240576956	;
    RnonCO2[	260	]=	0.258326044	;
    RnonCO2[	261	]=	0.276116517	;
    RnonCO2[	262	]=	0.29394187	;
    RnonCO2[	263	]=	0.311780272	;
    RnonCO2[	264	]=	0.329615086	;
    RnonCO2[	265	]=	0.348986214	;
    RnonCO2[	266	]=	0.37142882	;
    RnonCO2[	267	]=	0.394872321	;
    RnonCO2[	268	]=	0.417926425	;
    RnonCO2[	269	]=	0.441218554	;
    RnonCO2[	270	]=	0.464702083	;
    RnonCO2[	271	]=	0.488333479	;
    RnonCO2[	272	]=	0.512075403	;
    RnonCO2[	273	]=	0.535880676	;
    RnonCO2[	274	]=	0.559692195	;
    RnonCO2[	275	]=	0.583094257	;
    RnonCO2[	276	]=	0.60567867	;
    RnonCO2[	277	]=	0.627987231	;
    RnonCO2[	278	]=	0.650367412	;
    RnonCO2[	279	]=	0.672650114	;
    RnonCO2[	280	]=	0.69490502	;
    RnonCO2[	281	]=	0.717184654	;
    RnonCO2[	282	]=	0.739526114	;
    RnonCO2[	283	]=	0.761955068	;
    RnonCO2[	284	]=	0.784475878	;
    RnonCO2[	285	]=	0.805983293	;
    RnonCO2[	286	]=	0.825382732	;
    RnonCO2[	287	]=	0.843811364	;
    RnonCO2[	288	]=	0.862164274	;
    RnonCO2[	289	]=	0.88017297	;
    RnonCO2[	290	]=	0.897837466	;
    RnonCO2[	291	]=	0.915228374	;
    RnonCO2[	292	]=	0.932404856	;
    RnonCO2[	293	]=	0.949417839	;
    RnonCO2[	294	]=	0.966318914	;
    RnonCO2[	295	]=	0.982819486	;
    RnonCO2[	296	]=	0.998609085	;
    RnonCO2[	297	]=	1.013870109	;
    RnonCO2[	298	]=	1.028718413	;
    RnonCO2[	299	]=	1.043105084	;
    RnonCO2[	300	]=	1.057012563	;
    RnonCO2[	301	]=	1.070443452	;
    RnonCO2[	302	]=	1.083474934	;
    RnonCO2[	303	]=	1.096173716	;
    RnonCO2[	304	]=	1.108598174	;
    RnonCO2[	305	]=	1.12060237	;
    RnonCO2[	306	]=	1.132009413	;
    RnonCO2[	307	]=	1.143129004	;
    RnonCO2[	308	]=	1.154205373	;
    RnonCO2[	309	]=	1.165181375	;
    RnonCO2[	310	]=	1.176063128	;
    RnonCO2[	311	]=	1.186799276	;
    RnonCO2[	312	]=	1.197361335	;
    RnonCO2[	313	]=	1.20780055	;
    RnonCO2[	314	]=	1.218161605	;
    RnonCO2[	315	]=	1.229404767	;
    RnonCO2[	316	]=	1.242484643	;
    RnonCO2[	317	]=	1.256439183	;
    RnonCO2[	318	]=	1.270393379	;
    RnonCO2[	319	]=	1.284434844	;
    RnonCO2[	320	]=	1.298554619	;
    RnonCO2[	321	]=	1.312745936	;
    RnonCO2[	322	]=	1.326943443	;
    RnonCO2[	323	]=	1.341105165	;
    RnonCO2[	324	]=	1.35527369	;
    RnonCO2[	325	]=	1.368587592	;
    RnonCO2[	326	]=	1.3801878	;
    RnonCO2[	327	]=	1.390971947	;
    RnonCO2[	328	]=	1.401644792	;
    RnonCO2[	329	]=	1.41204733	;
    RnonCO2[	330	]=	1.422189875	;
    RnonCO2[	331	]=	1.432089402	;
    RnonCO2[	332	]=	1.441762212	;
    RnonCO2[	333	]=	1.451165105	;
    RnonCO2[	334	]=	1.460274813	;
    RnonCO2[	335	]=	1.468374652	;
    RnonCO2[	336	]=	1.474746424	;
    RnonCO2[	337	]=	1.480301476	;
    RnonCO2[	338	]=	1.485731533	;
    RnonCO2[	339	]=	1.490849211	;
    RnonCO2[	340	]=	1.495659068	;
    RnonCO2[	341	]=	1.500169653	;
    RnonCO2[	342	]=	1.504397544	;
    RnonCO2[	343	]=	1.508359214	;
    RnonCO2[	344	]=	1.512009834	;
    RnonCO2[	345	]=	1.51532465	;
    RnonCO2[	346	]=	1.518367002	;
    RnonCO2[	347	]=	1.521195686	;
    RnonCO2[	348	]=	1.523860963	;
    RnonCO2[	349	]=	1.52641356	;
    RnonCO2[	350	]=	1.528871707	;
    RnonCO2[	351	]=	1.531222737	;
    RnonCO2[	352	]=	1.533460345	;
    RnonCO2[	353	]=	1.535587165	;
    RnonCO2[	354	]=	1.537605959	;
    RnonCO2[	355	]=	1.539459965	;
    RnonCO2[	356	]=	1.541113783	;
    RnonCO2[	357	]=	1.542618783	;
    RnonCO2[	358	]=	1.544025951	;
    RnonCO2[	359	]=	1.545378349	;
    RnonCO2[	360	]=	1.546718751	;
    RnonCO2[	361	]=	1.548055941	;
    RnonCO2[	362	]=	1.549369713	;
    RnonCO2[	363	]=	1.550646585	;
    RnonCO2[	364	]=	1.551882676	;
    RnonCO2[	365	]=	1.553075063	;
    RnonCO2[	366	]=	1.554161867	;
    RnonCO2[	367	]=	1.555101796	;
    RnonCO2[	368	]=	1.555942834	;
    RnonCO2[	369	]=	1.556732334	;
    RnonCO2[	370	]=	1.557509895	;
    RnonCO2[	371	]=	1.558316771	;
    RnonCO2[	372	]=	1.559159287	;
    RnonCO2[	373	]=	1.560016734	;
    RnonCO2[	374	]=	1.560870324	;
    RnonCO2[	375	]=	1.561714212	;
    RnonCO2[	376	]=	1.562542262	;
    RnonCO2[	377	]=	1.563290939	;
    RnonCO2[	378	]=	1.563916226	;
    RnonCO2[	379	]=	1.564463579	;
    RnonCO2[	380	]=	1.564979531	;
    RnonCO2[	381	]=	1.565503311	;
    RnonCO2[	382	]=	1.566075173	;
    RnonCO2[	383	]=	1.566700798	;
    RnonCO2[	384	]=	1.567356549	;
    RnonCO2[	385	]=	1.568023908	;
    RnonCO2[	386	]=	1.568693378	;
    RnonCO2[	387	]=	1.569361956	;
    RnonCO2[	388	]=	1.569966223	;
    RnonCO2[	389	]=	1.570460069	;
    RnonCO2[	390	]=	1.570886164	;
    RnonCO2[	391	]=	1.57129172	;
    RnonCO2[	392	]=	1.571714342	;
    RnonCO2[	393	]=	1.572194218	;
    RnonCO2[	394	]=	1.572736425	;
    RnonCO2[	395	]=	1.573316462	;
    RnonCO2[	396	]=	1.573913961	;
    RnonCO2[	397	]=	1.574518489	;
    RnonCO2[	398	]=	1.575122951	;
    RnonCO2[	399	]=	1.575661885	;
    RnonCO2[	400	]=	1.576087712	;
    RnonCO2[	401	]=	1.576445094	;
    RnonCO2[	402	]=	1.576780719	;
    RnonCO2[	403	]=	1.577133646	;
    RnonCO2[	404	]=	1.577543971	;
    RnonCO2[	405	]=	1.57801753	;
    RnonCO2[	406	]=	1.578529885	;
    RnonCO2[	407	]=	1.57905987	;
    RnonCO2[	408	]=	1.579596243	;
    RnonCO2[	409	]=	1.5801324	;
    RnonCO2[	410	]=	1.580602658	;
    RnonCO2[	411	]=	1.580959115	;
    RnonCO2[	412	]=	1.581246127	;
    RnonCO2[	413	]=	1.581511182	;
    RnonCO2[	414	]=	1.581793935	;
    RnonCO2[	415	]=	1.582134962	;
    RnonCO2[	416	]=	1.582539659	;
    RnonCO2[	417	]=	1.582982927	;
    RnonCO2[	418	]=	1.58344402	;
    RnonCO2[	419	]=	1.583911568	;
    RnonCO2[	420	]=	1.584378025	;
    RnonCO2[	421	]=	1.584777859	;
    RnonCO2[	422	]=	1.585063744	;
    RnonCO2[	423	]=	1.585280055	;
    RnonCO2[	424	]=	1.585474046	;
    RnonCO2[	425	]=	1.585685963	;
    RnonCO2[	426	]=	1.585956484	;
    RnonCO2[	427	]=	1.586291104	;
    RnonCO2[	428	]=	1.58666443	;
    RnonCO2[	429	]=	1.587054917	;
    RnonCO2[	430	]=	1.587451207	;
    RnonCO2[	431	]=	1.587845901	;
    RnonCO2[	432	]=	1.588173799	;
    RnonCO2[	433	]=	1.588387781	;
    RnonCO2[	434	]=	1.588531449	;
    RnonCO2[	435	]=	1.588653082	;
    RnonCO2[	436	]=	1.588792336	;
    RnonCO2[	437	]=	1.58898945	;
    RnonCO2[	438	]=	1.589250031	;
    RnonCO2[	439	]=	1.589548396	;
    RnonCO2[	440	]=	1.589863	;
    RnonCO2[	441	]=	1.590182435	;
    RnonCO2[	442	]=	1.59049929	;
    RnonCO2[	443	]=	1.590748191	;
    RnonCO2[	444	]=	1.590882222	;
    RnonCO2[	445	]=	1.59094589	;
    RnonCO2[	446	]=	1.590987615	;
    RnonCO2[	447	]=	1.591046945	;
    RnonCO2[	448	]=	1.591164675	;
    RnonCO2[	449	]=	1.591345419	;
    RnonCO2[	450	]=	1.591563876	;
    RnonCO2[	451	]=	1.591798229	;
    RnonCO2[	452	]=	1.592037046	;
    RnonCO2[	453	]=	1.592273621	;
    RnonCO2[	454	]=	1.59244202	;
    RnonCO2[	455	]=	1.59249612	;
    RnonCO2[	456	]=	1.5924806	;
    RnonCO2[	457	]=	1.592443478	;
    RnonCO2[	458	]=	1.592425075	;
    RnonCO2[	459	]=	1.592465084	;
    RnonCO2[	460	]=	1.592568139	;
    RnonCO2[	461	]=	1.592708986	;
    RnonCO2[	462	]=	1.592865552	;
    RnonCO2[	463	]=	1.593026493	;
    RnonCO2[	464	]=	1.593185171	;
    RnonCO2[	465	]=	1.593275772	;
    RnonCO2[	466	]=	1.593252631	;
    RnonCO2[	467	]=	1.593160152	;
    RnonCO2[	468	]=	1.593046813	;
    RnonCO2[	469	]=	1.592952255	;
    RnonCO2[	470	]=	1.59291659	;
    RnonCO2[	471	]=	1.592944036	;
    RnonCO2[	472	]=	1.593009538	;
    RnonCO2[	473	]=	1.593090652	;
    RnonCO2[	474	]=	1.593176179	;
    RnonCO2[	475	]=	1.593259269	;
    RnonCO2[	476	]=	1.593274915	;
    RnonCO2[	477	]=	1.593177524	;
    RnonCO2[	478	]=	1.593011652	;
    RnonCO2[	479	]=	1.592825679	;
    RnonCO2[	480	]=	1.592659324	;
    RnonCO2[	481	]=	1.592551739	;
    RnonCO2[	482	]=	1.592508042	;
    RnonCO2[	483	]=	1.592502118	;
    RnonCO2[	484	]=	1.592512302	;
    RnonCO2[	485	]=	1.592519077	;
    RnonCO2[	486	]=	1.592519077	;
    RnonCO2[	487	]=	1.592519077	;
    RnonCO2[	488	]=	1.592519077	;
    RnonCO2[	489	]=	1.592519077	;
    RnonCO2[	490	]=	1.592519077	;
    RnonCO2[	491	]=	1.592519077	;
    RnonCO2[	492	]=	1.592519077	;
    RnonCO2[	493	]=	1.592519077	;
    RnonCO2[	494	]=	1.592519077	;
    RnonCO2[	495	]=	1.592519077	;
    RnonCO2[	496	]=	1.592519077	;
    RnonCO2[	497	]=	1.592519077	;
    RnonCO2[	498	]=	1.592519077	;
    RnonCO2[	499	]=	1.592519077	;
    RnonCO2[	500	]=	1.592519077	;
    RnonCO2[	501	]=	1.592519077	;
    RnonCO2[	502	]=	1.592519077	;
    RnonCO2[	503	]=	1.592519077	;
    RnonCO2[	504	]=	1.592519077	;
    RnonCO2[	505	]=	1.592519077	;
    RnonCO2[	506	]=	1.592519077	;
    RnonCO2[	507	]=	1.592519077	;
    RnonCO2[	508	]=	1.592519077	;
    RnonCO2[	509	]=	1.592519077	;
    RnonCO2[	510	]=	1.592519077	;
    RnonCO2[	511	]=	1.592519077	;
    RnonCO2[	512	]=	1.592519077	;
    RnonCO2[	513	]=	1.592519077	;
    RnonCO2[	514	]=	1.592519077	;
    RnonCO2[	515	]=	1.592519077	;
    RnonCO2[	516	]=	1.592519077	;
    RnonCO2[	517	]=	1.592519077	;
    RnonCO2[	518	]=	1.592519077	;
    RnonCO2[	519	]=	1.592519077	;
    RnonCO2[	520	]=	1.592519077	;
    RnonCO2[	521	]=	1.592519077	;
    RnonCO2[	522	]=	1.592519077	;
    RnonCO2[	523	]=	1.592519077	;
    RnonCO2[	524	]=	1.592519077	;
    RnonCO2[	525	]=	1.592519077	;
    RnonCO2[	526	]=	1.592519077	;
    RnonCO2[	527	]=	1.592519077	;
    RnonCO2[	528	]=	1.592519077	;
    RnonCO2[	529	]=	1.592519077	;
    RnonCO2[	530	]=	1.592519077	;
    RnonCO2[	531	]=	1.592519077	;
    RnonCO2[	532	]=	1.592519077	;
    RnonCO2[	533	]=	1.592519077	;
    RnonCO2[	534	]=	1.592519077	;
    RnonCO2[	535	]=	1.592519077	;
    RnonCO2[	536	]=	1.592519077	;
    RnonCO2[	537	]=	1.592519077	;
    RnonCO2[	538	]=	1.592519077	;
    RnonCO2[	539	]=	1.592519077	;
    RnonCO2[	540	]=	1.592519077	;
    RnonCO2[	541	]=	1.592519077	;
    RnonCO2[	542	]=	1.592519077	;
    RnonCO2[	543	]=	1.592519077	;
    RnonCO2[	544	]=	1.592519077	;
    RnonCO2[	545	]=	1.592519077	;
    RnonCO2[	546	]=	1.592519077	;
    RnonCO2[	547	]=	1.592519077	;
    RnonCO2[	548	]=	1.592519077	;
    RnonCO2[	549	]=	1.592519077	;
    RnonCO2[	550	]=	1.592519077	;
    RnonCO2[	551	]=	1.592519077	;
    RnonCO2[	552	]=	1.592519077	;
    RnonCO2[	553	]=	1.592519077	;
    RnonCO2[	554	]=	1.592519077	;
    RnonCO2[	555	]=	1.592519077	;
    RnonCO2[	556	]=	1.592519077	;
    RnonCO2[	557	]=	1.592519077	;
    RnonCO2[	558	]=	1.592519077	;
    RnonCO2[	559	]=	1.592519077	;
    RnonCO2[	560	]=	1.592519077	;
    RnonCO2[	561	]=	1.592519077	;
    RnonCO2[	562	]=	1.592519077	;
    RnonCO2[	563	]=	1.592519077	;
    RnonCO2[	564	]=	1.592519077	;
    RnonCO2[	565	]=	1.592519077	;
    RnonCO2[	566	]=	1.592519077	;
    RnonCO2[	567	]=	1.592519077	;
    RnonCO2[	568	]=	1.592519077	;
    RnonCO2[	569	]=	1.592519077	;
    RnonCO2[	570	]=	1.592519077	;
    RnonCO2[	571	]=	1.592519077	;
    RnonCO2[	572	]=	1.592519077	;
    RnonCO2[	573	]=	1.592519077	;
    RnonCO2[	574	]=	1.592519077	;
    RnonCO2[	575	]=	1.592519077	;
    RnonCO2[	576	]=	1.592519077	;
    RnonCO2[	577	]=	1.592519077	;
    RnonCO2[	578	]=	1.592519077	;
    RnonCO2[	579	]=	1.592519077	;
    RnonCO2[	580	]=	1.592519077	;
    RnonCO2[	581	]=	1.592519077	;
    RnonCO2[	582	]=	1.592519077	;
    RnonCO2[	583	]=	1.592519077	;
    RnonCO2[	584	]=	1.592519077	;
    RnonCO2[	585	]=	1.592519077	;
    RnonCO2[	586	]=	1.592519077	;
    RnonCO2[	587	]=	1.592519077	;
    RnonCO2[	588	]=	1.592519077	;
    RnonCO2[	589	]=	1.592519077	;
    RnonCO2[	590	]=	1.592519077	;
    RnonCO2[	591	]=	1.592519077	;
    RnonCO2[	592	]=	1.592519077	;
    RnonCO2[	593	]=	1.592519077	;
    RnonCO2[	594	]=	1.592519077	;
    RnonCO2[	595	]=	1.592519077	;
    RnonCO2[	596	]=	1.592519077	;
    RnonCO2[	597	]=	1.592519077	;
    RnonCO2[	598	]=	1.592519077	;
    RnonCO2[	599	]=	1.592519077	;
    RnonCO2[	600	]=	1.592519077	;
    RnonCO2[	601	]=	1.592519077	;
    RnonCO2[	602	]=	1.592519077	;
    RnonCO2[	603	]=	1.592519077	;
    RnonCO2[	604	]=	1.592519077	;
    RnonCO2[	605	]=	1.592519077	;
    RnonCO2[	606	]=	1.592519077	;
    RnonCO2[	607	]=	1.592519077	;
    RnonCO2[	608	]=	1.592519077	;
    RnonCO2[	609	]=	1.592519077	;
    RnonCO2[	610	]=	1.592519077	;
    RnonCO2[	611	]=	1.592519077	;
    RnonCO2[	612	]=	1.592519077	;
    RnonCO2[	613	]=	1.592519077	;
    RnonCO2[	614	]=	1.592519077	;
    RnonCO2[	615	]=	1.592519077	;
    RnonCO2[	616	]=	1.592519077	;
    RnonCO2[	617	]=	1.592519077	;
    RnonCO2[	618	]=	1.592519077	;
    RnonCO2[	619	]=	1.592519077	;
    RnonCO2[	620	]=	1.592519077	;
    RnonCO2[	621	]=	1.592519077	;
    RnonCO2[	622	]=	1.592519077	;
    RnonCO2[	623	]=	1.592519077	;
    RnonCO2[	624	]=	1.592519077	;
    RnonCO2[	625	]=	1.592519077	;
    RnonCO2[	626	]=	1.592519077	;
    RnonCO2[	627	]=	1.592519077	;
    RnonCO2[	628	]=	1.592519077	;
    RnonCO2[	629	]=	1.592519077	;
    RnonCO2[	630	]=	1.592519077	;
    RnonCO2[	631	]=	1.592519077	;
    RnonCO2[	632	]=	1.592519077	;
    RnonCO2[	633	]=	1.592519077	;
    RnonCO2[	634	]=	1.592519077	;
    RnonCO2[	635	]=	1.592519077	;
    RnonCO2[	636	]=	1.592519077	;
    RnonCO2[	637	]=	1.592519077	;
    RnonCO2[	638	]=	1.592519077	;
    RnonCO2[	639	]=	1.592519077	;
    RnonCO2[	640	]=	1.592519077	;
    RnonCO2[	641	]=	1.592519077	;
    RnonCO2[	642	]=	1.592519077	;
    RnonCO2[	643	]=	1.592519077	;
    RnonCO2[	644	]=	1.592519077	;
    RnonCO2[	645	]=	1.592519077	;
    RnonCO2[	646	]=	1.592519077	;
    RnonCO2[	647	]=	1.592519077	;
    RnonCO2[	648	]=	1.592519077	;
    RnonCO2[	649	]=	1.592519077	;
    RnonCO2[	650	]=	1.592519077	;
    RnonCO2[	651	]=	1.592519077	;
    RnonCO2[	652	]=	1.592519077	;
    RnonCO2[	653	]=	1.592519077	;
    RnonCO2[	654	]=	1.592519077	;
    RnonCO2[	655	]=	1.592519077	;
    RnonCO2[	656	]=	1.592519077	;
    RnonCO2[	657	]=	1.592519077	;
    RnonCO2[	658	]=	1.592519077	;
    RnonCO2[	659	]=	1.592519077	;
    RnonCO2[	660	]=	1.592519077	;
    RnonCO2[	661	]=	1.592519077	;
    RnonCO2[	662	]=	1.592519077	;
    RnonCO2[	663	]=	1.592519077	;
    RnonCO2[	664	]=	1.592519077	;
    RnonCO2[	665	]=	1.592519077	;
    RnonCO2[	666	]=	1.592519077	;
    RnonCO2[	667	]=	1.592519077	;
    RnonCO2[	668	]=	1.592519077	;
    RnonCO2[	669	]=	1.592519077	;
    RnonCO2[	670	]=	1.592519077	;
    RnonCO2[	671	]=	1.592519077	;
    RnonCO2[	672	]=	1.592519077	;
    RnonCO2[	673	]=	1.592519077	;
    RnonCO2[	674	]=	1.592519077	;
    RnonCO2[	675	]=	1.592519077	;
    RnonCO2[	676	]=	1.592519077	;
    RnonCO2[	677	]=	1.592519077	;
    RnonCO2[	678	]=	1.592519077	;
    RnonCO2[	679	]=	1.592519077	;
    RnonCO2[	680	]=	1.592519077	;
    RnonCO2[	681	]=	1.592519077	;
    RnonCO2[	682	]=	1.592519077	;
    RnonCO2[	683	]=	1.592519077	;
    RnonCO2[	684	]=	1.592519077	;
    RnonCO2[	685	]=	1.592519077	;
    RnonCO2[	686	]=	1.592519077	;
    RnonCO2[	687	]=	1.592519077	;
    RnonCO2[	688	]=	1.592519077	;
    RnonCO2[	689	]=	1.592519077	;
    RnonCO2[	690	]=	1.592519077	;
    RnonCO2[	691	]=	1.592519077	;
    RnonCO2[	692	]=	1.592519077	;
    RnonCO2[	693	]=	1.592519077	;
    RnonCO2[	694	]=	1.592519077	;
    RnonCO2[	695	]=	1.592519077	;
    RnonCO2[	696	]=	1.592519077	;
    RnonCO2[	697	]=	1.592519077	;
    RnonCO2[	698	]=	1.592519077	;
    RnonCO2[	699	]=	1.592519077	;
    RnonCO2[	700	]=	1.592519077	;
    RnonCO2[	701	]=	1.592519077	;
    RnonCO2[	702	]=	1.592519077	;
    RnonCO2[	703	]=	1.592519077	;
    RnonCO2[	704	]=	1.592519077	;
    RnonCO2[	705	]=	1.592519077	;
    RnonCO2[	706	]=	1.592519077	;
    RnonCO2[	707	]=	1.592519077	;
    RnonCO2[	708	]=	1.592519077	;
    RnonCO2[	709	]=	1.592519077	;
    RnonCO2[	710	]=	1.592519077	;
    RnonCO2[	711	]=	1.592519077	;
    RnonCO2[	712	]=	1.592519077	;
    RnonCO2[	713	]=	1.592519077	;
    RnonCO2[	714	]=	1.592519077	;
    RnonCO2[	715	]=	1.592519077	;
    RnonCO2[	716	]=	1.592519077	;
    RnonCO2[	717	]=	1.592519077	;
    RnonCO2[	718	]=	1.592519077	;
    RnonCO2[	719	]=	1.592519077	;
    RnonCO2[	720	]=	1.592519077	;
    RnonCO2[	721	]=	1.592519077	;
    RnonCO2[	722	]=	1.592519077	;
    RnonCO2[	723	]=	1.592519077	;
    RnonCO2[	724	]=	1.592519077	;
    RnonCO2[	725	]=	1.592519077	;
    RnonCO2[	726	]=	1.592519077	;
    RnonCO2[	727	]=	1.592519077	;
    RnonCO2[	728	]=	1.592519077	;
    RnonCO2[	729	]=	1.592519077	;
    RnonCO2[	730	]=	1.592519077	;
    RnonCO2[	731	]=	1.592519077	;
    RnonCO2[	732	]=	1.592519077	;
    RnonCO2[	733	]=	1.592519077	;
    RnonCO2[	734	]=	1.592519077	;
    RnonCO2[	735	]=	1.592519077	;
    
    
    for(int i = 736; i<tmax; i++)
    {
        RnonCO2[i] = RnonCO2[i-1];
    }
    
}

void setRnonCO2_RCP3PD_K(void)
{
    RnonCO2[	0	]=	-0.019988515	;
    RnonCO2[	1	]=	-0.005612142	;
    RnonCO2[	2	]=	0.008971239	;
    RnonCO2[	3	]=	0.009466172	;
    RnonCO2[	4	]=	0.009946833	;
    RnonCO2[	5	]=	0.010412166	;
    RnonCO2[	6	]=	0.010865438	;
    RnonCO2[	7	]=	0.011309666	;
    RnonCO2[	8	]=	0.011745593	;
    RnonCO2[	9	]=	0.012173039	;
    RnonCO2[	10	]=	0.012591208	;
    RnonCO2[	11	]=	0.012999626	;
    RnonCO2[	12	]=	0.013397144	;
    RnonCO2[	13	]=	0.013783559	;
    RnonCO2[	14	]=	0.014157995	;
    RnonCO2[	15	]=	0.014518817	;
    RnonCO2[	16	]=	0.014861789	;
    RnonCO2[	17	]=	0.015185328	;
    RnonCO2[	18	]=	0.015493438	;
    RnonCO2[	19	]=	0.015792171	;
    RnonCO2[	20	]=	0.01608699	;
    RnonCO2[	21	]=	0.016384013	;
    RnonCO2[	22	]=	0.016688621	;
    RnonCO2[	23	]=	0.017006739	;
    RnonCO2[	24	]=	0.017344141	;
    RnonCO2[	25	]=	0.017705767	;
    RnonCO2[	26	]=	0.018092359	;
    RnonCO2[	27	]=	0.018501885	;
    RnonCO2[	28	]=	0.01893258	;
    RnonCO2[	29	]=	0.019382558	;
    RnonCO2[	30	]=	0.019850303	;
    RnonCO2[	31	]=	0.020334085	;
    RnonCO2[	32	]=	0.020831989	;
    RnonCO2[	33	]=	0.021342706	;
    RnonCO2[	34	]=	0.02186457	;
    RnonCO2[	35	]=	0.022398729	;
    RnonCO2[	36	]=	0.022952686	;
    RnonCO2[	37	]=	0.023532049	;
    RnonCO2[	38	]=	0.024135881	;
    RnonCO2[	39	]=	0.024759327	;
    RnonCO2[	40	]=	0.025397837	;
    RnonCO2[	41	]=	0.02604713	;
    RnonCO2[	42	]=	0.026702498	;
    RnonCO2[	43	]=	0.02735936	;
    RnonCO2[	44	]=	0.028013477	;
    RnonCO2[	45	]=	0.028662413	;
    RnonCO2[	46	]=	0.029309198	;
    RnonCO2[	47	]=	0.029958961	;
    RnonCO2[	48	]=	0.030612845	;
    RnonCO2[	49	]=	0.031270152	;
    RnonCO2[	50	]=	0.03193112	;
    RnonCO2[	51	]=	0.032594644	;
    RnonCO2[	52	]=	0.033260857	;
    RnonCO2[	53	]=	0.033928967	;
    RnonCO2[	54	]=	0.034598246	;
    RnonCO2[	55	]=	0.035268641	;
    RnonCO2[	56	]=	0.035940357	;
    RnonCO2[	57	]=	0.036613717	;
    RnonCO2[	58	]=	0.037289228	;
    RnonCO2[	59	]=	0.037967253	;
    RnonCO2[	60	]=	0.038647628	;
    RnonCO2[	61	]=	0.039330893	;
    RnonCO2[	62	]=	0.040017299	;
    RnonCO2[	63	]=	0.040707068	;
    RnonCO2[	64	]=	0.041400385	;
    RnonCO2[	65	]=	0.042096128	;
    RnonCO2[	66	]=	0.042791515	;
    RnonCO2[	67	]=	0.04348473	;
    RnonCO2[	68	]=	0.044177707	;
    RnonCO2[	69	]=	0.044873365	;
    RnonCO2[	70	]=	0.045574528	;
    RnonCO2[	71	]=	0.04628438	;
    RnonCO2[	72	]=	0.047005924	;
    RnonCO2[	73	]=	0.047742202	;
    RnonCO2[	74	]=	0.048496247	;
    RnonCO2[	75	]=	0.049273302	;
    RnonCO2[	76	]=	0.050090919	;
    RnonCO2[	77	]=	0.050966437	;
    RnonCO2[	78	]=	0.051881638	;
    RnonCO2[	79	]=	0.052787675	;
    RnonCO2[	80	]=	0.053629807	;
    RnonCO2[	81	]=	0.054384454	;
    RnonCO2[	82	]=	0.055079735	;
    RnonCO2[	83	]=	0.055748092	;
    RnonCO2[	84	]=	0.056393965	;
    RnonCO2[	85	]=	0.057093398	;
    RnonCO2[	86	]=	0.05807606	;
    RnonCO2[	87	]=	0.059274862	;
    RnonCO2[	88	]=	0.060472717	;
    RnonCO2[	89	]=	0.061669627	;
    RnonCO2[	90	]=	0.062723294	;
    RnonCO2[	91	]=	0.063776572	;
    RnonCO2[	92	]=	0.064970976	;
    RnonCO2[	93	]=	0.066164765	;
    RnonCO2[	94	]=	0.067357381	;
    RnonCO2[	95	]=	0.06854924	;
    RnonCO2[	96	]=	0.069740139	;
    RnonCO2[	97	]=	0.070930146	;
    RnonCO2[	98	]=	0.072119404	;
    RnonCO2[	99	]=	0.073307822	;
    RnonCO2[	100	]=	0.074635976	;
    RnonCO2[	101	]=	0.075963149	;
    RnonCO2[	102	]=	0.077148556	;
    RnonCO2[	103	]=	0.078333267	;
    RnonCO2[	104	]=	0.079516971	;
    RnonCO2[	105	]=	0.080754531	;
    RnonCO2[	106	]=	0.081905797	;
    RnonCO2[	107	]=	0.083001419	;
    RnonCO2[	108	]=	0.084181461	;
    RnonCO2[	109	]=	0.085360786	;
    RnonCO2[	110	]=	0.086593261	;
    RnonCO2[	111	]=	0.087739428	;
    RnonCO2[	112	]=	0.088830817	;
    RnonCO2[	113	]=	0.090006409	;
    RnonCO2[	114	]=	0.091181226	;
    RnonCO2[	115	]=	0.092494155	;
    RnonCO2[	116	]=	0.093944335	;
    RnonCO2[	117	]=	0.095393111	;
    RnonCO2[	118	]=	0.096840323	;
    RnonCO2[	119	]=	0.098148223	;
    RnonCO2[	120	]=	0.09931678	;
    RnonCO2[	121	]=	0.100622634	;
    RnonCO2[	122	]=	0.101979664	;
    RnonCO2[	123	]=	0.10325049	;
    RnonCO2[	124	]=	0.104604961	;
    RnonCO2[	125	]=	0.106043011	;
    RnonCO2[	126	]=	0.107616808	;
    RnonCO2[	127	]=	0.109325606	;
    RnonCO2[	128	]=	0.110810739	;
    RnonCO2[	129	]=	0.112072699	;
    RnonCO2[	130	]=	0.113469426	;
    RnonCO2[	131	]=	0.115000677	;
    RnonCO2[	132	]=	0.116479164	;
    RnonCO2[	133	]=	0.117955956	;
    RnonCO2[	134	]=	0.119481632	;
    RnonCO2[	135	]=	0.121230683	;
    RnonCO2[	136	]=	0.123467298	;
    RnonCO2[	137	]=	0.125964505	;
    RnonCO2[	138	]=	0.128590857	;
    RnonCO2[	139	]=	0.131345679	;
    RnonCO2[	140	]=	0.134094266	;
    RnonCO2[	141	]=	0.136836274	;
    RnonCO2[	142	]=	0.139657159	;
    RnonCO2[	143	]=	0.142689092	;
    RnonCO2[	144	]=	0.145846541	;
    RnonCO2[	145	]=	0.148864539	;
    RnonCO2[	146	]=	0.151744205	;
    RnonCO2[	147	]=	0.154833698	;
    RnonCO2[	148	]=	0.158178102	;
    RnonCO2[	149	]=	0.161560167	;
    RnonCO2[	150	]=	0.165018225	;
    RnonCO2[	151	]=	0.168552404	;
    RnonCO2[	152	]=	0.172162279	;
    RnonCO2[	153	]=	0.175976823	;
    RnonCO2[	154	]=	0.179825831	;
    RnonCO2[	155	]=	0.18353714	;
    RnonCO2[	156	]=	0.18719586	;
    RnonCO2[	157	]=	0.190888601	;
    RnonCO2[	158	]=	0.194618313	;
    RnonCO2[	159	]=	0.198342475	;
    RnonCO2[	160	]=	0.202057589	;
    RnonCO2[	161	]=	0.20584786	;
    RnonCO2[	162	]=	0.209629466	;
    RnonCO2[	163	]=	0.213233356	;
    RnonCO2[	164	]=	0.216704098	;
    RnonCO2[	165	]=	0.220042715	;
    RnonCO2[	166	]=	0.223296521	;
    RnonCO2[	167	]=	0.226627686	;
    RnonCO2[	168	]=	0.230030664	;
    RnonCO2[	169	]=	0.233304361	;
    RnonCO2[	170	]=	0.236449311	;
    RnonCO2[	171	]=	0.239588758	;
    RnonCO2[	172	]=	0.24263915	;
    RnonCO2[	173	]=	0.245600759	;
    RnonCO2[	174	]=	0.248557521	;
    RnonCO2[	175	]=	0.251630431	;
    RnonCO2[	176	]=	0.254903022	;
    RnonCO2[	177	]=	0.258133328	;
    RnonCO2[	178	]=	0.26123828	;
    RnonCO2[	179	]=	0.264542	;
    RnonCO2[	180	]=	0.268044049	;
    RnonCO2[	181	]=	0.27165959	;
    RnonCO2[	182	]=	0.275506693	;
    RnonCO2[	183	]=	0.279465183	;
    RnonCO2[	184	]=	0.283533907	;
    RnonCO2[	185	]=	0.287746496	;
    RnonCO2[	186	]=	0.292103558	;
    RnonCO2[	187	]=	0.296886349	;
    RnonCO2[	188	]=	0.302089492	;
    RnonCO2[	189	]=	0.307309759	;
    RnonCO2[	190	]=	0.312629337	;
    RnonCO2[	191	]=	0.318245324	;
    RnonCO2[	192	]=	0.323988656	;
    RnonCO2[	193	]=	0.329826066	;
    RnonCO2[	194	]=	0.335923764	;
    RnonCO2[	195	]=	0.342309427	;
    RnonCO2[	196	]=	0.348815284	;
    RnonCO2[	197	]=	0.355439785	;
    RnonCO2[	198	]=	0.362318532	;
    RnonCO2[	199	]=	0.369551061	;
    RnonCO2[	200	]=	0.377072615	;
    RnonCO2[	201	]=	0.384665115	;
    RnonCO2[	202	]=	0.392331499	;
    RnonCO2[	203	]=	0.400273286	;
    RnonCO2[	204	]=	0.408559205	;
    RnonCO2[	205	]=	0.416991838	;
    RnonCO2[	206	]=	0.425389986	;
    RnonCO2[	207	]=	0.433949678	;
    RnonCO2[	208	]=	0.442740432	;
    RnonCO2[	209	]=	0.451573488	;
    RnonCO2[	210	]=	0.460373303	;
    RnonCO2[	211	]=	0.469303629	;
    RnonCO2[	212	]=	0.478507559	;
    RnonCO2[	213	]=	0.48782133	;
    RnonCO2[	214	]=	0.49689881	;
    RnonCO2[	215	]=	0.506302348	;
    RnonCO2[	216	]=	0.516911269	;
    RnonCO2[	217	]=	0.527818076	;
    RnonCO2[	218	]=	0.538162703	;
    RnonCO2[	219	]=	0.547403268	;
    RnonCO2[	220	]=	0.555524533	;
    RnonCO2[	221	]=	0.562888474	;
    RnonCO2[	222	]=	0.569356555	;
    RnonCO2[	223	]=	0.576908971	;
    RnonCO2[	224	]=	0.585481654	;
    RnonCO2[	225	]=	0.59287455	;
    RnonCO2[	226	]=	0.599146788	;
    RnonCO2[	227	]=	0.604130931	;
    RnonCO2[	228	]=	0.607750235	;
    RnonCO2[	229	]=	0.611664417	;
    RnonCO2[	230	]=	0.616444422	;
    RnonCO2[	231	]=	0.620912678	;
    RnonCO2[	232	]=	0.625821808	;
    RnonCO2[	233	]=	0.632177868	;
    RnonCO2[	234	]=	0.637926114	;
    RnonCO2[	235	]=	0.641532814	;
    RnonCO2[	236	]=	0.644675991	;
    RnonCO2[	237	]=	0.649064599	;
    RnonCO2[	238	]=	0.653980688	;
    RnonCO2[	239	]=	0.657858713	;
    RnonCO2[	240	]=	0.661298844	;
    RnonCO2[	241	]=	0.665634405	;
    RnonCO2[	242	]=	0.670709564	;
    RnonCO2[	243	]=	0.67628923	;
    RnonCO2[	244	]=	0.682302458	;
    RnonCO2[	245	]=	0.688639697	;
    RnonCO2[	246	]=	0.69527718	;
    RnonCO2[	247	]=	0.701403562	;
    RnonCO2[	248	]=	0.70627608	;
    RnonCO2[	249	]=	0.710008327	;
    RnonCO2[	250	]=	0.712705657	;
    RnonCO2[	251	]=	0.714449176	;
    RnonCO2[	252	]=	0.715301891	;
    RnonCO2[	253	]=	0.715323851	;
    RnonCO2[	254	]=	0.714572195	;
    RnonCO2[	255	]=	0.713098359	;
    RnonCO2[	256	]=	0.710923322	;
    RnonCO2[	257	]=	0.708448191	;
    RnonCO2[	258	]=	0.706115586	;
    RnonCO2[	259	]=	0.70402495	;
    RnonCO2[	260	]=	0.702225359	;
    RnonCO2[	261	]=	0.700699373	;
    RnonCO2[	262	]=	0.699417888	;
    RnonCO2[	263	]=	0.698344459	;
    RnonCO2[	264	]=	0.697449374	;
    RnonCO2[	265	]=	0.696708732	;
    RnonCO2[	266	]=	0.69610217	;
    RnonCO2[	267	]=	0.695628173	;
    RnonCO2[	268	]=	0.695294114	;
    RnonCO2[	269	]=	0.695099011	;
    RnonCO2[	270	]=	0.695038999	;
    RnonCO2[	271	]=	0.695109645	;
    RnonCO2[	272	]=	0.695309373	;
    RnonCO2[	273	]=	0.695623892	;
    RnonCO2[	274	]=	0.696030489	;
    RnonCO2[	275	]=	0.696511834	;
    RnonCO2[	276	]=	0.697056571	;
    RnonCO2[	277	]=	0.697402494	;
    RnonCO2[	278	]=	0.69728589	;
    RnonCO2[	279	]=	0.696728513	;
    RnonCO2[	280	]=	0.695783858	;
    RnonCO2[	281	]=	0.694495906	;
    RnonCO2[	282	]=	0.692903322	;
    RnonCO2[	283	]=	0.691042708	;
    RnonCO2[	284	]=	0.688934136	;
    RnonCO2[	285	]=	0.686585963	;
    RnonCO2[	286	]=	0.684008482	;
    RnonCO2[	287	]=	0.681383708	;
    RnonCO2[	288	]=	0.678877058	;
    RnonCO2[	289	]=	0.676445668	;
    RnonCO2[	290	]=	0.674060558	;
    RnonCO2[	291	]=	0.671730066	;
    RnonCO2[	292	]=	0.669457855	;
    RnonCO2[	293	]=	0.667246271	;
    RnonCO2[	294	]=	0.665099438	;
    RnonCO2[	295	]=	0.663009405	;
    RnonCO2[	296	]=	0.660958228	;
    RnonCO2[	297	]=	0.659086247	;
    RnonCO2[	298	]=	0.657524153	;
    RnonCO2[	299	]=	0.656237746	;
    RnonCO2[	300	]=	0.655171653	;
    RnonCO2[	301	]=	0.654286182	;
    RnonCO2[	302	]=	0.65357858	;
    RnonCO2[	303	]=	0.653043038	;
    RnonCO2[	304	]=	0.652672935	;
    RnonCO2[	305	]=	0.652465157	;
    RnonCO2[	306	]=	0.652404059	;
    RnonCO2[	307	]=	0.6524418	;
    RnonCO2[	308	]=	0.652536738	;
    RnonCO2[	309	]=	0.652678506	;
    RnonCO2[	310	]=	0.652857409	;
    RnonCO2[	311	]=	0.653041128	;
    RnonCO2[	312	]=	0.653210323	;
    RnonCO2[	313	]=	0.653380236	;
    RnonCO2[	314	]=	0.653560975	;
    RnonCO2[	315	]=	0.65376115	;
    RnonCO2[	316	]=	0.653990302	;
    RnonCO2[	317	]=	0.654213781	;
    RnonCO2[	318	]=	0.654389478	;
    RnonCO2[	319	]=	0.654513982	;
    RnonCO2[	320	]=	0.654587417	;
    RnonCO2[	321	]=	0.654609747	;
    RnonCO2[	322	]=	0.654557237	;
    RnonCO2[	323	]=	0.654417955	;
    RnonCO2[	324	]=	0.654213734	;
    RnonCO2[	325	]=	0.653961206	;
    RnonCO2[	326	]=	0.653673659	;
    RnonCO2[	327	]=	0.653359165	;
    RnonCO2[	328	]=	0.653011831	;
    RnonCO2[	329	]=	0.652623659	;
    RnonCO2[	330	]=	0.652190169	;
    RnonCO2[	331	]=	0.651709747	;
    RnonCO2[	332	]=	0.65118174	;
    RnonCO2[	333	]=	0.650581524	;
    RnonCO2[	334	]=	0.649896351	;
    RnonCO2[	335	]=	0.649147717	;
    RnonCO2[	336	]=	0.648351872	;
    RnonCO2[	337	]=	0.647587491	;
    RnonCO2[	338	]=	0.646927556	;
    RnonCO2[	339	]=	0.646361929	;
    RnonCO2[	340	]=	0.645872576	;
    RnonCO2[	341	]=	0.645446444	;
    RnonCO2[	342	]=	0.645074486	;
    RnonCO2[	343	]=	0.644749065	;
    RnonCO2[	344	]=	0.644439491	;
    RnonCO2[	345	]=	0.644127681	;
    RnonCO2[	346	]=	0.643829745	;
    RnonCO2[	347	]=	0.643556768	;
    RnonCO2[	348	]=	0.643317648	;
    RnonCO2[	349	]=	0.643122434	;
    RnonCO2[	350	]=	0.642968368	;
    RnonCO2[	351	]=	0.642843801	;
    RnonCO2[	352	]=	0.642741103	;
    RnonCO2[	353	]=	0.642656013	;
    RnonCO2[	354	]=	0.642585153	;
    RnonCO2[	355	]=	0.642501504	;
    RnonCO2[	356	]=	0.642390659	;
    RnonCO2[	357	]=	0.64227118	;
    RnonCO2[	358	]=	0.642157069	;
    RnonCO2[	359	]=	0.642059166	;
    RnonCO2[	360	]=	0.641989552	;
    RnonCO2[	361	]=	0.641947462	;
    RnonCO2[	362	]=	0.641922442	;
    RnonCO2[	363	]=	0.641908413	;
    RnonCO2[	364	]=	0.641902542	;
    RnonCO2[	365	]=	0.641902103	;
    RnonCO2[	366	]=	0.641881484	;
    RnonCO2[	367	]=	0.64182659	;
    RnonCO2[	368	]=	0.641757156	;
    RnonCO2[	369	]=	0.641687481	;
    RnonCO2[	370	]=	0.641629007	;
    RnonCO2[	371	]=	0.641594499	;
    RnonCO2[	372	]=	0.641583402	;
    RnonCO2[	373	]=	0.641585827	;
    RnonCO2[	374	]=	0.641596018	;
    RnonCO2[	375	]=	0.641611463	;
    RnonCO2[	376	]=	0.64162986	;
    RnonCO2[	377	]=	0.641625726	;
    RnonCO2[	378	]=	0.641585263	;
    RnonCO2[	379	]=	0.641528723	;
    RnonCO2[	380	]=	0.641470296	;
    RnonCO2[	381	]=	0.641421766	;
    RnonCO2[	382	]=	0.641395948	;
    RnonCO2[	383	]=	0.641392154	;
    RnonCO2[	384	]=	0.641401353	;
    RnonCO2[	385	]=	0.64141721	;
    RnonCO2[	386	]=	0.641437537	;
    RnonCO2[	387	]=	0.641460391	;
    RnonCO2[	388	]=	0.641459943	;
    RnonCO2[	389	]=	0.641423022	;
    RnonCO2[	390	]=	0.641369488	;
    RnonCO2[	391	]=	0.641313577	;
    RnonCO2[	392	]=	0.641267383	;
    RnonCO2[	393	]=	0.641243587	;
    RnonCO2[	394	]=	0.641241634	;
    RnonCO2[	395	]=	0.641252286	;
    RnonCO2[	396	]=	0.641269652	;
    RnonCO2[	397	]=	0.641291382	;
    RnonCO2[	398	]=	0.641315333	;
    RnonCO2[	399	]=	0.641316089	;
    RnonCO2[	400	]=	0.641280102	;
    RnonCO2[	401	]=	0.641227629	;
    RnonCO2[	402	]=	0.641172852	;
    RnonCO2[	403	]=	0.641127523	;
    RnonCO2[	404	]=	0.641104618	;
    RnonCO2[	405	]=	0.641103737	;
    RnonCO2[	406	]=	0.64111538	;
    RnonCO2[	407	]=	0.641133688	;
    RnonCO2[	408	]=	0.641156242	;
    RnonCO2[	409	]=	0.641181334	;
    RnonCO2[	410	]=	0.641183105	;
    RnonCO2[	411	]=	0.641148179	;
    RnonCO2[	412	]=	0.64109684	;
    RnonCO2[	413	]=	0.641043309	;
    RnonCO2[	414	]=	0.640999274	;
    RnonCO2[	415	]=	0.640977747	;
    RnonCO2[	416	]=	0.64097798	;
    RnonCO2[	417	]=	0.640990871	;
    RnonCO2[	418	]=	0.641010507	;
    RnonCO2[	419	]=	0.641034661	;
    RnonCO2[	420	]=	0.641061044	;
    RnonCO2[	421	]=	0.641064416	;
    RnonCO2[	422	]=	0.641031095	;
    RnonCO2[	423	]=	0.64098136	;
    RnonCO2[	424	]=	0.640929589	;
    RnonCO2[	425	]=	0.640887156	;
    RnonCO2[	426	]=	0.64086736	;
    RnonCO2[	427	]=	0.640869364	;
    RnonCO2[	428	]=	0.64088411	;
    RnonCO2[	429	]=	0.64090566	;
    RnonCO2[	430	]=	0.640931694	;
    RnonCO2[	431	]=	0.640960009	;
    RnonCO2[	432	]=	0.640965392	;
    RnonCO2[	433	]=	0.640934081	;
    RnonCO2[	434	]=	0.640886487	;
    RnonCO2[	435	]=	0.640836625	;
    RnonCO2[	436	]=	0.640796581	;
    RnonCO2[	437	]=	0.640778892	;
    RnonCO2[	438	]=	0.640783188	;
    RnonCO2[	439	]=	0.640800038	;
    RnonCO2[	440	]=	0.640823767	;
    RnonCO2[	441	]=	0.640851987	;
    RnonCO2[	442	]=	0.640882711	;
    RnonCO2[	443	]=	0.640890341	;
    RnonCO2[	444	]=	0.640861479	;
    RnonCO2[	445	]=	0.640816265	;
    RnonCO2[	446	]=	0.640768843	;
    RnonCO2[	447	]=	0.640731096	;
    RnonCO2[	448	]=	0.640715718	;
    RnonCO2[	449	]=	0.64072241	;
    RnonCO2[	450	]=	0.640741789	;
    RnonCO2[	451	]=	0.640767899	;
    RnonCO2[	452	]=	0.640798694	;
    RnonCO2[	453	]=	0.640831765	;
    RnonCO2[	454	]=	0.640841788	;
    RnonCO2[	455	]=	0.640815504	;
    RnonCO2[	456	]=	0.640772848	;
    RnonCO2[	457	]=	0.640727856	;
    RnonCO2[	458	]=	0.640692733	;
    RnonCO2[	459	]=	0.640680046	;
    RnonCO2[	460	]=	0.640689303	;
    RnonCO2[	461	]=	0.640711189	;
    RnonCO2[	462	]=	0.640739891	;
    RnonCO2[	463	]=	0.640773157	;
    RnonCO2[	464	]=	0.640808797	;
    RnonCO2[	465	]=	0.640821428	;
    RnonCO2[	466	]=	0.640797617	;
    RnonCO2[	467	]=	0.640757545	;
    RnonCO2[	468	]=	0.64071535	;
    RnonCO2[	469	]=	0.640682616	;
    RnonCO2[	470	]=	0.640672491	;
    RnonCO2[	471	]=	0.640684395	;
    RnonCO2[	472	]=	0.64070869	;
    RnonCO2[	473	]=	0.64074009	;
    RnonCO2[	474	]=	0.640775773	;
    RnonCO2[	475	]=	0.640814056	;
    RnonCO2[	476	]=	0.640829201	;
    RnonCO2[	477	]=	0.640808032	;
    RnonCO2[	478	]=	0.64077054	;
    RnonCO2[	479	]=	0.640730713	;
    RnonCO2[	480	]=	0.640700724	;
    RnonCO2[	481	]=	0.640693031	;
    RnonCO2[	482	]=	0.640707431	;
    RnonCO2[	483	]=	0.64073449	;
    RnonCO2[	484	]=	0.64076826	;
    RnonCO2[	485	]=	0.640806409	;
    RnonCO2[	486	]=	0.640847194	;
    RnonCO2[	487	]=	0.640864861	;
    RnonCO2[	488	]=	0.640846062	;
    RnonCO2[	489	]=	0.640811122	;
    RnonCO2[	490	]=	0.640773885	;
    RnonCO2[	491	]=	0.640746288	;
    RnonCO2[	492	]=	0.640741193	;
    RnonCO2[	493	]=	0.640757802	;
    RnonCO2[	494	]=	0.640787191	;
    RnonCO2[	495	]=	0.640823405	;
    RnonCO2[	496	]=	0.640864078	;
    RnonCO2[	497	]=	0.640907151	;
    RnonCO2[	498	]=	0.640927242	;
    RnonCO2[	499	]=	0.640910852	;
    RnonCO2[	500	]=	0.640878234	;
    RnonCO2[	501	]=	0.640843453	;
    RnonCO2[	502	]=	0.640818219	;
    RnonCO2[	503	]=	0.640815287	;
    RnonCO2[	504	]=	0.640834449	;
    RnonCO2[	505	]=	0.640866076	;
    RnonCO2[	506	]=	0.640904642	;
    RnonCO2[	507	]=	0.64094757	;
    RnonCO2[	508	]=	0.640992949	;
    RnonCO2[	509	]=	0.641015182	;
    RnonCO2[	510	]=	0.641001218	;
    RnonCO2[	511	]=	0.640970905	;
    RnonCO2[	512	]=	0.640938306	;
    RnonCO2[	513	]=	0.64091527	;
    RnonCO2[	514	]=	0.640914554	;
    RnonCO2[	515	]=	0.640935918	;
    RnonCO2[	516	]=	0.640969684	;
    RnonCO2[	517	]=	0.64101034	;
    RnonCO2[	518	]=	0.641055423	;
    RnonCO2[	519	]=	0.641102976	;
    RnonCO2[	520	]=	0.641127498	;
    RnonCO2[	521	]=	0.641115546	;
    RnonCO2[	522	]=	0.641087273	;
    RnonCO2[	523	]=	0.641056701	;
    RnonCO2[	524	]=	0.641035836	;
    RnonCO2[	525	]=	0.641037306	;
    RnonCO2[	526	]=	0.64106067	;
    RnonCO2[	527	]=	0.641096487	;
    RnonCO2[	528	]=	0.64113916	;
    RnonCO2[	529	]=	0.641186291	;
    RnonCO2[	530	]=	0.641235763	;
    RnonCO2[	531	]=	0.641262211	;
    RnonCO2[	532	]=	0.641252173	;
    RnonCO2[	533	]=	0.641225977	;
    RnonCO2[	534	]=	0.641197318	;
    RnonCO2[	535	]=	0.641178511	;
    RnonCO2[	536	]=	0.641181865	;
    RnonCO2[	537	]=	0.641207056	;
    RnonCO2[	538	]=	0.641244764	;
    RnonCO2[	539	]=	0.641289276	;
    RnonCO2[	540	]=	0.641338409	;
    RnonCO2[	541	]=	0.64138962	;
    RnonCO2[	542	]=	0.641418067	;
    RnonCO2[	543	]=	0.641409757	;
    RnonCO2[	544	]=	0.641385454	;
    RnonCO2[	545	]=	0.641358657	;
    RnonCO2[	546	]=	0.641341539	;
    RnonCO2[	547	]=	0.641346701	;
    RnonCO2[	548	]=	0.641373635	;
    RnonCO2[	549	]=	0.641413119	;
    RnonCO2[	550	]=	0.641459485	;
    RnonCO2[	551	]=	0.641510222	;
    RnonCO2[	552	]=	0.641563169	;
    RnonCO2[	553	]=	0.64159322	;
    RnonCO2[	554	]=	0.64158681	;
    RnonCO2[	555	]=	0.641563995	;
    RnonCO2[	556	]=	0.641539068	;
    RnonCO2[	557	]=	0.641523643	;
    RnonCO2[	558	]=	0.641530332	;
    RnonCO2[	559	]=	0.641558975	;
    RnonCO2[	560	]=	0.641600062	;
    RnonCO2[	561	]=	0.641647961	;
    RnonCO2[	562	]=	0.641700245	;
    RnonCO2[	563	]=	0.64175489	;
    RnonCO2[	564	]=	0.641786506	;
    RnonCO2[	565	]=	0.641781651	;
    RnonCO2[	566	]=	0.641760427	;
    RnonCO2[	567	]=	0.64173704	;
    RnonCO2[	568	]=	0.641723127	;
    RnonCO2[	569	]=	0.641731388	;
    RnonCO2[	570	]=	0.641761516	;
    RnonCO2[	571	]=	0.641803981	;
    RnonCO2[	572	]=	0.641853453	;
    RnonCO2[	573	]=	0.641907207	;
    RnonCO2[	574	]=	0.641963305	;
    RnonCO2[	575	]=	0.641996229	;
    RnonCO2[	576	]=	0.641992859	;
    RnonCO2[	577	]=	0.641973142	;
    RnonCO2[	578	]=	0.641951178	;
    RnonCO2[	579	]=	0.641938654	;
    RnonCO2[	580	]=	0.641948246	;
    RnonCO2[	581	]=	0.64197973	;
    RnonCO2[	582	]=	0.642023712	;
    RnonCO2[	583	]=	0.642074341	;
    RnonCO2[	584	]=	0.642129399	;
    RnonCO2[	585	]=	0.642186954	;
    RnonCO2[	586	]=	0.642221191	;
    RnonCO2[	587	]=	0.642219303	;
    RnonCO2[	588	]=	0.642200784	;
    RnonCO2[	589	]=	0.642179984	;
    RnonCO2[	590	]=	0.642168844	;
    RnonCO2[	591	]=	0.642179874	;
    RnonCO2[	592	]=	0.642212549	;
    RnonCO2[	593	]=	0.642257576	;
    RnonCO2[	594	]=	0.642309599	;
    RnonCO2[	595	]=	0.642365945	;
    RnonCO2[	596	]=	0.642424465	;
    RnonCO2[	597	]=	0.642460121	;
    RnonCO2[	598	]=	0.642459247	;
    RnonCO2[	599	]=	0.642442055	;
    RnonCO2[	600	]=	0.642422649	;
    RnonCO2[	601	]=	0.642412542	;
    RnonCO2[	602	]=	0.642424539	;
    RnonCO2[	603	]=	0.642458609	;
    RnonCO2[	604	]=	0.642504764	;
    RnonCO2[	605	]=	0.642557976	;
    RnonCO2[	606	]=	0.64261537	;
    RnonCO2[	607	]=	0.642675074	;
    RnonCO2[	608	]=	0.642711774	;
    RnonCO2[	609	]=	0.642711987	;
    RnonCO2[	610	]=	0.64269589	;
    RnonCO2[	611	]=	0.642677526	;
    RnonCO2[	612	]=	0.642668541	;
    RnonCO2[	613	]=	0.64268183	;
    RnonCO2[	614	]=	0.642716628	;
    RnonCO2[	615	]=	0.64276394	;
    RnonCO2[	616	]=	0.642817944	;
    RnonCO2[	617	]=	0.642876415	;
    RnonCO2[	618	]=	0.642937333	;
    RnonCO2[	619	]=	0.642974973	;
    RnonCO2[	620	]=	0.642976326	;
    RnonCO2[	621	]=	0.642961087	;
    RnonCO2[	622	]=	0.642943647	;
    RnonCO2[	623	]=	0.642935649	;
    RnonCO2[	624	]=	0.642949834	;
    RnonCO2[	625	]=	0.642985662	;
    RnonCO2[	626	]=	0.643034009	;
    RnonCO2[	627	]=	0.643089089	;
    RnonCO2[	628	]=	0.643148354	;
    RnonCO2[	629	]=	0.643210068	;
    RnonCO2[	630	]=	0.643248799	;
    RnonCO2[	631	]=	0.643250861	;
    RnonCO2[	632	]=	0.64323679	;
    RnonCO2[	633	]=	0.643220293	;
    RnonCO2[	634	]=	0.643213004	;
    RnonCO2[	635	]=	0.64322822	;
    RnonCO2[	636	]=	0.643264917	;
    RnonCO2[	637	]=	0.643313997	;
    RnonCO2[	638	]=	0.643369852	;
    RnonCO2[	639	]=	0.643430059	;
    RnonCO2[	640	]=	0.643492661	;
    RnonCO2[	641	]=	0.643532044	;
    RnonCO2[	642	]=	0.643535246	;
    RnonCO2[	643	]=	0.643521737	;
    RnonCO2[	644	]=	0.643506025	;
    RnonCO2[	645	]=	0.643499719	;
    RnonCO2[	646	]=	0.643515648	;
    RnonCO2[	647	]=	0.64355305	;
    RnonCO2[	648	]=	0.643603114	;
    RnonCO2[	649	]=	0.643659699	;
    RnonCO2[	650	]=	0.643720646	;
    RnonCO2[	651	]=	0.643783971	;
    RnonCO2[	652	]=	0.643824101	;
    RnonCO2[	653	]=	0.643827969	;
    RnonCO2[	654	]=	0.643815297	;
    RnonCO2[	655	]=	0.643800331	;
    RnonCO2[	656	]=	0.64379497	;
    RnonCO2[	657	]=	0.643811412	;
    RnonCO2[	658	]=	0.643849754	;
    RnonCO2[	659	]=	0.643900303	;
    RnonCO2[	660	]=	0.64395757	;
    RnonCO2[	661	]=	0.644019334	;
    RnonCO2[	662	]=	0.644083335	;
    RnonCO2[	663	]=	0.644124308	;
    RnonCO2[	664	]=	0.644128783	;
    RnonCO2[	665	]=	0.644116891	;
    RnonCO2[	666	]=	0.644102613	;
    RnonCO2[	667	]=	0.6440977	;
    RnonCO2[	668	]=	0.644114891	;
    RnonCO2[	669	]=	0.644153654	;
    RnonCO2[	670	]=	0.644205061	;
    RnonCO2[	671	]=	0.644262928	;
    RnonCO2[	672	]=	0.644325144	;
    RnonCO2[	673	]=	0.644390018	;
    RnonCO2[	674	]=	0.644431455	;
    RnonCO2[	675	]=	0.644436475	;
    RnonCO2[	676	]=	0.644425145	;
    RnonCO2[	677	]=	0.644411638	;
    RnonCO2[	678	]=	0.644407376	;
    RnonCO2[	679	]=	0.644425236	;
    RnonCO2[	680	]=	0.644464635	;
    RnonCO2[	681	]=	0.644516358	;
    RnonCO2[	682	]=	0.644575023	;
    RnonCO2[	683	]=	0.644637906	;
    RnonCO2[	684	]=	0.644703111	;
    RnonCO2[	685	]=	0.644745068	;
    RnonCO2[	686	]=	0.644750837	;
    RnonCO2[	687	]=	0.644740132	;
    RnonCO2[	688	]=	0.644727158	;
    RnonCO2[	689	]=	0.644723467	;
    RnonCO2[	690	]=	0.644741736	;
    RnonCO2[	691	]=	0.64478167	;
    RnonCO2[	692	]=	0.644834048	;
    RnonCO2[	693	]=	0.644893108	;
    RnonCO2[	694	]=	0.644956376	;
    RnonCO2[	695	]=	0.645022107	;
    RnonCO2[	696	]=	0.645064785	;
    RnonCO2[	697	]=	0.645071063	;
    RnonCO2[	698	]=	0.645060867	;
    RnonCO2[	699	]=	0.645048167	;
    RnonCO2[	700	]=	0.645045091	;
    RnonCO2[	701	]=	0.645063809	;
    RnonCO2[	702	]=	0.645104299	;
    RnonCO2[	703	]=	0.645157211	;
    RnonCO2[	704	]=	0.64521654	;
    RnonCO2[	705	]=	0.645280534	;
    RnonCO2[	706	]=	0.645346669	;
    RnonCO2[	707	]=	0.645389786	;
    RnonCO2[	708	]=	0.645396435	;
    RnonCO2[	709	]=	0.645386649	;
    RnonCO2[	710	]=	0.645374551	;
    RnonCO2[	711	]=	0.64537181	;
    RnonCO2[	712	]=	0.645391018	;
    RnonCO2[	713	]=	0.645431924	;
    RnonCO2[	714	]=	0.645485085	;
    RnonCO2[	715	]=	0.64554515	;
    RnonCO2[	716	]=	0.645609424	;
    RnonCO2[	717	]=	0.645676124	;
    RnonCO2[	718	]=	0.645719395	;
    RnonCO2[	719	]=	0.6457266	;
    RnonCO2[	720	]=	0.645717248	;
    RnonCO2[	721	]=	0.645705635	;
    RnonCO2[	722	]=	0.645703251	;
    RnonCO2[	723	]=	0.645722935	;
    RnonCO2[	724	]=	0.645764115	;
    RnonCO2[	725	]=	0.645817686	;
    RnonCO2[	726	]=	0.64587804	;
    RnonCO2[	727	]=	0.645942756	;
    RnonCO2[	728	]=	0.646009715	;
    RnonCO2[	729	]=	0.646053591	;
    RnonCO2[	730	]=	0.64606107	;
    RnonCO2[	731	]=	0.64605214	;
    RnonCO2[	732	]=	0.646040708	;
    RnonCO2[	733	]=	0.646038832	;
    RnonCO2[	734	]=	0.646058706	;
    RnonCO2[	735	]=	0.646075547	;
    
    for(int i = 736; i<tmax; i++)
    {
        RnonCO2[i] = RnonCO2[i-1];
    }

}

void setRnonCO2_RCP85_K(void)
{
    
    RnonCO2[	0	]=	-0.019988515	;
    RnonCO2[	1	]=	-0.005612142	;
    RnonCO2[	2	]=	0.008971239	;
    RnonCO2[	3	]=	0.009466172	;
    RnonCO2[	4	]=	0.009946833	;
    RnonCO2[	5	]=	0.010412166	;
    RnonCO2[	6	]=	0.010865438	;
    RnonCO2[	7	]=	0.011309666	;
    RnonCO2[	8	]=	0.011745593	;
    RnonCO2[	9	]=	0.012173039	;
    RnonCO2[	10	]=	0.012591208	;
    RnonCO2[	11	]=	0.012999626	;
    RnonCO2[	12	]=	0.013397144	;
    RnonCO2[	13	]=	0.013783559	;
    RnonCO2[	14	]=	0.014157995	;
    RnonCO2[	15	]=	0.014518817	;
    RnonCO2[	16	]=	0.014861789	;
    RnonCO2[	17	]=	0.015185328	;
    RnonCO2[	18	]=	0.015493438	;
    RnonCO2[	19	]=	0.015792171	;
    RnonCO2[	20	]=	0.01608699	;
    RnonCO2[	21	]=	0.016384013	;
    RnonCO2[	22	]=	0.016688621	;
    RnonCO2[	23	]=	0.017006739	;
    RnonCO2[	24	]=	0.017344141	;
    RnonCO2[	25	]=	0.017705767	;
    RnonCO2[	26	]=	0.018092359	;
    RnonCO2[	27	]=	0.018501885	;
    RnonCO2[	28	]=	0.01893258	;
    RnonCO2[	29	]=	0.019382558	;
    RnonCO2[	30	]=	0.019850303	;
    RnonCO2[	31	]=	0.020334085	;
    RnonCO2[	32	]=	0.020831989	;
    RnonCO2[	33	]=	0.021342706	;
    RnonCO2[	34	]=	0.02186457	;
    RnonCO2[	35	]=	0.022398729	;
    RnonCO2[	36	]=	0.022952686	;
    RnonCO2[	37	]=	0.023532049	;
    RnonCO2[	38	]=	0.024135881	;
    RnonCO2[	39	]=	0.024759327	;
    RnonCO2[	40	]=	0.025397837	;
    RnonCO2[	41	]=	0.02604713	;
    RnonCO2[	42	]=	0.026702498	;
    RnonCO2[	43	]=	0.02735936	;
    RnonCO2[	44	]=	0.028013477	;
    RnonCO2[	45	]=	0.028662413	;
    RnonCO2[	46	]=	0.029309198	;
    RnonCO2[	47	]=	0.029958961	;
    RnonCO2[	48	]=	0.030612845	;
    RnonCO2[	49	]=	0.031270152	;
    RnonCO2[	50	]=	0.03193112	;
    RnonCO2[	51	]=	0.032594644	;
    RnonCO2[	52	]=	0.033260857	;
    RnonCO2[	53	]=	0.033928967	;
    RnonCO2[	54	]=	0.034598246	;
    RnonCO2[	55	]=	0.035268641	;
    RnonCO2[	56	]=	0.035940357	;
    RnonCO2[	57	]=	0.036613717	;
    RnonCO2[	58	]=	0.037289228	;
    RnonCO2[	59	]=	0.037967253	;
    RnonCO2[	60	]=	0.038647628	;
    RnonCO2[	61	]=	0.039330893	;
    RnonCO2[	62	]=	0.040017299	;
    RnonCO2[	63	]=	0.040707068	;
    RnonCO2[	64	]=	0.041400385	;
    RnonCO2[	65	]=	0.042096128	;
    RnonCO2[	66	]=	0.042791515	;
    RnonCO2[	67	]=	0.04348473	;
    RnonCO2[	68	]=	0.044177707	;
    RnonCO2[	69	]=	0.044873365	;
    RnonCO2[	70	]=	0.045574528	;
    RnonCO2[	71	]=	0.04628438	;
    RnonCO2[	72	]=	0.047005924	;
    RnonCO2[	73	]=	0.047742202	;
    RnonCO2[	74	]=	0.048496247	;
    RnonCO2[	75	]=	0.049273302	;
    RnonCO2[	76	]=	0.050090919	;
    RnonCO2[	77	]=	0.050966437	;
    RnonCO2[	78	]=	0.051881638	;
    RnonCO2[	79	]=	0.052787675	;
    RnonCO2[	80	]=	0.053629807	;
    RnonCO2[	81	]=	0.054384454	;
    RnonCO2[	82	]=	0.055079735	;
    RnonCO2[	83	]=	0.055748092	;
    RnonCO2[	84	]=	0.056393965	;
    RnonCO2[	85	]=	0.057093398	;
    RnonCO2[	86	]=	0.05807606	;
    RnonCO2[	87	]=	0.059274862	;
    RnonCO2[	88	]=	0.060472717	;
    RnonCO2[	89	]=	0.061669627	;
    RnonCO2[	90	]=	0.062723294	;
    RnonCO2[	91	]=	0.063776572	;
    RnonCO2[	92	]=	0.064970976	;
    RnonCO2[	93	]=	0.066164765	;
    RnonCO2[	94	]=	0.067357381	;
    RnonCO2[	95	]=	0.06854924	;
    RnonCO2[	96	]=	0.069740139	;
    RnonCO2[	97	]=	0.070930146	;
    RnonCO2[	98	]=	0.072119404	;
    RnonCO2[	99	]=	0.073307822	;
    RnonCO2[	100	]=	0.074635976	;
    RnonCO2[	101	]=	0.075963149	;
    RnonCO2[	102	]=	0.077148556	;
    RnonCO2[	103	]=	0.078333267	;
    RnonCO2[	104	]=	0.079516971	;
    RnonCO2[	105	]=	0.080754531	;
    RnonCO2[	106	]=	0.081905797	;
    RnonCO2[	107	]=	0.083001419	;
    RnonCO2[	108	]=	0.084181461	;
    RnonCO2[	109	]=	0.085360786	;
    RnonCO2[	110	]=	0.086593261	;
    RnonCO2[	111	]=	0.087739428	;
    RnonCO2[	112	]=	0.088830817	;
    RnonCO2[	113	]=	0.090006409	;
    RnonCO2[	114	]=	0.091181226	;
    RnonCO2[	115	]=	0.092494155	;
    RnonCO2[	116	]=	0.093944335	;
    RnonCO2[	117	]=	0.095393111	;
    RnonCO2[	118	]=	0.096840323	;
    RnonCO2[	119	]=	0.098148223	;
    RnonCO2[	120	]=	0.09931678	;
    RnonCO2[	121	]=	0.100622634	;
    RnonCO2[	122	]=	0.101979664	;
    RnonCO2[	123	]=	0.10325049	;
    RnonCO2[	124	]=	0.104604961	;
    RnonCO2[	125	]=	0.106043011	;
    RnonCO2[	126	]=	0.107616808	;
    RnonCO2[	127	]=	0.109325606	;
    RnonCO2[	128	]=	0.110810739	;
    RnonCO2[	129	]=	0.112072699	;
    RnonCO2[	130	]=	0.113469426	;
    RnonCO2[	131	]=	0.115000677	;
    RnonCO2[	132	]=	0.116479164	;
    RnonCO2[	133	]=	0.117955956	;
    RnonCO2[	134	]=	0.119481632	;
    RnonCO2[	135	]=	0.121230683	;
    RnonCO2[	136	]=	0.123467298	;
    RnonCO2[	137	]=	0.125964505	;
    RnonCO2[	138	]=	0.128590857	;
    RnonCO2[	139	]=	0.131345679	;
    RnonCO2[	140	]=	0.134094266	;
    RnonCO2[	141	]=	0.136836274	;
    RnonCO2[	142	]=	0.139657159	;
    RnonCO2[	143	]=	0.142689092	;
    RnonCO2[	144	]=	0.145846541	;
    RnonCO2[	145	]=	0.148864539	;
    RnonCO2[	146	]=	0.151744205	;
    RnonCO2[	147	]=	0.154833698	;
    RnonCO2[	148	]=	0.158178102	;
    RnonCO2[	149	]=	0.161560167	;
    RnonCO2[	150	]=	0.165018225	;
    RnonCO2[	151	]=	0.168552404	;
    RnonCO2[	152	]=	0.172162279	;
    RnonCO2[	153	]=	0.175976823	;
    RnonCO2[	154	]=	0.179825831	;
    RnonCO2[	155	]=	0.18353714	;
    RnonCO2[	156	]=	0.18719586	;
    RnonCO2[	157	]=	0.190888601	;
    RnonCO2[	158	]=	0.194618313	;
    RnonCO2[	159	]=	0.198342475	;
    RnonCO2[	160	]=	0.202057589	;
    RnonCO2[	161	]=	0.20584786	;
    RnonCO2[	162	]=	0.209629466	;
    RnonCO2[	163	]=	0.213233356	;
    RnonCO2[	164	]=	0.216704098	;
    RnonCO2[	165	]=	0.220042715	;
    RnonCO2[	166	]=	0.223296521	;
    RnonCO2[	167	]=	0.226627686	;
    RnonCO2[	168	]=	0.230030664	;
    RnonCO2[	169	]=	0.233304361	;
    RnonCO2[	170	]=	0.236449311	;
    RnonCO2[	171	]=	0.239588758	;
    RnonCO2[	172	]=	0.24263915	;
    RnonCO2[	173	]=	0.245600759	;
    RnonCO2[	174	]=	0.248557521	;
    RnonCO2[	175	]=	0.251630431	;
    RnonCO2[	176	]=	0.254903022	;
    RnonCO2[	177	]=	0.258133328	;
    RnonCO2[	178	]=	0.26123828	;
    RnonCO2[	179	]=	0.264542	;
    RnonCO2[	180	]=	0.268044049	;
    RnonCO2[	181	]=	0.27165959	;
    RnonCO2[	182	]=	0.275506693	;
    RnonCO2[	183	]=	0.279465183	;
    RnonCO2[	184	]=	0.283533907	;
    RnonCO2[	185	]=	0.287746496	;
    RnonCO2[	186	]=	0.292103558	;
    RnonCO2[	187	]=	0.296886349	;
    RnonCO2[	188	]=	0.302089492	;
    RnonCO2[	189	]=	0.307309759	;
    RnonCO2[	190	]=	0.312629337	;
    RnonCO2[	191	]=	0.318245324	;
    RnonCO2[	192	]=	0.323988656	;
    RnonCO2[	193	]=	0.329826066	;
    RnonCO2[	194	]=	0.335923764	;
    RnonCO2[	195	]=	0.342309427	;
    RnonCO2[	196	]=	0.348815284	;
    RnonCO2[	197	]=	0.355439785	;
    RnonCO2[	198	]=	0.362318532	;
    RnonCO2[	199	]=	0.369551061	;
    RnonCO2[	200	]=	0.377072615	;
    RnonCO2[	201	]=	0.384665115	;
    RnonCO2[	202	]=	0.392331499	;
    RnonCO2[	203	]=	0.400273286	;
    RnonCO2[	204	]=	0.408559205	;
    RnonCO2[	205	]=	0.416991838	;
    RnonCO2[	206	]=	0.425389986	;
    RnonCO2[	207	]=	0.433949678	;
    RnonCO2[	208	]=	0.442740432	;
    RnonCO2[	209	]=	0.451573488	;
    RnonCO2[	210	]=	0.460373303	;
    RnonCO2[	211	]=	0.469303629	;
    RnonCO2[	212	]=	0.478507559	;
    RnonCO2[	213	]=	0.48782133	;
    RnonCO2[	214	]=	0.49689881	;
    RnonCO2[	215	]=	0.506302348	;
    RnonCO2[	216	]=	0.516911269	;
    RnonCO2[	217	]=	0.527818076	;
    RnonCO2[	218	]=	0.538162703	;
    RnonCO2[	219	]=	0.547403268	;
    RnonCO2[	220	]=	0.555524533	;
    RnonCO2[	221	]=	0.562888474	;
    RnonCO2[	222	]=	0.569356555	;
    RnonCO2[	223	]=	0.576908971	;
    RnonCO2[	224	]=	0.585481654	;
    RnonCO2[	225	]=	0.59287455	;
    RnonCO2[	226	]=	0.599146788	;
    RnonCO2[	227	]=	0.604130931	;
    RnonCO2[	228	]=	0.607750235	;
    RnonCO2[	229	]=	0.611664417	;
    RnonCO2[	230	]=	0.616444422	;
    RnonCO2[	231	]=	0.620912678	;
    RnonCO2[	232	]=	0.625821808	;
    RnonCO2[	233	]=	0.632177868	;
    RnonCO2[	234	]=	0.637926114	;
    RnonCO2[	235	]=	0.641532814	;
    RnonCO2[	236	]=	0.644675991	;
    RnonCO2[	237	]=	0.649064599	;
    RnonCO2[	238	]=	0.653980688	;
    RnonCO2[	239	]=	0.657858713	;
    RnonCO2[	240	]=	0.661298844	;
    RnonCO2[	241	]=	0.665633413	;
    RnonCO2[	242	]=	0.670918799	;
    RnonCO2[	243	]=	0.677115343	;
    RnonCO2[	244	]=	0.684117052	;
    RnonCO2[	245	]=	0.691779863	;
    RnonCO2[	246	]=	0.700055049	;
    RnonCO2[	247	]=	0.708955169	;
    RnonCO2[	248	]=	0.718480902	;
    RnonCO2[	249	]=	0.728589067	;
    RnonCO2[	250	]=	0.739243369	;
    RnonCO2[	251	]=	0.750397995	;
    RnonCO2[	252	]=	0.762000405	;
    RnonCO2[	253	]=	0.774005664	;
    RnonCO2[	254	]=	0.786375911	;
    RnonCO2[	255	]=	0.799075749	;
    RnonCO2[	256	]=	0.812044406	;
    RnonCO2[	257	]=	0.825155905	;
    RnonCO2[	258	]=	0.838337326	;
    RnonCO2[	259	]=	0.851599925	;
    RnonCO2[	260	]=	0.864953055	;
    RnonCO2[	261	]=	0.878407402	;
    RnonCO2[	262	]=	0.89195823	;
    RnonCO2[	263	]=	0.90558941	;
    RnonCO2[	264	]=	0.919289044	;
    RnonCO2[	265	]=	0.933049417	;
    RnonCO2[	266	]=	0.946861978	;
    RnonCO2[	267	]=	0.960865669	;
    RnonCO2[	268	]=	0.975199561	;
    RnonCO2[	269	]=	0.989856102	;
    RnonCO2[	270	]=	1.00482388	;
    RnonCO2[	271	]=	1.020090986	;
    RnonCO2[	272	]=	1.03564984	;
    RnonCO2[	273	]=	1.051477376	;
    RnonCO2[	274	]=	1.067540111	;
    RnonCO2[	275	]=	1.083810068	;
    RnonCO2[	276	]=	1.100266073	;
    RnonCO2[	277	]=	1.116869792	;
    RnonCO2[	278	]=	1.133550695	;
    RnonCO2[	279	]=	1.150273404	;
    RnonCO2[	280	]=	1.16705088	;
    RnonCO2[	281	]=	1.183891341	;
    RnonCO2[	282	]=	1.200801505	;
    RnonCO2[	283	]=	1.217791069	;
    RnonCO2[	284	]=	1.234851367	;
    RnonCO2[	285	]=	1.251960784	;
    RnonCO2[	286	]=	1.269103667	;
    RnonCO2[	287	]=	1.286084756	;
    RnonCO2[	288	]=	1.302726275	;
    RnonCO2[	289	]=	1.319010791	;
    RnonCO2[	290	]=	1.334936039	;
    RnonCO2[	291	]=	1.350549926	;
    RnonCO2[	292	]=	1.365893616	;
    RnonCO2[	293	]=	1.381003333	;
    RnonCO2[	294	]=	1.395916498	;
    RnonCO2[	295	]=	1.41064864	;
    RnonCO2[	296	]=	1.42519918	;
    RnonCO2[	297	]=	1.439408159	;
    RnonCO2[	298	]=	1.453136523	;
    RnonCO2[	299	]=	1.4664178	;
    RnonCO2[	300	]=	1.479242532	;
    RnonCO2[	301	]=	1.491615636	;
    RnonCO2[	302	]=	1.503595947	;
    RnonCO2[	303	]=	1.51523466	;
    RnonCO2[	304	]=	1.52657726	;
    RnonCO2[	305	]=	1.537669318	;
    RnonCO2[	306	]=	1.548533208	;
    RnonCO2[	307	]=	1.559202988	;
    RnonCO2[	308	]=	1.569713895	;
    RnonCO2[	309	]=	1.580073196	;
    RnonCO2[	310	]=	1.590288078	;
    RnonCO2[	311	]=	1.600323109	;
    RnonCO2[	312	]=	1.610159707	;
    RnonCO2[	313	]=	1.619837346	;
    RnonCO2[	314	]=	1.629391907	;
    RnonCO2[	315	]=	1.638852659	;
    RnonCO2[	316	]=	1.648250563	;
    RnonCO2[	317	]=	1.657630141	;
    RnonCO2[	318	]=	1.6670167	;
    RnonCO2[	319	]=	1.676400373	;
    RnonCO2[	320	]=	1.685775083	;
    RnonCO2[	321	]=	1.695137707	;
    RnonCO2[	322	]=	1.704440811	;
    RnonCO2[	323	]=	1.713654237	;
    RnonCO2[	324	]=	1.722810499	;
    RnonCO2[	325	]=	1.731938449	;
    RnonCO2[	326	]=	1.741063394	;
    RnonCO2[	327	]=	1.750064593	;
    RnonCO2[	328	]=	1.758809303	;
    RnonCO2[	329	]=	1.767302441	;
    RnonCO2[	330	]=	1.775552491	;
    RnonCO2[	331	]=	1.783572411	;
    RnonCO2[	332	]=	1.791376182	;
    RnonCO2[	333	]=	1.798930436	;
    RnonCO2[	334	]=	1.806218138	;
    RnonCO2[	335	]=	1.813287315	;
    RnonCO2[	336	]=	1.820181953	;
    RnonCO2[	337	]=	1.826831551	;
    RnonCO2[	338	]=	1.833173544	;
    RnonCO2[	339	]=	1.839234344	;
    RnonCO2[	340	]=	1.845018602	;
    RnonCO2[	341	]=	1.850534021	;
    RnonCO2[	342	]=	1.855794452	;
    RnonCO2[	343	]=	1.860812544	;
    RnonCO2[	344	]=	1.865555655	;
    RnonCO2[	345	]=	1.870006103	;
    RnonCO2[	346	]=	1.874212898	;
    RnonCO2[	347	]=	1.878221365	;
    RnonCO2[	348	]=	1.882071015	;
    RnonCO2[	349	]=	1.885801052	;
    RnonCO2[	350	]=	1.88942665	;
    RnonCO2[	351	]=	1.892938893	;
    RnonCO2[	352	]=	1.896333232	;
    RnonCO2[	353	]=	1.899612841	;
    RnonCO2[	354	]=	1.902780512	;
    RnonCO2[	355	]=	1.905793058	;
    RnonCO2[	356	]=	1.908624471	;
    RnonCO2[	357	]=	1.91131432	;
    RnonCO2[	358	]=	1.9139028	;
    RnonCO2[	359	]=	1.916423314	;
    RnonCO2[	360	]=	1.918909097	;
    RnonCO2[	361	]=	1.921369147	;
    RnonCO2[	362	]=	1.923789075	;
    RnonCO2[	363	]=	1.926158439	;
    RnonCO2[	364	]=	1.928475755	;
    RnonCO2[	365	]=	1.930738396	;
    RnonCO2[	366	]=	1.932900186	;
    RnonCO2[	367	]=	1.934928556	;
    RnonCO2[	368	]=	1.936861883	;
    RnonCO2[	369	]=	1.93873657	;
    RnonCO2[	370	]=	1.940582911	;
    RnonCO2[	371	]=	1.942433919	;
    RnonCO2[	372	]=	1.944294426	;
    RnonCO2[	373	]=	1.946148568	;
    RnonCO2[	374	]=	1.947983081	;
    RnonCO2[	375	]=	1.949792732	;
    RnonCO2[	376	]=	1.951573997	;
    RnonCO2[	377	]=	1.953278219	;
    RnonCO2[	378	]=	1.954871552	;
    RnonCO2[	379	]=	1.956389263	;
    RnonCO2[	380	]=	1.95786746	;
    RnonCO2[	381	]=	1.959336631	;
    RnonCO2[	382	]=	1.960828259	;
    RnonCO2[	383	]=	1.962346787	;
    RnonCO2[	384	]=	1.963874424	;
    RnonCO2[	385	]=	1.965397015	;
    RnonCO2[	386	]=	1.966907883	;
    RnonCO2[	387	]=	1.968402531	;
    RnonCO2[	388	]=	1.969831054	;
    RnonCO2[	389	]=	1.971157707	;
    RnonCO2[	390	]=	1.972415861	;
    RnonCO2[	391	]=	1.973641986	;
    RnonCO2[	392	]=	1.974865408	;
    RnonCO2[	393	]=	1.976117073	;
    RnonCO2[	394	]=	1.977401679	;
    RnonCO2[	395	]=	1.978700111	;
    RnonCO2[	396	]=	1.979997095	;
    RnonCO2[	397	]=	1.981284588	;
    RnonCO2[	398	]=	1.982557769	;
    RnonCO2[	399	]=	1.983766077	;
    RnonCO2[	400	]=	1.984873087	;
    RnonCO2[	401	]=	1.985913082	;
    RnonCO2[	402	]=	1.98692261	;
    RnonCO2[	403	]=	1.987932088	;
    RnonCO2[	404	]=	1.988972526	;
    RnonCO2[	405	]=	1.990048399	;
    RnonCO2[	406	]=	1.99114149	;
    RnonCO2[	407	]=	1.992235256	;
    RnonCO2[	408	]=	1.993321658	;
    RnonCO2[	409	]=	1.994395277	;
    RnonCO2[	410	]=	1.995405994	;
    RnonCO2[	411	]=	1.996316622	;
    RnonCO2[	412	]=	1.997161586	;
    RnonCO2[	413	]=	1.997978112	;
    RnonCO2[	414	]=	1.998796488	;
    RnonCO2[	415	]=	1.999648281	;
    RnonCO2[	416	]=	2.000537974	;
    RnonCO2[	417	]=	2.001446506	;
    RnonCO2[	418	]=	2.002357761	;
    RnonCO2[	419	]=	2.003263291	;
    RnonCO2[	420	]=	2.004157412	;
    RnonCO2[	421	]=	2.004989355	;
    RnonCO2[	422	]=	2.005722844	;
    RnonCO2[	423	]=	2.006392207	;
    RnonCO2[	424	]=	2.007034119	;
    RnonCO2[	425	]=	2.007680016	;
    RnonCO2[	426	]=	2.008361173	;
    RnonCO2[	427	]=	2.009082032	;
    RnonCO2[	428	]=	2.009823302	;
    RnonCO2[	429	]=	2.010568601	;
    RnonCO2[	430	]=	2.011308801	;
    RnonCO2[	431	]=	2.012038565	;
    RnonCO2[	432	]=	2.012707434	;
    RnonCO2[	433	]=	2.013279591	;
    RnonCO2[	434	]=	2.013788209	;
    RnonCO2[	435	]=	2.014270824	;
    RnonCO2[	436	]=	2.014758333	;
    RnonCO2[	437	]=	2.015281983	;
    RnonCO2[	438	]=	2.015846152	;
    RnonCO2[	439	]=	2.016431385	;
    RnonCO2[	440	]=	2.017020925	;
    RnonCO2[	441	]=	2.017606065	;
    RnonCO2[	442	]=	2.018181042	;
    RnonCO2[	443	]=	2.0186957	;
    RnonCO2[	444	]=	2.019113579	;
    RnonCO2[	445	]=	2.019469239	;
    RnonCO2[	446	]=	2.019800183	;
    RnonCO2[	447	]=	2.020137026	;
    RnonCO2[	448	]=	2.020511572	;
    RnonCO2[	449	]=	2.020927347	;
    RnonCO2[	450	]=	2.021365049	;
    RnonCO2[	451	]=	2.021807854	;
    RnonCO2[	452	]=	2.022247194	;
    RnonCO2[	453	]=	2.022677498	;
    RnonCO2[	454	]=	2.023048353	;
    RnonCO2[	455	]=	2.023323831	;
    RnonCO2[	456	]=	2.023538377	;
    RnonCO2[	457	]=	2.023729451	;
    RnonCO2[	458	]=	2.023928212	;
    RnonCO2[	459	]=	2.024165643	;
    RnonCO2[	460	]=	2.024445263	;
    RnonCO2[	461	]=	2.024747932	;
    RnonCO2[	462	]=	2.025056215	;
    RnonCO2[	463	]=	2.025361736	;
    RnonCO2[	464	]=	2.025659461	;
    RnonCO2[	465	]=	2.02589831	;
    RnonCO2[	466	]=	2.02604306	;
    RnonCO2[	467	]=	2.026128158	;
    RnonCO2[	468	]=	2.026190832	;
    RnonCO2[	469	]=	2.026262078	;
    RnonCO2[	470	]=	2.026373171	;
    RnonCO2[	471	]=	2.026527398	;
    RnonCO2[	472	]=	2.026705477	;
    RnonCO2[	473	]=	2.026889701	;
    RnonCO2[	474	]=	2.027072258	;
    RnonCO2[	475	]=	2.027247324	;
    RnonCO2[	476	]=	2.027364745	;
    RnonCO2[	477	]=	2.027389328	;
    RnonCO2[	478	]=	2.027355508	;
    RnonCO2[	479	]=	2.027300724	;
    RnonCO2[	480	]=	2.027255689	;
    RnonCO2[	481	]=	2.027251024	;
    RnonCO2[	482	]=	2.027290799	;
    RnonCO2[	483	]=	2.02735461	;
    RnonCO2[	484	]=	2.027426215	;
    RnonCO2[	485	]=	2.027461889	;
    RnonCO2[	486	]=	2.027461889	;
    RnonCO2[	487	]=	2.027461889	;
    RnonCO2[	488	]=	2.027461889	;
    RnonCO2[	489	]=	2.027461889	;
    RnonCO2[	490	]=	2.027461889	;
    RnonCO2[	491	]=	2.027461889	;
    RnonCO2[	492	]=	2.027461889	;
    RnonCO2[	493	]=	2.027461889	;
    RnonCO2[	494	]=	2.027461889	;
    RnonCO2[	495	]=	2.027461889	;
    RnonCO2[	496	]=	2.027461889	;
    RnonCO2[	497	]=	2.027461889	;
    RnonCO2[	498	]=	2.027461889	;
    RnonCO2[	499	]=	2.027461889	;
    RnonCO2[	500	]=	2.027461889	;
    RnonCO2[	501	]=	2.027461889	;
    RnonCO2[	502	]=	2.027461889	;
    RnonCO2[	503	]=	2.027461889	;
    RnonCO2[	504	]=	2.027461889	;
    RnonCO2[	505	]=	2.027461889	;
    RnonCO2[	506	]=	2.027461889	;
    RnonCO2[	507	]=	2.027461889	;
    RnonCO2[	508	]=	2.027461889	;
    RnonCO2[	509	]=	2.027461889	;
    RnonCO2[	510	]=	2.027461889	;
    RnonCO2[	511	]=	2.027461889	;
    RnonCO2[	512	]=	2.027461889	;
    RnonCO2[	513	]=	2.027461889	;
    RnonCO2[	514	]=	2.027461889	;
    RnonCO2[	515	]=	2.027461889	;
    RnonCO2[	516	]=	2.027461889	;
    RnonCO2[	517	]=	2.027461889	;
    RnonCO2[	518	]=	2.027461889	;
    RnonCO2[	519	]=	2.027461889	;
    RnonCO2[	520	]=	2.027461889	;
    RnonCO2[	521	]=	2.027461889	;
    RnonCO2[	522	]=	2.027461889	;
    RnonCO2[	523	]=	2.027461889	;
    RnonCO2[	524	]=	2.027461889	;
    RnonCO2[	525	]=	2.027461889	;
    RnonCO2[	526	]=	2.027461889	;
    RnonCO2[	527	]=	2.027461889	;
    RnonCO2[	528	]=	2.027461889	;
    RnonCO2[	529	]=	2.027461889	;
    RnonCO2[	530	]=	2.027461889	;
    RnonCO2[	531	]=	2.027461889	;
    RnonCO2[	532	]=	2.027461889	;
    RnonCO2[	533	]=	2.027461889	;
    RnonCO2[	534	]=	2.027461889	;
    RnonCO2[	535	]=	2.027461889	;
    RnonCO2[	536	]=	2.027461889	;
    RnonCO2[	537	]=	2.027461889	;
    RnonCO2[	538	]=	2.027461889	;
    RnonCO2[	539	]=	2.027461889	;
    RnonCO2[	540	]=	2.027461889	;
    RnonCO2[	541	]=	2.027461889	;
    RnonCO2[	542	]=	2.027461889	;
    RnonCO2[	543	]=	2.027461889	;
    RnonCO2[	544	]=	2.027461889	;
    RnonCO2[	545	]=	2.027461889	;
    RnonCO2[	546	]=	2.027461889	;
    RnonCO2[	547	]=	2.027461889	;
    RnonCO2[	548	]=	2.027461889	;
    RnonCO2[	549	]=	2.027461889	;
    RnonCO2[	550	]=	2.027461889	;
    RnonCO2[	551	]=	2.027461889	;
    RnonCO2[	552	]=	2.027461889	;
    RnonCO2[	553	]=	2.027461889	;
    RnonCO2[	554	]=	2.027461889	;
    RnonCO2[	555	]=	2.027461889	;
    RnonCO2[	556	]=	2.027461889	;
    RnonCO2[	557	]=	2.027461889	;
    RnonCO2[	558	]=	2.027461889	;
    RnonCO2[	559	]=	2.027461889	;
    RnonCO2[	560	]=	2.027461889	;
    RnonCO2[	561	]=	2.027461889	;
    RnonCO2[	562	]=	2.027461889	;
    RnonCO2[	563	]=	2.027461889	;
    RnonCO2[	564	]=	2.027461889	;
    RnonCO2[	565	]=	2.027461889	;
    RnonCO2[	566	]=	2.027461889	;
    RnonCO2[	567	]=	2.027461889	;
    RnonCO2[	568	]=	2.027461889	;
    RnonCO2[	569	]=	2.027461889	;
    RnonCO2[	570	]=	2.027461889	;
    RnonCO2[	571	]=	2.027461889	;
    RnonCO2[	572	]=	2.027461889	;
    RnonCO2[	573	]=	2.027461889	;
    RnonCO2[	574	]=	2.027461889	;
    RnonCO2[	575	]=	2.027461889	;
    RnonCO2[	576	]=	2.027461889	;
    RnonCO2[	577	]=	2.027461889	;
    RnonCO2[	578	]=	2.027461889	;
    RnonCO2[	579	]=	2.027461889	;
    RnonCO2[	580	]=	2.027461889	;
    RnonCO2[	581	]=	2.027461889	;
    RnonCO2[	582	]=	2.027461889	;
    RnonCO2[	583	]=	2.027461889	;
    RnonCO2[	584	]=	2.027461889	;
    RnonCO2[	585	]=	2.027461889	;
    RnonCO2[	586	]=	2.027461889	;
    RnonCO2[	587	]=	2.027461889	;
    RnonCO2[	588	]=	2.027461889	;
    RnonCO2[	589	]=	2.027461889	;
    RnonCO2[	590	]=	2.027461889	;
    RnonCO2[	591	]=	2.027461889	;
    RnonCO2[	592	]=	2.027461889	;
    RnonCO2[	593	]=	2.027461889	;
    RnonCO2[	594	]=	2.027461889	;
    RnonCO2[	595	]=	2.027461889	;
    RnonCO2[	596	]=	2.027461889	;
    RnonCO2[	597	]=	2.027461889	;
    RnonCO2[	598	]=	2.027461889	;
    RnonCO2[	599	]=	2.027461889	;
    RnonCO2[	600	]=	2.027461889	;
    RnonCO2[	601	]=	2.027461889	;
    RnonCO2[	602	]=	2.027461889	;
    RnonCO2[	603	]=	2.027461889	;
    RnonCO2[	604	]=	2.027461889	;
    RnonCO2[	605	]=	2.027461889	;
    RnonCO2[	606	]=	2.027461889	;
    RnonCO2[	607	]=	2.027461889	;
    RnonCO2[	608	]=	2.027461889	;
    RnonCO2[	609	]=	2.027461889	;
    RnonCO2[	610	]=	2.027461889	;
    RnonCO2[	611	]=	2.027461889	;
    RnonCO2[	612	]=	2.027461889	;
    RnonCO2[	613	]=	2.027461889	;
    RnonCO2[	614	]=	2.027461889	;
    RnonCO2[	615	]=	2.027461889	;
    RnonCO2[	616	]=	2.027461889	;
    RnonCO2[	617	]=	2.027461889	;
    RnonCO2[	618	]=	2.027461889	;
    RnonCO2[	619	]=	2.027461889	;
    RnonCO2[	620	]=	2.027461889	;
    RnonCO2[	621	]=	2.027461889	;
    RnonCO2[	622	]=	2.027461889	;
    RnonCO2[	623	]=	2.027461889	;
    RnonCO2[	624	]=	2.027461889	;
    RnonCO2[	625	]=	2.027461889	;
    RnonCO2[	626	]=	2.027461889	;
    RnonCO2[	627	]=	2.027461889	;
    RnonCO2[	628	]=	2.027461889	;
    RnonCO2[	629	]=	2.027461889	;
    RnonCO2[	630	]=	2.027461889	;
    RnonCO2[	631	]=	2.027461889	;
    RnonCO2[	632	]=	2.027461889	;
    RnonCO2[	633	]=	2.027461889	;
    RnonCO2[	634	]=	2.027461889	;
    RnonCO2[	635	]=	2.027461889	;
    RnonCO2[	636	]=	2.027461889	;
    RnonCO2[	637	]=	2.027461889	;
    RnonCO2[	638	]=	2.027461889	;
    RnonCO2[	639	]=	2.027461889	;
    RnonCO2[	640	]=	2.027461889	;
    RnonCO2[	641	]=	2.027461889	;
    RnonCO2[	642	]=	2.027461889	;
    RnonCO2[	643	]=	2.027461889	;
    RnonCO2[	644	]=	2.027461889	;
    RnonCO2[	645	]=	2.027461889	;
    RnonCO2[	646	]=	2.027461889	;
    RnonCO2[	647	]=	2.027461889	;
    RnonCO2[	648	]=	2.027461889	;
    RnonCO2[	649	]=	2.027461889	;
    RnonCO2[	650	]=	2.027461889	;
    RnonCO2[	651	]=	2.027461889	;
    RnonCO2[	652	]=	2.027461889	;
    RnonCO2[	653	]=	2.027461889	;
    RnonCO2[	654	]=	2.027461889	;
    RnonCO2[	655	]=	2.027461889	;
    RnonCO2[	656	]=	2.027461889	;
    RnonCO2[	657	]=	2.027461889	;
    RnonCO2[	658	]=	2.027461889	;
    RnonCO2[	659	]=	2.027461889	;
    RnonCO2[	660	]=	2.027461889	;
    RnonCO2[	661	]=	2.027461889	;
    RnonCO2[	662	]=	2.027461889	;
    RnonCO2[	663	]=	2.027461889	;
    RnonCO2[	664	]=	2.027461889	;
    RnonCO2[	665	]=	2.027461889	;
    RnonCO2[	666	]=	2.027461889	;
    RnonCO2[	667	]=	2.027461889	;
    RnonCO2[	668	]=	2.027461889	;
    RnonCO2[	669	]=	2.027461889	;
    RnonCO2[	670	]=	2.027461889	;
    RnonCO2[	671	]=	2.027461889	;
    RnonCO2[	672	]=	2.027461889	;
    RnonCO2[	673	]=	2.027461889	;
    RnonCO2[	674	]=	2.027461889	;
    RnonCO2[	675	]=	2.027461889	;
    RnonCO2[	676	]=	2.027461889	;
    RnonCO2[	677	]=	2.027461889	;
    RnonCO2[	678	]=	2.027461889	;
    RnonCO2[	679	]=	2.027461889	;
    RnonCO2[	680	]=	2.027461889	;
    RnonCO2[	681	]=	2.027461889	;
    RnonCO2[	682	]=	2.027461889	;
    RnonCO2[	683	]=	2.027461889	;
    RnonCO2[	684	]=	2.027461889	;
    RnonCO2[	685	]=	2.027461889	;
    RnonCO2[	686	]=	2.027461889	;
    RnonCO2[	687	]=	2.027461889	;
    RnonCO2[	688	]=	2.027461889	;
    RnonCO2[	689	]=	2.027461889	;
    RnonCO2[	690	]=	2.027461889	;
    RnonCO2[	691	]=	2.027461889	;
    RnonCO2[	692	]=	2.027461889	;
    RnonCO2[	693	]=	2.027461889	;
    RnonCO2[	694	]=	2.027461889	;
    RnonCO2[	695	]=	2.027461889	;
    RnonCO2[	696	]=	2.027461889	;
    RnonCO2[	697	]=	2.027461889	;
    RnonCO2[	698	]=	2.027461889	;
    RnonCO2[	699	]=	2.027461889	;
    RnonCO2[	700	]=	2.027461889	;
    RnonCO2[	701	]=	2.027461889	;
    RnonCO2[	702	]=	2.027461889	;
    RnonCO2[	703	]=	2.027461889	;
    RnonCO2[	704	]=	2.027461889	;
    RnonCO2[	705	]=	2.027461889	;
    RnonCO2[	706	]=	2.027461889	;
    RnonCO2[	707	]=	2.027461889	;
    RnonCO2[	708	]=	2.027461889	;
    RnonCO2[	709	]=	2.027461889	;
    RnonCO2[	710	]=	2.027461889	;
    RnonCO2[	711	]=	2.027461889	;
    RnonCO2[	712	]=	2.027461889	;
    RnonCO2[	713	]=	2.027461889	;
    RnonCO2[	714	]=	2.027461889	;
    RnonCO2[	715	]=	2.027461889	;
    RnonCO2[	716	]=	2.027461889	;
    RnonCO2[	717	]=	2.027461889	;
    RnonCO2[	718	]=	2.027461889	;
    RnonCO2[	719	]=	2.027461889	;
    RnonCO2[	720	]=	2.027461889	;
    RnonCO2[	721	]=	2.027461889	;
    RnonCO2[	722	]=	2.027461889	;
    RnonCO2[	723	]=	2.027461889	;
    RnonCO2[	724	]=	2.027461889	;
    RnonCO2[	725	]=	2.027461889	;
    RnonCO2[	726	]=	2.027461889	;
    RnonCO2[	727	]=	2.027461889	;
    RnonCO2[	728	]=	2.027461889	;
    RnonCO2[	729	]=	2.027461889	;
    RnonCO2[	730	]=	2.027461889	;
    RnonCO2[	731	]=	2.027461889	;
    RnonCO2[	732	]=	2.027461889	;
    RnonCO2[	733	]=	2.027461889	;
    RnonCO2[	734	]=	2.027461889	;
    RnonCO2[	735	]=	2.027461889	;
    
    for(int i = 736; i<tmax; i++)
    {
        RnonCO2[i] = RnonCO2[i-1];
    }

}

void setRnonCO2_RCP45_K(void)
{
    RnonCO2[	0	]=	-0.019988515	;
    RnonCO2[	1	]=	-0.005612142	;
    RnonCO2[	2	]=	0.008971239	;
    RnonCO2[	3	]=	0.009466172	;
    RnonCO2[	4	]=	0.009946833	;
    RnonCO2[	5	]=	0.010412166	;
    RnonCO2[	6	]=	0.010865438	;
    RnonCO2[	7	]=	0.011309666	;
    RnonCO2[	8	]=	0.011745593	;
    RnonCO2[	9	]=	0.012173039	;
    RnonCO2[	10	]=	0.012591208	;
    RnonCO2[	11	]=	0.012999626	;
    RnonCO2[	12	]=	0.013397144	;
    RnonCO2[	13	]=	0.013783559	;
    RnonCO2[	14	]=	0.014157995	;
    RnonCO2[	15	]=	0.014518817	;
    RnonCO2[	16	]=	0.014861789	;
    RnonCO2[	17	]=	0.015185328	;
    RnonCO2[	18	]=	0.015493438	;
    RnonCO2[	19	]=	0.015792171	;
    RnonCO2[	20	]=	0.01608699	;
    RnonCO2[	21	]=	0.016384013	;
    RnonCO2[	22	]=	0.016688621	;
    RnonCO2[	23	]=	0.017006739	;
    RnonCO2[	24	]=	0.017344141	;
    RnonCO2[	25	]=	0.017705767	;
    RnonCO2[	26	]=	0.018092359	;
    RnonCO2[	27	]=	0.018501885	;
    RnonCO2[	28	]=	0.01893258	;
    RnonCO2[	29	]=	0.019382558	;
    RnonCO2[	30	]=	0.019850303	;
    RnonCO2[	31	]=	0.020334085	;
    RnonCO2[	32	]=	0.020831989	;
    RnonCO2[	33	]=	0.021342706	;
    RnonCO2[	34	]=	0.02186457	;
    RnonCO2[	35	]=	0.022398729	;
    RnonCO2[	36	]=	0.022952686	;
    RnonCO2[	37	]=	0.023532049	;
    RnonCO2[	38	]=	0.024135881	;
    RnonCO2[	39	]=	0.024759327	;
    RnonCO2[	40	]=	0.025397837	;
    RnonCO2[	41	]=	0.02604713	;
    RnonCO2[	42	]=	0.026702498	;
    RnonCO2[	43	]=	0.02735936	;
    RnonCO2[	44	]=	0.028013477	;
    RnonCO2[	45	]=	0.028662413	;
    RnonCO2[	46	]=	0.029309198	;
    RnonCO2[	47	]=	0.029958961	;
    RnonCO2[	48	]=	0.030612845	;
    RnonCO2[	49	]=	0.031270152	;
    RnonCO2[	50	]=	0.03193112	;
    RnonCO2[	51	]=	0.032594644	;
    RnonCO2[	52	]=	0.033260857	;
    RnonCO2[	53	]=	0.033928967	;
    RnonCO2[	54	]=	0.034598246	;
    RnonCO2[	55	]=	0.035268641	;
    RnonCO2[	56	]=	0.035940357	;
    RnonCO2[	57	]=	0.036613717	;
    RnonCO2[	58	]=	0.037289228	;
    RnonCO2[	59	]=	0.037967253	;
    RnonCO2[	60	]=	0.038647628	;
    RnonCO2[	61	]=	0.039330893	;
    RnonCO2[	62	]=	0.040017299	;
    RnonCO2[	63	]=	0.040707068	;
    RnonCO2[	64	]=	0.041400385	;
    RnonCO2[	65	]=	0.042096128	;
    RnonCO2[	66	]=	0.042791515	;
    RnonCO2[	67	]=	0.04348473	;
    RnonCO2[	68	]=	0.044177707	;
    RnonCO2[	69	]=	0.044873365	;
    RnonCO2[	70	]=	0.045574528	;
    RnonCO2[	71	]=	0.04628438	;
    RnonCO2[	72	]=	0.047005924	;
    RnonCO2[	73	]=	0.047742202	;
    RnonCO2[	74	]=	0.048496247	;
    RnonCO2[	75	]=	0.049273302	;
    RnonCO2[	76	]=	0.050090919	;
    RnonCO2[	77	]=	0.050966437	;
    RnonCO2[	78	]=	0.051881638	;
    RnonCO2[	79	]=	0.052787675	;
    RnonCO2[	80	]=	0.053629807	;
    RnonCO2[	81	]=	0.054384454	;
    RnonCO2[	82	]=	0.055079735	;
    RnonCO2[	83	]=	0.055748092	;
    RnonCO2[	84	]=	0.056393965	;
    RnonCO2[	85	]=	0.057093398	;
    RnonCO2[	86	]=	0.05807606	;
    RnonCO2[	87	]=	0.059274862	;
    RnonCO2[	88	]=	0.060472717	;
    RnonCO2[	89	]=	0.061669627	;
    RnonCO2[	90	]=	0.062723294	;
    RnonCO2[	91	]=	0.063776572	;
    RnonCO2[	92	]=	0.064970976	;
    RnonCO2[	93	]=	0.066164765	;
    RnonCO2[	94	]=	0.067357381	;
    RnonCO2[	95	]=	0.06854924	;
    RnonCO2[	96	]=	0.069740139	;
    RnonCO2[	97	]=	0.070930146	;
    RnonCO2[	98	]=	0.072119404	;
    RnonCO2[	99	]=	0.073307822	;
    RnonCO2[	100	]=	0.074635976	;
    RnonCO2[	101	]=	0.075963149	;
    RnonCO2[	102	]=	0.077148556	;
    RnonCO2[	103	]=	0.078333267	;
    RnonCO2[	104	]=	0.079516971	;
    RnonCO2[	105	]=	0.080754531	;
    RnonCO2[	106	]=	0.081905797	;
    RnonCO2[	107	]=	0.083001419	;
    RnonCO2[	108	]=	0.084181461	;
    RnonCO2[	109	]=	0.085360786	;
    RnonCO2[	110	]=	0.086593261	;
    RnonCO2[	111	]=	0.087739428	;
    RnonCO2[	112	]=	0.088830817	;
    RnonCO2[	113	]=	0.090006409	;
    RnonCO2[	114	]=	0.091181226	;
    RnonCO2[	115	]=	0.092494155	;
    RnonCO2[	116	]=	0.093944335	;
    RnonCO2[	117	]=	0.095393111	;
    RnonCO2[	118	]=	0.096840323	;
    RnonCO2[	119	]=	0.098148223	;
    RnonCO2[	120	]=	0.09931678	;
    RnonCO2[	121	]=	0.100622634	;
    RnonCO2[	122	]=	0.101979664	;
    RnonCO2[	123	]=	0.10325049	;
    RnonCO2[	124	]=	0.104604961	;
    RnonCO2[	125	]=	0.106043011	;
    RnonCO2[	126	]=	0.107616808	;
    RnonCO2[	127	]=	0.109325606	;
    RnonCO2[	128	]=	0.110810739	;
    RnonCO2[	129	]=	0.112072699	;
    RnonCO2[	130	]=	0.113469426	;
    RnonCO2[	131	]=	0.115000677	;
    RnonCO2[	132	]=	0.116479164	;
    RnonCO2[	133	]=	0.117955956	;
    RnonCO2[	134	]=	0.119481632	;
    RnonCO2[	135	]=	0.121230683	;
    RnonCO2[	136	]=	0.123467298	;
    RnonCO2[	137	]=	0.125964505	;
    RnonCO2[	138	]=	0.128590857	;
    RnonCO2[	139	]=	0.131345679	;
    RnonCO2[	140	]=	0.134094266	;
    RnonCO2[	141	]=	0.136836274	;
    RnonCO2[	142	]=	0.139657159	;
    RnonCO2[	143	]=	0.142689092	;
    RnonCO2[	144	]=	0.145846541	;
    RnonCO2[	145	]=	0.148864539	;
    RnonCO2[	146	]=	0.151744205	;
    RnonCO2[	147	]=	0.154833698	;
    RnonCO2[	148	]=	0.158178102	;
    RnonCO2[	149	]=	0.161560167	;
    RnonCO2[	150	]=	0.165018225	;
    RnonCO2[	151	]=	0.168552404	;
    RnonCO2[	152	]=	0.172162279	;
    RnonCO2[	153	]=	0.175976823	;
    RnonCO2[	154	]=	0.179825831	;
    RnonCO2[	155	]=	0.18353714	;
    RnonCO2[	156	]=	0.18719586	;
    RnonCO2[	157	]=	0.190888601	;
    RnonCO2[	158	]=	0.194618313	;
    RnonCO2[	159	]=	0.198342475	;
    RnonCO2[	160	]=	0.202057589	;
    RnonCO2[	161	]=	0.20584786	;
    RnonCO2[	162	]=	0.209629466	;
    RnonCO2[	163	]=	0.213233356	;
    RnonCO2[	164	]=	0.216704098	;
    RnonCO2[	165	]=	0.220042715	;
    RnonCO2[	166	]=	0.223296521	;
    RnonCO2[	167	]=	0.226627686	;
    RnonCO2[	168	]=	0.230030664	;
    RnonCO2[	169	]=	0.233304361	;
    RnonCO2[	170	]=	0.236449311	;
    RnonCO2[	171	]=	0.239588758	;
    RnonCO2[	172	]=	0.24263915	;
    RnonCO2[	173	]=	0.245600759	;
    RnonCO2[	174	]=	0.248557521	;
    RnonCO2[	175	]=	0.251630431	;
    RnonCO2[	176	]=	0.254903022	;
    RnonCO2[	177	]=	0.258133328	;
    RnonCO2[	178	]=	0.26123828	;
    RnonCO2[	179	]=	0.264542	;
    RnonCO2[	180	]=	0.268044049	;
    RnonCO2[	181	]=	0.27165959	;
    RnonCO2[	182	]=	0.275506693	;
    RnonCO2[	183	]=	0.279465183	;
    RnonCO2[	184	]=	0.283533907	;
    RnonCO2[	185	]=	0.287746496	;
    RnonCO2[	186	]=	0.292103558	;
    RnonCO2[	187	]=	0.296886349	;
    RnonCO2[	188	]=	0.302089492	;
    RnonCO2[	189	]=	0.307309759	;
    RnonCO2[	190	]=	0.312629337	;
    RnonCO2[	191	]=	0.318245324	;
    RnonCO2[	192	]=	0.323988656	;
    RnonCO2[	193	]=	0.329826066	;
    RnonCO2[	194	]=	0.335923764	;
    RnonCO2[	195	]=	0.342309427	;
    RnonCO2[	196	]=	0.348815284	;
    RnonCO2[	197	]=	0.355439785	;
    RnonCO2[	198	]=	0.362318532	;
    RnonCO2[	199	]=	0.369551061	;
    RnonCO2[	200	]=	0.377072615	;
    RnonCO2[	201	]=	0.384665115	;
    RnonCO2[	202	]=	0.392331499	;
    RnonCO2[	203	]=	0.400273286	;
    RnonCO2[	204	]=	0.408559205	;
    RnonCO2[	205	]=	0.416991838	;
    RnonCO2[	206	]=	0.425389986	;
    RnonCO2[	207	]=	0.433949678	;
    RnonCO2[	208	]=	0.442740432	;
    RnonCO2[	209	]=	0.451573488	;
    RnonCO2[	210	]=	0.460373303	;
    RnonCO2[	211	]=	0.469303629	;
    RnonCO2[	212	]=	0.478507559	;
    RnonCO2[	213	]=	0.48782133	;
    RnonCO2[	214	]=	0.49689881	;
    RnonCO2[	215	]=	0.506302348	;
    RnonCO2[	216	]=	0.516911269	;
    RnonCO2[	217	]=	0.527818076	;
    RnonCO2[	218	]=	0.538162703	;
    RnonCO2[	219	]=	0.547403268	;
    RnonCO2[	220	]=	0.555524533	;
    RnonCO2[	221	]=	0.562888474	;
    RnonCO2[	222	]=	0.569356555	;
    RnonCO2[	223	]=	0.576908971	;
    RnonCO2[	224	]=	0.585481654	;
    RnonCO2[	225	]=	0.59287455	;
    RnonCO2[	226	]=	0.599146788	;
    RnonCO2[	227	]=	0.604130931	;
    RnonCO2[	228	]=	0.607750235	;
    RnonCO2[	229	]=	0.611664417	;
    RnonCO2[	230	]=	0.616444422	;
    RnonCO2[	231	]=	0.620912678	;
    RnonCO2[	232	]=	0.625821808	;
    RnonCO2[	233	]=	0.632177868	;
    RnonCO2[	234	]=	0.637926114	;
    RnonCO2[	235	]=	0.641532814	;
    RnonCO2[	236	]=	0.644675991	;
    RnonCO2[	237	]=	0.649064599	;
    RnonCO2[	238	]=	0.653980688	;
    RnonCO2[	239	]=	0.657858713	;
    RnonCO2[	240	]=	0.661298844	;
    RnonCO2[	241	]=	0.665634157	;
    RnonCO2[	242	]=	0.670529914	;
    RnonCO2[	243	]=	0.675587001	;
    RnonCO2[	244	]=	0.680736329	;
    RnonCO2[	245	]=	0.685870628	;
    RnonCO2[	246	]=	0.690993553	;
    RnonCO2[	247	]=	0.696130619	;
    RnonCO2[	248	]=	0.701297116	;
    RnonCO2[	249	]=	0.70649666	;
    RnonCO2[	250	]=	0.711736812	;
    RnonCO2[	251	]=	0.717011527	;
    RnonCO2[	252	]=	0.72230557	;
    RnonCO2[	253	]=	0.727608606	;
    RnonCO2[	254	]=	0.732913799	;
    RnonCO2[	255	]=	0.738215633	;
    RnonCO2[	256	]=	0.74348198	;
    RnonCO2[	257	]=	0.748612274	;
    RnonCO2[	258	]=	0.753553841	;
    RnonCO2[	259	]=	0.758340003	;
    RnonCO2[	260	]=	0.763000012	;
    RnonCO2[	261	]=	0.767562832	;
    RnonCO2[	262	]=	0.7720411	;
    RnonCO2[	263	]=	0.776435067	;
    RnonCO2[	264	]=	0.780747575	;
    RnonCO2[	265	]=	0.784984219	;
    RnonCO2[	266	]=	0.789149845	;
    RnonCO2[	267	]=	0.793173553	;
    RnonCO2[	268	]=	0.797002046	;
    RnonCO2[	269	]=	0.800669943	;
    RnonCO2[	270	]=	0.804204959	;
    RnonCO2[	271	]=	0.80763147	;
    RnonCO2[	272	]=	0.810974871	;
    RnonCO2[	273	]=	0.814243863	;
    RnonCO2[	274	]=	0.817436156	;
    RnonCO2[	275	]=	0.82055173	;
    RnonCO2[	276	]=	0.823594261	;
    RnonCO2[	277	]=	0.826514891	;
    RnonCO2[	278	]=	0.829241596	;
    RnonCO2[	279	]=	0.831770848	;
    RnonCO2[	280	]=	0.834137688	;
    RnonCO2[	281	]=	0.836370055	;
    RnonCO2[	282	]=	0.838492829	;
    RnonCO2[	283	]=	0.840530743	;
    RnonCO2[	284	]=	0.842492923	;
    RnonCO2[	285	]=	0.84437567	;
    RnonCO2[	286	]=	0.846180003	;
    RnonCO2[	287	]=	0.847843638	;
    RnonCO2[	288	]=	0.849308463	;
    RnonCO2[	289	]=	0.850557103	;
    RnonCO2[	290	]=	0.851584756	;
    RnonCO2[	291	]=	0.852426234	;
    RnonCO2[	292	]=	0.85310994	;
    RnonCO2[	293	]=	0.853661132	;
    RnonCO2[	294	]=	0.854105702	;
    RnonCO2[	295	]=	0.854453356	;
    RnonCO2[	296	]=	0.854701783	;
    RnonCO2[	297	]=	0.854837319	;
    RnonCO2[	298	]=	0.854850175	;
    RnonCO2[	299	]=	0.854747089	;
    RnonCO2[	300	]=	0.854505888	;
    RnonCO2[	301	]=	0.854116877	;
    RnonCO2[	302	]=	0.853609875	;
    RnonCO2[	303	]=	0.853008964	;
    RnonCO2[	304	]=	0.8523353	;
    RnonCO2[	305	]=	0.851610232	;
    RnonCO2[	306	]=	0.850839959	;
    RnonCO2[	307	]=	0.850000835	;
    RnonCO2[	308	]=	0.849074176	;
    RnonCO2[	309	]=	0.848064773	;
    RnonCO2[	310	]=	0.846977418	;
    RnonCO2[	311	]=	0.845788435	;
    RnonCO2[	312	]=	0.844486059	;
    RnonCO2[	313	]=	0.84310598	;
    RnonCO2[	314	]=	0.841720747	;
    RnonCO2[	315	]=	0.840386939	;
    RnonCO2[	316	]=	0.839115223	;
    RnonCO2[	317	]=	0.837993523	;
    RnonCO2[	318	]=	0.837088599	;
    RnonCO2[	319	]=	0.836371158	;
    RnonCO2[	320	]=	0.835818624	;
    RnonCO2[	321	]=	0.835411217	;
    RnonCO2[	322	]=	0.83510403	;
    RnonCO2[	323	]=	0.834866976	;
    RnonCO2[	324	]=	0.834710465	;
    RnonCO2[	325	]=	0.834640141	;
    RnonCO2[	326	]=	0.834659943	;
    RnonCO2[	327	]=	0.83477365	;
    RnonCO2[	328	]=	0.834971919	;
    RnonCO2[	329	]=	0.835236945	;
    RnonCO2[	330	]=	0.835556043	;
    RnonCO2[	331	]=	0.83592086	;
    RnonCO2[	332	]=	0.836324233	;
    RnonCO2[	333	]=	0.836732702	;
    RnonCO2[	334	]=	0.837126854	;
    RnonCO2[	335	]=	0.837525939	;
    RnonCO2[	336	]=	0.837944032	;
    RnonCO2[	337	]=	0.838420299	;
    RnonCO2[	338	]=	0.838988687	;
    RnonCO2[	339	]=	0.839635427	;
    RnonCO2[	340	]=	0.840337905	;
    RnonCO2[	341	]=	0.841079484	;
    RnonCO2[	342	]=	0.841848698	;
    RnonCO2[	343	]=	0.842635996	;
    RnonCO2[	344	]=	0.843406258	;
    RnonCO2[	345	]=	0.84413876	;
    RnonCO2[	346	]=	0.844851542	;
    RnonCO2[	347	]=	0.845557887	;
    RnonCO2[	348	]=	0.846268106	;
    RnonCO2[	349	]=	0.846993979	;
    RnonCO2[	350	]=	0.847732563	;
    RnonCO2[	351	]=	0.848470673	;
    RnonCO2[	352	]=	0.849199804	;
    RnonCO2[	353	]=	0.849915577	;
    RnonCO2[	354	]=	0.850614289	;
    RnonCO2[	355	]=	0.85126614	;
    RnonCO2[	356	]=	0.8518549	;
    RnonCO2[	357	]=	0.852402723	;
    RnonCO2[	358	]=	0.852925949	;
    RnonCO2[	359	]=	0.853437881	;
    RnonCO2[	360	]=	0.853952685	;
    RnonCO2[	361	]=	0.854469538	;
    RnonCO2[	362	]=	0.854977194	;
    RnonCO2[	363	]=	0.855468558	;
    RnonCO2[	364	]=	0.855940564	;
    RnonCO2[	365	]=	0.856390882	;
    RnonCO2[	366	]=	0.85679076	;
    RnonCO2[	367	]=	0.857124626	;
    RnonCO2[	368	]=	0.857415471	;
    RnonCO2[	369	]=	0.857680222	;
    RnonCO2[	370	]=	0.857932868	;
    RnonCO2[	371	]=	0.858188115	;
    RnonCO2[	372	]=	0.858445523	;
    RnonCO2[	373	]=	0.858694192	;
    RnonCO2[	374	]=	0.858927436	;
    RnonCO2[	375	]=	0.859142369	;
    RnonCO2[	376	]=	0.859336984	;
    RnonCO2[	377	]=	0.859482615	;
    RnonCO2[	378	]=	0.859563926	;
    RnonCO2[	379	]=	0.859603978	;
    RnonCO2[	380	]=	0.859620132	;
    RnonCO2[	381	]=	0.859626195	;
    RnonCO2[	382	]=	0.859636858	;
    RnonCO2[	383	]=	0.859651871	;
    RnonCO2[	384	]=	0.859660475	;
    RnonCO2[	385	]=	0.859662003	;
    RnonCO2[	386	]=	0.859662003	;
    RnonCO2[	387	]=	0.859662003	;
    RnonCO2[	388	]=	0.859662003	;
    RnonCO2[	389	]=	0.859662003	;
    RnonCO2[	390	]=	0.859662003	;
    RnonCO2[	391	]=	0.859662003	;
    RnonCO2[	392	]=	0.859662003	;
    RnonCO2[	393	]=	0.859662003	;
    RnonCO2[	394	]=	0.859662003	;
    RnonCO2[	395	]=	0.859662003	;
    RnonCO2[	396	]=	0.859662003	;
    RnonCO2[	397	]=	0.859662003	;
    RnonCO2[	398	]=	0.859662003	;
    RnonCO2[	399	]=	0.859662003	;
    RnonCO2[	400	]=	0.859662003	;
    RnonCO2[	401	]=	0.859662003	;
    RnonCO2[	402	]=	0.859662003	;
    RnonCO2[	403	]=	0.859662003	;
    RnonCO2[	404	]=	0.859662003	;
    RnonCO2[	405	]=	0.859662003	;
    RnonCO2[	406	]=	0.859662003	;
    RnonCO2[	407	]=	0.859662003	;
    RnonCO2[	408	]=	0.859662003	;
    RnonCO2[	409	]=	0.859662003	;
    RnonCO2[	410	]=	0.859662003	;
    RnonCO2[	411	]=	0.859662003	;
    RnonCO2[	412	]=	0.859662003	;
    RnonCO2[	413	]=	0.859662003	;
    RnonCO2[	414	]=	0.859662003	;
    RnonCO2[	415	]=	0.859662003	;
    RnonCO2[	416	]=	0.859662003	;
    RnonCO2[	417	]=	0.859662003	;
    RnonCO2[	418	]=	0.859662003	;
    RnonCO2[	419	]=	0.859662003	;
    RnonCO2[	420	]=	0.859662003	;
    RnonCO2[	421	]=	0.859662003	;
    RnonCO2[	422	]=	0.859662003	;
    RnonCO2[	423	]=	0.859662003	;
    RnonCO2[	424	]=	0.859662003	;
    RnonCO2[	425	]=	0.859662003	;
    RnonCO2[	426	]=	0.859662003	;
    RnonCO2[	427	]=	0.859662003	;
    RnonCO2[	428	]=	0.859662003	;
    RnonCO2[	429	]=	0.859662003	;
    RnonCO2[	430	]=	0.859662003	;
    RnonCO2[	431	]=	0.859662003	;
    RnonCO2[	432	]=	0.859662003	;
    RnonCO2[	433	]=	0.859662003	;
    RnonCO2[	434	]=	0.859662003	;
    RnonCO2[	435	]=	0.859662003	;
    RnonCO2[	436	]=	0.859662003	;
    RnonCO2[	437	]=	0.859662003	;
    RnonCO2[	438	]=	0.859662003	;
    RnonCO2[	439	]=	0.859662003	;
    RnonCO2[	440	]=	0.859662003	;
    RnonCO2[	441	]=	0.859662003	;
    RnonCO2[	442	]=	0.859662003	;
    RnonCO2[	443	]=	0.859662003	;
    RnonCO2[	444	]=	0.859662003	;
    RnonCO2[	445	]=	0.859662003	;
    RnonCO2[	446	]=	0.859662003	;
    RnonCO2[	447	]=	0.859662003	;
    RnonCO2[	448	]=	0.859662003	;
    RnonCO2[	449	]=	0.859662003	;
    RnonCO2[	450	]=	0.859662003	;
    RnonCO2[	451	]=	0.859662003	;
    RnonCO2[	452	]=	0.859662003	;
    RnonCO2[	453	]=	0.859662003	;
    RnonCO2[	454	]=	0.859662003	;
    RnonCO2[	455	]=	0.859662003	;
    RnonCO2[	456	]=	0.859662003	;
    RnonCO2[	457	]=	0.859662003	;
    RnonCO2[	458	]=	0.859662003	;
    RnonCO2[	459	]=	0.859662003	;
    RnonCO2[	460	]=	0.859662003	;
    RnonCO2[	461	]=	0.859662003	;
    RnonCO2[	462	]=	0.859662003	;
    RnonCO2[	463	]=	0.859662003	;
    RnonCO2[	464	]=	0.859662003	;
    RnonCO2[	465	]=	0.859662003	;
    RnonCO2[	466	]=	0.859662003	;
    RnonCO2[	467	]=	0.859662003	;
    RnonCO2[	468	]=	0.859662003	;
    RnonCO2[	469	]=	0.859662003	;
    RnonCO2[	470	]=	0.859662003	;
    RnonCO2[	471	]=	0.859662003	;
    RnonCO2[	472	]=	0.859662003	;
    RnonCO2[	473	]=	0.859662003	;
    RnonCO2[	474	]=	0.859662003	;
    RnonCO2[	475	]=	0.859662003	;
    RnonCO2[	476	]=	0.859662003	;
    RnonCO2[	477	]=	0.859662003	;
    RnonCO2[	478	]=	0.859662003	;
    RnonCO2[	479	]=	0.859662003	;
    RnonCO2[	480	]=	0.859662003	;
    RnonCO2[	481	]=	0.859662003	;
    RnonCO2[	482	]=	0.859662003	;
    RnonCO2[	483	]=	0.859662003	;
    RnonCO2[	484	]=	0.859662003	;
    RnonCO2[	485	]=	0.859662003	;
    RnonCO2[	486	]=	0.859662003	;
    RnonCO2[	487	]=	0.859662003	;
    RnonCO2[	488	]=	0.859662003	;
    RnonCO2[	489	]=	0.859662003	;
    RnonCO2[	490	]=	0.859662003	;
    RnonCO2[	491	]=	0.859662003	;
    RnonCO2[	492	]=	0.859662003	;
    RnonCO2[	493	]=	0.859662003	;
    RnonCO2[	494	]=	0.859662003	;
    RnonCO2[	495	]=	0.859662003	;
    RnonCO2[	496	]=	0.859662003	;
    RnonCO2[	497	]=	0.859662003	;
    RnonCO2[	498	]=	0.859662003	;
    RnonCO2[	499	]=	0.859662003	;
    RnonCO2[	500	]=	0.859662003	;
    RnonCO2[	501	]=	0.859662003	;
    RnonCO2[	502	]=	0.859662003	;
    RnonCO2[	503	]=	0.859662003	;
    RnonCO2[	504	]=	0.859662003	;
    RnonCO2[	505	]=	0.859662003	;
    RnonCO2[	506	]=	0.859662003	;
    RnonCO2[	507	]=	0.859662003	;
    RnonCO2[	508	]=	0.859662003	;
    RnonCO2[	509	]=	0.859662003	;
    RnonCO2[	510	]=	0.859662003	;
    RnonCO2[	511	]=	0.859662003	;
    RnonCO2[	512	]=	0.859662003	;
    RnonCO2[	513	]=	0.859662003	;
    RnonCO2[	514	]=	0.859662003	;
    RnonCO2[	515	]=	0.859662003	;
    RnonCO2[	516	]=	0.859662003	;
    RnonCO2[	517	]=	0.859662003	;
    RnonCO2[	518	]=	0.859662003	;
    RnonCO2[	519	]=	0.859662003	;
    RnonCO2[	520	]=	0.859662003	;
    RnonCO2[	521	]=	0.859662003	;
    RnonCO2[	522	]=	0.859662003	;
    RnonCO2[	523	]=	0.859662003	;
    RnonCO2[	524	]=	0.859662003	;
    RnonCO2[	525	]=	0.859662003	;
    RnonCO2[	526	]=	0.859662003	;
    RnonCO2[	527	]=	0.859662003	;
    RnonCO2[	528	]=	0.859662003	;
    RnonCO2[	529	]=	0.859662003	;
    RnonCO2[	530	]=	0.859662003	;
    RnonCO2[	531	]=	0.859662003	;
    RnonCO2[	532	]=	0.859662003	;
    RnonCO2[	533	]=	0.859662003	;
    RnonCO2[	534	]=	0.859662003	;
    RnonCO2[	535	]=	0.859662003	;
    RnonCO2[	536	]=	0.859662003	;
    RnonCO2[	537	]=	0.859662003	;
    RnonCO2[	538	]=	0.859662003	;
    RnonCO2[	539	]=	0.859662003	;
    RnonCO2[	540	]=	0.859662003	;
    RnonCO2[	541	]=	0.859662003	;
    RnonCO2[	542	]=	0.859662003	;
    RnonCO2[	543	]=	0.859662003	;
    RnonCO2[	544	]=	0.859662003	;
    RnonCO2[	545	]=	0.859662003	;
    RnonCO2[	546	]=	0.859662003	;
    RnonCO2[	547	]=	0.859662003	;
    RnonCO2[	548	]=	0.859662003	;
    RnonCO2[	549	]=	0.859662003	;
    RnonCO2[	550	]=	0.859662003	;
    RnonCO2[	551	]=	0.859662003	;
    RnonCO2[	552	]=	0.859662003	;
    RnonCO2[	553	]=	0.859662003	;
    RnonCO2[	554	]=	0.859662003	;
    RnonCO2[	555	]=	0.859662003	;
    RnonCO2[	556	]=	0.859662003	;
    RnonCO2[	557	]=	0.859662003	;
    RnonCO2[	558	]=	0.859662003	;
    RnonCO2[	559	]=	0.859662003	;
    RnonCO2[	560	]=	0.859662003	;
    RnonCO2[	561	]=	0.859662003	;
    RnonCO2[	562	]=	0.859662003	;
    RnonCO2[	563	]=	0.859662003	;
    RnonCO2[	564	]=	0.859662003	;
    RnonCO2[	565	]=	0.859662003	;
    RnonCO2[	566	]=	0.859662003	;
    RnonCO2[	567	]=	0.859662003	;
    RnonCO2[	568	]=	0.859662003	;
    RnonCO2[	569	]=	0.859662003	;
    RnonCO2[	570	]=	0.859662003	;
    RnonCO2[	571	]=	0.859662003	;
    RnonCO2[	572	]=	0.859662003	;
    RnonCO2[	573	]=	0.859662003	;
    RnonCO2[	574	]=	0.859662003	;
    RnonCO2[	575	]=	0.859662003	;
    RnonCO2[	576	]=	0.859662003	;
    RnonCO2[	577	]=	0.859662003	;
    RnonCO2[	578	]=	0.859662003	;
    RnonCO2[	579	]=	0.859662003	;
    RnonCO2[	580	]=	0.859662003	;
    RnonCO2[	581	]=	0.859662003	;
    RnonCO2[	582	]=	0.859662003	;
    RnonCO2[	583	]=	0.859662003	;
    RnonCO2[	584	]=	0.859662003	;
    RnonCO2[	585	]=	0.859662003	;
    RnonCO2[	586	]=	0.859662003	;
    RnonCO2[	587	]=	0.859662003	;
    RnonCO2[	588	]=	0.859662003	;
    RnonCO2[	589	]=	0.859662003	;
    RnonCO2[	590	]=	0.859662003	;
    RnonCO2[	591	]=	0.859662003	;
    RnonCO2[	592	]=	0.859662003	;
    RnonCO2[	593	]=	0.859662003	;
    RnonCO2[	594	]=	0.859662003	;
    RnonCO2[	595	]=	0.859662003	;
    RnonCO2[	596	]=	0.859662003	;
    RnonCO2[	597	]=	0.859662003	;
    RnonCO2[	598	]=	0.859662003	;
    RnonCO2[	599	]=	0.859662003	;
    RnonCO2[	600	]=	0.859662003	;
    RnonCO2[	601	]=	0.859662003	;
    RnonCO2[	602	]=	0.859662003	;
    RnonCO2[	603	]=	0.859662003	;
    RnonCO2[	604	]=	0.859662003	;
    RnonCO2[	605	]=	0.859662003	;
    RnonCO2[	606	]=	0.859662003	;
    RnonCO2[	607	]=	0.859662003	;
    RnonCO2[	608	]=	0.859662003	;
    RnonCO2[	609	]=	0.859662003	;
    RnonCO2[	610	]=	0.859662003	;
    RnonCO2[	611	]=	0.859662003	;
    RnonCO2[	612	]=	0.859662003	;
    RnonCO2[	613	]=	0.859662003	;
    RnonCO2[	614	]=	0.859662003	;
    RnonCO2[	615	]=	0.859662003	;
    RnonCO2[	616	]=	0.859662003	;
    RnonCO2[	617	]=	0.859662003	;
    RnonCO2[	618	]=	0.859662003	;
    RnonCO2[	619	]=	0.859662003	;
    RnonCO2[	620	]=	0.859662003	;
    RnonCO2[	621	]=	0.859662003	;
    RnonCO2[	622	]=	0.859662003	;
    RnonCO2[	623	]=	0.859662003	;
    RnonCO2[	624	]=	0.859662003	;
    RnonCO2[	625	]=	0.859662003	;
    RnonCO2[	626	]=	0.859662003	;
    RnonCO2[	627	]=	0.859662003	;
    RnonCO2[	628	]=	0.859662003	;
    RnonCO2[	629	]=	0.859662003	;
    RnonCO2[	630	]=	0.859662003	;
    RnonCO2[	631	]=	0.859662003	;
    RnonCO2[	632	]=	0.859662003	;
    RnonCO2[	633	]=	0.859662003	;
    RnonCO2[	634	]=	0.859662003	;
    RnonCO2[	635	]=	0.859662003	;
    RnonCO2[	636	]=	0.859662003	;
    RnonCO2[	637	]=	0.859662003	;
    RnonCO2[	638	]=	0.859662003	;
    RnonCO2[	639	]=	0.859662003	;
    RnonCO2[	640	]=	0.859662003	;
    RnonCO2[	641	]=	0.859662003	;
    RnonCO2[	642	]=	0.859662003	;
    RnonCO2[	643	]=	0.859662003	;
    RnonCO2[	644	]=	0.859662003	;
    RnonCO2[	645	]=	0.859662003	;
    RnonCO2[	646	]=	0.859662003	;
    RnonCO2[	647	]=	0.859662003	;
    RnonCO2[	648	]=	0.859662003	;
    RnonCO2[	649	]=	0.859662003	;
    RnonCO2[	650	]=	0.859662003	;
    RnonCO2[	651	]=	0.859662003	;
    RnonCO2[	652	]=	0.859662003	;
    RnonCO2[	653	]=	0.859662003	;
    RnonCO2[	654	]=	0.859662003	;
    RnonCO2[	655	]=	0.859662003	;
    RnonCO2[	656	]=	0.859662003	;
    RnonCO2[	657	]=	0.859662003	;
    RnonCO2[	658	]=	0.859662003	;
    RnonCO2[	659	]=	0.859662003	;
    RnonCO2[	660	]=	0.859662003	;
    RnonCO2[	661	]=	0.859662003	;
    RnonCO2[	662	]=	0.859662003	;
    RnonCO2[	663	]=	0.859662003	;
    RnonCO2[	664	]=	0.859662003	;
    RnonCO2[	665	]=	0.859662003	;
    RnonCO2[	666	]=	0.859662003	;
    RnonCO2[	667	]=	0.859662003	;
    RnonCO2[	668	]=	0.859662003	;
    RnonCO2[	669	]=	0.859662003	;
    RnonCO2[	670	]=	0.859662003	;
    RnonCO2[	671	]=	0.859662003	;
    RnonCO2[	672	]=	0.859662003	;
    RnonCO2[	673	]=	0.859662003	;
    RnonCO2[	674	]=	0.859662003	;
    RnonCO2[	675	]=	0.859662003	;
    RnonCO2[	676	]=	0.859662003	;
    RnonCO2[	677	]=	0.859662003	;
    RnonCO2[	678	]=	0.859662003	;
    RnonCO2[	679	]=	0.859662003	;
    RnonCO2[	680	]=	0.859662003	;
    RnonCO2[	681	]=	0.859662003	;
    RnonCO2[	682	]=	0.859662003	;
    RnonCO2[	683	]=	0.859662003	;
    RnonCO2[	684	]=	0.859662003	;
    RnonCO2[	685	]=	0.859662003	;
    RnonCO2[	686	]=	0.859662003	;
    RnonCO2[	687	]=	0.859662003	;
    RnonCO2[	688	]=	0.859662003	;
    RnonCO2[	689	]=	0.859662003	;
    RnonCO2[	690	]=	0.859662003	;
    RnonCO2[	691	]=	0.859662003	;
    RnonCO2[	692	]=	0.859662003	;
    RnonCO2[	693	]=	0.859662003	;
    RnonCO2[	694	]=	0.859662003	;
    RnonCO2[	695	]=	0.859662003	;
    RnonCO2[	696	]=	0.859662003	;
    RnonCO2[	697	]=	0.859662003	;
    RnonCO2[	698	]=	0.859662003	;
    RnonCO2[	699	]=	0.859662003	;
    RnonCO2[	700	]=	0.859662003	;
    RnonCO2[	701	]=	0.859662003	;
    RnonCO2[	702	]=	0.859662003	;
    RnonCO2[	703	]=	0.859662003	;
    RnonCO2[	704	]=	0.859662003	;
    RnonCO2[	705	]=	0.859662003	;
    RnonCO2[	706	]=	0.859662003	;
    RnonCO2[	707	]=	0.859662003	;
    RnonCO2[	708	]=	0.859662003	;
    RnonCO2[	709	]=	0.859662003	;
    RnonCO2[	710	]=	0.859662003	;
    RnonCO2[	711	]=	0.859662003	;
    RnonCO2[	712	]=	0.859662003	;
    RnonCO2[	713	]=	0.859662003	;
    RnonCO2[	714	]=	0.859662003	;
    RnonCO2[	715	]=	0.859662003	;
    RnonCO2[	716	]=	0.859662003	;
    RnonCO2[	717	]=	0.859662003	;
    RnonCO2[	718	]=	0.859662003	;
    RnonCO2[	719	]=	0.859662003	;
    RnonCO2[	720	]=	0.859662003	;
    RnonCO2[	721	]=	0.859662003	;
    RnonCO2[	722	]=	0.859662003	;
    RnonCO2[	723	]=	0.859662003	;
    RnonCO2[	724	]=	0.859662003	;
    RnonCO2[	725	]=	0.859662003	;
    RnonCO2[	726	]=	0.859662003	;
    RnonCO2[	727	]=	0.859662003	;
    RnonCO2[	728	]=	0.859662003	;
    RnonCO2[	729	]=	0.859662003	;
    RnonCO2[	730	]=	0.859662003	;
    RnonCO2[	731	]=	0.859662003	;
    RnonCO2[	732	]=	0.859662003	;
    RnonCO2[	733	]=	0.859662003	;
    RnonCO2[	734	]=	0.859662003	;
    RnonCO2[	735	]=	0.859662003	;
    
    for(int i = 736; i<tmax; i++)
    {
        RnonCO2[i] = RnonCO2[i-1];
    }
    
    
}



void setRnonCO2_RCP60K(void)
{
    RnonCO2[	0	]=	-0.01999748	;
    RnonCO2[	1	]=	-0.00561466	;
    RnonCO2[	2	]=	0.00897526	;
    RnonCO2[	3	]=	0.00947042	;
    RnonCO2[	4	]=	0.00995129	;
    RnonCO2[	5	]=	0.01041683	;
    RnonCO2[	6	]=	0.01087031	;
    RnonCO2[	7	]=	0.01131474	;
    RnonCO2[	8	]=	0.01175086	;
    RnonCO2[	9	]=	0.01217850	;
    RnonCO2[	10	]=	0.01259685	;
    RnonCO2[	11	]=	0.01300545	;
    RnonCO2[	12	]=	0.01340315	;
    RnonCO2[	13	]=	0.01378974	;
    RnonCO2[	14	]=	0.01416434	;
    RnonCO2[	15	]=	0.01452533	;
    RnonCO2[	16	]=	0.01486845	;
    RnonCO2[	17	]=	0.01519214	;
    RnonCO2[	18	]=	0.01550038	;
    RnonCO2[	19	]=	0.01579925	;
    RnonCO2[	20	]=	0.01609420	;
    RnonCO2[	21	]=	0.01639136	;
    RnonCO2[	22	]=	0.01669610	;
    RnonCO2[	23	]=	0.01701436	;
    RnonCO2[	24	]=	0.01735192	;
    RnonCO2[	25	]=	0.01771370	;
    RnonCO2[	26	]=	0.01810047	;
    RnonCO2[	27	]=	0.01851018	;
    RnonCO2[	28	]=	0.01894107	;
    RnonCO2[	29	]=	0.01939125	;
    RnonCO2[	30	]=	0.01985920	;
    RnonCO2[	31	]=	0.02034320	;
    RnonCO2[	32	]=	0.02084133	;
    RnonCO2[	33	]=	0.02135227	;
    RnonCO2[	34	]=	0.02187437	;
    RnonCO2[	35	]=	0.02240877	;
    RnonCO2[	36	]=	0.02296298	;
    RnonCO2[	37	]=	0.02354260	;
    RnonCO2[	38	]=	0.02414670	;
    RnonCO2[	39	]=	0.02477043	;
    RnonCO2[	40	]=	0.02540922	;
    RnonCO2[	41	]=	0.02605881	;
    RnonCO2[	42	]=	0.02671447	;
    RnonCO2[	43	]=	0.02737163	;
    RnonCO2[	44	]=	0.02802604	;
    RnonCO2[	45	]=	0.02867526	;
    RnonCO2[	46	]=	0.02932234	;
    RnonCO2[	47	]=	0.02997239	;
    RnonCO2[	48	]=	0.03062657	;
    RnonCO2[	49	]=	0.03128417	;
    RnonCO2[	50	]=	0.03194544	;
    RnonCO2[	51	]=	0.03260926	;
    RnonCO2[	52	]=	0.03327577	;
    RnonCO2[	53	]=	0.03394418	;
    RnonCO2[	54	]=	0.03461376	;
    RnonCO2[	55	]=	0.03528445	;
    RnonCO2[	56	]=	0.03595647	;
    RnonCO2[	57	]=	0.03663013	;
    RnonCO2[	58	]=	0.03730595	;
    RnonCO2[	59	]=	0.03798427	;
    RnonCO2[	60	]=	0.03866496	;
    RnonCO2[	61	]=	0.03934853	;
    RnonCO2[	62	]=	0.04003524	;
    RnonCO2[	63	]=	0.04072532	;
    RnonCO2[	64	]=	0.04141895	;
    RnonCO2[	65	]=	0.04211500	;
    RnonCO2[	66	]=	0.04281070	;
    RnonCO2[	67	]=	0.04350423	;
    RnonCO2[	68	]=	0.04419751	;
    RnonCO2[	69	]=	0.04489348	;
    RnonCO2[	70	]=	0.04559496	;
    RnonCO2[	71	]=	0.04630513	;
    RnonCO2[	72	]=	0.04702700	;
    RnonCO2[	73	]=	0.04776361	;
    RnonCO2[	74	]=	0.04851799	;
    RnonCO2[	75	]=	0.04929539	;
    RnonCO2[	76	]=	0.05011338	;
    RnonCO2[	77	]=	0.05098929	;
    RnonCO2[	78	]=	0.05190490	;
    RnonCO2[	79	]=	0.05281134	;
    RnonCO2[	80	]=	0.05365385	;
    RnonCO2[	81	]=	0.05440884	;
    RnonCO2[	82	]=	0.05510443	;
    RnonCO2[	83	]=	0.05577309	;
    RnonCO2[	84	]=	0.05641925	;
    RnonCO2[	85	]=	0.05711900	;
    RnonCO2[	86	]=	0.05810210	;
    RnonCO2[	87	]=	0.05930144	;
    RnonCO2[	88	]=	0.06049983	;
    RnonCO2[	89	]=	0.06169728	;
    RnonCO2[	90	]=	0.06275142	;
    RnonCO2[	91	]=	0.06380517	;
    RnonCO2[	92	]=	0.06500011	;
    RnonCO2[	93	]=	0.06619443	;
    RnonCO2[	94	]=	0.06738758	;
    RnonCO2[	95	]=	0.06857997	;
    RnonCO2[	96	]=	0.06977141	;
    RnonCO2[	97	]=	0.07096195	;
    RnonCO2[	98	]=	0.07215174	;
    RnonCO2[	99	]=	0.07334069	;
    RnonCO2[	100	]=	0.07466944	;
    RnonCO2[	101	]=	0.07599721	;
    RnonCO2[	102	]=	0.07718314	;
    RnonCO2[	103	]=	0.07836839	;
    RnonCO2[	104	]=	0.07955262	;
    RnonCO2[	105	]=	0.08079074	;
    RnonCO2[	106	]=	0.08194252	;
    RnonCO2[	107	]=	0.08303863	;
    RnonCO2[	108	]=	0.08421920	;
    RnonCO2[	109	]=	0.08539906	;
    RnonCO2[	110	]=	0.08663208	;
    RnonCO2[	111	]=	0.08777876	;
    RnonCO2[	112	]=	0.08887064	;
    RnonCO2[	113	]=	0.09004676	;
    RnonCO2[	114	]=	0.09122211	;
    RnonCO2[	115	]=	0.09253562	;
    RnonCO2[	116	]=	0.09398645	;
    RnonCO2[	117	]=	0.09543588	;
    RnonCO2[	118	]=	0.09688374	;
    RnonCO2[	119	]=	0.09819223	;
    RnonCO2[	120	]=	0.09936131	;
    RnonCO2[	121	]=	0.10066775	;
    RnonCO2[	122	]=	0.10202539	;
    RnonCO2[	123	]=	0.10329678	;
    RnonCO2[	124	]=	0.10465186	;
    RnonCO2[	125	]=	0.10609055	;
    RnonCO2[	126	]=	0.10766506	;
    RnonCO2[	127	]=	0.10937462	;
    RnonCO2[	128	]=	0.11086042	;
    RnonCO2[	129	]=	0.11212295	;
    RnonCO2[	130	]=	0.11352030	;
    RnonCO2[	131	]=	0.11505224	;
    RnonCO2[	132	]=	0.11653139	;
    RnonCO2[	133	]=	0.11800884	;
    RnonCO2[	134	]=	0.11953520	;
    RnonCO2[	135	]=	0.12128504	;
    RnonCO2[	136	]=	0.12352265	;
    RnonCO2[	137	]=	0.12602098	;
    RnonCO2[	138	]=	0.12864851	;
    RnonCO2[	139	]=	0.13140457	;
    RnonCO2[	140	]=	0.13415439	;
    RnonCO2[	141	]=	0.13689762	;
    RnonCO2[	142	]=	0.13971977	;
    RnonCO2[	143	]=	0.14275307	;
    RnonCO2[	144	]=	0.14591193	;
    RnonCO2[	145	]=	0.14893128	;
    RnonCO2[	146	]=	0.15181224	;
    RnonCO2[	147	]=	0.15490312	;
    RnonCO2[	148	]=	0.15824902	;
    RnonCO2[	149	]=	0.16163260	;
    RnonCO2[	150	]=	0.16509221	;
    RnonCO2[	151	]=	0.16862797	;
    RnonCO2[	152	]=	0.17223947	;
    RnonCO2[	153	]=	0.17605572	;
    RnonCO2[	154	]=	0.17990645	;
    RnonCO2[	155	]=	0.18361943	;
    RnonCO2[	156	]=	0.18727979	;
    RnonCO2[	157	]=	0.19097418	;
    RnonCO2[	158	]=	0.19470557	;
    RnonCO2[	159	]=	0.19843140	;
    RnonCO2[	160	]=	0.20214818	;
    RnonCO2[	161	]=	0.20594015	;
    RnonCO2[	162	]=	0.20972345	;
    RnonCO2[	163	]=	0.21332896	;
    RnonCO2[	164	]=	0.21680125	;
    RnonCO2[	165	]=	0.22014137	;
    RnonCO2[	166	]=	0.22339663	;
    RnonCO2[	167	]=	0.22672929	;
    RnonCO2[	168	]=	0.23013380	;
    RnonCO2[	169	]=	0.23340896	;
    RnonCO2[	170	]=	0.23655532	;
    RnonCO2[	171	]=	0.23969617	;
    RnonCO2[	172	]=	0.24274793	;
    RnonCO2[	173	]=	0.24571087	;
    RnonCO2[	174	]=	0.24866896	;
    RnonCO2[	175	]=	0.25174325	;
    RnonCO2[	176	]=	0.25501730	;
    RnonCO2[	177	]=	0.25824906	;
    RnonCO2[	178	]=	0.26135540	;
    RnonCO2[	179	]=	0.26466060	;
    RnonCO2[	180	]=	0.26816422	;
    RnonCO2[	181	]=	0.27178139	;
    RnonCO2[	182	]=	0.27563021	;
    RnonCO2[	183	]=	0.27959048	;
    RnonCO2[	184	]=	0.28366103	;
    RnonCO2[	185	]=	0.28787550	;
    RnonCO2[	186	]=	0.29223452	;
    RnonCO2[	187	]=	0.29701945	;
    RnonCO2[	188	]=	0.30222493	;
    RnonCO2[	189	]=	0.30744754	;
    RnonCO2[	190	]=	0.31276950	;
    RnonCO2[	191	]=	0.31838800	;
    RnonCO2[	192	]=	0.32413391	;
    RnonCO2[	193	]=	0.32997394	;
    RnonCO2[	194	]=	0.33607437	;
    RnonCO2[	195	]=	0.34246290	;
    RnonCO2[	196	]=	0.34897167	;
    RnonCO2[	197	]=	0.35559914	;
    RnonCO2[	198	]=	0.36248097	;
    RnonCO2[	199	]=	0.36971674	;
    RnonCO2[	200	]=	0.37724167	;
    RnonCO2[	201	]=	0.38483757	;
    RnonCO2[	202	]=	0.39250740	;
    RnonCO2[	203	]=	0.40045274	;
    RnonCO2[	204	]=	0.40874238	;
    RnonCO2[	205	]=	0.41717879	;
    RnonCO2[	206	]=	0.42558070	;
    RnonCO2[	207	]=	0.43414423	;
    RnonCO2[	208	]=	0.44293893	;
    RnonCO2[	209	]=	0.45177595	;
    RnonCO2[	210	]=	0.46057971	;
    RnonCO2[	211	]=	0.46951404	;
    RnonCO2[	212	]=	0.47872209	;
    RnonCO2[	213	]=	0.48804004	;
    RnonCO2[	214	]=	0.49712159	;
    RnonCO2[	215	]=	0.50652934	;
    RnonCO2[	216	]=	0.51714302	;
    RnonCO2[	217	]=	0.52805472	;
    RnonCO2[	218	]=	0.53840398	;
    RnonCO2[	219	]=	0.54764869	;
    RnonCO2[	220	]=	0.55577359	;
    RnonCO2[	221	]=	0.56314084	;
    RnonCO2[	222	]=	0.56961182	;
    RnonCO2[	223	]=	0.57716762	;
    RnonCO2[	224	]=	0.58574415	;
    RnonCO2[	225	]=	0.59314036	;
    RnonCO2[	226	]=	0.59941541	;
    RnonCO2[	227	]=	0.60440178	;
    RnonCO2[	228	]=	0.60802271	;
    RnonCO2[	229	]=	0.61193865	;
    RnonCO2[	230	]=	0.61672080	;
    RnonCO2[	231	]=	0.62119106	;
    RnonCO2[	232	]=	0.62610239	;
    RnonCO2[	233	]=	0.63246130	;
    RnonCO2[	234	]=	0.63821212	;
    RnonCO2[	235	]=	0.64182044	;
    RnonCO2[	236	]=	0.64496502	;
    RnonCO2[	237	]=	0.64935560	;
    RnonCO2[	238	]=	0.65427389	;
    RnonCO2[	239	]=	0.65815366	;
    RnonCO2[	240	]=	0.66159533	;
    RnonCO2[	241	]=	0.66593407	;
    RnonCO2[	242	]=	0.67090356	;
    RnonCO2[	243	]=	0.67616707	;
    RnonCO2[	244	]=	0.68165968	;
    RnonCO2[	245	]=	0.68727911	;
    RnonCO2[	246	]=	0.69301982	;
    RnonCO2[	247	]=	0.69861598	;
    RnonCO2[	248	]=	0.70381722	;
    RnonCO2[	249	]=	0.70866248	;
    RnonCO2[	250	]=	0.71319036	;
    RnonCO2[	251	]=	0.71742358	;
    RnonCO2[	252	]=	0.72137337	;
    RnonCO2[	253	]=	0.72505338	;
    RnonCO2[	254	]=	0.72847874	;
    RnonCO2[	255	]=	0.73166410	;
    RnonCO2[	256	]=	0.73459461	;
    RnonCO2[	257	]=	0.73747758	;
    RnonCO2[	258	]=	0.74054455	;
    RnonCO2[	259	]=	0.74380098	;
    RnonCO2[	260	]=	0.74725039	;
    RnonCO2[	261	]=	0.75089785	;
    RnonCO2[	262	]=	0.75473411	;
    RnonCO2[	263	]=	0.75873925	;
    RnonCO2[	264	]=	0.76289762	;
    RnonCO2[	265	]=	0.76719757	;
    RnonCO2[	266	]=	0.77163097	;
    RnonCO2[	267	]=	0.77617182	;
    RnonCO2[	268	]=	0.78080405	;
    RnonCO2[	269	]=	0.78554076	;
    RnonCO2[	270	]=	0.79039073	;
    RnonCO2[	271	]=	0.79536083	;
    RnonCO2[	272	]=	0.80046009	;
    RnonCO2[	273	]=	0.80568270	;
    RnonCO2[	274	]=	0.81101266	;
    RnonCO2[	275	]=	0.81643781	;
    RnonCO2[	276	]=	0.82194888	;
    RnonCO2[	277	]=	0.82748330	;
    RnonCO2[	278	]=	0.83295786	;
    RnonCO2[	279	]=	0.83836087	;
    RnonCO2[	280	]=	0.84371920	;
    RnonCO2[	281	]=	0.84905335	;
    RnonCO2[	282	]=	0.85438102	;
    RnonCO2[	283	]=	0.85972108	;
    RnonCO2[	284	]=	0.86507627	;
    RnonCO2[	285	]=	0.87043837	;
    RnonCO2[	286	]=	0.87580353	;
    RnonCO2[	287	]=	0.88115517	;
    RnonCO2[	288	]=	0.88647625	;
    RnonCO2[	289	]=	0.89173768	;
    RnonCO2[	290	]=	0.89692423	;
    RnonCO2[	291	]=	0.90206068	;
    RnonCO2[	292	]=	0.90716668	;
    RnonCO2[	293	]=	0.91225923	;
    RnonCO2[	294	]=	0.91735664	;
    RnonCO2[	295	]=	0.92246142	;
    RnonCO2[	296	]=	0.92756192	;
    RnonCO2[	297	]=	0.93258629	;
    RnonCO2[	298	]=	0.93747484	;
    RnonCO2[	299	]=	0.94223618	;
    RnonCO2[	300	]=	0.94684852	;
    RnonCO2[	301	]=	0.95130226	;
    RnonCO2[	302	]=	0.95562914	;
    RnonCO2[	303	]=	0.95985447	;
    RnonCO2[	304	]=	0.96400064	;
    RnonCO2[	305	]=	0.96809094	;
    RnonCO2[	306	]=	0.97213366	;
    RnonCO2[	307	]=	0.97599522	;
    RnonCO2[	308	]=	0.97955529	;
    RnonCO2[	309	]=	0.98283508	;
    RnonCO2[	310	]=	0.98585378	;
    RnonCO2[	311	]=	0.98859983	;
    RnonCO2[	312	]=	0.99107310	;
    RnonCO2[	313	]=	0.99331400	;
    RnonCO2[	314	]=	0.99535641	;
    RnonCO2[	315	]=	0.99723015	;
    RnonCO2[	316	]=	0.99896669	;
    RnonCO2[	317	]=	1.00024263	;
    RnonCO2[	318	]=	1.00074829	;
    RnonCO2[	319	]=	1.00053553	;
    RnonCO2[	320	]=	0.99965511	;
    RnonCO2[	321	]=	0.99815373	;
    RnonCO2[	322	]=	0.99604448	;
    RnonCO2[	323	]=	0.99335014	;
    RnonCO2[	324	]=	0.99013105	;
    RnonCO2[	325	]=	0.98644027	;
    RnonCO2[	326	]=	0.98232404	;
    RnonCO2[	327	]=	0.97811394	;
    RnonCO2[	328	]=	0.97410309	;
    RnonCO2[	329	]=	0.97026742	;
    RnonCO2[	330	]=	0.96658722	;
    RnonCO2[	331	]=	0.96304768	;
    RnonCO2[	332	]=	0.95963482	;
    RnonCO2[	333	]=	0.95637103	;
    RnonCO2[	334	]=	0.95331522	;
    RnonCO2[	335	]=	0.95049228	;
    RnonCO2[	336	]=	0.94788966	;
    RnonCO2[	337	]=	0.94564312	;
    RnonCO2[	338	]=	0.94387234	;
    RnonCO2[	339	]=	0.94252115	;
    RnonCO2[	340	]=	0.94152868	;
    RnonCO2[	341	]=	0.94084300	;
    RnonCO2[	342	]=	0.94042191	;
    RnonCO2[	343	]=	0.94022760	;
    RnonCO2[	344	]=	0.94020038	;
    RnonCO2[	345	]=	0.94029614	;
    RnonCO2[	346	]=	0.94051273	;
    RnonCO2[	347	]=	0.94084570	;
    RnonCO2[	348	]=	0.94128956	;
    RnonCO2[	349	]=	0.94184173	;
    RnonCO2[	350	]=	0.94248677	;
    RnonCO2[	351	]=	0.94320006	;
    RnonCO2[	352	]=	0.94396247	;
    RnonCO2[	353	]=	0.94476065	;
    RnonCO2[	354	]=	0.94558255	;
    RnonCO2[	355	]=	0.94639185	;
    RnonCO2[	356	]=	0.94716535	;
    RnonCO2[	357	]=	0.94791870	;
    RnonCO2[	358	]=	0.94866392	;
    RnonCO2[	359	]=	0.94941005	;
    RnonCO2[	360	]=	0.95016732	;
    RnonCO2[	361	]=	0.95093164	;
    RnonCO2[	362	]=	0.95168880	;
    RnonCO2[	363	]=	0.95242874	;
    RnonCO2[	364	]=	0.95314601	;
    RnonCO2[	365	]=	0.95383580	;
    RnonCO2[	366	]=	0.95446835	;
    RnonCO2[	367	]=	0.95502594	;
    RnonCO2[	368	]=	0.95552935	;
    RnonCO2[	369	]=	0.95599498	;
    RnonCO2[	370	]=	0.95643564	;
    RnonCO2[	371	]=	0.95686498	;
    RnonCO2[	372	]=	0.95728213	;
    RnonCO2[	373	]=	0.95767547	;
    RnonCO2[	374	]=	0.95803719	;
    RnonCO2[	375	]=	0.95836394	;
    RnonCO2[	376	]=	0.95865317	;
    RnonCO2[	377	]=	0.95887644	;
    RnonCO2[	378	]=	0.95901760	;
    RnonCO2[	379	]=	0.95909879	;
    RnonCO2[	380	]=	0.95913754	;
    RnonCO2[	381	]=	0.95914766	;
    RnonCO2[	382	]=	0.95914375	;
    RnonCO2[	383	]=	0.95912546	;
    RnonCO2[	384	]=	0.95908187	;
    RnonCO2[	385	]=	0.95905262	;
    RnonCO2[	386	]=	0.95905262	;
    RnonCO2[	387	]=	0.95905262	;
    RnonCO2[	388	]=	0.95905262	;
    RnonCO2[	389	]=	0.95905262	;
    RnonCO2[	390	]=	0.95905262	;
    RnonCO2[	391	]=	0.95905262	;
    RnonCO2[	392	]=	0.95905262	;
    RnonCO2[	393	]=	0.95905262	;
    RnonCO2[	394	]=	0.95905262	;
    RnonCO2[	395	]=	0.95905262	;
    RnonCO2[	396	]=	0.95905262	;
    RnonCO2[	397	]=	0.95905262	;
    RnonCO2[	398	]=	0.95905262	;
    RnonCO2[	399	]=	0.95905262	;
    RnonCO2[	400	]=	0.95905262	;
    RnonCO2[	401	]=	0.95905262	;
    RnonCO2[	402	]=	0.95905262	;
    RnonCO2[	403	]=	0.95905262	;
    RnonCO2[	404	]=	0.95905262	;
    RnonCO2[	405	]=	0.95905262	;
    RnonCO2[	406	]=	0.95905262	;
    RnonCO2[	407	]=	0.95905262	;
    RnonCO2[	408	]=	0.95905262	;
    RnonCO2[	409	]=	0.95905262	;
    RnonCO2[	410	]=	0.95905262	;
    RnonCO2[	411	]=	0.95905262	;
    RnonCO2[	412	]=	0.95905262	;
    RnonCO2[	413	]=	0.95905262	;
    RnonCO2[	414	]=	0.95905262	;
    RnonCO2[	415	]=	0.95905262	;
    RnonCO2[	416	]=	0.95905262	;
    RnonCO2[	417	]=	0.95905262	;
    RnonCO2[	418	]=	0.95905262	;
    RnonCO2[	419	]=	0.95905262	;
    RnonCO2[	420	]=	0.95905262	;
    RnonCO2[	421	]=	0.95905262	;
    RnonCO2[	422	]=	0.95905262	;
    RnonCO2[	423	]=	0.95905262	;
    RnonCO2[	424	]=	0.95905262	;
    RnonCO2[	425	]=	0.95905262	;
    RnonCO2[	426	]=	0.95905262	;
    RnonCO2[	427	]=	0.95905262	;
    RnonCO2[	428	]=	0.95905262	;
    RnonCO2[	429	]=	0.95905262	;
    RnonCO2[	430	]=	0.95905262	;
    RnonCO2[	431	]=	0.95905262	;
    RnonCO2[	432	]=	0.95905262	;
    RnonCO2[	433	]=	0.95905262	;
    RnonCO2[	434	]=	0.95905262	;
    RnonCO2[	435	]=	0.95905262	;
    RnonCO2[	436	]=	0.95905262	;
    RnonCO2[	437	]=	0.95905262	;
    RnonCO2[	438	]=	0.95905262	;
    RnonCO2[	439	]=	0.95905262	;
    RnonCO2[	440	]=	0.95905262	;
    RnonCO2[	441	]=	0.95905262	;
    RnonCO2[	442	]=	0.95905262	;
    RnonCO2[	443	]=	0.95905262	;
    RnonCO2[	444	]=	0.95905262	;
    RnonCO2[	445	]=	0.95905262	;
    RnonCO2[	446	]=	0.95905262	;
    RnonCO2[	447	]=	0.95905262	;
    RnonCO2[	448	]=	0.95905262	;
    RnonCO2[	449	]=	0.95905262	;
    RnonCO2[	450	]=	0.95905262	;
    RnonCO2[	451	]=	0.95905262	;
    RnonCO2[	452	]=	0.95905262	;
    RnonCO2[	453	]=	0.95905262	;
    RnonCO2[	454	]=	0.95905262	;
    RnonCO2[	455	]=	0.95905262	;
    RnonCO2[	456	]=	0.95905262	;
    RnonCO2[	457	]=	0.95905262	;
    RnonCO2[	458	]=	0.95905262	;
    RnonCO2[	459	]=	0.95905262	;
    RnonCO2[	460	]=	0.95905262	;
    RnonCO2[	461	]=	0.95905262	;
    RnonCO2[	462	]=	0.95905262	;
    RnonCO2[	463	]=	0.95905262	;
    RnonCO2[	464	]=	0.95905262	;
    RnonCO2[	465	]=	0.95905262	;
    RnonCO2[	466	]=	0.95905262	;
    RnonCO2[	467	]=	0.95905262	;
    RnonCO2[	468	]=	0.95905262	;
    RnonCO2[	469	]=	0.95905262	;
    RnonCO2[	470	]=	0.95905262	;
    RnonCO2[	471	]=	0.95905262	;
    RnonCO2[	472	]=	0.95905262	;
    RnonCO2[	473	]=	0.95905262	;
    RnonCO2[	474	]=	0.95905262	;
    RnonCO2[	475	]=	0.95905262	;
    RnonCO2[	476	]=	0.95905262	;
    RnonCO2[	477	]=	0.95905262	;
    RnonCO2[	478	]=	0.95905262	;
    RnonCO2[	479	]=	0.95905262	;
    RnonCO2[	480	]=	0.95905262	;
    RnonCO2[	481	]=	0.95905262	;
    RnonCO2[	482	]=	0.95905262	;
    RnonCO2[	483	]=	0.95905262	;
    RnonCO2[	484	]=	0.95905262	;
    RnonCO2[	485	]=	0.95905262	;
    RnonCO2[	486	]=	0.95905262	;
    RnonCO2[	487	]=	0.95905262	;
    RnonCO2[	488	]=	0.95905262	;
    RnonCO2[	489	]=	0.95905262	;
    RnonCO2[	490	]=	0.95905262	;
    RnonCO2[	491	]=	0.95905262	;
    RnonCO2[	492	]=	0.95905262	;
    RnonCO2[	493	]=	0.95905262	;
    RnonCO2[	494	]=	0.95905262	;
    RnonCO2[	495	]=	0.95905262	;
    RnonCO2[	496	]=	0.95905262	;
    RnonCO2[	497	]=	0.95905262	;
    RnonCO2[	498	]=	0.95905262	;
    RnonCO2[	499	]=	0.95905262	;
    RnonCO2[	500	]=	0.95905262	;
    RnonCO2[	501	]=	0.95905262	;
    RnonCO2[	502	]=	0.95905262	;
    RnonCO2[	503	]=	0.95905262	;
    RnonCO2[	504	]=	0.95905262	;
    RnonCO2[	505	]=	0.95905262	;
    RnonCO2[	506	]=	0.95905262	;
    RnonCO2[	507	]=	0.95905262	;
    RnonCO2[	508	]=	0.95905262	;
    RnonCO2[	509	]=	0.95905262	;
    RnonCO2[	510	]=	0.95905262	;
    RnonCO2[	511	]=	0.95905262	;
    RnonCO2[	512	]=	0.95905262	;
    RnonCO2[	513	]=	0.95905262	;
    RnonCO2[	514	]=	0.95905262	;
    RnonCO2[	515	]=	0.95905262	;
    RnonCO2[	516	]=	0.95905262	;
    RnonCO2[	517	]=	0.95905262	;
    RnonCO2[	518	]=	0.95905262	;
    RnonCO2[	519	]=	0.95905262	;
    RnonCO2[	520	]=	0.95905262	;
    RnonCO2[	521	]=	0.95905262	;
    RnonCO2[	522	]=	0.95905262	;
    RnonCO2[	523	]=	0.95905262	;
    RnonCO2[	524	]=	0.95905262	;
    RnonCO2[	525	]=	0.95905262	;
    RnonCO2[	526	]=	0.95905262	;
    RnonCO2[	527	]=	0.95905262	;
    RnonCO2[	528	]=	0.95905262	;
    RnonCO2[	529	]=	0.95905262	;
    RnonCO2[	530	]=	0.95905262	;
    RnonCO2[	531	]=	0.95905262	;
    RnonCO2[	532	]=	0.95905262	;
    RnonCO2[	533	]=	0.95905262	;
    RnonCO2[	534	]=	0.95905262	;
    RnonCO2[	535	]=	0.95905262	;
    RnonCO2[	536	]=	0.95905262	;
    RnonCO2[	537	]=	0.95905262	;
    RnonCO2[	538	]=	0.95905262	;
    RnonCO2[	539	]=	0.95905262	;
    RnonCO2[	540	]=	0.95905262	;
    RnonCO2[	541	]=	0.95905262	;
    RnonCO2[	542	]=	0.95905262	;
    RnonCO2[	543	]=	0.95905262	;
    RnonCO2[	544	]=	0.95905262	;
    RnonCO2[	545	]=	0.95905262	;
    RnonCO2[	546	]=	0.95905262	;
    RnonCO2[	547	]=	0.95905262	;
    RnonCO2[	548	]=	0.95905262	;
    RnonCO2[	549	]=	0.95905262	;
    RnonCO2[	550	]=	0.95905262	;
    RnonCO2[	551	]=	0.95905262	;
    RnonCO2[	552	]=	0.95905262	;
    RnonCO2[	553	]=	0.95905262	;
    RnonCO2[	554	]=	0.95905262	;
    RnonCO2[	555	]=	0.95905262	;
    RnonCO2[	556	]=	0.95905262	;
    RnonCO2[	557	]=	0.95905262	;
    RnonCO2[	558	]=	0.95905262	;
    RnonCO2[	559	]=	0.95905262	;
    RnonCO2[	560	]=	0.95905262	;
    RnonCO2[	561	]=	0.95905262	;
    RnonCO2[	562	]=	0.95905262	;
    RnonCO2[	563	]=	0.95905262	;
    RnonCO2[	564	]=	0.95905262	;
    RnonCO2[	565	]=	0.95905262	;
    RnonCO2[	566	]=	0.95905262	;
    RnonCO2[	567	]=	0.95905262	;
    RnonCO2[	568	]=	0.95905262	;
    RnonCO2[	569	]=	0.95905262	;
    RnonCO2[	570	]=	0.95905262	;
    RnonCO2[	571	]=	0.95905262	;
    RnonCO2[	572	]=	0.95905262	;
    RnonCO2[	573	]=	0.95905262	;
    RnonCO2[	574	]=	0.95905262	;
    RnonCO2[	575	]=	0.95905262	;
    RnonCO2[	576	]=	0.95905262	;
    RnonCO2[	577	]=	0.95905262	;
    RnonCO2[	578	]=	0.95905262	;
    RnonCO2[	579	]=	0.95905262	;
    RnonCO2[	580	]=	0.95905262	;
    RnonCO2[	581	]=	0.95905262	;
    RnonCO2[	582	]=	0.95905262	;
    RnonCO2[	583	]=	0.95905262	;
    RnonCO2[	584	]=	0.95905262	;
    RnonCO2[	585	]=	0.95905262	;
    RnonCO2[	586	]=	0.95905262	;
    RnonCO2[	587	]=	0.95905262	;
    RnonCO2[	588	]=	0.95905262	;
    RnonCO2[	589	]=	0.95905262	;
    RnonCO2[	590	]=	0.95905262	;
    RnonCO2[	591	]=	0.95905262	;
    RnonCO2[	592	]=	0.95905262	;
    RnonCO2[	593	]=	0.95905262	;
    RnonCO2[	594	]=	0.95905262	;
    RnonCO2[	595	]=	0.95905262	;
    RnonCO2[	596	]=	0.95905262	;
    RnonCO2[	597	]=	0.95905262	;
    RnonCO2[	598	]=	0.95905262	;
    RnonCO2[	599	]=	0.95905262	;
    RnonCO2[	600	]=	0.95905262	;
    RnonCO2[	601	]=	0.95905262	;
    RnonCO2[	602	]=	0.95905262	;
    RnonCO2[	603	]=	0.95905262	;
    RnonCO2[	604	]=	0.95905262	;
    RnonCO2[	605	]=	0.95905262	;
    RnonCO2[	606	]=	0.95905262	;
    RnonCO2[	607	]=	0.95905262	;
    RnonCO2[	608	]=	0.95905262	;
    RnonCO2[	609	]=	0.95905262	;
    RnonCO2[	610	]=	0.95905262	;
    RnonCO2[	611	]=	0.95905262	;
    RnonCO2[	612	]=	0.95905262	;
    RnonCO2[	613	]=	0.95905262	;
    RnonCO2[	614	]=	0.95905262	;
    RnonCO2[	615	]=	0.95905262	;
    RnonCO2[	616	]=	0.95905262	;
    RnonCO2[	617	]=	0.95905262	;
    RnonCO2[	618	]=	0.95905262	;
    RnonCO2[	619	]=	0.95905262	;
    RnonCO2[	620	]=	0.95905262	;
    RnonCO2[	621	]=	0.95905262	;
    RnonCO2[	622	]=	0.95905262	;
    RnonCO2[	623	]=	0.95905262	;
    RnonCO2[	624	]=	0.95905262	;
    RnonCO2[	625	]=	0.95905262	;
    RnonCO2[	626	]=	0.95905262	;
    RnonCO2[	627	]=	0.95905262	;
    RnonCO2[	628	]=	0.95905262	;
    RnonCO2[	629	]=	0.95905262	;
    RnonCO2[	630	]=	0.95905262	;
    RnonCO2[	631	]=	0.95905262	;
    RnonCO2[	632	]=	0.95905262	;
    RnonCO2[	633	]=	0.95905262	;
    RnonCO2[	634	]=	0.95905262	;
    RnonCO2[	635	]=	0.95905262	;
    RnonCO2[	636	]=	0.95905262	;
    RnonCO2[	637	]=	0.95905262	;
    RnonCO2[	638	]=	0.95905262	;
    RnonCO2[	639	]=	0.95905262	;
    RnonCO2[	640	]=	0.95905262	;
    RnonCO2[	641	]=	0.95905262	;
    RnonCO2[	642	]=	0.95905262	;
    RnonCO2[	643	]=	0.95905262	;
    RnonCO2[	644	]=	0.95905262	;
    RnonCO2[	645	]=	0.95905262	;
    RnonCO2[	646	]=	0.95905262	;
    RnonCO2[	647	]=	0.95905262	;
    RnonCO2[	648	]=	0.95905262	;
    RnonCO2[	649	]=	0.95905262	;
    RnonCO2[	650	]=	0.95905262	;
    RnonCO2[	651	]=	0.95905262	;
    RnonCO2[	652	]=	0.95905262	;
    RnonCO2[	653	]=	0.95905262	;
    RnonCO2[	654	]=	0.95905262	;
    RnonCO2[	655	]=	0.95905262	;
    RnonCO2[	656	]=	0.95905262	;
    RnonCO2[	657	]=	0.95905262	;
    RnonCO2[	658	]=	0.95905262	;
    RnonCO2[	659	]=	0.95905262	;
    RnonCO2[	660	]=	0.95905262	;
    RnonCO2[	661	]=	0.95905262	;
    RnonCO2[	662	]=	0.95905262	;
    RnonCO2[	663	]=	0.95905262	;
    RnonCO2[	664	]=	0.95905262	;
    RnonCO2[	665	]=	0.95905262	;
    RnonCO2[	666	]=	0.95905262	;
    RnonCO2[	667	]=	0.95905262	;
    RnonCO2[	668	]=	0.95905262	;
    RnonCO2[	669	]=	0.95905262	;
    RnonCO2[	670	]=	0.95905262	;
    RnonCO2[	671	]=	0.95905262	;
    RnonCO2[	672	]=	0.95905262	;
    RnonCO2[	673	]=	0.95905262	;
    RnonCO2[	674	]=	0.95905262	;
    RnonCO2[	675	]=	0.95905262	;
    RnonCO2[	676	]=	0.95905262	;
    RnonCO2[	677	]=	0.95905262	;
    RnonCO2[	678	]=	0.95905262	;
    RnonCO2[	679	]=	0.95905262	;
    RnonCO2[	680	]=	0.95905262	;
    RnonCO2[	681	]=	0.95905262	;
    RnonCO2[	682	]=	0.95905262	;
    RnonCO2[	683	]=	0.95905262	;
    RnonCO2[	684	]=	0.95905262	;
    RnonCO2[	685	]=	0.95905262	;
    RnonCO2[	686	]=	0.95905262	;
    RnonCO2[	687	]=	0.95905262	;
    RnonCO2[	688	]=	0.95905262	;
    RnonCO2[	689	]=	0.95905262	;
    RnonCO2[	690	]=	0.95905262	;
    RnonCO2[	691	]=	0.95905262	;
    RnonCO2[	692	]=	0.95905262	;
    RnonCO2[	693	]=	0.95905262	;
    RnonCO2[	694	]=	0.95905262	;
    RnonCO2[	695	]=	0.95905262	;
    RnonCO2[	696	]=	0.95905262	;
    RnonCO2[	697	]=	0.95905262	;
    RnonCO2[	698	]=	0.95905262	;
    RnonCO2[	699	]=	0.95905262	;
    RnonCO2[	700	]=	0.95905262	;
    RnonCO2[	701	]=	0.95905262	;
    RnonCO2[	702	]=	0.95905262	;
    RnonCO2[	703	]=	0.95905262	;
    RnonCO2[	704	]=	0.95905262	;
    RnonCO2[	705	]=	0.95905262	;
    RnonCO2[	706	]=	0.95905262	;
    RnonCO2[	707	]=	0.95905262	;
    RnonCO2[	708	]=	0.95905262	;
    RnonCO2[	709	]=	0.95905262	;
    RnonCO2[	710	]=	0.95905262	;
    RnonCO2[	711	]=	0.95905262	;
    RnonCO2[	712	]=	0.95905262	;
    RnonCO2[	713	]=	0.95905262	;
    RnonCO2[	714	]=	0.95905262	;
    RnonCO2[	715	]=	0.95905262	;
    RnonCO2[	716	]=	0.95905262	;
    RnonCO2[	717	]=	0.95905262	;
    RnonCO2[	718	]=	0.95905262	;
    RnonCO2[	719	]=	0.95905262	;
    RnonCO2[	720	]=	0.95905262	;
    RnonCO2[	721	]=	0.95905262	;
    RnonCO2[	722	]=	0.95905262	;
    RnonCO2[	723	]=	0.95905262	;
    RnonCO2[	724	]=	0.95905262	;
    RnonCO2[	725	]=	0.95905262	;
    RnonCO2[	726	]=	0.95905262	;
    RnonCO2[	727	]=	0.95905262	;
    RnonCO2[	728	]=	0.95905262	;
    RnonCO2[	729	]=	0.95905262	;
    RnonCO2[	730	]=	0.95905262	;
    RnonCO2[	731	]=	0.95905262	;
    RnonCO2[	732	]=	0.95905262	;
    RnonCO2[	733	]=	0.95905262	;
    RnonCO2[	734	]=	0.95905262	;
    RnonCO2[	735	]=	0.95905262	;
    
    for(int i = 736; i<tmax; i++)
    {
        RnonCO2[i] = RnonCO2[i-1];
    }

}

void setR_WMGHG_nonCO2_RCP85(void)
{
    RnonCO2[    0    ]=    0    ;
    RnonCO2[    1    ]=    0.004314432    ;
    RnonCO2[    2    ]=    0.008877103    ;
    RnonCO2[    3    ]=    0.009366935    ;
    RnonCO2[    4    ]=    0.00984243    ;
    RnonCO2[    5    ]=    0.010303014    ;
    RnonCO2[    6    ]=    0.010751385    ;
    RnonCO2[    7    ]=    0.011190869    ;
    RnonCO2[    8    ]=    0.01162232    ;
    RnonCO2[    9    ]=    0.012045243    ;
    RnonCO2[    10    ]=    0.012459081    ;
    RnonCO2[    11    ]=    0.012863113    ;
    RnonCO2[    12    ]=    0.013256576    ;
    RnonCO2[    13    ]=    0.013638884    ;
    RnonCO2[    14    ]=    0.014009567    ;
    RnonCO2[    15    ]=    0.014366637    ;
    RnonCO2[    16    ]=    0.014706055    ;
    RnonCO2[    17    ]=    0.015026378    ;
    RnonCO2[    18    ]=    0.015331384    ;
    RnonCO2[    19    ]=    0.015626956    ;
    RnonCO2[    20    ]=    0.015918925    ;
    RnonCO2[    21    ]=    0.01621293    ;
    RnonCO2[    22    ]=    0.016514514    ;
    RnonCO2[    23    ]=    0.016829376    ;
    RnonCO2[    24    ]=    0.017163338    ;
    RnonCO2[    25    ]=    0.017521083    ;
    RnonCO2[    26    ]=    0.017903698    ;
    RnonCO2[    27    ]=    0.01830912    ;
    RnonCO2[    28    ]=    0.018735197    ;
    RnonCO2[    29    ]=    0.019180394    ;
    RnonCO2[    30    ]=    0.01964318    ;
    RnonCO2[    31    ]=    0.02012184    ;
    RnonCO2[    32    ]=    0.02061459    ;
    RnonCO2[    33    ]=    0.02111978    ;
    RnonCO2[    34    ]=    0.02163591    ;
    RnonCO2[    35    ]=    0.02216446    ;
    RnonCO2[    36    ]=    0.02271248    ;
    RnonCO2[    37    ]=    0.02328571    ;
    RnonCO2[    38    ]=    0.0238829    ;
    RnonCO2[    39    ]=    0.02449959    ;
    RnonCO2[    40    ]=    0.02513135    ;
    RnonCO2[    41    ]=    0.02577366    ;
    RnonCO2[    42    ]=    0.02642202    ;
    RnonCO2[    43    ]=    0.02707192    ;
    RnonCO2[    44    ]=    0.02771893    ;
    RnonCO2[    45    ]=    0.02836089    ;
    RnonCO2[    46    ]=    0.02900087    ;
    RnonCO2[    47    ]=    0.02964356    ;
    RnonCO2[    48    ]=    0.03029043    ;
    RnonCO2[    49    ]=    0.03094103    ;
    RnonCO2[    50    ]=    0.03159488    ;
    RnonCO2[    51    ]=    0.03225155    ;
    RnonCO2[    52    ]=    0.0329106    ;
    RnonCO2[    53    ]=    0.03357157    ;
    RnonCO2[    54    ]=    0.03423393    ;
    RnonCO2[    55    ]=    0.03489733    ;
    RnonCO2[    56    ]=    0.03556198    ;
    RnonCO2[    57    ]=    0.03622841    ;
    RnonCO2[    58    ]=    0.03689698    ;
    RnonCO2[    59    ]=    0.03756791    ;
    RnonCO2[    60    ]=    0.03824135    ;
    RnonCO2[    61    ]=    0.03891758    ;
    RnonCO2[    62    ]=    0.03959693    ;
    RnonCO2[    63    ]=    0.04027961    ;
    RnonCO2[    64    ]=    0.04096581    ;
    RnonCO2[    65    ]=    0.04165447    ;
    RnonCO2[    66    ]=    0.04234275    ;
    RnonCO2[    67    ]=    0.04302909    ;
    RnonCO2[    68    ]=    0.04371511    ;
    RnonCO2[    69    ]=    0.04440374    ;
    RnonCO2[    70    ]=    0.0450979    ;
    RnonCO2[    71    ]=    0.04580059    ;
    RnonCO2[    72    ]=    0.04651493    ;
    RnonCO2[    73    ]=    0.0472439    ;
    RnonCO2[    74    ]=    0.04799039    ;
    RnonCO2[    75    ]=    0.04875957    ;
    RnonCO2[    76    ]=    0.04956908    ;
    RnonCO2[    77    ]=    0.05043603    ;
    RnonCO2[    78    ]=    0.05134235    ;
    RnonCO2[    79    ]=    0.05223923    ;
    RnonCO2[    80    ]=    0.05307263    ;
    RnonCO2[    81    ]=    0.05381898    ;
    RnonCO2[    82    ]=    0.05450612    ;
    RnonCO2[    83    ]=    0.05516657    ;
    RnonCO2[    84    ]=    0.05580469    ;
    RnonCO2[    85    ]=    0.05649661    ;
    RnonCO2[    86    ]=    0.05747303    ;
    RnonCO2[    87    ]=    0.05866701    ;
    RnonCO2[    88    ]=    0.0598601    ;
    RnonCO2[    89    ]=    0.0610523    ;
    RnonCO2[    90    ]=    0.06210314    ;
    RnonCO2[    91    ]=    0.06315323    ;
    RnonCO2[    92    ]=    0.06434304    ;
    RnonCO2[    93    ]=    0.06553198    ;
    RnonCO2[    94    ]=    0.06672003    ;
    RnonCO2[    95    ]=    0.06790722    ;
    RnonCO2[    96    ]=    0.06909353    ;
    RnonCO2[    97    ]=    0.07027898    ;
    RnonCO2[    98    ]=    0.07146355    ;
    RnonCO2[    99    ]=    0.07264725    ;
    RnonCO2[    100    ]=    0.07396922    ;
    RnonCO2[    101    ]=    0.0752902    ;
    RnonCO2[    102    ]=    0.07647106    ;
    RnonCO2[    103    ]=    0.07765105    ;
    RnonCO2[    104    ]=    0.07883019    ;
    RnonCO2[    105    ]=    0.08006188    ;
    RnonCO2[    106    ]=    0.0812076    ;
    RnonCO2[    107    ]=    0.08229901    ;
    RnonCO2[    108    ]=    0.08347458    ;
    RnonCO2[    109    ]=    0.08464931    ;
    RnonCO2[    110    ]=    0.08587598    ;
    RnonCO2[    111    ]=    0.08701708    ;
    RnonCO2[    112    ]=    0.08810499    ;
    RnonCO2[    113    ]=    0.08927741    ;
    RnonCO2[    114    ]=    0.09044939    ;
    RnonCO2[    115    ]=    0.09175792    ;
    RnonCO2[    116    ]=    0.09320276    ;
    RnonCO2[    117    ]=    0.09464664    ;
    RnonCO2[    118    ]=    0.09608949    ;
    RnonCO2[    119    ]=    0.09739496    ;
    RnonCO2[    120    ]=    0.0985634    ;
    RnonCO2[    121    ]=    0.09986768    ;
    RnonCO2[    122    ]=    0.10122271    ;
    RnonCO2[    123    ]=    0.10249213    ;
    RnonCO2[    124    ]=    0.10384549    ;
    RnonCO2[    125    ]=    0.10528284    ;
    RnonCO2[    126    ]=    0.10685466    ;
    RnonCO2[    127    ]=    0.1085606    ;
    RnonCO2[    128    ]=    0.11004559    ;
    RnonCO2[    129    ]=    0.11131004    ;
    RnonCO2[    130    ]=    0.11270838    ;
    RnonCO2[    131    ]=    0.1142403    ;
    RnonCO2[    132    ]=    0.11572158    ;
    RnonCO2[    133    ]=    0.11720182    ;
    RnonCO2[    134    ]=    0.11873056    ;
    RnonCO2[    135    ]=    0.12048167    ;
    RnonCO2[    136    ]=    0.12271612    ;
    RnonCO2[    137    ]=    0.12520844    ;
    RnonCO2[    138    ]=    0.12782821    ;
    RnonCO2[    139    ]=    0.13057484    ;
    RnonCO2[    140    ]=    0.13331526    ;
    RnonCO2[    141    ]=    0.13605041    ;
    RnonCO2[    142    ]=    0.13887031    ;
    RnonCO2[    143    ]=    0.14190497    ;
    RnonCO2[    144    ]=    0.14506366    ;
    RnonCO2[    145    ]=    0.14808479    ;
    RnonCO2[    146    ]=    0.15096905    ;
    RnonCO2[    147    ]=    0.15406256    ;
    RnonCO2[    148    ]=    0.15742306    ;
    RnonCO2[    149    ]=    0.16083056    ;
    RnonCO2[    150    ]=    0.16429017    ;
    RnonCO2[    151    ]=    0.16780522    ;
    RnonCO2[    152    ]=    0.17141349    ;
    RnonCO2[    153    ]=    0.17524964    ;
    RnonCO2[    154    ]=    0.17912698    ;
    RnonCO2[    155    ]=    0.18286803    ;
    RnonCO2[    156    ]=    0.18655793    ;
    RnonCO2[    157    ]=    0.19028114    ;
    RnonCO2[    158    ]=    0.19403945    ;
    RnonCO2[    159    ]=    0.19779207    ;
    RnonCO2[    160    ]=    0.2015367    ;
    RnonCO2[    161    ]=    0.20536218    ;
    RnonCO2[    162    ]=    0.20918465    ;
    RnonCO2[    163    ]=    0.21283052    ;
    RnonCO2[    164    ]=    0.2163449    ;
    RnonCO2[    165    ]=    0.21972929    ;
    RnonCO2[    166    ]=    0.22302895    ;
    RnonCO2[    167    ]=    0.22640626    ;
    RnonCO2[    168    ]=    0.22985789    ;
    RnonCO2[    169    ]=    0.23320109    ;
    RnonCO2[    170    ]=    0.2364361    ;
    RnonCO2[    171    ]=    0.23966811    ;
    RnonCO2[    172    ]=    0.2428137    ;
    RnonCO2[    173    ]=    0.24587625    ;
    RnonCO2[    174    ]=    0.248943    ;
    RnonCO2[    175    ]=    0.25213499    ;
    RnonCO2[    176    ]=    0.25554226    ;
    RnonCO2[    177    ]=    0.25892918    ;
    RnonCO2[    178    ]=    0.26220864    ;
    RnonCO2[    179    ]=    0.26570209    ;
    RnonCO2[    180    ]=    0.26941573    ;
    RnonCO2[    181    ]=    0.273294    ;
    RnonCO2[    182    ]=    0.27751544    ;
    RnonCO2[    183    ]=    0.28201064    ;
    RnonCO2[    184    ]=    0.28674226    ;
    RnonCO2[    185    ]=    0.29175313    ;
    RnonCO2[    186    ]=    0.29696911    ;
    RnonCO2[    187    ]=    0.30255457    ;
    RnonCO2[    188    ]=    0.30858029    ;
    RnonCO2[    189    ]=    0.31473264    ;
    RnonCO2[    190    ]=    0.32113183    ;
    RnonCO2[    191    ]=    0.32799691    ;
    RnonCO2[    192    ]=    0.33518916    ;
    RnonCO2[    193    ]=    0.34268013    ;
    RnonCO2[    194    ]=    0.35054912    ;
    RnonCO2[    195    ]=    0.3588091    ;
    RnonCO2[    196    ]=    0.36744837    ;
    RnonCO2[    197    ]=    0.37654998    ;
    RnonCO2[    198    ]=    0.38629282    ;
    RnonCO2[    199    ]=    0.39687085    ;
    RnonCO2[    200    ]=    0.40827014    ;
    RnonCO2[    201    ]=    0.4202703    ;
    RnonCO2[    202    ]=    0.43287302    ;
    RnonCO2[    203    ]=    0.44635066    ;
    RnonCO2[    204    ]=    0.46086195    ;
    RnonCO2[    205    ]=    0.47625741    ;
    RnonCO2[    206    ]=    0.49233715    ;
    RnonCO2[    207    ]=    0.50922498    ;
    RnonCO2[    208    ]=    0.52712569    ;
    RnonCO2[    209    ]=    0.54610485    ;
    RnonCO2[    210    ]=    0.56604558    ;
    RnonCO2[    211    ]=    0.58635079    ;
    RnonCO2[    212    ]=    0.6067464    ;
    RnonCO2[    213    ]=    0.6271609    ;
    RnonCO2[    214    ]=    0.6475189    ;
    RnonCO2[    215    ]=    0.6673124    ;
    RnonCO2[    216    ]=    0.6879383    ;
    RnonCO2[    217    ]=    0.7090557    ;
    RnonCO2[    218    ]=    0.7289172    ;
    RnonCO2[    219    ]=    0.7480442    ;
    RnonCO2[    220    ]=    0.766979    ;
    RnonCO2[    221    ]=    0.7850643    ;
    RnonCO2[    222    ]=    0.803085    ;
    RnonCO2[    223    ]=    0.8236521    ;
    RnonCO2[    224    ]=    0.8452713    ;
    RnonCO2[    225    ]=    0.864246    ;
    RnonCO2[    226    ]=    0.8809743    ;
    RnonCO2[    227    ]=    0.8949711    ;
    RnonCO2[    228    ]=    0.9053237    ;
    RnonCO2[    229    ]=    0.914042    ;
    RnonCO2[    230    ]=    0.9219104    ;
    RnonCO2[    231    ]=    0.9290394    ;
    RnonCO2[    232    ]=    0.9359876    ;
    RnonCO2[    233    ]=    0.9434778    ;
    RnonCO2[    234    ]=    0.9502339    ;
    RnonCO2[    235    ]=    0.9548618    ;
    RnonCO2[    236    ]=    0.9587726    ;
    RnonCO2[    237    ]=    0.9638465    ;
    RnonCO2[    238    ]=    0.9690029    ;
    RnonCO2[    239    ]=    0.972858    ;
    RnonCO2[    240    ]=    0.9761909    ;
    RnonCO2[    241    ]=    0.9798628    ;
    RnonCO2[    242    ]=    0.9841731    ;
    RnonCO2[    243    ]=    0.9894354    ;
    RnonCO2[    244    ]=    0.9954191    ;
    RnonCO2[    245    ]=    1.00198    ;
    RnonCO2[    246    ]=    1.0091146    ;
    RnonCO2[    247    ]=    1.0168268    ;
    RnonCO2[    248    ]=    1.0251231    ;
    RnonCO2[    249    ]=    1.0339608    ;
    RnonCO2[    250    ]=    1.0433067    ;
    RnonCO2[    251    ]=    1.0531035    ;
    RnonCO2[    252    ]=    1.0633085    ;
    RnonCO2[    253    ]=    1.0738918    ;
    RnonCO2[    254    ]=    1.084804    ;
    RnonCO2[    255    ]=    1.0960028    ;
    RnonCO2[    256    ]=    1.1074145    ;
    RnonCO2[    257    ]=    1.1189068    ;
    RnonCO2[    258    ]=    1.1304114    ;
    RnonCO2[    259    ]=    1.1419446    ;
    RnonCO2[    260    ]=    1.1535202    ;
    RnonCO2[    261    ]=    1.1651552    ;
    RnonCO2[    262    ]=    1.1768474    ;
    RnonCO2[    263    ]=    1.1885811    ;
    RnonCO2[    264    ]=    1.2003455    ;
    RnonCO2[    265    ]=    1.2121353    ;
    RnonCO2[    266    ]=    1.2239442    ;
    RnonCO2[    267    ]=    1.2358634    ;
    RnonCO2[    268    ]=    1.2479562    ;
    RnonCO2[    269    ]=    1.2601642    ;
    RnonCO2[    270    ]=    1.2724369    ;
    RnonCO2[    271    ]=    1.284734    ;
    RnonCO2[    272    ]=    1.297029    ;
    RnonCO2[    273    ]=    1.3092838    ;
    RnonCO2[    274    ]=    1.321453    ;
    RnonCO2[    275    ]=    1.3335026    ;
    RnonCO2[    276    ]=    1.3454097    ;
    RnonCO2[    277    ]=    1.3571802    ;
    RnonCO2[    278    ]=    1.3688174    ;
    RnonCO2[    279    ]=    1.3803494    ;
    RnonCO2[    280    ]=    1.39185    ;
    RnonCO2[    281    ]=    1.403377    ;
    RnonCO2[    282    ]=    1.4149773    ;
    RnonCO2[    283    ]=    1.4266939    ;
    RnonCO2[    284    ]=    1.4385426    ;
    RnonCO2[    285    ]=    1.450519    ;
    RnonCO2[    286    ]=    1.4626211    ;
    RnonCO2[    287    ]=    1.4746666    ;
    RnonCO2[    288    ]=    1.4864856    ;
    RnonCO2[    289    ]=    1.4980611    ;
    RnonCO2[    290    ]=    1.5093913    ;
    RnonCO2[    291    ]=    1.5205293    ;
    RnonCO2[    292    ]=    1.5315196    ;
    RnonCO2[    293    ]=    1.5424005    ;
    RnonCO2[    294    ]=    1.5532103    ;
    RnonCO2[    295    ]=    1.5639631    ;
    RnonCO2[    296    ]=    1.5746545    ;
    RnonCO2[    297    ]=    1.5851241    ;
    RnonCO2[    298    ]=    1.5952297    ;
    RnonCO2[    299    ]=    1.6050005    ;
    RnonCO2[    300    ]=    1.6144203    ;
    RnonCO2[    301    ]=    1.6234876    ;
    RnonCO2[    302    ]=    1.6322598    ;
    RnonCO2[    303    ]=    1.640786    ;
    RnonCO2[    304    ]=    1.6491097    ;
    RnonCO2[    305    ]=    1.6572747    ;
    RnonCO2[    306    ]=    1.6652999    ;
    RnonCO2[    307    ]=    1.6732147    ;
    RnonCO2[    308    ]=    1.6810498    ;
    RnonCO2[    309    ]=    1.6888091    ;
    RnonCO2[    310    ]=    1.6964967    ;
    RnonCO2[    311    ]=    1.7040719    ;
    RnonCO2[    312    ]=    1.7115112    ;
    RnonCO2[    313    ]=    1.7188534    ;
    RnonCO2[    314    ]=    1.7261326    ;
    RnonCO2[    315    ]=    1.7333772    ;
    RnonCO2[    316    ]=    1.740617    ;
    RnonCO2[    317    ]=    1.7478939    ;
    RnonCO2[    318    ]=    1.7552303    ;
    RnonCO2[    319    ]=    1.7626127    ;
    RnonCO2[    320    ]=    1.7700344    ;
    RnonCO2[    321    ]=    1.7774893    ;
    RnonCO2[    322    ]=    1.784927    ;
    RnonCO2[    323    ]=    1.7923144    ;
    RnonCO2[    324    ]=    1.7996837    ;
    RnonCO2[    325    ]=    1.8070631    ;
    RnonCO2[    326    ]=    1.8144777    ;
    RnonCO2[    327    ]=    1.8218089    ;
    RnonCO2[    328    ]=    1.8289238    ;
    RnonCO2[    329    ]=    1.8358256    ;
    RnonCO2[    330    ]=    1.8425207    ;
    RnonCO2[    331    ]=    1.8490211    ;
    RnonCO2[    332    ]=    1.8553386    ;
    RnonCO2[    333    ]=    1.861438    ;
    RnonCO2[    334    ]=    1.8673005    ;
    RnonCO2[    335    ]=    1.872973    ;
    RnonCO2[    336    ]=    1.8784998    ;
    RnonCO2[    337    ]=    1.8838116    ;
    RnonCO2[    338    ]=    1.8888456    ;
    RnonCO2[    339    ]=    1.8936276    ;
    RnonCO2[    340    ]=    1.898161    ;
    RnonCO2[    341    ]=    1.9024525    ;
    RnonCO2[    342    ]=    1.9065143    ;
    RnonCO2[    343    ]=    1.9103591    ;
    RnonCO2[    344    ]=    1.9139524    ;
    RnonCO2[    345    ]=    1.9172751    ;
    RnonCO2[    346    ]=    1.9203756    ;
    RnonCO2[    347    ]=    1.9232997    ;
    RnonCO2[    348    ]=    1.9260863    ;
    RnonCO2[    349    ]=    1.9287756    ;
    RnonCO2[    350    ]=    1.931381    ;
    RnonCO2[    351    ]=    1.9338935    ;
    RnonCO2[    352    ]=    1.9363084    ;
    RnonCO2[    353    ]=    1.9386273    ;
    RnonCO2[    354    ]=    1.940853    ;
    RnonCO2[    355    ]=    1.9429416    ;
    RnonCO2[    356    ]=    1.9448652    ;
    RnonCO2[    357    ]=    1.9466644    ;
    RnonCO2[    358    ]=    1.9483786    ;
    RnonCO2[    359    ]=    1.950041    ;
    RnonCO2[    360    ]=    1.9516856    ;
    RnonCO2[    361    ]=    1.9533204    ;
    RnonCO2[    362    ]=    1.9549312    ;
    RnonCO2[    363    ]=    1.9565073    ;
    RnonCO2[    364    ]=    1.9580454    ;
    RnonCO2[    365    ]=    1.9595441    ;
    RnonCO2[    366    ]=    1.9609555    ;
    RnonCO2[    367    ]=    1.9622472    ;
    RnonCO2[    368    ]=    1.9634567    ;
    RnonCO2[    369    ]=    1.9646211    ;
    RnonCO2[    370    ]=    1.9657705    ;
    RnonCO2[    371    ]=    1.9669362    ;
    RnonCO2[    372    ]=    1.9681259    ;
    RnonCO2[    373    ]=    1.9693208    ;
    RnonCO2[    374    ]=    1.9705091    ;
    RnonCO2[    375    ]=    1.9716846    ;
    RnonCO2[    376    ]=    1.9728438    ;
    RnonCO2[    377    ]=    1.9739366    ;
    RnonCO2[    378    ]=    1.9749294    ;
    RnonCO2[    379    ]=    1.9758573    ;
    RnonCO2[    380    ]=    1.9767565    ;
    RnonCO2[    381    ]=    1.9776574    ;
    RnonCO2[    382    ]=    1.9785912    ;
    RnonCO2[    383    ]=    1.9795626    ;
    RnonCO2[    384    ]=    1.9805528    ;
    RnonCO2[    385    ]=    1.9815485    ;
    RnonCO2[    386    ]=    1.9825418    ;
    RnonCO2[    387    ]=    1.9835297    ;
    RnonCO2[    388    ]=    1.9844595    ;
    RnonCO2[    389    ]=    1.9852968    ;
    RnonCO2[    390    ]=    1.9860737    ;
    RnonCO2[    391    ]=    1.9868278    ;
    RnonCO2[    392    ]=    1.9875876    ;
    RnonCO2[    393    ]=    1.9883847    ;
    RnonCO2[    394    ]=    1.9892229    ;
    RnonCO2[    395    ]=    1.9900824    ;
    RnonCO2[    396    ]=    1.9909497    ;
    RnonCO2[    397    ]=    1.9918147    ;
    RnonCO2[    398    ]=    1.9926738    ;
    RnonCO2[    399    ]=    1.9934757    ;
    RnonCO2[    400    ]=    1.9941844    ;
    RnonCO2[    401    ]=    1.9948328    ;
    RnonCO2[    402    ]=    1.9954579    ;
    RnonCO2[    403    ]=    1.9960894    ;
    RnonCO2[    404    ]=    1.9967601    ;
    RnonCO2[    405    ]=    1.9974721    ;
    RnonCO2[    406    ]=    1.9982083    ;
    RnonCO2[    407    ]=    1.9989518    ;
    RnonCO2[    408    ]=    1.9996957    ;
    RnonCO2[    409    ]=    2.0004323    ;
    RnonCO2[    410    ]=    2.0011115    ;
    RnonCO2[    411    ]=    2.0016989    ;
    RnonCO2[    412    ]=    2.0022257    ;
    RnonCO2[    413    ]=    2.0027304    ;
    RnonCO2[    414    ]=    2.0032428    ;
    RnonCO2[    415    ]=    2.0037938    ;
    RnonCO2[    416    ]=    2.0043879    ;
    RnonCO2[    417    ]=    2.0050072    ;
    RnonCO2[    418    ]=    2.0056344    ;
    RnonCO2[    419    ]=    2.0062615    ;
    RnonCO2[    420    ]=    2.0068815    ;
    RnonCO2[    421    ]=    2.0074457    ;
    RnonCO2[    422    ]=    2.0079169    ;
    RnonCO2[    423    ]=    2.0083285    ;
    RnonCO2[    424    ]=    2.0087185    ;
    RnonCO2[    425    ]=    2.0091161    ;
    RnonCO2[    426    ]=    2.0095542    ;
    RnonCO2[    427    ]=    2.010036    ;
    RnonCO2[    428    ]=    2.0105425    ;
    RnonCO2[    429    ]=    2.011058    ;
    RnonCO2[    430    ]=    2.011572    ;
    RnonCO2[    431    ]=    2.012081    ;
    RnonCO2[    432    ]=    2.012533    ;
    RnonCO2[    433    ]=    2.012893    ;
    RnonCO2[    434    ]=    2.013193    ;
    RnonCO2[    435    ]=    2.013472    ;
    RnonCO2[    436    ]=    2.013759    ;
    RnonCO2[    437    ]=    2.014085    ;
    RnonCO2[    438    ]=    2.014457    ;
    RnonCO2[    439    ]=    2.014853    ;
    RnonCO2[    440    ]=    2.015255    ;
    RnonCO2[    441    ]=    2.015658    ;
    RnonCO2[    442    ]=    2.016054    ;
    RnonCO2[    443    ]=    2.016393    ;
    RnonCO2[    444    ]=    2.01664    ;
    RnonCO2[    445    ]=    2.016827    ;
    RnonCO2[    446    ]=    2.016993    ;
    RnonCO2[    447    ]=    2.017169    ;
    RnonCO2[    448    ]=    2.017384    ;
    RnonCO2[    449    ]=    2.017644    ;
    RnonCO2[    450    ]=    2.017928    ;
    RnonCO2[    451    ]=    2.01822    ;
    RnonCO2[    452    ]=    2.018512    ;
    RnonCO2[    453    ]=    2.018798    ;
    RnonCO2[    454    ]=    2.019027    ;
    RnonCO2[    455    ]=    2.019164    ;
    RnonCO2[    456    ]=    2.019243    ;
    RnonCO2[    457    ]=    2.019301    ;
    RnonCO2[    458    ]=    2.019369    ;
    RnonCO2[    459    ]=    2.019478    ;
    RnonCO2[    460    ]=    2.019632    ;
    RnonCO2[    461    ]=    2.01981    ;
    RnonCO2[    462    ]=    2.019997    ;
    RnonCO2[    463    ]=    2.020183    ;
    RnonCO2[    464    ]=    2.020363    ;
    RnonCO2[    465    ]=    2.020487    ;
    RnonCO2[    466    ]=    2.02052    ;
    RnonCO2[    467    ]=    2.020496    ;
    RnonCO2[    468    ]=    2.020451    ;
    RnonCO2[    469    ]=    2.020417    ;
    RnonCO2[    470    ]=    2.020424    ;
    RnonCO2[    471    ]=    2.020477    ;
    RnonCO2[    472    ]=    2.020554    ;
    RnonCO2[    473    ]=    2.02064    ;
    RnonCO2[    474    ]=    2.020725    ;
    RnonCO2[    475    ]=    2.020805    ;
    RnonCO2[    476    ]=    2.02083    ;
    RnonCO2[    477    ]=    2.020763    ;
    RnonCO2[    478    ]=    2.020641    ;
    RnonCO2[    479    ]=    2.0205    ;
    RnonCO2[    480    ]=    2.020369    ;
    RnonCO2[    481    ]=    2.020281    ;
    RnonCO2[    482    ]=    2.020237    ;
    RnonCO2[    483    ]=    2.02022    ;
    RnonCO2[    484    ]=    2.020211    ;
    RnonCO2[    485    ]=    2.020207    ;
    RnonCO2[    486    ]=    2.020207    ;
    RnonCO2[    487    ]=    2.020207    ;
    RnonCO2[    488    ]=    2.020207    ;
    RnonCO2[    489    ]=    2.020207    ;
    RnonCO2[    490    ]=    2.020207    ;
    RnonCO2[    491    ]=    2.020207    ;
    RnonCO2[    492    ]=    2.020207    ;
    RnonCO2[    493    ]=    2.020207    ;
    RnonCO2[    494    ]=    2.020207    ;
    RnonCO2[    495    ]=    2.020207    ;
    RnonCO2[    496    ]=    2.020207    ;
    RnonCO2[    497    ]=    2.020207    ;
    RnonCO2[    498    ]=    2.020207    ;
    RnonCO2[    499    ]=    2.020207    ;
    RnonCO2[    500    ]=    2.020207    ;
    RnonCO2[    501    ]=    2.020207    ;
    RnonCO2[    502    ]=    2.020207    ;
    RnonCO2[    503    ]=    2.020207    ;
    RnonCO2[    504    ]=    2.020207    ;
    RnonCO2[    505    ]=    2.020207    ;
    RnonCO2[    506    ]=    2.020207    ;
    RnonCO2[    507    ]=    2.020207    ;
    RnonCO2[    508    ]=    2.020207    ;
    RnonCO2[    509    ]=    2.020207    ;
    RnonCO2[    510    ]=    2.020207    ;
    RnonCO2[    511    ]=    2.020207    ;
    RnonCO2[    512    ]=    2.020207    ;
    RnonCO2[    513    ]=    2.020207    ;
    RnonCO2[    514    ]=    2.020207    ;
    RnonCO2[    515    ]=    2.020207    ;
    RnonCO2[    516    ]=    2.020207    ;
    RnonCO2[    517    ]=    2.020207    ;
    RnonCO2[    518    ]=    2.020207    ;
    RnonCO2[    519    ]=    2.020207    ;
    RnonCO2[    520    ]=    2.020207    ;
    RnonCO2[    521    ]=    2.020207    ;
    RnonCO2[    522    ]=    2.020207    ;
    RnonCO2[    523    ]=    2.020207    ;
    RnonCO2[    524    ]=    2.020207    ;
    RnonCO2[    525    ]=    2.020207    ;
    RnonCO2[    526    ]=    2.020207    ;
    RnonCO2[    527    ]=    2.020207    ;
    RnonCO2[    528    ]=    2.020207    ;
    RnonCO2[    529    ]=    2.020207    ;
    RnonCO2[    530    ]=    2.020207    ;
    RnonCO2[    531    ]=    2.020207    ;
    RnonCO2[    532    ]=    2.020207    ;
    RnonCO2[    533    ]=    2.020207    ;
    RnonCO2[    534    ]=    2.020207    ;
    RnonCO2[    535    ]=    2.020207    ;
    RnonCO2[    536    ]=    2.020207    ;
    RnonCO2[    537    ]=    2.020207    ;
    RnonCO2[    538    ]=    2.020207    ;
    RnonCO2[    539    ]=    2.020207    ;
    RnonCO2[    540    ]=    2.020207    ;
    RnonCO2[    541    ]=    2.020207    ;
    RnonCO2[    542    ]=    2.020207    ;
    RnonCO2[    543    ]=    2.020207    ;
    RnonCO2[    544    ]=    2.020207    ;
    RnonCO2[    545    ]=    2.020207    ;
    RnonCO2[    546    ]=    2.020207    ;
    RnonCO2[    547    ]=    2.020207    ;
    RnonCO2[    548    ]=    2.020207    ;
    RnonCO2[    549    ]=    2.020207    ;
    RnonCO2[    550    ]=    2.020207    ;
    RnonCO2[    551    ]=    2.020207    ;
    RnonCO2[    552    ]=    2.020207    ;
    RnonCO2[    553    ]=    2.020207    ;
    RnonCO2[    554    ]=    2.020207    ;
    RnonCO2[    555    ]=    2.020207    ;
    RnonCO2[    556    ]=    2.020207    ;
    RnonCO2[    557    ]=    2.020207    ;
    RnonCO2[    558    ]=    2.020207    ;
    RnonCO2[    559    ]=    2.020207    ;
    RnonCO2[    560    ]=    2.020207    ;
    RnonCO2[    561    ]=    2.020207    ;
    RnonCO2[    562    ]=    2.020207    ;
    RnonCO2[    563    ]=    2.020207    ;
    RnonCO2[    564    ]=    2.020207    ;
    RnonCO2[    565    ]=    2.020207    ;
    RnonCO2[    566    ]=    2.020207    ;
    RnonCO2[    567    ]=    2.020207    ;
    RnonCO2[    568    ]=    2.020207    ;
    RnonCO2[    569    ]=    2.020207    ;
    RnonCO2[    570    ]=    2.020207    ;
    RnonCO2[    571    ]=    2.020207    ;
    RnonCO2[    572    ]=    2.020207    ;
    RnonCO2[    573    ]=    2.020207    ;
    RnonCO2[    574    ]=    2.020207    ;
    RnonCO2[    575    ]=    2.020207    ;
    RnonCO2[    576    ]=    2.020207    ;
    RnonCO2[    577    ]=    2.020207    ;
    RnonCO2[    578    ]=    2.020207    ;
    RnonCO2[    579    ]=    2.020207    ;
    RnonCO2[    580    ]=    2.020207    ;
    RnonCO2[    581    ]=    2.020207    ;
    RnonCO2[    582    ]=    2.020207    ;
    RnonCO2[    583    ]=    2.020207    ;
    RnonCO2[    584    ]=    2.020207    ;
    RnonCO2[    585    ]=    2.020207    ;
    RnonCO2[    586    ]=    2.020207    ;
    RnonCO2[    587    ]=    2.020207    ;
    RnonCO2[    588    ]=    2.020207    ;
    RnonCO2[    589    ]=    2.020207    ;
    RnonCO2[    590    ]=    2.020207    ;
    RnonCO2[    591    ]=    2.020207    ;
    RnonCO2[    592    ]=    2.020207    ;
    RnonCO2[    593    ]=    2.020207    ;
    RnonCO2[    594    ]=    2.020207    ;
    RnonCO2[    595    ]=    2.020207    ;
    RnonCO2[    596    ]=    2.020207    ;
    RnonCO2[    597    ]=    2.020207    ;
    RnonCO2[    598    ]=    2.020207    ;
    RnonCO2[    599    ]=    2.020207    ;
    RnonCO2[    600    ]=    2.020207    ;
    RnonCO2[    601    ]=    2.020207    ;
    RnonCO2[    602    ]=    2.020207    ;
    RnonCO2[    603    ]=    2.020207    ;
    RnonCO2[    604    ]=    2.020207    ;
    RnonCO2[    605    ]=    2.020207    ;
    RnonCO2[    606    ]=    2.020207    ;
    RnonCO2[    607    ]=    2.020207    ;
    RnonCO2[    608    ]=    2.020207    ;
    RnonCO2[    609    ]=    2.020207    ;
    RnonCO2[    610    ]=    2.020207    ;
    RnonCO2[    611    ]=    2.020207    ;
    RnonCO2[    612    ]=    2.020207    ;
    RnonCO2[    613    ]=    2.020207    ;
    RnonCO2[    614    ]=    2.020207    ;
    RnonCO2[    615    ]=    2.020207    ;
    RnonCO2[    616    ]=    2.020207    ;
    RnonCO2[    617    ]=    2.020207    ;
    RnonCO2[    618    ]=    2.020207    ;
    RnonCO2[    619    ]=    2.020207    ;
    RnonCO2[    620    ]=    2.020207    ;
    RnonCO2[    621    ]=    2.020207    ;
    RnonCO2[    622    ]=    2.020207    ;
    RnonCO2[    623    ]=    2.020207    ;
    RnonCO2[    624    ]=    2.020207    ;
    RnonCO2[    625    ]=    2.020207    ;
    RnonCO2[    626    ]=    2.020207    ;
    RnonCO2[    627    ]=    2.020207    ;
    RnonCO2[    628    ]=    2.020207    ;
    RnonCO2[    629    ]=    2.020207    ;
    RnonCO2[    630    ]=    2.020207    ;
    RnonCO2[    631    ]=    2.020207    ;
    RnonCO2[    632    ]=    2.020207    ;
    RnonCO2[    633    ]=    2.020207    ;
    RnonCO2[    634    ]=    2.020207    ;
    RnonCO2[    635    ]=    2.020207    ;
    RnonCO2[    636    ]=    2.020207    ;
    RnonCO2[    637    ]=    2.020207    ;
    RnonCO2[    638    ]=    2.020207    ;
    RnonCO2[    639    ]=    2.020207    ;
    RnonCO2[    640    ]=    2.020207    ;
    RnonCO2[    641    ]=    2.020207    ;
    RnonCO2[    642    ]=    2.020207    ;
    RnonCO2[    643    ]=    2.020207    ;
    RnonCO2[    644    ]=    2.020207    ;
    RnonCO2[    645    ]=    2.020207    ;
    RnonCO2[    646    ]=    2.020207    ;
    RnonCO2[    647    ]=    2.020207    ;
    RnonCO2[    648    ]=    2.020207    ;
    RnonCO2[    649    ]=    2.020207    ;
    RnonCO2[    650    ]=    2.020207    ;
    RnonCO2[    651    ]=    2.020207    ;
    RnonCO2[    652    ]=    2.020207    ;
    RnonCO2[    653    ]=    2.020207    ;
    RnonCO2[    654    ]=    2.020207    ;
    RnonCO2[    655    ]=    2.020207    ;
    RnonCO2[    656    ]=    2.020207    ;
    RnonCO2[    657    ]=    2.020207    ;
    RnonCO2[    658    ]=    2.020207    ;
    RnonCO2[    659    ]=    2.020207    ;
    RnonCO2[    660    ]=    2.020207    ;
    RnonCO2[    661    ]=    2.020207    ;
    RnonCO2[    662    ]=    2.020207    ;
    RnonCO2[    663    ]=    2.020207    ;
    RnonCO2[    664    ]=    2.020207    ;
    RnonCO2[    665    ]=    2.020207    ;
    RnonCO2[    666    ]=    2.020207    ;
    RnonCO2[    667    ]=    2.020207    ;
    RnonCO2[    668    ]=    2.020207    ;
    RnonCO2[    669    ]=    2.020207    ;
    RnonCO2[    670    ]=    2.020207    ;
    RnonCO2[    671    ]=    2.020207    ;
    RnonCO2[    672    ]=    2.020207    ;
    RnonCO2[    673    ]=    2.020207    ;
    RnonCO2[    674    ]=    2.020207    ;
    RnonCO2[    675    ]=    2.020207    ;
    RnonCO2[    676    ]=    2.020207    ;
    RnonCO2[    677    ]=    2.020207    ;
    RnonCO2[    678    ]=    2.020207    ;
    RnonCO2[    679    ]=    2.020207    ;
    RnonCO2[    680    ]=    2.020207    ;
    RnonCO2[    681    ]=    2.020207    ;
    RnonCO2[    682    ]=    2.020207    ;
    RnonCO2[    683    ]=    2.020207    ;
    RnonCO2[    684    ]=    2.020207    ;
    RnonCO2[    685    ]=    2.020207    ;
    RnonCO2[    686    ]=    2.020207    ;
    RnonCO2[    687    ]=    2.020207    ;
    RnonCO2[    688    ]=    2.020207    ;
    RnonCO2[    689    ]=    2.020207    ;
    RnonCO2[    690    ]=    2.020207    ;
    RnonCO2[    691    ]=    2.020207    ;
    RnonCO2[    692    ]=    2.020207    ;
    RnonCO2[    693    ]=    2.020207    ;
    RnonCO2[    694    ]=    2.020207    ;
    RnonCO2[    695    ]=    2.020207    ;
    RnonCO2[    696    ]=    2.020207    ;
    RnonCO2[    697    ]=    2.020207    ;
    RnonCO2[    698    ]=    2.020207    ;
    RnonCO2[    699    ]=    2.020207    ;
    RnonCO2[    700    ]=    2.020207    ;
    RnonCO2[    701    ]=    2.020207    ;
    RnonCO2[    702    ]=    2.020207    ;
    RnonCO2[    703    ]=    2.020207    ;
    RnonCO2[    704    ]=    2.020207    ;
    RnonCO2[    705    ]=    2.020207    ;
    RnonCO2[    706    ]=    2.020207    ;
    RnonCO2[    707    ]=    2.020207    ;
    RnonCO2[    708    ]=    2.020207    ;
    RnonCO2[    709    ]=    2.020207    ;
    RnonCO2[    710    ]=    2.020207    ;
    RnonCO2[    711    ]=    2.020207    ;
    RnonCO2[    712    ]=    2.020207    ;
    RnonCO2[    713    ]=    2.020207    ;
    RnonCO2[    714    ]=    2.020207    ;
    RnonCO2[    715    ]=    2.020207    ;
    RnonCO2[    716    ]=    2.020207    ;
    RnonCO2[    717    ]=    2.020207    ;
    RnonCO2[    718    ]=    2.020207    ;
    RnonCO2[    719    ]=    2.020207    ;
    RnonCO2[    720    ]=    2.020207    ;
    RnonCO2[    721    ]=    2.020207    ;
    RnonCO2[    722    ]=    2.020207    ;
    RnonCO2[    723    ]=    2.020207    ;
    RnonCO2[    724    ]=    2.020207    ;
    RnonCO2[    725    ]=    2.020207    ;
    RnonCO2[    726    ]=    2.020207    ;
    RnonCO2[    727    ]=    2.020207    ;
    RnonCO2[    728    ]=    2.020207    ;
    RnonCO2[    729    ]=    2.020207    ;
    RnonCO2[    730    ]=    2.020207    ;
    RnonCO2[    731    ]=    2.020207    ;
    RnonCO2[    732    ]=    2.020207    ;
    RnonCO2[    733    ]=    2.020207    ;
    RnonCO2[    734    ]=    2.020207    ;
    RnonCO2[    735    ]=    2.020207    ;
    for(int i = 736; i<tmax; i++)
    {
        RnonCO2[i] = RnonCO2[i-1];
    }
}

void setR_aerosol_RCP85(void)
{
    RnonCO2[    0    ]=    0    ;
    RnonCO2[    1    ]=    0.000955298    ;
    RnonCO2[    2    ]=    0.001424721    ;
    RnonCO2[    3    ]=    0.0001974    ;
    RnonCO2[    4    ]=    -0.001278437    ;
    RnonCO2[    5    ]=    -0.002733841    ;
    RnonCO2[    6    ]=    -0.004541334    ;
    RnonCO2[    7    ]=    -0.00632004    ;
    RnonCO2[    8    ]=    -0.007680948    ;
    RnonCO2[    9    ]=    -0.008994568    ;
    RnonCO2[    10    ]=    -0.010260775    ;
    RnonCO2[    11    ]=    -0.011479866    ;
    RnonCO2[    12    ]=    -0.012652425    ;
    RnonCO2[    13    ]=    -0.013779231    ;
    RnonCO2[    14    ]=    -0.014861204    ;
    RnonCO2[    15    ]=    -0.015900176    ;
    RnonCO2[    16    ]=    -0.017335746    ;
    RnonCO2[    17    ]=    -0.01873479    ;
    RnonCO2[    18    ]=    -0.019659073    ;
    RnonCO2[    19    ]=    -0.020540613    ;
    RnonCO2[    20    ]=    -0.02137714    ;
    RnonCO2[    21    ]=    -0.022166399    ;
    RnonCO2[    22    ]=    -0.022906132    ;
    RnonCO2[    23    ]=    -0.023594082    ;
    RnonCO2[    24    ]=    -0.024227995    ;
    RnonCO2[    25    ]=    -0.024806236    ;
    RnonCO2[    26    ]=    -0.025775743    ;
    RnonCO2[    27    ]=    -0.026691982    ;
    RnonCO2[    28    ]=    -0.027110281    ;
    RnonCO2[    29    ]=    -0.02747885    ;
    RnonCO2[    30    ]=    -0.027799526    ;
    RnonCO2[    31    ]=    -0.028074106    ;
    RnonCO2[    32    ]=    -0.02875039    ;
    RnonCO2[    33    ]=    -0.029383792    ;
    RnonCO2[    34    ]=    -0.02953002    ;
    RnonCO2[    35    ]=    -0.03007996    ;
    RnonCO2[    36    ]=    -0.03058546    ;
    RnonCO2[    37    ]=    -0.03148571    ;
    RnonCO2[    38    ]=    -0.03189741    ;
    RnonCO2[    39    ]=    -0.03138062    ;
    RnonCO2[    40    ]=    -0.03126848    ;
    RnonCO2[    41    ]=    -0.03156144    ;
    RnonCO2[    42    ]=    -0.03182181    ;
    RnonCO2[    43    ]=    -0.03161267    ;
    RnonCO2[    44    ]=    -0.03137821    ;
    RnonCO2[    45    ]=    -0.03112038    ;
    RnonCO2[    46    ]=    -0.03127367    ;
    RnonCO2[    47    ]=    -0.0313998    ;
    RnonCO2[    48    ]=    -0.03106322    ;
    RnonCO2[    49    ]=    -0.03070135    ;
    RnonCO2[    50    ]=    -0.03074616    ;
    RnonCO2[    51    ]=    -0.03119614    ;
    RnonCO2[    52    ]=    -0.03161872    ;
    RnonCO2[    53    ]=    -0.03158785    ;
    RnonCO2[    54    ]=    -0.03110696    ;
    RnonCO2[    55    ]=    -0.03060634    ;
    RnonCO2[    56    ]=    -0.0300866    ;
    RnonCO2[    57    ]=    -0.02996996    ;
    RnonCO2[    58    ]=    -0.03025404    ;
    RnonCO2[    59    ]=    -0.03009594    ;
    RnonCO2[    60    ]=    -0.02991636    ;
    RnonCO2[    61    ]=    -0.02971869    ;
    RnonCO2[    62    ]=    -0.02950042    ;
    RnonCO2[    63    ]=    -0.02926502    ;
    RnonCO2[    64    ]=    -0.02859871    ;
    RnonCO2[    65    ]=    -0.03037359    ;
    RnonCO2[    66    ]=    -0.03172034    ;
    RnonCO2[    67    ]=    -0.03059392    ;
    RnonCO2[    68    ]=    -0.03026959    ;
    RnonCO2[    69    ]=    -0.02993194    ;
    RnonCO2[    70    ]=    -0.02957653    ;
    RnonCO2[    71    ]=    -0.03080027    ;
    RnonCO2[    72    ]=    -0.03160022    ;
    RnonCO2[    73    ]=    -0.03117789    ;
    RnonCO2[    74    ]=    -0.03112982    ;
    RnonCO2[    75    ]=    -0.03144654    ;
    RnonCO2[    76    ]=    -0.03173222    ;
    RnonCO2[    77    ]=    -0.03198224    ;
    RnonCO2[    78    ]=    -0.03220368    ;
    RnonCO2[    79    ]=    -0.03239696    ;
    RnonCO2[    80    ]=    -0.03371239    ;
    RnonCO2[    81    ]=    -0.03424533    ;
    RnonCO2[    82    ]=    -0.0343764    ;
    RnonCO2[    83    ]=    -0.03487942    ;
    RnonCO2[    84    ]=    -0.03536957    ;
    RnonCO2[    85    ]=    -0.03686306    ;
    RnonCO2[    86    ]=    -0.03665392    ;
    RnonCO2[    87    ]=    -0.04248424    ;
    RnonCO2[    88    ]=    -0.04930444    ;
    RnonCO2[    89    ]=    -0.05182621    ;
    RnonCO2[    90    ]=    -0.0542671    ;
    RnonCO2[    91    ]=    -0.05573158    ;
    RnonCO2[    92    ]=    -0.05765815    ;
    RnonCO2[    93    ]=    -0.05821397    ;
    RnonCO2[    94    ]=    -0.06005825    ;
    RnonCO2[    95    ]=    -0.0683501    ;
    RnonCO2[    96    ]=    -0.08104112    ;
    RnonCO2[    97    ]=    -0.07832852    ;
    RnonCO2[    98    ]=    -0.07177024    ;
    RnonCO2[    99    ]=    -0.07363036    ;
    RnonCO2[    100    ]=    -0.07696442    ;
    RnonCO2[    101    ]=    -0.08462429    ;
    RnonCO2[    102    ]=    -0.09275025    ;
    RnonCO2[    103    ]=    -0.09511911    ;
    RnonCO2[    104    ]=    -0.0930339    ;
    RnonCO2[    105    ]=    -0.09237765    ;
    RnonCO2[    106    ]=    -0.10165608    ;
    RnonCO2[    107    ]=    -0.12466661    ;
    RnonCO2[    108    ]=    -0.14405058    ;
    RnonCO2[    109    ]=    -0.14773339    ;
    RnonCO2[    110    ]=    -0.14777672    ;
    RnonCO2[    111    ]=    -0.1470811    ;
    RnonCO2[    112    ]=    -0.14255587    ;
    RnonCO2[    113    ]=    -0.14024698    ;
    RnonCO2[    114    ]=    -0.14393886    ;
    RnonCO2[    115    ]=    -0.15612866    ;
    RnonCO2[    116    ]=    -0.164782    ;
    RnonCO2[    117    ]=    -0.16302445    ;
    RnonCO2[    118    ]=    -0.16326054    ;
    RnonCO2[    119    ]=    -0.16496204    ;
    RnonCO2[    120    ]=    -0.16272157    ;
    RnonCO2[    121    ]=    -0.1612244    ;
    RnonCO2[    122    ]=    -0.16356025    ;
    RnonCO2[    123    ]=    -0.17242315    ;
    RnonCO2[    124    ]=    -0.17632405    ;
    RnonCO2[    125    ]=    -0.17855867    ;
    RnonCO2[    126    ]=    -0.18527891    ;
    RnonCO2[    127    ]=    -0.18997972    ;
    RnonCO2[    128    ]=    -0.1933518    ;
    RnonCO2[    129    ]=    -0.19699663    ;
    RnonCO2[    130    ]=    -0.20233059    ;
    RnonCO2[    131    ]=    -0.2051441    ;
    RnonCO2[    132    ]=    -0.20577589    ;
    RnonCO2[    133    ]=    -0.20720143    ;
    RnonCO2[    134    ]=    -0.21030464    ;
    RnonCO2[    135    ]=    -0.21375193    ;
    RnonCO2[    136    ]=    -0.22004069    ;
    RnonCO2[    137    ]=    -0.22693377    ;
    RnonCO2[    138    ]=    -0.23402972    ;
    RnonCO2[    139    ]=    -0.2411565    ;
    RnonCO2[    140    ]=    -0.2471181    ;
    RnonCO2[    141    ]=    -0.255172    ;
    RnonCO2[    142    ]=    -0.26315304    ;
    RnonCO2[    143    ]=    -0.2636557    ;
    RnonCO2[    144    ]=    -0.26151664    ;
    RnonCO2[    145    ]=    -0.26499499    ;
    RnonCO2[    146    ]=    -0.26461175    ;
    RnonCO2[    147    ]=    -0.26044407    ;
    RnonCO2[    148    ]=    -0.25940869    ;
    RnonCO2[    149    ]=    -0.25560119    ;
    RnonCO2[    150    ]=    -0.25007798    ;
    RnonCO2[    151    ]=    -0.25106379    ;
    RnonCO2[    152    ]=    -0.25437609    ;
    RnonCO2[    153    ]=    -0.25544669    ;
    RnonCO2[    154    ]=    -0.2505011    ;
    RnonCO2[    155    ]=    -0.24919556    ;
    RnonCO2[    156    ]=    -0.24766564    ;
    RnonCO2[    157    ]=    -0.24454174    ;
    RnonCO2[    158    ]=    -0.25504195    ;
    RnonCO2[    159    ]=    -0.26312341    ;
    RnonCO2[    160    ]=    -0.26314088    ;
    RnonCO2[    161    ]=    -0.26376161    ;
    RnonCO2[    162    ]=    -0.26809378    ;
    RnonCO2[    163    ]=    -0.27284291    ;
    RnonCO2[    164    ]=    -0.27584022    ;
    RnonCO2[    165    ]=    -0.27557411    ;
    RnonCO2[    166    ]=    -0.26967283    ;
    RnonCO2[    167    ]=    -0.26315755    ;
    RnonCO2[    168    ]=    -0.26083606    ;
    RnonCO2[    169    ]=    -0.26325195    ;
    RnonCO2[    170    ]=    -0.2652416    ;
    RnonCO2[    171    ]=    -0.26824619    ;
    RnonCO2[    172    ]=    -0.26930622    ;
    RnonCO2[    173    ]=    -0.26149511    ;
    RnonCO2[    174    ]=    -0.25651091    ;
    RnonCO2[    175    ]=    -0.26035065    ;
    RnonCO2[    176    ]=    -0.26656695    ;
    RnonCO2[    177    ]=    -0.27194936    ;
    RnonCO2[    178    ]=    -0.274817    ;
    RnonCO2[    179    ]=    -0.27878403    ;
    RnonCO2[    180    ]=    -0.28302569    ;
    RnonCO2[    181    ]=    -0.29104814    ;
    RnonCO2[    182    ]=    -0.30416561    ;
    RnonCO2[    183    ]=    -0.31716714    ;
    RnonCO2[    184    ]=    -0.3246442    ;
    RnonCO2[    185    ]=    -0.3327935    ;
    RnonCO2[    186    ]=    -0.35639494    ;
    RnonCO2[    187    ]=    -0.38051016    ;
    RnonCO2[    188    ]=    -0.39378465    ;
    RnonCO2[    189    ]=    -0.40659776    ;
    RnonCO2[    190    ]=    -0.4196919    ;
    RnonCO2[    191    ]=    -0.43302942    ;
    RnonCO2[    192    ]=    -0.44651364    ;
    RnonCO2[    193    ]=    -0.45569127    ;
    RnonCO2[    194    ]=    -0.45642498    ;
    RnonCO2[    195    ]=    -0.45551768    ;
    RnonCO2[    196    ]=    -0.47183293    ;
    RnonCO2[    197    ]=    -0.49188519    ;
    RnonCO2[    198    ]=    -0.50135461    ;
    RnonCO2[    199    ]=    -0.5101619    ;
    RnonCO2[    200    ]=    -0.51738459    ;
    RnonCO2[    201    ]=    -0.52416219    ;
    RnonCO2[    202    ]=    -0.53325402    ;
    RnonCO2[    203    ]=    -0.54124791    ;
    RnonCO2[    204    ]=    -0.54379714    ;
    RnonCO2[    205    ]=    -0.53664772    ;
    RnonCO2[    206    ]=    -0.52607217    ;
    RnonCO2[    207    ]=    -0.52739363    ;
    RnonCO2[    208    ]=    -0.53724776    ;
    RnonCO2[    209    ]=    -0.54779467    ;
    RnonCO2[    210    ]=    -0.55942351    ;
    RnonCO2[    211    ]=    -0.57578959    ;
    RnonCO2[    212    ]=    -0.5916702    ;
    RnonCO2[    213    ]=    -0.5979117    ;
    RnonCO2[    214    ]=    -0.6009881    ;
    RnonCO2[    215    ]=    -0.6131431    ;
    RnonCO2[    216    ]=    -0.6319195    ;
    RnonCO2[    217    ]=    -0.6542028    ;
    RnonCO2[    218    ]=    -0.6747946    ;
    RnonCO2[    219    ]=    -0.6865596    ;
    RnonCO2[    220    ]=    -0.6942006    ;
    RnonCO2[    221    ]=    -0.7018816    ;
    RnonCO2[    222    ]=    -0.7121991    ;
    RnonCO2[    223    ]=    -0.7210817    ;
    RnonCO2[    224    ]=    -0.7262596    ;
    RnonCO2[    225    ]=    -0.7299457    ;
    RnonCO2[    226    ]=    -0.7418714    ;
    RnonCO2[    227    ]=    -0.7583892    ;
    RnonCO2[    228    ]=    -0.7656833    ;
    RnonCO2[    229    ]=    -0.768847    ;
    RnonCO2[    230    ]=    -0.7655134    ;
    RnonCO2[    231    ]=    -0.7588829    ;
    RnonCO2[    232    ]=    -0.7657109    ;
    RnonCO2[    233    ]=    -0.7804313    ;
    RnonCO2[    234    ]=    -0.7902367    ;
    RnonCO2[    235    ]=    -0.8012413    ;
    RnonCO2[    236    ]=    -0.8096987    ;
    RnonCO2[    237    ]=    -0.8105819    ;
    RnonCO2[    238    ]=    -0.8112899    ;
    RnonCO2[    239    ]=    -0.8123564    ;
    RnonCO2[    240    ]=    -0.8099077    ;
    RnonCO2[    241    ]=    -0.8000798    ;
    RnonCO2[    242    ]=    -0.7865755    ;
    RnonCO2[    243    ]=    -0.7727582    ;
    RnonCO2[    244    ]=    -0.7584482    ;
    RnonCO2[    245    ]=    -0.7472198    ;
    RnonCO2[    246    ]=    -0.742728    ;
    RnonCO2[    247    ]=    -0.7406136    ;
    RnonCO2[    248    ]=    -0.7377074    ;
    RnonCO2[    249    ]=    -0.7349095    ;
    RnonCO2[    250    ]=    -0.7320007    ;
    RnonCO2[    251    ]=    -0.7289608    ;
    RnonCO2[    252    ]=    -0.7257972    ;
    RnonCO2[    253    ]=    -0.7225277    ;
    RnonCO2[    254    ]=    -0.7187237    ;
    RnonCO2[    255    ]=    -0.7138313    ;
    RnonCO2[    256    ]=    -0.7074799    ;
    RnonCO2[    257    ]=    -0.7011371    ;
    RnonCO2[    258    ]=    -0.6956635    ;
    RnonCO2[    259    ]=    -0.6901843    ;
    RnonCO2[    260    ]=    -0.6846945    ;
    RnonCO2[    261    ]=    -0.6792039    ;
    RnonCO2[    262    ]=    -0.6737169    ;
    RnonCO2[    263    ]=    -0.6682408    ;
    RnonCO2[    264    ]=    -0.6627814    ;
    RnonCO2[    265    ]=    -0.6558025    ;
    RnonCO2[    266    ]=    -0.6457586    ;
    RnonCO2[    267    ]=    -0.6349733    ;
    RnonCO2[    268    ]=    -0.6249181    ;
    RnonCO2[    269    ]=    -0.614736    ;
    RnonCO2[    270    ]=    -0.604427    ;
    RnonCO2[    271    ]=    -0.5939962    ;
    RnonCO2[    272    ]=    -0.5834541    ;
    RnonCO2[    273    ]=    -0.572811    ;
    RnonCO2[    274    ]=    -0.5620802    ;
    RnonCO2[    275    ]=    -0.5516009    ;
    RnonCO2[    276    ]=    -0.5417168    ;
    RnonCO2[    277    ]=    -0.5318895    ;
    RnonCO2[    278    ]=    -0.5218095    ;
    RnonCO2[    279    ]=    -0.5117195    ;
    RnonCO2[    280    ]=    -0.5016219    ;
    RnonCO2[    281    ]=    -0.4915198    ;
    RnonCO2[    282    ]=    -0.4814209    ;
    RnonCO2[    283    ]=    -0.4713374    ;
    RnonCO2[    284    ]=    -0.4612791    ;
    RnonCO2[    285    ]=    -0.4523781    ;
    RnonCO2[    286    ]=    -0.4457713    ;
    RnonCO2[    287    ]=    -0.440062    ;
    RnonCO2[    288    ]=    -0.4341524    ;
    RnonCO2[    289    ]=    -0.4283564    ;
    RnonCO2[    290    ]=    -0.4226727    ;
    RnonCO2[    291    ]=    -0.4170824    ;
    RnonCO2[    292    ]=    -0.4115698    ;
    RnonCO2[    293    ]=    -0.4061213    ;
    RnonCO2[    294    ]=    -0.4007228    ;
    RnonCO2[    295    ]=    -0.3956366    ;
    RnonCO2[    296    ]=    -0.3911326    ;
    RnonCO2[    297    ]=    -0.3868856    ;
    RnonCO2[    298    ]=    -0.3826761    ;
    RnonCO2[    299    ]=    -0.378604    ;
    RnonCO2[    300    ]=    -0.3746703    ;
    RnonCO2[    301    ]=    -0.3708712    ;
    RnonCO2[    302    ]=    -0.3671855    ;
    RnonCO2[    303    ]=    -0.3635951    ;
    RnonCO2[    304    ]=    -0.3600835    ;
    RnonCO2[    305    ]=    -0.3568111    ;
    RnonCO2[    306    ]=    -0.3539465    ;
    RnonCO2[    307    ]=    -0.3512216    ;
    RnonCO2[    308    ]=    -0.3484468    ;
    RnonCO2[    309    ]=    -0.3456989    ;
    RnonCO2[    310    ]=    -0.3429758    ;
    RnonCO2[    311    ]=    -0.3402884    ;
    RnonCO2[    312    ]=    -0.3376425    ;
    RnonCO2[    313    ]=    -0.3350249    ;
    RnonCO2[    314    ]=    -0.3324246    ;
    RnonCO2[    315    ]=    -0.3289265    ;
    RnonCO2[    316    ]=    -0.3236119    ;
    RnonCO2[    317    ]=    -0.3174935    ;
    RnonCO2[    318    ]=    -0.3114646    ;
    RnonCO2[    319    ]=    -0.3053983    ;
    RnonCO2[    320    ]=    -0.2992967    ;
    RnonCO2[    321    ]=    -0.2931615    ;
    RnonCO2[    322    ]=    -0.2870076    ;
    RnonCO2[    323    ]=    -0.2808448    ;
    RnonCO2[    324    ]=    -0.2746629    ;
    RnonCO2[    325    ]=    -0.2693143    ;
    RnonCO2[    326    ]=    -0.2656573    ;
    RnonCO2[    327    ]=    -0.2626709    ;
    RnonCO2[    328    ]=    -0.2595414    ;
    RnonCO2[    329    ]=    -0.2564733    ;
    RnonCO2[    330    ]=    -0.2534625    ;
    RnonCO2[    331    ]=    -0.2505038    ;
    RnonCO2[    332    ]=    -0.2475918    ;
    RnonCO2[    333    ]=    -0.2447355    ;
    RnonCO2[    334    ]=    -0.2419394    ;
    RnonCO2[    335    ]=    -0.2398735    ;
    RnonCO2[    336    ]=    -0.2392144    ;
    RnonCO2[    337    ]=    -0.2390216    ;
    RnonCO2[    338    ]=    -0.2386237    ;
    RnonCO2[    339    ]=    -0.2382894    ;
    RnonCO2[    340    ]=    -0.2380172    ;
    RnonCO2[    341    ]=    -0.2378049    ;
    RnonCO2[    342    ]=    -0.2376482    ;
    RnonCO2[    343    ]=    -0.2375433    ;
    RnonCO2[    344    ]=    -0.2375006    ;
    RnonCO2[    345    ]=    -0.2375257    ;
    RnonCO2[    346    ]=    -0.2376038    ;
    RnonCO2[    347    ]=    -0.2377213    ;
    RnonCO2[    348    ]=    -0.2378659    ;
    RnonCO2[    349    ]=    -0.2380261    ;
    RnonCO2[    350    ]=    -0.2381976    ;
    RnonCO2[    351    ]=    -0.2383834    ;
    RnonCO2[    352    ]=    -0.2385853    ;
    RnonCO2[    353    ]=    -0.2388029    ;
    RnonCO2[    354    ]=    -0.2390356    ;
    RnonCO2[    355    ]=    -0.2392969    ;
    RnonCO2[    356    ]=    -0.2395954    ;
    RnonCO2[    357    ]=    -0.239919    ;
    RnonCO2[    358    ]=    -0.2402557    ;
    RnonCO2[    359    ]=    -0.2405956    ;
    RnonCO2[    360    ]=    -0.2409292    ;
    RnonCO2[    361    ]=    -0.2412561    ;
    RnonCO2[    362    ]=    -0.2415816    ;
    RnonCO2[    363    ]=    -0.2419093    ;
    RnonCO2[    364    ]=    -0.2422403    ;
    RnonCO2[    365    ]=    -0.2425753    ;
    RnonCO2[    366    ]=    -0.2429291    ;
    RnonCO2[    367    ]=    -0.2433117    ;
    RnonCO2[    368    ]=    -0.243712    ;
    RnonCO2[    369    ]=    -0.244119    ;
    RnonCO2[    370    ]=    -0.2445227    ;
    RnonCO2[    371    ]=    -0.244913    ;
    RnonCO2[    372    ]=    -0.2452901    ;
    RnonCO2[    373    ]=    -0.245658    ;
    RnonCO2[    374    ]=    -0.246021    ;
    RnonCO2[    375    ]=    -0.246382    ;
    RnonCO2[    376    ]=    -0.246742    ;
    RnonCO2[    377    ]=    -0.247115    ;
    RnonCO2[    378    ]=    -0.247514    ;
    RnonCO2[    379    ]=    -0.247925    ;
    RnonCO2[    380    ]=    -0.24834    ;
    RnonCO2[    381    ]=    -0.248748    ;
    RnonCO2[    382    ]=    -0.249141    ;
    RnonCO2[    383    ]=    -0.249516    ;
    RnonCO2[    384    ]=    -0.249878    ;
    RnonCO2[    385    ]=    -0.250234    ;
    RnonCO2[    386    ]=    -0.250585    ;
    RnonCO2[    387    ]=    -0.250932    ;
    RnonCO2[    388    ]=    -0.251284    ;
    RnonCO2[    389    ]=    -0.251656    ;
    RnonCO2[    390    ]=    -0.252035    ;
    RnonCO2[    391    ]=    -0.252412    ;
    RnonCO2[    392    ]=    -0.252776    ;
    RnonCO2[    393    ]=    -0.25312    ;
    RnonCO2[    394    ]=    -0.253441    ;
    RnonCO2[    395    ]=    -0.253744    ;
    RnonCO2[    396    ]=    -0.254037    ;
    RnonCO2[    397    ]=    -0.25432    ;
    RnonCO2[    398    ]=    -0.254597    ;
    RnonCO2[    399    ]=    -0.254882    ;
    RnonCO2[    400    ]=    -0.255188    ;
    RnonCO2[    401    ]=    -0.255503    ;
    RnonCO2[    402    ]=    -0.255816    ;
    RnonCO2[    403    ]=    -0.256117    ;
    RnonCO2[    404    ]=    -0.256399    ;
    RnonCO2[    405    ]=    -0.256657    ;
    RnonCO2[    406    ]=    -0.256901    ;
    RnonCO2[    407    ]=    -0.257133    ;
    RnonCO2[    408    ]=    -0.257358    ;
    RnonCO2[    409    ]=    -0.257576    ;
    RnonCO2[    410    ]=    -0.257804    ;
    RnonCO2[    411    ]=    -0.258053    ;
    RnonCO2[    412    ]=    -0.258312    ;
    RnonCO2[    413    ]=    -0.258571    ;
    RnonCO2[    414    ]=    -0.25882    ;
    RnonCO2[    415    ]=    -0.259047    ;
    RnonCO2[    416    ]=    -0.259253    ;
    RnonCO2[    417    ]=    -0.259444    ;
    RnonCO2[    418    ]=    -0.259625    ;
    RnonCO2[    419    ]=    -0.259799    ;
    RnonCO2[    420    ]=    -0.259966    ;
    RnonCO2[    421    ]=    -0.260145    ;
    RnonCO2[    422    ]=    -0.260345    ;
    RnonCO2[    423    ]=    -0.260556    ;
    RnonCO2[    424    ]=    -0.260768    ;
    RnonCO2[    425    ]=    -0.260968    ;
    RnonCO2[    426    ]=    -0.26115    ;
    RnonCO2[    427    ]=    -0.26131    ;
    RnonCO2[    428    ]=    -0.261455    ;
    RnonCO2[    429    ]=    -0.261591    ;
    RnonCO2[    430    ]=    -0.26172    ;
    RnonCO2[    431    ]=    -0.261844    ;
    RnonCO2[    432    ]=    -0.261979    ;
    RnonCO2[    433    ]=    -0.262137    ;
    RnonCO2[    434    ]=    -0.262306    ;
    RnonCO2[    435    ]=    -0.262476    ;
    RnonCO2[    436    ]=    -0.262635    ;
    RnonCO2[    437    ]=    -0.262775    ;
    RnonCO2[    438    ]=    -0.262896    ;
    RnonCO2[    439    ]=    -0.263002    ;
    RnonCO2[    440    ]=    -0.263099    ;
    RnonCO2[    441    ]=    -0.263189    ;
    RnonCO2[    442    ]=    -0.263277    ;
    RnonCO2[    443    ]=    -0.263375    ;
    RnonCO2[    444    ]=    -0.263496    ;
    RnonCO2[    445    ]=    -0.263629    ;
    RnonCO2[    446    ]=    -0.263764    ;
    RnonCO2[    447    ]=    -0.263889    ;
    RnonCO2[    448    ]=    -0.263995    ;
    RnonCO2[    449    ]=    -0.264081    ;
    RnonCO2[    450    ]=    -0.264153    ;
    RnonCO2[    451    ]=    -0.264216    ;
    RnonCO2[    452    ]=    -0.264274    ;
    RnonCO2[    453    ]=    -0.264329    ;
    RnonCO2[    454    ]=    -0.264394    ;
    RnonCO2[    455    ]=    -0.264484    ;
    RnonCO2[    456    ]=    -0.264586    ;
    RnonCO2[    457    ]=    -0.264688    ;
    RnonCO2[    458    ]=    -0.264782    ;
    RnonCO2[    459    ]=    -0.264856    ;
    RnonCO2[    460    ]=    -0.264912    ;
    RnonCO2[    461    ]=    -0.264954    ;
    RnonCO2[    462    ]=    -0.264987    ;
    RnonCO2[    463    ]=    -0.265015    ;
    RnonCO2[    464    ]=    -0.265039    ;
    RnonCO2[    465    ]=    -0.265076    ;
    RnonCO2[    466    ]=    -0.265136    ;
    RnonCO2[    467    ]=    -0.265209    ;
    RnonCO2[    468    ]=    -0.265283    ;
    RnonCO2[    469    ]=    -0.265348    ;
    RnonCO2[    470    ]=    -0.265395    ;
    RnonCO2[    471    ]=    -0.265422    ;
    RnonCO2[    472    ]=    -0.265435    ;
    RnonCO2[    473    ]=    -0.265442    ;
    RnonCO2[    474    ]=    -0.265442    ;
    RnonCO2[    475    ]=    -0.26544    ;
    RnonCO2[    476    ]=    -0.26545    ;
    RnonCO2[    477    ]=    -0.265484    ;
    RnonCO2[    478    ]=    -0.265531    ;
    RnonCO2[    479    ]=    -0.265578    ;
    RnonCO2[    480    ]=    -0.265617    ;
    RnonCO2[    481    ]=    -0.265638    ;
    RnonCO2[    482    ]=    -0.265638    ;
    RnonCO2[    483    ]=    -0.265627    ;
    RnonCO2[    484    ]=    -0.265607    ;
    RnonCO2[    485    ]=    -0.265596    ;
    RnonCO2[    486    ]=    -0.265596    ;
    RnonCO2[    487    ]=    -0.265596    ;
    RnonCO2[    488    ]=    -0.265596    ;
    RnonCO2[    489    ]=    -0.265596    ;
    RnonCO2[    490    ]=    -0.265596    ;
    RnonCO2[    491    ]=    -0.265596    ;
    RnonCO2[    492    ]=    -0.265596    ;
    RnonCO2[    493    ]=    -0.265596    ;
    RnonCO2[    494    ]=    -0.265596    ;
    RnonCO2[    495    ]=    -0.265596    ;
    RnonCO2[    496    ]=    -0.265596    ;
    RnonCO2[    497    ]=    -0.265596    ;
    RnonCO2[    498    ]=    -0.265596    ;
    RnonCO2[    499    ]=    -0.265596    ;
    RnonCO2[    500    ]=    -0.265596    ;
    RnonCO2[    501    ]=    -0.265596    ;
    RnonCO2[    502    ]=    -0.265596    ;
    RnonCO2[    503    ]=    -0.265596    ;
    RnonCO2[    504    ]=    -0.265596    ;
    RnonCO2[    505    ]=    -0.265596    ;
    RnonCO2[    506    ]=    -0.265596    ;
    RnonCO2[    507    ]=    -0.265596    ;
    RnonCO2[    508    ]=    -0.265596    ;
    RnonCO2[    509    ]=    -0.265596    ;
    RnonCO2[    510    ]=    -0.265596    ;
    RnonCO2[    511    ]=    -0.265596    ;
    RnonCO2[    512    ]=    -0.265596    ;
    RnonCO2[    513    ]=    -0.265596    ;
    RnonCO2[    514    ]=    -0.265596    ;
    RnonCO2[    515    ]=    -0.265596    ;
    RnonCO2[    516    ]=    -0.265596    ;
    RnonCO2[    517    ]=    -0.265596    ;
    RnonCO2[    518    ]=    -0.265596    ;
    RnonCO2[    519    ]=    -0.265596    ;
    RnonCO2[    520    ]=    -0.265596    ;
    RnonCO2[    521    ]=    -0.265596    ;
    RnonCO2[    522    ]=    -0.265596    ;
    RnonCO2[    523    ]=    -0.265596    ;
    RnonCO2[    524    ]=    -0.265596    ;
    RnonCO2[    525    ]=    -0.265596    ;
    RnonCO2[    526    ]=    -0.265596    ;
    RnonCO2[    527    ]=    -0.265596    ;
    RnonCO2[    528    ]=    -0.265596    ;
    RnonCO2[    529    ]=    -0.265596    ;
    RnonCO2[    530    ]=    -0.265596    ;
    RnonCO2[    531    ]=    -0.265596    ;
    RnonCO2[    532    ]=    -0.265596    ;
    RnonCO2[    533    ]=    -0.265596    ;
    RnonCO2[    534    ]=    -0.265596    ;
    RnonCO2[    535    ]=    -0.265596    ;
    RnonCO2[    536    ]=    -0.265596    ;
    RnonCO2[    537    ]=    -0.265596    ;
    RnonCO2[    538    ]=    -0.265596    ;
    RnonCO2[    539    ]=    -0.265596    ;
    RnonCO2[    540    ]=    -0.265596    ;
    RnonCO2[    541    ]=    -0.265596    ;
    RnonCO2[    542    ]=    -0.265596    ;
    RnonCO2[    543    ]=    -0.265596    ;
    RnonCO2[    544    ]=    -0.265596    ;
    RnonCO2[    545    ]=    -0.265596    ;
    RnonCO2[    546    ]=    -0.265596    ;
    RnonCO2[    547    ]=    -0.265596    ;
    RnonCO2[    548    ]=    -0.265596    ;
    RnonCO2[    549    ]=    -0.265596    ;
    RnonCO2[    550    ]=    -0.265596    ;
    RnonCO2[    551    ]=    -0.265596    ;
    RnonCO2[    552    ]=    -0.265596    ;
    RnonCO2[    553    ]=    -0.265596    ;
    RnonCO2[    554    ]=    -0.265596    ;
    RnonCO2[    555    ]=    -0.265596    ;
    RnonCO2[    556    ]=    -0.265596    ;
    RnonCO2[    557    ]=    -0.265596    ;
    RnonCO2[    558    ]=    -0.265596    ;
    RnonCO2[    559    ]=    -0.265596    ;
    RnonCO2[    560    ]=    -0.265596    ;
    RnonCO2[    561    ]=    -0.265596    ;
    RnonCO2[    562    ]=    -0.265596    ;
    RnonCO2[    563    ]=    -0.265596    ;
    RnonCO2[    564    ]=    -0.265596    ;
    RnonCO2[    565    ]=    -0.265596    ;
    RnonCO2[    566    ]=    -0.265596    ;
    RnonCO2[    567    ]=    -0.265596    ;
    RnonCO2[    568    ]=    -0.265596    ;
    RnonCO2[    569    ]=    -0.265596    ;
    RnonCO2[    570    ]=    -0.265596    ;
    RnonCO2[    571    ]=    -0.265596    ;
    RnonCO2[    572    ]=    -0.265596    ;
    RnonCO2[    573    ]=    -0.265596    ;
    RnonCO2[    574    ]=    -0.265596    ;
    RnonCO2[    575    ]=    -0.265596    ;
    RnonCO2[    576    ]=    -0.265596    ;
    RnonCO2[    577    ]=    -0.265596    ;
    RnonCO2[    578    ]=    -0.265596    ;
    RnonCO2[    579    ]=    -0.265596    ;
    RnonCO2[    580    ]=    -0.265596    ;
    RnonCO2[    581    ]=    -0.265596    ;
    RnonCO2[    582    ]=    -0.265596    ;
    RnonCO2[    583    ]=    -0.265596    ;
    RnonCO2[    584    ]=    -0.265596    ;
    RnonCO2[    585    ]=    -0.265596    ;
    RnonCO2[    586    ]=    -0.265596    ;
    RnonCO2[    587    ]=    -0.265596    ;
    RnonCO2[    588    ]=    -0.265596    ;
    RnonCO2[    589    ]=    -0.265596    ;
    RnonCO2[    590    ]=    -0.265596    ;
    RnonCO2[    591    ]=    -0.265596    ;
    RnonCO2[    592    ]=    -0.265596    ;
    RnonCO2[    593    ]=    -0.265596    ;
    RnonCO2[    594    ]=    -0.265596    ;
    RnonCO2[    595    ]=    -0.265596    ;
    RnonCO2[    596    ]=    -0.265596    ;
    RnonCO2[    597    ]=    -0.265596    ;
    RnonCO2[    598    ]=    -0.265596    ;
    RnonCO2[    599    ]=    -0.265596    ;
    RnonCO2[    600    ]=    -0.265596    ;
    RnonCO2[    601    ]=    -0.265596    ;
    RnonCO2[    602    ]=    -0.265596    ;
    RnonCO2[    603    ]=    -0.265596    ;
    RnonCO2[    604    ]=    -0.265596    ;
    RnonCO2[    605    ]=    -0.265596    ;
    RnonCO2[    606    ]=    -0.265596    ;
    RnonCO2[    607    ]=    -0.265596    ;
    RnonCO2[    608    ]=    -0.265596    ;
    RnonCO2[    609    ]=    -0.265596    ;
    RnonCO2[    610    ]=    -0.265596    ;
    RnonCO2[    611    ]=    -0.265596    ;
    RnonCO2[    612    ]=    -0.265596    ;
    RnonCO2[    613    ]=    -0.265596    ;
    RnonCO2[    614    ]=    -0.265596    ;
    RnonCO2[    615    ]=    -0.265596    ;
    RnonCO2[    616    ]=    -0.265596    ;
    RnonCO2[    617    ]=    -0.265596    ;
    RnonCO2[    618    ]=    -0.265596    ;
    RnonCO2[    619    ]=    -0.265596    ;
    RnonCO2[    620    ]=    -0.265596    ;
    RnonCO2[    621    ]=    -0.265596    ;
    RnonCO2[    622    ]=    -0.265596    ;
    RnonCO2[    623    ]=    -0.265596    ;
    RnonCO2[    624    ]=    -0.265596    ;
    RnonCO2[    625    ]=    -0.265596    ;
    RnonCO2[    626    ]=    -0.265596    ;
    RnonCO2[    627    ]=    -0.265596    ;
    RnonCO2[    628    ]=    -0.265596    ;
    RnonCO2[    629    ]=    -0.265596    ;
    RnonCO2[    630    ]=    -0.265596    ;
    RnonCO2[    631    ]=    -0.265596    ;
    RnonCO2[    632    ]=    -0.265596    ;
    RnonCO2[    633    ]=    -0.265596    ;
    RnonCO2[    634    ]=    -0.265596    ;
    RnonCO2[    635    ]=    -0.265596    ;
    RnonCO2[    636    ]=    -0.265596    ;
    RnonCO2[    637    ]=    -0.265596    ;
    RnonCO2[    638    ]=    -0.265596    ;
    RnonCO2[    639    ]=    -0.265596    ;
    RnonCO2[    640    ]=    -0.265596    ;
    RnonCO2[    641    ]=    -0.265596    ;
    RnonCO2[    642    ]=    -0.265596    ;
    RnonCO2[    643    ]=    -0.265596    ;
    RnonCO2[    644    ]=    -0.265596    ;
    RnonCO2[    645    ]=    -0.265596    ;
    RnonCO2[    646    ]=    -0.265596    ;
    RnonCO2[    647    ]=    -0.265596    ;
    RnonCO2[    648    ]=    -0.265596    ;
    RnonCO2[    649    ]=    -0.265596    ;
    RnonCO2[    650    ]=    -0.265596    ;
    RnonCO2[    651    ]=    -0.265596    ;
    RnonCO2[    652    ]=    -0.265596    ;
    RnonCO2[    653    ]=    -0.265596    ;
    RnonCO2[    654    ]=    -0.265596    ;
    RnonCO2[    655    ]=    -0.265596    ;
    RnonCO2[    656    ]=    -0.265596    ;
    RnonCO2[    657    ]=    -0.265596    ;
    RnonCO2[    658    ]=    -0.265596    ;
    RnonCO2[    659    ]=    -0.265596    ;
    RnonCO2[    660    ]=    -0.265596    ;
    RnonCO2[    661    ]=    -0.265596    ;
    RnonCO2[    662    ]=    -0.265596    ;
    RnonCO2[    663    ]=    -0.265596    ;
    RnonCO2[    664    ]=    -0.265596    ;
    RnonCO2[    665    ]=    -0.265596    ;
    RnonCO2[    666    ]=    -0.265596    ;
    RnonCO2[    667    ]=    -0.265596    ;
    RnonCO2[    668    ]=    -0.265596    ;
    RnonCO2[    669    ]=    -0.265596    ;
    RnonCO2[    670    ]=    -0.265596    ;
    RnonCO2[    671    ]=    -0.265596    ;
    RnonCO2[    672    ]=    -0.265596    ;
    RnonCO2[    673    ]=    -0.265596    ;
    RnonCO2[    674    ]=    -0.265596    ;
    RnonCO2[    675    ]=    -0.265596    ;
    RnonCO2[    676    ]=    -0.265596    ;
    RnonCO2[    677    ]=    -0.265596    ;
    RnonCO2[    678    ]=    -0.265596    ;
    RnonCO2[    679    ]=    -0.265596    ;
    RnonCO2[    680    ]=    -0.265596    ;
    RnonCO2[    681    ]=    -0.265596    ;
    RnonCO2[    682    ]=    -0.265596    ;
    RnonCO2[    683    ]=    -0.265596    ;
    RnonCO2[    684    ]=    -0.265596    ;
    RnonCO2[    685    ]=    -0.265596    ;
    RnonCO2[    686    ]=    -0.265596    ;
    RnonCO2[    687    ]=    -0.265596    ;
    RnonCO2[    688    ]=    -0.265596    ;
    RnonCO2[    689    ]=    -0.265596    ;
    RnonCO2[    690    ]=    -0.265596    ;
    RnonCO2[    691    ]=    -0.265596    ;
    RnonCO2[    692    ]=    -0.265596    ;
    RnonCO2[    693    ]=    -0.265596    ;
    RnonCO2[    694    ]=    -0.265596    ;
    RnonCO2[    695    ]=    -0.265596    ;
    RnonCO2[    696    ]=    -0.265596    ;
    RnonCO2[    697    ]=    -0.265596    ;
    RnonCO2[    698    ]=    -0.265596    ;
    RnonCO2[    699    ]=    -0.265596    ;
    RnonCO2[    700    ]=    -0.265596    ;
    RnonCO2[    701    ]=    -0.265596    ;
    RnonCO2[    702    ]=    -0.265596    ;
    RnonCO2[    703    ]=    -0.265596    ;
    RnonCO2[    704    ]=    -0.265596    ;
    RnonCO2[    705    ]=    -0.265596    ;
    RnonCO2[    706    ]=    -0.265596    ;
    RnonCO2[    707    ]=    -0.265596    ;
    RnonCO2[    708    ]=    -0.265596    ;
    RnonCO2[    709    ]=    -0.265596    ;
    RnonCO2[    710    ]=    -0.265596    ;
    RnonCO2[    711    ]=    -0.265596    ;
    RnonCO2[    712    ]=    -0.265596    ;
    RnonCO2[    713    ]=    -0.265596    ;
    RnonCO2[    714    ]=    -0.265596    ;
    RnonCO2[    715    ]=    -0.265596    ;
    RnonCO2[    716    ]=    -0.265596    ;
    RnonCO2[    717    ]=    -0.265596    ;
    RnonCO2[    718    ]=    -0.265596    ;
    RnonCO2[    719    ]=    -0.265596    ;
    RnonCO2[    720    ]=    -0.265596    ;
    RnonCO2[    721    ]=    -0.265596    ;
    RnonCO2[    722    ]=    -0.265596    ;
    RnonCO2[    723    ]=    -0.265596    ;
    RnonCO2[    724    ]=    -0.265596    ;
    RnonCO2[    725    ]=    -0.265596    ;
    RnonCO2[    726    ]=    -0.265596    ;
    RnonCO2[    727    ]=    -0.265596    ;
    RnonCO2[    728    ]=    -0.265596    ;
    RnonCO2[    729    ]=    -0.265596    ;
    RnonCO2[    730    ]=    -0.265596    ;
    RnonCO2[    731    ]=    -0.265596    ;
    RnonCO2[    732    ]=    -0.265596    ;
    RnonCO2[    733    ]=    -0.265596    ;
    RnonCO2[    734    ]=    -0.265596    ;
    RnonCO2[    735    ]=    -0.265596    ;
    for(int i = 736; i<tmax; i++)
    {
        RnonCO2[i] = RnonCO2[i-1];
    }
}

void setR_volcanic(void)
{
    RnonCO2[    0    ]=    0    ;
    RnonCO2[    1    ]=    0.11622211    ;
    RnonCO2[    2    ]=    0.23244422    ;
    RnonCO2[    3    ]=    0.19744422    ;
    RnonCO2[    4    ]=    0.13181922    ;
    RnonCO2[    5    ]=    0.12744422    ;
    RnonCO2[    6    ]=    0.17994422    ;
    RnonCO2[    7    ]=    0.21494422    ;
    RnonCO2[    8    ]=    0.22806922    ;
    RnonCO2[    9    ]=    0.23244422    ;
    RnonCO2[    10    ]=    0.23244422    ;
    RnonCO2[    11    ]=    0.23244422    ;
    RnonCO2[    12    ]=    0.23244422    ;
    RnonCO2[    13    ]=    0.23244422    ;
    RnonCO2[    14    ]=    0.23244422    ;
    RnonCO2[    15    ]=    0.23244422    ;
    RnonCO2[    16    ]=    0.23244422    ;
    RnonCO2[    17    ]=    0.21056922    ;
    RnonCO2[    18    ]=    0.11869422    ;
    RnonCO2[    19    ]=    0.044319219    ;
    RnonCO2[    20    ]=    0.10119422    ;
    RnonCO2[    21    ]=    0.18869422    ;
    RnonCO2[    22    ]=    0.22369422    ;
    RnonCO2[    23    ]=    0.22806922    ;
    RnonCO2[    24    ]=    0.12306922    ;
    RnonCO2[    25    ]=    -0.025680781    ;
    RnonCO2[    26    ]=    0.009319219    ;
    RnonCO2[    27    ]=    0.14056922    ;
    RnonCO2[    28    ]=    0.20619422    ;
    RnonCO2[    29    ]=    0.21494422    ;
    RnonCO2[    30    ]=    0.19306922    ;
    RnonCO2[    31    ]=    0.18869422    ;
    RnonCO2[    32    ]=    0.21056922    ;
    RnonCO2[    33    ]=    0.22806922    ;
    RnonCO2[    34    ]=    0.23244422    ;
    RnonCO2[    35    ]=    0.23244422    ;
    RnonCO2[    36    ]=    0.23244422    ;
    RnonCO2[    37    ]=    0.23244422    ;
    RnonCO2[    38    ]=    0.23244422    ;
    RnonCO2[    39    ]=    0.23244422    ;
    RnonCO2[    40    ]=    0.23244422    ;
    RnonCO2[    41    ]=    0.23244422    ;
    RnonCO2[    42    ]=    0.23244422    ;
    RnonCO2[    43    ]=    -0.25755578    ;
    RnonCO2[    44    ]=    -1.3994308    ;
    RnonCO2[    45    ]=    -1.8019308    ;
    RnonCO2[    46    ]=    -1.0100558    ;
    RnonCO2[    47    ]=    -0.34068078    ;
    RnonCO2[    48    ]=    -0.14818078    ;
    RnonCO2[    49    ]=    -0.67755578    ;
    RnonCO2[    50    ]=    -2.1519308    ;
    RnonCO2[    51    ]=    -2.6638058    ;
    RnonCO2[    52    ]=    -1.4956808    ;
    RnonCO2[    53    ]=    -0.36255578    ;
    RnonCO2[    54    ]=    0.092444219    ;
    RnonCO2[    55    ]=    0.22369422    ;
    RnonCO2[    56    ]=    0.23244422    ;
    RnonCO2[    57    ]=    0.23244422    ;
    RnonCO2[    58    ]=    0.23244422    ;
    RnonCO2[    59    ]=    0.23244422    ;
    RnonCO2[    60    ]=    0.23244422    ;
    RnonCO2[    61    ]=    0.23244422    ;
    RnonCO2[    62    ]=    0.23244422    ;
    RnonCO2[    63    ]=    0.22806922    ;
    RnonCO2[    64    ]=    0.19306922    ;
    RnonCO2[    65    ]=    -0.021305781    ;
    RnonCO2[    66    ]=    -0.39755578    ;
    RnonCO2[    67    ]=    -0.48505578    ;
    RnonCO2[    68    ]=    -0.18318078    ;
    RnonCO2[    69    ]=    -0.34505578    ;
    RnonCO2[    70    ]=    -0.98818078    ;
    RnonCO2[    71    ]=    -0.94880578    ;
    RnonCO2[    72    ]=    -0.34943078    ;
    RnonCO2[    73    ]=    0.044319219    ;
    RnonCO2[    74    ]=    0.17994422    ;
    RnonCO2[    75    ]=    0.17994422    ;
    RnonCO2[    76    ]=    0.17556922    ;
    RnonCO2[    77    ]=    0.18869422    ;
    RnonCO2[    78    ]=    0.16681922    ;
    RnonCO2[    79    ]=    0.15369422    ;
    RnonCO2[    80    ]=    0.18431922    ;
    RnonCO2[    81    ]=    0.21494422    ;
    RnonCO2[    82    ]=    0.22806922    ;
    RnonCO2[    83    ]=    0.23244422    ;
    RnonCO2[    84    ]=    0.21599422    ;
    RnonCO2[    85    ]=    0.19200464    ;
    RnonCO2[    86    ]=    0.19303276    ;
    RnonCO2[    87    ]=    0.20879734    ;
    RnonCO2[    88    ]=    0.2235338    ;
    RnonCO2[    89    ]=    0.2317588    ;
    RnonCO2[    90    ]=    0.10118693    ;
    RnonCO2[    91    ]=    -0.35667141    ;
    RnonCO2[    92    ]=    -0.61027557    ;
    RnonCO2[    93    ]=    -0.29772557    ;
    RnonCO2[    94    ]=    0.028190052    ;
    RnonCO2[    95    ]=    0.15053693    ;
    RnonCO2[    96    ]=    0.13957026    ;
    RnonCO2[    97    ]=    0.05526401    ;
    RnonCO2[    98    ]=    0.06760151    ;
    RnonCO2[    99    ]=    0.15087964    ;
    RnonCO2[    100    ]=    0.19851609    ;
    RnonCO2[    101    ]=    0.21907859    ;
    RnonCO2[    102    ]=    0.22901714    ;
    RnonCO2[    103    ]=    0.23107339    ;
    RnonCO2[    104    ]=    0.22284839    ;
    RnonCO2[    105    ]=    0.21873589    ;
    RnonCO2[    106    ]=    0.2194213    ;
    RnonCO2[    107    ]=    0.2070838    ;
    RnonCO2[    108    ]=    0.19337547    ;
    RnonCO2[    109    ]=    0.20125776    ;
    RnonCO2[    110    ]=    0.19406089    ;
    RnonCO2[    111    ]=    0.15293589    ;
    RnonCO2[    112    ]=    0.14848068    ;
    RnonCO2[    113    ]=    0.18035255    ;
    RnonCO2[    114    ]=    0.20194318    ;
    RnonCO2[    115    ]=    0.21565151    ;
    RnonCO2[    116    ]=    0.22284839    ;
    RnonCO2[    117    ]=    0.11112547    ;
    RnonCO2[    118    ]=    -0.80253495    ;
    RnonCO2[    119    ]=    -1.5373016    ;
    RnonCO2[    120    ]=    -0.94407349    ;
    RnonCO2[    121    ]=    -0.44851724    ;
    RnonCO2[    122    ]=    -0.32137245    ;
    RnonCO2[    123    ]=    -0.21102036    ;
    RnonCO2[    124    ]=    -0.27750578    ;
    RnonCO2[    125    ]=    -0.3316537    ;
    RnonCO2[    126    ]=    -0.2617412    ;
    RnonCO2[    127    ]=    -0.087645365    ;
    RnonCO2[    128    ]=    0.060061927    ;
    RnonCO2[    129    ]=    0.16938589    ;
    RnonCO2[    130    ]=    0.1618463    ;
    RnonCO2[    131    ]=    0.016537969    ;
    RnonCO2[    132    ]=    -0.064683906    ;
    RnonCO2[    133    ]=    0.037785885    ;
    RnonCO2[    134    ]=    0.15053693    ;
    RnonCO2[    135    ]=    0.20194318    ;
    RnonCO2[    136    ]=    0.19166193    ;
    RnonCO2[    137    ]=    -0.25694328    ;
    RnonCO2[    138    ]=    -0.65105786    ;
    RnonCO2[    139    ]=    -0.3563287    ;
    RnonCO2[    140    ]=    0.002144219    ;
    RnonCO2[    141    ]=    0.1001588    ;
    RnonCO2[    142    ]=    0.076169219    ;
    RnonCO2[    143    ]=    0.087821302    ;
    RnonCO2[    144    ]=    0.15396401    ;
    RnonCO2[    145    ]=    0.18892026    ;
    RnonCO2[    146    ]=    0.19508901    ;
    RnonCO2[    147    ]=    -0.048576615    ;
    RnonCO2[    148    ]=    -0.15652974    ;
    RnonCO2[    149    ]=    0.055606719    ;
    RnonCO2[    150    ]=    0.15944734    ;
    RnonCO2[    151    ]=    0.18754943    ;
    RnonCO2[    152    ]=    0.19611714    ;
    RnonCO2[    153    ]=    0.20194318    ;
    RnonCO2[    154    ]=    0.1947463    ;
    RnonCO2[    155    ]=    0.11078276    ;
    RnonCO2[    156    ]=    0.095703594    ;
    RnonCO2[    157    ]=    0.18103797    ;
    RnonCO2[    158    ]=    0.20228589    ;
    RnonCO2[    159    ]=    0.18926297    ;
    RnonCO2[    160    ]=    0.18926297    ;
    RnonCO2[    161    ]=    0.19714526    ;
    RnonCO2[    162    ]=    0.19508901    ;
    RnonCO2[    163    ]=    0.14573901    ;
    RnonCO2[    164    ]=    0.096046302    ;
    RnonCO2[    165    ]=    0.12174943    ;
    RnonCO2[    166    ]=    0.14642443    ;
    RnonCO2[    167    ]=    0.11489526    ;
    RnonCO2[    168    ]=    0.12380568    ;
    RnonCO2[    169    ]=    0.16218901    ;
    RnonCO2[    170    ]=    0.16938589    ;
    RnonCO2[    171    ]=    0.18069526    ;
    RnonCO2[    172    ]=    0.17932443    ;
    RnonCO2[    173    ]=    0.16321714    ;
    RnonCO2[    174    ]=    0.16390255    ;
    RnonCO2[    175    ]=    0.18377964    ;
    RnonCO2[    176    ]=    0.19303276    ;
    RnonCO2[    177    ]=    0.17144214    ;
    RnonCO2[    178    ]=    0.16630151    ;
    RnonCO2[    179    ]=    0.18926297    ;
    RnonCO2[    180    ]=    0.19988693    ;
    RnonCO2[    181    ]=    0.2070838    ;
    RnonCO2[    182    ]=    0.1988588    ;
    RnonCO2[    183    ]=    0.19680255    ;
    RnonCO2[    184    ]=    0.18926297    ;
    RnonCO2[    185    ]=    0.18686401    ;
    RnonCO2[    186    ]=    0.1906338    ;
    RnonCO2[    187    ]=    0.18446505    ;
    RnonCO2[    188    ]=    0.17555464    ;
    RnonCO2[    189    ]=    0.18069526    ;
    RnonCO2[    190    ]=    0.2029713    ;
    RnonCO2[    191    ]=    0.2153088    ;
    RnonCO2[    192    ]=    0.22559005    ;
    RnonCO2[    193    ]=    0.23073068    ;
    RnonCO2[    194    ]=    0.22696089    ;
    RnonCO2[    195    ]=    0.15190776    ;
    RnonCO2[    196    ]=    0.060747344    ;
    RnonCO2[    197    ]=    -0.013963073    ;
    RnonCO2[    198    ]=    -0.49546828    ;
    RnonCO2[    199    ]=    -0.85325578    ;
    RnonCO2[    200    ]=    -0.53248078    ;
    RnonCO2[    201    ]=    -0.15001828    ;
    RnonCO2[    202    ]=    -0.021502656    ;
    RnonCO2[    203    ]=    -0.22027349    ;
    RnonCO2[    204    ]=    -0.29635474    ;
    RnonCO2[    205    ]=    -0.055430781    ;
    RnonCO2[    206    ]=    0.11420984    ;
    RnonCO2[    207    ]=    0.15327859    ;
    RnonCO2[    208    ]=    0.10632755    ;
    RnonCO2[    209    ]=    -0.040694323    ;
    RnonCO2[    210    ]=    -0.14110786    ;
    RnonCO2[    211    ]=    -0.017390156    ;
    RnonCO2[    212    ]=    0.13408693    ;
    RnonCO2[    213    ]=    0.10906922    ;
    RnonCO2[    214    ]=    0.093990052    ;
    RnonCO2[    215    ]=    0.14813797    ;
    RnonCO2[    216    ]=    0.079596302    ;
    RnonCO2[    217    ]=    -0.59142661    ;
    RnonCO2[    218    ]=    -0.87621724    ;
    RnonCO2[    219    ]=    -0.32000161    ;
    RnonCO2[    220    ]=    -0.023216198    ;
    RnonCO2[    221    ]=    0.035044219    ;
    RnonCO2[    222    ]=    0.065887969    ;
    RnonCO2[    223    ]=    0.10598484    ;
    RnonCO2[    224    ]=    0.12928901    ;
    RnonCO2[    225    ]=    0.12654734    ;
    RnonCO2[    226    ]=    -0.81487245    ;
    RnonCO2[    227    ]=    -1.4197527    ;
    RnonCO2[    228    ]=    -0.68190161    ;
    RnonCO2[    229    ]=    -0.13151203    ;
    RnonCO2[    230    ]=    0.058691094    ;
    RnonCO2[    231    ]=    0.12380568    ;
    RnonCO2[    232    ]=    0.15327859    ;
    RnonCO2[    233    ]=    0.18754943    ;
    RnonCO2[    234    ]=    0.21633693    ;
    RnonCO2[    235    ]=    0.22867443    ;
    RnonCO2[    236    ]=    0.23244422    ;
    RnonCO2[    237    ]=    0.23244422    ;
    RnonCO2[    238    ]=    0.23244422    ;
    RnonCO2[    239    ]=    0.23244422    ;
    RnonCO2[    240    ]=    0.18401834    ;
    RnonCO2[    241    ]=    0.06779623    ;
    RnonCO2[    242    ]=    0    ;
    RnonCO2[    243    ]=    0    ;
    RnonCO2[    244    ]=    0    ;
    RnonCO2[    245    ]=    0    ;
    RnonCO2[    246    ]=    0    ;
    RnonCO2[    247    ]=    0    ;
    RnonCO2[    248    ]=    0    ;
    RnonCO2[    249    ]=    0    ;
    RnonCO2[    250    ]=    0    ;
    RnonCO2[    251    ]=    0    ;
    RnonCO2[    252    ]=    0    ;
    RnonCO2[    253    ]=    0    ;
    RnonCO2[    254    ]=    0    ;
    RnonCO2[    255    ]=    0    ;
    RnonCO2[    256    ]=    0    ;
    RnonCO2[    257    ]=    0    ;
    RnonCO2[    258    ]=    0    ;
    RnonCO2[    259    ]=    0    ;
    RnonCO2[    260    ]=    0    ;
    RnonCO2[    261    ]=    0    ;
    RnonCO2[    262    ]=    0    ;
    RnonCO2[    263    ]=    0    ;
    RnonCO2[    264    ]=    0    ;
    RnonCO2[    265    ]=    0    ;
    RnonCO2[    266    ]=    0    ;
    RnonCO2[    267    ]=    0    ;
    RnonCO2[    268    ]=    0    ;
    RnonCO2[    269    ]=    0    ;
    RnonCO2[    270    ]=    0    ;
    RnonCO2[    271    ]=    0    ;
    RnonCO2[    272    ]=    0    ;
    RnonCO2[    273    ]=    0    ;
    RnonCO2[    274    ]=    0    ;
    RnonCO2[    275    ]=    0    ;
    RnonCO2[    276    ]=    0    ;
    RnonCO2[    277    ]=    0    ;
    RnonCO2[    278    ]=    0    ;
    RnonCO2[    279    ]=    0    ;
    RnonCO2[    280    ]=    0    ;
    RnonCO2[    281    ]=    0    ;
    RnonCO2[    282    ]=    0    ;
    RnonCO2[    283    ]=    0    ;
    RnonCO2[    284    ]=    0    ;
    RnonCO2[    285    ]=    0    ;
    RnonCO2[    286    ]=    0    ;
    RnonCO2[    287    ]=    0    ;
    RnonCO2[    288    ]=    0    ;
    RnonCO2[    289    ]=    0    ;
    RnonCO2[    290    ]=    0    ;
    RnonCO2[    291    ]=    0    ;
    RnonCO2[    292    ]=    0    ;
    RnonCO2[    293    ]=    0    ;
    RnonCO2[    294    ]=    0    ;
    RnonCO2[    295    ]=    0    ;
    RnonCO2[    296    ]=    0    ;
    RnonCO2[    297    ]=    0    ;
    RnonCO2[    298    ]=    0    ;
    RnonCO2[    299    ]=    0    ;
    RnonCO2[    300    ]=    0    ;
    RnonCO2[    301    ]=    0    ;
    RnonCO2[    302    ]=    0    ;
    RnonCO2[    303    ]=    0    ;
    RnonCO2[    304    ]=    0    ;
    RnonCO2[    305    ]=    0    ;
    RnonCO2[    306    ]=    0    ;
    RnonCO2[    307    ]=    0    ;
    RnonCO2[    308    ]=    0    ;
    RnonCO2[    309    ]=    0    ;
    RnonCO2[    310    ]=    0    ;
    RnonCO2[    311    ]=    0    ;
    RnonCO2[    312    ]=    0    ;
    RnonCO2[    313    ]=    0    ;
    RnonCO2[    314    ]=    0    ;
    RnonCO2[    315    ]=    0    ;
    RnonCO2[    316    ]=    0    ;
    RnonCO2[    317    ]=    0    ;
    RnonCO2[    318    ]=    0    ;
    RnonCO2[    319    ]=    0    ;
    RnonCO2[    320    ]=    0    ;
    RnonCO2[    321    ]=    0    ;
    RnonCO2[    322    ]=    0    ;
    RnonCO2[    323    ]=    0    ;
    RnonCO2[    324    ]=    0    ;
    RnonCO2[    325    ]=    0    ;
    RnonCO2[    326    ]=    0    ;
    RnonCO2[    327    ]=    0    ;
    RnonCO2[    328    ]=    0    ;
    RnonCO2[    329    ]=    0    ;
    RnonCO2[    330    ]=    0    ;
    RnonCO2[    331    ]=    0    ;
    RnonCO2[    332    ]=    0    ;
    RnonCO2[    333    ]=    0    ;
    RnonCO2[    334    ]=    0    ;
    RnonCO2[    335    ]=    0    ;
    RnonCO2[    336    ]=    0    ;
    RnonCO2[    337    ]=    0    ;
    RnonCO2[    338    ]=    0    ;
    RnonCO2[    339    ]=    0    ;
    RnonCO2[    340    ]=    0    ;
    RnonCO2[    341    ]=    0    ;
    RnonCO2[    342    ]=    0    ;
    RnonCO2[    343    ]=    0    ;
    RnonCO2[    344    ]=    0    ;
    RnonCO2[    345    ]=    0    ;
    RnonCO2[    346    ]=    0    ;
    RnonCO2[    347    ]=    0    ;
    RnonCO2[    348    ]=    0    ;
    RnonCO2[    349    ]=    0    ;
    RnonCO2[    350    ]=    0    ;
    RnonCO2[    351    ]=    0    ;
    RnonCO2[    352    ]=    0    ;
    RnonCO2[    353    ]=    0    ;
    RnonCO2[    354    ]=    0    ;
    RnonCO2[    355    ]=    0    ;
    RnonCO2[    356    ]=    0    ;
    RnonCO2[    357    ]=    0    ;
    RnonCO2[    358    ]=    0    ;
    RnonCO2[    359    ]=    0    ;
    RnonCO2[    360    ]=    0    ;
    RnonCO2[    361    ]=    0    ;
    RnonCO2[    362    ]=    0    ;
    RnonCO2[    363    ]=    0    ;
    RnonCO2[    364    ]=    0    ;
    RnonCO2[    365    ]=    0    ;
    RnonCO2[    366    ]=    0    ;
    RnonCO2[    367    ]=    0    ;
    RnonCO2[    368    ]=    0    ;
    RnonCO2[    369    ]=    0    ;
    RnonCO2[    370    ]=    0    ;
    RnonCO2[    371    ]=    0    ;
    RnonCO2[    372    ]=    0    ;
    RnonCO2[    373    ]=    0    ;
    RnonCO2[    374    ]=    0    ;
    RnonCO2[    375    ]=    0    ;
    RnonCO2[    376    ]=    0    ;
    RnonCO2[    377    ]=    0    ;
    RnonCO2[    378    ]=    0    ;
    RnonCO2[    379    ]=    0    ;
    RnonCO2[    380    ]=    0    ;
    RnonCO2[    381    ]=    0    ;
    RnonCO2[    382    ]=    0    ;
    RnonCO2[    383    ]=    0    ;
    RnonCO2[    384    ]=    0    ;
    RnonCO2[    385    ]=    0    ;
    RnonCO2[    386    ]=    0    ;
    RnonCO2[    387    ]=    0    ;
    RnonCO2[    388    ]=    0    ;
    RnonCO2[    389    ]=    0    ;
    RnonCO2[    390    ]=    0    ;
    RnonCO2[    391    ]=    0    ;
    RnonCO2[    392    ]=    0    ;
    RnonCO2[    393    ]=    0    ;
    RnonCO2[    394    ]=    0    ;
    RnonCO2[    395    ]=    0    ;
    RnonCO2[    396    ]=    0    ;
    RnonCO2[    397    ]=    0    ;
    RnonCO2[    398    ]=    0    ;
    RnonCO2[    399    ]=    0    ;
    RnonCO2[    400    ]=    0    ;
    RnonCO2[    401    ]=    0    ;
    RnonCO2[    402    ]=    0    ;
    RnonCO2[    403    ]=    0    ;
    RnonCO2[    404    ]=    0    ;
    RnonCO2[    405    ]=    0    ;
    RnonCO2[    406    ]=    0    ;
    RnonCO2[    407    ]=    0    ;
    RnonCO2[    408    ]=    0    ;
    RnonCO2[    409    ]=    0    ;
    RnonCO2[    410    ]=    0    ;
    RnonCO2[    411    ]=    0    ;
    RnonCO2[    412    ]=    0    ;
    RnonCO2[    413    ]=    0    ;
    RnonCO2[    414    ]=    0    ;
    RnonCO2[    415    ]=    0    ;
    RnonCO2[    416    ]=    0    ;
    RnonCO2[    417    ]=    0    ;
    RnonCO2[    418    ]=    0    ;
    RnonCO2[    419    ]=    0    ;
    RnonCO2[    420    ]=    0    ;
    RnonCO2[    421    ]=    0    ;
    RnonCO2[    422    ]=    0    ;
    RnonCO2[    423    ]=    0    ;
    RnonCO2[    424    ]=    0    ;
    RnonCO2[    425    ]=    0    ;
    RnonCO2[    426    ]=    0    ;
    RnonCO2[    427    ]=    0    ;
    RnonCO2[    428    ]=    0    ;
    RnonCO2[    429    ]=    0    ;
    RnonCO2[    430    ]=    0    ;
    RnonCO2[    431    ]=    0    ;
    RnonCO2[    432    ]=    0    ;
    RnonCO2[    433    ]=    0    ;
    RnonCO2[    434    ]=    0    ;
    RnonCO2[    435    ]=    0    ;
    RnonCO2[    436    ]=    0    ;
    RnonCO2[    437    ]=    0    ;
    RnonCO2[    438    ]=    0    ;
    RnonCO2[    439    ]=    0    ;
    RnonCO2[    440    ]=    0    ;
    RnonCO2[    441    ]=    0    ;
    RnonCO2[    442    ]=    0    ;
    RnonCO2[    443    ]=    0    ;
    RnonCO2[    444    ]=    0    ;
    RnonCO2[    445    ]=    0    ;
    RnonCO2[    446    ]=    0    ;
    RnonCO2[    447    ]=    0    ;
    RnonCO2[    448    ]=    0    ;
    RnonCO2[    449    ]=    0    ;
    RnonCO2[    450    ]=    0    ;
    RnonCO2[    451    ]=    0    ;
    RnonCO2[    452    ]=    0    ;
    RnonCO2[    453    ]=    0    ;
    RnonCO2[    454    ]=    0    ;
    RnonCO2[    455    ]=    0    ;
    RnonCO2[    456    ]=    0    ;
    RnonCO2[    457    ]=    0    ;
    RnonCO2[    458    ]=    0    ;
    RnonCO2[    459    ]=    0    ;
    RnonCO2[    460    ]=    0    ;
    RnonCO2[    461    ]=    0    ;
    RnonCO2[    462    ]=    0    ;
    RnonCO2[    463    ]=    0    ;
    RnonCO2[    464    ]=    0    ;
    RnonCO2[    465    ]=    0    ;
    RnonCO2[    466    ]=    0    ;
    RnonCO2[    467    ]=    0    ;
    RnonCO2[    468    ]=    0    ;
    RnonCO2[    469    ]=    0    ;
    RnonCO2[    470    ]=    0    ;
    RnonCO2[    471    ]=    0    ;
    RnonCO2[    472    ]=    0    ;
    RnonCO2[    473    ]=    0    ;
    RnonCO2[    474    ]=    0    ;
    RnonCO2[    475    ]=    0    ;
    RnonCO2[    476    ]=    0    ;
    RnonCO2[    477    ]=    0    ;
    RnonCO2[    478    ]=    0    ;
    RnonCO2[    479    ]=    0    ;
    RnonCO2[    480    ]=    0    ;
    RnonCO2[    481    ]=    0    ;
    RnonCO2[    482    ]=    0    ;
    RnonCO2[    483    ]=    0    ;
    RnonCO2[    484    ]=    0    ;
    RnonCO2[    485    ]=    0    ;
    RnonCO2[    486    ]=    0    ;
    RnonCO2[    487    ]=    0    ;
    RnonCO2[    488    ]=    0    ;
    RnonCO2[    489    ]=    0    ;
    RnonCO2[    490    ]=    0    ;
    RnonCO2[    491    ]=    0    ;
    RnonCO2[    492    ]=    0    ;
    RnonCO2[    493    ]=    0    ;
    RnonCO2[    494    ]=    0    ;
    RnonCO2[    495    ]=    0    ;
    RnonCO2[    496    ]=    0    ;
    RnonCO2[    497    ]=    0    ;
    RnonCO2[    498    ]=    0    ;
    RnonCO2[    499    ]=    0    ;
    RnonCO2[    500    ]=    0    ;
    RnonCO2[    501    ]=    0    ;
    RnonCO2[    502    ]=    0    ;
    RnonCO2[    503    ]=    0    ;
    RnonCO2[    504    ]=    0    ;
    RnonCO2[    505    ]=    0    ;
    RnonCO2[    506    ]=    0    ;
    RnonCO2[    507    ]=    0    ;
    RnonCO2[    508    ]=    0    ;
    RnonCO2[    509    ]=    0    ;
    RnonCO2[    510    ]=    0    ;
    RnonCO2[    511    ]=    0    ;
    RnonCO2[    512    ]=    0    ;
    RnonCO2[    513    ]=    0    ;
    RnonCO2[    514    ]=    0    ;
    RnonCO2[    515    ]=    0    ;
    RnonCO2[    516    ]=    0    ;
    RnonCO2[    517    ]=    0    ;
    RnonCO2[    518    ]=    0    ;
    RnonCO2[    519    ]=    0    ;
    RnonCO2[    520    ]=    0    ;
    RnonCO2[    521    ]=    0    ;
    RnonCO2[    522    ]=    0    ;
    RnonCO2[    523    ]=    0    ;
    RnonCO2[    524    ]=    0    ;
    RnonCO2[    525    ]=    0    ;
    RnonCO2[    526    ]=    0    ;
    RnonCO2[    527    ]=    0    ;
    RnonCO2[    528    ]=    0    ;
    RnonCO2[    529    ]=    0    ;
    RnonCO2[    530    ]=    0    ;
    RnonCO2[    531    ]=    0    ;
    RnonCO2[    532    ]=    0    ;
    RnonCO2[    533    ]=    0    ;
    RnonCO2[    534    ]=    0    ;
    RnonCO2[    535    ]=    0    ;
    RnonCO2[    536    ]=    0    ;
    RnonCO2[    537    ]=    0    ;
    RnonCO2[    538    ]=    0    ;
    RnonCO2[    539    ]=    0    ;
    RnonCO2[    540    ]=    0    ;
    RnonCO2[    541    ]=    0    ;
    RnonCO2[    542    ]=    0    ;
    RnonCO2[    543    ]=    0    ;
    RnonCO2[    544    ]=    0    ;
    RnonCO2[    545    ]=    0    ;
    RnonCO2[    546    ]=    0    ;
    RnonCO2[    547    ]=    0    ;
    RnonCO2[    548    ]=    0    ;
    RnonCO2[    549    ]=    0    ;
    RnonCO2[    550    ]=    0    ;
    RnonCO2[    551    ]=    0    ;
    RnonCO2[    552    ]=    0    ;
    RnonCO2[    553    ]=    0    ;
    RnonCO2[    554    ]=    0    ;
    RnonCO2[    555    ]=    0    ;
    RnonCO2[    556    ]=    0    ;
    RnonCO2[    557    ]=    0    ;
    RnonCO2[    558    ]=    0    ;
    RnonCO2[    559    ]=    0    ;
    RnonCO2[    560    ]=    0    ;
    RnonCO2[    561    ]=    0    ;
    RnonCO2[    562    ]=    0    ;
    RnonCO2[    563    ]=    0    ;
    RnonCO2[    564    ]=    0    ;
    RnonCO2[    565    ]=    0    ;
    RnonCO2[    566    ]=    0    ;
    RnonCO2[    567    ]=    0    ;
    RnonCO2[    568    ]=    0    ;
    RnonCO2[    569    ]=    0    ;
    RnonCO2[    570    ]=    0    ;
    RnonCO2[    571    ]=    0    ;
    RnonCO2[    572    ]=    0    ;
    RnonCO2[    573    ]=    0    ;
    RnonCO2[    574    ]=    0    ;
    RnonCO2[    575    ]=    0    ;
    RnonCO2[    576    ]=    0    ;
    RnonCO2[    577    ]=    0    ;
    RnonCO2[    578    ]=    0    ;
    RnonCO2[    579    ]=    0    ;
    RnonCO2[    580    ]=    0    ;
    RnonCO2[    581    ]=    0    ;
    RnonCO2[    582    ]=    0    ;
    RnonCO2[    583    ]=    0    ;
    RnonCO2[    584    ]=    0    ;
    RnonCO2[    585    ]=    0    ;
    RnonCO2[    586    ]=    0    ;
    RnonCO2[    587    ]=    0    ;
    RnonCO2[    588    ]=    0    ;
    RnonCO2[    589    ]=    0    ;
    RnonCO2[    590    ]=    0    ;
    RnonCO2[    591    ]=    0    ;
    RnonCO2[    592    ]=    0    ;
    RnonCO2[    593    ]=    0    ;
    RnonCO2[    594    ]=    0    ;
    RnonCO2[    595    ]=    0    ;
    RnonCO2[    596    ]=    0    ;
    RnonCO2[    597    ]=    0    ;
    RnonCO2[    598    ]=    0    ;
    RnonCO2[    599    ]=    0    ;
    RnonCO2[    600    ]=    0    ;
    RnonCO2[    601    ]=    0    ;
    RnonCO2[    602    ]=    0    ;
    RnonCO2[    603    ]=    0    ;
    RnonCO2[    604    ]=    0    ;
    RnonCO2[    605    ]=    0    ;
    RnonCO2[    606    ]=    0    ;
    RnonCO2[    607    ]=    0    ;
    RnonCO2[    608    ]=    0    ;
    RnonCO2[    609    ]=    0    ;
    RnonCO2[    610    ]=    0    ;
    RnonCO2[    611    ]=    0    ;
    RnonCO2[    612    ]=    0    ;
    RnonCO2[    613    ]=    0    ;
    RnonCO2[    614    ]=    0    ;
    RnonCO2[    615    ]=    0    ;
    RnonCO2[    616    ]=    0    ;
    RnonCO2[    617    ]=    0    ;
    RnonCO2[    618    ]=    0    ;
    RnonCO2[    619    ]=    0    ;
    RnonCO2[    620    ]=    0    ;
    RnonCO2[    621    ]=    0    ;
    RnonCO2[    622    ]=    0    ;
    RnonCO2[    623    ]=    0    ;
    RnonCO2[    624    ]=    0    ;
    RnonCO2[    625    ]=    0    ;
    RnonCO2[    626    ]=    0    ;
    RnonCO2[    627    ]=    0    ;
    RnonCO2[    628    ]=    0    ;
    RnonCO2[    629    ]=    0    ;
    RnonCO2[    630    ]=    0    ;
    RnonCO2[    631    ]=    0    ;
    RnonCO2[    632    ]=    0    ;
    RnonCO2[    633    ]=    0    ;
    RnonCO2[    634    ]=    0    ;
    RnonCO2[    635    ]=    0    ;
    RnonCO2[    636    ]=    0    ;
    RnonCO2[    637    ]=    0    ;
    RnonCO2[    638    ]=    0    ;
    RnonCO2[    639    ]=    0    ;
    RnonCO2[    640    ]=    0    ;
    RnonCO2[    641    ]=    0    ;
    RnonCO2[    642    ]=    0    ;
    RnonCO2[    643    ]=    0    ;
    RnonCO2[    644    ]=    0    ;
    RnonCO2[    645    ]=    0    ;
    RnonCO2[    646    ]=    0    ;
    RnonCO2[    647    ]=    0    ;
    RnonCO2[    648    ]=    0    ;
    RnonCO2[    649    ]=    0    ;
    RnonCO2[    650    ]=    0    ;
    RnonCO2[    651    ]=    0    ;
    RnonCO2[    652    ]=    0    ;
    RnonCO2[    653    ]=    0    ;
    RnonCO2[    654    ]=    0    ;
    RnonCO2[    655    ]=    0    ;
    RnonCO2[    656    ]=    0    ;
    RnonCO2[    657    ]=    0    ;
    RnonCO2[    658    ]=    0    ;
    RnonCO2[    659    ]=    0    ;
    RnonCO2[    660    ]=    0    ;
    RnonCO2[    661    ]=    0    ;
    RnonCO2[    662    ]=    0    ;
    RnonCO2[    663    ]=    0    ;
    RnonCO2[    664    ]=    0    ;
    RnonCO2[    665    ]=    0    ;
    RnonCO2[    666    ]=    0    ;
    RnonCO2[    667    ]=    0    ;
    RnonCO2[    668    ]=    0    ;
    RnonCO2[    669    ]=    0    ;
    RnonCO2[    670    ]=    0    ;
    RnonCO2[    671    ]=    0    ;
    RnonCO2[    672    ]=    0    ;
    RnonCO2[    673    ]=    0    ;
    RnonCO2[    674    ]=    0    ;
    RnonCO2[    675    ]=    0    ;
    RnonCO2[    676    ]=    0    ;
    RnonCO2[    677    ]=    0    ;
    RnonCO2[    678    ]=    0    ;
    RnonCO2[    679    ]=    0    ;
    RnonCO2[    680    ]=    0    ;
    RnonCO2[    681    ]=    0    ;
    RnonCO2[    682    ]=    0    ;
    RnonCO2[    683    ]=    0    ;
    RnonCO2[    684    ]=    0    ;
    RnonCO2[    685    ]=    0    ;
    RnonCO2[    686    ]=    0    ;
    RnonCO2[    687    ]=    0    ;
    RnonCO2[    688    ]=    0    ;
    RnonCO2[    689    ]=    0    ;
    RnonCO2[    690    ]=    0    ;
    RnonCO2[    691    ]=    0    ;
    RnonCO2[    692    ]=    0    ;
    RnonCO2[    693    ]=    0    ;
    RnonCO2[    694    ]=    0    ;
    RnonCO2[    695    ]=    0    ;
    RnonCO2[    696    ]=    0    ;
    RnonCO2[    697    ]=    0    ;
    RnonCO2[    698    ]=    0    ;
    RnonCO2[    699    ]=    0    ;
    RnonCO2[    700    ]=    0    ;
    RnonCO2[    701    ]=    0    ;
    RnonCO2[    702    ]=    0    ;
    RnonCO2[    703    ]=    0    ;
    RnonCO2[    704    ]=    0    ;
    RnonCO2[    705    ]=    0    ;
    RnonCO2[    706    ]=    0    ;
    RnonCO2[    707    ]=    0    ;
    RnonCO2[    708    ]=    0    ;
    RnonCO2[    709    ]=    0    ;
    RnonCO2[    710    ]=    0    ;
    RnonCO2[    711    ]=    0    ;
    RnonCO2[    712    ]=    0    ;
    RnonCO2[    713    ]=    0    ;
    RnonCO2[    714    ]=    0    ;
    RnonCO2[    715    ]=    0    ;
    RnonCO2[    716    ]=    0    ;
    RnonCO2[    717    ]=    0    ;
    RnonCO2[    718    ]=    0    ;
    RnonCO2[    719    ]=    0    ;
    RnonCO2[    720    ]=    0    ;
    RnonCO2[    721    ]=    0    ;
    RnonCO2[    722    ]=    0    ;
    RnonCO2[    723    ]=    0    ;
    RnonCO2[    724    ]=    0    ;
    RnonCO2[    725    ]=    0    ;
    RnonCO2[    726    ]=    0    ;
    RnonCO2[    727    ]=    0    ;
    RnonCO2[    728    ]=    0    ;
    RnonCO2[    729    ]=    0    ;
    RnonCO2[    730    ]=    0    ;
    RnonCO2[    731    ]=    0    ;
    RnonCO2[    732    ]=    0    ;
    RnonCO2[    733    ]=    0    ;
    RnonCO2[    734    ]=    0    ;
    RnonCO2[    734    ]=    0    ;
    for(int i = 736; i<tmax; i++)
    {
        RnonCO2[i] = RnonCO2[i-1];
    }
}

void setR_solar(void)
{
    RnonCO2[    0    ]=    -0.02835875    ;
    RnonCO2[    1    ]=    -0.02069375    ;
    RnonCO2[    2    ]=    0.007039375    ;
    RnonCO2[    3    ]=    0.046256875    ;
    RnonCO2[    4    ]=    0.073605    ;
    RnonCO2[    5    ]=    0.077258125    ;
    RnonCO2[    6    ]=    0.064894375    ;
    RnonCO2[    7    ]=    0.04433625    ;
    RnonCO2[    8    ]=    0.020348125    ;
    RnonCO2[    9    ]=    0.00144375    ;
    RnonCO2[    10    ]=    -0.009813125    ;
    RnonCO2[    11    ]=    -0.003950625    ;
    RnonCO2[    12    ]=    0.022645    ;
    RnonCO2[    13    ]=    0.05236875    ;
    RnonCO2[    14    ]=    0.05970125    ;
    RnonCO2[    15    ]=    0.05152875    ;
    RnonCO2[    16    ]=    0.041895    ;
    RnonCO2[    17    ]=    0.019495    ;
    RnonCO2[    18    ]=    -0.006033125    ;
    RnonCO2[    19    ]=    -0.0190575    ;
    RnonCO2[    20    ]=    -0.00597625    ;
    RnonCO2[    21    ]=    0.031189375    ;
    RnonCO2[    22    ]=    0.056993125    ;
    RnonCO2[    23    ]=    0.0586425    ;
    RnonCO2[    24    ]=    0.050159375    ;
    RnonCO2[    25    ]=    0.03401125    ;
    RnonCO2[    26    ]=    0.015911875    ;
    RnonCO2[    27    ]=    0.007214375    ;
    RnonCO2[    28    ]=    0.000695625    ;
    RnonCO2[    29    ]=    -0.013549375    ;
    RnonCO2[    30    ]=    -0.0292075    ;
    RnonCO2[    31    ]=    -0.0392525    ;
    RnonCO2[    32    ]=    -0.046291875    ;
    RnonCO2[    33    ]=    -0.05053125    ;
    RnonCO2[    34    ]=    -0.04930625    ;
    RnonCO2[    35    ]=    -0.03357375    ;
    RnonCO2[    36    ]=    -0.0133    ;
    RnonCO2[    37    ]=    -0.017819375    ;
    RnonCO2[    38    ]=    -0.034545    ;
    RnonCO2[    39    ]=    -0.039536875    ;
    RnonCO2[    40    ]=    -0.04272625    ;
    RnonCO2[    41    ]=    -0.053764375    ;
    RnonCO2[    42    ]=    -0.065043125    ;
    RnonCO2[    43    ]=    -0.0714175    ;
    RnonCO2[    44    ]=    -0.07532875    ;
    RnonCO2[    45    ]=    -0.0767375    ;
    RnonCO2[    46    ]=    -0.074204375    ;
    RnonCO2[    47    ]=    -0.067939375    ;
    RnonCO2[    48    ]=    -0.06085625    ;
    RnonCO2[    49    ]=    -0.054236875    ;
    RnonCO2[    50    ]=    -0.044174375    ;
    RnonCO2[    51    ]=    -0.0345275    ;
    RnonCO2[    52    ]=    -0.034759375    ;
    RnonCO2[    53    ]=    -0.0407925    ;
    RnonCO2[    54    ]=    -0.046659375    ;
    RnonCO2[    55    ]=    -0.0531475    ;
    RnonCO2[    56    ]=    -0.0564375    ;
    RnonCO2[    57    ]=    -0.055076875    ;
    RnonCO2[    58    ]=    -0.051795625    ;
    RnonCO2[    59    ]=    -0.04477375    ;
    RnonCO2[    60    ]=    -0.0320425    ;
    RnonCO2[    61    ]=    -0.01493625    ;
    RnonCO2[    62    ]=    0.003136875    ;
    RnonCO2[    63    ]=    0.01644125    ;
    RnonCO2[    64    ]=    0.023629375    ;
    RnonCO2[    65    ]=    0.02078125    ;
    RnonCO2[    66    ]=    0.003675    ;
    RnonCO2[    67    ]=    -0.016638125    ;
    RnonCO2[    68    ]=    -0.027890625    ;
    RnonCO2[    69    ]=    -0.017539375    ;
    RnonCO2[    70    ]=    0.023550625    ;
    RnonCO2[    71    ]=    0.071575    ;
    RnonCO2[    72    ]=    0.08537375    ;
    RnonCO2[    73    ]=    0.066346875    ;
    RnonCO2[    74    ]=    0.04472125    ;
    RnonCO2[    75    ]=    0.0242725    ;
    RnonCO2[    76    ]=    0.003486875    ;
    RnonCO2[    77    ]=    -0.011038125    ;
    RnonCO2[    78    ]=    -0.018256875    ;
    RnonCO2[    79    ]=    -0.013068125    ;
    RnonCO2[    80    ]=    0.003745    ;
    RnonCO2[    81    ]=    0.022365    ;
    RnonCO2[    82    ]=    0.04412625    ;
    RnonCO2[    83    ]=    0.065358125    ;
    RnonCO2[    84    ]=    0.064448125    ;
    RnonCO2[    85    ]=    0.046816875    ;
    RnonCO2[    86    ]=    0.035013125    ;
    RnonCO2[    87    ]=    0.0234325    ;
    RnonCO2[    88    ]=    0.004930625    ;
    RnonCO2[    89    ]=    -0.01411375    ;
    RnonCO2[    90    ]=    -0.026280625    ;
    RnonCO2[    91    ]=    -0.02471875    ;
    RnonCO2[    92    ]=    -0.00660625    ;
    RnonCO2[    93    ]=    0.0245875    ;
    RnonCO2[    94    ]=    0.054998125    ;
    RnonCO2[    95    ]=    0.065550625    ;
    RnonCO2[    96    ]=    0.05229875    ;
    RnonCO2[    97    ]=    0.030695    ;
    RnonCO2[    98    ]=    0.01407    ;
    RnonCO2[    99    ]=    0.0009975    ;
    RnonCO2[    100    ]=    -0.012429375    ;
    RnonCO2[    101    ]=    -0.0242375    ;
    RnonCO2[    102    ]=    -0.024600625    ;
    RnonCO2[    103    ]=    -0.0025725    ;
    RnonCO2[    104    ]=    0.03553375    ;
    RnonCO2[    105    ]=    0.06412875    ;
    RnonCO2[    106    ]=    0.06727    ;
    RnonCO2[    107    ]=    0.051760625    ;
    RnonCO2[    108    ]=    0.0265125    ;
    RnonCO2[    109    ]=    -2.1875E-05    ;
    RnonCO2[    110    ]=    -0.020846875    ;
    RnonCO2[    111    ]=    -0.031731875    ;
    RnonCO2[    112    ]=    -0.0351575    ;
    RnonCO2[    113    ]=    -0.036675625    ;
    RnonCO2[    114    ]=    -0.029553125    ;
    RnonCO2[    115    ]=    -0.00942375    ;
    RnonCO2[    116    ]=    0.01064    ;
    RnonCO2[    117    ]=    0.0210525    ;
    RnonCO2[    118    ]=    0.02190125    ;
    RnonCO2[    119    ]=    0.010583125    ;
    RnonCO2[    120    ]=    -0.010530625    ;
    RnonCO2[    121    ]=    -0.0316925    ;
    RnonCO2[    122    ]=    -0.044279375    ;
    RnonCO2[    123    ]=    -0.0505225    ;
    RnonCO2[    124    ]=    -0.0530425    ;
    RnonCO2[    125    ]=    -0.04025875    ;
    RnonCO2[    126    ]=    -0.01243375    ;
    RnonCO2[    127    ]=    0.01336125    ;
    RnonCO2[    128    ]=    0.034181875    ;
    RnonCO2[    129    ]=    0.04360125    ;
    RnonCO2[    130    ]=    0.02968    ;
    RnonCO2[    131    ]=    0.0023975    ;
    RnonCO2[    132    ]=    -0.01924125    ;
    RnonCO2[    133    ]=    -0.029859375    ;
    RnonCO2[    134    ]=    -0.036728125    ;
    RnonCO2[    135    ]=    -0.045101875    ;
    RnonCO2[    136    ]=    -0.05291125    ;
    RnonCO2[    137    ]=    -0.046388125    ;
    RnonCO2[    138    ]=    -0.01581125    ;
    RnonCO2[    139    ]=    0.00875    ;
    RnonCO2[    140    ]=    0.012630625    ;
    RnonCO2[    141    ]=    0.016485    ;
    RnonCO2[    142    ]=    0.017793125    ;
    RnonCO2[    143    ]=    0.013584375    ;
    RnonCO2[    144    ]=    0.000923125    ;
    RnonCO2[    145    ]=    -0.019236875    ;
    RnonCO2[    146    ]=    -0.034409375    ;
    RnonCO2[    147    ]=    -0.04139625    ;
    RnonCO2[    148    ]=    -0.03905125    ;
    RnonCO2[    149    ]=    -0.019149375    ;
    RnonCO2[    150    ]=    0.0196175    ;
    RnonCO2[    151    ]=    0.0578375    ;
    RnonCO2[    152    ]=    0.0783825    ;
    RnonCO2[    153    ]=    0.071505    ;
    RnonCO2[    154    ]=    0.0426475    ;
    RnonCO2[    155    ]=    0.013803125    ;
    RnonCO2[    156    ]=    -0.00765625    ;
    RnonCO2[    157    ]=    -0.02118375    ;
    RnonCO2[    158    ]=    -0.022185625    ;
    RnonCO2[    159    ]=    -0.009104375    ;
    RnonCO2[    160    ]=    0.01456    ;
    RnonCO2[    161    ]=    0.04036375    ;
    RnonCO2[    162    ]=    0.0554925    ;
    RnonCO2[    163    ]=    0.051016875    ;
    RnonCO2[    164    ]=    0.0412825    ;
    RnonCO2[    165    ]=    0.032589375    ;
    RnonCO2[    166    ]=    0.016454375    ;
    RnonCO2[    167    ]=    -0.00354375    ;
    RnonCO2[    168    ]=    -0.013908125    ;
    RnonCO2[    169    ]=    6.5625E-05    ;
    RnonCO2[    170    ]=    0.0409325    ;
    RnonCO2[    171    ]=    0.081598125    ;
    RnonCO2[    172    ]=    0.09006375    ;
    RnonCO2[    173    ]=    0.0808325    ;
    RnonCO2[    174    ]=    0.073845625    ;
    RnonCO2[    175    ]=    0.064194375    ;
    RnonCO2[    176    ]=    0.050019375    ;
    RnonCO2[    177    ]=    0.030625    ;
    RnonCO2[    178    ]=    0.013610625    ;
    RnonCO2[    179    ]=    0.021004375    ;
    RnonCO2[    180    ]=    0.049966875    ;
    RnonCO2[    181    ]=    0.080705625    ;
    RnonCO2[    182    ]=    0.11165437    ;
    RnonCO2[    183    ]=    0.12945625    ;
    RnonCO2[    184    ]=    0.11637937    ;
    RnonCO2[    185    ]=    0.0807275    ;
    RnonCO2[    186    ]=    0.049030625    ;
    RnonCO2[    187    ]=    0.031810625    ;
    RnonCO2[    188    ]=    0.02046625    ;
    RnonCO2[    189    ]=    0.022115625    ;
    RnonCO2[    190    ]=    0.057264375    ;
    RnonCO2[    191    ]=    0.12487125    ;
    RnonCO2[    192    ]=    0.17792687    ;
    RnonCO2[    193    ]=    0.17952812    ;
    RnonCO2[    194    ]=    0.15146687    ;
    RnonCO2[    195    ]=    0.11563562    ;
    RnonCO2[    196    ]=    0.072270625    ;
    RnonCO2[    197    ]=    0.039375    ;
    RnonCO2[    198    ]=    0.02737    ;
    RnonCO2[    199    ]=    0.02680125    ;
    RnonCO2[    200    ]=    0.03808875    ;
    RnonCO2[    201    ]=    0.06435625    ;
    RnonCO2[    202    ]=    0.09177875    ;
    RnonCO2[    203    ]=    0.10839937    ;
    RnonCO2[    204    ]=    0.1176525    ;
    RnonCO2[    205    ]=    0.10840375    ;
    RnonCO2[    206    ]=    0.087416875    ;
    RnonCO2[    207    ]=    0.072148125    ;
    RnonCO2[    208    ]=    0.05096    ;
    RnonCO2[    209    ]=    0.0251475    ;
    RnonCO2[    210    ]=    0.0099925    ;
    RnonCO2[    211    ]=    0.0194775    ;
    RnonCO2[    212    ]=    0.0600775    ;
    RnonCO2[    213    ]=    0.12268812    ;
    RnonCO2[    214    ]=    0.17410312    ;
    RnonCO2[    215    ]=    0.19308187    ;
    RnonCO2[    216    ]=    0.17986937    ;
    RnonCO2[    217    ]=    0.14232312    ;
    RnonCO2[    218    ]=    0.09982    ;
    RnonCO2[    219    ]=    0.0546175    ;
    RnonCO2[    220    ]=    0.02268    ;
    RnonCO2[    221    ]=    0.021625625    ;
    RnonCO2[    222    ]=    0.047569375    ;
    RnonCO2[    223    ]=    0.1045625    ;
    RnonCO2[    224    ]=    0.16269312    ;
    RnonCO2[    225    ]=    0.17444    ;
    RnonCO2[    226    ]=    0.15460375    ;
    RnonCO2[    227    ]=    0.12536562    ;
    RnonCO2[    228    ]=    0.08502375    ;
    RnonCO2[    229    ]=    0.04853625    ;
    RnonCO2[    230    ]=    0.02604875    ;
    RnonCO2[    231    ]=    0.019718125    ;
    RnonCO2[    232    ]=    0.04291875    ;
    RnonCO2[    233    ]=    0.09258375    ;
    RnonCO2[    234    ]=    0.14625187    ;
    RnonCO2[    235    ]=    0.18119062    ;
    RnonCO2[    236    ]=    0.19056187    ;
    RnonCO2[    237    ]=    0.17415125    ;
    RnonCO2[    238    ]=    0.13018687    ;
    RnonCO2[    239    ]=    0.086078125    ;
    RnonCO2[    240    ]=    0.059268125    ;
    RnonCO2[    241    ]=    0.043176875    ;
    RnonCO2[    242    ]=    0.032265625    ;
    RnonCO2[    243    ]=    0.025655    ;
    RnonCO2[    244    ]=    0.07252875    ;
    RnonCO2[    245    ]=    0.14625187    ;
    RnonCO2[    246    ]=    0.18119062    ;
    RnonCO2[    247    ]=    0.19056187    ;
    RnonCO2[    248    ]=    0.17415125    ;
    RnonCO2[    249    ]=    0.13018687    ;
    RnonCO2[    250    ]=    0.086078125    ;
    RnonCO2[    251    ]=    0.059268125    ;
    RnonCO2[    252    ]=    0.043176875    ;
    RnonCO2[    253    ]=    0.032265625    ;
    RnonCO2[    254    ]=    0.025655    ;
    RnonCO2[    255    ]=    0.07252875    ;
    RnonCO2[    256    ]=    0.14625187    ;
    RnonCO2[    257    ]=    0.18119062    ;
    RnonCO2[    258    ]=    0.19056187    ;
    RnonCO2[    259    ]=    0.17415125    ;
    RnonCO2[    260    ]=    0.13018687    ;
    RnonCO2[    261    ]=    0.086078125    ;
    RnonCO2[    262    ]=    0.059268125    ;
    RnonCO2[    263    ]=    0.043176875    ;
    RnonCO2[    264    ]=    0.032265625    ;
    RnonCO2[    265    ]=    0.025655    ;
    RnonCO2[    266    ]=    0.07252875    ;
    RnonCO2[    267    ]=    0.14625187    ;
    RnonCO2[    268    ]=    0.18119062    ;
    RnonCO2[    269    ]=    0.19056187    ;
    RnonCO2[    270    ]=    0.17415125    ;
    RnonCO2[    271    ]=    0.13018687    ;
    RnonCO2[    272    ]=    0.086078125    ;
    RnonCO2[    273    ]=    0.059268125    ;
    RnonCO2[    274    ]=    0.043176875    ;
    RnonCO2[    275    ]=    0.032265625    ;
    RnonCO2[    276    ]=    0.025655    ;
    RnonCO2[    277    ]=    0.07252875    ;
    RnonCO2[    278    ]=    0.14625187    ;
    RnonCO2[    279    ]=    0.18119062    ;
    RnonCO2[    280    ]=    0.19056187    ;
    RnonCO2[    281    ]=    0.17415125    ;
    RnonCO2[    282    ]=    0.13018687    ;
    RnonCO2[    283    ]=    0.086078125    ;
    RnonCO2[    284    ]=    0.059268125    ;
    RnonCO2[    285    ]=    0.043176875    ;
    RnonCO2[    286    ]=    0.032265625    ;
    RnonCO2[    287    ]=    0.025655    ;
    RnonCO2[    288    ]=    0.07252875    ;
    RnonCO2[    289    ]=    0.14625187    ;
    RnonCO2[    290    ]=    0.18119062    ;
    RnonCO2[    291    ]=    0.19056187    ;
    RnonCO2[    292    ]=    0.17415125    ;
    RnonCO2[    293    ]=    0.13018687    ;
    RnonCO2[    294    ]=    0.086078125    ;
    RnonCO2[    295    ]=    0.059268125    ;
    RnonCO2[    296    ]=    0.043176875    ;
    RnonCO2[    297    ]=    0.032265625    ;
    RnonCO2[    298    ]=    0.025655    ;
    RnonCO2[    299    ]=    0.07252875    ;
    RnonCO2[    300    ]=    0.14625187    ;
    RnonCO2[    301    ]=    0.18119062    ;
    RnonCO2[    302    ]=    0.19056187    ;
    RnonCO2[    303    ]=    0.17415125    ;
    RnonCO2[    304    ]=    0.13018687    ;
    RnonCO2[    305    ]=    0.086078125    ;
    RnonCO2[    306    ]=    0.059268125    ;
    RnonCO2[    307    ]=    0.043176875    ;
    RnonCO2[    308    ]=    0.032265625    ;
    RnonCO2[    309    ]=    0.025655    ;
    RnonCO2[    310    ]=    0.07252875    ;
    RnonCO2[    311    ]=    0.14625187    ;
    RnonCO2[    312    ]=    0.18119062    ;
    RnonCO2[    313    ]=    0.19056187    ;
    RnonCO2[    314    ]=    0.17415125    ;
    RnonCO2[    315    ]=    0.13018687    ;
    RnonCO2[    316    ]=    0.086078125    ;
    RnonCO2[    317    ]=    0.059268125    ;
    RnonCO2[    318    ]=    0.043176875    ;
    RnonCO2[    319    ]=    0.032265625    ;
    RnonCO2[    320    ]=    0.025655    ;
    RnonCO2[    321    ]=    0.07252875    ;
    RnonCO2[    322    ]=    0.14625187    ;
    RnonCO2[    323    ]=    0.18119062    ;
    RnonCO2[    324    ]=    0.19056187    ;
    RnonCO2[    325    ]=    0.17415125    ;
    RnonCO2[    326    ]=    0.13018687    ;
    RnonCO2[    327    ]=    0.086078125    ;
    RnonCO2[    328    ]=    0.059268125    ;
    RnonCO2[    329    ]=    0.043176875    ;
    RnonCO2[    330    ]=    0.032265625    ;
    RnonCO2[    331    ]=    0.025655    ;
    RnonCO2[    332    ]=    0.07252875    ;
    RnonCO2[    333    ]=    0.14625187    ;
    RnonCO2[    334    ]=    0.18119062    ;
    RnonCO2[    335    ]=    0.19056187    ;
    RnonCO2[    336    ]=    0.17415125    ;
    RnonCO2[    337    ]=    0.13018687    ;
    RnonCO2[    338    ]=    0.086078125    ;
    RnonCO2[    339    ]=    0.059268125    ;
    RnonCO2[    340    ]=    0.043176875    ;
    RnonCO2[    341    ]=    0.032265625    ;
    RnonCO2[    342    ]=    0.025655    ;
    RnonCO2[    343    ]=    0.07252875    ;
    RnonCO2[    344    ]=    0.14625187    ;
    RnonCO2[    345    ]=    0.18119062    ;
    RnonCO2[    346    ]=    0.19056187    ;
    RnonCO2[    347    ]=    0.17415125    ;
    RnonCO2[    348    ]=    0.13018687    ;
    RnonCO2[    349    ]=    0.086078125    ;
    RnonCO2[    350    ]=    0.059268125    ;
    RnonCO2[    351    ]=    0.043176875    ;
    RnonCO2[    352    ]=    0.032265625    ;
    RnonCO2[    353    ]=    0.025655    ;
    RnonCO2[    354    ]=    0.07252875    ;
    RnonCO2[    355    ]=    0.14625187    ;
    RnonCO2[    356    ]=    0.18119062    ;
    RnonCO2[    357    ]=    0.19056187    ;
    RnonCO2[    358    ]=    0.17415125    ;
    RnonCO2[    359    ]=    0.13018687    ;
    RnonCO2[    360    ]=    0.086078125    ;
    RnonCO2[    361    ]=    0.059268125    ;
    RnonCO2[    362    ]=    0.043176875    ;
    RnonCO2[    363    ]=    0.032265625    ;
    RnonCO2[    364    ]=    0.025655    ;
    RnonCO2[    365    ]=    0.07252875    ;
    RnonCO2[    366    ]=    0.14625187    ;
    RnonCO2[    367    ]=    0.18119062    ;
    RnonCO2[    368    ]=    0.19056187    ;
    RnonCO2[    369    ]=    0.17415125    ;
    RnonCO2[    370    ]=    0.13018687    ;
    RnonCO2[    371    ]=    0.086078125    ;
    RnonCO2[    372    ]=    0.059268125    ;
    RnonCO2[    373    ]=    0.043176875    ;
    RnonCO2[    374    ]=    0.032265625    ;
    RnonCO2[    375    ]=    0.025655    ;
    RnonCO2[    376    ]=    0.07252875    ;
    RnonCO2[    377    ]=    0.14625187    ;
    RnonCO2[    378    ]=    0.18119062    ;
    RnonCO2[    379    ]=    0.19056187    ;
    RnonCO2[    380    ]=    0.17415125    ;
    RnonCO2[    381    ]=    0.13018687    ;
    RnonCO2[    382    ]=    0.086078125    ;
    RnonCO2[    383    ]=    0.059268125    ;
    RnonCO2[    384    ]=    0.043176875    ;
    RnonCO2[    385    ]=    0.032265625    ;
    RnonCO2[    386    ]=    0.025655    ;
    RnonCO2[    387    ]=    0.07252875    ;
    RnonCO2[    388    ]=    0.14625187    ;
    RnonCO2[    389    ]=    0.18119062    ;
    RnonCO2[    390    ]=    0.19056187    ;
    RnonCO2[    391    ]=    0.17415125    ;
    RnonCO2[    392    ]=    0.13018687    ;
    RnonCO2[    393    ]=    0.086078125    ;
    RnonCO2[    394    ]=    0.059268125    ;
    RnonCO2[    395    ]=    0.043176875    ;
    RnonCO2[    396    ]=    0.032265625    ;
    RnonCO2[    397    ]=    0.025655    ;
    RnonCO2[    398    ]=    0.07252875    ;
    RnonCO2[    399    ]=    0.14625187    ;
    RnonCO2[    400    ]=    0.18119062    ;
    RnonCO2[    401    ]=    0.19056187    ;
    RnonCO2[    402    ]=    0.17415125    ;
    RnonCO2[    403    ]=    0.13018687    ;
    RnonCO2[    404    ]=    0.086078125    ;
    RnonCO2[    405    ]=    0.059268125    ;
    RnonCO2[    406    ]=    0.043176875    ;
    RnonCO2[    407    ]=    0.032265625    ;
    RnonCO2[    408    ]=    0.025655    ;
    RnonCO2[    409    ]=    0.07252875    ;
    RnonCO2[    410    ]=    0.14625187    ;
    RnonCO2[    411    ]=    0.18119062    ;
    RnonCO2[    412    ]=    0.19056187    ;
    RnonCO2[    413    ]=    0.17415125    ;
    RnonCO2[    414    ]=    0.13018687    ;
    RnonCO2[    415    ]=    0.086078125    ;
    RnonCO2[    416    ]=    0.059268125    ;
    RnonCO2[    417    ]=    0.043176875    ;
    RnonCO2[    418    ]=    0.032265625    ;
    RnonCO2[    419    ]=    0.025655    ;
    RnonCO2[    420    ]=    0.07252875    ;
    RnonCO2[    421    ]=    0.14625187    ;
    RnonCO2[    422    ]=    0.18119062    ;
    RnonCO2[    423    ]=    0.19056187    ;
    RnonCO2[    424    ]=    0.17415125    ;
    RnonCO2[    425    ]=    0.13018687    ;
    RnonCO2[    426    ]=    0.086078125    ;
    RnonCO2[    427    ]=    0.059268125    ;
    RnonCO2[    428    ]=    0.043176875    ;
    RnonCO2[    429    ]=    0.032265625    ;
    RnonCO2[    430    ]=    0.025655    ;
    RnonCO2[    431    ]=    0.07252875    ;
    RnonCO2[    432    ]=    0.14625187    ;
    RnonCO2[    433    ]=    0.18119062    ;
    RnonCO2[    434    ]=    0.19056187    ;
    RnonCO2[    435    ]=    0.17415125    ;
    RnonCO2[    436    ]=    0.13018687    ;
    RnonCO2[    437    ]=    0.086078125    ;
    RnonCO2[    438    ]=    0.059268125    ;
    RnonCO2[    439    ]=    0.043176875    ;
    RnonCO2[    440    ]=    0.032265625    ;
    RnonCO2[    441    ]=    0.025655    ;
    RnonCO2[    442    ]=    0.07252875    ;
    RnonCO2[    443    ]=    0.14625187    ;
    RnonCO2[    444    ]=    0.18119062    ;
    RnonCO2[    445    ]=    0.19056187    ;
    RnonCO2[    446    ]=    0.17415125    ;
    RnonCO2[    447    ]=    0.13018687    ;
    RnonCO2[    448    ]=    0.086078125    ;
    RnonCO2[    449    ]=    0.059268125    ;
    RnonCO2[    450    ]=    0.043176875    ;
    RnonCO2[    451    ]=    0.032265625    ;
    RnonCO2[    452    ]=    0.025655    ;
    RnonCO2[    453    ]=    0.07252875    ;
    RnonCO2[    454    ]=    0.14625187    ;
    RnonCO2[    455    ]=    0.18119062    ;
    RnonCO2[    456    ]=    0.19056187    ;
    RnonCO2[    457    ]=    0.17415125    ;
    RnonCO2[    458    ]=    0.13018687    ;
    RnonCO2[    459    ]=    0.086078125    ;
    RnonCO2[    460    ]=    0.059268125    ;
    RnonCO2[    461    ]=    0.043176875    ;
    RnonCO2[    462    ]=    0.032265625    ;
    RnonCO2[    463    ]=    0.025655    ;
    RnonCO2[    464    ]=    0.07252875    ;
    RnonCO2[    465    ]=    0.14625187    ;
    RnonCO2[    466    ]=    0.18119062    ;
    RnonCO2[    467    ]=    0.19056187    ;
    RnonCO2[    468    ]=    0.17415125    ;
    RnonCO2[    469    ]=    0.13018687    ;
    RnonCO2[    470    ]=    0.086078125    ;
    RnonCO2[    471    ]=    0.059268125    ;
    RnonCO2[    472    ]=    0.043176875    ;
    RnonCO2[    473    ]=    0.032265625    ;
    RnonCO2[    474    ]=    0.025655    ;
    RnonCO2[    475    ]=    0.07252875    ;
    RnonCO2[    476    ]=    0.14625187    ;
    RnonCO2[    477    ]=    0.18119062    ;
    RnonCO2[    478    ]=    0.19056187    ;
    RnonCO2[    479    ]=    0.17415125    ;
    RnonCO2[    480    ]=    0.13018687    ;
    RnonCO2[    481    ]=    0.086078125    ;
    RnonCO2[    482    ]=    0.059268125    ;
    RnonCO2[    483    ]=    0.043176875    ;
    RnonCO2[    484    ]=    0.070611704    ;
    RnonCO2[    485    ]=    0.10375591    ;
    RnonCO2[    486    ]=    0.10375591    ;
    RnonCO2[    487    ]=    0.10375591    ;
    RnonCO2[    488    ]=    0.10375591    ;
    RnonCO2[    489    ]=    0.10375591    ;
    RnonCO2[    490    ]=    0.10375591    ;
    RnonCO2[    491    ]=    0.10375591    ;
    RnonCO2[    492    ]=    0.10375591    ;
    RnonCO2[    493    ]=    0.10375591    ;
    RnonCO2[    494    ]=    0.10375591    ;
    RnonCO2[    495    ]=    0.10375591    ;
    RnonCO2[    496    ]=    0.10375591    ;
    RnonCO2[    497    ]=    0.10375591    ;
    RnonCO2[    498    ]=    0.10375591    ;
    RnonCO2[    499    ]=    0.10375591    ;
    RnonCO2[    500    ]=    0.10375591    ;
    RnonCO2[    501    ]=    0.10375591    ;
    RnonCO2[    502    ]=    0.10375591    ;
    RnonCO2[    503    ]=    0.10375591    ;
    RnonCO2[    504    ]=    0.10375591    ;
    RnonCO2[    505    ]=    0.10375591    ;
    RnonCO2[    506    ]=    0.10375591    ;
    RnonCO2[    507    ]=    0.10375591    ;
    RnonCO2[    508    ]=    0.10375591    ;
    RnonCO2[    509    ]=    0.10375591    ;
    RnonCO2[    510    ]=    0.10375591    ;
    RnonCO2[    511    ]=    0.10375591    ;
    RnonCO2[    512    ]=    0.10375591    ;
    RnonCO2[    513    ]=    0.10375591    ;
    RnonCO2[    514    ]=    0.10375591    ;
    RnonCO2[    515    ]=    0.10375591    ;
    RnonCO2[    516    ]=    0.10375591    ;
    RnonCO2[    517    ]=    0.10375591    ;
    RnonCO2[    518    ]=    0.10375591    ;
    RnonCO2[    519    ]=    0.10375591    ;
    RnonCO2[    520    ]=    0.10375591    ;
    RnonCO2[    521    ]=    0.10375591    ;
    RnonCO2[    522    ]=    0.10375591    ;
    RnonCO2[    523    ]=    0.10375591    ;
    RnonCO2[    524    ]=    0.10375591    ;
    RnonCO2[    525    ]=    0.10375591    ;
    RnonCO2[    526    ]=    0.10375591    ;
    RnonCO2[    527    ]=    0.10375591    ;
    RnonCO2[    528    ]=    0.10375591    ;
    RnonCO2[    529    ]=    0.10375591    ;
    RnonCO2[    530    ]=    0.10375591    ;
    RnonCO2[    531    ]=    0.10375591    ;
    RnonCO2[    532    ]=    0.10375591    ;
    RnonCO2[    533    ]=    0.10375591    ;
    RnonCO2[    534    ]=    0.10375591    ;
    RnonCO2[    535    ]=    0.10375591    ;
    RnonCO2[    536    ]=    0.10375591    ;
    RnonCO2[    537    ]=    0.10375591    ;
    RnonCO2[    538    ]=    0.10375591    ;
    RnonCO2[    539    ]=    0.10375591    ;
    RnonCO2[    540    ]=    0.10375591    ;
    RnonCO2[    541    ]=    0.10375591    ;
    RnonCO2[    542    ]=    0.10375591    ;
    RnonCO2[    543    ]=    0.10375591    ;
    RnonCO2[    544    ]=    0.10375591    ;
    RnonCO2[    545    ]=    0.10375591    ;
    RnonCO2[    546    ]=    0.10375591    ;
    RnonCO2[    547    ]=    0.10375591    ;
    RnonCO2[    548    ]=    0.10375591    ;
    RnonCO2[    549    ]=    0.10375591    ;
    RnonCO2[    550    ]=    0.10375591    ;
    RnonCO2[    551    ]=    0.10375591    ;
    RnonCO2[    552    ]=    0.10375591    ;
    RnonCO2[    553    ]=    0.10375591    ;
    RnonCO2[    554    ]=    0.10375591    ;
    RnonCO2[    555    ]=    0.10375591    ;
    RnonCO2[    556    ]=    0.10375591    ;
    RnonCO2[    557    ]=    0.10375591    ;
    RnonCO2[    558    ]=    0.10375591    ;
    RnonCO2[    559    ]=    0.10375591    ;
    RnonCO2[    560    ]=    0.10375591    ;
    RnonCO2[    561    ]=    0.10375591    ;
    RnonCO2[    562    ]=    0.10375591    ;
    RnonCO2[    563    ]=    0.10375591    ;
    RnonCO2[    564    ]=    0.10375591    ;
    RnonCO2[    565    ]=    0.10375591    ;
    RnonCO2[    566    ]=    0.10375591    ;
    RnonCO2[    567    ]=    0.10375591    ;
    RnonCO2[    568    ]=    0.10375591    ;
    RnonCO2[    569    ]=    0.10375591    ;
    RnonCO2[    570    ]=    0.10375591    ;
    RnonCO2[    571    ]=    0.10375591    ;
    RnonCO2[    572    ]=    0.10375591    ;
    RnonCO2[    573    ]=    0.10375591    ;
    RnonCO2[    574    ]=    0.10375591    ;
    RnonCO2[    575    ]=    0.10375591    ;
    RnonCO2[    576    ]=    0.10375591    ;
    RnonCO2[    577    ]=    0.10375591    ;
    RnonCO2[    578    ]=    0.10375591    ;
    RnonCO2[    579    ]=    0.10375591    ;
    RnonCO2[    580    ]=    0.10375591    ;
    RnonCO2[    581    ]=    0.10375591    ;
    RnonCO2[    582    ]=    0.10375591    ;
    RnonCO2[    583    ]=    0.10375591    ;
    RnonCO2[    584    ]=    0.10375591    ;
    RnonCO2[    585    ]=    0.10375591    ;
    RnonCO2[    586    ]=    0.10375591    ;
    RnonCO2[    587    ]=    0.10375591    ;
    RnonCO2[    588    ]=    0.10375591    ;
    RnonCO2[    589    ]=    0.10375591    ;
    RnonCO2[    590    ]=    0.10375591    ;
    RnonCO2[    591    ]=    0.10375591    ;
    RnonCO2[    592    ]=    0.10375591    ;
    RnonCO2[    593    ]=    0.10375591    ;
    RnonCO2[    594    ]=    0.10375591    ;
    RnonCO2[    595    ]=    0.10375591    ;
    RnonCO2[    596    ]=    0.10375591    ;
    RnonCO2[    597    ]=    0.10375591    ;
    RnonCO2[    598    ]=    0.10375591    ;
    RnonCO2[    599    ]=    0.10375591    ;
    RnonCO2[    600    ]=    0.10375591    ;
    RnonCO2[    601    ]=    0.10375591    ;
    RnonCO2[    602    ]=    0.10375591    ;
    RnonCO2[    603    ]=    0.10375591    ;
    RnonCO2[    604    ]=    0.10375591    ;
    RnonCO2[    605    ]=    0.10375591    ;
    RnonCO2[    606    ]=    0.10375591    ;
    RnonCO2[    607    ]=    0.10375591    ;
    RnonCO2[    608    ]=    0.10375591    ;
    RnonCO2[    609    ]=    0.10375591    ;
    RnonCO2[    610    ]=    0.10375591    ;
    RnonCO2[    611    ]=    0.10375591    ;
    RnonCO2[    612    ]=    0.10375591    ;
    RnonCO2[    613    ]=    0.10375591    ;
    RnonCO2[    614    ]=    0.10375591    ;
    RnonCO2[    615    ]=    0.10375591    ;
    RnonCO2[    616    ]=    0.10375591    ;
    RnonCO2[    617    ]=    0.10375591    ;
    RnonCO2[    618    ]=    0.10375591    ;
    RnonCO2[    619    ]=    0.10375591    ;
    RnonCO2[    620    ]=    0.10375591    ;
    RnonCO2[    621    ]=    0.10375591    ;
    RnonCO2[    622    ]=    0.10375591    ;
    RnonCO2[    623    ]=    0.10375591    ;
    RnonCO2[    624    ]=    0.10375591    ;
    RnonCO2[    625    ]=    0.10375591    ;
    RnonCO2[    626    ]=    0.10375591    ;
    RnonCO2[    627    ]=    0.10375591    ;
    RnonCO2[    628    ]=    0.10375591    ;
    RnonCO2[    629    ]=    0.10375591    ;
    RnonCO2[    630    ]=    0.10375591    ;
    RnonCO2[    631    ]=    0.10375591    ;
    RnonCO2[    632    ]=    0.10375591    ;
    RnonCO2[    633    ]=    0.10375591    ;
    RnonCO2[    634    ]=    0.10375591    ;
    RnonCO2[    635    ]=    0.10375591    ;
    RnonCO2[    636    ]=    0.10375591    ;
    RnonCO2[    637    ]=    0.10375591    ;
    RnonCO2[    638    ]=    0.10375591    ;
    RnonCO2[    639    ]=    0.10375591    ;
    RnonCO2[    640    ]=    0.10375591    ;
    RnonCO2[    641    ]=    0.10375591    ;
    RnonCO2[    642    ]=    0.10375591    ;
    RnonCO2[    643    ]=    0.10375591    ;
    RnonCO2[    644    ]=    0.10375591    ;
    RnonCO2[    645    ]=    0.10375591    ;
    RnonCO2[    646    ]=    0.10375591    ;
    RnonCO2[    647    ]=    0.10375591    ;
    RnonCO2[    648    ]=    0.10375591    ;
    RnonCO2[    649    ]=    0.10375591    ;
    RnonCO2[    650    ]=    0.10375591    ;
    RnonCO2[    651    ]=    0.10375591    ;
    RnonCO2[    652    ]=    0.10375591    ;
    RnonCO2[    653    ]=    0.10375591    ;
    RnonCO2[    654    ]=    0.10375591    ;
    RnonCO2[    655    ]=    0.10375591    ;
    RnonCO2[    656    ]=    0.10375591    ;
    RnonCO2[    657    ]=    0.10375591    ;
    RnonCO2[    658    ]=    0.10375591    ;
    RnonCO2[    659    ]=    0.10375591    ;
    RnonCO2[    660    ]=    0.10375591    ;
    RnonCO2[    661    ]=    0.10375591    ;
    RnonCO2[    662    ]=    0.10375591    ;
    RnonCO2[    663    ]=    0.10375591    ;
    RnonCO2[    664    ]=    0.10375591    ;
    RnonCO2[    665    ]=    0.10375591    ;
    RnonCO2[    666    ]=    0.10375591    ;
    RnonCO2[    667    ]=    0.10375591    ;
    RnonCO2[    668    ]=    0.10375591    ;
    RnonCO2[    669    ]=    0.10375591    ;
    RnonCO2[    670    ]=    0.10375591    ;
    RnonCO2[    671    ]=    0.10375591    ;
    RnonCO2[    672    ]=    0.10375591    ;
    RnonCO2[    673    ]=    0.10375591    ;
    RnonCO2[    674    ]=    0.10375591    ;
    RnonCO2[    675    ]=    0.10375591    ;
    RnonCO2[    676    ]=    0.10375591    ;
    RnonCO2[    677    ]=    0.10375591    ;
    RnonCO2[    678    ]=    0.10375591    ;
    RnonCO2[    679    ]=    0.10375591    ;
    RnonCO2[    680    ]=    0.10375591    ;
    RnonCO2[    681    ]=    0.10375591    ;
    RnonCO2[    682    ]=    0.10375591    ;
    RnonCO2[    683    ]=    0.10375591    ;
    RnonCO2[    684    ]=    0.10375591    ;
    RnonCO2[    685    ]=    0.10375591    ;
    RnonCO2[    686    ]=    0.10375591    ;
    RnonCO2[    687    ]=    0.10375591    ;
    RnonCO2[    688    ]=    0.10375591    ;
    RnonCO2[    689    ]=    0.10375591    ;
    RnonCO2[    690    ]=    0.10375591    ;
    RnonCO2[    691    ]=    0.10375591    ;
    RnonCO2[    692    ]=    0.10375591    ;
    RnonCO2[    693    ]=    0.10375591    ;
    RnonCO2[    694    ]=    0.10375591    ;
    RnonCO2[    695    ]=    0.10375591    ;
    RnonCO2[    696    ]=    0.10375591    ;
    RnonCO2[    697    ]=    0.10375591    ;
    RnonCO2[    698    ]=    0.10375591    ;
    RnonCO2[    699    ]=    0.10375591    ;
    RnonCO2[    700    ]=    0.10375591    ;
    RnonCO2[    701    ]=    0.10375591    ;
    RnonCO2[    702    ]=    0.10375591    ;
    RnonCO2[    703    ]=    0.10375591    ;
    RnonCO2[    704    ]=    0.10375591    ;
    RnonCO2[    705    ]=    0.10375591    ;
    RnonCO2[    706    ]=    0.10375591    ;
    RnonCO2[    707    ]=    0.10375591    ;
    RnonCO2[    708    ]=    0.10375591    ;
    RnonCO2[    709    ]=    0.10375591    ;
    RnonCO2[    710    ]=    0.10375591    ;
    RnonCO2[    711    ]=    0.10375591    ;
    RnonCO2[    712    ]=    0.10375591    ;
    RnonCO2[    713    ]=    0.10375591    ;
    RnonCO2[    714    ]=    0.10375591    ;
    RnonCO2[    715    ]=    0.10375591    ;
    RnonCO2[    716    ]=    0.10375591    ;
    RnonCO2[    717    ]=    0.10375591    ;
    RnonCO2[    718    ]=    0.10375591    ;
    RnonCO2[    719    ]=    0.10375591    ;
    RnonCO2[    720    ]=    0.10375591    ;
    RnonCO2[    721    ]=    0.10375591    ;
    RnonCO2[    722    ]=    0.10375591    ;
    RnonCO2[    723    ]=    0.10375591    ;
    RnonCO2[    724    ]=    0.10375591    ;
    RnonCO2[    725    ]=    0.10375591    ;
    RnonCO2[    726    ]=    0.10375591    ;
    RnonCO2[    727    ]=    0.10375591    ;
    RnonCO2[    728    ]=    0.10375591    ;
    RnonCO2[    729    ]=    0.10375591    ;
    RnonCO2[    730    ]=    0.10375591    ;
    RnonCO2[    731    ]=    0.10375591    ;
    RnonCO2[    732    ]=    0.10375591    ;
    RnonCO2[    733    ]=    0.10375591    ;
    RnonCO2[    734    ]=    0.10375591    ;
    RnonCO2[    735    ]=    0.10375591    ;
    
    for(int i = 736; i<tmax; i++)
    {
        RnonCO2[i] = RnonCO2[i-1];
    }
}

void setR_volcanic2(void)
{
    RnonCO2[	0	]=	0.00000000	;
    RnonCO2[	1	]=	0.00000000	;
    RnonCO2[	2	]=	0.00000000	;
    RnonCO2[	3	]=	-0.03500000	;
    RnonCO2[	4	]=	-0.10062500	;
    RnonCO2[	5	]=	-0.10500000	;
    RnonCO2[	6	]=	-0.05250000	;
    RnonCO2[	7	]=	-0.01750000	;
    RnonCO2[	8	]=	-0.00437500	;
    RnonCO2[	9	]=	0.00000000	;
    RnonCO2[	10	]=	0.00000000	;
    RnonCO2[	11	]=	0.00000000	;
    RnonCO2[	12	]=	0.00000000	;
    RnonCO2[	13	]=	0.00000000	;
    RnonCO2[	14	]=	0.00000000	;
    RnonCO2[	15	]=	0.00000000	;
    RnonCO2[	16	]=	0.00000000	;
    RnonCO2[	17	]=	-0.02187500	;
    RnonCO2[	18	]=	-0.11375000	;
    RnonCO2[	19	]=	-0.18812500	;
    RnonCO2[	20	]=	-0.13125000	;
    RnonCO2[	21	]=	-0.04375000	;
    RnonCO2[	22	]=	-0.00875000	;
    RnonCO2[	23	]=	-0.00437500	;
    RnonCO2[	24	]=	-0.10937500	;
    RnonCO2[	25	]=	-0.25812500	;
    RnonCO2[	26	]=	-0.22312500	;
    RnonCO2[	27	]=	-0.09187500	;
    RnonCO2[	28	]=	-0.02625000	;
    RnonCO2[	29	]=	-0.01750000	;
    RnonCO2[	30	]=	-0.03937500	;
    RnonCO2[	31	]=	-0.04375000	;
    RnonCO2[	32	]=	-0.02187500	;
    RnonCO2[	33	]=	-0.00437500	;
    RnonCO2[	34	]=	0.00000000	;
    RnonCO2[	35	]=	0.00000000	;
    RnonCO2[	36	]=	0.00000000	;
    RnonCO2[	37	]=	0.00000000	;
    RnonCO2[	38	]=	0.00000000	;
    RnonCO2[	39	]=	0.00000000	;
    RnonCO2[	40	]=	0.00000000	;
    RnonCO2[	41	]=	0.00000000	;
    RnonCO2[	42	]=	0.00000000	;
    RnonCO2[	43	]=	-0.49000000	;
    RnonCO2[	44	]=	-1.63187502	;
    RnonCO2[	45	]=	-2.03437502	;
    RnonCO2[	46	]=	-1.24250002	;
    RnonCO2[	47	]=	-0.57312500	;
    RnonCO2[	48	]=	-0.38062500	;
    RnonCO2[	49	]=	-0.91000000	;
    RnonCO2[	50	]=	-2.38437502	;
    RnonCO2[	51	]=	-2.89625002	;
    RnonCO2[	52	]=	-1.72812502	;
    RnonCO2[	53	]=	-0.59500000	;
    RnonCO2[	54	]=	-0.14000000	;
    RnonCO2[	55	]=	-0.00875000	;
    RnonCO2[	56	]=	0.00000000	;
    RnonCO2[	57	]=	0.00000000	;
    RnonCO2[	58	]=	0.00000000	;
    RnonCO2[	59	]=	0.00000000	;
    RnonCO2[	60	]=	0.00000000	;
    RnonCO2[	61	]=	0.00000000	;
    RnonCO2[	62	]=	0.00000000	;
    RnonCO2[	63	]=	-0.00437500	;
    RnonCO2[	64	]=	-0.03937500	;
    RnonCO2[	65	]=	-0.25375000	;
    RnonCO2[	66	]=	-0.63000000	;
    RnonCO2[	67	]=	-0.71750000	;
    RnonCO2[	68	]=	-0.41562500	;
    RnonCO2[	69	]=	-0.57750000	;
    RnonCO2[	70	]=	-1.22062500	;
    RnonCO2[	71	]=	-1.18125000	;
    RnonCO2[	72	]=	-0.58187500	;
    RnonCO2[	73	]=	-0.18812500	;
    RnonCO2[	74	]=	-0.05250000	;
    RnonCO2[	75	]=	-0.05250000	;
    RnonCO2[	76	]=	-0.05687500	;
    RnonCO2[	77	]=	-0.04375000	;
    RnonCO2[	78	]=	-0.06562500	;
    RnonCO2[	79	]=	-0.07875000	;
    RnonCO2[	80	]=	-0.04812500	;
    RnonCO2[	81	]=	-0.01750000	;
    RnonCO2[	82	]=	-0.00437500	;
    RnonCO2[	83	]=	0.00000000	;
    RnonCO2[	84	]=	-0.01645000	;
    RnonCO2[	85	]=	-0.04043958	;
    RnonCO2[	86	]=	-0.03941146	;
    RnonCO2[	87	]=	-0.02364688	;
    RnonCO2[	88	]=	-0.00891042	;
    RnonCO2[	89	]=	-0.00068542	;
    RnonCO2[	90	]=	-0.13125729	;
    RnonCO2[	91	]=	-0.58911563	;
    RnonCO2[	92	]=	-0.84271979	;
    RnonCO2[	93	]=	-0.53016979	;
    RnonCO2[	94	]=	-0.20425417	;
    RnonCO2[	95	]=	-0.08190729	;
    RnonCO2[	96	]=	-0.09287396	;
    RnonCO2[	97	]=	-0.17718021	;
    RnonCO2[	98	]=	-0.16484271	;
    RnonCO2[	99	]=	-0.08156458	;
    RnonCO2[	100	]=	-0.03392813	;
    RnonCO2[	101	]=	-0.01336563	;
    RnonCO2[	102	]=	-0.00342708	;
    RnonCO2[	103	]=	-0.00137083	;
    RnonCO2[	104	]=	-0.00959583	;
    RnonCO2[	105	]=	-0.01370833	;
    RnonCO2[	106	]=	-0.01302292	;
    RnonCO2[	107	]=	-0.02536042	;
    RnonCO2[	108	]=	-0.03906875	;
    RnonCO2[	109	]=	-0.03118646	;
    RnonCO2[	110	]=	-0.03838333	;
    RnonCO2[	111	]=	-0.07950833	;
    RnonCO2[	112	]=	-0.08396354	;
    RnonCO2[	113	]=	-0.05209167	;
    RnonCO2[	114	]=	-0.03050104	;
    RnonCO2[	115	]=	-0.01679271	;
    RnonCO2[	116	]=	-0.00959583	;
    RnonCO2[	117	]=	-0.12131875	;
    RnonCO2[	118	]=	-1.03497917	;
    RnonCO2[	119	]=	-1.76974582	;
    RnonCO2[	120	]=	-1.17651771	;
    RnonCO2[	121	]=	-0.68096146	;
    RnonCO2[	122	]=	-0.55381667	;
    RnonCO2[	123	]=	-0.44346458	;
    RnonCO2[	124	]=	-0.50995000	;
    RnonCO2[	125	]=	-0.56409792	;
    RnonCO2[	126	]=	-0.49418542	;
    RnonCO2[	127	]=	-0.32008959	;
    RnonCO2[	128	]=	-0.17238229	;
    RnonCO2[	129	]=	-0.06305833	;
    RnonCO2[	130	]=	-0.07059792	;
    RnonCO2[	131	]=	-0.21590625	;
    RnonCO2[	132	]=	-0.29712813	;
    RnonCO2[	133	]=	-0.19465834	;
    RnonCO2[	134	]=	-0.08190729	;
    RnonCO2[	135	]=	-0.03050104	;
    RnonCO2[	136	]=	-0.04078229	;
    RnonCO2[	137	]=	-0.48938750	;
    RnonCO2[	138	]=	-0.88350208	;
    RnonCO2[	139	]=	-0.58877292	;
    RnonCO2[	140	]=	-0.23030000	;
    RnonCO2[	141	]=	-0.13228542	;
    RnonCO2[	142	]=	-0.15627500	;
    RnonCO2[	143	]=	-0.14462292	;
    RnonCO2[	144	]=	-0.07848021	;
    RnonCO2[	145	]=	-0.04352396	;
    RnonCO2[	146	]=	-0.03735521	;
    RnonCO2[	147	]=	-0.28102084	;
    RnonCO2[	148	]=	-0.38897396	;
    RnonCO2[	149	]=	-0.17683750	;
    RnonCO2[	150	]=	-0.07299688	;
    RnonCO2[	151	]=	-0.04489479	;
    RnonCO2[	152	]=	-0.03632708	;
    RnonCO2[	153	]=	-0.03050104	;
    RnonCO2[	154	]=	-0.03769792	;
    RnonCO2[	155	]=	-0.12166146	;
    RnonCO2[	156	]=	-0.13674063	;
    RnonCO2[	157	]=	-0.05140625	;
    RnonCO2[	158	]=	-0.03015833	;
    RnonCO2[	159	]=	-0.04318125	;
    RnonCO2[	160	]=	-0.04318125	;
    RnonCO2[	161	]=	-0.03529896	;
    RnonCO2[	162	]=	-0.03735521	;
    RnonCO2[	163	]=	-0.08670521	;
    RnonCO2[	164	]=	-0.13639792	;
    RnonCO2[	165	]=	-0.11069479	;
    RnonCO2[	166	]=	-0.08601979	;
    RnonCO2[	167	]=	-0.11754896	;
    RnonCO2[	168	]=	-0.10863854	;
    RnonCO2[	169	]=	-0.07025521	;
    RnonCO2[	170	]=	-0.06305833	;
    RnonCO2[	171	]=	-0.05174896	;
    RnonCO2[	172	]=	-0.05311979	;
    RnonCO2[	173	]=	-0.06922708	;
    RnonCO2[	174	]=	-0.06854167	;
    RnonCO2[	175	]=	-0.04866458	;
    RnonCO2[	176	]=	-0.03941146	;
    RnonCO2[	177	]=	-0.06100208	;
    RnonCO2[	178	]=	-0.06614271	;
    RnonCO2[	179	]=	-0.04318125	;
    RnonCO2[	180	]=	-0.03255729	;
    RnonCO2[	181	]=	-0.02536042	;
    RnonCO2[	182	]=	-0.03358542	;
    RnonCO2[	183	]=	-0.03564167	;
    RnonCO2[	184	]=	-0.04318125	;
    RnonCO2[	185	]=	-0.04558021	;
    RnonCO2[	186	]=	-0.04181042	;
    RnonCO2[	187	]=	-0.04797917	;
    RnonCO2[	188	]=	-0.05688958	;
    RnonCO2[	189	]=	-0.05174896	;
    RnonCO2[	190	]=	-0.02947292	;
    RnonCO2[	191	]=	-0.01713542	;
    RnonCO2[	192	]=	-0.00685417	;
    RnonCO2[	193	]=	-0.00171354	;
    RnonCO2[	194	]=	-0.00548333	;
    RnonCO2[	195	]=	-0.08053646	;
    RnonCO2[	196	]=	-0.17169688	;
    RnonCO2[	197	]=	-0.24640729	;
    RnonCO2[	198	]=	-0.72791250	;
    RnonCO2[	199	]=	-1.08570000	;
    RnonCO2[	200	]=	-0.76492500	;
    RnonCO2[	201	]=	-0.38246250	;
    RnonCO2[	202	]=	-0.25394688	;
    RnonCO2[	203	]=	-0.45271771	;
    RnonCO2[	204	]=	-0.52879896	;
    RnonCO2[	205	]=	-0.28787500	;
    RnonCO2[	206	]=	-0.11823438	;
    RnonCO2[	207	]=	-0.07916563	;
    RnonCO2[	208	]=	-0.12611667	;
    RnonCO2[	209	]=	-0.27313854	;
    RnonCO2[	210	]=	-0.37355208	;
    RnonCO2[	211	]=	-0.24983438	;
    RnonCO2[	212	]=	-0.09835729	;
    RnonCO2[	213	]=	-0.12337500	;
    RnonCO2[	214	]=	-0.13845417	;
    RnonCO2[	215	]=	-0.08430625	;
    RnonCO2[	216	]=	-0.15284792	;
    RnonCO2[	217	]=	-0.82387083	;
    RnonCO2[	218	]=	-1.10866146	;
    RnonCO2[	219	]=	-0.55244583	;
    RnonCO2[	220	]=	-0.25566042	;
    RnonCO2[	221	]=	-0.19740000	;
    RnonCO2[	222	]=	-0.16655625	;
    RnonCO2[	223	]=	-0.12645938	;
    RnonCO2[	224	]=	-0.10315521	;
    RnonCO2[	225	]=	-0.10589688	;
    RnonCO2[	226	]=	-1.04731667	;
    RnonCO2[	227	]=	-1.65219692	;
    RnonCO2[	228	]=	-0.91434583	;
    RnonCO2[	229	]=	-0.36395625	;
    RnonCO2[	230	]=	-0.17375313	;
    RnonCO2[	231	]=	-0.10863854	;
    RnonCO2[	232	]=	-0.07916563	;
    RnonCO2[	233	]=	-0.04489479	;
    RnonCO2[	234	]=	-0.01610729	;
    RnonCO2[	235	]=	-0.00376979	;
    RnonCO2[	236	]=	0.00000000	;
    RnonCO2[	237	]=	0.00000000	;
    RnonCO2[	238	]=	0.00000000	;
    RnonCO2[	239	]=	0.00000000	;
    RnonCO2[	240	]=	0.00000000	;
    RnonCO2[	241	]=	0.00000000	;
    RnonCO2[	242	]=	0.00000000	;
    RnonCO2[	243	]=	0.00000000	;
    RnonCO2[	244	]=	0.00000000	;
    RnonCO2[	245	]=	0.00000000	;
    RnonCO2[	246	]=	0.00000000	;
    RnonCO2[	247	]=	0.00000000	;
    RnonCO2[	248	]=	0.00000000	;
    RnonCO2[	249	]=	0.00000000	;
    RnonCO2[	250	]=	0.00000000	;
    RnonCO2[	251	]=	0.00000000	;
    RnonCO2[	252	]=	0.00000000	;
    RnonCO2[	253	]=	0.00000000	;
    RnonCO2[	254	]=	0.00000000	;
    RnonCO2[	255	]=	0.00000000	;
    RnonCO2[	256	]=	0.00000000	;
    RnonCO2[	257	]=	0.00000000	;
    RnonCO2[	258	]=	0.00000000	;
    RnonCO2[	259	]=	0.00000000	;
    RnonCO2[	260	]=	0.00000000	;
    RnonCO2[	261	]=	0.00000000	;
    RnonCO2[	262	]=	0.00000000	;
    RnonCO2[	263	]=	0.00000000	;
    RnonCO2[	264	]=	0.00000000	;
    RnonCO2[	265	]=	0.00000000	;
    RnonCO2[	266	]=	0.00000000	;
    RnonCO2[	267	]=	0.00000000	;
    RnonCO2[	268	]=	0.00000000	;
    RnonCO2[	269	]=	0.00000000	;
    RnonCO2[	270	]=	0.00000000	;
    RnonCO2[	271	]=	0.00000000	;
    RnonCO2[	272	]=	0.00000000	;
    RnonCO2[	273	]=	0.00000000	;
    RnonCO2[	274	]=	0.00000000	;
    RnonCO2[	275	]=	0.00000000	;
    RnonCO2[	276	]=	0.00000000	;
    RnonCO2[	277	]=	0.00000000	;
    RnonCO2[	278	]=	0.00000000	;
    RnonCO2[	279	]=	0.00000000	;
    RnonCO2[	280	]=	0.00000000	;
    RnonCO2[	281	]=	0.00000000	;
    RnonCO2[	282	]=	0.00000000	;
    RnonCO2[	283	]=	0.00000000	;
    RnonCO2[	284	]=	0.00000000	;
    RnonCO2[	285	]=	0.00000000	;
    RnonCO2[	286	]=	0.00000000	;
    RnonCO2[	287	]=	0.00000000	;
    RnonCO2[	288	]=	0.00000000	;
    RnonCO2[	289	]=	0.00000000	;
    RnonCO2[	290	]=	0.00000000	;
    RnonCO2[	291	]=	0.00000000	;
    RnonCO2[	292	]=	0.00000000	;
    RnonCO2[	293	]=	0.00000000	;
    RnonCO2[	294	]=	0.00000000	;
    RnonCO2[	295	]=	0.00000000	;
    RnonCO2[	296	]=	0.00000000	;
    RnonCO2[	297	]=	0.00000000	;
    RnonCO2[	298	]=	0.00000000	;
    RnonCO2[	299	]=	0.00000000	;
    RnonCO2[	300	]=	0.00000000	;
    RnonCO2[	301	]=	0.00000000	;
    RnonCO2[	302	]=	0.00000000	;
    RnonCO2[	303	]=	0.00000000	;
    RnonCO2[	304	]=	0.00000000	;
    RnonCO2[	305	]=	0.00000000	;
    RnonCO2[	306	]=	0.00000000	;
    RnonCO2[	307	]=	0.00000000	;
    RnonCO2[	308	]=	0.00000000	;
    RnonCO2[	309	]=	0.00000000	;
    RnonCO2[	310	]=	0.00000000	;
    RnonCO2[	311	]=	0.00000000	;
    RnonCO2[	312	]=	0.00000000	;
    RnonCO2[	313	]=	0.00000000	;
    RnonCO2[	314	]=	0.00000000	;
    RnonCO2[	315	]=	0.00000000	;
    RnonCO2[	316	]=	0.00000000	;
    RnonCO2[	317	]=	0.00000000	;
    RnonCO2[	318	]=	0.00000000	;
    RnonCO2[	319	]=	0.00000000	;
    RnonCO2[	320	]=	0.00000000	;
    RnonCO2[	321	]=	0.00000000	;
    RnonCO2[	322	]=	0.00000000	;
    RnonCO2[	323	]=	0.00000000	;
    RnonCO2[	324	]=	0.00000000	;
    RnonCO2[	325	]=	0.00000000	;
    RnonCO2[	326	]=	0.00000000	;
    RnonCO2[	327	]=	0.00000000	;
    RnonCO2[	328	]=	0.00000000	;
    RnonCO2[	329	]=	0.00000000	;
    RnonCO2[	330	]=	0.00000000	;
    RnonCO2[	331	]=	0.00000000	;
    RnonCO2[	332	]=	0.00000000	;
    RnonCO2[	333	]=	0.00000000	;
    RnonCO2[	334	]=	0.00000000	;
    RnonCO2[	335	]=	0.00000000	;
    RnonCO2[	336	]=	0.00000000	;
    RnonCO2[	337	]=	0.00000000	;
    RnonCO2[	338	]=	0.00000000	;
    RnonCO2[	339	]=	0.00000000	;
    RnonCO2[	340	]=	0.00000000	;
    RnonCO2[	341	]=	0.00000000	;
    RnonCO2[	342	]=	0.00000000	;
    RnonCO2[	343	]=	0.00000000	;
    RnonCO2[	344	]=	0.00000000	;
    RnonCO2[	345	]=	0.00000000	;
    RnonCO2[	346	]=	0.00000000	;
    RnonCO2[	347	]=	0.00000000	;
    RnonCO2[	348	]=	0.00000000	;
    RnonCO2[	349	]=	0.00000000	;
    RnonCO2[	350	]=	0.00000000	;
    RnonCO2[	351	]=	0.00000000	;
    RnonCO2[	352	]=	0.00000000	;
    RnonCO2[	353	]=	0.00000000	;
    RnonCO2[	354	]=	0.00000000	;
    RnonCO2[	355	]=	0.00000000	;
    RnonCO2[	356	]=	0.00000000	;
    RnonCO2[	357	]=	0.00000000	;
    RnonCO2[	358	]=	0.00000000	;
    RnonCO2[	359	]=	0.00000000	;
    RnonCO2[	360	]=	0.00000000	;
    RnonCO2[	361	]=	0.00000000	;
    RnonCO2[	362	]=	0.00000000	;
    RnonCO2[	363	]=	0.00000000	;
    RnonCO2[	364	]=	0.00000000	;
    RnonCO2[	365	]=	0.00000000	;
    RnonCO2[	366	]=	0.00000000	;
    RnonCO2[	367	]=	0.00000000	;
    RnonCO2[	368	]=	0.00000000	;
    RnonCO2[	369	]=	0.00000000	;
    RnonCO2[	370	]=	0.00000000	;
    RnonCO2[	371	]=	0.00000000	;
    RnonCO2[	372	]=	0.00000000	;
    RnonCO2[	373	]=	0.00000000	;
    RnonCO2[	374	]=	0.00000000	;
    RnonCO2[	375	]=	0.00000000	;
    RnonCO2[	376	]=	0.00000000	;
    RnonCO2[	377	]=	0.00000000	;
    RnonCO2[	378	]=	0.00000000	;
    RnonCO2[	379	]=	0.00000000	;
    RnonCO2[	380	]=	0.00000000	;
    RnonCO2[	381	]=	0.00000000	;
    RnonCO2[	382	]=	0.00000000	;
    RnonCO2[	383	]=	0.00000000	;
    RnonCO2[	384	]=	0.00000000	;
    RnonCO2[	385	]=	0.00000000	;
    RnonCO2[	386	]=	0.00000000	;
    RnonCO2[	387	]=	0.00000000	;
    RnonCO2[	388	]=	0.00000000	;
    RnonCO2[	389	]=	0.00000000	;
    RnonCO2[	390	]=	0.00000000	;
    RnonCO2[	391	]=	0.00000000	;
    RnonCO2[	392	]=	0.00000000	;
    RnonCO2[	393	]=	0.00000000	;
    RnonCO2[	394	]=	0.00000000	;
    RnonCO2[	395	]=	0.00000000	;
    RnonCO2[	396	]=	0.00000000	;
    RnonCO2[	397	]=	0.00000000	;
    RnonCO2[	398	]=	0.00000000	;
    RnonCO2[	399	]=	0.00000000	;
    RnonCO2[	400	]=	0.00000000	;
    RnonCO2[	401	]=	0.00000000	;
    RnonCO2[	402	]=	0.00000000	;
    RnonCO2[	403	]=	0.00000000	;
    RnonCO2[	404	]=	0.00000000	;
    RnonCO2[	405	]=	0.00000000	;
    RnonCO2[	406	]=	0.00000000	;
    RnonCO2[	407	]=	0.00000000	;
    RnonCO2[	408	]=	0.00000000	;
    RnonCO2[	409	]=	0.00000000	;
    RnonCO2[	410	]=	0.00000000	;
    RnonCO2[	411	]=	0.00000000	;
    RnonCO2[	412	]=	0.00000000	;
    RnonCO2[	413	]=	0.00000000	;
    RnonCO2[	414	]=	0.00000000	;
    RnonCO2[	415	]=	0.00000000	;
    RnonCO2[	416	]=	0.00000000	;
    RnonCO2[	417	]=	0.00000000	;
    RnonCO2[	418	]=	0.00000000	;
    RnonCO2[	419	]=	0.00000000	;
    RnonCO2[	420	]=	0.00000000	;
    RnonCO2[	421	]=	0.00000000	;
    RnonCO2[	422	]=	0.00000000	;
    RnonCO2[	423	]=	0.00000000	;
    RnonCO2[	424	]=	0.00000000	;
    RnonCO2[	425	]=	0.00000000	;
    RnonCO2[	426	]=	0.00000000	;
    RnonCO2[	427	]=	0.00000000	;
    RnonCO2[	428	]=	0.00000000	;
    RnonCO2[	429	]=	0.00000000	;
    RnonCO2[	430	]=	0.00000000	;
    RnonCO2[	431	]=	0.00000000	;
    RnonCO2[	432	]=	0.00000000	;
    RnonCO2[	433	]=	0.00000000	;
    RnonCO2[	434	]=	0.00000000	;
    RnonCO2[	435	]=	0.00000000	;
    RnonCO2[	436	]=	0.00000000	;
    RnonCO2[	437	]=	0.00000000	;
    RnonCO2[	438	]=	0.00000000	;
    RnonCO2[	439	]=	0.00000000	;
    RnonCO2[	440	]=	0.00000000	;
    RnonCO2[	441	]=	0.00000000	;
    RnonCO2[	442	]=	0.00000000	;
    RnonCO2[	443	]=	0.00000000	;
    RnonCO2[	444	]=	0.00000000	;
    RnonCO2[	445	]=	0.00000000	;
    RnonCO2[	446	]=	0.00000000	;
    RnonCO2[	447	]=	0.00000000	;
    RnonCO2[	448	]=	0.00000000	;
    RnonCO2[	449	]=	0.00000000	;
    RnonCO2[	450	]=	0.00000000	;
    RnonCO2[	451	]=	0.00000000	;
    RnonCO2[	452	]=	0.00000000	;
    RnonCO2[	453	]=	0.00000000	;
    RnonCO2[	454	]=	0.00000000	;
    RnonCO2[	455	]=	0.00000000	;
    RnonCO2[	456	]=	0.00000000	;
    RnonCO2[	457	]=	0.00000000	;
    RnonCO2[	458	]=	0.00000000	;
    RnonCO2[	459	]=	0.00000000	;
    RnonCO2[	460	]=	0.00000000	;
    RnonCO2[	461	]=	0.00000000	;
    RnonCO2[	462	]=	0.00000000	;
    RnonCO2[	463	]=	0.00000000	;
    RnonCO2[	464	]=	0.00000000	;
    RnonCO2[	465	]=	0.00000000	;
    RnonCO2[	466	]=	0.00000000	;
    RnonCO2[	467	]=	0.00000000	;
    RnonCO2[	468	]=	0.00000000	;
    RnonCO2[	469	]=	0.00000000	;
    RnonCO2[	470	]=	0.00000000	;
    RnonCO2[	471	]=	0.00000000	;
    RnonCO2[	472	]=	0.00000000	;
    RnonCO2[	473	]=	0.00000000	;
    RnonCO2[	474	]=	0.00000000	;
    RnonCO2[	475	]=	0.00000000	;
    RnonCO2[	476	]=	0.00000000	;
    RnonCO2[	477	]=	0.00000000	;
    RnonCO2[	478	]=	0.00000000	;
    RnonCO2[	479	]=	0.00000000	;
    RnonCO2[	480	]=	0.00000000	;
    RnonCO2[	481	]=	0.00000000	;
    RnonCO2[	482	]=	0.00000000	;
    RnonCO2[	483	]=	0.00000000	;
    RnonCO2[	484	]=	0.00000000	;
    RnonCO2[	485	]=	0.00000000	;
    RnonCO2[	486	]=	0.00000000	;
    RnonCO2[	487	]=	0.00000000	;
    RnonCO2[	488	]=	0.00000000	;
    RnonCO2[	489	]=	0.00000000	;
    RnonCO2[	490	]=	0.00000000	;
    RnonCO2[	491	]=	0.00000000	;
    RnonCO2[	492	]=	0.00000000	;
    RnonCO2[	493	]=	0.00000000	;
    RnonCO2[	494	]=	0.00000000	;
    RnonCO2[	495	]=	0.00000000	;
    RnonCO2[	496	]=	0.00000000	;
    RnonCO2[	497	]=	0.00000000	;
    RnonCO2[	498	]=	0.00000000	;
    RnonCO2[	499	]=	0.00000000	;
    RnonCO2[	500	]=	0.00000000	;
    RnonCO2[	501	]=	0.00000000	;
    RnonCO2[	502	]=	0.00000000	;
    RnonCO2[	503	]=	0.00000000	;
    RnonCO2[	504	]=	0.00000000	;
    RnonCO2[	505	]=	0.00000000	;
    RnonCO2[	506	]=	0.00000000	;
    RnonCO2[	507	]=	0.00000000	;
    RnonCO2[	508	]=	0.00000000	;
    RnonCO2[	509	]=	0.00000000	;
    RnonCO2[	510	]=	0.00000000	;
    RnonCO2[	511	]=	0.00000000	;
    RnonCO2[	512	]=	0.00000000	;
    RnonCO2[	513	]=	0.00000000	;
    RnonCO2[	514	]=	0.00000000	;
    RnonCO2[	515	]=	0.00000000	;
    RnonCO2[	516	]=	0.00000000	;
    RnonCO2[	517	]=	0.00000000	;
    RnonCO2[	518	]=	0.00000000	;
    RnonCO2[	519	]=	0.00000000	;
    RnonCO2[	520	]=	0.00000000	;
    RnonCO2[	521	]=	0.00000000	;
    RnonCO2[	522	]=	0.00000000	;
    RnonCO2[	523	]=	0.00000000	;
    RnonCO2[	524	]=	0.00000000	;
    RnonCO2[	525	]=	0.00000000	;
    RnonCO2[	526	]=	0.00000000	;
    RnonCO2[	527	]=	0.00000000	;
    RnonCO2[	528	]=	0.00000000	;
    RnonCO2[	529	]=	0.00000000	;
    RnonCO2[	530	]=	0.00000000	;
    RnonCO2[	531	]=	0.00000000	;
    RnonCO2[	532	]=	0.00000000	;
    RnonCO2[	533	]=	0.00000000	;
    RnonCO2[	534	]=	0.00000000	;
    RnonCO2[	535	]=	0.00000000	;
    RnonCO2[	536	]=	0.00000000	;
    RnonCO2[	537	]=	0.00000000	;
    RnonCO2[	538	]=	0.00000000	;
    RnonCO2[	539	]=	0.00000000	;
    RnonCO2[	540	]=	0.00000000	;
    RnonCO2[	541	]=	0.00000000	;
    RnonCO2[	542	]=	0.00000000	;
    RnonCO2[	543	]=	0.00000000	;
    RnonCO2[	544	]=	0.00000000	;
    RnonCO2[	545	]=	0.00000000	;
    RnonCO2[	546	]=	0.00000000	;
    RnonCO2[	547	]=	0.00000000	;
    RnonCO2[	548	]=	0.00000000	;
    RnonCO2[	549	]=	0.00000000	;
    RnonCO2[	550	]=	0.00000000	;
    RnonCO2[	551	]=	0.00000000	;
    RnonCO2[	552	]=	0.00000000	;
    RnonCO2[	553	]=	0.00000000	;
    RnonCO2[	554	]=	0.00000000	;
    RnonCO2[	555	]=	0.00000000	;
    RnonCO2[	556	]=	0.00000000	;
    RnonCO2[	557	]=	0.00000000	;
    RnonCO2[	558	]=	0.00000000	;
    RnonCO2[	559	]=	0.00000000	;
    RnonCO2[	560	]=	0.00000000	;
    RnonCO2[	561	]=	0.00000000	;
    RnonCO2[	562	]=	0.00000000	;
    RnonCO2[	563	]=	0.00000000	;
    RnonCO2[	564	]=	0.00000000	;
    RnonCO2[	565	]=	0.00000000	;
    RnonCO2[	566	]=	0.00000000	;
    RnonCO2[	567	]=	0.00000000	;
    RnonCO2[	568	]=	0.00000000	;
    RnonCO2[	569	]=	0.00000000	;
    RnonCO2[	570	]=	0.00000000	;
    RnonCO2[	571	]=	0.00000000	;
    RnonCO2[	572	]=	0.00000000	;
    RnonCO2[	573	]=	0.00000000	;
    RnonCO2[	574	]=	0.00000000	;
    RnonCO2[	575	]=	0.00000000	;
    RnonCO2[	576	]=	0.00000000	;
    RnonCO2[	577	]=	0.00000000	;
    RnonCO2[	578	]=	0.00000000	;
    RnonCO2[	579	]=	0.00000000	;
    RnonCO2[	580	]=	0.00000000	;
    RnonCO2[	581	]=	0.00000000	;
    RnonCO2[	582	]=	0.00000000	;
    RnonCO2[	583	]=	0.00000000	;
    RnonCO2[	584	]=	0.00000000	;
    RnonCO2[	585	]=	0.00000000	;
    RnonCO2[	586	]=	0.00000000	;
    RnonCO2[	587	]=	0.00000000	;
    RnonCO2[	588	]=	0.00000000	;
    RnonCO2[	589	]=	0.00000000	;
    RnonCO2[	590	]=	0.00000000	;
    RnonCO2[	591	]=	0.00000000	;
    RnonCO2[	592	]=	0.00000000	;
    RnonCO2[	593	]=	0.00000000	;
    RnonCO2[	594	]=	0.00000000	;
    RnonCO2[	595	]=	0.00000000	;
    RnonCO2[	596	]=	0.00000000	;
    RnonCO2[	597	]=	0.00000000	;
    RnonCO2[	598	]=	0.00000000	;
    RnonCO2[	599	]=	0.00000000	;
    RnonCO2[	600	]=	0.00000000	;
    RnonCO2[	601	]=	0.00000000	;
    RnonCO2[	602	]=	0.00000000	;
    RnonCO2[	603	]=	0.00000000	;
    RnonCO2[	604	]=	0.00000000	;
    RnonCO2[	605	]=	0.00000000	;
    RnonCO2[	606	]=	0.00000000	;
    RnonCO2[	607	]=	0.00000000	;
    RnonCO2[	608	]=	0.00000000	;
    RnonCO2[	609	]=	0.00000000	;
    RnonCO2[	610	]=	0.00000000	;
    RnonCO2[	611	]=	0.00000000	;
    RnonCO2[	612	]=	0.00000000	;
    RnonCO2[	613	]=	0.00000000	;
    RnonCO2[	614	]=	0.00000000	;
    RnonCO2[	615	]=	0.00000000	;
    RnonCO2[	616	]=	0.00000000	;
    RnonCO2[	617	]=	0.00000000	;
    RnonCO2[	618	]=	0.00000000	;
    RnonCO2[	619	]=	0.00000000	;
    RnonCO2[	620	]=	0.00000000	;
    RnonCO2[	621	]=	0.00000000	;
    RnonCO2[	622	]=	0.00000000	;
    RnonCO2[	623	]=	0.00000000	;
    RnonCO2[	624	]=	0.00000000	;
    RnonCO2[	625	]=	0.00000000	;
    RnonCO2[	626	]=	0.00000000	;
    RnonCO2[	627	]=	0.00000000	;
    RnonCO2[	628	]=	0.00000000	;
    RnonCO2[	629	]=	0.00000000	;
    RnonCO2[	630	]=	0.00000000	;
    RnonCO2[	631	]=	0.00000000	;
    RnonCO2[	632	]=	0.00000000	;
    RnonCO2[	633	]=	0.00000000	;
    RnonCO2[	634	]=	0.00000000	;
    RnonCO2[	635	]=	0.00000000	;
    RnonCO2[	636	]=	0.00000000	;
    RnonCO2[	637	]=	0.00000000	;
    RnonCO2[	638	]=	0.00000000	;
    RnonCO2[	639	]=	0.00000000	;
    RnonCO2[	640	]=	0.00000000	;
    RnonCO2[	641	]=	0.00000000	;
    RnonCO2[	642	]=	0.00000000	;
    RnonCO2[	643	]=	0.00000000	;
    RnonCO2[	644	]=	0.00000000	;
    RnonCO2[	645	]=	0.00000000	;
    RnonCO2[	646	]=	0.00000000	;
    RnonCO2[	647	]=	0.00000000	;
    RnonCO2[	648	]=	0.00000000	;
    RnonCO2[	649	]=	0.00000000	;
    RnonCO2[	650	]=	0.00000000	;
    RnonCO2[	651	]=	0.00000000	;
    RnonCO2[	652	]=	0.00000000	;
    RnonCO2[	653	]=	0.00000000	;
    RnonCO2[	654	]=	0.00000000	;
    RnonCO2[	655	]=	0.00000000	;
    RnonCO2[	656	]=	0.00000000	;
    RnonCO2[	657	]=	0.00000000	;
    RnonCO2[	658	]=	0.00000000	;
    RnonCO2[	659	]=	0.00000000	;
    RnonCO2[	660	]=	0.00000000	;
    RnonCO2[	661	]=	0.00000000	;
    RnonCO2[	662	]=	0.00000000	;
    RnonCO2[	663	]=	0.00000000	;
    RnonCO2[	664	]=	0.00000000	;
    RnonCO2[	665	]=	0.00000000	;
    RnonCO2[	666	]=	0.00000000	;
    RnonCO2[	667	]=	0.00000000	;
    RnonCO2[	668	]=	0.00000000	;
    RnonCO2[	669	]=	0.00000000	;
    RnonCO2[	670	]=	0.00000000	;
    RnonCO2[	671	]=	0.00000000	;
    RnonCO2[	672	]=	0.00000000	;
    RnonCO2[	673	]=	0.00000000	;
    RnonCO2[	674	]=	0.00000000	;
    RnonCO2[	675	]=	0.00000000	;
    RnonCO2[	676	]=	0.00000000	;
    RnonCO2[	677	]=	0.00000000	;
    RnonCO2[	678	]=	0.00000000	;
    RnonCO2[	679	]=	0.00000000	;
    RnonCO2[	680	]=	0.00000000	;
    RnonCO2[	681	]=	0.00000000	;
    RnonCO2[	682	]=	0.00000000	;
    RnonCO2[	683	]=	0.00000000	;
    RnonCO2[	684	]=	0.00000000	;
    RnonCO2[	685	]=	0.00000000	;
    RnonCO2[	686	]=	0.00000000	;
    RnonCO2[	687	]=	0.00000000	;
    RnonCO2[	688	]=	0.00000000	;
    RnonCO2[	689	]=	0.00000000	;
    RnonCO2[	690	]=	0.00000000	;
    RnonCO2[	691	]=	0.00000000	;
    RnonCO2[	692	]=	0.00000000	;
    RnonCO2[	693	]=	0.00000000	;
    RnonCO2[	694	]=	0.00000000	;
    RnonCO2[	695	]=	0.00000000	;
    RnonCO2[	696	]=	0.00000000	;
    RnonCO2[	697	]=	0.00000000	;
    RnonCO2[	698	]=	0.00000000	;
    RnonCO2[	699	]=	0.00000000	;
    RnonCO2[	700	]=	0.00000000	;
    RnonCO2[	701	]=	0.00000000	;
    RnonCO2[	702	]=	0.00000000	;
    RnonCO2[	703	]=	0.00000000	;
    RnonCO2[	704	]=	0.00000000	;
    RnonCO2[	705	]=	0.00000000	;
    RnonCO2[	706	]=	0.00000000	;
    RnonCO2[	707	]=	0.00000000	;
    RnonCO2[	708	]=	0.00000000	;
    RnonCO2[	709	]=	0.00000000	;
    RnonCO2[	710	]=	0.00000000	;
    RnonCO2[	711	]=	0.00000000	;
    RnonCO2[	712	]=	0.00000000	;
    RnonCO2[	713	]=	0.00000000	;
    RnonCO2[	714	]=	0.00000000	;
    RnonCO2[	715	]=	0.00000000	;
    RnonCO2[	716	]=	0.00000000	;
    RnonCO2[	717	]=	0.00000000	;
    RnonCO2[	718	]=	0.00000000	;
    RnonCO2[	719	]=	0.00000000	;
    RnonCO2[	720	]=	0.00000000	;
    RnonCO2[	721	]=	0.00000000	;
    RnonCO2[	722	]=	0.00000000	;
    RnonCO2[	723	]=	0.00000000	;
    RnonCO2[	724	]=	0.00000000	;
    RnonCO2[	725	]=	0.00000000	;
    RnonCO2[	726	]=	0.00000000	;
    RnonCO2[	727	]=	0.00000000	;
    RnonCO2[	728	]=	0.00000000	;
    RnonCO2[	729	]=	0.00000000	;
    RnonCO2[	730	]=	0.00000000	;
    RnonCO2[	731	]=	0.00000000	;
    RnonCO2[	732	]=	0.00000000	;
    RnonCO2[	733	]=	0.00000000	;
    RnonCO2[	734	]=	0.00000000	;
    RnonCO2[	734	]=	0.00000000	;
    
    for(int i = 736; i<tmax; i++)
    {

        RnonCO2[i] = RnonCO2[i-1];

    }
}

//Sets Aerosol Optical Depth (Must have 12 time steps per year to use
//From data.giss.nasa.gov/modelforce/strataer/reff_line.txt, downloaded 14th November 2017
void setAOD(void)
{
    for(int i=0; i<1020; i++)
    {
        AOD[i] = 0.0;
    }
    AOD[	1020	]=	0.0044	;
    AOD[	1021	]=	0.0042	;
    AOD[	1022	]=	0.0040	;
    AOD[	1023	]=	0.0039	;
    AOD[	1024	]=	0.0037	;
    AOD[	1025	]=	0.0036	;
    AOD[	1026	]=	0.0034	;
    AOD[	1027	]=	0.0034	;
    AOD[	1028	]=	0.0033	;
    AOD[	1029	]=	0.0032	;
    AOD[	1030	]=	0.0031	;
    AOD[	1031	]=	0.0030	;
    AOD[	1032	]=	0.0030	;
    AOD[	1033	]=	0.0029	;
    AOD[	1034	]=	0.0028	;
    AOD[	1035	]=	0.0027	;
    AOD[	1036	]=	0.0026	;
    AOD[	1037	]=	0.0026	;
    AOD[	1038	]=	0.0025	;
    AOD[	1039	]=	0.0024	;
    AOD[	1040	]=	0.0023	;
    AOD[	1041	]=	0.0022	;
    AOD[	1042	]=	0.0021	;
    AOD[	1043	]=	0.0020	;
    AOD[	1044	]=	0.0019	;
    AOD[	1045	]=	0.0018	;
    AOD[	1046	]=	0.0018	;
    AOD[	1047	]=	0.0017	;
    AOD[	1048	]=	0.0016	;
    AOD[	1049	]=	0.0015	;
    AOD[	1050	]=	0.0014	;
    AOD[	1051	]=	0.0013	;
    AOD[	1052	]=	0.0012	;
    AOD[	1053	]=	0.0011	;
    AOD[	1054	]=	0.0010	;
    AOD[	1055	]=	0.0010	;
    AOD[	1056	]=	0.0009	;
    AOD[	1057	]=	0.0008	;
    AOD[	1058	]=	0.0008	;
    AOD[	1059	]=	0.0007	;
    AOD[	1060	]=	0.0007	;
    AOD[	1061	]=	0.0006	;
    AOD[	1062	]=	0.0006	;
    AOD[	1063	]=	0.0005	;
    AOD[	1064	]=	0.0005	;
    AOD[	1065	]=	0.0005	;
    AOD[	1066	]=	0.0004	;
    AOD[	1067	]=	0.0004	;
    AOD[	1068	]=	0.0004	;
    AOD[	1069	]=	0.0004	;
    AOD[	1070	]=	0.0003	;
    AOD[	1071	]=	0.0003	;
    AOD[	1072	]=	0.0003	;
    AOD[	1073	]=	0.0003	;
    AOD[	1074	]=	0.0003	;
    AOD[	1075	]=	0.0003	;
    AOD[	1076	]=	0.0002	;
    AOD[	1077	]=	0.0002	;
    AOD[	1078	]=	0.0002	;
    AOD[	1079	]=	0.0002	;
    AOD[	1080	]=	0.0002	;
    AOD[	1081	]=	0.0002	;
    AOD[	1082	]=	0.0002	;
    AOD[	1083	]=	0.0002	;
    AOD[	1084	]=	0.0002	;
    AOD[	1085	]=	0.0002	;
    AOD[	1086	]=	0.0002	;
    AOD[	1087	]=	0.0002	;
    AOD[	1088	]=	0.0002	;
    AOD[	1089	]=	0.0029	;
    AOD[	1090	]=	0.0075	;
    AOD[	1091	]=	0.0122	;
    AOD[	1092	]=	0.0168	;
    AOD[	1093	]=	0.0214	;
    AOD[	1094	]=	0.0260	;
    AOD[	1095	]=	0.0306	;
    AOD[	1096	]=	0.0352	;
    AOD[	1097	]=	0.0399	;
    AOD[	1098	]=	0.0433	;
    AOD[	1099	]=	0.0456	;
    AOD[	1100	]=	0.0479	;
    AOD[	1101	]=	0.0501	;
    AOD[	1102	]=	0.0524	;
    AOD[	1103	]=	0.0547	;
    AOD[	1104	]=	0.0570	;
    AOD[	1105	]=	0.0593	;
    AOD[	1106	]=	0.0615	;
    AOD[	1107	]=	0.0638	;
    AOD[	1108	]=	0.0661	;
    AOD[	1109	]=	0.0684	;
    AOD[	1110	]=	0.0675	;
    AOD[	1111	]=	0.0636	;
    AOD[	1112	]=	0.0596	;
    AOD[	1113	]=	0.0557	;
    AOD[	1114	]=	0.0517	;
    AOD[	1115	]=	0.0478	;
    AOD[	1116	]=	0.0440	;
    AOD[	1117	]=	0.0405	;
    AOD[	1118	]=	0.0372	;
    AOD[	1119	]=	0.0343	;
    AOD[	1120	]=	0.0315	;
    AOD[	1121	]=	0.0290	;
    AOD[	1122	]=	0.0267	;
    AOD[	1123	]=	0.0246	;
    AOD[	1124	]=	0.0226	;
    AOD[	1125	]=	0.0208	;
    AOD[	1126	]=	0.0192	;
    AOD[	1127	]=	0.0176	;
    AOD[	1128	]=	0.0162	;
    AOD[	1129	]=	0.0150	;
    AOD[	1130	]=	0.0139	;
    AOD[	1131	]=	0.0130	;
    AOD[	1132	]=	0.0120	;
    AOD[	1133	]=	0.0110	;
    AOD[	1134	]=	0.0103	;
    AOD[	1135	]=	0.0097	;
    AOD[	1136	]=	0.0091	;
    AOD[	1137	]=	0.0086	;
    AOD[	1138	]=	0.0080	;
    AOD[	1139	]=	0.0074	;
    AOD[	1140	]=	0.0069	;
    AOD[	1141	]=	0.0064	;
    AOD[	1142	]=	0.0059	;
    AOD[	1143	]=	0.0054	;
    AOD[	1144	]=	0.0050	;
    AOD[	1145	]=	0.0046	;
    AOD[	1146	]=	0.0042	;
    AOD[	1147	]=	0.0039	;
    AOD[	1148	]=	0.0036	;
    AOD[	1149	]=	0.0033	;
    AOD[	1150	]=	0.0031	;
    AOD[	1151	]=	0.0028	;
    AOD[	1152	]=	0.0026	;
    AOD[	1153	]=	0.0024	;
    AOD[	1154	]=	0.0022	;
    AOD[	1155	]=	0.0020	;
    AOD[	1156	]=	0.0019	;
    AOD[	1157	]=	0.0017	;
    AOD[	1158	]=	0.0016	;
    AOD[	1159	]=	0.0027	;
    AOD[	1160	]=	0.0040	;
    AOD[	1161	]=	0.0053	;
    AOD[	1162	]=	0.0066	;
    AOD[	1163	]=	0.0079	;
    AOD[	1164	]=	0.0092	;
    AOD[	1165	]=	0.0105	;
    AOD[	1166	]=	0.0118	;
    AOD[	1167	]=	0.0132	;
    AOD[	1168	]=	0.0145	;
    AOD[	1169	]=	0.0158	;
    AOD[	1170	]=	0.0162	;
    AOD[	1171	]=	0.0156	;
    AOD[	1172	]=	0.0151	;
    AOD[	1173	]=	0.0145	;
    AOD[	1174	]=	0.0140	;
    AOD[	1175	]=	0.0135	;
    AOD[	1176	]=	0.0129	;
    AOD[	1177	]=	0.0124	;
    AOD[	1178	]=	0.0118	;
    AOD[	1179	]=	0.0113	;
    AOD[	1180	]=	0.0107	;
    AOD[	1181	]=	0.0102	;
    AOD[	1182	]=	0.0097	;
    AOD[	1183	]=	0.0092	;
    AOD[	1184	]=	0.0087	;
    AOD[	1185	]=	0.0082	;
    AOD[	1186	]=	0.0078	;
    AOD[	1187	]=	0.0073	;
    AOD[	1188	]=	0.0068	;
    AOD[	1189	]=	0.0063	;
    AOD[	1190	]=	0.0058	;
    AOD[	1191	]=	0.0054	;
    AOD[	1192	]=	0.0050	;
    AOD[	1193	]=	0.0046	;
    AOD[	1194	]=	0.0042	;
    AOD[	1195	]=	0.0039	;
    AOD[	1196	]=	0.0037	;
    AOD[	1197	]=	0.0035	;
    AOD[	1198	]=	0.0033	;
    AOD[	1199	]=	0.0031	;
    AOD[	1200	]=	0.0029	;
    AOD[	1201	]=	0.0027	;
    AOD[	1202	]=	0.0025	;
    AOD[	1203	]=	0.0023	;
    AOD[	1204	]=	0.0022	;
    AOD[	1205	]=	0.0020	;
    AOD[	1206	]=	0.0019	;
    AOD[	1207	]=	0.0017	;
    AOD[	1208	]=	0.0016	;
    AOD[	1209	]=	0.0015	;
    AOD[	1210	]=	0.0014	;
    AOD[	1211	]=	0.0013	;
    AOD[	1212	]=	0.0012	;
    AOD[	1213	]=	0.0011	;
    AOD[	1214	]=	0.0010	;
    AOD[	1215	]=	0.0010	;
    AOD[	1216	]=	0.0009	;
    AOD[	1217	]=	0.0008	;
    AOD[	1218	]=	0.0008	;
    AOD[	1219	]=	0.0007	;
    AOD[	1220	]=	0.0007	;
    AOD[	1221	]=	0.0006	;
    AOD[	1222	]=	0.0006	;
    AOD[	1223	]=	0.0005	;
    AOD[	1224	]=	0.0005	;
    AOD[	1225	]=	0.0005	;
    AOD[	1226	]=	0.0004	;
    AOD[	1227	]=	0.0004	;
    AOD[	1228	]=	0.0004	;
    AOD[	1229	]=	0.0004	;
    AOD[	1230	]=	0.0003	;
    AOD[	1231	]=	0.0003	;
    AOD[	1232	]=	0.0003	;
    AOD[	1233	]=	0.0003	;
    AOD[	1234	]=	0.0003	;
    AOD[	1235	]=	0.0003	;
    AOD[	1236	]=	0.0002	;
    AOD[	1237	]=	0.0002	;
    AOD[	1238	]=	0.0002	;
    AOD[	1239	]=	0.0002	;
    AOD[	1240	]=	0.0002	;
    AOD[	1241	]=	0.0002	;
    AOD[	1242	]=	0.0002	;
    AOD[	1243	]=	0.0002	;
    AOD[	1244	]=	0.0002	;
    AOD[	1245	]=	0.0002	;
    AOD[	1246	]=	0.0002	;
    AOD[	1247	]=	0.0003	;
    AOD[	1248	]=	0.0003	;
    AOD[	1249	]=	0.0004	;
    AOD[	1250	]=	0.0005	;
    AOD[	1251	]=	0.0005	;
    AOD[	1252	]=	0.0006	;
    AOD[	1253	]=	0.0007	;
    AOD[	1254	]=	0.0007	;
    AOD[	1255	]=	0.0007	;
    AOD[	1256	]=	0.0007	;
    AOD[	1257	]=	0.0007	;
    AOD[	1258	]=	0.0007	;
    AOD[	1259	]=	0.0007	;
    AOD[	1260	]=	0.0007	;
    AOD[	1261	]=	0.0006	;
    AOD[	1262	]=	0.0006	;
    AOD[	1263	]=	0.0006	;
    AOD[	1264	]=	0.0006	;
    AOD[	1265	]=	0.0006	;
    AOD[	1266	]=	0.0006	;
    AOD[	1267	]=	0.0006	;
    AOD[	1268	]=	0.0006	;
    AOD[	1269	]=	0.0006	;
    AOD[	1270	]=	0.0006	;
    AOD[	1271	]=	0.0006	;
    AOD[	1272	]=	0.0006	;
    AOD[	1273	]=	0.0006	;
    AOD[	1274	]=	0.0005	;
    AOD[	1275	]=	0.0005	;
    AOD[	1276	]=	0.0005	;
    AOD[	1277	]=	0.0005	;
    AOD[	1278	]=	0.0005	;
    AOD[	1279	]=	0.0006	;
    AOD[	1280	]=	0.0006	;
    AOD[	1281	]=	0.0007	;
    AOD[	1282	]=	0.0007	;
    AOD[	1283	]=	0.0008	;
    AOD[	1284	]=	0.0008	;
    AOD[	1285	]=	0.0009	;
    AOD[	1286	]=	0.0009	;
    AOD[	1287	]=	0.0010	;
    AOD[	1288	]=	0.0010	;
    AOD[	1289	]=	0.0010	;
    AOD[	1290	]=	0.0012	;
    AOD[	1291	]=	0.0014	;
    AOD[	1292	]=	0.0016	;
    AOD[	1293	]=	0.0018	;
    AOD[	1294	]=	0.0020	;
    AOD[	1295	]=	0.0022	;
    AOD[	1296	]=	0.0024	;
    AOD[	1297	]=	0.0026	;
    AOD[	1298	]=	0.0028	;
    AOD[	1299	]=	0.0030	;
    AOD[	1300	]=	0.0032	;
    AOD[	1301	]=	0.0034	;
    AOD[	1302	]=	0.0034	;
    AOD[	1303	]=	0.0033	;
    AOD[	1304	]=	0.0032	;
    AOD[	1305	]=	0.0031	;
    AOD[	1306	]=	0.0029	;
    AOD[	1307	]=	0.0028	;
    AOD[	1308	]=	0.0027	;
    AOD[	1309	]=	0.0025	;
    AOD[	1310	]=	0.0024	;
    AOD[	1311	]=	0.0023	;
    AOD[	1312	]=	0.0022	;
    AOD[	1313	]=	0.0020	;
    AOD[	1314	]=	0.0019	;
    AOD[	1315	]=	0.0017	;
    AOD[	1316	]=	0.0016	;
    AOD[	1317	]=	0.0015	;
    AOD[	1318	]=	0.0014	;
    AOD[	1319	]=	0.0013	;
    AOD[	1320	]=	0.0012	;
    AOD[	1321	]=	0.0011	;
    AOD[	1322	]=	0.0010	;
    AOD[	1323	]=	0.0009	;
    AOD[	1324	]=	0.0009	;
    AOD[	1325	]=	0.0008	;
    AOD[	1326	]=	0.0008	;
    AOD[	1327	]=	0.0007	;
    AOD[	1328	]=	0.0010	;
    AOD[	1329	]=	0.0017	;
    AOD[	1330	]=	0.0024	;
    AOD[	1331	]=	0.0030	;
    AOD[	1332	]=	0.0037	;
    AOD[	1333	]=	0.0044	;
    AOD[	1334	]=	0.0051	;
    AOD[	1335	]=	0.0058	;
    AOD[	1336	]=	0.0065	;
    AOD[	1337	]=	0.0072	;
    AOD[	1338	]=	0.0074	;
    AOD[	1339	]=	0.0072	;
    AOD[	1340	]=	0.0070	;
    AOD[	1341	]=	0.0069	;
    AOD[	1342	]=	0.0067	;
    AOD[	1343	]=	0.0065	;
    AOD[	1344	]=	0.0063	;
    AOD[	1345	]=	0.0061	;
    AOD[	1346	]=	0.0059	;
    AOD[	1347	]=	0.0057	;
    AOD[	1348	]=	0.0056	;
    AOD[	1349	]=	0.0054	;
    AOD[	1350	]=	0.0052	;
    AOD[	1351	]=	0.0050	;
    AOD[	1352	]=	0.0048	;
    AOD[	1353	]=	0.0046	;
    AOD[	1354	]=	0.0044	;
    AOD[	1355	]=	0.0042	;
    AOD[	1356	]=	0.0040	;
    AOD[	1357	]=	0.0039	;
    AOD[	1358	]=	0.0037	;
    AOD[	1359	]=	0.0035	;
    AOD[	1360	]=	0.0033	;
    AOD[	1361	]=	0.0031	;
    AOD[	1362	]=	0.0029	;
    AOD[	1363	]=	0.0029	;
    AOD[	1364	]=	0.0028	;
    AOD[	1365	]=	0.0027	;
    AOD[	1366	]=	0.0026	;
    AOD[	1367	]=	0.0025	;
    AOD[	1368	]=	0.0024	;
    AOD[	1369	]=	0.0024	;
    AOD[	1370	]=	0.0023	;
    AOD[	1371	]=	0.0022	;
    AOD[	1372	]=	0.0021	;
    AOD[	1373	]=	0.0020	;
    AOD[	1374	]=	0.0020	;
    AOD[	1375	]=	0.0019	;
    AOD[	1376	]=	0.0018	;
    AOD[	1377	]=	0.0017	;
    AOD[	1378	]=	0.0016	;
    AOD[	1379	]=	0.0016	;
    AOD[	1380	]=	0.0015	;
    AOD[	1381	]=	0.0014	;
    AOD[	1382	]=	0.0014	;
    AOD[	1383	]=	0.0013	;
    AOD[	1384	]=	0.0012	;
    AOD[	1385	]=	0.0011	;
    AOD[	1386	]=	0.0011	;
    AOD[	1387	]=	0.0010	;
    AOD[	1388	]=	0.0010	;
    AOD[	1389	]=	0.0009	;
    AOD[	1390	]=	0.0009	;
    AOD[	1391	]=	0.0009	;
    AOD[	1392	]=	0.0008	;
    AOD[	1393	]=	0.0008	;
    AOD[	1394	]=	0.0008	;
    AOD[	1395	]=	0.0007	;
    AOD[	1396	]=	0.0007	;
    AOD[	1397	]=	0.0006	;
    AOD[	1398	]=	0.0006	;
    AOD[	1399	]=	0.0006	;
    AOD[	1400	]=	0.0006	;
    AOD[	1401	]=	0.0006	;
    AOD[	1402	]=	0.0006	;
    AOD[	1403	]=	0.0006	;
    AOD[	1404	]=	0.0006	;
    AOD[	1405	]=	0.0006	;
    AOD[	1406	]=	0.0006	;
    AOD[	1407	]=	0.0006	;
    AOD[	1408	]=	0.0006	;
    AOD[	1409	]=	0.0006	;
    AOD[	1410	]=	0.0006	;
    AOD[	1411	]=	0.0006	;
    AOD[	1412	]=	0.0006	;
    AOD[	1413	]=	0.0005	;
    AOD[	1414	]=	0.0005	;
    AOD[	1415	]=	0.0005	;
    AOD[	1416	]=	0.0291	;
    AOD[	1417	]=	0.0328	;
    AOD[	1418	]=	0.0302	;
    AOD[	1419	]=	0.0278	;
    AOD[	1420	]=	0.0255	;
    AOD[	1421	]=	0.0235	;
    AOD[	1422	]=	0.0216	;
    AOD[	1423	]=	0.0197	;
    AOD[	1424	]=	0.0415	;
    AOD[	1425	]=	0.0781	;
    AOD[	1426	]=	0.1060	;
    AOD[	1427	]=	0.1317	;
    AOD[	1428	]=	0.1518	;
    AOD[	1429	]=	0.1523	;
    AOD[	1430	]=	0.1631	;
    AOD[	1431	]=	0.1644	;
    AOD[	1432	]=	0.1570	;
    AOD[	1433	]=	0.1525	;
    AOD[	1434	]=	0.1516	;
    AOD[	1435	]=	0.1392	;
    AOD[	1436	]=	0.1350	;
    AOD[	1437	]=	0.1255	;
    AOD[	1438	]=	0.1149	;
    AOD[	1439	]=	0.1076	;
    AOD[	1440	]=	0.1009	;
    AOD[	1441	]=	0.0872	;
    AOD[	1442	]=	0.0817	;
    AOD[	1443	]=	0.0727	;
    AOD[	1444	]=	0.0668	;
    AOD[	1445	]=	0.0615	;
    AOD[	1446	]=	0.0577	;
    AOD[	1447	]=	0.0530	;
    AOD[	1448	]=	0.0490	;
    AOD[	1449	]=	0.0468	;
    AOD[	1450	]=	0.0433	;
    AOD[	1451	]=	0.0413	;
    AOD[	1452	]=	0.0385	;
    AOD[	1453	]=	0.0343	;
    AOD[	1454	]=	0.0313	;
    AOD[	1455	]=	0.0291	;
    AOD[	1456	]=	0.0261	;
    AOD[	1457	]=	0.0239	;
    AOD[	1458	]=	0.0225	;
    AOD[	1459	]=	0.0203	;
    AOD[	1460	]=	0.0197	;
    AOD[	1461	]=	0.0692	;
    AOD[	1462	]=	0.0637	;
    AOD[	1463	]=	0.0586	;
    AOD[	1464	]=	0.0539	;
    AOD[	1465	]=	0.0496	;
    AOD[	1466	]=	0.0457	;
    AOD[	1467	]=	0.0420	;
    AOD[	1468	]=	0.0387	;
    AOD[	1469	]=	0.0356	;
    AOD[	1470	]=	0.0328	;
    AOD[	1471	]=	0.0345	;
    AOD[	1472	]=	0.0317	;
    AOD[	1473	]=	0.0292	;
    AOD[	1474	]=	0.0269	;
    AOD[	1475	]=	0.0247	;
    AOD[	1476	]=	0.0228	;
    AOD[	1477	]=	0.0210	;
    AOD[	1478	]=	0.0193	;
    AOD[	1479	]=	0.0178	;
    AOD[	1480	]=	0.0163	;
    AOD[	1481	]=	0.0110	;
    AOD[	1482	]=	0.0130	;
    AOD[	1483	]=	0.0149	;
    AOD[	1484	]=	0.0222	;
    AOD[	1485	]=	0.0296	;
    AOD[	1486	]=	0.0368	;
    AOD[	1487	]=	0.0376	;
    AOD[	1488	]=	0.0385	;
    AOD[	1489	]=	0.0393	;
    AOD[	1490	]=	0.0361	;
    AOD[	1491	]=	0.0332	;
    AOD[	1492	]=	0.0306	;
    AOD[	1493	]=	0.0282	;
    AOD[	1494	]=	0.0259	;
    AOD[	1495	]=	0.0238	;
    AOD[	1496	]=	0.0219	;
    AOD[	1497	]=	0.0202	;
    AOD[	1498	]=	0.0191	;
    AOD[	1499	]=	0.0256	;
    AOD[	1500	]=	0.0321	;
    AOD[	1501	]=	0.0387	;
    AOD[	1502	]=	0.0418	;
    AOD[	1503	]=	0.0450	;
    AOD[	1504	]=	0.0482	;
    AOD[	1505	]=	0.0443	;
    AOD[	1506	]=	0.0408	;
    AOD[	1507	]=	0.0375	;
    AOD[	1508	]=	0.0345	;
    AOD[	1509	]=	0.0331	;
    AOD[	1510	]=	0.0359	;
    AOD[	1511	]=	0.0376	;
    AOD[	1512	]=	0.0398	;
    AOD[	1513	]=	0.0422	;
    AOD[	1514	]=	0.0388	;
    AOD[	1515	]=	0.0357	;
    AOD[	1516	]=	0.0328	;
    AOD[	1517	]=	0.0303	;
    AOD[	1518	]=	0.0278	;
    AOD[	1519	]=	0.0256	;
    AOD[	1520	]=	0.0236	;
    AOD[	1521	]=	0.0217	;
    AOD[	1522	]=	0.0206	;
    AOD[	1523	]=	0.0208	;
    AOD[	1524	]=	0.0210	;
    AOD[	1525	]=	0.0212	;
    AOD[	1526	]=	0.0234	;
    AOD[	1527	]=	0.0255	;
    AOD[	1528	]=	0.0277	;
    AOD[	1529	]=	0.0255	;
    AOD[	1530	]=	0.0235	;
    AOD[	1531	]=	0.0216	;
    AOD[	1532	]=	0.0199	;
    AOD[	1533	]=	0.0183	;
    AOD[	1534	]=	0.0168	;
    AOD[	1535	]=	0.0155	;
    AOD[	1536	]=	0.0143	;
    AOD[	1537	]=	0.0131	;
    AOD[	1538	]=	0.0121	;
    AOD[	1539	]=	0.0111	;
    AOD[	1540	]=	0.0102	;
    AOD[	1541	]=	0.0095	;
    AOD[	1542	]=	0.0087	;
    AOD[	1543	]=	0.0080	;
    AOD[	1544	]=	0.0074	;
    AOD[	1545	]=	0.0068	;
    AOD[	1546	]=	0.0063	;
    AOD[	1547	]=	0.0058	;
    AOD[	1548	]=	0.0053	;
    AOD[	1549	]=	0.0049	;
    AOD[	1550	]=	0.0045	;
    AOD[	1551	]=	0.0042	;
    AOD[	1552	]=	0.0038	;
    AOD[	1553	]=	0.0035	;
    AOD[	1554	]=	0.0033	;
    AOD[	1555	]=	0.0030	;
    AOD[	1556	]=	0.0028	;
    AOD[	1557	]=	0.0025	;
    AOD[	1558	]=	0.0024	;
    AOD[	1559	]=	0.0022	;
    AOD[	1560	]=	0.0020	;
    AOD[	1561	]=	0.0019	;
    AOD[	1562	]=	0.0017	;
    AOD[	1563	]=	0.0016	;
    AOD[	1564	]=	0.0015	;
    AOD[	1565	]=	0.0014	;
    AOD[	1566	]=	0.0012	;
    AOD[	1567	]=	0.0012	;
    AOD[	1568	]=	0.0011	;
    AOD[	1569	]=	0.0010	;
    AOD[	1570	]=	0.0009	;
    AOD[	1571	]=	0.0009	;
    AOD[	1572	]=	0.0063	;
    AOD[	1573	]=	0.0058	;
    AOD[	1574	]=	0.0071	;
    AOD[	1575	]=	0.0134	;
    AOD[	1576	]=	0.0197	;
    AOD[	1577	]=	0.0225	;
    AOD[	1578	]=	0.0252	;
    AOD[	1579	]=	0.0280	;
    AOD[	1580	]=	0.0258	;
    AOD[	1581	]=	0.0237	;
    AOD[	1582	]=	0.0218	;
    AOD[	1583	]=	0.0201	;
    AOD[	1584	]=	0.0185	;
    AOD[	1585	]=	0.0171	;
    AOD[	1586	]=	0.0157	;
    AOD[	1587	]=	0.0144	;
    AOD[	1588	]=	0.0138	;
    AOD[	1589	]=	0.0143	;
    AOD[	1590	]=	0.0148	;
    AOD[	1591	]=	0.0153	;
    AOD[	1592	]=	0.0174	;
    AOD[	1593	]=	0.0195	;
    AOD[	1594	]=	0.0216	;
    AOD[	1595	]=	0.0199	;
    AOD[	1596	]=	0.0183	;
    AOD[	1597	]=	0.0169	;
    AOD[	1598	]=	0.0155	;
    AOD[	1599	]=	0.0143	;
    AOD[	1600	]=	0.0132	;
    AOD[	1601	]=	0.0121	;
    AOD[	1602	]=	0.0112	;
    AOD[	1603	]=	0.0103	;
    AOD[	1604	]=	0.0095	;
    AOD[	1605	]=	0.0087	;
    AOD[	1606	]=	0.0080	;
    AOD[	1607	]=	0.0074	;
    AOD[	1608	]=	0.0069	;
    AOD[	1609	]=	0.0063	;
    AOD[	1610	]=	0.0058	;
    AOD[	1611	]=	0.0054	;
    AOD[	1612	]=	0.0049	;
    AOD[	1613	]=	0.0046	;
    AOD[	1614	]=	0.0042	;
    AOD[	1615	]=	0.0039	;
    AOD[	1616	]=	0.0036	;
    AOD[	1617	]=	0.0033	;
    AOD[	1618	]=	0.0030	;
    AOD[	1619	]=	0.0028	;
    AOD[	1620	]=	0.0026	;
    AOD[	1621	]=	0.0024	;
    AOD[	1622	]=	0.0022	;
    AOD[	1623	]=	0.0021	;
    AOD[	1624	]=	0.0019	;
    AOD[	1625	]=	0.0018	;
    AOD[	1626	]=	0.0016	;
    AOD[	1627	]=	0.0015	;
    AOD[	1628	]=	0.0014	;
    AOD[	1629	]=	0.0013	;
    AOD[	1630	]=	0.0012	;
    AOD[	1631	]=	0.0011	;
    AOD[	1632	]=	0.0010	;
    AOD[	1633	]=	0.0010	;
    AOD[	1634	]=	0.0009	;
    AOD[	1635	]=	0.0009	;
    AOD[	1636	]=	0.0008	;
    AOD[	1637	]=	0.0007	;
    AOD[	1638	]=	0.0007	;
    AOD[	1639	]=	0.0007	;
    AOD[	1640	]=	0.0006	;
    AOD[	1641	]=	0.0006	;
    AOD[	1642	]=	0.0005	;
    AOD[	1643	]=	0.0005	;
    AOD[	1644	]=	0.0029	;
    AOD[	1645	]=	0.0043	;
    AOD[	1646	]=	0.0066	;
    AOD[	1647	]=	0.0089	;
    AOD[	1648	]=	0.0112	;
    AOD[	1649	]=	0.0103	;
    AOD[	1650	]=	0.0095	;
    AOD[	1651	]=	0.0099	;
    AOD[	1652	]=	0.0208	;
    AOD[	1653	]=	0.0391	;
    AOD[	1654	]=	0.0530	;
    AOD[	1655	]=	0.0659	;
    AOD[	1656	]=	0.0760	;
    AOD[	1657	]=	0.0762	;
    AOD[	1658	]=	0.0816	;
    AOD[	1659	]=	0.0822	;
    AOD[	1660	]=	0.0786	;
    AOD[	1661	]=	0.0763	;
    AOD[	1662	]=	0.0759	;
    AOD[	1663	]=	0.0696	;
    AOD[	1664	]=	0.0676	;
    AOD[	1665	]=	0.0628	;
    AOD[	1666	]=	0.0575	;
    AOD[	1667	]=	0.0539	;
    AOD[	1668	]=	0.0505	;
    AOD[	1669	]=	0.0437	;
    AOD[	1670	]=	0.0409	;
    AOD[	1671	]=	0.0364	;
    AOD[	1672	]=	0.0335	;
    AOD[	1673	]=	0.0308	;
    AOD[	1674	]=	0.0289	;
    AOD[	1675	]=	0.0265	;
    AOD[	1676	]=	0.0245	;
    AOD[	1677	]=	0.0234	;
    AOD[	1678	]=	0.0217	;
    AOD[	1679	]=	0.0207	;
    AOD[	1680	]=	0.0193	;
    AOD[	1681	]=	0.0172	;
    AOD[	1682	]=	0.0157	;
    AOD[	1683	]=	0.0146	;
    AOD[	1684	]=	0.0131	;
    AOD[	1685	]=	0.0120	;
    AOD[	1686	]=	0.0113	;
    AOD[	1687	]=	0.0102	;
    AOD[	1688	]=	0.0099	;
    AOD[	1689	]=	0.0095	;
    AOD[	1690	]=	0.0089	;
    AOD[	1691	]=	0.0092	;
    AOD[	1692	]=	0.0088	;
    AOD[	1693	]=	0.0088	;
    AOD[	1694	]=	0.0067	;
    AOD[	1695	]=	0.0065	;
    AOD[	1696	]=	0.0061	;
    AOD[	1697	]=	0.0056	;
    AOD[	1698	]=	0.0055	;
    AOD[	1699	]=	0.0091	;
    AOD[	1700	]=	0.0084	;
    AOD[	1701	]=	0.0078	;
    AOD[	1702	]=	0.0071	;
    AOD[	1703	]=	0.0066	;
    AOD[	1704	]=	0.0061	;
    AOD[	1705	]=	0.0058	;
    AOD[	1706	]=	0.0075	;
    AOD[	1707	]=	0.0091	;
    AOD[	1708	]=	0.0108	;
    AOD[	1709	]=	0.0108	;
    AOD[	1710	]=	0.0109	;
    AOD[	1711	]=	0.0109	;
    AOD[	1712	]=	0.0101	;
    AOD[	1713	]=	0.0093	;
    AOD[	1714	]=	0.0086	;
    AOD[	1715	]=	0.0105	;
    AOD[	1716	]=	0.0126	;
    AOD[	1717	]=	0.0147	;
    AOD[	1718	]=	0.0135	;
    AOD[	1719	]=	0.0125	;
    AOD[	1720	]=	0.0115	;
    AOD[	1721	]=	0.0106	;
    AOD[	1722	]=	0.0097	;
    AOD[	1723	]=	0.0089	;
    AOD[	1724	]=	0.0082	;
    AOD[	1725	]=	0.0076	;
    AOD[	1726	]=	0.0070	;
    AOD[	1727	]=	0.0065	;
    AOD[	1728	]=	0.0059	;
    AOD[	1729	]=	0.0056	;
    AOD[	1730	]=	0.0051	;
    AOD[	1731	]=	0.0047	;
    AOD[	1732	]=	0.0044	;
    AOD[	1733	]=	0.0040	;
    AOD[	1734	]=	0.0037	;
    AOD[	1735	]=	0.0034	;
    AOD[	1736	]=	0.0032	;
    AOD[	1737	]=	0.0029	;
    AOD[	1738	]=	0.0027	;
    AOD[	1739	]=	0.0025	;
    AOD[	1740	]=	0.0028	;
    AOD[	1741	]=	0.0041	;
    AOD[	1742	]=	0.0040	;
    AOD[	1743	]=	0.0039	;
    AOD[	1744	]=	0.0037	;
    AOD[	1745	]=	0.0034	;
    AOD[	1746	]=	0.0031	;
    AOD[	1747	]=	0.0029	;
    AOD[	1748	]=	0.0027	;
    AOD[	1749	]=	0.0025	;
    AOD[	1750	]=	0.0023	;
    AOD[	1751	]=	0.0021	;
    AOD[	1752	]=	0.0020	;
    AOD[	1753	]=	0.0023	;
    AOD[	1754	]=	0.0021	;
    AOD[	1755	]=	0.0020	;
    AOD[	1756	]=	0.0019	;
    AOD[	1757	]=	0.0017	;
    AOD[	1758	]=	0.0016	;
    AOD[	1759	]=	0.0015	;
    AOD[	1760	]=	0.0014	;
    AOD[	1761	]=	0.0013	;
    AOD[	1762	]=	0.0012	;
    AOD[	1763	]=	0.0011	;
    AOD[	1764	]=	0.0008	;
    AOD[	1765	]=	0.0017	;
    AOD[	1766	]=	0.0019	;
    AOD[	1767	]=	0.0021	;
    AOD[	1768	]=	0.0023	;
    AOD[	1769	]=	0.0149	;
    AOD[	1770	]=	0.0275	;
    AOD[	1771	]=	0.0401	;
    AOD[	1772	]=	0.0378	;
    AOD[	1773	]=	0.0357	;
    AOD[	1774	]=	0.0338	;
    AOD[	1775	]=	0.0330	;
    AOD[	1776	]=	0.0324	;
    AOD[	1777	]=	0.0319	;
    AOD[	1778	]=	0.0294	;
    AOD[	1779	]=	0.0271	;
    AOD[	1780	]=	0.0276	;
    AOD[	1781	]=	0.0254	;
    AOD[	1782	]=	0.0234	;
    AOD[	1783	]=	0.0215	;
    AOD[	1784	]=	0.0198	;
    AOD[	1785	]=	0.0182	;
    AOD[	1786	]=	0.0168	;
    AOD[	1787	]=	0.0155	;
    AOD[	1788	]=	0.0142	;
    AOD[	1789	]=	0.0134	;
    AOD[	1790	]=	0.0126	;
    AOD[	1791	]=	0.0117	;
    AOD[	1792	]=	0.0110	;
    AOD[	1793	]=	0.0101	;
    AOD[	1794	]=	0.0093	;
    AOD[	1795	]=	0.0086	;
    AOD[	1796	]=	0.0079	;
    AOD[	1797	]=	0.0073	;
    AOD[	1798	]=	0.0067	;
    AOD[	1799	]=	0.0062	;
    AOD[	1800	]=	0.0057	;
    AOD[	1801	]=	0.0052	;
    AOD[	1802	]=	0.0049	;
    AOD[	1803	]=	0.0045	;
    AOD[	1804	]=	0.0041	;
    AOD[	1805	]=	0.0038	;
    AOD[	1806	]=	0.0035	;
    AOD[	1807	]=	0.0033	;
    AOD[	1808	]=	0.0030	;
    AOD[	1809	]=	0.0028	;
    AOD[	1810	]=	0.0033	;
    AOD[	1811	]=	0.0032	;
    AOD[	1812	]=	0.0036	;
    AOD[	1813	]=	0.0033	;
    AOD[	1814	]=	0.0030	;
    AOD[	1815	]=	0.0028	;
    AOD[	1816	]=	0.0026	;
    AOD[	1817	]=	0.0024	;
    AOD[	1818	]=	0.0025	;
    AOD[	1819	]=	0.0023	;
    AOD[	1820	]=	0.0026	;
    AOD[	1821	]=	0.0024	;
    AOD[	1822	]=	0.0027	;
    AOD[	1823	]=	0.0027	;
    AOD[	1824	]=	0.0034	;
    AOD[	1825	]=	0.0031	;
    AOD[	1826	]=	0.0028	;
    AOD[	1827	]=	0.0026	;
    AOD[	1828	]=	0.0024	;
    AOD[	1829	]=	0.0022	;
    AOD[	1830	]=	0.0020	;
    AOD[	1831	]=	0.0019	;
    AOD[	1832	]=	0.0018	;
    AOD[	1833	]=	0.0016	;
    AOD[	1834	]=	0.0015	;
    AOD[	1835	]=	0.0014	;
    AOD[	1836	]=	0.0020	;
    AOD[	1837	]=	0.0020	;
    AOD[	1838	]=	0.0027	;
    AOD[	1839	]=	0.0025	;
    AOD[	1840	]=	0.0023	;
    AOD[	1841	]=	0.0021	;
    AOD[	1842	]=	0.0019	;
    AOD[	1843	]=	0.0018	;
    AOD[	1844	]=	0.0017	;
    AOD[	1845	]=	0.0016	;
    AOD[	1846	]=	0.0014	;
    AOD[	1847	]=	0.0014	;
    AOD[	1848	]=	0.0014	;
    AOD[	1849	]=	0.0020	;
    AOD[	1850	]=	0.0027	;
    AOD[	1851	]=	0.0027	;
    AOD[	1852	]=	0.0025	;
    AOD[	1853	]=	0.0023	;
    AOD[	1854	]=	0.0021	;
    AOD[	1855	]=	0.0019	;
    AOD[	1856	]=	0.0018	;
    AOD[	1857	]=	0.0017	;
    AOD[	1858	]=	0.0016	;
    AOD[	1859	]=	0.0014	;
    AOD[	1860	]=	0.0013	;
    AOD[	1861	]=	0.0012	;
    AOD[	1862	]=	0.0012	;
    AOD[	1863	]=	0.0031	;
    AOD[	1864	]=	0.0071	;
    AOD[	1865	]=	0.0101	;
    AOD[	1866	]=	0.0141	;
    AOD[	1867	]=	0.0171	;
    AOD[	1868	]=	0.0161	;
    AOD[	1869	]=	0.0148	;
    AOD[	1870	]=	0.0136	;
    AOD[	1871	]=	0.0126	;
    AOD[	1872	]=	0.0116	;
    AOD[	1873	]=	0.0106	;
    AOD[	1874	]=	0.0098	;
    AOD[	1875	]=	0.0090	;
    AOD[	1876	]=	0.0083	;
    AOD[	1877	]=	0.0077	;
    AOD[	1878	]=	0.0071	;
    AOD[	1879	]=	0.0065	;
    AOD[	1880	]=	0.0060	;
    AOD[	1881	]=	0.0055	;
    AOD[	1882	]=	0.0051	;
    AOD[	1883	]=	0.0047	;
    AOD[	1884	]=	0.0043	;
    AOD[	1885	]=	0.0040	;
    AOD[	1886	]=	0.0037	;
    AOD[	1887	]=	0.0034	;
    AOD[	1888	]=	0.0031	;
    AOD[	1889	]=	0.0029	;
    AOD[	1890	]=	0.0027	;
    AOD[	1891	]=	0.0025	;
    AOD[	1892	]=	0.0023	;
    AOD[	1893	]=	0.0021	;
    AOD[	1894	]=	0.0019	;
    AOD[	1895	]=	0.0018	;
    AOD[	1896	]=	0.0017	;
    AOD[	1897	]=	0.0015	;
    AOD[	1898	]=	0.0014	;
    AOD[	1899	]=	0.0013	;
    AOD[	1900	]=	0.0012	;
    AOD[	1901	]=	0.0011	;
    AOD[	1902	]=	0.0010	;
    AOD[	1903	]=	0.0010	;
    AOD[	1904	]=	0.0009	;
    AOD[	1905	]=	0.0008	;
    AOD[	1906	]=	0.0008	;
    AOD[	1907	]=	0.0007	;
    AOD[	1908	]=	0.0030	;
    AOD[	1909	]=	0.0027	;
    AOD[	1910	]=	0.0031	;
    AOD[	1911	]=	0.0031	;
    AOD[	1912	]=	0.0031	;
    AOD[	1913	]=	0.0035	;
    AOD[	1914	]=	0.0042	;
    AOD[	1915	]=	0.0040	;
    AOD[	1916	]=	0.0037	;
    AOD[	1917	]=	0.0037	;
    AOD[	1918	]=	0.0035	;
    AOD[	1919	]=	0.0033	;
    AOD[	1920	]=	0.0031	;
    AOD[	1921	]=	0.0028	;
    AOD[	1922	]=	0.0032	;
    AOD[	1923	]=	0.0032	;
    AOD[	1924	]=	0.0036	;
    AOD[	1925	]=	0.0033	;
    AOD[	1926	]=	0.0030	;
    AOD[	1927	]=	0.0028	;
    AOD[	1928	]=	0.0026	;
    AOD[	1929	]=	0.0024	;
    AOD[	1930	]=	0.0022	;
    AOD[	1931	]=	0.0021	;
    AOD[	1932	]=	0.0019	;
    AOD[	1933	]=	0.0018	;
    AOD[	1934	]=	0.0016	;
    AOD[	1935	]=	0.0015	;
    AOD[	1936	]=	0.0014	;
    AOD[	1937	]=	0.0027	;
    AOD[	1938	]=	0.0030	;
    AOD[	1939	]=	0.0033	;
    AOD[	1940	]=	0.0031	;
    AOD[	1941	]=	0.0028	;
    AOD[	1942	]=	0.0026	;
    AOD[	1943	]=	0.0024	;
    AOD[	1944	]=	0.0022	;
    AOD[	1945	]=	0.0021	;
    AOD[	1946	]=	0.0019	;
    AOD[	1947	]=	0.0018	;
    AOD[	1948	]=	0.0016	;
    AOD[	1949	]=	0.0015	;
    AOD[	1950	]=	0.0014	;
    AOD[	1951	]=	0.0013	;
    AOD[	1952	]=	0.0012	;
    AOD[	1953	]=	0.0011	;
    AOD[	1954	]=	0.0010	;
    AOD[	1955	]=	0.0010	;
    AOD[	1956	]=	0.0017	;
    AOD[	1957	]=	0.0024	;
    AOD[	1958	]=	0.0028	;
    AOD[	1959	]=	0.0033	;
    AOD[	1960	]=	0.0039	;
    AOD[	1961	]=	0.0058	;
    AOD[	1962	]=	0.0065	;
    AOD[	1963	]=	0.0072	;
    AOD[	1964	]=	0.0073	;
    AOD[	1965	]=	0.0069	;
    AOD[	1966	]=	0.0068	;
    AOD[	1967	]=	0.0084	;
    AOD[	1968	]=	0.0086	;
    AOD[	1969	]=	0.0100	;
    AOD[	1970	]=	0.0104	;
    AOD[	1971	]=	0.0099	;
    AOD[	1972	]=	0.0098	;
    AOD[	1973	]=	0.0098	;
    AOD[	1974	]=	0.0104	;
    AOD[	1975	]=	0.0108	;
    AOD[	1976	]=	0.0104	;
    AOD[	1977	]=	0.0099	;
    AOD[	1978	]=	0.0093	;
    AOD[	1979	]=	0.0086	;
    AOD[	1980	]=	0.0080	;
    AOD[	1981	]=	0.0075	;
    AOD[	1982	]=	0.0069	;
    AOD[	1983	]=	0.0064	;
    AOD[	1984	]=	0.0066	;
    AOD[	1985	]=	0.0064	;
    AOD[	1986	]=	0.0061	;
    AOD[	1987	]=	0.0058	;
    AOD[	1988	]=	0.0057	;
    AOD[	1989	]=	0.0053	;
    AOD[	1990	]=	0.0049	;
    AOD[	1991	]=	0.0049	;
    AOD[	1992	]=	0.0052	;
    AOD[	1993	]=	0.0051	;
    AOD[	1994	]=	0.0052	;
    AOD[	1995	]=	0.0052	;
    AOD[	1996	]=	0.0050	;
    AOD[	1997	]=	0.0049	;
    AOD[	1998	]=	0.0049	;
    AOD[	1999	]=	0.0045	;
    AOD[	2000	]=	0.0044	;
    AOD[	2001	]=	0.0041	;
    AOD[	2002	]=	0.0038	;
    AOD[	2003	]=	0.0041	;
    AOD[	2004	]=	0.0046	;
    AOD[	2005	]=	0.0057	;
    AOD[	2006	]=	0.0060	;
    AOD[	2007	]=	0.0066	;
    AOD[	2008	]=	0.0073	;
    AOD[	2009	]=	0.0088	;
    AOD[	2010	]=	0.0096	;
    AOD[	2011	]=	0.0102	;
    AOD[	2012	]=	0.0105	;
    AOD[	2013	]=	0.0101	;
    AOD[	2014	]=	0.0098	;
    AOD[	2015	]=	0.0097	;
    AOD[	2016	]=	0.0095	;
    AOD[	2017	]=	0.0088	;
    AOD[	2018	]=	0.0083	;
    AOD[	2019	]=	0.0079	;
    AOD[	2020	]=	0.0075	;
    AOD[	2021	]=	0.0069	;
    AOD[	2022	]=	0.0063	;
    AOD[	2023	]=	0.0059	;
    AOD[	2024	]=	0.0054	;
    AOD[	2025	]=	0.0049	;
    AOD[	2026	]=	0.0045	;
    AOD[	2027	]=	0.0042	;
    AOD[	2028	]=	0.0039	;
    AOD[	2029	]=	0.0036	;
    AOD[	2030	]=	0.0033	;
    AOD[	2031	]=	0.0030	;
    AOD[	2032	]=	0.0029	;
    AOD[	2033	]=	0.0032	;
    AOD[	2034	]=	0.0037	;
    AOD[	2035	]=	0.0039	;
    AOD[	2036	]=	0.0039	;
    AOD[	2037	]=	0.0044	;
    AOD[	2038	]=	0.0051	;
    AOD[	2039	]=	0.0047	;
    AOD[	2040	]=	0.0045	;
    AOD[	2041	]=	0.0045	;
    AOD[	2042	]=	0.0046	;
    AOD[	2043	]=	0.0048	;
    AOD[	2044	]=	0.0048	;
    AOD[	2045	]=	0.0046	;
    AOD[	2046	]=	0.0043	;
    AOD[	2047	]=	0.0041	;
    AOD[	2048	]=	0.0038	;
    AOD[	2049	]=	0.0037	;
    AOD[	2050	]=	0.0036	;
    AOD[	2051	]=	0.0035	;
    AOD[	2052	]=	0.0034	;
    AOD[	2053	]=	0.0032	;
    AOD[	2054	]=	0.0031	;
    AOD[	2055	]=	0.0030	;
    AOD[	2056	]=	0.0028	;
    AOD[	2057	]=	0.0026	;
    AOD[	2058	]=	0.0029	;
    AOD[	2059	]=	0.0035	;
    AOD[	2060	]=	0.0034	;
    AOD[	2061	]=	0.0037	;
    AOD[	2062	]=	0.0039	;
    AOD[	2063	]=	0.0036	;
    AOD[	2064	]=	0.0033	;
    AOD[	2065	]=	0.0030	;
    AOD[	2066	]=	0.0029	;
    AOD[	2067	]=	0.0026	;
    AOD[	2068	]=	0.0025	;
    AOD[	2069	]=	0.0022	;
    AOD[	2070	]=	0.0021	;
    AOD[	2071	]=	0.0026	;
    AOD[	2072	]=	0.0032	;
    AOD[	2073	]=	0.0032	;
    AOD[	2074	]=	0.0032	;
    AOD[	2075	]=	0.0030	;
    AOD[	2076	]=	0.0033	;
    AOD[	2077	]=	0.0035	;
    AOD[	2078	]=	0.0046	;
    AOD[	2079	]=	0.0046	;
    AOD[	2080	]=	0.0047	;
    AOD[	2081	]=	0.0049	;
    AOD[	2082	]=	0.0055	;
    AOD[	2083	]=	0.0055	;
    AOD[	2084	]=	0.0057	;
    AOD[	2085	]=	0.0059	;
    AOD[	2086	]=	0.0055	;
    AOD[	2087	]=	0.0052	;
    AOD[	2088	]=	0.0047	;
    AOD[	2089	]=	0.0044	;
    AOD[	2090	]=	0.0049	;
    AOD[	2091	]=	0.0047	;
    AOD[	2092	]=	0.0046	;
    AOD[	2093	]=	0.0046	;
    AOD[	2094	]=	0.0042	;
    AOD[	2095	]=	0.0039	;
    AOD[	2096	]=	0.0036	;
    AOD[	2097	]=	0.0033	;
    AOD[	2098	]=	0.0031	;
    AOD[	2099	]=	0.0032	;
    AOD[	2100	]=	0.0034	;
    AOD[	2101	]=	0.0036	;
    AOD[	2102	]=	0.0037	;
    AOD[	2103	]=	0.0034	;
    AOD[	2104	]=	0.0033	;
    AOD[	2105	]=	0.0031	;
    AOD[	2106	]=	0.0030	;
    AOD[	2107	]=	0.0027	;
    AOD[	2108	]=	0.0033	;
    AOD[	2109	]=	0.0030	;
    AOD[	2110	]=	0.0030	;
    AOD[	2111	]=	0.0028	;
    AOD[	2112	]=	0.0026	;
    AOD[	2113	]=	0.0023	;
    AOD[	2114	]=	0.0022	;
    AOD[	2115	]=	0.0020	;
    AOD[	2116	]=	0.0019	;
    AOD[	2117	]=	0.0017	;
    AOD[	2118	]=	0.0016	;
    AOD[	2119	]=	0.0015	;
    AOD[	2120	]=	0.0021	;
    AOD[	2121	]=	0.0019	;
    AOD[	2122	]=	0.0018	;
    AOD[	2123	]=	0.0017	;
    AOD[	2124	]=	0.0026	;
    AOD[	2125	]=	0.0029	;
    AOD[	2126	]=	0.0030	;
    AOD[	2127	]=	0.0031	;
    AOD[	2128	]=	0.0034	;
    AOD[	2129	]=	0.0035	;
    AOD[	2130	]=	0.0037	;
    AOD[	2131	]=	0.0038	;
    AOD[	2132	]=	0.0063	;
    AOD[	2133	]=	0.0061	;
    AOD[	2134	]=	0.0060	;
    AOD[	2135	]=	0.0060	;
    AOD[	2136	]=	0.0061	;
    AOD[	2137	]=	0.0056	;
    AOD[	2138	]=	0.0052	;
    AOD[	2139	]=	0.0048	;
    AOD[	2140	]=	0.0044	;
    AOD[	2141	]=	0.0040	;
    AOD[	2142	]=	0.0039	;
    AOD[	2143	]=	0.0039	;
    AOD[	2144	]=	0.0041	;
    AOD[	2145	]=	0.0038	;
    AOD[	2146	]=	0.0035	;
    AOD[	2147	]=	0.0032	;
    AOD[	2148	]=	0.0030	;
    AOD[	2149	]=	0.0027	;
    AOD[	2150	]=	0.0025	;
    AOD[	2151	]=	0.0023	;
    AOD[	2152	]=	0.0022	;
    AOD[	2153	]=	0.0020	;
    AOD[	2154	]=	0.0019	;
    AOD[	2155	]=	0.0022	;
    AOD[	2156	]=	0.0025	;
    AOD[	2157	]=	0.0025	;
    AOD[	2158	]=	0.0023	;
    AOD[	2159	]=	0.0021	;
    AOD[	2160	]=	0.0025	;
    AOD[	2161	]=	0.0027	;
    AOD[	2162	]=	0.0030	;
    AOD[	2163	]=	0.0028	;
    AOD[	2164	]=	0.0025	;
    AOD[	2165	]=	0.0023	;
    AOD[	2166	]=	0.0022	;
    AOD[	2167	]=	0.0020	;
    AOD[	2168	]=	0.0019	;
    AOD[	2169	]=	0.0017	;
    AOD[	2170	]=	0.0016	;
    AOD[	2171	]=	0.0014	;
    AOD[	2172	]=	0.0016	;
    AOD[	2173	]=	0.0015	;
    AOD[	2174	]=	0.0014	;
    AOD[	2175	]=	0.0014	;
    AOD[	2176	]=	0.0015	;
    AOD[	2177	]=	0.0026	;
    AOD[	2178	]=	0.0024	;
    AOD[	2179	]=	0.0022	;
    AOD[	2180	]=	0.0020	;
    AOD[	2181	]=	0.0019	;
    AOD[	2182	]=	0.0017	;
    AOD[	2183	]=	0.0016	;
    AOD[	2184	]=	0.0015	;
    AOD[	2185	]=	0.0014	;
    AOD[	2186	]=	0.0013	;
    AOD[	2187	]=	0.0012	;
    AOD[	2188	]=	0.0011	;
    AOD[	2189	]=	0.0010	;
    AOD[	2190	]=	0.0037	;
    AOD[	2191	]=	0.0036	;
    AOD[	2192	]=	0.0035	;
    AOD[	2193	]=	0.0033	;
    AOD[	2194	]=	0.0030	;
    AOD[	2195	]=	0.0028	;
    AOD[	2196	]=	0.0025	;
    AOD[	2197	]=	0.0023	;
    AOD[	2198	]=	0.0022	;
    AOD[	2199	]=	0.0020	;
    AOD[	2200	]=	0.0019	;
    AOD[	2201	]=	0.0017	;
    AOD[	2202	]=	0.0016	;
    AOD[	2203	]=	0.0015	;
    AOD[	2204	]=	0.0014	;
    AOD[	2205	]=	0.0012	;
    AOD[	2206	]=	0.0012	;
    AOD[	2207	]=	0.0010	;
    AOD[	2208	]=	0.0010	;
    AOD[	2209	]=	0.0013	;
    AOD[	2210	]=	0.0026	;
    AOD[	2211	]=	0.0027	;
    AOD[	2212	]=	0.0032	;
    AOD[	2213	]=	0.0038	;
    AOD[	2214	]=	0.0043	;
    AOD[	2215	]=	0.0045	;
    AOD[	2216	]=	0.0044	;
    AOD[	2217	]=	0.0041	;
    AOD[	2218	]=	0.0038	;
    AOD[	2219	]=	0.0035	;
    AOD[	2220	]=	0.0034	;
    AOD[	2221	]=	0.0031	;
    AOD[	2222	]=	0.0029	;
    AOD[	2223	]=	0.0029	;
    AOD[	2224	]=	0.0036	;
    AOD[	2225	]=	0.0033	;
    AOD[	2226	]=	0.0030	;
    AOD[	2227	]=	0.0029	;
    AOD[	2228	]=	0.0026	;
    AOD[	2229	]=	0.0024	;
    AOD[	2230	]=	0.0022	;
    AOD[	2231	]=	0.0021	;
    AOD[	2232	]=	0.0019	;
    AOD[	2233	]=	0.0017	;
    AOD[	2234	]=	0.0016	;
    AOD[	2235	]=	0.0016	;
    AOD[	2236	]=	0.0014	;
    AOD[	2237	]=	0.0013	;
    AOD[	2238	]=	0.0012	;
    AOD[	2239	]=	0.0011	;
    AOD[	2240	]=	0.0025	;
    AOD[	2241	]=	0.0034	;
    AOD[	2242	]=	0.0031	;
    AOD[	2243	]=	0.0029	;
    AOD[	2244	]=	0.0040	;
    AOD[	2245	]=	0.0042	;
    AOD[	2246	]=	0.0039	;
    AOD[	2247	]=	0.0039	;
    AOD[	2248	]=	0.0039	;
    AOD[	2249	]=	0.0037	;
    AOD[	2250	]=	0.0037	;
    AOD[	2251	]=	0.0036	;
    AOD[	2252	]=	0.0037	;
    AOD[	2253	]=	0.0034	;
    AOD[	2254	]=	0.0032	;
    AOD[	2255	]=	0.0029	;
    AOD[	2256	]=	0.0027	;
    AOD[	2257	]=	0.0025	;
    AOD[	2258	]=	0.0023	;
    AOD[	2259	]=	0.0021	;
    AOD[	2260	]=	0.0021	;
    AOD[	2261	]=	0.0034	;
    AOD[	2262	]=	0.0034	;
    AOD[	2263	]=	0.0040	;
    AOD[	2264	]=	0.0046	;
    AOD[	2265	]=	0.0045	;
    AOD[	2266	]=	0.0044	;
    AOD[	2267	]=	0.0047	;
    AOD[	2268	]=	0.0049	;
    AOD[	2269	]=	0.0047	;
    AOD[	2270	]=	0.0047	;
    AOD[	2271	]=	0.0043	;
    AOD[	2272	]=	0.0040	;
    AOD[	2273	]=	0.0037	;
    AOD[	2274	]=	0.0034	;
    AOD[	2275	]=	0.0032	;
    AOD[	2276	]=	0.0029	;
    AOD[	2277	]=	0.0027	;
    AOD[	2278	]=	0.0025	;
    AOD[	2279	]=	0.0023	;
    AOD[	2280	]=	0.0022	;
    AOD[	2281	]=	0.0019	;
    AOD[	2282	]=	0.0018	;
    AOD[	2283	]=	0.0017	;
    AOD[	2284	]=	0.0015	;
    AOD[	2285	]=	0.0014	;
    AOD[	2286	]=	0.0014	;
    AOD[	2287	]=	0.0016	;
    AOD[	2288	]=	0.0023	;
    AOD[	2289	]=	0.0021	;
    AOD[	2290	]=	0.0019	;
    AOD[	2291	]=	0.0018	;
    AOD[	2292	]=	0.0016	;
    AOD[	2293	]=	0.0015	;
    AOD[	2294	]=	0.0014	;
    AOD[	2295	]=	0.0013	;
    AOD[	2296	]=	0.0012	;
    AOD[	2297	]=	0.0012	;
    AOD[	2298	]=	0.0010	;
    AOD[	2299	]=	0.0010	;
    AOD[	2300	]=	0.0009	;
    AOD[	2301	]=	0.0009	;
    AOD[	2302	]=	0.0008	;
    AOD[	2303	]=	0.0008	;
    AOD[	2304	]=	0.0007	;
    AOD[	2305	]=	0.0007	;
    AOD[	2306	]=	0.0006	;
    AOD[	2307	]=	0.0006	;
    AOD[	2308	]=	0.0005	;
    AOD[	2309	]=	0.0005	;
    AOD[	2310	]=	0.0005	;
    AOD[	2311	]=	0.0005	;
    AOD[	2312	]=	0.0005	;
    AOD[	2313	]=	0.0004	;
    AOD[	2314	]=	0.0004	;
    AOD[	2315	]=	0.0003	;
    AOD[	2316	]=	0.0003	;
    AOD[	2317	]=	0.0003	;
    AOD[	2318	]=	0.0003	;
    AOD[	2319	]=	0.0003	;
    AOD[	2320	]=	0.0003	;
    AOD[	2321	]=	0.0003	;
    AOD[	2322	]=	0.0002	;
    AOD[	2323	]=	0.0002	;
    AOD[	2324	]=	0.0002	;
    AOD[	2325	]=	0.0002	;
    AOD[	2326	]=	0.0002	;
    AOD[	2327	]=	0.0002	;
    AOD[	2328	]=	0.0002	;
    AOD[	2329	]=	0.0002	;
    AOD[	2330	]=	0.0002	;
    AOD[	2331	]=	0.0002	;
    AOD[	2332	]=	0.0002	;
    AOD[	2333	]=	0.0002	;
    AOD[	2334	]=	0.0002	;
    AOD[	2335	]=	0.0002	;
    AOD[	2336	]=	0.0002	;
    AOD[	2337	]=	0.0002	;
    AOD[	2338	]=	0.0002	;
    AOD[	2339	]=	0.0002	;
    AOD[	2340	]=	0.0002	;
    AOD[	2341	]=	0.0002	;
    AOD[	2342	]=	0.0002	;
    AOD[	2343	]=	0.0015	;
    AOD[	2344	]=	0.0028	;
    AOD[	2345	]=	0.0042	;
    AOD[	2346	]=	0.0057	;
    AOD[	2347	]=	0.0070	;
    AOD[	2348	]=	0.0084	;
    AOD[	2349	]=	0.0084	;
    AOD[	2350	]=	0.0084	;
    AOD[	2351	]=	0.0084	;
    AOD[	2352	]=	0.0078	;
    AOD[	2353	]=	0.0086	;
    AOD[	2354	]=	0.0115	;
    AOD[	2355	]=	0.0126	;
    AOD[	2356	]=	0.0116	;
    AOD[	2357	]=	0.0112	;
    AOD[	2358	]=	0.0104	;
    AOD[	2359	]=	0.0107	;
    AOD[	2360	]=	0.0118	;
    AOD[	2361	]=	0.0116	;
    AOD[	2362	]=	0.0112	;
    AOD[	2363	]=	0.0103	;
    AOD[	2364	]=	0.0095	;
    AOD[	2365	]=	0.0090	;
    AOD[	2366	]=	0.0138	;
    AOD[	2367	]=	0.0151	;
    AOD[	2368	]=	0.0141	;
    AOD[	2369	]=	0.0132	;
    AOD[	2370	]=	0.0134	;
    AOD[	2371	]=	0.0160	;
    AOD[	2372	]=	0.0147	;
    AOD[	2373	]=	0.0143	;
    AOD[	2374	]=	0.0134	;
    AOD[	2375	]=	0.0125	;
    AOD[	2376	]=	0.0122	;
    AOD[	2377	]=	0.0119	;
    AOD[	2378	]=	0.0123	;
    AOD[	2379	]=	0.0173	;
    AOD[	2380	]=	0.0338	;
    AOD[	2381	]=	0.0470	;
    AOD[	2382	]=	0.0516	;
    AOD[	2383	]=	0.0562	;
    AOD[	2384	]=	0.0605	;
    AOD[	2385	]=	0.0754	;
    AOD[	2386	]=	0.0885	;
    AOD[	2387	]=	0.0849	;
    AOD[	2388	]=	0.0798	;
    AOD[	2389	]=	0.0746	;
    AOD[	2390	]=	0.0696	;
    AOD[	2391	]=	0.0668	;
    AOD[	2392	]=	0.0667	;
    AOD[	2393	]=	0.0700	;
    AOD[	2394	]=	0.0798	;
    AOD[	2395	]=	0.0797	;
    AOD[	2396	]=	0.0743	;
    AOD[	2397	]=	0.0695	;
    AOD[	2398	]=	0.0662	;
    AOD[	2399	]=	0.0633	;
    AOD[	2400	]=	0.0600	;
    AOD[	2401	]=	0.0567	;
    AOD[	2402	]=	0.0531	;
    AOD[	2403	]=	0.0493	;
    AOD[	2404	]=	0.0466	;
    AOD[	2405	]=	0.0435	;
    AOD[	2406	]=	0.0406	;
    AOD[	2407	]=	0.0374	;
    AOD[	2408	]=	0.0347	;
    AOD[	2409	]=	0.0325	;
    AOD[	2410	]=	0.0312	;
    AOD[	2411	]=	0.0326	;
    AOD[	2412	]=	0.0327	;
    AOD[	2413	]=	0.0307	;
    AOD[	2414	]=	0.0288	;
    AOD[	2415	]=	0.0269	;
    AOD[	2416	]=	0.0252	;
    AOD[	2417	]=	0.0236	;
    AOD[	2418	]=	0.0220	;
    AOD[	2419	]=	0.0206	;
    AOD[	2420	]=	0.0191	;
    AOD[	2421	]=	0.0177	;
    AOD[	2422	]=	0.0164	;
    AOD[	2423	]=	0.0151	;
    AOD[	2424	]=	0.0139	;
    AOD[	2425	]=	0.0128	;
    AOD[	2426	]=	0.0118	;
    AOD[	2427	]=	0.0108	;
    AOD[	2428	]=	0.0112	;
    AOD[	2429	]=	0.0126	;
    AOD[	2430	]=	0.0142	;
    AOD[	2431	]=	0.0159	;
    AOD[	2432	]=	0.0176	;
    AOD[	2433	]=	0.0192	;
    AOD[	2434	]=	0.0177	;
    AOD[	2435	]=	0.0163	;
    AOD[	2436	]=	0.0150	;
    AOD[	2437	]=	0.0138	;
    AOD[	2438	]=	0.0127	;
    AOD[	2439	]=	0.0117	;
    AOD[	2440	]=	0.0142	;
    AOD[	2441	]=	0.0209	;
    AOD[	2442	]=	0.0276	;
    AOD[	2443	]=	0.0342	;
    AOD[	2444	]=	0.0409	;
    AOD[	2445	]=	0.0475	;
    AOD[	2446	]=	0.0459	;
    AOD[	2447	]=	0.0445	;
    AOD[	2448	]=	0.0429	;
    AOD[	2449	]=	0.0414	;
    AOD[	2450	]=	0.0398	;
    AOD[	2451	]=	0.0383	;
    AOD[	2452	]=	0.0367	;
    AOD[	2453	]=	0.0351	;
    AOD[	2454	]=	0.0336	;
    AOD[	2455	]=	0.0320	;
    AOD[	2456	]=	0.0306	;
    AOD[	2457	]=	0.0290	;
    AOD[	2458	]=	0.0275	;
    AOD[	2459	]=	0.0259	;
    AOD[	2460	]=	0.0243	;
    AOD[	2461	]=	0.0228	;
    AOD[	2462	]=	0.0212	;
    AOD[	2463	]=	0.0197	;
    AOD[	2464	]=	0.0182	;
    AOD[	2465	]=	0.0167	;
    AOD[	2466	]=	0.0154	;
    AOD[	2467	]=	0.0142	;
    AOD[	2468	]=	0.0130	;
    AOD[	2469	]=	0.0120	;
    AOD[	2470	]=	0.0111	;
    AOD[	2471	]=	0.0102	;
    AOD[	2472	]=	0.0094	;
    AOD[	2473	]=	0.0086	;
    AOD[	2474	]=	0.0079	;
    AOD[	2475	]=	0.0073	;
    AOD[	2476	]=	0.0067	;
    AOD[	2477	]=	0.0062	;
    AOD[	2478	]=	0.0057	;
    AOD[	2479	]=	0.0053	;
    AOD[	2480	]=	0.0058	;
    AOD[	2481	]=	0.0054	;
    AOD[	2482	]=	0.0049	;
    AOD[	2483	]=	0.0046	;
    AOD[	2484	]=	0.0042	;
    AOD[	2485	]=	0.0039	;
    AOD[	2486	]=	0.0036	;
    AOD[	2487	]=	0.0033	;
    AOD[	2488	]=	0.0030	;
    AOD[	2489	]=	0.0030	;
    AOD[	2490	]=	0.0034	;
    AOD[	2491	]=	0.0038	;
    AOD[	2492	]=	0.0042	;
    AOD[	2493	]=	0.0045	;
    AOD[	2494	]=	0.0049	;
    AOD[	2495	]=	0.0053	;
    AOD[	2496	]=	0.0057	;
    AOD[	2497	]=	0.0061	;
    AOD[	2498	]=	0.0065	;
    AOD[	2499	]=	0.0068	;
    AOD[	2500	]=	0.0072	;
    AOD[	2501	]=	0.0076	;
    AOD[	2502	]=	0.0080	;
    AOD[	2503	]=	0.0084	;
    AOD[	2504	]=	0.0087	;
    AOD[	2505	]=	0.0091	;
    AOD[	2506	]=	0.0095	;
    AOD[	2507	]=	0.0099	;
    AOD[	2508	]=	0.0103	;
    AOD[	2509	]=	0.0107	;
    AOD[	2510	]=	0.0110	;
    AOD[	2511	]=	0.0114	;
    AOD[	2512	]=	0.0118	;
    AOD[	2513	]=	0.0122	;
    AOD[	2514	]=	0.0126	;
    AOD[	2515	]=	0.0130	;
    AOD[	2516	]=	0.0133	;
    AOD[	2517	]=	0.0137	;
    AOD[	2518	]=	0.0141	;
    AOD[	2519	]=	0.0184	;
    AOD[	2520	]=	0.0228	;
    AOD[	2521	]=	0.0271	;
    AOD[	2522	]=	0.0314	;
    AOD[	2523	]=	0.0358	;
    AOD[	2524	]=	0.0401	;
    AOD[	2525	]=	0.0369	;
    AOD[	2526	]=	0.0340	;
    AOD[	2527	]=	0.0313	;
    AOD[	2528	]=	0.0288	;
    AOD[	2529	]=	0.0265	;
    AOD[	2530	]=	0.0244	;
    AOD[	2531	]=	0.0224	;
    AOD[	2532	]=	0.0206	;
    AOD[	2533	]=	0.0190	;
    AOD[	2534	]=	0.0175	;
    AOD[	2535	]=	0.0161	;
    AOD[	2536	]=	0.0148	;
    AOD[	2537	]=	0.0136	;
    AOD[	2538	]=	0.0126	;
    AOD[	2539	]=	0.0116	;
    AOD[	2540	]=	0.0106	;
    AOD[	2541	]=	0.0098	;
    AOD[	2542	]=	0.0090	;
    AOD[	2543	]=	0.0083	;
    AOD[	2544	]=	0.0077	;
    AOD[	2545	]=	0.0071	;
    AOD[	2546	]=	0.0065	;
    AOD[	2547	]=	0.0060	;
    AOD[	2548	]=	0.0055	;
    AOD[	2549	]=	0.0051	;
    AOD[	2550	]=	0.0047	;
    AOD[	2551	]=	0.0043	;
    AOD[	2552	]=	0.0040	;
    AOD[	2553	]=	0.0037	;
    AOD[	2554	]=	0.0034	;
    AOD[	2555	]=	0.0031	;
    AOD[	2556	]=	0.0029	;
    AOD[	2557	]=	0.0027	;
    AOD[	2558	]=	0.0025	;
    AOD[	2559	]=	0.0029	;
    AOD[	2560	]=	0.0048	;
    AOD[	2561	]=	0.0066	;
    AOD[	2562	]=	0.0084	;
    AOD[	2563	]=	0.0103	;
    AOD[	2564	]=	0.0121	;
    AOD[	2565	]=	0.0123	;
    AOD[	2566	]=	0.0124	;
    AOD[	2567	]=	0.0126	;
    AOD[	2568	]=	0.0128	;
    AOD[	2569	]=	0.0129	;
    AOD[	2570	]=	0.0119	;
    AOD[	2571	]=	0.0109	;
    AOD[	2572	]=	0.0101	;
    AOD[	2573	]=	0.0093	;
    AOD[	2574	]=	0.0085	;
    AOD[	2575	]=	0.0079	;
    AOD[	2576	]=	0.0072	;
    AOD[	2577	]=	0.0067	;
    AOD[	2578	]=	0.0061	;
    AOD[	2579	]=	0.0057	;
    AOD[	2580	]=	0.0052	;
    AOD[	2581	]=	0.0048	;
    AOD[	2582	]=	0.0044	;
    AOD[	2583	]=	0.0041	;
    AOD[	2584	]=	0.0038	;
    AOD[	2585	]=	0.0039	;
    AOD[	2586	]=	0.0044	;
    AOD[	2587	]=	0.0049	;
    AOD[	2588	]=	0.0051	;
    AOD[	2589	]=	0.0052	;
    AOD[	2590	]=	0.0054	;
    AOD[	2591	]=	0.0052	;
    AOD[	2592	]=	0.0051	;
    AOD[	2593	]=	0.0049	;
    AOD[	2594	]=	0.0047	;
    AOD[	2595	]=	0.0045	;
    AOD[	2596	]=	0.0044	;
    AOD[	2597	]=	0.0049	;
    AOD[	2598	]=	0.0055	;
    AOD[	2599	]=	0.0055	;
    AOD[	2600	]=	0.0051	;
    AOD[	2601	]=	0.0051	;
    AOD[	2602	]=	0.0051	;
    AOD[	2603	]=	0.0051	;
    AOD[	2604	]=	0.0057	;
    AOD[	2605	]=	0.0057	;
    AOD[	2606	]=	0.0057	;
    AOD[	2607	]=	0.0245	;
    AOD[	2608	]=	0.0427	;
    AOD[	2609	]=	0.0534	;
    AOD[	2610	]=	0.0681	;
    AOD[	2611	]=	0.0741	;
    AOD[	2612	]=	0.0782	;
    AOD[	2613	]=	0.0843	;
    AOD[	2614	]=	0.0901	;
    AOD[	2615	]=	0.0978	;
    AOD[	2616	]=	0.0960	;
    AOD[	2617	]=	0.0939	;
    AOD[	2618	]=	0.0959	;
    AOD[	2619	]=	0.0912	;
    AOD[	2620	]=	0.0863	;
    AOD[	2621	]=	0.0794	;
    AOD[	2622	]=	0.0731	;
    AOD[	2623	]=	0.0673	;
    AOD[	2624	]=	0.0619	;
    AOD[	2625	]=	0.0569	;
    AOD[	2626	]=	0.0524	;
    AOD[	2627	]=	0.0486	;
    AOD[	2628	]=	0.0451	;
    AOD[	2629	]=	0.0416	;
    AOD[	2630	]=	0.0383	;
    AOD[	2631	]=	0.0352	;
    AOD[	2632	]=	0.0324	;
    AOD[	2633	]=	0.0302	;
    AOD[	2634	]=	0.0282	;
    AOD[	2635	]=	0.0260	;
    AOD[	2636	]=	0.0239	;
    AOD[	2637	]=	0.0220	;
    AOD[	2638	]=	0.0202	;
    AOD[	2639	]=	0.0188	;
    AOD[	2640	]=	0.0164	;
    AOD[	2641	]=	0.0160	;
    AOD[	2642	]=	0.0141	;
    AOD[	2643	]=	0.0138	;
    AOD[	2644	]=	0.0128	;
    AOD[	2645	]=	0.0126	;
    AOD[	2646	]=	0.0121	;
    AOD[	2647	]=	0.0116	;
    AOD[	2648	]=	0.0102	;
    AOD[	2649	]=	0.0101	;
    AOD[	2650	]=	0.0097	;
    AOD[	2651	]=	0.0122	;
    AOD[	2652	]=	0.0146	;
    AOD[	2653	]=	0.0158	;
    AOD[	2654	]=	0.0144	;
    AOD[	2655	]=	0.0147	;
    AOD[	2656	]=	0.0140	;
    AOD[	2657	]=	0.0130	;
    AOD[	2658	]=	0.0128	;
    AOD[	2659	]=	0.0140	;
    AOD[	2660	]=	0.0126	;
    AOD[	2661	]=	0.0126	;
    AOD[	2662	]=	0.0123	;
    AOD[	2663	]=	0.0120	;
    AOD[	2664	]=	0.0118	;
    AOD[	2665	]=	0.0111	;
    AOD[	2666	]=	0.0109	;
    AOD[	2667	]=	0.0104	;
    AOD[	2668	]=	0.0101	;
    AOD[	2669	]=	0.0105	;
    AOD[	2670	]=	0.0108	;
    AOD[	2671	]=	0.0108	;
    AOD[	2672	]=	0.0104	;
    AOD[	2673	]=	0.0094	;
    AOD[	2674	]=	0.0089	;
    AOD[	2675	]=	0.0089	;
    AOD[	2676	]=	0.0085	;
    AOD[	2677	]=	0.0088	;
    AOD[	2678	]=	0.0078	;
    AOD[	2679	]=	0.0074	;
    AOD[	2680	]=	0.0071	;
    AOD[	2681	]=	0.0071	;
    AOD[	2682	]=	0.0080	;
    AOD[	2683	]=	0.0096	;
    AOD[	2684	]=	0.0069	;
    AOD[	2685	]=	0.0065	;
    AOD[	2686	]=	0.0068	;
    AOD[	2687	]=	0.0068	;
    AOD[	2688	]=	0.0072	;
    AOD[	2689	]=	0.0064	;
    AOD[	2690	]=	0.0062	;
    AOD[	2691	]=	0.0060	;
    AOD[	2692	]=	0.0047	;
    AOD[	2693	]=	0.0057	;
    AOD[	2694	]=	0.0067	;
    AOD[	2695	]=	0.0083	;
    AOD[	2696	]=	0.0062	;
    AOD[	2697	]=	0.0054	;
    AOD[	2698	]=	0.0055	;
    AOD[	2699	]=	0.0054	;
    AOD[	2700	]=	0.0059	;
    AOD[	2701	]=	0.0074	;
    AOD[	2702	]=	0.0062	;
    AOD[	2703	]=	0.0044	;
    AOD[	2704	]=	0.0044	;
    AOD[	2705	]=	0.0060	;
    AOD[	2706	]=	0.0073	;
    AOD[	2707	]=	0.0074	;
    AOD[	2708	]=	0.0068	;
    AOD[	2709	]=	0.0058	;
    AOD[	2710	]=	0.0060	;
    AOD[	2711	]=	0.0060	;
    AOD[	2712	]=	0.0060	;
    AOD[	2713	]=	0.0057	;
    AOD[	2714	]=	0.0055	;
    AOD[	2715	]=	0.0053	;
    AOD[	2716	]=	0.0053	;
    AOD[	2717	]=	0.0179	;
    AOD[	2718	]=	0.0377	;
    AOD[	2719	]=	0.0710	;
    AOD[	2720	]=	0.0964	;
    AOD[	2721	]=	0.1197	;
    AOD[	2722	]=	0.1380	;
    AOD[	2723	]=	0.1385	;
    AOD[	2724	]=	0.1483	;
    AOD[	2725	]=	0.1494	;
    AOD[	2726	]=	0.1428	;
    AOD[	2727	]=	0.1386	;
    AOD[	2728	]=	0.1379	;
    AOD[	2729	]=	0.1265	;
    AOD[	2730	]=	0.1228	;
    AOD[	2731	]=	0.1141	;
    AOD[	2732	]=	0.1044	;
    AOD[	2733	]=	0.0978	;
    AOD[	2734	]=	0.0918	;
    AOD[	2735	]=	0.0793	;
    AOD[	2736	]=	0.0743	;
    AOD[	2737	]=	0.0661	;
    AOD[	2738	]=	0.0608	;
    AOD[	2739	]=	0.0559	;
    AOD[	2740	]=	0.0525	;
    AOD[	2741	]=	0.0482	;
    AOD[	2742	]=	0.0445	;
    AOD[	2743	]=	0.0425	;
    AOD[	2744	]=	0.0394	;
    AOD[	2745	]=	0.0376	;
    AOD[	2746	]=	0.0350	;
    AOD[	2747	]=	0.0312	;
    AOD[	2748	]=	0.0285	;
    AOD[	2749	]=	0.0265	;
    AOD[	2750	]=	0.0238	;
    AOD[	2751	]=	0.0218	;
    AOD[	2752	]=	0.0205	;
    AOD[	2753	]=	0.0185	;
    AOD[	2754	]=	0.0179	;
    AOD[	2755	]=	0.0173	;
    AOD[	2756	]=	0.0161	;
    AOD[	2757	]=	0.0166	;
    AOD[	2758	]=	0.0159	;
    AOD[	2759	]=	0.0160	;
    AOD[	2760	]=	0.0120	;
    AOD[	2761	]=	0.0117	;
    AOD[	2762	]=	0.0109	;
    AOD[	2763	]=	0.0100	;
    AOD[	2764	]=	0.0100	;
    AOD[	2765	]=	0.0093	;
    AOD[	2766	]=	0.0094	;
    AOD[	2767	]=	0.0087	;
    AOD[	2768	]=	0.0085	;
    AOD[	2769	]=	0.0088	;
    AOD[	2770	]=	0.0086	;
    AOD[	2771	]=	0.0077	;
    AOD[	2772	]=	0.0074	;
    AOD[	2773	]=	0.0072	;
    AOD[	2774	]=	0.0068	;
    AOD[	2775	]=	0.0063	;
    AOD[	2776	]=	0.0067	;
    AOD[	2777	]=	0.0060	;
    AOD[	2778	]=	0.0066	;
    AOD[	2779	]=	0.0062	;
    AOD[	2780	]=	0.0059	;
    AOD[	2781	]=	0.0064	;
    AOD[	2782	]=	0.0064	;
    AOD[	2783	]=	0.0061	;
    AOD[	2784	]=	0.0060	;
    AOD[	2785	]=	0.0057	;
    AOD[	2786	]=	0.0053	;
    AOD[	2787	]=	0.0052	;
    AOD[	2788	]=	0.0055	;
    AOD[	2789	]=	0.0050	;
    AOD[	2790	]=	0.0054	;
    AOD[	2791	]=	0.0051	;
    AOD[	2792	]=	0.0053	;
    AOD[	2793	]=	0.0051	;
    AOD[	2794	]=	0.0047	;
    AOD[	2795	]=	0.0043	;
    AOD[	2796	]=	0.0040	;
    AOD[	2797	]=	0.0037	;
    AOD[	2798	]=	0.0034	;
    AOD[	2799	]=	0.0031	;
    AOD[	2800	]=	0.0029	;
    AOD[	2801	]=	0.0027	;
    AOD[	2802	]=	0.0025	;
    AOD[	2803	]=	0.0024	;
    AOD[	2804	]=	0.0023	;
    AOD[	2805	]=	0.0022	;
    AOD[	2806	]=	0.0022	;
    AOD[	2807	]=	0.0022	;
    AOD[	2808	]=	0.0022	;
    AOD[	2809	]=	0.0022	;
    AOD[	2810	]=	0.0021	;
    AOD[	2811	]=	0.0021	;
    AOD[	2812	]=	0.0021	;
    AOD[	2813	]=	0.0021	;
    AOD[	2814	]=	0.0021	;
    AOD[	2815	]=	0.0021	;
    AOD[	2816	]=	0.0021	;
    AOD[	2817	]=	0.0021	;
    AOD[	2818	]=	0.0021	;
    AOD[	2819	]=	0.0021	;
    AOD[	2820	]=	0.0021	;
    AOD[	2821	]=	0.0021	;
    AOD[	2822	]=	0.0021	;
    AOD[	2823	]=	0.0021	;
    AOD[	2824	]=	0.0021	;
    AOD[	2825	]=	0.0021	;
    AOD[	2826	]=	0.0021	;
    AOD[	2827	]=	0.0021	;
    AOD[	2828	]=	0.0021	;
    AOD[	2829	]=	0.0021	;
    AOD[	2830	]=	0.0021	;
    AOD[	2831	]=	0.0021	;
    AOD[	2832	]=	0.0021	;
    AOD[	2833	]=	0.0021	;
    AOD[	2834	]=	0.0021	;
    AOD[	2835	]=	0.0021	;
    AOD[	2836	]=	0.0021	;
    AOD[	2837	]=	0.0021	;
    AOD[	2838	]=	0.0021	;
    AOD[	2839	]=	0.0021	;
    AOD[	2840	]=	0.0021	;
    AOD[	2841	]=	0.0021	;
    AOD[	2842	]=	0.0021	;
    AOD[	2843	]=	0.0022	;
    AOD[	2844	]=	0.0021	;
    AOD[	2845	]=	0.0021	;
    AOD[	2846	]=	0.0020	;
    AOD[	2847	]=	0.0020	;
    AOD[	2848	]=	0.0019	;
    AOD[	2849	]=	0.0019	;
    AOD[	2850	]=	0.0016	;
    AOD[	2851	]=	0.0021	;
    AOD[	2852	]=	0.0022	;
    AOD[	2853	]=	0.0022	;
    AOD[	2854	]=	0.0024	;
    AOD[	2855	]=	0.0026	;
    AOD[	2856	]=	0.0028	;
    AOD[	2857	]=	0.0028	;
    AOD[	2858	]=	0.0026	;
    AOD[	2859	]=	0.0026	;
    AOD[	2860	]=	0.0025	;
    AOD[	2861	]=	0.0025	;
    AOD[	2862	]=	0.0026	;
    AOD[	2863	]=	0.0028	;
    AOD[	2864	]=	0.0029	;
    AOD[	2865	]=	0.0028	;
    AOD[	2866	]=	0.0028	;
    AOD[	2867	]=	0.0027	;
    AOD[	2868	]=	0.0025	;
    AOD[	2869	]=	0.0024	;
    AOD[	2870	]=	0.0024	;
    AOD[	2871	]=	0.0024	;
    AOD[	2872	]=	0.0022	;
    AOD[	2873	]=	0.0022	;
    AOD[	2874	]=	0.0023	;
    AOD[	2875	]=	0.0025	;
    AOD[	2876	]=	0.0025	;
    AOD[	2877	]=	0.0025	;
    AOD[	2878	]=	0.0025	;
    AOD[	2879	]=	0.0027	;
    AOD[	2880	]=	0.0029	;
    AOD[	2881	]=	0.0031	;
    AOD[	2882	]=	0.0034	;
    AOD[	2883	]=	0.0035	;
    AOD[	2884	]=	0.0034	;
    AOD[	2885	]=	0.0033	;
    AOD[	2886	]=	0.0033	;
    AOD[	2887	]=	0.0036	;
    AOD[	2888	]=	0.0037	;
    AOD[	2889	]=	0.0037	;
    AOD[	2890	]=	0.0037	;
    AOD[	2891	]=	0.0036	;
    AOD[	2892	]=	0.0034	;
    AOD[	2893	]=	0.0033	;
    AOD[	2894	]=	0.0032	;
    AOD[	2895	]=	0.0032	;
    AOD[	2896	]=	0.0030	;
    AOD[	2897	]=	0.0033	;
    AOD[	2898	]=	0.0038	;
    AOD[	2899	]=	0.0041	;
    AOD[	2900	]=	0.0043	;
    AOD[	2901	]=	0.0044	;
    AOD[	2902	]=	0.0049	;
    AOD[	2903	]=	0.0054	;
    AOD[	2904	]=	0.0054	;
    AOD[	2905	]=	0.0051	;
    AOD[	2906	]=	0.0045	;
    AOD[	2907	]=	0.0045	;
    AOD[	2908	]=	0.0041	;
    AOD[	2909	]=	0.0040	;
    AOD[	2910	]=	0.0040	;
    AOD[	2911	]=	0.0041	;
    AOD[	2912	]=	0.0042	;
    AOD[	2913	]=	0.0041	;
    AOD[	2914	]=	0.0042	;
    AOD[	2915	]=	0.0040	;
    AOD[	2916	]=	0.0038	;
    AOD[	2917	]=	0.0036	;
    AOD[	2918	]=	0.0034	;
    AOD[	2919	]=	0.0033	;
    AOD[	2920	]=	0.0031	;
    AOD[	2921	]=	0.0031	;
    AOD[	2922	]=	0.0033	;
    AOD[	2923	]=	0.0036	;
    AOD[	2924	]=	0.0043	;
    AOD[	2925	]=	0.0046	;
    AOD[	2926	]=	0.0048	;
    AOD[	2927	]=	0.0046	;
    AOD[	2928	]=	0.0042	;
    AOD[	2929	]=	0.0039	;
    AOD[	2930	]=	0.0040	;
    AOD[	2931	]=	0.0038	;
    AOD[	2932	]=	0.0036	;
    AOD[	2933	]=	0.0036	;
    AOD[	2934	]=	0.0051	;
    AOD[	2935	]=	0.0062	;
    AOD[	2936	]=	0.0062	;
    AOD[	2937	]=	0.0056	;
    AOD[	2938	]=	0.0053	;
    AOD[	2939	]=	0.0046	;
    AOD[	2940	]=	0.0041	;
    AOD[	2941	]=	0.0037	;
    AOD[	2942	]=	0.0033	;
    AOD[	2943	]=	0.0034	;
    AOD[	2944	]=	0.0032	;
    AOD[	2945	]=	0.0032	;
    AOD[	2946	]=	0.0033	;
    AOD[	2947	]=	0.0035	;
    AOD[	2948	]=	0.0035	;
    AOD[	2949	]=	0.0034	;
    AOD[	2950	]=	0.0037	;
    AOD[	2951	]=	0.0045	;
    AOD[	2952	]=	0.0044	;
    AOD[	2953	]=	0.0041	;
    AOD[	2954	]=	0.0037	;
    AOD[	2955	]=	0.0035	;
    AOD[	2956	]=	0.0034	;
    AOD[	2957	]=	0.0034	;
    AOD[	2958	]=	0.0052	;
    AOD[	2959	]=	0.0068	;
    AOD[	2960	]=	0.0067	;
    AOD[	2961	]=	0.0064	;
    AOD[	2962	]=	0.0060	;
    AOD[	2963	]=	0.0055	;
    AOD[	2964	]=	0.0048	;
    AOD[	2965	]=	0.0040	;
    AOD[	2966	]=	0.0038	;
    AOD[	2967	]=	0.0038	;
    AOD[	2968	]=	0.0035	;
    AOD[	2969	]=	0.0034	;
    AOD[	2970	]=	0.0034	;
    AOD[	2971	]=	0.0034	;
    AOD[	2972	]=	0.0035	;
    
    for(int i = 2973; i<tmax; i++)
    {
        AOD[i] = 0.0;
    }
    
    
}

void setR_aerosol_RCP6(void)
{
    RnonCO2[	0	]=	0	;
    RnonCO2[	1	]=	0.000955298	;
    RnonCO2[	2	]=	0.001424721	;
    RnonCO2[	3	]=	0.0001974	;
    RnonCO2[	4	]=	-0.001278437	;
    RnonCO2[	5	]=	-0.002733841	;
    RnonCO2[	6	]=	-0.004541334	;
    RnonCO2[	7	]=	-0.00632004	;
    RnonCO2[	8	]=	-0.007680948	;
    RnonCO2[	9	]=	-0.008994568	;
    RnonCO2[	10	]=	-0.010260775	;
    RnonCO2[	11	]=	-0.011479866	;
    RnonCO2[	12	]=	-0.012652425	;
    RnonCO2[	13	]=	-0.013779231	;
    RnonCO2[	14	]=	-0.014861204	;
    RnonCO2[	15	]=	-0.015900176	;
    RnonCO2[	16	]=	-0.017335746	;
    RnonCO2[	17	]=	-0.01873479	;
    RnonCO2[	18	]=	-0.019659073	;
    RnonCO2[	19	]=	-0.020540613	;
    RnonCO2[	20	]=	-0.02137714	;
    RnonCO2[	21	]=	-0.022166399	;
    RnonCO2[	22	]=	-0.022906132	;
    RnonCO2[	23	]=	-0.023594082	;
    RnonCO2[	24	]=	-0.024227995	;
    RnonCO2[	25	]=	-0.024806236	;
    RnonCO2[	26	]=	-0.025775743	;
    RnonCO2[	27	]=	-0.026691982	;
    RnonCO2[	28	]=	-0.027110281	;
    RnonCO2[	29	]=	-0.02747885	;
    RnonCO2[	30	]=	-0.027799526	;
    RnonCO2[	31	]=	-0.028074106	;
    RnonCO2[	32	]=	-0.02875039	;
    RnonCO2[	33	]=	-0.029383792	;
    RnonCO2[	34	]=	-0.02953002	;
    RnonCO2[	35	]=	-0.03007996	;
    RnonCO2[	36	]=	-0.03058546	;
    RnonCO2[	37	]=	-0.03148571	;
    RnonCO2[	38	]=	-0.03189741	;
    RnonCO2[	39	]=	-0.03138062	;
    RnonCO2[	40	]=	-0.03126848	;
    RnonCO2[	41	]=	-0.03156144	;
    RnonCO2[	42	]=	-0.03182181	;
    RnonCO2[	43	]=	-0.03161267	;
    RnonCO2[	44	]=	-0.03137821	;
    RnonCO2[	45	]=	-0.03112038	;
    RnonCO2[	46	]=	-0.03127367	;
    RnonCO2[	47	]=	-0.0313998	;
    RnonCO2[	48	]=	-0.03106322	;
    RnonCO2[	49	]=	-0.03070135	;
    RnonCO2[	50	]=	-0.03074616	;
    RnonCO2[	51	]=	-0.03119614	;
    RnonCO2[	52	]=	-0.03161872	;
    RnonCO2[	53	]=	-0.03158785	;
    RnonCO2[	54	]=	-0.03110696	;
    RnonCO2[	55	]=	-0.03060634	;
    RnonCO2[	56	]=	-0.0300866	;
    RnonCO2[	57	]=	-0.02996996	;
    RnonCO2[	58	]=	-0.03025404	;
    RnonCO2[	59	]=	-0.03009594	;
    RnonCO2[	60	]=	-0.02991636	;
    RnonCO2[	61	]=	-0.02971869	;
    RnonCO2[	62	]=	-0.02950042	;
    RnonCO2[	63	]=	-0.02926502	;
    RnonCO2[	64	]=	-0.02859871	;
    RnonCO2[	65	]=	-0.03037359	;
    RnonCO2[	66	]=	-0.03172034	;
    RnonCO2[	67	]=	-0.03059392	;
    RnonCO2[	68	]=	-0.03026959	;
    RnonCO2[	69	]=	-0.02993194	;
    RnonCO2[	70	]=	-0.02957653	;
    RnonCO2[	71	]=	-0.03080027	;
    RnonCO2[	72	]=	-0.03160022	;
    RnonCO2[	73	]=	-0.03117789	;
    RnonCO2[	74	]=	-0.03112982	;
    RnonCO2[	75	]=	-0.03144654	;
    RnonCO2[	76	]=	-0.03173222	;
    RnonCO2[	77	]=	-0.03198224	;
    RnonCO2[	78	]=	-0.03220368	;
    RnonCO2[	79	]=	-0.03239696	;
    RnonCO2[	80	]=	-0.03371239	;
    RnonCO2[	81	]=	-0.03424533	;
    RnonCO2[	82	]=	-0.0343764	;
    RnonCO2[	83	]=	-0.03487942	;
    RnonCO2[	84	]=	-0.03536957	;
    RnonCO2[	85	]=	-0.03686306	;
    RnonCO2[	86	]=	-0.03665392	;
    RnonCO2[	87	]=	-0.04248424	;
    RnonCO2[	88	]=	-0.04930444	;
    RnonCO2[	89	]=	-0.05182621	;
    RnonCO2[	90	]=	-0.0542671	;
    RnonCO2[	91	]=	-0.05573158	;
    RnonCO2[	92	]=	-0.05765815	;
    RnonCO2[	93	]=	-0.05821397	;
    RnonCO2[	94	]=	-0.06005825	;
    RnonCO2[	95	]=	-0.0683501	;
    RnonCO2[	96	]=	-0.08104112	;
    RnonCO2[	97	]=	-0.07832852	;
    RnonCO2[	98	]=	-0.07177024	;
    RnonCO2[	99	]=	-0.07363036	;
    RnonCO2[	100	]=	-0.07696442	;
    RnonCO2[	101	]=	-0.08462429	;
    RnonCO2[	102	]=	-0.09275025	;
    RnonCO2[	103	]=	-0.09511911	;
    RnonCO2[	104	]=	-0.0930339	;
    RnonCO2[	105	]=	-0.09237765	;
    RnonCO2[	106	]=	-0.10165608	;
    RnonCO2[	107	]=	-0.12466661	;
    RnonCO2[	108	]=	-0.14405058	;
    RnonCO2[	109	]=	-0.14773339	;
    RnonCO2[	110	]=	-0.14777672	;
    RnonCO2[	111	]=	-0.1470811	;
    RnonCO2[	112	]=	-0.14255587	;
    RnonCO2[	113	]=	-0.14024698	;
    RnonCO2[	114	]=	-0.14393886	;
    RnonCO2[	115	]=	-0.15612866	;
    RnonCO2[	116	]=	-0.164782	;
    RnonCO2[	117	]=	-0.16302445	;
    RnonCO2[	118	]=	-0.16326054	;
    RnonCO2[	119	]=	-0.16496204	;
    RnonCO2[	120	]=	-0.16272157	;
    RnonCO2[	121	]=	-0.1612244	;
    RnonCO2[	122	]=	-0.16356025	;
    RnonCO2[	123	]=	-0.17242315	;
    RnonCO2[	124	]=	-0.17632405	;
    RnonCO2[	125	]=	-0.17855867	;
    RnonCO2[	126	]=	-0.18527891	;
    RnonCO2[	127	]=	-0.18997972	;
    RnonCO2[	128	]=	-0.1933518	;
    RnonCO2[	129	]=	-0.19699663	;
    RnonCO2[	130	]=	-0.20233059	;
    RnonCO2[	131	]=	-0.2051441	;
    RnonCO2[	132	]=	-0.20577589	;
    RnonCO2[	133	]=	-0.20720143	;
    RnonCO2[	134	]=	-0.21030464	;
    RnonCO2[	135	]=	-0.21375193	;
    RnonCO2[	136	]=	-0.22004069	;
    RnonCO2[	137	]=	-0.22693377	;
    RnonCO2[	138	]=	-0.23402972	;
    RnonCO2[	139	]=	-0.2411565	;
    RnonCO2[	140	]=	-0.2471181	;
    RnonCO2[	141	]=	-0.255172	;
    RnonCO2[	142	]=	-0.26315304	;
    RnonCO2[	143	]=	-0.2636557	;
    RnonCO2[	144	]=	-0.26151664	;
    RnonCO2[	145	]=	-0.26499499	;
    RnonCO2[	146	]=	-0.26461175	;
    RnonCO2[	147	]=	-0.26044407	;
    RnonCO2[	148	]=	-0.25940869	;
    RnonCO2[	149	]=	-0.25560119	;
    RnonCO2[	150	]=	-0.25007798	;
    RnonCO2[	151	]=	-0.25106379	;
    RnonCO2[	152	]=	-0.25437609	;
    RnonCO2[	153	]=	-0.25544669	;
    RnonCO2[	154	]=	-0.2505011	;
    RnonCO2[	155	]=	-0.24919556	;
    RnonCO2[	156	]=	-0.24766564	;
    RnonCO2[	157	]=	-0.24454174	;
    RnonCO2[	158	]=	-0.25504195	;
    RnonCO2[	159	]=	-0.26312341	;
    RnonCO2[	160	]=	-0.26314088	;
    RnonCO2[	161	]=	-0.26376161	;
    RnonCO2[	162	]=	-0.26809378	;
    RnonCO2[	163	]=	-0.27284291	;
    RnonCO2[	164	]=	-0.27584022	;
    RnonCO2[	165	]=	-0.27557411	;
    RnonCO2[	166	]=	-0.26967283	;
    RnonCO2[	167	]=	-0.26315755	;
    RnonCO2[	168	]=	-0.26083606	;
    RnonCO2[	169	]=	-0.26325195	;
    RnonCO2[	170	]=	-0.2652416	;
    RnonCO2[	171	]=	-0.26824619	;
    RnonCO2[	172	]=	-0.26930622	;
    RnonCO2[	173	]=	-0.26149511	;
    RnonCO2[	174	]=	-0.25651091	;
    RnonCO2[	175	]=	-0.26035065	;
    RnonCO2[	176	]=	-0.26656695	;
    RnonCO2[	177	]=	-0.27194936	;
    RnonCO2[	178	]=	-0.274817	;
    RnonCO2[	179	]=	-0.27878403	;
    RnonCO2[	180	]=	-0.28302569	;
    RnonCO2[	181	]=	-0.29104814	;
    RnonCO2[	182	]=	-0.30416561	;
    RnonCO2[	183	]=	-0.31716714	;
    RnonCO2[	184	]=	-0.3246442	;
    RnonCO2[	185	]=	-0.3327935	;
    RnonCO2[	186	]=	-0.35639494	;
    RnonCO2[	187	]=	-0.38051016	;
    RnonCO2[	188	]=	-0.39378465	;
    RnonCO2[	189	]=	-0.40659776	;
    RnonCO2[	190	]=	-0.4196919	;
    RnonCO2[	191	]=	-0.43302942	;
    RnonCO2[	192	]=	-0.44651364	;
    RnonCO2[	193	]=	-0.45569127	;
    RnonCO2[	194	]=	-0.45642498	;
    RnonCO2[	195	]=	-0.45551768	;
    RnonCO2[	196	]=	-0.47183293	;
    RnonCO2[	197	]=	-0.49188519	;
    RnonCO2[	198	]=	-0.50135461	;
    RnonCO2[	199	]=	-0.5101619	;
    RnonCO2[	200	]=	-0.51738459	;
    RnonCO2[	201	]=	-0.52416219	;
    RnonCO2[	202	]=	-0.53325402	;
    RnonCO2[	203	]=	-0.54124791	;
    RnonCO2[	204	]=	-0.54379714	;
    RnonCO2[	205	]=	-0.53664772	;
    RnonCO2[	206	]=	-0.52607217	;
    RnonCO2[	207	]=	-0.52739363	;
    RnonCO2[	208	]=	-0.53724776	;
    RnonCO2[	209	]=	-0.54779467	;
    RnonCO2[	210	]=	-0.55942351	;
    RnonCO2[	211	]=	-0.57578959	;
    RnonCO2[	212	]=	-0.5916702	;
    RnonCO2[	213	]=	-0.5979117	;
    RnonCO2[	214	]=	-0.6009881	;
    RnonCO2[	215	]=	-0.6131431	;
    RnonCO2[	216	]=	-0.6319195	;
    RnonCO2[	217	]=	-0.6542028	;
    RnonCO2[	218	]=	-0.6747946	;
    RnonCO2[	219	]=	-0.6865596	;
    RnonCO2[	220	]=	-0.6942006	;
    RnonCO2[	221	]=	-0.7018816	;
    RnonCO2[	222	]=	-0.7121991	;
    RnonCO2[	223	]=	-0.7210817	;
    RnonCO2[	224	]=	-0.7262596	;
    RnonCO2[	225	]=	-0.7299457	;
    RnonCO2[	226	]=	-0.7418714	;
    RnonCO2[	227	]=	-0.7583892	;
    RnonCO2[	228	]=	-0.7656833	;
    RnonCO2[	229	]=	-0.768847	;
    RnonCO2[	230	]=	-0.7655134	;
    RnonCO2[	231	]=	-0.7588829	;
    RnonCO2[	232	]=	-0.7657109	;
    RnonCO2[	233	]=	-0.7804313	;
    RnonCO2[	234	]=	-0.7902367	;
    RnonCO2[	235	]=	-0.8012413	;
    RnonCO2[	236	]=	-0.8096987	;
    RnonCO2[	237	]=	-0.8105819	;
    RnonCO2[	238	]=	-0.8112899	;
    RnonCO2[	239	]=	-0.8123564	;
    RnonCO2[	240	]=	-0.8127693	;
    RnonCO2[	241	]=	-0.8114998	;
    RnonCO2[	242	]=	-0.8100512	;
    RnonCO2[	243	]=	-0.8090083	;
    RnonCO2[	244	]=	-0.8076194	;
    RnonCO2[	245	]=	-0.8048381	;
    RnonCO2[	246	]=	-0.7996442	;
    RnonCO2[	247	]=	-0.792911	;
    RnonCO2[	248	]=	-0.7861671	;
    RnonCO2[	249	]=	-0.7797473	;
    RnonCO2[	250	]=	-0.7734094	;
    RnonCO2[	251	]=	-0.76711	;
    RnonCO2[	252	]=	-0.7608332	;
    RnonCO2[	253	]=	-0.7545757	;
    RnonCO2[	254	]=	-0.7478873	;
    RnonCO2[	255	]=	-0.7402884	;
    RnonCO2[	256	]=	-0.7314711	;
    RnonCO2[	257	]=	-0.7217277	;
    RnonCO2[	258	]=	-0.7117183	;
    RnonCO2[	259	]=	-0.7015213	;
    RnonCO2[	260	]=	-0.6911336	;
    RnonCO2[	261	]=	-0.6805672	;
    RnonCO2[	262	]=	-0.669829	;
    RnonCO2[	263	]=	-0.658928	;
    RnonCO2[	264	]=	-0.6478722	;
    RnonCO2[	265	]=	-0.6396725	;
    RnonCO2[	266	]=	-0.6373726	;
    RnonCO2[	267	]=	-0.6376422	;
    RnonCO2[	268	]=	-0.6375075	;
    RnonCO2[	269	]=	-0.637358	;
    RnonCO2[	270	]=	-0.6371818	;
    RnonCO2[	271	]=	-0.6369729	;
    RnonCO2[	272	]=	-0.636734	;
    RnonCO2[	273	]=	-0.6364677	;
    RnonCO2[	274	]=	-0.6361804	;
    RnonCO2[	275	]=	-0.634026	;
    RnonCO2[	276	]=	-0.6281673	;
    RnonCO2[	277	]=	-0.6210131	;
    RnonCO2[	278	]=	-0.6144609	;
    RnonCO2[	279	]=	-0.6080083	;
    RnonCO2[	280	]=	-0.6016499	;
    RnonCO2[	281	]=	-0.5953814	;
    RnonCO2[	282	]=	-0.5892031	;
    RnonCO2[	283	]=	-0.5831185	;
    RnonCO2[	284	]=	-0.5771411	;
    RnonCO2[	285	]=	-0.5725313	;
    RnonCO2[	286	]=	-0.5705237	;
    RnonCO2[	287	]=	-0.5698536	;
    RnonCO2[	288	]=	-0.5692522	;
    RnonCO2[	289	]=	-0.5687109	;
    RnonCO2[	290	]=	-0.5682345	;
    RnonCO2[	291	]=	-0.5678125	;
    RnonCO2[	292	]=	-0.5674364	;
    RnonCO2[	293	]=	-0.5670986	;
    RnonCO2[	294	]=	-0.5667917	;
    RnonCO2[	295	]=	-0.5632628	;
    RnonCO2[	296	]=	-0.5532447	;
    RnonCO2[	297	]=	-0.540553	;
    RnonCO2[	298	]=	-0.5284535	;
    RnonCO2[	299	]=	-0.5163756	;
    RnonCO2[	300	]=	-0.5043256	;
    RnonCO2[	301	]=	-0.4923045	;
    RnonCO2[	302	]=	-0.4802974	;
    RnonCO2[	303	]=	-0.4682919	;
    RnonCO2[	304	]=	-0.4562769	;
    RnonCO2[	305	]=	-0.4457461	;
    RnonCO2[	306	]=	-0.4382204	;
    RnonCO2[	307	]=	-0.4321175	;
    RnonCO2[	308	]=	-0.4259812	;
    RnonCO2[	309	]=	-0.4199528	;
    RnonCO2[	310	]=	-0.4140242	;
    RnonCO2[	311	]=	-0.4081992	;
    RnonCO2[	312	]=	-0.4024771	;
    RnonCO2[	313	]=	-0.3968416	;
    RnonCO2[	314	]=	-0.3912786	;
    RnonCO2[	315	]=	-0.3874316	;
    RnonCO2[	316	]=	-0.3869466	;
    RnonCO2[	317	]=	-0.387848	;
    RnonCO2[	318	]=	-0.3885955	;
    RnonCO2[	319	]=	-0.3896114	;
    RnonCO2[	320	]=	-0.3908763	;
    RnonCO2[	321	]=	-0.3923728	;
    RnonCO2[	322	]=	-0.3940965	;
    RnonCO2[	323	]=	-0.3960398	;
    RnonCO2[	324	]=	-0.3981799	;
    RnonCO2[	325	]=	-0.4003766	;
    RnonCO2[	326	]=	-0.4024909	;
    RnonCO2[	327	]=	-0.4044876	;
    RnonCO2[	328	]=	-0.4063598	;
    RnonCO2[	329	]=	-0.4081302	;
    RnonCO2[	330	]=	-0.4098073	;
    RnonCO2[	331	]=	-0.4113974	;
    RnonCO2[	332	]=	-0.4129068	;
    RnonCO2[	333	]=	-0.4143267	;
    RnonCO2[	334	]=	-0.4156335	;
    RnonCO2[	335	]=	-0.4174491	;
    RnonCO2[	336	]=	-0.4204126	;
    RnonCO2[	337	]=	-0.4233067	;
    RnonCO2[	338	]=	-0.4254415	;
    RnonCO2[	339	]=	-0.427363	;
    RnonCO2[	340	]=	-0.4290953	;
    RnonCO2[	341	]=	-0.4306596	;
    RnonCO2[	342	]=	-0.4320731	;
    RnonCO2[	343	]=	-0.4333513	;
    RnonCO2[	344	]=	-0.4345191	;
    RnonCO2[	345	]=	-0.4355948	;
    RnonCO2[	346	]=	-0.4365798	;
    RnonCO2[	347	]=	-0.4374762	;
    RnonCO2[	348	]=	-0.4382865	;
    RnonCO2[	349	]=	-0.4390119	;
    RnonCO2[	350	]=	-0.4396595	;
    RnonCO2[	351	]=	-0.4402397	;
    RnonCO2[	352	]=	-0.4407606	;
    RnonCO2[	353	]=	-0.4412283	;
    RnonCO2[	354	]=	-0.4416481	;
    RnonCO2[	355	]=	-0.4420352	;
    RnonCO2[	356	]=	-0.4424	;
    RnonCO2[	357	]=	-0.4427358	;
    RnonCO2[	358	]=	-0.4430382	;
    RnonCO2[	359	]=	-0.4433038	;
    RnonCO2[	360	]=	-0.4435315	;
    RnonCO2[	361	]=	-0.443729	;
    RnonCO2[	362	]=	-0.4439055	;
    RnonCO2[	363	]=	-0.4440656	;
    RnonCO2[	364	]=	-0.4442117	;
    RnonCO2[	365	]=	-0.4443461	;
    RnonCO2[	366	]=	-0.4444814	;
    RnonCO2[	367	]=	-0.4446253	;
    RnonCO2[	368	]=	-0.4447691	;
    RnonCO2[	369	]=	-0.4449066	;
    RnonCO2[	370	]=	-0.4450323	;
    RnonCO2[	371	]=	-0.4451409	;
    RnonCO2[	372	]=	-0.4452328	;
    RnonCO2[	373	]=	-0.4453132	;
    RnonCO2[	374	]=	-0.4453855	;
    RnonCO2[	375	]=	-0.4454513	;
    RnonCO2[	376	]=	-0.4455118	;
    RnonCO2[	377	]=	-0.4455791	;
    RnonCO2[	378	]=	-0.44566	;
    RnonCO2[	379	]=	-0.4457457	;
    RnonCO2[	380	]=	-0.4458289	;
    RnonCO2[	381	]=	-0.4459038	;
    RnonCO2[	382	]=	-0.4459649	;
    RnonCO2[	383	]=	-0.4460121	;
    RnonCO2[	384	]=	-0.4460503	;
    RnonCO2[	385	]=	-0.4460677	;
    RnonCO2[	386	]=	-0.4460677	;
    RnonCO2[	387	]=	-0.4460677	;
    RnonCO2[	388	]=	-0.4460677	;
    RnonCO2[	389	]=	-0.4460677	;
    RnonCO2[	390	]=	-0.4460677	;
    RnonCO2[	391	]=	-0.4460677	;
    RnonCO2[	392	]=	-0.4460677	;
    RnonCO2[	393	]=	-0.4460677	;
    RnonCO2[	394	]=	-0.4460677	;
    RnonCO2[	395	]=	-0.4460677	;
    RnonCO2[	396	]=	-0.4460677	;
    RnonCO2[	397	]=	-0.4460677	;
    RnonCO2[	398	]=	-0.4460677	;
    RnonCO2[	399	]=	-0.4460677	;
    RnonCO2[	400	]=	-0.4460677	;
    RnonCO2[	401	]=	-0.4460677	;
    RnonCO2[	402	]=	-0.4460677	;
    RnonCO2[	403	]=	-0.4460677	;
    RnonCO2[	404	]=	-0.4460677	;
    RnonCO2[	405	]=	-0.4460677	;
    RnonCO2[	406	]=	-0.4460677	;
    RnonCO2[	407	]=	-0.4460677	;
    RnonCO2[	408	]=	-0.4460677	;
    RnonCO2[	409	]=	-0.4460677	;
    RnonCO2[	410	]=	-0.4460677	;
    RnonCO2[	411	]=	-0.4460677	;
    RnonCO2[	412	]=	-0.4460677	;
    RnonCO2[	413	]=	-0.4460677	;
    RnonCO2[	414	]=	-0.4460677	;
    RnonCO2[	415	]=	-0.4460677	;
    RnonCO2[	416	]=	-0.4460677	;
    RnonCO2[	417	]=	-0.4460677	;
    RnonCO2[	418	]=	-0.4460677	;
    RnonCO2[	419	]=	-0.4460677	;
    RnonCO2[	420	]=	-0.4460677	;
    RnonCO2[	421	]=	-0.4460677	;
    RnonCO2[	422	]=	-0.4460677	;
    RnonCO2[	423	]=	-0.4460677	;
    RnonCO2[	424	]=	-0.4460677	;
    RnonCO2[	425	]=	-0.4460677	;
    RnonCO2[	426	]=	-0.4460677	;
    RnonCO2[	427	]=	-0.4460677	;
    RnonCO2[	428	]=	-0.4460677	;
    RnonCO2[	429	]=	-0.4460677	;
    RnonCO2[	430	]=	-0.4460677	;
    RnonCO2[	431	]=	-0.4460677	;
    RnonCO2[	432	]=	-0.4460677	;
    RnonCO2[	433	]=	-0.4460677	;
    RnonCO2[	434	]=	-0.4460677	;
    RnonCO2[	435	]=	-0.4460677	;
    RnonCO2[	436	]=	-0.4460677	;
    RnonCO2[	437	]=	-0.4460677	;
    RnonCO2[	438	]=	-0.4460677	;
    RnonCO2[	439	]=	-0.4460677	;
    RnonCO2[	440	]=	-0.4460677	;
    RnonCO2[	441	]=	-0.4460677	;
    RnonCO2[	442	]=	-0.4460677	;
    RnonCO2[	443	]=	-0.4460677	;
    RnonCO2[	444	]=	-0.4460677	;
    RnonCO2[	445	]=	-0.4460677	;
    RnonCO2[	446	]=	-0.4460677	;
    RnonCO2[	447	]=	-0.4460677	;
    RnonCO2[	448	]=	-0.4460677	;
    RnonCO2[	449	]=	-0.4460677	;
    RnonCO2[	450	]=	-0.4460677	;
    RnonCO2[	451	]=	-0.4460677	;
    RnonCO2[	452	]=	-0.4460677	;
    RnonCO2[	453	]=	-0.4460677	;
    RnonCO2[	454	]=	-0.4460677	;
    RnonCO2[	455	]=	-0.4460677	;
    RnonCO2[	456	]=	-0.4460677	;
    RnonCO2[	457	]=	-0.4460677	;
    RnonCO2[	458	]=	-0.4460677	;
    RnonCO2[	459	]=	-0.4460677	;
    RnonCO2[	460	]=	-0.4460677	;
    RnonCO2[	461	]=	-0.4460677	;
    RnonCO2[	462	]=	-0.4460677	;
    RnonCO2[	463	]=	-0.4460677	;
    RnonCO2[	464	]=	-0.4460677	;
    RnonCO2[	465	]=	-0.4460677	;
    RnonCO2[	466	]=	-0.4460677	;
    RnonCO2[	467	]=	-0.4460677	;
    RnonCO2[	468	]=	-0.4460677	;
    RnonCO2[	469	]=	-0.4460677	;
    RnonCO2[	470	]=	-0.4460677	;
    RnonCO2[	471	]=	-0.4460677	;
    RnonCO2[	472	]=	-0.4460677	;
    RnonCO2[	473	]=	-0.4460677	;
    RnonCO2[	474	]=	-0.4460677	;
    RnonCO2[	475	]=	-0.4460677	;
    RnonCO2[	476	]=	-0.4460677	;
    RnonCO2[	477	]=	-0.4460677	;
    RnonCO2[	478	]=	-0.4460677	;
    RnonCO2[	479	]=	-0.4460677	;
    RnonCO2[	480	]=	-0.4460677	;
    RnonCO2[	481	]=	-0.4460677	;
    RnonCO2[	482	]=	-0.4460677	;
    RnonCO2[	483	]=	-0.4460677	;
    RnonCO2[	484	]=	-0.4460677	;
    RnonCO2[	485	]=	-0.4460677	;
    RnonCO2[	486	]=	-0.4460677	;
    RnonCO2[	487	]=	-0.4460677	;
    RnonCO2[	488	]=	-0.4460677	;
    RnonCO2[	489	]=	-0.4460677	;
    RnonCO2[	490	]=	-0.4460677	;
    RnonCO2[	491	]=	-0.4460677	;
    RnonCO2[	492	]=	-0.4460677	;
    RnonCO2[	493	]=	-0.4460677	;
    RnonCO2[	494	]=	-0.4460677	;
    RnonCO2[	495	]=	-0.4460677	;
    RnonCO2[	496	]=	-0.4460677	;
    RnonCO2[	497	]=	-0.4460677	;
    RnonCO2[	498	]=	-0.4460677	;
    RnonCO2[	499	]=	-0.4460677	;
    RnonCO2[	500	]=	-0.4460677	;
    RnonCO2[	501	]=	-0.4460677	;
    RnonCO2[	502	]=	-0.4460677	;
    RnonCO2[	503	]=	-0.4460677	;
    RnonCO2[	504	]=	-0.4460677	;
    RnonCO2[	505	]=	-0.4460677	;
    RnonCO2[	506	]=	-0.4460677	;
    RnonCO2[	507	]=	-0.4460677	;
    RnonCO2[	508	]=	-0.4460677	;
    RnonCO2[	509	]=	-0.4460677	;
    RnonCO2[	510	]=	-0.4460677	;
    RnonCO2[	511	]=	-0.4460677	;
    RnonCO2[	512	]=	-0.4460677	;
    RnonCO2[	513	]=	-0.4460677	;
    RnonCO2[	514	]=	-0.4460677	;
    RnonCO2[	515	]=	-0.4460677	;
    RnonCO2[	516	]=	-0.4460677	;
    RnonCO2[	517	]=	-0.4460677	;
    RnonCO2[	518	]=	-0.4460677	;
    RnonCO2[	519	]=	-0.4460677	;
    RnonCO2[	520	]=	-0.4460677	;
    RnonCO2[	521	]=	-0.4460677	;
    RnonCO2[	522	]=	-0.4460677	;
    RnonCO2[	523	]=	-0.4460677	;
    RnonCO2[	524	]=	-0.4460677	;
    RnonCO2[	525	]=	-0.4460677	;
    RnonCO2[	526	]=	-0.4460677	;
    RnonCO2[	527	]=	-0.4460677	;
    RnonCO2[	528	]=	-0.4460677	;
    RnonCO2[	529	]=	-0.4460677	;
    RnonCO2[	530	]=	-0.4460677	;
    RnonCO2[	531	]=	-0.4460677	;
    RnonCO2[	532	]=	-0.4460677	;
    RnonCO2[	533	]=	-0.4460677	;
    RnonCO2[	534	]=	-0.4460677	;
    RnonCO2[	535	]=	-0.4460677	;
    RnonCO2[	536	]=	-0.4460677	;
    RnonCO2[	537	]=	-0.4460677	;
    RnonCO2[	538	]=	-0.4460677	;
    RnonCO2[	539	]=	-0.4460677	;
    RnonCO2[	540	]=	-0.4460677	;
    RnonCO2[	541	]=	-0.4460677	;
    RnonCO2[	542	]=	-0.4460677	;
    RnonCO2[	543	]=	-0.4460677	;
    RnonCO2[	544	]=	-0.4460677	;
    RnonCO2[	545	]=	-0.4460677	;
    RnonCO2[	546	]=	-0.4460677	;
    RnonCO2[	547	]=	-0.4460677	;
    RnonCO2[	548	]=	-0.4460677	;
    RnonCO2[	549	]=	-0.4460677	;
    RnonCO2[	550	]=	-0.4460677	;
    RnonCO2[	551	]=	-0.4460677	;
    RnonCO2[	552	]=	-0.4460677	;
    RnonCO2[	553	]=	-0.4460677	;
    RnonCO2[	554	]=	-0.4460677	;
    RnonCO2[	555	]=	-0.4460677	;
    RnonCO2[	556	]=	-0.4460677	;
    RnonCO2[	557	]=	-0.4460677	;
    RnonCO2[	558	]=	-0.4460677	;
    RnonCO2[	559	]=	-0.4460677	;
    RnonCO2[	560	]=	-0.4460677	;
    RnonCO2[	561	]=	-0.4460677	;
    RnonCO2[	562	]=	-0.4460677	;
    RnonCO2[	563	]=	-0.4460677	;
    RnonCO2[	564	]=	-0.4460677	;
    RnonCO2[	565	]=	-0.4460677	;
    RnonCO2[	566	]=	-0.4460677	;
    RnonCO2[	567	]=	-0.4460677	;
    RnonCO2[	568	]=	-0.4460677	;
    RnonCO2[	569	]=	-0.4460677	;
    RnonCO2[	570	]=	-0.4460677	;
    RnonCO2[	571	]=	-0.4460677	;
    RnonCO2[	572	]=	-0.4460677	;
    RnonCO2[	573	]=	-0.4460677	;
    RnonCO2[	574	]=	-0.4460677	;
    RnonCO2[	575	]=	-0.4460677	;
    RnonCO2[	576	]=	-0.4460677	;
    RnonCO2[	577	]=	-0.4460677	;
    RnonCO2[	578	]=	-0.4460677	;
    RnonCO2[	579	]=	-0.4460677	;
    RnonCO2[	580	]=	-0.4460677	;
    RnonCO2[	581	]=	-0.4460677	;
    RnonCO2[	582	]=	-0.4460677	;
    RnonCO2[	583	]=	-0.4460677	;
    RnonCO2[	584	]=	-0.4460677	;
    RnonCO2[	585	]=	-0.4460677	;
    RnonCO2[	586	]=	-0.4460677	;
    RnonCO2[	587	]=	-0.4460677	;
    RnonCO2[	588	]=	-0.4460677	;
    RnonCO2[	589	]=	-0.4460677	;
    RnonCO2[	590	]=	-0.4460677	;
    RnonCO2[	591	]=	-0.4460677	;
    RnonCO2[	592	]=	-0.4460677	;
    RnonCO2[	593	]=	-0.4460677	;
    RnonCO2[	594	]=	-0.4460677	;
    RnonCO2[	595	]=	-0.4460677	;
    RnonCO2[	596	]=	-0.4460677	;
    RnonCO2[	597	]=	-0.4460677	;
    RnonCO2[	598	]=	-0.4460677	;
    RnonCO2[	599	]=	-0.4460677	;
    RnonCO2[	600	]=	-0.4460677	;
    RnonCO2[	601	]=	-0.4460677	;
    RnonCO2[	602	]=	-0.4460677	;
    RnonCO2[	603	]=	-0.4460677	;
    RnonCO2[	604	]=	-0.4460677	;
    RnonCO2[	605	]=	-0.4460677	;
    RnonCO2[	606	]=	-0.4460677	;
    RnonCO2[	607	]=	-0.4460677	;
    RnonCO2[	608	]=	-0.4460677	;
    RnonCO2[	609	]=	-0.4460677	;
    RnonCO2[	610	]=	-0.4460677	;
    RnonCO2[	611	]=	-0.4460677	;
    RnonCO2[	612	]=	-0.4460677	;
    RnonCO2[	613	]=	-0.4460677	;
    RnonCO2[	614	]=	-0.4460677	;
    RnonCO2[	615	]=	-0.4460677	;
    RnonCO2[	616	]=	-0.4460677	;
    RnonCO2[	617	]=	-0.4460677	;
    RnonCO2[	618	]=	-0.4460677	;
    RnonCO2[	619	]=	-0.4460677	;
    RnonCO2[	620	]=	-0.4460677	;
    RnonCO2[	621	]=	-0.4460677	;
    RnonCO2[	622	]=	-0.4460677	;
    RnonCO2[	623	]=	-0.4460677	;
    RnonCO2[	624	]=	-0.4460677	;
    RnonCO2[	625	]=	-0.4460677	;
    RnonCO2[	626	]=	-0.4460677	;
    RnonCO2[	627	]=	-0.4460677	;
    RnonCO2[	628	]=	-0.4460677	;
    RnonCO2[	629	]=	-0.4460677	;
    RnonCO2[	630	]=	-0.4460677	;
    RnonCO2[	631	]=	-0.4460677	;
    RnonCO2[	632	]=	-0.4460677	;
    RnonCO2[	633	]=	-0.4460677	;
    RnonCO2[	634	]=	-0.4460677	;
    RnonCO2[	635	]=	-0.4460677	;
    RnonCO2[	636	]=	-0.4460677	;
    RnonCO2[	637	]=	-0.4460677	;
    RnonCO2[	638	]=	-0.4460677	;
    RnonCO2[	639	]=	-0.4460677	;
    RnonCO2[	640	]=	-0.4460677	;
    RnonCO2[	641	]=	-0.4460677	;
    RnonCO2[	642	]=	-0.4460677	;
    RnonCO2[	643	]=	-0.4460677	;
    RnonCO2[	644	]=	-0.4460677	;
    RnonCO2[	645	]=	-0.4460677	;
    RnonCO2[	646	]=	-0.4460677	;
    RnonCO2[	647	]=	-0.4460677	;
    RnonCO2[	648	]=	-0.4460677	;
    RnonCO2[	649	]=	-0.4460677	;
    RnonCO2[	650	]=	-0.4460677	;
    RnonCO2[	651	]=	-0.4460677	;
    RnonCO2[	652	]=	-0.4460677	;
    RnonCO2[	653	]=	-0.4460677	;
    RnonCO2[	654	]=	-0.4460677	;
    RnonCO2[	655	]=	-0.4460677	;
    RnonCO2[	656	]=	-0.4460677	;
    RnonCO2[	657	]=	-0.4460677	;
    RnonCO2[	658	]=	-0.4460677	;
    RnonCO2[	659	]=	-0.4460677	;
    RnonCO2[	660	]=	-0.4460677	;
    RnonCO2[	661	]=	-0.4460677	;
    RnonCO2[	662	]=	-0.4460677	;
    RnonCO2[	663	]=	-0.4460677	;
    RnonCO2[	664	]=	-0.4460677	;
    RnonCO2[	665	]=	-0.4460677	;
    RnonCO2[	666	]=	-0.4460677	;
    RnonCO2[	667	]=	-0.4460677	;
    RnonCO2[	668	]=	-0.4460677	;
    RnonCO2[	669	]=	-0.4460677	;
    RnonCO2[	670	]=	-0.4460677	;
    RnonCO2[	671	]=	-0.4460677	;
    RnonCO2[	672	]=	-0.4460677	;
    RnonCO2[	673	]=	-0.4460677	;
    RnonCO2[	674	]=	-0.4460677	;
    RnonCO2[	675	]=	-0.4460677	;
    RnonCO2[	676	]=	-0.4460677	;
    RnonCO2[	677	]=	-0.4460677	;
    RnonCO2[	678	]=	-0.4460677	;
    RnonCO2[	679	]=	-0.4460677	;
    RnonCO2[	680	]=	-0.4460677	;
    RnonCO2[	681	]=	-0.4460677	;
    RnonCO2[	682	]=	-0.4460677	;
    RnonCO2[	683	]=	-0.4460677	;
    RnonCO2[	684	]=	-0.4460677	;
    RnonCO2[	685	]=	-0.4460677	;
    RnonCO2[	686	]=	-0.4460677	;
    RnonCO2[	687	]=	-0.4460677	;
    RnonCO2[	688	]=	-0.4460677	;
    RnonCO2[	689	]=	-0.4460677	;
    RnonCO2[	690	]=	-0.4460677	;
    RnonCO2[	691	]=	-0.4460677	;
    RnonCO2[	692	]=	-0.4460677	;
    RnonCO2[	693	]=	-0.4460677	;
    RnonCO2[	694	]=	-0.4460677	;
    RnonCO2[	695	]=	-0.4460677	;
    RnonCO2[	696	]=	-0.4460677	;
    RnonCO2[	697	]=	-0.4460677	;
    RnonCO2[	698	]=	-0.4460677	;
    RnonCO2[	699	]=	-0.4460677	;
    RnonCO2[	700	]=	-0.4460677	;
    RnonCO2[	701	]=	-0.4460677	;
    RnonCO2[	702	]=	-0.4460677	;
    RnonCO2[	703	]=	-0.4460677	;
    RnonCO2[	704	]=	-0.4460677	;
    RnonCO2[	705	]=	-0.4460677	;
    RnonCO2[	706	]=	-0.4460677	;
    RnonCO2[	707	]=	-0.4460677	;
    RnonCO2[	708	]=	-0.4460677	;
    RnonCO2[	709	]=	-0.4460677	;
    RnonCO2[	710	]=	-0.4460677	;
    RnonCO2[	711	]=	-0.4460677	;
    RnonCO2[	712	]=	-0.4460677	;
    RnonCO2[	713	]=	-0.4460677	;
    RnonCO2[	714	]=	-0.4460677	;
    RnonCO2[	715	]=	-0.4460677	;
    RnonCO2[	716	]=	-0.4460677	;
    RnonCO2[	717	]=	-0.4460677	;
    RnonCO2[	718	]=	-0.4460677	;
    RnonCO2[	719	]=	-0.4460677	;
    RnonCO2[	720	]=	-0.4460677	;
    RnonCO2[	721	]=	-0.4460677	;
    RnonCO2[	722	]=	-0.4460677	;
    RnonCO2[	723	]=	-0.4460677	;
    RnonCO2[	724	]=	-0.4460677	;
    RnonCO2[	725	]=	-0.4460677	;
    RnonCO2[	726	]=	-0.4460677	;
    RnonCO2[	727	]=	-0.4460677	;
    RnonCO2[	728	]=	-0.4460677	;
    RnonCO2[	729	]=	-0.4460677	;
    RnonCO2[	730	]=	-0.4460677	;
    RnonCO2[	731	]=	-0.4460677	;
    RnonCO2[	732	]=	-0.4460677	;
    RnonCO2[	733	]=	-0.4460677	;
    RnonCO2[	734	]=	-0.4460677	;
    RnonCO2[	735	]=	-0.4460677	;
    
    for(int i = 736; i<tmax; i++)
    {
        
        RnonCO2[i] = RnonCO2[i-1];
        
    }
    
}

void setR_aerosol_RCP45(void)
{
    RnonCO2[	0	]=	0	;
    RnonCO2[	1	]=	0.000955298	;
    RnonCO2[	2	]=	0.001424721	;
    RnonCO2[	3	]=	0.0001974	;
    RnonCO2[	4	]=	-0.001278437	;
    RnonCO2[	5	]=	-0.002733841	;
    RnonCO2[	6	]=	-0.004541334	;
    RnonCO2[	7	]=	-0.00632004	;
    RnonCO2[	8	]=	-0.007680948	;
    RnonCO2[	9	]=	-0.008994568	;
    RnonCO2[	10	]=	-0.010260775	;
    RnonCO2[	11	]=	-0.011479866	;
    RnonCO2[	12	]=	-0.012652425	;
    RnonCO2[	13	]=	-0.013779231	;
    RnonCO2[	14	]=	-0.014861204	;
    RnonCO2[	15	]=	-0.015900176	;
    RnonCO2[	16	]=	-0.017335746	;
    RnonCO2[	17	]=	-0.01873479	;
    RnonCO2[	18	]=	-0.019659073	;
    RnonCO2[	19	]=	-0.020540613	;
    RnonCO2[	20	]=	-0.02137714	;
    RnonCO2[	21	]=	-0.022166399	;
    RnonCO2[	22	]=	-0.022906132	;
    RnonCO2[	23	]=	-0.023594082	;
    RnonCO2[	24	]=	-0.024227995	;
    RnonCO2[	25	]=	-0.024806236	;
    RnonCO2[	26	]=	-0.025775743	;
    RnonCO2[	27	]=	-0.026691982	;
    RnonCO2[	28	]=	-0.027110281	;
    RnonCO2[	29	]=	-0.02747885	;
    RnonCO2[	30	]=	-0.027799526	;
    RnonCO2[	31	]=	-0.028074106	;
    RnonCO2[	32	]=	-0.02875039	;
    RnonCO2[	33	]=	-0.029383792	;
    RnonCO2[	34	]=	-0.02953002	;
    RnonCO2[	35	]=	-0.03007996	;
    RnonCO2[	36	]=	-0.03058546	;
    RnonCO2[	37	]=	-0.03148571	;
    RnonCO2[	38	]=	-0.03189741	;
    RnonCO2[	39	]=	-0.03138062	;
    RnonCO2[	40	]=	-0.03126848	;
    RnonCO2[	41	]=	-0.03156144	;
    RnonCO2[	42	]=	-0.03182181	;
    RnonCO2[	43	]=	-0.03161267	;
    RnonCO2[	44	]=	-0.03137821	;
    RnonCO2[	45	]=	-0.03112038	;
    RnonCO2[	46	]=	-0.03127367	;
    RnonCO2[	47	]=	-0.0313998	;
    RnonCO2[	48	]=	-0.03106322	;
    RnonCO2[	49	]=	-0.03070135	;
    RnonCO2[	50	]=	-0.03074616	;
    RnonCO2[	51	]=	-0.03119614	;
    RnonCO2[	52	]=	-0.03161872	;
    RnonCO2[	53	]=	-0.03158785	;
    RnonCO2[	54	]=	-0.03110696	;
    RnonCO2[	55	]=	-0.03060634	;
    RnonCO2[	56	]=	-0.0300866	;
    RnonCO2[	57	]=	-0.02996996	;
    RnonCO2[	58	]=	-0.03025404	;
    RnonCO2[	59	]=	-0.03009594	;
    RnonCO2[	60	]=	-0.02991636	;
    RnonCO2[	61	]=	-0.02971869	;
    RnonCO2[	62	]=	-0.02950042	;
    RnonCO2[	63	]=	-0.02926502	;
    RnonCO2[	64	]=	-0.02859871	;
    RnonCO2[	65	]=	-0.03037359	;
    RnonCO2[	66	]=	-0.03172034	;
    RnonCO2[	67	]=	-0.03059392	;
    RnonCO2[	68	]=	-0.03026959	;
    RnonCO2[	69	]=	-0.02993194	;
    RnonCO2[	70	]=	-0.02957653	;
    RnonCO2[	71	]=	-0.03080027	;
    RnonCO2[	72	]=	-0.03160022	;
    RnonCO2[	73	]=	-0.03117789	;
    RnonCO2[	74	]=	-0.03112982	;
    RnonCO2[	75	]=	-0.03144654	;
    RnonCO2[	76	]=	-0.03173222	;
    RnonCO2[	77	]=	-0.03198224	;
    RnonCO2[	78	]=	-0.03220368	;
    RnonCO2[	79	]=	-0.03239696	;
    RnonCO2[	80	]=	-0.03371239	;
    RnonCO2[	81	]=	-0.03424533	;
    RnonCO2[	82	]=	-0.0343764	;
    RnonCO2[	83	]=	-0.03487942	;
    RnonCO2[	84	]=	-0.03536957	;
    RnonCO2[	85	]=	-0.03686306	;
    RnonCO2[	86	]=	-0.03665392	;
    RnonCO2[	87	]=	-0.04248424	;
    RnonCO2[	88	]=	-0.04930444	;
    RnonCO2[	89	]=	-0.05182621	;
    RnonCO2[	90	]=	-0.0542671	;
    RnonCO2[	91	]=	-0.05573158	;
    RnonCO2[	92	]=	-0.05765815	;
    RnonCO2[	93	]=	-0.05821397	;
    RnonCO2[	94	]=	-0.06005825	;
    RnonCO2[	95	]=	-0.0683501	;
    RnonCO2[	96	]=	-0.08104112	;
    RnonCO2[	97	]=	-0.07832852	;
    RnonCO2[	98	]=	-0.07177024	;
    RnonCO2[	99	]=	-0.07363036	;
    RnonCO2[	100	]=	-0.07696442	;
    RnonCO2[	101	]=	-0.08462429	;
    RnonCO2[	102	]=	-0.09275025	;
    RnonCO2[	103	]=	-0.09511911	;
    RnonCO2[	104	]=	-0.0930339	;
    RnonCO2[	105	]=	-0.09237765	;
    RnonCO2[	106	]=	-0.10165608	;
    RnonCO2[	107	]=	-0.12466661	;
    RnonCO2[	108	]=	-0.14405058	;
    RnonCO2[	109	]=	-0.14773339	;
    RnonCO2[	110	]=	-0.14777672	;
    RnonCO2[	111	]=	-0.1470811	;
    RnonCO2[	112	]=	-0.14255587	;
    RnonCO2[	113	]=	-0.14024698	;
    RnonCO2[	114	]=	-0.14393886	;
    RnonCO2[	115	]=	-0.15612866	;
    RnonCO2[	116	]=	-0.164782	;
    RnonCO2[	117	]=	-0.16302445	;
    RnonCO2[	118	]=	-0.16326054	;
    RnonCO2[	119	]=	-0.16496204	;
    RnonCO2[	120	]=	-0.16272157	;
    RnonCO2[	121	]=	-0.1612244	;
    RnonCO2[	122	]=	-0.16356025	;
    RnonCO2[	123	]=	-0.17242315	;
    RnonCO2[	124	]=	-0.17632405	;
    RnonCO2[	125	]=	-0.17855867	;
    RnonCO2[	126	]=	-0.18527891	;
    RnonCO2[	127	]=	-0.18997972	;
    RnonCO2[	128	]=	-0.1933518	;
    RnonCO2[	129	]=	-0.19699663	;
    RnonCO2[	130	]=	-0.20233059	;
    RnonCO2[	131	]=	-0.2051441	;
    RnonCO2[	132	]=	-0.20577589	;
    RnonCO2[	133	]=	-0.20720143	;
    RnonCO2[	134	]=	-0.21030464	;
    RnonCO2[	135	]=	-0.21375193	;
    RnonCO2[	136	]=	-0.22004069	;
    RnonCO2[	137	]=	-0.22693377	;
    RnonCO2[	138	]=	-0.23402972	;
    RnonCO2[	139	]=	-0.2411565	;
    RnonCO2[	140	]=	-0.2471181	;
    RnonCO2[	141	]=	-0.255172	;
    RnonCO2[	142	]=	-0.26315304	;
    RnonCO2[	143	]=	-0.2636557	;
    RnonCO2[	144	]=	-0.26151664	;
    RnonCO2[	145	]=	-0.26499499	;
    RnonCO2[	146	]=	-0.26461175	;
    RnonCO2[	147	]=	-0.26044407	;
    RnonCO2[	148	]=	-0.25940869	;
    RnonCO2[	149	]=	-0.25560119	;
    RnonCO2[	150	]=	-0.25007798	;
    RnonCO2[	151	]=	-0.25106379	;
    RnonCO2[	152	]=	-0.25437609	;
    RnonCO2[	153	]=	-0.25544669	;
    RnonCO2[	154	]=	-0.2505011	;
    RnonCO2[	155	]=	-0.24919556	;
    RnonCO2[	156	]=	-0.24766564	;
    RnonCO2[	157	]=	-0.24454174	;
    RnonCO2[	158	]=	-0.25504195	;
    RnonCO2[	159	]=	-0.26312341	;
    RnonCO2[	160	]=	-0.26314088	;
    RnonCO2[	161	]=	-0.26376161	;
    RnonCO2[	162	]=	-0.26809378	;
    RnonCO2[	163	]=	-0.27284291	;
    RnonCO2[	164	]=	-0.27584022	;
    RnonCO2[	165	]=	-0.27557411	;
    RnonCO2[	166	]=	-0.26967283	;
    RnonCO2[	167	]=	-0.26315755	;
    RnonCO2[	168	]=	-0.26083606	;
    RnonCO2[	169	]=	-0.26325195	;
    RnonCO2[	170	]=	-0.2652416	;
    RnonCO2[	171	]=	-0.26824619	;
    RnonCO2[	172	]=	-0.26930622	;
    RnonCO2[	173	]=	-0.26149511	;
    RnonCO2[	174	]=	-0.25651091	;
    RnonCO2[	175	]=	-0.26035065	;
    RnonCO2[	176	]=	-0.26656695	;
    RnonCO2[	177	]=	-0.27194936	;
    RnonCO2[	178	]=	-0.274817	;
    RnonCO2[	179	]=	-0.27878403	;
    RnonCO2[	180	]=	-0.28302569	;
    RnonCO2[	181	]=	-0.29104814	;
    RnonCO2[	182	]=	-0.30416561	;
    RnonCO2[	183	]=	-0.31716714	;
    RnonCO2[	184	]=	-0.3246442	;
    RnonCO2[	185	]=	-0.3327935	;
    RnonCO2[	186	]=	-0.35639494	;
    RnonCO2[	187	]=	-0.38051016	;
    RnonCO2[	188	]=	-0.39378465	;
    RnonCO2[	189	]=	-0.40659776	;
    RnonCO2[	190	]=	-0.4196919	;
    RnonCO2[	191	]=	-0.43302942	;
    RnonCO2[	192	]=	-0.44651364	;
    RnonCO2[	193	]=	-0.45569127	;
    RnonCO2[	194	]=	-0.45642498	;
    RnonCO2[	195	]=	-0.45551768	;
    RnonCO2[	196	]=	-0.47183293	;
    RnonCO2[	197	]=	-0.49188519	;
    RnonCO2[	198	]=	-0.50135461	;
    RnonCO2[	199	]=	-0.5101619	;
    RnonCO2[	200	]=	-0.51738459	;
    RnonCO2[	201	]=	-0.52416219	;
    RnonCO2[	202	]=	-0.53325402	;
    RnonCO2[	203	]=	-0.54124791	;
    RnonCO2[	204	]=	-0.54379714	;
    RnonCO2[	205	]=	-0.53664772	;
    RnonCO2[	206	]=	-0.52607217	;
    RnonCO2[	207	]=	-0.52739363	;
    RnonCO2[	208	]=	-0.53724776	;
    RnonCO2[	209	]=	-0.54779467	;
    RnonCO2[	210	]=	-0.55942351	;
    RnonCO2[	211	]=	-0.57578959	;
    RnonCO2[	212	]=	-0.5916702	;
    RnonCO2[	213	]=	-0.5979117	;
    RnonCO2[	214	]=	-0.6009881	;
    RnonCO2[	215	]=	-0.6131431	;
    RnonCO2[	216	]=	-0.6319195	;
    RnonCO2[	217	]=	-0.6542028	;
    RnonCO2[	218	]=	-0.6747946	;
    RnonCO2[	219	]=	-0.6865596	;
    RnonCO2[	220	]=	-0.6942006	;
    RnonCO2[	221	]=	-0.7018816	;
    RnonCO2[	222	]=	-0.7121991	;
    RnonCO2[	223	]=	-0.7210817	;
    RnonCO2[	224	]=	-0.7262596	;
    RnonCO2[	225	]=	-0.7299457	;
    RnonCO2[	226	]=	-0.7418714	;
    RnonCO2[	227	]=	-0.7583892	;
    RnonCO2[	228	]=	-0.7656833	;
    RnonCO2[	229	]=	-0.768847	;
    RnonCO2[	230	]=	-0.7655134	;
    RnonCO2[	231	]=	-0.7588829	;
    RnonCO2[	232	]=	-0.7657109	;
    RnonCO2[	233	]=	-0.7804313	;
    RnonCO2[	234	]=	-0.7902367	;
    RnonCO2[	235	]=	-0.8012413	;
    RnonCO2[	236	]=	-0.8096987	;
    RnonCO2[	237	]=	-0.8105819	;
    RnonCO2[	238	]=	-0.8112899	;
    RnonCO2[	239	]=	-0.8123564	;
    RnonCO2[	240	]=	-0.8108507	;
    RnonCO2[	241	]=	-0.8038388	;
    RnonCO2[	242	]=	-0.7949639	;
    RnonCO2[	243	]=	-0.7867463	;
    RnonCO2[	244	]=	-0.7782199	;
    RnonCO2[	245	]=	-0.7696101	;
    RnonCO2[	246	]=	-0.7611859	;
    RnonCO2[	247	]=	-0.7523398	;
    RnonCO2[	248	]=	-0.7432371	;
    RnonCO2[	249	]=	-0.7343269	;
    RnonCO2[	250	]=	-0.7253724	;
    RnonCO2[	251	]=	-0.7163378	;
    RnonCO2[	252	]=	-0.7072163	;
    RnonCO2[	253	]=	-0.698012	;
    RnonCO2[	254	]=	-0.6882828	;
    RnonCO2[	255	]=	-0.6789353	;
    RnonCO2[	256	]=	-0.6710819	;
    RnonCO2[	257	]=	-0.6634482	;
    RnonCO2[	258	]=	-0.6554378	;
    RnonCO2[	259	]=	-0.6474739	;
    RnonCO2[	260	]=	-0.6395458	;
    RnonCO2[	261	]=	-0.6316558	;
    RnonCO2[	262	]=	-0.6238013	;
    RnonCO2[	263	]=	-0.6159833	;
    RnonCO2[	264	]=	-0.6082026	;
    RnonCO2[	265	]=	-0.6001577	;
    RnonCO2[	266	]=	-0.5915484	;
    RnonCO2[	267	]=	-0.5830978	;
    RnonCO2[	268	]=	-0.575125	;
    RnonCO2[	269	]=	-0.5672197	;
    RnonCO2[	270	]=	-0.5593621	;
    RnonCO2[	271	]=	-0.5515396	;
    RnonCO2[	272	]=	-0.5437488	;
    RnonCO2[	273	]=	-0.5359863	;
    RnonCO2[	274	]=	-0.5282529	;
    RnonCO2[	275	]=	-0.5202403	;
    RnonCO2[	276	]=	-0.5116363	;
    RnonCO2[	277	]=	-0.5031575	;
    RnonCO2[	278	]=	-0.4951399	;
    RnonCO2[	279	]=	-0.4871949	;
    RnonCO2[	280	]=	-0.4793139	;
    RnonCO2[	281	]=	-0.4714904	;
    RnonCO2[	282	]=	-0.4637225	;
    RnonCO2[	283	]=	-0.4560139	;
    RnonCO2[	284	]=	-0.4483667	;
    RnonCO2[	285	]=	-0.4418114	;
    RnonCO2[	286	]=	-0.4373793	;
    RnonCO2[	287	]=	-0.4340576	;
    RnonCO2[	288	]=	-0.4307923	;
    RnonCO2[	289	]=	-0.4275505	;
    RnonCO2[	290	]=	-0.4243342	;
    RnonCO2[	291	]=	-0.4211294	;
    RnonCO2[	292	]=	-0.4179249	;
    RnonCO2[	293	]=	-0.4147107	;
    RnonCO2[	294	]=	-0.4114766	;
    RnonCO2[	295	]=	-0.4084679	;
    RnonCO2[	296	]=	-0.4059352	;
    RnonCO2[	297	]=	-0.403637	;
    RnonCO2[	298	]=	-0.4013333	;
    RnonCO2[	299	]=	-0.399025	;
    RnonCO2[	300	]=	-0.39672	;
    RnonCO2[	301	]=	-0.3944212	;
    RnonCO2[	302	]=	-0.3921159	;
    RnonCO2[	303	]=	-0.3897938	;
    RnonCO2[	304	]=	-0.3874458	;
    RnonCO2[	305	]=	-0.3853167	;
    RnonCO2[	306	]=	-0.3836585	;
    RnonCO2[	307	]=	-0.382227	;
    RnonCO2[	308	]=	-0.3807793	;
    RnonCO2[	309	]=	-0.3793168	;
    RnonCO2[	310	]=	-0.3778368	;
    RnonCO2[	311	]=	-0.3763473	;
    RnonCO2[	312	]=	-0.3748516	;
    RnonCO2[	313	]=	-0.3733347	;
    RnonCO2[	314	]=	-0.3717663	;
    RnonCO2[	315	]=	-0.3711127	;
    RnonCO2[	316	]=	-0.3723784	;
    RnonCO2[	317	]=	-0.3738112	;
    RnonCO2[	318	]=	-0.3744043	;
    RnonCO2[	319	]=	-0.3749248	;
    RnonCO2[	320	]=	-0.375382	;
    RnonCO2[	321	]=	-0.3757837	;
    RnonCO2[	322	]=	-0.3761478	;
    RnonCO2[	323	]=	-0.3764863	;
    RnonCO2[	324	]=	-0.3767948	;
    RnonCO2[	325	]=	-0.3770756	;
    RnonCO2[	326	]=	-0.3773313	;
    RnonCO2[	327	]=	-0.377555	;
    RnonCO2[	328	]=	-0.3777453	;
    RnonCO2[	329	]=	-0.3779096	;
    RnonCO2[	330	]=	-0.3780531	;
    RnonCO2[	331	]=	-0.3781791	;
    RnonCO2[	332	]=	-0.3782905	;
    RnonCO2[	333	]=	-0.3784007	;
    RnonCO2[	334	]=	-0.3785174	;
    RnonCO2[	335	]=	-0.3788718	;
    RnonCO2[	336	]=	-0.3796964	;
    RnonCO2[	337	]=	-0.3805122	;
    RnonCO2[	338	]=	-0.3810503	;
    RnonCO2[	339	]=	-0.3815251	;
    RnonCO2[	340	]=	-0.3819464	;
    RnonCO2[	341	]=	-0.3823224	;
    RnonCO2[	342	]=	-0.3826584	;
    RnonCO2[	343	]=	-0.3829595	;
    RnonCO2[	344	]=	-0.3832407	;
    RnonCO2[	345	]=	-0.3835115	;
    RnonCO2[	346	]=	-0.3837653	;
    RnonCO2[	347	]=	-0.3839978	;
    RnonCO2[	348	]=	-0.3842056	;
    RnonCO2[	349	]=	-0.3843847	;
    RnonCO2[	350	]=	-0.3845373	;
    RnonCO2[	351	]=	-0.384669	;
    RnonCO2[	352	]=	-0.3847842	;
    RnonCO2[	353	]=	-0.3848851	;
    RnonCO2[	354	]=	-0.3849739	;
    RnonCO2[	355	]=	-0.3850633	;
    RnonCO2[	356	]=	-0.3851609	;
    RnonCO2[	357	]=	-0.3852587	;
    RnonCO2[	358	]=	-0.3853504	;
    RnonCO2[	359	]=	-0.3854312	;
    RnonCO2[	360	]=	-0.3854929	;
    RnonCO2[	361	]=	-0.3855371	;
    RnonCO2[	362	]=	-0.3855717	;
    RnonCO2[	363	]=	-0.3856003	;
    RnonCO2[	364	]=	-0.3856241	;
    RnonCO2[	365	]=	-0.3856445	;
    RnonCO2[	366	]=	-0.3856732	;
    RnonCO2[	367	]=	-0.3857168	;
    RnonCO2[	368	]=	-0.3857662	;
    RnonCO2[	369	]=	-0.3858148	;
    RnonCO2[	370	]=	-0.3858573	;
    RnonCO2[	371	]=	-0.3858879	;
    RnonCO2[	372	]=	-0.3859072	;
    RnonCO2[	373	]=	-0.3859195	;
    RnonCO2[	374	]=	-0.3859278	;
    RnonCO2[	375	]=	-0.3859334	;
    RnonCO2[	376	]=	-0.3859374	;
    RnonCO2[	377	]=	-0.3859512	;
    RnonCO2[	378	]=	-0.3859812	;
    RnonCO2[	379	]=	-0.3860183	;
    RnonCO2[	380	]=	-0.3860556	;
    RnonCO2[	381	]=	-0.3860878	;
    RnonCO2[	382	]=	-0.3861089	;
    RnonCO2[	383	]=	-0.3861194	;
    RnonCO2[	384	]=	-0.3861236	;
    RnonCO2[	385	]=	-0.3861246	;
    RnonCO2[	386	]=	-0.3861246	;
    RnonCO2[	387	]=	-0.3861246	;
    RnonCO2[	388	]=	-0.3861246	;
    RnonCO2[	389	]=	-0.3861246	;
    RnonCO2[	390	]=	-0.3861246	;
    RnonCO2[	391	]=	-0.3861246	;
    RnonCO2[	392	]=	-0.3861246	;
    RnonCO2[	393	]=	-0.3861246	;
    RnonCO2[	394	]=	-0.3861246	;
    RnonCO2[	395	]=	-0.3861246	;
    RnonCO2[	396	]=	-0.3861246	;
    RnonCO2[	397	]=	-0.3861246	;
    RnonCO2[	398	]=	-0.3861246	;
    RnonCO2[	399	]=	-0.3861246	;
    RnonCO2[	400	]=	-0.3861246	;
    RnonCO2[	401	]=	-0.3861246	;
    RnonCO2[	402	]=	-0.3861246	;
    RnonCO2[	403	]=	-0.3861246	;
    RnonCO2[	404	]=	-0.3861246	;
    RnonCO2[	405	]=	-0.3861246	;
    RnonCO2[	406	]=	-0.3861246	;
    RnonCO2[	407	]=	-0.3861246	;
    RnonCO2[	408	]=	-0.3861246	;
    RnonCO2[	409	]=	-0.3861246	;
    RnonCO2[	410	]=	-0.3861246	;
    RnonCO2[	411	]=	-0.3861246	;
    RnonCO2[	412	]=	-0.3861246	;
    RnonCO2[	413	]=	-0.3861246	;
    RnonCO2[	414	]=	-0.3861246	;
    RnonCO2[	415	]=	-0.3861246	;
    RnonCO2[	416	]=	-0.3861246	;
    RnonCO2[	417	]=	-0.3861246	;
    RnonCO2[	418	]=	-0.3861246	;
    RnonCO2[	419	]=	-0.3861246	;
    RnonCO2[	420	]=	-0.3861246	;
    RnonCO2[	421	]=	-0.3861246	;
    RnonCO2[	422	]=	-0.3861246	;
    RnonCO2[	423	]=	-0.3861246	;
    RnonCO2[	424	]=	-0.3861246	;
    RnonCO2[	425	]=	-0.3861246	;
    RnonCO2[	426	]=	-0.3861246	;
    RnonCO2[	427	]=	-0.3861246	;
    RnonCO2[	428	]=	-0.3861246	;
    RnonCO2[	429	]=	-0.3861246	;
    RnonCO2[	430	]=	-0.3861246	;
    RnonCO2[	431	]=	-0.3861246	;
    RnonCO2[	432	]=	-0.3861246	;
    RnonCO2[	433	]=	-0.3861246	;
    RnonCO2[	434	]=	-0.3861246	;
    RnonCO2[	435	]=	-0.3861246	;
    RnonCO2[	436	]=	-0.3861246	;
    RnonCO2[	437	]=	-0.3861246	;
    RnonCO2[	438	]=	-0.3861246	;
    RnonCO2[	439	]=	-0.3861246	;
    RnonCO2[	440	]=	-0.3861246	;
    RnonCO2[	441	]=	-0.3861246	;
    RnonCO2[	442	]=	-0.3861246	;
    RnonCO2[	443	]=	-0.3861246	;
    RnonCO2[	444	]=	-0.3861246	;
    RnonCO2[	445	]=	-0.3861246	;
    RnonCO2[	446	]=	-0.3861246	;
    RnonCO2[	447	]=	-0.3861246	;
    RnonCO2[	448	]=	-0.3861246	;
    RnonCO2[	449	]=	-0.3861246	;
    RnonCO2[	450	]=	-0.3861246	;
    RnonCO2[	451	]=	-0.3861246	;
    RnonCO2[	452	]=	-0.3861246	;
    RnonCO2[	453	]=	-0.3861246	;
    RnonCO2[	454	]=	-0.3861246	;
    RnonCO2[	455	]=	-0.3861246	;
    RnonCO2[	456	]=	-0.3861246	;
    RnonCO2[	457	]=	-0.3861246	;
    RnonCO2[	458	]=	-0.3861246	;
    RnonCO2[	459	]=	-0.3861246	;
    RnonCO2[	460	]=	-0.3861246	;
    RnonCO2[	461	]=	-0.3861246	;
    RnonCO2[	462	]=	-0.3861246	;
    RnonCO2[	463	]=	-0.3861246	;
    RnonCO2[	464	]=	-0.3861246	;
    RnonCO2[	465	]=	-0.3861246	;
    RnonCO2[	466	]=	-0.3861246	;
    RnonCO2[	467	]=	-0.3861246	;
    RnonCO2[	468	]=	-0.3861246	;
    RnonCO2[	469	]=	-0.3861246	;
    RnonCO2[	470	]=	-0.3861246	;
    RnonCO2[	471	]=	-0.3861246	;
    RnonCO2[	472	]=	-0.3861246	;
    RnonCO2[	473	]=	-0.3861246	;
    RnonCO2[	474	]=	-0.3861246	;
    RnonCO2[	475	]=	-0.3861246	;
    RnonCO2[	476	]=	-0.3861246	;
    RnonCO2[	477	]=	-0.3861246	;
    RnonCO2[	478	]=	-0.3861246	;
    RnonCO2[	479	]=	-0.3861246	;
    RnonCO2[	480	]=	-0.3861246	;
    RnonCO2[	481	]=	-0.3861246	;
    RnonCO2[	482	]=	-0.3861246	;
    RnonCO2[	483	]=	-0.3861246	;
    RnonCO2[	484	]=	-0.3861246	;
    RnonCO2[	485	]=	-0.3861246	;
    RnonCO2[	486	]=	-0.3861246	;
    RnonCO2[	487	]=	-0.3861246	;
    RnonCO2[	488	]=	-0.3861246	;
    RnonCO2[	489	]=	-0.3861246	;
    RnonCO2[	490	]=	-0.3861246	;
    RnonCO2[	491	]=	-0.3861246	;
    RnonCO2[	492	]=	-0.3861246	;
    RnonCO2[	493	]=	-0.3861246	;
    RnonCO2[	494	]=	-0.3861246	;
    RnonCO2[	495	]=	-0.3861246	;
    RnonCO2[	496	]=	-0.3861246	;
    RnonCO2[	497	]=	-0.3861246	;
    RnonCO2[	498	]=	-0.3861246	;
    RnonCO2[	499	]=	-0.3861246	;
    RnonCO2[	500	]=	-0.3861246	;
    RnonCO2[	501	]=	-0.3861246	;
    RnonCO2[	502	]=	-0.3861246	;
    RnonCO2[	503	]=	-0.3861246	;
    RnonCO2[	504	]=	-0.3861246	;
    RnonCO2[	505	]=	-0.3861246	;
    RnonCO2[	506	]=	-0.3861246	;
    RnonCO2[	507	]=	-0.3861246	;
    RnonCO2[	508	]=	-0.3861246	;
    RnonCO2[	509	]=	-0.3861246	;
    RnonCO2[	510	]=	-0.3861246	;
    RnonCO2[	511	]=	-0.3861246	;
    RnonCO2[	512	]=	-0.3861246	;
    RnonCO2[	513	]=	-0.3861246	;
    RnonCO2[	514	]=	-0.3861246	;
    RnonCO2[	515	]=	-0.3861246	;
    RnonCO2[	516	]=	-0.3861246	;
    RnonCO2[	517	]=	-0.3861246	;
    RnonCO2[	518	]=	-0.3861246	;
    RnonCO2[	519	]=	-0.3861246	;
    RnonCO2[	520	]=	-0.3861246	;
    RnonCO2[	521	]=	-0.3861246	;
    RnonCO2[	522	]=	-0.3861246	;
    RnonCO2[	523	]=	-0.3861246	;
    RnonCO2[	524	]=	-0.3861246	;
    RnonCO2[	525	]=	-0.3861246	;
    RnonCO2[	526	]=	-0.3861246	;
    RnonCO2[	527	]=	-0.3861246	;
    RnonCO2[	528	]=	-0.3861246	;
    RnonCO2[	529	]=	-0.3861246	;
    RnonCO2[	530	]=	-0.3861246	;
    RnonCO2[	531	]=	-0.3861246	;
    RnonCO2[	532	]=	-0.3861246	;
    RnonCO2[	533	]=	-0.3861246	;
    RnonCO2[	534	]=	-0.3861246	;
    RnonCO2[	535	]=	-0.3861246	;
    RnonCO2[	536	]=	-0.3861246	;
    RnonCO2[	537	]=	-0.3861246	;
    RnonCO2[	538	]=	-0.3861246	;
    RnonCO2[	539	]=	-0.3861246	;
    RnonCO2[	540	]=	-0.3861246	;
    RnonCO2[	541	]=	-0.3861246	;
    RnonCO2[	542	]=	-0.3861246	;
    RnonCO2[	543	]=	-0.3861246	;
    RnonCO2[	544	]=	-0.3861246	;
    RnonCO2[	545	]=	-0.3861246	;
    RnonCO2[	546	]=	-0.3861246	;
    RnonCO2[	547	]=	-0.3861246	;
    RnonCO2[	548	]=	-0.3861246	;
    RnonCO2[	549	]=	-0.3861246	;
    RnonCO2[	550	]=	-0.3861246	;
    RnonCO2[	551	]=	-0.3861246	;
    RnonCO2[	552	]=	-0.3861246	;
    RnonCO2[	553	]=	-0.3861246	;
    RnonCO2[	554	]=	-0.3861246	;
    RnonCO2[	555	]=	-0.3861246	;
    RnonCO2[	556	]=	-0.3861246	;
    RnonCO2[	557	]=	-0.3861246	;
    RnonCO2[	558	]=	-0.3861246	;
    RnonCO2[	559	]=	-0.3861246	;
    RnonCO2[	560	]=	-0.3861246	;
    RnonCO2[	561	]=	-0.3861246	;
    RnonCO2[	562	]=	-0.3861246	;
    RnonCO2[	563	]=	-0.3861246	;
    RnonCO2[	564	]=	-0.3861246	;
    RnonCO2[	565	]=	-0.3861246	;
    RnonCO2[	566	]=	-0.3861246	;
    RnonCO2[	567	]=	-0.3861246	;
    RnonCO2[	568	]=	-0.3861246	;
    RnonCO2[	569	]=	-0.3861246	;
    RnonCO2[	570	]=	-0.3861246	;
    RnonCO2[	571	]=	-0.3861246	;
    RnonCO2[	572	]=	-0.3861246	;
    RnonCO2[	573	]=	-0.3861246	;
    RnonCO2[	574	]=	-0.3861246	;
    RnonCO2[	575	]=	-0.3861246	;
    RnonCO2[	576	]=	-0.3861246	;
    RnonCO2[	577	]=	-0.3861246	;
    RnonCO2[	578	]=	-0.3861246	;
    RnonCO2[	579	]=	-0.3861246	;
    RnonCO2[	580	]=	-0.3861246	;
    RnonCO2[	581	]=	-0.3861246	;
    RnonCO2[	582	]=	-0.3861246	;
    RnonCO2[	583	]=	-0.3861246	;
    RnonCO2[	584	]=	-0.3861246	;
    RnonCO2[	585	]=	-0.3861246	;
    RnonCO2[	586	]=	-0.3861246	;
    RnonCO2[	587	]=	-0.3861246	;
    RnonCO2[	588	]=	-0.3861246	;
    RnonCO2[	589	]=	-0.3861246	;
    RnonCO2[	590	]=	-0.3861246	;
    RnonCO2[	591	]=	-0.3861246	;
    RnonCO2[	592	]=	-0.3861246	;
    RnonCO2[	593	]=	-0.3861246	;
    RnonCO2[	594	]=	-0.3861246	;
    RnonCO2[	595	]=	-0.3861246	;
    RnonCO2[	596	]=	-0.3861246	;
    RnonCO2[	597	]=	-0.3861246	;
    RnonCO2[	598	]=	-0.3861246	;
    RnonCO2[	599	]=	-0.3861246	;
    RnonCO2[	600	]=	-0.3861246	;
    RnonCO2[	601	]=	-0.3861246	;
    RnonCO2[	602	]=	-0.3861246	;
    RnonCO2[	603	]=	-0.3861246	;
    RnonCO2[	604	]=	-0.3861246	;
    RnonCO2[	605	]=	-0.3861246	;
    RnonCO2[	606	]=	-0.3861246	;
    RnonCO2[	607	]=	-0.3861246	;
    RnonCO2[	608	]=	-0.3861246	;
    RnonCO2[	609	]=	-0.3861246	;
    RnonCO2[	610	]=	-0.3861246	;
    RnonCO2[	611	]=	-0.3861246	;
    RnonCO2[	612	]=	-0.3861246	;
    RnonCO2[	613	]=	-0.3861246	;
    RnonCO2[	614	]=	-0.3861246	;
    RnonCO2[	615	]=	-0.3861246	;
    RnonCO2[	616	]=	-0.3861246	;
    RnonCO2[	617	]=	-0.3861246	;
    RnonCO2[	618	]=	-0.3861246	;
    RnonCO2[	619	]=	-0.3861246	;
    RnonCO2[	620	]=	-0.3861246	;
    RnonCO2[	621	]=	-0.3861246	;
    RnonCO2[	622	]=	-0.3861246	;
    RnonCO2[	623	]=	-0.3861246	;
    RnonCO2[	624	]=	-0.3861246	;
    RnonCO2[	625	]=	-0.3861246	;
    RnonCO2[	626	]=	-0.3861246	;
    RnonCO2[	627	]=	-0.3861246	;
    RnonCO2[	628	]=	-0.3861246	;
    RnonCO2[	629	]=	-0.3861246	;
    RnonCO2[	630	]=	-0.3861246	;
    RnonCO2[	631	]=	-0.3861246	;
    RnonCO2[	632	]=	-0.3861246	;
    RnonCO2[	633	]=	-0.3861246	;
    RnonCO2[	634	]=	-0.3861246	;
    RnonCO2[	635	]=	-0.3861246	;
    RnonCO2[	636	]=	-0.3861246	;
    RnonCO2[	637	]=	-0.3861246	;
    RnonCO2[	638	]=	-0.3861246	;
    RnonCO2[	639	]=	-0.3861246	;
    RnonCO2[	640	]=	-0.3861246	;
    RnonCO2[	641	]=	-0.3861246	;
    RnonCO2[	642	]=	-0.3861246	;
    RnonCO2[	643	]=	-0.3861246	;
    RnonCO2[	644	]=	-0.3861246	;
    RnonCO2[	645	]=	-0.3861246	;
    RnonCO2[	646	]=	-0.3861246	;
    RnonCO2[	647	]=	-0.3861246	;
    RnonCO2[	648	]=	-0.3861246	;
    RnonCO2[	649	]=	-0.3861246	;
    RnonCO2[	650	]=	-0.3861246	;
    RnonCO2[	651	]=	-0.3861246	;
    RnonCO2[	652	]=	-0.3861246	;
    RnonCO2[	653	]=	-0.3861246	;
    RnonCO2[	654	]=	-0.3861246	;
    RnonCO2[	655	]=	-0.3861246	;
    RnonCO2[	656	]=	-0.3861246	;
    RnonCO2[	657	]=	-0.3861246	;
    RnonCO2[	658	]=	-0.3861246	;
    RnonCO2[	659	]=	-0.3861246	;
    RnonCO2[	660	]=	-0.3861246	;
    RnonCO2[	661	]=	-0.3861246	;
    RnonCO2[	662	]=	-0.3861246	;
    RnonCO2[	663	]=	-0.3861246	;
    RnonCO2[	664	]=	-0.3861246	;
    RnonCO2[	665	]=	-0.3861246	;
    RnonCO2[	666	]=	-0.3861246	;
    RnonCO2[	667	]=	-0.3861246	;
    RnonCO2[	668	]=	-0.3861246	;
    RnonCO2[	669	]=	-0.3861246	;
    RnonCO2[	670	]=	-0.3861246	;
    RnonCO2[	671	]=	-0.3861246	;
    RnonCO2[	672	]=	-0.3861246	;
    RnonCO2[	673	]=	-0.3861246	;
    RnonCO2[	674	]=	-0.3861246	;
    RnonCO2[	675	]=	-0.3861246	;
    RnonCO2[	676	]=	-0.3861246	;
    RnonCO2[	677	]=	-0.3861246	;
    RnonCO2[	678	]=	-0.3861246	;
    RnonCO2[	679	]=	-0.3861246	;
    RnonCO2[	680	]=	-0.3861246	;
    RnonCO2[	681	]=	-0.3861246	;
    RnonCO2[	682	]=	-0.3861246	;
    RnonCO2[	683	]=	-0.3861246	;
    RnonCO2[	684	]=	-0.3861246	;
    RnonCO2[	685	]=	-0.3861246	;
    RnonCO2[	686	]=	-0.3861246	;
    RnonCO2[	687	]=	-0.3861246	;
    RnonCO2[	688	]=	-0.3861246	;
    RnonCO2[	689	]=	-0.3861246	;
    RnonCO2[	690	]=	-0.3861246	;
    RnonCO2[	691	]=	-0.3861246	;
    RnonCO2[	692	]=	-0.3861246	;
    RnonCO2[	693	]=	-0.3861246	;
    RnonCO2[	694	]=	-0.3861246	;
    RnonCO2[	695	]=	-0.3861246	;
    RnonCO2[	696	]=	-0.3861246	;
    RnonCO2[	697	]=	-0.3861246	;
    RnonCO2[	698	]=	-0.3861246	;
    RnonCO2[	699	]=	-0.3861246	;
    RnonCO2[	700	]=	-0.3861246	;
    RnonCO2[	701	]=	-0.3861246	;
    RnonCO2[	702	]=	-0.3861246	;
    RnonCO2[	703	]=	-0.3861246	;
    RnonCO2[	704	]=	-0.3861246	;
    RnonCO2[	705	]=	-0.3861246	;
    RnonCO2[	706	]=	-0.3861246	;
    RnonCO2[	707	]=	-0.3861246	;
    RnonCO2[	708	]=	-0.3861246	;
    RnonCO2[	709	]=	-0.3861246	;
    RnonCO2[	710	]=	-0.3861246	;
    RnonCO2[	711	]=	-0.3861246	;
    RnonCO2[	712	]=	-0.3861246	;
    RnonCO2[	713	]=	-0.3861246	;
    RnonCO2[	714	]=	-0.3861246	;
    RnonCO2[	715	]=	-0.3861246	;
    RnonCO2[	716	]=	-0.3861246	;
    RnonCO2[	717	]=	-0.3861246	;
    RnonCO2[	718	]=	-0.3861246	;
    RnonCO2[	719	]=	-0.3861246	;
    RnonCO2[	720	]=	-0.3861246	;
    RnonCO2[	721	]=	-0.3861246	;
    RnonCO2[	722	]=	-0.3861246	;
    RnonCO2[	723	]=	-0.3861246	;
    RnonCO2[	724	]=	-0.3861246	;
    RnonCO2[	725	]=	-0.3861246	;
    RnonCO2[	726	]=	-0.3861246	;
    RnonCO2[	727	]=	-0.3861246	;
    RnonCO2[	728	]=	-0.3861246	;
    RnonCO2[	729	]=	-0.3861246	;
    RnonCO2[	730	]=	-0.3861246	;
    RnonCO2[	731	]=	-0.3861246	;
    RnonCO2[	732	]=	-0.3861246	;
    RnonCO2[	733	]=	-0.3861246	;
    RnonCO2[	734	]=	-0.3861246	;
    RnonCO2[	735	]=	-0.3861246	;
    
    for(int i = 736; i<tmax; i++)
    {
        
        RnonCO2[i] = RnonCO2[i-1];
        
    }
    
}

void setR_aerosol_RCP3PD(void)
{
    
    RnonCO2[	0	]=	0	;
    RnonCO2[	1	]=	0.000955298	;
    RnonCO2[	2	]=	0.001424721	;
    RnonCO2[	3	]=	0.0001974	;
    RnonCO2[	4	]=	-0.001278437	;
    RnonCO2[	5	]=	-0.002733841	;
    RnonCO2[	6	]=	-0.004541334	;
    RnonCO2[	7	]=	-0.00632004	;
    RnonCO2[	8	]=	-0.007680948	;
    RnonCO2[	9	]=	-0.008994568	;
    RnonCO2[	10	]=	-0.010260775	;
    RnonCO2[	11	]=	-0.011479866	;
    RnonCO2[	12	]=	-0.012652425	;
    RnonCO2[	13	]=	-0.013779231	;
    RnonCO2[	14	]=	-0.014861204	;
    RnonCO2[	15	]=	-0.015900176	;
    RnonCO2[	16	]=	-0.017335746	;
    RnonCO2[	17	]=	-0.01873479	;
    RnonCO2[	18	]=	-0.019659073	;
    RnonCO2[	19	]=	-0.020540613	;
    RnonCO2[	20	]=	-0.02137714	;
    RnonCO2[	21	]=	-0.022166399	;
    RnonCO2[	22	]=	-0.022906132	;
    RnonCO2[	23	]=	-0.023594082	;
    RnonCO2[	24	]=	-0.024227995	;
    RnonCO2[	25	]=	-0.024806236	;
    RnonCO2[	26	]=	-0.025775743	;
    RnonCO2[	27	]=	-0.026691982	;
    RnonCO2[	28	]=	-0.027110281	;
    RnonCO2[	29	]=	-0.02747885	;
    RnonCO2[	30	]=	-0.027799526	;
    RnonCO2[	31	]=	-0.028074106	;
    RnonCO2[	32	]=	-0.02875039	;
    RnonCO2[	33	]=	-0.029383792	;
    RnonCO2[	34	]=	-0.02953002	;
    RnonCO2[	35	]=	-0.03007996	;
    RnonCO2[	36	]=	-0.03058546	;
    RnonCO2[	37	]=	-0.03148571	;
    RnonCO2[	38	]=	-0.03189741	;
    RnonCO2[	39	]=	-0.03138062	;
    RnonCO2[	40	]=	-0.03126848	;
    RnonCO2[	41	]=	-0.03156144	;
    RnonCO2[	42	]=	-0.03182181	;
    RnonCO2[	43	]=	-0.03161267	;
    RnonCO2[	44	]=	-0.03137821	;
    RnonCO2[	45	]=	-0.03112038	;
    RnonCO2[	46	]=	-0.03127367	;
    RnonCO2[	47	]=	-0.0313998	;
    RnonCO2[	48	]=	-0.03106322	;
    RnonCO2[	49	]=	-0.03070135	;
    RnonCO2[	50	]=	-0.03074616	;
    RnonCO2[	51	]=	-0.03119614	;
    RnonCO2[	52	]=	-0.03161872	;
    RnonCO2[	53	]=	-0.03158785	;
    RnonCO2[	54	]=	-0.03110696	;
    RnonCO2[	55	]=	-0.03060634	;
    RnonCO2[	56	]=	-0.0300866	;
    RnonCO2[	57	]=	-0.02996996	;
    RnonCO2[	58	]=	-0.03025404	;
    RnonCO2[	59	]=	-0.03009594	;
    RnonCO2[	60	]=	-0.02991636	;
    RnonCO2[	61	]=	-0.02971869	;
    RnonCO2[	62	]=	-0.02950042	;
    RnonCO2[	63	]=	-0.02926502	;
    RnonCO2[	64	]=	-0.02859871	;
    RnonCO2[	65	]=	-0.03037359	;
    RnonCO2[	66	]=	-0.03172034	;
    RnonCO2[	67	]=	-0.03059392	;
    RnonCO2[	68	]=	-0.03026959	;
    RnonCO2[	69	]=	-0.02993194	;
    RnonCO2[	70	]=	-0.02957653	;
    RnonCO2[	71	]=	-0.03080027	;
    RnonCO2[	72	]=	-0.03160022	;
    RnonCO2[	73	]=	-0.03117789	;
    RnonCO2[	74	]=	-0.03112982	;
    RnonCO2[	75	]=	-0.03144654	;
    RnonCO2[	76	]=	-0.03173222	;
    RnonCO2[	77	]=	-0.03198224	;
    RnonCO2[	78	]=	-0.03220368	;
    RnonCO2[	79	]=	-0.03239696	;
    RnonCO2[	80	]=	-0.03371239	;
    RnonCO2[	81	]=	-0.03424533	;
    RnonCO2[	82	]=	-0.0343764	;
    RnonCO2[	83	]=	-0.03487942	;
    RnonCO2[	84	]=	-0.03536957	;
    RnonCO2[	85	]=	-0.03686306	;
    RnonCO2[	86	]=	-0.03665392	;
    RnonCO2[	87	]=	-0.04248424	;
    RnonCO2[	88	]=	-0.04930444	;
    RnonCO2[	89	]=	-0.05182621	;
    RnonCO2[	90	]=	-0.0542671	;
    RnonCO2[	91	]=	-0.05573158	;
    RnonCO2[	92	]=	-0.05765815	;
    RnonCO2[	93	]=	-0.05821397	;
    RnonCO2[	94	]=	-0.06005825	;
    RnonCO2[	95	]=	-0.0683501	;
    RnonCO2[	96	]=	-0.08104112	;
    RnonCO2[	97	]=	-0.07832852	;
    RnonCO2[	98	]=	-0.07177024	;
    RnonCO2[	99	]=	-0.07363036	;
    RnonCO2[	100	]=	-0.07696442	;
    RnonCO2[	101	]=	-0.08462429	;
    RnonCO2[	102	]=	-0.09275025	;
    RnonCO2[	103	]=	-0.09511911	;
    RnonCO2[	104	]=	-0.0930339	;
    RnonCO2[	105	]=	-0.09237765	;
    RnonCO2[	106	]=	-0.10165608	;
    RnonCO2[	107	]=	-0.12466661	;
    RnonCO2[	108	]=	-0.14405058	;
    RnonCO2[	109	]=	-0.14773339	;
    RnonCO2[	110	]=	-0.14777672	;
    RnonCO2[	111	]=	-0.1470811	;
    RnonCO2[	112	]=	-0.14255587	;
    RnonCO2[	113	]=	-0.14024698	;
    RnonCO2[	114	]=	-0.14393886	;
    RnonCO2[	115	]=	-0.15612866	;
    RnonCO2[	116	]=	-0.164782	;
    RnonCO2[	117	]=	-0.16302445	;
    RnonCO2[	118	]=	-0.16326054	;
    RnonCO2[	119	]=	-0.16496204	;
    RnonCO2[	120	]=	-0.16272157	;
    RnonCO2[	121	]=	-0.1612244	;
    RnonCO2[	122	]=	-0.16356025	;
    RnonCO2[	123	]=	-0.17242315	;
    RnonCO2[	124	]=	-0.17632405	;
    RnonCO2[	125	]=	-0.17855867	;
    RnonCO2[	126	]=	-0.18527891	;
    RnonCO2[	127	]=	-0.18997972	;
    RnonCO2[	128	]=	-0.1933518	;
    RnonCO2[	129	]=	-0.19699663	;
    RnonCO2[	130	]=	-0.20233059	;
    RnonCO2[	131	]=	-0.2051441	;
    RnonCO2[	132	]=	-0.20577589	;
    RnonCO2[	133	]=	-0.20720143	;
    RnonCO2[	134	]=	-0.21030464	;
    RnonCO2[	135	]=	-0.21375193	;
    RnonCO2[	136	]=	-0.22004069	;
    RnonCO2[	137	]=	-0.22693377	;
    RnonCO2[	138	]=	-0.23402972	;
    RnonCO2[	139	]=	-0.2411565	;
    RnonCO2[	140	]=	-0.2471181	;
    RnonCO2[	141	]=	-0.255172	;
    RnonCO2[	142	]=	-0.26315304	;
    RnonCO2[	143	]=	-0.2636557	;
    RnonCO2[	144	]=	-0.26151664	;
    RnonCO2[	145	]=	-0.26499499	;
    RnonCO2[	146	]=	-0.26461175	;
    RnonCO2[	147	]=	-0.26044407	;
    RnonCO2[	148	]=	-0.25940869	;
    RnonCO2[	149	]=	-0.25560119	;
    RnonCO2[	150	]=	-0.25007798	;
    RnonCO2[	151	]=	-0.25106379	;
    RnonCO2[	152	]=	-0.25437609	;
    RnonCO2[	153	]=	-0.25544669	;
    RnonCO2[	154	]=	-0.2505011	;
    RnonCO2[	155	]=	-0.24919556	;
    RnonCO2[	156	]=	-0.24766564	;
    RnonCO2[	157	]=	-0.24454174	;
    RnonCO2[	158	]=	-0.25504195	;
    RnonCO2[	159	]=	-0.26312341	;
    RnonCO2[	160	]=	-0.26314088	;
    RnonCO2[	161	]=	-0.26376161	;
    RnonCO2[	162	]=	-0.26809378	;
    RnonCO2[	163	]=	-0.27284291	;
    RnonCO2[	164	]=	-0.27584022	;
    RnonCO2[	165	]=	-0.27557411	;
    RnonCO2[	166	]=	-0.26967283	;
    RnonCO2[	167	]=	-0.26315755	;
    RnonCO2[	168	]=	-0.26083606	;
    RnonCO2[	169	]=	-0.26325195	;
    RnonCO2[	170	]=	-0.2652416	;
    RnonCO2[	171	]=	-0.26824619	;
    RnonCO2[	172	]=	-0.26930622	;
    RnonCO2[	173	]=	-0.26149511	;
    RnonCO2[	174	]=	-0.25651091	;
    RnonCO2[	175	]=	-0.26035065	;
    RnonCO2[	176	]=	-0.26656695	;
    RnonCO2[	177	]=	-0.27194936	;
    RnonCO2[	178	]=	-0.274817	;
    RnonCO2[	179	]=	-0.27878403	;
    RnonCO2[	180	]=	-0.28302569	;
    RnonCO2[	181	]=	-0.29104814	;
    RnonCO2[	182	]=	-0.30416561	;
    RnonCO2[	183	]=	-0.31716714	;
    RnonCO2[	184	]=	-0.3246442	;
    RnonCO2[	185	]=	-0.3327935	;
    RnonCO2[	186	]=	-0.35639494	;
    RnonCO2[	187	]=	-0.38051016	;
    RnonCO2[	188	]=	-0.39378465	;
    RnonCO2[	189	]=	-0.40659776	;
    RnonCO2[	190	]=	-0.4196919	;
    RnonCO2[	191	]=	-0.43302942	;
    RnonCO2[	192	]=	-0.44651364	;
    RnonCO2[	193	]=	-0.45569127	;
    RnonCO2[	194	]=	-0.45642498	;
    RnonCO2[	195	]=	-0.45551768	;
    RnonCO2[	196	]=	-0.47183293	;
    RnonCO2[	197	]=	-0.49188519	;
    RnonCO2[	198	]=	-0.50135461	;
    RnonCO2[	199	]=	-0.5101619	;
    RnonCO2[	200	]=	-0.51738459	;
    RnonCO2[	201	]=	-0.52416219	;
    RnonCO2[	202	]=	-0.53325402	;
    RnonCO2[	203	]=	-0.54124791	;
    RnonCO2[	204	]=	-0.54379714	;
    RnonCO2[	205	]=	-0.53664772	;
    RnonCO2[	206	]=	-0.52607217	;
    RnonCO2[	207	]=	-0.52739363	;
    RnonCO2[	208	]=	-0.53724776	;
    RnonCO2[	209	]=	-0.54779467	;
    RnonCO2[	210	]=	-0.55942351	;
    RnonCO2[	211	]=	-0.57578959	;
    RnonCO2[	212	]=	-0.5916702	;
    RnonCO2[	213	]=	-0.5979117	;
    RnonCO2[	214	]=	-0.6009881	;
    RnonCO2[	215	]=	-0.6131431	;
    RnonCO2[	216	]=	-0.6319195	;
    RnonCO2[	217	]=	-0.6542028	;
    RnonCO2[	218	]=	-0.6747946	;
    RnonCO2[	219	]=	-0.6865596	;
    RnonCO2[	220	]=	-0.6942006	;
    RnonCO2[	221	]=	-0.7018816	;
    RnonCO2[	222	]=	-0.7121991	;
    RnonCO2[	223	]=	-0.7210817	;
    RnonCO2[	224	]=	-0.7262596	;
    RnonCO2[	225	]=	-0.7299457	;
    RnonCO2[	226	]=	-0.7418714	;
    RnonCO2[	227	]=	-0.7583892	;
    RnonCO2[	228	]=	-0.7656833	;
    RnonCO2[	229	]=	-0.768847	;
    RnonCO2[	230	]=	-0.7655134	;
    RnonCO2[	231	]=	-0.7588829	;
    RnonCO2[	232	]=	-0.7657109	;
    RnonCO2[	233	]=	-0.7804313	;
    RnonCO2[	234	]=	-0.7902367	;
    RnonCO2[	235	]=	-0.8012413	;
    RnonCO2[	236	]=	-0.8096987	;
    RnonCO2[	237	]=	-0.8105819	;
    RnonCO2[	238	]=	-0.8112899	;
    RnonCO2[	239	]=	-0.8123564	;
    RnonCO2[	240	]=	-0.8111513	;
    RnonCO2[	241	]=	-0.8050465	;
    RnonCO2[	242	]=	-0.7966306	;
    RnonCO2[	243	]=	-0.7880713	;
    RnonCO2[	244	]=	-0.7791377	;
    RnonCO2[	245	]=	-0.7684838	;
    RnonCO2[	246	]=	-0.7547825	;
    RnonCO2[	247	]=	-0.7400687	;
    RnonCO2[	248	]=	-0.7264014	;
    RnonCO2[	249	]=	-0.7134622	;
    RnonCO2[	250	]=	-0.7009668	;
    RnonCO2[	251	]=	-0.6888369	;
    RnonCO2[	252	]=	-0.6770345	;
    RnonCO2[	253	]=	-0.6655394	;
    RnonCO2[	254	]=	-0.653891	;
    RnonCO2[	255	]=	-0.6434029	;
    RnonCO2[	256	]=	-0.6355575	;
    RnonCO2[	257	]=	-0.6294243	;
    RnonCO2[	258	]=	-0.6237506	;
    RnonCO2[	259	]=	-0.6178977	;
    RnonCO2[	260	]=	-0.611837	;
    RnonCO2[	261	]=	-0.6055847	;
    RnonCO2[	262	]=	-0.59916	;
    RnonCO2[	263	]=	-0.5925781	;
    RnonCO2[	264	]=	-0.5858487	;
    RnonCO2[	265	]=	-0.580215	;
    RnonCO2[	266	]=	-0.5769285	;
    RnonCO2[	267	]=	-0.5739943	;
    RnonCO2[	268	]=	-0.5701748	;
    RnonCO2[	269	]=	-0.5662388	;
    RnonCO2[	270	]=	-0.5621802	;
    RnonCO2[	271	]=	-0.5579998	;
    RnonCO2[	272	]=	-0.5537056	;
    RnonCO2[	273	]=	-0.5493032	;
    RnonCO2[	274	]=	-0.5448003	;
    RnonCO2[	275	]=	-0.5412975	;
    RnonCO2[	276	]=	-0.539911	;
    RnonCO2[	277	]=	-0.5396073	;
    RnonCO2[	278	]=	-0.5393865	;
    RnonCO2[	279	]=	-0.5392914	;
    RnonCO2[	280	]=	-0.5393053	;
    RnonCO2[	281	]=	-0.5394198	;
    RnonCO2[	282	]=	-0.5396188	;
    RnonCO2[	283	]=	-0.5398498	;
    RnonCO2[	284	]=	-0.5400737	;
    RnonCO2[	285	]=	-0.5410681	;
    RnonCO2[	286	]=	-0.5436393	;
    RnonCO2[	287	]=	-0.5470987	;
    RnonCO2[	288	]=	-0.5506475	;
    RnonCO2[	289	]=	-0.5542153	;
    RnonCO2[	290	]=	-0.557813	;
    RnonCO2[	291	]=	-0.5614358	;
    RnonCO2[	292	]=	-0.5650812	;
    RnonCO2[	293	]=	-0.568747	;
    RnonCO2[	294	]=	-0.5724309	;
    RnonCO2[	295	]=	-0.5753782	;
    RnonCO2[	296	]=	-0.5768182	;
    RnonCO2[	297	]=	-0.577794	;
    RnonCO2[	298	]=	-0.5789932	;
    RnonCO2[	299	]=	-0.5800496	;
    RnonCO2[	300	]=	-0.5809866	;
    RnonCO2[	301	]=	-0.5818207	;
    RnonCO2[	302	]=	-0.582553	;
    RnonCO2[	303	]=	-0.5831855	;
    RnonCO2[	304	]=	-0.5837211	;
    RnonCO2[	305	]=	-0.5836775	;
    RnonCO2[	306	]=	-0.5825824	;
    RnonCO2[	307	]=	-0.580916	;
    RnonCO2[	308	]=	-0.5791711	;
    RnonCO2[	309	]=	-0.5773716	;
    RnonCO2[	310	]=	-0.5755228	;
    RnonCO2[	311	]=	-0.5736392	;
    RnonCO2[	312	]=	-0.5717301	;
    RnonCO2[	313	]=	-0.5697893	;
    RnonCO2[	314	]=	-0.5678132	;
    RnonCO2[	315	]=	-0.5660861	;
    RnonCO2[	316	]=	-0.5648969	;
    RnonCO2[	317	]=	-0.5636325	;
    RnonCO2[	318	]=	-0.5620179	;
    RnonCO2[	319	]=	-0.5603957	;
    RnonCO2[	320	]=	-0.5587675	;
    RnonCO2[	321	]=	-0.5571346	;
    RnonCO2[	322	]=	-0.5555085	;
    RnonCO2[	323	]=	-0.5538953	;
    RnonCO2[	324	]=	-0.5522866	;
    RnonCO2[	325	]=	-0.550654	;
    RnonCO2[	326	]=	-0.5489639	;
    RnonCO2[	327	]=	-0.5473193	;
    RnonCO2[	328	]=	-0.5457334	;
    RnonCO2[	329	]=	-0.54411	;
    RnonCO2[	330	]=	-0.5424524	;
    RnonCO2[	331	]=	-0.5407624	;
    RnonCO2[	332	]=	-0.5390414	;
    RnonCO2[	333	]=	-0.5373012	;
    RnonCO2[	334	]=	-0.5355478	;
    RnonCO2[	335	]=	-0.5347717	;
    RnonCO2[	336	]=	-0.5359662	;
    RnonCO2[	337	]=	-0.5374878	;
    RnonCO2[	338	]=	-0.5383173	;
    RnonCO2[	339	]=	-0.5390826	;
    RnonCO2[	340	]=	-0.5397906	;
    RnonCO2[	341	]=	-0.5404467	;
    RnonCO2[	342	]=	-0.5410543	;
    RnonCO2[	343	]=	-0.5416166	;
    RnonCO2[	344	]=	-0.5421464	;
    RnonCO2[	345	]=	-0.542651	;
    RnonCO2[	346	]=	-0.5431236	;
    RnonCO2[	347	]=	-0.5435591	;
    RnonCO2[	348	]=	-0.5439538	;
    RnonCO2[	349	]=	-0.5443033	;
    RnonCO2[	350	]=	-0.5446088	;
    RnonCO2[	351	]=	-0.5448751	;
    RnonCO2[	352	]=	-0.5451056	;
    RnonCO2[	353	]=	-0.545302	;
    RnonCO2[	354	]=	-0.5454656	;
    RnonCO2[	355	]=	-0.5456081	;
    RnonCO2[	356	]=	-0.5457354	;
    RnonCO2[	357	]=	-0.5458396	;
    RnonCO2[	358	]=	-0.5459148	;
    RnonCO2[	359	]=	-0.5459561	;
    RnonCO2[	360	]=	-0.5459653	;
    RnonCO2[	361	]=	-0.5459561	;
    RnonCO2[	362	]=	-0.5459399	;
    RnonCO2[	363	]=	-0.5459192	;
    RnonCO2[	364	]=	-0.5458953	;
    RnonCO2[	365	]=	-0.5458692	;
    RnonCO2[	366	]=	-0.545852	;
    RnonCO2[	367	]=	-0.5458494	;
    RnonCO2[	368	]=	-0.5458532	;
    RnonCO2[	369	]=	-0.5458572	;
    RnonCO2[	370	]=	-0.5458563	;
    RnonCO2[	371	]=	-0.5458454	;
    RnonCO2[	372	]=	-0.5458246	;
    RnonCO2[	373	]=	-0.5457981	;
    RnonCO2[	374	]=	-0.5457684	;
    RnonCO2[	375	]=	-0.5457367	;
    RnonCO2[	376	]=	-0.5457039	;
    RnonCO2[	377	]=	-0.5456809	;
    RnonCO2[	378	]=	-0.5456733	;
    RnonCO2[	379	]=	-0.5456728	;
    RnonCO2[	380	]=	-0.5456731	;
    RnonCO2[	381	]=	-0.545669	;
    RnonCO2[	382	]=	-0.5456555	;
    RnonCO2[	383	]=	-0.5456325	;
    RnonCO2[	384	]=	-0.5456043	;
    RnonCO2[	385	]=	-0.5455733	;
    RnonCO2[	386	]=	-0.5455404	;
    RnonCO2[	387	]=	-0.5455068	;
    RnonCO2[	388	]=	-0.5454831	;
    RnonCO2[	389	]=	-0.5454751	;
    RnonCO2[	390	]=	-0.5454742	;
    RnonCO2[	391	]=	-0.5454743	;
    RnonCO2[	392	]=	-0.5454704	;
    RnonCO2[	393	]=	-0.5454569	;
    RnonCO2[	394	]=	-0.5454342	;
    RnonCO2[	395	]=	-0.5454062	;
    RnonCO2[	396	]=	-0.5453757	;
    RnonCO2[	397	]=	-0.5453433	;
    RnonCO2[	398	]=	-0.5453102	;
    RnonCO2[	399	]=	-0.545287	;
    RnonCO2[	400	]=	-0.5452796	;
    RnonCO2[	401	]=	-0.5452793	;
    RnonCO2[	402	]=	-0.5452801	;
    RnonCO2[	403	]=	-0.5452767	;
    RnonCO2[	404	]=	-0.5452639	;
    RnonCO2[	405	]=	-0.545242	;
    RnonCO2[	406	]=	-0.5452148	;
    RnonCO2[	407	]=	-0.5451848	;
    RnonCO2[	408	]=	-0.5451532	;
    RnonCO2[	409	]=	-0.5451208	;
    RnonCO2[	410	]=	-0.5450983	;
    RnonCO2[	411	]=	-0.5450917	;
    RnonCO2[	412	]=	-0.5450921	;
    RnonCO2[	413	]=	-0.5450935	;
    RnonCO2[	414	]=	-0.5450909	;
    RnonCO2[	415	]=	-0.5450788	;
    RnonCO2[	416	]=	-0.5450575	;
    RnonCO2[	417	]=	-0.545031	;
    RnonCO2[	418	]=	-0.5450018	;
    RnonCO2[	419	]=	-0.5449708	;
    RnonCO2[	420	]=	-0.5449391	;
    RnonCO2[	421	]=	-0.5449173	;
    RnonCO2[	422	]=	-0.5449113	;
    RnonCO2[	423	]=	-0.5449123	;
    RnonCO2[	424	]=	-0.5449143	;
    RnonCO2[	425	]=	-0.5449123	;
    RnonCO2[	426	]=	-0.5449008	;
    RnonCO2[	427	]=	-0.5448801	;
    RnonCO2[	428	]=	-0.5448542	;
    RnonCO2[	429	]=	-0.5448255	;
    RnonCO2[	430	]=	-0.5447951	;
    RnonCO2[	431	]=	-0.544764	;
    RnonCO2[	432	]=	-0.5447428	;
    RnonCO2[	433	]=	-0.5447373	;
    RnonCO2[	434	]=	-0.5447388	;
    RnonCO2[	435	]=	-0.5447413	;
    RnonCO2[	436	]=	-0.5447398	;
    RnonCO2[	437	]=	-0.5447288	;
    RnonCO2[	438	]=	-0.5447085	;
    RnonCO2[	439	]=	-0.5446832	;
    RnonCO2[	440	]=	-0.544655	;
    RnonCO2[	441	]=	-0.5446251	;
    RnonCO2[	442	]=	-0.5445943	;
    RnonCO2[	443	]=	-0.5445736	;
    RnonCO2[	444	]=	-0.5445685	;
    RnonCO2[	445	]=	-0.5445705	;
    RnonCO2[	446	]=	-0.5445734	;
    RnonCO2[	447	]=	-0.5445723	;
    RnonCO2[	448	]=	-0.5445617	;
    RnonCO2[	449	]=	-0.5445419	;
    RnonCO2[	450	]=	-0.544517	;
    RnonCO2[	451	]=	-0.5444891	;
    RnonCO2[	452	]=	-0.5444597	;
    RnonCO2[	453	]=	-0.5444293	;
    RnonCO2[	454	]=	-0.544409	;
    RnonCO2[	455	]=	-0.5444042	;
    RnonCO2[	456	]=	-0.5444066	;
    RnonCO2[	457	]=	-0.5444099	;
    RnonCO2[	458	]=	-0.5444091	;
    RnonCO2[	459	]=	-0.5443989	;
    RnonCO2[	460	]=	-0.5443795	;
    RnonCO2[	461	]=	-0.5443548	;
    RnonCO2[	462	]=	-0.5443274	;
    RnonCO2[	463	]=	-0.5442982	;
    RnonCO2[	464	]=	-0.5442682	;
    RnonCO2[	465	]=	-0.5442482	;
    RnonCO2[	466	]=	-0.5442438	;
    RnonCO2[	467	]=	-0.5442464	;
    RnonCO2[	468	]=	-0.54425	;
    RnonCO2[	469	]=	-0.5442495	;
    RnonCO2[	470	]=	-0.5442396	;
    RnonCO2[	471	]=	-0.5442204	;
    RnonCO2[	472	]=	-0.5441961	;
    RnonCO2[	473	]=	-0.544169	;
    RnonCO2[	474	]=	-0.5441401	;
    RnonCO2[	475	]=	-0.5441105	;
    RnonCO2[	476	]=	-0.5440907	;
    RnonCO2[	477	]=	-0.5440866	;
    RnonCO2[	478	]=	-0.5440894	;
    RnonCO2[	479	]=	-0.5440933	;
    RnonCO2[	480	]=	-0.5440931	;
    RnonCO2[	481	]=	-0.5440835	;
    RnonCO2[	482	]=	-0.5440646	;
    RnonCO2[	483	]=	-0.5440405	;
    RnonCO2[	484	]=	-0.5440137	;
    RnonCO2[	485	]=	-0.5439851	;
    RnonCO2[	486	]=	-0.5439556	;
    RnonCO2[	487	]=	-0.5439362	;
    RnonCO2[	488	]=	-0.5439322	;
    RnonCO2[	489	]=	-0.5439354	;
    RnonCO2[	490	]=	-0.5439394	;
    RnonCO2[	491	]=	-0.5439395	;
    RnonCO2[	492	]=	-0.54393	;
    RnonCO2[	493	]=	-0.5439114	;
    RnonCO2[	494	]=	-0.5438876	;
    RnonCO2[	495	]=	-0.543861	;
    RnonCO2[	496	]=	-0.5438326	;
    RnonCO2[	497	]=	-0.5438034	;
    RnonCO2[	498	]=	-0.5437841	;
    RnonCO2[	499	]=	-0.5437805	;
    RnonCO2[	500	]=	-0.5437838	;
    RnonCO2[	501	]=	-0.5437881	;
    RnonCO2[	502	]=	-0.5437883	;
    RnonCO2[	503	]=	-0.5437791	;
    RnonCO2[	504	]=	-0.5437606	;
    RnonCO2[	505	]=	-0.5437371	;
    RnonCO2[	506	]=	-0.5437107	;
    RnonCO2[	507	]=	-0.5436826	;
    RnonCO2[	508	]=	-0.5436536	;
    RnonCO2[	509	]=	-0.5436345	;
    RnonCO2[	510	]=	-0.543631	;
    RnonCO2[	511	]=	-0.5436345	;
    RnonCO2[	512	]=	-0.543639	;
    RnonCO2[	513	]=	-0.5436394	;
    RnonCO2[	514	]=	-0.5436304	;
    RnonCO2[	515	]=	-0.5436122	;
    RnonCO2[	516	]=	-0.5435888	;
    RnonCO2[	517	]=	-0.5435626	;
    RnonCO2[	518	]=	-0.5435346	;
    RnonCO2[	519	]=	-0.5435059	;
    RnonCO2[	520	]=	-0.5434869	;
    RnonCO2[	521	]=	-0.5434837	;
    RnonCO2[	522	]=	-0.5434874	;
    RnonCO2[	523	]=	-0.543492	;
    RnonCO2[	524	]=	-0.5434926	;
    RnonCO2[	525	]=	-0.5434838	;
    RnonCO2[	526	]=	-0.5434658	;
    RnonCO2[	527	]=	-0.5434425	;
    RnonCO2[	528	]=	-0.5434165	;
    RnonCO2[	529	]=	-0.5433887	;
    RnonCO2[	530	]=	-0.5433601	;
    RnonCO2[	531	]=	-0.5433414	;
    RnonCO2[	532	]=	-0.5433383	;
    RnonCO2[	533	]=	-0.5433421	;
    RnonCO2[	534	]=	-0.5433469	;
    RnonCO2[	535	]=	-0.5433476	;
    RnonCO2[	536	]=	-0.543339	;
    RnonCO2[	537	]=	-0.5433211	;
    RnonCO2[	538	]=	-0.543298	;
    RnonCO2[	539	]=	-0.5432722	;
    RnonCO2[	540	]=	-0.5432446	;
    RnonCO2[	541	]=	-0.5432162	;
    RnonCO2[	542	]=	-0.5431976	;
    RnonCO2[	543	]=	-0.5431946	;
    RnonCO2[	544	]=	-0.5431986	;
    RnonCO2[	545	]=	-0.5432035	;
    RnonCO2[	546	]=	-0.5432044	;
    RnonCO2[	547	]=	-0.5431959	;
    RnonCO2[	548	]=	-0.5431783	;
    RnonCO2[	549	]=	-0.5431553	;
    RnonCO2[	550	]=	-0.5431296	;
    RnonCO2[	551	]=	-0.5431022	;
    RnonCO2[	552	]=	-0.5430738	;
    RnonCO2[	553	]=	-0.5430555	;
    RnonCO2[	554	]=	-0.5430527	;
    RnonCO2[	555	]=	-0.5430567	;
    RnonCO2[	556	]=	-0.5430619	;
    RnonCO2[	557	]=	-0.5430629	;
    RnonCO2[	558	]=	-0.5430545	;
    RnonCO2[	559	]=	-0.543037	;
    RnonCO2[	560	]=	-0.5430142	;
    RnonCO2[	561	]=	-0.5429887	;
    RnonCO2[	562	]=	-0.5429614	;
    RnonCO2[	563	]=	-0.5429332	;
    RnonCO2[	564	]=	-0.5429149	;
    RnonCO2[	565	]=	-0.5429123	;
    RnonCO2[	566	]=	-0.5429164	;
    RnonCO2[	567	]=	-0.5429217	;
    RnonCO2[	568	]=	-0.5429229	;
    RnonCO2[	569	]=	-0.5429147	;
    RnonCO2[	570	]=	-0.5428972	;
    RnonCO2[	571	]=	-0.5428746	;
    RnonCO2[	572	]=	-0.5428492	;
    RnonCO2[	573	]=	-0.542822	;
    RnonCO2[	574	]=	-0.542794	;
    RnonCO2[	575	]=	-0.5427759	;
    RnonCO2[	576	]=	-0.5427733	;
    RnonCO2[	577	]=	-0.5427776	;
    RnonCO2[	578	]=	-0.5427829	;
    RnonCO2[	579	]=	-0.5427842	;
    RnonCO2[	580	]=	-0.5427761	;
    RnonCO2[	581	]=	-0.5427588	;
    RnonCO2[	582	]=	-0.5427364	;
    RnonCO2[	583	]=	-0.5427111	;
    RnonCO2[	584	]=	-0.5426841	;
    RnonCO2[	585	]=	-0.5426562	;
    RnonCO2[	586	]=	-0.5426382	;
    RnonCO2[	587	]=	-0.5426357	;
    RnonCO2[	588	]=	-0.5426401	;
    RnonCO2[	589	]=	-0.5426456	;
    RnonCO2[	590	]=	-0.542647	;
    RnonCO2[	591	]=	-0.542639	;
    RnonCO2[	592	]=	-0.5426218	;
    RnonCO2[	593	]=	-0.5425995	;
    RnonCO2[	594	]=	-0.5425743	;
    RnonCO2[	595	]=	-0.5425474	;
    RnonCO2[	596	]=	-0.5425196	;
    RnonCO2[	597	]=	-0.5425018	;
    RnonCO2[	598	]=	-0.5424994	;
    RnonCO2[	599	]=	-0.5425039	;
    RnonCO2[	600	]=	-0.5425095	;
    RnonCO2[	601	]=	-0.542511	;
    RnonCO2[	602	]=	-0.5425031	;
    RnonCO2[	603	]=	-0.542486	;
    RnonCO2[	604	]=	-0.5424638	;
    RnonCO2[	605	]=	-0.5424388	;
    RnonCO2[	606	]=	-0.5424119	;
    RnonCO2[	607	]=	-0.5423844	;
    RnonCO2[	608	]=	-0.5423665	;
    RnonCO2[	609	]=	-0.5423643	;
    RnonCO2[	610	]=	-0.5423689	;
    RnonCO2[	611	]=	-0.5423746	;
    RnonCO2[	612	]=	-0.5423762	;
    RnonCO2[	613	]=	-0.5423684	;
    RnonCO2[	614	]=	-0.5423514	;
    RnonCO2[	615	]=	-0.5423294	;
    RnonCO2[	616	]=	-0.5423044	;
    RnonCO2[	617	]=	-0.5422777	;
    RnonCO2[	618	]=	-0.5422502	;
    RnonCO2[	619	]=	-0.5422325	;
    RnonCO2[	620	]=	-0.5422303	;
    RnonCO2[	621	]=	-0.5422351	;
    RnonCO2[	622	]=	-0.5422408	;
    RnonCO2[	623	]=	-0.5422426	;
    RnonCO2[	624	]=	-0.5422348	;
    RnonCO2[	625	]=	-0.542218	;
    RnonCO2[	626	]=	-0.542196	;
    RnonCO2[	627	]=	-0.5421712	;
    RnonCO2[	628	]=	-0.5421446	;
    RnonCO2[	629	]=	-0.5421171	;
    RnonCO2[	630	]=	-0.5420996	;
    RnonCO2[	631	]=	-0.5420975	;
    RnonCO2[	632	]=	-0.5421023	;
    RnonCO2[	633	]=	-0.5421081	;
    RnonCO2[	634	]=	-0.54211	;
    RnonCO2[	635	]=	-0.5421024	;
    RnonCO2[	636	]=	-0.5420857	;
    RnonCO2[	637	]=	-0.5420637	;
    RnonCO2[	638	]=	-0.542039	;
    RnonCO2[	639	]=	-0.5420126	;
    RnonCO2[	640	]=	-0.5419852	;
    RnonCO2[	641	]=	-0.5419677	;
    RnonCO2[	642	]=	-0.5419656	;
    RnonCO2[	643	]=	-0.5419706	;
    RnonCO2[	644	]=	-0.5419765	;
    RnonCO2[	645	]=	-0.5419784	;
    RnonCO2[	646	]=	-0.541971	;
    RnonCO2[	647	]=	-0.5419542	;
    RnonCO2[	648	]=	-0.5419325	;
    RnonCO2[	649	]=	-0.5419079	;
    RnonCO2[	650	]=	-0.5418814	;
    RnonCO2[	651	]=	-0.5418542	;
    RnonCO2[	652	]=	-0.5418368	;
    RnonCO2[	653	]=	-0.5418349	;
    RnonCO2[	654	]=	-0.5418399	;
    RnonCO2[	655	]=	-0.5418459	;
    RnonCO2[	656	]=	-0.5418478	;
    RnonCO2[	657	]=	-0.5418405	;
    RnonCO2[	658	]=	-0.5418239	;
    RnonCO2[	659	]=	-0.5418021	;
    RnonCO2[	660	]=	-0.5417777	;
    RnonCO2[	661	]=	-0.5417513	;
    RnonCO2[	662	]=	-0.5417242	;
    RnonCO2[	663	]=	-0.5417069	;
    RnonCO2[	664	]=	-0.541705	;
    RnonCO2[	665	]=	-0.5417102	;
    RnonCO2[	666	]=	-0.5417162	;
    RnonCO2[	667	]=	-0.5417182	;
    RnonCO2[	668	]=	-0.5417109	;
    RnonCO2[	669	]=	-0.5416945	;
    RnonCO2[	670	]=	-0.5416728	;
    RnonCO2[	671	]=	-0.5416484	;
    RnonCO2[	672	]=	-0.5416222	;
    RnonCO2[	673	]=	-0.5415951	;
    RnonCO2[	674	]=	-0.5415779	;
    RnonCO2[	675	]=	-0.5415761	;
    RnonCO2[	676	]=	-0.5415812	;
    RnonCO2[	677	]=	-0.5415873	;
    RnonCO2[	678	]=	-0.5415895	;
    RnonCO2[	679	]=	-0.5415823	;
    RnonCO2[	680	]=	-0.5415659	;
    RnonCO2[	681	]=	-0.5415443	;
    RnonCO2[	682	]=	-0.5415199	;
    RnonCO2[	683	]=	-0.5414939	;
    RnonCO2[	684	]=	-0.5414669	;
    RnonCO2[	685	]=	-0.5414497	;
    RnonCO2[	686	]=	-0.541448	;
    RnonCO2[	687	]=	-0.5414532	;
    RnonCO2[	688	]=	-0.5414594	;
    RnonCO2[	689	]=	-0.5414616	;
    RnonCO2[	690	]=	-0.5414545	;
    RnonCO2[	691	]=	-0.5414382	;
    RnonCO2[	692	]=	-0.5414166	;
    RnonCO2[	693	]=	-0.5413924	;
    RnonCO2[	694	]=	-0.5413663	;
    RnonCO2[	695	]=	-0.5413394	;
    RnonCO2[	696	]=	-0.5413224	;
    RnonCO2[	697	]=	-0.5413208	;
    RnonCO2[	698	]=	-0.541326	;
    RnonCO2[	699	]=	-0.5413324	;
    RnonCO2[	700	]=	-0.5413346	;
    RnonCO2[	701	]=	-0.5413275	;
    RnonCO2[	702	]=	-0.5413112	;
    RnonCO2[	703	]=	-0.5412899	;
    RnonCO2[	704	]=	-0.5412657	;
    RnonCO2[	705	]=	-0.5412398	;
    RnonCO2[	706	]=	-0.5412128	;
    RnonCO2[	707	]=	-0.5411959	;
    RnonCO2[	708	]=	-0.5411943	;
    RnonCO2[	709	]=	-0.5411997	;
    RnonCO2[	710	]=	-0.541206	;
    RnonCO2[	711	]=	-0.5412084	;
    RnonCO2[	712	]=	-0.5412014	;
    RnonCO2[	713	]=	-0.5411852	;
    RnonCO2[	714	]=	-0.5411638	;
    RnonCO2[	715	]=	-0.5411398	;
    RnonCO2[	716	]=	-0.5411139	;
    RnonCO2[	717	]=	-0.5410871	;
    RnonCO2[	718	]=	-0.5410701	;
    RnonCO2[	719	]=	-0.5410686	;
    RnonCO2[	720	]=	-0.5410741	;
    RnonCO2[	721	]=	-0.5410805	;
    RnonCO2[	722	]=	-0.5410829	;
    RnonCO2[	723	]=	-0.5410759	;
    RnonCO2[	724	]=	-0.5410599	;
    RnonCO2[	725	]=	-0.5410386	;
    RnonCO2[	726	]=	-0.5410145	;
    RnonCO2[	727	]=	-0.5409887	;
    RnonCO2[	728	]=	-0.540962	;
    RnonCO2[	729	]=	-0.5409451	;
    RnonCO2[	730	]=	-0.5409437	;
    RnonCO2[	731	]=	-0.5409492	;
    RnonCO2[	732	]=	-0.5409557	;
    RnonCO2[	733	]=	-0.54095816	;
    RnonCO2[	734	]=	-0.54095127	;
    RnonCO2[	735	]=	-0.5409449	;
    
    
    for(int i = 736; i<tmax; i++)
    {
        
        RnonCO2[i] = RnonCO2[i-1];
        
    }
    
}

void setR_WMGHG_nonCO2_RCP6(void)
{
    RnonCO2[	0	]=	0	;
    RnonCO2[	1	]=	0.004314432	;
    RnonCO2[	2	]=	0.008877103	;
    RnonCO2[	3	]=	0.009366935	;
    RnonCO2[	4	]=	0.00984243	;
    RnonCO2[	5	]=	0.010303014	;
    RnonCO2[	6	]=	0.010751385	;
    RnonCO2[	7	]=	0.011190869	;
    RnonCO2[	8	]=	0.01162232	;
    RnonCO2[	9	]=	0.012045243	;
    RnonCO2[	10	]=	0.012459081	;
    RnonCO2[	11	]=	0.012863113	;
    RnonCO2[	12	]=	0.013256576	;
    RnonCO2[	13	]=	0.013638884	;
    RnonCO2[	14	]=	0.014009567	;
    RnonCO2[	15	]=	0.014366637	;
    RnonCO2[	16	]=	0.014706055	;
    RnonCO2[	17	]=	0.015026378	;
    RnonCO2[	18	]=	0.015331384	;
    RnonCO2[	19	]=	0.015626956	;
    RnonCO2[	20	]=	0.015918925	;
    RnonCO2[	21	]=	0.01621293	;
    RnonCO2[	22	]=	0.016514514	;
    RnonCO2[	23	]=	0.016829376	;
    RnonCO2[	24	]=	0.017163338	;
    RnonCO2[	25	]=	0.017521083	;
    RnonCO2[	26	]=	0.017903698	;
    RnonCO2[	27	]=	0.01830912	;
    RnonCO2[	28	]=	0.018735197	;
    RnonCO2[	29	]=	0.019180394	;
    RnonCO2[	30	]=	0.01964318	;
    RnonCO2[	31	]=	0.02012184	;
    RnonCO2[	32	]=	0.02061459	;
    RnonCO2[	33	]=	0.02111978	;
    RnonCO2[	34	]=	0.02163591	;
    RnonCO2[	35	]=	0.02216446	;
    RnonCO2[	36	]=	0.02271248	;
    RnonCO2[	37	]=	0.02328571	;
    RnonCO2[	38	]=	0.0238829	;
    RnonCO2[	39	]=	0.02449959	;
    RnonCO2[	40	]=	0.02513135	;
    RnonCO2[	41	]=	0.02577366	;
    RnonCO2[	42	]=	0.02642202	;
    RnonCO2[	43	]=	0.02707192	;
    RnonCO2[	44	]=	0.02771893	;
    RnonCO2[	45	]=	0.02836089	;
    RnonCO2[	46	]=	0.02900087	;
    RnonCO2[	47	]=	0.02964356	;
    RnonCO2[	48	]=	0.03029043	;
    RnonCO2[	49	]=	0.03094103	;
    RnonCO2[	50	]=	0.03159488	;
    RnonCO2[	51	]=	0.03225155	;
    RnonCO2[	52	]=	0.0329106	;
    RnonCO2[	53	]=	0.03357157	;
    RnonCO2[	54	]=	0.03423393	;
    RnonCO2[	55	]=	0.03489733	;
    RnonCO2[	56	]=	0.03556198	;
    RnonCO2[	57	]=	0.03622841	;
    RnonCO2[	58	]=	0.03689698	;
    RnonCO2[	59	]=	0.03756791	;
    RnonCO2[	60	]=	0.03824135	;
    RnonCO2[	61	]=	0.03891758	;
    RnonCO2[	62	]=	0.03959693	;
    RnonCO2[	63	]=	0.04027961	;
    RnonCO2[	64	]=	0.04096581	;
    RnonCO2[	65	]=	0.04165447	;
    RnonCO2[	66	]=	0.04234275	;
    RnonCO2[	67	]=	0.04302909	;
    RnonCO2[	68	]=	0.04371511	;
    RnonCO2[	69	]=	0.04440374	;
    RnonCO2[	70	]=	0.0450979	;
    RnonCO2[	71	]=	0.04580059	;
    RnonCO2[	72	]=	0.04651493	;
    RnonCO2[	73	]=	0.0472439	;
    RnonCO2[	74	]=	0.04799039	;
    RnonCO2[	75	]=	0.04875957	;
    RnonCO2[	76	]=	0.04956908	;
    RnonCO2[	77	]=	0.05043603	;
    RnonCO2[	78	]=	0.05134235	;
    RnonCO2[	79	]=	0.05223923	;
    RnonCO2[	80	]=	0.05307263	;
    RnonCO2[	81	]=	0.05381898	;
    RnonCO2[	82	]=	0.05450612	;
    RnonCO2[	83	]=	0.05516657	;
    RnonCO2[	84	]=	0.05580469	;
    RnonCO2[	85	]=	0.05649661	;
    RnonCO2[	86	]=	0.05747303	;
    RnonCO2[	87	]=	0.05866701	;
    RnonCO2[	88	]=	0.0598601	;
    RnonCO2[	89	]=	0.0610523	;
    RnonCO2[	90	]=	0.06210314	;
    RnonCO2[	91	]=	0.06315323	;
    RnonCO2[	92	]=	0.06434304	;
    RnonCO2[	93	]=	0.06553198	;
    RnonCO2[	94	]=	0.06672003	;
    RnonCO2[	95	]=	0.06790722	;
    RnonCO2[	96	]=	0.06909353	;
    RnonCO2[	97	]=	0.07027898	;
    RnonCO2[	98	]=	0.07146355	;
    RnonCO2[	99	]=	0.07264725	;
    RnonCO2[	100	]=	0.07396922	;
    RnonCO2[	101	]=	0.0752902	;
    RnonCO2[	102	]=	0.07647106	;
    RnonCO2[	103	]=	0.07765105	;
    RnonCO2[	104	]=	0.07883019	;
    RnonCO2[	105	]=	0.08006188	;
    RnonCO2[	106	]=	0.0812076	;
    RnonCO2[	107	]=	0.08229901	;
    RnonCO2[	108	]=	0.08347458	;
    RnonCO2[	109	]=	0.08464931	;
    RnonCO2[	110	]=	0.08587598	;
    RnonCO2[	111	]=	0.08701708	;
    RnonCO2[	112	]=	0.08810499	;
    RnonCO2[	113	]=	0.08927741	;
    RnonCO2[	114	]=	0.09044939	;
    RnonCO2[	115	]=	0.09175792	;
    RnonCO2[	116	]=	0.09320276	;
    RnonCO2[	117	]=	0.09464664	;
    RnonCO2[	118	]=	0.09608949	;
    RnonCO2[	119	]=	0.09739496	;
    RnonCO2[	120	]=	0.0985634	;
    RnonCO2[	121	]=	0.09986768	;
    RnonCO2[	122	]=	0.10122271	;
    RnonCO2[	123	]=	0.10249213	;
    RnonCO2[	124	]=	0.10384549	;
    RnonCO2[	125	]=	0.10528284	;
    RnonCO2[	126	]=	0.10685466	;
    RnonCO2[	127	]=	0.1085606	;
    RnonCO2[	128	]=	0.11004559	;
    RnonCO2[	129	]=	0.11131004	;
    RnonCO2[	130	]=	0.11270838	;
    RnonCO2[	131	]=	0.1142403	;
    RnonCO2[	132	]=	0.11572158	;
    RnonCO2[	133	]=	0.11720182	;
    RnonCO2[	134	]=	0.11873056	;
    RnonCO2[	135	]=	0.12048167	;
    RnonCO2[	136	]=	0.12271612	;
    RnonCO2[	137	]=	0.12520844	;
    RnonCO2[	138	]=	0.12782821	;
    RnonCO2[	139	]=	0.13057484	;
    RnonCO2[	140	]=	0.13331526	;
    RnonCO2[	141	]=	0.13605041	;
    RnonCO2[	142	]=	0.13887031	;
    RnonCO2[	143	]=	0.14190497	;
    RnonCO2[	144	]=	0.14506366	;
    RnonCO2[	145	]=	0.14808479	;
    RnonCO2[	146	]=	0.15096905	;
    RnonCO2[	147	]=	0.15406256	;
    RnonCO2[	148	]=	0.15742306	;
    RnonCO2[	149	]=	0.16083056	;
    RnonCO2[	150	]=	0.16429017	;
    RnonCO2[	151	]=	0.16780522	;
    RnonCO2[	152	]=	0.17141349	;
    RnonCO2[	153	]=	0.17524964	;
    RnonCO2[	154	]=	0.17912698	;
    RnonCO2[	155	]=	0.18286803	;
    RnonCO2[	156	]=	0.18655793	;
    RnonCO2[	157	]=	0.19028114	;
    RnonCO2[	158	]=	0.19403945	;
    RnonCO2[	159	]=	0.19779207	;
    RnonCO2[	160	]=	0.2015367	;
    RnonCO2[	161	]=	0.20536218	;
    RnonCO2[	162	]=	0.20918465	;
    RnonCO2[	163	]=	0.21283052	;
    RnonCO2[	164	]=	0.2163449	;
    RnonCO2[	165	]=	0.21972929	;
    RnonCO2[	166	]=	0.22302895	;
    RnonCO2[	167	]=	0.22640626	;
    RnonCO2[	168	]=	0.22985789	;
    RnonCO2[	169	]=	0.23320109	;
    RnonCO2[	170	]=	0.2364361	;
    RnonCO2[	171	]=	0.23966811	;
    RnonCO2[	172	]=	0.2428137	;
    RnonCO2[	173	]=	0.24587625	;
    RnonCO2[	174	]=	0.248943	;
    RnonCO2[	175	]=	0.25213499	;
    RnonCO2[	176	]=	0.25554226	;
    RnonCO2[	177	]=	0.25892918	;
    RnonCO2[	178	]=	0.26220864	;
    RnonCO2[	179	]=	0.26570209	;
    RnonCO2[	180	]=	0.26941573	;
    RnonCO2[	181	]=	0.273294	;
    RnonCO2[	182	]=	0.27751544	;
    RnonCO2[	183	]=	0.28201064	;
    RnonCO2[	184	]=	0.28674226	;
    RnonCO2[	185	]=	0.29175313	;
    RnonCO2[	186	]=	0.29696911	;
    RnonCO2[	187	]=	0.30255457	;
    RnonCO2[	188	]=	0.30858029	;
    RnonCO2[	189	]=	0.31473264	;
    RnonCO2[	190	]=	0.32113183	;
    RnonCO2[	191	]=	0.32799691	;
    RnonCO2[	192	]=	0.33518916	;
    RnonCO2[	193	]=	0.34268013	;
    RnonCO2[	194	]=	0.35054912	;
    RnonCO2[	195	]=	0.3588091	;
    RnonCO2[	196	]=	0.36744837	;
    RnonCO2[	197	]=	0.37654998	;
    RnonCO2[	198	]=	0.38629282	;
    RnonCO2[	199	]=	0.39687085	;
    RnonCO2[	200	]=	0.40827014	;
    RnonCO2[	201	]=	0.4202703	;
    RnonCO2[	202	]=	0.43287302	;
    RnonCO2[	203	]=	0.44635066	;
    RnonCO2[	204	]=	0.46086195	;
    RnonCO2[	205	]=	0.47625741	;
    RnonCO2[	206	]=	0.49233715	;
    RnonCO2[	207	]=	0.50922498	;
    RnonCO2[	208	]=	0.52712569	;
    RnonCO2[	209	]=	0.54610485	;
    RnonCO2[	210	]=	0.56604558	;
    RnonCO2[	211	]=	0.58635079	;
    RnonCO2[	212	]=	0.6067464	;
    RnonCO2[	213	]=	0.6271609	;
    RnonCO2[	214	]=	0.6475189	;
    RnonCO2[	215	]=	0.6673124	;
    RnonCO2[	216	]=	0.6879383	;
    RnonCO2[	217	]=	0.7090557	;
    RnonCO2[	218	]=	0.7289172	;
    RnonCO2[	219	]=	0.7480442	;
    RnonCO2[	220	]=	0.766979	;
    RnonCO2[	221	]=	0.7850643	;
    RnonCO2[	222	]=	0.803085	;
    RnonCO2[	223	]=	0.8236521	;
    RnonCO2[	224	]=	0.8452713	;
    RnonCO2[	225	]=	0.864246	;
    RnonCO2[	226	]=	0.8809743	;
    RnonCO2[	227	]=	0.8949711	;
    RnonCO2[	228	]=	0.9053237	;
    RnonCO2[	229	]=	0.914042	;
    RnonCO2[	230	]=	0.9219104	;
    RnonCO2[	231	]=	0.9290394	;
    RnonCO2[	232	]=	0.9359876	;
    RnonCO2[	233	]=	0.9434778	;
    RnonCO2[	234	]=	0.9502339	;
    RnonCO2[	235	]=	0.9548618	;
    RnonCO2[	236	]=	0.9587726	;
    RnonCO2[	237	]=	0.9638465	;
    RnonCO2[	238	]=	0.9690029	;
    RnonCO2[	239	]=	0.972858	;
    RnonCO2[	240	]=	0.9761909	;
    RnonCO2[	241	]=	0.979865	;
    RnonCO2[	242	]=	0.9838607	;
    RnonCO2[	243	]=	0.9881962	;
    RnonCO2[	244	]=	0.9926917	;
    RnonCO2[	245	]=	0.9972453	;
    RnonCO2[	246	]=	1.0018881	;
    RnonCO2[	247	]=	1.0063484	;
    RnonCO2[	248	]=	1.0103817	;
    RnonCO2[	249	]=	1.0140262	;
    RnonCO2[	250	]=	1.0173217	;
    RnonCO2[	251	]=	1.0202781	;
    RnonCO2[	252	]=	1.0229149	;
    RnonCO2[	253	]=	1.0252597	;
    RnonCO2[	254	]=	1.0273159	;
    RnonCO2[	255	]=	1.0290893	;
    RnonCO2[	256	]=	1.0305521	;
    RnonCO2[	257	]=	1.0319004	;
    RnonCO2[	258	]=	1.0333688	;
    RnonCO2[	259	]=	1.0349681	;
    RnonCO2[	260	]=	1.0367072	;
    RnonCO2[	261	]=	1.0385984	;
    RnonCO2[	262	]=	1.0406353	;
    RnonCO2[	263	]=	1.042799	;
    RnonCO2[	264	]=	1.045076	;
    RnonCO2[	265	]=	1.0474575	;
    RnonCO2[	266	]=	1.0499391	;
    RnonCO2[	267	]=	1.0524481	;
    RnonCO2[	268	]=	1.054891	;
    RnonCO2[	269	]=	1.0572297	;
    RnonCO2[	270	]=	1.0594342	;
    RnonCO2[	271	]=	1.0614837	;
    RnonCO2[	272	]=	1.0633689	;
    RnonCO2[	273	]=	1.0650698	;
    RnonCO2[	274	]=	1.0665599	;
    RnonCO2[	275	]=	1.0678224	;
    RnonCO2[	276	]=	1.0688481	;
    RnonCO2[	277	]=	1.0696204	;
    RnonCO2[	278	]=	1.070133	;
    RnonCO2[	279	]=	1.0704387	;
    RnonCO2[	280	]=	1.0706254	;
    RnonCO2[	281	]=	1.0707631	;
    RnonCO2[	282	]=	1.0709097	;
    RnonCO2[	283	]=	1.0711173	;
    RnonCO2[	284	]=	1.0714131	;
    RnonCO2[	285	]=	1.0718053	;
    RnonCO2[	286	]=	1.0723038	;
    RnonCO2[	287	]=	1.0729023	;
    RnonCO2[	288	]=	1.0735907	;
    RnonCO2[	289	]=	1.0743409	;
    RnonCO2[	290	]=	1.0751375	;
    RnonCO2[	291	]=	1.0760106	;
    RnonCO2[	292	]=	1.0769825	;
    RnonCO2[	293	]=	1.0780711	;
    RnonCO2[	294	]=	1.0792954	;
    RnonCO2[	295	]=	1.0806557	;
    RnonCO2[	296	]=	1.0821355	;
    RnonCO2[	297	]=	1.0836592	;
    RnonCO2[	298	]=	1.0851638	;
    RnonCO2[	299	]=	1.0866533	;
    RnonCO2[	300	]=	1.0880982	;
    RnonCO2[	301	]=	1.0894825	;
    RnonCO2[	302	]=	1.0908356	;
    RnonCO2[	303	]=	1.0921806	;
    RnonCO2[	304	]=	1.0935374	;
    RnonCO2[	305	]=	1.0949267	;
    RnonCO2[	306	]=	1.0963533	;
    RnonCO2[	307	]=	1.0976809	;
    RnonCO2[	308	]=	1.0987856	;
    RnonCO2[	309	]=	1.0996843	;
    RnonCO2[	310	]=	1.1003929	;
    RnonCO2[	311	]=	1.100894	;
    RnonCO2[	312	]=	1.1011824	;
    RnonCO2[	313	]=	1.1012974	;
    RnonCO2[	314	]=	1.1012716	;
    RnonCO2[	315	]=	1.1011335	;
    RnonCO2[	316	]=	1.1009134	;
    RnonCO2[	317	]=	1.1002909	;
    RnonCO2[	318	]=	1.0989556	;
    RnonCO2[	319	]=	1.0969561	;
    RnonCO2[	320	]=	1.0943407	;
    RnonCO2[	321	]=	1.0911536	;
    RnonCO2[	322	]=	1.0874039	;
    RnonCO2[	323	]=	1.0831111	;
    RnonCO2[	324	]=	1.0783352	;
    RnonCO2[	325	]=	1.0731283	;
    RnonCO2[	326	]=	1.0675364	;
    RnonCO2[	327	]=	1.0618862	;
    RnonCO2[	328	]=	1.0564668	;
    RnonCO2[	329	]=	1.0512526	;
    RnonCO2[	330	]=	1.0462228	;
    RnonCO2[	331	]=	1.0413613	;
    RnonCO2[	332	]=	1.0366536	;
    RnonCO2[	333	]=	1.0321188	;
    RnonCO2[	334	]=	1.0278132	;
    RnonCO2[	335	]=	1.0237623	;
    RnonCO2[	336	]=	1.0199536	;
    RnonCO2[	337	]=	1.0165209	;
    RnonCO2[	338	]=	1.0135833	;
    RnonCO2[	339	]=	1.0110843	;
    RnonCO2[	340	]=	1.0089625	;
    RnonCO2[	341	]=	1.0071658	;
    RnonCO2[	342	]=	1.0056514	;
    RnonCO2[	343	]=	1.0043812	;
    RnonCO2[	344	]=	1.0032942	;
    RnonCO2[	345	]=	1.0023453	;
    RnonCO2[	346	]=	1.0015329	;
    RnonCO2[	347	]=	1.000853	;
    RnonCO2[	348	]=	1.0003004	;
    RnonCO2[	349	]=	0.9998729	;
    RnonCO2[	350	]=	0.999555	;
    RnonCO2[	351	]=	0.9993217	;
    RnonCO2[	352	]=	0.999153	;
    RnonCO2[	353	]=	0.9990352	;
    RnonCO2[	354	]=	0.9989564	;
    RnonCO2[	355	]=	0.9988787	;
    RnonCO2[	356	]=	0.9987782	;
    RnonCO2[	357	]=	0.9986708	;
    RnonCO2[	358	]=	0.9985689	;
    RnonCO2[	359	]=	0.9984817	;
    RnonCO2[	360	]=	0.9984199	;
    RnonCO2[	361	]=	0.9983791	;
    RnonCO2[	362	]=	0.9983445	;
    RnonCO2[	363	]=	0.9983059	;
    RnonCO2[	364	]=	0.9982573	;
    RnonCO2[	365	]=	0.998194	;
    RnonCO2[	366	]=	0.9980849	;
    RnonCO2[	367	]=	0.9979118	;
    RnonCO2[	368	]=	0.9976956	;
    RnonCO2[	369	]=	0.9974531	;
    RnonCO2[	370	]=	0.9971971	;
    RnonCO2[	371	]=	0.9969416	;
    RnonCO2[	372	]=	0.9966855	;
    RnonCO2[	373	]=	0.9964167	;
    RnonCO2[	374	]=	0.9961273	;
    RnonCO2[	375	]=	0.9958138	;
    RnonCO2[	376	]=	0.995473	;
    RnonCO2[	377	]=	0.9950759	;
    RnonCO2[	378	]=	0.994606	;
    RnonCO2[	379	]=	0.9940855	;
    RnonCO2[	380	]=	0.9935321	;
    RnonCO2[	381	]=	0.9929597	;
    RnonCO2[	382	]=	0.992383	;
    RnonCO2[	383	]=	0.9918017	;
    RnonCO2[	384	]=	0.9912045	;
    RnonCO2[	385	]=	0.9909006	;
    RnonCO2[	386	]=	0.9909006	;
    RnonCO2[	387	]=	0.9909006	;
    RnonCO2[	388	]=	0.9909006	;
    RnonCO2[	389	]=	0.9909006	;
    RnonCO2[	390	]=	0.9909006	;
    RnonCO2[	391	]=	0.9909006	;
    RnonCO2[	392	]=	0.9909006	;
    RnonCO2[	393	]=	0.9909006	;
    RnonCO2[	394	]=	0.9909006	;
    RnonCO2[	395	]=	0.9909006	;
    RnonCO2[	396	]=	0.9909006	;
    RnonCO2[	397	]=	0.9909006	;
    RnonCO2[	398	]=	0.9909006	;
    RnonCO2[	399	]=	0.9909006	;
    RnonCO2[	400	]=	0.9909006	;
    RnonCO2[	401	]=	0.9909006	;
    RnonCO2[	402	]=	0.9909006	;
    RnonCO2[	403	]=	0.9909006	;
    RnonCO2[	404	]=	0.9909006	;
    RnonCO2[	405	]=	0.9909006	;
    RnonCO2[	406	]=	0.9909006	;
    RnonCO2[	407	]=	0.9909006	;
    RnonCO2[	408	]=	0.9909006	;
    RnonCO2[	409	]=	0.9909006	;
    RnonCO2[	410	]=	0.9909006	;
    RnonCO2[	411	]=	0.9909006	;
    RnonCO2[	412	]=	0.9909006	;
    RnonCO2[	413	]=	0.9909006	;
    RnonCO2[	414	]=	0.9909006	;
    RnonCO2[	415	]=	0.9909006	;
    RnonCO2[	416	]=	0.9909006	;
    RnonCO2[	417	]=	0.9909006	;
    RnonCO2[	418	]=	0.9909006	;
    RnonCO2[	419	]=	0.9909006	;
    RnonCO2[	420	]=	0.9909006	;
    RnonCO2[	421	]=	0.9909006	;
    RnonCO2[	422	]=	0.9909006	;
    RnonCO2[	423	]=	0.9909006	;
    RnonCO2[	424	]=	0.9909006	;
    RnonCO2[	425	]=	0.9909006	;
    RnonCO2[	426	]=	0.9909006	;
    RnonCO2[	427	]=	0.9909006	;
    RnonCO2[	428	]=	0.9909006	;
    RnonCO2[	429	]=	0.9909006	;
    RnonCO2[	430	]=	0.9909006	;
    RnonCO2[	431	]=	0.9909006	;
    RnonCO2[	432	]=	0.9909006	;
    RnonCO2[	433	]=	0.9909006	;
    RnonCO2[	434	]=	0.9909006	;
    RnonCO2[	435	]=	0.9909006	;
    RnonCO2[	436	]=	0.9909006	;
    RnonCO2[	437	]=	0.9909006	;
    RnonCO2[	438	]=	0.9909006	;
    RnonCO2[	439	]=	0.9909006	;
    RnonCO2[	440	]=	0.9909006	;
    RnonCO2[	441	]=	0.9909006	;
    RnonCO2[	442	]=	0.9909006	;
    RnonCO2[	443	]=	0.9909006	;
    RnonCO2[	444	]=	0.9909006	;
    RnonCO2[	445	]=	0.9909006	;
    RnonCO2[	446	]=	0.9909006	;
    RnonCO2[	447	]=	0.9909006	;
    RnonCO2[	448	]=	0.9909006	;
    RnonCO2[	449	]=	0.9909006	;
    RnonCO2[	450	]=	0.9909006	;
    RnonCO2[	451	]=	0.9909006	;
    RnonCO2[	452	]=	0.9909006	;
    RnonCO2[	453	]=	0.9909006	;
    RnonCO2[	454	]=	0.9909006	;
    RnonCO2[	455	]=	0.9909006	;
    RnonCO2[	456	]=	0.9909006	;
    RnonCO2[	457	]=	0.9909006	;
    RnonCO2[	458	]=	0.9909006	;
    RnonCO2[	459	]=	0.9909006	;
    RnonCO2[	460	]=	0.9909006	;
    RnonCO2[	461	]=	0.9909006	;
    RnonCO2[	462	]=	0.9909006	;
    RnonCO2[	463	]=	0.9909006	;
    RnonCO2[	464	]=	0.9909006	;
    RnonCO2[	465	]=	0.9909006	;
    RnonCO2[	466	]=	0.9909006	;
    RnonCO2[	467	]=	0.9909006	;
    RnonCO2[	468	]=	0.9909006	;
    RnonCO2[	469	]=	0.9909006	;
    RnonCO2[	470	]=	0.9909006	;
    RnonCO2[	471	]=	0.9909006	;
    RnonCO2[	472	]=	0.9909006	;
    RnonCO2[	473	]=	0.9909006	;
    RnonCO2[	474	]=	0.9909006	;
    RnonCO2[	475	]=	0.9909006	;
    RnonCO2[	476	]=	0.9909006	;
    RnonCO2[	477	]=	0.9909006	;
    RnonCO2[	478	]=	0.9909006	;
    RnonCO2[	479	]=	0.9909006	;
    RnonCO2[	480	]=	0.9909006	;
    RnonCO2[	481	]=	0.9909006	;
    RnonCO2[	482	]=	0.9909006	;
    RnonCO2[	483	]=	0.9909006	;
    RnonCO2[	484	]=	0.9909006	;
    RnonCO2[	485	]=	0.9909006	;
    RnonCO2[	486	]=	0.9909006	;
    RnonCO2[	487	]=	0.9909006	;
    RnonCO2[	488	]=	0.9909006	;
    RnonCO2[	489	]=	0.9909006	;
    RnonCO2[	490	]=	0.9909006	;
    RnonCO2[	491	]=	0.9909006	;
    RnonCO2[	492	]=	0.9909006	;
    RnonCO2[	493	]=	0.9909006	;
    RnonCO2[	494	]=	0.9909006	;
    RnonCO2[	495	]=	0.9909006	;
    RnonCO2[	496	]=	0.9909006	;
    RnonCO2[	497	]=	0.9909006	;
    RnonCO2[	498	]=	0.9909006	;
    RnonCO2[	499	]=	0.9909006	;
    RnonCO2[	500	]=	0.9909006	;
    RnonCO2[	501	]=	0.9909006	;
    RnonCO2[	502	]=	0.9909006	;
    RnonCO2[	503	]=	0.9909006	;
    RnonCO2[	504	]=	0.9909006	;
    RnonCO2[	505	]=	0.9909006	;
    RnonCO2[	506	]=	0.9909006	;
    RnonCO2[	507	]=	0.9909006	;
    RnonCO2[	508	]=	0.9909006	;
    RnonCO2[	509	]=	0.9909006	;
    RnonCO2[	510	]=	0.9909006	;
    RnonCO2[	511	]=	0.9909006	;
    RnonCO2[	512	]=	0.9909006	;
    RnonCO2[	513	]=	0.9909006	;
    RnonCO2[	514	]=	0.9909006	;
    RnonCO2[	515	]=	0.9909006	;
    RnonCO2[	516	]=	0.9909006	;
    RnonCO2[	517	]=	0.9909006	;
    RnonCO2[	518	]=	0.9909006	;
    RnonCO2[	519	]=	0.9909006	;
    RnonCO2[	520	]=	0.9909006	;
    RnonCO2[	521	]=	0.9909006	;
    RnonCO2[	522	]=	0.9909006	;
    RnonCO2[	523	]=	0.9909006	;
    RnonCO2[	524	]=	0.9909006	;
    RnonCO2[	525	]=	0.9909006	;
    RnonCO2[	526	]=	0.9909006	;
    RnonCO2[	527	]=	0.9909006	;
    RnonCO2[	528	]=	0.9909006	;
    RnonCO2[	529	]=	0.9909006	;
    RnonCO2[	530	]=	0.9909006	;
    RnonCO2[	531	]=	0.9909006	;
    RnonCO2[	532	]=	0.9909006	;
    RnonCO2[	533	]=	0.9909006	;
    RnonCO2[	534	]=	0.9909006	;
    RnonCO2[	535	]=	0.9909006	;
    RnonCO2[	536	]=	0.9909006	;
    RnonCO2[	537	]=	0.9909006	;
    RnonCO2[	538	]=	0.9909006	;
    RnonCO2[	539	]=	0.9909006	;
    RnonCO2[	540	]=	0.9909006	;
    RnonCO2[	541	]=	0.9909006	;
    RnonCO2[	542	]=	0.9909006	;
    RnonCO2[	543	]=	0.9909006	;
    RnonCO2[	544	]=	0.9909006	;
    RnonCO2[	545	]=	0.9909006	;
    RnonCO2[	546	]=	0.9909006	;
    RnonCO2[	547	]=	0.9909006	;
    RnonCO2[	548	]=	0.9909006	;
    RnonCO2[	549	]=	0.9909006	;
    RnonCO2[	550	]=	0.9909006	;
    RnonCO2[	551	]=	0.9909006	;
    RnonCO2[	552	]=	0.9909006	;
    RnonCO2[	553	]=	0.9909006	;
    RnonCO2[	554	]=	0.9909006	;
    RnonCO2[	555	]=	0.9909006	;
    RnonCO2[	556	]=	0.9909006	;
    RnonCO2[	557	]=	0.9909006	;
    RnonCO2[	558	]=	0.9909006	;
    RnonCO2[	559	]=	0.9909006	;
    RnonCO2[	560	]=	0.9909006	;
    RnonCO2[	561	]=	0.9909006	;
    RnonCO2[	562	]=	0.9909006	;
    RnonCO2[	563	]=	0.9909006	;
    RnonCO2[	564	]=	0.9909006	;
    RnonCO2[	565	]=	0.9909006	;
    RnonCO2[	566	]=	0.9909006	;
    RnonCO2[	567	]=	0.9909006	;
    RnonCO2[	568	]=	0.9909006	;
    RnonCO2[	569	]=	0.9909006	;
    RnonCO2[	570	]=	0.9909006	;
    RnonCO2[	571	]=	0.9909006	;
    RnonCO2[	572	]=	0.9909006	;
    RnonCO2[	573	]=	0.9909006	;
    RnonCO2[	574	]=	0.9909006	;
    RnonCO2[	575	]=	0.9909006	;
    RnonCO2[	576	]=	0.9909006	;
    RnonCO2[	577	]=	0.9909006	;
    RnonCO2[	578	]=	0.9909006	;
    RnonCO2[	579	]=	0.9909006	;
    RnonCO2[	580	]=	0.9909006	;
    RnonCO2[	581	]=	0.9909006	;
    RnonCO2[	582	]=	0.9909006	;
    RnonCO2[	583	]=	0.9909006	;
    RnonCO2[	584	]=	0.9909006	;
    RnonCO2[	585	]=	0.9909006	;
    RnonCO2[	586	]=	0.9909006	;
    RnonCO2[	587	]=	0.9909006	;
    RnonCO2[	588	]=	0.9909006	;
    RnonCO2[	589	]=	0.9909006	;
    RnonCO2[	590	]=	0.9909006	;
    RnonCO2[	591	]=	0.9909006	;
    RnonCO2[	592	]=	0.9909006	;
    RnonCO2[	593	]=	0.9909006	;
    RnonCO2[	594	]=	0.9909006	;
    RnonCO2[	595	]=	0.9909006	;
    RnonCO2[	596	]=	0.9909006	;
    RnonCO2[	597	]=	0.9909006	;
    RnonCO2[	598	]=	0.9909006	;
    RnonCO2[	599	]=	0.9909006	;
    RnonCO2[	600	]=	0.9909006	;
    RnonCO2[	601	]=	0.9909006	;
    RnonCO2[	602	]=	0.9909006	;
    RnonCO2[	603	]=	0.9909006	;
    RnonCO2[	604	]=	0.9909006	;
    RnonCO2[	605	]=	0.9909006	;
    RnonCO2[	606	]=	0.9909006	;
    RnonCO2[	607	]=	0.9909006	;
    RnonCO2[	608	]=	0.9909006	;
    RnonCO2[	609	]=	0.9909006	;
    RnonCO2[	610	]=	0.9909006	;
    RnonCO2[	611	]=	0.9909006	;
    RnonCO2[	612	]=	0.9909006	;
    RnonCO2[	613	]=	0.9909006	;
    RnonCO2[	614	]=	0.9909006	;
    RnonCO2[	615	]=	0.9909006	;
    RnonCO2[	616	]=	0.9909006	;
    RnonCO2[	617	]=	0.9909006	;
    RnonCO2[	618	]=	0.9909006	;
    RnonCO2[	619	]=	0.9909006	;
    RnonCO2[	620	]=	0.9909006	;
    RnonCO2[	621	]=	0.9909006	;
    RnonCO2[	622	]=	0.9909006	;
    RnonCO2[	623	]=	0.9909006	;
    RnonCO2[	624	]=	0.9909006	;
    RnonCO2[	625	]=	0.9909006	;
    RnonCO2[	626	]=	0.9909006	;
    RnonCO2[	627	]=	0.9909006	;
    RnonCO2[	628	]=	0.9909006	;
    RnonCO2[	629	]=	0.9909006	;
    RnonCO2[	630	]=	0.9909006	;
    RnonCO2[	631	]=	0.9909006	;
    RnonCO2[	632	]=	0.9909006	;
    RnonCO2[	633	]=	0.9909006	;
    RnonCO2[	634	]=	0.9909006	;
    RnonCO2[	635	]=	0.9909006	;
    RnonCO2[	636	]=	0.9909006	;
    RnonCO2[	637	]=	0.9909006	;
    RnonCO2[	638	]=	0.9909006	;
    RnonCO2[	639	]=	0.9909006	;
    RnonCO2[	640	]=	0.9909006	;
    RnonCO2[	641	]=	0.9909006	;
    RnonCO2[	642	]=	0.9909006	;
    RnonCO2[	643	]=	0.9909006	;
    RnonCO2[	644	]=	0.9909006	;
    RnonCO2[	645	]=	0.9909006	;
    RnonCO2[	646	]=	0.9909006	;
    RnonCO2[	647	]=	0.9909006	;
    RnonCO2[	648	]=	0.9909006	;
    RnonCO2[	649	]=	0.9909006	;
    RnonCO2[	650	]=	0.9909006	;
    RnonCO2[	651	]=	0.9909006	;
    RnonCO2[	652	]=	0.9909006	;
    RnonCO2[	653	]=	0.9909006	;
    RnonCO2[	654	]=	0.9909006	;
    RnonCO2[	655	]=	0.9909006	;
    RnonCO2[	656	]=	0.9909006	;
    RnonCO2[	657	]=	0.9909006	;
    RnonCO2[	658	]=	0.9909006	;
    RnonCO2[	659	]=	0.9909006	;
    RnonCO2[	660	]=	0.9909006	;
    RnonCO2[	661	]=	0.9909006	;
    RnonCO2[	662	]=	0.9909006	;
    RnonCO2[	663	]=	0.9909006	;
    RnonCO2[	664	]=	0.9909006	;
    RnonCO2[	665	]=	0.9909006	;
    RnonCO2[	666	]=	0.9909006	;
    RnonCO2[	667	]=	0.9909006	;
    RnonCO2[	668	]=	0.9909006	;
    RnonCO2[	669	]=	0.9909006	;
    RnonCO2[	670	]=	0.9909006	;
    RnonCO2[	671	]=	0.9909006	;
    RnonCO2[	672	]=	0.9909006	;
    RnonCO2[	673	]=	0.9909006	;
    RnonCO2[	674	]=	0.9909006	;
    RnonCO2[	675	]=	0.9909006	;
    RnonCO2[	676	]=	0.9909006	;
    RnonCO2[	677	]=	0.9909006	;
    RnonCO2[	678	]=	0.9909006	;
    RnonCO2[	679	]=	0.9909006	;
    RnonCO2[	680	]=	0.9909006	;
    RnonCO2[	681	]=	0.9909006	;
    RnonCO2[	682	]=	0.9909006	;
    RnonCO2[	683	]=	0.9909006	;
    RnonCO2[	684	]=	0.9909006	;
    RnonCO2[	685	]=	0.9909006	;
    RnonCO2[	686	]=	0.9909006	;
    RnonCO2[	687	]=	0.9909006	;
    RnonCO2[	688	]=	0.9909006	;
    RnonCO2[	689	]=	0.9909006	;
    RnonCO2[	690	]=	0.9909006	;
    RnonCO2[	691	]=	0.9909006	;
    RnonCO2[	692	]=	0.9909006	;
    RnonCO2[	693	]=	0.9909006	;
    RnonCO2[	694	]=	0.9909006	;
    RnonCO2[	695	]=	0.9909006	;
    RnonCO2[	696	]=	0.9909006	;
    RnonCO2[	697	]=	0.9909006	;
    RnonCO2[	698	]=	0.9909006	;
    RnonCO2[	699	]=	0.9909006	;
    RnonCO2[	700	]=	0.9909006	;
    RnonCO2[	701	]=	0.9909006	;
    RnonCO2[	702	]=	0.9909006	;
    RnonCO2[	703	]=	0.9909006	;
    RnonCO2[	704	]=	0.9909006	;
    RnonCO2[	705	]=	0.9909006	;
    RnonCO2[	706	]=	0.9909006	;
    RnonCO2[	707	]=	0.9909006	;
    RnonCO2[	708	]=	0.9909006	;
    RnonCO2[	709	]=	0.9909006	;
    RnonCO2[	710	]=	0.9909006	;
    RnonCO2[	711	]=	0.9909006	;
    RnonCO2[	712	]=	0.9909006	;
    RnonCO2[	713	]=	0.9909006	;
    RnonCO2[	714	]=	0.9909006	;
    RnonCO2[	715	]=	0.9909006	;
    RnonCO2[	716	]=	0.9909006	;
    RnonCO2[	717	]=	0.9909006	;
    RnonCO2[	718	]=	0.9909006	;
    RnonCO2[	719	]=	0.9909006	;
    RnonCO2[	720	]=	0.9909006	;
    RnonCO2[	721	]=	0.9909006	;
    RnonCO2[	722	]=	0.9909006	;
    RnonCO2[	723	]=	0.9909006	;
    RnonCO2[	724	]=	0.9909006	;
    RnonCO2[	725	]=	0.9909006	;
    RnonCO2[	726	]=	0.9909006	;
    RnonCO2[	727	]=	0.9909006	;
    RnonCO2[	728	]=	0.9909006	;
    RnonCO2[	729	]=	0.9909006	;
    RnonCO2[	730	]=	0.9909006	;
    RnonCO2[	731	]=	0.9909006	;
    RnonCO2[	732	]=	0.9909006	;
    RnonCO2[	733	]=	0.9909006	;
    RnonCO2[	734	]=	0.9909006	;
    RnonCO2[	735	]=	0.9909006	;
    
    for(int i = 736; i<tmax; i++)
    {
        
        RnonCO2[i] = RnonCO2[i-1];
        
    }
    
}

void setR_WMGHG_nonCO2_RCP45(void)
{
    RnonCO2[	0	]=	0	;
    RnonCO2[	1	]=	0.004314432	;
    RnonCO2[	2	]=	0.008877103	;
    RnonCO2[	3	]=	0.009366935	;
    RnonCO2[	4	]=	0.00984243	;
    RnonCO2[	5	]=	0.010303014	;
    RnonCO2[	6	]=	0.010751385	;
    RnonCO2[	7	]=	0.011190869	;
    RnonCO2[	8	]=	0.01162232	;
    RnonCO2[	9	]=	0.012045243	;
    RnonCO2[	10	]=	0.012459081	;
    RnonCO2[	11	]=	0.012863113	;
    RnonCO2[	12	]=	0.013256576	;
    RnonCO2[	13	]=	0.013638884	;
    RnonCO2[	14	]=	0.014009567	;
    RnonCO2[	15	]=	0.014366637	;
    RnonCO2[	16	]=	0.014706055	;
    RnonCO2[	17	]=	0.015026378	;
    RnonCO2[	18	]=	0.015331384	;
    RnonCO2[	19	]=	0.015626956	;
    RnonCO2[	20	]=	0.015918925	;
    RnonCO2[	21	]=	0.01621293	;
    RnonCO2[	22	]=	0.016514514	;
    RnonCO2[	23	]=	0.016829376	;
    RnonCO2[	24	]=	0.017163338	;
    RnonCO2[	25	]=	0.017521083	;
    RnonCO2[	26	]=	0.017903698	;
    RnonCO2[	27	]=	0.01830912	;
    RnonCO2[	28	]=	0.018735197	;
    RnonCO2[	29	]=	0.019180394	;
    RnonCO2[	30	]=	0.01964318	;
    RnonCO2[	31	]=	0.02012184	;
    RnonCO2[	32	]=	0.02061459	;
    RnonCO2[	33	]=	0.02111978	;
    RnonCO2[	34	]=	0.02163591	;
    RnonCO2[	35	]=	0.02216446	;
    RnonCO2[	36	]=	0.02271248	;
    RnonCO2[	37	]=	0.02328571	;
    RnonCO2[	38	]=	0.0238829	;
    RnonCO2[	39	]=	0.02449959	;
    RnonCO2[	40	]=	0.02513135	;
    RnonCO2[	41	]=	0.02577366	;
    RnonCO2[	42	]=	0.02642202	;
    RnonCO2[	43	]=	0.02707192	;
    RnonCO2[	44	]=	0.02771893	;
    RnonCO2[	45	]=	0.02836089	;
    RnonCO2[	46	]=	0.02900087	;
    RnonCO2[	47	]=	0.02964356	;
    RnonCO2[	48	]=	0.03029043	;
    RnonCO2[	49	]=	0.03094103	;
    RnonCO2[	50	]=	0.03159488	;
    RnonCO2[	51	]=	0.03225155	;
    RnonCO2[	52	]=	0.0329106	;
    RnonCO2[	53	]=	0.03357157	;
    RnonCO2[	54	]=	0.03423393	;
    RnonCO2[	55	]=	0.03489733	;
    RnonCO2[	56	]=	0.03556198	;
    RnonCO2[	57	]=	0.03622841	;
    RnonCO2[	58	]=	0.03689698	;
    RnonCO2[	59	]=	0.03756791	;
    RnonCO2[	60	]=	0.03824135	;
    RnonCO2[	61	]=	0.03891758	;
    RnonCO2[	62	]=	0.03959693	;
    RnonCO2[	63	]=	0.04027961	;
    RnonCO2[	64	]=	0.04096581	;
    RnonCO2[	65	]=	0.04165447	;
    RnonCO2[	66	]=	0.04234275	;
    RnonCO2[	67	]=	0.04302909	;
    RnonCO2[	68	]=	0.04371511	;
    RnonCO2[	69	]=	0.04440374	;
    RnonCO2[	70	]=	0.0450979	;
    RnonCO2[	71	]=	0.04580059	;
    RnonCO2[	72	]=	0.04651493	;
    RnonCO2[	73	]=	0.0472439	;
    RnonCO2[	74	]=	0.04799039	;
    RnonCO2[	75	]=	0.04875957	;
    RnonCO2[	76	]=	0.04956908	;
    RnonCO2[	77	]=	0.05043603	;
    RnonCO2[	78	]=	0.05134235	;
    RnonCO2[	79	]=	0.05223923	;
    RnonCO2[	80	]=	0.05307263	;
    RnonCO2[	81	]=	0.05381898	;
    RnonCO2[	82	]=	0.05450612	;
    RnonCO2[	83	]=	0.05516657	;
    RnonCO2[	84	]=	0.05580469	;
    RnonCO2[	85	]=	0.05649661	;
    RnonCO2[	86	]=	0.05747303	;
    RnonCO2[	87	]=	0.05866701	;
    RnonCO2[	88	]=	0.0598601	;
    RnonCO2[	89	]=	0.0610523	;
    RnonCO2[	90	]=	0.06210314	;
    RnonCO2[	91	]=	0.06315323	;
    RnonCO2[	92	]=	0.06434304	;
    RnonCO2[	93	]=	0.06553198	;
    RnonCO2[	94	]=	0.06672003	;
    RnonCO2[	95	]=	0.06790722	;
    RnonCO2[	96	]=	0.06909353	;
    RnonCO2[	97	]=	0.07027898	;
    RnonCO2[	98	]=	0.07146355	;
    RnonCO2[	99	]=	0.07264725	;
    RnonCO2[	100	]=	0.07396922	;
    RnonCO2[	101	]=	0.0752902	;
    RnonCO2[	102	]=	0.07647106	;
    RnonCO2[	103	]=	0.07765105	;
    RnonCO2[	104	]=	0.07883019	;
    RnonCO2[	105	]=	0.08006188	;
    RnonCO2[	106	]=	0.0812076	;
    RnonCO2[	107	]=	0.08229901	;
    RnonCO2[	108	]=	0.08347458	;
    RnonCO2[	109	]=	0.08464931	;
    RnonCO2[	110	]=	0.08587598	;
    RnonCO2[	111	]=	0.08701708	;
    RnonCO2[	112	]=	0.08810499	;
    RnonCO2[	113	]=	0.08927741	;
    RnonCO2[	114	]=	0.09044939	;
    RnonCO2[	115	]=	0.09175792	;
    RnonCO2[	116	]=	0.09320276	;
    RnonCO2[	117	]=	0.09464664	;
    RnonCO2[	118	]=	0.09608949	;
    RnonCO2[	119	]=	0.09739496	;
    RnonCO2[	120	]=	0.0985634	;
    RnonCO2[	121	]=	0.09986768	;
    RnonCO2[	122	]=	0.10122271	;
    RnonCO2[	123	]=	0.10249213	;
    RnonCO2[	124	]=	0.10384549	;
    RnonCO2[	125	]=	0.10528284	;
    RnonCO2[	126	]=	0.10685466	;
    RnonCO2[	127	]=	0.1085606	;
    RnonCO2[	128	]=	0.11004559	;
    RnonCO2[	129	]=	0.11131004	;
    RnonCO2[	130	]=	0.11270838	;
    RnonCO2[	131	]=	0.1142403	;
    RnonCO2[	132	]=	0.11572158	;
    RnonCO2[	133	]=	0.11720182	;
    RnonCO2[	134	]=	0.11873056	;
    RnonCO2[	135	]=	0.12048167	;
    RnonCO2[	136	]=	0.12271612	;
    RnonCO2[	137	]=	0.12520844	;
    RnonCO2[	138	]=	0.12782821	;
    RnonCO2[	139	]=	0.13057484	;
    RnonCO2[	140	]=	0.13331526	;
    RnonCO2[	141	]=	0.13605041	;
    RnonCO2[	142	]=	0.13887031	;
    RnonCO2[	143	]=	0.14190497	;
    RnonCO2[	144	]=	0.14506366	;
    RnonCO2[	145	]=	0.14808479	;
    RnonCO2[	146	]=	0.15096905	;
    RnonCO2[	147	]=	0.15406256	;
    RnonCO2[	148	]=	0.15742306	;
    RnonCO2[	149	]=	0.16083056	;
    RnonCO2[	150	]=	0.16429017	;
    RnonCO2[	151	]=	0.16780522	;
    RnonCO2[	152	]=	0.17141349	;
    RnonCO2[	153	]=	0.17524964	;
    RnonCO2[	154	]=	0.17912698	;
    RnonCO2[	155	]=	0.18286803	;
    RnonCO2[	156	]=	0.18655793	;
    RnonCO2[	157	]=	0.19028114	;
    RnonCO2[	158	]=	0.19403945	;
    RnonCO2[	159	]=	0.19779207	;
    RnonCO2[	160	]=	0.2015367	;
    RnonCO2[	161	]=	0.20536218	;
    RnonCO2[	162	]=	0.20918465	;
    RnonCO2[	163	]=	0.21283052	;
    RnonCO2[	164	]=	0.2163449	;
    RnonCO2[	165	]=	0.21972929	;
    RnonCO2[	166	]=	0.22302895	;
    RnonCO2[	167	]=	0.22640626	;
    RnonCO2[	168	]=	0.22985789	;
    RnonCO2[	169	]=	0.23320109	;
    RnonCO2[	170	]=	0.2364361	;
    RnonCO2[	171	]=	0.23966811	;
    RnonCO2[	172	]=	0.2428137	;
    RnonCO2[	173	]=	0.24587625	;
    RnonCO2[	174	]=	0.248943	;
    RnonCO2[	175	]=	0.25213499	;
    RnonCO2[	176	]=	0.25554226	;
    RnonCO2[	177	]=	0.25892918	;
    RnonCO2[	178	]=	0.26220864	;
    RnonCO2[	179	]=	0.26570209	;
    RnonCO2[	180	]=	0.26941573	;
    RnonCO2[	181	]=	0.273294	;
    RnonCO2[	182	]=	0.27751544	;
    RnonCO2[	183	]=	0.28201064	;
    RnonCO2[	184	]=	0.28674226	;
    RnonCO2[	185	]=	0.29175313	;
    RnonCO2[	186	]=	0.29696911	;
    RnonCO2[	187	]=	0.30255457	;
    RnonCO2[	188	]=	0.30858029	;
    RnonCO2[	189	]=	0.31473264	;
    RnonCO2[	190	]=	0.32113183	;
    RnonCO2[	191	]=	0.32799691	;
    RnonCO2[	192	]=	0.33518916	;
    RnonCO2[	193	]=	0.34268013	;
    RnonCO2[	194	]=	0.35054912	;
    RnonCO2[	195	]=	0.3588091	;
    RnonCO2[	196	]=	0.36744837	;
    RnonCO2[	197	]=	0.37654998	;
    RnonCO2[	198	]=	0.38629282	;
    RnonCO2[	199	]=	0.39687085	;
    RnonCO2[	200	]=	0.40827014	;
    RnonCO2[	201	]=	0.4202703	;
    RnonCO2[	202	]=	0.43287302	;
    RnonCO2[	203	]=	0.44635066	;
    RnonCO2[	204	]=	0.46086195	;
    RnonCO2[	205	]=	0.47625741	;
    RnonCO2[	206	]=	0.49233715	;
    RnonCO2[	207	]=	0.50922498	;
    RnonCO2[	208	]=	0.52712569	;
    RnonCO2[	209	]=	0.54610485	;
    RnonCO2[	210	]=	0.56604558	;
    RnonCO2[	211	]=	0.58635079	;
    RnonCO2[	212	]=	0.6067464	;
    RnonCO2[	213	]=	0.6271609	;
    RnonCO2[	214	]=	0.6475189	;
    RnonCO2[	215	]=	0.6673124	;
    RnonCO2[	216	]=	0.6879383	;
    RnonCO2[	217	]=	0.7090557	;
    RnonCO2[	218	]=	0.7289172	;
    RnonCO2[	219	]=	0.7480442	;
    RnonCO2[	220	]=	0.766979	;
    RnonCO2[	221	]=	0.7850643	;
    RnonCO2[	222	]=	0.803085	;
    RnonCO2[	223	]=	0.8236521	;
    RnonCO2[	224	]=	0.8452713	;
    RnonCO2[	225	]=	0.864246	;
    RnonCO2[	226	]=	0.8809743	;
    RnonCO2[	227	]=	0.8949711	;
    RnonCO2[	228	]=	0.9053237	;
    RnonCO2[	229	]=	0.914042	;
    RnonCO2[	230	]=	0.9219104	;
    RnonCO2[	231	]=	0.9290394	;
    RnonCO2[	232	]=	0.9359876	;
    RnonCO2[	233	]=	0.9434778	;
    RnonCO2[	234	]=	0.9502339	;
    RnonCO2[	235	]=	0.9548618	;
    RnonCO2[	236	]=	0.9587726	;
    RnonCO2[	237	]=	0.9638465	;
    RnonCO2[	238	]=	0.9690029	;
    RnonCO2[	239	]=	0.972858	;
    RnonCO2[	240	]=	0.9761909	;
    RnonCO2[	241	]=	0.9798636	;
    RnonCO2[	242	]=	0.9837885	;
    RnonCO2[	243	]=	0.9879216	;
    RnonCO2[	244	]=	0.9920736	;
    RnonCO2[	245	]=	0.9961362	;
    RnonCO2[	246	]=	1.0001534	;
    RnonCO2[	247	]=	1.0041406	;
    RnonCO2[	248	]=	1.0081179	;
    RnonCO2[	249	]=	1.0120888	;
    RnonCO2[	250	]=	1.0160626	;
    RnonCO2[	251	]=	1.0200208	;
    RnonCO2[	252	]=	1.0239574	;
    RnonCO2[	253	]=	1.0278762	;
    RnonCO2[	254	]=	1.0317588	;
    RnonCO2[	255	]=	1.0355918	;
    RnonCO2[	256	]=	1.0393303	;
    RnonCO2[	257	]=	1.0428685	;
    RnonCO2[	258	]=	1.04616	;
    RnonCO2[	259	]=	1.0492419	;
    RnonCO2[	260	]=	1.0521483	;
    RnonCO2[	261	]=	1.054914	;
    RnonCO2[	262	]=	1.0575542	;
    RnonCO2[	263	]=	1.0600695	;
    RnonCO2[	264	]=	1.0624641	;
    RnonCO2[	265	]=	1.0647461	;
    RnonCO2[	266	]=	1.0669226	;
    RnonCO2[	267	]=	1.0688765	;
    RnonCO2[	268	]=	1.0704761	;
    RnonCO2[	269	]=	1.0717049	;
    RnonCO2[	270	]=	1.0725519	;
    RnonCO2[	271	]=	1.0730135	;
    RnonCO2[	272	]=	1.0730966	;
    RnonCO2[	273	]=	1.0727968	;
    RnonCO2[	274	]=	1.072101	;
    RnonCO2[	275	]=	1.0710055	;
    RnonCO2[	276	]=	1.0695146	;
    RnonCO2[	277	]=	1.0676264	;
    RnonCO2[	278	]=	1.0653457	;
    RnonCO2[	279	]=	1.0627346	;
    RnonCO2[	280	]=	1.0598897	;
    RnonCO2[	281	]=	1.0568894	;
    RnonCO2[	282	]=	1.0537991	;
    RnonCO2[	283	]=	1.0506775	;
    RnonCO2[	284	]=	1.0475581	;
    RnonCO2[	285	]=	1.0444547	;
    RnonCO2[	286	]=	1.0413816	;
    RnonCO2[	287	]=	1.0382878	;
    RnonCO2[	288	]=	1.0351228	;
    RnonCO2[	289	]=	1.0318695	;
    RnonCO2[	290	]=	1.0285234	;
    RnonCO2[	291	]=	1.025124	;
    RnonCO2[	292	]=	1.0217023	;
    RnonCO2[	293	]=	1.0182846	;
    RnonCO2[	294	]=	1.0148969	;
    RnonCO2[	295	]=	1.0115463	;
    RnonCO2[	296	]=	1.0082259	;
    RnonCO2[	297	]=	1.0049176	;
    RnonCO2[	298	]=	1.0016079	;
    RnonCO2[	299	]=	0.9982987	;
    RnonCO2[	300	]=	0.9949603	;
    RnonCO2[	301	]=	0.9915764	;
    RnonCO2[	302	]=	0.988175	;
    RnonCO2[	303	]=	0.9847778	;
    RnonCO2[	304	]=	0.981403	;
    RnonCO2[	305	]=	0.9780696	;
    RnonCO2[	306	]=	0.97478	;
    RnonCO2[	307	]=	0.9715057	;
    RnonCO2[	308	]=	0.968224	;
    RnonCO2[	309	]=	0.9649359	;
    RnonCO2[	310	]=	0.9616425	;
    RnonCO2[	311	]=	0.958314	;
    RnonCO2[	312	]=	0.9549343	;
    RnonCO2[	313	]=	0.9515372	;
    RnonCO2[	314	]=	0.948194	;
    RnonCO2[	315	]=	0.9449594	;
    RnonCO2[	316	]=	0.9418428	;
    RnonCO2[	317	]=	0.9389288	;
    RnonCO2[	318	]=	0.9362802	;
    RnonCO2[	319	]=	0.933865	;
    RnonCO2[	320	]=	0.9316588	;
    RnonCO2[	321	]=	0.9296395	;
    RnonCO2[	322	]=	0.9277583	;
    RnonCO2[	323	]=	0.9259826	;
    RnonCO2[	324	]=	0.9243221	;
    RnonCO2[	325	]=	0.9227825	;
    RnonCO2[	326	]=	0.9213668	;
    RnonCO2[	327	]=	0.9200788	;
    RnonCO2[	328	]=	0.9189076	;
    RnonCO2[	329	]=	0.9178341	;
    RnonCO2[	330	]=	0.9168437	;
    RnonCO2[	331	]=	0.9159271	;
    RnonCO2[	332	]=	0.9150758	;
    RnonCO2[	333	]=	0.9142542	;
    RnonCO2[	334	]=	0.9134406	;
    RnonCO2[	335	]=	0.9126545	;
    RnonCO2[	336	]=	0.9119101	;
    RnonCO2[	337	]=	0.911246	;
    RnonCO2[	338	]=	0.9106951	;
    RnonCO2[	339	]=	0.9102435	;
    RnonCO2[	340	]=	0.9098674	;
    RnonCO2[	341	]=	0.9095496	;
    RnonCO2[	342	]=	0.909278	;
    RnonCO2[	343	]=	0.9090426	;
    RnonCO2[	344	]=	0.9088065	;
    RnonCO2[	345	]=	0.9085477	;
    RnonCO2[	346	]=	0.9082848	;
    RnonCO2[	347	]=	0.9080314	;
    RnonCO2[	348	]=	0.9077981	;
    RnonCO2[	349	]=	0.9075967	;
    RnonCO2[	350	]=	0.9074242	;
    RnonCO2[	351	]=	0.9072668	;
    RnonCO2[	352	]=	0.9071152	;
    RnonCO2[	353	]=	0.9069647	;
    RnonCO2[	354	]=	0.9068114	;
    RnonCO2[	355	]=	0.9066242	;
    RnonCO2[	356	]=	0.9063857	;
    RnonCO2[	357	]=	0.9061185	;
    RnonCO2[	358	]=	0.9058394	;
    RnonCO2[	359	]=	0.9055618	;
    RnonCO2[	360	]=	0.9053001	;
    RnonCO2[	361	]=	0.9050535	;
    RnonCO2[	362	]=	0.9048099	;
    RnonCO2[	363	]=	0.9045621	;
    RnonCO2[	364	]=	0.9043065	;
    RnonCO2[	365	]=	0.9040407	;
    RnonCO2[	366	]=	0.9037348	;
    RnonCO2[	367	]=	0.9033725	;
    RnonCO2[	368	]=	0.9029771	;
    RnonCO2[	369	]=	0.902566	;
    RnonCO2[	370	]=	0.9021532	;
    RnonCO2[	371	]=	0.9017535	;
    RnonCO2[	372	]=	0.9013665	;
    RnonCO2[	373	]=	0.9009809	;
    RnonCO2[	374	]=	0.9005896	;
    RnonCO2[	375	]=	0.9001895	;
    RnonCO2[	376	]=	0.8997785	;
    RnonCO2[	377	]=	0.899327	;
    RnonCO2[	378	]=	0.8988193	;
    RnonCO2[	379	]=	0.8982785	;
    RnonCO2[	380	]=	0.8977223	;
    RnonCO2[	381	]=	0.8971646	;
    RnonCO2[	382	]=	0.8966201	;
    RnonCO2[	383	]=	0.8960888	;
    RnonCO2[	384	]=	0.8955593	;
    RnonCO2[	385	]=	0.8952938	;
    RnonCO2[	386	]=	0.8952938	;
    RnonCO2[	387	]=	0.8952938	;
    RnonCO2[	388	]=	0.8952938	;
    RnonCO2[	389	]=	0.8952938	;
    RnonCO2[	390	]=	0.8952938	;
    RnonCO2[	391	]=	0.8952938	;
    RnonCO2[	392	]=	0.8952938	;
    RnonCO2[	393	]=	0.8952938	;
    RnonCO2[	394	]=	0.8952938	;
    RnonCO2[	395	]=	0.8952938	;
    RnonCO2[	396	]=	0.8952938	;
    RnonCO2[	397	]=	0.8952938	;
    RnonCO2[	398	]=	0.8952938	;
    RnonCO2[	399	]=	0.8952938	;
    RnonCO2[	400	]=	0.8952938	;
    RnonCO2[	401	]=	0.8952938	;
    RnonCO2[	402	]=	0.8952938	;
    RnonCO2[	403	]=	0.8952938	;
    RnonCO2[	404	]=	0.8952938	;
    RnonCO2[	405	]=	0.8952938	;
    RnonCO2[	406	]=	0.8952938	;
    RnonCO2[	407	]=	0.8952938	;
    RnonCO2[	408	]=	0.8952938	;
    RnonCO2[	409	]=	0.8952938	;
    RnonCO2[	410	]=	0.8952938	;
    RnonCO2[	411	]=	0.8952938	;
    RnonCO2[	412	]=	0.8952938	;
    RnonCO2[	413	]=	0.8952938	;
    RnonCO2[	414	]=	0.8952938	;
    RnonCO2[	415	]=	0.8952938	;
    RnonCO2[	416	]=	0.8952938	;
    RnonCO2[	417	]=	0.8952938	;
    RnonCO2[	418	]=	0.8952938	;
    RnonCO2[	419	]=	0.8952938	;
    RnonCO2[	420	]=	0.8952938	;
    RnonCO2[	421	]=	0.8952938	;
    RnonCO2[	422	]=	0.8952938	;
    RnonCO2[	423	]=	0.8952938	;
    RnonCO2[	424	]=	0.8952938	;
    RnonCO2[	425	]=	0.8952938	;
    RnonCO2[	426	]=	0.8952938	;
    RnonCO2[	427	]=	0.8952938	;
    RnonCO2[	428	]=	0.8952938	;
    RnonCO2[	429	]=	0.8952938	;
    RnonCO2[	430	]=	0.8952938	;
    RnonCO2[	431	]=	0.8952938	;
    RnonCO2[	432	]=	0.8952938	;
    RnonCO2[	433	]=	0.8952938	;
    RnonCO2[	434	]=	0.8952938	;
    RnonCO2[	435	]=	0.8952938	;
    RnonCO2[	436	]=	0.8952938	;
    RnonCO2[	437	]=	0.8952938	;
    RnonCO2[	438	]=	0.8952938	;
    RnonCO2[	439	]=	0.8952938	;
    RnonCO2[	440	]=	0.8952938	;
    RnonCO2[	441	]=	0.8952938	;
    RnonCO2[	442	]=	0.8952938	;
    RnonCO2[	443	]=	0.8952938	;
    RnonCO2[	444	]=	0.8952938	;
    RnonCO2[	445	]=	0.8952938	;
    RnonCO2[	446	]=	0.8952938	;
    RnonCO2[	447	]=	0.8952938	;
    RnonCO2[	448	]=	0.8952938	;
    RnonCO2[	449	]=	0.8952938	;
    RnonCO2[	450	]=	0.8952938	;
    RnonCO2[	451	]=	0.8952938	;
    RnonCO2[	452	]=	0.8952938	;
    RnonCO2[	453	]=	0.8952938	;
    RnonCO2[	454	]=	0.8952938	;
    RnonCO2[	455	]=	0.8952938	;
    RnonCO2[	456	]=	0.8952938	;
    RnonCO2[	457	]=	0.8952938	;
    RnonCO2[	458	]=	0.8952938	;
    RnonCO2[	459	]=	0.8952938	;
    RnonCO2[	460	]=	0.8952938	;
    RnonCO2[	461	]=	0.8952938	;
    RnonCO2[	462	]=	0.8952938	;
    RnonCO2[	463	]=	0.8952938	;
    RnonCO2[	464	]=	0.8952938	;
    RnonCO2[	465	]=	0.8952938	;
    RnonCO2[	466	]=	0.8952938	;
    RnonCO2[	467	]=	0.8952938	;
    RnonCO2[	468	]=	0.8952938	;
    RnonCO2[	469	]=	0.8952938	;
    RnonCO2[	470	]=	0.8952938	;
    RnonCO2[	471	]=	0.8952938	;
    RnonCO2[	472	]=	0.8952938	;
    RnonCO2[	473	]=	0.8952938	;
    RnonCO2[	474	]=	0.8952938	;
    RnonCO2[	475	]=	0.8952938	;
    RnonCO2[	476	]=	0.8952938	;
    RnonCO2[	477	]=	0.8952938	;
    RnonCO2[	478	]=	0.8952938	;
    RnonCO2[	479	]=	0.8952938	;
    RnonCO2[	480	]=	0.8952938	;
    RnonCO2[	481	]=	0.8952938	;
    RnonCO2[	482	]=	0.8952938	;
    RnonCO2[	483	]=	0.8952938	;
    RnonCO2[	484	]=	0.8952938	;
    RnonCO2[	485	]=	0.8952938	;
    RnonCO2[	486	]=	0.8952938	;
    RnonCO2[	487	]=	0.8952938	;
    RnonCO2[	488	]=	0.8952938	;
    RnonCO2[	489	]=	0.8952938	;
    RnonCO2[	490	]=	0.8952938	;
    RnonCO2[	491	]=	0.8952938	;
    RnonCO2[	492	]=	0.8952938	;
    RnonCO2[	493	]=	0.8952938	;
    RnonCO2[	494	]=	0.8952938	;
    RnonCO2[	495	]=	0.8952938	;
    RnonCO2[	496	]=	0.8952938	;
    RnonCO2[	497	]=	0.8952938	;
    RnonCO2[	498	]=	0.8952938	;
    RnonCO2[	499	]=	0.8952938	;
    RnonCO2[	500	]=	0.8952938	;
    RnonCO2[	501	]=	0.8952938	;
    RnonCO2[	502	]=	0.8952938	;
    RnonCO2[	503	]=	0.8952938	;
    RnonCO2[	504	]=	0.8952938	;
    RnonCO2[	505	]=	0.8952938	;
    RnonCO2[	506	]=	0.8952938	;
    RnonCO2[	507	]=	0.8952938	;
    RnonCO2[	508	]=	0.8952938	;
    RnonCO2[	509	]=	0.8952938	;
    RnonCO2[	510	]=	0.8952938	;
    RnonCO2[	511	]=	0.8952938	;
    RnonCO2[	512	]=	0.8952938	;
    RnonCO2[	513	]=	0.8952938	;
    RnonCO2[	514	]=	0.8952938	;
    RnonCO2[	515	]=	0.8952938	;
    RnonCO2[	516	]=	0.8952938	;
    RnonCO2[	517	]=	0.8952938	;
    RnonCO2[	518	]=	0.8952938	;
    RnonCO2[	519	]=	0.8952938	;
    RnonCO2[	520	]=	0.8952938	;
    RnonCO2[	521	]=	0.8952938	;
    RnonCO2[	522	]=	0.8952938	;
    RnonCO2[	523	]=	0.8952938	;
    RnonCO2[	524	]=	0.8952938	;
    RnonCO2[	525	]=	0.8952938	;
    RnonCO2[	526	]=	0.8952938	;
    RnonCO2[	527	]=	0.8952938	;
    RnonCO2[	528	]=	0.8952938	;
    RnonCO2[	529	]=	0.8952938	;
    RnonCO2[	530	]=	0.8952938	;
    RnonCO2[	531	]=	0.8952938	;
    RnonCO2[	532	]=	0.8952938	;
    RnonCO2[	533	]=	0.8952938	;
    RnonCO2[	534	]=	0.8952938	;
    RnonCO2[	535	]=	0.8952938	;
    RnonCO2[	536	]=	0.8952938	;
    RnonCO2[	537	]=	0.8952938	;
    RnonCO2[	538	]=	0.8952938	;
    RnonCO2[	539	]=	0.8952938	;
    RnonCO2[	540	]=	0.8952938	;
    RnonCO2[	541	]=	0.8952938	;
    RnonCO2[	542	]=	0.8952938	;
    RnonCO2[	543	]=	0.8952938	;
    RnonCO2[	544	]=	0.8952938	;
    RnonCO2[	545	]=	0.8952938	;
    RnonCO2[	546	]=	0.8952938	;
    RnonCO2[	547	]=	0.8952938	;
    RnonCO2[	548	]=	0.8952938	;
    RnonCO2[	549	]=	0.8952938	;
    RnonCO2[	550	]=	0.8952938	;
    RnonCO2[	551	]=	0.8952938	;
    RnonCO2[	552	]=	0.8952938	;
    RnonCO2[	553	]=	0.8952938	;
    RnonCO2[	554	]=	0.8952938	;
    RnonCO2[	555	]=	0.8952938	;
    RnonCO2[	556	]=	0.8952938	;
    RnonCO2[	557	]=	0.8952938	;
    RnonCO2[	558	]=	0.8952938	;
    RnonCO2[	559	]=	0.8952938	;
    RnonCO2[	560	]=	0.8952938	;
    RnonCO2[	561	]=	0.8952938	;
    RnonCO2[	562	]=	0.8952938	;
    RnonCO2[	563	]=	0.8952938	;
    RnonCO2[	564	]=	0.8952938	;
    RnonCO2[	565	]=	0.8952938	;
    RnonCO2[	566	]=	0.8952938	;
    RnonCO2[	567	]=	0.8952938	;
    RnonCO2[	568	]=	0.8952938	;
    RnonCO2[	569	]=	0.8952938	;
    RnonCO2[	570	]=	0.8952938	;
    RnonCO2[	571	]=	0.8952938	;
    RnonCO2[	572	]=	0.8952938	;
    RnonCO2[	573	]=	0.8952938	;
    RnonCO2[	574	]=	0.8952938	;
    RnonCO2[	575	]=	0.8952938	;
    RnonCO2[	576	]=	0.8952938	;
    RnonCO2[	577	]=	0.8952938	;
    RnonCO2[	578	]=	0.8952938	;
    RnonCO2[	579	]=	0.8952938	;
    RnonCO2[	580	]=	0.8952938	;
    RnonCO2[	581	]=	0.8952938	;
    RnonCO2[	582	]=	0.8952938	;
    RnonCO2[	583	]=	0.8952938	;
    RnonCO2[	584	]=	0.8952938	;
    RnonCO2[	585	]=	0.8952938	;
    RnonCO2[	586	]=	0.8952938	;
    RnonCO2[	587	]=	0.8952938	;
    RnonCO2[	588	]=	0.8952938	;
    RnonCO2[	589	]=	0.8952938	;
    RnonCO2[	590	]=	0.8952938	;
    RnonCO2[	591	]=	0.8952938	;
    RnonCO2[	592	]=	0.8952938	;
    RnonCO2[	593	]=	0.8952938	;
    RnonCO2[	594	]=	0.8952938	;
    RnonCO2[	595	]=	0.8952938	;
    RnonCO2[	596	]=	0.8952938	;
    RnonCO2[	597	]=	0.8952938	;
    RnonCO2[	598	]=	0.8952938	;
    RnonCO2[	599	]=	0.8952938	;
    RnonCO2[	600	]=	0.8952938	;
    RnonCO2[	601	]=	0.8952938	;
    RnonCO2[	602	]=	0.8952938	;
    RnonCO2[	603	]=	0.8952938	;
    RnonCO2[	604	]=	0.8952938	;
    RnonCO2[	605	]=	0.8952938	;
    RnonCO2[	606	]=	0.8952938	;
    RnonCO2[	607	]=	0.8952938	;
    RnonCO2[	608	]=	0.8952938	;
    RnonCO2[	609	]=	0.8952938	;
    RnonCO2[	610	]=	0.8952938	;
    RnonCO2[	611	]=	0.8952938	;
    RnonCO2[	612	]=	0.8952938	;
    RnonCO2[	613	]=	0.8952938	;
    RnonCO2[	614	]=	0.8952938	;
    RnonCO2[	615	]=	0.8952938	;
    RnonCO2[	616	]=	0.8952938	;
    RnonCO2[	617	]=	0.8952938	;
    RnonCO2[	618	]=	0.8952938	;
    RnonCO2[	619	]=	0.8952938	;
    RnonCO2[	620	]=	0.8952938	;
    RnonCO2[	621	]=	0.8952938	;
    RnonCO2[	622	]=	0.8952938	;
    RnonCO2[	623	]=	0.8952938	;
    RnonCO2[	624	]=	0.8952938	;
    RnonCO2[	625	]=	0.8952938	;
    RnonCO2[	626	]=	0.8952938	;
    RnonCO2[	627	]=	0.8952938	;
    RnonCO2[	628	]=	0.8952938	;
    RnonCO2[	629	]=	0.8952938	;
    RnonCO2[	630	]=	0.8952938	;
    RnonCO2[	631	]=	0.8952938	;
    RnonCO2[	632	]=	0.8952938	;
    RnonCO2[	633	]=	0.8952938	;
    RnonCO2[	634	]=	0.8952938	;
    RnonCO2[	635	]=	0.8952938	;
    RnonCO2[	636	]=	0.8952938	;
    RnonCO2[	637	]=	0.8952938	;
    RnonCO2[	638	]=	0.8952938	;
    RnonCO2[	639	]=	0.8952938	;
    RnonCO2[	640	]=	0.8952938	;
    RnonCO2[	641	]=	0.8952938	;
    RnonCO2[	642	]=	0.8952938	;
    RnonCO2[	643	]=	0.8952938	;
    RnonCO2[	644	]=	0.8952938	;
    RnonCO2[	645	]=	0.8952938	;
    RnonCO2[	646	]=	0.8952938	;
    RnonCO2[	647	]=	0.8952938	;
    RnonCO2[	648	]=	0.8952938	;
    RnonCO2[	649	]=	0.8952938	;
    RnonCO2[	650	]=	0.8952938	;
    RnonCO2[	651	]=	0.8952938	;
    RnonCO2[	652	]=	0.8952938	;
    RnonCO2[	653	]=	0.8952938	;
    RnonCO2[	654	]=	0.8952938	;
    RnonCO2[	655	]=	0.8952938	;
    RnonCO2[	656	]=	0.8952938	;
    RnonCO2[	657	]=	0.8952938	;
    RnonCO2[	658	]=	0.8952938	;
    RnonCO2[	659	]=	0.8952938	;
    RnonCO2[	660	]=	0.8952938	;
    RnonCO2[	661	]=	0.8952938	;
    RnonCO2[	662	]=	0.8952938	;
    RnonCO2[	663	]=	0.8952938	;
    RnonCO2[	664	]=	0.8952938	;
    RnonCO2[	665	]=	0.8952938	;
    RnonCO2[	666	]=	0.8952938	;
    RnonCO2[	667	]=	0.8952938	;
    RnonCO2[	668	]=	0.8952938	;
    RnonCO2[	669	]=	0.8952938	;
    RnonCO2[	670	]=	0.8952938	;
    RnonCO2[	671	]=	0.8952938	;
    RnonCO2[	672	]=	0.8952938	;
    RnonCO2[	673	]=	0.8952938	;
    RnonCO2[	674	]=	0.8952938	;
    RnonCO2[	675	]=	0.8952938	;
    RnonCO2[	676	]=	0.8952938	;
    RnonCO2[	677	]=	0.8952938	;
    RnonCO2[	678	]=	0.8952938	;
    RnonCO2[	679	]=	0.8952938	;
    RnonCO2[	680	]=	0.8952938	;
    RnonCO2[	681	]=	0.8952938	;
    RnonCO2[	682	]=	0.8952938	;
    RnonCO2[	683	]=	0.8952938	;
    RnonCO2[	684	]=	0.8952938	;
    RnonCO2[	685	]=	0.8952938	;
    RnonCO2[	686	]=	0.8952938	;
    RnonCO2[	687	]=	0.8952938	;
    RnonCO2[	688	]=	0.8952938	;
    RnonCO2[	689	]=	0.8952938	;
    RnonCO2[	690	]=	0.8952938	;
    RnonCO2[	691	]=	0.8952938	;
    RnonCO2[	692	]=	0.8952938	;
    RnonCO2[	693	]=	0.8952938	;
    RnonCO2[	694	]=	0.8952938	;
    RnonCO2[	695	]=	0.8952938	;
    RnonCO2[	696	]=	0.8952938	;
    RnonCO2[	697	]=	0.8952938	;
    RnonCO2[	698	]=	0.8952938	;
    RnonCO2[	699	]=	0.8952938	;
    RnonCO2[	700	]=	0.8952938	;
    RnonCO2[	701	]=	0.8952938	;
    RnonCO2[	702	]=	0.8952938	;
    RnonCO2[	703	]=	0.8952938	;
    RnonCO2[	704	]=	0.8952938	;
    RnonCO2[	705	]=	0.8952938	;
    RnonCO2[	706	]=	0.8952938	;
    RnonCO2[	707	]=	0.8952938	;
    RnonCO2[	708	]=	0.8952938	;
    RnonCO2[	709	]=	0.8952938	;
    RnonCO2[	710	]=	0.8952938	;
    RnonCO2[	711	]=	0.8952938	;
    RnonCO2[	712	]=	0.8952938	;
    RnonCO2[	713	]=	0.8952938	;
    RnonCO2[	714	]=	0.8952938	;
    RnonCO2[	715	]=	0.8952938	;
    RnonCO2[	716	]=	0.8952938	;
    RnonCO2[	717	]=	0.8952938	;
    RnonCO2[	718	]=	0.8952938	;
    RnonCO2[	719	]=	0.8952938	;
    RnonCO2[	720	]=	0.8952938	;
    RnonCO2[	721	]=	0.8952938	;
    RnonCO2[	722	]=	0.8952938	;
    RnonCO2[	723	]=	0.8952938	;
    RnonCO2[	724	]=	0.8952938	;
    RnonCO2[	725	]=	0.8952938	;
    RnonCO2[	726	]=	0.8952938	;
    RnonCO2[	727	]=	0.8952938	;
    RnonCO2[	728	]=	0.8952938	;
    RnonCO2[	729	]=	0.8952938	;
    RnonCO2[	730	]=	0.8952938	;
    RnonCO2[	731	]=	0.8952938	;
    RnonCO2[	732	]=	0.8952938	;
    RnonCO2[	733	]=	0.8952938	;
    RnonCO2[	734	]=	0.8952938	;
    RnonCO2[	735	]=	0.8952938	;
    
    for(int i = 736; i<tmax; i++)
    {
        
        RnonCO2[i] = RnonCO2[i-1];
        
    }
    
}

void setR_WMGHG_nonCO2_RCP3PD(void)
{
    RnonCO2[	0	]=	0	;
    RnonCO2[	1	]=	0.004314432	;
    RnonCO2[	2	]=	0.008877103	;
    RnonCO2[	3	]=	0.009366935	;
    RnonCO2[	4	]=	0.00984243	;
    RnonCO2[	5	]=	0.010303014	;
    RnonCO2[	6	]=	0.010751385	;
    RnonCO2[	7	]=	0.011190869	;
    RnonCO2[	8	]=	0.01162232	;
    RnonCO2[	9	]=	0.012045243	;
    RnonCO2[	10	]=	0.012459081	;
    RnonCO2[	11	]=	0.012863113	;
    RnonCO2[	12	]=	0.013256576	;
    RnonCO2[	13	]=	0.013638884	;
    RnonCO2[	14	]=	0.014009567	;
    RnonCO2[	15	]=	0.014366637	;
    RnonCO2[	16	]=	0.014706055	;
    RnonCO2[	17	]=	0.015026378	;
    RnonCO2[	18	]=	0.015331384	;
    RnonCO2[	19	]=	0.015626956	;
    RnonCO2[	20	]=	0.015918925	;
    RnonCO2[	21	]=	0.01621293	;
    RnonCO2[	22	]=	0.016514514	;
    RnonCO2[	23	]=	0.016829376	;
    RnonCO2[	24	]=	0.017163338	;
    RnonCO2[	25	]=	0.017521083	;
    RnonCO2[	26	]=	0.017903698	;
    RnonCO2[	27	]=	0.01830912	;
    RnonCO2[	28	]=	0.018735197	;
    RnonCO2[	29	]=	0.019180394	;
    RnonCO2[	30	]=	0.01964318	;
    RnonCO2[	31	]=	0.02012184	;
    RnonCO2[	32	]=	0.02061459	;
    RnonCO2[	33	]=	0.02111978	;
    RnonCO2[	34	]=	0.02163591	;
    RnonCO2[	35	]=	0.02216446	;
    RnonCO2[	36	]=	0.02271248	;
    RnonCO2[	37	]=	0.02328571	;
    RnonCO2[	38	]=	0.0238829	;
    RnonCO2[	39	]=	0.02449959	;
    RnonCO2[	40	]=	0.02513135	;
    RnonCO2[	41	]=	0.02577366	;
    RnonCO2[	42	]=	0.02642202	;
    RnonCO2[	43	]=	0.02707192	;
    RnonCO2[	44	]=	0.02771893	;
    RnonCO2[	45	]=	0.02836089	;
    RnonCO2[	46	]=	0.02900087	;
    RnonCO2[	47	]=	0.02964356	;
    RnonCO2[	48	]=	0.03029043	;
    RnonCO2[	49	]=	0.03094103	;
    RnonCO2[	50	]=	0.03159488	;
    RnonCO2[	51	]=	0.03225155	;
    RnonCO2[	52	]=	0.0329106	;
    RnonCO2[	53	]=	0.03357157	;
    RnonCO2[	54	]=	0.03423393	;
    RnonCO2[	55	]=	0.03489733	;
    RnonCO2[	56	]=	0.03556198	;
    RnonCO2[	57	]=	0.03622841	;
    RnonCO2[	58	]=	0.03689698	;
    RnonCO2[	59	]=	0.03756791	;
    RnonCO2[	60	]=	0.03824135	;
    RnonCO2[	61	]=	0.03891758	;
    RnonCO2[	62	]=	0.03959693	;
    RnonCO2[	63	]=	0.04027961	;
    RnonCO2[	64	]=	0.04096581	;
    RnonCO2[	65	]=	0.04165447	;
    RnonCO2[	66	]=	0.04234275	;
    RnonCO2[	67	]=	0.04302909	;
    RnonCO2[	68	]=	0.04371511	;
    RnonCO2[	69	]=	0.04440374	;
    RnonCO2[	70	]=	0.0450979	;
    RnonCO2[	71	]=	0.04580059	;
    RnonCO2[	72	]=	0.04651493	;
    RnonCO2[	73	]=	0.0472439	;
    RnonCO2[	74	]=	0.04799039	;
    RnonCO2[	75	]=	0.04875957	;
    RnonCO2[	76	]=	0.04956908	;
    RnonCO2[	77	]=	0.05043603	;
    RnonCO2[	78	]=	0.05134235	;
    RnonCO2[	79	]=	0.05223923	;
    RnonCO2[	80	]=	0.05307263	;
    RnonCO2[	81	]=	0.05381898	;
    RnonCO2[	82	]=	0.05450612	;
    RnonCO2[	83	]=	0.05516657	;
    RnonCO2[	84	]=	0.05580469	;
    RnonCO2[	85	]=	0.05649661	;
    RnonCO2[	86	]=	0.05747303	;
    RnonCO2[	87	]=	0.05866701	;
    RnonCO2[	88	]=	0.0598601	;
    RnonCO2[	89	]=	0.0610523	;
    RnonCO2[	90	]=	0.06210314	;
    RnonCO2[	91	]=	0.06315323	;
    RnonCO2[	92	]=	0.06434304	;
    RnonCO2[	93	]=	0.06553198	;
    RnonCO2[	94	]=	0.06672003	;
    RnonCO2[	95	]=	0.06790722	;
    RnonCO2[	96	]=	0.06909353	;
    RnonCO2[	97	]=	0.07027898	;
    RnonCO2[	98	]=	0.07146355	;
    RnonCO2[	99	]=	0.07264725	;
    RnonCO2[	100	]=	0.07396922	;
    RnonCO2[	101	]=	0.0752902	;
    RnonCO2[	102	]=	0.07647106	;
    RnonCO2[	103	]=	0.07765105	;
    RnonCO2[	104	]=	0.07883019	;
    RnonCO2[	105	]=	0.08006188	;
    RnonCO2[	106	]=	0.0812076	;
    RnonCO2[	107	]=	0.08229901	;
    RnonCO2[	108	]=	0.08347458	;
    RnonCO2[	109	]=	0.08464931	;
    RnonCO2[	110	]=	0.08587598	;
    RnonCO2[	111	]=	0.08701708	;
    RnonCO2[	112	]=	0.08810499	;
    RnonCO2[	113	]=	0.08927741	;
    RnonCO2[	114	]=	0.09044939	;
    RnonCO2[	115	]=	0.09175792	;
    RnonCO2[	116	]=	0.09320276	;
    RnonCO2[	117	]=	0.09464664	;
    RnonCO2[	118	]=	0.09608949	;
    RnonCO2[	119	]=	0.09739496	;
    RnonCO2[	120	]=	0.0985634	;
    RnonCO2[	121	]=	0.09986768	;
    RnonCO2[	122	]=	0.10122271	;
    RnonCO2[	123	]=	0.10249213	;
    RnonCO2[	124	]=	0.10384549	;
    RnonCO2[	125	]=	0.10528284	;
    RnonCO2[	126	]=	0.10685466	;
    RnonCO2[	127	]=	0.1085606	;
    RnonCO2[	128	]=	0.11004559	;
    RnonCO2[	129	]=	0.11131004	;
    RnonCO2[	130	]=	0.11270838	;
    RnonCO2[	131	]=	0.1142403	;
    RnonCO2[	132	]=	0.11572158	;
    RnonCO2[	133	]=	0.11720182	;
    RnonCO2[	134	]=	0.11873056	;
    RnonCO2[	135	]=	0.12048167	;
    RnonCO2[	136	]=	0.12271612	;
    RnonCO2[	137	]=	0.12520844	;
    RnonCO2[	138	]=	0.12782821	;
    RnonCO2[	139	]=	0.13057484	;
    RnonCO2[	140	]=	0.13331526	;
    RnonCO2[	141	]=	0.13605041	;
    RnonCO2[	142	]=	0.13887031	;
    RnonCO2[	143	]=	0.14190497	;
    RnonCO2[	144	]=	0.14506366	;
    RnonCO2[	145	]=	0.14808479	;
    RnonCO2[	146	]=	0.15096905	;
    RnonCO2[	147	]=	0.15406256	;
    RnonCO2[	148	]=	0.15742306	;
    RnonCO2[	149	]=	0.16083056	;
    RnonCO2[	150	]=	0.16429017	;
    RnonCO2[	151	]=	0.16780522	;
    RnonCO2[	152	]=	0.17141349	;
    RnonCO2[	153	]=	0.17524964	;
    RnonCO2[	154	]=	0.17912698	;
    RnonCO2[	155	]=	0.18286803	;
    RnonCO2[	156	]=	0.18655793	;
    RnonCO2[	157	]=	0.19028114	;
    RnonCO2[	158	]=	0.19403945	;
    RnonCO2[	159	]=	0.19779207	;
    RnonCO2[	160	]=	0.2015367	;
    RnonCO2[	161	]=	0.20536218	;
    RnonCO2[	162	]=	0.20918465	;
    RnonCO2[	163	]=	0.21283052	;
    RnonCO2[	164	]=	0.2163449	;
    RnonCO2[	165	]=	0.21972929	;
    RnonCO2[	166	]=	0.22302895	;
    RnonCO2[	167	]=	0.22640626	;
    RnonCO2[	168	]=	0.22985789	;
    RnonCO2[	169	]=	0.23320109	;
    RnonCO2[	170	]=	0.2364361	;
    RnonCO2[	171	]=	0.23966811	;
    RnonCO2[	172	]=	0.2428137	;
    RnonCO2[	173	]=	0.24587625	;
    RnonCO2[	174	]=	0.248943	;
    RnonCO2[	175	]=	0.25213499	;
    RnonCO2[	176	]=	0.25554226	;
    RnonCO2[	177	]=	0.25892918	;
    RnonCO2[	178	]=	0.26220864	;
    RnonCO2[	179	]=	0.26570209	;
    RnonCO2[	180	]=	0.26941573	;
    RnonCO2[	181	]=	0.273294	;
    RnonCO2[	182	]=	0.27751544	;
    RnonCO2[	183	]=	0.28201064	;
    RnonCO2[	184	]=	0.28674226	;
    RnonCO2[	185	]=	0.29175313	;
    RnonCO2[	186	]=	0.29696911	;
    RnonCO2[	187	]=	0.30255457	;
    RnonCO2[	188	]=	0.30858029	;
    RnonCO2[	189	]=	0.31473264	;
    RnonCO2[	190	]=	0.32113183	;
    RnonCO2[	191	]=	0.32799691	;
    RnonCO2[	192	]=	0.33518916	;
    RnonCO2[	193	]=	0.34268013	;
    RnonCO2[	194	]=	0.35054912	;
    RnonCO2[	195	]=	0.3588091	;
    RnonCO2[	196	]=	0.36744837	;
    RnonCO2[	197	]=	0.37654998	;
    RnonCO2[	198	]=	0.38629282	;
    RnonCO2[	199	]=	0.39687085	;
    RnonCO2[	200	]=	0.40827014	;
    RnonCO2[	201	]=	0.4202703	;
    RnonCO2[	202	]=	0.43287302	;
    RnonCO2[	203	]=	0.44635066	;
    RnonCO2[	204	]=	0.46086195	;
    RnonCO2[	205	]=	0.47625741	;
    RnonCO2[	206	]=	0.49233715	;
    RnonCO2[	207	]=	0.50922498	;
    RnonCO2[	208	]=	0.52712569	;
    RnonCO2[	209	]=	0.54610485	;
    RnonCO2[	210	]=	0.56604558	;
    RnonCO2[	211	]=	0.58635079	;
    RnonCO2[	212	]=	0.6067464	;
    RnonCO2[	213	]=	0.6271609	;
    RnonCO2[	214	]=	0.6475189	;
    RnonCO2[	215	]=	0.6673124	;
    RnonCO2[	216	]=	0.6879383	;
    RnonCO2[	217	]=	0.7090557	;
    RnonCO2[	218	]=	0.7289172	;
    RnonCO2[	219	]=	0.7480442	;
    RnonCO2[	220	]=	0.766979	;
    RnonCO2[	221	]=	0.7850643	;
    RnonCO2[	222	]=	0.803085	;
    RnonCO2[	223	]=	0.8236521	;
    RnonCO2[	224	]=	0.8452713	;
    RnonCO2[	225	]=	0.864246	;
    RnonCO2[	226	]=	0.8809743	;
    RnonCO2[	227	]=	0.8949711	;
    RnonCO2[	228	]=	0.9053237	;
    RnonCO2[	229	]=	0.914042	;
    RnonCO2[	230	]=	0.9219104	;
    RnonCO2[	231	]=	0.9290394	;
    RnonCO2[	232	]=	0.9359876	;
    RnonCO2[	233	]=	0.9434778	;
    RnonCO2[	234	]=	0.9502339	;
    RnonCO2[	235	]=	0.9548618	;
    RnonCO2[	236	]=	0.9587726	;
    RnonCO2[	237	]=	0.9638465	;
    RnonCO2[	238	]=	0.9690029	;
    RnonCO2[	239	]=	0.972858	;
    RnonCO2[	240	]=	0.9761909	;
    RnonCO2[	241	]=	0.9798637	;
    RnonCO2[	242	]=	0.9839659	;
    RnonCO2[	243	]=	0.9886163	;
    RnonCO2[	244	]=	0.9935961	;
    RnonCO2[	245	]=	0.9987971	;
    RnonCO2[	246	]=	1.0042671	;
    RnonCO2[	247	]=	1.0091057	;
    RnonCO2[	248	]=	1.0124942	;
    RnonCO2[	249	]=	1.0145622	;
    RnonCO2[	250	]=	1.0154303	;
    RnonCO2[	251	]=	1.0151798	;
    RnonCO2[	252	]=	1.0139019	;
    RnonCO2[	253	]=	1.0116952	;
    RnonCO2[	254	]=	1.0086249	;
    RnonCO2[	255	]=	1.004751	;
    RnonCO2[	256	]=	1.0000955	;
    RnonCO2[	257	]=	0.9949606	;
    RnonCO2[	258	]=	0.9897129	;
    RnonCO2[	259	]=	0.9844862	;
    RnonCO2[	260	]=	0.9793593	;
    RnonCO2[	261	]=	0.9743385	;
    RnonCO2[	262	]=	0.9694131	;
    RnonCO2[	263	]=	0.9645612	;
    RnonCO2[	264	]=	0.9597663	;
    RnonCO2[	265	]=	0.9550179	;
    RnonCO2[	266	]=	0.9503079	;
    RnonCO2[	267	]=	0.9456576	;
    RnonCO2[	268	]=	0.9410975	;
    RnonCO2[	269	]=	0.9366385	;
    RnonCO2[	270	]=	0.9322865	;
    RnonCO2[	271	]=	0.928046	;
    RnonCO2[	272	]=	0.9239233	;
    RnonCO2[	273	]=	0.9199093	;
    RnonCO2[	274	]=	0.9159837	;
    RnonCO2[	275	]=	0.9121317	;
    RnonCO2[	276	]=	0.9083448	;
    RnonCO2[	277	]=	0.9044256	;
    RnonCO2[	278	]=	0.9001644	;
    RnonCO2[	279	]=	0.8955727	;
    RnonCO2[	280	]=	0.8907008	;
    RnonCO2[	281	]=	0.88559	;
    RnonCO2[	282	]=	0.8802749	;
    RnonCO2[	283	]=	0.8747896	;
    RnonCO2[	284	]=	0.8691492	;
    RnonCO2[	285	]=	0.8633559	;
    RnonCO2[	286	]=	0.8574152	;
    RnonCO2[	287	]=	0.8515233	;
    RnonCO2[	288	]=	0.8458588	;
    RnonCO2[	289	]=	0.8403682	;
    RnonCO2[	290	]=	0.8350142	;
    RnonCO2[	291	]=	0.8298023	;
    RnonCO2[	292	]=	0.8247332	;
    RnonCO2[	293	]=	0.819806	;
    RnonCO2[	294	]=	0.8150227	;
    RnonCO2[	295	]=	0.810371	;
    RnonCO2[	296	]=	0.8058283	;
    RnonCO2[	297	]=	0.801534	;
    RnonCO2[	298	]=	0.7976189	;
    RnonCO2[	299	]=	0.7940447	;
    RnonCO2[	300	]=	0.7907496	;
    RnonCO2[	301	]=	0.7876889	;
    RnonCO2[	302	]=	0.784859	;
    RnonCO2[	303	]=	0.7822529	;
    RnonCO2[	304	]=	0.7798632	;
    RnonCO2[	305	]=	0.7776856	;
    RnonCO2[	306	]=	0.7757026	;
    RnonCO2[	307	]=	0.7738647	;
    RnonCO2[	308	]=	0.7721296	;
    RnonCO2[	309	]=	0.7704841	;
    RnonCO2[	310	]=	0.7689169	;
    RnonCO2[	311	]=	0.767391	;
    RnonCO2[	312	]=	0.7658839	;
    RnonCO2[	313	]=	0.7644109	;
    RnonCO2[	314	]=	0.7629824	;
    RnonCO2[	315	]=	0.7616064	;
    RnonCO2[	316	]=	0.7602926	;
    RnonCO2[	317	]=	0.7590056	;
    RnonCO2[	318	]=	0.7577023	;
    RnonCO2[	319	]=	0.7563777	;
    RnonCO2[	320	]=	0.7550304	;
    RnonCO2[	321	]=	0.7536595	;
    RnonCO2[	322	]=	0.752238	;
    RnonCO2[	323	]=	0.7507519	;
    RnonCO2[	324	]=	0.7492239	;
    RnonCO2[	325	]=	0.7476704	;
    RnonCO2[	326	]=	0.7461052	;
    RnonCO2[	327	]=	0.7445368	;
    RnonCO2[	328	]=	0.7429588	;
    RnonCO2[	329	]=	0.7413619	;
    RnonCO2[	330	]=	0.7397406	;
    RnonCO2[	331	]=	0.7380929	;
    RnonCO2[	332	]=	0.736417	;
    RnonCO2[	333	]=	0.7346863	;
    RnonCO2[	334	]=	0.7328865	;
    RnonCO2[	335	]=	0.7310399	;
    RnonCO2[	336	]=	0.729163	;
    RnonCO2[	337	]=	0.727335	;
    RnonCO2[	338	]=	0.7256289	;
    RnonCO2[	339	]=	0.7240345	;
    RnonCO2[	340	]=	0.7225328	;
    RnonCO2[	341	]=	0.7211102	;
    RnonCO2[	342	]=	0.7197575	;
    RnonCO2[	343	]=	0.7184664	;
    RnonCO2[	344	]=	0.7172045	;
    RnonCO2[	345	]=	0.7159525	;
    RnonCO2[	346	]=	0.7147276	;
    RnonCO2[	347	]=	0.713541	;
    RnonCO2[	348	]=	0.712402	;
    RnonCO2[	349	]=	0.7113211	;
    RnonCO2[	350	]=	0.7102955	;
    RnonCO2[	351	]=	0.7093128	;
    RnonCO2[	352	]=	0.7083648	;
    RnonCO2[	353	]=	0.7074472	;
    RnonCO2[	354	]=	0.7065561	;
    RnonCO2[	355	]=	0.7056632	;
    RnonCO2[	356	]=	0.7047529	;
    RnonCO2[	357	]=	0.7038449	;
    RnonCO2[	358	]=	0.702953	;
    RnonCO2[	359	]=	0.7020886	;
    RnonCO2[	360	]=	0.7012643	;
    RnonCO2[	361	]=	0.7004787	;
    RnonCO2[	362	]=	0.6997216	;
    RnonCO2[	363	]=	0.6989861	;
    RnonCO2[	364	]=	0.6982691	;
    RnonCO2[	365	]=	0.6975678	;
    RnonCO2[	366	]=	0.6968552	;
    RnonCO2[	367	]=	0.6961166	;
    RnonCO2[	368	]=	0.6953724	;
    RnonCO2[	369	]=	0.694637	;
    RnonCO2[	370	]=	0.6939221	;
    RnonCO2[	371	]=	0.693241	;
    RnonCO2[	372	]=	0.6925926	;
    RnonCO2[	373	]=	0.6919671	;
    RnonCO2[	374	]=	0.6913582	;
    RnonCO2[	375	]=	0.6907631	;
    RnonCO2[	376	]=	0.6901796	;
    RnonCO2[	377	]=	0.689581	;
    RnonCO2[	378	]=	0.6889531	;
    RnonCO2[	379	]=	0.6883164	;
    RnonCO2[	380	]=	0.6876854	;
    RnonCO2[	381	]=	0.6870721	;
    RnonCO2[	382	]=	0.6864896	;
    RnonCO2[	383	]=	0.6859372	;
    RnonCO2[	384	]=	0.6854053	;
    RnonCO2[	385	]=	0.6848879	;
    RnonCO2[	386	]=	0.684382	;
    RnonCO2[	387	]=	0.6838858	;
    RnonCO2[	388	]=	0.6833726	;
    RnonCO2[	389	]=	0.6828287	;
    RnonCO2[	390	]=	0.6822745	;
    RnonCO2[	391	]=	0.6817244	;
    RnonCO2[	392	]=	0.6811905	;
    RnonCO2[	393	]=	0.6806857	;
    RnonCO2[	394	]=	0.6802097	;
    RnonCO2[	395	]=	0.6797527	;
    RnonCO2[	396	]=	0.6793088	;
    RnonCO2[	397	]=	0.6788753	;
    RnonCO2[	398	]=	0.6784502	;
    RnonCO2[	399	]=	0.6780071	;
    RnonCO2[	400	]=	0.6775325	;
    RnonCO2[	401	]=	0.6770464	;
    RnonCO2[	402	]=	0.6765636	;
    RnonCO2[	403	]=	0.6760958	;
    RnonCO2[	404	]=	0.6756562	;
    RnonCO2[	405	]=	0.6752443	;
    RnonCO2[	406	]=	0.6748504	;
    RnonCO2[	407	]=	0.6744685	;
    RnonCO2[	408	]=	0.6740963	;
    RnonCO2[	409	]=	0.6737315	;
    RnonCO2[	410	]=	0.6733481	;
    RnonCO2[	411	]=	0.6729325	;
    RnonCO2[	412	]=	0.6725047	;
    RnonCO2[	413	]=	0.6720793	;
    RnonCO2[	414	]=	0.6716683	;
    RnonCO2[	415	]=	0.6712846	;
    RnonCO2[	416	]=	0.6709277	;
    RnonCO2[	417	]=	0.6705881	;
    RnonCO2[	418	]=	0.6702598	;
    RnonCO2[	419	]=	0.6699403	;
    RnonCO2[	420	]=	0.6696277	;
    RnonCO2[	421	]=	0.6692957	;
    RnonCO2[	422	]=	0.6689311	;
    RnonCO2[	423	]=	0.6685537	;
    RnonCO2[	424	]=	0.6681781	;
    RnonCO2[	425	]=	0.6678163	;
    RnonCO2[	426	]=	0.667481	;
    RnonCO2[	427	]=	0.6671719	;
    RnonCO2[	428	]=	0.6668794	;
    RnonCO2[	429	]=	0.6665976	;
    RnonCO2[	430	]=	0.666324	;
    RnonCO2[	431	]=	0.6660566	;
    RnonCO2[	432	]=	0.6657696	;
    RnonCO2[	433	]=	0.6654492	;
    RnonCO2[	434	]=	0.6651158	;
    RnonCO2[	435	]=	0.6647835	;
    RnonCO2[	436	]=	0.6644645	;
    RnonCO2[	437	]=	0.6641714	;
    RnonCO2[	438	]=	0.6639039	;
    RnonCO2[	439	]=	0.6636525	;
    RnonCO2[	440	]=	0.6634112	;
    RnonCO2[	441	]=	0.6631777	;
    RnonCO2[	442	]=	0.6629498	;
    RnonCO2[	443	]=	0.6627018	;
    RnonCO2[	444	]=	0.6624202	;
    RnonCO2[	445	]=	0.6621251	;
    RnonCO2[	446	]=	0.6618307	;
    RnonCO2[	447	]=	0.6615491	;
    RnonCO2[	448	]=	0.661293	;
    RnonCO2[	449	]=	0.6610618	;
    RnonCO2[	450	]=	0.6608463	;
    RnonCO2[	451	]=	0.6606404	;
    RnonCO2[	452	]=	0.6604419	;
    RnonCO2[	453	]=	0.6602486	;
    RnonCO2[	454	]=	0.6600348	;
    RnonCO2[	455	]=	0.6597871	;
    RnonCO2[	456	]=	0.6595255	;
    RnonCO2[	457	]=	0.6592643	;
    RnonCO2[	458	]=	0.6590154	;
    RnonCO2[	459	]=	0.6587916	;
    RnonCO2[	460	]=	0.6585923	;
    RnonCO2[	461	]=	0.6584081	;
    RnonCO2[	462	]=	0.6582334	;
    RnonCO2[	463	]=	0.6580655	;
    RnonCO2[	464	]=	0.6579025	;
    RnonCO2[	465	]=	0.6577187	;
    RnonCO2[	466	]=	0.6575007	;
    RnonCO2[	467	]=	0.6572685	;
    RnonCO2[	468	]=	0.6570364	;
    RnonCO2[	469	]=	0.6568161	;
    RnonCO2[	470	]=	0.6566206	;
    RnonCO2[	471	]=	0.6564492	;
    RnonCO2[	472	]=	0.6562926	;
    RnonCO2[	473	]=	0.6561451	;
    RnonCO2[	474	]=	0.6560041	;
    RnonCO2[	475	]=	0.6558677	;
    RnonCO2[	476	]=	0.6557102	;
    RnonCO2[	477	]=	0.6555183	;
    RnonCO2[	478	]=	0.6553118	;
    RnonCO2[	479	]=	0.6551053	;
    RnonCO2[	480	]=	0.6549103	;
    RnonCO2[	481	]=	0.6547395	;
    RnonCO2[	482	]=	0.6545928	;
    RnonCO2[	483	]=	0.6544603	;
    RnonCO2[	484	]=	0.6543368	;
    RnonCO2[	485	]=	0.6542194	;
    RnonCO2[	486	]=	0.6541063	;
    RnonCO2[	487	]=	0.653972	;
    RnonCO2[	488	]=	0.6538028	;
    RnonCO2[	489	]=	0.6536192	;
    RnonCO2[	490	]=	0.6534349	;
    RnonCO2[	491	]=	0.6532621	;
    RnonCO2[	492	]=	0.6531132	;
    RnonCO2[	493	]=	0.652988	;
    RnonCO2[	494	]=	0.6528769	;
    RnonCO2[	495	]=	0.6527743	;
    RnonCO2[	496	]=	0.6526777	;
    RnonCO2[	497	]=	0.6525852	;
    RnonCO2[	498	]=	0.6524711	;
    RnonCO2[	499	]=	0.6523222	;
    RnonCO2[	500	]=	0.6521584	;
    RnonCO2[	501	]=	0.651994	;
    RnonCO2[	502	]=	0.6518406	;
    RnonCO2[	503	]=	0.651711	;
    RnonCO2[	504	]=	0.6516046	;
    RnonCO2[	505	]=	0.6515124	;
    RnonCO2[	506	]=	0.6514283	;
    RnonCO2[	507	]=	0.65135	;
    RnonCO2[	508	]=	0.6512756	;
    RnonCO2[	509	]=	0.6511794	;
    RnonCO2[	510	]=	0.6510483	;
    RnonCO2[	511	]=	0.6509021	;
    RnonCO2[	512	]=	0.650755	;
    RnonCO2[	513	]=	0.6506187	;
    RnonCO2[	514	]=	0.6505061	;
    RnonCO2[	515	]=	0.6504165	;
    RnonCO2[	516	]=	0.6503408	;
    RnonCO2[	517	]=	0.650273	;
    RnonCO2[	518	]=	0.6502108	;
    RnonCO2[	519	]=	0.6501524	;
    RnonCO2[	520	]=	0.6500719	;
    RnonCO2[	521	]=	0.6499565	;
    RnonCO2[	522	]=	0.6498258	;
    RnonCO2[	523	]=	0.6496941	;
    RnonCO2[	524	]=	0.649573	;
    RnonCO2[	525	]=	0.6494753	;
    RnonCO2[	526	]=	0.6494005	;
    RnonCO2[	527	]=	0.6493393	;
    RnonCO2[	528	]=	0.6492859	;
    RnonCO2[	529	]=	0.649238	;
    RnonCO2[	530	]=	0.6491935	;
    RnonCO2[	531	]=	0.6491272	;
    RnonCO2[	532	]=	0.6490255	;
    RnonCO2[	533	]=	0.6489084	;
    RnonCO2[	534	]=	0.6487903	;
    RnonCO2[	535	]=	0.6486826	;
    RnonCO2[	536	]=	0.6485981	;
    RnonCO2[	537	]=	0.6485363	;
    RnonCO2[	538	]=	0.6484879	;
    RnonCO2[	539	]=	0.6484472	;
    RnonCO2[	540	]=	0.648412	;
    RnonCO2[	541	]=	0.64838	;
    RnonCO2[	542	]=	0.6483259	;
    RnonCO2[	543	]=	0.6482364	;
    RnonCO2[	544	]=	0.6481315	;
    RnonCO2[	545	]=	0.6480252	;
    RnonCO2[	546	]=	0.6479294	;
    RnonCO2[	547	]=	0.6478566	;
    RnonCO2[	548	]=	0.6478064	;
    RnonCO2[	549	]=	0.6477694	;
    RnonCO2[	550	]=	0.6477399	;
    RnonCO2[	551	]=	0.6477158	;
    RnonCO2[	552	]=	0.6476947	;
    RnonCO2[	553	]=	0.6476515	;
    RnonCO2[	554	]=	0.6475729	;
    RnonCO2[	555	]=	0.6474786	;
    RnonCO2[	556	]=	0.6473831	;
    RnonCO2[	557	]=	0.6472977	;
    RnonCO2[	558	]=	0.6472351	;
    RnonCO2[	559	]=	0.6471951	;
    RnonCO2[	560	]=	0.6471682	;
    RnonCO2[	561	]=	0.6471487	;
    RnonCO2[	562	]=	0.6471344	;
    RnonCO2[	563	]=	0.6471231	;
    RnonCO2[	564	]=	0.6470895	;
    RnonCO2[	565	]=	0.6470204	;
    RnonCO2[	566	]=	0.6469357	;
    RnonCO2[	567	]=	0.6468494	;
    RnonCO2[	568	]=	0.6467734	;
    RnonCO2[	569	]=	0.64672	;
    RnonCO2[	570	]=	0.6466889	;
    RnonCO2[	571	]=	0.646671	;
    RnonCO2[	572	]=	0.6466604	;
    RnonCO2[	573	]=	0.6466547	;
    RnonCO2[	574	]=	0.6466521	;
    RnonCO2[	575	]=	0.6466271	;
    RnonCO2[	576	]=	0.6465664	;
    RnonCO2[	577	]=	0.6464901	;
    RnonCO2[	578	]=	0.6464122	;
    RnonCO2[	579	]=	0.6463442	;
    RnonCO2[	580	]=	0.646299	;
    RnonCO2[	581	]=	0.646276	;
    RnonCO2[	582	]=	0.646266	;
    RnonCO2[	583	]=	0.6462631	;
    RnonCO2[	584	]=	0.6462652	;
    RnonCO2[	585	]=	0.6462702	;
    RnonCO2[	586	]=	0.6462528	;
    RnonCO2[	587	]=	0.6461997	;
    RnonCO2[	588	]=	0.6461308	;
    RnonCO2[	589	]=	0.6460603	;
    RnonCO2[	590	]=	0.6459996	;
    RnonCO2[	591	]=	0.6459616	;
    RnonCO2[	592	]=	0.6459456	;
    RnonCO2[	593	]=	0.6459425	;
    RnonCO2[	594	]=	0.6459467	;
    RnonCO2[	595	]=	0.6459556	;
    RnonCO2[	596	]=	0.6459674	;
    RnonCO2[	597	]=	0.6459567	;
    RnonCO2[	598	]=	0.6459103	;
    RnonCO2[	599	]=	0.645848	;
    RnonCO2[	600	]=	0.645784	;
    RnonCO2[	601	]=	0.6457299	;
    RnonCO2[	602	]=	0.6456982	;
    RnonCO2[	603	]=	0.6456885	;
    RnonCO2[	604	]=	0.6456917	;
    RnonCO2[	605	]=	0.645702	;
    RnonCO2[	606	]=	0.645717	;
    RnonCO2[	607	]=	0.6457349	;
    RnonCO2[	608	]=	0.6457301	;
    RnonCO2[	609	]=	0.6456896	;
    RnonCO2[	610	]=	0.6456333	;
    RnonCO2[	611	]=	0.6455751	;
    RnonCO2[	612	]=	0.6455267	;
    RnonCO2[	613	]=	0.6455006	;
    RnonCO2[	614	]=	0.6454966	;
    RnonCO2[	615	]=	0.6455053	;
    RnonCO2[	616	]=	0.645521	;
    RnonCO2[	617	]=	0.6455415	;
    RnonCO2[	618	]=	0.6455647	;
    RnonCO2[	619	]=	0.6455653	;
    RnonCO2[	620	]=	0.64553	;
    RnonCO2[	621	]=	0.645479	;
    RnonCO2[	622	]=	0.6454259	;
    RnonCO2[	623	]=	0.6453826	;
    RnonCO2[	624	]=	0.6453617	;
    RnonCO2[	625	]=	0.6453627	;
    RnonCO2[	626	]=	0.6453763	;
    RnonCO2[	627	]=	0.6453968	;
    RnonCO2[	628	]=	0.6454222	;
    RnonCO2[	629	]=	0.6454501	;
    RnonCO2[	630	]=	0.6454554	;
    RnonCO2[	631	]=	0.6454249	;
    RnonCO2[	632	]=	0.6453784	;
    RnonCO2[	633	]=	0.64533	;
    RnonCO2[	634	]=	0.6452912	;
    RnonCO2[	635	]=	0.6452748	;
    RnonCO2[	636	]=	0.6452802	;
    RnonCO2[	637	]=	0.6452981	;
    RnonCO2[	638	]=	0.645323	;
    RnonCO2[	639	]=	0.6453527	;
    RnonCO2[	640	]=	0.6453848	;
    RnonCO2[	641	]=	0.6453944	;
    RnonCO2[	642	]=	0.645368	;
    RnonCO2[	643	]=	0.6453257	;
    RnonCO2[	644	]=	0.6452813	;
    RnonCO2[	645	]=	0.6452467	;
    RnonCO2[	646	]=	0.6452342	;
    RnonCO2[	647	]=	0.6452435	;
    RnonCO2[	648	]=	0.6452654	;
    RnonCO2[	649	]=	0.6452942	;
    RnonCO2[	650	]=	0.6453276	;
    RnonCO2[	651	]=	0.6453636	;
    RnonCO2[	652	]=	0.6453768	;
    RnonCO2[	653	]=	0.6453542	;
    RnonCO2[	654	]=	0.6453156	;
    RnonCO2[	655	]=	0.645275	;
    RnonCO2[	656	]=	0.6452438	;
    RnonCO2[	657	]=	0.6452349	;
    RnonCO2[	658	]=	0.6452478	;
    RnonCO2[	659	]=	0.6452731	;
    RnonCO2[	660	]=	0.6453054	;
    RnonCO2[	661	]=	0.6453421	;
    RnonCO2[	662	]=	0.6453815	;
    RnonCO2[	663	]=	0.6453981	;
    RnonCO2[	664	]=	0.6453788	;
    RnonCO2[	665	]=	0.6453435	;
    RnonCO2[	666	]=	0.645306	;
    RnonCO2[	667	]=	0.6452781	;
    RnonCO2[	668	]=	0.6452724	;
    RnonCO2[	669	]=	0.6452884	;
    RnonCO2[	670	]=	0.6453168	;
    RnonCO2[	671	]=	0.6453521	;
    RnonCO2[	672	]=	0.645392	;
    RnonCO2[	673	]=	0.6454343	;
    RnonCO2[	674	]=	0.6454539	;
    RnonCO2[	675	]=	0.6454375	;
    RnonCO2[	676	]=	0.6454051	;
    RnonCO2[	677	]=	0.6453706	;
    RnonCO2[	678	]=	0.6453456	;
    RnonCO2[	679	]=	0.6453427	;
    RnonCO2[	680	]=	0.6453615	;
    RnonCO2[	681	]=	0.6453926	;
    RnonCO2[	682	]=	0.6454306	;
    RnonCO2[	683	]=	0.6454732	;
    RnonCO2[	684	]=	0.6455183	;
    RnonCO2[	685	]=	0.6455405	;
    RnonCO2[	686	]=	0.6455268	;
    RnonCO2[	687	]=	0.645497	;
    RnonCO2[	688	]=	0.6454651	;
    RnonCO2[	689	]=	0.64544266	;
    RnonCO2[	690	]=	0.64544228	;
    RnonCO2[	691	]=	0.64546356	;
    RnonCO2[	692	]=	0.64549714	;
    RnonCO2[	693	]=	0.64553763	;
    RnonCO2[	694	]=	0.64558254	;
    RnonCO2[	695	]=	0.64562997	;
    RnonCO2[	696	]=	0.64565467	;
    RnonCO2[	697	]=	0.6456433	;
    RnonCO2[	698	]=	0.64561584	;
    RnonCO2[	699	]=	0.64558629	;
    RnonCO2[	700	]=	0.64556609	;
    RnonCO2[	701	]=	0.645568	;
    RnonCO2[	702	]=	0.6455914	;
    RnonCO2[	703	]=	0.64562733	;
    RnonCO2[	704	]=	0.64566993	;
    RnonCO2[	705	]=	0.64571707	;
    RnonCO2[	706	]=	0.64576657	;
    RnonCO2[	707	]=	0.64579335	;
    RnonCO2[	708	]=	0.64578408	;
    RnonCO2[	709	]=	0.64575878	;
    RnonCO2[	710	]=	0.64573126	;
    RnonCO2[	711	]=	0.64571315	;
    RnonCO2[	712	]=	0.64571703	;
    RnonCO2[	713	]=	0.64574252	;
    RnonCO2[	714	]=	0.64578025	;
    RnonCO2[	715	]=	0.6458249	;
    RnonCO2[	716	]=	0.64587391	;
    RnonCO2[	717	]=	0.64592541	;
    RnonCO2[	718	]=	0.64595398	;
    RnonCO2[	719	]=	0.64594665	;
    RnonCO2[	720	]=	0.64592321	;
    RnonCO2[	721	]=	0.64589755	;
    RnonCO2[	722	]=	0.64588124	;
    RnonCO2[	723	]=	0.6458869	;
    RnonCO2[	724	]=	0.64591419	;
    RnonCO2[	725	]=	0.64595372	;
    RnonCO2[	726	]=	0.646	;
    RnonCO2[	727	]=	0.64605074	;
    RnonCO2[	728	]=	0.64610391	;
    RnonCO2[	729	]=	0.64613424	;
    RnonCO2[	730	]=	0.64612859	;
    RnonCO2[	731	]=	0.64610687	;
    RnonCO2[	732	]=	0.64608283	;
    RnonCO2[	733	]=	0.64606819	;
    RnonCO2[	734	]=	0.64607547	;
    RnonCO2[	735	]=	0.6460859	;
    
    for(int i = 736; i<tmax; i++)
    {
        
        RnonCO2[i] = RnonCO2[i-1];
        
    }
    
}


void setRnonCO2_zero(void)
{
    for(int i = 0; i<tmax; i++)
    {
        RnonCO2[i] = 0.0;
    }
}

//These are the Local to Global Ratio of Temperature Change (LGRTC) from Leduc et al, 2016 and from subsequent work for RCPs 2.6 (peak warming), 4.5 and 8.5 from the CMIP5 ensemble
double LGRTC_CO2[64][128] = {
    {    1.13235    ,    1.127828    ,    1.124168    ,    1.118937    ,    1.115888    ,    1.114195    ,    1.106491    ,    1.102244    ,    1.096694    ,    1.093992    ,    1.091868    ,    1.084419    ,    1.082446    ,    1.086774    ,    1.088757    ,    1.088975    ,    1.092242    ,    1.095997    ,    1.104449    ,    1.110543    ,    1.11606    ,    1.12313    ,    1.131713    ,    1.137215    ,    1.145362    ,    1.157466    ,    1.163238    ,    1.16725    ,    1.168526    ,    1.16791    ,    1.162586    ,    1.160189    ,    1.165801    ,    1.159892    ,    1.154024    ,    1.153295    ,    1.149689    ,    1.151192    ,    1.151361    ,    1.149562    ,    1.152473    ,    1.152176    ,    1.150893    ,    1.151609    ,    1.159264    ,    1.166224    ,    1.1735    ,    1.181385    ,    1.187529    ,    1.188959    ,    1.188946    ,    1.190093    ,    1.19324    ,    1.19365    ,    1.198705    ,    1.200222    ,    1.207871    ,    1.211357    ,    1.212117    ,    1.216293    ,    1.21953    ,    1.221233    ,    1.219083    ,    1.228929    ,    1.231389    ,    1.234443    ,    1.234736    ,    1.234468    ,    1.23564    ,    1.239622    ,    1.238075    ,    1.238103    ,    1.246355    ,    1.249745    ,    1.245284    ,    1.240901    ,    1.246108    ,    1.250639    ,    1.255662    ,    1.269594    ,    1.265832    ,    1.264232    ,    1.270232    ,    1.27048    ,    1.275645    ,    1.275718    ,    1.28304    ,    1.289628    ,    1.288649    ,    1.265916    ,    1.282132    ,    1.289289    ,    1.28845    ,    1.273018    ,    1.274879    ,    1.273939    ,    1.277156    ,    1.263502    ,    1.271355    ,    1.270741    ,    1.26137    ,    1.26088    ,    1.253134    ,    1.253341    ,    1.241976    ,    1.251273    ,    1.246784    ,    1.233077    ,    1.226569    ,    1.217757    ,    1.209813    ,    1.198953    ,    1.198163    ,    1.185609    ,    1.179484    ,    1.175523    ,    1.176119    ,    1.173318    ,    1.163545    ,    1.160026    ,    1.155296    ,    1.161159    ,    1.145316    ,    1.139239    ,    1.138002    ,    1.129376    ,    1.128965    ,    1.133326    },
    {    0.942619    ,    0.946712    ,    0.945145    ,    0.940673    ,    0.932225    ,    0.937265    ,    0.943783    ,    0.953031    ,    0.956795    ,    0.961117    ,    0.967392    ,    0.97262    ,    0.965327    ,    0.967938    ,    0.970104    ,    0.970916    ,    0.965475    ,    0.957139    ,    0.954204    ,    0.955479    ,    0.960411    ,    0.962546    ,    0.974431    ,    0.986059    ,    0.999145    ,    1.015671    ,    1.037686    ,    1.052922    ,    1.077097    ,    1.092867    ,    1.107893    ,    1.118256    ,    1.124255    ,    1.111455    ,    1.107992    ,    1.09993    ,    1.097876    ,    1.10122    ,    1.102725    ,    1.100804    ,    1.102085    ,    1.100994    ,    1.098413    ,    1.091127    ,    1.09121    ,    1.087252    ,    1.08748    ,    1.095465    ,    1.103088    ,    1.10548    ,    1.108248    ,    1.116284    ,    1.127188    ,    1.133677    ,    1.142406    ,    1.149121    ,    1.153229    ,    1.159528    ,    1.16636    ,    1.176251    ,    1.187771    ,    1.195406    ,    1.200695    ,    1.213358    ,    1.220931    ,    1.232874    ,    1.237614    ,    1.23167    ,    1.241032    ,    1.240132    ,    1.237564    ,    1.245007    ,    1.243843    ,    1.239092    ,    1.239312    ,    1.23598    ,    1.230361    ,    1.239717    ,    1.247225    ,    1.236815    ,    1.254377    ,    1.254374    ,    1.253958    ,    1.240216    ,    1.269772    ,    1.257358    ,    1.249364    ,    1.259846    ,    1.244331    ,    1.235997    ,    1.247028    ,    1.234673    ,    1.239301    ,    1.248761    ,    1.253064    ,    1.260953    ,    1.259491    ,    1.246115    ,    1.264319    ,    1.264379    ,    1.254498    ,    1.251079    ,    1.238063    ,    1.217289    ,    1.220088    ,    1.215584    ,    1.210081    ,    1.196508    ,    1.182629    ,    1.170317    ,    1.157356    ,    1.135872    ,    1.119262    ,    1.106451    ,    1.09727    ,    1.090143    ,    1.081604    ,    1.067488    ,    1.055205    ,    1.046229    ,    1.034859    ,    1.024407    ,    1.017369    ,    1.004537    ,    0.985035    ,    0.972618    ,    0.965159    ,    0.9547    },
    {    1.061763    ,    1.074548    ,    1.087022    ,    1.097067    ,    1.097025    ,    1.095313    ,    1.083755    ,    1.075888    ,    1.067507    ,    1.051342    ,    1.036852    ,    1.033921    ,    1.026764    ,    1.011793    ,    0.990481    ,    0.968898    ,    0.955465    ,    0.940991    ,    0.935555    ,    0.926852    ,    0.918644    ,    0.909521    ,    0.901323    ,    0.898123    ,    0.897871    ,    0.903608    ,    0.912157    ,    0.92151    ,    0.936078    ,    0.945385    ,    0.959837    ,    0.974036    ,    0.98911    ,    1.007181    ,    1.02382    ,    1.036759    ,    1.041433    ,    1.050577    ,    1.065341    ,    1.08391    ,    1.110044    ,    1.127647    ,    1.144851    ,    1.142154    ,    1.135922    ,    1.130398    ,    1.123339    ,    1.121338    ,    1.106233    ,    1.090379    ,    1.085317    ,    1.088962    ,    1.089986    ,    1.10081    ,    1.110807    ,    1.120556    ,    1.128582    ,    1.122564    ,    1.132084    ,    1.134002    ,    1.147905    ,    1.160885    ,    1.179912    ,    1.189119    ,    1.191857    ,    1.209137    ,    1.221203    ,    1.217474    ,    1.237076    ,    1.233213    ,    1.240889    ,    1.234238    ,    1.245664    ,    1.241821    ,    1.241257    ,    1.256644    ,    1.265361    ,    1.261733    ,    1.275151    ,    1.304948    ,    1.310232    ,    1.300268    ,    1.309345    ,    1.296345    ,    1.28975    ,    1.2902    ,    1.284684    ,    1.277697    ,    1.287898    ,    1.290008    ,    1.285883    ,    1.257006    ,    1.271901    ,    1.270032    ,    1.264697    ,    1.254124    ,    1.25541    ,    1.236078    ,    1.238479    ,    1.217891    ,    1.233277    ,    1.230045    ,    1.222972    ,    1.213851    ,    1.217646    ,    1.212388    ,    1.199178    ,    1.187552    ,    1.18291    ,    1.171269    ,    1.152842    ,    1.141426    ,    1.123865    ,    1.112045    ,    1.103716    ,    1.095149    ,    1.079499    ,    1.063612    ,    1.048363    ,    1.035928    ,    1.024727    ,    1.014702    ,    1.007057    ,    1.010255    ,    1.027335    ,    1.038127    ,    1.046835    ,    1.052717    },
    {    1.146024    ,    1.128625    ,    1.124941    ,    1.120316    ,    1.112673    ,    1.103232    ,    1.087243    ,    1.052807    ,    1.022437    ,    1.019085    ,    0.989409    ,    0.953906    ,    0.936505    ,    0.917156    ,    0.895289    ,    0.872911    ,    0.857692    ,    0.850449    ,    0.844662    ,    0.846348    ,    0.856806    ,    0.872581    ,    0.882609    ,    0.891881    ,    0.903731    ,    0.903047    ,    0.90718    ,    0.909511    ,    0.914081    ,    0.920136    ,    0.922519    ,    0.934219    ,    0.946993    ,    0.955141    ,    0.979434    ,    1.021363    ,    1.051608    ,    1.087696    ,    1.123668    ,    1.16982    ,    1.191162    ,    1.224937    ,    1.241235    ,    1.250134    ,    1.257034    ,    1.257649    ,    1.253877    ,    1.249133    ,    1.255151    ,    1.244888    ,    1.22698    ,    1.189565    ,    1.142139    ,    1.115213    ,    1.091912    ,    1.082545    ,    1.075001    ,    1.06636    ,    1.07038    ,    1.074129    ,    1.085378    ,    1.086571    ,    1.091042    ,    1.096658    ,    1.10235    ,    1.107242    ,    1.118884    ,    1.13105    ,    1.139756    ,    1.15053    ,    1.162863    ,    1.200235    ,    1.216996    ,    1.234477    ,    1.246383    ,    1.256764    ,    1.260608    ,    1.279271    ,    1.312177    ,    1.332008    ,    1.347068    ,    1.343506    ,    1.314561    ,    1.277238    ,    1.234407    ,    1.20588    ,    1.175052    ,    1.14258    ,    1.10255    ,    1.09974    ,    1.100929    ,    1.113824    ,    1.138857    ,    1.168094    ,    1.176292    ,    1.211005    ,    1.24573    ,    1.223268    ,    1.21002    ,    1.195358    ,    1.18446    ,    1.185143    ,    1.177773    ,    1.172694    ,    1.161513    ,    1.155227    ,    1.147936    ,    1.158167    ,    1.166497    ,    1.168457    ,    1.171343    ,    1.160175    ,    1.151431    ,    1.152175    ,    1.143208    ,    1.135369    ,    1.121004    ,    1.090286    ,    1.064972    ,    1.036064    ,    1.008068    ,    0.996409    ,    1.017604    ,    1.051872    ,    1.087396    ,    1.120681    ,    1.137826    ,    1.141174    },
    {    1.105742    ,    1.14359    ,    1.180795    ,    1.203993    ,    1.216475    ,    1.221961    ,    1.222251    ,    1.203502    ,    1.150449    ,    1.140137    ,    1.135213    ,    1.068939    ,    1.03758    ,    0.943758    ,    0.908736    ,    0.889267    ,    0.878086    ,    0.87332    ,    0.869451    ,    0.870635    ,    0.865854    ,    0.868437    ,    0.87786    ,    0.889138    ,    0.896512    ,    0.901574    ,    0.906223    ,    0.911465    ,    0.913566    ,    0.916468    ,    0.926041    ,    0.939141    ,    0.966873    ,    0.985129    ,    1.012983    ,    1.051087    ,    1.097234    ,    1.146029    ,    1.184196    ,    1.206893    ,    1.222426    ,    1.240082    ,    1.261871    ,    1.253993    ,    1.260522    ,    1.263733    ,    1.256352    ,    1.257668    ,    1.271502    ,    1.290598    ,    1.295312    ,    1.273809    ,    1.257497    ,    1.229887    ,    1.185005    ,    1.149071    ,    1.112203    ,    1.06552    ,    1.013222    ,    1.017467    ,    1.028874    ,    1.043204    ,    1.062262    ,    1.070838    ,    1.087656    ,    1.109282    ,    1.147381    ,    1.158494    ,    1.173753    ,    1.207412    ,    1.229295    ,    1.262421    ,    1.276616    ,    1.282399    ,    1.273487    ,    1.265155    ,    1.292482    ,    1.299351    ,    1.294762    ,    1.294391    ,    1.245489    ,    1.203369    ,    1.181484    ,    1.147117    ,    1.134489    ,    1.117222    ,    1.094566    ,    1.064598    ,    1.027749    ,    0.992973    ,    0.975699    ,    0.962027    ,    0.948104    ,    0.945973    ,    0.960303    ,    0.968548    ,    1.003875    ,    1.087968    ,    1.116047    ,    1.107198    ,    1.092909    ,    1.095644    ,    1.129099    ,    1.147079    ,    1.168585    ,    1.168865    ,    1.185466    ,    1.195549    ,    1.179132    ,    1.161384    ,    1.146985    ,    1.157483    ,    1.174844    ,    1.168342    ,    1.173268    ,    1.167665    ,    1.143093    ,    1.114278    ,    1.083313    ,    1.049482    ,    1.004645    ,    0.983709    ,    0.892173    ,    0.93328    ,    0.955094    ,    0.954912    ,    1.001445    ,    1.049986    },
    {    1.094838    ,    1.109341    ,    1.109508    ,    1.094233    ,    1.079657    ,    1.074149    ,    1.071772    ,    1.075207    ,    1.086121    ,    1.098731    ,    1.111152    ,    1.120261    ,    1.129134    ,    1.129903    ,    1.117832    ,    1.089347    ,    1.078424    ,    1.051901    ,    1.022708    ,    1.002936    ,    1.041818    ,    1.064555    ,    1.070153    ,    1.069876    ,    1.051103    ,    1.039947    ,    1.048563    ,    1.048472    ,    1.086429    ,    1.079176    ,    1.066367    ,    1.052856    ,    1.046581    ,    1.05428    ,    1.061533    ,    1.07647    ,    1.121921    ,    1.130781    ,    1.14015    ,    1.144699    ,    1.161715    ,    1.16902    ,    1.19094    ,    1.243841    ,    1.288322    ,    1.287769    ,    1.281933    ,    1.272194    ,    1.276203    ,    1.284351    ,    1.293384    ,    1.306649    ,    1.319846    ,    1.323169    ,    1.330513    ,    1.332131    ,    1.291749    ,    1.224513    ,    1.140911    ,    1.039037    ,    0.981306    ,    0.973731    ,    0.994777    ,    1.014954    ,    1.023868    ,    1.048501    ,    1.070571    ,    1.091692    ,    1.102341    ,    1.112219    ,    1.117033    ,    1.10754    ,    1.106537    ,    1.112717    ,    1.115261    ,    1.106114    ,    1.112644    ,    1.129008    ,    1.157534    ,    1.182649    ,    1.203845    ,    1.211534    ,    1.204817    ,    1.164001    ,    1.12278    ,    1.093841    ,    1.068998    ,    1.057345    ,    1.074631    ,    1.046717    ,    0.956151    ,    0.907359    ,    0.890226    ,    0.89444    ,    0.91396    ,    0.950372    ,    0.997158    ,    1.027216    ,    1.059491    ,    1.07653    ,    1.077045    ,    1.098819    ,    1.1289    ,    1.168202    ,    1.181639    ,    1.188423    ,    1.166421    ,    1.148324    ,    1.138542    ,    1.118331    ,    1.086431    ,    1.083516    ,    1.087564    ,    1.107639    ,    1.125978    ,    1.126194    ,    1.114645    ,    1.091288    ,    1.064687    ,    1.038978    ,    1.009109    ,    0.979334    ,    0.94758    ,    0.969921    ,    1.01323    ,    1.034461    ,    1.060088    ,    1.085051    },
    {    1.010621    ,    0.99045    ,    0.971649    ,    0.94857    ,    0.923685    ,    0.908219    ,    0.901257    ,    0.905365    ,    0.917944    ,    0.935473    ,    0.953175    ,    0.968286    ,    0.978633    ,    0.988226    ,    1.000737    ,    1.011573    ,    1.018172    ,    1.028079    ,    1.044716    ,    1.066079    ,    1.084435    ,    1.102045    ,    1.117877    ,    1.12987    ,    1.139765    ,    1.150463    ,    1.167478    ,    1.191588    ,    1.228602    ,    1.268426    ,    1.292932    ,    1.294199    ,    1.284543    ,    1.273165    ,    1.262709    ,    1.256677    ,    1.253726    ,    1.273229    ,    1.302135    ,    1.26041    ,    1.227007    ,    1.23269    ,    1.261312    ,    1.327841    ,    1.374682    ,    1.369603    ,    1.344779    ,    1.31947    ,    1.30082    ,    1.282385    ,    1.270936    ,    1.266839    ,    1.268881    ,    1.273913    ,    1.280539    ,    1.287246    ,    1.296298    ,    1.309606    ,    1.308993    ,    1.285061    ,    1.242574    ,    1.185239    ,    1.137967    ,    1.089864    ,    1.012566    ,    0.948521    ,    0.947377    ,    0.937986    ,    0.937278    ,    0.936866    ,    0.947569    ,    0.953967    ,    0.954422    ,    0.949932    ,    0.95311    ,    0.951802    ,    0.952264    ,    0.962546    ,    0.977396    ,    1.001273    ,    1.026698    ,    1.050683    ,    1.089938    ,    1.122851    ,    1.0973    ,    1.035449    ,    0.996918    ,    0.992832    ,    1.001713    ,    1.038776    ,    0.996501    ,    0.902038    ,    0.858163    ,    0.844266    ,    0.849863    ,    0.873237    ,    0.889961    ,    0.914761    ,    0.943667    ,    0.95491    ,    0.972652    ,    1.01071    ,    1.038423    ,    1.048861    ,    1.044173    ,    1.026543    ,    1.016518    ,    1.012704    ,    1.017492    ,    1.026829    ,    1.029281    ,    1.024715    ,    1.028382    ,    1.030325    ,    1.027075    ,    1.025732    ,    1.010936    ,    0.980011    ,    0.959702    ,    0.943161    ,    0.943449    ,    0.958992    ,    0.978822    ,    0.990727    ,    1.008261    ,    1.097791    ,    1.076295    ,    1.048861    },
    {    0.917947    ,    0.886281    ,    0.861027    ,    0.842017    ,    0.823903    ,    0.805418    ,    0.791112    ,    0.786609    ,    0.794955    ,    0.808599    ,    0.818345    ,    0.828691    ,    0.840753    ,    0.857372    ,    0.880535    ,    0.902403    ,    0.917191    ,    0.920063    ,    0.921236    ,    0.92468    ,    0.932504    ,    0.946404    ,    0.968545    ,    0.996438    ,    1.026277    ,    1.056792    ,    1.088313    ,    1.119971    ,    1.150628    ,    1.176683    ,    1.196416    ,    1.206462    ,    1.212308    ,    1.217471    ,    1.220482    ,    1.229085    ,    1.248251    ,    1.276543    ,    1.296872    ,    1.311002    ,    1.370178    ,    1.411944    ,    1.474449    ,    1.490726    ,    1.467809    ,    1.419307    ,    1.36125    ,    1.308383    ,    1.272353    ,    1.241509    ,    1.215487    ,    1.196059    ,    1.185424    ,    1.183946    ,    1.184825    ,    1.188372    ,    1.193146    ,    1.200525    ,    1.208153    ,    1.220293    ,    1.233638    ,    1.249243    ,    1.256632    ,    1.256633    ,    1.266608    ,    1.278086    ,    1.279203    ,    1.278003    ,    1.269978    ,    1.269722    ,    1.274353    ,    1.274249    ,    1.26078    ,    1.239875    ,    1.220158    ,    1.205011    ,    1.182463    ,    1.181416    ,    1.15912    ,    1.034803    ,    0.988591    ,    0.961463    ,    0.938433    ,    0.934515    ,    0.92582    ,    0.930829    ,    0.929818    ,    0.969764    ,    1.012562    ,    1.116929    ,    1.121301    ,    1.059286    ,    0.946951    ,    0.856302    ,    0.829524    ,    0.803375    ,    0.790654    ,    0.797133    ,    0.807519    ,    0.816066    ,    0.821406    ,    0.840568    ,    0.86589    ,    0.886528    ,    0.902965    ,    0.903103    ,    0.91078    ,    0.932765    ,    0.900272    ,    0.912646    ,    0.91283    ,    0.909738    ,    0.914517    ,    0.914777    ,    0.914285    ,    0.917049    ,    0.951312    ,    0.994896    ,    1.035325    ,    1.076036    ,    1.093992    ,    1.112853    ,    1.108192    ,    1.08936    ,    1.0695    ,    1.036512    ,    0.996462    ,    0.955814    },
    {    0.755853    ,    0.742877    ,    0.744112    ,    0.751701    ,    0.751311    ,    0.743061    ,    0.732267    ,    0.728925    ,    0.728559    ,    0.726507    ,    0.719693    ,    0.714121    ,    0.707506    ,    0.709688    ,    0.729312    ,    0.75337    ,    0.772301    ,    0.784973    ,    0.793935    ,    0.803519    ,    0.817157    ,    0.834233    ,    0.85296    ,    0.875651    ,    0.901294    ,    0.928654    ,    0.958291    ,    0.986728    ,    1.011366    ,    1.031138    ,    1.047587    ,    1.062818    ,    1.075554    ,    1.090535    ,    1.1065    ,    1.124364    ,    1.140789    ,    1.161509    ,    1.18593    ,    1.210161    ,    1.235987    ,    1.342536    ,    1.44129    ,    1.488647    ,    1.469902    ,    1.391694    ,    1.309866    ,    1.244139    ,    1.199201    ,    1.165925    ,    1.1392    ,    1.116427    ,    1.099007    ,    1.088416    ,    1.081737    ,    1.076117    ,    1.070475    ,    1.063503    ,    1.054283    ,    1.053463    ,    1.061034    ,    1.073677    ,    1.086832    ,    1.096336    ,    1.10224    ,    1.10105    ,    1.110942    ,    1.123285    ,    1.131769    ,    1.137033    ,    1.145857    ,    1.151705    ,    1.152316    ,    1.156053    ,    1.163038    ,    1.169351    ,    1.172944    ,    1.172113    ,    1.166129    ,    1.160086    ,    1.163213    ,    1.170524    ,    1.174075    ,    1.168516    ,    1.18775    ,    1.180478    ,    1.133241    ,    1.097964    ,    1.077296    ,    1.049011    ,    1.025098    ,    1.017812    ,    1.025949    ,    1.039818    ,    1.047465    ,    1.061279    ,    1.050993    ,    0.997376    ,    1.020308    ,    1.052671    ,    1.000988    ,    1.064642    ,    1.093465    ,    1.085611    ,    1.118525    ,    1.161916    ,    1.170859    ,    1.16244    ,    1.12468    ,    1.131491    ,    1.115562    ,    1.101211    ,    1.088412    ,    1.075532    ,    1.054281    ,    1.054457    ,    1.075264    ,    1.060459    ,    1.039771    ,    1.019787    ,    0.996469    ,    0.978441    ,    0.972728    ,    0.955383    ,    0.918388    ,    0.878974    ,    0.840367    ,    0.792358    },
    {    0.607226    ,    0.58237    ,    0.573153    ,    0.580155    ,    0.591101    ,    0.599065    ,    0.601466    ,    0.601172    ,    0.603189    ,    0.602044    ,    0.592665    ,    0.581121    ,    0.577809    ,    0.584235    ,    0.605084    ,    0.616083    ,    0.612881    ,    0.606708    ,    0.605297    ,    0.606404    ,    0.615188    ,    0.629468    ,    0.646352    ,    0.661078    ,    0.675551    ,    0.695548    ,    0.7184    ,    0.73916    ,    0.758082    ,    0.773265    ,    0.788585    ,    0.805924    ,    0.821053    ,    0.838103    ,    0.857427    ,    0.882225    ,    0.913422    ,    0.950202    ,    0.995432    ,    1.047012    ,    1.098279    ,    1.138465    ,    1.19764    ,    1.250165    ,    1.296101    ,    1.276196    ,    1.220259    ,    1.156384    ,    1.112812    ,    1.088581    ,    1.068029    ,    1.045157    ,    1.019339    ,    0.997127    ,    0.979756    ,    0.967201    ,    0.952829    ,    0.93874    ,    0.924664    ,    0.918837    ,    0.927803    ,    0.94884    ,    0.962642    ,    0.968885    ,    0.971437    ,    0.969712    ,    0.97224    ,    0.97784    ,    0.981458    ,    0.986395    ,    0.99079    ,    0.991659    ,    0.987775    ,    0.985314    ,    0.984218    ,    0.982644    ,    0.97727    ,    0.973797    ,    0.971662    ,    0.973575    ,    0.988256    ,    1.0108    ,    1.034785    ,    1.055786    ,    1.063437    ,    1.053961    ,    1.036836    ,    1.012191    ,    0.980232    ,    0.932948    ,    0.884524    ,    0.849101    ,    0.839469    ,    0.854691    ,    0.873762    ,    0.908146    ,    0.943134    ,    0.979263    ,    1.017315    ,    1.048187    ,    1.067302    ,    1.091918    ,    1.109183    ,    1.116617    ,    1.119914    ,    1.106325    ,    1.078835    ,    1.053793    ,    1.032948    ,    1.013693    ,    0.99661    ,    0.984517    ,    0.973462    ,    0.965159    ,    0.953541    ,    0.939958    ,    0.907768    ,    0.866754    ,    0.833619    ,    0.819797    ,    0.823171    ,    0.828014    ,    0.831912    ,    0.80687    ,    0.769927    ,    0.731748    ,    0.69514    ,    0.646422    },
    {    0.474693    ,    0.458657    ,    0.455064    ,    0.46396    ,    0.467755    ,    0.462477    ,    0.457367    ,    0.462066    ,    0.47486    ,    0.483641    ,    0.472945    ,    0.454881    ,    0.451078    ,    0.458563    ,    0.470728    ,    0.476097    ,    0.47651    ,    0.481219    ,    0.484339    ,    0.48662    ,    0.486981    ,    0.496477    ,    0.508119    ,    0.522183    ,    0.535176    ,    0.549224    ,    0.565286    ,    0.580336    ,    0.593712    ,    0.60457    ,    0.615907    ,    0.62746    ,    0.638341    ,    0.647724    ,    0.657738    ,    0.674106    ,    0.695699    ,    0.715348    ,    0.730108    ,    0.752392    ,    0.802837    ,    0.882856    ,    0.939448    ,    0.994645    ,    1.061711    ,    1.092138    ,    1.083133    ,    1.041557    ,    1.012712    ,    1.001131    ,    0.991844    ,    0.974373    ,    0.961136    ,    0.942767    ,    0.923532    ,    0.903406    ,    0.885726    ,    0.873762    ,    0.859504    ,    0.84241    ,    0.829807    ,    0.831026    ,    0.840661    ,    0.842867    ,    0.843561    ,    0.848539    ,    0.855837    ,    0.855965    ,    0.851233    ,    0.851515    ,    0.85488    ,    0.856996    ,    0.855104    ,    0.853831    ,    0.85538    ,    0.860696    ,    0.86442    ,    0.86391    ,    0.865927    ,    0.868683    ,    0.873833    ,    0.884412    ,    0.902735    ,    0.921375    ,    0.934024    ,    0.941116    ,    0.949056    ,    0.958125    ,    0.960093    ,    0.925069    ,    0.869903    ,    0.824781    ,    0.79222    ,    0.767182    ,    0.749983    ,    0.758792    ,    0.775681    ,    0.794637    ,    0.811044    ,    0.826644    ,    0.830344    ,    0.832496    ,    0.831591    ,    0.830007    ,    0.829623    ,    0.828371    ,    0.818963    ,    0.804865    ,    0.792032    ,    0.775257    ,    0.760399    ,    0.746078    ,    0.735053    ,    0.732196    ,    0.729251    ,    0.727553    ,    0.710652    ,    0.683902    ,    0.671844    ,    0.66476    ,    0.654948    ,    0.652755    ,    0.647263    ,    0.616751    ,    0.579296    ,    0.551603    ,    0.527101    ,    0.497999    },
    {    0.460658    ,    0.445285    ,    0.444717    ,    0.455436    ,    0.458843    ,    0.44708    ,    0.432404    ,    0.423284    ,    0.419045    ,    0.411806    ,    0.407901    ,    0.400696    ,    0.388107    ,    0.374812    ,    0.359613    ,    0.353023    ,    0.358655    ,    0.38298    ,    0.410392    ,    0.431833    ,    0.440053    ,    0.443572    ,    0.450708    ,    0.461998    ,    0.478256    ,    0.494768    ,    0.508045    ,    0.520701    ,    0.531594    ,    0.539874    ,    0.547477    ,    0.554741    ,    0.560821    ,    0.564949    ,    0.567097    ,    0.572703    ,    0.577619    ,    0.575594    ,    0.576631    ,    0.591278    ,    0.621218    ,    0.657109    ,    0.694414    ,    0.730945    ,    0.779359    ,    0.831178    ,    0.85035    ,    0.848717    ,    0.842467    ,    0.84809    ,    0.851265    ,    0.85474    ,    0.847908    ,    0.845308    ,    0.846069    ,    0.844815    ,    0.84748    ,    0.841189    ,    0.829265    ,    0.81812    ,    0.803989    ,    0.786211    ,    0.770106    ,    0.752294    ,    0.733443    ,    0.725056    ,    0.721595    ,    0.723024    ,    0.719013    ,    0.717832    ,    0.724174    ,    0.735041    ,    0.743758    ,    0.743482    ,    0.732997    ,    0.726177    ,    0.726098    ,    0.723495    ,    0.724257    ,    0.728917    ,    0.735738    ,    0.74189    ,    0.754329    ,    0.770191    ,    0.788792    ,    0.805846    ,    0.823388    ,    0.845763    ,    0.867353    ,    0.871093    ,    0.85304    ,    0.82235    ,    0.785177    ,    0.73646    ,    0.710867    ,    0.700383    ,    0.695755    ,    0.688337    ,    0.673196    ,    0.662187    ,    0.653975    ,    0.650363    ,    0.64752    ,    0.643675    ,    0.637798    ,    0.634517    ,    0.626344    ,    0.616161    ,    0.604841    ,    0.596211    ,    0.586445    ,    0.58031    ,    0.577813    ,    0.580086    ,    0.58225    ,    0.581048    ,    0.575287    ,    0.565789    ,    0.558704    ,    0.551879    ,    0.543735    ,    0.540614    ,    0.533455    ,    0.526204    ,    0.514086    ,    0.504903    ,    0.495422    ,    0.48198    },
    {    0.485162    ,    0.479984    ,    0.481783    ,    0.490628    ,    0.488813    ,    0.475045    ,    0.452285    ,    0.438014    ,    0.424207    ,    0.401091    ,    0.384629    ,    0.370599    ,    0.356792    ,    0.355736    ,    0.355099    ,    0.354958    ,    0.360151    ,    0.380654    ,    0.412151    ,    0.435635    ,    0.442788    ,    0.443867    ,    0.448646    ,    0.459976    ,    0.477024    ,    0.492777    ,    0.503704    ,    0.513721    ,    0.523312    ,    0.529291    ,    0.532002    ,    0.532951    ,    0.535834    ,    0.534356    ,    0.528135    ,    0.521148    ,    0.517157    ,    0.5271    ,    0.581222    ,    0.657883    ,    0.682992    ,    0.626483    ,    0.613559    ,    0.618774    ,    0.622686    ,    0.638394    ,    0.66862    ,    0.694842    ,    0.695576    ,    0.688394    ,    0.68708    ,    0.700286    ,    0.694827    ,    0.699506    ,    0.706738    ,    0.71572    ,    0.728825    ,    0.723749    ,    0.709107    ,    0.699063    ,    0.688144    ,    0.673533    ,    0.664942    ,    0.653431    ,    0.636853    ,    0.625547    ,    0.616065    ,    0.611035    ,    0.606452    ,    0.600479    ,    0.599467    ,    0.604004    ,    0.612221    ,    0.624179    ,    0.627403    ,    0.624437    ,    0.626808    ,    0.63085    ,    0.630372    ,    0.628176    ,    0.63281    ,    0.6383    ,    0.647872    ,    0.662397    ,    0.675433    ,    0.686239    ,    0.700447    ,    0.718979    ,    0.741557    ,    0.76205    ,    0.77759    ,    0.759632    ,    0.702197    ,    0.650735    ,    0.621021    ,    0.606421    ,    0.601712    ,    0.596824    ,    0.597726    ,    0.591988    ,    0.586772    ,    0.581705    ,    0.573401    ,    0.567642    ,    0.569369    ,    0.57623    ,    0.577488    ,    0.573196    ,    0.56484    ,    0.559826    ,    0.553721    ,    0.548774    ,    0.546389    ,    0.544714    ,    0.547449    ,    0.548858    ,    0.539583    ,    0.535047    ,    0.534991    ,    0.525826    ,    0.517619    ,    0.517611    ,    0.515501    ,    0.52269    ,    0.524979    ,    0.518884    ,    0.506159    ,    0.493334    },
    {    0.520907    ,    0.526504    ,    0.532123    ,    0.532938    ,    0.529508    ,    0.514212    ,    0.485255    ,    0.472236    ,    0.464056    ,    0.442993    ,    0.430436    ,    0.422584    ,    0.421998    ,    0.434287    ,    0.441835    ,    0.44923    ,    0.453846    ,    0.458756    ,    0.466627    ,    0.473327    ,    0.475431    ,    0.478357    ,    0.485283    ,    0.493354    ,    0.503015    ,    0.51151    ,    0.517971    ,    0.525315    ,    0.532559    ,    0.534536    ,    0.533205    ,    0.532613    ,    0.530888    ,    0.526829    ,    0.521138    ,    0.520445    ,    0.53142    ,    0.564717    ,    0.668488    ,    0.744353    ,    0.70169    ,    0.65935    ,    0.642853    ,    0.639991    ,    0.609509    ,    0.594431    ,    0.612958    ,    0.636951    ,    0.6428    ,    0.64434    ,    0.652941    ,    0.654084    ,    0.652882    ,    0.660149    ,    0.659439    ,    0.657858    ,    0.656204    ,    0.647151    ,    0.62863    ,    0.613037    ,    0.600462    ,    0.590995    ,    0.595806    ,    0.600283    ,    0.589405    ,    0.580687    ,    0.577109    ,    0.587212    ,    0.597953    ,    0.584021    ,    0.574005    ,    0.574378    ,    0.580419    ,    0.586474    ,    0.596776    ,    0.610265    ,    0.617197    ,    0.618172    ,    0.618049    ,    0.623293    ,    0.633563    ,    0.638565    ,    0.643716    ,    0.655809    ,    0.666362    ,    0.670797    ,    0.68606    ,    0.707435    ,    0.721041    ,    0.718485    ,    0.70517    ,    0.683585    ,    0.665562    ,    0.667976    ,    0.661562    ,    0.656099    ,    0.652091    ,    0.647349    ,    0.648591    ,    0.644765    ,    0.639097    ,    0.636861    ,    0.634854    ,    0.629498    ,    0.62908    ,    0.625815    ,    0.611712    ,    0.597839    ,    0.58529    ,    0.571031    ,    0.558215    ,    0.553461    ,    0.552768    ,    0.553196    ,    0.556724    ,    0.561536    ,    0.562824    ,    0.558544    ,    0.549242    ,    0.533462    ,    0.521087    ,    0.522301    ,    0.532169    ,    0.544149    ,    0.543894    ,    0.541814    ,    0.529491    ,    0.517492    },
    {    0.610608    ,    0.629876    ,    0.630777    ,    0.621578    ,    0.601946    ,    0.58767    ,    0.565706    ,    0.553369    ,    0.543093    ,    0.531064    ,    0.518148    ,    0.515054    ,    0.518924    ,    0.519503    ,    0.517475    ,    0.514133    ,    0.512007    ,    0.511474    ,    0.512128    ,    0.514185    ,    0.516338    ,    0.517664    ,    0.521648    ,    0.527486    ,    0.533315    ,    0.539627    ,    0.544784    ,    0.549459    ,    0.551209    ,    0.550097    ,    0.547434    ,    0.546943    ,    0.544863    ,    0.541735    ,    0.539253    ,    0.544134    ,    0.557929    ,    0.599313    ,    0.732334    ,    0.837656    ,    0.803598    ,    0.726128    ,    0.676932    ,    0.631103    ,    0.607207    ,    0.619442    ,    0.653557    ,    0.677087    ,    0.693144    ,    0.701605    ,    0.708671    ,    0.701737    ,    0.698329    ,    0.699495    ,    0.69468    ,    0.680269    ,    0.668464    ,    0.661329    ,    0.647283    ,    0.630546    ,    0.610782    ,    0.607143    ,    0.625353    ,    0.638955    ,    0.640704    ,    0.636445    ,    0.629496    ,    0.636501    ,    0.644714    ,    0.643016    ,    0.63559    ,    0.634601    ,    0.637064    ,    0.630729    ,    0.627658    ,    0.639443    ,    0.666367    ,    0.672252    ,    0.676675    ,    0.675569    ,    0.680008    ,    0.678367    ,    0.675265    ,    0.674712    ,    0.682775    ,    0.687923    ,    0.704704    ,    0.72538    ,    0.731701    ,    0.729585    ,    0.739147    ,    0.766104    ,    0.78761    ,    0.787673    ,    0.772771    ,    0.758168    ,    0.766614    ,    0.767628    ,    0.761304    ,    0.745382    ,    0.733517    ,    0.719714    ,    0.703317    ,    0.684087    ,    0.663823    ,    0.650251    ,    0.63162    ,    0.615152    ,    0.595278    ,    0.574517    ,    0.558813    ,    0.556141    ,    0.562763    ,    0.575919    ,    0.578156    ,    0.57714    ,    0.584902    ,    0.583554    ,    0.574543    ,    0.573398    ,    0.572867    ,    0.571705    ,    0.580443    ,    0.610898    ,    0.610323    ,    0.599797    ,    0.590641    ,    0.59971    },
    {    0.747967    ,    0.748375    ,    0.726845    ,    0.698395    ,    0.674878    ,    0.65671    ,    0.639412    ,    0.619482    ,    0.598833    ,    0.585314    ,    0.577796    ,    0.572596    ,    0.568926    ,    0.564345    ,    0.55747    ,    0.549326    ,    0.544478    ,    0.543942    ,    0.54464    ,    0.545625    ,    0.545098    ,    0.543035    ,    0.545182    ,    0.552001    ,    0.558095    ,    0.561657    ,    0.568287    ,    0.573356    ,    0.575305    ,    0.576415    ,    0.576243    ,    0.575185    ,    0.572486    ,    0.573617    ,    0.573096    ,    0.577671    ,    0.58315    ,    0.613471    ,    0.746232    ,    0.856313    ,    0.857549    ,    0.75941    ,    0.704778    ,    0.659396    ,    0.691795    ,    0.734698    ,    0.765495    ,    0.777042    ,    0.798198    ,    0.800923    ,    0.790322    ,    0.778076    ,    0.756322    ,    0.744017    ,    0.73798    ,    0.726984    ,    0.72546    ,    0.730147    ,    0.724719    ,    0.711906    ,    0.696683    ,    0.69488    ,    0.708575    ,    0.714402    ,    0.710248    ,    0.699899    ,    0.69925    ,    0.704732    ,    0.716356    ,    0.719302    ,    0.71178    ,    0.708209    ,    0.716831    ,    0.720083    ,    0.723099    ,    0.73503    ,    0.75828    ,    0.766518    ,    0.769835    ,    0.767598    ,    0.764613    ,    0.760634    ,    0.757898    ,    0.752474    ,    0.742256    ,    0.734243    ,    0.735992    ,    0.747201    ,    0.764179    ,    0.776521    ,    0.807614    ,    0.842908    ,    0.857134    ,    0.866384    ,    0.864057    ,    0.849146    ,    0.833473    ,    0.81238    ,    0.788959    ,    0.757918    ,    0.733442    ,    0.713604    ,    0.692084    ,    0.672479    ,    0.65716    ,    0.642752    ,    0.625522    ,    0.60593    ,    0.585935    ,    0.569889    ,    0.559205    ,    0.558534    ,    0.56755    ,    0.58167    ,    0.598696    ,    0.639825    ,    0.707757    ,    0.738086    ,    0.70435    ,    0.679379    ,    0.670698    ,    0.659508    ,    0.667301    ,    0.730452    ,    0.79101    ,    0.73954    ,    0.711803    ,    0.733142    },
    {    0.81634    ,    0.800832    ,    0.759905    ,    0.7159    ,    0.687986    ,    0.671281    ,    0.654162    ,    0.639039    ,    0.626041    ,    0.618097    ,    0.610765    ,    0.605262    ,    0.598464    ,    0.589406    ,    0.580721    ,    0.571661    ,    0.566816    ,    0.566391    ,    0.567077    ,    0.566427    ,    0.565856    ,    0.567238    ,    0.573894    ,    0.580253    ,    0.586294    ,    0.591386    ,    0.596033    ,    0.602508    ,    0.607577    ,    0.607703    ,    0.607644    ,    0.607537    ,    0.607596    ,    0.611229    ,    0.613184    ,    0.622543    ,    0.608559    ,    0.620396    ,    0.768745    ,    0.907465    ,    0.890696    ,    0.790671    ,    0.731238    ,    0.764458    ,    0.837332    ,    0.862256    ,    0.857342    ,    0.853819    ,    0.855444    ,    0.861252    ,    0.85714    ,    0.847529    ,    0.826439    ,    0.813923    ,    0.804948    ,    0.802503    ,    0.813361    ,    0.827901    ,    0.83024    ,    0.812668    ,    0.793595    ,    0.787951    ,    0.785024    ,    0.786463    ,    0.788862    ,    0.779343    ,    0.781173    ,    0.797047    ,    0.808496    ,    0.808319    ,    0.805086    ,    0.804847    ,    0.821966    ,    0.841994    ,    0.841928    ,    0.837318    ,    0.82108    ,    0.851825    ,    0.879824    ,    0.893878    ,    0.876969    ,    0.876866    ,    0.860275    ,    0.848498    ,    0.835392    ,    0.818775    ,    0.82009    ,    0.796737    ,    0.811382    ,    0.822334    ,    0.840111    ,    0.857104    ,    0.864392    ,    0.860825    ,    0.845072    ,    0.820323    ,    0.800282    ,    0.77338    ,    0.738638    ,    0.708996    ,    0.685938    ,    0.668571    ,    0.651374    ,    0.638484    ,    0.626689    ,    0.613686    ,    0.600522    ,    0.586888    ,    0.577021    ,    0.569638    ,    0.564625    ,    0.568963    ,    0.583553    ,    0.603689    ,    0.631285    ,    0.691455    ,    0.814646    ,    0.942551    ,    0.899074    ,    0.828389    ,    0.805306    ,    0.793246    ,    0.787199    ,    0.821095    ,    0.874796    ,    0.894624    ,    0.854918    ,    0.83454    },
    {    0.806577    ,    0.76549    ,    0.740362    ,    0.71682    ,    0.691831    ,    0.674723    ,    0.663778    ,    0.6548    ,    0.646491    ,    0.64079    ,    0.632515    ,    0.624333    ,    0.615838    ,    0.60503    ,    0.596025    ,    0.589043    ,    0.584614    ,    0.583314    ,    0.582775    ,    0.584894    ,    0.586533    ,    0.591529    ,    0.599452    ,    0.603477    ,    0.610527    ,    0.619573    ,    0.62416    ,    0.632766    ,    0.639522    ,    0.638447    ,    0.640093    ,    0.644671    ,    0.645014    ,    0.645149    ,    0.646154    ,    0.644832    ,    0.605835    ,    0.599988    ,    0.789783    ,    0.931855    ,    0.882231    ,    0.805417    ,    0.741792    ,    0.788044    ,    0.883637    ,    0.902452    ,    0.865251    ,    0.822923    ,    0.809267    ,    0.815304    ,    0.823904    ,    0.825685    ,    0.822948    ,    0.823046    ,    0.826738    ,    0.83171    ,    0.838849    ,    0.866624    ,    0.88988    ,    0.852065    ,    0.822245    ,    0.807277    ,    0.802136    ,    0.808469    ,    0.812147    ,    0.814586    ,    0.833591    ,    0.868593    ,    0.892801    ,    0.912986    ,    0.934048    ,    0.949925    ,    0.942436    ,    0.936056    ,    0.926787    ,    0.881977    ,    0.886127    ,    0.903566    ,    0.940384    ,    0.940585    ,    0.884437    ,    0.853782    ,    0.812149    ,    0.794573    ,    0.798835    ,    0.801262    ,    0.795048    ,    0.78216    ,    0.777317    ,    0.766406    ,    0.765785    ,    0.767211    ,    0.773665    ,    0.765314    ,    0.74534    ,    0.728671    ,    0.709736    ,    0.686379    ,    0.664918    ,    0.647697    ,    0.634729    ,    0.623666    ,    0.615048    ,    0.614577    ,    0.616087    ,    0.609421    ,    0.607301    ,    0.609168    ,    0.614562    ,    0.617511    ,    0.618311    ,    0.627067    ,    0.633054    ,    0.64017    ,    0.667861    ,    0.738499    ,    0.851251    ,    1.011163    ,    1.063623    ,    0.973465    ,    0.912804    ,    0.886213    ,    0.868625    ,    0.875569    ,    0.886541    ,    0.910333    ,    0.948259    ,    0.895697    },
    {    0.799234    ,    0.735861    ,    0.712708    ,    0.700124    ,    0.686909    ,    0.676013    ,    0.672304    ,    0.669281    ,    0.664382    ,    0.661803    ,    0.655269    ,    0.647211    ,    0.640088    ,    0.633592    ,    0.627718    ,    0.620806    ,    0.617588    ,    0.618672    ,    0.617882    ,    0.617823    ,    0.617433    ,    0.620877    ,    0.627448    ,    0.634005    ,    0.641853    ,    0.651498    ,    0.6534    ,    0.656564    ,    0.660539    ,    0.659708    ,    0.660812    ,    0.660558    ,    0.656199    ,    0.658374    ,    0.656681    ,    0.634189    ,    0.57759    ,    0.573649    ,    0.820625    ,    0.992018    ,    0.89586    ,    0.798572    ,    0.776508    ,    0.783081    ,    0.873002    ,    0.878861    ,    0.736815    ,    0.703124    ,    0.714141    ,    0.735335    ,    0.751478    ,    0.760068    ,    0.772027    ,    0.778285    ,    0.786246    ,    0.798299    ,    0.805378    ,    0.816065    ,    0.814895    ,    0.798593    ,    0.791763    ,    0.789282    ,    0.791279    ,    0.794783    ,    0.795272    ,    0.799543    ,    0.819229    ,    0.860227    ,    0.910991    ,    0.962367    ,    0.940384    ,    0.879817    ,    0.836256    ,    0.801954    ,    0.764196    ,    0.725495    ,    0.718471    ,    0.715951    ,    0.732416    ,    0.734239    ,    0.716374    ,    0.701048    ,    0.691084    ,    0.691139    ,    0.699064    ,    0.70449    ,    0.699251    ,    0.693034    ,    0.684621    ,    0.682982    ,    0.686326    ,    0.685966    ,    0.682259    ,    0.675404    ,    0.667216    ,    0.659393    ,    0.653586    ,    0.653249    ,    0.655734    ,    0.65409    ,    0.653555    ,    0.649442    ,    0.650215    ,    0.660134    ,    0.65727    ,    0.645131    ,    0.650834    ,    0.661094    ,    0.670967    ,    0.677179    ,    0.674932    ,    0.669095    ,    0.662411    ,    0.691952    ,    0.792389    ,    0.901436    ,    0.950225    ,    0.998089    ,    1.081168    ,    1.032661    ,    0.95122    ,    0.926382    ,    0.916467    ,    0.912894    ,    0.908488    ,    0.906757    ,    0.92975    ,    0.910782    },
    {    0.763159    ,    0.719053    ,    0.705435    ,    0.698029    ,    0.690383    ,    0.684318    ,    0.683374    ,    0.685874    ,    0.685801    ,    0.683165    ,    0.680484    ,    0.677865    ,    0.673382    ,    0.66707    ,    0.659134    ,    0.652926    ,    0.652697    ,    0.656672    ,    0.655852    ,    0.655228    ,    0.652052    ,    0.646244    ,    0.646932    ,    0.653878    ,    0.659172    ,    0.661109    ,    0.657581    ,    0.654705    ,    0.654132    ,    0.648308    ,    0.641815    ,    0.642173    ,    0.639734    ,    0.634999    ,    0.618957    ,    0.580555    ,    0.529195    ,    0.555742    ,    0.779786    ,    1.038804    ,    0.904855    ,    0.774247    ,    0.723475    ,    0.765298    ,    0.810914    ,    0.825543    ,    0.699242    ,    0.663748    ,    0.678065    ,    0.695026    ,    0.708039    ,    0.715853    ,    0.727035    ,    0.732501    ,    0.743497    ,    0.757504    ,    0.764177    ,    0.767474    ,    0.760955    ,    0.754635    ,    0.760087    ,    0.765169    ,    0.761568    ,    0.753318    ,    0.750822    ,    0.766305    ,    0.803795    ,    0.851136    ,    0.880256    ,    0.85907    ,    0.818428    ,    0.818516    ,    0.819655    ,    0.801458    ,    0.762523    ,    0.704062    ,    0.634664    ,    0.614399    ,    0.632754    ,    0.651933    ,    0.665323    ,    0.676415    ,    0.687093    ,    0.689661    ,    0.688593    ,    0.690468    ,    0.69437    ,    0.693645    ,    0.690389    ,    0.688658    ,    0.694057    ,    0.698794    ,    0.700589    ,    0.697152    ,    0.692031    ,    0.68872    ,    0.690316    ,    0.692483    ,    0.695904    ,    0.696997    ,    0.702374    ,    0.693399    ,    0.68837    ,    0.702951    ,    0.692264    ,    0.720606    ,    0.759296    ,    0.711788    ,    0.690714    ,    0.687682    ,    0.683213    ,    0.682976    ,    0.716181    ,    0.848931    ,    0.956832    ,    1.011656    ,    1.050757    ,    1.043526    ,    1.016289    ,    0.940038    ,    0.880266    ,    0.879949    ,    0.882612    ,    0.883846    ,    0.899175    ,    0.900325    ,    0.879552    ,    0.825301    },
    {    0.731312    ,    0.711159    ,    0.708152    ,    0.701768    ,    0.69986    ,    0.699952    ,    0.70073    ,    0.706007    ,    0.707825    ,    0.707597    ,    0.708406    ,    0.70795    ,    0.702735    ,    0.694218    ,    0.685951    ,    0.680456    ,    0.678203    ,    0.676853    ,    0.672733    ,    0.666852    ,    0.66255    ,    0.655278    ,    0.646136    ,    0.646    ,    0.646269    ,    0.636287    ,    0.626065    ,    0.617083    ,    0.610109    ,    0.604979    ,    0.600799    ,    0.600462    ,    0.592148    ,    0.575533    ,    0.550606    ,    0.512811    ,    0.487178    ,    0.536312    ,    0.729403    ,    1.05512    ,    0.965643    ,    0.851378    ,    0.807571    ,    0.868063    ,    0.912568    ,    0.862632    ,    0.702735    ,    0.676239    ,    0.672227    ,    0.681434    ,    0.695003    ,    0.698797    ,    0.70303    ,    0.709534    ,    0.721812    ,    0.732535    ,    0.731461    ,    0.721674    ,    0.710037    ,    0.704017    ,    0.703011    ,    0.703229    ,    0.70118    ,    0.697821    ,    0.705203    ,    0.733029    ,    0.770974    ,    0.792056    ,    0.793319    ,    0.766071    ,    0.816304    ,    1.013951    ,    1.117627    ,    1.101668    ,    1.008801    ,    0.815005    ,    0.739291    ,    0.683557    ,    0.6905    ,    0.717506    ,    0.728274    ,    0.739129    ,    0.750418    ,    0.754048    ,    0.758572    ,    0.763258    ,    0.767095    ,    0.767163    ,    0.765041    ,    0.761659    ,    0.76245    ,    0.765498    ,    0.762235    ,    0.755512    ,    0.748616    ,    0.742372    ,    0.731494    ,    0.716613    ,    0.711746    ,    0.722122    ,    0.725764    ,    0.702826    ,    0.692176    ,    0.712923    ,    0.720233    ,    0.865712    ,    0.980677    ,    0.916197    ,    0.850358    ,    0.789002    ,    0.757537    ,    0.787886    ,    0.940622    ,    0.995145    ,    1.030589    ,    1.0807    ,    1.099285    ,    1.089342    ,    0.997315    ,    0.822865    ,    0.758326    ,    0.770513    ,    0.784608    ,    0.793981    ,    0.811428    ,    0.81046    ,    0.788183    ,    0.763462    },
    {    0.743876    ,    0.734803    ,    0.725914    ,    0.717351    ,    0.713636    ,    0.716902    ,    0.723641    ,    0.728347    ,    0.72528    ,    0.720824    ,    0.718315    ,    0.716676    ,    0.711858    ,    0.707331    ,    0.699832    ,    0.693193    ,    0.684576    ,    0.673854    ,    0.664739    ,    0.65347    ,    0.643293    ,    0.63149    ,    0.618115    ,    0.610359    ,    0.604593    ,    0.587685    ,    0.57291    ,    0.566301    ,    0.558137    ,    0.55468    ,    0.551158    ,    0.546081    ,    0.532787    ,    0.510535    ,    0.490455    ,    0.471091    ,    0.485246    ,    0.55822    ,    0.743704    ,    1.112214    ,    1.114942    ,    0.932839    ,    0.977906    ,    0.965331    ,    1.012045    ,    1.031363    ,    0.904461    ,    0.704831    ,    0.663805    ,    0.674044    ,    0.684053    ,    0.686332    ,    0.690041    ,    0.69275    ,    0.695288    ,    0.695988    ,    0.688817    ,    0.671604    ,    0.660771    ,    0.653293    ,    0.647812    ,    0.649142    ,    0.656363    ,    0.664096    ,    0.677845    ,    0.700408    ,    0.72094    ,    0.729728    ,    0.735625    ,    0.75795    ,    0.943945    ,    1.192747    ,    1.326397    ,    1.326288    ,    1.224467    ,    1.021072    ,    0.792517    ,    0.763501    ,    0.774996    ,    0.796023    ,    0.805885    ,    0.814372    ,    0.828821    ,    0.827305    ,    0.822303    ,    0.819462    ,    0.81414    ,    0.808143    ,    0.803476    ,    0.798484    ,    0.791849    ,    0.785264    ,    0.773219    ,    0.765071    ,    0.752279    ,    0.735444    ,    0.720924    ,    0.716332    ,    0.723014    ,    0.727211    ,    0.713215    ,    0.693239    ,    0.694807    ,    0.711479    ,    0.749942    ,    0.97932    ,    1.16893    ,    1.058943    ,    1.005432    ,    0.99307    ,    0.978634    ,    0.992136    ,    1.051309    ,    1.057768    ,    1.092544    ,    1.115531    ,    1.121629    ,    1.098035    ,    0.999037    ,    0.775015    ,    0.720378    ,    0.732665    ,    0.745956    ,    0.755719    ,    0.760463    ,    0.759277    ,    0.753267    ,    0.749701    },
    {    0.752865    ,    0.745597    ,    0.734685    ,    0.723687    ,    0.717529    ,    0.718946    ,    0.721212    ,    0.718668    ,    0.713382    ,    0.708377    ,    0.704482    ,    0.702309    ,    0.69888    ,    0.689505    ,    0.677774    ,    0.666918    ,    0.652733    ,    0.635996    ,    0.623285    ,    0.611268    ,    0.59858    ,    0.584457    ,    0.569241    ,    0.556662    ,    0.54852    ,    0.539518    ,    0.531333    ,    0.529139    ,    0.523631    ,    0.517402    ,    0.505955    ,    0.492327    ,    0.480505    ,    0.460753    ,    0.455016    ,    0.472589    ,    0.522486    ,    0.611986    ,    0.762166    ,    1.108608    ,    1.370342    ,    1.087629    ,    1.023302    ,    1.100763    ,    1.134741    ,    1.125458    ,    1.037658    ,    0.817881    ,    0.677093    ,    0.670212    ,    0.677225    ,    0.686122    ,    0.68625    ,    0.676754    ,    0.667808    ,    0.657897    ,    0.64726    ,    0.635251    ,    0.627831    ,    0.622342    ,    0.620766    ,    0.622743    ,    0.634378    ,    0.651467    ,    0.669972    ,    0.685868    ,    0.700861    ,    0.726015    ,    0.771916    ,    0.870461    ,    1.155547    ,    1.35664    ,    1.374141    ,    1.348486    ,    1.272558    ,    1.126725    ,    0.860054    ,    0.796374    ,    0.804068    ,    0.82811    ,    0.854631    ,    0.812819    ,    0.818657    ,    0.822361    ,    0.811601    ,    0.805417    ,    0.794268    ,    0.788419    ,    0.779392    ,    0.76633    ,    0.754822    ,    0.746166    ,    0.733874    ,    0.720185    ,    0.708569    ,    0.70373    ,    0.705773    ,    0.70535    ,    0.702268    ,    0.690978    ,    0.68438    ,    0.68498    ,    0.69537    ,    0.722195    ,    0.797109    ,    1.08299    ,    1.158147    ,    1.149029    ,    1.105461    ,    1.118425    ,    1.110687    ,    1.067621    ,    1.102255    ,    1.142438    ,    1.134016    ,    1.138818    ,    1.113022    ,    1.052249    ,    0.940042    ,    0.745896    ,    0.73462    ,    0.749269    ,    0.753352    ,    0.761199    ,    0.762357    ,    0.763585    ,    0.758829    ,    0.754531    },
    {    0.747742    ,    0.739899    ,    0.73059    ,    0.721675    ,    0.714981    ,    0.709148    ,    0.705204    ,    0.700073    ,    0.696001    ,    0.693838    ,    0.690305    ,    0.685936    ,    0.676639    ,    0.664618    ,    0.65264    ,    0.641159    ,    0.626109    ,    0.610676    ,    0.598819    ,    0.586735    ,    0.574507    ,    0.562623    ,    0.550806    ,    0.541579    ,    0.534029    ,    0.52704    ,    0.518268    ,    0.508096    ,    0.495544    ,    0.481528    ,    0.466208    ,    0.456665    ,    0.453893    ,    0.454773    ,    0.473087    ,    0.511045    ,    0.561085    ,    0.655918    ,    0.78172    ,    1.095354    ,    1.375897    ,    1.197996    ,    1.136475    ,    1.174872    ,    1.194238    ,    1.174342    ,    1.166528    ,    1.053882    ,    0.910115    ,    0.728133    ,    0.677656    ,    0.68195    ,    0.676964    ,    0.66127    ,    0.649891    ,    0.636396    ,    0.62481    ,    0.620305    ,    0.619199    ,    0.621058    ,    0.629175    ,    0.63687    ,    0.650617    ,    0.66079    ,    0.66975    ,    0.687866    ,    0.719025    ,    0.773512    ,    0.829629    ,    0.984    ,    1.273811    ,    1.3674    ,    1.409705    ,    1.31078    ,    1.263005    ,    1.199292    ,    1.064527    ,    0.834377    ,    0.816723    ,    0.879594    ,    1.049736    ,    0.858593    ,    0.776747    ,    0.777128    ,    0.771514    ,    0.768363    ,    0.755162    ,    0.749357    ,    0.739129    ,    0.728784    ,    0.720524    ,    0.710643    ,    0.699034    ,    0.691072    ,    0.690007    ,    0.692197    ,    0.691389    ,    0.684101    ,    0.679987    ,    0.676486    ,    0.682768    ,    0.694548    ,    0.710405    ,    0.740244    ,    0.821532    ,    1.166499    ,    1.291604    ,    1.171279    ,    1.272898    ,    1.223782    ,    1.172242    ,    1.14343    ,    1.172494    ,    1.190877    ,    1.140155    ,    1.100872    ,    1.035017    ,    0.976165    ,    0.794175    ,    0.73541    ,    0.748607    ,    0.75698    ,    0.759848    ,    0.761948    ,    0.761197    ,    0.759791    ,    0.754796    ,    0.75069    },
    {    0.745782    ,    0.739185    ,    0.732576    ,    0.726166    ,    0.719161    ,    0.713296    ,    0.709753    ,    0.709595    ,    0.706913    ,    0.701473    ,    0.693545    ,    0.684152    ,    0.672212    ,    0.660367    ,    0.649225    ,    0.63933    ,    0.627664    ,    0.614604    ,    0.606114    ,    0.596357    ,    0.585589    ,    0.573899    ,    0.558848    ,    0.546322    ,    0.534952    ,    0.522091    ,    0.509821    ,    0.496934    ,    0.485915    ,    0.478458    ,    0.472418    ,    0.473602    ,    0.480735    ,    0.505435    ,    0.53237    ,    0.572928    ,    0.632822    ,    0.709828    ,    0.808388    ,    1.142205    ,    1.391821    ,    1.261937    ,    1.247077    ,    1.295034    ,    1.290417    ,    1.259609    ,    1.248649    ,    1.206744    ,    1.12085    ,    1.007854    ,    0.734676    ,    0.682021    ,    0.676685    ,    0.661223    ,    0.650652    ,    0.641968    ,    0.639055    ,    0.638754    ,    0.641174    ,    0.646164    ,    0.653173    ,    0.659231    ,    0.668035    ,    0.675602    ,    0.694097    ,    0.725447    ,    0.772325    ,    0.823018    ,    0.85375    ,    1.080403    ,    1.34421    ,    1.377215    ,    1.346264    ,    1.325283    ,    1.293738    ,    1.276665    ,    1.115057    ,    0.842586    ,    0.820815    ,    0.875058    ,    1.067991    ,    0.984769    ,    0.771047    ,    0.752723    ,    0.748285    ,    0.744614    ,    0.737948    ,    0.734938    ,    0.729877    ,    0.723616    ,    0.7185    ,    0.711891    ,    0.706048    ,    0.702715    ,    0.704504    ,    0.701934    ,    0.697654    ,    0.695981    ,    0.698383    ,    0.703753    ,    0.713496    ,    0.72211    ,    0.73806    ,    0.75729    ,    0.793425    ,    0.931614    ,    1.08972    ,    1.14718    ,    1.183992    ,    1.196085    ,    1.206874    ,    1.199059    ,    1.163965    ,    1.119137    ,    1.089957    ,    1.03441    ,    0.99099    ,    0.840226    ,    0.741084    ,    0.752668    ,    0.767906    ,    0.769334    ,    0.769075    ,    0.768615    ,    0.765906    ,    0.760762    ,    0.754164    ,    0.750641    },
    {    0.759287    ,    0.752137    ,    0.746536    ,    0.742936    ,    0.737051    ,    0.734257    ,    0.728255    ,    0.72352    ,    0.718907    ,    0.712199    ,    0.702889    ,    0.693076    ,    0.68397    ,    0.673353    ,    0.663741    ,    0.656145    ,    0.645962    ,    0.632765    ,    0.621578    ,    0.609467    ,    0.592842    ,    0.580496    ,    0.568901    ,    0.557264    ,    0.548514    ,    0.542495    ,    0.539705    ,    0.537405    ,    0.537009    ,    0.539897    ,    0.546967    ,    0.562199    ,    0.583356    ,    0.613142    ,    0.635226    ,    0.681257    ,    0.742645    ,    0.797119    ,    0.906724    ,    1.18622    ,    1.282673    ,    1.263103    ,    1.352787    ,    1.357101    ,    1.353474    ,    1.283473    ,    1.253457    ,    1.249067    ,    1.199887    ,    1.065758    ,    0.827825    ,    0.693383    ,    0.689508    ,    0.675285    ,    0.666586    ,    0.661948    ,    0.663028    ,    0.664685    ,    0.669643    ,    0.675135    ,    0.684207    ,    0.693603    ,    0.705563    ,    0.723139    ,    0.749822    ,    0.785356    ,    0.829116    ,    0.860847    ,    0.906773    ,    1.221406    ,    1.380141    ,    1.343503    ,    1.355543    ,    1.320225    ,    1.316817    ,    1.223112    ,    1.137189    ,    0.954951    ,    0.829776    ,    0.835681    ,    0.997575    ,    0.975339    ,    0.777045    ,    0.746913    ,    0.747232    ,    0.744473    ,    0.739822    ,    0.738171    ,    0.737449    ,    0.733658    ,    0.729327    ,    0.725467    ,    0.723646    ,    0.721573    ,    0.721711    ,    0.720132    ,    0.722054    ,    0.725904    ,    0.732053    ,    0.739365    ,    0.748013    ,    0.758206    ,    0.771716    ,    0.784112    ,    0.795663    ,    0.82315    ,    0.850197    ,    0.899255    ,    1.106755    ,    1.094473    ,    1.1325    ,    1.08251    ,    1.033786    ,    1.045624    ,    1.022344    ,    0.99375    ,    0.867247    ,    0.757602    ,    0.758926    ,    0.771291    ,    0.77687    ,    0.773877    ,    0.771617    ,    0.771706    ,    0.771999    ,    0.768426    ,    0.766307    ,    0.768121    },
    {    0.764683    ,    0.760054    ,    0.756094    ,    0.752643    ,    0.747313    ,    0.743265    ,    0.736795    ,    0.72989    ,    0.72514    ,    0.719381    ,    0.711752    ,    0.704789    ,    0.698372    ,    0.689435    ,    0.682584    ,    0.674322    ,    0.663593    ,    0.653855    ,    0.6423    ,    0.633067    ,    0.624954    ,    0.616509    ,    0.611749    ,    0.608096    ,    0.607843    ,    0.611938    ,    0.615619    ,    0.620916    ,    0.629827    ,    0.645005    ,    0.663928    ,    0.684383    ,    0.710568    ,    0.733181    ,    0.761747    ,    0.804236    ,    0.852925    ,    0.985997    ,    1.184076    ,    1.231751    ,    1.2086    ,    1.311399    ,    1.404555    ,    1.370025    ,    1.319024    ,    1.309639    ,    1.232645    ,    1.217627    ,    1.196005    ,    1.099631    ,    0.882919    ,    0.70203    ,    0.700464    ,    0.694216    ,    0.689092    ,    0.687356    ,    0.691673    ,    0.69818    ,    0.70543    ,    0.715815    ,    0.727947    ,    0.741701    ,    0.760068    ,    0.779065    ,    0.79714    ,    0.818816    ,    0.839651    ,    0.84784    ,    0.896145    ,    1.165192    ,    1.310882    ,    1.326279    ,    1.333633    ,    1.251235    ,    1.253247    ,    1.225581    ,    1.165291    ,    1.108433    ,    0.965194    ,    0.832097    ,    0.870678    ,    0.955017    ,    0.806287    ,    0.753202    ,    0.755945    ,    0.755803    ,    0.755201    ,    0.755718    ,    0.757097    ,    0.753321    ,    0.751028    ,    0.748472    ,    0.746773    ,    0.748128    ,    0.74929    ,    0.751636    ,    0.754965    ,    0.761301    ,    0.770462    ,    0.777747    ,    0.787044    ,    0.799025    ,    0.807763    ,    0.81486    ,    0.823533    ,    0.839349    ,    0.851529    ,    0.855119    ,    0.893352    ,    1.001003    ,    1.010476    ,    1.048555    ,    1.008404    ,    0.8586    ,    0.863638    ,    0.917569    ,    0.75938    ,    0.759955    ,    0.774856    ,    0.776517    ,    0.777364    ,    0.776937    ,    0.778203    ,    0.778819    ,    0.778172    ,    0.775152    ,    0.773027    ,    0.770649    },
    {    0.763411    ,    0.760152    ,    0.756048    ,    0.752272    ,    0.748824    ,    0.742904    ,    0.737933    ,    0.733568    ,    0.729367    ,    0.724483    ,    0.719045    ,    0.714125    ,    0.708998    ,    0.705755    ,    0.699554    ,    0.695851    ,    0.692595    ,    0.685872    ,    0.682983    ,    0.68028    ,    0.677968    ,    0.675388    ,    0.673033    ,    0.674197    ,    0.67554    ,    0.680553    ,    0.686547    ,    0.69748    ,    0.710623    ,    0.728355    ,    0.751465    ,    0.774663    ,    0.806488    ,    0.826361    ,    0.85559    ,    0.880356    ,    0.92785    ,    1.15188    ,    1.179682    ,    1.225095    ,    1.343606    ,    1.362415    ,    1.323644    ,    1.348142    ,    1.276977    ,    1.291157    ,    1.252167    ,    1.165601    ,    1.196844    ,    1.09059    ,    0.949216    ,    0.745232    ,    0.726828    ,    0.732935    ,    0.728667    ,    0.729954    ,    0.736284    ,    0.745737    ,    0.75439    ,    0.765208    ,    0.774358    ,    0.787154    ,    0.798811    ,    0.808092    ,    0.816515    ,    0.825376    ,    0.833717    ,    0.836341    ,    0.82201    ,    1.077977    ,    1.21805    ,    1.253058    ,    1.264002    ,    1.226761    ,    1.195694    ,    1.183401    ,    1.158027    ,    1.111091    ,    0.971798    ,    0.817798    ,    0.80125    ,    0.821092    ,    0.797244    ,    0.776546    ,    0.77891    ,    0.77967    ,    0.782237    ,    0.782449    ,    0.782579    ,    0.779946    ,    0.779008    ,    0.778463    ,    0.77687    ,    0.776263    ,    0.77799    ,    0.781774    ,    0.784778    ,    0.790498    ,    0.795    ,    0.79994    ,    0.808847    ,    0.816505    ,    0.821498    ,    0.826014    ,    0.829393    ,    0.831893    ,    0.838081    ,    0.838489    ,    0.835839    ,    0.834192    ,    0.864113    ,    0.975474    ,    0.926328    ,    0.80723    ,    0.811941    ,    0.840864    ,    0.747201    ,    0.760616    ,    0.769907    ,    0.779109    ,    0.783005    ,    0.782333    ,    0.781908    ,    0.779545    ,    0.776594    ,    0.774449    ,    0.770599    ,    0.767576    },
    {    0.770101    ,    0.765812    ,    0.763178    ,    0.759161    ,    0.754348    ,    0.749635    ,    0.746241    ,    0.743185    ,    0.740315    ,    0.736887    ,    0.733403    ,    0.730208    ,    0.727573    ,    0.72826    ,    0.726858    ,    0.7278    ,    0.73122    ,    0.729481    ,    0.732099    ,    0.733022    ,    0.731562    ,    0.731701    ,    0.731396    ,    0.732859    ,    0.734535    ,    0.741873    ,    0.751292    ,    0.760864    ,    0.773313    ,    0.791259    ,    0.812496    ,    0.832561    ,    0.854769    ,    0.878749    ,    0.904196    ,    0.902947    ,    0.990233    ,    1.188472    ,    1.242541    ,    1.322345    ,    1.28025    ,    1.288467    ,    1.302234    ,    1.293639    ,    1.33302    ,    1.295918    ,    1.239299    ,    1.191203    ,    1.108936    ,    1.077431    ,    0.970114    ,    0.879808    ,    0.762528    ,    0.770079    ,    0.770472    ,    0.773122    ,    0.775112    ,    0.779433    ,    0.783861    ,    0.791941    ,    0.80012    ,    0.808029    ,    0.813124    ,    0.819244    ,    0.823126    ,    0.825529    ,    0.832459    ,    0.839067    ,    0.816296    ,    1.01069    ,    1.189032    ,    1.223195    ,    1.19447    ,    1.166234    ,    1.158676    ,    1.160039    ,    1.137034    ,    1.089549    ,    0.908395    ,    0.800255    ,    0.793002    ,    0.79438    ,    0.795449    ,    0.795891    ,    0.801403    ,    0.805626    ,    0.808271    ,    0.809362    ,    0.807365    ,    0.807275    ,    0.806828    ,    0.806011    ,    0.806439    ,    0.804181    ,    0.802041    ,    0.797571    ,    0.794536    ,    0.796416    ,    0.794843    ,    0.798962    ,    0.800114    ,    0.803459    ,    0.811988    ,    0.815319    ,    0.82262    ,    0.824337    ,    0.82621    ,    0.827238    ,    0.831047    ,    0.813458    ,    0.798824    ,    0.792087    ,    0.774586    ,    0.771613    ,    0.786108    ,    0.792031    ,    0.781522    ,    0.79197    ,    0.768058    ,    0.779193    ,    0.784028    ,    0.789081    ,    0.786119    ,    0.784996    ,    0.781551    ,    0.779418    ,    0.776234    ,    0.772163    },
    {    0.775138    ,    0.771971    ,    0.770106    ,    0.769356    ,    0.768002    ,    0.767389    ,    0.764896    ,    0.763445    ,    0.761517    ,    0.760119    ,    0.759585    ,    0.760712    ,    0.761181    ,    0.762715    ,    0.766864    ,    0.769546    ,    0.773605    ,    0.777046    ,    0.778939    ,    0.782402    ,    0.786511    ,    0.787888    ,    0.789876    ,    0.793437    ,    0.798782    ,    0.803259    ,    0.806055    ,    0.814929    ,    0.828229    ,    0.843068    ,    0.85405    ,    0.861664    ,    0.87648    ,    0.902664    ,    0.909317    ,    0.901329    ,    1.068226    ,    1.195797    ,    1.240181    ,    1.200239    ,    1.21309    ,    1.25243    ,    1.295575    ,    1.294061    ,    1.279116    ,    1.250719    ,    1.195323    ,    1.157633    ,    1.166458    ,    1.096699    ,    1.091241    ,    0.975139    ,    0.798654    ,    0.800308    ,    0.803012    ,    0.803368    ,    0.805147    ,    0.806904    ,    0.80976    ,    0.815011    ,    0.820271    ,    0.824897    ,    0.826424    ,    0.830006    ,    0.828698    ,    0.832938    ,    0.83822    ,    0.844256    ,    0.855561    ,    1.046739    ,    1.181762    ,    1.184153    ,    1.175997    ,    1.178005    ,    1.161561    ,    1.105326    ,    1.100924    ,    1.075518    ,    0.883606    ,    0.799893    ,    0.808283    ,    0.812814    ,    0.817132    ,    0.819973    ,    0.823577    ,    0.823945    ,    0.82424    ,    0.825906    ,    0.823526    ,    0.821603    ,    0.822178    ,    0.821202    ,    0.819062    ,    0.816041    ,    0.81303    ,    0.809693    ,    0.807939    ,    0.804102    ,    0.801431    ,    0.793124    ,    0.779817    ,    0.807297    ,    0.89196    ,    0.853813    ,    0.823326    ,    0.811037    ,    0.801593    ,    0.803735    ,    0.797049    ,    0.790761    ,    0.781298    ,    0.772182    ,    0.762782    ,    0.813527    ,    0.921559    ,    0.962446    ,    0.911123    ,    0.804095    ,    0.78406    ,    0.784835    ,    0.787884    ,    0.788318    ,    0.786967    ,    0.787336    ,    0.785107    ,    0.783352    ,    0.779919    ,    0.776545    },
    {    0.80117    ,    0.802254    ,    0.802997    ,    0.805265    ,    0.806182    ,    0.807243    ,    0.807644    ,    0.808965    ,    0.81129    ,    0.812394    ,    0.814173    ,    0.817047    ,    0.820232    ,    0.822348    ,    0.825268    ,    0.826975    ,    0.830068    ,    0.833143    ,    0.837255    ,    0.839058    ,    0.840835    ,    0.844565    ,    0.850895    ,    0.856089    ,    0.862049    ,    0.871215    ,    0.878575    ,    0.886022    ,    0.893273    ,    0.89814    ,    0.898861    ,    0.898074    ,    0.901742    ,    0.903995    ,    0.881187    ,    0.877515    ,    1.062034    ,    1.1774    ,    1.184321    ,    1.142731    ,    1.217542    ,    1.247042    ,    1.279635    ,    1.259221    ,    1.251589    ,    1.234586    ,    1.131438    ,    1.072304    ,    1.089358    ,    1.024727    ,    0.964522    ,    0.844433    ,    0.811356    ,    0.819362    ,    0.823679    ,    0.828217    ,    0.834629    ,    0.837478    ,    0.842233    ,    0.843719    ,    0.843554    ,    0.842368    ,    0.841919    ,    0.840445    ,    0.837867    ,    0.840768    ,    0.844739    ,    0.859702    ,    0.954847    ,    1.072554    ,    1.13884    ,    1.171425    ,    1.194505    ,    1.192595    ,    1.156835    ,    1.136693    ,    1.111717    ,    1.051656    ,    0.9334    ,    0.81365    ,    0.824506    ,    0.832885    ,    0.833529    ,    0.835273    ,    0.835424    ,    0.833473    ,    0.833129    ,    0.831139    ,    0.830249    ,    0.829977    ,    0.829297    ,    0.828034    ,    0.824152    ,    0.822932    ,    0.818862    ,    0.81572    ,    0.813823    ,    0.809788    ,    0.803191    ,    0.784612    ,    0.801145    ,    0.919111    ,    0.858033    ,    0.809068    ,    0.829177    ,    0.835665    ,    0.815217    ,    0.874161    ,    0.805903    ,    0.7986    ,    0.796183    ,    0.79913    ,    0.864172    ,    0.890074    ,    0.949938    ,    0.90703    ,    0.811502    ,    0.790909    ,    0.811759    ,    0.795    ,    0.793179    ,    0.793433    ,    0.795922    ,    0.799362    ,    0.800183    ,    0.801235    ,    0.800651    ,    0.800606    },
    {    0.831413    ,    0.834538    ,    0.838259    ,    0.840358    ,    0.841751    ,    0.844495    ,    0.846114    ,    0.846525    ,    0.848448    ,    0.848606    ,    0.847835    ,    0.849216    ,    0.849443    ,    0.850288    ,    0.852345    ,    0.854801    ,    0.8572    ,    0.857274    ,    0.860285    ,    0.861572    ,    0.865789    ,    0.87459    ,    0.886804    ,    0.899706    ,    0.908861    ,    0.922327    ,    0.933358    ,    0.938323    ,    0.942772    ,    0.944664    ,    0.942229    ,    0.930251    ,    0.90709    ,    0.881984    ,    0.848483    ,    0.84877    ,    1.022272    ,    1.145305    ,    1.11041    ,    1.102904    ,    1.159539    ,    1.216864    ,    1.245942    ,    1.277607    ,    1.248338    ,    1.209749    ,    1.088576    ,    0.975022    ,    0.855626    ,    0.833113    ,    0.824249    ,    0.822783    ,    0.828927    ,    0.837266    ,    0.842731    ,    0.848863    ,    0.855268    ,    0.860243    ,    0.867437    ,    0.867335    ,    0.865548    ,    0.860073    ,    0.855776    ,    0.852239    ,    0.851185    ,    0.852196    ,    0.856921    ,    0.88704    ,    1.008204    ,    1.06194    ,    1.121315    ,    1.134211    ,    1.136876    ,    1.148589    ,    1.168936    ,    1.099463    ,    1.051269    ,    1.072377    ,    1.017237    ,    0.883056    ,    0.827117    ,    0.845766    ,    0.852739    ,    0.852066    ,    0.852004    ,    0.845731    ,    0.840122    ,    0.835355    ,    0.829876    ,    0.82632    ,    0.821603    ,    0.820211    ,    0.81895    ,    0.816865    ,    0.813791    ,    0.811603    ,    0.810154    ,    0.805632    ,    0.793344    ,    0.792576    ,    0.925988    ,    0.934257    ,    0.83638    ,    0.887054    ,    0.971259    ,    0.961767    ,    0.869859    ,    0.88928    ,    0.827519    ,    0.810172    ,    0.81202    ,    0.874678    ,    0.857495    ,    0.825169    ,    0.789431    ,    0.786745    ,    0.786385    ,    0.792811    ,    0.801447    ,    0.804869    ,    0.808939    ,    0.811654    ,    0.815811    ,    0.818807    ,    0.823545    ,    0.826442    ,    0.828482    ,    0.830413    },
    {    0.824211    ,    0.826    ,    0.827682    ,    0.829434    ,    0.831144    ,    0.8333    ,    0.834816    ,    0.837054    ,    0.837989    ,    0.838463    ,    0.836448    ,    0.837276    ,    0.839047    ,    0.840488    ,    0.842514    ,    0.846055    ,    0.846545    ,    0.849882    ,    0.854417    ,    0.860466    ,    0.869779    ,    0.87912    ,    0.88836    ,    0.89981    ,    0.909461    ,    0.918137    ,    0.92095    ,    0.91976    ,    0.913924    ,    0.904278    ,    0.894956    ,    0.877938    ,    0.858891    ,    0.842571    ,    0.825461    ,    0.809027    ,    0.928725    ,    1.086956    ,    1.128897    ,    1.128239    ,    1.152762    ,    1.229354    ,    1.29584    ,    1.293322    ,    1.273271    ,    1.199485    ,    0.930675    ,    0.797789    ,    0.811665    ,    0.813609    ,    0.814814    ,    0.819709    ,    0.827627    ,    0.836477    ,    0.841141    ,    0.846728    ,    0.852073    ,    0.855016    ,    0.859411    ,    0.861612    ,    0.85807    ,    0.850332    ,    0.84401    ,    0.841927    ,    0.842927    ,    0.842028    ,    0.846639    ,    0.866395    ,    1.022564    ,    1.082292    ,    1.12099    ,    1.139737    ,    1.13002    ,    1.156642    ,    1.169104    ,    1.137352    ,    1.063565    ,    1.102473    ,    1.06778    ,    1.046871    ,    0.887606    ,    0.852741    ,    0.858763    ,    0.860969    ,    0.858914    ,    0.856709    ,    0.851533    ,    0.844507    ,    0.835343    ,    0.82858    ,    0.823674    ,    0.819561    ,    0.817925    ,    0.817344    ,    0.815614    ,    0.812537    ,    0.808968    ,    0.802967    ,    0.796848    ,    0.847597    ,    0.956903    ,    0.851856    ,    0.823662    ,    0.892222    ,    0.975499    ,    0.966261    ,    0.854356    ,    0.81584    ,    0.823246    ,    0.828887    ,    0.810786    ,    0.791824    ,    0.790736    ,    0.792289    ,    0.791035    ,    0.791894    ,    0.790918    ,    0.788244    ,    0.793032    ,    0.798423    ,    0.803573    ,    0.80677    ,    0.80923    ,    0.811145    ,    0.814022    ,    0.817379    ,    0.82058    ,    0.822334    },
    {    0.794941    ,    0.795677    ,    0.798911    ,    0.800029    ,    0.801039    ,    0.803777    ,    0.805369    ,    0.805445    ,    0.803987    ,    0.805158    ,    0.80625    ,    0.806869    ,    0.808259    ,    0.809203    ,    0.810423    ,    0.816216    ,    0.81954    ,    0.8246    ,    0.828683    ,    0.832785    ,    0.838774    ,    0.845991    ,    0.851768    ,    0.853366    ,    0.858523    ,    0.858025    ,    0.85541    ,    0.851681    ,    0.842188    ,    0.832021    ,    0.821464    ,    0.808784    ,    0.803182    ,    0.798871    ,    0.80233    ,    0.792643    ,    0.80141    ,    1.019368    ,    1.149325    ,    1.188592    ,    1.21583    ,    1.255643    ,    1.256711    ,    1.209789    ,    1.13176    ,    1.007862    ,    0.821178    ,    0.796398    ,    0.798527    ,    0.79846    ,    0.803314    ,    0.808052    ,    0.8145    ,    0.817281    ,    0.819477    ,    0.823612    ,    0.828342    ,    0.835692    ,    0.8365    ,    0.831399    ,    0.82685    ,    0.859193    ,    0.853416    ,    0.838911    ,    0.821586    ,    0.816649    ,    0.85491    ,    0.938892    ,    1.050814    ,    1.102122    ,    1.136758    ,    1.156949    ,    1.168637    ,    1.153792    ,    1.145924    ,    1.164425    ,    1.124044    ,    1.099034    ,    1.083811    ,    1.122902    ,    1.006614    ,    0.908318    ,    0.863028    ,    0.870236    ,    0.868363    ,    0.866174    ,    0.858934    ,    0.850977    ,    0.843943    ,    0.835061    ,    0.829711    ,    0.823351    ,    0.819995    ,    0.819495    ,    0.817455    ,    0.813412    ,    0.810108    ,    0.808621    ,    0.832256    ,    0.914689    ,    0.91967    ,    0.822976    ,    0.799702    ,    0.803538    ,    0.825191    ,    0.927184    ,    0.860515    ,    0.800465    ,    0.79488    ,    0.794367    ,    0.791313    ,    0.788663    ,    0.789905    ,    0.789886    ,    0.786852    ,    0.783706    ,    0.782634    ,    0.781638    ,    0.781384    ,    0.784823    ,    0.789007    ,    0.789309    ,    0.788239    ,    0.789887    ,    0.791592    ,    0.79249    ,    0.792836    ,    0.79258    },
    {    0.778269    ,    0.777543    ,    0.776763    ,    0.775168    ,    0.774625    ,    0.773454    ,    0.772934    ,    0.77107    ,    0.769688    ,    0.76823    ,    0.767101    ,    0.766931    ,    0.767988    ,    0.76877    ,    0.76838    ,    0.771756    ,    0.774635    ,    0.777814    ,    0.780874    ,    0.786081    ,    0.792489    ,    0.799275    ,    0.803762    ,    0.806823    ,    0.810061    ,    0.810912    ,    0.812192    ,    0.808867    ,    0.804491    ,    0.796973    ,    0.793977    ,    0.796041    ,    0.794722    ,    0.805662    ,    0.829248    ,    0.853618    ,    0.866643    ,    1.06016    ,    1.179057    ,    1.282301    ,    1.314003    ,    1.247923    ,    1.131815    ,    0.990522    ,    0.788466    ,    0.778405    ,    0.77459    ,    0.774062    ,    0.776028    ,    0.778499    ,    0.78552    ,    0.788298    ,    0.793734    ,    0.797877    ,    0.802859    ,    0.805837    ,    0.806443    ,    0.817441    ,    0.810976    ,    0.810354    ,    0.961187    ,    1.062043    ,    1.081775    ,    1.08538    ,    1.059959    ,    1.029632    ,    1.047457    ,    1.11596    ,    1.121316    ,    1.144326    ,    1.175151    ,    1.189403    ,    1.20535    ,    1.187409    ,    1.147761    ,    1.136873    ,    1.154952    ,    1.123504    ,    1.129853    ,    1.045714    ,    1.09242    ,    1.011521    ,    0.883731    ,    0.876601    ,    0.881788    ,    0.877623    ,    0.866425    ,    0.857584    ,    0.844819    ,    0.836021    ,    0.827138    ,    0.820931    ,    0.833676    ,    0.834001    ,    0.80894    ,    0.804552    ,    0.803791    ,    0.798307    ,    0.803196    ,    0.859166    ,    0.863838    ,    0.803952    ,    0.788403    ,    0.78754    ,    0.791513    ,    0.806859    ,    0.813365    ,    0.807002    ,    0.862838    ,    0.852405    ,    0.797318    ,    0.791631    ,    0.79225    ,    0.790815    ,    0.788311    ,    0.788658    ,    0.787927    ,    0.787349    ,    0.786261    ,    0.785973    ,    0.786506    ,    0.78581    ,    0.784242    ,    0.783421    ,    0.783134    ,    0.781307    ,    0.779065    ,    0.778566    },
    {    0.777817    ,    0.778211    ,    0.775207    ,    0.772162    ,    0.770516    ,    0.768679    ,    0.768304    ,    0.763806    ,    0.758409    ,    0.755979    ,    0.754279    ,    0.754136    ,    0.755305    ,    0.75789    ,    0.760454    ,    0.764103    ,    0.767969    ,    0.769499    ,    0.772939    ,    0.778124    ,    0.785245    ,    0.790475    ,    0.795715    ,    0.801834    ,    0.803058    ,    0.808089    ,    0.811422    ,    0.811382    ,    0.812719    ,    0.813388    ,    0.820252    ,    0.819054    ,    0.820772    ,    0.857011    ,    0.949142    ,    0.830159    ,    0.830034    ,    1.018539    ,    1.234228    ,    1.214075    ,    1.085905    ,    0.99935    ,    0.868021    ,    0.764966    ,    0.745459    ,    0.748504    ,    0.750529    ,    0.749966    ,    0.749477    ,    0.752062    ,    0.757858    ,    0.763415    ,    0.769901    ,    0.775415    ,    0.783378    ,    0.788138    ,    0.790669    ,    0.793641    ,    0.79625    ,    0.948475    ,    1.16159    ,    1.185248    ,    1.155041    ,    1.140236    ,    1.138609    ,    1.13689    ,    1.159477    ,    1.148535    ,    1.147323    ,    1.152667    ,    1.145576    ,    1.189808    ,    1.206214    ,    1.180916    ,    1.129364    ,    1.097351    ,    1.135354    ,    1.147852    ,    1.21839    ,    1.131322    ,    1.100272    ,    1.117382    ,    1.042838    ,    0.892197    ,    0.905076    ,    0.907785    ,    0.897238    ,    0.880607    ,    0.857794    ,    0.847062    ,    0.830512    ,    0.857512    ,    0.887887    ,    0.820801    ,    0.808222    ,    0.805243    ,    0.806703    ,    0.805891    ,    0.81515    ,    0.886431    ,    0.827719    ,    0.857737    ,    0.839887    ,    0.777373    ,    0.774642    ,    0.784035    ,    0.796879    ,    0.810273    ,    0.881278    ,    0.832196    ,    0.801266    ,    0.796863    ,    0.79447    ,    0.79434    ,    0.794533    ,    0.795258    ,    0.794812    ,    0.79409    ,    0.794264    ,    0.793469    ,    0.792099    ,    0.789805    ,    0.788086    ,    0.786135    ,    0.782891    ,    0.781034    ,    0.780232    ,    0.779368    },
    {    0.789964    ,    0.786851    ,    0.783058    ,    0.779075    ,    0.775846    ,    0.773748    ,    0.771048    ,    0.767323    ,    0.76363    ,    0.762553    ,    0.761057    ,    0.761659    ,    0.763323    ,    0.763899    ,    0.765886    ,    0.768979    ,    0.771747    ,    0.771584    ,    0.772719    ,    0.776021    ,    0.781237    ,    0.787937    ,    0.791257    ,    0.793223    ,    0.798316    ,    0.80491    ,    0.813548    ,    0.824736    ,    0.830457    ,    0.84322    ,    0.847725    ,    0.863031    ,    0.904531    ,    1.028868    ,    0.940883    ,    0.72304    ,    0.712283    ,    0.708783    ,    0.721478    ,    0.729891    ,    0.721928    ,    0.719217    ,    0.723588    ,    0.722419    ,    0.718439    ,    0.711994    ,    0.707397    ,    0.706983    ,    0.711315    ,    0.717952    ,    0.723856    ,    0.730638    ,    0.736269    ,    0.742161    ,    0.750923    ,    0.759113    ,    0.765705    ,    0.758941    ,    0.852775    ,    1.144317    ,    1.234314    ,    1.22855    ,    1.227334    ,    1.202195    ,    1.179143    ,    1.188456    ,    1.181332    ,    1.204752    ,    1.215701    ,    1.168616    ,    1.164225    ,    1.223682    ,    1.248395    ,    1.232749    ,    1.247146    ,    1.213172    ,    1.178478    ,    1.159699    ,    1.201474    ,    1.154636    ,    1.020939    ,    0.976233    ,    0.938798    ,    0.897398    ,    0.909411    ,    0.918803    ,    0.911472    ,    0.895876    ,    0.874274    ,    0.858928    ,    0.842522    ,    0.914797    ,    0.939653    ,    0.820648    ,    0.803789    ,    0.805532    ,    0.807251    ,    0.812335    ,    0.819475    ,    0.936367    ,    0.899441    ,    0.985851    ,    0.942517    ,    0.823277    ,    0.770041    ,    0.783618    ,    0.798578    ,    0.813627    ,    0.862932    ,    0.80798    ,    0.799517    ,    0.801491    ,    0.800973    ,    0.802766    ,    0.805212    ,    0.806255    ,    0.808784    ,    0.807564    ,    0.805694    ,    0.804019    ,    0.803319    ,    0.802197    ,    0.801373    ,    0.800924    ,    0.797862    ,    0.796268    ,    0.794816    ,    0.792783    },
    {    0.796797    ,    0.793544    ,    0.789504    ,    0.786205    ,    0.78297    ,    0.78241    ,    0.781439    ,    0.781158    ,    0.781139    ,    0.781602    ,    0.782331    ,    0.782004    ,    0.781276    ,    0.780432    ,    0.779471    ,    0.776658    ,    0.772975    ,    0.767225    ,    0.761024    ,    0.756065    ,    0.75427    ,    0.755073    ,    0.75953    ,    0.763154    ,    0.773404    ,    0.788785    ,    0.811641    ,    0.834885    ,    0.863491    ,    0.966251    ,    1.013116    ,    1.088441    ,    1.147024    ,    0.9949    ,    0.829292    ,    0.735146    ,    0.728582    ,    0.718124    ,    0.715508    ,    0.714149    ,    0.710476    ,    0.70932    ,    0.708217    ,    0.701612    ,    0.692221    ,    0.680446    ,    0.671901    ,    0.667896    ,    0.66585    ,    0.668081    ,    0.675202    ,    0.681484    ,    0.688287    ,    0.697449    ,    0.712876    ,    0.730788    ,    0.740093    ,    0.732025    ,    0.861813    ,    1.138886    ,    1.270881    ,    1.340874    ,    1.313655    ,    1.289906    ,    1.277608    ,    1.27847    ,    1.219322    ,    1.233891    ,    1.283374    ,    1.25775    ,    1.24746    ,    1.241778    ,    1.19553    ,    1.303555    ,    1.290595    ,    1.295143    ,    1.284951    ,    1.293823    ,    1.148807    ,    1.094399    ,    1.294426    ,    1.208649    ,    1.044231    ,    0.925255    ,    0.900647    ,    0.91532    ,    0.91489    ,    0.899807    ,    0.883641    ,    0.87275    ,    0.888408    ,    0.997961    ,    0.952536    ,    0.831134    ,    0.800233    ,    0.806976    ,    0.819921    ,    0.823333    ,    0.852243    ,    0.990873    ,    1.018385    ,    0.979914    ,    0.932972    ,    0.801347    ,    0.773763    ,    0.780909    ,    0.792632    ,    0.861878    ,    0.816483    ,    0.798139    ,    0.799122    ,    0.800438    ,    0.804156    ,    0.80649    ,    0.809413    ,    0.809641    ,    0.809826    ,    0.809657    ,    0.809876    ,    0.809628    ,    0.808969    ,    0.809737    ,    0.809377    ,    0.808689    ,    0.80692    ,    0.805284    ,    0.802619    ,    0.799474    },
    {    0.803703    ,    0.803173    ,    0.801081    ,    0.80045    ,    0.798638    ,    0.798921    ,    0.801584    ,    0.801651    ,    0.799951    ,    0.797592    ,    0.79664    ,    0.795945    ,    0.796034    ,    0.795134    ,    0.790562    ,    0.783266    ,    0.773847    ,    0.759504    ,    0.740539    ,    0.722103    ,    0.707983    ,    0.701827    ,    0.703504    ,    0.707621    ,    0.73318    ,    0.760281    ,    0.80943    ,    0.93329    ,    1.082581    ,    1.153745    ,    1.097799    ,    1.102512    ,    1.088507    ,    0.808362    ,    0.761094    ,    0.770075    ,    0.779456    ,    0.792064    ,    0.823337    ,    0.836381    ,    0.754051    ,    0.731189    ,    0.720322    ,    0.709007    ,    0.696673    ,    0.685364    ,    0.674349    ,    0.662086    ,    0.65222    ,    0.645205    ,    0.63892    ,    0.633591    ,    0.632012    ,    0.635813    ,    0.652666    ,    0.675948    ,    0.692376    ,    0.703622    ,    0.845431    ,    1.170133    ,    1.300855    ,    1.333231    ,    1.358177    ,    1.347319    ,    1.341668    ,    1.312846    ,    1.275802    ,    1.270376    ,    1.310404    ,    1.325312    ,    1.271425    ,    1.235933    ,    1.212076    ,    1.271202    ,    1.299059    ,    1.296672    ,    1.3142    ,    1.239717    ,    1.006386    ,    1.185368    ,    1.36252    ,    1.387399    ,    1.262375    ,    1.118246    ,    0.937299    ,    0.881839    ,    0.899417    ,    0.897742    ,    0.896257    ,    0.900222    ,    0.960092    ,    1.038391    ,    1.033811    ,    0.996975    ,    0.858885    ,    0.812578    ,    0.828733    ,    0.863916    ,    0.990111    ,    1.025413    ,    1.001577    ,    0.971746    ,    0.862566    ,    0.801397    ,    0.779767    ,    0.780163    ,    0.784782    ,    0.824335    ,    0.80529    ,    0.787578    ,    0.790483    ,    0.793727    ,    0.795721    ,    0.795844    ,    0.798159    ,    0.800662    ,    0.801131    ,    0.801915    ,    0.800913    ,    0.802523    ,    0.804056    ,    0.80355    ,    0.805102    ,    0.806164    ,    0.805813    ,    0.805434    ,    0.805151    ,    0.804224    },
    {    0.815223    ,    0.81532    ,    0.814209    ,    0.814176    ,    0.812871    ,    0.811588    ,    0.811916    ,    0.810797    ,    0.808646    ,    0.805076    ,    0.796785    ,    0.792573    ,    0.789796    ,    0.786654    ,    0.781953    ,    0.776397    ,    0.768884    ,    0.757189    ,    0.742885    ,    0.724981    ,    0.702799    ,    0.67859    ,    0.651148    ,    0.645731    ,    0.685094    ,    0.729898    ,    0.851797    ,    1.10947    ,    1.156037    ,    1.062949    ,    0.850536    ,    0.787631    ,    0.861187    ,    0.817614    ,    0.802005    ,    0.835004    ,    0.876583    ,    0.844868    ,    0.809018    ,    0.792308    ,    0.772368    ,    0.759293    ,    0.748194    ,    0.742082    ,    0.734444    ,    0.72633    ,    0.714528    ,    0.70098    ,    0.689341    ,    0.672406    ,    0.655541    ,    0.637282    ,    0.619509    ,    0.605048    ,    0.599737    ,    0.607991    ,    0.623519    ,    0.664606    ,    0.83992    ,    1.202539    ,    1.34108    ,    1.370015    ,    1.389348    ,    1.393784    ,    1.379005    ,    1.329865    ,    1.327041    ,    1.294693    ,    1.297447    ,    1.288709    ,    1.249618    ,    1.243117    ,    1.261734    ,    1.280715    ,    1.280885    ,    1.311032    ,    1.321671    ,    1.176372    ,    1.118023    ,    1.327669    ,    1.360758    ,    1.348651    ,    1.333571    ,    1.287871    ,    1.200583    ,    0.955542    ,    0.882774    ,    0.895676    ,    0.918181    ,    0.97148    ,    1.048633    ,    1.107708    ,    1.096851    ,    1.101119    ,    1.081766    ,    0.917239    ,    0.875298    ,    0.976098    ,    1.040544    ,    1.086961    ,    1.105931    ,    1.077331    ,    0.988802    ,    0.936855    ,    0.850648    ,    0.815455    ,    0.78984    ,    0.777544    ,    0.778806    ,    0.779274    ,    0.785438    ,    0.792294    ,    0.791179    ,    0.791079    ,    0.792075    ,    0.795723    ,    0.797566    ,    0.799014    ,    0.799109    ,    0.797476    ,    0.796233    ,    0.796155    ,    0.797715    ,    0.800412    ,    0.802719    ,    0.806391    ,    0.809667    ,    0.81343    },
    {    0.836455    ,    0.837737    ,    0.8362    ,    0.831405    ,    0.827718    ,    0.824814    ,    0.820141    ,    0.814959    ,    0.809255    ,    0.806403    ,    0.803613    ,    0.800169    ,    0.796504    ,    0.790169    ,    0.784803    ,    0.781334    ,    0.77687    ,    0.771676    ,    0.759597    ,    0.742067    ,    0.719698    ,    0.680858    ,    0.631792    ,    0.624737    ,    0.682714    ,    0.830237    ,    1.032436    ,    1.163301    ,    1.14298    ,    1.047222    ,    0.822627    ,    0.777763    ,    0.770442    ,    0.778594    ,    0.793566    ,    0.798612    ,    0.783744    ,    0.766368    ,    0.768348    ,    0.772837    ,    0.782416    ,    0.787046    ,    0.786017    ,    0.783592    ,    0.77717    ,    0.770975    ,    0.763835    ,    0.755775    ,    0.744658    ,    0.732608    ,    0.719095    ,    0.696286    ,    0.671099    ,    0.647753    ,    0.620224    ,    0.597727    ,    0.580997    ,    0.597152    ,    0.717627    ,    1.068203    ,    1.288199    ,    1.356106    ,    1.372626    ,    1.3793    ,    1.376689    ,    1.347173    ,    1.370982    ,    1.306315    ,    1.274068    ,    1.223391    ,    1.203286    ,    1.212549    ,    1.184795    ,    1.205655    ,    1.234705    ,    1.285613    ,    1.283375    ,    1.107406    ,    1.331357    ,    1.400493    ,    1.415742    ,    1.406444    ,    1.34416    ,    1.275773    ,    1.269153    ,    1.008134    ,    0.929797    ,    0.9421    ,    1.004569    ,    1.141723    ,    1.116634    ,    1.173005    ,    1.153589    ,    1.142956    ,    1.150511    ,    1.112069    ,    1.094816    ,    1.096565    ,    1.120043    ,    1.16101    ,    1.146615    ,    1.131417    ,    1.117727    ,    1.130389    ,    1.106855    ,    1.044735    ,    0.891684    ,    0.833272    ,    0.783928    ,    0.772913    ,    0.781077    ,    0.79128    ,    0.789398    ,    0.78495    ,    0.787758    ,    0.793016    ,    0.798261    ,    0.800679    ,    0.805437    ,    0.805709    ,    0.806717    ,    0.810942    ,    0.813896    ,    0.817379    ,    0.821423    ,    0.825094    ,    0.82843    ,    0.832873    },
    {    0.859704    ,    0.863217    ,    0.864012    ,    0.862429    ,    0.855907    ,    0.849476    ,    0.842759    ,    0.835699    ,    0.827055    ,    0.820012    ,    0.813982    ,    0.814116    ,    0.81618    ,    0.815247    ,    0.810945    ,    0.801076    ,    0.78962    ,    0.786274    ,    0.784631    ,    0.774037    ,    0.749583    ,    0.703428    ,    0.648034    ,    0.663855    ,    0.894078    ,    1.081251    ,    1.213889    ,    1.216125    ,    1.151345    ,    1.085304    ,    0.867137    ,    0.812973    ,    0.80488    ,    0.813892    ,    0.847568    ,    0.874133    ,    0.793607    ,    0.772473    ,    0.784257    ,    0.787291    ,    0.794062    ,    0.804753    ,    0.806769    ,    0.803607    ,    0.802741    ,    0.805028    ,    0.804508    ,    0.801412    ,    0.794052    ,    0.79291    ,    0.78737    ,    0.769796    ,    0.7538    ,    0.733344    ,    0.702473    ,    0.661318    ,    0.619293    ,    0.583975    ,    0.592003    ,    0.757887    ,    1.104024    ,    1.298938    ,    1.361698    ,    1.356288    ,    1.359913    ,    1.372019    ,    1.346629    ,    1.301183    ,    1.230327    ,    1.194209    ,    1.16591    ,    1.14874    ,    1.17882    ,    1.17838    ,    1.191213    ,    1.24252    ,    1.234669    ,    1.295733    ,    1.408564    ,    1.407568    ,    1.412696    ,    1.394884    ,    1.234046    ,    1.212633    ,    1.281619    ,    1.301667    ,    1.277957    ,    1.268881    ,    1.323764    ,    1.286041    ,    1.275357    ,    1.227173    ,    1.198364    ,    1.220354    ,    1.14477    ,    1.145712    ,    1.139688    ,    1.131745    ,    1.132137    ,    1.147899    ,    1.154028    ,    1.116049    ,    1.155364    ,    1.186738    ,    1.20028    ,    1.15224    ,    1.101964    ,    0.961517    ,    0.856973    ,    0.808673    ,    0.789958    ,    0.803733    ,    0.803093    ,    0.794397    ,    0.792323    ,    0.794079    ,    0.799886    ,    0.80351    ,    0.806958    ,    0.812194    ,    0.817922    ,    0.824988    ,    0.827316    ,    0.8336    ,    0.838984    ,    0.843775    ,    0.849871    ,    0.8543    },
    {    0.867322    ,    0.870271    ,    0.873403    ,    0.877297    ,    0.877493    ,    0.874817    ,    0.870009    ,    0.862902    ,    0.854409    ,    0.84907    ,    0.840559    ,    0.837422    ,    0.839177    ,    0.841792    ,    0.839714    ,    0.82871    ,    0.81076    ,    0.796105    ,    0.793641    ,    0.790323    ,    0.764112    ,    0.715193    ,    0.683807    ,    0.804279    ,    1.08143    ,    1.197341    ,    1.234231    ,    1.256651    ,    1.214138    ,    1.157312    ,    1.037961    ,    0.928035    ,    0.914683    ,    0.912046    ,    0.945134    ,    0.944918    ,    0.823106    ,    0.792047    ,    0.790836    ,    0.791465    ,    0.792924    ,    0.79307    ,    0.792369    ,    0.793326    ,    0.799495    ,    0.807286    ,    0.817694    ,    0.825394    ,    0.821643    ,    0.815555    ,    0.807062    ,    0.799152    ,    0.793643    ,    0.781108    ,    0.767068    ,    0.740632    ,    0.700813    ,    0.653321    ,    0.603864    ,    0.596785    ,    0.750772    ,    1.217038    ,    1.327619    ,    1.2886    ,    1.302916    ,    1.305441    ,    1.272414    ,    1.262028    ,    1.22598    ,    1.169564    ,    1.110078    ,    1.087829    ,    1.120529    ,    1.114793    ,    1.112791    ,    1.14651    ,    1.189094    ,    1.305191    ,    1.358681    ,    1.374838    ,    1.374071    ,    1.318036    ,    1.338093    ,    1.420727    ,    1.434796    ,    1.427934    ,    1.413728    ,    1.40307    ,    1.464364    ,    1.404397    ,    1.34157    ,    1.317384    ,    1.23179    ,    1.37821    ,    1.586746    ,    1.606402    ,    1.553763    ,    1.513317    ,    1.412658    ,    1.405962    ,    1.246882    ,    1.167975    ,    1.187189    ,    1.237647    ,    1.234799    ,    1.2097    ,    1.166255    ,    1.112877    ,    0.962957    ,    0.895344    ,    0.848604    ,    0.832245    ,    0.819976    ,    0.814576    ,    0.815908    ,    0.813074    ,    0.813379    ,    0.815973    ,    0.818538    ,    0.82188    ,    0.824311    ,    0.828584    ,    0.829994    ,    0.831153    ,    0.837503    ,    0.846183    ,    0.855279    ,    0.862017    },
    {    0.881326    ,    0.883787    ,    0.884048    ,    0.883462    ,    0.884233    ,    0.883241    ,    0.880172    ,    0.876618    ,    0.871223    ,    0.865281    ,    0.857055    ,    0.848631    ,    0.844779    ,    0.842864    ,    0.838543    ,    0.83149    ,    0.82223    ,    0.804799    ,    0.795159    ,    0.785784    ,    0.751113    ,    0.742437    ,    0.79228    ,    1.046856    ,    1.115057    ,    1.23288    ,    1.308913    ,    1.221591    ,    1.231641    ,    1.206052    ,    1.182116    ,    1.196056    ,    1.207813    ,    1.121664    ,    1.109908    ,    1.048498    ,    0.917517    ,    0.852817    ,    0.815346    ,    0.800592    ,    0.798292    ,    0.795481    ,    0.790478    ,    0.785354    ,    0.783636    ,    0.791859    ,    0.804372    ,    0.813333    ,    0.818914    ,    0.821546    ,    0.819564    ,    0.81494    ,    0.802921    ,    0.79362    ,    0.785133    ,    0.775011    ,    0.757261    ,    0.714195    ,    0.66175    ,    0.601781    ,    0.624025    ,    1.047037    ,    1.336274    ,    1.3256    ,    1.284493    ,    1.270824    ,    1.231017    ,    1.188215    ,    1.138638    ,    1.035268    ,    0.918653    ,    0.931247    ,    1.004524    ,    0.935938    ,    0.894687    ,    0.906771    ,    0.964041    ,    1.191794    ,    1.299293    ,    1.341731    ,    1.350344    ,    1.41026    ,    1.451916    ,    1.442621    ,    1.393266    ,    1.422216    ,    1.44645    ,    1.466772    ,    1.496452    ,    1.422922    ,    1.440724    ,    1.400771    ,    1.519335    ,    1.607302    ,    1.512407    ,    1.46065    ,    1.433818    ,    1.498461    ,    1.490443    ,    1.409191    ,    1.383376    ,    1.248122    ,    1.211147    ,    1.242281    ,    1.255563    ,    1.237785    ,    1.207248    ,    1.116841    ,    1.014811    ,    0.976857    ,    0.980857    ,    0.963443    ,    0.907805    ,    0.875369    ,    0.857171    ,    0.849754    ,    0.846412    ,    0.847821    ,    0.851022    ,    0.853573    ,    0.854666    ,    0.857278    ,    0.856898    ,    0.85316    ,    0.85405    ,    0.858369    ,    0.866509    ,    0.875983    },
    {    0.912892    ,    0.912989    ,    0.911177    ,    0.905902    ,    0.899402    ,    0.891681    ,    0.887114    ,    0.884952    ,    0.880334    ,    0.873719    ,    0.866747    ,    0.859988    ,    0.852656    ,    0.849742    ,    0.845598    ,    0.84119    ,    0.83377    ,    0.814764    ,    0.796827    ,    0.779125    ,    0.767227    ,    0.877855    ,    1.090562    ,    1.137001    ,    1.293286    ,    1.309259    ,    1.302183    ,    1.244294    ,    1.271421    ,    1.260776    ,    1.27458    ,    1.276854    ,    1.252015    ,    1.226643    ,    1.209377    ,    1.212129    ,    1.189412    ,    1.032414    ,    0.925229    ,    0.89339    ,    0.883105    ,    0.8632    ,    0.849847    ,    0.839131    ,    0.830537    ,    0.823981    ,    0.819033    ,    0.817258    ,    0.817957    ,    0.816375    ,    0.809997    ,    0.798446    ,    0.784997    ,    0.782283    ,    0.774828    ,    0.774134    ,    0.775127    ,    0.732673    ,    0.688173    ,    0.632548    ,    0.577803    ,    0.736103    ,    1.09189    ,    1.189348    ,    1.239685    ,    1.252982    ,    1.247425    ,    1.172527    ,    0.979617    ,    0.923465    ,    0.908708    ,    0.90822    ,    0.927513    ,    0.935126    ,    0.943288    ,    0.954109    ,    0.941726    ,    1.121094    ,    1.257961    ,    1.312101    ,    1.381693    ,    1.452394    ,    1.376494    ,    1.323396    ,    1.38163    ,    1.377482    ,    1.390853    ,    1.496795    ,    1.542732    ,    1.561482    ,    1.536006    ,    1.530318    ,    1.569799    ,    1.563388    ,    1.494876    ,    1.461527    ,    1.431425    ,    1.420857    ,    1.417737    ,    1.408071    ,    1.376909    ,    1.325315    ,    1.267347    ,    1.26717    ,    1.274703    ,    1.239759    ,    1.217542    ,    1.11307    ,    1.071109    ,    1.090404    ,    1.06314    ,    1.089609    ,    1.098676    ,    1.050034    ,    0.961984    ,    0.928092    ,    0.920381    ,    0.919383    ,    0.917065    ,    0.917595    ,    0.918839    ,    0.92244    ,    0.922383    ,    0.917832    ,    0.913188    ,    0.911476    ,    0.911144    ,    0.913467    },
    {    0.959073    ,    0.952452    ,    0.945738    ,    0.937844    ,    0.928081    ,    0.916621    ,    0.908597    ,    0.901459    ,    0.893939    ,    0.889525    ,    0.88699    ,    0.882304    ,    0.873459    ,    0.866153    ,    0.859455    ,    0.850319    ,    0.840161    ,    0.823425    ,    0.798228    ,    0.798682    ,    0.865738    ,    1.109549    ,    1.306187    ,    1.351818    ,    1.394313    ,    1.424567    ,    1.357565    ,    1.311621    ,    1.317828    ,    1.346721    ,    1.349934    ,    1.327911    ,    1.328861    ,    1.310942    ,    1.295249    ,    1.314001    ,    1.331255    ,    1.240748    ,    1.103602    ,    1.074229    ,    1.070266    ,    1.033156    ,    1.000388    ,    0.978195    ,    0.960822    ,    0.95113    ,    0.944735    ,    0.936328    ,    0.903709    ,    0.859328    ,    0.831623    ,    0.796712    ,    0.757954    ,    0.747381    ,    0.740479    ,    0.742927    ,    0.759808    ,    0.733736    ,    0.683502    ,    0.631424    ,    0.603719    ,    0.843295    ,    1.113401    ,    1.117752    ,    0.900548    ,    0.921526    ,    0.973127    ,    0.975527    ,    0.939906    ,    0.958396    ,    0.945636    ,    0.967863    ,    1.108674    ,    1.044285    ,    1.186819    ,    1.255917    ,    1.249015    ,    1.289148    ,    1.325921    ,    1.399834    ,    1.425601    ,    1.342468    ,    1.182541    ,    1.209453    ,    1.317779    ,    1.343163    ,    1.290882    ,    1.294953    ,    1.389672    ,    1.49693    ,    1.507632    ,    1.450226    ,    1.420845    ,    1.39772    ,    1.385287    ,    1.368504    ,    1.384621    ,    1.353722    ,    1.356521    ,    1.377112    ,    1.367249    ,    1.378976    ,    1.358658    ,    1.326929    ,    1.304617    ,    1.259071    ,    1.253585    ,    1.202201    ,    1.167504    ,    1.215151    ,    1.156653    ,    1.102277    ,    1.07617    ,    1.104631    ,    1.101558    ,    1.02037    ,    1.015113    ,    1.01571    ,    1.010916    ,    1.006038    ,    0.998557    ,    0.998722    ,    0.998079    ,    0.995266    ,    0.987615    ,    0.980331    ,    0.973162    ,    0.967026    },
    {    1.004015    ,    0.994762    ,    0.984493    ,    0.973198    ,    0.960789    ,    0.946494    ,    0.934353    ,    0.924198    ,    0.915034    ,    0.910134    ,    0.909513    ,    0.909188    ,    0.90308    ,    0.895032    ,    0.887528    ,    0.875401    ,    0.863035    ,    0.846502    ,    0.820034    ,    0.845157    ,    1.04271    ,    1.209405    ,    1.314942    ,    1.400699    ,    1.470694    ,    1.486603    ,    1.358778    ,    1.29886    ,    1.338032    ,    1.409371    ,    1.446188    ,    1.442377    ,    1.439958    ,    1.429839    ,    1.419641    ,    1.422632    ,    1.445718    ,    1.428385    ,    1.328152    ,    1.270704    ,    1.244189    ,    1.19393    ,    1.157804    ,    1.126692    ,    1.124797    ,    1.143278    ,    1.163888    ,    1.164793    ,    1.09054    ,    0.984438    ,    0.909278    ,    0.840209    ,    0.752802    ,    0.707763    ,    0.695643    ,    0.689575    ,    0.711352    ,    0.705768    ,    0.668458    ,    0.60723    ,    0.588589    ,    0.921632    ,    1.166447    ,    1.250056    ,    1.084395    ,    0.907334    ,    0.899597    ,    0.958651    ,    0.970873    ,    1.049973    ,    1.119007    ,    1.185828    ,    1.241215    ,    1.159458    ,    1.146937    ,    1.172366    ,    1.19914    ,    1.160328    ,    1.194415    ,    1.299907    ,    1.319191    ,    1.252673    ,    1.142077    ,    1.223854    ,    1.335416    ,    1.340822    ,    1.330833    ,    1.330445    ,    1.305988    ,    1.437891    ,    1.432881    ,    1.412333    ,    1.400597    ,    1.403575    ,    1.40843    ,    1.423191    ,    1.448044    ,    1.44685    ,    1.442766    ,    1.452212    ,    1.459285    ,    1.474733    ,    1.463368    ,    1.397623    ,    1.371696    ,    1.344847    ,    1.343821    ,    1.323272    ,    1.352178    ,    1.344019    ,    1.26506    ,    1.186372    ,    1.157925    ,    1.1315    ,    1.143198    ,    1.142606    ,    1.130053    ,    1.10008    ,    1.086004    ,    1.076999    ,    1.060648    ,    1.050584    ,    1.044667    ,    1.042277    ,    1.036589    ,    1.028753    ,    1.021035    ,    1.012914    },
    {    1.016238    ,    1.008436    ,    0.99974    ,    0.988542    ,    0.976631    ,    0.965267    ,    0.956424    ,    0.947018    ,    0.937477    ,    0.934193    ,    0.935574    ,    0.937717    ,    0.935369    ,    0.930955    ,    0.924583    ,    0.911378    ,    0.898157    ,    0.879106    ,    0.853659    ,    0.884785    ,    1.080993    ,    1.258257    ,    1.337038    ,    1.433248    ,    1.503211    ,    1.39643    ,    1.374008    ,    1.332814    ,    1.403543    ,    1.489409    ,    1.537295    ,    1.556479    ,    1.547047    ,    1.521792    ,    1.540091    ,    1.532517    ,    1.542283    ,    1.567982    ,    1.552632    ,    1.471029    ,    1.37598    ,    1.35031    ,    1.318212    ,    1.272889    ,    1.242116    ,    1.238712    ,    1.276102    ,    1.334121    ,    1.235716    ,    1.096123    ,    0.961412    ,    0.851927    ,    0.728822    ,    0.646076    ,    0.625121    ,    0.618371    ,    0.618791    ,    0.634848    ,    0.628134    ,    0.589411    ,    0.570094    ,    0.751149    ,    0.881464    ,    0.965498    ,    1.084663    ,    1.097397    ,    1.048728    ,    1.048796    ,    1.146096    ,    1.167494    ,    1.19434    ,    1.264386    ,    1.283327    ,    1.28156    ,    1.163474    ,    1.053745    ,    1.038949    ,    1.028416    ,    1.065979    ,    1.17729    ,    1.233629    ,    1.181296    ,    1.189813    ,    1.324061    ,    1.39656    ,    1.398771    ,    1.413179    ,    1.383921    ,    1.387503    ,    1.354368    ,    1.377488    ,    1.396253    ,    1.38581    ,    1.418304    ,    1.42395    ,    1.493458    ,    1.552384    ,    1.54204    ,    1.516925    ,    1.510358    ,    1.505748    ,    1.529025    ,    1.50967    ,    1.451548    ,    1.450525    ,    1.428025    ,    1.390537    ,    1.400975    ,    1.442481    ,    1.446791    ,    1.423004    ,    1.370095    ,    1.252524    ,    1.207737    ,    1.255291    ,    1.301755    ,    1.252576    ,    1.1632    ,    1.1265    ,    1.108422    ,    1.092961    ,    1.08046    ,    1.072039    ,    1.058914    ,    1.04909    ,    1.036833    ,    1.027988    ,    1.022097    },
    {    1.014111    ,    1.008008    ,    1.001807    ,    0.994128    ,    0.985558    ,    0.978651    ,    0.976064    ,    0.971453    ,    0.964982    ,    0.96402    ,    0.967218    ,    0.971985    ,    0.97577    ,    0.975561    ,    0.968051    ,    0.950602    ,    0.93314    ,    0.91485    ,    0.896834    ,    0.925239    ,    1.082523    ,    1.25365    ,    1.321201    ,    1.388135    ,    1.398572    ,    1.391235    ,    1.418015    ,    1.445504    ,    1.504222    ,    1.584848    ,    1.627546    ,    1.628723    ,    1.631944    ,    1.657817    ,    1.681229    ,    1.703788    ,    1.711181    ,    1.710048    ,    1.684017    ,    1.652502    ,    1.60903    ,    1.553473    ,    1.490095    ,    1.412775    ,    1.310237    ,    1.208333    ,    1.144449    ,    1.148872    ,    1.120304    ,    1.00345    ,    0.850563    ,    0.726829    ,    0.616258    ,    0.540336    ,    0.512471    ,    0.49954    ,    0.490601    ,    0.507906    ,    0.535471    ,    0.546466    ,    0.556009    ,    0.587958    ,    0.643183    ,    0.778016    ,    0.993807    ,    1.07664    ,    1.125091    ,    1.164098    ,    1.203381    ,    1.228956    ,    1.253297    ,    1.264381    ,    1.280346    ,    1.308009    ,    1.295904    ,    1.238716    ,    1.245263    ,    1.234185    ,    1.265487    ,    1.284649    ,    1.310549    ,    1.262458    ,    1.257648    ,    1.366937    ,    1.457177    ,    1.491753    ,    1.521798    ,    1.524647    ,    1.502513    ,    1.496752    ,    1.495361    ,    1.482108    ,    1.473412    ,    1.490297    ,    1.567794    ,    1.59371    ,    1.550876    ,    1.545423    ,    1.523064    ,    1.507081    ,    1.464039    ,    1.469991    ,    1.452781    ,    1.430411    ,    1.446341    ,    1.453634    ,    1.449119    ,    1.440022    ,    1.498613    ,    1.546024    ,    1.516992    ,    1.516703    ,    1.474532    ,    1.37385    ,    1.384842    ,    1.426375    ,    1.371593    ,    1.300556    ,    1.223281    ,    1.164316    ,    1.126188    ,    1.099045    ,    1.080521    ,    1.066831    ,    1.052657    ,    1.035941    ,    1.022793    ,    1.017688    },
    {    1.030225    ,    1.028295    ,    1.026975    ,    1.025254    ,    1.022348    ,    1.019739    ,    1.015573    ,    1.011183    ,    1.007544    ,    1.008347    ,    1.010979    ,    1.015357    ,    1.016886    ,    1.014402    ,    1.004133    ,    0.984229    ,    0.963728    ,    0.95006    ,    0.953762    ,    1.041856    ,    1.225564    ,    1.354103    ,    1.400789    ,    1.412969    ,    1.447192    ,    1.469158    ,    1.507507    ,    1.570719    ,    1.627255    ,    1.687765    ,    1.682756    ,    1.677871    ,    1.69454    ,    1.72353    ,    1.767322    ,    1.784606    ,    1.782694    ,    1.766113    ,    1.74165    ,    1.697833    ,    1.679857    ,    1.598174    ,    1.527822    ,    1.495423    ,    1.444288    ,    1.254697    ,    1.069206    ,    0.95517    ,    0.881941    ,    0.802425    ,    0.679368    ,    0.57628    ,    0.508647    ,    0.46226    ,    0.433089    ,    0.396021    ,    0.365693    ,    0.390198    ,    0.430315    ,    0.487367    ,    0.51491    ,    0.568797    ,    0.653399    ,    0.794568    ,    0.951716    ,    1.028309    ,    1.080287    ,    1.112125    ,    1.152016    ,    1.200385    ,    1.237769    ,    1.270317    ,    1.287316    ,    1.319523    ,    1.354065    ,    1.375912    ,    1.393076    ,    1.392523    ,    1.406747    ,    1.436241    ,    1.465359    ,    1.451425    ,    1.44567    ,    1.522544    ,    1.552677    ,    1.589491    ,    1.611801    ,    1.590898    ,    1.568242    ,    1.579155    ,    1.570961    ,    1.574921    ,    1.581039    ,    1.578453    ,    1.532233    ,    1.487299    ,    1.453548    ,    1.503487    ,    1.51591    ,    1.472037    ,    1.428992    ,    1.458867    ,    1.468457    ,    1.438818    ,    1.449507    ,    1.490655    ,    1.508791    ,    1.506424    ,    1.515306    ,    1.546412    ,    1.545839    ,    1.536178    ,    1.530889    ,    1.528471    ,    1.539611    ,    1.577786    ,    1.548778    ,    1.471417    ,    1.365987    ,    1.28115    ,    1.212396    ,    1.162532    ,    1.124626    ,    1.093647    ,    1.074082    ,    1.058906    ,    1.045815    ,    1.03626    },
    {    1.076563    ,    1.075645    ,    1.078956    ,    1.087922    ,    1.096932    ,    1.09582    ,    1.090049    ,    1.080218    ,    1.071757    ,    1.069532    ,    1.066254    ,    1.061469    ,    1.051735    ,    1.042756    ,    1.028933    ,    1.007985    ,    0.991696    ,    0.999459    ,    1.059326    ,    1.274498    ,    1.406869    ,    1.444212    ,    1.446722    ,    1.449261    ,    1.500185    ,    1.5618    ,    1.638109    ,    1.67294    ,    1.691046    ,    1.721551    ,    1.745623    ,    1.693596    ,    1.742605    ,    1.779862    ,    1.840155    ,    1.89149    ,    1.904709    ,    1.824657    ,    1.76176    ,    1.678385    ,    1.660702    ,    1.651726    ,    1.65135    ,    1.624351    ,    1.486355    ,    1.197164    ,    0.989122    ,    0.834692    ,    0.704208    ,    0.593221    ,    0.511565    ,    0.463934    ,    0.431698    ,    0.394652    ,    0.364454    ,    0.345826    ,    0.350814    ,    0.379657    ,    0.406451    ,    0.467081    ,    0.52857    ,    0.655142    ,    0.722692    ,    0.868946    ,    0.937851    ,    0.956264    ,    1.023299    ,    1.09315    ,    1.142095    ,    1.190871    ,    1.235065    ,    1.279684    ,    1.311898    ,    1.36326    ,    1.419963    ,    1.469928    ,    1.479499    ,    1.51495    ,    1.546595    ,    1.555926    ,    1.577978    ,    1.614907    ,    1.637328    ,    1.654988    ,    1.623779    ,    1.62095    ,    1.649228    ,    1.663595    ,    1.668832    ,    1.681365    ,    1.667958    ,    1.674586    ,    1.662666    ,    1.637589    ,    1.547695    ,    1.45746    ,    1.404865    ,    1.477272    ,    1.483637    ,    1.494258    ,    1.454637    ,    1.492403    ,    1.557834    ,    1.545332    ,    1.522088    ,    1.523444    ,    1.586057    ,    1.580853    ,    1.582706    ,    1.591414    ,    1.611017    ,    1.573725    ,    1.580824    ,    1.615661    ,    1.683365    ,    1.73571    ,    1.724977    ,    1.654406    ,    1.547775    ,    1.473125    ,    1.43979    ,    1.308148    ,    1.226403    ,    1.154115    ,    1.121114    ,    1.110011    ,    1.097803    ,    1.084927    },
    {    1.173925    ,    1.185333    ,    1.203795    ,    1.225142    ,    1.248675    ,    1.258538    ,    1.255333    ,    1.239421    ,    1.206596    ,    1.165829    ,    1.134249    ,    1.11425    ,    1.095731    ,    1.07776    ,    1.059051    ,    1.051409    ,    1.062217    ,    1.157789    ,    1.35635    ,    1.471833    ,    1.51878    ,    1.497926    ,    1.49657    ,    1.526304    ,    1.59873    ,    1.648398    ,    1.696637    ,    1.72472    ,    1.752678    ,    1.793041    ,    1.818071    ,    1.831945    ,    1.865523    ,    1.921116    ,    2.0203    ,    2.129832    ,    2.120793    ,    1.924638    ,    1.829202    ,    1.778449    ,    1.736779    ,    1.718088    ,    1.675272    ,    1.594534    ,    1.375027    ,    1.116946    ,    0.936735    ,    0.776275    ,    0.650067    ,    0.562312    ,    0.498957    ,    0.450656    ,    0.415291    ,    0.387516    ,    0.368743    ,    0.358566    ,    0.379382    ,    0.41771    ,    0.438294    ,    0.462774    ,    0.507021    ,    0.664269    ,    0.791853    ,    0.894068    ,    0.880137    ,    0.890032    ,    0.942313    ,    1.036601    ,    1.129887    ,    1.157782    ,    1.22311    ,    1.282154    ,    1.354297    ,    1.429122    ,    1.497744    ,    1.540997    ,    1.588677    ,    1.624239    ,    1.645566    ,    1.654474    ,    1.64382    ,    1.675365    ,    1.691422    ,    1.686469    ,    1.628861    ,    1.624645    ,    1.68229    ,    1.721057    ,    1.742244    ,    1.751233    ,    1.781391    ,    1.788256    ,    1.761096    ,    1.733071    ,    1.685089    ,    1.609372    ,    1.567649    ,    1.553062    ,    1.535311    ,    1.557196    ,    1.57789    ,    1.595361    ,    1.621863    ,    1.605952    ,    1.574037    ,    1.566374    ,    1.60155    ,    1.641489    ,    1.627244    ,    1.63734    ,    1.652553    ,    1.623855    ,    1.64565    ,    1.828463    ,    1.960367    ,    1.943251    ,    1.869427    ,    1.788386    ,    1.69808    ,    1.653518    ,    1.669055    ,    1.585657    ,    1.459706    ,    1.309537    ,    1.231692    ,    1.20634    ,    1.190521    ,    1.179212    },
    {    1.38077    ,    1.401074    ,    1.432873    ,    1.467738    ,    1.490554    ,    1.511754    ,    1.539348    ,    1.548292    ,    1.537739    ,    1.380777    ,    1.273706    ,    1.206308    ,    1.165316    ,    1.142334    ,    1.13397    ,    1.166891    ,    1.303377    ,    1.502629    ,    1.533509    ,    1.541463    ,    1.582001    ,    1.629416    ,    1.647962    ,    1.69652    ,    1.725292    ,    1.757822    ,    1.777367    ,    1.782459    ,    1.845454    ,    1.892197    ,    1.927655    ,    1.95252    ,    2.053029    ,    2.125157    ,    2.159344    ,    2.179307    ,    2.159313    ,    2.032026    ,    1.92745    ,    1.861437    ,    1.812261    ,    1.77789    ,    1.664164    ,    1.443    ,    1.201676    ,    1.045141    ,    0.935374    ,    0.841843    ,    0.801341    ,    0.755353    ,    0.664422    ,    0.5851    ,    0.532174    ,    0.510551    ,    0.492514    ,    0.467457    ,    0.466389    ,    0.498101    ,    0.522566    ,    0.521957    ,    0.518478    ,    0.571783    ,    0.747358    ,    0.851326    ,    0.859491    ,    0.916568    ,    1.004582    ,    1.125365    ,    1.218314    ,    1.278876    ,    1.303618    ,    1.346623    ,    1.427104    ,    1.512321    ,    1.577539    ,    1.610542    ,    1.637896    ,    1.680161    ,    1.694126    ,    1.704699    ,    1.704649    ,    1.719458    ,    1.722811    ,    1.725952    ,    1.71321    ,    1.699829    ,    1.739063    ,    1.763345    ,    1.786588    ,    1.797659    ,    1.823782    ,    1.813965    ,    1.802338    ,    1.80622    ,    1.813424    ,    1.787256    ,    1.75505    ,    1.742263    ,    1.718005    ,    1.737613    ,    1.752093    ,    1.710721    ,    1.710528    ,    1.671241    ,    1.671108    ,    1.666377    ,    1.689339    ,    1.706205    ,    1.7241    ,    1.735431    ,    1.72951    ,    1.73721    ,    1.731629    ,    1.80061    ,    1.987474    ,    1.986501    ,    1.928678    ,    1.877279    ,    1.834904    ,    1.796602    ,    1.813274    ,    1.786954    ,    1.785609    ,    1.583683    ,    1.468855    ,    1.423311    ,    1.399414    ,    1.383616    },
    {    1.763433    ,    1.74643    ,    1.737967    ,    1.729663    ,    1.717417    ,    1.757146    ,    1.840538    ,    1.892708    ,    1.866572    ,    1.789669    ,    1.633998    ,    1.510707    ,    1.441037    ,    1.416418    ,    1.364959    ,    1.522968    ,    1.637965    ,    1.619718    ,    1.624287    ,    1.657502    ,    1.711811    ,    1.76575    ,    1.82755    ,    1.850367    ,    1.854525    ,    1.880155    ,    1.84142    ,    1.883844    ,    1.917477    ,    1.961335    ,    2.012196    ,    2.035633    ,    2.070835    ,    2.110315    ,    2.14491    ,    2.164738    ,    2.15386    ,    2.10234    ,    2.029266    ,    1.935972    ,    1.791032    ,    1.755543    ,    1.574329    ,    1.398391    ,    1.200929    ,    1.0786    ,    1.021548    ,    1.03975    ,    1.093742    ,    1.164616    ,    0.972309    ,    0.797019    ,    0.696234    ,    0.660833    ,    0.651604    ,    0.632577    ,    0.624695    ,    0.623886    ,    0.623938    ,    0.627116    ,    0.637257    ,    0.667795    ,    0.738936    ,    0.81943    ,    0.893838    ,    0.974513    ,    1.137769    ,    1.301894    ,    1.406383    ,    1.460918    ,    1.485065    ,    1.522471    ,    1.570814    ,    1.654968    ,    1.700279    ,    1.717886    ,    1.720936    ,    1.737162    ,    1.759628    ,    1.761438    ,    1.787269    ,    1.787821    ,    1.788735    ,    1.803494    ,    1.801589    ,    1.810581    ,    1.857003    ,    1.877584    ,    1.943467    ,    1.926804    ,    1.926119    ,    1.926582    ,    1.958488    ,    1.934968    ,    1.952767    ,    1.941038    ,    1.89796    ,    1.890411    ,    1.857664    ,    1.852303    ,    1.868359    ,    1.843759    ,    1.84702    ,    1.85059    ,    1.842014    ,    1.810691    ,    1.808106    ,    1.831274    ,    1.827724    ,    1.81832    ,    1.836707    ,    1.827721    ,    1.841251    ,    1.865643    ,    1.852646    ,    1.842745    ,    1.868194    ,    1.854877    ,    1.860493    ,    1.878021    ,    1.765156    ,    1.841308    ,    1.962212    ,    1.915387    ,    1.87772    ,    1.864822    ,    1.819798    ,    1.78883    },
    {    2.154883    ,    2.05518    ,    1.987178    ,    1.9399    ,    1.913401    ,    1.916632    ,    2.01037    ,    2.020942    ,    1.987903    ,    1.907452    ,    1.833611    ,    1.806787    ,    1.753449    ,    1.750654    ,    1.753955    ,    1.773174    ,    1.722965    ,    1.703467    ,    1.720893    ,    1.768643    ,    1.847736    ,    1.924084    ,    1.974497    ,    1.946436    ,    1.956332    ,    1.955413    ,    1.984254    ,    2.010738    ,    2.050294    ,    2.079718    ,    2.122118    ,    2.142502    ,    2.084342    ,    2.085202    ,    2.141119    ,    2.158308    ,    2.152822    ,    2.155703    ,    2.058172    ,    1.954396    ,    1.881407    ,    1.700588    ,    1.602695    ,    1.444423    ,    1.288754    ,    1.194294    ,    1.160691    ,    1.173731    ,    1.265106    ,    1.39194    ,    1.358794    ,    1.11083    ,    0.941741    ,    0.862787    ,    0.851002    ,    0.865642    ,    0.904887    ,    0.924398    ,    0.902432    ,    0.913199    ,    0.915562    ,    0.95019    ,    0.99786    ,    1.033355    ,    1.063247    ,    1.103134    ,    1.171405    ,    1.317289    ,    1.445109    ,    1.536986    ,    1.628381    ,    1.709999    ,    1.778401    ,    1.81761    ,    1.844638    ,    1.849689    ,    1.867294    ,    1.88427    ,    1.873864    ,    1.880886    ,    1.899915    ,    1.908472    ,    1.929447    ,    1.938636    ,    1.953316    ,    1.97513    ,    2.011296    ,    2.04499    ,    2.070503    ,    2.066884    ,    2.116817    ,    2.116205    ,    2.102329    ,    2.068012    ,    2.067086    ,    2.004892    ,    1.972547    ,    1.949355    ,    1.941988    ,    1.929893    ,    1.958518    ,    1.949709    ,    1.959089    ,    1.968781    ,    1.975973    ,    1.975013    ,    1.97322    ,    1.959288    ,    1.938626    ,    1.920099    ,    1.891156    ,    1.876939    ,    1.887335    ,    1.853503    ,    1.820432    ,    1.841439    ,    1.881771    ,    1.935551    ,    1.991952    ,    2.008844    ,    1.966185    ,    1.984725    ,    2.015072    ,    2.046361    ,    2.061627    ,    2.109584    ,    2.159389    ,    2.182406    },
    {    2.306496    ,    2.333924    ,    2.395781    ,    2.376952    ,    2.314696    ,    2.220485    ,    2.175687    ,    2.125561    ,    2.066982    ,    2.017676    ,    1.98704    ,    1.966865    ,    1.955718    ,    1.938064    ,    1.930306    ,    1.89795    ,    1.882936    ,    1.87449    ,    1.901066    ,    1.930589    ,    1.997948    ,    2.032229    ,    2.049712    ,    2.052996    ,    2.070323    ,    2.090057    ,    2.104584    ,    2.1298    ,    2.163803    ,    2.190288    ,    2.195673    ,    2.186718    ,    2.178553    ,    2.192864    ,    2.222149    ,    2.156358    ,    2.132222    ,    2.152575    ,    2.112705    ,    2.040449    ,    1.949861    ,    1.80896    ,    1.740194    ,    1.60361    ,    1.437142    ,    1.393774    ,    1.32572    ,    1.253527    ,    1.375069    ,    1.413678    ,    1.56053    ,    1.610067    ,    1.579666    ,    1.397999    ,    1.296215    ,    1.29845    ,    1.425273    ,    1.4241    ,    1.397311    ,    1.389853    ,    1.357509    ,    1.354013    ,    1.36144    ,    1.345738    ,    1.341218    ,    1.33569    ,    1.32426    ,    1.338542    ,    1.403914    ,    1.590968    ,    1.697797    ,    1.804215    ,    1.873861    ,    1.920666    ,    1.949014    ,    1.981913    ,    2.026908    ,    2.045839    ,    2.066554    ,    2.066758    ,    2.093724    ,    2.10557    ,    2.114519    ,    2.120786    ,    2.138055    ,    2.152107    ,    2.170055    ,    2.21364    ,    2.248499    ,    2.262224    ,    2.248705    ,    2.24303    ,    2.226842    ,    2.206836    ,    2.160491    ,    2.122323    ,    2.096186    ,    2.087654    ,    2.070886    ,    2.056416    ,    2.042048    ,    2.072933    ,    2.083871    ,    2.116066    ,    2.106858    ,    2.114456    ,    2.093784    ,    2.055125    ,    2.043139    ,    1.979896    ,    1.90352    ,    1.909767    ,    1.905686    ,    1.880906    ,    1.871737    ,    1.917273    ,    1.956908    ,    2.035922    ,    2.141125    ,    2.164801    ,    2.140019    ,    2.125341    ,    2.098666    ,    2.123471    ,    2.190196    ,    2.251861    ,    2.291813    ,    2.320474    },
    {    2.575307    ,    2.691031    ,    2.725449    ,    2.686899    ,    2.627052    ,    2.552568    ,    2.406048    ,    2.328565    ,    2.248955    ,    2.198354    ,    2.166953    ,    2.136716    ,    2.114856    ,    2.138646    ,    2.19252    ,    2.232259    ,    2.21274    ,    2.195223    ,    2.187689    ,    2.198434    ,    2.206259    ,    2.213659    ,    2.214811    ,    2.224729    ,    2.230414    ,    2.223937    ,    2.253314    ,    2.279044    ,    2.269597    ,    2.296453    ,    2.275112    ,    2.2587    ,    2.254826    ,    2.257201    ,    2.242845    ,    2.206572    ,    2.135257    ,    2.137855    ,    2.071804    ,    2.010512    ,    1.94092    ,    1.914303    ,    1.852178    ,    1.729086    ,    1.586561    ,    1.57093    ,    1.400466    ,    1.294479    ,    1.357985    ,    1.429381    ,    1.449781    ,    1.413978    ,    1.43004    ,    1.589143    ,    1.708297    ,    1.912441    ,    1.993067    ,    1.925294    ,    1.83359    ,    1.744171    ,    1.693827    ,    1.692212    ,    1.706851    ,    1.714921    ,    1.695002    ,    1.655687    ,    1.608859    ,    1.556469    ,    1.519541    ,    1.53042    ,    1.730184    ,    1.828315    ,    1.906197    ,    1.970399    ,    2.027086    ,    2.052535    ,    2.086169    ,    2.150064    ,    2.203513    ,    2.242669    ,    2.336854    ,    2.356127    ,    2.450531    ,    2.48424    ,    2.493056    ,    2.490628    ,    2.453088    ,    2.445381    ,    2.452479    ,    2.455578    ,    2.432956    ,    2.419362    ,    2.398047    ,    2.368664    ,    2.329058    ,    2.276672    ,    2.20734    ,    2.152472    ,    2.131524    ,    2.135005    ,    2.172386    ,    2.207274    ,    2.222388    ,    2.203269    ,    2.20509    ,    2.212475    ,    2.200752    ,    2.172027    ,    2.122491    ,    2.050524    ,    2.012503    ,    2.05151    ,    2.079306    ,    2.065997    ,    2.079248    ,    2.14208    ,    2.205095    ,    2.270125    ,    2.312772    ,    2.357283    ,    2.367534    ,    2.335192    ,    2.326094    ,    2.317916    ,    2.335801    ,    2.38852    ,    2.423608    ,    2.48327    },
    {    2.836067    ,    2.847594    ,    2.861442    ,    2.845438    ,    2.811157    ,    2.767306    ,    2.719242    ,    2.692981    ,    2.704411    ,    2.66105    ,    2.6323    ,    2.597762    ,    2.561465    ,    2.553407    ,    2.550279    ,    2.534435    ,    2.520119    ,    2.487032    ,    2.446358    ,    2.413251    ,    2.387445    ,    2.365402    ,    2.351389    ,    2.347726    ,    2.34815    ,    2.349315    ,    2.369287    ,    2.394454    ,    2.445995    ,    2.408985    ,    2.382368    ,    2.388139    ,    2.388468    ,    2.315952    ,    2.282358    ,    2.241679    ,    2.172113    ,    2.113716    ,    2.062964    ,    2.059695    ,    2.057578    ,    2.002832    ,    1.927886    ,    1.809286    ,    1.674876    ,    1.571816    ,    1.363751    ,    1.334637    ,    1.394552    ,    1.487478    ,    1.549112    ,    1.534619    ,    1.481672    ,    1.4395    ,    1.47439    ,    1.628198    ,    1.98842    ,    2.289241    ,    2.297208    ,    2.20064    ,    2.080071    ,    1.952806    ,    1.888554    ,    1.862654    ,    1.848395    ,    1.830881    ,    1.800994    ,    1.744621    ,    1.677663    ,    1.660728    ,    1.692019    ,    1.776092    ,    1.846247    ,    1.98832    ,    2.048062    ,    2.08486    ,    2.168114    ,    2.276264    ,    2.391043    ,    2.50499    ,    2.618274    ,    2.708823    ,    2.774699    ,    2.868578    ,    2.866361    ,    2.867659    ,    2.854783    ,    2.784584    ,    2.706282    ,    2.641693    ,    2.589514    ,    2.554961    ,    2.535605    ,    2.507559    ,    2.480453    ,    2.437754    ,    2.380044    ,    2.328999    ,    2.315462    ,    2.318511    ,    2.341225    ,    2.348555    ,    2.33774    ,    2.336323    ,    2.346203    ,    2.348993    ,    2.338539    ,    2.320368    ,    2.299354    ,    2.267277    ,    2.250082    ,    2.287032    ,    2.30493    ,    2.339645    ,    2.388909    ,    2.442165    ,    2.489455    ,    2.521473    ,    2.538378    ,    2.559274    ,    2.617216    ,    2.650595    ,    2.675515    ,    2.696991    ,    2.717881    ,    2.744812    ,    2.778051    ,    2.804867    },
    {    2.880345    ,    2.88319    ,    2.882575    ,    2.877723    ,    2.869167    ,    2.85928    ,    2.844884    ,    2.828077    ,    2.807263    ,    2.780311    ,    2.746815    ,    2.706694    ,    2.665585    ,    2.6266    ,    2.589421    ,    2.554265    ,    2.519609    ,    2.490015    ,    2.461577    ,    2.421259    ,    2.410383    ,    2.436022    ,    2.442642    ,    2.444222    ,    2.459559    ,    2.462833    ,    2.522355    ,    2.530797    ,    2.549649    ,    2.525012    ,    2.506457    ,    2.45874    ,    2.40256    ,    2.377964    ,    2.26676    ,    2.208088    ,    2.213552    ,    2.166771    ,    2.138185    ,    2.118363    ,    2.092491    ,    2.034268    ,    1.950049    ,    1.832957    ,    1.683699    ,    1.458701    ,    1.41537    ,    1.453679    ,    1.500516    ,    1.542388    ,    1.555089    ,    1.538855    ,    1.529405    ,    1.495779    ,    1.523367    ,    1.618469    ,    1.794843    ,    2.21412    ,    2.441957    ,    2.494895    ,    2.463753    ,    2.342773    ,    2.191431    ,    2.064762    ,    1.99033    ,    1.953854    ,    1.938347    ,    1.936198    ,    1.952223    ,    2.011937    ,    2.129429    ,    2.227152    ,    2.286029    ,    2.345745    ,    2.416388    ,    2.497499    ,    2.60733    ,    2.715473    ,    2.796975    ,    2.866423    ,    2.918656    ,    2.966059    ,    2.982532    ,    2.99252    ,    2.931929    ,    2.98052    ,    2.979857    ,    2.960065    ,    2.890952    ,    2.823563    ,    2.735391    ,    2.678242    ,    2.641752    ,    2.624569    ,    2.611034    ,    2.595967    ,    2.57425    ,    2.547263    ,    2.534452    ,    2.526237    ,    2.522239    ,    2.525506    ,    2.527343    ,    2.525573    ,    2.517066    ,    2.501177    ,    2.477891    ,    2.47266    ,    2.491717    ,    2.498831    ,    2.480324    ,    2.491132    ,    2.509914    ,    2.540369    ,    2.608508    ,    2.639486    ,    2.671118    ,    2.702275    ,    2.732169    ,    2.754106    ,    2.767165    ,    2.781065    ,    2.794988    ,    2.809677    ,    2.826548    ,    2.843537    ,    2.858912    ,    2.871908    },
    {    2.830105    ,    2.82272    ,    2.815261    ,    2.807891    ,    2.800863    ,    2.790458    ,    2.77616    ,    2.75724    ,    2.73596    ,    2.711415    ,    2.68685    ,    2.660832    ,    2.636878    ,    2.613802    ,    2.591056    ,    2.567198    ,    2.544091    ,    2.520888    ,    2.497406    ,    2.468964    ,    2.435795    ,    2.419357    ,    2.455542    ,    2.493944    ,    2.500499    ,    2.526488    ,    2.552217    ,    2.53796    ,    2.52188    ,    2.498538    ,    2.456003    ,    2.426291    ,    2.412819    ,    2.394202    ,    2.315512    ,    2.22349    ,    2.192985    ,    2.107235    ,    2.047018    ,    2.012564    ,    1.908339    ,    1.798228    ,    1.748285    ,    1.65945    ,    1.554884    ,    1.545258    ,    1.549866    ,    1.557674    ,    1.569215    ,    1.567444    ,    1.556059    ,    1.553208    ,    1.538651    ,    1.538228    ,    1.566162    ,    1.615152    ,    1.734884    ,    2.200274    ,    2.462612    ,    2.580355    ,    2.60817    ,    2.563091    ,    2.481614    ,    2.387965    ,    2.279244    ,    2.207187    ,    2.186844    ,    2.238836    ,    2.363648    ,    2.532367    ,    2.694093    ,    2.919545    ,    2.961395    ,    3.0786    ,    3.160931    ,    3.243189    ,    3.293622    ,    3.311222    ,    3.337592    ,    3.371999    ,    3.391793    ,    3.402636    ,    3.413531    ,    3.401157    ,    3.376904    ,    3.308558    ,    3.282312    ,    3.304311    ,    3.270597    ,    3.236024    ,    3.162134    ,    3.071127    ,    2.988517    ,    2.910676    ,    2.843012    ,    2.778392    ,    2.716254    ,    2.682119    ,    2.664248    ,    2.642593    ,    2.659807    ,    2.67512    ,    2.665996    ,    2.681168    ,    2.655829    ,    2.645931    ,    2.672395    ,    2.682665    ,    2.683958    ,    2.683706    ,    2.684353    ,    2.683195    ,    2.679911    ,    2.69288    ,    2.707687    ,    2.733771    ,    2.754088    ,    2.767015    ,    2.780317    ,    2.788669    ,    2.795228    ,    2.802236    ,    2.808577    ,    2.815785    ,    2.823109    ,    2.829956    ,    2.834086    ,    2.836436    },
    {    2.76523    ,    2.758429    ,    2.752485    ,    2.746074    ,    2.739662    ,    2.732332    ,    2.722658    ,    2.710992    ,    2.697882    ,    2.682213    ,    2.665536    ,    2.651845    ,    2.638833    ,    2.626526    ,    2.616846    ,    2.605642    ,    2.593086    ,    2.580632    ,    2.568922    ,    2.557329    ,    2.546075    ,    2.53439    ,    2.531398    ,    2.533064    ,    2.532506    ,    2.531875    ,    2.527465    ,    2.518821    ,    2.503343    ,    2.45726    ,    2.418458    ,    2.33705    ,    2.291389    ,    2.205186    ,    2.128569    ,    2.102802    ,    2.091559    ,    2.075346    ,    2.003103    ,    1.980719    ,    1.879238    ,    1.810619    ,    1.702283    ,    1.648696    ,    1.627831    ,    1.6119    ,    1.59782    ,    1.596772    ,    1.601823    ,    1.59852    ,    1.593574    ,    1.591811    ,    1.610392    ,    1.65716    ,    1.72289    ,    1.815227    ,    1.957468    ,    2.223633    ,    2.457573    ,    2.571344    ,    2.617247    ,    2.626163    ,    2.610002    ,    2.580492    ,    2.562353    ,    2.555918    ,    2.529219    ,    2.536932    ,    2.563482    ,    2.659356    ,    2.753709    ,    2.87035    ,    3.032187    ,    3.192517    ,    3.319935    ,    3.385135    ,    3.435262    ,    3.470922    ,    3.501548    ,    3.533318    ,    3.550203    ,    3.550334    ,    3.534388    ,    3.513726    ,    3.478574    ,    3.447914    ,    3.432351    ,    3.429632    ,    3.428197    ,    3.425249    ,    3.401657    ,    3.341103    ,    3.27403    ,    3.215792    ,    3.162728    ,    3.091618    ,    2.988823    ,    2.911554    ,    2.850236    ,    2.861115    ,    2.865278    ,    2.863628    ,    2.864235    ,    2.863746    ,    2.855327    ,    2.850459    ,    2.84779    ,    2.841425    ,    2.83543    ,    2.832774    ,    2.828742    ,    2.823692    ,    2.819929    ,    2.812959    ,    2.806497    ,    2.800512    ,    2.795    ,    2.791322    ,    2.788981    ,    2.790542    ,    2.789743    ,    2.789922    ,    2.789274    ,    2.786963    ,    2.783865    ,    2.780348    ,    2.776459    ,    2.772193    },
    {    2.72972    ,    2.724805    ,    2.719342    ,    2.712397    ,    2.70625    ,    2.700415    ,    2.693846    ,    2.687118    ,    2.680599    ,    2.673959    ,    2.667251    ,    2.660403    ,    2.65391    ,    2.64904    ,    2.643044    ,    2.639597    ,    2.638045    ,    2.636096    ,    2.63552    ,    2.636479    ,    2.6349    ,    2.636303    ,    2.634469    ,    2.634161    ,    2.629157    ,    2.624783    ,    2.614875    ,    2.600436    ,    2.586635    ,    2.557714    ,    2.538043    ,    2.508779    ,    2.461415    ,    2.447362    ,    2.365138    ,    2.330803    ,    2.282675    ,    2.252444    ,    2.228225    ,    2.201232    ,    2.182252    ,    2.184992    ,    2.171772    ,    2.167717    ,    2.182316    ,    2.163017    ,    2.116734    ,    2.102348    ,    2.102298    ,    2.054293    ,    2.018782    ,    1.989796    ,    2.044409    ,    2.102708    ,    2.146286    ,    2.261104    ,    2.427462    ,    2.492833    ,    2.546962    ,    2.60653    ,    2.649028    ,    2.67734    ,    2.709881    ,    2.741953    ,    2.761997    ,    2.777682    ,    2.801534    ,    2.835841    ,    2.874452    ,    2.916841    ,    2.953974    ,    3.001265    ,    3.04457    ,    3.088315    ,    3.124837    ,    3.150006    ,    3.169689    ,    3.183556    ,    3.195556    ,    3.202023    ,    3.20414    ,    3.200186    ,    3.191471    ,    3.178163    ,    3.169652    ,    3.175284    ,    3.175126    ,    3.181895    ,    3.184311    ,    3.190845    ,    3.190797    ,    3.182793    ,    3.163298    ,    3.140584    ,    3.113464    ,    3.08518    ,    3.0451    ,    3.014382    ,    2.997099    ,    2.985523    ,    2.970749    ,    2.954384    ,    2.938355    ,    2.92533    ,    2.913167    ,    2.903382    ,    2.894518    ,    2.887686    ,    2.882563    ,    2.878429    ,    2.87362    ,    2.868598    ,    2.860508    ,    2.851469    ,    2.842026    ,    2.831119    ,    2.819937    ,    2.810371    ,    2.799435    ,    2.788559    ,    2.780466    ,    2.77149    ,    2.76631    ,    2.760983    ,    2.754963    ,    2.748268    ,    2.741474    ,    2.735494    },
    {    2.735367    ,    2.730538    ,    2.725408    ,    2.720613    ,    2.717227    ,    2.715075    ,    2.712365    ,    2.710094    ,    2.708162    ,    2.705267    ,    2.70337    ,    2.70188    ,    2.700017    ,    2.700064    ,    2.699605    ,    2.698782    ,    2.699158    ,    2.700695    ,    2.700333    ,    2.701281    ,    2.703201    ,    2.703354    ,    2.705305    ,    2.706132    ,    2.706002    ,    2.706001    ,    2.702825    ,    2.702536    ,    2.699596    ,    2.692244    ,    2.687551    ,    2.679845    ,    2.670829    ,    2.660235    ,    2.654412    ,    2.647713    ,    2.639234    ,    2.638327    ,    2.637542    ,    2.63799    ,    2.639613    ,    2.642199    ,    2.645021    ,    2.64635    ,    2.646742    ,    2.648046    ,    2.64858    ,    2.64693    ,    2.640893    ,    2.639373    ,    2.640778    ,    2.643831    ,    2.645824    ,    2.656816    ,    2.671385    ,    2.685638    ,    2.698324    ,    2.721761    ,    2.734535    ,    2.747236    ,    2.7671    ,    2.782333    ,    2.795662    ,    2.810488    ,    2.827172    ,    2.840488    ,    2.8544    ,    2.869467    ,    2.882391    ,    2.893905    ,    2.905501    ,    2.916158    ,    2.925502    ,    2.936081    ,    2.944918    ,    2.952456    ,    2.957429    ,    2.96378    ,    2.968166    ,    2.971772    ,    2.973693    ,    2.972555    ,    2.971289    ,    2.971349    ,    2.969679    ,    2.967243    ,    2.965872    ,    2.964948    ,    2.962216    ,    2.961542    ,    2.960539    ,    2.955406    ,    2.948737    ,    2.946363    ,    2.937018    ,    2.926633    ,    2.920433    ,    2.912563    ,    2.903409    ,    2.895541    ,    2.88979    ,    2.882421    ,    2.875815    ,    2.871299    ,    2.867285    ,    2.862938    ,    2.85873    ,    2.854677    ,    2.851351    ,    2.847079    ,    2.843101    ,    2.837286    ,    2.832062    ,    2.826631    ,    2.821773    ,    2.815521    ,    2.809553    ,    2.803048    ,    2.796176    ,    2.789504    ,    2.782279    ,    2.776188    ,    2.769322    ,    2.763833    ,    2.75919    ,    2.751954    ,    2.744139    ,    2.739092    },
    {    2.761649    ,    2.76152    ,    2.761634    ,    2.76207    ,    2.762452    ,    2.76266    ,    2.762551    ,    2.763712    ,    2.765347    ,    2.766632    ,    2.767355    ,    2.768212    ,    2.769437    ,    2.771508    ,    2.772761    ,    2.773756    ,    2.774072    ,    2.774935    ,    2.77538    ,    2.775736    ,    2.779024    ,    2.780258    ,    2.780806    ,    2.780795    ,    2.781323    ,    2.782764    ,    2.784918    ,    2.786285    ,    2.786892    ,    2.789043    ,    2.789915    ,    2.791087    ,    2.792955    ,    2.794636    ,    2.795511    ,    2.795951    ,    2.796727    ,    2.79756    ,    2.799041    ,    2.800737    ,    2.801882    ,    2.803556    ,    2.804649    ,    2.805699    ,    2.80613    ,    2.805838    ,    2.806196    ,    2.805603    ,    2.804585    ,    2.804274    ,    2.803604    ,    2.802714    ,    2.803091    ,    2.803845    ,    2.804273    ,    2.805091    ,    2.80643    ,    2.807017    ,    2.807147    ,    2.808955    ,    2.813035    ,    2.815228    ,    2.816869    ,    2.817697    ,    2.819653    ,    2.823355    ,    2.826291    ,    2.829153    ,    2.831432    ,    2.833718    ,    2.835963    ,    2.837975    ,    2.839814    ,    2.840915    ,    2.841433    ,    2.841563    ,    2.841292    ,    2.841112    ,    2.841202    ,    2.841601    ,    2.840827    ,    2.840201    ,    2.839899    ,    2.839849    ,    2.838853    ,    2.839097    ,    2.838335    ,    2.838068    ,    2.837486    ,    2.835823    ,    2.833636    ,    2.831983    ,    2.830489    ,    2.830292    ,    2.829012    ,    2.826953    ,    2.823058    ,    2.820062    ,    2.816937    ,    2.814717    ,    2.812811    ,    2.811127    ,    2.808485    ,    2.806768    ,    2.805347    ,    2.803254    ,    2.801627    ,    2.799497    ,    2.797636    ,    2.795891    ,    2.794501    ,    2.791504    ,    2.788584    ,    2.786414    ,    2.784503    ,    2.782495    ,    2.780746    ,    2.779115    ,    2.777001    ,    2.775261    ,    2.773437    ,    2.771334    ,    2.770087    ,    2.769069    ,    2.768245    ,    2.765374    ,    2.763531    ,    2.762223    }
    
}; //The Local to Global Ratio of Temperature Change from Leduc et al (2016), analysed as the average from CMIP5 models
double Stdev_LGRTC_CO2[64][128] = {
    {    0.245648    ,    0.24635    ,    0.250567    ,    0.250953    ,    0.252451    ,    0.25655    ,    0.25802    ,    0.260217    ,    0.256415    ,    0.262809    ,    0.264406    ,    0.25641    ,    0.252314    ,    0.258467    ,    0.260964    ,    0.262339    ,    0.265354    ,    0.264839    ,    0.270447    ,    0.272604    ,    0.275312    ,    0.281497    ,    0.288423    ,    0.288629    ,    0.291672    ,    0.302358    ,    0.302249    ,    0.310019    ,    0.31645    ,    0.307559    ,    0.29685    ,    0.297942    ,    0.305611    ,    0.298013    ,    0.286172    ,    0.284446    ,    0.282945    ,    0.279878    ,    0.278677    ,    0.274295    ,    0.273728    ,    0.273415    ,    0.270263    ,    0.268702    ,    0.277428    ,    0.283821    ,    0.292377    ,    0.301337    ,    0.304802    ,    0.305642    ,    0.303443    ,    0.301061    ,    0.305714    ,    0.307615    ,    0.309901    ,    0.308737    ,    0.313081    ,    0.3162    ,    0.31938    ,    0.321494    ,    0.320326    ,    0.321987    ,    0.324092    ,    0.325627    ,    0.323967    ,    0.319496    ,    0.317777    ,    0.314288    ,    0.309801    ,    0.31308    ,    0.30991    ,    0.31378    ,    0.304638    ,    0.309479    ,    0.298936    ,    0.280268    ,    0.29012    ,    0.285107    ,    0.281274    ,    0.291432    ,    0.286944    ,    0.273488    ,    0.275547    ,    0.26487    ,    0.26838    ,    0.262378    ,    0.270342    ,    0.271422    ,    0.269347    ,    0.264311    ,    0.264558    ,    0.273367    ,    0.266066    ,    0.273175    ,    0.266482    ,    0.273877    ,    0.279227    ,    0.280288    ,    0.290312    ,    0.287669    ,    0.297461    ,    0.291576    ,    0.28175    ,    0.290284    ,    0.297528    ,    0.292339    ,    0.291687    ,    0.279446    ,    0.277491    ,    0.273123    ,    0.266017    ,    0.266782    ,    0.27033    ,    0.254514    ,    0.254072    ,    0.256964    ,    0.253038    ,    0.253322    ,    0.255036    ,    0.260178    ,    0.257598    ,    0.257899    ,    0.243875    ,    0.241336    ,    0.246092    ,    0.233172    ,    0.235063    ,    0.241639    },
    {    0.235408    ,    0.244673    ,    0.250285    ,    0.241034    ,    0.230472    ,    0.235529    ,    0.238788    ,    0.24055    ,    0.234804    ,    0.236342    ,    0.241391    ,    0.251203    ,    0.251355    ,    0.264903    ,    0.28117    ,    0.291222    ,    0.280606    ,    0.272503    ,    0.273306    ,    0.271966    ,    0.278666    ,    0.272804    ,    0.286224    ,    0.281004    ,    0.275332    ,    0.269876    ,    0.265907    ,    0.260626    ,    0.258047    ,    0.25869    ,    0.266063    ,    0.271972    ,    0.290733    ,    0.260826    ,    0.253974    ,    0.239802    ,    0.240373    ,    0.242584    ,    0.244254    ,    0.248938    ,    0.25845    ,    0.265795    ,    0.262274    ,    0.248177    ,    0.24917    ,    0.24044    ,    0.24649    ,    0.261373    ,    0.279581    ,    0.279261    ,    0.282641    ,    0.289389    ,    0.301777    ,    0.306686    ,    0.312087    ,    0.318739    ,    0.316033    ,    0.313673    ,    0.315946    ,    0.327854    ,    0.338455    ,    0.340056    ,    0.338283    ,    0.348036    ,    0.358158    ,    0.364368    ,    0.372735    ,    0.350313    ,    0.369705    ,    0.357786    ,    0.355897    ,    0.349994    ,    0.338881    ,    0.335927    ,    0.318841    ,    0.311237    ,    0.302683    ,    0.308927    ,    0.312948    ,    0.305339    ,    0.305227    ,    0.293769    ,    0.298239    ,    0.275105    ,    0.300092    ,    0.287725    ,    0.275717    ,    0.279506    ,    0.271577    ,    0.252031    ,    0.270314    ,    0.266372    ,    0.272469    ,    0.269565    ,    0.274935    ,    0.280065    ,    0.282279    ,    0.282266    ,    0.286643    ,    0.286673    ,    0.277418    ,    0.282584    ,    0.265149    ,    0.255925    ,    0.253729    ,    0.241629    ,    0.242751    ,    0.235495    ,    0.226764    ,    0.225284    ,    0.224014    ,    0.213322    ,    0.206678    ,    0.205834    ,    0.210073    ,    0.216994    ,    0.220541    ,    0.224036    ,    0.229408    ,    0.238468    ,    0.23893    ,    0.232318    ,    0.23107    ,    0.231664    ,    0.215414    ,    0.221276    ,    0.236002    ,    0.232553    },
    {    0.352054    ,    0.348513    ,    0.350486    ,    0.348973    ,    0.343553    ,    0.329471    ,    0.320808    ,    0.314562    ,    0.307496    ,    0.303187    ,    0.298116    ,    0.299471    ,    0.301521    ,    0.307736    ,    0.303899    ,    0.308792    ,    0.30865    ,    0.301566    ,    0.2962    ,    0.296436    ,    0.296028    ,    0.290198    ,    0.291058    ,    0.290503    ,    0.296165    ,    0.304805    ,    0.302108    ,    0.297004    ,    0.289129    ,    0.276655    ,    0.267297    ,    0.268588    ,    0.261656    ,    0.286855    ,    0.2965    ,    0.280018    ,    0.249153    ,    0.227453    ,    0.217555    ,    0.216802    ,    0.234214    ,    0.24989    ,    0.247245    ,    0.222306    ,    0.205834    ,    0.196299    ,    0.191891    ,    0.19579    ,    0.199533    ,    0.216348    ,    0.22908    ,    0.246033    ,    0.249655    ,    0.267255    ,    0.278622    ,    0.286525    ,    0.293092    ,    0.285938    ,    0.29643    ,    0.294251    ,    0.304172    ,    0.31409    ,    0.319448    ,    0.313115    ,    0.322342    ,    0.329139    ,    0.339103    ,    0.327766    ,    0.348848    ,    0.341095    ,    0.359931    ,    0.345975    ,    0.332915    ,    0.316397    ,    0.308965    ,    0.303835    ,    0.298769    ,    0.29537    ,    0.288965    ,    0.297239    ,    0.297343    ,    0.286783    ,    0.307952    ,    0.290934    ,    0.285814    ,    0.289975    ,    0.285977    ,    0.281949    ,    0.299835    ,    0.300208    ,    0.288742    ,    0.237247    ,    0.250549    ,    0.256742    ,    0.258923    ,    0.261171    ,    0.262299    ,    0.247847    ,    0.264366    ,    0.24323    ,    0.249539    ,    0.243359    ,    0.249227    ,    0.249612    ,    0.257922    ,    0.257123    ,    0.254795    ,    0.247246    ,    0.235724    ,    0.226    ,    0.215327    ,    0.209262    ,    0.203302    ,    0.201504    ,    0.209754    ,    0.22346    ,    0.226015    ,    0.239322    ,    0.252004    ,    0.256857    ,    0.260562    ,    0.269511    ,    0.274211    ,    0.292533    ,    0.329685    ,    0.351294    ,    0.361621    ,    0.355928    },
    {    0.314767    ,    0.312136    ,    0.314253    ,    0.318257    ,    0.322506    ,    0.33051    ,    0.338915    ,    0.346848    ,    0.349134    ,    0.383252    ,    0.380532    ,    0.356265    ,    0.357823    ,    0.349593    ,    0.341164    ,    0.329312    ,    0.323178    ,    0.316975    ,    0.30558    ,    0.302284    ,    0.301946    ,    0.297015    ,    0.293413    ,    0.288291    ,    0.285472    ,    0.277375    ,    0.273944    ,    0.270994    ,    0.263685    ,    0.262878    ,    0.261441    ,    0.255487    ,    0.258147    ,    0.250051    ,    0.256431    ,    0.272265    ,    0.265869    ,    0.257174    ,    0.25531    ,    0.26589    ,    0.25749    ,    0.265159    ,    0.26848    ,    0.271171    ,    0.274534    ,    0.259465    ,    0.23813    ,    0.23703    ,    0.255047    ,    0.250609    ,    0.248531    ,    0.23871    ,    0.221784    ,    0.219272    ,    0.223895    ,    0.240312    ,    0.249408    ,    0.263617    ,    0.273694    ,    0.279674    ,    0.28376    ,    0.275409    ,    0.264777    ,    0.257355    ,    0.261779    ,    0.267704    ,    0.27954    ,    0.291375    ,    0.295313    ,    0.302761    ,    0.303737    ,    0.316317    ,    0.328318    ,    0.341729    ,    0.324272    ,    0.324757    ,    0.307117    ,    0.296506    ,    0.325624    ,    0.343968    ,    0.344223    ,    0.338049    ,    0.313091    ,    0.294273    ,    0.268419    ,    0.27473    ,    0.265979    ,    0.253231    ,    0.220321    ,    0.232715    ,    0.236383    ,    0.2377    ,    0.244349    ,    0.247218    ,    0.228905    ,    0.262246    ,    0.297346    ,    0.294686    ,    0.297396    ,    0.290184    ,    0.280081    ,    0.284755    ,    0.279517    ,    0.269637    ,    0.265224    ,    0.254907    ,    0.257428    ,    0.26716    ,    0.272814    ,    0.270133    ,    0.260337    ,    0.241676    ,    0.231243    ,    0.235441    ,    0.235826    ,    0.24456    ,    0.248361    ,    0.236188    ,    0.235775    ,    0.230338    ,    0.231476    ,    0.237346    ,    0.256336    ,    0.29737    ,    0.312002    ,    0.33336    ,    0.336034    ,    0.330316    },
    {    0.276739    ,    0.325501    ,    0.351753    ,    0.367218    ,    0.36855    ,    0.367058    ,    0.366904    ,    0.369119    ,    0.384958    ,    0.422508    ,    0.436478    ,    0.453273    ,    0.495253    ,    0.412487    ,    0.380885    ,    0.361104    ,    0.346741    ,    0.335409    ,    0.318621    ,    0.308556    ,    0.296052    ,    0.285928    ,    0.275162    ,    0.265108    ,    0.252543    ,    0.249848    ,    0.250703    ,    0.251804    ,    0.254029    ,    0.252968    ,    0.257664    ,    0.254005    ,    0.263454    ,    0.250667    ,    0.252868    ,    0.257697    ,    0.254173    ,    0.265335    ,    0.273156    ,    0.274035    ,    0.285308    ,    0.291777    ,    0.296482    ,    0.304338    ,    0.307922    ,    0.31725    ,    0.322416    ,    0.33325    ,    0.346322    ,    0.350794    ,    0.363295    ,    0.374866    ,    0.365145    ,    0.322541    ,    0.300544    ,    0.312667    ,    0.31172    ,    0.301374    ,    0.274898    ,    0.285468    ,    0.271701    ,    0.27037    ,    0.265248    ,    0.250297    ,    0.250373    ,    0.260478    ,    0.28044    ,    0.286173    ,    0.294502    ,    0.320137    ,    0.34796    ,    0.34595    ,    0.340193    ,    0.32006    ,    0.322956    ,    0.304654    ,    0.274289    ,    0.258247    ,    0.237336    ,    0.260667    ,    0.263496    ,    0.271015    ,    0.275502    ,    0.26196    ,    0.252629    ,    0.254018    ,    0.259812    ,    0.257857    ,    0.255212    ,    0.255252    ,    0.263556    ,    0.270477    ,    0.270865    ,    0.271485    ,    0.279934    ,    0.281048    ,    0.260027    ,    0.291713    ,    0.290962    ,    0.276503    ,    0.301867    ,    0.273738    ,    0.304545    ,    0.307149    ,    0.321549    ,    0.30778    ,    0.317097    ,    0.344248    ,    0.338323    ,    0.323429    ,    0.313966    ,    0.319175    ,    0.326765    ,    0.30804    ,    0.305255    ,    0.293836    ,    0.269727    ,    0.250796    ,    0.234322    ,    0.219361    ,    0.204682    ,    0.214992    ,    0.347008    ,    0.297308    ,    0.280554    ,    0.240249    ,    0.225665    ,    0.238501    },
    {    0.388279    ,    0.391291    ,    0.408998    ,    0.408697    ,    0.399521    ,    0.392354    ,    0.390213    ,    0.397666    ,    0.413739    ,    0.431579    ,    0.448419    ,    0.464673    ,    0.483332    ,    0.504831    ,    0.504919    ,    0.486503    ,    0.450272    ,    0.423736    ,    0.383292    ,    0.352248    ,    0.344576    ,    0.362576    ,    0.35638    ,    0.345307    ,    0.340946    ,    0.334443    ,    0.333823    ,    0.312878    ,    0.298074    ,    0.284132    ,    0.283722    ,    0.281464    ,    0.28193    ,    0.287414    ,    0.291644    ,    0.282815    ,    0.287645    ,    0.261788    ,    0.257589    ,    0.260491    ,    0.256616    ,    0.259553    ,    0.283823    ,    0.349957    ,    0.369081    ,    0.337646    ,    0.327843    ,    0.337928    ,    0.358753    ,    0.382685    ,    0.396993    ,    0.400663    ,    0.39819    ,    0.397045    ,    0.396474    ,    0.394214    ,    0.404564    ,    0.418041    ,    0.379689    ,    0.311657    ,    0.271485    ,    0.245526    ,    0.239945    ,    0.254817    ,    0.266223    ,    0.280886    ,    0.288067    ,    0.299923    ,    0.301891    ,    0.308205    ,    0.301684    ,    0.277889    ,    0.263355    ,    0.249913    ,    0.245551    ,    0.218905    ,    0.213635    ,    0.215164    ,    0.232721    ,    0.238896    ,    0.252183    ,    0.243749    ,    0.244072    ,    0.243314    ,    0.247518    ,    0.258085    ,    0.257569    ,    0.262608    ,    0.254231    ,    0.235506    ,    0.251087    ,    0.257063    ,    0.263963    ,    0.269196    ,    0.274027    ,    0.271896    ,    0.276593    ,    0.284957    ,    0.292542    ,    0.299995    ,    0.310436    ,    0.31908    ,    0.327238    ,    0.333528    ,    0.316523    ,    0.301953    ,    0.287875    ,    0.285518    ,    0.294438    ,    0.319526    ,    0.331473    ,    0.353908    ,    0.341329    ,    0.327105    ,    0.318397    ,    0.299107    ,    0.282494    ,    0.270825    ,    0.260689    ,    0.2504    ,    0.250599    ,    0.242372    ,    0.246038    ,    0.242397    ,    0.265438    ,    0.324472    ,    0.346461    ,    0.354966    },
    {    0.433136    ,    0.432834    ,    0.425263    ,    0.405159    ,    0.387802    ,    0.376698    ,    0.372733    ,    0.374509    ,    0.383573    ,    0.398396    ,    0.419356    ,    0.440113    ,    0.457125    ,    0.472862    ,    0.490795    ,    0.499809    ,    0.494283    ,    0.47788    ,    0.460762    ,    0.452979    ,    0.443121    ,    0.434582    ,    0.422738    ,    0.401949    ,    0.38151    ,    0.363557    ,    0.348297    ,    0.324979    ,    0.29594    ,    0.284191    ,    0.28786    ,    0.30884    ,    0.325615    ,    0.332405    ,    0.337391    ,    0.339167    ,    0.342114    ,    0.3526    ,    0.317338    ,    0.273837    ,    0.276292    ,    0.282309    ,    0.30918    ,    0.342951    ,    0.38591    ,    0.370543    ,    0.367811    ,    0.369641    ,    0.370072    ,    0.389283    ,    0.411344    ,    0.430087    ,    0.445834    ,    0.454563    ,    0.461839    ,    0.465339    ,    0.468116    ,    0.467705    ,    0.461451    ,    0.448128    ,    0.474007    ,    0.466622    ,    0.451239    ,    0.419336    ,    0.354242    ,    0.289264    ,    0.276952    ,    0.262917    ,    0.256778    ,    0.248584    ,    0.247578    ,    0.249054    ,    0.249787    ,    0.249733    ,    0.250059    ,    0.244682    ,    0.230191    ,    0.215844    ,    0.203932    ,    0.202754    ,    0.203202    ,    0.207873    ,    0.227798    ,    0.236317    ,    0.236981    ,    0.248013    ,    0.249928    ,    0.247932    ,    0.236773    ,    0.284889    ,    0.24804    ,    0.268907    ,    0.281798    ,    0.290435    ,    0.29068    ,    0.286575    ,    0.283268    ,    0.286939    ,    0.295965    ,    0.290564    ,    0.291254    ,    0.297685    ,    0.297133    ,    0.29831    ,    0.305594    ,    0.295876    ,    0.293668    ,    0.293852    ,    0.294475    ,    0.300179    ,    0.304234    ,    0.293854    ,    0.297256    ,    0.294441    ,    0.273881    ,    0.268933    ,    0.258305    ,    0.243271    ,    0.248747    ,    0.256285    ,    0.28142    ,    0.277141    ,    0.281603    ,    0.266728    ,    0.33677    ,    0.327079    ,    0.383745    ,    0.40629    },
    {    0.395108    ,    0.403091    ,    0.405106    ,    0.397423    ,    0.389971    ,    0.385761    ,    0.388794    ,    0.394441    ,    0.400074    ,    0.408611    ,    0.423671    ,    0.441008    ,    0.456161    ,    0.470346    ,    0.491686    ,    0.512729    ,    0.524784    ,    0.516492    ,    0.493324    ,    0.473418    ,    0.457593    ,    0.444072    ,    0.429367    ,    0.413741    ,    0.399289    ,    0.384209    ,    0.368615    ,    0.351123    ,    0.335333    ,    0.323192    ,    0.308882    ,    0.299383    ,    0.289272    ,    0.279984    ,    0.269632    ,    0.259661    ,    0.249267    ,    0.238181    ,    0.233551    ,    0.231682    ,    0.272146    ,    0.311718    ,    0.3607    ,    0.368735    ,    0.360732    ,    0.348883    ,    0.335946    ,    0.33678    ,    0.352981    ,    0.384004    ,    0.423223    ,    0.463714    ,    0.500111    ,    0.526353    ,    0.546604    ,    0.559006    ,    0.565948    ,    0.568027    ,    0.570145    ,    0.571423    ,    0.564061    ,    0.553282    ,    0.540275    ,    0.535251    ,    0.543976    ,    0.547262    ,    0.542132    ,    0.531147    ,    0.519991    ,    0.523481    ,    0.52425    ,    0.529299    ,    0.525982    ,    0.519982    ,    0.505087    ,    0.490469    ,    0.457035    ,    0.451882    ,    0.448159    ,    0.310971    ,    0.262152    ,    0.250018    ,    0.242663    ,    0.248584    ,    0.26892    ,    0.291075    ,    0.302262    ,    0.318012    ,    0.310159    ,    0.313903    ,    0.355699    ,    0.350224    ,    0.288418    ,    0.284848    ,    0.281624    ,    0.27728    ,    0.269302    ,    0.277775    ,    0.289779    ,    0.295749    ,    0.287397    ,    0.28244    ,    0.282791    ,    0.284076    ,    0.281921    ,    0.289826    ,    0.299356    ,    0.291806    ,    0.291203    ,    0.303702    ,    0.292845    ,    0.281679    ,    0.280454    ,    0.278673    ,    0.278994    ,    0.272654    ,    0.261571    ,    0.282525    ,    0.311538    ,    0.315931    ,    0.321143    ,    0.323683    ,    0.331962    ,    0.338044    ,    0.344962    ,    0.363124    ,    0.376064    ,    0.38381    },
    {    0.389775    ,    0.386403    ,    0.376883    ,    0.369605    ,    0.368982    ,    0.375708    ,    0.382479    ,    0.38869    ,    0.393405    ,    0.400546    ,    0.40658    ,    0.407232    ,    0.406667    ,    0.408014    ,    0.409073    ,    0.409095    ,    0.409376    ,    0.417549    ,    0.425526    ,    0.426197    ,    0.424636    ,    0.421525    ,    0.417901    ,    0.414333    ,    0.411675    ,    0.411577    ,    0.41179    ,    0.41025    ,    0.412383    ,    0.421426    ,    0.432717    ,    0.437269    ,    0.441198    ,    0.442561    ,    0.434108    ,    0.415638    ,    0.38945    ,    0.353005    ,    0.315451    ,    0.282088    ,    0.248585    ,    0.253678    ,    0.284125    ,    0.298184    ,    0.304309    ,    0.298221    ,    0.292304    ,    0.292079    ,    0.308938    ,    0.341344    ,    0.37714    ,    0.415598    ,    0.459425    ,    0.50233    ,    0.543038    ,    0.572976    ,    0.594861    ,    0.608617    ,    0.613788    ,    0.61007    ,    0.607841    ,    0.6103    ,    0.618725    ,    0.626527    ,    0.633852    ,    0.63752    ,    0.639886    ,    0.638171    ,    0.636044    ,    0.632939    ,    0.634775    ,    0.637968    ,    0.638798    ,    0.639814    ,    0.63947    ,    0.636556    ,    0.625994    ,    0.605639    ,    0.581177    ,    0.564963    ,    0.549459    ,    0.535015    ,    0.512522    ,    0.49047    ,    0.46622    ,    0.430507    ,    0.410258    ,    0.400533    ,    0.396334    ,    0.403677    ,    0.413883    ,    0.404588    ,    0.394238    ,    0.400823    ,    0.401206    ,    0.398405    ,    0.36684    ,    0.308946    ,    0.333272    ,    0.389915    ,    0.368963    ,    0.391921    ,    0.396711    ,    0.390558    ,    0.436842    ,    0.471213    ,    0.493835    ,    0.464353    ,    0.442194    ,    0.452811    ,    0.440486    ,    0.428844    ,    0.41863    ,    0.394407    ,    0.37056    ,    0.371684    ,    0.394747    ,    0.397869    ,    0.390293    ,    0.384938    ,    0.384151    ,    0.381072    ,    0.363937    ,    0.353908    ,    0.355641    ,    0.367285    ,    0.370537    ,    0.380759    },
    {    0.406119    ,    0.425585    ,    0.436198    ,    0.432934    ,    0.422122    ,    0.419171    ,    0.419034    ,    0.414614    ,    0.405783    ,    0.397008    ,    0.389665    ,    0.381004    ,    0.367749    ,    0.361373    ,    0.363187    ,    0.357063    ,    0.338615    ,    0.324773    ,    0.318866    ,    0.318433    ,    0.318995    ,    0.317554    ,    0.315914    ,    0.316424    ,    0.318265    ,    0.321317    ,    0.323673    ,    0.324134    ,    0.326197    ,    0.332107    ,    0.34205    ,    0.352264    ,    0.360567    ,    0.364314    ,    0.363647    ,    0.361457    ,    0.364846    ,    0.373651    ,    0.380901    ,    0.382215    ,    0.35661    ,    0.298951    ,    0.253516    ,    0.26044    ,    0.26852    ,    0.296092    ,    0.332928    ,    0.363221    ,    0.362615    ,    0.352251    ,    0.350385    ,    0.353356    ,    0.365638    ,    0.393527    ,    0.428759    ,    0.46637    ,    0.495116    ,    0.516797    ,    0.531958    ,    0.537311    ,    0.538996    ,    0.54517    ,    0.555892    ,    0.565257    ,    0.574185    ,    0.579434    ,    0.587117    ,    0.597439    ,    0.605924    ,    0.609085    ,    0.608662    ,    0.607689    ,    0.607735    ,    0.60562    ,    0.601932    ,    0.597565    ,    0.59456    ,    0.592606    ,    0.586971    ,    0.574907    ,    0.559884    ,    0.553035    ,    0.55201    ,    0.546366    ,    0.532174    ,    0.519702    ,    0.504476    ,    0.493571    ,    0.484603    ,    0.480403    ,    0.474801    ,    0.470291    ,    0.456867    ,    0.431761    ,    0.419479    ,    0.420108    ,    0.435447    ,    0.452888    ,    0.473288    ,    0.481928    ,    0.483458    ,    0.491806    ,    0.503302    ,    0.511485    ,    0.526113    ,    0.536401    ,    0.540741    ,    0.536393    ,    0.525115    ,    0.51368    ,    0.502731    ,    0.489496    ,    0.468137    ,    0.453346    ,    0.453284    ,    0.455254    ,    0.473414    ,    0.4673    ,    0.427615    ,    0.393636    ,    0.376406    ,    0.373109    ,    0.362488    ,    0.354588    ,    0.358966    ,    0.374199    ,    0.38431    ,    0.389753    },
    {    0.301529    ,    0.326833    ,    0.345145    ,    0.352254    ,    0.357384    ,    0.36999    ,    0.37999    ,    0.374949    ,    0.355423    ,    0.345881    ,    0.344701    ,    0.33708    ,    0.320542    ,    0.306986    ,    0.298873    ,    0.289662    ,    0.275177    ,    0.260704    ,    0.254818    ,    0.255791    ,    0.252723    ,    0.243182    ,    0.237194    ,    0.233673    ,    0.231285    ,    0.230115    ,    0.229945    ,    0.229919    ,    0.232094    ,    0.237148    ,    0.243092    ,    0.248897    ,    0.253636    ,    0.257915    ,    0.261624    ,    0.262338    ,    0.262415    ,    0.26598    ,    0.270706    ,    0.276554    ,    0.299455    ,    0.310778    ,    0.281028    ,    0.259567    ,    0.248306    ,    0.276263    ,    0.331383    ,    0.389606    ,    0.408922    ,    0.400789    ,    0.385051    ,    0.374496    ,    0.367054    ,    0.371052    ,    0.382972    ,    0.401766    ,    0.423239    ,    0.443434    ,    0.462411    ,    0.480413    ,    0.494769    ,    0.502319    ,    0.507462    ,    0.512321    ,    0.518696    ,    0.523834    ,    0.53255    ,    0.542934    ,    0.553784    ,    0.563172    ,    0.567692    ,    0.567448    ,    0.568327    ,    0.567148    ,    0.56625    ,    0.567721    ,    0.573813    ,    0.583358    ,    0.589447    ,    0.593693    ,    0.598812    ,    0.60275    ,    0.607574    ,    0.611512    ,    0.613735    ,    0.61242    ,    0.609377    ,    0.603743    ,    0.586041    ,    0.549654    ,    0.505738    ,    0.474015    ,    0.454968    ,    0.444114    ,    0.436133    ,    0.435687    ,    0.447389    ,    0.462659    ,    0.476703    ,    0.490306    ,    0.502731    ,    0.51852    ,    0.535979    ,    0.556155    ,    0.574427    ,    0.588523    ,    0.591039    ,    0.580794    ,    0.562419    ,    0.541487    ,    0.521129    ,    0.502279    ,    0.48036    ,    0.459804    ,    0.455232    ,    0.455942    ,    0.453787    ,    0.436738    ,    0.405917    ,    0.367949    ,    0.329452    ,    0.306462    ,    0.285028    ,    0.262349    ,    0.24944    ,    0.252784    ,    0.263659    ,    0.277148    },
    {    0.200135    ,    0.22461    ,    0.239393    ,    0.248506    ,    0.260743    ,    0.279388    ,    0.291896    ,    0.286152    ,    0.267538    ,    0.259777    ,    0.256963    ,    0.251271    ,    0.241989    ,    0.238123    ,    0.249115    ,    0.264201    ,    0.261927    ,    0.235879    ,    0.210946    ,    0.20142    ,    0.200574    ,    0.200437    ,    0.197804    ,    0.190887    ,    0.182545    ,    0.176581    ,    0.174262    ,    0.171426    ,    0.168362    ,    0.168436    ,    0.169621    ,    0.171861    ,    0.174769    ,    0.178453    ,    0.183444    ,    0.187433    ,    0.190053    ,    0.192996    ,    0.194349    ,    0.192689    ,    0.195505    ,    0.209211    ,    0.203788    ,    0.198748    ,    0.197931    ,    0.199166    ,    0.22792    ,    0.265635    ,    0.300762    ,    0.305294    ,    0.313471    ,    0.321463    ,    0.333888    ,    0.343094    ,    0.353765    ,    0.370876    ,    0.390682    ,    0.409646    ,    0.423892    ,    0.432669    ,    0.442112    ,    0.460645    ,    0.481054    ,    0.492452    ,    0.49748    ,    0.499283    ,    0.499701    ,    0.499937    ,    0.507434    ,    0.52    ,    0.534153    ,    0.543325    ,    0.542413    ,    0.527239    ,    0.502921    ,    0.486253    ,    0.481528    ,    0.485104    ,    0.48921    ,    0.492679    ,    0.497205    ,    0.50078    ,    0.501939    ,    0.506204    ,    0.511832    ,    0.5171    ,    0.524327    ,    0.532778    ,    0.537769    ,    0.529458    ,    0.511513    ,    0.491513    ,    0.469159    ,    0.462465    ,    0.455195    ,    0.448326    ,    0.442562    ,    0.433837    ,    0.441671    ,    0.457008    ,    0.470094    ,    0.481308    ,    0.48354    ,    0.476572    ,    0.460913    ,    0.44684    ,    0.427272    ,    0.401066    ,    0.370213    ,    0.344212    ,    0.328837    ,    0.317097    ,    0.306608    ,    0.29696    ,    0.291711    ,    0.285073    ,    0.279449    ,    0.265097    ,    0.243789    ,    0.227463    ,    0.225288    ,    0.214471    ,    0.199139    ,    0.178204    ,    0.159113    ,    0.158531    ,    0.164413    ,    0.176689    },
    {    0.142953    ,    0.153588    ,    0.165393    ,    0.176597    ,    0.195232    ,    0.210422    ,    0.224962    ,    0.21488    ,    0.192838    ,    0.188997    ,    0.190886    ,    0.192746    ,    0.191533    ,    0.193437    ,    0.209111    ,    0.225961    ,    0.230613    ,    0.214256    ,    0.193953    ,    0.178157    ,    0.168756    ,    0.167462    ,    0.164885    ,    0.158089    ,    0.149237    ,    0.142335    ,    0.139301    ,    0.137632    ,    0.136536    ,    0.137074    ,    0.139666    ,    0.143532    ,    0.14489    ,    0.148687    ,    0.154714    ,    0.157981    ,    0.160059    ,    0.152678    ,    0.152451    ,    0.137822    ,    0.125857    ,    0.151085    ,    0.165064    ,    0.166928    ,    0.175113    ,    0.184923    ,    0.184228    ,    0.184124    ,    0.204158    ,    0.220454    ,    0.229137    ,    0.229604    ,    0.239957    ,    0.252032    ,    0.259843    ,    0.271234    ,    0.283903    ,    0.295889    ,    0.312022    ,    0.324777    ,    0.335548    ,    0.350292    ,    0.368513    ,    0.385267    ,    0.40271    ,    0.418452    ,    0.425732    ,    0.426185    ,    0.429034    ,    0.435422    ,    0.439498    ,    0.437421    ,    0.428984    ,    0.409616    ,    0.38668    ,    0.372346    ,    0.369309    ,    0.368815    ,    0.372685    ,    0.375558    ,    0.373109    ,    0.368023    ,    0.361374    ,    0.353504    ,    0.345824    ,    0.340769    ,    0.338967    ,    0.34091    ,    0.347032    ,    0.356983    ,    0.368639    ,    0.378568    ,    0.387873    ,    0.392355    ,    0.383016    ,    0.359977    ,    0.346346    ,    0.349426    ,    0.354044    ,    0.36501    ,    0.371433    ,    0.372266    ,    0.368498    ,    0.366843    ,    0.355243    ,    0.335069    ,    0.311411    ,    0.288535    ,    0.267891    ,    0.253695    ,    0.248983    ,    0.245598    ,    0.241054    ,    0.239615    ,    0.235452    ,    0.229689    ,    0.213773    ,    0.201094    ,    0.190017    ,    0.187396    ,    0.184404    ,    0.17102    ,    0.14941    ,    0.138674    ,    0.132328    ,    0.126014    ,    0.125909    ,    0.13359    },
    {    0.127595    ,    0.134053    ,    0.140159    ,    0.153719    ,    0.166871    ,    0.178131    ,    0.195682    ,    0.170424    ,    0.139221    ,    0.131737    ,    0.123722    ,    0.125407    ,    0.126928    ,    0.133967    ,    0.142101    ,    0.14809    ,    0.154139    ,    0.157644    ,    0.155241    ,    0.149548    ,    0.144656    ,    0.142914    ,    0.140731    ,    0.137791    ,    0.136844    ,    0.135892    ,    0.134658    ,    0.132292    ,    0.131738    ,    0.135211    ,    0.141928    ,    0.146854    ,    0.150947    ,    0.156218    ,    0.160309    ,    0.153189    ,    0.139539    ,    0.128016    ,    0.124693    ,    0.122355    ,    0.109739    ,    0.134156    ,    0.143902    ,    0.147019    ,    0.156004    ,    0.166574    ,    0.158925    ,    0.152238    ,    0.159422    ,    0.169086    ,    0.169858    ,    0.164868    ,    0.163387    ,    0.166412    ,    0.178451    ,    0.186554    ,    0.194544    ,    0.207183    ,    0.225807    ,    0.241015    ,    0.252802    ,    0.260849    ,    0.267909    ,    0.276451    ,    0.289616    ,    0.303582    ,    0.310671    ,    0.304498    ,    0.3061    ,    0.300453    ,    0.296578    ,    0.290334    ,    0.285759    ,    0.289696    ,    0.285862    ,    0.273907    ,    0.269792    ,    0.272815    ,    0.277922    ,    0.279002    ,    0.271957    ,    0.263277    ,    0.263306    ,    0.260789    ,    0.261062    ,    0.263514    ,    0.265853    ,    0.276513    ,    0.284756    ,    0.290414    ,    0.299804    ,    0.308397    ,    0.305841    ,    0.336177    ,    0.312039    ,    0.308183    ,    0.301994    ,    0.290152    ,    0.294518    ,    0.302254    ,    0.304448    ,    0.298304    ,    0.295776    ,    0.296156    ,    0.289608    ,    0.27432    ,    0.254947    ,    0.234859    ,    0.220964    ,    0.213032    ,    0.210672    ,    0.213743    ,    0.218077    ,    0.219968    ,    0.215651    ,    0.207599    ,    0.196102    ,    0.17814    ,    0.162985    ,    0.155065    ,    0.148339    ,    0.13933    ,    0.131362    ,    0.136189    ,    0.140918    ,    0.141164    ,    0.135421    ,    0.130148    },
    {    0.163731    ,    0.178122    ,    0.16228    ,    0.149323    ,    0.149812    ,    0.144308    ,    0.141093    ,    0.114834    ,    0.092577    ,    0.079144    ,    0.072518    ,    0.073783    ,    0.08158    ,    0.092913    ,    0.10399    ,    0.113068    ,    0.121863    ,    0.128772    ,    0.134391    ,    0.135351    ,    0.132957    ,    0.131277    ,    0.131853    ,    0.134496    ,    0.13751    ,    0.13574    ,    0.130548    ,    0.125382    ,    0.126606    ,    0.132469    ,    0.139726    ,    0.144169    ,    0.148342    ,    0.15484    ,    0.16092    ,    0.152953    ,    0.134195    ,    0.125791    ,    0.148081    ,    0.140836    ,    0.11056    ,    0.094745    ,    0.113494    ,    0.133543    ,    0.148072    ,    0.140761    ,    0.121605    ,    0.113911    ,    0.116337    ,    0.117971    ,    0.120702    ,    0.122411    ,    0.123946    ,    0.124966    ,    0.133718    ,    0.151126    ,    0.168384    ,    0.182542    ,    0.193931    ,    0.200581    ,    0.204011    ,    0.201264    ,    0.204781    ,    0.206499    ,    0.206312    ,    0.211488    ,    0.221743    ,    0.219869    ,    0.201729    ,    0.195969    ,    0.193354    ,    0.197565    ,    0.202763    ,    0.210314    ,    0.2187    ,    0.216475    ,    0.200454    ,    0.206122    ,    0.212394    ,    0.214724    ,    0.207435    ,    0.201706    ,    0.205431    ,    0.214141    ,    0.215228    ,    0.208842    ,    0.216398    ,    0.23456    ,    0.247063    ,    0.25149    ,    0.27152    ,    0.281916    ,    0.278837    ,    0.265388    ,    0.252124    ,    0.251002    ,    0.229074    ,    0.227568    ,    0.235584    ,    0.246671    ,    0.254949    ,    0.261253    ,    0.259946    ,    0.244003    ,    0.230965    ,    0.228243    ,    0.226552    ,    0.212823    ,    0.20162    ,    0.195553    ,    0.196357    ,    0.204734    ,    0.219749    ,    0.228232    ,    0.216211    ,    0.189922    ,    0.164586    ,    0.152845    ,    0.156174    ,    0.152487    ,    0.148329    ,    0.143551    ,    0.145405    ,    0.152012    ,    0.155801    ,    0.158349    ,    0.155028    ,    0.159254    },
    {    0.17446    ,    0.163214    ,    0.149041    ,    0.127361    ,    0.11316    ,    0.108405    ,    0.097694    ,    0.079182    ,    0.067584    ,    0.067127    ,    0.068239    ,    0.071796    ,    0.077559    ,    0.089153    ,    0.103722    ,    0.116266    ,    0.123865    ,    0.127537    ,    0.129865    ,    0.1278    ,    0.126647    ,    0.130406    ,    0.13573    ,    0.138749    ,    0.139995    ,    0.135436    ,    0.126188    ,    0.122222    ,    0.125014    ,    0.128832    ,    0.132964    ,    0.137294    ,    0.141587    ,    0.147355    ,    0.158185    ,    0.146686    ,    0.122702    ,    0.135217    ,    0.132343    ,    0.140933    ,    0.106048    ,    0.080283    ,    0.088086    ,    0.095723    ,    0.10008    ,    0.107947    ,    0.124004    ,    0.117679    ,    0.098098    ,    0.103964    ,    0.109784    ,    0.118663    ,    0.134853    ,    0.142137    ,    0.153191    ,    0.169514    ,    0.185224    ,    0.194116    ,    0.195727    ,    0.19476    ,    0.187536    ,    0.177594    ,    0.169783    ,    0.168293    ,    0.168869    ,    0.172535    ,    0.17972    ,    0.182619    ,    0.168147    ,    0.161006    ,    0.161448    ,    0.167829    ,    0.173533    ,    0.181247    ,    0.195048    ,    0.191239    ,    0.181503    ,    0.185675    ,    0.193813    ,    0.196852    ,    0.199485    ,    0.202504    ,    0.202727    ,    0.197846    ,    0.186804    ,    0.172109    ,    0.166348    ,    0.176964    ,    0.19343    ,    0.200464    ,    0.202043    ,    0.203719    ,    0.21415    ,    0.211786    ,    0.199503    ,    0.194231    ,    0.193057    ,    0.203664    ,    0.209769    ,    0.193234    ,    0.181846    ,    0.176308    ,    0.177705    ,    0.178092    ,    0.180574    ,    0.188006    ,    0.186747    ,    0.179109    ,    0.176802    ,    0.182594    ,    0.192085    ,    0.198423    ,    0.199482    ,    0.198834    ,    0.192314    ,    0.175647    ,    0.176763    ,    0.17695    ,    0.16823    ,    0.168337    ,    0.159758    ,    0.155971    ,    0.163466    ,    0.17188    ,    0.18021    ,    0.168917    ,    0.172384    ,    0.178179    },
    {    0.176852    ,    0.187908    ,    0.180456    ,    0.153668    ,    0.14777    ,    0.140845    ,    0.133061    ,    0.127593    ,    0.127009    ,    0.124183    ,    0.120062    ,    0.113787    ,    0.106383    ,    0.107007    ,    0.110234    ,    0.11563    ,    0.120098    ,    0.122684    ,    0.123672    ,    0.123243    ,    0.124852    ,    0.126523    ,    0.126785    ,    0.13058    ,    0.132819    ,    0.130897    ,    0.129205    ,    0.126961    ,    0.125018    ,    0.127865    ,    0.130102    ,    0.129269    ,    0.129752    ,    0.135483    ,    0.138569    ,    0.112728    ,    0.10032    ,    0.125741    ,    0.120338    ,    0.138181    ,    0.11816    ,    0.074263    ,    0.066644    ,    0.158322    ,    0.239474    ,    0.215862    ,    0.148143    ,    0.11528    ,    0.108916    ,    0.116765    ,    0.11919    ,    0.133037    ,    0.151484    ,    0.181131    ,    0.202362    ,    0.208281    ,    0.211683    ,    0.217005    ,    0.225954    ,    0.214455    ,    0.194812    ,    0.190047    ,    0.18439    ,    0.179305    ,    0.17404    ,    0.164947    ,    0.162662    ,    0.168272    ,    0.164077    ,    0.160379    ,    0.163936    ,    0.180383    ,    0.198531    ,    0.199288    ,    0.194624    ,    0.192462    ,    0.215308    ,    0.202333    ,    0.211577    ,    0.212521    ,    0.213319    ,    0.203328    ,    0.192    ,    0.174742    ,    0.158819    ,    0.154211    ,    0.146037    ,    0.151316    ,    0.146766    ,    0.148469    ,    0.147391    ,    0.147763    ,    0.144461    ,    0.143218    ,    0.142005    ,    0.145987    ,    0.146607    ,    0.14585    ,    0.141784    ,    0.130967    ,    0.13296    ,    0.13579    ,    0.139277    ,    0.143318    ,    0.150638    ,    0.158779    ,    0.159604    ,    0.162274    ,    0.165243    ,    0.168963    ,    0.170578    ,    0.165077    ,    0.160147    ,    0.166671    ,    0.167662    ,    0.161374    ,    0.136851    ,    0.261177    ,    0.221315    ,    0.186622    ,    0.180627    ,    0.16844    ,    0.170881    ,    0.178945    ,    0.19293    ,    0.181631    ,    0.154848    ,    0.145153    },
    {    0.158582    ,    0.146147    ,    0.126959    ,    0.115331    ,    0.124282    ,    0.129681    ,    0.12745    ,    0.127302    ,    0.130228    ,    0.129992    ,    0.128554    ,    0.123713    ,    0.11745    ,    0.118522    ,    0.120871    ,    0.121268    ,    0.122846    ,    0.129808    ,    0.13473    ,    0.133672    ,    0.132702    ,    0.129461    ,    0.12752    ,    0.134024    ,    0.135784    ,    0.13195    ,    0.131658    ,    0.127289    ,    0.122615    ,    0.125617    ,    0.127557    ,    0.122525    ,    0.119484    ,    0.122235    ,    0.111427    ,    0.091978    ,    0.084843    ,    0.12504    ,    0.13871    ,    0.114499    ,    0.100589    ,    0.114311    ,    0.10106    ,    0.159873    ,    0.15585    ,    0.188043    ,    0.199035    ,    0.189468    ,    0.177952    ,    0.167889    ,    0.167661    ,    0.184565    ,    0.202018    ,    0.207539    ,    0.192647    ,    0.166185    ,    0.151399    ,    0.168362    ,    0.206221    ,    0.202361    ,    0.195758    ,    0.17998    ,    0.166942    ,    0.160959    ,    0.155397    ,    0.147599    ,    0.149001    ,    0.192638    ,    0.250605    ,    0.277941    ,    0.288284    ,    0.325144    ,    0.328799    ,    0.296707    ,    0.262036    ,    0.239409    ,    0.216519    ,    0.232668    ,    0.260923    ,    0.260165    ,    0.252167    ,    0.209508    ,    0.177849    ,    0.159705    ,    0.161477    ,    0.157718    ,    0.148709    ,    0.145596    ,    0.134401    ,    0.126672    ,    0.121747    ,    0.12898    ,    0.135265    ,    0.135768    ,    0.131326    ,    0.132442    ,    0.133763    ,    0.131283    ,    0.125968    ,    0.125259    ,    0.128153    ,    0.132164    ,    0.134235    ,    0.132688    ,    0.135336    ,    0.139257    ,    0.142818    ,    0.1414    ,    0.140441    ,    0.144693    ,    0.140743    ,    0.135578    ,    0.146036    ,    0.135355    ,    0.142313    ,    0.141562    ,    0.141758    ,    0.273387    ,    0.309143    ,    0.243312    ,    0.199872    ,    0.187079    ,    0.179574    ,    0.181403    ,    0.193162    ,    0.195671    ,    0.164207    ,    0.147196    },
    {    0.147175    ,    0.138231    ,    0.119577    ,    0.109931    ,    0.110257    ,    0.113636    ,    0.115161    ,    0.117944    ,    0.120595    ,    0.116877    ,    0.115016    ,    0.112485    ,    0.110357    ,    0.111452    ,    0.113767    ,    0.117922    ,    0.120368    ,    0.123453    ,    0.127616    ,    0.128157    ,    0.131067    ,    0.134156    ,    0.13579    ,    0.135351    ,    0.131661    ,    0.127574    ,    0.128474    ,    0.126947    ,    0.12365    ,    0.121094    ,    0.116986    ,    0.112402    ,    0.107962    ,    0.101707    ,    0.093083    ,    0.086576    ,    0.087231    ,    0.1322    ,    0.152606    ,    0.096045    ,    0.113041    ,    0.144576    ,    0.135575    ,    0.085561    ,    0.196866    ,    0.287275    ,    0.173127    ,    0.129346    ,    0.127024    ,    0.135544    ,    0.146102    ,    0.157702    ,    0.159013    ,    0.145637    ,    0.125027    ,    0.111298    ,    0.120664    ,    0.152108    ,    0.180417    ,    0.180854    ,    0.163114    ,    0.150565    ,    0.149529    ,    0.149616    ,    0.145825    ,    0.148928    ,    0.171491    ,    0.217469    ,    0.276124    ,    0.33345    ,    0.312739    ,    0.260872    ,    0.25028    ,    0.258014    ,    0.264653    ,    0.259025    ,    0.218221    ,    0.198745    ,    0.205308    ,    0.213001    ,    0.206718    ,    0.185084    ,    0.167631    ,    0.15515    ,    0.153134    ,    0.146882    ,    0.14279    ,    0.141296    ,    0.142351    ,    0.137646    ,    0.137433    ,    0.144719    ,    0.156247    ,    0.153167    ,    0.139724    ,    0.127289    ,    0.120486    ,    0.113298    ,    0.11249    ,    0.119765    ,    0.121696    ,    0.119622    ,    0.121314    ,    0.128641    ,    0.142037    ,    0.13263    ,    0.1252    ,    0.130774    ,    0.130548    ,    0.128953    ,    0.120164    ,    0.12063    ,    0.134772    ,    0.128673    ,    0.111563    ,    0.12447    ,    0.138711    ,    0.205345    ,    0.287523    ,    0.251565    ,    0.196417    ,    0.184229    ,    0.184115    ,    0.188807    ,    0.196201    ,    0.202575    ,    0.185661    ,    0.158332    },
    {    0.125766    ,    0.108005    ,    0.097747    ,    0.096442    ,    0.099927    ,    0.107675    ,    0.112545    ,    0.112633    ,    0.112725    ,    0.11273    ,    0.11072    ,    0.105574    ,    0.102908    ,    0.101311    ,    0.104463    ,    0.108336    ,    0.110991    ,    0.113683    ,    0.119885    ,    0.123751    ,    0.128812    ,    0.13905    ,    0.143497    ,    0.13803    ,    0.133206    ,    0.13116    ,    0.13035    ,    0.126842    ,    0.116829    ,    0.11217    ,    0.108956    ,    0.104734    ,    0.101809    ,    0.101017    ,    0.097473    ,    0.08583    ,    0.109059    ,    0.14495    ,    0.120845    ,    0.140735    ,    0.101908    ,    0.140709    ,    0.177065    ,    0.13076    ,    0.071487    ,    0.251212    ,    0.137336    ,    0.123801    ,    0.118813    ,    0.125393    ,    0.129333    ,    0.133622    ,    0.127795    ,    0.120527    ,    0.122732    ,    0.13303    ,    0.148591    ,    0.159601    ,    0.156001    ,    0.144585    ,    0.140102    ,    0.142155    ,    0.138509    ,    0.133912    ,    0.138191    ,    0.155831    ,    0.19216    ,    0.239602    ,    0.259562    ,    0.240724    ,    0.206254    ,    0.172132    ,    0.174757    ,    0.15387    ,    0.14257    ,    0.138575    ,    0.130266    ,    0.110991    ,    0.127805    ,    0.149211    ,    0.157604    ,    0.151383    ,    0.146555    ,    0.152605    ,    0.154039    ,    0.148391    ,    0.146964    ,    0.14634    ,    0.139539    ,    0.12368    ,    0.118789    ,    0.116708    ,    0.115128    ,    0.108588    ,    0.100914    ,    0.100449    ,    0.095576    ,    0.094412    ,    0.102252    ,    0.114671    ,    0.116391    ,    0.112233    ,    0.116171    ,    0.140149    ,    0.141643    ,    0.11872    ,    0.124436    ,    0.128728    ,    0.132769    ,    0.124116    ,    0.119324    ,    0.112024    ,    0.10546    ,    0.112068    ,    0.136956    ,    0.154017    ,    0.149727    ,    0.149169    ,    0.167982    ,    0.138628    ,    0.117251    ,    0.118192    ,    0.126306    ,    0.144014    ,    0.166616    ,    0.176943    ,    0.169046    ,    0.151851    },
    {    0.079738    ,    0.082655    ,    0.09328    ,    0.102384    ,    0.105072    ,    0.106469    ,    0.106218    ,    0.105204    ,    0.107161    ,    0.11111    ,    0.1114    ,    0.1093    ,    0.109787    ,    0.110362    ,    0.110962    ,    0.113376    ,    0.116976    ,    0.120121    ,    0.123886    ,    0.129162    ,    0.131811    ,    0.137109    ,    0.139758    ,    0.133343    ,    0.129178    ,    0.126983    ,    0.12246    ,    0.111784    ,    0.101464    ,    0.100043    ,    0.097684    ,    0.096599    ,    0.094639    ,    0.091929    ,    0.083036    ,    0.096208    ,    0.127588    ,    0.15951    ,    0.148372    ,    0.143714    ,    0.117778    ,    0.139626    ,    0.155419    ,    0.147101    ,    0.098713    ,    0.082839    ,    0.116533    ,    0.130697    ,    0.125141    ,    0.118056    ,    0.124351    ,    0.128337    ,    0.126393    ,    0.126357    ,    0.128637    ,    0.129253    ,    0.129885    ,    0.127048    ,    0.122405    ,    0.119911    ,    0.118745    ,    0.117309    ,    0.113513    ,    0.114871    ,    0.124863    ,    0.149229    ,    0.184217    ,    0.199604    ,    0.201681    ,    0.194347    ,    0.203881    ,    0.301951    ,    0.164397    ,    0.141385    ,    0.122026    ,    0.101487    ,    0.082941    ,    0.112078    ,    0.122164    ,    0.123754    ,    0.121017    ,    0.119857    ,    0.118275    ,    0.118446    ,    0.117966    ,    0.112974    ,    0.101765    ,    0.094328    ,    0.093569    ,    0.091895    ,    0.09387    ,    0.097107    ,    0.098879    ,    0.105241    ,    0.109698    ,    0.104707    ,    0.096188    ,    0.104918    ,    0.111724    ,    0.10879    ,    0.115367    ,    0.114875    ,    0.119767    ,    0.135695    ,    0.127297    ,    0.111881    ,    0.114563    ,    0.093861    ,    0.091187    ,    0.07738    ,    0.083712    ,    0.091628    ,    0.127157    ,    0.162804    ,    0.175669    ,    0.146728    ,    0.152407    ,    0.147335    ,    0.123047    ,    0.062654    ,    0.053577    ,    0.062809    ,    0.060384    ,    0.076    ,    0.096297    ,    0.104986    ,    0.108085    ,    0.095308    },
    {    0.086256    ,    0.094286    ,    0.094105    ,    0.095643    ,    0.09798    ,    0.099866    ,    0.106576    ,    0.117003    ,    0.124184    ,    0.126542    ,    0.124992    ,    0.123815    ,    0.127104    ,    0.124466    ,    0.122538    ,    0.123744    ,    0.125503    ,    0.123364    ,    0.122476    ,    0.124174    ,    0.120953    ,    0.116541    ,    0.114178    ,    0.1095    ,    0.105915    ,    0.101017    ,    0.094939    ,    0.086011    ,    0.081266    ,    0.08273    ,    0.08403    ,    0.080796    ,    0.077573    ,    0.081615    ,    0.087035    ,    0.107409    ,    0.136438    ,    0.153609    ,    0.141042    ,    0.131388    ,    0.116656    ,    0.126879    ,    0.198403    ,    0.169361    ,    0.158532    ,    0.113494    ,    0.086411    ,    0.103478    ,    0.093647    ,    0.091365    ,    0.097857    ,    0.102048    ,    0.103349    ,    0.103336    ,    0.100874    ,    0.095739    ,    0.09298    ,    0.09236    ,    0.093047    ,    0.095199    ,    0.094778    ,    0.096761    ,    0.104417    ,    0.112019    ,    0.119726    ,    0.13867    ,    0.156661    ,    0.166982    ,    0.175991    ,    0.165101    ,    0.155847    ,    0.219905    ,    0.163475    ,    0.157109    ,    0.153234    ,    0.132849    ,    0.078578    ,    0.074529    ,    0.08061    ,    0.089466    ,    0.098    ,    0.104302    ,    0.113447    ,    0.111979    ,    0.117619    ,    0.116972    ,    0.117294    ,    0.116817    ,    0.116976    ,    0.12046    ,    0.121487    ,    0.121613    ,    0.122649    ,    0.118154    ,    0.110061    ,    0.104928    ,    0.103979    ,    0.104009    ,    0.09999    ,    0.101722    ,    0.102593    ,    0.103742    ,    0.106619    ,    0.112198    ,    0.111914    ,    0.1164    ,    0.192768    ,    0.115371    ,    0.09796    ,    0.08589    ,    0.104099    ,    0.131061    ,    0.11447    ,    0.169725    ,    0.164661    ,    0.173662    ,    0.1943    ,    0.171443    ,    0.121817    ,    0.053929    ,    0.060226    ,    0.05315    ,    0.056074    ,    0.060744    ,    0.066642    ,    0.068071    ,    0.069462    ,    0.076676    },
    {    0.078321    ,    0.084426    ,    0.089703    ,    0.099676    ,    0.110119    ,    0.118197    ,    0.125253    ,    0.131224    ,    0.133413    ,    0.132237    ,    0.129338    ,    0.12921    ,    0.13166    ,    0.131358    ,    0.130642    ,    0.127231    ,    0.121235    ,    0.117331    ,    0.113915    ,    0.109216    ,    0.103201    ,    0.095419    ,    0.091051    ,    0.084526    ,    0.079023    ,    0.078049    ,    0.077005    ,    0.074048    ,    0.070619    ,    0.071968    ,    0.074364    ,    0.0772    ,    0.080155    ,    0.089552    ,    0.097137    ,    0.109714    ,    0.131426    ,    0.136523    ,    0.158156    ,    0.174153    ,    0.195131    ,    0.092094    ,    0.115147    ,    0.157935    ,    0.160113    ,    0.157362    ,    0.116949    ,    0.147549    ,    0.069375    ,    0.063807    ,    0.066659    ,    0.071476    ,    0.078076    ,    0.082634    ,    0.086854    ,    0.088181    ,    0.086796    ,    0.087708    ,    0.092362    ,    0.09235    ,    0.091547    ,    0.093195    ,    0.100752    ,    0.111116    ,    0.123518    ,    0.131791    ,    0.137287    ,    0.144103    ,    0.149381    ,    0.151565    ,    0.110781    ,    0.191937    ,    0.174811    ,    0.153683    ,    0.147935    ,    0.116804    ,    0.102546    ,    0.075084    ,    0.079211    ,    0.088407    ,    0.114673    ,    0.089431    ,    0.10428    ,    0.12024    ,    0.123024    ,    0.118344    ,    0.117127    ,    0.114521    ,    0.11327    ,    0.111763    ,    0.109156    ,    0.106506    ,    0.102402    ,    0.09502    ,    0.091168    ,    0.093146    ,    0.090596    ,    0.088564    ,    0.089654    ,    0.091033    ,    0.091661    ,    0.099755    ,    0.099199    ,    0.095111    ,    0.093841    ,    0.146881    ,    0.177734    ,    0.160609    ,    0.19211    ,    0.122122    ,    0.123006    ,    0.176125    ,    0.15059    ,    0.17176    ,    0.234657    ,    0.250352    ,    0.211482    ,    0.192278    ,    0.136488    ,    0.070806    ,    0.088637    ,    0.081722    ,    0.076007    ,    0.076706    ,    0.076489    ,    0.078353    ,    0.078577    ,    0.076918    },
    {    0.090369    ,    0.096473    ,    0.10122    ,    0.1087    ,    0.115164    ,    0.121154    ,    0.124239    ,    0.125104    ,    0.126763    ,    0.126963    ,    0.125879    ,    0.125845    ,    0.127562    ,    0.12887    ,    0.124889    ,    0.116027    ,    0.106142    ,    0.097997    ,    0.093273    ,    0.090302    ,    0.087464    ,    0.083326    ,    0.079759    ,    0.078154    ,    0.076263    ,    0.073509    ,    0.07511    ,    0.075495    ,    0.074898    ,    0.078567    ,    0.084373    ,    0.086889    ,    0.084541    ,    0.090542    ,    0.098658    ,    0.1103    ,    0.124006    ,    0.134869    ,    0.149694    ,    0.20378    ,    0.222719    ,    0.112381    ,    0.109143    ,    0.161637    ,    0.215462    ,    0.220418    ,    0.138785    ,    0.160428    ,    0.124459    ,    0.063655    ,    0.066488    ,    0.079804    ,    0.081228    ,    0.0891    ,    0.097139    ,    0.095206    ,    0.095723    ,    0.09688    ,    0.098121    ,    0.098893    ,    0.105676    ,    0.110359    ,    0.114764    ,    0.114999    ,    0.112837    ,    0.120498    ,    0.135357    ,    0.143427    ,    0.144834    ,    0.150092    ,    0.173384    ,    0.17029    ,    0.203115    ,    0.198377    ,    0.174754    ,    0.150899    ,    0.111857    ,    0.07957    ,    0.076854    ,    0.085531    ,    0.137415    ,    0.078494    ,    0.078178    ,    0.086594    ,    0.081603    ,    0.077493    ,    0.079879    ,    0.080183    ,    0.081848    ,    0.079959    ,    0.076012    ,    0.075808    ,    0.075691    ,    0.078636    ,    0.082461    ,    0.0901    ,    0.09637    ,    0.097774    ,    0.095086    ,    0.093529    ,    0.09483    ,    0.09507    ,    0.09108    ,    0.090173    ,    0.107608    ,    0.110781    ,    0.194747    ,    0.284348    ,    0.195627    ,    0.159989    ,    0.194553    ,    0.225468    ,    0.188054    ,    0.205726    ,    0.266819    ,    0.279673    ,    0.262459    ,    0.182147    ,    0.089519    ,    0.080921    ,    0.085401    ,    0.079256    ,    0.078901    ,    0.080305    ,    0.07972    ,    0.078936    ,    0.081049    ,    0.084763    },
    {    0.098278    ,    0.100605    ,    0.104441    ,    0.110003    ,    0.114776    ,    0.11809    ,    0.120274    ,    0.122438    ,    0.126166    ,    0.128364    ,    0.127545    ,    0.126763    ,    0.12814    ,    0.127505    ,    0.121238    ,    0.113201    ,    0.105268    ,    0.098365    ,    0.096464    ,    0.094412    ,    0.093495    ,    0.090922    ,    0.086413    ,    0.089659    ,    0.093862    ,    0.094986    ,    0.096911    ,    0.096443    ,    0.098271    ,    0.097627    ,    0.093714    ,    0.087522    ,    0.088421    ,    0.091939    ,    0.100512    ,    0.107267    ,    0.125734    ,    0.15274    ,    0.13568    ,    0.238543    ,    0.225983    ,    0.137742    ,    0.177681    ,    0.160705    ,    0.181354    ,    0.179515    ,    0.157597    ,    0.143424    ,    0.124919    ,    0.074601    ,    0.080518    ,    0.08811    ,    0.087429    ,    0.087639    ,    0.09041    ,    0.092112    ,    0.09869    ,    0.104481    ,    0.11164    ,    0.115859    ,    0.11763    ,    0.117134    ,    0.117188    ,    0.117719    ,    0.120319    ,    0.127806    ,    0.130508    ,    0.1333    ,    0.125228    ,    0.144503    ,    0.175078    ,    0.213347    ,    0.205734    ,    0.183015    ,    0.169415    ,    0.193156    ,    0.13951    ,    0.074339    ,    0.068063    ,    0.077363    ,    0.113312    ,    0.137175    ,    0.072985    ,    0.065806    ,    0.060304    ,    0.062792    ,    0.067875    ,    0.070487    ,    0.071363    ,    0.072441    ,    0.074396    ,    0.079547    ,    0.085319    ,    0.091917    ,    0.096862    ,    0.104271    ,    0.105625    ,    0.10374    ,    0.100577    ,    0.10143    ,    0.100341    ,    0.096235    ,    0.092975    ,    0.087908    ,    0.082016    ,    0.107494    ,    0.130164    ,    0.148569    ,    0.164168    ,    0.167552    ,    0.177203    ,    0.198005    ,    0.20795    ,    0.247357    ,    0.248286    ,    0.24615    ,    0.222741    ,    0.126658    ,    0.073456    ,    0.080542    ,    0.085441    ,    0.084292    ,    0.085814    ,    0.085834    ,    0.086175    ,    0.086858    ,    0.090871    ,    0.095041    },
    {    0.104865    ,    0.106723    ,    0.110465    ,    0.11533    ,    0.117817    ,    0.120417    ,    0.124043    ,    0.129761    ,    0.133124    ,    0.13452    ,    0.133965    ,    0.133284    ,    0.134828    ,    0.133792    ,    0.12649    ,    0.117836    ,    0.111847    ,    0.108567    ,    0.105893    ,    0.107778    ,    0.110235    ,    0.115772    ,    0.118646    ,    0.117771    ,    0.113384    ,    0.107025    ,    0.103128    ,    0.100253    ,    0.099457    ,    0.096375    ,    0.094855    ,    0.094425    ,    0.097326    ,    0.100525    ,    0.107717    ,    0.117378    ,    0.129142    ,    0.126165    ,    0.141496    ,    0.157204    ,    0.221267    ,    0.148574    ,    0.17158    ,    0.157603    ,    0.159814    ,    0.152323    ,    0.144701    ,    0.140711    ,    0.169067    ,    0.128713    ,    0.101067    ,    0.087545    ,    0.094147    ,    0.096503    ,    0.106509    ,    0.115785    ,    0.123169    ,    0.126627    ,    0.13023    ,    0.130752    ,    0.130911    ,    0.131829    ,    0.133237    ,    0.131901    ,    0.128143    ,    0.129667    ,    0.130139    ,    0.132551    ,    0.138606    ,    0.146915    ,    0.223835    ,    0.237612    ,    0.214773    ,    0.199561    ,    0.225411    ,    0.168518    ,    0.138003    ,    0.109158    ,    0.065795    ,    0.067213    ,    0.082732    ,    0.092191    ,    0.06983    ,    0.073419    ,    0.07134    ,    0.073545    ,    0.078943    ,    0.084661    ,    0.089962    ,    0.095549    ,    0.100429    ,    0.103835    ,    0.106403    ,    0.109579    ,    0.11099    ,    0.111839    ,    0.112529    ,    0.114985    ,    0.114321    ,    0.113469    ,    0.106199    ,    0.096242    ,    0.089598    ,    0.083497    ,    0.074893    ,    0.075816    ,    0.072507    ,    0.10375    ,    0.183291    ,    0.162689    ,    0.149836    ,    0.15095    ,    0.189428    ,    0.16252    ,    0.186273    ,    0.216177    ,    0.128076    ,    0.069859    ,    0.079688    ,    0.086793    ,    0.09211    ,    0.091691    ,    0.094323    ,    0.095942    ,    0.097431    ,    0.098092    ,    0.099647    ,    0.101243    },
    {    0.108307    ,    0.108971    ,    0.111446    ,    0.114959    ,    0.116456    ,    0.118701    ,    0.122113    ,    0.123882    ,    0.125461    ,    0.124903    ,    0.125662    ,    0.125636    ,    0.125428    ,    0.120718    ,    0.115631    ,    0.111986    ,    0.111413    ,    0.119078    ,    0.121452    ,    0.127611    ,    0.131122    ,    0.126539    ,    0.12167    ,    0.115239    ,    0.112596    ,    0.111626    ,    0.111589    ,    0.112565    ,    0.11494    ,    0.113246    ,    0.11077    ,    0.110335    ,    0.110781    ,    0.112333    ,    0.121733    ,    0.129246    ,    0.108371    ,    0.122287    ,    0.18004    ,    0.186985    ,    0.153296    ,    0.197963    ,    0.202964    ,    0.164747    ,    0.166192    ,    0.172602    ,    0.138199    ,    0.150465    ,    0.261175    ,    0.13463    ,    0.125161    ,    0.104918    ,    0.118354    ,    0.12472    ,    0.135514    ,    0.141271    ,    0.147694    ,    0.149306    ,    0.148404    ,    0.146106    ,    0.143652    ,    0.141474    ,    0.138733    ,    0.135512    ,    0.131163    ,    0.133799    ,    0.136303    ,    0.123313    ,    0.127083    ,    0.222879    ,    0.193619    ,    0.204206    ,    0.238313    ,    0.186458    ,    0.182609    ,    0.15699    ,    0.120046    ,    0.117568    ,    0.101763    ,    0.080252    ,    0.092654    ,    0.098376    ,    0.086582    ,    0.092323    ,    0.096066    ,    0.098483    ,    0.101789    ,    0.106091    ,    0.10852    ,    0.110659    ,    0.112552    ,    0.112009    ,    0.110768    ,    0.110526    ,    0.111357    ,    0.113582    ,    0.11587    ,    0.114043    ,    0.10932    ,    0.106665    ,    0.102905    ,    0.09952    ,    0.096412    ,    0.091194    ,    0.08979    ,    0.089827    ,    0.08779    ,    0.085405    ,    0.10884    ,    0.1266    ,    0.139146    ,    0.150697    ,    0.137695    ,    0.088151    ,    0.10275    ,    0.125155    ,    0.070648    ,    0.081452    ,    0.09223    ,    0.09638    ,    0.101561    ,    0.10398    ,    0.105731    ,    0.107018    ,    0.107983    ,    0.107592    ,    0.1074    ,    0.107691    },
    {    0.103538    ,    0.106279    ,    0.107813    ,    0.110286    ,    0.114363    ,    0.114112    ,    0.115201    ,    0.116553    ,    0.117844    ,    0.116308    ,    0.115393    ,    0.115592    ,    0.113447    ,    0.112763    ,    0.1096    ,    0.113413    ,    0.116961    ,    0.115005    ,    0.116383    ,    0.114021    ,    0.111717    ,    0.107902    ,    0.10631    ,    0.107633    ,    0.110698    ,    0.115434    ,    0.12013    ,    0.121588    ,    0.120737    ,    0.11829    ,    0.116191    ,    0.116303    ,    0.117888    ,    0.11985    ,    0.135664    ,    0.133216    ,    0.106349    ,    0.155101    ,    0.192623    ,    0.174538    ,    0.190669    ,    0.204461    ,    0.164663    ,    0.202878    ,    0.143267    ,    0.160397    ,    0.174693    ,    0.144312    ,    0.20558    ,    0.127972    ,    0.196937    ,    0.106249    ,    0.126181    ,    0.130707    ,    0.139016    ,    0.144351    ,    0.148768    ,    0.150021    ,    0.151169    ,    0.153784    ,    0.152798    ,    0.149457    ,    0.147015    ,    0.141143    ,    0.135575    ,    0.133645    ,    0.127525    ,    0.119572    ,    0.107452    ,    0.12317    ,    0.171345    ,    0.178775    ,    0.209975    ,    0.148961    ,    0.170004    ,    0.143345    ,    0.141566    ,    0.106884    ,    0.113683    ,    0.092964    ,    0.087547    ,    0.100364    ,    0.113707    ,    0.111039    ,    0.113511    ,    0.113912    ,    0.115032    ,    0.115405    ,    0.115842    ,    0.114767    ,    0.11404    ,    0.112014    ,    0.110856    ,    0.111007    ,    0.110473    ,    0.110284    ,    0.109551    ,    0.107116    ,    0.103768    ,    0.102226    ,    0.100522    ,    0.099267    ,    0.09809    ,    0.097403    ,    0.097203    ,    0.094842    ,    0.09315    ,    0.09169    ,    0.090937    ,    0.087814    ,    0.10562    ,    0.151097    ,    0.158434    ,    0.076077    ,    0.076035    ,    0.128397    ,    0.082949    ,    0.099277    ,    0.103041    ,    0.103108    ,    0.106219    ,    0.108157    ,    0.107967    ,    0.104689    ,    0.10268    ,    0.101091    ,    0.100508    ,    0.101724    },
    {    0.109286    ,    0.111794    ,    0.116513    ,    0.118236    ,    0.120365    ,    0.120214    ,    0.119926    ,    0.120493    ,    0.120831    ,    0.120014    ,    0.117521    ,    0.116824    ,    0.11345    ,    0.114312    ,    0.109372    ,    0.106929    ,    0.107668    ,    0.101178    ,    0.101556    ,    0.099144    ,    0.096241    ,    0.097352    ,    0.099924    ,    0.103861    ,    0.107628    ,    0.112524    ,    0.115889    ,    0.117893    ,    0.117596    ,    0.116826    ,    0.115278    ,    0.11186    ,    0.109329    ,    0.123154    ,    0.150336    ,    0.126822    ,    0.123279    ,    0.146765    ,    0.218243    ,    0.240076    ,    0.198005    ,    0.204745    ,    0.22834    ,    0.21525    ,    0.258091    ,    0.190884    ,    0.176411    ,    0.14904    ,    0.212751    ,    0.12754    ,    0.114959    ,    0.116841    ,    0.124121    ,    0.131987    ,    0.13843    ,    0.143323    ,    0.148286    ,    0.154428    ,    0.159079    ,    0.160828    ,    0.159022    ,    0.157449    ,    0.149604    ,    0.140644    ,    0.133299    ,    0.122964    ,    0.109659    ,    0.106506    ,    0.089346    ,    0.141291    ,    0.183743    ,    0.160546    ,    0.176967    ,    0.16078    ,    0.168086    ,    0.165671    ,    0.167285    ,    0.131855    ,    0.105015    ,    0.095367    ,    0.101288    ,    0.105216    ,    0.108889    ,    0.108974    ,    0.112833    ,    0.115101    ,    0.113677    ,    0.113997    ,    0.114468    ,    0.114563    ,    0.113671    ,    0.112315    ,    0.11194    ,    0.109706    ,    0.107137    ,    0.104577    ,    0.103616    ,    0.102831    ,    0.102672    ,    0.103672    ,    0.106459    ,    0.108595    ,    0.109593    ,    0.114705    ,    0.122884    ,    0.11783    ,    0.113491    ,    0.112498    ,    0.107145    ,    0.093573    ,    0.087871    ,    0.089087    ,    0.082562    ,    0.083322    ,    0.082585    ,    0.088627    ,    0.099362    ,    0.105152    ,    0.101625    ,    0.101076    ,    0.101509    ,    0.099871    ,    0.101906    ,    0.10229    ,    0.10191    ,    0.102816    ,    0.105455    ,    0.106923    },
    {    0.123263    ,    0.124483    ,    0.126664    ,    0.127917    ,    0.128741    ,    0.128913    ,    0.128821    ,    0.129624    ,    0.129576    ,    0.129607    ,    0.128302    ,    0.128135    ,    0.12756    ,    0.125936    ,    0.125567    ,    0.122289    ,    0.120074    ,    0.116115    ,    0.111691    ,    0.11229    ,    0.115109    ,    0.113601    ,    0.112041    ,    0.113079    ,    0.114525    ,    0.114866    ,    0.115223    ,    0.117406    ,    0.119215    ,    0.122054    ,    0.121469    ,    0.11569    ,    0.115716    ,    0.142414    ,    0.158265    ,    0.125461    ,    0.182837    ,    0.282827    ,    0.290995    ,    0.250487    ,    0.280419    ,    0.318273    ,    0.298978    ,    0.277334    ,    0.27503    ,    0.252959    ,    0.219501    ,    0.224071    ,    0.194973    ,    0.173957    ,    0.134637    ,    0.104107    ,    0.125745    ,    0.130085    ,    0.136584    ,    0.140426    ,    0.145341    ,    0.149625    ,    0.151853    ,    0.15316    ,    0.151655    ,    0.147063    ,    0.140045    ,    0.132963    ,    0.120439    ,    0.107148    ,    0.095236    ,    0.087043    ,    0.084507    ,    0.154861    ,    0.184152    ,    0.215318    ,    0.196053    ,    0.182439    ,    0.196667    ,    0.192656    ,    0.169296    ,    0.122765    ,    0.098546    ,    0.093562    ,    0.094621    ,    0.095621    ,    0.100603    ,    0.105066    ,    0.10924    ,    0.110118    ,    0.110224    ,    0.112969    ,    0.113129    ,    0.111233    ,    0.111803    ,    0.111279    ,    0.111643    ,    0.109025    ,    0.105704    ,    0.102688    ,    0.099888    ,    0.09696    ,    0.097918    ,    0.101913    ,    0.111157    ,    0.122464    ,    0.140052    ,    0.134091    ,    0.132499    ,    0.118782    ,    0.117308    ,    0.114332    ,    0.108032    ,    0.097862    ,    0.096328    ,    0.094339    ,    0.094547    ,    0.111674    ,    0.143813    ,    0.140031    ,    0.140813    ,    0.130032    ,    0.102386    ,    0.100415    ,    0.099947    ,    0.105881    ,    0.105393    ,    0.109494    ,    0.111028    ,    0.114138    ,    0.118139    ,    0.118899    },
    {    0.129388    ,    0.131291    ,    0.131024    ,    0.131621    ,    0.131399    ,    0.130483    ,    0.129334    ,    0.12927    ,    0.129232    ,    0.127993    ,    0.127184    ,    0.12623    ,    0.127025    ,    0.126406    ,    0.12594    ,    0.124711    ,    0.125859    ,    0.125021    ,    0.124907    ,    0.127286    ,    0.130389    ,    0.130938    ,    0.128347    ,    0.126955    ,    0.126    ,    0.124751    ,    0.123913    ,    0.125842    ,    0.126189    ,    0.123274    ,    0.123808    ,    0.126972    ,    0.131475    ,    0.155337    ,    0.165143    ,    0.139556    ,    0.200513    ,    0.363257    ,    0.353234    ,    0.351608    ,    0.415529    ,    0.362252    ,    0.346433    ,    0.33047    ,    0.327397    ,    0.317254    ,    0.28711    ,    0.268867    ,    0.199023    ,    0.171772    ,    0.126812    ,    0.116945    ,    0.120483    ,    0.128079    ,    0.131767    ,    0.134484    ,    0.135641    ,    0.134251    ,    0.13484    ,    0.135122    ,    0.133901    ,    0.129564    ,    0.125476    ,    0.120169    ,    0.113255    ,    0.104668    ,    0.091364    ,    0.085899    ,    0.136722    ,    0.180667    ,    0.232705    ,    0.214026    ,    0.178061    ,    0.181258    ,    0.197144    ,    0.194709    ,    0.173161    ,    0.131751    ,    0.114295    ,    0.094122    ,    0.09546    ,    0.093805    ,    0.094308    ,    0.099161    ,    0.104031    ,    0.108448    ,    0.112001    ,    0.109017    ,    0.109848    ,    0.108829    ,    0.108242    ,    0.109505    ,    0.108775    ,    0.109168    ,    0.106965    ,    0.104302    ,    0.099215    ,    0.095424    ,    0.095886    ,    0.10158    ,    0.112987    ,    0.148154    ,    0.133755    ,    0.108085    ,    0.122102    ,    0.116639    ,    0.105732    ,    0.121154    ,    0.119552    ,    0.109183    ,    0.10974    ,    0.11878    ,    0.148502    ,    0.172385    ,    0.132266    ,    0.140898    ,    0.123101    ,    0.113079    ,    0.110226    ,    0.113837    ,    0.116936    ,    0.121447    ,    0.124101    ,    0.125585    ,    0.126583    ,    0.128355    ,    0.129275    ,    0.128926    },
    {    0.134703    ,    0.132945    ,    0.131266    ,    0.131353    ,    0.130603    ,    0.129223    ,    0.126945    ,    0.124723    ,    0.123772    ,    0.12371    ,    0.124061    ,    0.122135    ,    0.118893    ,    0.119056    ,    0.119927    ,    0.121006    ,    0.123864    ,    0.12133    ,    0.120619    ,    0.12176    ,    0.124951    ,    0.128373    ,    0.128243    ,    0.13106    ,    0.13487    ,    0.137946    ,    0.138024    ,    0.13647    ,    0.140398    ,    0.144222    ,    0.149941    ,    0.1535    ,    0.151498    ,    0.152477    ,    0.146956    ,    0.124342    ,    0.153214    ,    0.275299    ,    0.308205    ,    0.308193    ,    0.407015    ,    0.372984    ,    0.39599    ,    0.377532    ,    0.331491    ,    0.28395    ,    0.189881    ,    0.137185    ,    0.119066    ,    0.115538    ,    0.11242    ,    0.116015    ,    0.120072    ,    0.125022    ,    0.124655    ,    0.124981    ,    0.121768    ,    0.118943    ,    0.117087    ,    0.112264    ,    0.110381    ,    0.106683    ,    0.106738    ,    0.105499    ,    0.105074    ,    0.099795    ,    0.095466    ,    0.099281    ,    0.146329    ,    0.161506    ,    0.208419    ,    0.191218    ,    0.183357    ,    0.183427    ,    0.175631    ,    0.168917    ,    0.201433    ,    0.163419    ,    0.14675    ,    0.112252    ,    0.092009    ,    0.096836    ,    0.097734    ,    0.099302    ,    0.106887    ,    0.107966    ,    0.109066    ,    0.107636    ,    0.105734    ,    0.103834    ,    0.099719    ,    0.100413    ,    0.101148    ,    0.101253    ,    0.10196    ,    0.101486    ,    0.098738    ,    0.097212    ,    0.095128    ,    0.106742    ,    0.142098    ,    0.125659    ,    0.104077    ,    0.126192    ,    0.161359    ,    0.12909    ,    0.137192    ,    0.147908    ,    0.133727    ,    0.115999    ,    0.124356    ,    0.136842    ,    0.147113    ,    0.118279    ,    0.106534    ,    0.106375    ,    0.107053    ,    0.108662    ,    0.114608    ,    0.1196    ,    0.125461    ,    0.128455    ,    0.130859    ,    0.130658    ,    0.131855    ,    0.13275    ,    0.132737    ,    0.133469    },
    {    0.136138    ,    0.134271    ,    0.132711    ,    0.133586    ,    0.132525    ,    0.128446    ,    0.125088    ,    0.122986    ,    0.120551    ,    0.119057    ,    0.118141    ,    0.117358    ,    0.115523    ,    0.117    ,    0.118985    ,    0.120052    ,    0.120733    ,    0.121874    ,    0.122266    ,    0.122675    ,    0.125924    ,    0.128417    ,    0.12868    ,    0.131588    ,    0.133714    ,    0.134398    ,    0.132574    ,    0.13272    ,    0.140139    ,    0.146032    ,    0.15012    ,    0.146352    ,    0.136175    ,    0.123759    ,    0.121037    ,    0.109523    ,    0.132773    ,    0.209419    ,    0.238143    ,    0.266623    ,    0.339059    ,    0.348681    ,    0.355951    ,    0.322942    ,    0.314104    ,    0.342685    ,    0.152172    ,    0.1039    ,    0.111258    ,    0.110672    ,    0.105567    ,    0.103248    ,    0.102174    ,    0.105692    ,    0.10617    ,    0.105529    ,    0.104076    ,    0.101417    ,    0.095908    ,    0.093455    ,    0.093386    ,    0.094219    ,    0.098253    ,    0.098488    ,    0.099324    ,    0.0956    ,    0.096624    ,    0.101036    ,    0.149733    ,    0.179266    ,    0.194543    ,    0.19176    ,    0.190622    ,    0.184472    ,    0.188775    ,    0.187794    ,    0.254454    ,    0.209033    ,    0.194455    ,    0.117953    ,    0.098677    ,    0.090242    ,    0.090156    ,    0.090726    ,    0.094492    ,    0.0996    ,    0.100014    ,    0.101918    ,    0.101579    ,    0.101139    ,    0.099519    ,    0.099191    ,    0.099552    ,    0.098608    ,    0.100353    ,    0.099988    ,    0.098415    ,    0.097071    ,    0.101234    ,    0.12874    ,    0.13473    ,    0.126314    ,    0.114597    ,    0.138723    ,    0.134669    ,    0.124016    ,    0.100804    ,    0.098237    ,    0.11531    ,    0.121249    ,    0.111714    ,    0.101534    ,    0.099129    ,    0.09962    ,    0.099124    ,    0.104677    ,    0.10683    ,    0.108482    ,    0.11243    ,    0.1151    ,    0.120199    ,    0.124201    ,    0.127284    ,    0.129004    ,    0.129933    ,    0.131872    ,    0.133364    ,    0.133795    },
    {    0.117664    ,    0.11746    ,    0.119718    ,    0.120852    ,    0.120194    ,    0.117459    ,    0.115946    ,    0.114702    ,    0.112099    ,    0.110278    ,    0.108912    ,    0.10861    ,    0.108496    ,    0.108536    ,    0.106398    ,    0.107517    ,    0.108259    ,    0.109432    ,    0.110098    ,    0.110899    ,    0.111154    ,    0.113222    ,    0.115071    ,    0.112817    ,    0.11252    ,    0.113287    ,    0.112139    ,    0.112083    ,    0.113359    ,    0.115524    ,    0.118673    ,    0.114081    ,    0.116926    ,    0.115377    ,    0.118303    ,    0.109827    ,    0.109378    ,    0.163545    ,    0.224283    ,    0.297574    ,    0.310429    ,    0.29055    ,    0.252334    ,    0.252319    ,    0.25029    ,    0.177244    ,    0.104268    ,    0.103747    ,    0.100875    ,    0.089986    ,    0.085399    ,    0.082728    ,    0.082978    ,    0.082225    ,    0.08265    ,    0.084433    ,    0.084422    ,    0.087269    ,    0.089662    ,    0.089091    ,    0.091457    ,    0.11343    ,    0.111096    ,    0.104741    ,    0.125542    ,    0.115646    ,    0.110495    ,    0.126888    ,    0.1533    ,    0.171388    ,    0.200115    ,    0.208448    ,    0.188629    ,    0.196374    ,    0.177668    ,    0.204732    ,    0.302705    ,    0.244794    ,    0.198563    ,    0.159833    ,    0.168    ,    0.11559    ,    0.094994    ,    0.0972    ,    0.093068    ,    0.098183    ,    0.099523    ,    0.101076    ,    0.104065    ,    0.101853    ,    0.102129    ,    0.100659    ,    0.101166    ,    0.100349    ,    0.100485    ,    0.098332    ,    0.094395    ,    0.093689    ,    0.10654    ,    0.12281    ,    0.132995    ,    0.096655    ,    0.100292    ,    0.094972    ,    0.109039    ,    0.126701    ,    0.12107    ,    0.100287    ,    0.102758    ,    0.101899    ,    0.101225    ,    0.098606    ,    0.094873    ,    0.095106    ,    0.095407    ,    0.094167    ,    0.094543    ,    0.095838    ,    0.096837    ,    0.099063    ,    0.101226    ,    0.104084    ,    0.106313    ,    0.108282    ,    0.109441    ,    0.11142    ,    0.1143    ,    0.115086    },
    {    0.11677    ,    0.118237    ,    0.120406    ,    0.122631    ,    0.124274    ,    0.123626    ,    0.124253    ,    0.123661    ,    0.123992    ,    0.120732    ,    0.116735    ,    0.115268    ,    0.112844    ,    0.109484    ,    0.103263    ,    0.100463    ,    0.095909    ,    0.091718    ,    0.089753    ,    0.091141    ,    0.092387    ,    0.092358    ,    0.092943    ,    0.091241    ,    0.09099    ,    0.091094    ,    0.095783    ,    0.097921    ,    0.105969    ,    0.11358    ,    0.117831    ,    0.12183    ,    0.121252    ,    0.117593    ,    0.125824    ,    0.134303    ,    0.130843    ,    0.165212    ,    0.199026    ,    0.283985    ,    0.271107    ,    0.198851    ,    0.157788    ,    0.202019    ,    0.100306    ,    0.090811    ,    0.089628    ,    0.098322    ,    0.091751    ,    0.082675    ,    0.078029    ,    0.072227    ,    0.072398    ,    0.069383    ,    0.068386    ,    0.069241    ,    0.069322    ,    0.084264    ,    0.097432    ,    0.106772    ,    0.130001    ,    0.138635    ,    0.153452    ,    0.153473    ,    0.190366    ,    0.168745    ,    0.18759    ,    0.168098    ,    0.139781    ,    0.171811    ,    0.17794    ,    0.224574    ,    0.188356    ,    0.192006    ,    0.22089    ,    0.239368    ,    0.229864    ,    0.248571    ,    0.245113    ,    0.279044    ,    0.185124    ,    0.159195    ,    0.119208    ,    0.10661    ,    0.106155    ,    0.1098    ,    0.109854    ,    0.110181    ,    0.104741    ,    0.10415    ,    0.101152    ,    0.099634    ,    0.113809    ,    0.111594    ,    0.097404    ,    0.095303    ,    0.092112    ,    0.086807    ,    0.090134    ,    0.138555    ,    0.105852    ,    0.103219    ,    0.097149    ,    0.092093    ,    0.091388    ,    0.102092    ,    0.104274    ,    0.10691    ,    0.133337    ,    0.104692    ,    0.098169    ,    0.094315    ,    0.093569    ,    0.091067    ,    0.091665    ,    0.092956    ,    0.090977    ,    0.089209    ,    0.087586    ,    0.088765    ,    0.090115    ,    0.092059    ,    0.095382    ,    0.097949    ,    0.102808    ,    0.106633    ,    0.110225    ,    0.11425    },
    {    0.115826    ,    0.121993    ,    0.125336    ,    0.128389    ,    0.130777    ,    0.130633    ,    0.132155    ,    0.132099    ,    0.13188    ,    0.131093    ,    0.128795    ,    0.126913    ,    0.124668    ,    0.121955    ,    0.118247    ,    0.113329    ,    0.108737    ,    0.10064    ,    0.097426    ,    0.095604    ,    0.095248    ,    0.096909    ,    0.096899    ,    0.096626    ,    0.095226    ,    0.094653    ,    0.096081    ,    0.097297    ,    0.10364    ,    0.10559    ,    0.1128    ,    0.105854    ,    0.093269    ,    0.10897    ,    0.150032    ,    0.185178    ,    0.179628    ,    0.176315    ,    0.198936    ,    0.207594    ,    0.252551    ,    0.174375    ,    0.109422    ,    0.084688    ,    0.085977    ,    0.089211    ,    0.094441    ,    0.096713    ,    0.088857    ,    0.080808    ,    0.07423    ,    0.067942    ,    0.066759    ,    0.067417    ,    0.071251    ,    0.075761    ,    0.081716    ,    0.098473    ,    0.10791    ,    0.132583    ,    0.156643    ,    0.160708    ,    0.143804    ,    0.147637    ,    0.163151    ,    0.176976    ,    0.165029    ,    0.163087    ,    0.16249    ,    0.172075    ,    0.224114    ,    0.21013    ,    0.205197    ,    0.224884    ,    0.314311    ,    0.34719    ,    0.231476    ,    0.210318    ,    0.227492    ,    0.314867    ,    0.186582    ,    0.15748    ,    0.253097    ,    0.113198    ,    0.118636    ,    0.131312    ,    0.137826    ,    0.131536    ,    0.116026    ,    0.114288    ,    0.107042    ,    0.114965    ,    0.160215    ,    0.113041    ,    0.103213    ,    0.097049    ,    0.093622    ,    0.093173    ,    0.103261    ,    0.165258    ,    0.104348    ,    0.125166    ,    0.126084    ,    0.09077    ,    0.085886    ,    0.087702    ,    0.083793    ,    0.100768    ,    0.146225    ,    0.105563    ,    0.091577    ,    0.087899    ,    0.089648    ,    0.088253    ,    0.088331    ,    0.088025    ,    0.085214    ,    0.082348    ,    0.08221    ,    0.082337    ,    0.084731    ,    0.086438    ,    0.090312    ,    0.094148    ,    0.097498    ,    0.102362    ,    0.107245    ,    0.111637    },
    {    0.114136    ,    0.119033    ,    0.124829    ,    0.130195    ,    0.134288    ,    0.138092    ,    0.140024    ,    0.141725    ,    0.141269    ,    0.142067    ,    0.141879    ,    0.141799    ,    0.141416    ,    0.139197    ,    0.137672    ,    0.134248    ,    0.130853    ,    0.122851    ,    0.119486    ,    0.116472    ,    0.115599    ,    0.114089    ,    0.109672    ,    0.104798    ,    0.106173    ,    0.104103    ,    0.105128    ,    0.107446    ,    0.094232    ,    0.091668    ,    0.091732    ,    0.088188    ,    0.101439    ,    0.177092    ,    0.136435    ,    0.109197    ,    0.108542    ,    0.094644    ,    0.093738    ,    0.101426    ,    0.106783    ,    0.098582    ,    0.093471    ,    0.096504    ,    0.1013    ,    0.10146    ,    0.100616    ,    0.098014    ,    0.091414    ,    0.084954    ,    0.078086    ,    0.07341    ,    0.066896    ,    0.067581    ,    0.075069    ,    0.079373    ,    0.08575    ,    0.104874    ,    0.098779    ,    0.144449    ,    0.183321    ,    0.220969    ,    0.186608    ,    0.191718    ,    0.213617    ,    0.196873    ,    0.195524    ,    0.198316    ,    0.202485    ,    0.262195    ,    0.361071    ,    0.346185    ,    0.220517    ,    0.234532    ,    0.259936    ,    0.28534    ,    0.279172    ,    0.322482    ,    0.201449    ,    0.195432    ,    0.129829    ,    0.126701    ,    0.107107    ,    0.100305    ,    0.120197    ,    0.134552    ,    0.147027    ,    0.147653    ,    0.133198    ,    0.124215    ,    0.116091    ,    0.159962    ,    0.183079    ,    0.118214    ,    0.103738    ,    0.097728    ,    0.093474    ,    0.090708    ,    0.105653    ,    0.139382    ,    0.159929    ,    0.138665    ,    0.202265    ,    0.110976    ,    0.082913    ,    0.08394    ,    0.088591    ,    0.135844    ,    0.12554    ,    0.095764    ,    0.086679    ,    0.086484    ,    0.087641    ,    0.086608    ,    0.087453    ,    0.086041    ,    0.085287    ,    0.082635    ,    0.08128    ,    0.080672    ,    0.084134    ,    0.088384    ,    0.091415    ,    0.09566    ,    0.098532    ,    0.102317    ,    0.10585    ,    0.10924    },
    {    0.103308    ,    0.11025    ,    0.118131    ,    0.126695    ,    0.134984    ,    0.142828    ,    0.148681    ,    0.154786    ,    0.159655    ,    0.161913    ,    0.164456    ,    0.166595    ,    0.168137    ,    0.168832    ,    0.170498    ,    0.172413    ,    0.178139    ,    0.180223    ,    0.177963    ,    0.17389    ,    0.170416    ,    0.164007    ,    0.153241    ,    0.129566    ,    0.110936    ,    0.097439    ,    0.101286    ,    0.09352    ,    0.086463    ,    0.131385    ,    0.124228    ,    0.125645    ,    0.166073    ,    0.139998    ,    0.09335    ,    0.099807    ,    0.10584    ,    0.099656    ,    0.095144    ,    0.095323    ,    0.096628    ,    0.097567    ,    0.100737    ,    0.099491    ,    0.098072    ,    0.095625    ,    0.094008    ,    0.092711    ,    0.087091    ,    0.082243    ,    0.079228    ,    0.076416    ,    0.074531    ,    0.07448    ,    0.072996    ,    0.072246    ,    0.079369    ,    0.101783    ,    0.0845    ,    0.131094    ,    0.165183    ,    0.202515    ,    0.286224    ,    0.256749    ,    0.224867    ,    0.230649    ,    0.264238    ,    0.227423    ,    0.196292    ,    0.197998    ,    0.240881    ,    0.303014    ,    0.293172    ,    0.244354    ,    0.24606    ,    0.272131    ,    0.256055    ,    0.227717    ,    0.177229    ,    0.175449    ,    0.2985    ,    0.20997    ,    0.120072    ,    0.081087    ,    0.111791    ,    0.134057    ,    0.147139    ,    0.146461    ,    0.136628    ,    0.130778    ,    0.129745    ,    0.123293    ,    0.176185    ,    0.121505    ,    0.105215    ,    0.10453    ,    0.101448    ,    0.091069    ,    0.116041    ,    0.157617    ,    0.159584    ,    0.187752    ,    0.140409    ,    0.085557    ,    0.082433    ,    0.082447    ,    0.088303    ,    0.105627    ,    0.106434    ,    0.09714    ,    0.099834    ,    0.098502    ,    0.096693    ,    0.092982    ,    0.092128    ,    0.089865    ,    0.086729    ,    0.08472    ,    0.083031    ,    0.081217    ,    0.079336    ,    0.081705    ,    0.082287    ,    0.08319    ,    0.08486    ,    0.087416    ,    0.091727    ,    0.096112    },
    {    0.087608    ,    0.095608    ,    0.10317    ,    0.112716    ,    0.122469    ,    0.133176    ,    0.144503    ,    0.155061    ,    0.164886    ,    0.172961    ,    0.180972    ,    0.187439    ,    0.193646    ,    0.199138    ,    0.206293    ,    0.21565    ,    0.227671    ,    0.236726    ,    0.238379    ,    0.232863    ,    0.222497    ,    0.214801    ,    0.202812    ,    0.165577    ,    0.13027    ,    0.109682    ,    0.113798    ,    0.115558    ,    0.164462    ,    0.177655    ,    0.128533    ,    0.113236    ,    0.141859    ,    0.095392    ,    0.083673    ,    0.089443    ,    0.085836    ,    0.092827    ,    0.094786    ,    0.11867    ,    0.096183    ,    0.105782    ,    0.10387    ,    0.101677    ,    0.099426    ,    0.097893    ,    0.096179    ,    0.090982    ,    0.087385    ,    0.085152    ,    0.08391    ,    0.083042    ,    0.084723    ,    0.084419    ,    0.079003    ,    0.072612    ,    0.065259    ,    0.083189    ,    0.102742    ,    0.138477    ,    0.151693    ,    0.173239    ,    0.172267    ,    0.147562    ,    0.147496    ,    0.155772    ,    0.211998    ,    0.232301    ,    0.173742    ,    0.197609    ,    0.168809    ,    0.174826    ,    0.211515    ,    0.188167    ,    0.201006    ,    0.206128    ,    0.224994    ,    0.195982    ,    0.115032    ,    0.216887    ,    0.202811    ,    0.12071    ,    0.177428    ,    0.124415    ,    0.083123    ,    0.118232    ,    0.128936    ,    0.135687    ,    0.130381    ,    0.125596    ,    0.118633    ,    0.169356    ,    0.200785    ,    0.161508    ,    0.11212    ,    0.093561    ,    0.099566    ,    0.099561    ,    0.14301    ,    0.175308    ,    0.200568    ,    0.196566    ,    0.108096    ,    0.068295    ,    0.073855    ,    0.080637    ,    0.088985    ,    0.105458    ,    0.103435    ,    0.103344    ,    0.110052    ,    0.110041    ,    0.106632    ,    0.103523    ,    0.101646    ,    0.10179    ,    0.098781    ,    0.093375    ,    0.086824    ,    0.082019    ,    0.078796    ,    0.076044    ,    0.074373    ,    0.072484    ,    0.071542    ,    0.072381    ,    0.07593    ,    0.079881    },
    {    0.076072    ,    0.077644    ,    0.083476    ,    0.092728    ,    0.10333    ,    0.116314    ,    0.129674    ,    0.143814    ,    0.15733    ,    0.169089    ,    0.180124    ,    0.187082    ,    0.191359    ,    0.196077    ,    0.203398    ,    0.214143    ,    0.22674    ,    0.237968    ,    0.243661    ,    0.242277    ,    0.233771    ,    0.218098    ,    0.197982    ,    0.175987    ,    0.146089    ,    0.109264    ,    0.121754    ,    0.148473    ,    0.161456    ,    0.121876    ,    0.108612    ,    0.086935    ,    0.104696    ,    0.080408    ,    0.099982    ,    0.121083    ,    0.143002    ,    0.123923    ,    0.117134    ,    0.109311    ,    0.106434    ,    0.103774    ,    0.100669    ,    0.102368    ,    0.104729    ,    0.110479    ,    0.110772    ,    0.107814    ,    0.106324    ,    0.096931    ,    0.091261    ,    0.090276    ,    0.094506    ,    0.100165    ,    0.10448    ,    0.100971    ,    0.091236    ,    0.083103    ,    0.105304    ,    0.09371    ,    0.11406    ,    0.110904    ,    0.110281    ,    0.109722    ,    0.106227    ,    0.112692    ,    0.180569    ,    0.183958    ,    0.166685    ,    0.15464    ,    0.146399    ,    0.159655    ,    0.127402    ,    0.139386    ,    0.135396    ,    0.161095    ,    0.195697    ,    0.185683    ,    0.174764    ,    0.218331    ,    0.247956    ,    0.185279    ,    0.169852    ,    0.137976    ,    0.122316    ,    0.095299    ,    0.111023    ,    0.12263    ,    0.124404    ,    0.12085    ,    0.126977    ,    0.136225    ,    0.217059    ,    0.201874    ,    0.183643    ,    0.103545    ,    0.083853    ,    0.114741    ,    0.205408    ,    0.173872    ,    0.159122    ,    0.170838    ,    0.158121    ,    0.129473    ,    0.102085    ,    0.087864    ,    0.09121    ,    0.093709    ,    0.102893    ,    0.106476    ,    0.112536    ,    0.112249    ,    0.107803    ,    0.106237    ,    0.106977    ,    0.108074    ,    0.109237    ,    0.107891    ,    0.104017    ,    0.098943    ,    0.09591    ,    0.094187    ,    0.094498    ,    0.091191    ,    0.087786    ,    0.084739    ,    0.080124    ,    0.076997    },
    {    0.098642    ,    0.088403    ,    0.080441    ,    0.07568    ,    0.076434    ,    0.085468    ,    0.099423    ,    0.115962    ,    0.130801    ,    0.144212    ,    0.155641    ,    0.166458    ,    0.174376    ,    0.179212    ,    0.18359    ,    0.186289    ,    0.191144    ,    0.19899    ,    0.203699    ,    0.206617    ,    0.206362    ,    0.196713    ,    0.176224    ,    0.162507    ,    0.136397    ,    0.108215    ,    0.132685    ,    0.173299    ,    0.171898    ,    0.131254    ,    0.10602    ,    0.09292    ,    0.083854    ,    0.084997    ,    0.083451    ,    0.084939    ,    0.101433    ,    0.096048    ,    0.09694    ,    0.099183    ,    0.103378    ,    0.102784    ,    0.104181    ,    0.10746    ,    0.106709    ,    0.113283    ,    0.119558    ,    0.125106    ,    0.125161    ,    0.121313    ,    0.118324    ,    0.108908    ,    0.098894    ,    0.093882    ,    0.094645    ,    0.104611    ,    0.11986    ,    0.11535    ,    0.099068    ,    0.084587    ,    0.079268    ,    0.090953    ,    0.087405    ,    0.087735    ,    0.096778    ,    0.106266    ,    0.177118    ,    0.143407    ,    0.14856    ,    0.188358    ,    0.133992    ,    0.126708    ,    0.145455    ,    0.147543    ,    0.127439    ,    0.139477    ,    0.164376    ,    0.14189    ,    0.226248    ,    0.190777    ,    0.153428    ,    0.122217    ,    0.102857    ,    0.137487    ,    0.128998    ,    0.11518    ,    0.123239    ,    0.133234    ,    0.137396    ,    0.186881    ,    0.251678    ,    0.160386    ,    0.17449    ,    0.235608    ,    0.273956    ,    0.227835    ,    0.18829    ,    0.193287    ,    0.248878    ,    0.205702    ,    0.149488    ,    0.186778    ,    0.186336    ,    0.22147    ,    0.227831    ,    0.181404    ,    0.124316    ,    0.086903    ,    0.10102    ,    0.111043    ,    0.104448    ,    0.104142    ,    0.101566    ,    0.101985    ,    0.104994    ,    0.113285    ,    0.119924    ,    0.119358    ,    0.117843    ,    0.116596    ,    0.115915    ,    0.115666    ,    0.11585    ,    0.117872    ,    0.11788    ,    0.116508    ,    0.113062    ,    0.106535    },
    {    0.113129    ,    0.106047    ,    0.09942    ,    0.08961    ,    0.081074    ,    0.075305    ,    0.07578    ,    0.082949    ,    0.093879    ,    0.105839    ,    0.118274    ,    0.13108    ,    0.141219    ,    0.14925    ,    0.154394    ,    0.156179    ,    0.155594    ,    0.158568    ,    0.166193    ,    0.172135    ,    0.173073    ,    0.166134    ,    0.16158    ,    0.16364    ,    0.154686    ,    0.179403    ,    0.176035    ,    0.210436    ,    0.216692    ,    0.147199    ,    0.117957    ,    0.097781    ,    0.083536    ,    0.087302    ,    0.091273    ,    0.082652    ,    0.079632    ,    0.078069    ,    0.0872    ,    0.084352    ,    0.088336    ,    0.096238    ,    0.099784    ,    0.103471    ,    0.108972    ,    0.112303    ,    0.11508    ,    0.119995    ,    0.117926    ,    0.125546    ,    0.131883    ,    0.129003    ,    0.132493    ,    0.130536    ,    0.114412    ,    0.094866    ,    0.091977    ,    0.109555    ,    0.136527    ,    0.154037    ,    0.100749    ,    0.073609    ,    0.092927    ,    0.095218    ,    0.096583    ,    0.087193    ,    0.089853    ,    0.110295    ,    0.135028    ,    0.128311    ,    0.128263    ,    0.132453    ,    0.113589    ,    0.113415    ,    0.109296    ,    0.129805    ,    0.170893    ,    0.153227    ,    0.16515    ,    0.135317    ,    0.130264    ,    0.122194    ,    0.138391    ,    0.164635    ,    0.144221    ,    0.127059    ,    0.154905    ,    0.181494    ,    0.155374    ,    0.155456    ,    0.176721    ,    0.248132    ,    0.208398    ,    0.157347    ,    0.178052    ,    0.139987    ,    0.124741    ,    0.113465    ,    0.186256    ,    0.193919    ,    0.177722    ,    0.158916    ,    0.147654    ,    0.165516    ,    0.24232    ,    0.176597    ,    0.149388    ,    0.115337    ,    0.092818    ,    0.107477    ,    0.112585    ,    0.099066    ,    0.101292    ,    0.106584    ,    0.113202    ,    0.117746    ,    0.123507    ,    0.124179    ,    0.125569    ,    0.126974    ,    0.12983    ,    0.130817    ,    0.132073    ,    0.133443    ,    0.13268    ,    0.128326    ,    0.124391    ,    0.11897    },
    {    0.111488    ,    0.102201    ,    0.096262    ,    0.089866    ,    0.084516    ,    0.082441    ,    0.083346    ,    0.083522    ,    0.084655    ,    0.085993    ,    0.092852    ,    0.105789    ,    0.11676    ,    0.126814    ,    0.135386    ,    0.142708    ,    0.147212    ,    0.146325    ,    0.144531    ,    0.150815    ,    0.155952    ,    0.147839    ,    0.153104    ,    0.148515    ,    0.21463    ,    0.194091    ,    0.29043    ,    0.224883    ,    0.230384    ,    0.199627    ,    0.142885    ,    0.129582    ,    0.122791    ,    0.124576    ,    0.108969    ,    0.093654    ,    0.077162    ,    0.066989    ,    0.065184    ,    0.066022    ,    0.067094    ,    0.073085    ,    0.081793    ,    0.088465    ,    0.096028    ,    0.09551    ,    0.099872    ,    0.110491    ,    0.112074    ,    0.115568    ,    0.117761    ,    0.125695    ,    0.135274    ,    0.139846    ,    0.145462    ,    0.133942    ,    0.110253    ,    0.089754    ,    0.103937    ,    0.156945    ,    0.136326    ,    0.080531    ,    0.080996    ,    0.083126    ,    0.072955    ,    0.084154    ,    0.086301    ,    0.10547    ,    0.115859    ,    0.115022    ,    0.098987    ,    0.091767    ,    0.07767    ,    0.056213    ,    0.057622    ,    0.079088    ,    0.123276    ,    0.135973    ,    0.122362    ,    0.109957    ,    0.13629    ,    0.194135    ,    0.15109    ,    0.12274    ,    0.148394    ,    0.151094    ,    0.157203    ,    0.235475    ,    0.179187    ,    0.168093    ,    0.240086    ,    0.208112    ,    0.199543    ,    0.172348    ,    0.334243    ,    0.38854    ,    0.291105    ,    0.302645    ,    0.222992    ,    0.23127    ,    0.202942    ,    0.166449    ,    0.168843    ,    0.133188    ,    0.131517    ,    0.128319    ,    0.1458    ,    0.104745    ,    0.115756    ,    0.096871    ,    0.09829    ,    0.103279    ,    0.107655    ,    0.109275    ,    0.114211    ,    0.11781    ,    0.123876    ,    0.129402    ,    0.130618    ,    0.130094    ,    0.131286    ,    0.131493    ,    0.131288    ,    0.130222    ,    0.127901    ,    0.123673    ,    0.119826    ,    0.115976    },
    {    0.122018    ,    0.111429    ,    0.099358    ,    0.089196    ,    0.084076    ,    0.085233    ,    0.086101    ,    0.082064    ,    0.07774    ,    0.073628    ,    0.070732    ,    0.077868    ,    0.089088    ,    0.100748    ,    0.112497    ,    0.125064    ,    0.137279    ,    0.14239    ,    0.1448    ,    0.144987    ,    0.139227    ,    0.146069    ,    0.168676    ,    0.148939    ,    0.26033    ,    0.193359    ,    0.232154    ,    0.280115    ,    0.235881    ,    0.197459    ,    0.18997    ,    0.159209    ,    0.138293    ,    0.188583    ,    0.146565    ,    0.141595    ,    0.121147    ,    0.103816    ,    0.09411    ,    0.088487    ,    0.079883    ,    0.076976    ,    0.073786    ,    0.071604    ,    0.072008    ,    0.076993    ,    0.087126    ,    0.104477    ,    0.119472    ,    0.123868    ,    0.129073    ,    0.137126    ,    0.136249    ,    0.143574    ,    0.152704    ,    0.153581    ,    0.144335    ,    0.114641    ,    0.090839    ,    0.126    ,    0.214935    ,    0.121819    ,    0.083603    ,    0.107541    ,    0.110293    ,    0.103207    ,    0.095458    ,    0.092371    ,    0.077292    ,    0.06913    ,    0.05432    ,    0.046165    ,    0.060876    ,    0.040269    ,    0.047894    ,    0.039285    ,    0.086916    ,    0.117954    ,    0.107684    ,    0.078224    ,    0.102228    ,    0.111528    ,    0.113172    ,    0.117342    ,    0.13319    ,    0.146351    ,    0.131266    ,    0.150832    ,    0.161801    ,    0.177759    ,    0.207135    ,    0.193011    ,    0.204651    ,    0.250704    ,    0.232981    ,    0.220108    ,    0.224299    ,    0.268151    ,    0.259045    ,    0.247291    ,    0.201644    ,    0.1408    ,    0.169168    ,    0.149416    ,    0.141037    ,    0.14033    ,    0.123864    ,    0.131862    ,    0.114245    ,    0.100115    ,    0.093725    ,    0.119644    ,    0.121163    ,    0.119109    ,    0.123608    ,    0.127571    ,    0.137535    ,    0.145619    ,    0.150991    ,    0.152926    ,    0.152216    ,    0.150019    ,    0.145349    ,    0.14292    ,    0.139786    ,    0.135721    ,    0.131717    ,    0.128415    },
    {    0.138705    ,    0.127795    ,    0.115503    ,    0.103389    ,    0.098722    ,    0.095529    ,    0.088038    ,    0.083345    ,    0.079861    ,    0.073639    ,    0.062995    ,    0.060977    ,    0.07359    ,    0.084901    ,    0.095263    ,    0.111095    ,    0.128101    ,    0.143665    ,    0.155176    ,    0.148534    ,    0.14051    ,    0.148063    ,    0.148594    ,    0.288585    ,    0.257119    ,    0.202114    ,    0.189783    ,    0.243726    ,    0.220499    ,    0.227094    ,    0.214855    ,    0.170447    ,    0.172675    ,    0.160923    ,    0.172969    ,    0.172507    ,    0.178337    ,    0.173529    ,    0.160455    ,    0.145574    ,    0.146869    ,    0.15356    ,    0.149282    ,    0.139536    ,    0.127988    ,    0.126747    ,    0.132872    ,    0.141411    ,    0.154832    ,    0.170539    ,    0.180994    ,    0.18484    ,    0.179299    ,    0.177586    ,    0.177623    ,    0.177102    ,    0.17139    ,    0.147242    ,    0.118721    ,    0.109039    ,    0.183478    ,    0.196352    ,    0.132574    ,    0.091637    ,    0.096007    ,    0.114405    ,    0.109633    ,    0.091604    ,    0.07298    ,    0.060404    ,    0.060152    ,    0.051431    ,    0.058272    ,    0.090388    ,    0.089225    ,    0.091112    ,    0.093806    ,    0.120662    ,    0.129611    ,    0.112776    ,    0.117179    ,    0.10723    ,    0.161344    ,    0.230809    ,    0.121474    ,    0.143624    ,    0.155453    ,    0.103167    ,    0.182738    ,    0.150096    ,    0.12426    ,    0.181283    ,    0.266817    ,    0.305874    ,    0.245187    ,    0.195845    ,    0.202289    ,    0.199828    ,    0.203234    ,    0.178245    ,    0.197176    ,    0.148796    ,    0.172467    ,    0.167471    ,    0.166898    ,    0.15522    ,    0.128709    ,    0.143036    ,    0.123517    ,    0.116175    ,    0.11631    ,    0.115681    ,    0.140364    ,    0.140387    ,    0.161506    ,    0.174458    ,    0.186283    ,    0.191507    ,    0.193004    ,    0.192992    ,    0.192341    ,    0.189239    ,    0.182209    ,    0.175196    ,    0.167774    ,    0.161295    ,    0.156201    ,    0.150212    },
    {    0.143818    ,    0.140739    ,    0.136144    ,    0.126452    ,    0.116133    ,    0.105088    ,    0.094144    ,    0.084961    ,    0.078103    ,    0.070823    ,    0.059169    ,    0.050955    ,    0.05885    ,    0.075687    ,    0.093841    ,    0.114379    ,    0.134688    ,    0.151253    ,    0.162651    ,    0.166297    ,    0.155861    ,    0.162955    ,    0.188327    ,    0.138284    ,    0.207222    ,    0.223025    ,    0.188376    ,    0.176592    ,    0.229625    ,    0.240863    ,    0.28804    ,    0.228584    ,    0.212918    ,    0.198119    ,    0.205135    ,    0.204225    ,    0.201296    ,    0.206833    ,    0.236957    ,    0.245885    ,    0.261445    ,    0.25333    ,    0.248343    ,    0.237301    ,    0.224269    ,    0.217684    ,    0.215304    ,    0.215046    ,    0.218237    ,    0.238735    ,    0.252757    ,    0.265826    ,    0.270139    ,    0.255172    ,    0.239431    ,    0.218642    ,    0.195089    ,    0.162609    ,    0.144138    ,    0.135403    ,    0.17068    ,    0.176271    ,    0.12545    ,    0.130625    ,    0.153012    ,    0.114736    ,    0.087831    ,    0.08206    ,    0.064715    ,    0.091304    ,    0.090039    ,    0.078368    ,    0.103876    ,    0.125484    ,    0.094198    ,    0.136081    ,    0.130636    ,    0.12472    ,    0.126047    ,    0.132151    ,    0.135133    ,    0.123328    ,    0.188035    ,    0.156613    ,    0.119509    ,    0.105321    ,    0.157327    ,    0.154762    ,    0.097764    ,    0.140204    ,    0.124454    ,    0.151249    ,    0.104983    ,    0.094602    ,    0.090202    ,    0.106415    ,    0.138433    ,    0.109498    ,    0.132056    ,    0.130506    ,    0.130742    ,    0.120948    ,    0.154963    ,    0.153904    ,    0.139979    ,    0.157813    ,    0.135095    ,    0.127522    ,    0.11576    ,    0.12342    ,    0.146198    ,    0.139427    ,    0.146634    ,    0.15511    ,    0.168921    ,    0.209233    ,    0.233828    ,    0.237455    ,    0.232803    ,    0.228711    ,    0.22509    ,    0.218999    ,    0.209425    ,    0.196895    ,    0.182541    ,    0.169134    ,    0.16185    ,    0.152943    },
    {    0.130772    ,    0.129667    ,    0.131965    ,    0.130117    ,    0.124755    ,    0.119435    ,    0.110545    ,    0.098331    ,    0.086818    ,    0.081765    ,    0.073494    ,    0.069345    ,    0.076743    ,    0.095046    ,    0.116864    ,    0.132956    ,    0.149747    ,    0.163533    ,    0.168523    ,    0.172469    ,    0.144686    ,    0.12852    ,    0.151074    ,    0.167899    ,    0.203885    ,    0.214289    ,    0.177813    ,    0.163951    ,    0.235213    ,    0.261625    ,    0.240754    ,    0.232045    ,    0.205556    ,    0.190079    ,    0.192579    ,    0.180486    ,    0.177661    ,    0.185191    ,    0.22745    ,    0.252885    ,    0.27479    ,    0.283656    ,    0.291155    ,    0.299045    ,    0.278413    ,    0.25931    ,    0.255316    ,    0.287977    ,    0.267105    ,    0.276063    ,    0.277745    ,    0.296717    ,    0.340126    ,    0.339585    ,    0.317247    ,    0.292677    ,    0.242669    ,    0.185013    ,    0.154019    ,    0.155377    ,    0.174476    ,    0.10757    ,    0.111212    ,    0.184015    ,    0.100971    ,    0.094468    ,    0.081011    ,    0.080942    ,    0.095551    ,    0.104066    ,    0.057704    ,    0.084148    ,    0.14029    ,    0.120081    ,    0.123117    ,    0.118995    ,    0.102222    ,    0.154534    ,    0.163939    ,    0.161246    ,    0.134651    ,    0.106135    ,    0.234162    ,    0.173795    ,    0.120518    ,    0.127215    ,    0.138141    ,    0.122034    ,    0.155514    ,    0.127215    ,    0.101232    ,    0.113736    ,    0.13132    ,    0.089417    ,    0.118889    ,    0.142575    ,    0.121425    ,    0.126222    ,    0.166376    ,    0.14379    ,    0.155    ,    0.175176    ,    0.185856    ,    0.180246    ,    0.177345    ,    0.14662    ,    0.129472    ,    0.15687    ,    0.120455    ,    0.115532    ,    0.159579    ,    0.161365    ,    0.155651    ,    0.143214    ,    0.148247    ,    0.197878    ,    0.258496    ,    0.275128    ,    0.261993    ,    0.243325    ,    0.223185    ,    0.205771    ,    0.191731    ,    0.177964    ,    0.164156    ,    0.151293    ,    0.146593    ,    0.139097    },
    {    0.121962    ,    0.124899    ,    0.128958    ,    0.13057    ,    0.128858    ,    0.126742    ,    0.123598    ,    0.117511    ,    0.111695    ,    0.107335    ,    0.101942    ,    0.099447    ,    0.103833    ,    0.116492    ,    0.134964    ,    0.149343    ,    0.160741    ,    0.167115    ,    0.168886    ,    0.178147    ,    0.15686    ,    0.100665    ,    0.099881    ,    0.126424    ,    0.204883    ,    0.163348    ,    0.159438    ,    0.159488    ,    0.189118    ,    0.225328    ,    0.225631    ,    0.234068    ,    0.236049    ,    0.212898    ,    0.178426    ,    0.17997    ,    0.164701    ,    0.160639    ,    0.144045    ,    0.143251    ,    0.175934    ,    0.204239    ,    0.21559    ,    0.289006    ,    0.335814    ,    0.276224    ,    0.263116    ,    0.31797    ,    0.30268    ,    0.314928    ,    0.338472    ,    0.343056    ,    0.382649    ,    0.407066    ,    0.395369    ,    0.375997    ,    0.347219    ,    0.259063    ,    0.190654    ,    0.167444    ,    0.169372    ,    0.149058    ,    0.139887    ,    0.124397    ,    0.094595    ,    0.090581    ,    0.090904    ,    0.10081    ,    0.131206    ,    0.165711    ,    0.14439    ,    0.126314    ,    0.163618    ,    0.173202    ,    0.189457    ,    0.122909    ,    0.096248    ,    0.097404    ,    0.12676    ,    0.17787    ,    0.16675    ,    0.124146    ,    0.161709    ,    0.146142    ,    0.14727    ,    0.158854    ,    0.163918    ,    0.162298    ,    0.134027    ,    0.133505    ,    0.171523    ,    0.149233    ,    0.126165    ,    0.124841    ,    0.167402    ,    0.183537    ,    0.171434    ,    0.17255    ,    0.185262    ,    0.207171    ,    0.187441    ,    0.20957    ,    0.207544    ,    0.18742    ,    0.196884    ,    0.165853    ,    0.172442    ,    0.151897    ,    0.141172    ,    0.118567    ,    0.109694    ,    0.102175    ,    0.112909    ,    0.110206    ,    0.111395    ,    0.181388    ,    0.170781    ,    0.164199    ,    0.147448    ,    0.140265    ,    0.12625    ,    0.110425    ,    0.10158    ,    0.105501    ,    0.108288    ,    0.108362    ,    0.110302    ,    0.116441    },
    {    0.175069    ,    0.172392    ,    0.167057    ,    0.161834    ,    0.158422    ,    0.155434    ,    0.152378    ,    0.151197    ,    0.152823    ,    0.15195    ,    0.146423    ,    0.140798    ,    0.139685    ,    0.145236    ,    0.153956    ,    0.160105    ,    0.166133    ,    0.172528    ,    0.176276    ,    0.182088    ,    0.182385    ,    0.113636    ,    0.112352    ,    0.121939    ,    0.128918    ,    0.126135    ,    0.137265    ,    0.148004    ,    0.159066    ,    0.172508    ,    0.198525    ,    0.218614    ,    0.224541    ,    0.268535    ,    0.272974    ,    0.24984    ,    0.243788    ,    0.222331    ,    0.178961    ,    0.143074    ,    0.134454    ,    0.137934    ,    0.156994    ,    0.212196    ,    0.229755    ,    0.201909    ,    0.240407    ,    0.331267    ,    0.349379    ,    0.350789    ,    0.400586    ,    0.434049    ,    0.457109    ,    0.477745    ,    0.478373    ,    0.473403    ,    0.464164    ,    0.390986    ,    0.303475    ,    0.228284    ,    0.188924    ,    0.175498    ,    0.164368    ,    0.133433    ,    0.119228    ,    0.115226    ,    0.141225    ,    0.143301    ,    0.162867    ,    0.17716    ,    0.197826    ,    0.183326    ,    0.174421    ,    0.185721    ,    0.217422    ,    0.208249    ,    0.19088    ,    0.174182    ,    0.191788    ,    0.200794    ,    0.183169    ,    0.177022    ,    0.177613    ,    0.18987    ,    0.175829    ,    0.183974    ,    0.215263    ,    0.215416    ,    0.19702    ,    0.192368    ,    0.185296    ,    0.176904    ,    0.158248    ,    0.148125    ,    0.211215    ,    0.244848    ,    0.20182    ,    0.214851    ,    0.21863    ,    0.233512    ,    0.199785    ,    0.164376    ,    0.163487    ,    0.174041    ,    0.179474    ,    0.175206    ,    0.159313    ,    0.149431    ,    0.153905    ,    0.141986    ,    0.124847    ,    0.126095    ,    0.115981    ,    0.14536    ,    0.152119    ,    0.246091    ,    0.298974    ,    0.253436    ,    0.212916    ,    0.189432    ,    0.175105    ,    0.171515    ,    0.165884    ,    0.162628    ,    0.160961    ,    0.165393    ,    0.17354    ,    0.175997    },
    {    0.230863    ,    0.218024    ,    0.207641    ,    0.202464    ,    0.20017    ,    0.200637    ,    0.200711    ,    0.20188    ,    0.205706    ,    0.208767    ,    0.208032    ,    0.203449    ,    0.19816    ,    0.195134    ,    0.193753    ,    0.188983    ,    0.184113    ,    0.184266    ,    0.189266    ,    0.166225    ,    0.159669    ,    0.115115    ,    0.102806    ,    0.109533    ,    0.113499    ,    0.116644    ,    0.11943    ,    0.116688    ,    0.136969    ,    0.175084    ,    0.209205    ,    0.228795    ,    0.250017    ,    0.267044    ,    0.24242    ,    0.260422    ,    0.216057    ,    0.200991    ,    0.221932    ,    0.184244    ,    0.165544    ,    0.19557    ,    0.196232    ,    0.173428    ,    0.173418    ,    0.220048    ,    0.298983    ,    0.344181    ,    0.354555    ,    0.338884    ,    0.366747    ,    0.415918    ,    0.439346    ,    0.455112    ,    0.481456    ,    0.527938    ,    0.564531    ,    0.506527    ,    0.397829    ,    0.268983    ,    0.20308    ,    0.189298    ,    0.16952    ,    0.159762    ,    0.163556    ,    0.146758    ,    0.152871    ,    0.161357    ,    0.175168    ,    0.16906    ,    0.199626    ,    0.202278    ,    0.192907    ,    0.206238    ,    0.223009    ,    0.228602    ,    0.239411    ,    0.241421    ,    0.247067    ,    0.236813    ,    0.204445    ,    0.195062    ,    0.21581    ,    0.18462    ,    0.20049    ,    0.215936    ,    0.218732    ,    0.206861    ,    0.191487    ,    0.187479    ,    0.197874    ,    0.194515    ,    0.184745    ,    0.191048    ,    0.199525    ,    0.205866    ,    0.193378    ,    0.193177    ,    0.162954    ,    0.19275    ,    0.188464    ,    0.18001    ,    0.179099    ,    0.193022    ,    0.183598    ,    0.188156    ,    0.185977    ,    0.164182    ,    0.172739    ,    0.169117    ,    0.156646    ,    0.142804    ,    0.157927    ,    0.146214    ,    0.165834    ,    0.286726    ,    0.444943    ,    0.438674    ,    0.37705    ,    0.300937    ,    0.261539    ,    0.25897    ,    0.257441    ,    0.245578    ,    0.243813    ,    0.250148    ,    0.251582    ,    0.242395    },
    {    0.3106    ,    0.291657    ,    0.272386    ,    0.261918    ,    0.257879    ,    0.257486    ,    0.257227    ,    0.256732    ,    0.257745    ,    0.262322    ,    0.264378    ,    0.261597    ,    0.254398    ,    0.247451    ,    0.239331    ,    0.22279    ,    0.20812    ,    0.206928    ,    0.205186    ,    0.161483    ,    0.132628    ,    0.11619    ,    0.116413    ,    0.119034    ,    0.106078    ,    0.132046    ,    0.140304    ,    0.151014    ,    0.168234    ,    0.201283    ,    0.219036    ,    0.215308    ,    0.217557    ,    0.205326    ,    0.2299    ,    0.241766    ,    0.259395    ,    0.209897    ,    0.19894    ,    0.128359    ,    0.147454    ,    0.162992    ,    0.17545    ,    0.180919    ,    0.221231    ,    0.317527    ,    0.394726    ,    0.400488    ,    0.402209    ,    0.414537    ,    0.43647    ,    0.44649    ,    0.457856    ,    0.489775    ,    0.519096    ,    0.536769    ,    0.537026    ,    0.493164    ,    0.389301    ,    0.246126    ,    0.200047    ,    0.209701    ,    0.221793    ,    0.212848    ,    0.2269    ,    0.233415    ,    0.214813    ,    0.205683    ,    0.20697    ,    0.201665    ,    0.203924    ,    0.208012    ,    0.228919    ,    0.209162    ,    0.210864    ,    0.223551    ,    0.252733    ,    0.247916    ,    0.251387    ,    0.226605    ,    0.201031    ,    0.175143    ,    0.169409    ,    0.176875    ,    0.18075    ,    0.190442    ,    0.201527    ,    0.187781    ,    0.191675    ,    0.205866    ,    0.222677    ,    0.236422    ,    0.236539    ,    0.239788    ,    0.22016    ,    0.188352    ,    0.180637    ,    0.17809    ,    0.186092    ,    0.218636    ,    0.193298    ,    0.171815    ,    0.186439    ,    0.194011    ,    0.203282    ,    0.212684    ,    0.189327    ,    0.173096    ,    0.177178    ,    0.191098    ,    0.193518    ,    0.17505    ,    0.186596    ,    0.207411    ,    0.257607    ,    0.392369    ,    0.571204    ,    0.611589    ,    0.536659    ,    0.427906    ,    0.333871    ,    0.339297    ,    0.342453    ,    0.332494    ,    0.337717    ,    0.34021    ,    0.336035    ,    0.3261    },
    {    0.421116    ,    0.399744    ,    0.381373    ,    0.374416    ,    0.384088    ,    0.383364    ,    0.359819    ,    0.340676    ,    0.326731    ,    0.317104    ,    0.310146    ,    0.30185    ,    0.288247    ,    0.274229    ,    0.256353    ,    0.239301    ,    0.243194    ,    0.260625    ,    0.241266    ,    0.173112    ,    0.180251    ,    0.152666    ,    0.133764    ,    0.130573    ,    0.128136    ,    0.14074    ,    0.185835    ,    0.196265    ,    0.203117    ,    0.206985    ,    0.219329    ,    0.217362    ,    0.231257    ,    0.238849    ,    0.269259    ,    0.297909    ,    0.292591    ,    0.228845    ,    0.215721    ,    0.224649    ,    0.215202    ,    0.225074    ,    0.230048    ,    0.263644    ,    0.366119    ,    0.425643    ,    0.440934    ,    0.434479    ,    0.439478    ,    0.449313    ,    0.454809    ,    0.451123    ,    0.456767    ,    0.465248    ,    0.463831    ,    0.435856    ,    0.381324    ,    0.327251    ,    0.256891    ,    0.199709    ,    0.181983    ,    0.227942    ,    0.274698    ,    0.262718    ,    0.241487    ,    0.237466    ,    0.244964    ,    0.237352    ,    0.233813    ,    0.249424    ,    0.231545    ,    0.229009    ,    0.233006    ,    0.22613    ,    0.223043    ,    0.221051    ,    0.189294    ,    0.19083    ,    0.191134    ,    0.165988    ,    0.180144    ,    0.179165    ,    0.179749    ,    0.187194    ,    0.200755    ,    0.186363    ,    0.199682    ,    0.221667    ,    0.242559    ,    0.241355    ,    0.252841    ,    0.257916    ,    0.267424    ,    0.263116    ,    0.235223    ,    0.210521    ,    0.189104    ,    0.183755    ,    0.193608    ,    0.203673    ,    0.183682    ,    0.230465    ,    0.204907    ,    0.211529    ,    0.210862    ,    0.209553    ,    0.198149    ,    0.217063    ,    0.212786    ,    0.217913    ,    0.232578    ,    0.215066    ,    0.228561    ,    0.357423    ,    0.506906    ,    0.553831    ,    0.687118    ,    0.709298    ,    0.632241    ,    0.507592    ,    0.388901    ,    0.37246    ,    0.416983    ,    0.42168    ,    0.439172    ,    0.44772    ,    0.44629    ,    0.437497    },
    {    0.538769    ,    0.531336    ,    0.530904    ,    0.531523    ,    0.525613    ,    0.524995    ,    0.529568    ,    0.500452    ,    0.445477    ,    0.397277    ,    0.368155    ,    0.354659    ,    0.332915    ,    0.313829    ,    0.301337    ,    0.297608    ,    0.321723    ,    0.251635    ,    0.251338    ,    0.225188    ,    0.196441    ,    0.192136    ,    0.17515    ,    0.176846    ,    0.165303    ,    0.175956    ,    0.195398    ,    0.196642    ,    0.189333    ,    0.212147    ,    0.207897    ,    0.221715    ,    0.247064    ,    0.337042    ,    0.361011    ,    0.373926    ,    0.364274    ,    0.324342    ,    0.287041    ,    0.293527    ,    0.296347    ,    0.297264    ,    0.345147    ,    0.453528    ,    0.494333    ,    0.474099    ,    0.471723    ,    0.454326    ,    0.448662    ,    0.441185    ,    0.442869    ,    0.443559    ,    0.425881    ,    0.369424    ,    0.326573    ,    0.306184    ,    0.288891    ,    0.240529    ,    0.185502    ,    0.160446    ,    0.163263    ,    0.188845    ,    0.241076    ,    0.305772    ,    0.298318    ,    0.297999    ,    0.306163    ,    0.307167    ,    0.319966    ,    0.29538    ,    0.299149    ,    0.30097    ,    0.279965    ,    0.260976    ,    0.247677    ,    0.228413    ,    0.20123    ,    0.180388    ,    0.181526    ,    0.169237    ,    0.167275    ,    0.169232    ,    0.187144    ,    0.194978    ,    0.225165    ,    0.222219    ,    0.228687    ,    0.247836    ,    0.263715    ,    0.269776    ,    0.273506    ,    0.267824    ,    0.260433    ,    0.25959    ,    0.269639    ,    0.26122    ,    0.242952    ,    0.229425    ,    0.226726    ,    0.217791    ,    0.242859    ,    0.250664    ,    0.246108    ,    0.221571    ,    0.219496    ,    0.233543    ,    0.224332    ,    0.233588    ,    0.256255    ,    0.272183    ,    0.262312    ,    0.266744    ,    0.275536    ,    0.362039    ,    0.565533    ,    0.599247    ,    0.648225    ,    0.642785    ,    0.594687    ,    0.54234    ,    0.469862    ,    0.428069    ,    0.461305    ,    0.469253    ,    0.505504    ,    0.521079    ,    0.536232    ,    0.541813    },
    {    0.597517    ,    0.587426    ,    0.561479    ,    0.542793    ,    0.539686    ,    0.552679    ,    0.557883    ,    0.561197    ,    0.518941    ,    0.491362    ,    0.461801    ,    0.457482    ,    0.424539    ,    0.412013    ,    0.390821    ,    0.401417    ,    0.302788    ,    0.304446    ,    0.2984    ,    0.28227    ,    0.284196    ,    0.255776    ,    0.251402    ,    0.224556    ,    0.230437    ,    0.241351    ,    0.243632    ,    0.231026    ,    0.222664    ,    0.233379    ,    0.252259    ,    0.338636    ,    0.375279    ,    0.417797    ,    0.442546    ,    0.451227    ,    0.439949    ,    0.437174    ,    0.41377    ,    0.391497    ,    0.36378    ,    0.404661    ,    0.478773    ,    0.59467    ,    0.563487    ,    0.515786    ,    0.485845    ,    0.457647    ,    0.444654    ,    0.530142    ,    0.512578    ,    0.475677    ,    0.449641    ,    0.39326    ,    0.331987    ,    0.316077    ,    0.3247    ,    0.322752    ,    0.309209    ,    0.29136    ,    0.27222    ,    0.28032    ,    0.326756    ,    0.379273    ,    0.400073    ,    0.397988    ,    0.359342    ,    0.320038    ,    0.317126    ,    0.320682    ,    0.33717    ,    0.34924    ,    0.327166    ,    0.292042    ,    0.270365    ,    0.235128    ,    0.217977    ,    0.209477    ,    0.201112    ,    0.195434    ,    0.19607    ,    0.20733    ,    0.231676    ,    0.230577    ,    0.246815    ,    0.253637    ,    0.260584    ,    0.285744    ,    0.293633    ,    0.290459    ,    0.298669    ,    0.299333    ,    0.30217    ,    0.293246    ,    0.301747    ,    0.283389    ,    0.279896    ,    0.30083    ,    0.281429    ,    0.281382    ,    0.287249    ,    0.286309    ,    0.291727    ,    0.291985    ,    0.306037    ,    0.271753    ,    0.278303    ,    0.294697    ,    0.288562    ,    0.277159    ,    0.289277    ,    0.279756    ,    0.312775    ,    0.375718    ,    0.403152    ,    0.436918    ,    0.444824    ,    0.438391    ,    0.445054    ,    0.481202    ,    0.465124    ,    0.452661    ,    0.505541    ,    0.526931    ,    0.55043    ,    0.555035    ,    0.56548    ,    0.578832    },
    {    0.554719    ,    0.522221    ,    0.490381    ,    0.501025    ,    0.528746    ,    0.566215    ,    0.573034    ,    0.549907    ,    0.524576    ,    0.501814    ,    0.482506    ,    0.484719    ,    0.445712    ,    0.471729    ,    0.458209    ,    0.457664    ,    0.376984    ,    0.361267    ,    0.364349    ,    0.337161    ,    0.316802    ,    0.281504    ,    0.268974    ,    0.280174    ,    0.284824    ,    0.294356    ,    0.288773    ,    0.288353    ,    0.291708    ,    0.299465    ,    0.335295    ,    0.397676    ,    0.436717    ,    0.460599    ,    0.480906    ,    0.486038    ,    0.484843    ,    0.509037    ,    0.466925    ,    0.441763    ,    0.439793    ,    0.41818    ,    0.522767    ,    0.569403    ,    0.606646    ,    0.521829    ,    0.365583    ,    0.30935    ,    0.307329    ,    0.467624    ,    0.542384    ,    0.610254    ,    0.626644    ,    0.587279    ,    0.539835    ,    0.531738    ,    0.569265    ,    0.599076    ,    0.618676    ,    0.629079    ,    0.607304    ,    0.577801    ,    0.576153    ,    0.597597    ,    0.631058    ,    0.633132    ,    0.574615    ,    0.454993    ,    0.372516    ,    0.348391    ,    0.344763    ,    0.353556    ,    0.317415    ,    0.284738    ,    0.261468    ,    0.25431    ,    0.248611    ,    0.230749    ,    0.231339    ,    0.245331    ,    0.244019    ,    0.253086    ,    0.270763    ,    0.281095    ,    0.294659    ,    0.314137    ,    0.331691    ,    0.344196    ,    0.348018    ,    0.380634    ,    0.360742    ,    0.362842    ,    0.356876    ,    0.34745    ,    0.344218    ,    0.323147    ,    0.313448    ,    0.311982    ,    0.307096    ,    0.314031    ,    0.318905    ,    0.324007    ,    0.332659    ,    0.33102    ,    0.342007    ,    0.349482    ,    0.359384    ,    0.356919    ,    0.36325    ,    0.354576    ,    0.346963    ,    0.347846    ,    0.357688    ,    0.386981    ,    0.398104    ,    0.419177    ,    0.437648    ,    0.447377    ,    0.463514    ,    0.476591    ,    0.491157    ,    0.519233    ,    0.549775    ,    0.561165    ,    0.563928    ,    0.554157    ,    0.548539    ,    0.557871    },
    {    0.618044    ,    0.625484    ,    0.638368    ,    0.670611    ,    0.687975    ,    0.644848    ,    0.608277    ,    0.576698    ,    0.543088    ,    0.519843    ,    0.503503    ,    0.487684    ,    0.470121    ,    0.474996    ,    0.472577    ,    0.461325    ,    0.441826    ,    0.393525    ,    0.371984    ,    0.367275    ,    0.360096    ,    0.348442    ,    0.341447    ,    0.334852    ,    0.328062    ,    0.328774    ,    0.334227    ,    0.345013    ,    0.366278    ,    0.389628    ,    0.410755    ,    0.421955    ,    0.443865    ,    0.484379    ,    0.499822    ,    0.488253    ,    0.512527    ,    0.565606    ,    0.571536    ,    0.555786    ,    0.508558    ,    0.442833    ,    0.398891    ,    0.424768    ,    0.519287    ,    0.486128    ,    0.313034    ,    0.263675    ,    0.260867    ,    0.273914    ,    0.400574    ,    0.500318    ,    0.540161    ,    0.575805    ,    0.560355    ,    0.558137    ,    0.61502    ,    0.653924    ,    0.716234    ,    0.79421    ,    0.866464    ,    0.820559    ,    0.797438    ,    0.807198    ,    0.825012    ,    0.83658    ,    0.82442    ,    0.736951    ,    0.570129    ,    0.4319    ,    0.368735    ,    0.324831    ,    0.314749    ,    0.306691    ,    0.299761    ,    0.297476    ,    0.284558    ,    0.278627    ,    0.284148    ,    0.284835    ,    0.307906    ,    0.313501    ,    0.32447    ,    0.336545    ,    0.361474    ,    0.396468    ,    0.401567    ,    0.399683    ,    0.414389    ,    0.431888    ,    0.398797    ,    0.394832    ,    0.380099    ,    0.363183    ,    0.34603    ,    0.331166    ,    0.322941    ,    0.321987    ,    0.329758    ,    0.349905    ,    0.361651    ,    0.38712    ,    0.394186    ,    0.415543    ,    0.412288    ,    0.402763    ,    0.40335    ,    0.399281    ,    0.403057    ,    0.414845    ,    0.398929    ,    0.417218    ,    0.403418    ,    0.404359    ,    0.416548    ,    0.429412    ,    0.438091    ,    0.465601    ,    0.500062    ,    0.499111    ,    0.505309    ,    0.536008    ,    0.562021    ,    0.586612    ,    0.61798    ,    0.630821    ,    0.642363    ,    0.662576    },
    {    0.691977    ,    0.669605    ,    0.677125    ,    0.721582    ,    0.762904    ,    0.813856    ,    0.711815    ,    0.692725    ,    0.631653    ,    0.602177    ,    0.589256    ,    0.587627    ,    0.584626    ,    0.56586    ,    0.55586    ,    0.562258    ,    0.502235    ,    0.448065    ,    0.434877    ,    0.417447    ,    0.410654    ,    0.413069    ,    0.421951    ,    0.406566    ,    0.404153    ,    0.419773    ,    0.413465    ,    0.429419    ,    0.428609    ,    0.483638    ,    0.465265    ,    0.466151    ,    0.485948    ,    0.521383    ,    0.531021    ,    0.537734    ,    0.526004    ,    0.573005    ,    0.551698    ,    0.542926    ,    0.504961    ,    0.416772    ,    0.341729    ,    0.348654    ,    0.394513    ,    0.318175    ,    0.303808    ,    0.255392    ,    0.2643    ,    0.306063    ,    0.279364    ,    0.21072    ,    0.200616    ,    0.217251    ,    0.353482    ,    0.453236    ,    0.632516    ,    0.769115    ,    0.861993    ,    0.960862    ,    1.003169    ,    0.992828    ,    0.995624    ,    1.000607    ,    0.98585    ,    0.975574    ,    0.968229    ,    0.918384    ,    0.800778    ,    0.645814    ,    0.514944    ,    0.437369    ,    0.408869    ,    0.361776    ,    0.35751    ,    0.358111    ,    0.372912    ,    0.396626    ,    0.402729    ,    0.395654    ,    0.368429    ,    0.390343    ,    0.440514    ,    0.460679    ,    0.475202    ,    0.48029    ,    0.444715    ,    0.438824    ,    0.440894    ,    0.4342    ,    0.416757    ,    0.40513    ,    0.391481    ,    0.382544    ,    0.377944    ,    0.371333    ,    0.368999    ,    0.367316    ,    0.390996    ,    0.418653    ,    0.426212    ,    0.434546    ,    0.430545    ,    0.415153    ,    0.425556    ,    0.424627    ,    0.434806    ,    0.444305    ,    0.441085    ,    0.451033    ,    0.455133    ,    0.471185    ,    0.489007    ,    0.493973    ,    0.483497    ,    0.486531    ,    0.484086    ,    0.491035    ,    0.50111    ,    0.511895    ,    0.527748    ,    0.530215    ,    0.571487    ,    0.603446    ,    0.633293    ,    0.660645    ,    0.691649    ,    0.715102    },
    {    0.596567    ,    0.623152    ,    0.668113    ,    0.703244    ,    0.721745    ,    0.727038    ,    0.715719    ,    0.718325    ,    0.706735    ,    0.703568    ,    0.682673    ,    0.662916    ,    0.661977    ,    0.647633    ,    0.634498    ,    0.621701    ,    0.60411    ,    0.577134    ,    0.54588    ,    0.510783    ,    0.493685    ,    0.464094    ,    0.462206    ,    0.454736    ,    0.446804    ,    0.454855    ,    0.457461    ,    0.512274    ,    0.607302    ,    0.554447    ,    0.512045    ,    0.577462    ,    0.625434    ,    0.566358    ,    0.565989    ,    0.554055    ,    0.525082    ,    0.479423    ,    0.458419    ,    0.459519    ,    0.459473    ,    0.427512    ,    0.388596    ,    0.365084    ,    0.333217    ,    0.26811    ,    0.211207    ,    0.21335    ,    0.27507    ,    0.333797    ,    0.34413    ,    0.302186    ,    0.238735    ,    0.174338    ,    0.161084    ,    0.19094    ,    0.318615    ,    0.545905    ,    0.775499    ,    0.948635    ,    1.096051    ,    1.140087    ,    1.138649    ,    1.128123    ,    1.106821    ,    1.078041    ,    1.048644    ,    1.000828    ,    0.929033    ,    0.871208    ,    0.821176    ,    0.776363    ,    0.734266    ,    0.704702    ,    0.679387    ,    0.695413    ,    0.70595    ,    0.64649    ,    0.56628    ,    0.488513    ,    0.448342    ,    0.479107    ,    0.542875    ,    0.606878    ,    0.629905    ,    0.597474    ,    0.582796    ,    0.531987    ,    0.497814    ,    0.47841    ,    0.453924    ,    0.447146    ,    0.459078    ,    0.456522    ,    0.457792    ,    0.460157    ,    0.467794    ,    0.472289    ,    0.46604    ,    0.462854    ,    0.468619    ,    0.474501    ,    0.486151    ,    0.493799    ,    0.492573    ,    0.495236    ,    0.508292    ,    0.509957    ,    0.513711    ,    0.5179    ,    0.507859    ,    0.531783    ,    0.541    ,    0.563361    ,    0.554336    ,    0.55812    ,    0.54749    ,    0.53168    ,    0.531599    ,    0.525471    ,    0.4999    ,    0.487666    ,    0.489741    ,    0.498222    ,    0.510838    ,    0.527565    ,    0.5523    ,    0.568623    },
    {    0.557727    ,    0.574943    ,    0.594051    ,    0.61263    ,    0.630248    ,    0.643028    ,    0.652787    ,    0.660541    ,    0.66841    ,    0.675872    ,    0.676073    ,    0.669406    ,    0.662173    ,    0.652207    ,    0.637928    ,    0.622664    ,    0.603508    ,    0.587397    ,    0.57536    ,    0.552901    ,    0.543814    ,    0.517201    ,    0.50418    ,    0.48949    ,    0.495266    ,    0.497663    ,    0.584691    ,    0.579242    ,    0.609376    ,    0.613043    ,    0.625897    ,    0.627217    ,    0.624404    ,    0.585926    ,    0.540558    ,    0.485787    ,    0.541676    ,    0.507355    ,    0.509519    ,    0.513014    ,    0.510451    ,    0.501911    ,    0.487556    ,    0.452395    ,    0.374073    ,    0.220146    ,    0.176273    ,    0.263623    ,    0.336356    ,    0.392639    ,    0.384863    ,    0.348315    ,    0.303739    ,    0.21813    ,    0.181178    ,    0.256245    ,    0.313222    ,    0.368899    ,    0.495831    ,    0.705804    ,    0.912742    ,    1.089421    ,    1.19791    ,    1.234055    ,    1.197782    ,    1.143706    ,    1.096665    ,    1.051064    ,    1.014577    ,    0.973351    ,    0.921024    ,    0.877509    ,    0.849434    ,    0.822952    ,    0.812199    ,    0.787537    ,    0.733185    ,    0.661424    ,    0.594448    ,    0.529572    ,    0.474965    ,    0.476595    ,    0.515105    ,    0.556659    ,    0.570968    ,    0.585396    ,    0.56067    ,    0.548736    ,    0.507415    ,    0.504989    ,    0.487081    ,    0.477799    ,    0.480542    ,    0.50818    ,    0.491797    ,    0.505927    ,    0.515932    ,    0.51656    ,    0.522507    ,    0.530015    ,    0.541634    ,    0.55211    ,    0.563348    ,    0.571792    ,    0.567707    ,    0.544192    ,    0.561759    ,    0.558786    ,    0.575908    ,    0.568578    ,    0.560065    ,    0.561698    ,    0.560347    ,    0.538327    ,    0.537156    ,    0.523696    ,    0.520301    ,    0.502924    ,    0.498605    ,    0.500721    ,    0.499109    ,    0.495684    ,    0.491544    ,    0.490446    ,    0.493356    ,    0.503659    ,    0.522746    ,    0.539968    },
    {    0.584236    ,    0.587638    ,    0.593593    ,    0.601229    ,    0.609929    ,    0.62068    ,    0.628417    ,    0.634384    ,    0.640373    ,    0.644708    ,    0.648989    ,    0.651852    ,    0.650248    ,    0.64522    ,    0.63791    ,    0.627394    ,    0.615861    ,    0.600461    ,    0.589297    ,    0.578087    ,    0.558847    ,    0.542812    ,    0.556635    ,    0.563115    ,    0.532304    ,    0.57133    ,    0.592431    ,    0.572329    ,    0.56252    ,    0.560533    ,    0.550509    ,    0.560678    ,    0.574568    ,    0.621568    ,    0.571156    ,    0.513677    ,    0.551375    ,    0.452215    ,    0.445034    ,    0.426765    ,    0.39944    ,    0.383316    ,    0.363465    ,    0.271805    ,    0.192294    ,    0.193345    ,    0.242058    ,    0.286772    ,    0.329349    ,    0.341873    ,    0.333423    ,    0.294523    ,    0.226505    ,    0.173312    ,    0.151517    ,    0.182284    ,    0.223838    ,    0.371628    ,    0.428938    ,    0.528721    ,    0.642671    ,    0.808862    ,    0.97611    ,    1.096754    ,    1.149308    ,    1.123495    ,    1.082987    ,    1.031201    ,    0.967394    ,    0.858705    ,    0.812463    ,    0.788665    ,    0.738117    ,    0.755009    ,    0.761793    ,    0.731258    ,    0.671359    ,    0.629225    ,    0.617125    ,    0.627255    ,    0.637406    ,    0.651859    ,    0.685654    ,    0.72043    ,    0.749175    ,    0.732121    ,    0.756997    ,    0.811772    ,    0.7882    ,    0.774028    ,    0.732435    ,    0.675721    ,    0.639166    ,    0.616466    ,    0.593734    ,    0.568274    ,    0.550661    ,    0.553675    ,    0.550232    ,    0.555234    ,    0.586904    ,    0.598824    ,    0.578735    ,    0.593117    ,    0.57144    ,    0.557567    ,    0.57131    ,    0.571086    ,    0.56477    ,    0.555209    ,    0.546393    ,    0.540665    ,    0.534173    ,    0.516201    ,    0.517415    ,    0.522157    ,    0.516113    ,    0.512721    ,    0.516905    ,    0.52762    ,    0.538392    ,    0.54493    ,    0.5526    ,    0.558039    ,    0.562363    ,    0.568716    ,    0.573355    ,    0.576975    },
    {    0.632167    ,    0.634038    ,    0.636067    ,    0.639748    ,    0.643281    ,    0.646039    ,    0.644661    ,    0.639943    ,    0.633705    ,    0.629008    ,    0.625542    ,    0.623705    ,    0.622438    ,    0.616833    ,    0.610527    ,    0.605607    ,    0.600537    ,    0.596199    ,    0.59255    ,    0.586672    ,    0.580964    ,    0.577272    ,    0.583704    ,    0.594661    ,    0.604076    ,    0.621716    ,    0.627267    ,    0.625642    ,    0.616069    ,    0.606007    ,    0.587611    ,    0.512274    ,    0.471798    ,    0.389819    ,    0.354307    ,    0.372843    ,    0.355884    ,    0.362634    ,    0.365021    ,    0.38763    ,    0.322557    ,    0.306401    ,    0.246152    ,    0.212286    ,    0.182069    ,    0.164155    ,    0.157587    ,    0.155114    ,    0.158733    ,    0.153726    ,    0.150194    ,    0.138939    ,    0.133928    ,    0.141054    ,    0.158906    ,    0.179185    ,    0.184999    ,    0.300768    ,    0.425545    ,    0.443335    ,    0.447953    ,    0.478807    ,    0.532568    ,    0.592709    ,    0.654116    ,    0.719501    ,    0.763577    ,    0.742614    ,    0.75043    ,    0.648619    ,    0.613168    ,    0.567947    ,    0.66966    ,    0.653329    ,    0.682128    ,    0.668847    ,    0.651271    ,    0.633495    ,    0.628312    ,    0.645186    ,    0.669389    ,    0.686794    ,    0.714732    ,    0.744274    ,    0.762229    ,    0.779303    ,    0.781995    ,    0.806626    ,    0.834331    ,    0.872317    ,    0.887745    ,    0.872217    ,    0.851258    ,    0.858268    ,    0.861607    ,    0.825338    ,    0.726738    ,    0.697985    ,    0.693183    ,    0.713297    ,    0.711504    ,    0.658219    ,    0.638878    ,    0.649539    ,    0.65448    ,    0.652663    ,    0.649337    ,    0.639914    ,    0.630845    ,    0.619516    ,    0.608385    ,    0.599175    ,    0.591931    ,    0.587762    ,    0.584823    ,    0.582876    ,    0.582133    ,    0.58347    ,    0.587864    ,    0.592653    ,    0.600461    ,    0.606641    ,    0.613791    ,    0.61978    ,    0.624299    ,    0.628618    ,    0.630755    ,    0.630352    },
    {    0.679412    ,    0.674947    ,    0.671644    ,    0.667707    ,    0.663403    ,    0.660701    ,    0.656964    ,    0.651881    ,    0.643981    ,    0.638099    ,    0.63257    ,    0.626282    ,    0.619686    ,    0.61415    ,    0.608867    ,    0.601581    ,    0.596534    ,    0.591389    ,    0.58878    ,    0.587122    ,    0.583841    ,    0.58281    ,    0.580523    ,    0.582477    ,    0.585935    ,    0.594178    ,    0.602175    ,    0.609127    ,    0.619594    ,    0.624505    ,    0.628195    ,    0.620432    ,    0.581305    ,    0.579232    ,    0.521934    ,    0.508964    ,    0.488778    ,    0.48226    ,    0.459117    ,    0.423444    ,    0.416317    ,    0.424141    ,    0.439464    ,    0.451885    ,    0.505953    ,    0.479365    ,    0.439715    ,    0.437793    ,    0.391737    ,    0.375596    ,    0.353083    ,    0.30775    ,    0.314879    ,    0.310602    ,    0.332405    ,    0.389053    ,    0.469013    ,    0.448955    ,    0.437793    ,    0.456223    ,    0.429818    ,    0.424715    ,    0.418687    ,    0.421656    ,    0.425208    ,    0.422552    ,    0.433474    ,    0.459198    ,    0.488288    ,    0.515602    ,    0.531724    ,    0.561752    ,    0.595765    ,    0.628719    ,    0.651457    ,    0.664134    ,    0.672574    ,    0.67976    ,    0.683212    ,    0.688706    ,    0.695917    ,    0.699205    ,    0.70638    ,    0.705216    ,    0.722421    ,    0.747736    ,    0.74966    ,    0.766205    ,    0.781746    ,    0.798723    ,    0.8156    ,    0.826625    ,    0.829556    ,    0.822989    ,    0.814703    ,    0.789761    ,    0.75153    ,    0.710442    ,    0.685001    ,    0.685251    ,    0.695562    ,    0.698247    ,    0.698711    ,    0.70307    ,    0.710276    ,    0.711192    ,    0.712531    ,    0.713972    ,    0.713024    ,    0.714519    ,    0.712657    ,    0.71054    ,    0.707177    ,    0.704146    ,    0.701795    ,    0.699702    ,    0.698963    ,    0.69852    ,    0.697666    ,    0.699982    ,    0.698064    ,    0.699744    ,    0.698627    ,    0.695735    ,    0.693604    ,    0.691262    ,    0.687729    ,    0.683347    },
    {    0.672431    ,    0.666859    ,    0.663032    ,    0.659288    ,    0.654196    ,    0.649217    ,    0.645572    ,    0.641942    ,    0.637086    ,    0.634282    ,    0.630817    ,    0.626203    ,    0.621837    ,    0.616982    ,    0.611841    ,    0.60628    ,    0.603543    ,    0.600687    ,    0.598282    ,    0.596616    ,    0.596275    ,    0.595414    ,    0.593558    ,    0.594212    ,    0.595394    ,    0.59673    ,    0.597884    ,    0.599945    ,    0.603168    ,    0.601181    ,    0.605254    ,    0.607384    ,    0.606412    ,    0.609027    ,    0.608823    ,    0.608348    ,    0.60898    ,    0.60533    ,    0.601841    ,    0.601525    ,    0.60195    ,    0.598929    ,    0.596972    ,    0.598057    ,    0.600949    ,    0.602108    ,    0.602639    ,    0.603406    ,    0.601905    ,    0.589862    ,    0.579494    ,    0.5731    ,    0.56712    ,    0.556267    ,    0.547773    ,    0.542961    ,    0.541892    ,    0.526765    ,    0.520428    ,    0.518006    ,    0.513403    ,    0.515933    ,    0.516547    ,    0.516647    ,    0.516784    ,    0.52228    ,    0.528545    ,    0.53676    ,    0.544167    ,    0.552089    ,    0.561658    ,    0.572865    ,    0.582961    ,    0.59635    ,    0.607047    ,    0.617487    ,    0.628711    ,    0.638587    ,    0.647759    ,    0.653775    ,    0.658537    ,    0.662536    ,    0.665601    ,    0.667543    ,    0.668647    ,    0.670952    ,    0.673235    ,    0.674891    ,    0.677009    ,    0.677755    ,    0.676063    ,    0.673537    ,    0.672798    ,    0.668771    ,    0.666689    ,    0.666431    ,    0.663912    ,    0.662278    ,    0.661397    ,    0.661914    ,    0.663285    ,    0.666321    ,    0.670393    ,    0.676497    ,    0.681676    ,    0.684986    ,    0.689459    ,    0.69504    ,    0.698231    ,    0.702356    ,    0.706756    ,    0.710087    ,    0.711295    ,    0.712072    ,    0.711005    ,    0.709208    ,    0.708004    ,    0.707787    ,    0.706387    ,    0.704841    ,    0.703771    ,    0.700632    ,    0.697909    ,    0.694749    ,    0.689773    ,    0.686105    ,    0.681952    ,    0.677591    },
    {    0.624602    ,    0.622909    ,    0.621698    ,    0.620259    ,    0.619251    ,    0.618648    ,    0.617447    ,    0.615369    ,    0.613692    ,    0.612069    ,    0.611405    ,    0.610677    ,    0.609479    ,    0.607111    ,    0.606026    ,    0.60534    ,    0.605256    ,    0.603988    ,    0.602484    ,    0.601398    ,    0.59995    ,    0.59889    ,    0.597333    ,    0.596696    ,    0.595393    ,    0.594241    ,    0.593438    ,    0.593163    ,    0.593453    ,    0.593736    ,    0.593567    ,    0.593259    ,    0.592389    ,    0.591869    ,    0.591918    ,    0.591262    ,    0.590734    ,    0.590448    ,    0.589703    ,    0.588346    ,    0.587921    ,    0.586309    ,    0.58489    ,    0.583768    ,    0.582947    ,    0.582241    ,    0.580727    ,    0.579146    ,    0.577006    ,    0.575177    ,    0.572969    ,    0.571462    ,    0.569068    ,    0.567039    ,    0.564928    ,    0.56367    ,    0.562308    ,    0.562479    ,    0.563258    ,    0.563808    ,    0.563445    ,    0.563923    ,    0.565617    ,    0.568264    ,    0.571169    ,    0.57454    ,    0.577561    ,    0.580563    ,    0.583158    ,    0.586033    ,    0.589337    ,    0.592707    ,    0.59543    ,    0.597547    ,    0.599564    ,    0.60235    ,    0.605299    ,    0.606942    ,    0.60852    ,    0.609959    ,    0.613008    ,    0.615005    ,    0.616101    ,    0.616394    ,    0.617858    ,    0.618282    ,    0.619221    ,    0.619888    ,    0.62038    ,    0.621452    ,    0.621835    ,    0.622111    ,    0.621805    ,    0.621614    ,    0.621547    ,    0.622007    ,    0.623827    ,    0.625016    ,    0.62598    ,    0.627198    ,    0.628455    ,    0.629529    ,    0.631607    ,    0.632626    ,    0.633034    ,    0.634295    ,    0.635509    ,    0.636936    ,    0.638426    ,    0.639304    ,    0.639879    ,    0.639801    ,    0.637762    ,    0.636513    ,    0.63598    ,    0.635597    ,    0.635398    ,    0.635889    ,    0.635481    ,    0.634529    ,    0.633579    ,    0.632965    ,    0.632032    ,    0.631016    ,    0.629296    ,    0.628864    ,    0.627797    ,    0.626499    }
}; //The standard deviation in the LGRTC from Leduc et al (2016), analysed from CMIP5 models

double LGRTC_RCP85[64][128]={
    {    1.14429    ,    1.1388    ,    1.13579    ,    1.13276    ,    1.12683    ,    1.1208    ,    1.11566    ,    1.11077    ,    1.10646    ,    1.10403    ,    1.10065    ,    1.09891    ,    1.09883    ,    1.09895    ,    1.0978    ,    1.09763    ,    1.09706    ,    1.09718    ,    1.09926    ,    1.10204    ,    1.1063    ,    1.10943    ,    1.11394    ,    1.11903    ,    1.12179    ,    1.12724    ,    1.1311    ,    1.13408    ,    1.13504    ,    1.13276    ,    1.13479    ,    1.13303    ,    1.12998    ,    1.13026    ,    1.12752    ,    1.1278    ,    1.12829    ,    1.13065    ,    1.13294    ,    1.13425    ,    1.13605    ,    1.13921    ,    1.13991    ,    1.14155    ,    1.14301    ,    1.14413    ,    1.1489    ,    1.15294    ,    1.15621    ,    1.15793    ,    1.16069    ,    1.16318    ,    1.16521    ,    1.16833    ,    1.16962    ,    1.17057    ,    1.1749    ,    1.17682    ,    1.1825    ,    1.18526    ,    1.18982    ,    1.19258    ,    1.19873    ,    1.20029    ,    1.20398    ,    1.20546    ,    1.21002    ,    1.20846    ,    1.21259    ,    1.21695    ,    1.21815    ,    1.21925    ,    1.22664    ,    1.22423    ,    1.22605    ,    1.22659    ,    1.23558    ,    1.23531    ,    1.23459    ,    1.23615    ,    1.23658    ,    1.23446    ,    1.24421    ,    1.24264    ,    1.24583    ,    1.24992    ,    1.24768    ,    1.25341    ,    1.24803    ,    1.25    ,    1.25415    ,    1.25706    ,    1.25062    ,    1.2548    ,    1.25432    ,    1.25326    ,    1.26007    ,    1.25866    ,    1.26069    ,    1.26233    ,    1.26272    ,    1.25897    ,    1.25058    ,    1.25272    ,    1.24869    ,    1.24851    ,    1.24199    ,    1.23665    ,    1.23205    ,    1.22564    ,    1.22093    ,    1.21589    ,    1.21101    ,    1.20635    ,    1.20292    ,    1.20023    ,    1.19447    ,    1.18998    ,    1.18504    ,    1.18098    ,    1.17457    ,    1.16955    ,    1.16366    ,    1.16011    ,    1.15799    ,    1.15366    ,    1.15186    ,    1.14904    },
    {    0.968872    ,    0.964813    ,    0.961898    ,    0.956593    ,    0.955523    ,    0.951961    ,    0.949986    ,    0.949465    ,    0.953657    ,    0.95472    ,    0.955259    ,    0.95358    ,    0.950256    ,    0.947202    ,    0.9459    ,    0.945991    ,    0.945368    ,    0.942178    ,    0.939285    ,    0.940267    ,    0.948444    ,    0.958371    ,    0.970323    ,    0.985003    ,    0.999836    ,    1.02056    ,    1.0416    ,    1.05478    ,    1.07305    ,    1.07806    ,    1.08708    ,    1.08436    ,    1.08126    ,    1.07993    ,    1.07024    ,    1.06728    ,    1.0666    ,    1.0701    ,    1.07047    ,    1.06925    ,    1.0669    ,    1.06469    ,    1.06233    ,    1.06084    ,    1.05738    ,    1.05699    ,    1.05685    ,    1.06238    ,    1.06812    ,    1.07162    ,    1.07212    ,    1.07413    ,    1.07962    ,    1.08504    ,    1.09068    ,    1.09578    ,    1.1016    ,    1.11045    ,    1.11983    ,    1.12855    ,    1.13715    ,    1.14759    ,    1.16086    ,    1.17176    ,    1.18448    ,    1.19751    ,    1.20845    ,    1.2116    ,    1.21256    ,    1.21013    ,    1.21709    ,    1.20476    ,    1.21039    ,    1.20929    ,    1.21259    ,    1.21318    ,    1.21721    ,    1.22127    ,    1.22718    ,    1.23944    ,    1.24297    ,    1.24392    ,    1.24215    ,    1.25249    ,    1.25227    ,    1.25466    ,    1.24923    ,    1.24879    ,    1.25122    ,    1.25202    ,    1.24956    ,    1.24601    ,    1.24715    ,    1.23857    ,    1.24517    ,    1.2532    ,    1.25407    ,    1.25656    ,    1.26129    ,    1.26272    ,    1.26632    ,    1.26284    ,    1.25413    ,    1.25442    ,    1.24811    ,    1.2382    ,    1.22734    ,    1.21939    ,    1.20586    ,    1.19827    ,    1.1795    ,    1.16508    ,    1.15009    ,    1.13752    ,    1.12484    ,    1.11015    ,    1.10067    ,    1.08782    ,    1.07995    ,    1.06549    ,    1.05204    ,    1.04029    ,    1.0307    ,    1.0206    ,    1.01027    ,    0.999376    ,    0.993562    ,    0.98277    },
    {    1.02108    ,    1.03058    ,    1.04104    ,    1.04758    ,    1.04839    ,    1.04965    ,    1.04856    ,    1.04202    ,    1.02886    ,    1.01151    ,    0.999328    ,    0.981993    ,    0.973108    ,    0.963204    ,    0.952288    ,    0.942311    ,    0.93322    ,    0.922228    ,    0.913945    ,    0.905412    ,    0.897535    ,    0.891597    ,    0.888057    ,    0.891762    ,    0.900767    ,    0.915939    ,    0.932832    ,    0.947601    ,    0.967396    ,    0.977296    ,    0.979324    ,    0.978397    ,    0.977298    ,    0.975857    ,    0.978762    ,    0.988553    ,    1.00306    ,    1.01794    ,    1.03438    ,    1.05285    ,    1.07603    ,    1.09821    ,    1.11657    ,    1.11705    ,    1.10822    ,    1.10504    ,    1.09688    ,    1.08809    ,    1.07292    ,    1.05047    ,    1.04112    ,    1.03789    ,    1.03842    ,    1.03859    ,    1.04261    ,    1.05069    ,    1.05985    ,    1.06745    ,    1.07755    ,    1.09155    ,    1.10636    ,    1.11888    ,    1.12825    ,    1.13579    ,    1.14504    ,    1.15393    ,    1.16528    ,    1.16963    ,    1.18049    ,    1.18214    ,    1.18095    ,    1.18218    ,    1.19537    ,    1.20444    ,    1.21493    ,    1.22793    ,    1.23862    ,    1.23815    ,    1.24609    ,    1.25658    ,    1.26629    ,    1.26169    ,    1.28004    ,    1.28256    ,    1.29127    ,    1.27775    ,    1.29564    ,    1.29355    ,    1.3039    ,    1.29401    ,    1.28169    ,    1.27244    ,    1.26855    ,    1.26186    ,    1.26497    ,    1.2606    ,    1.26592    ,    1.257    ,    1.24767    ,    1.24874    ,    1.24407    ,    1.23476    ,    1.23725    ,    1.23097    ,    1.22761    ,    1.22167    ,    1.2145    ,    1.2101    ,    1.19728    ,    1.17888    ,    1.16486    ,    1.15005    ,    1.13532    ,    1.12121    ,    1.10909    ,    1.09675    ,    1.07995    ,    1.06423    ,    1.0438    ,    1.02535    ,    1.00666    ,    0.988073    ,    0.973051    ,    0.975829    ,    0.980693    ,    0.993474    ,    1.00401    ,    1.01456    },
    {    1.05907    ,    1.06088    ,    1.06047    ,    1.05872    ,    1.05558    ,    1.0559    ,    1.04525    ,    1.02662    ,    1.00403    ,    0.984295    ,    0.968769    ,    0.951756    ,    0.930949    ,    0.909952    ,    0.896208    ,    0.883075    ,    0.869982    ,    0.86266    ,    0.861324    ,    0.867062    ,    0.877116    ,    0.889336    ,    0.9043    ,    0.911972    ,    0.923221    ,    0.928683    ,    0.931587    ,    0.935757    ,    0.939573    ,    0.939797    ,    0.941514    ,    0.940218    ,    0.941448    ,    0.941713    ,    0.948087    ,    0.975736    ,    1.00937    ,    1.04994    ,    1.08879    ,    1.12624    ,    1.15109    ,    1.17686    ,    1.20472    ,    1.2184    ,    1.23025    ,    1.22986    ,    1.217    ,    1.20947    ,    1.20863    ,    1.20128    ,    1.18079    ,    1.1438    ,    1.0976    ,    1.06862    ,    1.0432    ,    1.0223    ,    1.01409    ,    1.01265    ,    1.01971    ,    1.03057    ,    1.04538    ,    1.06302    ,    1.07789    ,    1.09297    ,    1.10308    ,    1.11745    ,    1.12942    ,    1.13618    ,    1.14721    ,    1.16436    ,    1.17209    ,    1.19506    ,    1.21302    ,    1.2241    ,    1.22676    ,    1.2445    ,    1.25483    ,    1.25566    ,    1.2717    ,    1.28657    ,    1.28265    ,    1.26569    ,    1.26202    ,    1.23575    ,    1.24277    ,    1.22823    ,    1.21229    ,    1.20747    ,    1.20496    ,    1.2029    ,    1.19963    ,    1.20663    ,    1.22303    ,    1.23182    ,    1.2564    ,    1.26273    ,    1.2795    ,    1.26768    ,    1.22666    ,    1.22354    ,    1.23249    ,    1.22286    ,    1.2121    ,    1.2141    ,    1.2171    ,    1.21637    ,    1.21106    ,    1.20931    ,    1.20293    ,    1.19094    ,    1.18009    ,    1.16912    ,    1.16365    ,    1.15665    ,    1.14983    ,    1.13711    ,    1.1217    ,    1.09894    ,    1.0665    ,    1.03664    ,    1.0122    ,    0.996676    ,    0.99659    ,    1.00653    ,    1.03159    ,    1.05457    ,    1.06815    ,    1.07038    },
    {    1.0164    ,    1.04687    ,    1.08171    ,    1.10948    ,    1.13285    ,    1.15433    ,    1.16994    ,    1.1681    ,    1.14798    ,    1.14899    ,    1.14729    ,    1.0698    ,    1.02019    ,    0.941299    ,    0.906138    ,    0.889821    ,    0.889707    ,    0.891185    ,    0.894793    ,    0.900018    ,    0.905666    ,    0.904248    ,    0.908691    ,    0.911401    ,    0.915564    ,    0.920659    ,    0.922929    ,    0.926701    ,    0.930012    ,    0.933301    ,    0.937889    ,    0.945473    ,    0.957145    ,    0.974739    ,    0.993553    ,    1.02939    ,    1.06939    ,    1.11285    ,    1.14016    ,    1.14654    ,    1.14627    ,    1.15102    ,    1.17747    ,    1.19855    ,    1.20586    ,    1.21992    ,    1.21804    ,    1.21667    ,    1.23001    ,    1.23936    ,    1.24275    ,    1.23534    ,    1.21978    ,    1.17745    ,    1.12939    ,    1.06555    ,    1.03678    ,    1.01016    ,    0.98407    ,    0.99285    ,    1.0151    ,    1.0481    ,    1.07503    ,    1.09588    ,    1.10687    ,    1.12223    ,    1.15067    ,    1.16007    ,    1.18272    ,    1.19656    ,    1.19921    ,    1.21351    ,    1.22794    ,    1.25006    ,    1.26334    ,    1.27181    ,    1.26322    ,    1.27077    ,    1.26758    ,    1.26925    ,    1.26499    ,    1.22987    ,    1.20623    ,    1.18666    ,    1.17549    ,    1.16279    ,    1.14861    ,    1.13278    ,    1.11569    ,    1.09875    ,    1.08946    ,    1.08225    ,    1.07188    ,    1.07222    ,    1.08442    ,    1.10716    ,    1.13698    ,    1.17467    ,    1.17633    ,    1.18515    ,    1.20068    ,    1.18705    ,    1.18014    ,    1.18481    ,    1.20711    ,    1.20658    ,    1.20747    ,    1.21641    ,    1.20624    ,    1.18317    ,    1.17114    ,    1.17029    ,    1.16848    ,    1.17067    ,    1.16779    ,    1.15509    ,    1.13517    ,    1.11134    ,    1.08725    ,    1.0544    ,    1.01702    ,    0.99938    ,    0.965837    ,    0.96853    ,    0.975695    ,    0.984262    ,    0.990995    ,    0.999541    },
    {    0.966843    ,    0.973404    ,    0.981549    ,    0.993776    ,    1.01607    ,    1.04448    ,    1.07234    ,    1.09857    ,    1.12254    ,    1.141    ,    1.15576    ,    1.16518    ,    1.16735    ,    1.16107    ,    1.149    ,    1.12275    ,    1.09762    ,    1.07512    ,    1.03446    ,    0.995615    ,    1.01497    ,    1.02426    ,    1.02658    ,    1.03012    ,    1.03601    ,    1.01916    ,    1.01848    ,    1.01204    ,    1.00263    ,    1.01052    ,    1.01915    ,    1.01369    ,    1.00504    ,    1.0123    ,    1.03113    ,    1.04974    ,    1.07306    ,    1.0958    ,    1.1028    ,    1.11058    ,    1.12306    ,    1.12592    ,    1.13831    ,    1.1499    ,    1.14987    ,    1.15235    ,    1.15538    ,    1.16642    ,    1.18476    ,    1.20028    ,    1.2123    ,    1.22447    ,    1.23395    ,    1.23353    ,    1.22035    ,    1.19155    ,    1.18299    ,    1.12079    ,    1.03638    ,    0.958977    ,    0.927347    ,    0.940852    ,    0.982005    ,    1.01456    ,    1.02806    ,    1.04512    ,    1.06305    ,    1.08535    ,    1.10202    ,    1.10972    ,    1.11179    ,    1.10253    ,    1.10477    ,    1.10976    ,    1.10831    ,    1.10742    ,    1.11408    ,    1.13157    ,    1.15433    ,    1.16934    ,    1.18469    ,    1.19102    ,    1.18592    ,    1.16246    ,    1.13653    ,    1.12411    ,    1.11475    ,    1.1141    ,    1.13076    ,    1.1063    ,    1.0448    ,    1.01616    ,    0.998    ,    0.996288    ,    1.00826    ,    1.03441    ,    1.06911    ,    1.10161    ,    1.11713    ,    1.12454    ,    1.13814    ,    1.14389    ,    1.15775    ,    1.17852    ,    1.18688    ,    1.1782    ,    1.16235    ,    1.15032    ,    1.13485    ,    1.10647    ,    1.08713    ,    1.09156    ,    1.10094    ,    1.11523    ,    1.12612    ,    1.12634    ,    1.11388    ,    1.09282    ,    1.06631    ,    1.03389    ,    0.993843    ,    0.961188    ,    0.944157    ,    0.971088    ,    0.996306    ,    0.996434    ,    0.985907    ,    0.977703    },
    {    0.894336    ,    0.88972    ,    0.892784    ,    0.905403    ,    0.925577    ,    0.94705    ,    0.968959    ,    0.992304    ,    1.01485    ,    1.0341    ,    1.04907    ,    1.06032    ,    1.06756    ,    1.06979    ,    1.06862    ,    1.0643    ,    1.06077    ,    1.06231    ,    1.06594    ,    1.07155    ,    1.0736    ,    1.0709    ,    1.06306    ,    1.05231    ,    1.04081    ,    1.02958    ,    1.02587    ,    1.02981    ,    1.03841    ,    1.05137    ,    1.06014    ,    1.0696    ,    1.07532    ,    1.07663    ,    1.07891    ,    1.08619    ,    1.0947    ,    1.12516    ,    1.18984    ,    1.17925    ,    1.1504    ,    1.1568    ,    1.15937    ,    1.19356    ,    1.19608    ,    1.18716    ,    1.17435    ,    1.16454    ,    1.15693    ,    1.15231    ,    1.15172    ,    1.14916    ,    1.14817    ,    1.14696    ,    1.14492    ,    1.14225    ,    1.1439    ,    1.14287    ,    1.13254    ,    1.09969    ,    1.07443    ,    1.0279    ,    0.994917    ,    0.969737    ,    0.950349    ,    0.926979    ,    0.919637    ,    0.918729    ,    0.914223    ,    0.916671    ,    0.92037    ,    0.920857    ,    0.917802    ,    0.913307    ,    0.915886    ,    0.925527    ,    0.935335    ,    0.955019    ,    0.970195    ,    0.98984    ,    1.01941    ,    1.05326    ,    1.08368    ,    1.09695    ,    1.07772    ,    1.04686    ,    1.02523    ,    1.02588    ,    1.04802    ,    1.1185    ,    1.03202    ,    0.960958    ,    0.939809    ,    0.935589    ,    0.950039    ,    0.969517    ,    0.984312    ,    0.999493    ,    1.00816    ,    1.01685    ,    1.02344    ,    1.03816    ,    1.04208    ,    1.03464    ,    1.02301    ,    1.01028    ,    0.995339    ,    0.985001    ,    0.986053    ,    0.989656    ,    0.989872    ,    0.995069    ,    1.00046    ,    1.00556    ,    1.00554    ,    1.00194    ,    0.996007    ,    0.975848    ,    0.95536    ,    0.932902    ,    0.915041    ,    0.908351    ,    0.911114    ,    0.922017    ,    0.956367    ,    0.955284    ,    0.929078    ,    0.914168    },
    {    0.838984    ,    0.829241    ,    0.823529    ,    0.826215    ,    0.834053    ,    0.844618    ,    0.857309    ,    0.872216    ,    0.890057    ,    0.90696    ,    0.922587    ,    0.935302    ,    0.94348    ,    0.948406    ,    0.953197    ,    0.954987    ,    0.957804    ,    0.959259    ,    0.957044    ,    0.953589    ,    0.951987    ,    0.953187    ,    0.955839    ,    0.959537    ,    0.964148    ,    0.966176    ,    0.969163    ,    0.976491    ,    0.985743    ,    0.992372    ,    0.995287    ,    0.993551    ,    0.988564    ,    0.986685    ,    0.990146    ,    1.00021    ,    1.01997    ,    1.04881    ,    1.08564    ,    1.12328    ,    1.16294    ,    1.18232    ,    1.20034    ,    1.2314    ,    1.20337    ,    1.16773    ,    1.12915    ,    1.0965    ,    1.07599    ,    1.06292    ,    1.054    ,    1.04812    ,    1.04172    ,    1.03539    ,    1.03004    ,    1.03045    ,    1.03702    ,    1.04756    ,    1.05911    ,    1.06577    ,    1.06572    ,    1.06653    ,    1.06183    ,    1.05596    ,    1.05969    ,    1.06141    ,    1.05486    ,    1.0483    ,    1.04693    ,    1.0549    ,    1.067    ,    1.07857    ,    1.08265    ,    1.08059    ,    1.05335    ,    1.03265    ,    1.02006    ,    1.04023    ,    1.02148    ,    0.928079    ,    0.902557    ,    0.91517    ,    0.91918    ,    0.933874    ,    0.931943    ,    0.933926    ,    0.943769    ,    0.958948    ,    0.992734    ,    1.04223    ,    1.02152    ,    0.953366    ,    0.908215    ,    0.876838    ,    0.867705    ,    0.861345    ,    0.852998    ,    0.849202    ,    0.850559    ,    0.852937    ,    0.851309    ,    0.857465    ,    0.85455    ,    0.853976    ,    0.854429    ,    0.854116    ,    0.862096    ,    0.84217    ,    0.83211    ,    0.833342    ,    0.833268    ,    0.838704    ,    0.853568    ,    0.857125    ,    0.855857    ,    0.862625    ,    0.88356    ,    0.896122    ,    0.903321    ,    0.91404    ,    0.911305    ,    0.917549    ,    0.917154    ,    0.91526    ,    0.909666    ,    0.900908    ,    0.880977    ,    0.860412    },
    {    0.689622    ,    0.688595    ,    0.689897    ,    0.6915    ,    0.699634    ,    0.709532    ,    0.71867    ,    0.727532    ,    0.735767    ,    0.744902    ,    0.753286    ,    0.759766    ,    0.765696    ,    0.770344    ,    0.773388    ,    0.774129    ,    0.770192    ,    0.766095    ,    0.761388    ,    0.75729    ,    0.757953    ,    0.761114    ,    0.765199    ,    0.769838    ,    0.775975    ,    0.78312    ,    0.790064    ,    0.798504    ,    0.809373    ,    0.821155    ,    0.833655    ,    0.846287    ,    0.851202    ,    0.852087    ,    0.854999    ,    0.864389    ,    0.88413    ,    0.909013    ,    0.941411    ,    0.982002    ,    1.02818    ,    1.07926    ,    1.15337    ,    1.18604    ,    1.15666    ,    1.09346    ,    1.03704    ,    1.00517    ,    0.990168    ,    0.978912    ,    0.968207    ,    0.958779    ,    0.951036    ,    0.943529    ,    0.939603    ,    0.939904    ,    0.941724    ,    0.945677    ,    0.952626    ,    0.960374    ,    0.965397    ,    0.967338    ,    0.965778    ,    0.960946    ,    0.954914    ,    0.945829    ,    0.934919    ,    0.929974    ,    0.934296    ,    0.939457    ,    0.940981    ,    0.942948    ,    0.950158    ,    0.964692    ,    0.983231    ,    0.996107    ,    1.00116    ,    1.00339    ,    1.00801    ,    1.00719    ,    0.994593    ,    0.979336    ,    0.964125    ,    0.965825    ,    0.984967    ,    0.973052    ,    0.941382    ,    0.927232    ,    0.922093    ,    0.91677    ,    0.900232    ,    0.887655    ,    0.886425    ,    0.901431    ,    0.911176    ,    0.934518    ,    0.922423    ,    0.88116    ,    0.874485    ,    0.898621    ,    0.86952    ,    0.916517    ,    0.935617    ,    0.934663    ,    0.954005    ,    0.975841    ,    0.967207    ,    0.943194    ,    0.90076    ,    0.894066    ,    0.859975    ,    0.843882    ,    0.826725    ,    0.817168    ,    0.800071    ,    0.799655    ,    0.80879    ,    0.790211    ,    0.777534    ,    0.768393    ,    0.757231    ,    0.754921    ,    0.761877    ,    0.765612    ,    0.756942    ,    0.736566    ,    0.714556    ,    0.699896    },
    {    0.535518    ,    0.528774    ,    0.529831    ,    0.535106    ,    0.544015    ,    0.55024    ,    0.551956    ,    0.55164    ,    0.554501    ,    0.563873    ,    0.575255    ,    0.585034    ,    0.592905    ,    0.59756    ,    0.595186    ,    0.584121    ,    0.568066    ,    0.555828    ,    0.548809    ,    0.544764    ,    0.543431    ,    0.545407    ,    0.5514    ,    0.557529    ,    0.563955    ,    0.57238    ,    0.582419    ,    0.59391    ,    0.606946    ,    0.618988    ,    0.631953    ,    0.646257    ,    0.657135    ,    0.664105    ,    0.669625    ,    0.676606    ,    0.688597    ,    0.7067    ,    0.731542    ,    0.764865    ,    0.809713    ,    0.846712    ,    0.879776    ,    0.935409    ,    0.962835    ,    0.940884    ,    0.91348    ,    0.890536    ,    0.878639    ,    0.872412    ,    0.868205    ,    0.866022    ,    0.863394    ,    0.86123    ,    0.860893    ,    0.860997    ,    0.861273    ,    0.864247    ,    0.870225    ,    0.877889    ,    0.884324    ,    0.886705    ,    0.884648    ,    0.880139    ,    0.874537    ,    0.868544    ,    0.858953    ,    0.849108    ,    0.844418    ,    0.846732    ,    0.848774    ,    0.850156    ,    0.853641    ,    0.856636    ,    0.860933    ,    0.867419    ,    0.878128    ,    0.89123    ,    0.905498    ,    0.921056    ,    0.936523    ,    0.947856    ,    0.956438    ,    0.956888    ,    0.936554    ,    0.898505    ,    0.859634    ,    0.823272    ,    0.789526    ,    0.759939    ,    0.726736    ,    0.696405    ,    0.677535    ,    0.674299    ,    0.691788    ,    0.736035    ,    0.784919    ,    0.819548    ,    0.83897    ,    0.852185    ,    0.857401    ,    0.861919    ,    0.862145    ,    0.866272    ,    0.869633    ,    0.861524    ,    0.843531    ,    0.825722    ,    0.813034    ,    0.806336    ,    0.80298    ,    0.802806    ,    0.797042    ,    0.787272    ,    0.771449    ,    0.747246    ,    0.715592    ,    0.674675    ,    0.635043    ,    0.605362    ,    0.587038    ,    0.584602    ,    0.581154    ,    0.580781    ,    0.577579    ,    0.564293    ,    0.553006    ,    0.544563    },
    {    0.42041    ,    0.416266    ,    0.413044    ,    0.410149    ,    0.408587    ,    0.407892    ,    0.40286    ,    0.397249    ,    0.396521    ,    0.403718    ,    0.415751    ,    0.429734    ,    0.443002    ,    0.452547    ,    0.456767    ,    0.455383    ,    0.450717    ,    0.448443    ,    0.449696    ,    0.453184    ,    0.454902    ,    0.459286    ,    0.464039    ,    0.471137    ,    0.479672    ,    0.488561    ,    0.49949    ,    0.511286    ,    0.522266    ,    0.531802    ,    0.541301    ,    0.550787    ,    0.55922    ,    0.565472    ,    0.570418    ,    0.576649    ,    0.586617    ,    0.596744    ,    0.603892    ,    0.614629    ,    0.636514    ,    0.675487    ,    0.708448    ,    0.747153    ,    0.788283    ,    0.806237    ,    0.802108    ,    0.780378    ,    0.767987    ,    0.762647    ,    0.762128    ,    0.765721    ,    0.770949    ,    0.776763    ,    0.779525    ,    0.781615    ,    0.785422    ,    0.787858    ,    0.787674    ,    0.790808    ,    0.797206    ,    0.801023    ,    0.80137    ,    0.796974    ,    0.790094    ,    0.786485    ,    0.786232    ,    0.782452    ,    0.778253    ,    0.779188    ,    0.779478    ,    0.774654    ,    0.766714    ,    0.75187    ,    0.734263    ,    0.722846    ,    0.722293    ,    0.727112    ,    0.736498    ,    0.74452    ,    0.753699    ,    0.762726    ,    0.771264    ,    0.778835    ,    0.784027    ,    0.783801    ,    0.776688    ,    0.767131    ,    0.752881    ,    0.718869    ,    0.6717    ,    0.632548    ,    0.607176    ,    0.593962    ,    0.582344    ,    0.589271    ,    0.611622    ,    0.633491    ,    0.65498    ,    0.671089    ,    0.685316    ,    0.691867    ,    0.684017    ,    0.669218    ,    0.654928    ,    0.638532    ,    0.619719    ,    0.6017    ,    0.585636    ,    0.575113    ,    0.569155    ,    0.564529    ,    0.562296    ,    0.566426    ,    0.572104    ,    0.574552    ,    0.565685    ,    0.532665    ,    0.501872    ,    0.488975    ,    0.47992    ,    0.482795    ,    0.485555    ,    0.476792    ,    0.461948    ,    0.447157    ,    0.434239    ,    0.426399    },
    {    0.427186    ,    0.425913    ,    0.427318    ,    0.428892    ,    0.425184    ,    0.418738    ,    0.407895    ,    0.392905    ,    0.381707    ,    0.379357    ,    0.381069    ,    0.376189    ,    0.373423    ,    0.378853    ,    0.381413    ,    0.382542    ,    0.383621    ,    0.394827    ,    0.415044    ,    0.434181    ,    0.442736    ,    0.443721    ,    0.444441    ,    0.449588    ,    0.459716    ,    0.473056    ,    0.4851    ,    0.496625    ,    0.50722    ,    0.517087    ,    0.525456    ,    0.532896    ,    0.539398    ,    0.545267    ,    0.549051    ,    0.553918    ,    0.555986    ,    0.554402    ,    0.561003    ,    0.576054    ,    0.587111    ,    0.590025    ,    0.600636    ,    0.620174    ,    0.646465    ,    0.672786    ,    0.681211    ,    0.675588    ,    0.665605    ,    0.659778    ,    0.647536    ,    0.64309    ,    0.647344    ,    0.655623    ,    0.666493    ,    0.681619    ,    0.696039    ,    0.699007    ,    0.698125    ,    0.697589    ,    0.695578    ,    0.690787    ,    0.684607    ,    0.678003    ,    0.670458    ,    0.665411    ,    0.664412    ,    0.668607    ,    0.673975    ,    0.677629    ,    0.674006    ,    0.662676    ,    0.644572    ,    0.621835    ,    0.596963    ,    0.579277    ,    0.57283    ,    0.574776    ,    0.581489    ,    0.589065    ,    0.598251    ,    0.606107    ,    0.613469    ,    0.620917    ,    0.628619    ,    0.636086    ,    0.644494    ,    0.655103    ,    0.665963    ,    0.669276    ,    0.654703    ,    0.61895    ,    0.579749    ,    0.558357    ,    0.545551    ,    0.536634    ,    0.541758    ,    0.544529    ,    0.548745    ,    0.554744    ,    0.561733    ,    0.562587    ,    0.556173    ,    0.547573    ,    0.54088    ,    0.534424    ,    0.525725    ,    0.517639    ,    0.510673    ,    0.506665    ,    0.503641    ,    0.502432    ,    0.502926    ,    0.505674    ,    0.512307    ,    0.521641    ,    0.519864    ,    0.498974    ,    0.491786    ,    0.489438    ,    0.490239    ,    0.49464    ,    0.484867    ,    0.467222    ,    0.453639    ,    0.443688    ,    0.435782    ,    0.432414    },
    {    0.456545    ,    0.459213    ,    0.466992    ,    0.471717    ,    0.467616    ,    0.459117    ,    0.440676    ,    0.421526    ,    0.411037    ,    0.412224    ,    0.414121    ,    0.401499    ,    0.386683    ,    0.380766    ,    0.387278    ,    0.389836    ,    0.391024    ,    0.406044    ,    0.434506    ,    0.454642    ,    0.459353    ,    0.457919    ,    0.457929    ,    0.463728    ,    0.476254    ,    0.491002    ,    0.504038    ,    0.513698    ,    0.522699    ,    0.529943    ,    0.536199    ,    0.540962    ,    0.54436    ,    0.547093    ,    0.548619    ,    0.547396    ,    0.548038    ,    0.558299    ,    0.604656    ,    0.666862    ,    0.668046    ,    0.611434    ,    0.59661    ,    0.597179    ,    0.591645    ,    0.588074    ,    0.59468    ,    0.603085    ,    0.602153    ,    0.596077    ,    0.589854    ,    0.590837    ,    0.582212    ,    0.591447    ,    0.60528    ,    0.617048    ,    0.624104    ,    0.625602    ,    0.622235    ,    0.618412    ,    0.61289    ,    0.602498    ,    0.594211    ,    0.588019    ,    0.579978    ,    0.572871    ,    0.564471    ,    0.564303    ,    0.572814    ,    0.580906    ,    0.587129    ,    0.588864    ,    0.584473    ,    0.579693    ,    0.572617    ,    0.565163    ,    0.563529    ,    0.567284    ,    0.570652    ,    0.573523    ,    0.580891    ,    0.588307    ,    0.593082    ,    0.598257    ,    0.605235    ,    0.613635    ,    0.62536    ,    0.638624    ,    0.654035    ,    0.667728    ,    0.673364    ,    0.648808    ,    0.589331    ,    0.561009    ,    0.557888    ,    0.549788    ,    0.552825    ,    0.549215    ,    0.547494    ,    0.549843    ,    0.546916    ,    0.545627    ,    0.547591    ,    0.549203    ,    0.550903    ,    0.549179    ,    0.541623    ,    0.534428    ,    0.528372    ,    0.524822    ,    0.520237    ,    0.517884    ,    0.516925    ,    0.517139    ,    0.522279    ,    0.526006    ,    0.524033    ,    0.508818    ,    0.501971    ,    0.497238    ,    0.495202    ,    0.49914    ,    0.488719    ,    0.485075    ,    0.48583    ,    0.480376    ,    0.468316    ,    0.460278    },
    {    0.507469    ,    0.518346    ,    0.527776    ,    0.529863    ,    0.526196    ,    0.517187    ,    0.496318    ,    0.472957    ,    0.46553    ,    0.464707    ,    0.462257    ,    0.452728    ,    0.443801    ,    0.448161    ,    0.460376    ,    0.469088    ,    0.476504    ,    0.486    ,    0.495823    ,    0.499414    ,    0.497378    ,    0.495716    ,    0.496544    ,    0.502113    ,    0.513789    ,    0.524779    ,    0.532085    ,    0.537776    ,    0.542944    ,    0.545888    ,    0.548922    ,    0.551637    ,    0.552549    ,    0.55434    ,    0.557085    ,    0.559232    ,    0.564485    ,    0.593067    ,    0.688684    ,    0.739187    ,    0.685251    ,    0.641817    ,    0.633938    ,    0.636016    ,    0.599358    ,    0.575014    ,    0.582364    ,    0.596775    ,    0.603063    ,    0.605692    ,    0.61354    ,    0.618137    ,    0.619907    ,    0.631292    ,    0.639089    ,    0.636832    ,    0.636131    ,    0.637246    ,    0.63498    ,    0.632718    ,    0.628067    ,    0.610784    ,    0.603942    ,    0.607699    ,    0.60539    ,    0.601336    ,    0.596193    ,    0.595945    ,    0.595717    ,    0.590137    ,    0.582718    ,    0.584309    ,    0.58985    ,    0.592282    ,    0.591647    ,    0.598052    ,    0.60029    ,    0.60221    ,    0.602021    ,    0.601656    ,    0.603951    ,    0.609779    ,    0.615977    ,    0.621126    ,    0.626895    ,    0.633283    ,    0.647991    ,    0.666152    ,    0.680016    ,    0.68491    ,    0.67824    ,    0.654801    ,    0.612517    ,    0.62289    ,    0.642973    ,    0.632611    ,    0.627918    ,    0.626419    ,    0.626687    ,    0.628121    ,    0.628639    ,    0.633538    ,    0.638064    ,    0.640165    ,    0.638491    ,    0.628352    ,    0.607948    ,    0.585766    ,    0.569202    ,    0.557248    ,    0.54806    ,    0.545546    ,    0.544698    ,    0.540785    ,    0.532293    ,    0.520782    ,    0.517425    ,    0.509828    ,    0.509419    ,    0.505535    ,    0.505796    ,    0.516504    ,    0.522009    ,    0.524284    ,    0.527749    ,    0.52326    ,    0.508587    ,    0.500886    },
    {    0.619208    ,    0.62587    ,    0.630519    ,    0.620868    ,    0.604667    ,    0.591703    ,    0.573965    ,    0.559744    ,    0.55468    ,    0.546738    ,    0.538721    ,    0.534672    ,    0.533414    ,    0.536503    ,    0.542814    ,    0.546036    ,    0.545531    ,    0.544739    ,    0.544312    ,    0.543904    ,    0.541441    ,    0.539742    ,    0.540337    ,    0.543001    ,    0.548009    ,    0.551824    ,    0.555221    ,    0.55818    ,    0.560756    ,    0.562322    ,    0.564196    ,    0.566569    ,    0.567394    ,    0.568703    ,    0.573761    ,    0.579252    ,    0.5823    ,    0.619372    ,    0.756966    ,    0.803545    ,    0.768366    ,    0.695387    ,    0.65681    ,    0.621834    ,    0.597045    ,    0.603022    ,    0.631007    ,    0.649245    ,    0.65868    ,    0.666784    ,    0.673589    ,    0.671894    ,    0.67044    ,    0.67331    ,    0.680688    ,    0.674937    ,    0.676308    ,    0.687333    ,    0.701616    ,    0.709809    ,    0.703015    ,    0.683755    ,    0.678965    ,    0.680744    ,    0.676941    ,    0.671307    ,    0.664545    ,    0.664657    ,    0.654786    ,    0.643598    ,    0.637701    ,    0.637556    ,    0.636388    ,    0.630698    ,    0.629632    ,    0.643121    ,    0.660875    ,    0.667446    ,    0.668228    ,    0.661029    ,    0.65967    ,    0.662978    ,    0.665996    ,    0.666777    ,    0.667763    ,    0.67096    ,    0.687241    ,    0.69962    ,    0.70202    ,    0.70148    ,    0.715943    ,    0.743442    ,    0.75249    ,    0.740592    ,    0.723022    ,    0.713141    ,    0.712104    ,    0.712213    ,    0.713874    ,    0.710146    ,    0.708282    ,    0.707166    ,    0.695482    ,    0.679489    ,    0.660547    ,    0.642162    ,    0.618128    ,    0.593345    ,    0.572318    ,    0.558766    ,    0.555225    ,    0.560533    ,    0.563309    ,    0.554343    ,    0.533017    ,    0.527223    ,    0.539941    ,    0.551627    ,    0.554864    ,    0.556327    ,    0.562258    ,    0.572945    ,    0.58246    ,    0.596052    ,    0.594762    ,    0.588759    ,    0.58771    ,    0.602274    },
    {    0.727756    ,    0.739865    ,    0.734551    ,    0.704491    ,    0.678772    ,    0.661449    ,    0.646462    ,    0.631177    ,    0.616997    ,    0.608572    ,    0.602174    ,    0.599643    ,    0.597773    ,    0.596397    ,    0.594522    ,    0.589999    ,    0.585355    ,    0.581252    ,    0.576642    ,    0.57356    ,    0.570775    ,    0.569262    ,    0.56955    ,    0.570762    ,    0.571925    ,    0.57396    ,    0.577417    ,    0.580001    ,    0.582198    ,    0.584588    ,    0.586111    ,    0.58783    ,    0.588113    ,    0.590974    ,    0.597385    ,    0.602796    ,    0.596814    ,    0.625137    ,    0.754813    ,    0.832164    ,    0.823909    ,    0.722807    ,    0.678425    ,    0.653526    ,    0.679885    ,    0.699413    ,    0.715226    ,    0.72879    ,    0.734067    ,    0.742327    ,    0.735944    ,    0.728279    ,    0.720124    ,    0.726423    ,    0.734867    ,    0.737386    ,    0.746976    ,    0.771974    ,    0.802403    ,    0.810017    ,    0.787246    ,    0.762618    ,    0.751015    ,    0.742929    ,    0.734891    ,    0.731216    ,    0.732338    ,    0.733085    ,    0.72491    ,    0.71138    ,    0.707862    ,    0.700932    ,    0.700127    ,    0.701982    ,    0.709059    ,    0.72041    ,    0.739463    ,    0.751356    ,    0.745927    ,    0.732675    ,    0.727453    ,    0.726796    ,    0.725675    ,    0.73041    ,    0.721402    ,    0.70816    ,    0.708539    ,    0.716733    ,    0.725631    ,    0.732963    ,    0.754688    ,    0.778222    ,    0.788417    ,    0.779881    ,    0.758472    ,    0.742836    ,    0.736993    ,    0.737201    ,    0.73276    ,    0.71386    ,    0.696167    ,    0.680624    ,    0.659754    ,    0.640909    ,    0.625323    ,    0.61135    ,    0.593502    ,    0.575785    ,    0.562731    ,    0.553318    ,    0.550064    ,    0.550573    ,    0.550323    ,    0.553484    ,    0.566461    ,    0.610746    ,    0.663296    ,    0.698498    ,    0.684754    ,    0.663825    ,    0.656362    ,    0.655914    ,    0.66564    ,    0.706999    ,    0.765968    ,    0.726039    ,    0.706557    ,    0.707896    },
    {    0.806581    ,    0.798691    ,    0.760303    ,    0.714833    ,    0.689051    ,    0.675129    ,    0.662695    ,    0.651899    ,    0.643831    ,    0.638715    ,    0.636603    ,    0.635223    ,    0.630714    ,    0.625293    ,    0.620317    ,    0.614552    ,    0.610363    ,    0.606407    ,    0.603171    ,    0.602095    ,    0.599398    ,    0.597174    ,    0.597343    ,    0.597215    ,    0.598252    ,    0.601487    ,    0.604921    ,    0.607323    ,    0.608753    ,    0.609465    ,    0.609842    ,    0.610798    ,    0.610414    ,    0.610887    ,    0.612649    ,    0.613782    ,    0.601892    ,    0.618774    ,    0.746518    ,    0.86864    ,    0.860059    ,    0.751404    ,    0.69599    ,    0.739534    ,    0.792106    ,    0.82354    ,    0.820163    ,    0.804924    ,    0.786408    ,    0.786135    ,    0.783435    ,    0.787124    ,    0.789281    ,    0.79973    ,    0.81013    ,    0.815735    ,    0.827138    ,    0.851906    ,    0.88238    ,    0.857446    ,    0.804653    ,    0.785028    ,    0.779397    ,    0.779631    ,    0.78024    ,    0.776458    ,    0.781075    ,    0.784182    ,    0.774826    ,    0.769087    ,    0.777529    ,    0.781192    ,    0.790685    ,    0.807082    ,    0.807783    ,    0.7989    ,    0.804091    ,    0.828282    ,    0.840134    ,    0.816499    ,    0.779918    ,    0.765561    ,    0.752703    ,    0.751441    ,    0.741624    ,    0.728077    ,    0.726841    ,    0.712355    ,    0.721056    ,    0.731359    ,    0.738337    ,    0.741587    ,    0.740788    ,    0.737586    ,    0.725564    ,    0.714923    ,    0.711284    ,    0.702447    ,    0.681794    ,    0.656815    ,    0.639251    ,    0.629114    ,    0.617177    ,    0.607447    ,    0.599531    ,    0.590103    ,    0.581147    ,    0.573788    ,    0.56809    ,    0.563104    ,    0.561961    ,    0.565763    ,    0.574299    ,    0.59278    ,    0.620392    ,    0.668124    ,    0.760331    ,    0.848661    ,    0.851064    ,    0.791471    ,    0.75975    ,    0.750827    ,    0.749941    ,    0.767375    ,    0.812499    ,    0.84922    ,    0.809427    ,    0.790967    },
    {    0.793728    ,    0.760046    ,    0.731293    ,    0.705559    ,    0.688046    ,    0.680172    ,    0.675937    ,    0.673268    ,    0.670858    ,    0.666682    ,    0.663585    ,    0.661265    ,    0.657425    ,    0.653895    ,    0.650644    ,    0.647891    ,    0.645983    ,    0.642341    ,    0.639229    ,    0.63662    ,    0.631994    ,    0.629335    ,    0.628202    ,    0.626437    ,    0.628092    ,    0.629385    ,    0.63172    ,    0.631789    ,    0.627594    ,    0.624988    ,    0.624394    ,    0.621876    ,    0.617347    ,    0.611962    ,    0.606152    ,    0.599164    ,    0.584502    ,    0.599872    ,    0.7739    ,    0.896372    ,    0.845717    ,    0.76908    ,    0.714261    ,    0.745744    ,    0.837866    ,    0.897915    ,    0.86871    ,    0.835558    ,    0.814366    ,    0.807058    ,    0.807653    ,    0.80968    ,    0.812706    ,    0.817353    ,    0.821542    ,    0.824438    ,    0.831424    ,    0.851595    ,    0.859628    ,    0.810806    ,    0.76296    ,    0.750006    ,    0.754014    ,    0.763765    ,    0.772374    ,    0.78072    ,    0.798473    ,    0.815042    ,    0.829089    ,    0.845042    ,    0.851317    ,    0.842462    ,    0.827176    ,    0.836245    ,    0.819594    ,    0.792219    ,    0.812604    ,    0.826959    ,    0.84431    ,    0.813578    ,    0.74433    ,    0.701069    ,    0.670482    ,    0.660222    ,    0.665085    ,    0.665431    ,    0.663625    ,    0.660882    ,    0.658017    ,    0.654345    ,    0.653113    ,    0.654516    ,    0.663487    ,    0.667075    ,    0.665381    ,    0.663301    ,    0.65742    ,    0.643462    ,    0.631214    ,    0.624878    ,    0.624995    ,    0.623275    ,    0.621357    ,    0.621667    ,    0.618936    ,    0.614673    ,    0.614158    ,    0.615566    ,    0.614025    ,    0.61069    ,    0.61058    ,    0.613355    ,    0.617476    ,    0.628552    ,    0.649817    ,    0.705505    ,    0.804051    ,    0.890574    ,    0.942662    ,    0.888924    ,    0.835119    ,    0.815409    ,    0.809405    ,    0.810079    ,    0.818571    ,    0.847742    ,    0.883175    ,    0.852728    },
    {    0.787296    ,    0.746314    ,    0.725413    ,    0.718678    ,    0.714148    ,    0.710027    ,    0.709798    ,    0.709197    ,    0.705914    ,    0.702772    ,    0.699813    ,    0.698963    ,    0.695922    ,    0.691789    ,    0.687995    ,    0.684287    ,    0.680879    ,    0.676616    ,    0.672237    ,    0.665854    ,    0.659066    ,    0.655818    ,    0.653069    ,    0.648527    ,    0.645306    ,    0.642437    ,    0.641033    ,    0.637014    ,    0.630691    ,    0.626556    ,    0.623451    ,    0.616452    ,    0.608262    ,    0.596797    ,    0.581218    ,    0.564947    ,    0.550967    ,    0.583293    ,    0.803891    ,    0.957918    ,    0.857202    ,    0.806275    ,    0.788599    ,    0.779709    ,    0.847105    ,    0.879896    ,    0.802951    ,    0.773209    ,    0.764622    ,    0.763161    ,    0.759873    ,    0.755912    ,    0.755994    ,    0.754674    ,    0.757219    ,    0.762893    ,    0.766914    ,    0.767312    ,    0.751183    ,    0.722944    ,    0.71054    ,    0.718008    ,    0.728797    ,    0.743774    ,    0.760415    ,    0.787172    ,    0.815663    ,    0.839931    ,    0.85664    ,    0.851054    ,    0.816161    ,    0.778232    ,    0.752287    ,    0.72338    ,    0.679399    ,    0.646827    ,    0.638104    ,    0.642723    ,    0.647504    ,    0.637484    ,    0.615318    ,    0.604714    ,    0.599915    ,    0.601413    ,    0.609454    ,    0.615495    ,    0.614499    ,    0.61455    ,    0.616936    ,    0.618372    ,    0.618629    ,    0.625585    ,    0.636751    ,    0.643775    ,    0.649023    ,    0.651927    ,    0.651069    ,    0.652425    ,    0.659522    ,    0.666277    ,    0.67236    ,    0.670591    ,    0.670697    ,    0.67223    ,    0.662991    ,    0.661305    ,    0.668338    ,    0.674284    ,    0.672023    ,    0.666829    ,    0.658752    ,    0.647814    ,    0.644696    ,    0.671715    ,    0.7802    ,    0.893827    ,    0.94673    ,    0.954372    ,    0.964678    ,    0.931216    ,    0.879247    ,    0.857399    ,    0.853299    ,    0.850119    ,    0.846581    ,    0.852217    ,    0.878766    ,    0.869059    },
    {    0.764995    ,    0.746356    ,    0.743082    ,    0.742962    ,    0.741505    ,    0.739112    ,    0.738932    ,    0.73861    ,    0.737164    ,    0.735599    ,    0.732161    ,    0.730361    ,    0.726447    ,    0.720319    ,    0.71536    ,    0.709855    ,    0.704726    ,    0.696611    ,    0.689034    ,    0.680503    ,    0.672047    ,    0.666105    ,    0.65803    ,    0.647685    ,    0.637966    ,    0.631093    ,    0.624925    ,    0.617063    ,    0.612929    ,    0.606511    ,    0.593832    ,    0.585028    ,    0.573481    ,    0.555363    ,    0.536875    ,    0.523018    ,    0.520959    ,    0.582601    ,    0.780482    ,    1.01387    ,    0.901126    ,    0.820469    ,    0.815572    ,    0.828372    ,    0.829727    ,    0.811552    ,    0.756149    ,    0.721343    ,    0.70225    ,    0.696767    ,    0.688661    ,    0.686475    ,    0.686891    ,    0.686428    ,    0.686773    ,    0.686431    ,    0.682002    ,    0.67454    ,    0.66795    ,    0.671028    ,    0.681859    ,    0.691743    ,    0.704983    ,    0.72445    ,    0.747987    ,    0.776774    ,    0.797292    ,    0.798218    ,    0.785198    ,    0.752074    ,    0.721375    ,    0.750784    ,    0.761696    ,    0.757809    ,    0.727747    ,    0.671494    ,    0.614476    ,    0.600736    ,    0.609283    ,    0.621797    ,    0.631585    ,    0.636875    ,    0.639284    ,    0.639661    ,    0.641126    ,    0.643098    ,    0.646888    ,    0.654274    ,    0.660995    ,    0.662695    ,    0.665452    ,    0.675879    ,    0.687756    ,    0.696441    ,    0.703478    ,    0.708838    ,    0.710268    ,    0.711655    ,    0.714881    ,    0.71436    ,    0.710953    ,    0.701172    ,    0.695653    ,    0.695915    ,    0.690256    ,    0.738392    ,    0.784335    ,    0.733749    ,    0.70552    ,    0.684796    ,    0.674476    ,    0.672389    ,    0.713501    ,    0.846994    ,    0.976031    ,    1.05001    ,    1.10969    ,    1.07642    ,    0.962218    ,    0.894756    ,    0.843705    ,    0.835154    ,    0.841085    ,    0.845046    ,    0.85067    ,    0.852933    ,    0.851492    ,    0.812361    },
    {    0.767308    ,    0.76647    ,    0.767887    ,    0.765785    ,    0.763931    ,    0.761085    ,    0.75984    ,    0.760057    ,    0.760194    ,    0.75743    ,    0.750613    ,    0.745986    ,    0.740273    ,    0.733304    ,    0.726707    ,    0.718835    ,    0.711    ,    0.700319    ,    0.688461    ,    0.676067    ,    0.663751    ,    0.652118    ,    0.639137    ,    0.625585    ,    0.611245    ,    0.59942    ,    0.590754    ,    0.581082    ,    0.572372    ,    0.56152    ,    0.549537    ,    0.539986    ,    0.526125    ,    0.511811    ,    0.500001    ,    0.493491    ,    0.51196    ,    0.584266    ,    0.757108    ,    1.04746    ,    0.976792    ,    0.885396    ,    0.891627    ,    0.920457    ,    0.939976    ,    0.872061    ,    0.751905    ,    0.724206    ,    0.687179    ,    0.674752    ,    0.670835    ,    0.668918    ,    0.66641    ,    0.661575    ,    0.654606    ,    0.647934    ,    0.64046    ,    0.634371    ,    0.637103    ,    0.646167    ,    0.656122    ,    0.668243    ,    0.682969    ,    0.705443    ,    0.727507    ,    0.743689    ,    0.749102    ,    0.738562    ,    0.713318    ,    0.677297    ,    0.734873    ,    1.06231    ,    1.15633    ,    1.10921    ,    0.983127    ,    0.796238    ,    0.732795    ,    0.700055    ,    0.706097    ,    0.716007    ,    0.726933    ,    0.728318    ,    0.72765    ,    0.72477    ,    0.721036    ,    0.72219    ,    0.725196    ,    0.729035    ,    0.731843    ,    0.73331    ,    0.736525    ,    0.742642    ,    0.746643    ,    0.750788    ,    0.752707    ,    0.75187    ,    0.744571    ,    0.736013    ,    0.733158    ,    0.730116    ,    0.721712    ,    0.706699    ,    0.698866    ,    0.704829    ,    0.729622    ,    0.891208    ,    1.04532    ,    0.984913    ,    0.894343    ,    0.799759    ,    0.764482    ,    0.800221    ,    0.966748    ,    1.06126    ,    1.1005    ,    1.13928    ,    1.17663    ,    1.15543    ,    1.03218    ,    0.84352    ,    0.787233    ,    0.784023    ,    0.795219    ,    0.804994    ,    0.814203    ,    0.812754    ,    0.802534    ,    0.78639    },
    {    0.788392    ,    0.786385    ,    0.781187    ,    0.775003    ,    0.769147    ,    0.767642    ,    0.769196    ,    0.770884    ,    0.768577    ,    0.762114    ,    0.754736    ,    0.747735    ,    0.738678    ,    0.730485    ,    0.721406    ,    0.709193    ,    0.695217    ,    0.680333    ,    0.663879    ,    0.645351    ,    0.628957    ,    0.613172    ,    0.594537    ,    0.577824    ,    0.564142    ,    0.550916    ,    0.539619    ,    0.529084    ,    0.518128    ,    0.510428    ,    0.504624    ,    0.49587    ,    0.48663    ,    0.479033    ,    0.474479    ,    0.482149    ,    0.522462    ,    0.606519    ,    0.773458    ,    1.09349    ,    1.11819    ,    0.987158    ,    1.00981    ,    1.03174    ,    1.06744    ,    1.02637    ,    0.90757    ,    0.74219    ,    0.707091    ,    0.691023    ,    0.68222    ,    0.676153    ,    0.668696    ,    0.657322    ,    0.645878    ,    0.637255    ,    0.63081    ,    0.625167    ,    0.628573    ,    0.638101    ,    0.651217    ,    0.667819    ,    0.681871    ,    0.696602    ,    0.706751    ,    0.712513    ,    0.707126    ,    0.691685    ,    0.675213    ,    0.684305    ,    0.89762    ,    1.27686    ,    1.43645    ,    1.42239    ,    1.25438    ,    1.01896    ,    0.810926    ,    0.790649    ,    0.790733    ,    0.795923    ,    0.795708    ,    0.792429    ,    0.801148    ,    0.800086    ,    0.787863    ,    0.781687    ,    0.777696    ,    0.774209    ,    0.772335    ,    0.771635    ,    0.770871    ,    0.770776    ,    0.768287    ,    0.767265    ,    0.762663    ,    0.754767    ,    0.747342    ,    0.741421    ,    0.737774    ,    0.727373    ,    0.712525    ,    0.701394    ,    0.702182    ,    0.720108    ,    0.771964    ,    1.02243    ,    1.16476    ,    1.10972    ,    1.06945    ,    1.04128    ,    1.03086    ,    1.03883    ,    1.11869    ,    1.17019    ,    1.1925    ,    1.21022    ,    1.2183    ,    1.18423    ,    1.07879    ,    0.827464    ,    0.773241    ,    0.776488    ,    0.788439    ,    0.795092    ,    0.795673    ,    0.796129    ,    0.792503    ,    0.790922    },
    {    0.79024    ,    0.783945    ,    0.776001    ,    0.769101    ,    0.764086    ,    0.762246    ,    0.761866    ,    0.759313    ,    0.753991    ,    0.746973    ,    0.738097    ,    0.727518    ,    0.715107    ,    0.70054    ,    0.685351    ,    0.668887    ,    0.650334    ,    0.631265    ,    0.611873    ,    0.593734    ,    0.576261    ,    0.558678    ,    0.541008    ,    0.526846    ,    0.515028    ,    0.503881    ,    0.493761    ,    0.485479    ,    0.478189    ,    0.472362    ,    0.467956    ,    0.462892    ,    0.461019    ,    0.456003    ,    0.464168    ,    0.498167    ,    0.557064    ,    0.646003    ,    0.783072    ,    1.09106    ,    1.30625    ,    1.11071    ,    1.13296    ,    1.16335    ,    1.21608    ,    1.14389    ,    1.03858    ,    0.822011    ,    0.728257    ,    0.706676    ,    0.692653    ,    0.686591    ,    0.674745    ,    0.658968    ,    0.648629    ,    0.641586    ,    0.63885    ,    0.638863    ,    0.644781    ,    0.652997    ,    0.664347    ,    0.675591    ,    0.685102    ,    0.689695    ,    0.689228    ,    0.682656    ,    0.674108    ,    0.677789    ,    0.702084    ,    0.799218    ,    1.20357    ,    1.43176    ,    1.51069    ,    1.47857    ,    1.34927    ,    1.17238    ,    0.893926    ,    0.824185    ,    0.827899    ,    0.844174    ,    0.861772    ,    0.808724    ,    0.801195    ,    0.803206    ,    0.794061    ,    0.787547    ,    0.77858    ,    0.77464    ,    0.771019    ,    0.767768    ,    0.765524    ,    0.76411    ,    0.758919    ,    0.753422    ,    0.748539    ,    0.743055    ,    0.736666    ,    0.728446    ,    0.718867    ,    0.708035    ,    0.700501    ,    0.702309    ,    0.715495    ,    0.744955    ,    0.829112    ,    1.16394    ,    1.2599    ,    1.2233    ,    1.20364    ,    1.20239    ,    1.20254    ,    1.19586    ,    1.19308    ,    1.22986    ,    1.23218    ,    1.24714    ,    1.2148    ,    1.16195    ,    1.03685    ,    0.7963    ,    0.782447    ,    0.792985    ,    0.796382    ,    0.798616    ,    0.798139    ,    0.798963    ,    0.797721    ,    0.794001    },
    {    0.770355    ,    0.764932    ,    0.758421    ,    0.752468    ,    0.747846    ,    0.744155    ,    0.741217    ,    0.735874    ,    0.727338    ,    0.717943    ,    0.705579    ,    0.692396    ,    0.678041    ,    0.663019    ,    0.647691    ,    0.630147    ,    0.613111    ,    0.596531    ,    0.579604    ,    0.561561    ,    0.544276    ,    0.52877    ,    0.515418    ,    0.502621    ,    0.490126    ,    0.481521    ,    0.472034    ,    0.463109    ,    0.455672    ,    0.450846    ,    0.448104    ,    0.448973    ,    0.453094    ,    0.460258    ,    0.487146    ,    0.536958    ,    0.597428    ,    0.688077    ,    0.805553    ,    1.11791    ,    1.37546    ,    1.22963    ,    1.24683    ,    1.28501    ,    1.3072    ,    1.25004    ,    1.17214    ,    1.05857    ,    0.898057    ,    0.754843    ,    0.701032    ,    0.688128    ,    0.681164    ,    0.669397    ,    0.665831    ,    0.662936    ,    0.662315    ,    0.666217    ,    0.6722    ,    0.677888    ,    0.681134    ,    0.679198    ,    0.678203    ,    0.671339    ,    0.664486    ,    0.663895    ,    0.676766    ,    0.707661    ,    0.753466    ,    0.949388    ,    1.3812    ,    1.50132    ,    1.51537    ,    1.46643    ,    1.38307    ,    1.26113    ,    1.10196    ,    0.856443    ,    0.848032    ,    0.903833    ,    1.06519    ,    0.882251    ,    0.785728    ,    0.78264    ,    0.776656    ,    0.771694    ,    0.763688    ,    0.76097    ,    0.757628    ,    0.755005    ,    0.752988    ,    0.750654    ,    0.747793    ,    0.744099    ,    0.741343    ,    0.736481    ,    0.729493    ,    0.721395    ,    0.717444    ,    0.714648    ,    0.714569    ,    0.720904    ,    0.735004    ,    0.764377    ,    0.855226    ,    1.203    ,    1.2855    ,    1.27662    ,    1.28166    ,    1.28249    ,    1.27839    ,    1.2537    ,    1.23198    ,    1.2384    ,    1.23394    ,    1.22777    ,    1.17015    ,    1.03778    ,    0.856944    ,    0.77909    ,    0.787321    ,    0.793992    ,    0.793031    ,    0.793981    ,    0.793451    ,    0.789925    ,    0.785272    ,    0.778889    },
    {    0.754507    ,    0.750942    ,    0.74627    ,    0.742839    ,    0.739916    ,    0.733872    ,    0.726989    ,    0.719105    ,    0.708412    ,    0.697016    ,    0.683733    ,    0.670907    ,    0.658316    ,    0.646173    ,    0.634685    ,    0.621609    ,    0.60754    ,    0.592196    ,    0.576646    ,    0.558754    ,    0.54356    ,    0.530476    ,    0.515516    ,    0.504181    ,    0.495163    ,    0.487065    ,    0.478068    ,    0.472371    ,    0.469586    ,    0.467967    ,    0.472195    ,    0.480206    ,    0.492912    ,    0.51563    ,    0.549711    ,    0.60339    ,    0.670246    ,    0.745698    ,    0.828659    ,    1.13139    ,    1.37184    ,    1.28407    ,    1.32426    ,    1.34759    ,    1.35579    ,    1.30123    ,    1.24644    ,    1.21242    ,    1.11761    ,    0.984613    ,    0.747767    ,    0.700073    ,    0.698399    ,    0.694342    ,    0.691259    ,    0.690391    ,    0.691498    ,    0.691354    ,    0.689529    ,    0.685487    ,    0.678361    ,    0.670225    ,    0.666401    ,    0.663656    ,    0.669658    ,    0.684981    ,    0.710984    ,    0.749555    ,    0.800426    ,    1.08104    ,    1.44091    ,    1.51225    ,    1.4856    ,    1.4211    ,    1.35889    ,    1.295    ,    1.153    ,    0.861611    ,    0.851835    ,    0.901712    ,    1.07588    ,    0.940614    ,    0.780193    ,    0.769704    ,    0.767959    ,    0.764644    ,    0.762201    ,    0.760131    ,    0.758914    ,    0.757916    ,    0.756583    ,    0.755719    ,    0.753555    ,    0.749345    ,    0.74725    ,    0.743025    ,    0.739508    ,    0.736855    ,    0.735958    ,    0.734351    ,    0.735696    ,    0.74243    ,    0.756178    ,    0.776388    ,    0.823112    ,    0.983681    ,    1.1395    ,    1.22004    ,    1.24796    ,    1.25606    ,    1.25656    ,    1.24821    ,    1.21384    ,    1.18988    ,    1.18158    ,    1.14109    ,    1.03171    ,    0.874728    ,    0.777163    ,    0.776203    ,    0.784976    ,    0.788303    ,    0.787495    ,    0.787241    ,    0.780026    ,    0.775895    ,    0.769238    ,    0.762518    },
    {    0.751069    ,    0.747018    ,    0.74317    ,    0.740579    ,    0.736018    ,    0.729398    ,    0.722369    ,    0.714827    ,    0.705139    ,    0.694007    ,    0.683685    ,    0.673016    ,    0.663697    ,    0.654916    ,    0.644221    ,    0.632972    ,    0.61877    ,    0.603271    ,    0.588247    ,    0.574688    ,    0.561103    ,    0.552054    ,    0.542997    ,    0.535756    ,    0.530149    ,    0.527741    ,    0.526533    ,    0.526324    ,    0.52916    ,    0.536783    ,    0.550338    ,    0.568444    ,    0.592259    ,    0.621825    ,    0.658107    ,    0.705534    ,    0.763703    ,    0.825879    ,    0.939791    ,    1.19776    ,    1.2992    ,    1.30465    ,    1.36941    ,    1.35958    ,    1.34982    ,    1.29464    ,    1.24338    ,    1.19518    ,    1.1424    ,    1.0352    ,    0.830841    ,    0.721038    ,    0.715258    ,    0.710457    ,    0.705711    ,    0.701628    ,    0.699138    ,    0.694626    ,    0.688427    ,    0.681279    ,    0.677014    ,    0.676753    ,    0.680904    ,    0.690565    ,    0.708211    ,    0.735182    ,    0.77082    ,    0.806615    ,    0.887421    ,    1.26899    ,    1.43472    ,    1.45991    ,    1.42924    ,    1.37139    ,    1.32011    ,    1.25233    ,    1.21254    ,    1.01732    ,    0.859707    ,    0.862743    ,    1.01453    ,    0.983838    ,    0.789813    ,    0.77104    ,    0.769918    ,    0.768818    ,    0.766199    ,    0.764147    ,    0.763144    ,    0.761578    ,    0.759713    ,    0.758756    ,    0.757157    ,    0.755315    ,    0.75502    ,    0.754726    ,    0.754794    ,    0.754345    ,    0.755773    ,    0.759351    ,    0.764334    ,    0.769462    ,    0.777525    ,    0.792021    ,    0.817548    ,    0.853953    ,    0.881296    ,    0.947364    ,    1.13366    ,    1.19446    ,    1.21368    ,    1.20287    ,    1.18481    ,    1.1122    ,    1.06553    ,    1.0195    ,    0.869098    ,    0.782779    ,    0.777622    ,    0.780753    ,    0.781369    ,    0.778988    ,    0.774601    ,    0.771016    ,    0.768967    ,    0.764771    ,    0.760495    ,    0.761332    },
    {    0.752579    ,    0.749651    ,    0.746878    ,    0.744017    ,    0.739396    ,    0.734781    ,    0.729962    ,    0.723774    ,    0.716329    ,    0.707509    ,    0.699175    ,    0.690883    ,    0.684155    ,    0.675358    ,    0.666658    ,    0.656799    ,    0.646146    ,    0.636623    ,    0.626085    ,    0.618157    ,    0.610132    ,    0.604138    ,    0.599819    ,    0.597055    ,    0.595838    ,    0.598475    ,    0.601927    ,    0.609075    ,    0.619231    ,    0.634591    ,    0.654406    ,    0.676438    ,    0.704588    ,    0.734845    ,    0.773061    ,    0.80989    ,    0.851962    ,    0.986859    ,    1.20609    ,    1.33608    ,    1.28985    ,    1.42495    ,    1.39879    ,    1.35742    ,    1.3204    ,    1.27941    ,    1.23444    ,    1.18958    ,    1.14635    ,    1.0568    ,    0.881036    ,    0.727075    ,    0.71721    ,    0.71098    ,    0.703418    ,    0.699447    ,    0.696008    ,    0.693525    ,    0.692236    ,    0.694319    ,    0.700246    ,    0.708781    ,    0.721916    ,    0.736368    ,    0.754892    ,    0.780387    ,    0.807995    ,    0.826759    ,    0.903223    ,    1.25204    ,    1.36622    ,    1.3971    ,    1.3679    ,    1.3363    ,    1.2977    ,    1.26285    ,    1.22891    ,    1.16003    ,    1.01203    ,    0.849692    ,    0.879746    ,    0.959149    ,    0.817411    ,    0.778842    ,    0.779476    ,    0.776872    ,    0.775541    ,    0.773817    ,    0.772408    ,    0.770815    ,    0.770153    ,    0.769398    ,    0.768564    ,    0.768969    ,    0.769745    ,    0.77074    ,    0.771004    ,    0.773306    ,    0.779706    ,    0.784658    ,    0.789896    ,    0.796602    ,    0.802806    ,    0.812766    ,    0.82937    ,    0.849473    ,    0.864924    ,    0.878416    ,    0.932172    ,    1.06152    ,    1.08378    ,    1.138    ,    1.08035    ,    0.890664    ,    0.881582    ,    0.932435    ,    0.784411    ,    0.782056    ,    0.786801    ,    0.785267    ,    0.7826    ,    0.77823    ,    0.774118    ,    0.771237    ,    0.766532    ,    0.761705    ,    0.758284    ,    0.755012    },
    {    0.758857    ,    0.758135    ,    0.757372    ,    0.755004    ,    0.751605    ,    0.747041    ,    0.743087    ,    0.737704    ,    0.732422    ,    0.725989    ,    0.71925    ,    0.714648    ,    0.712178    ,    0.706017    ,    0.701545    ,    0.697598    ,    0.691434    ,    0.684077    ,    0.6781    ,    0.672778    ,    0.667445    ,    0.663187    ,    0.659356    ,    0.658883    ,    0.659187    ,    0.661855    ,    0.666934    ,    0.679211    ,    0.694038    ,    0.711285    ,    0.73361    ,    0.756893    ,    0.784536    ,    0.813915    ,    0.847464    ,    0.873639    ,    0.920698    ,    1.15227    ,    1.22537    ,    1.30676    ,    1.41493    ,    1.44074    ,    1.39014    ,    1.37434    ,    1.34819    ,    1.32546    ,    1.26744    ,    1.20207    ,    1.17599    ,    1.06761    ,    0.942085    ,    0.754642    ,    0.724805    ,    0.724442    ,    0.7185    ,    0.718438    ,    0.719413    ,    0.72286    ,    0.727108    ,    0.73493    ,    0.742177    ,    0.751155    ,    0.76209    ,    0.772234    ,    0.786827    ,    0.804967    ,    0.825886    ,    0.841566    ,    0.845019    ,    1.08243    ,    1.28554    ,    1.31957    ,    1.30114    ,    1.27967    ,    1.2595    ,    1.24742    ,    1.19498    ,    1.15532    ,    0.992479    ,    0.833139    ,    0.825936    ,    0.846623    ,    0.815332    ,    0.800224    ,    0.799546    ,    0.79882    ,    0.798227    ,    0.797086    ,    0.796349    ,    0.794329    ,    0.791552    ,    0.790871    ,    0.789292    ,    0.78897    ,    0.788855    ,    0.789634    ,    0.788184    ,    0.790513    ,    0.793136    ,    0.796199    ,    0.801757    ,    0.810121    ,    0.814507    ,    0.821725    ,    0.829192    ,    0.835314    ,    0.846135    ,    0.853535    ,    0.858427    ,    0.860816    ,    0.892019    ,    1.00104    ,    0.957    ,    0.828788    ,    0.827594    ,    0.84287    ,    0.773869    ,    0.782896    ,    0.787707    ,    0.794168    ,    0.791939    ,    0.786632    ,    0.783018    ,    0.776556    ,    0.771813    ,    0.767818    ,    0.764156    ,    0.761665    },
    {    0.771259    ,    0.769116    ,    0.767969    ,    0.7644    ,    0.761615    ,    0.757804    ,    0.754645    ,    0.750438    ,    0.746486    ,    0.74331    ,    0.742262    ,    0.740485    ,    0.739714    ,    0.737909    ,    0.737256    ,    0.735843    ,    0.734381    ,    0.729698    ,    0.726571    ,    0.72241    ,    0.718733    ,    0.717777    ,    0.716999    ,    0.718219    ,    0.720337    ,    0.725305    ,    0.732254    ,    0.741544    ,    0.753774    ,    0.770118    ,    0.790252    ,    0.812416    ,    0.837141    ,    0.870018    ,    0.897323    ,    0.901932    ,    0.999201    ,    1.2101    ,    1.32389    ,    1.39128    ,    1.39372    ,    1.36701    ,    1.37364    ,    1.38554    ,    1.36024    ,    1.34153    ,    1.3113    ,    1.23738    ,    1.17915    ,    1.10707    ,    1.02164    ,    0.880345    ,    0.75559    ,    0.759296    ,    0.757273    ,    0.757153    ,    0.759673    ,    0.762837    ,    0.767232    ,    0.775216    ,    0.780921    ,    0.789017    ,    0.795057    ,    0.800888    ,    0.809033    ,    0.820986    ,    0.838547    ,    0.853641    ,    0.834186    ,    1.01685    ,    1.21031    ,    1.24837    ,    1.22178    ,    1.21543    ,    1.20938    ,    1.22096    ,    1.19255    ,    1.12879    ,    0.95889    ,    0.828131    ,    0.824565    ,    0.821662    ,    0.822371    ,    0.824369    ,    0.825397    ,    0.825382    ,    0.825027    ,    0.823897    ,    0.822897    ,    0.822113    ,    0.819269    ,    0.816761    ,    0.81649    ,    0.81201    ,    0.809228    ,    0.804705    ,    0.80121    ,    0.800738    ,    0.796521    ,    0.796666    ,    0.794749    ,    0.792388    ,    0.796678    ,    0.808461    ,    0.822969    ,    0.830752    ,    0.841256    ,    0.847228    ,    0.84809    ,    0.836223    ,    0.82458    ,    0.821997    ,    0.804253    ,    0.794898    ,    0.813631    ,    0.807746    ,    0.809522    ,    0.824557    ,    0.795418    ,    0.800992    ,    0.796421    ,    0.796376    ,    0.789831    ,    0.785409    ,    0.782461    ,    0.778403    ,    0.774587    ,    0.773308    },
    {    0.786462    ,    0.785508    ,    0.783186    ,    0.781798    ,    0.779642    ,    0.778561    ,    0.776479    ,    0.775541    ,    0.774814    ,    0.774081    ,    0.775582    ,    0.77545    ,    0.775555    ,    0.777741    ,    0.777195    ,    0.776042    ,    0.774467    ,    0.77286    ,    0.772585    ,    0.7724    ,    0.773033    ,    0.774598    ,    0.777373    ,    0.780011    ,    0.785123    ,    0.790133    ,    0.795279    ,    0.803292    ,    0.813964    ,    0.827552    ,    0.841245    ,    0.857039    ,    0.880667    ,    0.908368    ,    0.918709    ,    0.90655    ,    1.06265    ,    1.27823    ,    1.30857    ,    1.30622    ,    1.31967    ,    1.33597    ,    1.35557    ,    1.35584    ,    1.34829    ,    1.32034    ,    1.29846    ,    1.23132    ,    1.19549    ,    1.14116    ,    1.09149    ,    0.967294    ,    0.792397    ,    0.797896    ,    0.798583    ,    0.797633    ,    0.800004    ,    0.801019    ,    0.805751    ,    0.810537    ,    0.81607    ,    0.821575    ,    0.825595    ,    0.828627    ,    0.829172    ,    0.836247    ,    0.849786    ,    0.857436    ,    0.868915    ,    1.07759    ,    1.2036    ,    1.21138    ,    1.19982    ,    1.17727    ,    1.16789    ,    1.16819    ,    1.14676    ,    1.0747    ,    0.902433    ,    0.840829    ,    0.847756    ,    0.848741    ,    0.849697    ,    0.849434    ,    0.852745    ,    0.850052    ,    0.849326    ,    0.846968    ,    0.844793    ,    0.843881    ,    0.842256    ,    0.837765    ,    0.83359    ,    0.827902    ,    0.82296    ,    0.817555    ,    0.812244    ,    0.805457    ,    0.798623    ,    0.789447    ,    0.773581    ,    0.796944    ,    0.903627    ,    0.892331    ,    0.870013    ,    0.836312    ,    0.829233    ,    0.826505    ,    0.816944    ,    0.818062    ,    0.811508    ,    0.802749    ,    0.790548    ,    0.827907    ,    0.917407    ,    0.926989    ,    0.902402    ,    0.819212    ,    0.803889    ,    0.80398    ,    0.801284    ,    0.797245    ,    0.793015    ,    0.792826    ,    0.790842    ,    0.790038    ,    0.788614    ,    0.787431    },
    {    0.829079    ,    0.831037    ,    0.8321    ,    0.833398    ,    0.833709    ,    0.835158    ,    0.836614    ,    0.838997    ,    0.842758    ,    0.844142    ,    0.845512    ,    0.846932    ,    0.84942    ,    0.847627    ,    0.848365    ,    0.847754    ,    0.846584    ,    0.845879    ,    0.844923    ,    0.844604    ,    0.84355    ,    0.846296    ,    0.849878    ,    0.853278    ,    0.858145    ,    0.861613    ,    0.867358    ,    0.874366    ,    0.880056    ,    0.886002    ,    0.892965    ,    0.899248    ,    0.910636    ,    0.919443    ,    0.908512    ,    0.893842    ,    1.06037    ,    1.24027    ,    1.26814    ,    1.2572    ,    1.28117    ,    1.30416    ,    1.34492    ,    1.34627    ,    1.31434    ,    1.28631    ,    1.24529    ,    1.2148    ,    1.1656    ,    1.0755    ,    0.984483    ,    0.853745    ,    0.818554    ,    0.828468    ,    0.829094    ,    0.832299    ,    0.83457    ,    0.835372    ,    0.838451    ,    0.8408    ,    0.84419    ,    0.845641    ,    0.849204    ,    0.850235    ,    0.849946    ,    0.855448    ,    0.864399    ,    0.876868    ,    0.968403    ,    1.11481    ,    1.17166    ,    1.18811    ,    1.19675    ,    1.19578    ,    1.18932    ,    1.15804    ,    1.12373    ,    1.04833    ,    0.909303    ,    0.856312    ,    0.869981    ,    0.872563    ,    0.868278    ,    0.867711    ,    0.866433    ,    0.864113    ,    0.861511    ,    0.856478    ,    0.853481    ,    0.851168    ,    0.848439    ,    0.844575    ,    0.841162    ,    0.836536    ,    0.830799    ,    0.824715    ,    0.817935    ,    0.80739    ,    0.796831    ,    0.780056    ,    0.796726    ,    0.938032    ,    0.869825    ,    0.826401    ,    0.841611    ,    0.855112    ,    0.832012    ,    0.876798    ,    0.818143    ,    0.818528    ,    0.818884    ,    0.817879    ,    0.862979    ,    0.918691    ,    0.928075    ,    0.892824    ,    0.818834    ,    0.807187    ,    0.827687    ,    0.819061    ,    0.817221    ,    0.817956    ,    0.819328    ,    0.821865    ,    0.82413    ,    0.826205    ,    0.827938    ,    0.82985    },
    {    0.880967    ,    0.883741    ,    0.885022    ,    0.886764    ,    0.887723    ,    0.887595    ,    0.888862    ,    0.88958    ,    0.89135    ,    0.892565    ,    0.894746    ,    0.89713    ,    0.899868    ,    0.901777    ,    0.904016    ,    0.905439    ,    0.905851    ,    0.907155    ,    0.909822    ,    0.911164    ,    0.914178    ,    0.918886    ,    0.924882    ,    0.930656    ,    0.936901    ,    0.943495    ,    0.949991    ,    0.952707    ,    0.955037    ,    0.956345    ,    0.95443    ,    0.946444    ,    0.935642    ,    0.923557    ,    0.900382    ,    0.881573    ,    1.00754    ,    1.17854    ,    1.21882    ,    1.21084    ,    1.25289    ,    1.29835    ,    1.33225    ,    1.34007    ,    1.31191    ,    1.19201    ,    1.10551    ,    1.03985    ,    0.895318    ,    0.849746    ,    0.840334    ,    0.837298    ,    0.846212    ,    0.851877    ,    0.853891    ,    0.856922    ,    0.860139    ,    0.863117    ,    0.866157    ,    0.868191    ,    0.870823    ,    0.870745    ,    0.873114    ,    0.871693    ,    0.870971    ,    0.873527    ,    0.880511    ,    0.904121    ,    1.03165    ,    1.08942    ,    1.15743    ,    1.15142    ,    1.16318    ,    1.16376    ,    1.15658    ,    1.12363    ,    1.0682    ,    1.04327    ,    0.982423    ,    0.893742    ,    0.876476    ,    0.889373    ,    0.888374    ,    0.881944    ,    0.875004    ,    0.867476    ,    0.860143    ,    0.85338    ,    0.847201    ,    0.842333    ,    0.838054    ,    0.833136    ,    0.832174    ,    0.828746    ,    0.823743    ,    0.81808    ,    0.813051    ,    0.804436    ,    0.795051    ,    0.797194    ,    0.944222    ,    0.944067    ,    0.842236    ,    0.896815    ,    0.988025    ,    0.974256    ,    0.870531    ,    0.904754    ,    0.834312    ,    0.820971    ,    0.823703    ,    0.882087    ,    0.859996    ,    0.845844    ,    0.821487    ,    0.81871    ,    0.820664    ,    0.831314    ,    0.841017    ,    0.844883    ,    0.849789    ,    0.853865    ,    0.858713    ,    0.86408    ,    0.870489    ,    0.87182    ,    0.874444    ,    0.879277    },
    {    0.879052    ,    0.880219    ,    0.881933    ,    0.882615    ,    0.884002    ,    0.884214    ,    0.883277    ,    0.88398    ,    0.885464    ,    0.887211    ,    0.88847    ,    0.890915    ,    0.894888    ,    0.89459    ,    0.897078    ,    0.900337    ,    0.90269    ,    0.907692    ,    0.913043    ,    0.918285    ,    0.924432    ,    0.930999    ,    0.93803    ,    0.946914    ,    0.954204    ,    0.959993    ,    0.961843    ,    0.961998    ,    0.960085    ,    0.952454    ,    0.944054    ,    0.92893    ,    0.910132    ,    0.892662    ,    0.877949    ,    0.85442    ,    0.922153    ,    1.09535    ,    1.20991    ,    1.23135    ,    1.2523    ,    1.29912    ,    1.32583    ,    1.30466    ,    1.20698    ,    1.09482    ,    0.950628    ,    0.837507    ,    0.838747    ,    0.841839    ,    0.845074    ,    0.850923    ,    0.857546    ,    0.86228    ,    0.864685    ,    0.867684    ,    0.869117    ,    0.87052    ,    0.870392    ,    0.872241    ,    0.87024    ,    0.869777    ,    0.866955    ,    0.865633    ,    0.867971    ,    0.869604    ,    0.874219    ,    0.887529    ,    1.04164    ,    1.11745    ,    1.14835    ,    1.15462    ,    1.16022    ,    1.16424    ,    1.13899    ,    1.10549    ,    1.02393    ,    1.06484    ,    1.03182    ,    0.984871    ,    0.910359    ,    0.894497    ,    0.899736    ,    0.89372    ,    0.887661    ,    0.881674    ,    0.872005    ,    0.863637    ,    0.853732    ,    0.846994    ,    0.839324    ,    0.831852    ,    0.829597    ,    0.82704    ,    0.824016    ,    0.822094    ,    0.818497    ,    0.81549    ,    0.814039    ,    0.883361    ,    0.977336    ,    0.877908    ,    0.819334    ,    0.905965    ,    0.971101    ,    0.967972    ,    0.876858    ,    0.843343    ,    0.848296    ,    0.832599    ,    0.829836    ,    0.82354    ,    0.82419    ,    0.824051    ,    0.826513    ,    0.825661    ,    0.828712    ,    0.836147    ,    0.843563    ,    0.84559    ,    0.848006    ,    0.852247    ,    0.856726    ,    0.863506    ,    0.869218    ,    0.872572    ,    0.875455    ,    0.877639    },
    {    0.838098    ,    0.838659    ,    0.839811    ,    0.840983    ,    0.840499    ,    0.842699    ,    0.842611    ,    0.841922    ,    0.843379    ,    0.845529    ,    0.846599    ,    0.848152    ,    0.850507    ,    0.85044    ,    0.852544    ,    0.854987    ,    0.857169    ,    0.860501    ,    0.862754    ,    0.865609    ,    0.868696    ,    0.872254    ,    0.877318    ,    0.879949    ,    0.882996    ,    0.883789    ,    0.881655    ,    0.878542    ,    0.874005    ,    0.869663    ,    0.864504    ,    0.852197    ,    0.8434    ,    0.84009    ,    0.843984    ,    0.844219    ,    0.842251    ,    1.03456    ,    1.19084    ,    1.26048    ,    1.27421    ,    1.28055    ,    1.28094    ,    1.1678    ,    1.09217    ,    1.01073    ,    0.854318    ,    0.832998    ,    0.838837    ,    0.838862    ,    0.841596    ,    0.847753    ,    0.8531    ,    0.85565    ,    0.860744    ,    0.862915    ,    0.863966    ,    0.86405    ,    0.860176    ,    0.853071    ,    0.846593    ,    0.881997    ,    0.874835    ,    0.866367    ,    0.854099    ,    0.854178    ,    0.88794    ,    0.953994    ,    1.06899    ,    1.13823    ,    1.15948    ,    1.17872    ,    1.17922    ,    1.16192    ,    1.18013    ,    1.13503    ,    1.08568    ,    1.06231    ,    1.03918    ,    1.03384    ,    0.986898    ,    0.930382    ,    0.900849    ,    0.90634    ,    0.90305    ,    0.89759    ,    0.88776    ,    0.878356    ,    0.869104    ,    0.859133    ,    0.850256    ,    0.842584    ,    0.837569    ,    0.832406    ,    0.829811    ,    0.827574    ,    0.822128    ,    0.820694    ,    0.848307    ,    0.899061    ,    0.938558    ,    0.834018    ,    0.798017    ,    0.810926    ,    0.834047    ,    0.938366    ,    0.879292    ,    0.821081    ,    0.822134    ,    0.819706    ,    0.817113    ,    0.817933    ,    0.818282    ,    0.818179    ,    0.81823    ,    0.816657    ,    0.815919    ,    0.818434    ,    0.819617    ,    0.819793    ,    0.821133    ,    0.823866    ,    0.826462    ,    0.830024    ,    0.833804    ,    0.833448    ,    0.835888    ,    0.837346    },
    {    0.807683    ,    0.808066    ,    0.808141    ,    0.808308    ,    0.808096    ,    0.804935    ,    0.805792    ,    0.806056    ,    0.806088    ,    0.805948    ,    0.806005    ,    0.807037    ,    0.806532    ,    0.808539    ,    0.80841    ,    0.809461    ,    0.810299    ,    0.811676    ,    0.813161    ,    0.815308    ,    0.81794    ,    0.82035    ,    0.822086    ,    0.822976    ,    0.825381    ,    0.828826    ,    0.830717    ,    0.830744    ,    0.832112    ,    0.83624    ,    0.837353    ,    0.840686    ,    0.838818    ,    0.842022    ,    0.859538    ,    0.884036    ,    0.894017    ,    1.0665    ,    1.18102    ,    1.29205    ,    1.28664    ,    1.23585    ,    1.14557    ,    0.995525    ,    0.821418    ,    0.811594    ,    0.813476    ,    0.820368    ,    0.82358    ,    0.826187    ,    0.832613    ,    0.837381    ,    0.843306    ,    0.848252    ,    0.855676    ,    0.858328    ,    0.857215    ,    0.859417    ,    0.844196    ,    0.836298    ,    0.971121    ,    1.06573    ,    1.07885    ,    1.07816    ,    1.06265    ,    1.04547    ,    1.08064    ,    1.11429    ,    1.12469    ,    1.15956    ,    1.1974    ,    1.20386    ,    1.20208    ,    1.18322    ,    1.19081    ,    1.19458    ,    1.15045    ,    1.09879    ,    1.09576    ,    1.06159    ,    1.04291    ,    0.990597    ,    0.918623    ,    0.915982    ,    0.91896    ,    0.914214    ,    0.902994    ,    0.893514    ,    0.880778    ,    0.869802    ,    0.857235    ,    0.84898    ,    0.850543    ,    0.859733    ,    0.821304    ,    0.818461    ,    0.815625    ,    0.811059    ,    0.811926    ,    0.852157    ,    0.864383    ,    0.804221    ,    0.795235    ,    0.800338    ,    0.807971    ,    0.824129    ,    0.826509    ,    0.824195    ,    0.881304    ,    0.86829    ,    0.826363    ,    0.821757    ,    0.816521    ,    0.813347    ,    0.811542    ,    0.811961    ,    0.811346    ,    0.810259    ,    0.809561    ,    0.809493    ,    0.810625    ,    0.810235    ,    0.810251    ,    0.809352    ,    0.810643    ,    0.809451    ,    0.809881    ,    0.810312    },
    {    0.814444    ,    0.814366    ,    0.814055    ,    0.813846    ,    0.814647    ,    0.812651    ,    0.811086    ,    0.810273    ,    0.810194    ,    0.8104    ,    0.810171    ,    0.810262    ,    0.809575    ,    0.808477    ,    0.808831    ,    0.808968    ,    0.808497    ,    0.808488    ,    0.808636    ,    0.809306    ,    0.808374    ,    0.809811    ,    0.812875    ,    0.817845    ,    0.823219    ,    0.828767    ,    0.833263    ,    0.836803    ,    0.841617    ,    0.848622    ,    0.854014    ,    0.855964    ,    0.850626    ,    0.87409    ,    0.959197    ,    0.84619    ,    0.854257    ,    0.987559    ,    1.1606    ,    1.16209    ,    1.04991    ,    0.995151    ,    0.905513    ,    0.800997    ,    0.790075    ,    0.793686    ,    0.801336    ,    0.805506    ,    0.808744    ,    0.813058    ,    0.818869    ,    0.825347    ,    0.831667    ,    0.838737    ,    0.849007    ,    0.853753    ,    0.850993    ,    0.844001    ,    0.832785    ,    0.96782    ,    1.148    ,    1.21561    ,    1.19722    ,    1.20208    ,    1.18291    ,    1.17621    ,    1.17708    ,    1.15872    ,    1.17353    ,    1.19758    ,    1.22503    ,    1.19254    ,    1.20868    ,    1.2083    ,    1.17246    ,    1.14189    ,    1.14689    ,    1.13331    ,    1.17851    ,    1.15067    ,    1.0868    ,    1.07292    ,    1.00363    ,    0.934443    ,    0.940666    ,    0.938238    ,    0.927817    ,    0.91379    ,    0.893211    ,    0.878835    ,    0.858651    ,    0.885268    ,    0.893909    ,    0.826371    ,    0.814337    ,    0.812744    ,    0.813162    ,    0.816008    ,    0.819936    ,    0.892479    ,    0.827183    ,    0.852278    ,    0.843335    ,    0.793089    ,    0.79947    ,    0.803935    ,    0.81695    ,    0.82846    ,    0.887891    ,    0.845244    ,    0.824325    ,    0.823383    ,    0.819786    ,    0.818491    ,    0.816777    ,    0.816032    ,    0.81438    ,    0.815065    ,    0.817096    ,    0.817349    ,    0.817461    ,    0.817878    ,    0.817727    ,    0.817991    ,    0.817499    ,    0.816979    ,    0.816756    ,    0.81572    },
    {    0.829797    ,    0.830271    ,    0.831504    ,    0.832617    ,    0.832355    ,    0.832246    ,    0.831505    ,    0.831382    ,    0.83158    ,    0.832787    ,    0.833385    ,    0.833005    ,    0.833369    ,    0.832838    ,    0.831542    ,    0.829086    ,    0.827202    ,    0.824902    ,    0.821876    ,    0.81804    ,    0.812442    ,    0.811755    ,    0.812911    ,    0.815158    ,    0.821134    ,    0.826922    ,    0.835782    ,    0.847608    ,    0.85489    ,    0.863657    ,    0.857641    ,    0.869292    ,    0.905656    ,    1.00959    ,    0.977489    ,    0.787132    ,    0.78397    ,    0.783363    ,    0.795964    ,    0.79921    ,    0.785887    ,    0.777945    ,    0.774888    ,    0.77361    ,    0.773865    ,    0.773733    ,    0.776288    ,    0.778959    ,    0.785827    ,    0.793057    ,    0.799612    ,    0.807312    ,    0.812833    ,    0.818519    ,    0.827462    ,    0.836398    ,    0.843339    ,    0.83454    ,    0.902211    ,    1.18612    ,    1.31075    ,    1.36202    ,    1.33467    ,    1.29273    ,    1.26717    ,    1.24865    ,    1.23801    ,    1.23444    ,    1.23053    ,    1.21326    ,    1.23989    ,    1.25186    ,    1.26617    ,    1.26212    ,    1.23698    ,    1.21827    ,    1.19828    ,    1.19678    ,    1.19854    ,    1.14131    ,    1.03137    ,    0.986403    ,    0.956171    ,    0.932158    ,    0.945145    ,    0.948417    ,    0.942678    ,    0.926923    ,    0.902215    ,    0.879868    ,    0.861301    ,    0.971512    ,    0.965597    ,    0.820602    ,    0.809408    ,    0.810986    ,    0.812066    ,    0.817367    ,    0.821871    ,    0.918498    ,    0.899566    ,    0.970392    ,    0.944533    ,    0.820417    ,    0.791658    ,    0.798986    ,    0.809412    ,    0.841531    ,    0.859061    ,    0.822224    ,    0.821706    ,    0.823177    ,    0.822594    ,    0.822409    ,    0.821804    ,    0.820699    ,    0.820142    ,    0.820219    ,    0.821303    ,    0.822292    ,    0.824314    ,    0.825626    ,    0.826038    ,    0.827505    ,    0.827037    ,    0.828008    ,    0.828479    ,    0.828784    },
    {    0.832402    ,    0.834407    ,    0.83686    ,    0.83982    ,    0.84315    ,    0.845167    ,    0.847902    ,    0.850185    ,    0.852371    ,    0.85481    ,    0.857329    ,    0.8586    ,    0.859996    ,    0.859632    ,    0.857545    ,    0.85346    ,    0.848631    ,    0.841814    ,    0.833379    ,    0.822982    ,    0.813153    ,    0.806435    ,    0.804249    ,    0.807582    ,    0.815804    ,    0.822274    ,    0.833915    ,    0.849329    ,    0.865273    ,    0.948812    ,    1.00152    ,    1.06962    ,    1.16044    ,    1.02485    ,    0.895811    ,    0.795697    ,    0.790594    ,    0.782548    ,    0.778702    ,    0.774349    ,    0.770516    ,    0.767667    ,    0.765511    ,    0.760663    ,    0.75619    ,    0.750412    ,    0.746157    ,    0.747237    ,    0.749546    ,    0.752489    ,    0.759265    ,    0.764776    ,    0.772173    ,    0.780367    ,    0.794471    ,    0.811028    ,    0.829035    ,    0.835044    ,    0.940589    ,    1.20253    ,    1.35172    ,    1.42295    ,    1.41153    ,    1.36527    ,    1.36849    ,    1.31253    ,    1.29435    ,    1.29565    ,    1.31616    ,    1.33816    ,    1.34012    ,    1.32608    ,    1.28396    ,    1.34435    ,    1.36963    ,    1.34339    ,    1.30811    ,    1.29975    ,    1.19743    ,    1.09889    ,    1.25705    ,    1.21241    ,    1.06823    ,    0.957245    ,    0.939431    ,    0.948107    ,    0.943288    ,    0.931911    ,    0.90909    ,    0.885553    ,    0.901459    ,    1.0241    ,    0.993337    ,    0.83991    ,    0.803565    ,    0.807511    ,    0.819101    ,    0.823971    ,    0.850961    ,    0.943786    ,    1.00814    ,    1.01266    ,    0.953209    ,    0.81567    ,    0.793567    ,    0.793541    ,    0.803667    ,    0.856486    ,    0.816234    ,    0.811522    ,    0.816748    ,    0.818842    ,    0.824098    ,    0.824738    ,    0.825987    ,    0.82556    ,    0.824317    ,    0.822312    ,    0.821543    ,    0.822325    ,    0.823245    ,    0.82608    ,    0.827003    ,    0.82855    ,    0.829281    ,    0.829949    ,    0.83025    ,    0.831597    },
    {    0.822527    ,    0.82359    ,    0.825448    ,    0.828512    ,    0.832551    ,    0.837753    ,    0.844564    ,    0.850057    ,    0.855831    ,    0.858453    ,    0.862159    ,    0.866203    ,    0.869376    ,    0.871876    ,    0.872335    ,    0.870615    ,    0.867384    ,    0.860913    ,    0.851389    ,    0.84043    ,    0.828346    ,    0.814728    ,    0.804134    ,    0.80151    ,    0.808553    ,    0.813344    ,    0.829636    ,    0.949217    ,    1.10206    ,    1.16492    ,    1.10695    ,    1.07486    ,    1.08776    ,    0.84322    ,    0.805089    ,    0.81293    ,    0.818573    ,    0.820329    ,    0.849556    ,    0.859619    ,    0.79529    ,    0.775979    ,    0.767508    ,    0.760201    ,    0.752983    ,    0.744454    ,    0.738214    ,    0.731029    ,    0.726084    ,    0.724822    ,    0.723257    ,    0.720367    ,    0.723734    ,    0.729061    ,    0.74359    ,    0.763948    ,    0.787529    ,    0.809164    ,    0.921478    ,    1.24655    ,    1.37624    ,    1.42528    ,    1.45811    ,    1.43546    ,    1.41675    ,    1.38811    ,    1.36698    ,    1.35666    ,    1.3875    ,    1.39192    ,    1.35916    ,    1.33274    ,    1.33002    ,    1.36685    ,    1.37715    ,    1.38091    ,    1.37539    ,    1.28871    ,    1.05086    ,    1.20581    ,    1.41093    ,    1.42261    ,    1.32638    ,    1.17124    ,    0.982886    ,    0.918148    ,    0.929162    ,    0.930974    ,    0.917267    ,    0.910691    ,    0.992495    ,    1.09847    ,    1.09861    ,    1.04954    ,    0.877681    ,    0.817592    ,    0.824264    ,    0.864358    ,    1.00382    ,    1.02232    ,    1.02798    ,    1.03162    ,    0.910758    ,    0.835913    ,    0.798316    ,    0.793695    ,    0.793095    ,    0.826637    ,    0.803102    ,    0.805714    ,    0.811038    ,    0.815105    ,    0.821756    ,    0.82354    ,    0.826638    ,    0.828702    ,    0.827801    ,    0.825382    ,    0.822187    ,    0.821084    ,    0.818761    ,    0.818471    ,    0.819095    ,    0.819769    ,    0.820438    ,    0.820735    ,    0.821054    ,    0.822184    },
    {    0.813113    ,    0.81112    ,    0.81012    ,    0.811178    ,    0.81304    ,    0.816897    ,    0.822303    ,    0.830279    ,    0.838327    ,    0.842234    ,    0.845903    ,    0.853832    ,    0.861998    ,    0.866868    ,    0.87051    ,    0.871901    ,    0.874537    ,    0.875638    ,    0.872228    ,    0.865329    ,    0.854802    ,    0.834219    ,    0.807532    ,    0.788396    ,    0.786205    ,    0.799998    ,    0.878404    ,    1.20115    ,    1.23765    ,    1.13099    ,    0.887161    ,    0.828954    ,    0.905585    ,    0.864383    ,    0.82537    ,    0.848922    ,    0.883985    ,    0.858575    ,    0.826469    ,    0.821264    ,    0.804267    ,    0.793439    ,    0.788112    ,    0.784029    ,    0.779923    ,    0.773674    ,    0.764885    ,    0.755665    ,    0.748639    ,    0.738589    ,    0.72791    ,    0.720026    ,    0.712025    ,    0.706937    ,    0.708806    ,    0.715509    ,    0.732033    ,    0.76733    ,    0.937668    ,    1.28359    ,    1.40881    ,    1.45306    ,    1.48018    ,    1.49096    ,    1.47775    ,    1.44263    ,    1.40956    ,    1.3925    ,    1.38428    ,    1.37856    ,    1.33765    ,    1.32782    ,    1.3484    ,    1.36295    ,    1.37239    ,    1.39202    ,    1.39064    ,    1.24238    ,    1.18566    ,    1.41909    ,    1.4794    ,    1.49775    ,    1.46924    ,    1.37822    ,    1.24111    ,    0.993508    ,    0.905464    ,    0.916963    ,    0.92941    ,    1.01034    ,    1.11555    ,    1.17638    ,    1.1912    ,    1.20522    ,    1.13622    ,    0.941236    ,    0.875607    ,    0.990905    ,    1.0618    ,    1.07207    ,    1.10628    ,    1.09656    ,    1.02757    ,    0.979033    ,    0.876531    ,    0.826207    ,    0.803172    ,    0.793104    ,    0.790073    ,    0.802672    ,    0.811416    ,    0.818102    ,    0.824195    ,    0.830339    ,    0.834838    ,    0.838042    ,    0.840267    ,    0.84051    ,    0.838182    ,    0.834269    ,    0.829547    ,    0.825447    ,    0.822987    ,    0.820237    ,    0.818187    ,    0.816648    ,    0.814481    ,    0.814528    },
    {    0.831121    ,    0.825135    ,    0.818244    ,    0.812026    ,    0.806817    ,    0.804496    ,    0.803348    ,    0.806238    ,    0.809786    ,    0.815263    ,    0.823313    ,    0.832602    ,    0.844293    ,    0.855563    ,    0.866728    ,    0.872426    ,    0.876514    ,    0.882006    ,    0.88541    ,    0.885669    ,    0.87997    ,    0.854623    ,    0.811062    ,    0.783482    ,    0.799293    ,    0.900816    ,    1.07787    ,    1.24108    ,    1.2443    ,    1.10436    ,    0.864053    ,    0.823313    ,    0.812276    ,    0.815063    ,    0.829256    ,    0.837507    ,    0.810217    ,    0.792977    ,    0.800655    ,    0.809959    ,    0.815621    ,    0.818822    ,    0.820108    ,    0.818322    ,    0.814058    ,    0.812054    ,    0.809871    ,    0.803618    ,    0.795561    ,    0.786458    ,    0.778805    ,    0.768941    ,    0.753434    ,    0.737266    ,    0.721894    ,    0.708841    ,    0.702699    ,    0.710346    ,    0.796811    ,    1.15877    ,    1.36761    ,    1.4344    ,    1.47363    ,    1.48982    ,    1.49596    ,    1.47819    ,    1.42783    ,    1.37401    ,    1.34506    ,    1.33188    ,    1.30879    ,    1.29432    ,    1.31308    ,    1.31402    ,    1.3446    ,    1.38472    ,    1.37514    ,    1.18606    ,    1.39653    ,    1.48413    ,    1.49971    ,    1.50772    ,    1.45846    ,    1.35443    ,    1.33278    ,    1.05854    ,    0.950565    ,    0.948236    ,    1.01513    ,    1.17795    ,    1.23751    ,    1.25974    ,    1.25318    ,    1.26956    ,    1.23031    ,    1.15092    ,    1.09812    ,    1.1218    ,    1.12708    ,    1.14442    ,    1.12556    ,    1.1153    ,    1.15043    ,    1.17242    ,    1.14208    ,    1.066    ,    0.921975    ,    0.878989    ,    0.804533    ,    0.799095    ,    0.813135    ,    0.826665    ,    0.83233    ,    0.839041    ,    0.845665    ,    0.851805    ,    0.857462    ,    0.860429    ,    0.862001    ,    0.86048    ,    0.858433    ,    0.856102    ,    0.853454    ,    0.850392    ,    0.846498    ,    0.842644    ,    0.838179    ,    0.835347    },
    {    0.868364    ,    0.861498    ,    0.851773    ,    0.840582    ,    0.829651    ,    0.81903    ,    0.810619    ,    0.805244    ,    0.800548    ,    0.800292    ,    0.804776    ,    0.812386    ,    0.825347    ,    0.840457    ,    0.855434    ,    0.865619    ,    0.873271    ,    0.880192    ,    0.885796    ,    0.887913    ,    0.885765    ,    0.865344    ,    0.819903    ,    0.819924    ,    1.00499    ,    1.13784    ,    1.27754    ,    1.29302    ,    1.31083    ,    1.14967    ,    0.907064    ,    0.860941    ,    0.84838    ,    0.854614    ,    0.879148    ,    0.926208    ,    0.814721    ,    0.801019    ,    0.821934    ,    0.832282    ,    0.836386    ,    0.841187    ,    0.841779    ,    0.838354    ,    0.835554    ,    0.833111    ,    0.831781    ,    0.830129    ,    0.82732    ,    0.825647    ,    0.820561    ,    0.811316    ,    0.800694    ,    0.78351    ,    0.762522    ,    0.741368    ,    0.718272    ,    0.699071    ,    0.702655    ,    0.869132    ,    1.1965    ,    1.39343    ,    1.44627    ,    1.45922    ,    1.48847    ,    1.47802    ,    1.43955    ,    1.37828    ,    1.32923    ,    1.3012    ,    1.25962    ,    1.24918    ,    1.27519    ,    1.28536    ,    1.30881    ,    1.35362    ,    1.33501    ,    1.36728    ,    1.48578    ,    1.49275    ,    1.49921    ,    1.48417    ,    1.30168    ,    1.27002    ,    1.31772    ,    1.33494    ,    1.32463    ,    1.28796    ,    1.3222    ,    1.3305    ,    1.32824    ,    1.32896    ,    1.32382    ,    1.26426    ,    1.21317    ,    1.18371    ,    1.15703    ,    1.1508    ,    1.16059    ,    1.15355    ,    1.15543    ,    1.16635    ,    1.22526    ,    1.2674    ,    1.25968    ,    1.22001    ,    1.13877    ,    0.984804    ,    0.881818    ,    0.83768    ,    0.8234    ,    0.836345    ,    0.844625    ,    0.850991    ,    0.859904    ,    0.86598    ,    0.87267    ,    0.876587    ,    0.880384    ,    0.883296    ,    0.884564    ,    0.885721    ,    0.88538    ,    0.88427    ,    0.882696    ,    0.880087    ,    0.8762    ,    0.872337    },
    {    0.901087    ,    0.895904    ,    0.887561    ,    0.879082    ,    0.869091    ,    0.855081    ,    0.841149    ,    0.830334    ,    0.820359    ,    0.813313    ,    0.812592    ,    0.815731    ,    0.823491    ,    0.834078    ,    0.844548    ,    0.854223    ,    0.86417    ,    0.871608    ,    0.881038    ,    0.886098    ,    0.879256    ,    0.859297    ,    0.836031    ,    0.953436    ,    1.20471    ,    1.29167    ,    1.34068    ,    1.3475    ,    1.30309    ,    1.25872    ,    1.08766    ,    0.984137    ,    0.963709    ,    0.949398    ,    0.968861    ,    0.98989    ,    0.834238    ,    0.811819    ,    0.831851    ,    0.839612    ,    0.840426    ,    0.840793    ,    0.843206    ,    0.844199    ,    0.846064    ,    0.842335    ,    0.836862    ,    0.837516    ,    0.838831    ,    0.835385    ,    0.828844    ,    0.824865    ,    0.820497    ,    0.808572    ,    0.794987    ,    0.775864    ,    0.753773    ,    0.732313    ,    0.708669    ,    0.704017    ,    0.842358    ,    1.33856    ,    1.42357    ,    1.41001    ,    1.4184    ,    1.40443    ,    1.37023    ,    1.35287    ,    1.31676    ,    1.25382    ,    1.18431    ,    1.17273    ,    1.21605    ,    1.21932    ,    1.22061    ,    1.26238    ,    1.33227    ,    1.42397    ,    1.46036    ,    1.47346    ,    1.48709    ,    1.4037    ,    1.38448    ,    1.47621    ,    1.4819    ,    1.47224    ,    1.47474    ,    1.48667    ,    1.47034    ,    1.43891    ,    1.43214    ,    1.39193    ,    1.32494    ,    1.36752    ,    1.56074    ,    1.59859    ,    1.53402    ,    1.45602    ,    1.39835    ,    1.38658    ,    1.26323    ,    1.27071    ,    1.32078    ,    1.32349    ,    1.3345    ,    1.29996    ,    1.25996    ,    1.15639    ,    0.988384    ,    0.926873    ,    0.879514    ,    0.864181    ,    0.857729    ,    0.859383    ,    0.867394    ,    0.872609    ,    0.879638    ,    0.885909    ,    0.889179    ,    0.891856    ,    0.894563    ,    0.899366    ,    0.903536    ,    0.904117    ,    0.904251    ,    0.904578    ,    0.902915    ,    0.90146    },
    {    0.928224    ,    0.924114    ,    0.918091    ,    0.91264    ,    0.906413    ,    0.897064    ,    0.884026    ,    0.870318    ,    0.857103    ,    0.84563    ,    0.836756    ,    0.833472    ,    0.834459    ,    0.838014    ,    0.84262    ,    0.849041    ,    0.856383    ,    0.862934    ,    0.872162    ,    0.878735    ,    0.865575    ,    0.861294    ,    0.911505    ,    1.16965    ,    1.29071    ,    1.34312    ,    1.36508    ,    1.37689    ,    1.35727    ,    1.30421    ,    1.2652    ,    1.2236    ,    1.20983    ,    1.14708    ,    1.13302    ,    1.07825    ,    0.907885    ,    0.847557    ,    0.831378    ,    0.829205    ,    0.830621    ,    0.829879    ,    0.830485    ,    0.829642    ,    0.831267    ,    0.834128    ,    0.833437    ,    0.834502    ,    0.835565    ,    0.834997    ,    0.832864    ,    0.827912    ,    0.819107    ,    0.809572    ,    0.800465    ,    0.790147    ,    0.780518    ,    0.760136    ,    0.73282    ,    0.705779    ,    0.742594    ,    1.16276    ,    1.4434    ,    1.42996    ,    1.39777    ,    1.357    ,    1.31944    ,    1.2788    ,    1.2094    ,    1.10217    ,    0.969725    ,    0.979585    ,    1.07675    ,    0.998284    ,    0.969656    ,    0.990289    ,    1.06809    ,    1.34386    ,    1.4201    ,    1.45787    ,    1.48437    ,    1.47853    ,    1.487    ,    1.47324    ,    1.44918    ,    1.45668    ,    1.47216    ,    1.50947    ,    1.51418    ,    1.46308    ,    1.45913    ,    1.40437    ,    1.561    ,    1.65747    ,    1.56492    ,    1.52601    ,    1.50851    ,    1.50469    ,    1.51521    ,    1.46525    ,    1.41589    ,    1.32932    ,    1.31588    ,    1.32423    ,    1.33176    ,    1.32607    ,    1.28982    ,    1.19228    ,    1.06382    ,    1.02103    ,    1.00303    ,    0.995136    ,    0.939754    ,    0.913391    ,    0.900416    ,    0.897309    ,    0.898586    ,    0.905329    ,    0.911464    ,    0.914921    ,    0.916862    ,    0.919608    ,    0.923459    ,    0.927979    ,    0.933379    ,    0.933866    ,    0.931042    ,    0.929295    },
    {    0.96814    ,    0.961662    ,    0.952914    ,    0.944477    ,    0.937418    ,    0.928783    ,    0.918762    ,    0.907856    ,    0.896045    ,    0.882713    ,    0.870303    ,    0.861948    ,    0.858545    ,    0.854661    ,    0.852098    ,    0.853203    ,    0.856031    ,    0.858004    ,    0.863484    ,    0.870574    ,    0.870514    ,    0.960521    ,    1.17818    ,    1.28906    ,    1.34293    ,    1.37951    ,    1.39738    ,    1.38092    ,    1.39664    ,    1.35535    ,    1.33174    ,    1.31372    ,    1.26967    ,    1.22645    ,    1.19523    ,    1.18623    ,    1.14387    ,    0.974162    ,    0.872563    ,    0.844898    ,    0.838579    ,    0.833104    ,    0.829631    ,    0.825668    ,    0.821869    ,    0.820892    ,    0.820863    ,    0.8228    ,    0.826106    ,    0.825758    ,    0.824342    ,    0.821773    ,    0.818974    ,    0.813883    ,    0.799103    ,    0.785572    ,    0.781658    ,    0.768515    ,    0.746223    ,    0.715591    ,    0.709245    ,    0.846718    ,    1.20115    ,    1.28234    ,    1.35089    ,    1.37138    ,    1.33858    ,    1.26069    ,    1.02845    ,    0.962142    ,    0.956705    ,    0.958913    ,    0.97796    ,    0.98468    ,    1.00334    ,    1.02735    ,    1.04673    ,    1.2584    ,    1.3898    ,    1.4334    ,    1.46183    ,    1.48175    ,    1.42313    ,    1.41447    ,    1.42885    ,    1.40604    ,    1.41409    ,    1.49235    ,    1.56009    ,    1.57433    ,    1.54499    ,    1.55239    ,    1.5568    ,    1.53718    ,    1.50263    ,    1.50885    ,    1.5212    ,    1.48624    ,    1.44752    ,    1.44288    ,    1.40871    ,    1.36051    ,    1.30122    ,    1.30401    ,    1.31407    ,    1.31976    ,    1.29113    ,    1.19826    ,    1.13842    ,    1.15151    ,    1.10268    ,    1.11412    ,    1.11826    ,    1.07738    ,    1.00322    ,    0.972933    ,    0.968318    ,    0.973172    ,    0.976088    ,    0.978987    ,    0.980246    ,    0.979921    ,    0.979921    ,    0.980603    ,    0.983284    ,    0.983213    ,    0.978936    ,    0.973886    },
    {    1.01336    ,    1.0033    ,    0.992668    ,    0.982827    ,    0.973784    ,    0.963946    ,    0.954438    ,    0.944424    ,    0.933392    ,    0.92123    ,    0.909901    ,    0.900579    ,    0.892927    ,    0.886577    ,    0.88097    ,    0.874864    ,    0.868565    ,    0.865955    ,    0.865409    ,    0.877663    ,    0.940316    ,    1.13895    ,    1.33044    ,    1.3639    ,    1.39332    ,    1.4266    ,    1.39656    ,    1.36091    ,    1.389    ,    1.37859    ,    1.35079    ,    1.35569    ,    1.34464    ,    1.31285    ,    1.28626    ,    1.2616    ,    1.26533    ,    1.15665    ,    0.975631    ,    0.921132    ,    0.907611    ,    0.891129    ,    0.879819    ,    0.869623    ,    0.859064    ,    0.852996    ,    0.847302    ,    0.841756    ,    0.836787    ,    0.832738    ,    0.828829    ,    0.818589    ,    0.807104    ,    0.803365    ,    0.797325    ,    0.782696    ,    0.764075    ,    0.757697    ,    0.745415    ,    0.706767    ,    0.712796    ,    0.990236    ,    1.2768    ,    1.22932    ,    0.987069    ,    1.00936    ,    1.04018    ,    1.03864    ,    0.984531    ,    1.02257    ,    1.0166    ,    1.03678    ,    1.17702    ,    1.08125    ,    1.26305    ,    1.34543    ,    1.3496    ,    1.38616    ,    1.43549    ,    1.48558    ,    1.48849    ,    1.3808    ,    1.18547    ,    1.23178    ,    1.36685    ,    1.36601    ,    1.34249    ,    1.36418    ,    1.43959    ,    1.67833    ,    1.67167    ,    1.5214    ,    1.47389    ,    1.45348    ,    1.43194    ,    1.42027    ,    1.43539    ,    1.40946    ,    1.41287    ,    1.40908    ,    1.39386    ,    1.38631    ,    1.34715    ,    1.33709    ,    1.33468    ,    1.31925    ,    1.3169    ,    1.25616    ,    1.22966    ,    1.2633    ,    1.19791    ,    1.14483    ,    1.12391    ,    1.14532    ,    1.146    ,    1.0924    ,    1.10053    ,    1.10325    ,    1.09876    ,    1.09026    ,    1.08237    ,    1.07452    ,    1.06651    ,    1.05786    ,    1.05023    ,    1.04312    ,    1.03394    ,    1.02409    },
    {    1.0573    ,    1.04589    ,    1.03462    ,    1.02347    ,    1.01467    ,    1.00561    ,    0.996029    ,    0.986498    ,    0.976376    ,    0.966535    ,    0.956518    ,    0.948284    ,    0.939893    ,    0.931453    ,    0.921269    ,    0.909923    ,    0.897843    ,    0.889581    ,    0.882482    ,    0.898513    ,    1.067    ,    1.24952    ,    1.34604    ,    1.40492    ,    1.43688    ,    1.50708    ,    1.41622    ,    1.34041    ,    1.36972    ,    1.40059    ,    1.40019    ,    1.40191    ,    1.38303    ,    1.37476    ,    1.37481    ,    1.36395    ,    1.3612    ,    1.34692    ,    1.20789    ,    1.09712    ,    1.04497    ,    1.00764    ,    0.981071    ,    0.954045    ,    0.938532    ,    0.94089    ,    0.950875    ,    0.958019    ,    0.950408    ,    0.920521    ,    0.894211    ,    0.859977    ,    0.81814    ,    0.790211    ,    0.784605    ,    0.776332    ,    0.748027    ,    0.734043    ,    0.725724    ,    0.69301    ,    0.699385    ,    1.05731    ,    1.31356    ,    1.34401    ,    1.1799    ,    0.981554    ,    0.957347    ,    1.02723    ,    1.00917    ,    1.09775    ,    1.1868    ,    1.22939    ,    1.32132    ,    1.19566    ,    1.19925    ,    1.22292    ,    1.26523    ,    1.22969    ,    1.25682    ,    1.3837    ,    1.36957    ,    1.29149    ,    1.1307    ,    1.2352    ,    1.35488    ,    1.35447    ,    1.3512    ,    1.35129    ,    1.37255    ,    1.49013    ,    1.52911    ,    1.50574    ,    1.46615    ,    1.45195    ,    1.45901    ,    1.47106    ,    1.48506    ,    1.47997    ,    1.46888    ,    1.47002    ,    1.46576    ,    1.46736    ,    1.42198    ,    1.39078    ,    1.38808    ,    1.3683    ,    1.3776    ,    1.36886    ,    1.40191    ,    1.37672    ,    1.29971    ,    1.23938    ,    1.20804    ,    1.17882    ,    1.20846    ,    1.21648    ,    1.2295    ,    1.20985    ,    1.18865    ,    1.17694    ,    1.16575    ,    1.15721    ,    1.14699    ,    1.13181    ,    1.11196    ,    1.09509    ,    1.08138    ,    1.06935    },
    {    1.08691    ,    1.07794    ,    1.0694    ,    1.0601    ,    1.05034    ,    1.04127    ,    1.03309    ,    1.02609    ,    1.01821    ,    1.00975    ,    1.00115    ,    0.993203    ,    0.984948    ,    0.976254    ,    0.965718    ,    0.951663    ,    0.937734    ,    0.92448    ,    0.913736    ,    0.924708    ,    1.07327    ,    1.26077    ,    1.32118    ,    1.42694    ,    1.50316    ,    1.44816    ,    1.41505    ,    1.36675    ,    1.39439    ,    1.44388    ,    1.47519    ,    1.48612    ,    1.4797    ,    1.44834    ,    1.46334    ,    1.46509    ,    1.46103    ,    1.46247    ,    1.45121    ,    1.35049    ,    1.2362    ,    1.18658    ,    1.13784    ,    1.08362    ,    1.04725    ,    1.04916    ,    1.08382    ,    1.13415    ,    1.12025    ,    1.02966    ,    0.939737    ,    0.860945    ,    0.796673    ,    0.746769    ,    0.729216    ,    0.731614    ,    0.721389    ,    0.702755    ,    0.68881    ,    0.675824    ,    0.671069    ,    0.839699    ,    0.98038    ,    1.05511    ,    1.20652    ,    1.18013    ,    1.11639    ,    1.0993    ,    1.19971    ,    1.2058    ,    1.2218    ,    1.30422    ,    1.34568    ,    1.33693    ,    1.18993    ,    1.0606    ,    1.0596    ,    1.05085    ,    1.12245    ,    1.27562    ,    1.30588    ,    1.19394    ,    1.16823    ,    1.31576    ,    1.37334    ,    1.36578    ,    1.39285    ,    1.3823    ,    1.39005    ,    1.3727    ,    1.41013    ,    1.4425    ,    1.47992    ,    1.48972    ,    1.49584    ,    1.53391    ,    1.56606    ,    1.57346    ,    1.55855    ,    1.54447    ,    1.52951    ,    1.4928    ,    1.46636    ,    1.45318    ,    1.45662    ,    1.4289    ,    1.41938    ,    1.45908    ,    1.48235    ,    1.48275    ,    1.44989    ,    1.40677    ,    1.31139    ,    1.24709    ,    1.30005    ,    1.34525    ,    1.30549    ,    1.24372    ,    1.22013    ,    1.21037    ,    1.19895    ,    1.18827    ,    1.18134    ,    1.16797    ,    1.14618    ,    1.1226    ,    1.10682    ,    1.09633    },
    {    1.09233    ,    1.08423    ,    1.07725    ,    1.07239    ,    1.06791    ,    1.06248    ,    1.05716    ,    1.05402    ,    1.05019    ,    1.04347    ,    1.0372    ,    1.03165    ,    1.02499    ,    1.01708    ,    1.00588    ,    0.989531    ,    0.970394    ,    0.953132    ,    0.94322    ,    0.953713    ,    1.06813    ,    1.2212    ,    1.29335    ,    1.37697    ,    1.38815    ,    1.38283    ,    1.38671    ,    1.41981    ,    1.4623    ,    1.52668    ,    1.55985    ,    1.55607    ,    1.55108    ,    1.5543    ,    1.56414    ,    1.59215    ,    1.57889    ,    1.57897    ,    1.5777    ,    1.53945    ,    1.49824    ,    1.44512    ,    1.36858    ,    1.27363    ,    1.19255    ,    1.14449    ,    1.11632    ,    1.12182    ,    1.08934    ,    1.03396    ,    0.92312    ,    0.822947    ,    0.7356    ,    0.677186    ,    0.65879    ,    0.657176    ,    0.654361    ,    0.651221    ,    0.635647    ,    0.634817    ,    0.649403    ,    0.67538    ,    0.715342    ,    0.842142    ,    1.08377    ,    1.16119    ,    1.20036    ,    1.21955    ,    1.24707    ,    1.25262    ,    1.27699    ,    1.2903    ,    1.30016    ,    1.33233    ,    1.31478    ,    1.22862    ,    1.25365    ,    1.22436    ,    1.28231    ,    1.30722    ,    1.31915    ,    1.24214    ,    1.20925    ,    1.30292    ,    1.41104    ,    1.41695    ,    1.44677    ,    1.47408    ,    1.46897    ,    1.45844    ,    1.45803    ,    1.4616    ,    1.46892    ,    1.50191    ,    1.5638    ,    1.59613    ,    1.56934    ,    1.56041    ,    1.55153    ,    1.50512    ,    1.45487    ,    1.45034    ,    1.45458    ,    1.45485    ,    1.45287    ,    1.45761    ,    1.4513    ,    1.46394    ,    1.4844    ,    1.50705    ,    1.49927    ,    1.51754    ,    1.48495    ,    1.39793    ,    1.43137    ,    1.45341    ,    1.41016    ,    1.35073    ,    1.29895    ,    1.26795    ,    1.23399    ,    1.20769    ,    1.19338    ,    1.17926    ,    1.16073    ,    1.13873    ,    1.11948    ,    1.10464    },
    {    1.10086    ,    1.09162    ,    1.08687    ,    1.08596    ,    1.08712    ,    1.08601    ,    1.08486    ,    1.08481    ,    1.08228    ,    1.07797    ,    1.07309    ,    1.06876    ,    1.06112    ,    1.05088    ,    1.0363    ,    1.01439    ,    0.991532    ,    0.977175    ,    0.976666    ,    1.03113    ,    1.18596    ,    1.30982    ,    1.35842    ,    1.358    ,    1.37433    ,    1.38984    ,    1.43705    ,    1.49662    ,    1.55494    ,    1.61231    ,    1.61515    ,    1.60552    ,    1.6144    ,    1.6416    ,    1.66316    ,    1.67777    ,    1.68407    ,    1.65995    ,    1.64205    ,    1.62081    ,    1.60161    ,    1.52318    ,    1.43605    ,    1.39496    ,    1.34462    ,    1.20237    ,    1.05732    ,    0.931115    ,    0.855234    ,    0.843386    ,    0.786777    ,    0.712155    ,    0.628983    ,    0.578049    ,    0.565532    ,    0.562705    ,    0.568745    ,    0.586117    ,    0.590039    ,    0.591407    ,    0.60322    ,    0.645116    ,    0.715223    ,    0.848084    ,    0.996111    ,    1.07105    ,    1.10927    ,    1.12705    ,    1.15543    ,    1.1875    ,    1.21883    ,    1.24885    ,    1.26699    ,    1.2892    ,    1.31782    ,    1.32886    ,    1.33739    ,    1.34338    ,    1.3582    ,    1.37781    ,    1.40321    ,    1.38536    ,    1.37597    ,    1.41956    ,    1.47631    ,    1.52134    ,    1.5371    ,    1.54111    ,    1.54438    ,    1.55221    ,    1.55823    ,    1.55842    ,    1.56073    ,    1.55896    ,    1.54937    ,    1.51313    ,    1.47493    ,    1.47884    ,    1.48828    ,    1.44519    ,    1.41226    ,    1.43143    ,    1.43797    ,    1.41279    ,    1.4168    ,    1.45893    ,    1.49099    ,    1.49416    ,    1.4865    ,    1.52612    ,    1.51081    ,    1.52375    ,    1.52476    ,    1.51328    ,    1.56865    ,    1.56864    ,    1.54301    ,    1.45616    ,    1.3883    ,    1.34107    ,    1.2967    ,    1.25542    ,    1.22571    ,    1.19851    ,    1.1744    ,    1.15302    ,    1.13214    ,    1.11509    },
    {    1.13486    ,    1.12278    ,    1.11653    ,    1.11901    ,    1.12392    ,    1.12635    ,    1.12845    ,    1.12759    ,    1.12274    ,    1.11486    ,    1.10592    ,    1.09853    ,    1.09003    ,    1.07909    ,    1.05936    ,    1.03234    ,    1.01341    ,    1.01026    ,    1.04661    ,    1.23358    ,    1.33056    ,    1.3657    ,    1.3763    ,    1.36923    ,    1.41806    ,    1.47786    ,    1.53684    ,    1.59046    ,    1.61861    ,    1.66997    ,    1.66853    ,    1.67105    ,    1.68679    ,    1.72011    ,    1.78583    ,    1.84315    ,    1.83715    ,    1.76784    ,    1.69936    ,    1.66009    ,    1.63913    ,    1.61503    ,    1.58513    ,    1.54423    ,    1.40161    ,    1.14303    ,    0.96801    ,    0.797054    ,    0.68195    ,    0.624192    ,    0.586766    ,    0.545358    ,    0.498734    ,    0.470252    ,    0.459262    ,    0.457793    ,    0.47771    ,    0.509294    ,    0.529364    ,    0.544781    ,    0.579758    ,    0.695246    ,    0.743874    ,    0.869679    ,    0.909854    ,    0.912522    ,    0.999528    ,    1.06131    ,    1.10683    ,    1.1349    ,    1.16918    ,    1.20453    ,    1.24459    ,    1.28481    ,    1.32495    ,    1.35076    ,    1.3772    ,    1.3957    ,    1.41263    ,    1.43855    ,    1.46146    ,    1.49238    ,    1.50999    ,    1.53165    ,    1.55975    ,    1.58135    ,    1.61531    ,    1.63863    ,    1.63615    ,    1.62317    ,    1.62599    ,    1.63345    ,    1.65271    ,    1.62993    ,    1.54552    ,    1.44181    ,    1.40981    ,    1.42623    ,    1.4356    ,    1.43021    ,    1.40778    ,    1.43797    ,    1.47916    ,    1.47025    ,    1.45684    ,    1.48109    ,    1.49535    ,    1.50455    ,    1.51514    ,    1.54501    ,    1.53343    ,    1.52918    ,    1.55539    ,    1.59878    ,    1.67387    ,    1.7337    ,    1.69299    ,    1.5747    ,    1.48874    ,    1.44369    ,    1.44841    ,    1.33927    ,    1.28879    ,    1.2447    ,    1.213    ,    1.19179    ,    1.17224    ,    1.15315    },
    {    1.19544    ,    1.18773    ,    1.18603    ,    1.19116    ,    1.20204    ,    1.21386    ,    1.22872    ,    1.22168    ,    1.19872    ,    1.17155    ,    1.14749    ,    1.12984    ,    1.118    ,    1.10311    ,    1.07865    ,    1.05816    ,    1.05307    ,    1.10656    ,    1.25715    ,    1.3522    ,    1.36229    ,    1.37231    ,    1.35683    ,    1.39467    ,    1.49528    ,    1.54142    ,    1.5894    ,    1.63285    ,    1.68242    ,    1.73337    ,    1.73917    ,    1.77471    ,    1.81217    ,    1.87148    ,    1.97676    ,    2.10349    ,    2.09342    ,    1.88612    ,    1.79818    ,    1.74818    ,    1.71859    ,    1.69382    ,    1.6577    ,    1.5708    ,    1.33863    ,    1.08069    ,    0.886103    ,    0.745068    ,    0.654375    ,    0.579708    ,    0.514492    ,    0.457386    ,    0.426853    ,    0.415509    ,    0.405323    ,    0.396406    ,    0.413723    ,    0.457793    ,    0.495899    ,    0.51542    ,    0.546328    ,    0.678443    ,    0.762548    ,    0.837378    ,    0.818716    ,    0.814466    ,    0.849565    ,    0.952532    ,    1.03162    ,    1.06098    ,    1.10137    ,    1.1592    ,    1.23514    ,    1.29207    ,    1.34541    ,    1.385    ,    1.43101    ,    1.46271    ,    1.49562    ,    1.51846    ,    1.5303    ,    1.54928    ,    1.57347    ,    1.59188    ,    1.57982    ,    1.58464    ,    1.6428    ,    1.67545    ,    1.69819    ,    1.71347    ,    1.73031    ,    1.75708    ,    1.7606    ,    1.74042    ,    1.68418    ,    1.60573    ,    1.5634    ,    1.52085    ,    1.48363    ,    1.46577    ,    1.48803    ,    1.52875    ,    1.54409    ,    1.55093    ,    1.5162    ,    1.51122    ,    1.52667    ,    1.54511    ,    1.5616    ,    1.58451    ,    1.57882    ,    1.58743    ,    1.62799    ,    1.82582    ,    1.95973    ,    1.91553    ,    1.79172    ,    1.6831    ,    1.6025    ,    1.57948    ,    1.65467    ,    1.57072    ,    1.42282    ,    1.33228    ,    1.28112    ,    1.25332    ,    1.23333    ,    1.21484    },
    {    1.32987    ,    1.33272    ,    1.34898    ,    1.37642    ,    1.40365    ,    1.42615    ,    1.43761    ,    1.4435    ,    1.43489    ,    1.34675    ,    1.25544    ,    1.18654    ,    1.14953    ,    1.12788    ,    1.11433    ,    1.12694    ,    1.20936    ,    1.40018    ,    1.46255    ,    1.44554    ,    1.43643    ,    1.44629    ,    1.49226    ,    1.52829    ,    1.5796    ,    1.62467    ,    1.67829    ,    1.72829    ,    1.78043    ,    1.83198    ,    1.87227    ,    1.95408    ,    2.04422    ,    2.12717    ,    2.18127    ,    2.21908    ,    2.21142    ,    2.05246    ,    1.94416    ,    1.88989    ,    1.84758    ,    1.82207    ,    1.7122    ,    1.42151    ,    1.11231    ,    0.935001    ,    0.833646    ,    0.765507    ,    0.72505    ,    0.674656    ,    0.604367    ,    0.529162    ,    0.482072    ,    0.473141    ,    0.459241    ,    0.440202    ,    0.433232    ,    0.465476    ,    0.505963    ,    0.526347    ,    0.530779    ,    0.571856    ,    0.705855    ,    0.771158    ,    0.756477    ,    0.794691    ,    0.860135    ,    0.965431    ,    1.04804    ,    1.12415    ,    1.13346    ,    1.16334    ,    1.24417    ,    1.31981    ,    1.3837    ,    1.42923    ,    1.46516    ,    1.51176    ,    1.5431    ,    1.56461    ,    1.56961    ,    1.59337    ,    1.61081    ,    1.61676    ,    1.62783    ,    1.62271    ,    1.66839    ,    1.70893    ,    1.73705    ,    1.75292    ,    1.76293    ,    1.76376    ,    1.77107    ,    1.77031    ,    1.7527    ,    1.71033    ,    1.67622    ,    1.65307    ,    1.63126    ,    1.62687    ,    1.63236    ,    1.61917    ,    1.60947    ,    1.60139    ,    1.59935    ,    1.59836    ,    1.60606    ,    1.63278    ,    1.64815    ,    1.65878    ,    1.66055    ,    1.67278    ,    1.68814    ,    1.76693    ,    1.89064    ,    1.8904    ,    1.84606    ,    1.80266    ,    1.77346    ,    1.74392    ,    1.76876    ,    1.79005    ,    1.65817    ,    1.49674    ,    1.4243    ,    1.39068    ,    1.36395    ,    1.34451    },
    {    1.65998    ,    1.65783    ,    1.67457    ,    1.69142    ,    1.69029    ,    1.68097    ,    1.71048    ,    1.74524    ,    1.72496    ,    1.6765    ,    1.55253    ,    1.41963    ,    1.35017    ,    1.33835    ,    1.27978    ,    1.44317    ,    1.53025    ,    1.51384    ,    1.50162    ,    1.49761    ,    1.52751    ,    1.59126    ,    1.65788    ,    1.70401    ,    1.74467    ,    1.76673    ,    1.80516    ,    1.84851    ,    1.90098    ,    1.94271    ,    1.99739    ,    2.03319    ,    2.08395    ,    2.13594    ,    2.17892    ,    2.22104    ,    2.22086    ,    2.12734    ,    2.07367    ,    2.00696    ,    1.91207    ,    1.83443    ,    1.62823    ,    1.30509    ,    1.04437    ,    0.940886    ,    0.89976    ,    0.921992    ,    0.96599    ,    0.963347    ,    0.834149    ,    0.721559    ,    0.637688    ,    0.593355    ,    0.560027    ,    0.53092    ,    0.516465    ,    0.52103    ,    0.541847    ,    0.556177    ,    0.559619    ,    0.575248    ,    0.619065    ,    0.668005    ,    0.720295    ,    0.789732    ,    0.962714    ,    1.13629    ,    1.20376    ,    1.24962    ,    1.2549    ,    1.26992    ,    1.32985    ,    1.41219    ,    1.47724    ,    1.51904    ,    1.53439    ,    1.55731    ,    1.5754    ,    1.58224    ,    1.59742    ,    1.62071    ,    1.63797    ,    1.65066    ,    1.66243    ,    1.67974    ,    1.71318    ,    1.75088    ,    1.78883    ,    1.79028    ,    1.79751    ,    1.80635    ,    1.81818    ,    1.82542    ,    1.82077    ,    1.8081    ,    1.78524    ,    1.766    ,    1.76611    ,    1.75214    ,    1.75408    ,    1.75234    ,    1.75047    ,    1.73939    ,    1.72047    ,    1.71463    ,    1.71903    ,    1.73974    ,    1.75143    ,    1.75181    ,    1.77059    ,    1.78555    ,    1.78752    ,    1.76835    ,    1.73615    ,    1.76375    ,    1.7868    ,    1.79458    ,    1.82695    ,    1.87783    ,    1.80452    ,    1.83261    ,    1.87309    ,    1.76462    ,    1.71852    ,    1.72136    ,    1.69538    ,    1.67646    },
    {    2.09889    ,    2.02559    ,    1.98782    ,    1.95612    ,    1.92205    ,    1.877    ,    1.89552    ,    1.88005    ,    1.83583    ,    1.78241    ,    1.752    ,    1.69394    ,    1.6488    ,    1.6158    ,    1.60041    ,    1.57798    ,    1.57273    ,    1.56558    ,    1.58717    ,    1.60694    ,    1.66171    ,    1.75301    ,    1.8151    ,    1.84465    ,    1.87363    ,    1.90262    ,    1.95308    ,    2.00137    ,    2.04486    ,    2.07938    ,    2.11487    ,    2.12615    ,    2.08899    ,    2.10605    ,    2.17236    ,    2.22907    ,    2.25175    ,    2.24526    ,    2.17219    ,    2.07324    ,    1.95805    ,    1.77242    ,    1.61557    ,    1.37014    ,    1.12348    ,    1.01367    ,    1.05579    ,    1.11027    ,    1.17334    ,    1.17884    ,    1.13739    ,    0.972414    ,    0.861783    ,    0.77905    ,    0.720838    ,    0.67508    ,    0.680886    ,    0.709935    ,    0.69713    ,    0.698356    ,    0.688115    ,    0.690943    ,    0.720883    ,    0.743228    ,    0.768992    ,    0.80935    ,    0.905309    ,    1.07818    ,    1.21344    ,    1.29349    ,    1.34929    ,    1.40552    ,    1.47864    ,    1.5281    ,    1.5651    ,    1.59569    ,    1.61973    ,    1.64236    ,    1.65482    ,    1.67471    ,    1.68669    ,    1.70593    ,    1.73312    ,    1.74969    ,    1.76141    ,    1.78571    ,    1.82975    ,    1.86843    ,    1.89381    ,    1.93647    ,    1.96167    ,    1.97617    ,    1.96606    ,    1.95663    ,    1.93866    ,    1.91013    ,    1.88682    ,    1.86919    ,    1.86487    ,    1.87213    ,    1.88049    ,    1.882    ,    1.88377    ,    1.88627    ,    1.89034    ,    1.89045    ,    1.89509    ,    1.89905    ,    1.89322    ,    1.87669    ,    1.8612    ,    1.83615    ,    1.79861    ,    1.76436    ,    1.7605    ,    1.76842    ,    1.78472    ,    1.82533    ,    1.8739    ,    1.90535    ,    1.89704    ,    1.92269    ,    1.9711    ,    2.00894    ,    2.04317    ,    2.09653    ,    2.14465    ,    2.15229    },
    {    2.33073    ,    2.33064    ,    2.37185    ,    2.31608    ,    2.21701    ,    2.1585    ,    2.09818    ,    2.01744    ,    1.93436    ,    1.8789    ,    1.83537    ,    1.8017    ,    1.78424    ,    1.7731    ,    1.76894    ,    1.7694    ,    1.76528    ,    1.7575    ,    1.76671    ,    1.80185    ,    1.86845    ,    1.9262    ,    1.95503    ,    1.97774    ,    2.01208    ,    2.06576    ,    2.10961    ,    2.1521    ,    2.19388    ,    2.22543    ,    2.24472    ,    2.24327    ,    2.24648    ,    2.26723    ,    2.30544    ,    2.28    ,    2.26593    ,    2.24727    ,    2.18517    ,    2.11653    ,    2.03845    ,    1.90524    ,    1.8105    ,    1.6543    ,    1.40779    ,    1.26971    ,    1.27738    ,    1.20722    ,    1.31841    ,    1.31873    ,    1.40416    ,    1.40352    ,    1.39211    ,    1.28152    ,    1.20977    ,    1.18059    ,    1.28756    ,    1.34604    ,    1.30322    ,    1.19622    ,    1.07132    ,    1.02115    ,    1.00944    ,    1.00337    ,    0.992889    ,    0.971533    ,    0.952913    ,    0.982289    ,    1.09026    ,    1.31413    ,    1.41685    ,    1.48334    ,    1.56866    ,    1.60193    ,    1.62771    ,    1.66359    ,    1.70974    ,    1.76798    ,    1.80155    ,    1.79822    ,    1.8185    ,    1.83641    ,    1.85772    ,    1.887    ,    1.90582    ,    1.93614    ,    1.96884    ,    2.01346    ,    2.05433    ,    2.08418    ,    2.08938    ,    2.09921    ,    2.09055    ,    2.08541    ,    2.051    ,    2.01734    ,    1.9995    ,    1.99596    ,    1.98579    ,    1.9804    ,    1.97722    ,    1.98719    ,    1.9835    ,    1.99192    ,    2.01275    ,    2.01954    ,    2.01906    ,    2.00633    ,    1.96707    ,    1.90898    ,    1.87435    ,    1.86008    ,    1.83253    ,    1.81069    ,    1.78543    ,    1.79987    ,    1.82782    ,    1.87805    ,    1.96583    ,    2.01218    ,    2.00138    ,    2.00652    ,    2.01739    ,    2.06137    ,    2.11883    ,    2.19386    ,    2.25233    ,    2.29787    },
    {    2.53036    ,    2.63765    ,    2.65412    ,    2.5865    ,    2.49062    ,    2.38272    ,    2.29002    ,    2.21441    ,    2.14268    ,    2.08339    ,    2.04156    ,    2.01159    ,    1.99303    ,    2.0215    ,    2.08088    ,    2.13877    ,    2.12616    ,    2.10962    ,    2.09308    ,    2.10694    ,    2.12408    ,    2.15389    ,    2.17983    ,    2.20167    ,    2.23393    ,    2.25571    ,    2.29922    ,    2.34213    ,    2.35956    ,    2.37593    ,    2.36897    ,    2.36348    ,    2.36805    ,    2.37771    ,    2.36418    ,    2.32321    ,    2.25925    ,    2.22794    ,    2.17649    ,    2.11766    ,    2.03839    ,    2.00968    ,    1.95096    ,    1.83452    ,    1.65139    ,    1.50697    ,    1.36821    ,    1.22528    ,    1.287    ,    1.34853    ,    1.35468    ,    1.33141    ,    1.33685    ,    1.43343    ,    1.55536    ,    1.7537    ,    1.885    ,    1.87207    ,    1.78497    ,    1.66914    ,    1.56158    ,    1.46644    ,    1.39213    ,    1.34004    ,    1.29798    ,    1.24608    ,    1.18899    ,    1.14516    ,    1.12863    ,    1.19772    ,    1.39951    ,    1.52389    ,    1.59669    ,    1.6505    ,    1.6979    ,    1.7353    ,    1.78509    ,    1.84245    ,    1.88296    ,    1.92219    ,    2.00698    ,    2.03783    ,    2.11203    ,    2.15526    ,    2.18459    ,    2.20431    ,    2.21364    ,    2.22717    ,    2.24546    ,    2.25683    ,    2.26626    ,    2.26846    ,    2.26    ,    2.2271    ,    2.1969    ,    2.14261    ,    2.08525    ,    2.04755    ,    2.03755    ,    2.03773    ,    2.0638    ,    2.09842    ,    2.11042    ,    2.11071    ,    2.12372    ,    2.11813    ,    2.11778    ,    2.09205    ,    2.02743    ,    1.96699    ,    1.94786    ,    1.95199    ,    1.9617    ,    1.95366    ,    1.95375    ,    1.98016    ,    2.03357    ,    2.09338    ,    2.14083    ,    2.17535    ,    2.17842    ,    2.16975    ,    2.18502    ,    2.21473    ,    2.25176    ,    2.31728    ,    2.37002    ,    2.42409    },
    {    2.74845    ,    2.77053    ,    2.78599    ,    2.7654    ,    2.72328    ,    2.67322    ,    2.62837    ,    2.60051    ,    2.59055    ,    2.55385    ,    2.53053    ,    2.5091    ,    2.49983    ,    2.51057    ,    2.53176    ,    2.53206    ,    2.50952    ,    2.46955    ,    2.42708    ,    2.39447    ,    2.37851    ,    2.3823    ,    2.39428    ,    2.39352    ,    2.40521    ,    2.42423    ,    2.45283    ,    2.49725    ,    2.52584    ,    2.50857    ,    2.50294    ,    2.5071    ,    2.5124    ,    2.46014    ,    2.40642    ,    2.33859    ,    2.26366    ,    2.21599    ,    2.17561    ,    2.12612    ,    2.11422    ,    2.07804    ,    2.00249    ,    1.8776    ,    1.70049    ,    1.53512    ,    1.3384    ,    1.26756    ,    1.32879    ,    1.39626    ,    1.4474    ,    1.44478    ,    1.40644    ,    1.3668    ,    1.41613    ,    1.58587    ,    1.8924    ,    2.13046    ,    2.17429    ,    2.1142    ,    2.00577    ,    1.88745    ,    1.78234    ,    1.67814    ,    1.58847    ,    1.51327    ,    1.44504    ,    1.37045    ,    1.29754    ,    1.27349    ,    1.31092    ,    1.4036    ,    1.52401    ,    1.66939    ,    1.76556    ,    1.78839    ,    1.82853    ,    1.89384    ,    1.97196    ,    2.06068    ,    2.16141    ,    2.2642    ,    2.35159    ,    2.44699    ,    2.52625    ,    2.57718    ,    2.56789    ,    2.51378    ,    2.46546    ,    2.42738    ,    2.40378    ,    2.39465    ,    2.38147    ,    2.36355    ,    2.33543    ,    2.2962    ,    2.24805    ,    2.20815    ,    2.19768    ,    2.20351    ,    2.22184    ,    2.2335    ,    2.23504    ,    2.23606    ,    2.24728    ,    2.24806    ,    2.2332    ,    2.21259    ,    2.18708    ,    2.16006    ,    2.14231    ,    2.14705    ,    2.15765    ,    2.18653    ,    2.21063    ,    2.26053    ,    2.30497    ,    2.34034    ,    2.37556    ,    2.41317    ,    2.44611    ,    2.45725    ,    2.48726    ,    2.52092    ,    2.55561    ,    2.59758    ,    2.64105    ,    2.68829    },
    {    2.86154    ,    2.8798    ,    2.88953    ,    2.88907    ,    2.87894    ,    2.86126    ,    2.83987    ,    2.81892    ,    2.80121    ,    2.78542    ,    2.77015    ,    2.75679    ,    2.74219    ,    2.72719    ,    2.70918    ,    2.68701    ,    2.65865    ,    2.62358    ,    2.58139    ,    2.53038    ,    2.52026    ,    2.54814    ,    2.57007    ,    2.59331    ,    2.62748    ,    2.64951    ,    2.68191    ,    2.69476    ,    2.6999    ,    2.68436    ,    2.67468    ,    2.63082    ,    2.59342    ,    2.52442    ,    2.44348    ,    2.36322    ,    2.2861    ,    2.24029    ,    2.22689    ,    2.20429    ,    2.15342    ,    2.07254    ,    1.96443    ,    1.82547    ,    1.60879    ,    1.39577    ,    1.35186    ,    1.36921    ,    1.41035    ,    1.4592    ,    1.48805    ,    1.47302    ,    1.46241    ,    1.44415    ,    1.46532    ,    1.56762    ,    1.74228    ,    2.04927    ,    2.2521    ,    2.34372    ,    2.32795    ,    2.22678    ,    2.08729    ,    1.94658    ,    1.82905    ,    1.73969    ,    1.66578    ,    1.61527    ,    1.5868    ,    1.61619    ,    1.709    ,    1.82676    ,    1.9083    ,    1.97018    ,    2.02525    ,    2.07684    ,    2.13789    ,    2.21588    ,    2.27779    ,    2.32996    ,    2.38069    ,    2.42901    ,    2.4726    ,    2.53093    ,    2.57598    ,    2.69805    ,    2.71735    ,    2.69968    ,    2.64893    ,    2.59707    ,    2.53137    ,    2.48492    ,    2.45523    ,    2.43873    ,    2.44702    ,    2.43562    ,    2.42596    ,    2.41764    ,    2.41406    ,    2.40993    ,    2.40207    ,    2.39943    ,    2.39725    ,    2.39299    ,    2.37541    ,    2.34652    ,    2.31779    ,    2.30336    ,    2.30435    ,    2.30679    ,    2.28981    ,    2.30166    ,    2.32506    ,    2.35752    ,    2.41284    ,    2.43809    ,    2.47476    ,    2.50301    ,    2.53528    ,    2.57201    ,    2.60735    ,    2.63984    ,    2.6719    ,    2.70214    ,    2.73254    ,    2.76359    ,    2.79541    ,    2.82826    },
    {    2.89728    ,    2.9039    ,    2.9068    ,    2.90579    ,    2.90018    ,    2.89161    ,    2.88216    ,    2.87421    ,    2.86564    ,    2.85743    ,    2.84796    ,    2.83718    ,    2.82439    ,    2.80977    ,    2.79414    ,    2.77787    ,    2.76054    ,    2.74336    ,    2.71889    ,    2.68649    ,    2.64643    ,    2.63811    ,    2.67665    ,    2.72742    ,    2.75103    ,    2.76349    ,    2.76912    ,    2.77324    ,    2.75429    ,    2.72684    ,    2.66519    ,    2.62701    ,    2.5556    ,    2.50085    ,    2.38454    ,    2.28065    ,    2.18209    ,    2.14507    ,    2.11373    ,    2.03729    ,    1.89526    ,    1.76789    ,    1.66953    ,    1.55843    ,    1.47504    ,    1.46638    ,    1.46363    ,    1.47351    ,    1.49904    ,    1.50555    ,    1.51488    ,    1.51087    ,    1.49953    ,    1.5036    ,    1.53006    ,    1.59256    ,    1.72555    ,    2.09201    ,    2.31348    ,    2.41333    ,    2.4231    ,    2.3574    ,    2.2462    ,    2.14456    ,    2.04937    ,    1.96695    ,    1.90666    ,    1.90632    ,    1.98025    ,    2.11551    ,    2.28094    ,    2.45919    ,    2.50777    ,    2.60298    ,    2.65015    ,    2.68375    ,    2.71409    ,    2.73047    ,    2.74086    ,    2.75208    ,    2.76287    ,    2.77516    ,    2.78638    ,    2.80005    ,    2.8222    ,    2.83694    ,    2.86832    ,    2.90512    ,    2.90403    ,    2.90207    ,    2.85704    ,    2.79384    ,    2.72717    ,    2.65887    ,    2.59871    ,    2.54722    ,    2.50579    ,    2.4907    ,    2.49172    ,    2.4831    ,    2.50903    ,    2.50829    ,    2.48482    ,    2.4941    ,    2.4681    ,    2.45743    ,    2.47104    ,    2.4753    ,    2.47587    ,    2.47928    ,    2.48391    ,    2.49171    ,    2.50399    ,    2.52583    ,    2.54171    ,    2.57062    ,    2.59487    ,    2.62532    ,    2.65337    ,    2.68419    ,    2.71463    ,    2.74597    ,    2.77614    ,    2.80345    ,    2.82783    ,    2.85031    ,    2.87012    ,    2.88629    },
    {    2.89176    ,    2.89391    ,    2.89521    ,    2.89548    ,    2.89463    ,    2.89334    ,    2.88977    ,    2.88568    ,    2.88139    ,    2.87577    ,    2.8695    ,    2.86138    ,    2.85256    ,    2.84379    ,    2.83523    ,    2.82647    ,    2.81869    ,    2.80888    ,    2.79931    ,    2.78789    ,    2.77543    ,    2.76225    ,    2.75052    ,    2.73967    ,    2.73101    ,    2.72492    ,    2.70832    ,    2.6793    ,    2.65217    ,    2.60823    ,    2.56324    ,    2.47956    ,    2.42448    ,    2.37254    ,    2.29354    ,    2.20256    ,    2.15974    ,    2.14205    ,    2.11873    ,    2.0969    ,    2.0055    ,    1.86919    ,    1.72934    ,    1.64948    ,    1.60395    ,    1.58277    ,    1.5726    ,    1.57238    ,    1.57667    ,    1.58481    ,    1.59113    ,    1.59672    ,    1.62106    ,    1.67265    ,    1.73526    ,    1.82142    ,    1.95191    ,    2.15153    ,    2.31352    ,    2.40839    ,    2.44978    ,    2.43898    ,    2.39492    ,    2.35664    ,    2.34318    ,    2.31956    ,    2.29492    ,    2.29433    ,    2.32432    ,    2.43245    ,    2.54069    ,    2.61847    ,    2.70278    ,    2.83277    ,    2.93252    ,    2.96694    ,    2.98042    ,    2.98878    ,    2.99966    ,    3.0161    ,    3.03506    ,    3.04423    ,    3.05317    ,    3.06069    ,    3.0533    ,    3.05741    ,    3.07152    ,    3.09927    ,    3.1264    ,    3.1375    ,    3.11799    ,    3.06506    ,    3.01025    ,    2.94693    ,    2.88291    ,    2.81675    ,    2.74515    ,    2.70074    ,    2.66456    ,    2.66359    ,    2.67998    ,    2.67823    ,    2.67106    ,    2.67554    ,    2.67869    ,    2.67986    ,    2.67792    ,    2.67458    ,    2.67176    ,    2.67083    ,    2.67411    ,    2.67845    ,    2.68288    ,    2.68935    ,    2.70014    ,    2.71507    ,    2.73169    ,    2.74891    ,    2.76713    ,    2.78687    ,    2.80647    ,    2.82512    ,    2.84073    ,    2.85322    ,    2.86393    ,    2.87295    ,    2.8814    ,    2.88794    },
    {    2.88126    ,    2.88096    ,    2.88008    ,    2.87861    ,    2.87782    ,    2.87669    ,    2.87586    ,    2.87381    ,    2.87149    ,    2.86828    ,    2.86483    ,    2.86176    ,    2.85832    ,    2.85349    ,    2.84907    ,    2.84516    ,    2.8411    ,    2.83605    ,    2.83193    ,    2.82748    ,    2.82249    ,    2.81651    ,    2.80992    ,    2.80087    ,    2.79151    ,    2.7791    ,    2.76394    ,    2.74569    ,    2.72328    ,    2.69005    ,    2.65792    ,    2.62396    ,    2.5941    ,    2.54083    ,    2.47417    ,    2.44447    ,    2.40983    ,    2.38823    ,    2.36791    ,    2.34763    ,    2.33656    ,    2.35208    ,    2.35103    ,    2.33608    ,    2.29913    ,    2.25715    ,    2.21478    ,    2.17102    ,    2.13107    ,    2.08365    ,    2.06684    ,    2.05021    ,    2.10371    ,    2.16233    ,    2.22193    ,    2.32156    ,    2.46445    ,    2.55691    ,    2.61566    ,    2.6495    ,    2.6668    ,    2.67017    ,    2.6666    ,    2.66749    ,    2.67113    ,    2.68272    ,    2.70376    ,    2.73199    ,    2.76546    ,    2.80475    ,    2.83864    ,    2.87202    ,    2.89696    ,    2.9248    ,    2.94785    ,    2.96487    ,    2.97892    ,    2.99159    ,    3.00505    ,    3.01374    ,    3.01921    ,    3.01693    ,    3.0158    ,    3.01548    ,    3.01824    ,    3.03236    ,    3.043    ,    3.05641    ,    3.07108    ,    3.08429    ,    3.08933    ,    3.08158    ,    3.06549    ,    3.0397    ,    3.01442    ,    2.98522    ,    2.95458    ,    2.92836    ,    2.91439    ,    2.90306    ,    2.8883    ,    2.87471    ,    2.86177    ,    2.84987    ,    2.83638    ,    2.8283    ,    2.81995    ,    2.81428    ,    2.81183    ,    2.81047    ,    2.80918    ,    2.8087    ,    2.81032    ,    2.81275    ,    2.81597    ,    2.82092    ,    2.82652    ,    2.83312    ,    2.83978    ,    2.84719    ,    2.85436    ,    2.86141    ,    2.8673    ,    2.87227    ,    2.87653    ,    2.878    ,    2.87945    ,    2.88112    },
    {    2.87532    ,    2.87513    ,    2.87483    ,    2.87485    ,    2.87475    ,    2.87456    ,    2.87417    ,    2.87449    ,    2.87419    ,    2.87305    ,    2.8724    ,    2.87116    ,    2.86993    ,    2.86825    ,    2.86651    ,    2.86546    ,    2.86396    ,    2.86235    ,    2.86053    ,    2.85816    ,    2.85627    ,    2.85377    ,    2.84938    ,    2.84575    ,    2.84297    ,    2.83551    ,    2.83042    ,    2.82416    ,    2.81811    ,    2.80837    ,    2.80159    ,    2.79903    ,    2.79715    ,    2.79097    ,    2.78748    ,    2.78357    ,    2.78357    ,    2.78471    ,    2.78874    ,    2.79055    ,    2.79459    ,    2.79626    ,    2.79474    ,    2.79045    ,    2.78334    ,    2.77874    ,    2.77426    ,    2.76827    ,    2.76107    ,    2.75904    ,    2.75712    ,    2.75661    ,    2.76184    ,    2.77139    ,    2.78119    ,    2.79367    ,    2.80604    ,    2.81989    ,    2.82733    ,    2.83539    ,    2.84151    ,    2.8441    ,    2.84694    ,    2.84945    ,    2.85138    ,    2.8556    ,    2.86088    ,    2.867    ,    2.87441    ,    2.88294    ,    2.8917    ,    2.9012    ,    2.90862    ,    2.92009    ,    2.9268    ,    2.934    ,    2.94182    ,    2.9461    ,    2.95154    ,    2.95467    ,    2.9576    ,    2.96082    ,    2.96119    ,    2.96182    ,    2.96448    ,    2.96662    ,    2.96836    ,    2.97078    ,    2.97254    ,    2.97339    ,    2.97407    ,    2.97329    ,    2.97086    ,    2.96782    ,    2.96187    ,    2.95469    ,    2.94669    ,    2.93711    ,    2.92814    ,    2.91973    ,    2.91232    ,    2.90458    ,    2.89901    ,    2.89391    ,    2.88986    ,    2.88573    ,    2.88184    ,    2.87888    ,    2.87569    ,    2.87308    ,    2.87151    ,    2.86899    ,    2.86669    ,    2.8659    ,    2.8658    ,    2.86684    ,    2.86766    ,    2.86868    ,    2.86973    ,    2.87104    ,    2.87211    ,    2.87272    ,    2.87364    ,    2.87401    ,    2.87441    ,    2.87446    ,    2.87457    ,    2.8747    },
    {    2.89035    ,    2.89068    ,    2.89105    ,    2.89147    ,    2.89204    ,    2.89231    ,    2.89304    ,    2.89383    ,    2.89452    ,    2.89487    ,    2.89487    ,    2.8948    ,    2.89498    ,    2.89546    ,    2.89502    ,    2.89453    ,    2.89447    ,    2.89405    ,    2.89375    ,    2.89337    ,    2.89159    ,    2.89082    ,    2.89064    ,    2.89131    ,    2.89115    ,    2.89055    ,    2.89033    ,    2.89073    ,    2.89095    ,    2.89162    ,    2.89115    ,    2.89206    ,    2.89317    ,    2.89452    ,    2.89525    ,    2.89558    ,    2.89514    ,    2.89777    ,    2.89911    ,    2.901    ,    2.90244    ,    2.90258    ,    2.90284    ,    2.90412    ,    2.90471    ,    2.90485    ,    2.90437    ,    2.9043    ,    2.90363    ,    2.90251    ,    2.90271    ,    2.90306    ,    2.90447    ,    2.90664    ,    2.90653    ,    2.90645    ,    2.9084    ,    2.91048    ,    2.91206    ,    2.91329    ,    2.91397    ,    2.91422    ,    2.91533    ,    2.91612    ,    2.91703    ,    2.91729    ,    2.91846    ,    2.91931    ,    2.92055    ,    2.92203    ,    2.92313    ,    2.92468    ,    2.92592    ,    2.92692    ,    2.92778    ,    2.92904    ,    2.93018    ,    2.93068    ,    2.9308    ,    2.93094    ,    2.93258    ,    2.93317    ,    2.93291    ,    2.93271    ,    2.93276    ,    2.93337    ,    2.93367    ,    2.93359    ,    2.93401    ,    2.93367    ,    2.93275    ,    2.93158    ,    2.9311    ,    2.93259    ,    2.93227    ,    2.93081    ,    2.92896    ,    2.92743    ,    2.92567    ,    2.92329    ,    2.92135    ,    2.91924    ,    2.91714    ,    2.91512    ,    2.91283    ,    2.9107    ,    2.90837    ,    2.90672    ,    2.90537    ,    2.90385    ,    2.90246    ,    2.89997    ,    2.89732    ,    2.89573    ,    2.89452    ,    2.89361    ,    2.89394    ,    2.89411    ,    2.89362    ,    2.89268    ,    2.89209    ,    2.8914    ,    2.89083    ,    2.89038    ,    2.89078    ,    2.89064    ,    2.89054    ,    2.89038    }
};


double Stdev_LGRTC_RCP85[64][128] = {
    {    0.272276    ,    0.268094    ,    0.26733    ,    0.266339    ,    0.264758    ,    0.262524    ,    0.262692    ,    0.262027    ,    0.260143    ,    0.260349    ,    0.259258    ,    0.259325    ,    0.259426    ,    0.258896    ,    0.257546    ,    0.257012    ,    0.255583    ,    0.25341    ,    0.253455    ,    0.254262    ,    0.2558    ,    0.256736    ,    0.257862    ,    0.258671    ,    0.258534    ,    0.260581    ,    0.260705    ,    0.261731    ,    0.258339    ,    0.253414    ,    0.25239    ,    0.250149    ,    0.246458    ,    0.243614    ,    0.241322    ,    0.240805    ,    0.239758    ,    0.2406    ,    0.241577    ,    0.238798    ,    0.238734    ,    0.240121    ,    0.23823    ,    0.23717    ,    0.23638    ,    0.235276    ,    0.238089    ,    0.240657    ,    0.242201    ,    0.241522    ,    0.241952    ,    0.242016    ,    0.242546    ,    0.243361    ,    0.244127    ,    0.243885    ,    0.245952    ,    0.24697    ,    0.250144    ,    0.252643    ,    0.256973    ,    0.25836    ,    0.261081    ,    0.263548    ,    0.264907    ,    0.267808    ,    0.267586    ,    0.267173    ,    0.269792    ,    0.269682    ,    0.271408    ,    0.274656    ,    0.272071    ,    0.272848    ,    0.27118    ,    0.272684    ,    0.269692    ,    0.267115    ,    0.269692    ,    0.267658    ,    0.265247    ,    0.265777    ,    0.267245    ,    0.267302    ,    0.263571    ,    0.265429    ,    0.258832    ,    0.262683    ,    0.262429    ,    0.257472    ,    0.261001    ,    0.26246    ,    0.260706    ,    0.267251    ,    0.260273    ,    0.263612    ,    0.261459    ,    0.263739    ,    0.265545    ,    0.270858    ,    0.27306    ,    0.270947    ,    0.272849    ,    0.277731    ,    0.274033    ,    0.276255    ,    0.279888    ,    0.281301    ,    0.282994    ,    0.282727    ,    0.281252    ,    0.279331    ,    0.280901    ,    0.285173    ,    0.285835    ,    0.283505    ,    0.280769    ,    0.280751    ,    0.275849    ,    0.27575    ,    0.273759    ,    0.273132    ,    0.272378    ,    0.271788    ,    0.272591    ,    0.271051    ,    0.272835    ,    0.273102    },
    {    0.255644    ,    0.254021    ,    0.254223    ,    0.249945    ,    0.252891    ,    0.249427    ,    0.248152    ,    0.246649    ,    0.251681    ,    0.257543    ,    0.263114    ,    0.268326    ,    0.270802    ,    0.275352    ,    0.282433    ,    0.287404    ,    0.289115    ,    0.289512    ,    0.286887    ,    0.28736    ,    0.290311    ,    0.296293    ,    0.30257    ,    0.309693    ,    0.312637    ,    0.316652    ,    0.31876    ,    0.319362    ,    0.314981    ,    0.309874    ,    0.298307    ,    0.282523    ,    0.268786    ,    0.25384    ,    0.24    ,    0.228134    ,    0.223016    ,    0.226098    ,    0.230147    ,    0.233298    ,    0.23275    ,    0.230639    ,    0.22369    ,    0.216801    ,    0.20933    ,    0.20666    ,    0.205328    ,    0.20828    ,    0.211572    ,    0.215089    ,    0.212859    ,    0.21445    ,    0.219286    ,    0.222483    ,    0.225802    ,    0.226998    ,    0.229439    ,    0.233338    ,    0.239574    ,    0.244208    ,    0.24995    ,    0.255694    ,    0.263635    ,    0.272397    ,    0.281411    ,    0.290814    ,    0.297717    ,    0.298222    ,    0.292222    ,    0.286343    ,    0.288565    ,    0.277474    ,    0.276126    ,    0.267823    ,    0.261877    ,    0.257583    ,    0.253357    ,    0.253556    ,    0.257856    ,    0.269383    ,    0.280517    ,    0.276025    ,    0.269644    ,    0.27949    ,    0.27615    ,    0.273703    ,    0.262219    ,    0.261194    ,    0.259961    ,    0.254436    ,    0.257062    ,    0.250889    ,    0.253243    ,    0.255595    ,    0.256188    ,    0.256051    ,    0.261213    ,    0.257325    ,    0.260321    ,    0.269962    ,    0.275931    ,    0.274371    ,    0.275384    ,    0.27272    ,    0.273723    ,    0.265084    ,    0.257032    ,    0.253072    ,    0.248625    ,    0.256216    ,    0.24913    ,    0.246493    ,    0.243718    ,    0.24361    ,    0.242722    ,    0.241225    ,    0.249426    ,    0.250346    ,    0.259156    ,    0.257338    ,    0.255098    ,    0.256156    ,    0.255779    ,    0.259253    ,    0.259622    ,    0.260864    ,    0.263481    ,    0.258452    },
    {    0.289901    ,    0.301006    ,    0.307851    ,    0.317562    ,    0.32229    ,    0.33092    ,    0.340587    ,    0.344653    ,    0.34769    ,    0.346376    ,    0.350259    ,    0.353262    ,    0.353089    ,    0.359    ,    0.364457    ,    0.368417    ,    0.371538    ,    0.373435    ,    0.372791    ,    0.370729    ,    0.365746    ,    0.357997    ,    0.353226    ,    0.354488    ,    0.358573    ,    0.369108    ,    0.382883    ,    0.396344    ,    0.404261    ,    0.398257    ,    0.384533    ,    0.361576    ,    0.329687    ,    0.30038    ,    0.273809    ,    0.245244    ,    0.228811    ,    0.223146    ,    0.225641    ,    0.234049    ,    0.249839    ,    0.263343    ,    0.272946    ,    0.269533    ,    0.258941    ,    0.250885    ,    0.24046    ,    0.229243    ,    0.215998    ,    0.204201    ,    0.203102    ,    0.20758    ,    0.211196    ,    0.218531    ,    0.224977    ,    0.231745    ,    0.235143    ,    0.2421    ,    0.248498    ,    0.258579    ,    0.269917    ,    0.274654    ,    0.274981    ,    0.277207    ,    0.281267    ,    0.286187    ,    0.294098    ,    0.301022    ,    0.311027    ,    0.310645    ,    0.301918    ,    0.291914    ,    0.286311    ,    0.279989    ,    0.27617    ,    0.277852    ,    0.278717    ,    0.271011    ,    0.275805    ,    0.274404    ,    0.278153    ,    0.30917    ,    0.304115    ,    0.274432    ,    0.281054    ,    0.265375    ,    0.276886    ,    0.27115    ,    0.251863    ,    0.241113    ,    0.238205    ,    0.249392    ,    0.253948    ,    0.270427    ,    0.277078    ,    0.266292    ,    0.253425    ,    0.248974    ,    0.244775    ,    0.248396    ,    0.251031    ,    0.24949    ,    0.247894    ,    0.240083    ,    0.242063    ,    0.234554    ,    0.232637    ,    0.231456    ,    0.227817    ,    0.221547    ,    0.219694    ,    0.217784    ,    0.216565    ,    0.215925    ,    0.217899    ,    0.223631    ,    0.227856    ,    0.234508    ,    0.234628    ,    0.238912    ,    0.24137    ,    0.240458    ,    0.232893    ,    0.24148    ,    0.249714    ,    0.269068    ,    0.281425    ,    0.284528    },
    {    0.307857    ,    0.319435    ,    0.329056    ,    0.340307    ,    0.347519    ,    0.358629    ,    0.371201    ,    0.382996    ,    0.394162    ,    0.404496    ,    0.413657    ,    0.422247    ,    0.42764    ,    0.428225    ,    0.425642    ,    0.418845    ,    0.409616    ,    0.401311    ,    0.394465    ,    0.390761    ,    0.391213    ,    0.391907    ,    0.393252    ,    0.393777    ,    0.390745    ,    0.387416    ,    0.387741    ,    0.389167    ,    0.389715    ,    0.386745    ,    0.379872    ,    0.367354    ,    0.347979    ,    0.325209    ,    0.304638    ,    0.286308    ,    0.271156    ,    0.265902    ,    0.266206    ,    0.275173    ,    0.282358    ,    0.293577    ,    0.298666    ,    0.303417    ,    0.305982    ,    0.305784    ,    0.301199    ,    0.288528    ,    0.285952    ,    0.300827    ,    0.309485    ,    0.297447    ,    0.270109    ,    0.266464    ,    0.260169    ,    0.255176    ,    0.250599    ,    0.254533    ,    0.257376    ,    0.257186    ,    0.256656    ,    0.264351    ,    0.268308    ,    0.271488    ,    0.271691    ,    0.273328    ,    0.275444    ,    0.272832    ,    0.274713    ,    0.286021    ,    0.283667    ,    0.297698    ,    0.307337    ,    0.295928    ,    0.292891    ,    0.295024    ,    0.311804    ,    0.286545    ,    0.288299    ,    0.293854    ,    0.26748    ,    0.237115    ,    0.227594    ,    0.236721    ,    0.221415    ,    0.217173    ,    0.216524    ,    0.225546    ,    0.225853    ,    0.225591    ,    0.227546    ,    0.224751    ,    0.234633    ,    0.223818    ,    0.233915    ,    0.229674    ,    0.246413    ,    0.251522    ,    0.269228    ,    0.247587    ,    0.256916    ,    0.265137    ,    0.258026    ,    0.248802    ,    0.252363    ,    0.246409    ,    0.247424    ,    0.247665    ,    0.246714    ,    0.241419    ,    0.235654    ,    0.232336    ,    0.228821    ,    0.225978    ,    0.227439    ,    0.228976    ,    0.233578    ,    0.235594    ,    0.228211    ,    0.224425    ,    0.225974    ,    0.234651    ,    0.233913    ,    0.244001    ,    0.26318    ,    0.280209    ,    0.288882    ,    0.295985    },
    {    0.38201    ,    0.418211    ,    0.447378    ,    0.471212    ,    0.502053    ,    0.537631    ,    0.569689    ,    0.586347    ,    0.582711    ,    0.602936    ,    0.613984    ,    0.551508    ,    0.541527    ,    0.474309    ,    0.447139    ,    0.4316    ,    0.422459    ,    0.416385    ,    0.413629    ,    0.40723    ,    0.398455    ,    0.388365    ,    0.378729    ,    0.365465    ,    0.3485    ,    0.338996    ,    0.334818    ,    0.33449    ,    0.331675    ,    0.330208    ,    0.329373    ,    0.33379    ,    0.334124    ,    0.328382    ,    0.314035    ,    0.309137    ,    0.307149    ,    0.304147    ,    0.29359    ,    0.280207    ,    0.273898    ,    0.27045    ,    0.310993    ,    0.34515    ,    0.364267    ,    0.388452    ,    0.408724    ,    0.429022    ,    0.452624    ,    0.477018    ,    0.496643    ,    0.506816    ,    0.490753    ,    0.431033    ,    0.399835    ,    0.333447    ,    0.297861    ,    0.266001    ,    0.256573    ,    0.257608    ,    0.260528    ,    0.275556    ,    0.288792    ,    0.297483    ,    0.296581    ,    0.292632    ,    0.318925    ,    0.298725    ,    0.301745    ,    0.28818    ,    0.271994    ,    0.272782    ,    0.263164    ,    0.258285    ,    0.25368    ,    0.257917    ,    0.236884    ,    0.251375    ,    0.23346    ,    0.250533    ,    0.255909    ,    0.245345    ,    0.24073    ,    0.238631    ,    0.239108    ,    0.238727    ,    0.239116    ,    0.239911    ,    0.241353    ,    0.243853    ,    0.25154    ,    0.261116    ,    0.265841    ,    0.266385    ,    0.269888    ,    0.274141    ,    0.274837    ,    0.286847    ,    0.297069    ,    0.31176    ,    0.280208    ,    0.276878    ,    0.298522    ,    0.306023    ,    0.298457    ,    0.296874    ,    0.295843    ,    0.295403    ,    0.307097    ,    0.301122    ,    0.297464    ,    0.296064    ,    0.289757    ,    0.285578    ,    0.27935    ,    0.269068    ,    0.260287    ,    0.252966    ,    0.252413    ,    0.249226    ,    0.247248    ,    0.257607    ,    0.309593    ,    0.292587    ,    0.290407    ,    0.306582    ,    0.324339    ,    0.350275    },
    {    0.54916    ,    0.613818    ,    0.662512    ,    0.690876    ,    0.699253    ,    0.69686    ,    0.693976    ,    0.697369    ,    0.7043    ,    0.710125    ,    0.713597    ,    0.713626    ,    0.707317    ,    0.695418    ,    0.675056    ,    0.644826    ,    0.605291    ,    0.58686    ,    0.547313    ,    0.509067    ,    0.5056    ,    0.503088    ,    0.498626    ,    0.489861    ,    0.488793    ,    0.463434    ,    0.438068    ,    0.410044    ,    0.350041    ,    0.337011    ,    0.341999    ,    0.319184    ,    0.303515    ,    0.303727    ,    0.30272    ,    0.28665    ,    0.252941    ,    0.245141    ,    0.245096    ,    0.250184    ,    0.266613    ,    0.265091    ,    0.304641    ,    0.349998    ,    0.37341    ,    0.398919    ,    0.424437    ,    0.447085    ,    0.465335    ,    0.482077    ,    0.498928    ,    0.511863    ,    0.521151    ,    0.528385    ,    0.532863    ,    0.528989    ,    0.519399    ,    0.4626    ,    0.383484    ,    0.285873    ,    0.258475    ,    0.249138    ,    0.257454    ,    0.267077    ,    0.264    ,    0.2586    ,    0.25091    ,    0.25142    ,    0.251261    ,    0.247307    ,    0.243767    ,    0.235224    ,    0.236979    ,    0.238585    ,    0.232821    ,    0.223624    ,    0.216896    ,    0.212412    ,    0.213398    ,    0.209781    ,    0.211648    ,    0.213607    ,    0.217426    ,    0.22213    ,    0.23379    ,    0.248112    ,    0.260841    ,    0.28194    ,    0.291066    ,    0.251753    ,    0.256753    ,    0.274478    ,    0.278095    ,    0.282715    ,    0.284996    ,    0.284895    ,    0.28511    ,    0.290533    ,    0.296345    ,    0.294813    ,    0.290987    ,    0.29004    ,    0.29083    ,    0.295033    ,    0.297195    ,    0.292245    ,    0.284766    ,    0.291982    ,    0.302236    ,    0.304575    ,    0.313331    ,    0.318282    ,    0.311256    ,    0.299831    ,    0.291008    ,    0.28475    ,    0.277722    ,    0.272643    ,    0.268807    ,    0.270858    ,    0.267982    ,    0.26689    ,    0.273592    ,    0.291824    ,    0.291143    ,    0.321827    ,    0.377824    ,    0.463461    },
    {    0.67301    ,    0.684415    ,    0.6811    ,    0.669609    ,    0.658824    ,    0.664433    ,    0.681309    ,    0.701496    ,    0.724696    ,    0.746234    ,    0.763364    ,    0.771892    ,    0.773526    ,    0.769103    ,    0.760359    ,    0.746764    ,    0.727065    ,    0.704488    ,    0.683741    ,    0.667669    ,    0.654715    ,    0.646623    ,    0.639818    ,    0.633586    ,    0.625198    ,    0.610718    ,    0.591458    ,    0.564118    ,    0.539348    ,    0.525317    ,    0.509048    ,    0.489791    ,    0.47401    ,    0.454683    ,    0.437195    ,    0.425924    ,    0.408926    ,    0.357874    ,    0.334994    ,    0.284741    ,    0.260589    ,    0.259408    ,    0.283832    ,    0.341461    ,    0.379956    ,    0.407818    ,    0.427711    ,    0.442941    ,    0.45884    ,    0.48055    ,    0.508244    ,    0.534766    ,    0.558516    ,    0.579119    ,    0.591616    ,    0.594395    ,    0.587135    ,    0.571026    ,    0.543414    ,    0.498153    ,    0.437057    ,    0.388432    ,    0.360277    ,    0.343821    ,    0.361209    ,    0.324117    ,    0.295675    ,    0.280213    ,    0.265013    ,    0.261762    ,    0.260554    ,    0.263364    ,    0.264676    ,    0.259671    ,    0.252429    ,    0.246341    ,    0.23771    ,    0.233339    ,    0.222686    ,    0.214182    ,    0.208533    ,    0.203934    ,    0.205754    ,    0.217755    ,    0.223238    ,    0.228432    ,    0.230339    ,    0.23363    ,    0.251535    ,    0.303422    ,    0.26526    ,    0.272419    ,    0.28774    ,    0.297181    ,    0.297963    ,    0.288925    ,    0.282126    ,    0.280259    ,    0.277593    ,    0.27691    ,    0.278736    ,    0.27952    ,    0.275051    ,    0.270983    ,    0.272289    ,    0.273683    ,    0.271567    ,    0.267732    ,    0.269228    ,    0.272289    ,    0.278302    ,    0.287644    ,    0.291425    ,    0.286257    ,    0.275833    ,    0.269582    ,    0.27048    ,    0.267039    ,    0.266535    ,    0.269003    ,    0.280204    ,    0.290431    ,    0.317325    ,    0.351349    ,    0.429759    ,    0.491641    ,    0.584512    ,    0.643203    },
    {    0.542618    ,    0.549634    ,    0.556856    ,    0.5607    ,    0.565758    ,    0.579467    ,    0.599855    ,    0.621134    ,    0.643355    ,    0.663875    ,    0.68338    ,    0.696608    ,    0.698826    ,    0.69465    ,    0.69578    ,    0.697291    ,    0.692901    ,    0.682139    ,    0.666267    ,    0.651597    ,    0.637562    ,    0.624215    ,    0.612042    ,    0.602462    ,    0.592328    ,    0.578969    ,    0.566804    ,    0.558817    ,    0.550152    ,    0.537598    ,    0.519746    ,    0.495558    ,    0.466704    ,    0.438753    ,    0.414945    ,    0.397967    ,    0.390867    ,    0.389887    ,    0.36882    ,    0.314604    ,    0.247535    ,    0.236233    ,    0.338702    ,    0.322834    ,    0.340771    ,    0.364866    ,    0.40069    ,    0.444872    ,    0.485233    ,    0.514482    ,    0.536139    ,    0.562484    ,    0.594745    ,    0.618308    ,    0.630986    ,    0.636384    ,    0.636634    ,    0.632621    ,    0.626991    ,    0.621199    ,    0.615602    ,    0.612716    ,    0.601272    ,    0.590681    ,    0.582959    ,    0.572919    ,    0.552531    ,    0.525037    ,    0.506135    ,    0.515768    ,    0.53404    ,    0.550356    ,    0.559441    ,    0.557358    ,    0.519747    ,    0.504328    ,    0.500458    ,    0.551909    ,    0.513037    ,    0.33765    ,    0.272969    ,    0.264756    ,    0.241487    ,    0.23228    ,    0.237784    ,    0.241723    ,    0.256844    ,    0.267972    ,    0.278479    ,    0.304072    ,    0.339433    ,    0.330792    ,    0.309386    ,    0.286956    ,    0.282879    ,    0.280926    ,    0.278205    ,    0.280343    ,    0.286419    ,    0.28866    ,    0.286925    ,    0.290841    ,    0.292246    ,    0.292121    ,    0.292283    ,    0.298394    ,    0.315122    ,    0.301368    ,    0.299395    ,    0.303492    ,    0.300406    ,    0.298246    ,    0.297019    ,    0.28934    ,    0.279886    ,    0.282846    ,    0.311955    ,    0.364334    ,    0.399667    ,    0.448924    ,    0.483616    ,    0.521575    ,    0.54476    ,    0.54802    ,    0.546052    ,    0.544997    ,    0.537949    ,    0.533788    },
    {    0.512469    ,    0.515831    ,    0.516676    ,    0.512723    ,    0.509231    ,    0.50923    ,    0.514477    ,    0.5218    ,    0.530672    ,    0.543934    ,    0.561694    ,    0.576416    ,    0.582215    ,    0.579106    ,    0.576975    ,    0.576163    ,    0.569893    ,    0.559151    ,    0.547859    ,    0.536913    ,    0.527222    ,    0.516813    ,    0.504335    ,    0.490903    ,    0.476206    ,    0.461923    ,    0.448512    ,    0.435812    ,    0.425946    ,    0.418838    ,    0.412639    ,    0.403139    ,    0.391632    ,    0.383909    ,    0.37663    ,    0.36784    ,    0.358209    ,    0.345926    ,    0.337003    ,    0.334024    ,    0.327156    ,    0.261444    ,    0.23991    ,    0.264152    ,    0.275237    ,    0.291104    ,    0.336231    ,    0.388048    ,    0.421432    ,    0.439528    ,    0.454731    ,    0.475811    ,    0.504766    ,    0.530023    ,    0.551672    ,    0.576496    ,    0.600726    ,    0.614664    ,    0.618222    ,    0.621558    ,    0.626374    ,    0.629984    ,    0.633907    ,    0.634876    ,    0.631345    ,    0.623648    ,    0.621589    ,    0.623253    ,    0.616923    ,    0.610824    ,    0.612722    ,    0.617852    ,    0.618827    ,    0.616728    ,    0.615094    ,    0.61395    ,    0.608757    ,    0.604889    ,    0.604654    ,    0.597136    ,    0.581823    ,    0.553318    ,    0.496557    ,    0.46459    ,    0.464954    ,    0.45766    ,    0.438223    ,    0.416061    ,    0.412515    ,    0.432655    ,    0.461603    ,    0.474945    ,    0.459473    ,    0.431244    ,    0.411154    ,    0.401371    ,    0.377459    ,    0.344779    ,    0.347695    ,    0.374527    ,    0.351602    ,    0.393434    ,    0.39239    ,    0.391783    ,    0.432967    ,    0.465928    ,    0.472756    ,    0.461036    ,    0.432617    ,    0.450938    ,    0.43567    ,    0.444336    ,    0.438082    ,    0.443026    ,    0.443596    ,    0.460694    ,    0.481977    ,    0.494854    ,    0.502829    ,    0.505492    ,    0.503552    ,    0.499856    ,    0.492568    ,    0.485999    ,    0.489017    ,    0.497281    ,    0.504466    ,    0.507767    },
    {    0.467501    ,    0.468344    ,    0.467518    ,    0.465111    ,    0.460267    ,    0.458976    ,    0.464811    ,    0.474705    ,    0.479069    ,    0.477896    ,    0.479038    ,    0.480449    ,    0.477783    ,    0.472791    ,    0.464284    ,    0.446734    ,    0.423719    ,    0.409104    ,    0.400867    ,    0.393498    ,    0.38481    ,    0.376302    ,    0.367066    ,    0.356999    ,    0.345324    ,    0.334067    ,    0.323056    ,    0.312126    ,    0.301769    ,    0.293364    ,    0.286881    ,    0.280774    ,    0.275346    ,    0.270642    ,    0.2673    ,    0.265391    ,    0.264659    ,    0.263973    ,    0.258495    ,    0.250914    ,    0.257266    ,    0.27806    ,    0.284029    ,    0.270569    ,    0.235025    ,    0.232759    ,    0.271458    ,    0.315682    ,    0.347711    ,    0.363539    ,    0.377509    ,    0.391535    ,    0.406667    ,    0.424875    ,    0.445479    ,    0.46866    ,    0.493964    ,    0.51916    ,    0.53938    ,    0.554806    ,    0.569817    ,    0.580664    ,    0.584604    ,    0.584683    ,    0.586044    ,    0.584689    ,    0.588153    ,    0.599218    ,    0.612138    ,    0.619744    ,    0.623116    ,    0.630526    ,    0.642866    ,    0.650692    ,    0.652512    ,    0.648582    ,    0.645428    ,    0.641026    ,    0.635038    ,    0.632385    ,    0.631933    ,    0.628235    ,    0.622234    ,    0.616601    ,    0.626012    ,    0.664707    ,    0.702965    ,    0.719614    ,    0.730955    ,    0.74724    ,    0.75504    ,    0.73483    ,    0.699991    ,    0.661902    ,    0.607731    ,    0.551276    ,    0.50317    ,    0.480162    ,    0.474483    ,    0.477128    ,    0.479933    ,    0.489076    ,    0.497045    ,    0.500268    ,    0.504749    ,    0.507545    ,    0.509581    ,    0.51192    ,    0.512712    ,    0.510262    ,    0.509252    ,    0.509331    ,    0.503939    ,    0.499824    ,    0.497833    ,    0.500217    ,    0.507677    ,    0.497935    ,    0.477709    ,    0.461311    ,    0.454651    ,    0.454741    ,    0.45236    ,    0.449643    ,    0.449963    ,    0.454146    ,    0.458015    ,    0.462184    },
    {    0.313347    ,    0.315016    ,    0.316553    ,    0.318442    ,    0.322067    ,    0.326955    ,    0.337297    ,    0.354347    ,    0.369832    ,    0.380234    ,    0.386734    ,    0.376345    ,    0.3645    ,    0.353556    ,    0.350414    ,    0.349967    ,    0.345282    ,    0.33411    ,    0.323007    ,    0.315671    ,    0.30936    ,    0.303711    ,    0.297451    ,    0.289667    ,    0.280682    ,    0.271628    ,    0.261424    ,    0.251153    ,    0.242691    ,    0.234342    ,    0.226384    ,    0.219155    ,    0.212723    ,    0.20701    ,    0.203361    ,    0.200565    ,    0.197664    ,    0.196056    ,    0.195407    ,    0.194501    ,    0.194508    ,    0.20027    ,    0.207787    ,    0.221847    ,    0.230567    ,    0.239158    ,    0.25688    ,    0.276028    ,    0.295409    ,    0.304691    ,    0.316369    ,    0.327778    ,    0.33607    ,    0.346908    ,    0.362667    ,    0.383984    ,    0.40658    ,    0.427655    ,    0.447344    ,    0.468457    ,    0.491876    ,    0.509456    ,    0.518987    ,    0.527059    ,    0.533541    ,    0.536193    ,    0.540795    ,    0.546131    ,    0.554947    ,    0.56796    ,    0.573401    ,    0.568893    ,    0.563553    ,    0.552341    ,    0.537005    ,    0.5235    ,    0.51811    ,    0.509438    ,    0.495126    ,    0.479704    ,    0.474319    ,    0.471245    ,    0.464704    ,    0.458794    ,    0.459826    ,    0.464677    ,    0.472931    ,    0.483166    ,    0.504565    ,    0.555824    ,    0.589283    ,    0.585456    ,    0.588771    ,    0.585326    ,    0.565814    ,    0.536318    ,    0.520549    ,    0.502954    ,    0.495205    ,    0.500396    ,    0.510874    ,    0.512349    ,    0.503188    ,    0.48982    ,    0.472655    ,    0.451213    ,    0.426408    ,    0.40334    ,    0.380883    ,    0.361218    ,    0.345433    ,    0.331808    ,    0.32255    ,    0.319352    ,    0.318913    ,    0.321398    ,    0.334279    ,    0.338489    ,    0.331185    ,    0.315664    ,    0.301448    ,    0.293427    ,    0.286944    ,    0.286394    ,    0.286668    ,    0.293247    ,    0.29845    ,    0.304715    },
    {    0.22875    ,    0.232527    ,    0.234898    ,    0.239206    ,    0.245621    ,    0.253739    ,    0.266657    ,    0.28732    ,    0.303379    ,    0.29727    ,    0.292124    ,    0.300746    ,    0.306787    ,    0.298651    ,    0.297577    ,    0.303869    ,    0.308714    ,    0.297311    ,    0.274667    ,    0.259094    ,    0.252986    ,    0.251222    ,    0.251203    ,    0.247351    ,    0.239681    ,    0.231025    ,    0.222458    ,    0.214022    ,    0.205899    ,    0.197522    ,    0.189978    ,    0.183276    ,    0.178042    ,    0.173838    ,    0.171137    ,    0.168232    ,    0.166001    ,    0.165497    ,    0.161281    ,    0.157101    ,    0.15776    ,    0.16084    ,    0.156336    ,    0.153688    ,    0.155814    ,    0.163878    ,    0.18012    ,    0.198359    ,    0.21813    ,    0.228479    ,    0.235137    ,    0.244512    ,    0.249901    ,    0.259886    ,    0.27671    ,    0.298996    ,    0.320081    ,    0.334684    ,    0.347562    ,    0.358845    ,    0.37115    ,    0.38519    ,    0.400155    ,    0.415371    ,    0.428001    ,    0.433757    ,    0.436706    ,    0.442621    ,    0.453972    ,    0.456027    ,    0.442168    ,    0.424265    ,    0.403918    ,    0.377403    ,    0.353087    ,    0.341648    ,    0.335411    ,    0.330967    ,    0.32852    ,    0.324554    ,    0.320619    ,    0.314618    ,    0.30807    ,    0.301738    ,    0.295963    ,    0.290777    ,    0.289981    ,    0.294172    ,    0.304656    ,    0.319165    ,    0.339702    ,    0.365744    ,    0.390323    ,    0.406489    ,    0.411793    ,    0.399652    ,    0.406945    ,    0.397102    ,    0.38742    ,    0.389379    ,    0.390461    ,    0.376501    ,    0.354274    ,    0.333    ,    0.31348    ,    0.298696    ,    0.282319    ,    0.267757    ,    0.255262    ,    0.246684    ,    0.242808    ,    0.240609    ,    0.239436    ,    0.236622    ,    0.235735    ,    0.237978    ,    0.23839    ,    0.2231    ,    0.218907    ,    0.21438    ,    0.221145    ,    0.220343    ,    0.209173    ,    0.199122    ,    0.197059    ,    0.205935    ,    0.213634    ,    0.219719    },
    {    0.183208    ,    0.184001    ,    0.185612    ,    0.191497    ,    0.198162    ,    0.207725    ,    0.225569    ,    0.246675    ,    0.251304    ,    0.236545    ,    0.227763    ,    0.234101    ,    0.244371    ,    0.24896    ,    0.255323    ,    0.259445    ,    0.256782    ,    0.242809    ,    0.222281    ,    0.209119    ,    0.206816    ,    0.212125    ,    0.21533    ,    0.211192    ,    0.20173    ,    0.192433    ,    0.185356    ,    0.179707    ,    0.174729    ,    0.169093    ,    0.163702    ,    0.160293    ,    0.159325    ,    0.159351    ,    0.158749    ,    0.15858    ,    0.15646    ,    0.154537    ,    0.150565    ,    0.150055    ,    0.141833    ,    0.14609    ,    0.147423    ,    0.143933    ,    0.141705    ,    0.140455    ,    0.143581    ,    0.158401    ,    0.176109    ,    0.181318    ,    0.18452    ,    0.189412    ,    0.19382    ,    0.206987    ,    0.228674    ,    0.240277    ,    0.238904    ,    0.238827    ,    0.244057    ,    0.252323    ,    0.264476    ,    0.276963    ,    0.287975    ,    0.296168    ,    0.300628    ,    0.30376    ,    0.309359    ,    0.312695    ,    0.323229    ,    0.328805    ,    0.326035    ,    0.317343    ,    0.303732    ,    0.286369    ,    0.267289    ,    0.255658    ,    0.251489    ,    0.251949    ,    0.257059    ,    0.260527    ,    0.260141    ,    0.253283    ,    0.246616    ,    0.242306    ,    0.236656    ,    0.230403    ,    0.224372    ,    0.219663    ,    0.215601    ,    0.210161    ,    0.20224    ,    0.204325    ,    0.227788    ,    0.25424    ,    0.264213    ,    0.255816    ,    0.285716    ,    0.287818    ,    0.279568    ,    0.295681    ,    0.288039    ,    0.278202    ,    0.275801    ,    0.269631    ,    0.260758    ,    0.24638    ,    0.232796    ,    0.223759    ,    0.215446    ,    0.208728    ,    0.205139    ,    0.203177    ,    0.202347    ,    0.202716    ,    0.206519    ,    0.209888    ,    0.210634    ,    0.190088    ,    0.182204    ,    0.182526    ,    0.183197    ,    0.178052    ,    0.162012    ,    0.158363    ,    0.157711    ,    0.15799    ,    0.164315    ,    0.175357    },
    {    0.175392    ,    0.176386    ,    0.175325    ,    0.178445    ,    0.182523    ,    0.186812    ,    0.201138    ,    0.220071    ,    0.215394    ,    0.200176    ,    0.187866    ,    0.188982    ,    0.192735    ,    0.19699    ,    0.197675    ,    0.195115    ,    0.19064    ,    0.182546    ,    0.170551    ,    0.164601    ,    0.169421    ,    0.177955    ,    0.18305    ,    0.178889    ,    0.166207    ,    0.155835    ,    0.152439    ,    0.151984    ,    0.151856    ,    0.151652    ,    0.152341    ,    0.154385    ,    0.157503    ,    0.159633    ,    0.15915    ,    0.158341    ,    0.156013    ,    0.152202    ,    0.152902    ,    0.140505    ,    0.134905    ,    0.139809    ,    0.139201    ,    0.13873    ,    0.135772    ,    0.131691    ,    0.132112    ,    0.142968    ,    0.151535    ,    0.156061    ,    0.157549    ,    0.151798    ,    0.152755    ,    0.160387    ,    0.175477    ,    0.190864    ,    0.196603    ,    0.194268    ,    0.199039    ,    0.214635    ,    0.236807    ,    0.236718    ,    0.234664    ,    0.236456    ,    0.239217    ,    0.249474    ,    0.25945    ,    0.261705    ,    0.275459    ,    0.272496    ,    0.260577    ,    0.254051    ,    0.250583    ,    0.255297    ,    0.245766    ,    0.2243    ,    0.213287    ,    0.211356    ,    0.217012    ,    0.216231    ,    0.207381    ,    0.201806    ,    0.2017    ,    0.201132    ,    0.198186    ,    0.194107    ,    0.185755    ,    0.175543    ,    0.169701    ,    0.167324    ,    0.160606    ,    0.15192    ,    0.141638    ,    0.201943    ,    0.252301    ,    0.219528    ,    0.210828    ,    0.204462    ,    0.20025    ,    0.212141    ,    0.210648    ,    0.205344    ,    0.209947    ,    0.211516    ,    0.20793    ,    0.197883    ,    0.187953    ,    0.184028    ,    0.181118    ,    0.178489    ,    0.18189    ,    0.188494    ,    0.194448    ,    0.197976    ,    0.191313    ,    0.169783    ,    0.163824    ,    0.154214    ,    0.152484    ,    0.150378    ,    0.151619    ,    0.148326    ,    0.145984    ,    0.1529    ,    0.152495    ,    0.153823    ,    0.154901    ,    0.163357    },
    {    0.186888    ,    0.169479    ,    0.154408    ,    0.150411    ,    0.154578    ,    0.158393    ,    0.169079    ,    0.170486    ,    0.160298    ,    0.154633    ,    0.153246    ,    0.1521    ,    0.152146    ,    0.152049    ,    0.148733    ,    0.145954    ,    0.14366    ,    0.140848    ,    0.138418    ,    0.138369    ,    0.142713    ,    0.147525    ,    0.149992    ,    0.147974    ,    0.141496    ,    0.137105    ,    0.137671    ,    0.139997    ,    0.142559    ,    0.145659    ,    0.150417    ,    0.153719    ,    0.158749    ,    0.16682    ,    0.169283    ,    0.160759    ,    0.157847    ,    0.155363    ,    0.209563    ,    0.160619    ,    0.133996    ,    0.126375    ,    0.12919    ,    0.128249    ,    0.127996    ,    0.127257    ,    0.135402    ,    0.135656    ,    0.136169    ,    0.145631    ,    0.150313    ,    0.148452    ,    0.147919    ,    0.150422    ,    0.157283    ,    0.171779    ,    0.182455    ,    0.184255    ,    0.194449    ,    0.217979    ,    0.243413    ,    0.225611    ,    0.212277    ,    0.21123    ,    0.207962    ,    0.216798    ,    0.231465    ,    0.237323    ,    0.214049    ,    0.203466    ,    0.201843    ,    0.209636    ,    0.218317    ,    0.23039    ,    0.225211    ,    0.204739    ,    0.191002    ,    0.187509    ,    0.185357    ,    0.18309    ,    0.177578    ,    0.177705    ,    0.183287    ,    0.186678    ,    0.183387    ,    0.176149    ,    0.164354    ,    0.157066    ,    0.156692    ,    0.15216    ,    0.148851    ,    0.143054    ,    0.128091    ,    0.124537    ,    0.139318    ,    0.144575    ,    0.137976    ,    0.135415    ,    0.139431    ,    0.145537    ,    0.151102    ,    0.152913    ,    0.153173    ,    0.151366    ,    0.14823    ,    0.150368    ,    0.157776    ,    0.163893    ,    0.167864    ,    0.168764    ,    0.173963    ,    0.185694    ,    0.19955    ,    0.193459    ,    0.165379    ,    0.148034    ,    0.139717    ,    0.140592    ,    0.142427    ,    0.143155    ,    0.142128    ,    0.142927    ,    0.149353    ,    0.150025    ,    0.147224    ,    0.155913    ,    0.164537    ,    0.17964    },
    {    0.160423    ,    0.169011    ,    0.164822    ,    0.163144    ,    0.159101    ,    0.157242    ,    0.153506    ,    0.145453    ,    0.14008    ,    0.13525    ,    0.130726    ,    0.127825    ,    0.12964    ,    0.131803    ,    0.134005    ,    0.134877    ,    0.133259    ,    0.131007    ,    0.130246    ,    0.130186    ,    0.130913    ,    0.132088    ,    0.132093    ,    0.130093    ,    0.128689    ,    0.127682    ,    0.127991    ,    0.130896    ,    0.13477    ,    0.139245    ,    0.143582    ,    0.149184    ,    0.161141    ,    0.174717    ,    0.169575    ,    0.151499    ,    0.147551    ,    0.14786    ,    0.165198    ,    0.145118    ,    0.114409    ,    0.114068    ,    0.116475    ,    0.118929    ,    0.164139    ,    0.146847    ,    0.152761    ,    0.168649    ,    0.152094    ,    0.156487    ,    0.148107    ,    0.146809    ,    0.143004    ,    0.14139    ,    0.14308    ,    0.14895    ,    0.158467    ,    0.166139    ,    0.193861    ,    0.226891    ,    0.222943    ,    0.204211    ,    0.181178    ,    0.168611    ,    0.165909    ,    0.176692    ,    0.186686    ,    0.17917    ,    0.166635    ,    0.173312    ,    0.181948    ,    0.181475    ,    0.182996    ,    0.180857    ,    0.167955    ,    0.157207    ,    0.153082    ,    0.1581    ,    0.167749    ,    0.177541    ,    0.181662    ,    0.180617    ,    0.176955    ,    0.17612    ,    0.169956    ,    0.169629    ,    0.166602    ,    0.157098    ,    0.149942    ,    0.146456    ,    0.144152    ,    0.133606    ,    0.128795    ,    0.124851    ,    0.124311    ,    0.121535    ,    0.117533    ,    0.117281    ,    0.116812    ,    0.112766    ,    0.116018    ,    0.120428    ,    0.12778    ,    0.134982    ,    0.139814    ,    0.146015    ,    0.152877    ,    0.158033    ,    0.162063    ,    0.165361    ,    0.166274    ,    0.166509    ,    0.166185    ,    0.162258    ,    0.153708    ,    0.146246    ,    0.143775    ,    0.136681    ,    0.132886    ,    0.138268    ,    0.136124    ,    0.131469    ,    0.141542    ,    0.146868    ,    0.149156    ,    0.152695    ,    0.175832    ,    0.174465    },
    {    0.187672    ,    0.186931    ,    0.177296    ,    0.179355    ,    0.166005    ,    0.14925    ,    0.138757    ,    0.134542    ,    0.134527    ,    0.134655    ,    0.130806    ,    0.128367    ,    0.129644    ,    0.130682    ,    0.131114    ,    0.131066    ,    0.129233    ,    0.126711    ,    0.124297    ,    0.12159    ,    0.120549    ,    0.12051    ,    0.11909    ,    0.118323    ,    0.117673    ,    0.115054    ,    0.115192    ,    0.117755    ,    0.120552    ,    0.123536    ,    0.128412    ,    0.136065    ,    0.147691    ,    0.156264    ,    0.149009    ,    0.132796    ,    0.128723    ,    0.137184    ,    0.137752    ,    0.134492    ,    0.104005    ,    0.0967925    ,    0.115558    ,    0.199192    ,    0.223454    ,    0.180542    ,    0.149257    ,    0.141213    ,    0.135267    ,    0.139323    ,    0.131227    ,    0.133059    ,    0.123634    ,    0.124963    ,    0.13018    ,    0.135215    ,    0.143999    ,    0.161779    ,    0.196873    ,    0.206814    ,    0.170094    ,    0.15971    ,    0.152981    ,    0.152761    ,    0.154252    ,    0.152413    ,    0.158216    ,    0.161946    ,    0.161902    ,    0.163219    ,    0.15517    ,    0.146173    ,    0.154287    ,    0.158934    ,    0.151926    ,    0.148346    ,    0.15827    ,    0.165079    ,    0.177983    ,    0.187895    ,    0.189292    ,    0.176835    ,    0.163329    ,    0.158017    ,    0.170982    ,    0.178673    ,    0.177472    ,    0.179728    ,    0.177471    ,    0.168062    ,    0.154449    ,    0.146474    ,    0.145993    ,    0.143486    ,    0.13898    ,    0.134931    ,    0.132764    ,    0.131253    ,    0.127142    ,    0.127337    ,    0.133255    ,    0.13716    ,    0.140655    ,    0.143957    ,    0.145993    ,    0.14826    ,    0.150339    ,    0.153045    ,    0.155434    ,    0.153636    ,    0.150867    ,    0.148625    ,    0.145776    ,    0.145524    ,    0.147642    ,    0.147545    ,    0.139159    ,    0.189057    ,    0.180978    ,    0.150542    ,    0.136388    ,    0.130326    ,    0.137483    ,    0.148772    ,    0.15723    ,    0.154114    ,    0.148476    ,    0.143556    },
    {    0.125369    ,    0.113628    ,    0.104268    ,    0.104364    ,    0.113574    ,    0.116982    ,    0.117342    ,    0.120043    ,    0.123073    ,    0.126652    ,    0.127237    ,    0.124975    ,    0.124299    ,    0.123661    ,    0.122242    ,    0.121672    ,    0.120281    ,    0.117864    ,    0.115336    ,    0.112104    ,    0.109551    ,    0.107886    ,    0.106475    ,    0.106582    ,    0.105633    ,    0.103659    ,    0.104219    ,    0.105621    ,    0.10797    ,    0.10819    ,    0.109086    ,    0.11485    ,    0.121682    ,    0.12357    ,    0.119909    ,    0.113996    ,    0.115696    ,    0.132634    ,    0.13399    ,    0.111898    ,    0.0922053    ,    0.0857528    ,    0.102105    ,    0.135565    ,    0.157313    ,    0.193114    ,    0.181374    ,    0.146258    ,    0.13217    ,    0.129746    ,    0.125827    ,    0.127878    ,    0.131921    ,    0.13475    ,    0.131596    ,    0.128551    ,    0.129355    ,    0.13925    ,    0.154697    ,    0.153319    ,    0.143085    ,    0.134363    ,    0.133082    ,    0.14149    ,    0.14887    ,    0.16303    ,    0.189703    ,    0.222981    ,    0.235839    ,    0.224974    ,    0.211321    ,    0.221535    ,    0.229286    ,    0.224199    ,    0.205035    ,    0.203566    ,    0.189149    ,    0.214076    ,    0.226597    ,    0.227609    ,    0.222671    ,    0.193315    ,    0.181142    ,    0.173471    ,    0.172095    ,    0.176781    ,    0.177194    ,    0.17664    ,    0.171557    ,    0.164503    ,    0.160096    ,    0.160271    ,    0.160757    ,    0.1518    ,    0.141056    ,    0.135684    ,    0.132313    ,    0.13044    ,    0.128873    ,    0.128933    ,    0.128167    ,    0.128398    ,    0.128893    ,    0.130452    ,    0.132882    ,    0.134341    ,    0.136415    ,    0.136807    ,    0.136258    ,    0.13481    ,    0.133298    ,    0.128424    ,    0.121857    ,    0.121958    ,    0.132291    ,    0.129256    ,    0.117625    ,    0.164956    ,    0.201558    ,    0.17537    ,    0.147804    ,    0.145542    ,    0.14326    ,    0.146839    ,    0.156308    ,    0.157259    ,    0.145431    ,    0.131174    },
    {    0.112261    ,    0.107031    ,    0.0980168    ,    0.0928816    ,    0.0949441    ,    0.101438    ,    0.105169    ,    0.108499    ,    0.110376    ,    0.110689    ,    0.112267    ,    0.111139    ,    0.111421    ,    0.112729    ,    0.112799    ,    0.11329    ,    0.112395    ,    0.111258    ,    0.109432    ,    0.107851    ,    0.105597    ,    0.103755    ,    0.102282    ,    0.101732    ,    0.100849    ,    0.0992941    ,    0.0987978    ,    0.0975991    ,    0.095918    ,    0.094251    ,    0.094607    ,    0.10085    ,    0.104835    ,    0.0989156    ,    0.1002    ,    0.103207    ,    0.111228    ,    0.131815    ,    0.124442    ,    0.14883    ,    0.107345    ,    0.114913    ,    0.122127    ,    0.12894    ,    0.229889    ,    0.250214    ,    0.165504    ,    0.151775    ,    0.147832    ,    0.142495    ,    0.137137    ,    0.135412    ,    0.129951    ,    0.122482    ,    0.114578    ,    0.111111    ,    0.11089    ,    0.1188    ,    0.132097    ,    0.137844    ,    0.13804    ,    0.1299    ,    0.134087    ,    0.14105    ,    0.150628    ,    0.171692    ,    0.195666    ,    0.219486    ,    0.231332    ,    0.225946    ,    0.197148    ,    0.176326    ,    0.177243    ,    0.186733    ,    0.198292    ,    0.195    ,    0.176867    ,    0.174511    ,    0.17944    ,    0.179246    ,    0.176922    ,    0.16716    ,    0.157966    ,    0.1545    ,    0.153938    ,    0.152423    ,    0.154677    ,    0.152413    ,    0.146873    ,    0.143667    ,    0.144748    ,    0.14716    ,    0.145846    ,    0.133189    ,    0.123111    ,    0.118358    ,    0.11613    ,    0.112504    ,    0.107626    ,    0.105231    ,    0.10484    ,    0.107769    ,    0.112579    ,    0.118266    ,    0.124529    ,    0.121682    ,    0.122676    ,    0.123556    ,    0.125009    ,    0.122617    ,    0.118603    ,    0.11389    ,    0.113475    ,    0.111238    ,    0.108662    ,    0.11744    ,    0.121484    ,    0.13149    ,    0.196233    ,    0.191513    ,    0.155665    ,    0.150783    ,    0.148129    ,    0.147461    ,    0.152364    ,    0.156917    ,    0.14311    ,    0.127093    },
    {    0.0964448    ,    0.0928955    ,    0.0849407    ,    0.0833443    ,    0.0871942    ,    0.0928163    ,    0.0972821    ,    0.10067    ,    0.102168    ,    0.101956    ,    0.105661    ,    0.104757    ,    0.104551    ,    0.105812    ,    0.106831    ,    0.108092    ,    0.107917    ,    0.10803    ,    0.108036    ,    0.107192    ,    0.106443    ,    0.105838    ,    0.104472    ,    0.103767    ,    0.102768    ,    0.100278    ,    0.0983824    ,    0.0953631    ,    0.0916755    ,    0.0888846    ,    0.0869567    ,    0.0904235    ,    0.089849    ,    0.0860402    ,    0.0977111    ,    0.0977369    ,    0.101368    ,    0.117119    ,    0.0988704    ,    0.147908    ,    0.126126    ,    0.142871    ,    0.145589    ,    0.141721    ,    0.149853    ,    0.157906    ,    0.114283    ,    0.116924    ,    0.110412    ,    0.110965    ,    0.108488    ,    0.100818    ,    0.095361    ,    0.0934393    ,    0.0961011    ,    0.102854    ,    0.114368    ,    0.126772    ,    0.13432    ,    0.136192    ,    0.136079    ,    0.1363    ,    0.134147    ,    0.133437    ,    0.141192    ,    0.163247    ,    0.178142    ,    0.180439    ,    0.18181    ,    0.169679    ,    0.143108    ,    0.124567    ,    0.133716    ,    0.119618    ,    0.109518    ,    0.115923    ,    0.121971    ,    0.123212    ,    0.123327    ,    0.123088    ,    0.125999    ,    0.126463    ,    0.124401    ,    0.122222    ,    0.121802    ,    0.120458    ,    0.117073    ,    0.113413    ,    0.109395    ,    0.107499    ,    0.105608    ,    0.105249    ,    0.106242    ,    0.104675    ,    0.104353    ,    0.103495    ,    0.101221    ,    0.0988569    ,    0.0964127    ,    0.0984238    ,    0.101377    ,    0.102991    ,    0.106989    ,    0.118371    ,    0.11493    ,    0.11373    ,    0.116728    ,    0.107395    ,    0.105208    ,    0.10093    ,    0.0973646    ,    0.0930902    ,    0.0985702    ,    0.109158    ,    0.119939    ,    0.140018    ,    0.152195    ,    0.121371    ,    0.133072    ,    0.142124    ,    0.129012    ,    0.123042    ,    0.122931    ,    0.129029    ,    0.137838    ,    0.14284    ,    0.131047    ,    0.118183    },
    {    0.0757816    ,    0.0745564    ,    0.0765749    ,    0.0794815    ,    0.0842825    ,    0.0908658    ,    0.0971383    ,    0.100623    ,    0.102727    ,    0.105463    ,    0.109184    ,    0.108294    ,    0.107855    ,    0.106879    ,    0.106659    ,    0.108356    ,    0.109702    ,    0.111008    ,    0.110968    ,    0.110423    ,    0.110493    ,    0.109124    ,    0.107905    ,    0.107574    ,    0.105875    ,    0.103259    ,    0.101772    ,    0.0979484    ,    0.0952614    ,    0.0915937    ,    0.087209    ,    0.0840265    ,    0.082691    ,    0.0879015    ,    0.0924067    ,    0.0903325    ,    0.100346    ,    0.112384    ,    0.100534    ,    0.155093    ,    0.17709    ,    0.185686    ,    0.180554    ,    0.16714    ,    0.177022    ,    0.152688    ,    0.105532    ,    0.0983182    ,    0.09263    ,    0.097396    ,    0.10313    ,    0.101194    ,    0.101929    ,    0.105299    ,    0.111815    ,    0.117365    ,    0.12275    ,    0.127966    ,    0.134681    ,    0.14091    ,    0.145433    ,    0.140722    ,    0.133914    ,    0.133453    ,    0.139415    ,    0.150215    ,    0.151252    ,    0.149184    ,    0.146296    ,    0.135994    ,    0.1526    ,    0.144173    ,    0.149597    ,    0.129698    ,    0.118449    ,    0.0923621    ,    0.0862724    ,    0.103342    ,    0.109124    ,    0.106605    ,    0.111684    ,    0.123345    ,    0.120993    ,    0.108298    ,    0.10249    ,    0.0988246    ,    0.0968446    ,    0.0999523    ,    0.102809    ,    0.105561    ,    0.10542    ,    0.106902    ,    0.110514    ,    0.114431    ,    0.11362    ,    0.106719    ,    0.099199    ,    0.0974675    ,    0.0985614    ,    0.105363    ,    0.106024    ,    0.104049    ,    0.104486    ,    0.112836    ,    0.108622    ,    0.12799    ,    0.125425    ,    0.113123    ,    0.101593    ,    0.0885449    ,    0.0823373    ,    0.0880016    ,    0.121098    ,    0.13013    ,    0.139936    ,    0.167245    ,    0.206041    ,    0.155265    ,    0.118384    ,    0.0950336    ,    0.0860767    ,    0.0894072    ,    0.0880124    ,    0.0895579    ,    0.0931933    ,    0.0945417    ,    0.0891305    ,    0.0794871    },
    {    0.0778737    ,    0.0841414    ,    0.0875107    ,    0.0910086    ,    0.097344    ,    0.104039    ,    0.109095    ,    0.111979    ,    0.113936    ,    0.11559    ,    0.114509    ,    0.112268    ,    0.111097    ,    0.110159    ,    0.110395    ,    0.111309    ,    0.111178    ,    0.110902    ,    0.109775    ,    0.108134    ,    0.108276    ,    0.107935    ,    0.107655    ,    0.107333    ,    0.106909    ,    0.105976    ,    0.103771    ,    0.0999172    ,    0.0954512    ,    0.0889331    ,    0.0838389    ,    0.082724    ,    0.0840472    ,    0.0838642    ,    0.0843308    ,    0.0905906    ,    0.0983836    ,    0.099475    ,    0.0903473    ,    0.151827    ,    0.212461    ,    0.2248    ,    0.227149    ,    0.242968    ,    0.230992    ,    0.19663    ,    0.141237    ,    0.099416    ,    0.0969261    ,    0.102047    ,    0.103614    ,    0.102323    ,    0.10152    ,    0.106045    ,    0.111887    ,    0.118649    ,    0.126217    ,    0.133328    ,    0.139453    ,    0.142776    ,    0.141894    ,    0.137219    ,    0.132756    ,    0.132568    ,    0.134383    ,    0.131076    ,    0.126937    ,    0.127644    ,    0.130325    ,    0.13465    ,    0.13991    ,    0.136149    ,    0.185822    ,    0.179492    ,    0.145833    ,    0.122993    ,    0.093669    ,    0.100698    ,    0.10935    ,    0.110412    ,    0.114806    ,    0.120536    ,    0.126991    ,    0.119583    ,    0.115478    ,    0.11316    ,    0.114279    ,    0.115731    ,    0.115412    ,    0.115913    ,    0.115239    ,    0.115486    ,    0.115528    ,    0.11172    ,    0.106369    ,    0.100201    ,    0.0955631    ,    0.0952782    ,    0.0988361    ,    0.100145    ,    0.0957831    ,    0.0959819    ,    0.0993086    ,    0.0984837    ,    0.101284    ,    0.122661    ,    0.174323    ,    0.196827    ,    0.121659    ,    0.112222    ,    0.111382    ,    0.181909    ,    0.153868    ,    0.179647    ,    0.179655    ,    0.213039    ,    0.257171    ,    0.245935    ,    0.131012    ,    0.0829989    ,    0.0726188    ,    0.0732687    ,    0.0771748    ,    0.0770501    ,    0.0752627    ,    0.0752433    ,    0.0730179    ,    0.0737094    },
    {    0.0882628    ,    0.0957609    ,    0.098553    ,    0.102905    ,    0.109833    ,    0.115502    ,    0.119469    ,    0.121481    ,    0.122418    ,    0.12082    ,    0.118272    ,    0.115848    ,    0.113568    ,    0.111023    ,    0.109411    ,    0.108622    ,    0.107861    ,    0.106586    ,    0.104874    ,    0.104211    ,    0.105081    ,    0.106063    ,    0.105941    ,    0.104154    ,    0.101869    ,    0.0999747    ,    0.0962332    ,    0.0916959    ,    0.0870424    ,    0.0850327    ,    0.0831391    ,    0.0836794    ,    0.0849982    ,    0.0836982    ,    0.0833816    ,    0.0845579    ,    0.0865972    ,    0.0772594    ,    0.083043    ,    0.165247    ,    0.199989    ,    0.219969    ,    0.283537    ,    0.30991    ,    0.31743    ,    0.218811    ,    0.160401    ,    0.140647    ,    0.093662    ,    0.0914057    ,    0.0930264    ,    0.0962961    ,    0.100346    ,    0.10848    ,    0.117552    ,    0.126091    ,    0.133181    ,    0.136831    ,    0.137289    ,    0.135355    ,    0.133527    ,    0.1332    ,    0.132025    ,    0.129824    ,    0.123326    ,    0.116302    ,    0.116844    ,    0.125194    ,    0.149098    ,    0.149414    ,    0.133578    ,    0.159689    ,    0.196824    ,    0.202694    ,    0.154869    ,    0.146713    ,    0.1236    ,    0.098539    ,    0.108288    ,    0.107706    ,    0.112663    ,    0.100962    ,    0.103019    ,    0.100424    ,    0.0953437    ,    0.0950632    ,    0.0949606    ,    0.0931831    ,    0.0934536    ,    0.0936225    ,    0.0918655    ,    0.0913896    ,    0.0906764    ,    0.0887168    ,    0.0880061    ,    0.0886086    ,    0.0891172    ,    0.0882333    ,    0.0889929    ,    0.086985    ,    0.0876055    ,    0.0885365    ,    0.0922402    ,    0.098051    ,    0.123779    ,    0.174007    ,    0.183924    ,    0.191422    ,    0.160966    ,    0.160073    ,    0.170358    ,    0.178427    ,    0.201034    ,    0.191846    ,    0.232604    ,    0.235392    ,    0.237895    ,    0.233263    ,    0.118483    ,    0.070548    ,    0.0768085    ,    0.0769776    ,    0.075064    ,    0.0746656    ,    0.0723932    ,    0.0708187    ,    0.0713541    ,    0.0778298    },
    {    0.0916333    ,    0.0949488    ,    0.0988314    ,    0.104774    ,    0.109618    ,    0.113529    ,    0.115537    ,    0.115914    ,    0.114865    ,    0.112996    ,    0.112113    ,    0.110942    ,    0.109441    ,    0.108276    ,    0.108226    ,    0.108495    ,    0.108014    ,    0.107175    ,    0.106814    ,    0.105798    ,    0.104509    ,    0.101911    ,    0.0993264    ,    0.0968614    ,    0.0946858    ,    0.0935069    ,    0.0920935    ,    0.0913837    ,    0.0923391    ,    0.093795    ,    0.0927573    ,    0.092002    ,    0.0896325    ,    0.087902    ,    0.0839    ,    0.0771321    ,    0.072274    ,    0.074547    ,    0.083031    ,    0.171916    ,    0.236468    ,    0.208901    ,    0.310776    ,    0.336848    ,    0.329664    ,    0.225681    ,    0.183246    ,    0.143923    ,    0.128344    ,    0.0882101    ,    0.0965787    ,    0.107323    ,    0.115547    ,    0.122708    ,    0.129207    ,    0.132287    ,    0.134665    ,    0.136695    ,    0.13829    ,    0.139286    ,    0.140262    ,    0.136622    ,    0.130001    ,    0.12158    ,    0.114823    ,    0.115552    ,    0.123776    ,    0.139482    ,    0.174598    ,    0.156359    ,    0.209856    ,    0.207865    ,    0.227738    ,    0.206574    ,    0.184786    ,    0.167653    ,    0.15485    ,    0.0969007    ,    0.109597    ,    0.122965    ,    0.137134    ,    0.0976414    ,    0.0770772    ,    0.0759221    ,    0.0704025    ,    0.0708215    ,    0.0720527    ,    0.0737977    ,    0.0762828    ,    0.0780722    ,    0.0784121    ,    0.0779213    ,    0.0803465    ,    0.0842329    ,    0.0869552    ,    0.0882234    ,    0.0875272    ,    0.0837999    ,    0.0828487    ,    0.0846852    ,    0.0868427    ,    0.0886131    ,    0.0960705    ,    0.109894    ,    0.146144    ,    0.194569    ,    0.200973    ,    0.22491    ,    0.200059    ,    0.201199    ,    0.207352    ,    0.217132    ,    0.22453    ,    0.250228    ,    0.24794    ,    0.241386    ,    0.251557    ,    0.180871    ,    0.0851483    ,    0.0765901    ,    0.0794444    ,    0.0772146    ,    0.0774933    ,    0.0786423    ,    0.0779258    ,    0.0785812    ,    0.0839836    ,    0.0895103    },
    {    0.0958056    ,    0.101071    ,    0.105942    ,    0.110323    ,    0.113579    ,    0.115438    ,    0.116601    ,    0.117045    ,    0.117032    ,    0.117025    ,    0.117369    ,    0.116726    ,    0.114826    ,    0.11432    ,    0.114588    ,    0.115504    ,    0.115332    ,    0.113641    ,    0.112085    ,    0.108726    ,    0.106423    ,    0.105889    ,    0.103926    ,    0.103831    ,    0.10328    ,    0.104747    ,    0.106875    ,    0.107427    ,    0.107621    ,    0.105835    ,    0.101472    ,    0.0975673    ,    0.0915657    ,    0.0877772    ,    0.0863733    ,    0.0800382    ,    0.0818213    ,    0.10015    ,    0.0861053    ,    0.192107    ,    0.208311    ,    0.185619    ,    0.324108    ,    0.317905    ,    0.261114    ,    0.211569    ,    0.178857    ,    0.128591    ,    0.12199    ,    0.122387    ,    0.103492    ,    0.119792    ,    0.13211    ,    0.137345    ,    0.14456    ,    0.148186    ,    0.151006    ,    0.151105    ,    0.149575    ,    0.145267    ,    0.141282    ,    0.135465    ,    0.131991    ,    0.128265    ,    0.126709    ,    0.128163    ,    0.134144    ,    0.143512    ,    0.154567    ,    0.133905    ,    0.206943    ,    0.232278    ,    0.222036    ,    0.1953    ,    0.161307    ,    0.1674    ,    0.171066    ,    0.0965341    ,    0.102068    ,    0.12392    ,    0.141495    ,    0.112713    ,    0.0691037    ,    0.0712215    ,    0.0687902    ,    0.0701368    ,    0.0743019    ,    0.0776445    ,    0.0800565    ,    0.08285    ,    0.0850825    ,    0.0873399    ,    0.089448    ,    0.0909949    ,    0.0912449    ,    0.0903745    ,    0.0881632    ,    0.0854546    ,    0.0869635    ,    0.0900717    ,    0.0939273    ,    0.0982694    ,    0.107228    ,    0.119921    ,    0.125867    ,    0.144708    ,    0.163815    ,    0.200609    ,    0.203708    ,    0.208134    ,    0.212039    ,    0.215021    ,    0.244392    ,    0.273451    ,    0.270474    ,    0.232136    ,    0.20231    ,    0.0973295    ,    0.0682808    ,    0.0800544    ,    0.0838779    ,    0.081899    ,    0.0844089    ,    0.0831733    ,    0.0863478    ,    0.0870207    ,    0.0892067    ,    0.0928155    },
    {    0.107713    ,    0.112528    ,    0.115582    ,    0.116629    ,    0.117701    ,    0.119255    ,    0.120042    ,    0.120928    ,    0.121336    ,    0.12194    ,    0.122521    ,    0.122286    ,    0.122263    ,    0.12392    ,    0.123985    ,    0.123725    ,    0.123026    ,    0.12223    ,    0.123182    ,    0.123623    ,    0.120958    ,    0.120412    ,    0.119039    ,    0.119506    ,    0.121535    ,    0.124471    ,    0.126164    ,    0.125227    ,    0.123527    ,    0.119266    ,    0.114456    ,    0.111006    ,    0.10787    ,    0.106927    ,    0.101399    ,    0.0987947    ,    0.104806    ,    0.097814    ,    0.113191    ,    0.164249    ,    0.165725    ,    0.187376    ,    0.249037    ,    0.234855    ,    0.201055    ,    0.15615    ,    0.157239    ,    0.139781    ,    0.104433    ,    0.124654    ,    0.128423    ,    0.128115    ,    0.144218    ,    0.150757    ,    0.15631    ,    0.156874    ,    0.155869    ,    0.152951    ,    0.151901    ,    0.15033    ,    0.150529    ,    0.148745    ,    0.145378    ,    0.141451    ,    0.136484    ,    0.132457    ,    0.133181    ,    0.131097    ,    0.12    ,    0.156477    ,    0.212647    ,    0.223646    ,    0.221519    ,    0.198273    ,    0.173722    ,    0.167775    ,    0.191867    ,    0.140757    ,    0.110316    ,    0.100067    ,    0.111142    ,    0.0887699    ,    0.0720082    ,    0.0747959    ,    0.0761999    ,    0.0792804    ,    0.0830006    ,    0.0845717    ,    0.0848495    ,    0.0868757    ,    0.087523    ,    0.0876573    ,    0.0873041    ,    0.0886288    ,    0.0901578    ,    0.0921783    ,    0.0945784    ,    0.0969387    ,    0.100251    ,    0.103475    ,    0.105258    ,    0.106227    ,    0.112786    ,    0.11896    ,    0.118895    ,    0.113313    ,    0.111147    ,    0.129855    ,    0.179694    ,    0.217111    ,    0.209105    ,    0.195995    ,    0.214249    ,    0.198773    ,    0.204248    ,    0.172863    ,    0.103755    ,    0.0722268    ,    0.0785309    ,    0.0829821    ,    0.0862421    ,    0.0871265    ,    0.0901058    ,    0.0919918    ,    0.0936239    ,    0.0962806    ,    0.0986282    ,    0.0978214    },
    {    0.111088    ,    0.113656    ,    0.115997    ,    0.116425    ,    0.116162    ,    0.116434    ,    0.115962    ,    0.115705    ,    0.117095    ,    0.118022    ,    0.119171    ,    0.119073    ,    0.119941    ,    0.120498    ,    0.121484    ,    0.122801    ,    0.124698    ,    0.128754    ,    0.129046    ,    0.130924    ,    0.130483    ,    0.130864    ,    0.132742    ,    0.1355    ,    0.140409    ,    0.14413    ,    0.145756    ,    0.14482    ,    0.141858    ,    0.136484    ,    0.131831    ,    0.127806    ,    0.127775    ,    0.123025    ,    0.122805    ,    0.127883    ,    0.116538    ,    0.114946    ,    0.169214    ,    0.40657    ,    0.15687    ,    0.208639    ,    0.181863    ,    0.173536    ,    0.16068    ,    0.149159    ,    0.140635    ,    0.136284    ,    0.121321    ,    0.122265    ,    0.134119    ,    0.13504    ,    0.145547    ,    0.151954    ,    0.158985    ,    0.161091    ,    0.162819    ,    0.163703    ,    0.164327    ,    0.163372    ,    0.160834    ,    0.156945    ,    0.15163    ,    0.146228    ,    0.136849    ,    0.128662    ,    0.122411    ,    0.106905    ,    0.103895    ,    0.189775    ,    0.188314    ,    0.194672    ,    0.202441    ,    0.206833    ,    0.183967    ,    0.175811    ,    0.181379    ,    0.199225    ,    0.153762    ,    0.0892917    ,    0.096862    ,    0.106766    ,    0.0833646    ,    0.0794994    ,    0.0836683    ,    0.0839044    ,    0.0867067    ,    0.0870277    ,    0.085898    ,    0.0866277    ,    0.0871161    ,    0.0879434    ,    0.0898906    ,    0.0931152    ,    0.0968902    ,    0.098887    ,    0.10024    ,    0.10136    ,    0.102669    ,    0.102045    ,    0.103304    ,    0.101796    ,    0.100095    ,    0.0987579    ,    0.0969025    ,    0.0960558    ,    0.0988624    ,    0.0982912    ,    0.125944    ,    0.145659    ,    0.158754    ,    0.186232    ,    0.173591    ,    0.10619    ,    0.108549    ,    0.110123    ,    0.0759578    ,    0.0838709    ,    0.0936271    ,    0.0947877    ,    0.0979415    ,    0.101872    ,    0.103387    ,    0.103322    ,    0.102308    ,    0.103342    ,    0.104597    ,    0.106521    },
    {    0.108962    ,    0.111359    ,    0.11397    ,    0.114163    ,    0.115664    ,    0.114465    ,    0.11467    ,    0.113592    ,    0.11376    ,    0.112829    ,    0.111241    ,    0.111795    ,    0.113518    ,    0.110675    ,    0.112092    ,    0.11388    ,    0.114468    ,    0.115278    ,    0.118273    ,    0.121441    ,    0.124714    ,    0.128646    ,    0.132933    ,    0.137231    ,    0.140442    ,    0.142939    ,    0.143954    ,    0.143485    ,    0.13996    ,    0.133532    ,    0.127057    ,    0.120738    ,    0.118006    ,    0.116641    ,    0.132015    ,    0.137697    ,    0.108192    ,    0.139691    ,    0.159157    ,    0.166443    ,    0.209998    ,    0.199904    ,    0.185456    ,    0.18419    ,    0.179142    ,    0.16983    ,    0.177501    ,    0.160101    ,    0.12082    ,    0.13787    ,    0.139814    ,    0.146988    ,    0.1461    ,    0.149437    ,    0.156096    ,    0.159896    ,    0.160976    ,    0.162018    ,    0.161711    ,    0.160427    ,    0.1575    ,    0.153247    ,    0.146858    ,    0.136946    ,    0.124069    ,    0.114934    ,    0.106483    ,    0.100949    ,    0.0953986    ,    0.170895    ,    0.166237    ,    0.175832    ,    0.190391    ,    0.186759    ,    0.184086    ,    0.178663    ,    0.151056    ,    0.127294    ,    0.147629    ,    0.0845728    ,    0.0868421    ,    0.0912121    ,    0.0905644    ,    0.0856387    ,    0.0877397    ,    0.0900991    ,    0.0916393    ,    0.0913243    ,    0.0920824    ,    0.0924344    ,    0.0916972    ,    0.091262    ,    0.0920484    ,    0.0923483    ,    0.0942888    ,    0.093849    ,    0.0933381    ,    0.0929231    ,    0.0919514    ,    0.0934099    ,    0.0913352    ,    0.0889291    ,    0.0880966    ,    0.0883078    ,    0.0867838    ,    0.0855099    ,    0.0847208    ,    0.0835231    ,    0.0832481    ,    0.0865219    ,    0.112152    ,    0.150388    ,    0.143724    ,    0.0890154    ,    0.0924106    ,    0.115165    ,    0.086404    ,    0.0975266    ,    0.104022    ,    0.108528    ,    0.109419    ,    0.109489    ,    0.108368    ,    0.106623    ,    0.105337    ,    0.105047    ,    0.106511    ,    0.108842    },
    {    0.112542    ,    0.11501    ,    0.117128    ,    0.117943    ,    0.119003    ,    0.117363    ,    0.116149    ,    0.112995    ,    0.110939    ,    0.108073    ,    0.105345    ,    0.102136    ,    0.0996276    ,    0.097382    ,    0.0981279    ,    0.0987227    ,    0.100515    ,    0.100931    ,    0.103867    ,    0.10511    ,    0.108649    ,    0.113393    ,    0.117749    ,    0.120247    ,    0.122044    ,    0.124301    ,    0.124359    ,    0.122088    ,    0.118214    ,    0.111094    ,    0.104205    ,    0.100584    ,    0.101715    ,    0.115554    ,    0.146627    ,    0.144292    ,    0.13365    ,    0.160123    ,    0.183067    ,    0.227013    ,    0.209098    ,    0.190227    ,    0.206523    ,    0.209404    ,    0.187811    ,    0.182331    ,    0.188572    ,    0.160436    ,    0.147394    ,    0.120579    ,    0.119606    ,    0.126227    ,    0.1339    ,    0.132079    ,    0.136996    ,    0.14096    ,    0.144125    ,    0.146276    ,    0.146522    ,    0.146937    ,    0.144907    ,    0.141965    ,    0.133168    ,    0.122082    ,    0.111227    ,    0.102201    ,    0.0928898    ,    0.0998656    ,    0.0945939    ,    0.167433    ,    0.182507    ,    0.156713    ,    0.172224    ,    0.173196    ,    0.178573    ,    0.183566    ,    0.151395    ,    0.127891    ,    0.104374    ,    0.091326    ,    0.0919842    ,    0.0889824    ,    0.0898485    ,    0.0902954    ,    0.0909221    ,    0.0934849    ,    0.0944545    ,    0.0937085    ,    0.0930486    ,    0.094863    ,    0.0927393    ,    0.0917809    ,    0.0958207    ,    0.091298    ,    0.0897876    ,    0.0871221    ,    0.0859013    ,    0.0867589    ,    0.0879502    ,    0.0865295    ,    0.0881678    ,    0.0945977    ,    0.0998602    ,    0.0929957    ,    0.089729    ,    0.0870685    ,    0.0880409    ,    0.0872265    ,    0.0814009    ,    0.0775626    ,    0.0798009    ,    0.0814447    ,    0.0789676    ,    0.0830202    ,    0.0831534    ,    0.0842537    ,    0.0986438    ,    0.103713    ,    0.104583    ,    0.112597    ,    0.112327    ,    0.111556    ,    0.110953    ,    0.110852    ,    0.11076    ,    0.110054    ,    0.111648    ,    0.112023    },
    {    0.136136    ,    0.136603    ,    0.135592    ,    0.134446    ,    0.133489    ,    0.131366    ,    0.128155    ,    0.12436    ,    0.121191    ,    0.11657    ,    0.113299    ,    0.108449    ,    0.104985    ,    0.106995    ,    0.104289    ,    0.101163    ,    0.0978522    ,    0.0967546    ,    0.0993052    ,    0.101684    ,    0.102841    ,    0.105298    ,    0.108212    ,    0.107762    ,    0.107972    ,    0.108223    ,    0.107824    ,    0.105879    ,    0.104226    ,    0.102735    ,    0.101292    ,    0.102789    ,    0.109274    ,    0.128588    ,    0.158122    ,    0.152805    ,    0.16402    ,    0.226585    ,    0.240607    ,    0.217772    ,    0.220535    ,    0.231007    ,    0.23466    ,    0.21816    ,    0.22692    ,    0.204721    ,    0.188495    ,    0.158765    ,    0.165728    ,    0.152802    ,    0.145641    ,    0.151923    ,    0.120344    ,    0.118238    ,    0.122279    ,    0.124846    ,    0.128808    ,    0.129742    ,    0.130778    ,    0.131453    ,    0.130545    ,    0.126874    ,    0.121053    ,    0.111886    ,    0.100544    ,    0.0932068    ,    0.0910911    ,    0.0949754    ,    0.10877    ,    0.150328    ,    0.149339    ,    0.145074    ,    0.154878    ,    0.166611    ,    0.159717    ,    0.168059    ,    0.136091    ,    0.119269    ,    0.0999738    ,    0.0933925    ,    0.0929555    ,    0.0936371    ,    0.0951222    ,    0.0930843    ,    0.0952479    ,    0.0942106    ,    0.0984257    ,    0.0993532    ,    0.0993813    ,    0.0994294    ,    0.0980916    ,    0.0951248    ,    0.0959505    ,    0.0928756    ,    0.0892089    ,    0.0847533    ,    0.0844041    ,    0.0838137    ,    0.0871124    ,    0.0882802    ,    0.103068    ,    0.109845    ,    0.100318    ,    0.0965729    ,    0.0914902    ,    0.101479    ,    0.0961496    ,    0.0892888    ,    0.0844327    ,    0.0793107    ,    0.0766321    ,    0.0751339    ,    0.0763166    ,    0.0899801    ,    0.100928    ,    0.108757    ,    0.113555    ,    0.12034    ,    0.110367    ,    0.117262    ,    0.121336    ,    0.126433    ,    0.127666    ,    0.130578    ,    0.131764    ,    0.133698    ,    0.136052    ,    0.135297    },
    {    0.162826    ,    0.160163    ,    0.15553    ,    0.152628    ,    0.148745    ,    0.145695    ,    0.142603    ,    0.139682    ,    0.137615    ,    0.134156    ,    0.130527    ,    0.126631    ,    0.123471    ,    0.116971    ,    0.113361    ,    0.110061    ,    0.107717    ,    0.107218    ,    0.10554    ,    0.104958    ,    0.102168    ,    0.103732    ,    0.102535    ,    0.0995893    ,    0.0988796    ,    0.0994148    ,    0.101794    ,    0.103591    ,    0.104767    ,    0.105894    ,    0.109654    ,    0.112309    ,    0.117114    ,    0.129872    ,    0.152615    ,    0.151261    ,    0.166698    ,    0.252111    ,    0.245606    ,    0.238838    ,    0.252091    ,    0.264734    ,    0.272495    ,    0.262378    ,    0.251471    ,    0.231118    ,    0.196084    ,    0.147489    ,    0.132748    ,    0.134053    ,    0.107476    ,    0.107927    ,    0.108667    ,    0.112543    ,    0.11381    ,    0.114961    ,    0.116044    ,    0.11502    ,    0.114719    ,    0.113831    ,    0.113766    ,    0.110521    ,    0.109048    ,    0.103501    ,    0.0966784    ,    0.0934286    ,    0.0924558    ,    0.095184    ,    0.127168    ,    0.143106    ,    0.139697    ,    0.143814    ,    0.141549    ,    0.142746    ,    0.144783    ,    0.174631    ,    0.134959    ,    0.133559    ,    0.0926902    ,    0.0905164    ,    0.0905371    ,    0.0906185    ,    0.0884587    ,    0.0916041    ,    0.0942851    ,    0.0963038    ,    0.0980446    ,    0.0974639    ,    0.0981475    ,    0.0975324    ,    0.09663    ,    0.0963798    ,    0.0964135    ,    0.0950055    ,    0.0938608    ,    0.0917043    ,    0.0919821    ,    0.0906217    ,    0.0916727    ,    0.101854    ,    0.103208    ,    0.0928559    ,    0.10022    ,    0.0878993    ,    0.100123    ,    0.0996331    ,    0.0865071    ,    0.0873201    ,    0.0889681    ,    0.0812773    ,    0.0814019    ,    0.0867891    ,    0.0924801    ,    0.0984504    ,    0.0871615    ,    0.0958549    ,    0.113741    ,    0.119697    ,    0.131284    ,    0.150527    ,    0.158528    ,    0.164798    ,    0.171456    ,    0.175049    ,    0.175503    ,    0.174392    ,    0.170883    ,    0.167292    },
    {    0.184636    ,    0.178333    ,    0.17068    ,    0.165492    ,    0.160566    ,    0.156193    ,    0.152841    ,    0.149186    ,    0.147945    ,    0.145632    ,    0.144088    ,    0.142011    ,    0.139162    ,    0.135925    ,    0.13215    ,    0.128785    ,    0.12543    ,    0.122168    ,    0.116463    ,    0.109789    ,    0.107238    ,    0.103887    ,    0.101826    ,    0.100022    ,    0.100615    ,    0.10316    ,    0.104607    ,    0.105557    ,    0.1109    ,    0.116982    ,    0.121742    ,    0.124056    ,    0.129098    ,    0.13197    ,    0.138212    ,    0.125551    ,    0.124525    ,    0.220255    ,    0.236821    ,    0.231915    ,    0.269756    ,    0.298797    ,    0.31247    ,    0.286268    ,    0.243346    ,    0.170482    ,    0.151224    ,    0.113729    ,    0.107166    ,    0.102772    ,    0.100466    ,    0.0997982    ,    0.102887    ,    0.107562    ,    0.109755    ,    0.110616    ,    0.110348    ,    0.108702    ,    0.107286    ,    0.10494    ,    0.104158    ,    0.100864    ,    0.100458    ,    0.0971662    ,    0.0954164    ,    0.0932892    ,    0.0959338    ,    0.0996448    ,    0.12259    ,    0.130517    ,    0.144531    ,    0.158311    ,    0.140058    ,    0.145906    ,    0.149355    ,    0.145501    ,    0.152518    ,    0.139596    ,    0.131817    ,    0.101325    ,    0.0872265    ,    0.0936867    ,    0.0951115    ,    0.0985596    ,    0.101835    ,    0.103    ,    0.100171    ,    0.0982178    ,    0.0958156    ,    0.095475    ,    0.0934299    ,    0.0919791    ,    0.0931282    ,    0.0931967    ,    0.0934373    ,    0.0915349    ,    0.0908877    ,    0.09259    ,    0.0973461    ,    0.101227    ,    0.0988704    ,    0.106859    ,    0.0867568    ,    0.0942379    ,    0.121562    ,    0.0963318    ,    0.0934021    ,    0.093248    ,    0.0949188    ,    0.0898937    ,    0.0931665    ,    0.0917055    ,    0.110725    ,    0.101445    ,    0.108907    ,    0.127947    ,    0.146466    ,    0.162115    ,    0.184395    ,    0.198259    ,    0.2064    ,    0.211867    ,    0.215782    ,    0.216602    ,    0.215637    ,    0.209439    ,    0.202255    ,    0.194246    },
    {    0.180624    ,    0.172549    ,    0.164522    ,    0.15786    ,    0.152507    ,    0.148594    ,    0.144838    ,    0.141749    ,    0.139239    ,    0.136409    ,    0.133742    ,    0.13059    ,    0.128524    ,    0.123522    ,    0.120933    ,    0.119464    ,    0.116311    ,    0.114037    ,    0.109724    ,    0.104846    ,    0.10337    ,    0.100797    ,    0.0989827    ,    0.0995409    ,    0.100015    ,    0.102178    ,    0.10408    ,    0.107166    ,    0.111385    ,    0.113397    ,    0.117204    ,    0.118734    ,    0.121492    ,    0.111834    ,    0.11209    ,    0.105574    ,    0.118932    ,    0.17353    ,    0.219639    ,    0.249658    ,    0.290278    ,    0.296998    ,    0.266    ,    0.24665    ,    0.23814    ,    0.156998    ,    0.1169    ,    0.0979745    ,    0.104386    ,    0.101872    ,    0.0973356    ,    0.0938267    ,    0.0935298    ,    0.0953295    ,    0.0970447    ,    0.0979236    ,    0.10021    ,    0.100821    ,    0.0985989    ,    0.0999349    ,    0.099472    ,    0.0984291    ,    0.0975199    ,    0.0975556    ,    0.0990779    ,    0.0940247    ,    0.0924546    ,    0.0942855    ,    0.127577    ,    0.146461    ,    0.160158    ,    0.152422    ,    0.144183    ,    0.157202    ,    0.164621    ,    0.204845    ,    0.244354    ,    0.209863    ,    0.1815    ,    0.165985    ,    0.106013    ,    0.0959082    ,    0.100049    ,    0.100109    ,    0.102015    ,    0.105389    ,    0.103583    ,    0.102881    ,    0.0999989    ,    0.100618    ,    0.0977323    ,    0.0936281    ,    0.0926178    ,    0.0910203    ,    0.0913919    ,    0.0926846    ,    0.0924367    ,    0.0947981    ,    0.101299    ,    0.103866    ,    0.123641    ,    0.0899176    ,    0.0887055    ,    0.100421    ,    0.101802    ,    0.0838376    ,    0.0790391    ,    0.0809101    ,    0.0952562    ,    0.0909859    ,    0.0923696    ,    0.100183    ,    0.108898    ,    0.114056    ,    0.122978    ,    0.136812    ,    0.151102    ,    0.170284    ,    0.185572    ,    0.193615    ,    0.2003    ,    0.206006    ,    0.209335    ,    0.211388    ,    0.211209    ,    0.206289    ,    0.198489    ,    0.189305    },
    {    0.149053    ,    0.144245    ,    0.140588    ,    0.136173    ,    0.132032    ,    0.130289    ,    0.128658    ,    0.126709    ,    0.124591    ,    0.122941    ,    0.120571    ,    0.118303    ,    0.115771    ,    0.111253    ,    0.108046    ,    0.104899    ,    0.0999788    ,    0.0968763    ,    0.0931441    ,    0.0917331    ,    0.0907758    ,    0.0892408    ,    0.0899661    ,    0.0890792    ,    0.0901896    ,    0.0920661    ,    0.0925371    ,    0.094657    ,    0.0950828    ,    0.0962057    ,    0.100858    ,    0.0960408    ,    0.0939584    ,    0.0873059    ,    0.0882101    ,    0.0899016    ,    0.0954038    ,    0.136702    ,    0.187546    ,    0.255359    ,    0.282153    ,    0.240746    ,    0.175066    ,    0.182488    ,    0.141281    ,    0.122972    ,    0.0977773    ,    0.100435    ,    0.0992387    ,    0.0939238    ,    0.0918245    ,    0.0898708    ,    0.089458    ,    0.0892293    ,    0.093181    ,    0.0940659    ,    0.0932639    ,    0.0925605    ,    0.0939948    ,    0.0976142    ,    0.0978418    ,    0.107787    ,    0.109436    ,    0.104004    ,    0.11457    ,    0.100105    ,    0.0973978    ,    0.111833    ,    0.131089    ,    0.135565    ,    0.14137    ,    0.168109    ,    0.155811    ,    0.164856    ,    0.141434    ,    0.219447    ,    0.246493    ,    0.235335    ,    0.210505    ,    0.189226    ,    0.1619    ,    0.104413    ,    0.102001    ,    0.105485    ,    0.104625    ,    0.104591    ,    0.103661    ,    0.102169    ,    0.100243    ,    0.0983659    ,    0.0954286    ,    0.0929723    ,    0.0914316    ,    0.0884724    ,    0.0881382    ,    0.0872173    ,    0.0857362    ,    0.0914817    ,    0.0939194    ,    0.107059    ,    0.102638    ,    0.0758049    ,    0.0624245    ,    0.0701128    ,    0.0837893    ,    0.0738507    ,    0.0891007    ,    0.0794674    ,    0.0868989    ,    0.0923244    ,    0.0952065    ,    0.0977456    ,    0.0991511    ,    0.102722    ,    0.107677    ,    0.113455    ,    0.120446    ,    0.127235    ,    0.132162    ,    0.137036    ,    0.141789    ,    0.146312    ,    0.151488    ,    0.155745    ,    0.157737    ,    0.157479    ,    0.15586    ,    0.152519    },
    {    0.127249    ,    0.127097    ,    0.125502    ,    0.124431    ,    0.123854    ,    0.121556    ,    0.121854    ,    0.121688    ,    0.121426    ,    0.119782    ,    0.117784    ,    0.116166    ,    0.112636    ,    0.110531    ,    0.106581    ,    0.104102    ,    0.10084    ,    0.0976098    ,    0.0940598    ,    0.0921479    ,    0.089259    ,    0.0861785    ,    0.0840034    ,    0.0814797    ,    0.0818548    ,    0.0831761    ,    0.0842587    ,    0.0853232    ,    0.0890647    ,    0.0924622    ,    0.0902863    ,    0.0949279    ,    0.0941066    ,    0.0885701    ,    0.096012    ,    0.105832    ,    0.104118    ,    0.131988    ,    0.148682    ,    0.205867    ,    0.211538    ,    0.147182    ,    0.123267    ,    0.130607    ,    0.0963947    ,    0.0899571    ,    0.0949274    ,    0.09724    ,    0.0910828    ,    0.0874437    ,    0.0879696    ,    0.0861136    ,    0.0844604    ,    0.0818847    ,    0.0830137    ,    0.0852445    ,    0.0877201    ,    0.0940186    ,    0.107982    ,    0.12285    ,    0.126146    ,    0.135123    ,    0.162329    ,    0.155262    ,    0.18483    ,    0.173668    ,    0.147923    ,    0.153782    ,    0.134652    ,    0.153307    ,    0.175819    ,    0.191444    ,    0.169235    ,    0.151056    ,    0.205662    ,    0.192173    ,    0.166209    ,    0.206823    ,    0.21485    ,    0.24344    ,    0.212953    ,    0.201019    ,    0.111221    ,    0.109317    ,    0.110578    ,    0.110244    ,    0.106675    ,    0.101929    ,    0.0961829    ,    0.0940761    ,    0.0921536    ,    0.0912222    ,    0.09193    ,    0.0816332    ,    0.0781107    ,    0.0773141    ,    0.0784139    ,    0.0799713    ,    0.079798    ,    0.103728    ,    0.0998578    ,    0.0735816    ,    0.068391    ,    0.0662708    ,    0.0718628    ,    0.0834828    ,    0.0767056    ,    0.0836856    ,    0.0982619    ,    0.0945056    ,    0.100648    ,    0.0986915    ,    0.100744    ,    0.101883    ,    0.105084    ,    0.107634    ,    0.108723    ,    0.110629    ,    0.112316    ,    0.114511    ,    0.117975    ,    0.119885    ,    0.121851    ,    0.123752    ,    0.126004    ,    0.126734    ,    0.127692    ,    0.128739    },
    {    0.131288    ,    0.133017    ,    0.132934    ,    0.132817    ,    0.133229    ,    0.131914    ,    0.131263    ,    0.130701    ,    0.131105    ,    0.130979    ,    0.131049    ,    0.131437    ,    0.129052    ,    0.12569    ,    0.12333    ,    0.120224    ,    0.114395    ,    0.109193    ,    0.103238    ,    0.097123    ,    0.0888061    ,    0.0837635    ,    0.0802508    ,    0.0791755    ,    0.0781767    ,    0.078645    ,    0.0797101    ,    0.0801093    ,    0.0838971    ,    0.0886251    ,    0.0931189    ,    0.0938746    ,    0.0856372    ,    0.104019    ,    0.13375    ,    0.128421    ,    0.120063    ,    0.116545    ,    0.136663    ,    0.162074    ,    0.173574    ,    0.129392    ,    0.109004    ,    0.0983577    ,    0.0942147    ,    0.0927498    ,    0.0946733    ,    0.0941683    ,    0.0901751    ,    0.0888043    ,    0.0889477    ,    0.08917    ,    0.0905799    ,    0.0891615    ,    0.0888474    ,    0.0878387    ,    0.0892589    ,    0.0999868    ,    0.114502    ,    0.142299    ,    0.161987    ,    0.174089    ,    0.223618    ,    0.18457    ,    0.192214    ,    0.186198    ,    0.18876    ,    0.18715    ,    0.176376    ,    0.232126    ,    0.252839    ,    0.288895    ,    0.208658    ,    0.209386    ,    0.319663    ,    0.329807    ,    0.17694    ,    0.170447    ,    0.21259    ,    0.238106    ,    0.209384    ,    0.170179    ,    0.126515    ,    0.10973    ,    0.111179    ,    0.115618    ,    0.114697    ,    0.109881    ,    0.099312    ,    0.0962071    ,    0.092722    ,    0.0917807    ,    0.10045    ,    0.0797998    ,    0.079487    ,    0.0760238    ,    0.0760884    ,    0.0819911    ,    0.0876603    ,    0.117949    ,    0.100439    ,    0.10069    ,    0.0988933    ,    0.0754083    ,    0.0755388    ,    0.0771096    ,    0.0780744    ,    0.0849413    ,    0.109192    ,    0.0943378    ,    0.0955623    ,    0.0968753    ,    0.100372    ,    0.101431    ,    0.102962    ,    0.104618    ,    0.1062    ,    0.108106    ,    0.110747    ,    0.112926    ,    0.115296    ,    0.117401    ,    0.119932    ,    0.121832    ,    0.123844    ,    0.126552    ,    0.12877    ,    0.130195    },
    {    0.133832    ,    0.135838    ,    0.138343    ,    0.139791    ,    0.140279    ,    0.141245    ,    0.141444    ,    0.14123    ,    0.141108    ,    0.142182    ,    0.143651    ,    0.143854    ,    0.144149    ,    0.142893    ,    0.141788    ,    0.138357    ,    0.134846    ,    0.129868    ,    0.124148    ,    0.117095    ,    0.107154    ,    0.0985637    ,    0.0901836    ,    0.0836321    ,    0.0820049    ,    0.0787593    ,    0.0766315    ,    0.0793978    ,    0.0758249    ,    0.07895    ,    0.0746121    ,    0.085417    ,    0.108369    ,    0.159168    ,    0.163304    ,    0.115419    ,    0.113755    ,    0.112473    ,    0.114723    ,    0.115301    ,    0.116103    ,    0.112863    ,    0.108228    ,    0.105011    ,    0.104745    ,    0.106364    ,    0.105738    ,    0.103541    ,    0.102038    ,    0.0995389    ,    0.0961941    ,    0.0968983    ,    0.0988029    ,    0.0998861    ,    0.102421    ,    0.104803    ,    0.108606    ,    0.117587    ,    0.118211    ,    0.195067    ,    0.214713    ,    0.218767    ,    0.248545    ,    0.210984    ,    0.209761    ,    0.213386    ,    0.227563    ,    0.219928    ,    0.230104    ,    0.274228    ,    0.260459    ,    0.267728    ,    0.213773    ,    0.223604    ,    0.27315    ,    0.278871    ,    0.275456    ,    0.195238    ,    0.22887    ,    0.226434    ,    0.145935    ,    0.130367    ,    0.104165    ,    0.0924838    ,    0.100382    ,    0.107177    ,    0.112593    ,    0.113458    ,    0.105705    ,    0.101746    ,    0.098993    ,    0.117134    ,    0.134632    ,    0.0759878    ,    0.0788801    ,    0.0778622    ,    0.0761719    ,    0.0784522    ,    0.0931357    ,    0.125529    ,    0.162946    ,    0.153621    ,    0.125885    ,    0.0838839    ,    0.0827571    ,    0.0790917    ,    0.0848656    ,    0.0928891    ,    0.0923974    ,    0.0914186    ,    0.0915812    ,    0.0951628    ,    0.0980557    ,    0.10031    ,    0.102428    ,    0.104726    ,    0.107437    ,    0.109329    ,    0.111413    ,    0.113581    ,    0.115885    ,    0.118144    ,    0.120366    ,    0.122676    ,    0.124535    ,    0.126949    ,    0.129355    ,    0.131698    },
    {    0.128495    ,    0.131207    ,    0.133503    ,    0.135721    ,    0.138651    ,    0.142165    ,    0.145955    ,    0.150224    ,    0.153999    ,    0.15768    ,    0.161725    ,    0.165169    ,    0.168615    ,    0.170922    ,    0.173111    ,    0.173916    ,    0.173695    ,    0.171536    ,    0.170565    ,    0.167517    ,    0.162437    ,    0.150598    ,    0.134423    ,    0.11325    ,    0.0952697    ,    0.0811377    ,    0.0763748    ,    0.0714562    ,    0.0650232    ,    0.108556    ,    0.114394    ,    0.132016    ,    0.17292    ,    0.14079    ,    0.108367    ,    0.10298    ,    0.109987    ,    0.111524    ,    0.109299    ,    0.108918    ,    0.108245    ,    0.106428    ,    0.106896    ,    0.10655    ,    0.108039    ,    0.109894    ,    0.107872    ,    0.107305    ,    0.106002    ,    0.103171    ,    0.101689    ,    0.101577    ,    0.1059    ,    0.109556    ,    0.112697    ,    0.115173    ,    0.117956    ,    0.128597    ,    0.126892    ,    0.172831    ,    0.219177    ,    0.21741    ,    0.234056    ,    0.214138    ,    0.230799    ,    0.263724    ,    0.260173    ,    0.2493    ,    0.23443    ,    0.225622    ,    0.22985    ,    0.25793    ,    0.266429    ,    0.255026    ,    0.24323    ,    0.239703    ,    0.235888    ,    0.22979    ,    0.195643    ,    0.18879    ,    0.229576    ,    0.16573    ,    0.123505    ,    0.106147    ,    0.0949911    ,    0.103345    ,    0.111203    ,    0.111807    ,    0.107023    ,    0.104503    ,    0.118253    ,    0.139753    ,    0.156957    ,    0.0874455    ,    0.0782896    ,    0.0791493    ,    0.078817    ,    0.0830407    ,    0.10557    ,    0.140242    ,    0.156039    ,    0.15043    ,    0.118814    ,    0.0902985    ,    0.0949572    ,    0.087424    ,    0.0945874    ,    0.0876288    ,    0.0999707    ,    0.0970372    ,    0.0981967    ,    0.09868    ,    0.0995541    ,    0.100587    ,    0.101277    ,    0.102421    ,    0.104209    ,    0.106817    ,    0.108691    ,    0.110065    ,    0.110629    ,    0.111748    ,    0.112922    ,    0.114558    ,    0.116739    ,    0.120006    ,    0.123468    ,    0.125873    },
    {    0.118663    ,    0.121815    ,    0.125323    ,    0.128612    ,    0.131363    ,    0.135001    ,    0.139317    ,    0.145178    ,    0.153617    ,    0.160351    ,    0.169429    ,    0.175929    ,    0.182671    ,    0.188804    ,    0.19424    ,    0.200333    ,    0.206078    ,    0.209715    ,    0.213006    ,    0.213042    ,    0.212454    ,    0.202315    ,    0.185013    ,    0.159352    ,    0.128777    ,    0.100663    ,    0.0902987    ,    0.0985188    ,    0.106855    ,    0.11237    ,    0.104026    ,    0.129557    ,    0.141151    ,    0.0934449    ,    0.087071    ,    0.0879663    ,    0.087559    ,    0.0926078    ,    0.0923157    ,    0.0893554    ,    0.0940162    ,    0.0968055    ,    0.0974307    ,    0.0976319    ,    0.0983565    ,    0.099388    ,    0.100677    ,    0.0988114    ,    0.0986235    ,    0.0983688    ,    0.09799    ,    0.100027    ,    0.104725    ,    0.109009    ,    0.113775    ,    0.117986    ,    0.119909    ,    0.128503    ,    0.135655    ,    0.174942    ,    0.19613    ,    0.199161    ,    0.184921    ,    0.177258    ,    0.177844    ,    0.196311    ,    0.19552    ,    0.196762    ,    0.184848    ,    0.176427    ,    0.183835    ,    0.183084    ,    0.185263    ,    0.176249    ,    0.176462    ,    0.186119    ,    0.196759    ,    0.17605    ,    0.177809    ,    0.197381    ,    0.160918    ,    0.142154    ,    0.138371    ,    0.105745    ,    0.0994763    ,    0.101654    ,    0.105286    ,    0.105367    ,    0.103129    ,    0.106902    ,    0.151022    ,    0.177238    ,    0.20441    ,    0.17    ,    0.112412    ,    0.0827203    ,    0.081655    ,    0.102144    ,    0.130485    ,    0.150544    ,    0.141291    ,    0.147758    ,    0.124062    ,    0.0967616    ,    0.101048    ,    0.102735    ,    0.104289    ,    0.0965776    ,    0.113597    ,    0.116394    ,    0.11454    ,    0.111103    ,    0.109161    ,    0.106618    ,    0.104824    ,    0.102549    ,    0.100678    ,    0.100848    ,    0.100741    ,    0.100127    ,    0.0998237    ,    0.100236    ,    0.100927    ,    0.101676    ,    0.103657    ,    0.10686    ,    0.110661    ,    0.114161    },
    {    0.105352    ,    0.108175    ,    0.111996    ,    0.115431    ,    0.118539    ,    0.122325    ,    0.126351    ,    0.13092    ,    0.137814    ,    0.143642    ,    0.149722    ,    0.156223    ,    0.164462    ,    0.171306    ,    0.178793    ,    0.186718    ,    0.193298    ,    0.198253    ,    0.201238    ,    0.201885    ,    0.201687    ,    0.199075    ,    0.193325    ,    0.172584    ,    0.13708    ,    0.105891    ,    0.105689    ,    0.129556    ,    0.172451    ,    0.110384    ,    0.0814287    ,    0.0802017    ,    0.0852216    ,    0.0789003    ,    0.0827222    ,    0.0979193    ,    0.104704    ,    0.0946362    ,    0.0953125    ,    0.103025    ,    0.101128    ,    0.0907758    ,    0.0888843    ,    0.0892833    ,    0.0903834    ,    0.090681    ,    0.0901053    ,    0.0890143    ,    0.0909219    ,    0.0909386    ,    0.0918281    ,    0.0940567    ,    0.0991639    ,    0.102694    ,    0.105777    ,    0.11138    ,    0.114978    ,    0.121391    ,    0.13518    ,    0.123509    ,    0.143324    ,    0.135014    ,    0.136652    ,    0.134903    ,    0.144876    ,    0.152554    ,    0.141689    ,    0.129489    ,    0.124392    ,    0.121142    ,    0.122052    ,    0.12701    ,    0.137637    ,    0.128222    ,    0.134746    ,    0.149125    ,    0.157955    ,    0.168783    ,    0.178682    ,    0.169406    ,    0.1339    ,    0.136305    ,    0.114925    ,    0.111754    ,    0.13106    ,    0.115111    ,    0.0996683    ,    0.099131    ,    0.103853    ,    0.135591    ,    0.212904    ,    0.240432    ,    0.227284    ,    0.212998    ,    0.193816    ,    0.131839    ,    0.0969862    ,    0.121821    ,    0.141842    ,    0.144455    ,    0.157917    ,    0.17596    ,    0.145456    ,    0.138334    ,    0.129432    ,    0.115371    ,    0.114147    ,    0.11624    ,    0.129034    ,    0.137147    ,    0.137557    ,    0.135284    ,    0.133562    ,    0.133744    ,    0.131784    ,    0.128452    ,    0.123326    ,    0.118126    ,    0.114848    ,    0.109271    ,    0.104228    ,    0.100019    ,    0.0985506    ,    0.0963737    ,    0.095058    ,    0.0958184    ,    0.0975447    ,    0.100616    },
    {    0.116589    ,    0.116149    ,    0.115954    ,    0.115204    ,    0.116089    ,    0.115803    ,    0.114647    ,    0.113973    ,    0.116102    ,    0.119533    ,    0.123478    ,    0.127333    ,    0.133862    ,    0.138595    ,    0.144157    ,    0.148643    ,    0.153014    ,    0.157666    ,    0.162375    ,    0.163742    ,    0.160825    ,    0.16177    ,    0.166627    ,    0.158947    ,    0.140598    ,    0.117031    ,    0.121187    ,    0.139769    ,    0.156527    ,    0.122263    ,    0.0778845    ,    0.0887103    ,    0.0865168    ,    0.0882062    ,    0.0892818    ,    0.0918588    ,    0.0854581    ,    0.0768509    ,    0.0822707    ,    0.0856086    ,    0.0840263    ,    0.085461    ,    0.0923617    ,    0.0994387    ,    0.104988    ,    0.109818    ,    0.113685    ,    0.113614    ,    0.11043    ,    0.103393    ,    0.101098    ,    0.0983902    ,    0.0972233    ,    0.0957214    ,    0.0957624    ,    0.0994159    ,    0.10659    ,    0.120589    ,    0.135248    ,    0.0954357    ,    0.0967941    ,    0.107844    ,    0.112569    ,    0.115979    ,    0.127649    ,    0.134649    ,    0.142624    ,    0.125243    ,    0.121747    ,    0.11153    ,    0.10524    ,    0.105007    ,    0.104942    ,    0.118031    ,    0.119601    ,    0.139993    ,    0.133641    ,    0.214843    ,    0.172627    ,    0.154313    ,    0.144576    ,    0.134265    ,    0.14599    ,    0.157385    ,    0.132317    ,    0.137252    ,    0.123859    ,    0.12047    ,    0.129356    ,    0.158943    ,    0.228758    ,    0.265255    ,    0.269618    ,    0.230445    ,    0.209006    ,    0.206758    ,    0.174897    ,    0.14818    ,    0.169501    ,    0.131083    ,    0.135792    ,    0.155247    ,    0.160243    ,    0.157207    ,    0.16885    ,    0.154419    ,    0.145077    ,    0.133309    ,    0.135884    ,    0.146783    ,    0.149325    ,    0.154519    ,    0.155361    ,    0.159439    ,    0.160721    ,    0.160722    ,    0.159673    ,    0.157389    ,    0.154301    ,    0.148201    ,    0.142867    ,    0.136562    ,    0.131365    ,    0.126615    ,    0.122483    ,    0.118601    ,    0.116006    ,    0.115258    },
    {    0.144336    ,    0.144901    ,    0.145124    ,    0.144785    ,    0.143728    ,    0.141657    ,    0.13626    ,    0.128255    ,    0.119718    ,    0.112331    ,    0.108523    ,    0.107108    ,    0.109385    ,    0.111941    ,    0.115206    ,    0.11855    ,    0.12379    ,    0.126385    ,    0.129197    ,    0.131255    ,    0.131015    ,    0.131686    ,    0.136936    ,    0.154439    ,    0.146638    ,    0.193952    ,    0.18572    ,    0.14483    ,    0.144861    ,    0.135399    ,    0.0921663    ,    0.0902869    ,    0.0907834    ,    0.0951061    ,    0.101211    ,    0.109586    ,    0.0764524    ,    0.0787219    ,    0.0941364    ,    0.0860516    ,    0.0814317    ,    0.0842736    ,    0.087099    ,    0.093946    ,    0.105922    ,    0.115133    ,    0.122519    ,    0.127439    ,    0.129525    ,    0.132137    ,    0.131068    ,    0.125754    ,    0.118326    ,    0.107821    ,    0.100835    ,    0.102707    ,    0.106322    ,    0.111799    ,    0.123094    ,    0.125617    ,    0.0893549    ,    0.0974906    ,    0.112176    ,    0.114949    ,    0.123079    ,    0.125955    ,    0.132903    ,    0.133837    ,    0.130486    ,    0.112455    ,    0.100575    ,    0.103975    ,    0.0910713    ,    0.102935    ,    0.10557    ,    0.132244    ,    0.150607    ,    0.172445    ,    0.142525    ,    0.140478    ,    0.13708    ,    0.140186    ,    0.249228    ,    0.233677    ,    0.189492    ,    0.106008    ,    0.11331    ,    0.168708    ,    0.12562    ,    0.141009    ,    0.188078    ,    0.242732    ,    0.251772    ,    0.191495    ,    0.162407    ,    0.138221    ,    0.130094    ,    0.125396    ,    0.11808    ,    0.124254    ,    0.13333    ,    0.144772    ,    0.155593    ,    0.165517    ,    0.191408    ,    0.181868    ,    0.16671    ,    0.153289    ,    0.139411    ,    0.147363    ,    0.149492    ,    0.151628    ,    0.155497    ,    0.158642    ,    0.163047    ,    0.166418    ,    0.171857    ,    0.17421    ,    0.175997    ,    0.175382    ,    0.173076    ,    0.168205    ,    0.163513    ,    0.160068    ,    0.154729    ,    0.14812    ,    0.145124    ,    0.143754    },
    {    0.158794    ,    0.160817    ,    0.162589    ,    0.164077    ,    0.164979    ,    0.166084    ,    0.162435    ,    0.155551    ,    0.14458    ,    0.130523    ,    0.116099    ,    0.103441    ,    0.0950002    ,    0.092655    ,    0.095135    ,    0.0995262    ,    0.104567    ,    0.107713    ,    0.111442    ,    0.114699    ,    0.109285    ,    0.109391    ,    0.118704    ,    0.139737    ,    0.210091    ,    0.204627    ,    0.200557    ,    0.157201    ,    0.161598    ,    0.177307    ,    0.142757    ,    0.158158    ,    0.14352    ,    0.140545    ,    0.132479    ,    0.125124    ,    0.0764992    ,    0.0782662    ,    0.0980964    ,    0.0981765    ,    0.090702    ,    0.0886904    ,    0.0868013    ,    0.0847337    ,    0.0877768    ,    0.0900468    ,    0.0936646    ,    0.101051    ,    0.106513    ,    0.109851    ,    0.11512    ,    0.12498    ,    0.136048    ,    0.135513    ,    0.126892    ,    0.113629    ,    0.110879    ,    0.116504    ,    0.120415    ,    0.133215    ,    0.127808    ,    0.144425    ,    0.138675    ,    0.129362    ,    0.124229    ,    0.12925    ,    0.136274    ,    0.139146    ,    0.116583    ,    0.105174    ,    0.0854712    ,    0.0810737    ,    0.0843689    ,    0.0840574    ,    0.0930109    ,    0.108718    ,    0.130991    ,    0.13773    ,    0.12756    ,    0.123683    ,    0.127566    ,    0.202145    ,    0.224605    ,    0.141861    ,    0.139004    ,    0.129758    ,    0.133806    ,    0.136777    ,    0.152055    ,    0.175755    ,    0.175084    ,    0.239726    ,    0.186896    ,    0.157436    ,    0.230731    ,    0.288418    ,    0.256485    ,    0.209589    ,    0.207503    ,    0.201918    ,    0.158635    ,    0.154357    ,    0.17763    ,    0.137141    ,    0.162226    ,    0.17285    ,    0.166656    ,    0.149537    ,    0.152416    ,    0.151674    ,    0.15248    ,    0.152774    ,    0.15364    ,    0.155574    ,    0.155918    ,    0.157588    ,    0.162445    ,    0.168379    ,    0.171342    ,    0.171618    ,    0.171813    ,    0.173061    ,    0.17262    ,    0.171592    ,    0.169929    ,    0.165228    ,    0.161377    ,    0.159387    },
    {    0.177775    ,    0.178203    ,    0.177568    ,    0.176749    ,    0.176021    ,    0.176654    ,    0.174717    ,    0.170862    ,    0.164808    ,    0.155844    ,    0.141963    ,    0.124859    ,    0.109555    ,    0.095909    ,    0.0885357    ,    0.0880346    ,    0.0923711    ,    0.0971795    ,    0.101934    ,    0.103212    ,    0.0909564    ,    0.0995796    ,    0.125957    ,    0.14277    ,    0.167362    ,    0.180255    ,    0.172629    ,    0.174459    ,    0.174949    ,    0.197482    ,    0.197264    ,    0.188766    ,    0.189831    ,    0.16126    ,    0.155174    ,    0.14232    ,    0.106651    ,    0.0866006    ,    0.090899    ,    0.0955289    ,    0.0944984    ,    0.0935103    ,    0.0925598    ,    0.0935072    ,    0.0939464    ,    0.0924898    ,    0.0920134    ,    0.0915139    ,    0.0908532    ,    0.087439    ,    0.0871729    ,    0.0929127    ,    0.104692    ,    0.119679    ,    0.134042    ,    0.141241    ,    0.135177    ,    0.124814    ,    0.123203    ,    0.13225    ,    0.150593    ,    0.153285    ,    0.154435    ,    0.149719    ,    0.13296    ,    0.125958    ,    0.118276    ,    0.131712    ,    0.114406    ,    0.100818    ,    0.0952118    ,    0.0930064    ,    0.101597    ,    0.102731    ,    0.113883    ,    0.1219    ,    0.154532    ,    0.158966    ,    0.131137    ,    0.126441    ,    0.134346    ,    0.155067    ,    0.137086    ,    0.138521    ,    0.126965    ,    0.130807    ,    0.146872    ,    0.122322    ,    0.152193    ,    0.160544    ,    0.219135    ,    0.175037    ,    0.22522    ,    0.336972    ,    0.324668    ,    0.302292    ,    0.292125    ,    0.295399    ,    0.29496    ,    0.24663    ,    0.158312    ,    0.115949    ,    0.130881    ,    0.130668    ,    0.142502    ,    0.152592    ,    0.151101    ,    0.166178    ,    0.166347    ,    0.160502    ,    0.147997    ,    0.160289    ,    0.161027    ,    0.162027    ,    0.165239    ,    0.166869    ,    0.17029    ,    0.174904    ,    0.179338    ,    0.181623    ,    0.182757    ,    0.183966    ,    0.184482    ,    0.184945    ,    0.184045    ,    0.18186    ,    0.180937    ,    0.178551    },
    {    0.202065    ,    0.201843    ,    0.201684    ,    0.197781    ,    0.193916    ,    0.189585    ,    0.185406    ,    0.181472    ,    0.178057    ,    0.17098    ,    0.162246    ,    0.149334    ,    0.134206    ,    0.118458    ,    0.104847    ,    0.0955786    ,    0.092981    ,    0.0935987    ,    0.0942917    ,    0.0960514    ,    0.097614    ,    0.0963206    ,    0.106657    ,    0.137503    ,    0.173766    ,    0.187645    ,    0.155872    ,    0.181349    ,    0.183422    ,    0.204295    ,    0.199972    ,    0.187222    ,    0.173171    ,    0.15423    ,    0.147675    ,    0.144679    ,    0.135368    ,    0.114871    ,    0.106507    ,    0.107673    ,    0.109656    ,    0.115396    ,    0.118398    ,    0.12227    ,    0.126897    ,    0.129    ,    0.130483    ,    0.130392    ,    0.128944    ,    0.12701    ,    0.11971    ,    0.116286    ,    0.120852    ,    0.129718    ,    0.137631    ,    0.147407    ,    0.159879    ,    0.16072    ,    0.142285    ,    0.136192    ,    0.160097    ,    0.186953    ,    0.17797    ,    0.153347    ,    0.109231    ,    0.124883    ,    0.122217    ,    0.11789    ,    0.10988    ,    0.0973611    ,    0.0941815    ,    0.0892853    ,    0.0983122    ,    0.111809    ,    0.126259    ,    0.136458    ,    0.152022    ,    0.139221    ,    0.132081    ,    0.132927    ,    0.138885    ,    0.135948    ,    0.129482    ,    0.124415    ,    0.134507    ,    0.15072    ,    0.127128    ,    0.111597    ,    0.173403    ,    0.178857    ,    0.200709    ,    0.22323    ,    0.251979    ,    0.242584    ,    0.264099    ,    0.292239    ,    0.272591    ,    0.282242    ,    0.271965    ,    0.202501    ,    0.138895    ,    0.134834    ,    0.155001    ,    0.141235    ,    0.130509    ,    0.151145    ,    0.146495    ,    0.17805    ,    0.165448    ,    0.149308    ,    0.15928    ,    0.15108    ,    0.156522    ,    0.157859    ,    0.178754    ,    0.194593    ,    0.20404    ,    0.208424    ,    0.211116    ,    0.215714    ,    0.219063    ,    0.220917    ,    0.220042    ,    0.218096    ,    0.215253    ,    0.210942    ,    0.207468    ,    0.205024    },
    {    0.222451    ,    0.221008    ,    0.219665    ,    0.216337    ,    0.21187    ,    0.20715    ,    0.202219    ,    0.197736    ,    0.194042    ,    0.186678    ,    0.178197    ,    0.168109    ,    0.156315    ,    0.142397    ,    0.127223    ,    0.114478    ,    0.106639    ,    0.0991307    ,    0.0929176    ,    0.0986285    ,    0.100908    ,    0.117747    ,    0.145213    ,    0.134589    ,    0.169795    ,    0.188097    ,    0.174074    ,    0.165059    ,    0.190408    ,    0.215602    ,    0.207717    ,    0.190076    ,    0.191753    ,    0.180165    ,    0.169566    ,    0.156399    ,    0.143734    ,    0.130367    ,    0.142459    ,    0.140238    ,    0.146358    ,    0.151562    ,    0.155234    ,    0.15991    ,    0.167608    ,    0.176049    ,    0.181998    ,    0.186209    ,    0.185395    ,    0.182561    ,    0.175503    ,    0.171442    ,    0.174692    ,    0.176975    ,    0.177278    ,    0.177154    ,    0.179594    ,    0.188043    ,    0.17269    ,    0.154858    ,    0.157605    ,    0.178091    ,    0.152197    ,    0.115559    ,    0.132852    ,    0.108502    ,    0.109748    ,    0.103503    ,    0.0928405    ,    0.100242    ,    0.106414    ,    0.10517    ,    0.11037    ,    0.162484    ,    0.144603    ,    0.142826    ,    0.130076    ,    0.119634    ,    0.140541    ,    0.144741    ,    0.141558    ,    0.15222    ,    0.214366    ,    0.161983    ,    0.132025    ,    0.130613    ,    0.146674    ,    0.121833    ,    0.119533    ,    0.310465    ,    0.345386    ,    0.188978    ,    0.149864    ,    0.14232    ,    0.144696    ,    0.157407    ,    0.155664    ,    0.144749    ,    0.14741    ,    0.135631    ,    0.108654    ,    0.140894    ,    0.12934    ,    0.12638    ,    0.12191    ,    0.169189    ,    0.15544    ,    0.154922    ,    0.158376    ,    0.130393    ,    0.172162    ,    0.180675    ,    0.17023    ,    0.160551    ,    0.170276    ,    0.244789    ,    0.282808    ,    0.284523    ,    0.276579    ,    0.27478    ,    0.274159    ,    0.269692    ,    0.261668    ,    0.252197    ,    0.243764    ,    0.237056    ,    0.231609    ,    0.227077    },
    {    0.227396    ,    0.230767    ,    0.233711    ,    0.233285    ,    0.230737    ,    0.226482    ,    0.221033    ,    0.215089    ,    0.20817    ,    0.200606    ,    0.193561    ,    0.184437    ,    0.173545    ,    0.160881    ,    0.146793    ,    0.133812    ,    0.121231    ,    0.10692    ,    0.0962414    ,    0.0982978    ,    0.0835355    ,    0.113041    ,    0.15131    ,    0.157745    ,    0.180459    ,    0.190934    ,    0.161797    ,    0.162746    ,    0.216912    ,    0.252108    ,    0.225016    ,    0.205329    ,    0.198067    ,    0.178154    ,    0.168984    ,    0.160045    ,    0.157947    ,    0.151582    ,    0.156776    ,    0.172766    ,    0.197335    ,    0.217365    ,    0.221068    ,    0.223545    ,    0.234546    ,    0.250156    ,    0.265332    ,    0.282764    ,    0.295384    ,    0.280089    ,    0.255442    ,    0.235888    ,    0.229204    ,    0.228652    ,    0.221847    ,    0.210981    ,    0.204399    ,    0.202975    ,    0.189849    ,    0.172796    ,    0.164702    ,    0.138645    ,    0.145352    ,    0.126836    ,    0.104258    ,    0.10527    ,    0.0905607    ,    0.0874277    ,    0.0915525    ,    0.0962108    ,    0.12132    ,    0.124145    ,    0.138126    ,    0.150914    ,    0.16898    ,    0.158856    ,    0.16099    ,    0.137334    ,    0.139922    ,    0.147575    ,    0.149346    ,    0.161797    ,    0.258541    ,    0.201176    ,    0.153599    ,    0.149313    ,    0.140797    ,    0.125371    ,    0.128402    ,    0.131339    ,    0.154742    ,    0.149631    ,    0.141331    ,    0.130797    ,    0.13516    ,    0.147625    ,    0.12962    ,    0.145849    ,    0.121845    ,    0.129226    ,    0.116455    ,    0.130021    ,    0.128301    ,    0.132193    ,    0.131029    ,    0.120372    ,    0.123531    ,    0.137184    ,    0.126502    ,    0.129421    ,    0.156045    ,    0.172931    ,    0.163556    ,    0.162549    ,    0.147939    ,    0.253328    ,    0.319511    ,    0.305809    ,    0.283204    ,    0.272954    ,    0.268849    ,    0.260302    ,    0.251128    ,    0.239958    ,    0.23294    ,    0.228577    ,    0.226433    ,    0.225782    },
    {    0.23296    ,    0.241926    ,    0.248989    ,    0.25176    ,    0.249287    ,    0.243747    ,    0.237962    ,    0.23153    ,    0.225487    ,    0.21876    ,    0.211666    ,    0.202659    ,    0.192163    ,    0.178686    ,    0.164957    ,    0.151455    ,    0.134458    ,    0.116719    ,    0.103862    ,    0.101845    ,    0.099547    ,    0.118739    ,    0.139238    ,    0.142982    ,    0.175779    ,    0.170139    ,    0.209977    ,    0.202144    ,    0.236586    ,    0.262332    ,    0.24751    ,    0.225859    ,    0.21841    ,    0.197959    ,    0.181645    ,    0.188997    ,    0.192575    ,    0.184151    ,    0.183432    ,    0.191432    ,    0.216035    ,    0.250165    ,    0.289599    ,    0.321021    ,    0.335491    ,    0.330129    ,    0.307962    ,    0.388901    ,    0.47464    ,    0.467495    ,    0.432852    ,    0.384798    ,    0.343376    ,    0.314439    ,    0.294937    ,    0.26481    ,    0.233095    ,    0.21891    ,    0.200862    ,    0.177056    ,    0.162255    ,    0.14155    ,    0.144893    ,    0.155434    ,    0.138322    ,    0.121189    ,    0.116066    ,    0.119973    ,    0.126194    ,    0.171353    ,    0.139464    ,    0.128301    ,    0.129631    ,    0.144441    ,    0.167666    ,    0.161556    ,    0.158907    ,    0.151789    ,    0.143914    ,    0.125902    ,    0.123817    ,    0.167994    ,    0.227341    ,    0.172117    ,    0.160389    ,    0.167079    ,    0.136576    ,    0.135999    ,    0.115849    ,    0.137599    ,    0.159059    ,    0.177135    ,    0.179381    ,    0.179857    ,    0.187337    ,    0.184372    ,    0.182633    ,    0.182887    ,    0.179606    ,    0.187528    ,    0.185049    ,    0.128947    ,    0.140058    ,    0.145102    ,    0.155596    ,    0.149362    ,    0.1448    ,    0.138598    ,    0.152331    ,    0.160318    ,    0.147834    ,    0.138369    ,    0.161712    ,    0.168367    ,    0.171013    ,    0.165649    ,    0.169478    ,    0.165352    ,    0.163672    ,    0.168191    ,    0.172261    ,    0.183891    ,    0.196068    ,    0.205596    ,    0.206224    ,    0.205907    ,    0.212194    ,    0.220756    },
    {    0.247694    ,    0.254217    ,    0.259091    ,    0.259204    ,    0.256467    ,    0.251323    ,    0.245335    ,    0.241217    ,    0.238978    ,    0.235797    ,    0.229984    ,    0.221152    ,    0.209585    ,    0.194369    ,    0.179991    ,    0.164932    ,    0.145715    ,    0.126566    ,    0.111061    ,    0.107059    ,    0.124124    ,    0.115786    ,    0.143095    ,    0.145115    ,    0.157184    ,    0.188267    ,    0.216553    ,    0.231521    ,    0.256658    ,    0.276227    ,    0.274112    ,    0.252648    ,    0.218629    ,    0.216519    ,    0.226644    ,    0.224177    ,    0.242684    ,    0.224277    ,    0.22634    ,    0.220716    ,    0.199902    ,    0.198802    ,    0.225964    ,    0.27985    ,    0.298084    ,    0.282307    ,    0.28432    ,    0.371365    ,    0.45487    ,    0.459509    ,    0.456499    ,    0.447486    ,    0.425703    ,    0.385916    ,    0.350195    ,    0.315303    ,    0.279116    ,    0.256254    ,    0.238693    ,    0.202678    ,    0.16779    ,    0.160652    ,    0.168901    ,    0.149466    ,    0.111213    ,    0.12491    ,    0.125997    ,    0.127314    ,    0.13085    ,    0.126225    ,    0.127592    ,    0.131201    ,    0.140387    ,    0.127909    ,    0.147688    ,    0.175308    ,    0.15507    ,    0.153348    ,    0.149242    ,    0.155649    ,    0.168899    ,    0.18387    ,    0.189088    ,    0.194723    ,    0.169066    ,    0.175647    ,    0.171857    ,    0.159727    ,    0.161702    ,    0.165327    ,    0.165552    ,    0.154164    ,    0.168354    ,    0.159699    ,    0.223979    ,    0.247295    ,    0.218599    ,    0.198505    ,    0.171382    ,    0.167513    ,    0.159617    ,    0.139076    ,    0.14189    ,    0.142054    ,    0.142951    ,    0.155726    ,    0.155419    ,    0.153746    ,    0.160348    ,    0.17025    ,    0.175559    ,    0.170387    ,    0.171237    ,    0.192696    ,    0.22432    ,    0.207837    ,    0.237948    ,    0.23778    ,    0.231829    ,    0.226886    ,    0.22401    ,    0.226121    ,    0.228542    ,    0.231143    ,    0.232717    ,    0.230911    ,    0.235037    ,    0.241528    },
    {    0.26005    ,    0.258862    ,    0.255226    ,    0.24932    ,    0.244702    ,    0.242964    ,    0.24233    ,    0.242812    ,    0.24186    ,    0.23934    ,    0.235761    ,    0.228265    ,    0.216873    ,    0.200541    ,    0.184083    ,    0.166238    ,    0.146248    ,    0.1295    ,    0.114657    ,    0.108063    ,    0.133483    ,    0.138132    ,    0.156658    ,    0.150343    ,    0.184849    ,    0.200648    ,    0.221318    ,    0.239527    ,    0.262348    ,    0.276373    ,    0.266036    ,    0.243507    ,    0.237508    ,    0.239899    ,    0.24615    ,    0.25621    ,    0.252214    ,    0.257357    ,    0.261268    ,    0.247569    ,    0.237589    ,    0.236261    ,    0.244139    ,    0.24601    ,    0.229395    ,    0.275829    ,    0.344608    ,    0.413694    ,    0.438549    ,    0.395362    ,    0.451195    ,    0.471879    ,    0.461851    ,    0.415011    ,    0.375383    ,    0.337008    ,    0.30103    ,    0.269744    ,    0.243492    ,    0.208038    ,    0.183866    ,    0.176948    ,    0.168707    ,    0.149208    ,    0.123833    ,    0.119645    ,    0.125378    ,    0.134135    ,    0.141954    ,    0.135918    ,    0.140047    ,    0.148239    ,    0.153887    ,    0.157601    ,    0.165823    ,    0.178176    ,    0.179193    ,    0.182902    ,    0.187496    ,    0.195001    ,    0.197792    ,    0.206811    ,    0.219878    ,    0.20289    ,    0.202008    ,    0.202299    ,    0.208151    ,    0.203349    ,    0.196339    ,    0.193619    ,    0.188225    ,    0.187916    ,    0.188641    ,    0.20422    ,    0.201312    ,    0.185827    ,    0.182087    ,    0.201569    ,    0.183387    ,    0.152528    ,    0.165018    ,    0.147552    ,    0.154919    ,    0.159081    ,    0.172214    ,    0.162296    ,    0.180764    ,    0.186369    ,    0.198469    ,    0.207545    ,    0.215197    ,    0.217192    ,    0.211925    ,    0.210099    ,    0.236711    ,    0.257352    ,    0.338341    ,    0.348459    ,    0.333764    ,    0.31468    ,    0.306954    ,    0.312266    ,    0.310386    ,    0.299106    ,    0.288591    ,    0.278177    ,    0.271124    ,    0.265469    },
    {    0.272844    ,    0.261663    ,    0.25009    ,    0.24308    ,    0.239232    ,    0.23923    ,    0.242468    ,    0.24377    ,    0.240578    ,    0.236693    ,    0.233072    ,    0.225643    ,    0.213112    ,    0.194944    ,    0.177202    ,    0.158556    ,    0.141869    ,    0.132768    ,    0.128886    ,    0.128318    ,    0.150396    ,    0.151399    ,    0.165121    ,    0.175941    ,    0.203594    ,    0.2198    ,    0.234808    ,    0.254217    ,    0.262435    ,    0.267695    ,    0.251186    ,    0.245522    ,    0.250007    ,    0.248171    ,    0.278011    ,    0.317312    ,    0.307067    ,    0.299859    ,    0.283563    ,    0.258532    ,    0.255429    ,    0.268168    ,    0.265767    ,    0.271109    ,    0.292041    ,    0.353529    ,    0.420126    ,    0.426539    ,    0.422999    ,    0.39406    ,    0.413879    ,    0.430598    ,    0.457391    ,    0.44495    ,    0.405121    ,    0.357763    ,    0.313997    ,    0.28701    ,    0.251907    ,    0.207845    ,    0.192943    ,    0.175404    ,    0.173171    ,    0.148152    ,    0.1439    ,    0.15141    ,    0.143644    ,    0.155013    ,    0.153353    ,    0.156159    ,    0.167908    ,    0.175401    ,    0.183436    ,    0.189487    ,    0.198686    ,    0.212358    ,    0.222466    ,    0.229821    ,    0.238243    ,    0.247388    ,    0.247147    ,    0.241833    ,    0.24033    ,    0.231889    ,    0.211958    ,    0.215459    ,    0.209843    ,    0.206612    ,    0.217563    ,    0.229943    ,    0.221545    ,    0.217898    ,    0.214593    ,    0.225278    ,    0.201299    ,    0.188822    ,    0.17615    ,    0.176226    ,    0.157119    ,    0.153572    ,    0.163833    ,    0.162772    ,    0.17166    ,    0.171681    ,    0.18683    ,    0.193578    ,    0.192273    ,    0.210246    ,    0.225925    ,    0.243939    ,    0.246162    ,    0.239233    ,    0.240099    ,    0.254282    ,    0.276312    ,    0.328176    ,    0.430694    ,    0.432718    ,    0.401054    ,    0.362088    ,    0.332573    ,    0.354023    ,    0.370807    ,    0.364805    ,    0.349757    ,    0.328917    ,    0.310236    ,    0.293741    },
    {    0.294424    ,    0.28067    ,    0.26764    ,    0.262991    ,    0.267634    ,    0.266383    ,    0.258843    ,    0.252427    ,    0.242224    ,    0.230395    ,    0.218764    ,    0.20749    ,    0.193573    ,    0.178271    ,    0.166232    ,    0.155256    ,    0.144726    ,    0.153304    ,    0.154221    ,    0.151891    ,    0.157029    ,    0.157175    ,    0.161707    ,    0.178261    ,    0.214165    ,    0.220264    ,    0.231796    ,    0.243144    ,    0.250405    ,    0.283118    ,    0.275968    ,    0.296211    ,    0.315124    ,    0.333737    ,    0.371424    ,    0.405849    ,    0.401216    ,    0.321583    ,    0.29083    ,    0.286858    ,    0.287987    ,    0.282121    ,    0.288713    ,    0.309412    ,    0.372482    ,    0.464639    ,    0.519911    ,    0.487362    ,    0.434083    ,    0.417134    ,    0.436104    ,    0.446477    ,    0.442458    ,    0.426082    ,    0.401091    ,    0.362357    ,    0.316849    ,    0.287468    ,    0.25835    ,    0.219707    ,    0.202769    ,    0.180201    ,    0.179024    ,    0.155243    ,    0.163222    ,    0.175536    ,    0.184448    ,    0.182327    ,    0.191358    ,    0.201985    ,    0.208996    ,    0.208668    ,    0.218519    ,    0.226538    ,    0.243749    ,    0.252881    ,    0.253904    ,    0.253716    ,    0.253326    ,    0.255443    ,    0.257193    ,    0.253806    ,    0.250236    ,    0.243404    ,    0.224356    ,    0.226842    ,    0.23256    ,    0.247162    ,    0.252386    ,    0.262405    ,    0.262754    ,    0.245715    ,    0.242731    ,    0.248369    ,    0.240609    ,    0.213287    ,    0.214963    ,    0.190463    ,    0.183905    ,    0.179964    ,    0.182859    ,    0.188385    ,    0.18361    ,    0.181043    ,    0.199162    ,    0.210831    ,    0.218726    ,    0.2309    ,    0.245962    ,    0.271919    ,    0.26868    ,    0.2581    ,    0.25718    ,    0.354407    ,    0.473374    ,    0.49806    ,    0.505886    ,    0.476195    ,    0.435492    ,    0.383979    ,    0.386282    ,    0.350367    ,    0.393987    ,    0.416952    ,    0.408086    ,    0.384295    ,    0.356304    ,    0.327354    },
    {    0.338838    ,    0.328987    ,    0.320999    ,    0.32016    ,    0.324788    ,    0.326133    ,    0.324632    ,    0.303822    ,    0.28014    ,    0.250087    ,    0.226875    ,    0.209196    ,    0.193307    ,    0.180919    ,    0.173583    ,    0.168462    ,    0.180313    ,    0.196074    ,    0.200997    ,    0.197291    ,    0.173683    ,    0.192292    ,    0.209634    ,    0.212416    ,    0.216285    ,    0.222312    ,    0.242885    ,    0.264315    ,    0.274927    ,    0.297534    ,    0.323243    ,    0.367632    ,    0.419966    ,    0.430537    ,    0.430853    ,    0.427728    ,    0.399716    ,    0.366852    ,    0.34341    ,    0.333641    ,    0.327406    ,    0.340212    ,    0.351562    ,    0.396699    ,    0.550253    ,    0.604356    ,    0.608533    ,    0.599774    ,    0.596971    ,    0.599855    ,    0.588018    ,    0.54964    ,    0.516924    ,    0.487677    ,    0.430666    ,    0.377136    ,    0.331149    ,    0.298917    ,    0.268245    ,    0.235402    ,    0.224049    ,    0.217197    ,    0.193212    ,    0.194708    ,    0.206353    ,    0.215332    ,    0.22255    ,    0.216586    ,    0.230254    ,    0.225903    ,    0.247363    ,    0.263687    ,    0.265289    ,    0.273039    ,    0.278751    ,    0.281863    ,    0.284187    ,    0.283512    ,    0.282103    ,    0.279886    ,    0.273362    ,    0.269978    ,    0.267912    ,    0.256099    ,    0.250408    ,    0.244741    ,    0.255281    ,    0.26682    ,    0.261635    ,    0.252905    ,    0.253214    ,    0.250415    ,    0.260018    ,    0.260648    ,    0.255005    ,    0.247032    ,    0.24196    ,    0.244829    ,    0.230413    ,    0.225957    ,    0.223289    ,    0.223568    ,    0.215477    ,    0.215487    ,    0.238123    ,    0.236679    ,    0.241792    ,    0.261405    ,    0.267583    ,    0.275751    ,    0.277843    ,    0.274245    ,    0.278478    ,    0.331614    ,    0.445405    ,    0.49023    ,    0.494576    ,    0.473726    ,    0.453586    ,    0.425386    ,    0.400126    ,    0.37401    ,    0.379556    ,    0.430374    ,    0.431594    ,    0.41438    ,    0.394168    ,    0.368276    },
    {    0.390891    ,    0.374753    ,    0.350214    ,    0.330236    ,    0.326017    ,    0.333899    ,    0.334099    ,    0.339552    ,    0.350783    ,    0.327276    ,    0.302521    ,    0.29096    ,    0.274276    ,    0.257545    ,    0.186836    ,    0.278983    ,    0.245149    ,    0.23041    ,    0.220139    ,    0.216733    ,    0.217242    ,    0.228274    ,    0.24062    ,    0.23682    ,    0.243587    ,    0.247416    ,    0.261915    ,    0.273758    ,    0.292048    ,    0.312834    ,    0.352279    ,    0.422758    ,    0.445389    ,    0.45107    ,    0.453541    ,    0.450314    ,    0.430898    ,    0.423554    ,    0.408715    ,    0.39815    ,    0.40171    ,    0.404957    ,    0.359531    ,    0.499717    ,    0.639811    ,    0.63069    ,    0.584778    ,    0.513161    ,    0.481071    ,    0.54266    ,    0.635196    ,    0.623704    ,    0.580801    ,    0.513635    ,    0.419595    ,    0.348195    ,    0.321477    ,    0.299104    ,    0.284228    ,    0.269267    ,    0.262993    ,    0.259491    ,    0.254039    ,    0.261328    ,    0.268813    ,    0.264933    ,    0.245297    ,    0.233498    ,    0.238788    ,    0.248003    ,    0.268444    ,    0.305495    ,    0.317171    ,    0.314647    ,    0.313818    ,    0.315461    ,    0.312972    ,    0.310188    ,    0.302083    ,    0.292389    ,    0.284443    ,    0.273324    ,    0.26623    ,    0.264668    ,    0.260238    ,    0.253352    ,    0.243494    ,    0.246253    ,    0.262367    ,    0.24658    ,    0.248738    ,    0.254842    ,    0.260207    ,    0.262099    ,    0.267145    ,    0.256425    ,    0.250238    ,    0.245339    ,    0.268359    ,    0.251575    ,    0.256893    ,    0.259532    ,    0.262312    ,    0.261479    ,    0.260505    ,    0.267072    ,    0.269603    ,    0.271461    ,    0.276756    ,    0.271957    ,    0.284048    ,    0.289795    ,    0.304101    ,    0.339072    ,    0.345152    ,    0.376063    ,    0.39647    ,    0.410226    ,    0.41933    ,    0.427631    ,    0.397988    ,    0.349653    ,    0.370987    ,    0.36018    ,    0.349793    ,    0.368835    ,    0.394073    ,    0.401502    },
    {    0.330178    ,    0.325442    ,    0.313743    ,    0.311747    ,    0.316143    ,    0.323253    ,    0.333656    ,    0.350158    ,    0.354985    ,    0.346013    ,    0.341525    ,    0.325987    ,    0.313704    ,    0.302694    ,    0.323678    ,    0.267988    ,    0.249227    ,    0.241315    ,    0.233213    ,    0.237031    ,    0.235964    ,    0.26261    ,    0.280872    ,    0.274712    ,    0.274625    ,    0.291144    ,    0.312665    ,    0.327924    ,    0.343845    ,    0.364318    ,    0.391405    ,    0.417903    ,    0.439837    ,    0.450009    ,    0.462172    ,    0.466379    ,    0.480457    ,    0.489827    ,    0.470954    ,    0.417952    ,    0.399623    ,    0.365213    ,    0.355136    ,    0.492874    ,    0.636551    ,    0.606203    ,    0.419488    ,    0.303746    ,    0.295961    ,    0.386757    ,    0.543078    ,    0.631682    ,    0.616274    ,    0.54881    ,    0.435779    ,    0.354928    ,    0.349629    ,    0.357986    ,    0.377084    ,    0.406821    ,    0.414745    ,    0.388667    ,    0.365775    ,    0.358034    ,    0.354481    ,    0.331796    ,    0.312928    ,    0.289795    ,    0.269988    ,    0.275156    ,    0.30168    ,    0.344686    ,    0.342669    ,    0.340516    ,    0.341702    ,    0.34123    ,    0.339638    ,    0.335941    ,    0.328865    ,    0.316306    ,    0.300792    ,    0.287895    ,    0.280937    ,    0.277562    ,    0.268305    ,    0.267709    ,    0.269481    ,    0.271825    ,    0.277273    ,    0.294315    ,    0.308166    ,    0.31582    ,    0.309319    ,    0.31034    ,    0.301676    ,    0.293569    ,    0.292502    ,    0.293875    ,    0.300176    ,    0.310955    ,    0.320957    ,    0.321658    ,    0.324242    ,    0.328513    ,    0.332555    ,    0.328693    ,    0.32878    ,    0.328279    ,    0.332457    ,    0.320329    ,    0.319607    ,    0.323041    ,    0.326465    ,    0.344499    ,    0.344066    ,    0.357637    ,    0.379484    ,    0.392465    ,    0.404694    ,    0.418146    ,    0.417851    ,    0.420007    ,    0.437852    ,    0.446465    ,    0.43801    ,    0.403699    ,    0.373337    ,    0.346082    },
    {    0.403571    ,    0.391471    ,    0.401989    ,    0.385612    ,    0.333566    ,    0.370904    ,    0.393073    ,    0.395333    ,    0.385796    ,    0.378613    ,    0.371575    ,    0.358615    ,    0.344755    ,    0.333981    ,    0.326953    ,    0.305971    ,    0.289518    ,    0.280049    ,    0.270813    ,    0.275652    ,    0.301046    ,    0.321391    ,    0.328681    ,    0.328179    ,    0.333611    ,    0.350906    ,    0.370767    ,    0.394953    ,    0.421589    ,    0.448366    ,    0.463691    ,    0.468069    ,    0.473624    ,    0.486685    ,    0.499778    ,    0.49109    ,    0.510287    ,    0.501975    ,    0.475074    ,    0.449392    ,    0.429933    ,    0.408689    ,    0.381575    ,    0.393638    ,    0.476847    ,    0.442868    ,    0.299327    ,    0.29447    ,    0.309481    ,    0.316064    ,    0.400091    ,    0.410537    ,    0.450174    ,    0.502772    ,    0.466404    ,    0.397595    ,    0.393413    ,    0.465744    ,    0.513783    ,    0.559289    ,    0.620669    ,    0.584465    ,    0.55384    ,    0.518376    ,    0.484223    ,    0.454775    ,    0.443073    ,    0.426944    ,    0.36994    ,    0.327831    ,    0.314046    ,    0.322733    ,    0.347781    ,    0.350345    ,    0.350822    ,    0.350206    ,    0.345766    ,    0.362103    ,    0.34722    ,    0.346608    ,    0.32513    ,    0.319638    ,    0.307842    ,    0.300623    ,    0.290621    ,    0.295516    ,    0.303911    ,    0.314156    ,    0.335284    ,    0.347638    ,    0.346039    ,    0.354486    ,    0.353358    ,    0.356104    ,    0.347082    ,    0.344216    ,    0.347166    ,    0.34611    ,    0.351916    ,    0.355273    ,    0.360994    ,    0.382717    ,    0.384135    ,    0.38661    ,    0.393601    ,    0.393046    ,    0.386737    ,    0.380735    ,    0.37478    ,    0.376561    ,    0.359449    ,    0.363776    ,    0.359684    ,    0.369184    ,    0.367224    ,    0.376381    ,    0.3804    ,    0.398071    ,    0.429916    ,    0.445808    ,    0.448533    ,    0.454126    ,    0.446519    ,    0.449093    ,    0.45908    ,    0.464987    ,    0.454694    ,    0.43905    },
    {    0.47695    ,    0.493241    ,    0.494972    ,    0.47302    ,    0.447057    ,    0.432746    ,    0.439864    ,    0.449068    ,    0.431207    ,    0.421723    ,    0.419202    ,    0.421877    ,    0.420749    ,    0.41626    ,    0.428748    ,    0.444118    ,    0.426175    ,    0.409739    ,    0.398856    ,    0.403306    ,    0.4086    ,    0.403745    ,    0.404925    ,    0.406731    ,    0.414588    ,    0.43205    ,    0.453949    ,    0.507419    ,    0.542595    ,    0.543028    ,    0.516177    ,    0.514368    ,    0.539997    ,    0.557794    ,    0.540649    ,    0.500856    ,    0.473508    ,    0.491092    ,    0.471037    ,    0.435285    ,    0.390768    ,    0.369467    ,    0.375009    ,    0.389344    ,    0.400108    ,    0.349499    ,    0.314911    ,    0.296255    ,    0.290747    ,    0.281632    ,    0.267076    ,    0.26379    ,    0.292159    ,    0.303923    ,    0.347272    ,    0.406463    ,    0.515122    ,    0.581545    ,    0.636961    ,    0.682236    ,    0.704942    ,    0.69809    ,    0.670453    ,    0.635402    ,    0.608014    ,    0.591373    ,    0.578239    ,    0.554574    ,    0.514728    ,    0.447534    ,    0.376692    ,    0.35976    ,    0.356985    ,    0.367448    ,    0.371589    ,    0.377387    ,    0.394963    ,    0.403337    ,    0.453284    ,    0.470447    ,    0.425541    ,    0.442518    ,    0.413832    ,    0.39724    ,    0.386716    ,    0.370686    ,    0.380817    ,    0.39153    ,    0.393536    ,    0.394288    ,    0.399243    ,    0.408703    ,    0.407471    ,    0.413945    ,    0.396983    ,    0.396612    ,    0.393093    ,    0.381871    ,    0.387489    ,    0.401144    ,    0.41159    ,    0.419541    ,    0.418084    ,    0.419545    ,    0.426776    ,    0.429548    ,    0.437402    ,    0.437659    ,    0.416288    ,    0.412849    ,    0.397225    ,    0.399616    ,    0.401944    ,    0.401195    ,    0.393604    ,    0.396183    ,    0.409823    ,    0.426519    ,    0.438525    ,    0.462001    ,    0.468582    ,    0.468893    ,    0.476517    ,    0.472498    ,    0.474208    ,    0.477376    ,    0.477395    ,    0.473384    },
    {    0.548413    ,    0.563586    ,    0.577517    ,    0.575242    ,    0.56221    ,    0.554001    ,    0.544965    ,    0.541723    ,    0.524241    ,    0.522326    ,    0.512583    ,    0.509659    ,    0.522776    ,    0.526411    ,    0.534279    ,    0.538873    ,    0.528783    ,    0.503805    ,    0.47729    ,    0.455    ,    0.457317    ,    0.465224    ,    0.475209    ,    0.48284    ,    0.48541    ,    0.499541    ,    0.521951    ,    0.585405    ,    0.640491    ,    0.630037    ,    0.606643    ,    0.622725    ,    0.655201    ,    0.604545    ,    0.565912    ,    0.517291    ,    0.486707    ,    0.453816    ,    0.438863    ,    0.413156    ,    0.388989    ,    0.389366    ,    0.393886    ,    0.386384    ,    0.368131    ,    0.300053    ,    0.274124    ,    0.254466    ,    0.255438    ,    0.274221    ,    0.279141    ,    0.262051    ,    0.246368    ,    0.245799    ,    0.251747    ,    0.298588    ,    0.401054    ,    0.527287    ,    0.645685    ,    0.734681    ,    0.809019    ,    0.830847    ,    0.828602    ,    0.790784    ,    0.746111    ,    0.711643    ,    0.693496    ,    0.662848    ,    0.618339    ,    0.585354    ,    0.566475    ,    0.567966    ,    0.600867    ,    0.643934    ,    0.673149    ,    0.679603    ,    0.673482    ,    0.669515    ,    0.676367    ,    0.676913    ,    0.652391    ,    0.637346    ,    0.6081    ,    0.54598    ,    0.61135    ,    0.668704    ,    0.661872    ,    0.635937    ,    0.562535    ,    0.503926    ,    0.479571    ,    0.470803    ,    0.467516    ,    0.45401    ,    0.436656    ,    0.424384    ,    0.421749    ,    0.423523    ,    0.42814    ,    0.430788    ,    0.432722    ,    0.432047    ,    0.433296    ,    0.437542    ,    0.445633    ,    0.449377    ,    0.458164    ,    0.452738    ,    0.445705    ,    0.434871    ,    0.417349    ,    0.423842    ,    0.410351    ,    0.422225    ,    0.420484    ,    0.435328    ,    0.447346    ,    0.457776    ,    0.471063    ,    0.485575    ,    0.498515    ,    0.513123    ,    0.517134    ,    0.523324    ,    0.525183    ,    0.527194    ,    0.528062    ,    0.530026    },
    {    0.546665    ,    0.557298    ,    0.567644    ,    0.57588    ,    0.580334    ,    0.582659    ,    0.579663    ,    0.573351    ,    0.566659    ,    0.564308    ,    0.563618    ,    0.563026    ,    0.561238    ,    0.557168    ,    0.553362    ,    0.548369    ,    0.53828    ,    0.526618    ,    0.517578    ,    0.512485    ,    0.535401    ,    0.546524    ,    0.551818    ,    0.560262    ,    0.565595    ,    0.572469    ,    0.605953    ,    0.628079    ,    0.648739    ,    0.652938    ,    0.658824    ,    0.64783    ,    0.639076    ,    0.598218    ,    0.568972    ,    0.513954    ,    0.504474    ,    0.478868    ,    0.468347    ,    0.465248    ,    0.463336    ,    0.44974    ,    0.436405    ,    0.411768    ,    0.322376    ,    0.252111    ,    0.240279    ,    0.245239    ,    0.253825    ,    0.26792    ,    0.273644    ,    0.279487    ,    0.252273    ,    0.228795    ,    0.23012    ,    0.263049    ,    0.331688    ,    0.400921    ,    0.511655    ,    0.632825    ,    0.725736    ,    0.800174    ,    0.847348    ,    0.856215    ,    0.828335    ,    0.784683    ,    0.746477    ,    0.713201    ,    0.680212    ,    0.677794    ,    0.70857    ,    0.748043    ,    0.777903    ,    0.806288    ,    0.81651    ,    0.798249    ,    0.779318    ,    0.790683    ,    0.810502    ,    0.828245    ,    0.831651    ,    0.810617    ,    0.761342    ,    0.698034    ,    0.726977    ,    0.748391    ,    0.749936    ,    0.727256    ,    0.699674    ,    0.673275    ,    0.658074    ,    0.640412    ,    0.612088    ,    0.558279    ,    0.5151    ,    0.483583    ,    0.46697    ,    0.457263    ,    0.457227    ,    0.45549    ,    0.459369    ,    0.464835    ,    0.469895    ,    0.473718    ,    0.476016    ,    0.477201    ,    0.484302    ,    0.49051    ,    0.487371    ,    0.475108    ,    0.466741    ,    0.456673    ,    0.448494    ,    0.448299    ,    0.470244    ,    0.488281    ,    0.517973    ,    0.528599    ,    0.528458    ,    0.527617    ,    0.529608    ,    0.531483    ,    0.533893    ,    0.537409    ,    0.539195    ,    0.536826    ,    0.536195    ,    0.540541    },
    {    0.5566    ,    0.567562    ,    0.577573    ,    0.58518    ,    0.589533    ,    0.59271    ,    0.595008    ,    0.596918    ,    0.597699    ,    0.598004    ,    0.598481    ,    0.599578    ,    0.597807    ,    0.593535    ,    0.586929    ,    0.578522    ,    0.569781    ,    0.559848    ,    0.550048    ,    0.541254    ,    0.534633    ,    0.541411    ,    0.575494    ,    0.616035    ,    0.610346    ,    0.620885    ,    0.623081    ,    0.615883    ,    0.612622    ,    0.599269    ,    0.586315    ,    0.576809    ,    0.571335    ,    0.568478    ,    0.510774    ,    0.483553    ,    0.464712    ,    0.441106    ,    0.434264    ,    0.408905    ,    0.350395    ,    0.319178    ,    0.301543    ,    0.270353    ,    0.24087    ,    0.226916    ,    0.231761    ,    0.246111    ,    0.254326    ,    0.265898    ,    0.254439    ,    0.241554    ,    0.237734    ,    0.228091    ,    0.235736    ,    0.26646    ,    0.284361    ,    0.335286    ,    0.403426    ,    0.470126    ,    0.531373    ,    0.587495    ,    0.647476    ,    0.684702    ,    0.711085    ,    0.726693    ,    0.72607    ,    0.729421    ,    0.724771    ,    0.709977    ,    0.730961    ,    0.749292    ,    0.77593    ,    0.784131    ,    0.827352    ,    0.854971    ,    0.869937    ,    0.887301    ,    0.916912    ,    0.951882    ,    0.970558    ,    0.963868    ,    0.944938    ,    0.924185    ,    0.916045    ,    0.929644    ,    0.942053    ,    0.93436    ,    0.881289    ,    0.857796    ,    0.807592    ,    0.756458    ,    0.706463    ,    0.659652    ,    0.618841    ,    0.584827    ,    0.55751    ,    0.535633    ,    0.527953    ,    0.519542    ,    0.532067    ,    0.548829    ,    0.559325    ,    0.563267    ,    0.557163    ,    0.548288    ,    0.540603    ,    0.534188    ,    0.530072    ,    0.523163    ,    0.516277    ,    0.509049    ,    0.504098    ,    0.504959    ,    0.5164    ,    0.521187    ,    0.52501    ,    0.527749    ,    0.529825    ,    0.529177    ,    0.528135    ,    0.527659    ,    0.527435    ,    0.532066    ,    0.537053    ,    0.540502    ,    0.542776    ,    0.549907    },
    {    0.586284    ,    0.595041    ,    0.60229    ,    0.60825    ,    0.611846    ,    0.613949    ,    0.615998    ,    0.618001    ,    0.6197    ,    0.619892    ,    0.619072    ,    0.617516    ,    0.614783    ,    0.610662    ,    0.606008    ,    0.602263    ,    0.597265    ,    0.592646    ,    0.587276    ,    0.581916    ,    0.576523    ,    0.569611    ,    0.562837    ,    0.557802    ,    0.556588    ,    0.561664    ,    0.563338    ,    0.58142    ,    0.580139    ,    0.570442    ,    0.574111    ,    0.554184    ,    0.52558    ,    0.503745    ,    0.490508    ,    0.470228    ,    0.438377    ,    0.435755    ,    0.440566    ,    0.457912    ,    0.397322    ,    0.343584    ,    0.290292    ,    0.262388    ,    0.241021    ,    0.230141    ,    0.223631    ,    0.222258    ,    0.223411    ,    0.227741    ,    0.23162    ,    0.2393    ,    0.241135    ,    0.241096    ,    0.246981    ,    0.268171    ,    0.281914    ,    0.343388    ,    0.403148    ,    0.41752    ,    0.393865    ,    0.390755    ,    0.425121    ,    0.466418    ,    0.496642    ,    0.54806    ,    0.585801    ,    0.584398    ,    0.592872    ,    0.600111    ,    0.648015    ,    0.682008    ,    0.701297    ,    0.690091    ,    0.692032    ,    0.714901    ,    0.73437    ,    0.760222    ,    0.793399    ,    0.828964    ,    0.86293    ,    0.885755    ,    0.901669    ,    0.901322    ,    0.886729    ,    0.884558    ,    0.893403    ,    0.911292    ,    0.927895    ,    0.932254    ,    0.911953    ,    0.871783    ,    0.821347    ,    0.767781    ,    0.714855    ,    0.665117    ,    0.620904    ,    0.607583    ,    0.621492    ,    0.65989    ,    0.692743    ,    0.697846    ,    0.679508    ,    0.663219    ,    0.643002    ,    0.622581    ,    0.608155    ,    0.595516    ,    0.586258    ,    0.5801    ,    0.57422    ,    0.568236    ,    0.563396    ,    0.559313    ,    0.555022    ,    0.551444    ,    0.547821    ,    0.546896    ,    0.547987    ,    0.547716    ,    0.547171    ,    0.546211    ,    0.546692    ,    0.549008    ,    0.553838    ,    0.559833    ,    0.567583    ,    0.576932    },
    {    0.617071    ,    0.619311    ,    0.621593    ,    0.624272    ,    0.626391    ,    0.627607    ,    0.627932    ,    0.628237    ,    0.628546    ,    0.629243    ,    0.628937    ,    0.627094    ,    0.624258    ,    0.621109    ,    0.617025    ,    0.611229    ,    0.606032    ,    0.599525    ,    0.593188    ,    0.586839    ,    0.581166    ,    0.576163    ,    0.572355    ,    0.566734    ,    0.561695    ,    0.554049    ,    0.548091    ,    0.541418    ,    0.535747    ,    0.530654    ,    0.528834    ,    0.529522    ,    0.516365    ,    0.526749    ,    0.521179    ,    0.540182    ,    0.549062    ,    0.546206    ,    0.522059    ,    0.478487    ,    0.437266    ,    0.411249    ,    0.408614    ,    0.392208    ,    0.397756    ,    0.386311    ,    0.366937    ,    0.3571    ,    0.343689    ,    0.363259    ,    0.3723    ,    0.340388    ,    0.367077    ,    0.370005    ,    0.38559    ,    0.417075    ,    0.455756    ,    0.476227    ,    0.480566    ,    0.475053    ,    0.462423    ,    0.454848    ,    0.451477    ,    0.455236    ,    0.462197    ,    0.475002    ,    0.487631    ,    0.515453    ,    0.548717    ,    0.58034    ,    0.613527    ,    0.637987    ,    0.658547    ,    0.670581    ,    0.679947    ,    0.693936    ,    0.709507    ,    0.730152    ,    0.750167    ,    0.763169    ,    0.775557    ,    0.781114    ,    0.785921    ,    0.793455    ,    0.808189    ,    0.820063    ,    0.831009    ,    0.839988    ,    0.850709    ,    0.854975    ,    0.855721    ,    0.849448    ,    0.837211    ,    0.823907    ,    0.808308    ,    0.79434    ,    0.780579    ,    0.7679    ,    0.758925    ,    0.754801    ,    0.747161    ,    0.732141    ,    0.717786    ,    0.702023    ,    0.684808    ,    0.668039    ,    0.653796    ,    0.644872    ,    0.636554    ,    0.631005    ,    0.626423    ,    0.62118    ,    0.615732    ,    0.611163    ,    0.606204    ,    0.60245    ,    0.599817    ,    0.597841    ,    0.596458    ,    0.595824    ,    0.596609    ,    0.597875    ,    0.600978    ,    0.604768    ,    0.609    ,    0.613162    ,    0.615316    ,    0.617055    },
    {    0.634331    ,    0.634441    ,    0.634313    ,    0.633876    ,    0.633728    ,    0.633557    ,    0.633013    ,    0.632444    ,    0.631806    ,    0.630372    ,    0.628234    ,    0.626046    ,    0.622219    ,    0.618657    ,    0.614702    ,    0.610255    ,    0.606253    ,    0.602671    ,    0.599302    ,    0.595878    ,    0.591823    ,    0.587433    ,    0.583034    ,    0.577838    ,    0.572506    ,    0.566689    ,    0.56145    ,    0.556107    ,    0.551373    ,    0.549494    ,    0.546386    ,    0.540495    ,    0.535102    ,    0.534172    ,    0.532803    ,    0.534846    ,    0.537746    ,    0.536753    ,    0.530947    ,    0.530097    ,    0.527323    ,    0.518805    ,    0.514042    ,    0.51194    ,    0.512319    ,    0.511206    ,    0.510739    ,    0.51209    ,    0.514271    ,    0.51279    ,    0.513792    ,    0.518753    ,    0.518669    ,    0.518315    ,    0.519969    ,    0.517492    ,    0.520541    ,    0.521937    ,    0.526474    ,    0.53275    ,    0.539701    ,    0.547402    ,    0.554625    ,    0.562557    ,    0.570444    ,    0.579137    ,    0.588341    ,    0.598913    ,    0.609991    ,    0.61968    ,    0.630857    ,    0.641437    ,    0.653864    ,    0.663336    ,    0.675722    ,    0.686146    ,    0.696364    ,    0.706759    ,    0.714314    ,    0.721886    ,    0.729138    ,    0.733568    ,    0.739741    ,    0.745834    ,    0.750617    ,    0.754691    ,    0.758746    ,    0.761338    ,    0.763799    ,    0.764242    ,    0.763725    ,    0.761552    ,    0.757759    ,    0.75329    ,    0.746717    ,    0.738784    ,    0.730118    ,    0.720424    ,    0.71194    ,    0.703898    ,    0.696673    ,    0.691002    ,    0.686745    ,    0.683038    ,    0.678177    ,    0.672834    ,    0.667274    ,    0.661621    ,    0.657095    ,    0.65175    ,    0.646724    ,    0.642202    ,    0.638283    ,    0.635809    ,    0.632799    ,    0.630428    ,    0.628958    ,    0.628904    ,    0.628389    ,    0.627975    ,    0.627511    ,    0.62775    ,    0.628881    ,    0.630604    ,    0.631686    ,    0.632954    ,    0.633887    ,    0.634211    },
    {    0.621405    ,    0.621713    ,    0.622018    ,    0.6223    ,    0.622238    ,    0.622276    ,    0.621944    ,    0.621337    ,    0.621001    ,    0.61982    ,    0.617922    ,    0.61643    ,    0.614695    ,    0.613096    ,    0.610679    ,    0.608682    ,    0.60656    ,    0.604542    ,    0.60268    ,    0.600565    ,    0.598568    ,    0.595768    ,    0.594223    ,    0.592753    ,    0.591219    ,    0.589148    ,    0.587489    ,    0.586902    ,    0.586205    ,    0.585902    ,    0.586176    ,    0.585486    ,    0.584513    ,    0.583457    ,    0.583672    ,    0.582746    ,    0.583847    ,    0.581673    ,    0.581833    ,    0.579827    ,    0.578111    ,    0.578184    ,    0.578513    ,    0.577551    ,    0.576426    ,    0.575743    ,    0.575996    ,    0.575039    ,    0.574992    ,    0.576462    ,    0.576386    ,    0.575482    ,    0.576291    ,    0.576963    ,    0.579003    ,    0.58067    ,    0.58202    ,    0.585081    ,    0.588228    ,    0.591406    ,    0.595775    ,    0.601291    ,    0.606123    ,    0.611131    ,    0.616506    ,    0.620439    ,    0.624695    ,    0.629512    ,    0.633875    ,    0.635985    ,    0.638591    ,    0.641288    ,    0.644317    ,    0.646477    ,    0.647798    ,    0.649692    ,    0.651395    ,    0.653303    ,    0.654135    ,    0.654973    ,    0.656141    ,    0.657336    ,    0.658314    ,    0.659139    ,    0.660179    ,    0.661069    ,    0.661825    ,    0.662954    ,    0.6637    ,    0.665072    ,    0.665884    ,    0.665708    ,    0.665493    ,    0.66424    ,    0.664556    ,    0.663297    ,    0.662908    ,    0.662287    ,    0.662063    ,    0.660808    ,    0.659024    ,    0.657698    ,    0.656077    ,    0.654338    ,    0.652734    ,    0.65042    ,    0.648125    ,    0.645718    ,    0.643904    ,    0.64227    ,    0.640554    ,    0.638203    ,    0.636013    ,    0.634368    ,    0.632442    ,    0.630395    ,    0.629341    ,    0.628809    ,    0.627516    ,    0.625991    ,    0.624743    ,    0.624105    ,    0.623562    ,    0.623169    ,    0.622136    ,    0.622278    ,    0.622501    ,    0.622327    }
};


double LGRTC_RCP45[64][128]={
    {    1.13449    ,    1.12999    ,    1.12308    ,    1.12238    ,    1.12218    ,    1.11832    ,    1.1118    ,    1.10758    ,    1.1081    ,    1.10803    ,    1.10439    ,    1.09826    ,    1.09942    ,    1.099    ,    1.10044    ,    1.10104    ,    1.1024    ,    1.10771    ,    1.11506    ,    1.11981    ,    1.12329    ,    1.12644    ,    1.1327    ,    1.13713    ,    1.14026    ,    1.14557    ,    1.15194    ,    1.16223    ,    1.16424    ,    1.16098    ,    1.16756    ,    1.16696    ,    1.171    ,    1.17418    ,    1.17156    ,    1.17121    ,    1.17645    ,    1.17945    ,    1.18081    ,    1.18399    ,    1.18708    ,    1.19086    ,    1.19135    ,    1.19442    ,    1.19364    ,    1.19417    ,    1.19703    ,    1.20068    ,    1.20025    ,    1.20322    ,    1.20778    ,    1.21332    ,    1.21384    ,    1.2156    ,    1.21378    ,    1.21456    ,    1.21998    ,    1.22285    ,    1.22373    ,    1.22396    ,    1.22168    ,    1.22716    ,    1.23289    ,    1.24121    ,    1.24458    ,    1.24982    ,    1.25403    ,    1.25565    ,    1.25473    ,    1.26996    ,    1.28427    ,    1.27906    ,    1.28277    ,    1.28964    ,    1.28059    ,    1.29445    ,    1.30541    ,    1.30852    ,    1.31674    ,    1.3237    ,    1.33441    ,    1.3208    ,    1.32835    ,    1.3259    ,    1.33112    ,    1.33023    ,    1.32869    ,    1.33415    ,    1.33699    ,    1.34193    ,    1.34098    ,    1.3476    ,    1.33555    ,    1.35153    ,    1.33298    ,    1.34314    ,    1.33332    ,    1.32751    ,    1.32169    ,    1.32021    ,    1.31871    ,    1.32516    ,    1.31947    ,    1.31984    ,    1.31104    ,    1.30443    ,    1.2874    ,    1.26156    ,    1.25763    ,    1.23646    ,    1.23281    ,    1.23107    ,    1.22532    ,    1.22516    ,    1.21913    ,    1.20618    ,    1.20439    ,    1.20779    ,    1.20362    ,    1.19669    ,    1.18847    ,    1.18049    ,    1.17043    ,    1.16434    ,    1.16659    ,    1.15421    ,    1.14953    ,    1.1436    },
    {    0.927385    ,    0.923142    ,    0.919532    ,    0.920952    ,    0.914019    ,    0.915578    ,    0.909447    ,    0.909171    ,    0.909106    ,    0.907769    ,    0.904065    ,    0.902558    ,    0.902596    ,    0.901953    ,    0.903047    ,    0.904733    ,    0.900466    ,    0.894465    ,    0.894012    ,    0.897281    ,    0.902999    ,    0.910254    ,    0.922725    ,    0.93642    ,    0.953245    ,    0.969256    ,    0.995124    ,    1.03032    ,    1.0688    ,    1.07332    ,    1.07272    ,    1.08264    ,    1.08461    ,    1.07871    ,    1.0648    ,    1.06438    ,    1.06177    ,    1.05941    ,    1.05916    ,    1.06049    ,    1.05934    ,    1.05715    ,    1.05936    ,    1.0653    ,    1.06014    ,    1.05804    ,    1.0647    ,    1.06639    ,    1.07049    ,    1.07161    ,    1.07507    ,    1.07871    ,    1.07591    ,    1.08412    ,    1.09247    ,    1.103    ,    1.10644    ,    1.12029    ,    1.13448    ,    1.14605    ,    1.16082    ,    1.17075    ,    1.18876    ,    1.19545    ,    1.21434    ,    1.22724    ,    1.24289    ,    1.24978    ,    1.2536    ,    1.24907    ,    1.24413    ,    1.23673    ,    1.24777    ,    1.23989    ,    1.2464    ,    1.25792    ,    1.25506    ,    1.26711    ,    1.28935    ,    1.29826    ,    1.30839    ,    1.29462    ,    1.28366    ,    1.27006    ,    1.31077    ,    1.30338    ,    1.31373    ,    1.3161    ,    1.28488    ,    1.29693    ,    1.29267    ,    1.29688    ,    1.30513    ,    1.30598    ,    1.28939    ,    1.28553    ,    1.28879    ,    1.28612    ,    1.27972    ,    1.28567    ,    1.29169    ,    1.24563    ,    1.26645    ,    1.26034    ,    1.25078    ,    1.23803    ,    1.22807    ,    1.22051    ,    1.19029    ,    1.17847    ,    1.16333    ,    1.13797    ,    1.12656    ,    1.10831    ,    1.09023    ,    1.07782    ,    1.05672    ,    1.03689    ,    1.02846    ,    1.00349    ,    0.99534    ,    0.991236    ,    0.986218    ,    0.978253    ,    0.966749    ,    0.946386    ,    0.943767    ,    0.932621    },
    {    0.975961    ,    0.969577    ,    0.973115    ,    0.974846    ,    0.97088    ,    0.966774    ,    0.955226    ,    0.950685    ,    0.952034    ,    0.930138    ,    0.924154    ,    0.909366    ,    0.896501    ,    0.892562    ,    0.884248    ,    0.874123    ,    0.869849    ,    0.860462    ,    0.852492    ,    0.84721    ,    0.840196    ,    0.834906    ,    0.834911    ,    0.840932    ,    0.848799    ,    0.865654    ,    0.891117    ,    0.918329    ,    0.940234    ,    0.948379    ,    0.968283    ,    0.977239    ,    0.980155    ,    0.980769    ,    0.983566    ,    0.991932    ,    0.996169    ,    1.00076    ,    1.01352    ,    1.03104    ,    1.05445    ,    1.0708    ,    1.0864    ,    1.08361    ,    1.07808    ,    1.08107    ,    1.08471    ,    1.06838    ,    1.05542    ,    1.03895    ,    1.03564    ,    1.03388    ,    1.0333    ,    1.03408    ,    1.03507    ,    1.03931    ,    1.04743    ,    1.04901    ,    1.05893    ,    1.07527    ,    1.09758    ,    1.11483    ,    1.13599    ,    1.14774    ,    1.15749    ,    1.17456    ,    1.19745    ,    1.2256    ,    1.23393    ,    1.24249    ,    1.24747    ,    1.26524    ,    1.27188    ,    1.28437    ,    1.29925    ,    1.30367    ,    1.31451    ,    1.32015    ,    1.32034    ,    1.31771    ,    1.32866    ,    1.32437    ,    1.35357    ,    1.34117    ,    1.33321    ,    1.34021    ,    1.34531    ,    1.32689    ,    1.33904    ,    1.34786    ,    1.33324    ,    1.31684    ,    1.32495    ,    1.30814    ,    1.3038    ,    1.2908    ,    1.32729    ,    1.29531    ,    1.28568    ,    1.28587    ,    1.25885    ,    1.24386    ,    1.24956    ,    1.23557    ,    1.23982    ,    1.22381    ,    1.21706    ,    1.22138    ,    1.20193    ,    1.18042    ,    1.15298    ,    1.13796    ,    1.11552    ,    1.09481    ,    1.07775    ,    1.05941    ,    1.03394    ,    1.00782    ,    0.992813    ,    0.971287    ,    0.961288    ,    0.952965    ,    0.949862    ,    0.955342    ,    0.973097    ,    0.97205    ,    0.973396    ,    0.98166    },
    {    0.963412    ,    0.964219    ,    0.962121    ,    0.953274    ,    0.958965    ,    0.958555    ,    0.959956    ,    0.948621    ,    0.932857    ,    0.912288    ,    0.909928    ,    0.900718    ,    0.879679    ,    0.864204    ,    0.840797    ,    0.824366    ,    0.812914    ,    0.801854    ,    0.794106    ,    0.800307    ,    0.809794    ,    0.822643    ,    0.844399    ,    0.865611    ,    0.883638    ,    0.886101    ,    0.895058    ,    0.89995    ,    0.906168    ,    0.909522    ,    0.910386    ,    0.906722    ,    0.917402    ,    0.925268    ,    0.930296    ,    0.951594    ,    0.977875    ,    1.00779    ,    1.03147    ,    1.05985    ,    1.08892    ,    1.12067    ,    1.14813    ,    1.16421    ,    1.17604    ,    1.16184    ,    1.14983    ,    1.15191    ,    1.16068    ,    1.16629    ,    1.15624    ,    1.1316    ,    1.08868    ,    1.07735    ,    1.04567    ,    1.01845    ,    1.00662    ,    1.0032    ,    1.01131    ,    1.02416    ,    1.02974    ,    1.0589    ,    1.10067    ,    1.12589    ,    1.14038    ,    1.14834    ,    1.1476    ,    1.19792    ,    1.20268    ,    1.24594    ,    1.28492    ,    1.30173    ,    1.3092    ,    1.30166    ,    1.32816    ,    1.32958    ,    1.25798    ,    1.27511    ,    1.27959    ,    1.30815    ,    1.39101    ,    1.36845    ,    1.36145    ,    1.31989    ,    1.29622    ,    1.26168    ,    1.2398    ,    1.19735    ,    1.19614    ,    1.20916    ,    1.1656    ,    1.18436    ,    1.21065    ,    1.1915    ,    1.22395    ,    1.25481    ,    1.28815    ,    1.32661    ,    1.34256    ,    1.31357    ,    1.30816    ,    1.28781    ,    1.24337    ,    1.24958    ,    1.22726    ,    1.21801    ,    1.21427    ,    1.21667    ,    1.20223    ,    1.18884    ,    1.16825    ,    1.14831    ,    1.13082    ,    1.11975    ,    1.10072    ,    1.08222    ,    1.06345    ,    1.03286    ,    1.00321    ,    0.973318    ,    0.946887    ,    0.931696    ,    0.934255    ,    0.939602    ,    0.97633    ,    0.973377    ,    0.986159    ,    0.96836    },
    {    1.00715    ,    1.03238    ,    1.049    ,    1.06158    ,    1.08658    ,    1.10926    ,    1.13633    ,    1.13797    ,    1.10802    ,    1.10091    ,    1.08957    ,    0.994871    ,    0.948388    ,    0.879432    ,    0.848274    ,    0.83683    ,    0.821813    ,    0.816918    ,    0.818639    ,    0.821402    ,    0.820461    ,    0.822788    ,    0.836899    ,    0.834107    ,    0.838754    ,    0.85104    ,    0.863564    ,    0.867936    ,    0.866118    ,    0.86362    ,    0.869521    ,    0.884154    ,    0.899905    ,    0.918121    ,    0.942347    ,    0.973356    ,    0.997831    ,    1.03239    ,    1.06456    ,    1.08438    ,    1.09305    ,    1.08675    ,    1.11054    ,    1.11981    ,    1.1113    ,    1.11887    ,    1.12169    ,    1.12891    ,    1.1548    ,    1.18202    ,    1.20036    ,    1.20231    ,    1.20534    ,    1.17231    ,    1.12035    ,    1.06609    ,    1.02337    ,    0.996389    ,    0.98806    ,    1.00691    ,    1.04154    ,    1.07887    ,    1.1001    ,    1.13    ,    1.16346    ,    1.17788    ,    1.19038    ,    1.24139    ,    1.23501    ,    1.22987    ,    1.24056    ,    1.26377    ,    1.26177    ,    1.25213    ,    1.27422    ,    1.30954    ,    1.24952    ,    1.24815    ,    1.28251    ,    1.24181    ,    1.25376    ,    1.22084    ,    1.22546    ,    1.20935    ,    1.20563    ,    1.19105    ,    1.1721    ,    1.15512    ,    1.14735    ,    1.13906    ,    1.14035    ,    1.1385    ,    1.13828    ,    1.14448    ,    1.16309    ,    1.18527    ,    1.23269    ,    1.26449    ,    1.30206    ,    1.30767    ,    1.2494    ,    1.25573    ,    1.23267    ,    1.24137    ,    1.25707    ,    1.23212    ,    1.21277    ,    1.1963    ,    1.18921    ,    1.15146    ,    1.12947    ,    1.10755    ,    1.09469    ,    1.08859    ,    1.07414    ,    1.05915    ,    1.04623    ,    1.03181    ,    1.01423    ,    0.983793    ,    0.95633    ,    0.934692    ,    0.885277    ,    0.90161    ,    0.910376    ,    0.927543    ,    0.937319    ,    0.967277    },
    {    0.870947    ,    0.862501    ,    0.86061    ,    0.86951    ,    0.896527    ,    0.934497    ,    0.969616    ,    1.00325    ,    1.0375    ,    1.0691    ,    1.08999    ,    1.09756    ,    1.10076    ,    1.10069    ,    1.09748    ,    1.08405    ,    1.07094    ,    1.05661    ,    1.02042    ,    0.974177    ,    0.988481    ,    1.00373    ,    1.00708    ,    1.01439    ,    1.01113    ,    1.01958    ,    1.02778    ,    1.00846    ,    0.979095    ,    0.979414    ,    0.979065    ,    0.964997    ,    0.951287    ,    0.963442    ,    0.97519    ,    0.998379    ,    1.01818    ,    1.03584    ,    1.04681    ,    1.04936    ,    1.06462    ,    1.06438    ,    1.06769    ,    1.07994    ,    1.06411    ,    1.04991    ,    1.06309    ,    1.09963    ,    1.14324    ,    1.18651    ,    1.22856    ,    1.26859    ,    1.3007    ,    1.3088    ,    1.30388    ,    1.27864    ,    1.25239    ,    1.18941    ,    1.05959    ,    0.957768    ,    0.942308    ,    0.968471    ,    1.0158    ,    1.04723    ,    1.06363    ,    1.08325    ,    1.11605    ,    1.13305    ,    1.14989    ,    1.1506    ,    1.15171    ,    1.14773    ,    1.14999    ,    1.16432    ,    1.15881    ,    1.16947    ,    1.18803    ,    1.20304    ,    1.22176    ,    1.24242    ,    1.24797    ,    1.23946    ,    1.21662    ,    1.1981    ,    1.17261    ,    1.15515    ,    1.14255    ,    1.14851    ,    1.16679    ,    1.16033    ,    1.11669    ,    1.09994    ,    1.08786    ,    1.09541    ,    1.10456    ,    1.13265    ,    1.16795    ,    1.19989    ,    1.21252    ,    1.2133    ,    1.18373    ,    1.17259    ,    1.17531    ,    1.17402    ,    1.19286    ,    1.1876    ,    1.16612    ,    1.14751    ,    1.10032    ,    1.07084    ,    1.07255    ,    1.0476    ,    1.03723    ,    1.02786    ,    1.01309    ,    0.999158    ,    0.989733    ,    0.983846    ,    0.968348    ,    0.952087    ,    0.922064    ,    0.903633    ,    0.889671    ,    0.920991    ,    0.931837    ,    0.926343    ,    0.91111    ,    0.900461    },
    {    0.7585    ,    0.755071    ,    0.763606    ,    0.788392    ,    0.826917    ,    0.86178    ,    0.889266    ,    0.920079    ,    0.955148    ,    0.988575    ,    1.01076    ,    1.02004    ,    1.01926    ,    1.0131    ,    1.00369    ,    0.998365    ,    1.00124    ,    1.00704    ,    1.01636    ,    1.03435    ,    1.05788    ,    1.07648    ,    1.0855    ,    1.0848    ,    1.07751    ,    1.06589    ,    1.06071    ,    1.05467    ,    1.06041    ,    1.08151    ,    1.10357    ,    1.11875    ,    1.13075    ,    1.1411    ,    1.14895    ,    1.15935    ,    1.16207    ,    1.1379    ,    1.16045    ,    1.10092    ,    1.05948    ,    1.08228    ,    1.10198    ,    1.10953    ,    1.10814    ,    1.10288    ,    1.11235    ,    1.13593    ,    1.17285    ,    1.21445    ,    1.25854    ,    1.29274    ,    1.31735    ,    1.32612    ,    1.32479    ,    1.3173    ,    1.30943    ,    1.29786    ,    1.27016    ,    1.22576    ,    1.177    ,    1.10009    ,    1.06083    ,    1.02504    ,    0.98267    ,    0.968222    ,    0.96165    ,    0.963715    ,    0.956453    ,    0.954897    ,    0.958919    ,    0.964067    ,    0.962863    ,    0.958002    ,    0.964255    ,    0.97638    ,    0.988176    ,    1.01272    ,    1.03867    ,    1.07251    ,    1.1142    ,    1.15262    ,    1.17566    ,    1.16869    ,    1.13902    ,    1.09579    ,    1.08246    ,    1.09321    ,    1.1314    ,    1.19338    ,    1.12027    ,    1.05529    ,    1.04465    ,    1.04817    ,    1.06188    ,    1.07441    ,    1.09591    ,    1.11525    ,    1.12021    ,    1.13146    ,    1.14247    ,    1.15327    ,    1.14191    ,    1.12491    ,    1.08858    ,    1.05226    ,    1.03579    ,    1.0185    ,    1.01286    ,    1.00751    ,    1.00408    ,    0.99791    ,    0.989615    ,    0.982219    ,    0.970518    ,    0.94967    ,    0.92803    ,    0.901537    ,    0.875202    ,    0.867577    ,    0.855767    ,    0.835277    ,    0.832791    ,    0.847866    ,    0.860371    ,    0.850778    ,    0.819856    ,    0.792889    },
    {    0.722766    ,    0.719442    ,    0.721287    ,    0.733288    ,    0.745352    ,    0.75595    ,    0.774344    ,    0.807136    ,    0.839024    ,    0.869528    ,    0.891803    ,    0.906968    ,    0.915766    ,    0.913082    ,    0.909761    ,    0.908743    ,    0.916426    ,    0.919567    ,    0.908994    ,    0.892114    ,    0.883388    ,    0.8855    ,    0.89752    ,    0.912779    ,    0.931639    ,    0.945321    ,    0.955537    ,    0.9708    ,    0.990459    ,    1.00372    ,    1.01335    ,    1.01544    ,    1.01379    ,    1.01769    ,    1.02319    ,    1.03292    ,    1.05234    ,    1.08482    ,    1.11779    ,    1.119    ,    1.1122    ,    1.11665    ,    1.12538    ,    1.15396    ,    1.12357    ,    1.10273    ,    1.08883    ,    1.09198    ,    1.11918    ,    1.15597    ,    1.19533    ,    1.2359    ,    1.26637    ,    1.28163    ,    1.28676    ,    1.29225    ,    1.29335    ,    1.29215    ,    1.29119    ,    1.28597    ,    1.27709    ,    1.26714    ,    1.25464    ,    1.24695    ,    1.24615    ,    1.23822    ,    1.21403    ,    1.19652    ,    1.19319    ,    1.20727    ,    1.22119    ,    1.22968    ,    1.22662    ,    1.20886    ,    1.15945    ,    1.11651    ,    1.08559    ,    1.09559    ,    1.07846    ,    0.968346    ,    0.924382    ,    0.977919    ,    1.00307    ,    1.03217    ,    1.02327    ,    1.02514    ,    1.03624    ,    1.06154    ,    1.10242    ,    1.15803    ,    1.12259    ,    1.0403    ,    1.01402    ,    1.00998    ,    0.990233    ,    0.970514    ,    0.961983    ,    0.967429    ,    0.977772    ,    0.989332    ,    0.995337    ,    1.00065    ,    0.983685    ,    0.967567    ,    0.947363    ,    0.919391    ,    0.928219    ,    0.907218    ,    0.884212    ,    0.875925    ,    0.869743    ,    0.871131    ,    0.876551    ,    0.874274    ,    0.857344    ,    0.839489    ,    0.844676    ,    0.816369    ,    0.807486    ,    0.807317    ,    0.794346    ,    0.801129    ,    0.809586    ,    0.80574    ,    0.797032    ,    0.785388    ,    0.768134    ,    0.749936    },
    {    0.613635    ,    0.622939    ,    0.63272    ,    0.636477    ,    0.641422    ,    0.64801    ,    0.655133    ,    0.674584    ,    0.696821    ,    0.715035    ,    0.733742    ,    0.747906    ,    0.750254    ,    0.748333    ,    0.745328    ,    0.737694    ,    0.727156    ,    0.713415    ,    0.698482    ,    0.685754    ,    0.681823    ,    0.683484    ,    0.688623    ,    0.698471    ,    0.713031    ,    0.733415    ,    0.753986    ,    0.772811    ,    0.796391    ,    0.821615    ,    0.842946    ,    0.858556    ,    0.859887    ,    0.856791    ,    0.859102    ,    0.869173    ,    0.886381    ,    0.914937    ,    0.953355    ,    0.989477    ,    1.01298    ,    1.03702    ,    1.07728    ,    1.13007    ,    1.10867    ,    1.05683    ,    1.01659    ,    1.0106    ,    1.02504    ,    1.04585    ,    1.07341    ,    1.10178    ,    1.12793    ,    1.141    ,    1.14676    ,    1.15344    ,    1.15595    ,    1.15672    ,    1.16218    ,    1.17882    ,    1.20318    ,    1.2208    ,    1.22158    ,    1.20439    ,    1.17625    ,    1.14364    ,    1.10369    ,    1.07244    ,    1.05397    ,    1.04986    ,    1.04949    ,    1.04846    ,    1.04819    ,    1.05695    ,    1.06646    ,    1.06469    ,    1.0591    ,    1.04545    ,    1.03613    ,    1.01877    ,    0.988422    ,    0.97781    ,    0.99076    ,    1.00762    ,    1.04018    ,    1.02275    ,    0.971174    ,    0.947564    ,    0.948281    ,    0.94801    ,    0.912742    ,    0.881021    ,    0.88487    ,    0.940019    ,    0.991615    ,    1.04949    ,    1.04373    ,    1.00199    ,    1.0285    ,    1.04404    ,    1.00804    ,    1.05514    ,    1.06376    ,    1.0536    ,    1.06191    ,    1.07448    ,    1.06976    ,    1.02592    ,    0.944256    ,    0.91134    ,    0.856771    ,    0.824195    ,    0.790639    ,    0.752438    ,    0.706851    ,    0.681727    ,    0.675788    ,    0.63306    ,    0.610343    ,    0.600604    ,    0.594729    ,    0.601747    ,    0.624818    ,    0.64832    ,    0.652571    ,    0.638695    ,    0.621706    ,    0.614485    },
    {    0.480956    ,    0.47381    ,    0.473304    ,    0.485833    ,    0.5065    ,    0.517648    ,    0.514549    ,    0.511421    ,    0.521227    ,    0.543305    ,    0.570213    ,    0.593981    ,    0.60814    ,    0.620697    ,    0.619597    ,    0.602744    ,    0.574722    ,    0.550602    ,    0.534048    ,    0.521245    ,    0.51103    ,    0.511304    ,    0.515546    ,    0.523198    ,    0.529723    ,    0.540133    ,    0.555912    ,    0.575098    ,    0.597882    ,    0.618537    ,    0.637274    ,    0.655744    ,    0.669729    ,    0.676008    ,    0.678187    ,    0.681401    ,    0.694505    ,    0.720187    ,    0.757896    ,    0.797296    ,    0.838529    ,    0.869594    ,    0.883409    ,    0.902754    ,    0.924817    ,    0.915184    ,    0.89868    ,    0.894862    ,    0.903121    ,    0.918621    ,    0.937113    ,    0.956253    ,    0.976021    ,    0.990593    ,    1.00237    ,    1.01046    ,    1.01192    ,    1.01584    ,    1.02275    ,    1.03594    ,    1.05322    ,    1.07281    ,    1.08184    ,    1.07441    ,    1.05041    ,    1.03112    ,    1.01356    ,    0.990775    ,    0.974645    ,    0.967664    ,    0.963957    ,    0.958689    ,    0.94563    ,    0.928858    ,    0.915674    ,    0.916433    ,    0.927808    ,    0.93771    ,    0.94573    ,    0.951802    ,    0.955933    ,    0.956675    ,    0.955764    ,    0.942097    ,    0.896227    ,    0.834268    ,    0.778455    ,    0.726167    ,    0.67312    ,    0.61598    ,    0.56412    ,    0.533124    ,    0.526709    ,    0.543884    ,    0.615333    ,    0.752641    ,    0.864106    ,    0.93375    ,    0.983491    ,    1.01595    ,    1.03495    ,    1.05457    ,    1.06268    ,    1.06488    ,    1.06388    ,    1.04638    ,    1.00485    ,    0.955536    ,    0.919018    ,    0.896341    ,    0.880228    ,    0.868047    ,    0.843498    ,    0.812317    ,    0.784577    ,    0.754583    ,    0.714261    ,    0.652755    ,    0.586186    ,    0.527192    ,    0.496177    ,    0.508301    ,    0.519846    ,    0.524104    ,    0.52092    ,    0.508144    ,    0.494011    ,    0.48769    },
    {    0.381221    ,    0.380026    ,    0.373802    ,    0.370488    ,    0.372055    ,    0.376867    ,    0.371766    ,    0.360395    ,    0.362603    ,    0.380246    ,    0.403503    ,    0.42549    ,    0.453531    ,    0.47543    ,    0.477255    ,    0.468359    ,    0.451512    ,    0.440556    ,    0.437186    ,    0.439003    ,    0.43708    ,    0.436158    ,    0.434309    ,    0.438026    ,    0.445736    ,    0.458349    ,    0.474729    ,    0.492688    ,    0.510044    ,    0.525925    ,    0.53892    ,    0.552149    ,    0.565563    ,    0.573804    ,    0.577458    ,    0.581516    ,    0.592112    ,    0.605855    ,    0.6181    ,    0.634582    ,    0.664981    ,    0.721753    ,    0.770295    ,    0.809521    ,    0.84615    ,    0.84994    ,    0.838605    ,    0.799426    ,    0.782889    ,    0.79079    ,    0.807981    ,    0.823534    ,    0.844639    ,    0.865663    ,    0.881911    ,    0.890412    ,    0.894926    ,    0.902045    ,    0.911997    ,    0.918239    ,    0.925268    ,    0.933824    ,    0.9411    ,    0.940926    ,    0.929296    ,    0.915126    ,    0.909688    ,    0.899545    ,    0.889125    ,    0.884261    ,    0.870705    ,    0.843147    ,    0.811949    ,    0.779609    ,    0.751085    ,    0.738469    ,    0.742545    ,    0.75371    ,    0.771771    ,    0.790671    ,    0.814258    ,    0.844863    ,    0.8724    ,    0.891977    ,    0.899197    ,    0.896567    ,    0.886242    ,    0.86514    ,    0.823302    ,    0.729279    ,    0.61503    ,    0.525018    ,    0.472126    ,    0.45555    ,    0.476052    ,    0.55376    ,    0.62022    ,    0.672691    ,    0.720363    ,    0.762583    ,    0.797076    ,    0.823158    ,    0.827362    ,    0.820266    ,    0.811721    ,    0.799457    ,    0.781381    ,    0.760001    ,    0.736999    ,    0.719171    ,    0.708056    ,    0.697643    ,    0.688388    ,    0.689717    ,    0.69917    ,    0.703235    ,    0.685038    ,    0.630259    ,    0.583817    ,    0.558663    ,    0.534802    ,    0.533329    ,    0.522854    ,    0.48661    ,    0.441887    ,    0.417017    ,    0.400584    ,    0.389444    },
    {    0.395888    ,    0.400134    ,    0.406122    ,    0.414477    ,    0.410807    ,    0.407112    ,    0.394085    ,    0.373153    ,    0.363726    ,    0.364931    ,    0.372982    ,    0.369854    ,    0.364644    ,    0.369253    ,    0.360579    ,    0.357167    ,    0.357463    ,    0.365041    ,    0.37867    ,    0.398057    ,    0.409418    ,    0.413849    ,    0.413917    ,    0.417988    ,    0.432503    ,    0.4523    ,    0.469712    ,    0.485583    ,    0.498568    ,    0.511387    ,    0.521303    ,    0.531892    ,    0.542403    ,    0.551622    ,    0.558045    ,    0.563698    ,    0.573094    ,    0.578247    ,    0.595314    ,    0.617039    ,    0.63085    ,    0.638681    ,    0.659925    ,    0.693992    ,    0.732929    ,    0.763698    ,    0.758611    ,    0.721619    ,    0.684157    ,    0.659601    ,    0.652214    ,    0.662416    ,    0.671657    ,    0.684236    ,    0.702147    ,    0.724579    ,    0.749152    ,    0.762744    ,    0.774018    ,    0.781848    ,    0.782897    ,    0.783724    ,    0.789275    ,    0.79149    ,    0.786164    ,    0.773145    ,    0.763109    ,    0.764886    ,    0.76896    ,    0.771995    ,    0.766924    ,    0.752773    ,    0.725089    ,    0.682799    ,    0.641145    ,    0.614738    ,    0.607338    ,    0.61177    ,    0.627241    ,    0.647275    ,    0.668667    ,    0.687091    ,    0.705502    ,    0.725064    ,    0.74191    ,    0.758642    ,    0.777554    ,    0.798852    ,    0.814727    ,    0.804591    ,    0.751504    ,    0.662397    ,    0.571954    ,    0.533213    ,    0.525657    ,    0.521067    ,    0.539765    ,    0.562908    ,    0.58049    ,    0.611533    ,    0.650938    ,    0.6692    ,    0.665773    ,    0.660044    ,    0.654736    ,    0.649691    ,    0.641025    ,    0.629904    ,    0.619917    ,    0.61777    ,    0.617134    ,    0.618401    ,    0.623942    ,    0.635202    ,    0.649654    ,    0.668503    ,    0.664876    ,    0.624279    ,    0.606339    ,    0.598488    ,    0.596077    ,    0.590533    ,    0.539791    ,    0.479327    ,    0.442714    ,    0.426751    ,    0.415462    ,    0.406697    },
    {    0.422456    ,    0.430039    ,    0.448172    ,    0.464906    ,    0.462625    ,    0.44791    ,    0.421012    ,    0.393528    ,    0.383506    ,    0.381021    ,    0.373035    ,    0.348053    ,    0.326164    ,    0.312483    ,    0.319795    ,    0.332765    ,    0.342714    ,    0.355976    ,    0.384856    ,    0.414305    ,    0.427818    ,    0.434245    ,    0.439448    ,    0.448898    ,    0.467755    ,    0.486941    ,    0.503352    ,    0.515804    ,    0.526732    ,    0.537249    ,    0.546162    ,    0.55419    ,    0.559065    ,    0.564289    ,    0.569777    ,    0.573138    ,    0.579079    ,    0.594219    ,    0.6309    ,    0.670808    ,    0.672972    ,    0.650695    ,    0.647535    ,    0.652249    ,    0.652939    ,    0.654578    ,    0.660763    ,    0.654109    ,    0.616515    ,    0.57762    ,    0.557055    ,    0.562348    ,    0.554303    ,    0.561686    ,    0.570322    ,    0.581778    ,    0.599155    ,    0.610311    ,    0.619626    ,    0.62302    ,    0.620667    ,    0.616035    ,    0.619122    ,    0.627217    ,    0.630167    ,    0.623958    ,    0.610333    ,    0.607768    ,    0.616746    ,    0.626693    ,    0.635267    ,    0.637569    ,    0.632149    ,    0.625486    ,    0.613966    ,    0.605325    ,    0.603946    ,    0.617644    ,    0.632558    ,    0.642193    ,    0.657012    ,    0.672259    ,    0.683339    ,    0.693898    ,    0.705692    ,    0.720079    ,    0.742131    ,    0.766208    ,    0.79094    ,    0.809382    ,    0.815041    ,    0.780799    ,    0.675297    ,    0.606105    ,    0.586311    ,    0.560814    ,    0.548817    ,    0.553518    ,    0.571714    ,    0.596917    ,    0.606016    ,    0.616312    ,    0.63299    ,    0.648037    ,    0.657844    ,    0.653432    ,    0.642352    ,    0.635012    ,    0.629983    ,    0.630748    ,    0.631043    ,    0.635492    ,    0.641547    ,    0.647068    ,    0.662715    ,    0.672991    ,    0.666856    ,    0.619644    ,    0.593389    ,    0.578475    ,    0.572609    ,    0.56794    ,    0.517913    ,    0.484194    ,    0.472869    ,    0.458242    ,    0.439469    ,    0.428085    },
    {    0.475378    ,    0.490998    ,    0.506344    ,    0.512718    ,    0.512476    ,    0.496297    ,    0.457581    ,    0.428456    ,    0.425926    ,    0.425895    ,    0.419618    ,    0.400886    ,    0.380287    ,    0.375246    ,    0.391398    ,    0.412478    ,    0.429524    ,    0.445664    ,    0.467022    ,    0.483044    ,    0.488614    ,    0.490766    ,    0.492498    ,    0.497249    ,    0.513105    ,    0.531352    ,    0.543656    ,    0.551554    ,    0.557707    ,    0.562742    ,    0.567981    ,    0.573448    ,    0.576731    ,    0.579455    ,    0.584009    ,    0.590267    ,    0.588398    ,    0.608063    ,    0.673836    ,    0.710356    ,    0.67028    ,    0.64974    ,    0.652034    ,    0.657832    ,    0.641653    ,    0.624874    ,    0.62969    ,    0.629756    ,    0.607016    ,    0.584238    ,    0.568707    ,    0.565071    ,    0.568459    ,    0.572371    ,    0.574337    ,    0.574774    ,    0.579057    ,    0.589457    ,    0.602023    ,    0.610148    ,    0.611638    ,    0.597069    ,    0.594096    ,    0.601905    ,    0.610563    ,    0.614742    ,    0.607986    ,    0.604313    ,    0.606164    ,    0.596868    ,    0.58051    ,    0.579698    ,    0.585549    ,    0.593716    ,    0.602131    ,    0.616746    ,    0.62646    ,    0.640862    ,    0.653341    ,    0.659839    ,    0.664865    ,    0.675872    ,    0.69063    ,    0.702452    ,    0.710694    ,    0.720206    ,    0.751701    ,    0.783072    ,    0.800802    ,    0.806269    ,    0.796411    ,    0.761686    ,    0.681847    ,    0.634606    ,    0.64813    ,    0.629954    ,    0.629284    ,    0.634428    ,    0.645813    ,    0.664719    ,    0.676202    ,    0.696924    ,    0.720083    ,    0.736923    ,    0.741335    ,    0.725132    ,    0.693144    ,    0.666107    ,    0.647648    ,    0.635609    ,    0.626817    ,    0.637241    ,    0.647214    ,    0.645547    ,    0.628039    ,    0.599621    ,    0.583565    ,    0.55307    ,    0.543757    ,    0.528539    ,    0.52649    ,    0.53946    ,    0.531946    ,    0.515583    ,    0.514155    ,    0.501648    ,    0.477762    ,    0.466838    },
    {    0.572955    ,    0.592955    ,    0.599548    ,    0.58381    ,    0.567781    ,    0.557788    ,    0.537737    ,    0.526924    ,    0.529171    ,    0.523817    ,    0.511439    ,    0.498787    ,    0.491306    ,    0.488005    ,    0.500091    ,    0.51578    ,    0.52327    ,    0.53067    ,    0.538656    ,    0.541927    ,    0.539302    ,    0.537869    ,    0.537467    ,    0.539204    ,    0.550213    ,    0.563333    ,    0.569493    ,    0.57398    ,    0.577252    ,    0.580496    ,    0.583202    ,    0.585577    ,    0.58599    ,    0.586215    ,    0.58965    ,    0.597648    ,    0.600288    ,    0.623767    ,    0.738695    ,    0.767771    ,    0.729489    ,    0.669953    ,    0.650565    ,    0.632545    ,    0.621426    ,    0.629258    ,    0.655762    ,    0.667633    ,    0.665596    ,    0.655869    ,    0.647548    ,    0.635391    ,    0.625115    ,    0.623311    ,    0.631042    ,    0.62975    ,    0.634256    ,    0.660591    ,    0.692606    ,    0.714655    ,    0.713249    ,    0.687219    ,    0.675456    ,    0.668604    ,    0.662463    ,    0.662192    ,    0.660724    ,    0.659735    ,    0.633289    ,    0.612439    ,    0.601702    ,    0.601376    ,    0.597292    ,    0.585648    ,    0.590904    ,    0.625213    ,    0.666036    ,    0.690838    ,    0.702518    ,    0.696189    ,    0.696371    ,    0.703289    ,    0.714739    ,    0.724602    ,    0.726116    ,    0.730426    ,    0.768302    ,    0.792538    ,    0.787056    ,    0.783607    ,    0.797916    ,    0.817366    ,    0.804044    ,    0.754637    ,    0.713832    ,    0.696981    ,    0.697246    ,    0.704911    ,    0.724016    ,    0.73819    ,    0.751233    ,    0.764751    ,    0.756806    ,    0.742572    ,    0.719552    ,    0.696281    ,    0.661761    ,    0.626164    ,    0.598653    ,    0.591075    ,    0.599094    ,    0.62672    ,    0.647019    ,    0.630378    ,    0.570471    ,    0.536503    ,    0.528747    ,    0.529868    ,    0.537194    ,    0.537734    ,    0.540636    ,    0.557448    ,    0.570662    ,    0.572761    ,    0.565556    ,    0.557007    ,    0.546515    ,    0.549704    },
    {    0.662091    ,    0.682857    ,    0.689264    ,    0.668909    ,    0.654937    ,    0.649664    ,    0.64379    ,    0.628761    ,    0.613365    ,    0.600712    ,    0.587863    ,    0.580625    ,    0.580055    ,    0.581464    ,    0.588454    ,    0.585403    ,    0.576546    ,    0.572374    ,    0.5686    ,    0.563818    ,    0.558867    ,    0.557402    ,    0.55968    ,    0.56353    ,    0.568347    ,    0.574613    ,    0.580288    ,    0.585712    ,    0.587666    ,    0.588978    ,    0.589982    ,    0.591552    ,    0.591481    ,    0.590906    ,    0.592367    ,    0.602106    ,    0.611652    ,    0.63343    ,    0.723742    ,    0.780155    ,    0.763275    ,    0.681406    ,    0.646196    ,    0.62797    ,    0.639735    ,    0.666248    ,    0.696785    ,    0.717229    ,    0.723924    ,    0.728759    ,    0.717186    ,    0.703782    ,    0.683915    ,    0.685062    ,    0.690419    ,    0.698096    ,    0.717114    ,    0.777277    ,    0.836866    ,    0.853112    ,    0.812415    ,    0.76763    ,    0.74398    ,    0.729396    ,    0.719579    ,    0.722712    ,    0.734249    ,    0.726023    ,    0.692775    ,    0.661497    ,    0.655616    ,    0.652915    ,    0.655184    ,    0.666355    ,    0.701215    ,    0.73086    ,    0.759505    ,    0.774644    ,    0.764885    ,    0.745792    ,    0.742612    ,    0.749525    ,    0.756287    ,    0.770667    ,    0.748965    ,    0.7224    ,    0.731605    ,    0.755547    ,    0.76931    ,    0.771969    ,    0.791586    ,    0.813676    ,    0.815028    ,    0.79129    ,    0.748601    ,    0.722526    ,    0.721961    ,    0.740516    ,    0.743532    ,    0.720907    ,    0.699763    ,    0.682875    ,    0.657201    ,    0.634763    ,    0.615654    ,    0.601152    ,    0.579161    ,    0.559688    ,    0.550139    ,    0.549005    ,    0.558138    ,    0.572527    ,    0.57897    ,    0.573267    ,    0.557654    ,    0.555974    ,    0.567749    ,    0.592359    ,    0.600531    ,    0.593755    ,    0.588385    ,    0.605136    ,    0.628503    ,    0.652555    ,    0.69098    ,    0.657601    ,    0.637449    ,    0.639457    },
    {    0.763938    ,    0.771081    ,    0.753212    ,    0.727368    ,    0.712358    ,    0.701266    ,    0.684361    ,    0.666799    ,    0.654085    ,    0.646153    ,    0.642605    ,    0.640934    ,    0.635909    ,    0.626676    ,    0.616358    ,    0.602451    ,    0.59034    ,    0.58449    ,    0.581482    ,    0.58028    ,    0.577453    ,    0.575703    ,    0.577848    ,    0.581948    ,    0.585302    ,    0.588288    ,    0.59271    ,    0.594755    ,    0.591383    ,    0.587409    ,    0.587205    ,    0.59066    ,    0.594504    ,    0.597001    ,    0.591056    ,    0.58901    ,    0.603888    ,    0.627808    ,    0.70983    ,    0.808113    ,    0.786783    ,    0.697719    ,    0.652643    ,    0.671422    ,    0.716552    ,    0.765255    ,    0.782636    ,    0.779673    ,    0.776512    ,    0.788319    ,    0.782125    ,    0.77776    ,    0.771297    ,    0.786255    ,    0.80768    ,    0.832566    ,    0.875806    ,    0.940723    ,    0.97946    ,    0.920192    ,    0.821011    ,    0.793362    ,    0.786987    ,    0.785089    ,    0.782466    ,    0.770333    ,    0.772234    ,    0.768141    ,    0.74131    ,    0.725344    ,    0.739067    ,    0.75136    ,    0.784659    ,    0.832609    ,    0.843273    ,    0.82023    ,    0.815232    ,    0.849059    ,    0.864278    ,    0.831528    ,    0.778124    ,    0.770979    ,    0.766159    ,    0.761285    ,    0.723963    ,    0.693202    ,    0.692531    ,    0.666727    ,    0.676993    ,    0.691001    ,    0.700734    ,    0.709163    ,    0.712448    ,    0.709327    ,    0.693526    ,    0.685238    ,    0.691774    ,    0.683284    ,    0.648175    ,    0.604147    ,    0.577525    ,    0.568488    ,    0.555319    ,    0.546414    ,    0.541529    ,    0.535263    ,    0.531058    ,    0.532776    ,    0.539087    ,    0.547641    ,    0.553835    ,    0.562385    ,    0.573871    ,    0.592755    ,    0.607688    ,    0.62132    ,    0.682968    ,    0.719966    ,    0.716942    ,    0.687519    ,    0.673518    ,    0.673568    ,    0.675262    ,    0.677968    ,    0.711768    ,    0.75555    ,    0.729292    ,    0.726672    },
    {    0.747798    ,    0.728984    ,    0.717459    ,    0.710653    ,    0.708538    ,    0.702473    ,    0.696722    ,    0.695032    ,    0.693213    ,    0.689014    ,    0.681736    ,    0.674077    ,    0.664745    ,    0.653431    ,    0.643066    ,    0.634017    ,    0.626196    ,    0.617771    ,    0.608538    ,    0.603209    ,    0.599763    ,    0.59832    ,    0.598046    ,    0.598508    ,    0.599141    ,    0.600321    ,    0.603033    ,    0.599505    ,    0.590606    ,    0.581699    ,    0.579407    ,    0.580156    ,    0.588728    ,    0.597618    ,    0.579801    ,    0.576859    ,    0.597807    ,    0.615113    ,    0.716096    ,    0.817086    ,    0.787556    ,    0.721167    ,    0.677005    ,    0.690053    ,    0.776782    ,    0.909567    ,    0.950598    ,    0.908991    ,    0.865038    ,    0.85953    ,    0.855774    ,    0.85789    ,    0.864349    ,    0.883148    ,    0.903944    ,    0.924261    ,    0.948158    ,    0.973693    ,    0.959608    ,    0.860642    ,    0.78229    ,    0.75855    ,    0.749445    ,    0.746504    ,    0.746119    ,    0.741644    ,    0.751883    ,    0.770457    ,    0.800307    ,    0.831304    ,    0.846324    ,    0.846975    ,    0.853804    ,    0.88667    ,    0.868956    ,    0.81478    ,    0.835833    ,    0.865712    ,    0.901714    ,    0.869325    ,    0.764771    ,    0.700485    ,    0.646908    ,    0.612979    ,    0.603205    ,    0.597737    ,    0.589871    ,    0.58118    ,    0.578299    ,    0.57075    ,    0.568412    ,    0.574434    ,    0.590678    ,    0.594097    ,    0.588796    ,    0.586564    ,    0.581035    ,    0.561471    ,    0.543251    ,    0.538058    ,    0.54322    ,    0.545836    ,    0.54248    ,    0.543657    ,    0.547946    ,    0.550649    ,    0.559543    ,    0.575095    ,    0.592163    ,    0.599388    ,    0.601926    ,    0.606291    ,    0.617    ,    0.628261    ,    0.643477    ,    0.672386    ,    0.740002    ,    0.787753    ,    0.811964    ,    0.777743    ,    0.746611    ,    0.725714    ,    0.713039    ,    0.70718    ,    0.712542    ,    0.742629    ,    0.785778    ,    0.79387    },
    {    0.737013    ,    0.705846    ,    0.702396    ,    0.711994    ,    0.71658    ,    0.715481    ,    0.71831    ,    0.720465    ,    0.714658    ,    0.7106    ,    0.704719    ,    0.69861    ,    0.690171    ,    0.682059    ,    0.675457    ,    0.667433    ,    0.656649    ,    0.646939    ,    0.637907    ,    0.629105    ,    0.62409    ,    0.618771    ,    0.612421    ,    0.607432    ,    0.608065    ,    0.610414    ,    0.610686    ,    0.600205    ,    0.586612    ,    0.578367    ,    0.574223    ,    0.572075    ,    0.582518    ,    0.594045    ,    0.576539    ,    0.570322    ,    0.584801    ,    0.611322    ,    0.757312    ,    0.868389    ,    0.783796    ,    0.767903    ,    0.771353    ,    0.760622    ,    0.854262    ,    1.01341    ,    0.958457    ,    0.913026    ,    0.89426    ,    0.87827    ,    0.865083    ,    0.857353    ,    0.854912    ,    0.850709    ,    0.849346    ,    0.855739    ,    0.859083    ,    0.843173    ,    0.792991    ,    0.727266    ,    0.693816    ,    0.690087    ,    0.687565    ,    0.69946    ,    0.717124    ,    0.741141    ,    0.778864    ,    0.820735    ,    0.852197    ,    0.849053    ,    0.81327    ,    0.777933    ,    0.753661    ,    0.723683    ,    0.673289    ,    0.6276    ,    0.614363    ,    0.622472    ,    0.626947    ,    0.606565    ,    0.569317    ,    0.545071    ,    0.52742    ,    0.519235    ,    0.525426    ,    0.533011    ,    0.527115    ,    0.522548    ,    0.524059    ,    0.523907    ,    0.523308    ,    0.534037    ,    0.544761    ,    0.545205    ,    0.542257    ,    0.536944    ,    0.536659    ,    0.54444    ,    0.560087    ,    0.579116    ,    0.595991    ,    0.602176    ,    0.605162    ,    0.616784    ,    0.630728    ,    0.633242    ,    0.638372    ,    0.646693    ,    0.654085    ,    0.657274    ,    0.655842    ,    0.650156    ,    0.654154    ,    0.674455    ,    0.740481    ,    0.806928    ,    0.894477    ,    0.871784    ,    0.860898    ,    0.8376    ,    0.797319    ,    0.775271    ,    0.761389    ,    0.748369    ,    0.737597    ,    0.749862    ,    0.784855    ,    0.790924    },
    {    0.729404    ,    0.72084    ,    0.727108    ,    0.731648    ,    0.730168    ,    0.728176    ,    0.730252    ,    0.730702    ,    0.726042    ,    0.725953    ,    0.724808    ,    0.71949    ,    0.709682    ,    0.702075    ,    0.693747    ,    0.683558    ,    0.6736    ,    0.66373    ,    0.65885    ,    0.652948    ,    0.64776    ,    0.637767    ,    0.62232    ,    0.610307    ,    0.605316    ,    0.605948    ,    0.606866    ,    0.596374    ,    0.579763    ,    0.567468    ,    0.555724    ,    0.557185    ,    0.56639    ,    0.569069    ,    0.553484    ,    0.549361    ,    0.559951    ,    0.614737    ,    0.76106    ,    0.937002    ,    0.874622    ,    0.814124    ,    0.830356    ,    0.826606    ,    0.841305    ,    0.897397    ,    0.863666    ,    0.807706    ,    0.778822    ,    0.761326    ,    0.739812    ,    0.73094    ,    0.727138    ,    0.724076    ,    0.724178    ,    0.724346    ,    0.712479    ,    0.688114    ,    0.663219    ,    0.655023    ,    0.657699    ,    0.654449    ,    0.65917    ,    0.685229    ,    0.71947    ,    0.759077    ,    0.793135    ,    0.800552    ,    0.779598    ,    0.744109    ,    0.712711    ,    0.733694    ,    0.749085    ,    0.757372    ,    0.725222    ,    0.646667    ,    0.563367    ,    0.539311    ,    0.544775    ,    0.554625    ,    0.566543    ,    0.573113    ,    0.573071    ,    0.569071    ,    0.569251    ,    0.573513    ,    0.574278    ,    0.577616    ,    0.589237    ,    0.592404    ,    0.591342    ,    0.59769    ,    0.602655    ,    0.609653    ,    0.616956    ,    0.621337    ,    0.628097    ,    0.632748    ,    0.637209    ,    0.642073    ,    0.65463    ,    0.658545    ,    0.665158    ,    0.674182    ,    0.664122    ,    0.698467    ,    0.74782    ,    0.712025    ,    0.694583    ,    0.684655    ,    0.682189    ,    0.687635    ,    0.71794    ,    0.787593    ,    0.866314    ,    0.910894    ,    0.981863    ,    0.993786    ,    0.90483    ,    0.844675    ,    0.806897    ,    0.789154    ,    0.779272    ,    0.769294    ,    0.76366    ,    0.767269    ,    0.77707    ,    0.755705    },
    {    0.741162    ,    0.739774    ,    0.74327    ,    0.740211    ,    0.739366    ,    0.737746    ,    0.735446    ,    0.736936    ,    0.739092    ,    0.738022    ,    0.732917    ,    0.727844    ,    0.719316    ,    0.70856    ,    0.697355    ,    0.687132    ,    0.677095    ,    0.670115    ,    0.664281    ,    0.657887    ,    0.650754    ,    0.636438    ,    0.613947    ,    0.596678    ,    0.585248    ,    0.579307    ,    0.577976    ,    0.567843    ,    0.552169    ,    0.539946    ,    0.532292    ,    0.534639    ,    0.535698    ,    0.527695    ,    0.519303    ,    0.521443    ,    0.5383    ,    0.603652    ,    0.754471    ,    0.985864    ,    0.940293    ,    0.900287    ,    0.902095    ,    0.900967    ,    0.934036    ,    0.888821    ,    0.810182    ,    0.796882    ,    0.730627    ,    0.697745    ,    0.688523    ,    0.683789    ,    0.679248    ,    0.677792    ,    0.672974    ,    0.666791    ,    0.653327    ,    0.638701    ,    0.633983    ,    0.632624    ,    0.630792    ,    0.633106    ,    0.64379    ,    0.6741    ,    0.711346    ,    0.741731    ,    0.753294    ,    0.742021    ,    0.715664    ,    0.678927    ,    0.711304    ,    0.979792    ,    1.09834    ,    1.10808    ,    1.00232    ,    0.807137    ,    0.725615    ,    0.667158    ,    0.675613    ,    0.685445    ,    0.695893    ,    0.699687    ,    0.700313    ,    0.698626    ,    0.695173    ,    0.696874    ,    0.697874    ,    0.698778    ,    0.707363    ,    0.706104    ,    0.702029    ,    0.703897    ,    0.704618    ,    0.708731    ,    0.710032    ,    0.706688    ,    0.696224    ,    0.68536    ,    0.688345    ,    0.696542    ,    0.698893    ,    0.686396    ,    0.678937    ,    0.680073    ,    0.686455    ,    0.814384    ,    0.949914    ,    0.908252    ,    0.843973    ,    0.757571    ,    0.737502    ,    0.777651    ,    0.893832    ,    0.95026    ,    0.976825    ,    1.01482    ,    1.05922    ,    1.06404    ,    0.982227    ,    0.824285    ,    0.782557    ,    0.778328    ,    0.773458    ,    0.771282    ,    0.773228    ,    0.767712    ,    0.758955    ,    0.749685    },
    {    0.753566    ,    0.753652    ,    0.754372    ,    0.752804    ,    0.750665    ,    0.749286    ,    0.752008    ,    0.756207    ,    0.755849    ,    0.74752    ,    0.736091    ,    0.726327    ,    0.713846    ,    0.703098    ,    0.693739    ,    0.68462    ,    0.672623    ,    0.663316    ,    0.653164    ,    0.635454    ,    0.617154    ,    0.598924    ,    0.57468    ,    0.553935    ,    0.54293    ,    0.53452    ,    0.524795    ,    0.513297    ,    0.50538    ,    0.50378    ,    0.501136    ,    0.497938    ,    0.495343    ,    0.490833    ,    0.490813    ,    0.498938    ,    0.53591    ,    0.630715    ,    0.785537    ,    1.03606    ,    1.08723    ,    0.995979    ,    0.947838    ,    0.977901    ,    1.0237    ,    1.00783    ,    0.908391    ,    0.790305    ,    0.749186    ,    0.715654    ,    0.701246    ,    0.698083    ,    0.690042    ,    0.681943    ,    0.673924    ,    0.66209    ,    0.64831    ,    0.632283    ,    0.621534    ,    0.616218    ,    0.620323    ,    0.637037    ,    0.661211    ,    0.686959    ,    0.707291    ,    0.719816    ,    0.71719    ,    0.704364    ,    0.681664    ,    0.671736    ,    0.836058    ,    1.17958    ,    1.35257    ,    1.36997    ,    1.27089    ,    1.03859    ,    0.824388    ,    0.794533    ,    0.796225    ,    0.802187    ,    0.801179    ,    0.802826    ,    0.819163    ,    0.821791    ,    0.802669    ,    0.792562    ,    0.787093    ,    0.780902    ,    0.780548    ,    0.775548    ,    0.767576    ,    0.761324    ,    0.756863    ,    0.752407    ,    0.739498    ,    0.726097    ,    0.716892    ,    0.715386    ,    0.7291    ,    0.725233    ,    0.702314    ,    0.678268    ,    0.669407    ,    0.682244    ,    0.721392    ,    0.91521    ,    1.04165    ,    1.0259    ,    0.968083    ,    0.927322    ,    0.927807    ,    0.96498    ,    1.00441    ,    1.03262    ,    1.05245    ,    1.1027    ,    1.13533    ,    1.12346    ,    1.02671    ,    0.828976    ,    0.770073    ,    0.768162    ,    0.764072    ,    0.762457    ,    0.761525    ,    0.757069    ,    0.754859    ,    0.755734    },
    {    0.754759    ,    0.758341    ,    0.758903    ,    0.758861    ,    0.758517    ,    0.754985    ,    0.751529    ,    0.745555    ,    0.735574    ,    0.724697    ,    0.714364    ,    0.700816    ,    0.686657    ,    0.677049    ,    0.666239    ,    0.651491    ,    0.633074    ,    0.618868    ,    0.601508    ,    0.577835    ,    0.556476    ,    0.536592    ,    0.516109    ,    0.501232    ,    0.492902    ,    0.485524    ,    0.475356    ,    0.46872    ,    0.467412    ,    0.464756    ,    0.4636    ,    0.462029    ,    0.461039    ,    0.459232    ,    0.46748    ,    0.499381    ,    0.562053    ,    0.663642    ,    0.809004    ,    1.0474    ,    1.27597    ,    1.06657    ,    1.04438    ,    1.06741    ,    1.10524    ,    1.10245    ,    1.01778    ,    0.830727    ,    0.763104    ,    0.740547    ,    0.716986    ,    0.707258    ,    0.696751    ,    0.680311    ,    0.664821    ,    0.648856    ,    0.637078    ,    0.630288    ,    0.629339    ,    0.63217    ,    0.642999    ,    0.661625    ,    0.683747    ,    0.697457    ,    0.697539    ,    0.694174    ,    0.684685    ,    0.676248    ,    0.676206    ,    0.755607    ,    1.12803    ,    1.31075    ,    1.41414    ,    1.41368    ,    1.34183    ,    1.19877    ,    0.900582    ,    0.824655    ,    0.836482    ,    0.858844    ,    0.880645    ,    0.826053    ,    0.823618    ,    0.828482    ,    0.814912    ,    0.806281    ,    0.795605    ,    0.789183    ,    0.784255    ,    0.779644    ,    0.773946    ,    0.765353    ,    0.75797    ,    0.747271    ,    0.73586    ,    0.723749    ,    0.713761    ,    0.708786    ,    0.7063    ,    0.690275    ,    0.670756    ,    0.664441    ,    0.674755    ,    0.699632    ,    0.760173    ,    1.01378    ,    1.1255    ,    1.10112    ,    1.08614    ,    1.07987    ,    1.06605    ,    1.07989    ,    1.06729    ,    1.08356    ,    1.09303    ,    1.11493    ,    1.121    ,    1.09982    ,    0.993265    ,    0.790566    ,    0.766281    ,    0.764598    ,    0.759838    ,    0.758242    ,    0.756859    ,    0.756399    ,    0.756383    ,    0.754815    },
    {    0.748855    ,    0.748639    ,    0.749679    ,    0.750773    ,    0.747058    ,    0.735266    ,    0.722604    ,    0.710692    ,    0.695591    ,    0.680096    ,    0.665169    ,    0.650044    ,    0.636554    ,    0.625247    ,    0.610518    ,    0.591071    ,    0.572852    ,    0.560693    ,    0.547772    ,    0.530131    ,    0.513511    ,    0.500667    ,    0.491913    ,    0.486206    ,    0.475299    ,    0.464939    ,    0.454246    ,    0.447433    ,    0.442636    ,    0.437873    ,    0.436707    ,    0.433361    ,    0.436417    ,    0.441841    ,    0.465782    ,    0.51974    ,    0.594791    ,    0.696568    ,    0.824763    ,    1.11774    ,    1.42662    ,    1.22563    ,    1.16958    ,    1.17691    ,    1.18753    ,    1.19641    ,    1.12706    ,    1.0227    ,    0.919995    ,    0.788924    ,    0.724698    ,    0.696921    ,    0.689074    ,    0.674603    ,    0.66141    ,    0.651609    ,    0.648018    ,    0.654873    ,    0.662273    ,    0.669093    ,    0.676011    ,    0.677655    ,    0.67816    ,    0.674125    ,    0.66825    ,    0.665406    ,    0.668084    ,    0.683013    ,    0.712431    ,    0.90468    ,    1.28208    ,    1.42667    ,    1.46685    ,    1.47124    ,    1.38131    ,    1.261    ,    1.10502    ,    0.847481    ,    0.846039    ,    0.901565    ,    1.11186    ,    0.910481    ,    0.796063    ,    0.787953    ,    0.777071    ,    0.775032    ,    0.763054    ,    0.755419    ,    0.749129    ,    0.745594    ,    0.738468    ,    0.727527    ,    0.719305    ,    0.712199    ,    0.710057    ,    0.703185    ,    0.696891    ,    0.693701    ,    0.685612    ,    0.674151    ,    0.672362    ,    0.675588    ,    0.687355    ,    0.713643    ,    0.782748    ,    1.04718    ,    1.14061    ,    1.15112    ,    1.14658    ,    1.13295    ,    1.13416    ,    1.10033    ,    1.12362    ,    1.11581    ,    1.12183    ,    1.08708    ,    1.07688    ,    0.926641    ,    0.815701    ,    0.764805    ,    0.761804    ,    0.762255    ,    0.76088    ,    0.760216    ,    0.758542    ,    0.755563    ,    0.754353    ,    0.751085    },
    {    0.742027    ,    0.741294    ,    0.740221    ,    0.737733    ,    0.729767    ,    0.713931    ,    0.69752    ,    0.68264    ,    0.664637    ,    0.646145    ,    0.631848    ,    0.619018    ,    0.607024    ,    0.593429    ,    0.577449    ,    0.561406    ,    0.549325    ,    0.540269    ,    0.530271    ,    0.517812    ,    0.507887    ,    0.500266    ,    0.491034    ,    0.482644    ,    0.471428    ,    0.461201    ,    0.453588    ,    0.446893    ,    0.441152    ,    0.440201    ,    0.440909    ,    0.444229    ,    0.457495    ,    0.478492    ,    0.514507    ,    0.575637    ,    0.649475    ,    0.737113    ,    0.826967    ,    1.12254    ,    1.41413    ,    1.27501    ,    1.27071    ,    1.18529    ,    1.21983    ,    1.25638    ,    1.1816    ,    1.18553    ,    1.13449    ,    1.01481    ,    0.759414    ,    0.699385    ,    0.688959    ,    0.682882    ,    0.674988    ,    0.666145    ,    0.666476    ,    0.668005    ,    0.669586    ,    0.668669    ,    0.662767    ,    0.654287    ,    0.648914    ,    0.648228    ,    0.65618    ,    0.667602    ,    0.683606    ,    0.714004    ,    0.759876    ,    1.03377    ,    1.35971    ,    1.42244    ,    1.45306    ,    1.41217    ,    1.37914    ,    1.30773    ,    1.14898    ,    0.849353    ,    0.849208    ,    0.894306    ,    1.11084    ,    0.969446    ,    0.770978    ,    0.754869    ,    0.7469    ,    0.740388    ,    0.728904    ,    0.723491    ,    0.71898    ,    0.712403    ,    0.703854    ,    0.699053    ,    0.696375    ,    0.696163    ,    0.698108    ,    0.694367    ,    0.690494    ,    0.685543    ,    0.676588    ,    0.672123    ,    0.677123    ,    0.686541    ,    0.705476    ,    0.726575    ,    0.766028    ,    0.908197    ,    1.03986    ,    1.11384    ,    1.15618    ,    1.1626    ,    1.19546    ,    1.13367    ,    1.16841    ,    1.14277    ,    1.08865    ,    1.03861    ,    0.991858    ,    0.837757    ,    0.759798    ,    0.756694    ,    0.757475    ,    0.755042    ,    0.754837    ,    0.756636    ,    0.749053    ,    0.747962    ,    0.746417    ,    0.744611    },
    {    0.731137    ,    0.727512    ,    0.7247    ,    0.720091    ,    0.713662    ,    0.701867    ,    0.688818    ,    0.675868    ,    0.661071    ,    0.647159    ,    0.634218    ,    0.620208    ,    0.609381    ,    0.592839    ,    0.577243    ,    0.566697    ,    0.556212    ,    0.545369    ,    0.535444    ,    0.529336    ,    0.522345    ,    0.517955    ,    0.512908    ,    0.505191    ,    0.496065    ,    0.489597    ,    0.485484    ,    0.482289    ,    0.485207    ,    0.492969    ,    0.507525    ,    0.525224    ,    0.551762    ,    0.582153    ,    0.624715    ,    0.679054    ,    0.753277    ,    0.837622    ,    0.956093    ,    1.23493    ,    1.29152    ,    1.27822    ,    1.32541    ,    1.2676    ,    1.24725    ,    1.29552    ,    1.26686    ,    1.27306    ,    1.24077    ,    1.12964    ,    0.838354    ,    0.703714    ,    0.685102    ,    0.67983    ,    0.672542    ,    0.663139    ,    0.660849    ,    0.658297    ,    0.656624    ,    0.653122    ,    0.64848    ,    0.648895    ,    0.653516    ,    0.665718    ,    0.68557    ,    0.711217    ,    0.736653    ,    0.765467    ,    0.854627    ,    1.21449    ,    1.38024    ,    1.41619    ,    1.4067    ,    1.38057    ,    1.33635    ,    1.28062    ,    1.20281    ,    0.994251    ,    0.847294    ,    0.848316    ,    1.02711    ,    0.983648    ,    0.759276    ,    0.736937    ,    0.731981    ,    0.72393    ,    0.712783    ,    0.70775    ,    0.703193    ,    0.697949    ,    0.695156    ,    0.694284    ,    0.692247    ,    0.692813    ,    0.694494    ,    0.69504    ,    0.694369    ,    0.692879    ,    0.690877    ,    0.6947    ,    0.704167    ,    0.719528    ,    0.735743    ,    0.749159    ,    0.770764    ,    0.810395    ,    0.842047    ,    0.911141    ,    1.09853    ,    1.17263    ,    1.13809    ,    1.1547    ,    1.14079    ,    1.08013    ,    1.01743    ,    0.95157    ,    0.833167    ,    0.756209    ,    0.754048    ,    0.752946    ,    0.7508    ,    0.745568    ,    0.739464    ,    0.736587    ,    0.735925    ,    0.735543    ,    0.736224    ,    0.740249    },
    {    0.715861    ,    0.713757    ,    0.710027    ,    0.706393    ,    0.702668    ,    0.696535    ,    0.690476    ,    0.682909    ,    0.673096    ,    0.662499    ,    0.65293    ,    0.641196    ,    0.629861    ,    0.615484    ,    0.607992    ,    0.600142    ,    0.593778    ,    0.588591    ,    0.582612    ,    0.579071    ,    0.575799    ,    0.570636    ,    0.568121    ,    0.563881    ,    0.557687    ,    0.556567    ,    0.558626    ,    0.564689    ,    0.577337    ,    0.594861    ,    0.616608    ,    0.640785    ,    0.673152    ,    0.712843    ,    0.760684    ,    0.814174    ,    0.87813    ,    1.01286    ,    1.22484    ,    1.42228    ,    1.22759    ,    1.36786    ,    1.3425    ,    1.31263    ,    1.30313    ,    1.29536    ,    1.26505    ,    1.23133    ,    1.20128    ,    1.10248    ,    0.884024    ,    0.700698    ,    0.688883    ,    0.6798    ,    0.669516    ,    0.665621    ,    0.664576    ,    0.664186    ,    0.662798    ,    0.663259    ,    0.668253    ,    0.682532    ,    0.698059    ,    0.712081    ,    0.727836    ,    0.748372    ,    0.762386    ,    0.772809    ,    0.881691    ,    1.21883    ,    1.29648    ,    1.32604    ,    1.34722    ,    1.33268    ,    1.31698    ,    1.29003    ,    1.23739    ,    1.20886    ,    0.990586    ,    0.823709    ,    0.854794    ,    0.950954    ,    0.774255    ,    0.729706    ,    0.729495    ,    0.720208    ,    0.713592    ,    0.709396    ,    0.706331    ,    0.703543    ,    0.703335    ,    0.703698    ,    0.705831    ,    0.710388    ,    0.714047    ,    0.717551    ,    0.720147    ,    0.726862    ,    0.736447    ,    0.74413    ,    0.749945    ,    0.761124    ,    0.775744    ,    0.783848    ,    0.795835    ,    0.814644    ,    0.827641    ,    0.843786    ,    0.90549    ,    1.03321    ,    1.07362    ,    1.12052    ,    1.05787    ,    0.848092    ,    0.83502    ,    0.890893    ,    0.748733    ,    0.753499    ,    0.757593    ,    0.754215    ,    0.750476    ,    0.747074    ,    0.74426    ,    0.740915    ,    0.735809    ,    0.730619    ,    0.730998    ,    0.725436    },
    {    0.719084    ,    0.714929    ,    0.711856    ,    0.70991    ,    0.707447    ,    0.706079    ,    0.699369    ,    0.695024    ,    0.691409    ,    0.683744    ,    0.68063    ,    0.676248    ,    0.670188    ,    0.666874    ,    0.666652    ,    0.662854    ,    0.658327    ,    0.654317    ,    0.649381    ,    0.646586    ,    0.641568    ,    0.637981    ,    0.629938    ,    0.627004    ,    0.627638    ,    0.629025    ,    0.635328    ,    0.646836    ,    0.664593    ,    0.68294    ,    0.707649    ,    0.736965    ,    0.776249    ,    0.820193    ,    0.864358    ,    0.90711    ,    0.945603    ,    1.14344    ,    1.17066    ,    1.24019    ,    1.35115    ,    1.36738    ,    1.37013    ,    1.35777    ,    1.37311    ,    1.31303    ,    1.28687    ,    1.23083    ,    1.22033    ,    1.10319    ,    0.923337    ,    0.732307    ,    0.700037    ,    0.697741    ,    0.687557    ,    0.683773    ,    0.68021    ,    0.6819    ,    0.687362    ,    0.696465    ,    0.709382    ,    0.723598    ,    0.735027    ,    0.743086    ,    0.753054    ,    0.761581    ,    0.769019    ,    0.78393    ,    0.818182    ,    1.05565    ,    1.21083    ,    1.25185    ,    1.25841    ,    1.22029    ,    1.23783    ,    1.24254    ,    1.17774    ,    1.12378    ,    0.962851    ,    0.795738    ,    0.790109    ,    0.81828    ,    0.769731    ,    0.747423    ,    0.746966    ,    0.741562    ,    0.742924    ,    0.739972    ,    0.738505    ,    0.736893    ,    0.736521    ,    0.738742    ,    0.740453    ,    0.743116    ,    0.744373    ,    0.750033    ,    0.75409    ,    0.762223    ,    0.768862    ,    0.774505    ,    0.783329    ,    0.795319    ,    0.800351    ,    0.802153    ,    0.807707    ,    0.814571    ,    0.825547    ,    0.830726    ,    0.839576    ,    0.84913    ,    0.869328    ,    0.96603    ,    0.918425    ,    0.786215    ,    0.793915    ,    0.810031    ,    0.747127    ,    0.757885    ,    0.761602    ,    0.762659    ,    0.758654    ,    0.754813    ,    0.748802    ,    0.744481    ,    0.744225    ,    0.74108    ,    0.736676    ,    0.729612    },
    {    0.730726    ,    0.725503    ,    0.725733    ,    0.723117    ,    0.724345    ,    0.720371    ,    0.717088    ,    0.713727    ,    0.707846    ,    0.706598    ,    0.707543    ,    0.706353    ,    0.706774    ,    0.707903    ,    0.707539    ,    0.712583    ,    0.712287    ,    0.707205    ,    0.706001    ,    0.704885    ,    0.699476    ,    0.697401    ,    0.696255    ,    0.697761    ,    0.699484    ,    0.704125    ,    0.712379    ,    0.723266    ,    0.734633    ,    0.752401    ,    0.776084    ,    0.80422    ,    0.842771    ,    0.887176    ,    0.928126    ,    0.935213    ,    1.00458    ,    1.18076    ,    1.2549    ,    1.31754    ,    1.32608    ,    1.31933    ,    1.36534    ,    1.35143    ,    1.37852    ,    1.35596    ,    1.293    ,    1.26839    ,    1.19772    ,    1.14217    ,    1.03836    ,    0.860899    ,    0.729116    ,    0.733165    ,    0.726408    ,    0.72421    ,    0.724099    ,    0.726686    ,    0.73109    ,    0.738649    ,    0.749661    ,    0.756874    ,    0.761836    ,    0.766859    ,    0.768423    ,    0.769089    ,    0.777874    ,    0.804084    ,    0.80638    ,    0.98365    ,    1.13992    ,    1.18524    ,    1.18896    ,    1.16935    ,    1.16746    ,    1.20416    ,    1.14344    ,    1.08116    ,    0.892122    ,    0.78479    ,    0.779905    ,    0.776415    ,    0.772378    ,    0.77304    ,    0.773549    ,    0.773489    ,    0.77427    ,    0.771437    ,    0.771271    ,    0.772    ,    0.771236    ,    0.773257    ,    0.773267    ,    0.77417    ,    0.773666    ,    0.776091    ,    0.777525    ,    0.780849    ,    0.779734    ,    0.784384    ,    0.7835    ,    0.782019    ,    0.785676    ,    0.796289    ,    0.811451    ,    0.813319    ,    0.812408    ,    0.816611    ,    0.816708    ,    0.810348    ,    0.793314    ,    0.786686    ,    0.767871    ,    0.760806    ,    0.780379    ,    0.780931    ,    0.783786    ,    0.795673    ,    0.77006    ,    0.769431    ,    0.765016    ,    0.759167    ,    0.750146    ,    0.748645    ,    0.746349    ,    0.749008    ,    0.743636    ,    0.736429    },
    {    0.747538    ,    0.744217    ,    0.744975    ,    0.747187    ,    0.748522    ,    0.745969    ,    0.746857    ,    0.747975    ,    0.746866    ,    0.744658    ,    0.747556    ,    0.750582    ,    0.748465    ,    0.749228    ,    0.753521    ,    0.756146    ,    0.760048    ,    0.761952    ,    0.762433    ,    0.761066    ,    0.7621    ,    0.76436    ,    0.771745    ,    0.77428    ,    0.777806    ,    0.784212    ,    0.790904    ,    0.802999    ,    0.814993    ,    0.834534    ,    0.852799    ,    0.870023    ,    0.896999    ,    0.934511    ,    0.949074    ,    0.928315    ,    1.05877    ,    1.25777    ,    1.22833    ,    1.22537    ,    1.25812    ,    1.28993    ,    1.33554    ,    1.33928    ,    1.33154    ,    1.32046    ,    1.26211    ,    1.22205    ,    1.18474    ,    1.15733    ,    1.12218    ,    0.949812    ,    0.769814    ,    0.775403    ,    0.772921    ,    0.771075    ,    0.77129    ,    0.772631    ,    0.775164    ,    0.781179    ,    0.785774    ,    0.789254    ,    0.792781    ,    0.790545    ,    0.790668    ,    0.793118    ,    0.800167    ,    0.815372    ,    0.837752    ,    1.01645    ,    1.15677    ,    1.17055    ,    1.14219    ,    1.13235    ,    1.12654    ,    1.10485    ,    1.11467    ,    1.02601    ,    0.857396    ,    0.80335    ,    0.806148    ,    0.803397    ,    0.805198    ,    0.808587    ,    0.810272    ,    0.806623    ,    0.803363    ,    0.796703    ,    0.795256    ,    0.798416    ,    0.799459    ,    0.798762    ,    0.793829    ,    0.796501    ,    0.795826    ,    0.790368    ,    0.78895    ,    0.783719    ,    0.781519    ,    0.777304    ,    0.75996    ,    0.777549    ,    0.852502    ,    0.838264    ,    0.831474    ,    0.831252    ,    0.820864    ,    0.815729    ,    0.804798    ,    0.798578    ,    0.786835    ,    0.775927    ,    0.765002    ,    0.804308    ,    0.888313    ,    0.889192    ,    0.860522    ,    0.785998    ,    0.770877    ,    0.765583    ,    0.760517    ,    0.757984    ,    0.751663    ,    0.753196    ,    0.748589    ,    0.751203    ,    0.753269    ,    0.750562    },
    {    0.789805    ,    0.79662    ,    0.803788    ,    0.808551    ,    0.814644    ,    0.814283    ,    0.814971    ,    0.819254    ,    0.825711    ,    0.829248    ,    0.838588    ,    0.841716    ,    0.844693    ,    0.845036    ,    0.847059    ,    0.847084    ,    0.848094    ,    0.852278    ,    0.853762    ,    0.854771    ,    0.859488    ,    0.860657    ,    0.864601    ,    0.869447    ,    0.876153    ,    0.886078    ,    0.897527    ,    0.904274    ,    0.91105    ,    0.916672    ,    0.923729    ,    0.928704    ,    0.935847    ,    0.94698    ,    0.939496    ,    0.901315    ,    1.07129    ,    1.19356    ,    1.18725    ,    1.16919    ,    1.21744    ,    1.26791    ,    1.29489    ,    1.28153    ,    1.25669    ,    1.23542    ,    1.18927    ,    1.15514    ,    1.13402    ,    1.08648    ,    0.973213    ,    0.83592    ,    0.800883    ,    0.809504    ,    0.806876    ,    0.806507    ,    0.808025    ,    0.809444    ,    0.814302    ,    0.815274    ,    0.814096    ,    0.820752    ,    0.822972    ,    0.818596    ,    0.817026    ,    0.818196    ,    0.825597    ,    0.841774    ,    0.924354    ,    1.03269    ,    1.13447    ,    1.15344    ,    1.1652    ,    1.16547    ,    1.14777    ,    1.11209    ,    1.04858    ,    1.02071    ,    0.877777    ,    0.824149    ,    0.835634    ,    0.833282    ,    0.832297    ,    0.833736    ,    0.829606    ,    0.820621    ,    0.817551    ,    0.811446    ,    0.811273    ,    0.81357    ,    0.814011    ,    0.812811    ,    0.813282    ,    0.807086    ,    0.806838    ,    0.800398    ,    0.795932    ,    0.784628    ,    0.776318    ,    0.758716    ,    0.775005    ,    0.8935    ,    0.857597    ,    0.828301    ,    0.835412    ,    0.850641    ,    0.816512    ,    0.844197    ,    0.805861    ,    0.80592    ,    0.805268    ,    0.798779    ,    0.827675    ,    0.882819    ,    0.900811    ,    0.866097    ,    0.788793    ,    0.774973    ,    0.792935    ,    0.780039    ,    0.776232    ,    0.775507    ,    0.778158    ,    0.782399    ,    0.783565    ,    0.785814    ,    0.791022    ,    0.792303    },
    {    0.851325    ,    0.85433    ,    0.859584    ,    0.865692    ,    0.87225    ,    0.872725    ,    0.876848    ,    0.882162    ,    0.885548    ,    0.890635    ,    0.896801    ,    0.901548    ,    0.907274    ,    0.907248    ,    0.914175    ,    0.917296    ,    0.916186    ,    0.920622    ,    0.923734    ,    0.932321    ,    0.941374    ,    0.947444    ,    0.95432    ,    0.963206    ,    0.974216    ,    0.989303    ,    1.00507    ,    1.01305    ,    1.01231    ,    1.01011    ,    1.00554    ,    0.996401    ,    0.974499    ,    0.960188    ,    0.933802    ,    0.894037    ,    0.978939    ,    1.11459    ,    1.1414    ,    1.12533    ,    1.18529    ,    1.21719    ,    1.24249    ,    1.27009    ,    1.22785    ,    1.16005    ,    1.07899    ,    0.995007    ,    0.872605    ,    0.835778    ,    0.825219    ,    0.821902    ,    0.829259    ,    0.834444    ,    0.833947    ,    0.833879    ,    0.830934    ,    0.831673    ,    0.835878    ,    0.835618    ,    0.836319    ,    0.836848    ,    0.84007    ,    0.837008    ,    0.832393    ,    0.833783    ,    0.841857    ,    0.859641    ,    0.977041    ,    1.03256    ,    1.10588    ,    1.1186    ,    1.13955    ,    1.13131    ,    1.10197    ,    1.07438    ,    1.00739    ,    1.0053    ,    0.94239    ,    0.870575    ,    0.84197    ,    0.853511    ,    0.854012    ,    0.849399    ,    0.838927    ,    0.826443    ,    0.819146    ,    0.813898    ,    0.810811    ,    0.812343    ,    0.811292    ,    0.809683    ,    0.811193    ,    0.806482    ,    0.804622    ,    0.798579    ,    0.792825    ,    0.781987    ,    0.771587    ,    0.767213    ,    0.881804    ,    0.913812    ,    0.844857    ,    0.873359    ,    0.951891    ,    0.933506    ,    0.843793    ,    0.866392    ,    0.820017    ,    0.8083    ,    0.804414    ,    0.842387    ,    0.817258    ,    0.802794    ,    0.783198    ,    0.785155    ,    0.788994    ,    0.797615    ,    0.812139    ,    0.815984    ,    0.821283    ,    0.823612    ,    0.826556    ,    0.83375    ,    0.839254    ,    0.843602    ,    0.847678    ,    0.850922    },
    {    0.843927    ,    0.843381    ,    0.842604    ,    0.840813    ,    0.845226    ,    0.852631    ,    0.85661    ,    0.86023    ,    0.866561    ,    0.873407    ,    0.88246    ,    0.888861    ,    0.893947    ,    0.898268    ,    0.90289    ,    0.908573    ,    0.911081    ,    0.917213    ,    0.925176    ,    0.93585    ,    0.948263    ,    0.958982    ,    0.969127    ,    0.980252    ,    0.992721    ,    1.0061    ,    1.01803    ,    1.02133    ,    1.02093    ,    1.01354    ,    1.00242    ,    0.98112    ,    0.959154    ,    0.93668    ,    0.915808    ,    0.887461    ,    0.914107    ,    1.043    ,    1.14153    ,    1.13976    ,    1.1587    ,    1.22102    ,    1.28796    ,    1.2672    ,    1.24419    ,    1.09174    ,    0.930989    ,    0.811044    ,    0.826009    ,    0.830948    ,    0.830712    ,    0.835406    ,    0.840817    ,    0.842839    ,    0.845887    ,    0.844763    ,    0.842703    ,    0.838538    ,    0.840529    ,    0.841242    ,    0.837629    ,    0.840059    ,    0.837581    ,    0.835305    ,    0.836668    ,    0.836242    ,    0.841571    ,    0.856098    ,    0.978847    ,    1.07244    ,    1.09757    ,    1.10801    ,    1.11715    ,    1.13284    ,    1.10513    ,    1.06119    ,    0.994618    ,    1.02155    ,    1.02553    ,    0.977795    ,    0.880513    ,    0.856385    ,    0.863472    ,    0.859303    ,    0.851573    ,    0.847073    ,    0.83836    ,    0.834097    ,    0.827119    ,    0.822355    ,    0.819646    ,    0.819584    ,    0.820186    ,    0.817271    ,    0.815249    ,    0.813862    ,    0.814937    ,    0.805713    ,    0.793715    ,    0.839931    ,    0.911127    ,    0.856591    ,    0.826317    ,    0.868541    ,    0.934241    ,    0.92371    ,    0.850925    ,    0.824175    ,    0.829282    ,    0.818812    ,    0.803534    ,    0.788031    ,    0.791498    ,    0.790733    ,    0.789528    ,    0.791536    ,    0.796081    ,    0.803048    ,    0.810606    ,    0.8116    ,    0.817398    ,    0.821253    ,    0.827444    ,    0.831639    ,    0.837303    ,    0.844721    ,    0.850172    ,    0.848899    },
    {    0.789049    ,    0.786026    ,    0.786363    ,    0.791678    ,    0.793821    ,    0.799872    ,    0.807717    ,    0.810852    ,    0.818695    ,    0.825184    ,    0.83225    ,    0.83672    ,    0.838321    ,    0.845411    ,    0.848046    ,    0.853875    ,    0.858953    ,    0.866111    ,    0.873705    ,    0.876851    ,    0.884669    ,    0.890008    ,    0.896247    ,    0.905173    ,    0.912984    ,    0.917544    ,    0.920346    ,    0.922235    ,    0.920706    ,    0.915884    ,    0.904027    ,    0.884801    ,    0.870832    ,    0.863519    ,    0.866025    ,    0.866431    ,    0.854057    ,    1.00173    ,    1.13545    ,    1.18437    ,    1.19723    ,    1.20636    ,    1.25075    ,    1.14554    ,    1.08781    ,    0.957586    ,    0.831451    ,    0.82439    ,    0.835054    ,    0.830617    ,    0.828547    ,    0.832688    ,    0.834564    ,    0.837987    ,    0.837947    ,    0.837127    ,    0.838871    ,    0.840811    ,    0.840315    ,    0.832221    ,    0.826779    ,    0.861791    ,    0.856336    ,    0.840191    ,    0.834731    ,    0.837509    ,    0.865746    ,    0.932128    ,    1.02376    ,    1.11519    ,    1.12901    ,    1.15125    ,    1.1307    ,    1.13896    ,    1.14964    ,    1.09722    ,    1.10749    ,    1.01889    ,    1.02966    ,    1.06962    ,    0.987846    ,    0.89787    ,    0.870476    ,    0.875591    ,    0.874419    ,    0.872778    ,    0.867948    ,    0.864215    ,    0.859979    ,    0.852846    ,    0.845889    ,    0.8405    ,    0.837316    ,    0.835782    ,    0.83826    ,    0.837436    ,    0.833197    ,    0.82535    ,    0.829765    ,    0.855609    ,    0.881085    ,    0.819806    ,    0.810123    ,    0.819566    ,    0.835309    ,    0.905662    ,    0.847953    ,    0.803718    ,    0.804246    ,    0.800222    ,    0.789915    ,    0.787737    ,    0.787267    ,    0.786634    ,    0.785371    ,    0.781635    ,    0.782531    ,    0.785904    ,    0.781134    ,    0.775654    ,    0.775268    ,    0.776466    ,    0.779789    ,    0.779973    ,    0.781501    ,    0.784642    ,    0.789639    ,    0.790703    },
    {    0.770756    ,    0.769223    ,    0.772325    ,    0.771309    ,    0.769227    ,    0.769409    ,    0.775273    ,    0.775121    ,    0.78092    ,    0.785466    ,    0.785741    ,    0.789072    ,    0.791453    ,    0.789576    ,    0.79254    ,    0.79688    ,    0.800769    ,    0.804873    ,    0.811894    ,    0.816188    ,    0.825926    ,    0.829316    ,    0.834575    ,    0.836811    ,    0.840546    ,    0.843313    ,    0.848834    ,    0.851181    ,    0.854383    ,    0.856515    ,    0.864621    ,    0.86938    ,    0.859534    ,    0.859731    ,    0.870484    ,    0.893109    ,    0.895216    ,    1.00674    ,    1.12228    ,    1.23102    ,    1.28814    ,    1.20056    ,    1.07661    ,    0.967732    ,    0.816893    ,    0.804416    ,    0.807658    ,    0.81631    ,    0.821259    ,    0.818253    ,    0.823047    ,    0.828391    ,    0.834238    ,    0.838864    ,    0.84294    ,    0.839199    ,    0.834451    ,    0.840119    ,    0.830414    ,    0.827359    ,    0.939121    ,    1.01022    ,    1.03965    ,    1.03603    ,    1.00702    ,    0.980514    ,    1.00977    ,    1.08911    ,    1.08452    ,    1.1384    ,    1.16794    ,    1.18378    ,    1.16967    ,    1.17905    ,    1.2085    ,    1.22036    ,    1.15935    ,    1.06768    ,    1.10025    ,    1.06414    ,    1.06152    ,    1.00629    ,    0.887971    ,    0.88382    ,    0.895953    ,    0.896965    ,    0.894295    ,    0.890603    ,    0.880476    ,    0.86984    ,    0.858429    ,    0.850212    ,    0.853806    ,    0.838213    ,    0.840145    ,    0.842922    ,    0.840334    ,    0.832833    ,    0.829943    ,    0.848038    ,    0.831026    ,    0.812044    ,    0.809278    ,    0.80962    ,    0.811683    ,    0.821349    ,    0.813834    ,    0.815275    ,    0.857737    ,    0.835619    ,    0.800298    ,    0.790797    ,    0.79135    ,    0.791074    ,    0.785237    ,    0.7793    ,    0.776289    ,    0.772236    ,    0.769978    ,    0.76907    ,    0.768147    ,    0.771575    ,    0.770709    ,    0.76709    ,    0.770797    ,    0.770385    ,    0.770463    ,    0.770771    },
    {    0.793594    ,    0.793704    ,    0.792505    ,    0.792607    ,    0.793389    ,    0.793018    ,    0.794124    ,    0.796467    ,    0.796348    ,    0.797343    ,    0.799538    ,    0.802128    ,    0.803292    ,    0.801211    ,    0.800356    ,    0.801905    ,    0.803898    ,    0.804669    ,    0.807112    ,    0.811507    ,    0.808832    ,    0.810995    ,    0.812576    ,    0.815241    ,    0.819432    ,    0.828899    ,    0.836519    ,    0.845307    ,    0.855281    ,    0.864821    ,    0.874453    ,    0.881111    ,    0.872785    ,    0.895513    ,    0.927643    ,    0.834551    ,    0.842225    ,    0.988662    ,    1.12814    ,    1.13375    ,    1.04437    ,    0.977203    ,    0.888584    ,    0.801439    ,    0.792719    ,    0.799318    ,    0.804353    ,    0.804572    ,    0.805742    ,    0.810499    ,    0.815866    ,    0.822271    ,    0.836066    ,    0.840453    ,    0.845985    ,    0.849045    ,    0.846756    ,    0.845255    ,    0.834981    ,    0.945984    ,    1.08329    ,    1.12187    ,    1.11575    ,    1.11871    ,    1.12934    ,    1.07496    ,    1.10254    ,    1.1486    ,    1.17348    ,    1.18525    ,    1.20093    ,    1.14314    ,    1.16395    ,    1.21857    ,    1.12828    ,    1.09648    ,    1.15735    ,    1.21066    ,    1.21454    ,    1.17839    ,    1.10542    ,    1.10604    ,    1.00975    ,    0.905495    ,    0.919842    ,    0.922585    ,    0.920744    ,    0.914056    ,    0.898044    ,    0.88128    ,    0.863444    ,    0.879198    ,    0.877583    ,    0.847127    ,    0.843743    ,    0.844175    ,    0.846775    ,    0.849052    ,    0.854396    ,    0.889728    ,    0.840123    ,    0.853252    ,    0.841896    ,    0.810679    ,    0.809853    ,    0.808738    ,    0.809856    ,    0.821247    ,    0.855217    ,    0.822806    ,    0.800951    ,    0.796617    ,    0.795241    ,    0.793121    ,    0.788814    ,    0.786403    ,    0.784335    ,    0.780938    ,    0.782857    ,    0.783897    ,    0.784622    ,    0.786079    ,    0.790404    ,    0.791993    ,    0.796632    ,    0.795461    ,    0.796499    ,    0.796436    },
    {    0.816292    ,    0.817123    ,    0.821846    ,    0.822356    ,    0.825182    ,    0.827546    ,    0.830048    ,    0.833073    ,    0.835197    ,    0.836558    ,    0.836896    ,    0.838714    ,    0.841294    ,    0.841476    ,    0.838671    ,    0.83718    ,    0.831855    ,    0.824265    ,    0.820349    ,    0.813472    ,    0.803832    ,    0.798771    ,    0.795681    ,    0.795842    ,    0.802005    ,    0.812234    ,    0.824979    ,    0.84724    ,    0.859974    ,    0.868499    ,    0.868152    ,    0.881995    ,    0.921063    ,    1.03117    ,    0.972609    ,    0.794654    ,    0.79832    ,    0.801082    ,    0.810791    ,    0.816445    ,    0.805937    ,    0.793754    ,    0.786094    ,    0.78282    ,    0.784257    ,    0.78232    ,    0.779925    ,    0.776661    ,    0.780465    ,    0.787154    ,    0.794577    ,    0.804575    ,    0.820011    ,    0.828182    ,    0.833501    ,    0.841028    ,    0.851329    ,    0.854613    ,    0.902197    ,    1.15671    ,    1.24959    ,    1.25267    ,    1.22071    ,    1.22292    ,    1.20218    ,    1.19514    ,    1.17795    ,    1.16931    ,    1.19233    ,    1.21402    ,    1.25356    ,    1.24063    ,    1.22371    ,    1.23103    ,    1.19632    ,    1.19542    ,    1.1721    ,    1.19792    ,    1.19582    ,    1.11748    ,    1.00365    ,    0.958665    ,    0.937757    ,    0.909535    ,    0.922043    ,    0.922751    ,    0.922807    ,    0.919066    ,    0.901888    ,    0.88335    ,    0.86839    ,    0.957602    ,    0.953429    ,    0.849668    ,    0.839054    ,    0.841053    ,    0.846137    ,    0.856058    ,    0.869151    ,    0.932187    ,    0.917069    ,    0.970567    ,    0.949166    ,    0.833275    ,    0.811796    ,    0.809514    ,    0.820697    ,    0.836221    ,    0.842465    ,    0.805979    ,    0.803793    ,    0.799816    ,    0.797894    ,    0.796225    ,    0.795902    ,    0.796748    ,    0.795864    ,    0.79574    ,    0.796691    ,    0.799206    ,    0.802084    ,    0.804402    ,    0.804339    ,    0.807249    ,    0.810331    ,    0.810592    ,    0.811533    ,    0.813375    },
    {    0.818345    ,    0.820972    ,    0.825057    ,    0.830047    ,    0.838006    ,    0.844073    ,    0.849287    ,    0.85462    ,    0.862011    ,    0.86878    ,    0.874405    ,    0.879629    ,    0.883144    ,    0.885822    ,    0.883818    ,    0.877523    ,    0.866757    ,    0.854592    ,    0.841897    ,    0.824052    ,    0.803858    ,    0.788178    ,    0.783967    ,    0.785927    ,    0.793115    ,    0.797887    ,    0.81713    ,    0.845021    ,    0.862232    ,    0.932151    ,    0.971573    ,    1.04118    ,    1.1512    ,    1.03195    ,    0.901373    ,    0.801028    ,    0.801738    ,    0.793206    ,    0.789025    ,    0.785428    ,    0.784267    ,    0.78362    ,    0.785615    ,    0.783515    ,    0.777949    ,    0.766336    ,    0.759313    ,    0.755406    ,    0.754405    ,    0.755058    ,    0.758945    ,    0.767212    ,    0.778829    ,    0.792486    ,    0.805801    ,    0.820999    ,    0.843606    ,    0.865938    ,    0.949452    ,    1.17411    ,    1.28877    ,    1.30857    ,    1.30747    ,    1.30141    ,    1.31288    ,    1.32392    ,    1.25776    ,    1.31988    ,    1.28743    ,    1.26555    ,    1.2893    ,    1.2899    ,    1.24498    ,    1.30508    ,    1.30723    ,    1.29738    ,    1.30112    ,    1.3377    ,    1.17101    ,    1.06747    ,    1.22872    ,    1.16116    ,    1.06401    ,    0.940261    ,    0.912987    ,    0.921665    ,    0.921103    ,    0.921745    ,    0.912013    ,    0.896998    ,    0.904389    ,    1.03558    ,    1.02068    ,    0.86715    ,    0.832688    ,    0.838114    ,    0.848757    ,    0.857306    ,    0.894789    ,    0.974899    ,    1.05539    ,    1.04323    ,    0.995287    ,    0.847881    ,    0.820645    ,    0.81258    ,    0.828539    ,    0.857856    ,    0.820893    ,    0.806534    ,    0.807187    ,    0.801733    ,    0.804889    ,    0.805339    ,    0.804505    ,    0.80707    ,    0.804035    ,    0.807655    ,    0.805009    ,    0.80427    ,    0.803614    ,    0.805209    ,    0.805708    ,    0.806403    ,    0.807693    ,    0.809979    ,    0.812218    ,    0.815757    },
    {    0.811681    ,    0.812328    ,    0.813763    ,    0.817685    ,    0.824509    ,    0.832368    ,    0.84223    ,    0.852425    ,    0.863921    ,    0.871562    ,    0.878959    ,    0.886533    ,    0.892366    ,    0.89844    ,    0.904496    ,    0.907296    ,    0.907273    ,    0.903867    ,    0.892945    ,    0.873448    ,    0.847596    ,    0.821881    ,    0.810323    ,    0.801951    ,    0.799103    ,    0.796153    ,    0.821585    ,    0.935737    ,    1.06262    ,    1.13789    ,    1.08125    ,    1.06182    ,    1.07213    ,    0.849941    ,    0.808172    ,    0.812813    ,    0.820406    ,    0.824708    ,    0.860379    ,    0.868912    ,    0.812679    ,    0.796598    ,    0.788936    ,    0.781412    ,    0.772972    ,    0.763422    ,    0.754778    ,    0.743167    ,    0.737492    ,    0.732259    ,    0.728094    ,    0.725156    ,    0.728594    ,    0.737715    ,    0.755695    ,    0.781649    ,    0.81354    ,    0.849977    ,    0.936751    ,    1.16372    ,    1.2795    ,    1.35207    ,    1.37731    ,    1.37822    ,    1.35519    ,    1.33546    ,    1.32493    ,    1.28469    ,    1.29801    ,    1.28847    ,    1.2616    ,    1.23058    ,    1.22183    ,    1.23771    ,    1.2812    ,    1.31114    ,    1.30557    ,    1.25191    ,    1.03103    ,    1.18147    ,    1.36017    ,    1.38017    ,    1.29526    ,    1.14418    ,    0.974394    ,    0.903823    ,    0.91636    ,    0.92697    ,    0.926917    ,    0.930635    ,    1.00364    ,    1.10856    ,    1.11886    ,    1.05644    ,    0.886125    ,    0.848499    ,    0.856619    ,    0.894302    ,    1.03831    ,    1.06729    ,    1.07769    ,    1.07665    ,    0.974618    ,    0.889189    ,    0.838223    ,    0.825034    ,    0.8288    ,    0.841998    ,    0.823987    ,    0.826076    ,    0.822605    ,    0.817491    ,    0.819609    ,    0.824335    ,    0.824452    ,    0.822371    ,    0.81978    ,    0.816475    ,    0.81218    ,    0.809081    ,    0.807993    ,    0.805857    ,    0.804463    ,    0.805718    ,    0.807679    ,    0.809042    ,    0.809084    ,    0.811336    },
    {    0.80657    ,    0.802539    ,    0.799972    ,    0.800988    ,    0.80617    ,    0.81321    ,    0.821987    ,    0.834215    ,    0.850133    ,    0.857595    ,    0.864831    ,    0.875191    ,    0.884641    ,    0.8918    ,    0.900638    ,    0.909564    ,    0.915183    ,    0.920427    ,    0.923491    ,    0.92312    ,    0.919845    ,    0.899996    ,    0.859751    ,    0.82317    ,    0.804261    ,    0.803709    ,    0.860177    ,    1.15473    ,    1.21324    ,    1.11018    ,    0.887975    ,    0.823116    ,    0.887318    ,    0.855671    ,    0.823973    ,    0.844872    ,    0.872938    ,    0.863084    ,    0.839196    ,    0.843304    ,    0.827398    ,    0.814058    ,    0.807893    ,    0.806547    ,    0.801246    ,    0.794455    ,    0.784617    ,    0.769129    ,    0.755769    ,    0.744561    ,    0.736241    ,    0.728093    ,    0.718972    ,    0.713073    ,    0.715559    ,    0.728319    ,    0.760298    ,    0.818935    ,    0.966305    ,    1.22422    ,    1.31111    ,    1.36613    ,    1.39385    ,    1.40048    ,    1.40852    ,    1.37245    ,    1.32739    ,    1.29602    ,    1.29336    ,    1.2694    ,    1.25016    ,    1.2255    ,    1.25531    ,    1.2482    ,    1.27621    ,    1.31841    ,    1.34523    ,    1.23026    ,    1.17677    ,    1.38431    ,    1.39405    ,    1.42729    ,    1.42934    ,    1.35726    ,    1.23796    ,    1.00883    ,    0.914452    ,    0.930866    ,    0.951517    ,    1.03969    ,    1.10568    ,    1.15445    ,    1.14618    ,    1.17063    ,    1.09601    ,    0.966481    ,    0.926671    ,    0.997632    ,    1.082    ,    1.09765    ,    1.12564    ,    1.18102    ,    1.09997    ,    1.04312    ,    0.942316    ,    0.877742    ,    0.846434    ,    0.835388    ,    0.837294    ,    0.855394    ,    0.85771    ,    0.855437    ,    0.8562    ,    0.858866    ,    0.86082    ,    0.860234    ,    0.857394    ,    0.85312    ,    0.848424    ,    0.842032    ,    0.837137    ,    0.830007    ,    0.827226    ,    0.823886    ,    0.821306    ,    0.818567    ,    0.815744    ,    0.813506    },
    {    0.837181    ,    0.826658    ,    0.813727    ,    0.804505    ,    0.798743    ,    0.793249    ,    0.794647    ,    0.802895    ,    0.813247    ,    0.824545    ,    0.837075    ,    0.848841    ,    0.861923    ,    0.874058    ,    0.885994    ,    0.898027    ,    0.910011    ,    0.922709    ,    0.93613    ,    0.947723    ,    0.958217    ,    0.939881    ,    0.889373    ,    0.843332    ,    0.830363    ,    0.916264    ,    1.04626    ,    1.19194    ,    1.21188    ,    1.09448    ,    0.862264    ,    0.81294    ,    0.805113    ,    0.81875    ,    0.841967    ,    0.850324    ,    0.822683    ,    0.806638    ,    0.815483    ,    0.828266    ,    0.836545    ,    0.8385    ,    0.834358    ,    0.830956    ,    0.826966    ,    0.826982    ,    0.826997    ,    0.821357    ,    0.809834    ,    0.798699    ,    0.787449    ,    0.774422    ,    0.756947    ,    0.741368    ,    0.729163    ,    0.723484    ,    0.730174    ,    0.757159    ,    0.851137    ,    1.15422    ,    1.29443    ,    1.35725    ,    1.3637    ,    1.36721    ,    1.39808    ,    1.36716    ,    1.34169    ,    1.26609    ,    1.20628    ,    1.18009    ,    1.16428    ,    1.15135    ,    1.19473    ,    1.18842    ,    1.21707    ,    1.29439    ,    1.32478    ,    1.18192    ,    1.37344    ,    1.43559    ,    1.45539    ,    1.44441    ,    1.36599    ,    1.30392    ,    1.33963    ,    1.08164    ,    0.979865    ,    0.977641    ,    1.04455    ,    1.20807    ,    1.29631    ,    1.24187    ,    1.16754    ,    1.19151    ,    1.21605    ,    1.15981    ,    1.16687    ,    1.1635    ,    1.1999    ,    1.2033    ,    1.15683    ,    1.1881    ,    1.22161    ,    1.25692    ,    1.21828    ,    1.12574    ,    0.986502    ,    0.936047    ,    0.864116    ,    0.871945    ,    0.886775    ,    0.896748    ,    0.902731    ,    0.90428    ,    0.903193    ,    0.905775    ,    0.906372    ,    0.902214    ,    0.898855    ,    0.89538    ,    0.889637    ,    0.884218    ,    0.880262    ,    0.875509    ,    0.870047    ,    0.861468    ,    0.85487    ,    0.847751    },
    {    0.901802    ,    0.887193    ,    0.868924    ,    0.852881    ,    0.83482    ,    0.814713    ,    0.80169    ,    0.796639    ,    0.797194    ,    0.805562    ,    0.813388    ,    0.822973    ,    0.837654    ,    0.853538    ,    0.867061    ,    0.879348    ,    0.893658    ,    0.91176    ,    0.932024    ,    0.95686    ,    0.971178    ,    0.946901    ,    0.892743    ,    0.88483    ,    1.05254    ,    1.1307    ,    1.26158    ,    1.32864    ,    1.30995    ,    1.11854    ,    0.917769    ,    0.871953    ,    0.858268    ,    0.875947    ,    0.907008    ,    0.937961    ,    0.831016    ,    0.825637    ,    0.856194    ,    0.865155    ,    0.867373    ,    0.871715    ,    0.863399    ,    0.855818    ,    0.848267    ,    0.841038    ,    0.837945    ,    0.838528    ,    0.837166    ,    0.83632    ,    0.830395    ,    0.82072    ,    0.809007    ,    0.792237    ,    0.771309    ,    0.749813    ,    0.734228    ,    0.72763    ,    0.744916    ,    0.902501    ,    1.17252    ,    1.32174    ,    1.34779    ,    1.36239    ,    1.40012    ,    1.38168    ,    1.32328    ,    1.22699    ,    1.16392    ,    1.12943    ,    1.07813    ,    1.07767    ,    1.10355    ,    1.133    ,    1.17988    ,    1.24628    ,    1.27334    ,    1.31604    ,    1.43187    ,    1.45206    ,    1.45777    ,    1.44519    ,    1.27465    ,    1.24967    ,    1.29781    ,    1.33401    ,    1.33689    ,    1.32264    ,    1.32371    ,    1.36392    ,    1.35039    ,    1.28968    ,    1.2798    ,    1.20038    ,    1.22528    ,    1.2387    ,    1.25167    ,    1.26953    ,    1.27407    ,    1.21388    ,    1.19984    ,    1.23382    ,    1.30697    ,    1.3325    ,    1.33239    ,    1.27633    ,    1.19604    ,    1.04987    ,    0.937116    ,    0.907207    ,    0.903759    ,    0.924533    ,    0.938175    ,    0.943225    ,    0.945443    ,    0.948902    ,    0.954445    ,    0.957158    ,    0.959688    ,    0.957795    ,    0.953637    ,    0.948598    ,    0.943311    ,    0.940073    ,    0.931952    ,    0.923634    ,    0.915964    ,    0.910385    },
    {    0.966999    ,    0.952759    ,    0.936628    ,    0.921039    ,    0.900412    ,    0.87527    ,    0.850034    ,    0.830332    ,    0.819819    ,    0.818918    ,    0.825062    ,    0.836784    ,    0.846583    ,    0.854663    ,    0.865841    ,    0.874143    ,    0.883622    ,    0.896378    ,    0.921741    ,    0.955351    ,    0.96905    ,    0.939554    ,    0.91533    ,    1.02745    ,    1.22138    ,    1.29707    ,    1.39654    ,    1.37537    ,    1.26999    ,    1.23616    ,    1.07977    ,    1.01165    ,    0.994183    ,    0.989387    ,    0.99732    ,    0.985826    ,    0.864224    ,    0.850163    ,    0.885529    ,    0.887645    ,    0.88595    ,    0.886472    ,    0.882609    ,    0.876217    ,    0.869007    ,    0.85447    ,    0.84479    ,    0.837211    ,    0.828033    ,    0.822309    ,    0.814574    ,    0.809331    ,    0.803304    ,    0.802699    ,    0.802716    ,    0.791643    ,    0.769207    ,    0.743958    ,    0.728414    ,    0.740413    ,    0.873744    ,    1.30923    ,    1.35219    ,    1.31504    ,    1.34397    ,    1.32611    ,    1.28687    ,    1.24909    ,    1.22269    ,    1.14086    ,    1.06001    ,    1.03863    ,    1.07219    ,    1.09872    ,    1.09406    ,    1.14553    ,    1.2343    ,    1.34306    ,    1.41124    ,    1.43185    ,    1.44325    ,    1.38066    ,    1.37358    ,    1.48969    ,    1.47973    ,    1.43882    ,    1.46357    ,    1.50828    ,    1.48538    ,    1.47389    ,    1.42581    ,    1.33454    ,    1.26643    ,    1.39513    ,    1.63626    ,    1.67643    ,    1.56989    ,    1.53553    ,    1.5055    ,    1.46609    ,    1.30662    ,    1.34938    ,    1.37465    ,    1.34624    ,    1.38217    ,    1.34488    ,    1.2934    ,    1.19714    ,    1.0353    ,    0.985219    ,    0.954502    ,    0.949695    ,    0.950912    ,    0.955388    ,    0.965414    ,    0.970323    ,    0.979353    ,    0.98572    ,    0.987648    ,    0.991288    ,    0.994104    ,    0.99346    ,    0.994618    ,    0.995699    ,    0.993311    ,    0.98788    ,    0.984434    ,    0.978359    },
    {    1.03515    ,    1.02601    ,    1.01197    ,    0.992422    ,    0.967199    ,    0.938417    ,    0.90845    ,    0.884521    ,    0.867592    ,    0.853926    ,    0.848746    ,    0.852502    ,    0.8585    ,    0.86409    ,    0.87099    ,    0.87298    ,    0.876756    ,    0.889721    ,    0.915797    ,    0.946712    ,    0.950614    ,    0.942609    ,    0.98408    ,    1.20288    ,    1.2911    ,    1.34708    ,    1.46015    ,    1.36176    ,    1.37005    ,    1.3165    ,    1.25197    ,    1.23388    ,    1.24518    ,    1.1918    ,    1.15562    ,    1.1005    ,    0.957363    ,    0.902126    ,    0.890013    ,    0.885817    ,    0.884979    ,    0.884596    ,    0.881769    ,    0.877285    ,    0.871149    ,    0.862376    ,    0.853279    ,    0.841165    ,    0.830193    ,    0.819691    ,    0.80722    ,    0.793744    ,    0.772891    ,    0.75568    ,    0.751829    ,    0.767057    ,    0.786459    ,    0.77361    ,    0.744664    ,    0.730147    ,    0.775693    ,    1.14672    ,    1.41672    ,    1.39323    ,    1.32784    ,    1.28888    ,    1.2432    ,    1.18718    ,    1.13876    ,    1.02888    ,    0.929847    ,    0.921019    ,    1.00099    ,    0.938079    ,    0.926706    ,    0.959522    ,    1.0118    ,    1.24846    ,    1.34323    ,    1.395    ,    1.4409    ,    1.46238    ,    1.51094    ,    1.50792    ,    1.45534    ,    1.43029    ,    1.46665    ,    1.53861    ,    1.57377    ,    1.48973    ,    1.4984    ,    1.44607    ,    1.49485    ,    1.75475    ,    1.60784    ,    1.54491    ,    1.53976    ,    1.51903    ,    1.54248    ,    1.53296    ,    1.44164    ,    1.34664    ,    1.31799    ,    1.30442    ,    1.32997    ,    1.32337    ,    1.3176    ,    1.23844    ,    1.12825    ,    1.08209    ,    1.06916    ,    1.05546    ,    1.02075    ,    1.00756    ,    1.00142    ,    1.00269    ,    1.00719    ,    1.01781    ,    1.02495    ,    1.03048    ,    1.03744    ,    1.04127    ,    1.04537    ,    1.04851    ,    1.04962    ,    1.05006    ,    1.04696    ,    1.04166    },
    {    1.09179    ,    1.0808    ,    1.06388    ,    1.0439    ,    1.02108    ,    0.995322    ,    0.966476    ,    0.940258    ,    0.921347    ,    0.906531    ,    0.894424    ,    0.892369    ,    0.892695    ,    0.889056    ,    0.887065    ,    0.888879    ,    0.892295    ,    0.90154    ,    0.923334    ,    0.947804    ,    0.956731    ,    1.03701    ,    1.21992    ,    1.28391    ,    1.3463    ,    1.38102    ,    1.38703    ,    1.38411    ,    1.41391    ,    1.39492    ,    1.3184    ,    1.28858    ,    1.24716    ,    1.25486    ,    1.24249    ,    1.23114    ,    1.21716    ,    1.05141    ,    0.942327    ,    0.906605    ,    0.900495    ,    0.894657    ,    0.890076    ,    0.887767    ,    0.881054    ,    0.871079    ,    0.862988    ,    0.854051    ,    0.841039    ,    0.825573    ,    0.808282    ,    0.786404    ,    0.773179    ,    0.758296    ,    0.729792    ,    0.713756    ,    0.732501    ,    0.756365    ,    0.745325    ,    0.707298    ,    0.721789    ,    0.857609    ,    1.19162    ,    1.24379    ,    1.29853    ,    1.31051    ,    1.27181    ,    1.20086    ,    1.00627    ,    0.941709    ,    0.935248    ,    0.93019    ,    0.943532    ,    0.955754    ,    0.995414    ,    1.03041    ,    1.03867    ,    1.17148    ,    1.31969    ,    1.36698    ,    1.44511    ,    1.51398    ,    1.45356    ,    1.42343    ,    1.44314    ,    1.41553    ,    1.42162    ,    1.52445    ,    1.6591    ,    1.65859    ,    1.56335    ,    1.53557    ,    1.53632    ,    1.5338    ,    1.48997    ,    1.49779    ,    1.51991    ,    1.53203    ,    1.48041    ,    1.44973    ,    1.38515    ,    1.3439    ,    1.29444    ,    1.29983    ,    1.30664    ,    1.30689    ,    1.30688    ,    1.25799    ,    1.213    ,    1.20383    ,    1.15904    ,    1.1595    ,    1.16171    ,    1.13326    ,    1.08097    ,    1.06613    ,    1.07112    ,    1.0799    ,    1.08546    ,    1.0968    ,    1.10991    ,    1.12099    ,    1.12371    ,    1.12109    ,    1.11641    ,    1.11358    ,    1.1093    ,    1.10287    },
    {    1.11765    ,    1.10631    ,    1.09205    ,    1.07385    ,    1.05418    ,    1.03266    ,    1.00984    ,    0.988492    ,    0.972603    ,    0.957849    ,    0.948589    ,    0.946593    ,    0.942725    ,    0.934363    ,    0.929399    ,    0.924703    ,    0.920628    ,    0.922817    ,    0.938962    ,    0.959689    ,    1.01788    ,    1.16977    ,    1.38792    ,    1.40822    ,    1.41702    ,    1.44074    ,    1.40064    ,    1.36495    ,    1.41757    ,    1.38749    ,    1.32289    ,    1.3374    ,    1.34311    ,    1.34426    ,    1.3493    ,    1.34317    ,    1.34831    ,    1.24906    ,    1.06936    ,    1.01473    ,    1.00691    ,    0.989764    ,    0.973101    ,    0.957047    ,    0.939971    ,    0.923249    ,    0.910887    ,    0.899612    ,    0.877363    ,    0.852612    ,    0.828616    ,    0.796025    ,    0.763749    ,    0.744386    ,    0.723732    ,    0.703428    ,    0.692625    ,    0.712006    ,    0.724023    ,    0.681128    ,    0.694803    ,    0.976477    ,    1.24456    ,    1.17851    ,    0.960417    ,    0.9824    ,    1.01703    ,    1.02237    ,    0.977318    ,    1.01157    ,    1.00155    ,    1.01662    ,    1.13502    ,    1.04989    ,    1.21445    ,    1.26915    ,    1.28749    ,    1.32679    ,    1.38869    ,    1.46193    ,    1.48493    ,    1.40597    ,    1.21306    ,    1.23583    ,    1.36355    ,    1.36145    ,    1.29561    ,    1.34893    ,    1.47234    ,    1.7156    ,    1.75469    ,    1.4954    ,    1.37813    ,    1.37164    ,    1.34395    ,    1.34836    ,    1.37478    ,    1.37648    ,    1.37742    ,    1.3715    ,    1.32907    ,    1.37516    ,    1.33073    ,    1.32527    ,    1.32806    ,    1.3303    ,    1.35081    ,    1.32926    ,    1.33183    ,    1.30365    ,    1.27033    ,    1.2261    ,    1.19249    ,    1.19483    ,    1.17998    ,    1.14425    ,    1.16308    ,    1.18159    ,    1.19011    ,    1.20255    ,    1.21295    ,    1.22009    ,    1.21611    ,    1.20151    ,    1.18297    ,    1.16683    ,    1.1528    ,    1.13553    },
    {    1.11463    ,    1.10036    ,    1.08876    ,    1.07477    ,    1.06017    ,    1.04071    ,    1.02351    ,    1.01172    ,    1.00465    ,    0.999033    ,    0.995604    ,    0.99668    ,    0.996022    ,    0.990765    ,    0.984253    ,    0.973246    ,    0.962474    ,    0.955442    ,    0.961046    ,    0.977151    ,    1.11252    ,    1.31742    ,    1.41786    ,    1.47007    ,    1.49525    ,    1.54222    ,    1.44528    ,    1.34945    ,    1.38259    ,    1.41008    ,    1.40311    ,    1.41931    ,    1.4571    ,    1.4334    ,    1.43847    ,    1.45619    ,    1.46767    ,    1.46363    ,    1.35246    ,    1.24974    ,    1.18989    ,    1.14702    ,    1.11333    ,    1.0763    ,    1.06322    ,    1.04897    ,    1.04134    ,    1.06705    ,    1.06536    ,    1.00641    ,    0.943063    ,    0.873648    ,    0.795518    ,    0.723567    ,    0.699355    ,    0.690462    ,    0.666534    ,    0.661117    ,    0.6652    ,    0.648383    ,    0.657163    ,    0.996083    ,    1.24649    ,    1.2961    ,    1.11435    ,    0.95887    ,    0.950632    ,    1.02191    ,    1.01181    ,    1.09266    ,    1.17515    ,    1.18797    ,    1.22839    ,    1.13604    ,    1.11927    ,    1.1273    ,    1.18371    ,    1.18122    ,    1.22308    ,    1.37676    ,    1.36796    ,    1.31096    ,    1.15508    ,    1.23094    ,    1.31017    ,    1.29222    ,    1.25961    ,    1.29656    ,    1.35437    ,    1.48833    ,    1.4536    ,    1.40083    ,    1.39668    ,    1.3916    ,    1.41233    ,    1.43628    ,    1.43416    ,    1.42549    ,    1.43405    ,    1.40725    ,    1.41296    ,    1.43256    ,    1.39675    ,    1.36918    ,    1.37667    ,    1.38842    ,    1.40674    ,    1.42588    ,    1.4543    ,    1.41329    ,    1.38691    ,    1.36595    ,    1.33179    ,    1.27188    ,    1.25627    ,    1.2571    ,    1.26909    ,    1.26474    ,    1.26303    ,    1.27011    ,    1.27298    ,    1.272    ,    1.26445    ,    1.24229    ,    1.20918    ,    1.18177    ,    1.16037    ,    1.13746    },
    {    1.09347    ,    1.08277    ,    1.07451    ,    1.06537    ,    1.05598    ,    1.0447    ,    1.03307    ,    1.02701    ,    1.02654    ,    1.02928    ,    1.03282    ,    1.03695    ,    1.0402    ,    1.03643    ,    1.03096    ,    1.02161    ,    1.00845    ,    0.994667    ,    0.989337    ,    0.996958    ,    1.12116    ,    1.30134    ,    1.36392    ,    1.47479    ,    1.53394    ,    1.49357    ,    1.42048    ,    1.36755    ,    1.40536    ,    1.4707    ,    1.51481    ,    1.56571    ,    1.52794    ,    1.52498    ,    1.53944    ,    1.58498    ,    1.5827    ,    1.58426    ,    1.56628    ,    1.48018    ,    1.36191    ,    1.30358    ,    1.24622    ,    1.18742    ,    1.15478    ,    1.1185    ,    1.11578    ,    1.2582    ,    1.29682    ,    1.15716    ,    0.986133    ,    0.834699    ,    0.725417    ,    0.643549    ,    0.617555    ,    0.624795    ,    0.612252    ,    0.606451    ,    0.601276    ,    0.596772    ,    0.613617    ,    0.792617    ,    0.918148    ,    0.973997    ,    1.11986    ,    1.10339    ,    1.08433    ,    1.08304    ,    1.16727    ,    1.18918    ,    1.20778    ,    1.25167    ,    1.24853    ,    1.23506    ,    1.14302    ,    1.06326    ,    1.0595    ,    1.05215    ,    1.11279    ,    1.22165    ,    1.25894    ,    1.19567    ,    1.19344    ,    1.32558    ,    1.35603    ,    1.34296    ,    1.34973    ,    1.3373    ,    1.37655    ,    1.33705    ,    1.37291    ,    1.39227    ,    1.4189    ,    1.43172    ,    1.44655    ,    1.48331    ,    1.51226    ,    1.51657    ,    1.48209    ,    1.46956    ,    1.46256    ,    1.44864    ,    1.40429    ,    1.40146    ,    1.40939    ,    1.41184    ,    1.42898    ,    1.47995    ,    1.48445    ,    1.48077    ,    1.46601    ,    1.47871    ,    1.39022    ,    1.33407    ,    1.38446    ,    1.43404    ,    1.38756    ,    1.29937    ,    1.2607    ,    1.2434    ,    1.23619    ,    1.23716    ,    1.23619    ,    1.22014    ,    1.18608    ,    1.15443    ,    1.12738    ,    1.10983    },
    {    1.08747    ,    1.07307    ,    1.06589    ,    1.06499    ,    1.06619    ,    1.06525    ,    1.06188    ,    1.06276    ,    1.06468    ,    1.07112    ,    1.07823    ,    1.0821    ,    1.08498    ,    1.08059    ,    1.07176    ,    1.05956    ,    1.04638    ,    1.03071    ,    1.02253    ,    1.02593    ,    1.12518    ,    1.2552    ,    1.32257    ,    1.39822    ,    1.41723    ,    1.39947    ,    1.39423    ,    1.42718    ,    1.48189    ,    1.55111    ,    1.59776    ,    1.59402    ,    1.61376    ,    1.64192    ,    1.65248    ,    1.68375    ,    1.6616    ,    1.67117    ,    1.68683    ,    1.64209    ,    1.58697    ,    1.52687    ,    1.45169    ,    1.33312    ,    1.22389    ,    1.12692    ,    1.04622    ,    1.12274    ,    1.13203    ,    1.053    ,    0.854118    ,    0.65637    ,    0.50764    ,    0.449002    ,    0.477755    ,    0.495311    ,    0.497556    ,    0.499623    ,    0.509472    ,    0.530675    ,    0.568604    ,    0.631841    ,    0.685015    ,    0.804558    ,    1.01639    ,    1.09644    ,    1.1573    ,    1.17468    ,    1.2096    ,    1.22756    ,    1.25365    ,    1.29457    ,    1.2386    ,    1.25947    ,    1.23425    ,    1.19857    ,    1.20794    ,    1.17807    ,    1.19777    ,    1.25994    ,    1.30049    ,    1.25457    ,    1.24806    ,    1.33529    ,    1.44523    ,    1.46056    ,    1.48006    ,    1.49813    ,    1.49981    ,    1.48999    ,    1.48778    ,    1.48923    ,    1.48806    ,    1.47652    ,    1.49993    ,    1.51532    ,    1.49926    ,    1.48218    ,    1.46784    ,    1.40836    ,    1.39395    ,    1.3993    ,    1.4086    ,    1.41103    ,    1.41274    ,    1.42657    ,    1.4161    ,    1.45246    ,    1.45381    ,    1.48215    ,    1.46767    ,    1.49414    ,    1.48337    ,    1.46661    ,    1.55696    ,    1.63033    ,    1.62175    ,    1.52378    ,    1.43326    ,    1.38158    ,    1.34622    ,    1.32216    ,    1.2939    ,    1.26265    ,    1.21924    ,    1.1738    ,    1.13548    ,    1.10972    },
    {    1.13647    ,    1.12403    ,    1.1127    ,    1.11532    ,    1.12189    ,    1.12169    ,    1.12347    ,    1.12819    ,    1.13251    ,    1.13856    ,    1.14583    ,    1.14746    ,    1.14203    ,    1.13361    ,    1.11925    ,    1.10149    ,    1.08694    ,    1.07058    ,    1.06042    ,    1.10819    ,    1.24031    ,    1.37042    ,    1.42151    ,    1.41544    ,    1.44759    ,    1.43439    ,    1.46295    ,    1.5431    ,    1.59516    ,    1.66114    ,    1.68127    ,    1.65619    ,    1.66171    ,    1.70384    ,    1.7295    ,    1.74408    ,    1.76055    ,    1.72642    ,    1.70006    ,    1.66193    ,    1.64654    ,    1.58263    ,    1.49241    ,    1.4253    ,    1.31716    ,    1.09036    ,    0.898962    ,    0.803042    ,    0.754226    ,    0.714338    ,    0.581575    ,    0.442222    ,    0.309738    ,    0.279844    ,    0.34402    ,    0.36849    ,    0.374806    ,    0.391201    ,    0.422931    ,    0.480158    ,    0.531012    ,    0.596179    ,    0.682995    ,    0.815202    ,    0.963847    ,    1.03295    ,    1.09095    ,    1.12848    ,    1.1877    ,    1.23792    ,    1.29924    ,    1.31396    ,    1.31001    ,    1.27082    ,    1.26475    ,    1.28303    ,    1.29814    ,    1.30345    ,    1.32878    ,    1.38675    ,    1.42625    ,    1.41978    ,    1.41497    ,    1.462    ,    1.52689    ,    1.59465    ,    1.60069    ,    1.60132    ,    1.61208    ,    1.62682    ,    1.61729    ,    1.60396    ,    1.5929    ,    1.56914    ,    1.52404    ,    1.4842    ,    1.42748    ,    1.46376    ,    1.42402    ,    1.42347    ,    1.3763    ,    1.38593    ,    1.40657    ,    1.40819    ,    1.4119    ,    1.42421    ,    1.4668    ,    1.47544    ,    1.44878    ,    1.47765    ,    1.4595    ,    1.51775    ,    1.5547    ,    1.53898    ,    1.63786    ,    1.70485    ,    1.80013    ,    1.67682    ,    1.55616    ,    1.50442    ,    1.47776    ,    1.43483    ,    1.3846    ,    1.32867    ,    1.27634    ,    1.22316    ,    1.18099    ,    1.15462    },
    {    1.23868    ,    1.22618    ,    1.21075    ,    1.2099    ,    1.21321    ,    1.21013    ,    1.20898    ,    1.20826    ,    1.2108    ,    1.2152    ,    1.2175    ,    1.21881    ,    1.21163    ,    1.19631    ,    1.16994    ,    1.13866    ,    1.11419    ,    1.10119    ,    1.13289    ,    1.30905    ,    1.41157    ,    1.44006    ,    1.45454    ,    1.44504    ,    1.4783    ,    1.51792    ,    1.56969    ,    1.63096    ,    1.65591    ,    1.70714    ,    1.67855    ,    1.69186    ,    1.70601    ,    1.74123    ,    1.81951    ,    1.90204    ,    1.89794    ,    1.82573    ,    1.75774    ,    1.69715    ,    1.65014    ,    1.62096    ,    1.55979    ,    1.49782    ,    1.27735    ,    0.950032    ,    0.753591    ,    0.585489    ,    0.440064    ,    0.334297    ,    0.284194    ,    0.275099    ,    0.236865    ,    0.211823    ,    0.246649    ,    0.270113    ,    0.28828    ,    0.32697    ,    0.369472    ,    0.45059    ,    0.511597    ,    0.63721    ,    0.706621    ,    0.847167    ,    0.904063    ,    0.926575    ,    1.0129    ,    1.0979    ,    1.15405    ,    1.21128    ,    1.2619    ,    1.28118    ,    1.30745    ,    1.32535    ,    1.3447    ,    1.35798    ,    1.37611    ,    1.39416    ,    1.41331    ,    1.44881    ,    1.51229    ,    1.56527    ,    1.57103    ,    1.58564    ,    1.60256    ,    1.63973    ,    1.69744    ,    1.72383    ,    1.72036    ,    1.70576    ,    1.68749    ,    1.67296    ,    1.68547    ,    1.64176    ,    1.52614    ,    1.42861    ,    1.37799    ,    1.41374    ,    1.41413    ,    1.44779    ,    1.3926    ,    1.42813    ,    1.47303    ,    1.47215    ,    1.44264    ,    1.44922    ,    1.4754    ,    1.46056    ,    1.4537    ,    1.51326    ,    1.53877    ,    1.54473    ,    1.58111    ,    1.63608    ,    1.71653    ,    1.87657    ,    1.98573    ,    1.82618    ,    1.68837    ,    1.62585    ,    1.63608    ,    1.56613    ,    1.49764    ,    1.41992    ,    1.36152    ,    1.31799    ,    1.28252    ,    1.2566    },
    {    1.38602    ,    1.38059    ,    1.37334    ,    1.37268    ,    1.37521    ,    1.37137    ,    1.36812    ,    1.33931    ,    1.30832    ,    1.28325    ,    1.27154    ,    1.26662    ,    1.25591    ,    1.23545    ,    1.20404    ,    1.17378    ,    1.16161    ,    1.21354    ,    1.36678    ,    1.45738    ,    1.45922    ,    1.45402    ,    1.41511    ,    1.44948    ,    1.53199    ,    1.55732    ,    1.57592    ,    1.61145    ,    1.67185    ,    1.72611    ,    1.71585    ,    1.75293    ,    1.82323    ,    1.8939    ,    2.02367    ,    2.18126    ,    2.17592    ,    1.94081    ,    1.82303    ,    1.75815    ,    1.71078    ,    1.6649    ,    1.59458    ,    1.45273    ,    1.1378    ,    0.839249    ,    0.627193    ,    0.440042    ,    0.29728    ,    0.201154    ,    0.157462    ,    0.15458    ,    0.150097    ,    0.151192    ,    0.169512    ,    0.196682    ,    0.236519    ,    0.305332    ,    0.375801    ,    0.437329    ,    0.480396    ,    0.628924    ,    0.733727    ,    0.834229    ,    0.853479    ,    0.876047    ,    0.929833    ,    1.03858    ,    1.1295    ,    1.16585    ,    1.22915    ,    1.2977    ,    1.36715    ,    1.41878    ,    1.45123    ,    1.47441    ,    1.52578    ,    1.55353    ,    1.58652    ,    1.59844    ,    1.61202    ,    1.63715    ,    1.65458    ,    1.66136    ,    1.63864    ,    1.63688    ,    1.73761    ,    1.77124    ,    1.78154    ,    1.78174    ,    1.76937    ,    1.78939    ,    1.78729    ,    1.73251    ,    1.65534    ,    1.56705    ,    1.54413    ,    1.49709    ,    1.46422    ,    1.46615    ,    1.4945    ,    1.55759    ,    1.56436    ,    1.5465    ,    1.49641    ,    1.48229    ,    1.48554    ,    1.50669    ,    1.51437    ,    1.54265    ,    1.54066    ,    1.55162    ,    1.61697    ,    1.8709    ,    2.13158    ,    2.21809    ,    2.14618    ,    1.97434    ,    1.83345    ,    1.76682    ,    1.783    ,    1.75823    ,    1.70535    ,    1.59764    ,    1.5162    ,    1.46649    ,    1.43154    ,    1.40698    },
    {    1.62555    ,    1.63069    ,    1.64048    ,    1.64575    ,    1.63209    ,    1.6131    ,    1.59863    ,    1.59793    ,    1.57548    ,    1.47902    ,    1.39284    ,    1.33325    ,    1.29389    ,    1.26709    ,    1.24625    ,    1.25723    ,    1.34016    ,    1.52091    ,    1.55585    ,    1.52089    ,    1.51158    ,    1.51193    ,    1.546    ,    1.55727    ,    1.60243    ,    1.6228    ,    1.66128    ,    1.70422    ,    1.73768    ,    1.80284    ,    1.83337    ,    1.93466    ,    2.03756    ,    2.1521    ,    2.22808    ,    2.28551    ,    2.28423    ,    2.07359    ,    1.9541    ,    1.88359    ,    1.81782    ,    1.76846    ,    1.60009    ,    1.2323    ,    0.865128    ,    0.674405    ,    0.563416    ,    0.475345    ,    0.427876    ,    0.354181    ,    0.263998    ,    0.206222    ,    0.176516    ,    0.17659    ,    0.211527    ,    0.255639    ,    0.291237    ,    0.357355    ,    0.428442    ,    0.482639    ,    0.495586    ,    0.543956    ,    0.713279    ,    0.809796    ,    0.829717    ,    0.895354    ,    0.989002    ,    1.10722    ,    1.19485    ,    1.29368    ,    1.32332    ,    1.36022    ,    1.42545    ,    1.49684    ,    1.54084    ,    1.56167    ,    1.59214    ,    1.63955    ,    1.68794    ,    1.69368    ,    1.68087    ,    1.69808    ,    1.70944    ,    1.70525    ,    1.68517    ,    1.68154    ,    1.72992    ,    1.76792    ,    1.78612    ,    1.78173    ,    1.76087    ,    1.73257    ,    1.73427    ,    1.71642    ,    1.69918    ,    1.66001    ,    1.66476    ,    1.66177    ,    1.64495    ,    1.62725    ,    1.64606    ,    1.63453    ,    1.61975    ,    1.59502    ,    1.58859    ,    1.57155    ,    1.56459    ,    1.5989    ,    1.59831    ,    1.62077    ,    1.62769    ,    1.63028    ,    1.64389    ,    1.78507    ,    2.07229    ,    2.1794    ,    2.15954    ,    2.10638    ,    2.05352    ,    1.96223    ,    1.91955    ,    1.91675    ,    1.94531    ,    1.83953    ,    1.75991    ,    1.71087    ,    1.67322    ,    1.64728    },
    {    2.02237    ,    1.99885    ,    1.96975    ,    1.92887    ,    1.857    ,    1.8097    ,    1.83698    ,    1.88654    ,    1.87254    ,    1.84242    ,    1.71575    ,    1.5917    ,    1.51206    ,    1.47957    ,    1.44962    ,    1.59013    ,    1.65234    ,    1.60294    ,    1.58478    ,    1.58203    ,    1.61844    ,    1.64679    ,    1.6991    ,    1.72554    ,    1.75849    ,    1.75549    ,    1.79232    ,    1.82357    ,    1.87192    ,    1.91207    ,    1.98155    ,    2.04369    ,    2.11196    ,    2.16792    ,    2.21616    ,    2.27194    ,    2.26552    ,    2.14389    ,    2.07052    ,    1.98156    ,    1.84993    ,    1.75824    ,    1.5131    ,    1.10383    ,    0.793413    ,    0.693461    ,    0.673874    ,    0.735284    ,    0.834145    ,    0.831625    ,    0.57748    ,    0.420051    ,    0.328814    ,    0.312784    ,    0.348243    ,    0.386227    ,    0.41778    ,    0.456239    ,    0.505579    ,    0.550824    ,    0.570343    ,    0.600587    ,    0.676464    ,    0.757681    ,    0.839577    ,    0.937333    ,    1.13823    ,    1.32687    ,    1.4159    ,    1.47178    ,    1.49241    ,    1.52571    ,    1.55457    ,    1.61178    ,    1.65355    ,    1.67643    ,    1.68552    ,    1.69746    ,    1.70716    ,    1.71077    ,    1.70764    ,    1.71336    ,    1.70869    ,    1.71675    ,    1.7434    ,    1.75383    ,    1.77193    ,    1.79949    ,    1.80222    ,    1.76874    ,    1.79074    ,    1.80057    ,    1.79962    ,    1.79355    ,    1.78213    ,    1.77031    ,    1.77229    ,    1.77395    ,    1.79668    ,    1.77145    ,    1.76392    ,    1.74781    ,    1.73925    ,    1.7205    ,    1.68773    ,    1.67862    ,    1.66986    ,    1.68159    ,    1.70765    ,    1.70282    ,    1.71127    ,    1.7409    ,    1.75901    ,    1.78933    ,    1.74037    ,    1.8003    ,    1.84331    ,    1.86968    ,    1.91061    ,    1.98152    ,    1.89926    ,    1.93186    ,    2.05678    ,    2.03655    ,    2.02495    ,    2.06781    ,    2.06945    ,    2.05021    },
    {    2.35577    ,    2.27299    ,    2.19378    ,    2.11171    ,    2.03881    ,    1.98795    ,    2.0074    ,    1.99555    ,    1.97912    ,    1.9349    ,    1.89348    ,    1.85368    ,    1.82089    ,    1.78225    ,    1.77717    ,    1.7233    ,    1.6957    ,    1.66906    ,    1.69636    ,    1.72207    ,    1.7555    ,    1.82123    ,    1.86039    ,    1.87193    ,    1.89922    ,    1.92006    ,    1.95092    ,    1.98408    ,    2.01456    ,    2.0505    ,    2.11235    ,    2.15462    ,    2.13528    ,    2.14903    ,    2.20187    ,    2.26149    ,    2.28549    ,    2.26902    ,    2.16081    ,    2.03262    ,    1.92302    ,    1.72455    ,    1.52857    ,    1.19582    ,    0.917651    ,    0.840841    ,    0.951452    ,    1.03867    ,    1.12558    ,    1.20968    ,    1.10586    ,    0.795671    ,    0.622598    ,    0.535257    ,    0.543041    ,    0.595226    ,    0.669449    ,    0.735809    ,    0.740123    ,    0.745878    ,    0.746776    ,    0.794367    ,    0.873586    ,    0.928378    ,    0.979606    ,    1.03085    ,    1.11976    ,    1.29417    ,    1.42722    ,    1.50903    ,    1.58784    ,    1.68743    ,    1.72044    ,    1.74086    ,    1.76716    ,    1.79296    ,    1.81659    ,    1.83546    ,    1.82867    ,    1.83307    ,    1.83065    ,    1.84331    ,    1.86346    ,    1.87017    ,    1.88425    ,    1.90234    ,    1.93335    ,    1.9517    ,    1.9468    ,    1.98207    ,    1.99164    ,    1.97706    ,    1.95961    ,    1.93375    ,    1.92638    ,    1.89593    ,    1.8817    ,    1.86581    ,    1.86303    ,    1.87419    ,    1.87881    ,    1.85937    ,    1.85741    ,    1.84868    ,    1.84856    ,    1.84391    ,    1.8341    ,    1.81079    ,    1.81049    ,    1.7853    ,    1.76633    ,    1.7716    ,    1.76268    ,    1.75093    ,    1.75166    ,    1.77363    ,    1.81773    ,    1.84868    ,    1.90347    ,    1.96592    ,    1.96395    ,    2.00795    ,    2.05514    ,    2.09942    ,    2.14016    ,    2.19246    ,    2.2829    ,    2.34035    },
    {    2.42517    ,    2.4381    ,    2.49794    ,    2.46229    ,    2.39944    ,    2.29923    ,    2.20589    ,    2.13364    ,    2.07477    ,    2.02415    ,    1.99296    ,    1.96093    ,    1.93813    ,    1.93501    ,    1.92185    ,    1.92545    ,    1.91699    ,    1.89457    ,    1.90357    ,    1.91569    ,    1.95859    ,    1.9966    ,    2.00782    ,    2.02971    ,    2.04644    ,    2.08022    ,    2.09996    ,    2.11844    ,    2.1566    ,    2.20341    ,    2.23671    ,    2.23508    ,    2.23342    ,    2.25251    ,    2.29349    ,    2.27453    ,    2.25534    ,    2.23589    ,    2.14265    ,    2.06129    ,    1.99467    ,    1.87556    ,    1.78172    ,    1.57827    ,    1.29202    ,    1.18326    ,    1.21837    ,    1.17156    ,    1.29129    ,    1.31088    ,    1.48324    ,    1.51245    ,    1.45788    ,    1.27785    ,    1.17578    ,    1.17466    ,    1.34992    ,    1.46221    ,    1.42205    ,    1.34555    ,    1.23992    ,    1.21602    ,    1.23748    ,    1.25186    ,    1.27738    ,    1.28301    ,    1.25261    ,    1.25198    ,    1.31712    ,    1.52977    ,    1.62819    ,    1.71521    ,    1.83553    ,    1.85456    ,    1.87672    ,    1.9314    ,    1.95667    ,    2.01833    ,    2.03759    ,    2.01818    ,    2.01924    ,    2.01616    ,    2.01926    ,    2.03911    ,    2.04099    ,    2.07365    ,    2.09674    ,    2.13821    ,    2.16444    ,    2.18609    ,    2.16037    ,    2.15381    ,    2.13012    ,    2.10103    ,    2.05231    ,    2.01259    ,    1.98617    ,    1.98236    ,    1.96584    ,    1.95682    ,    1.96121    ,    1.94411    ,    1.93911    ,    1.93932    ,    1.95571    ,    1.96204    ,    1.95947    ,    1.93547    ,    1.90245    ,    1.85072    ,    1.81867    ,    1.80555    ,    1.79725    ,    1.79275    ,    1.75916    ,    1.7725    ,    1.79483    ,    1.86417    ,    1.95402    ,    2.03254    ,    2.06765    ,    2.08526    ,    2.07605    ,    2.1173    ,    2.17118    ,    2.2664    ,    2.34362    ,    2.38792    },
    {    2.56929    ,    2.68882    ,    2.76263    ,    2.74318    ,    2.69599    ,    2.58395    ,    2.44986    ,    2.3547    ,    2.28461    ,    2.23755    ,    2.18971    ,    2.161    ,    2.15092    ,    2.1788    ,    2.24324    ,    2.30996    ,    2.28346    ,    2.24955    ,    2.21188    ,    2.21348    ,    2.21406    ,    2.23887    ,    2.24639    ,    2.24773    ,    2.25783    ,    2.2624    ,    2.30089    ,    2.33188    ,    2.34164    ,    2.35676    ,    2.3461    ,    2.35203    ,    2.3667    ,    2.38149    ,    2.34241    ,    2.30382    ,    2.23171    ,    2.17899    ,    2.11364    ,    2.05353    ,    2.02027    ,    2.03597    ,    1.98038    ,    1.85915    ,    1.6764    ,    1.51089    ,    1.30973    ,    1.18008    ,    1.22505    ,    1.25383    ,    1.25559    ,    1.25207    ,    1.29233    ,    1.44955    ,    1.59616    ,    1.81325    ,    1.99356    ,    2.02968    ,    1.95518    ,    1.84391    ,    1.7309    ,    1.65808    ,    1.63367    ,    1.61755    ,    1.60428    ,    1.58513    ,    1.54359    ,    1.48462    ,    1.42105    ,    1.43489    ,    1.62679    ,    1.76448    ,    1.85236    ,    1.92213    ,    1.98991    ,    2.03766    ,    2.10703    ,    2.18496    ,    2.25076    ,    2.26766    ,    2.33974    ,    2.3401    ,    2.38948    ,    2.40185    ,    2.40247    ,    2.40971    ,    2.39206    ,    2.3986    ,    2.40124    ,    2.3776    ,    2.3742    ,    2.34992    ,    2.31382    ,    2.2593    ,    2.20893    ,    2.14652    ,    2.07817    ,    2.02681    ,    2.00453    ,    2.00495    ,    2.03137    ,    2.05811    ,    2.06188    ,    2.06101    ,    2.0798    ,    2.08713    ,    2.08297    ,    2.04862    ,    1.993    ,    1.95327    ,    1.91489    ,    1.91581    ,    1.91947    ,    1.90665    ,    1.90744    ,    1.94005    ,    1.97362    ,    2.02955    ,    2.0749    ,    2.13544    ,    2.14422    ,    2.15742    ,    2.16419    ,    2.20721    ,    2.24766    ,    2.32179    ,    2.38685    ,    2.45264    },
    {    2.82417    ,    2.86103    ,    2.89671    ,    2.89694    ,    2.87903    ,    2.84573    ,    2.80686    ,    2.78796    ,    2.77596    ,    2.73893    ,    2.71781    ,    2.69091    ,    2.66245    ,    2.6515    ,    2.65777    ,    2.64865    ,    2.62901    ,    2.58931    ,    2.54741    ,    2.5116    ,    2.48361    ,    2.45996    ,    2.43898    ,    2.4113    ,    2.4103    ,    2.41939    ,    2.4403    ,    2.49913    ,    2.55407    ,    2.51388    ,    2.48686    ,    2.53561    ,    2.55189    ,    2.43174    ,    2.33996    ,    2.25355    ,    2.18455    ,    2.14284    ,    2.14123    ,    2.12878    ,    2.1429    ,    2.13414    ,    2.0838    ,    1.97775    ,    1.77955    ,    1.50578    ,    1.27196    ,    1.21621    ,    1.2701    ,    1.32705    ,    1.34952    ,    1.34196    ,    1.32297    ,    1.29689    ,    1.33085    ,    1.5052    ,    1.89316    ,    2.25529    ,    2.3439    ,    2.28435    ,    2.1592    ,    1.99818    ,    1.88592    ,    1.81074    ,    1.75371    ,    1.72094    ,    1.70057    ,    1.66163    ,    1.59606    ,    1.56504    ,    1.61263    ,    1.72474    ,    1.86923    ,    2.05625    ,    2.15341    ,    2.20442    ,    2.28238    ,    2.36396    ,    2.45516    ,    2.55185    ,    2.64983    ,    2.72008    ,    2.74151    ,    2.78783    ,    2.83903    ,    2.89148    ,    2.87783    ,    2.79687    ,    2.67755    ,    2.57845    ,    2.51817    ,    2.47274    ,    2.43709    ,    2.40197    ,    2.35995    ,    2.31016    ,    2.2588    ,    2.20028    ,    2.17161    ,    2.16404    ,    2.17105    ,    2.16797    ,    2.17232    ,    2.17662    ,    2.18007    ,    2.18898    ,    2.18013    ,    2.15459    ,    2.1173    ,    2.09326    ,    2.07583    ,    2.08427    ,    2.07147    ,    2.09174    ,    2.12382    ,    2.17544    ,    2.22702    ,    2.28084    ,    2.33984    ,    2.38845    ,    2.43194    ,    2.44882    ,    2.48281    ,    2.52359    ,    2.57222    ,    2.63125    ,    2.68924    ,    2.75782    },
    {    2.97257    ,    2.9956    ,    3.01015    ,    3.01655    ,    3.01534    ,    3.00427    ,    2.98552    ,    2.96038    ,    2.9304    ,    2.89653    ,    2.86068    ,    2.82463    ,    2.78219    ,    2.73818    ,    2.69794    ,    2.66341    ,    2.63352    ,    2.60664    ,    2.58309    ,    2.54949    ,    2.53253    ,    2.54327    ,    2.5493    ,    2.58216    ,    2.63353    ,    2.65927    ,    2.69151    ,    2.73147    ,    2.71606    ,    2.67077    ,    2.67608    ,    2.60776    ,    2.58017    ,    2.49069    ,    2.40957    ,    2.32564    ,    2.24054    ,    2.2044    ,    2.21813    ,    2.22691    ,    2.22026    ,    2.1786    ,    2.09571    ,    1.95205    ,    1.63821    ,    1.35956    ,    1.29097    ,    1.28244    ,    1.31995    ,    1.35757    ,    1.37651    ,    1.37136    ,    1.37548    ,    1.35803    ,    1.38162    ,    1.50619    ,    1.69265    ,    2.08516    ,    2.3586    ,    2.47425    ,    2.48882    ,    2.37258    ,    2.16068    ,    1.96151    ,    1.84328    ,    1.80788    ,    1.80669    ,    1.84066    ,    1.89706    ,    2.02401    ,    2.17155    ,    2.31435    ,    2.41365    ,    2.48779    ,    2.54032    ,    2.59796    ,    2.67825    ,    2.77268    ,    2.84156    ,    2.89645    ,    2.93615    ,    2.94875    ,    2.92937    ,    2.93818    ,    2.92191    ,    3.02478    ,    3.02886    ,    2.99507    ,    2.91679    ,    2.83015    ,    2.73797    ,    2.66862    ,    2.61929    ,    2.56587    ,    2.52346    ,    2.47298    ,    2.43626    ,    2.41513    ,    2.41195    ,    2.39299    ,    2.37246    ,    2.36467    ,    2.35066    ,    2.33631    ,    2.31534    ,    2.29741    ,    2.26845    ,    2.27065    ,    2.27323    ,    2.27957    ,    2.25674    ,    2.26171    ,    2.27716    ,    2.31335    ,    2.39081    ,    2.42444    ,    2.46896    ,    2.5274    ,    2.58162    ,    2.6296    ,    2.66499    ,    2.69356    ,    2.72571    ,    2.76259    ,    2.80273    ,    2.84598    ,    2.8889    ,    2.93242    },
    {    2.97572    ,    2.97191    ,    2.96648    ,    2.96408    ,    2.96237    ,    2.95603    ,    2.94309    ,    2.92308    ,    2.89556    ,    2.86689    ,    2.83517    ,    2.80605    ,    2.77608    ,    2.74182    ,    2.7089    ,    2.67758    ,    2.65054    ,    2.62957    ,    2.60783    ,    2.58295    ,    2.55135    ,    2.53268    ,    2.57339    ,    2.65069    ,    2.69231    ,    2.70094    ,    2.75492    ,    2.78451    ,    2.74916    ,    2.70743    ,    2.60943    ,    2.57054    ,    2.49608    ,    2.42161    ,    2.2684    ,    2.1721    ,    2.12071    ,    2.08594    ,    2.07276    ,    2.02004    ,    1.89108    ,    1.76178    ,    1.67046    ,    1.54811    ,    1.43328    ,    1.40173    ,    1.38928    ,    1.39414    ,    1.40482    ,    1.39507    ,    1.39292    ,    1.40476    ,    1.40336    ,    1.42259    ,    1.46088    ,    1.54418    ,    1.69219    ,    2.12194    ,    2.37994    ,    2.4886    ,    2.49694    ,    2.44574    ,    2.35845    ,    2.25227    ,    2.14375    ,    2.06475    ,    2.05559    ,    2.15756    ,    2.35342    ,    2.60473    ,    2.85996    ,    3.07726    ,    3.11568    ,    3.23275    ,    3.323    ,    3.39567    ,    3.42409    ,    3.41699    ,    3.41409    ,    3.42771    ,    3.43222    ,    3.42444    ,    3.4043    ,    3.37608    ,    3.35889    ,    3.33529    ,    3.33414    ,    3.34045    ,    3.28849    ,    3.24028    ,    3.14457    ,    3.03244    ,    2.92961    ,    2.83179    ,    2.74711    ,    2.67588    ,    2.61726    ,    2.58455    ,    2.56221    ,    2.54057    ,    2.53118    ,    2.52292    ,    2.49545    ,    2.50533    ,    2.47054    ,    2.43731    ,    2.44695    ,    2.44757    ,    2.44794    ,    2.45086    ,    2.46147    ,    2.48126    ,    2.50334    ,    2.53743    ,    2.56665    ,    2.61552    ,    2.65719    ,    2.69786    ,    2.73317    ,    2.7633    ,    2.78756    ,    2.8141    ,    2.84629    ,    2.88064    ,    2.91236    ,    2.93835    ,    2.96046    ,    2.97406    },
    {    2.93414    ,    2.92572    ,    2.9174    ,    2.9069    ,    2.89468    ,    2.88038    ,    2.86325    ,    2.8448    ,    2.8253    ,    2.80619    ,    2.78573    ,    2.76546    ,    2.74151    ,    2.71954    ,    2.69684    ,    2.6773    ,    2.65848    ,    2.64148    ,    2.62772    ,    2.61556    ,    2.60306    ,    2.59068    ,    2.58311    ,    2.58082    ,    2.58022    ,    2.58531    ,    2.57856    ,    2.56381    ,    2.5458    ,    2.50579    ,    2.44794    ,    2.34035    ,    2.28375    ,    2.22742    ,    2.16458    ,    2.09939    ,    2.07687    ,    2.04962    ,    2.01712    ,    1.99185    ,    1.91532    ,    1.78948    ,    1.673    ,    1.5917    ,    1.54152    ,    1.51146    ,    1.49333    ,    1.48839    ,    1.48615    ,    1.48599    ,    1.48433    ,    1.49038    ,    1.52417    ,    1.58155    ,    1.6479    ,    1.74223    ,    1.87323    ,    2.11218    ,    2.32782    ,    2.43874    ,    2.48267    ,    2.48021    ,    2.44843    ,    2.4301    ,    2.45486    ,    2.47196    ,    2.47378    ,    2.49266    ,    2.57738    ,    2.72523    ,    2.85141    ,    2.96093    ,    3.08955    ,    3.26084    ,    3.39167    ,    3.44111    ,    3.47256    ,    3.49351    ,    3.5123    ,    3.53498    ,    3.55283    ,    3.5428    ,    3.52125    ,    3.49619    ,    3.45196    ,    3.42357    ,    3.40922    ,    3.41295    ,    3.42161    ,    3.4153    ,    3.38161    ,    3.296    ,    3.20039    ,    3.11661    ,    3.04288    ,    2.96205    ,    2.86511    ,    2.80344    ,    2.74392    ,    2.7431    ,    2.75478    ,    2.74956    ,    2.73383    ,    2.71816    ,    2.69731    ,    2.68053    ,    2.66955    ,    2.661    ,    2.65688    ,    2.6606    ,    2.66637    ,    2.67705    ,    2.68817    ,    2.70543    ,    2.72544    ,    2.74797    ,    2.7688    ,    2.78792    ,    2.81046    ,    2.83133    ,    2.8518    ,    2.87279    ,    2.89215    ,    2.90714    ,    2.91896    ,    2.92789    ,    2.93536    ,    2.93781    },
    {    2.85148    ,    2.84273    ,    2.83391    ,    2.82501    ,    2.81583    ,    2.80439    ,    2.79142    ,    2.77668    ,    2.76091    ,    2.7462    ,    2.73155    ,    2.71664    ,    2.69997    ,    2.6855    ,    2.67288    ,    2.66014    ,    2.64899    ,    2.63871    ,    2.63099    ,    2.625    ,    2.61792    ,    2.61099    ,    2.60372    ,    2.59592    ,    2.58764    ,    2.57786    ,    2.56546    ,    2.54645    ,    2.52889    ,    2.50622    ,    2.4836    ,    2.46225    ,    2.42398    ,    2.3616    ,    2.32145    ,    2.30973    ,    2.26585    ,    2.24781    ,    2.21936    ,    2.18744    ,    2.18712    ,    2.18516    ,    2.19945    ,    2.17468    ,    2.14703    ,    2.14146    ,    2.07444    ,    2.03955    ,    1.9967    ,    1.95777    ,    1.94323    ,    1.95511    ,    1.9912    ,    2.04905    ,    2.10656    ,    2.22223    ,    2.37048    ,    2.467    ,    2.51021    ,    2.54342    ,    2.57434    ,    2.59242    ,    2.61382    ,    2.64661    ,    2.677    ,    2.7149    ,    2.75874    ,    2.81344    ,    2.87303    ,    2.93618    ,    2.99002    ,    3.03752    ,    3.07558    ,    3.11407    ,    3.14188    ,    3.15984    ,    3.17467    ,    3.18759    ,    3.19624    ,    3.20218    ,    3.20414    ,    3.19231    ,    3.17862    ,    3.16636    ,    3.15857    ,    3.16563    ,    3.16791    ,    3.17649    ,    3.18526    ,    3.18721    ,    3.18826    ,    3.17398    ,    3.14878    ,    3.12373    ,    3.09124    ,    3.06109    ,    3.02285    ,    2.98916    ,    2.96493    ,    2.93997    ,    2.91857    ,    2.89809    ,    2.87722    ,    2.85909    ,    2.84083    ,    2.82989    ,    2.81918    ,    2.81225    ,    2.80788    ,    2.80717    ,    2.80573    ,    2.80508    ,    2.80714    ,    2.81119    ,    2.81739    ,    2.82368    ,    2.83023    ,    2.8382    ,    2.84556    ,    2.85124    ,    2.85567    ,    2.86079    ,    2.86417    ,    2.86553    ,    2.86729    ,    2.86666    ,    2.86488    ,    2.85959    },
    {    2.7942    ,    2.78887    ,    2.78253    ,    2.77659    ,    2.76979    ,    2.76259    ,    2.75641    ,    2.75013    ,    2.7434    ,    2.73546    ,    2.72694    ,    2.71872    ,    2.71284    ,    2.70522    ,    2.69578    ,    2.68879    ,    2.6824    ,    2.67586    ,    2.66821    ,    2.66044    ,    2.65488    ,    2.64916    ,    2.64171    ,    2.63523    ,    2.62742    ,    2.61797    ,    2.60873    ,    2.60128    ,    2.59367    ,    2.56784    ,    2.563    ,    2.56594    ,    2.56939    ,    2.57088    ,    2.57441    ,    2.57426    ,    2.57748    ,    2.58308    ,    2.59058    ,    2.59706    ,    2.60631    ,    2.60832    ,    2.60745    ,    2.60214    ,    2.59717    ,    2.59575    ,    2.5937    ,    2.58689    ,    2.58032    ,    2.578    ,    2.57501    ,    2.57793    ,    2.58753    ,    2.59887    ,    2.61493    ,    2.63728    ,    2.65721    ,    2.67628    ,    2.68839    ,    2.70359    ,    2.72287    ,    2.73851    ,    2.75292    ,    2.76942    ,    2.78631    ,    2.80312    ,    2.8226    ,    2.84115    ,    2.8571    ,    2.872    ,    2.88672    ,    2.90025    ,    2.91142    ,    2.91929    ,    2.92839    ,    2.93639    ,    2.94178    ,    2.94627    ,    2.9498    ,    2.95178    ,    2.95273    ,    2.95312    ,    2.95245    ,    2.95106    ,    2.94984    ,    2.94935    ,    2.94905    ,    2.94835    ,    2.94848    ,    2.94857    ,    2.94847    ,    2.94717    ,    2.94347    ,    2.94494    ,    2.9398    ,    2.93354    ,    2.92579    ,    2.91663    ,    2.908    ,    2.89869    ,    2.89136    ,    2.88573    ,    2.87863    ,    2.873    ,    2.86781    ,    2.86326    ,    2.85827    ,    2.85289    ,    2.8468    ,    2.84191    ,    2.8394    ,    2.83506    ,    2.82909    ,    2.82697    ,    2.82614    ,    2.82396    ,    2.82179    ,    2.82136    ,    2.81995    ,    2.81803    ,    2.81613    ,    2.81325    ,    2.81026    ,    2.8091    ,    2.80679    ,    2.80384    ,    2.80008    ,    2.79797    },
    {    2.77767    ,    2.77662    ,    2.77547    ,    2.77373    ,    2.77187    ,    2.76964    ,    2.76714    ,    2.76494    ,    2.76256    ,    2.76045    ,    2.75816    ,    2.75617    ,    2.75495    ,    2.75338    ,    2.75044    ,    2.74714    ,    2.74309    ,    2.73905    ,    2.73656    ,    2.73389    ,    2.73155    ,    2.72891    ,    2.72665    ,    2.72465    ,    2.72248    ,    2.72105    ,    2.71999    ,    2.71986    ,    2.71881    ,    2.71026    ,    2.71044    ,    2.71162    ,    2.71261    ,    2.71605    ,    2.71826    ,    2.71943    ,    2.72179    ,    2.72551    ,    2.72807    ,    2.73195    ,    2.73357    ,    2.73552    ,    2.73695    ,    2.73843    ,    2.74197    ,    2.74426    ,    2.74549    ,    2.74804    ,    2.74946    ,    2.75111    ,    2.75281    ,    2.75555    ,    2.7606    ,    2.76596    ,    2.76912    ,    2.77274    ,    2.7775    ,    2.78152    ,    2.78384    ,    2.78741    ,    2.79296    ,    2.80073    ,    2.80496    ,    2.80917    ,    2.81269    ,    2.81513    ,    2.81895    ,    2.82391    ,    2.82941    ,    2.83253    ,    2.83666    ,    2.84031    ,    2.84435    ,    2.84688    ,    2.84876    ,    2.84982    ,    2.85133    ,    2.85344    ,    2.85539    ,    2.85607    ,    2.85517    ,    2.85438    ,    2.85444    ,    2.85395    ,    2.85363    ,    2.85436    ,    2.85426    ,    2.85496    ,    2.85464    ,    2.85403    ,    2.85348    ,    2.85255    ,    2.85112    ,    2.85131    ,    2.84992    ,    2.84814    ,    2.84669    ,    2.84452    ,    2.84291    ,    2.84113    ,    2.83865    ,    2.8362    ,    2.83297    ,    2.83088    ,    2.82895    ,    2.82656    ,    2.82455    ,    2.82198    ,    2.81899    ,    2.81683    ,    2.81477    ,    2.81172    ,    2.80732    ,    2.8045    ,    2.80151    ,    2.79999    ,    2.79805    ,    2.79704    ,    2.79558    ,    2.79381    ,    2.79287    ,    2.79145    ,    2.7902    ,    2.78905    ,    2.78845    ,    2.7841    ,    2.78101    ,    2.77928    }
};

double Stdev_LGRTC_RCP45[64][128] = {
    {    0.339874    ,    0.340371    ,    0.34033    ,    0.342509    ,    0.346735    ,    0.345728    ,    0.344676    ,    0.343683    ,    0.347153    ,    0.349919    ,    0.350445    ,    0.346539    ,    0.3481    ,    0.346363    ,    0.345647    ,    0.343359    ,    0.343245    ,    0.345412    ,    0.353197    ,    0.356612    ,    0.353924    ,    0.350555    ,    0.352    ,    0.349571    ,    0.346518    ,    0.347703    ,    0.34711    ,    0.3493    ,    0.344193    ,    0.334314    ,    0.329643    ,    0.329479    ,    0.326493    ,    0.324845    ,    0.323235    ,    0.318427    ,    0.320045    ,    0.319216    ,    0.322123    ,    0.324892    ,    0.325468    ,    0.327207    ,    0.326889    ,    0.327584    ,    0.322336    ,    0.322881    ,    0.326603    ,    0.328969    ,    0.327888    ,    0.333545    ,    0.33816    ,    0.341741    ,    0.342181    ,    0.343776    ,    0.345227    ,    0.345769    ,    0.350668    ,    0.34658    ,    0.350825    ,    0.352663    ,    0.359009    ,    0.362556    ,    0.369271    ,    0.375862    ,    0.375087    ,    0.383838    ,    0.377721    ,    0.378344    ,    0.384421    ,    0.380943    ,    0.39649    ,    0.389122    ,    0.385827    ,    0.387411    ,    0.394061    ,    0.402861    ,    0.385672    ,    0.384127    ,    0.400089    ,    0.404814    ,    0.399536    ,    0.403316    ,    0.407249    ,    0.408602    ,    0.398919    ,    0.388095    ,    0.384518    ,    0.385038    ,    0.394635    ,    0.401617    ,    0.399796    ,    0.409561    ,    0.404208    ,    0.419136    ,    0.395218    ,    0.416639    ,    0.401577    ,    0.39742    ,    0.393059    ,    0.403378    ,    0.395019    ,    0.407338    ,    0.415841    ,    0.424202    ,    0.395394    ,    0.389708    ,    0.379684    ,    0.370695    ,    0.381194    ,    0.369174    ,    0.354518    ,    0.341908    ,    0.342592    ,    0.345437    ,    0.351113    ,    0.328081    ,    0.333822    ,    0.348517    ,    0.347317    ,    0.34494    ,    0.342932    ,    0.337359    ,    0.333673    ,    0.339618    ,    0.345604    ,    0.334266    ,    0.338837    ,    0.341003    },
    {    0.299374    ,    0.297371    ,    0.298698    ,    0.304636    ,    0.314371    ,    0.322158    ,    0.329259    ,    0.339935    ,    0.354678    ,    0.368992    ,    0.380226    ,    0.390815    ,    0.397877    ,    0.404033    ,    0.409799    ,    0.416004    ,    0.412225    ,    0.410926    ,    0.411034    ,    0.414792    ,    0.414494    ,    0.418717    ,    0.42678    ,    0.433497    ,    0.436875    ,    0.443199    ,    0.446615    ,    0.435404    ,    0.422915    ,    0.400761    ,    0.374565    ,    0.347085    ,    0.331892    ,    0.335722    ,    0.318412    ,    0.317524    ,    0.320949    ,    0.317728    ,    0.322076    ,    0.317351    ,    0.314834    ,    0.309151    ,    0.310776    ,    0.313222    ,    0.299995    ,    0.295607    ,    0.294632    ,    0.29234    ,    0.295159    ,    0.296633    ,    0.292863    ,    0.290639    ,    0.293856    ,    0.295173    ,    0.299419    ,    0.309404    ,    0.314531    ,    0.320864    ,    0.330202    ,    0.334747    ,    0.342762    ,    0.350893    ,    0.360154    ,    0.370723    ,    0.383832    ,    0.394685    ,    0.407977    ,    0.403457    ,    0.389929    ,    0.376813    ,    0.356428    ,    0.353209    ,    0.358956    ,    0.349808    ,    0.339542    ,    0.337403    ,    0.334076    ,    0.332422    ,    0.356508    ,    0.350225    ,    0.382231    ,    0.364771    ,    0.354691    ,    0.359479    ,    0.374028    ,    0.346054    ,    0.353458    ,    0.357168    ,    0.365504    ,    0.342394    ,    0.342587    ,    0.34436    ,    0.354459    ,    0.363188    ,    0.352783    ,    0.346059    ,    0.357721    ,    0.355045    ,    0.361766    ,    0.376037    ,    0.382252    ,    0.419183    ,    0.376803    ,    0.375306    ,    0.374257    ,    0.353274    ,    0.345989    ,    0.33258    ,    0.321076    ,    0.311609    ,    0.298601    ,    0.288444    ,    0.27185    ,    0.263962    ,    0.260629    ,    0.258636    ,    0.258034    ,    0.25879    ,    0.260016    ,    0.264545    ,    0.267478    ,    0.274117    ,    0.276234    ,    0.287733    ,    0.283844    ,    0.282393    ,    0.286583    ,    0.291283    },
    {    0.417757    ,    0.435701    ,    0.465444    ,    0.49042    ,    0.514629    ,    0.551436    ,    0.575617    ,    0.603971    ,    0.61785    ,    0.630338    ,    0.642472    ,    0.640917    ,    0.636799    ,    0.637439    ,    0.636329    ,    0.62465    ,    0.613025    ,    0.598494    ,    0.585182    ,    0.571928    ,    0.556732    ,    0.551011    ,    0.546206    ,    0.547327    ,    0.551813    ,    0.562586    ,    0.568938    ,    0.574704    ,    0.573663    ,    0.553111    ,    0.52182    ,    0.474705    ,    0.428286    ,    0.378785    ,    0.336815    ,    0.304094    ,    0.2874    ,    0.294481    ,    0.305766    ,    0.308478    ,    0.319674    ,    0.330215    ,    0.353302    ,    0.356016    ,    0.362811    ,    0.353079    ,    0.338409    ,    0.329894    ,    0.318601    ,    0.317424    ,    0.31809    ,    0.315592    ,    0.314758    ,    0.319345    ,    0.317409    ,    0.32163    ,    0.322588    ,    0.323156    ,    0.328984    ,    0.343852    ,    0.360647    ,    0.37287    ,    0.379779    ,    0.385823    ,    0.39429    ,    0.404393    ,    0.416205    ,    0.430436    ,    0.434991    ,    0.465894    ,    0.462998    ,    0.430485    ,    0.406701    ,    0.394795    ,    0.391174    ,    0.386392    ,    0.400174    ,    0.423911    ,    0.400514    ,    0.396546    ,    0.382006    ,    0.385277    ,    0.401165    ,    0.353698    ,    0.353043    ,    0.34584    ,    0.376621    ,    0.354077    ,    0.335211    ,    0.344812    ,    0.335807    ,    0.347477    ,    0.386321    ,    0.337626    ,    0.359531    ,    0.407614    ,    0.343026    ,    0.356856    ,    0.318235    ,    0.336517    ,    0.342041    ,    0.340005    ,    0.335812    ,    0.330353    ,    0.334577    ,    0.316961    ,    0.312711    ,    0.307374    ,    0.29649    ,    0.283033    ,    0.270906    ,    0.259443    ,    0.245742    ,    0.240192    ,    0.238177    ,    0.240097    ,    0.24392    ,    0.254057    ,    0.26246    ,    0.275057    ,    0.287062    ,    0.299171    ,    0.313843    ,    0.336404    ,    0.36675    ,    0.378716    ,    0.393358    ,    0.405796    },
    {    0.54823    ,    0.579649    ,    0.612591    ,    0.657127    ,    0.680468    ,    0.689192    ,    0.697904    ,    0.726801    ,    0.742793    ,    0.760733    ,    0.779476    ,    0.772513    ,    0.741783    ,    0.714851    ,    0.690612    ,    0.667866    ,    0.643452    ,    0.628814    ,    0.622742    ,    0.628566    ,    0.637895    ,    0.645946    ,    0.649838    ,    0.657774    ,    0.650529    ,    0.636964    ,    0.621633    ,    0.607458    ,    0.588046    ,    0.559998    ,    0.533456    ,    0.503735    ,    0.468273    ,    0.430733    ,    0.391212    ,    0.3553    ,    0.343733    ,    0.353082    ,    0.37846    ,    0.387307    ,    0.382486    ,    0.38721    ,    0.395536    ,    0.399907    ,    0.403964    ,    0.400796    ,    0.398037    ,    0.384314    ,    0.389866    ,    0.412064    ,    0.435833    ,    0.451153    ,    0.421286    ,    0.413199    ,    0.41438    ,    0.390755    ,    0.367013    ,    0.353904    ,    0.347901    ,    0.343505    ,    0.34833    ,    0.363267    ,    0.399175    ,    0.425094    ,    0.436338    ,    0.43472    ,    0.424653    ,    0.468636    ,    0.446392    ,    0.495479    ,    0.547163    ,    0.552497    ,    0.511627    ,    0.446234    ,    0.459192    ,    0.455378    ,    0.485663    ,    0.452882    ,    0.442536    ,    0.422328    ,    0.409409    ,    0.404607    ,    0.362999    ,    0.356644    ,    0.338517    ,    0.324437    ,    0.322749    ,    0.325634    ,    0.296826    ,    0.281291    ,    0.302814    ,    0.281432    ,    0.318336    ,    0.330095    ,    0.335646    ,    0.338751    ,    0.335418    ,    0.386181    ,    0.485781    ,    0.374531    ,    0.342382    ,    0.325873    ,    0.315564    ,    0.315968    ,    0.291202    ,    0.287327    ,    0.327026    ,    0.338406    ,    0.341164    ,    0.343388    ,    0.316491    ,    0.301338    ,    0.291062    ,    0.284077    ,    0.280663    ,    0.281075    ,    0.286234    ,    0.28291    ,    0.278242    ,    0.279839    ,    0.28394    ,    0.303784    ,    0.326425    ,    0.35085    ,    0.393175    ,    0.434633    ,    0.462004    ,    0.507528    },
    {    0.662382    ,    0.718905    ,    0.76695    ,    0.800596    ,    0.840741    ,    0.880037    ,    0.906725    ,    0.918528    ,    0.911878    ,    0.934217    ,    0.947934    ,    0.836948    ,    0.788887    ,    0.72305    ,    0.676644    ,    0.65248    ,    0.648468    ,    0.648359    ,    0.645443    ,    0.645521    ,    0.642051    ,    0.634812    ,    0.622647    ,    0.606297    ,    0.585814    ,    0.571466    ,    0.558618    ,    0.546986    ,    0.537055    ,    0.529794    ,    0.5166    ,    0.500151    ,    0.487963    ,    0.463314    ,    0.436655    ,    0.418679    ,    0.403072    ,    0.397025    ,    0.394648    ,    0.389299    ,    0.378177    ,    0.363663    ,    0.383412    ,    0.385198    ,    0.393881    ,    0.40659    ,    0.409495    ,    0.44568    ,    0.490068    ,    0.5369    ,    0.566229    ,    0.591283    ,    0.586712    ,    0.519953    ,    0.445994    ,    0.393755    ,    0.350016    ,    0.335058    ,    0.334565    ,    0.339547    ,    0.358039    ,    0.386623    ,    0.39918    ,    0.423641    ,    0.475588    ,    0.490299    ,    0.488654    ,    0.532877    ,    0.474127    ,    0.459605    ,    0.45434    ,    0.465384    ,    0.413119    ,    0.422549    ,    0.411775    ,    0.34831    ,    0.443746    ,    0.423443    ,    0.393211    ,    0.466078    ,    0.373202    ,    0.307059    ,    0.305734    ,    0.310055    ,    0.294224    ,    0.288703    ,    0.281758    ,    0.279573    ,    0.288735    ,    0.301042    ,    0.308679    ,    0.321759    ,    0.3276    ,    0.337052    ,    0.341874    ,    0.348549    ,    0.344618    ,    0.374523    ,    0.422166    ,    0.401397    ,    0.336618    ,    0.394562    ,    0.329272    ,    0.318328    ,    0.350494    ,    0.353447    ,    0.35576    ,    0.342839    ,    0.365373    ,    0.350579    ,    0.356725    ,    0.353513    ,    0.354966    ,    0.352828    ,    0.352019    ,    0.35088    ,    0.349361    ,    0.351397    ,    0.346039    ,    0.335601    ,    0.340333    ,    0.348871    ,    0.484997    ,    0.452881    ,    0.469043    ,    0.489477    ,    0.529704    ,    0.583063    },
    {    0.801211    ,    0.888058    ,    0.972187    ,    1.02696    ,    1.07119    ,    1.10913    ,    1.12644    ,    1.14002    ,    1.15361    ,    1.1556    ,    1.14656    ,    1.1289    ,    1.10699    ,    1.08004    ,    1.03937    ,    0.995074    ,    0.941782    ,    0.937115    ,    0.877288    ,    0.817929    ,    0.800468    ,    0.804432    ,    0.796063    ,    0.778752    ,    0.765399    ,    0.741997    ,    0.717071    ,    0.700116    ,    0.623828    ,    0.600996    ,    0.589841    ,    0.556919    ,    0.525068    ,    0.524313    ,    0.524362    ,    0.507185    ,    0.474623    ,    0.44106    ,    0.410505    ,    0.391043    ,    0.384475    ,    0.370266    ,    0.373407    ,    0.403286    ,    0.42931    ,    0.463381    ,    0.504699    ,    0.545608    ,    0.585853    ,    0.635298    ,    0.686708    ,    0.725702    ,    0.744284    ,    0.749953    ,    0.746709    ,    0.735811    ,    0.722911    ,    0.68315    ,    0.551356    ,    0.393262    ,    0.356706    ,    0.357178    ,    0.384082    ,    0.407835    ,    0.417513    ,    0.411748    ,    0.409996    ,    0.405943    ,    0.401196    ,    0.385682    ,    0.369874    ,    0.352577    ,    0.34395    ,    0.349094    ,    0.351748    ,    0.339309    ,    0.334431    ,    0.328913    ,    0.324851    ,    0.314212    ,    0.306857    ,    0.30318    ,    0.318962    ,    0.308244    ,    0.306476    ,    0.304435    ,    0.291097    ,    0.280669    ,    0.287482    ,    0.279608    ,    0.299101    ,    0.32966    ,    0.33716    ,    0.35268    ,    0.346218    ,    0.35347    ,    0.364053    ,    0.365421    ,    0.377418    ,    0.372691    ,    0.337531    ,    0.342    ,    0.354751    ,    0.360616    ,    0.412908    ,    0.416029    ,    0.404379    ,    0.454972    ,    0.407904    ,    0.382634    ,    0.410194    ,    0.396736    ,    0.387508    ,    0.384666    ,    0.388311    ,    0.403338    ,    0.406807    ,    0.406076    ,    0.395814    ,    0.408371    ,    0.405839    ,    0.399057    ,    0.411499    ,    0.427962    ,    0.455025    ,    0.513033    ,    0.572709    ,    0.671086    },
    {    0.907551    ,    0.955597    ,    0.992283    ,    1.03846    ,    1.11529    ,    1.1905    ,    1.24525    ,    1.28621    ,    1.32311    ,    1.35804    ,    1.38117    ,    1.38521    ,    1.3714    ,    1.33963    ,    1.29632    ,    1.24731    ,    1.20323    ,    1.15984    ,    1.11811    ,    1.08749    ,    1.05747    ,    1.03186    ,    1.0068    ,    0.986383    ,    0.964375    ,    0.929619    ,    0.882559    ,    0.834692    ,    0.805747    ,    0.794477    ,    0.774418    ,    0.747439    ,    0.717112    ,    0.676286    ,    0.643623    ,    0.622112    ,    0.606513    ,    0.545294    ,    0.536063    ,    0.442051    ,    0.392869    ,    0.36665    ,    0.37679    ,    0.450079    ,    0.494573    ,    0.527106    ,    0.569396    ,    0.609807    ,    0.651795    ,    0.69885    ,    0.744512    ,    0.776282    ,    0.797277    ,    0.812229    ,    0.822613    ,    0.832044    ,    0.835869    ,    0.812751    ,    0.775058    ,    0.724972    ,    0.627741    ,    0.490567    ,    0.475869    ,    0.493152    ,    0.497618    ,    0.478387    ,    0.462768    ,    0.450526    ,    0.437846    ,    0.429824    ,    0.432519    ,    0.437754    ,    0.440734    ,    0.437162    ,    0.425086    ,    0.409575    ,    0.398457    ,    0.386379    ,    0.368458    ,    0.351848    ,    0.337403    ,    0.316794    ,    0.297941    ,    0.304866    ,    0.29261    ,    0.292431    ,    0.280531    ,    0.26903    ,    0.283592    ,    0.351872    ,    0.321966    ,    0.308901    ,    0.326382    ,    0.334853    ,    0.340612    ,    0.336932    ,    0.343134    ,    0.348551    ,    0.348594    ,    0.352152    ,    0.371883    ,    0.390905    ,    0.398398    ,    0.416786    ,    0.417146    ,    0.396913    ,    0.399361    ,    0.386914    ,    0.379729    ,    0.3742    ,    0.380726    ,    0.385993    ,    0.393408    ,    0.397901    ,    0.404418    ,    0.421947    ,    0.434698    ,    0.428513    ,    0.422089    ,    0.429362    ,    0.421318    ,    0.407783    ,    0.41912    ,    0.450389    ,    0.517724    ,    0.572395    ,    0.672475    ,    0.790157    },
    {    0.81254    ,    0.831857    ,    0.855371    ,    0.880482    ,    0.900971    ,    0.928797    ,    0.981517    ,    1.05986    ,    1.12581    ,    1.18176    ,    1.2313    ,    1.26956    ,    1.30039    ,    1.3125    ,    1.29906    ,    1.27734    ,    1.26013    ,    1.24327    ,    1.21449    ,    1.17131    ,    1.11931    ,    1.06632    ,    1.02336    ,    0.988627    ,    0.95906    ,    0.923396    ,    0.882048    ,    0.84631    ,    0.809758    ,    0.761843    ,    0.704101    ,    0.648351    ,    0.596174    ,    0.550368    ,    0.51108    ,    0.482266    ,    0.466278    ,    0.464633    ,    0.460985    ,    0.435625    ,    0.419058    ,    0.418219    ,    0.501743    ,    0.509423    ,    0.513003    ,    0.543322    ,    0.595886    ,    0.641782    ,    0.673152    ,    0.695443    ,    0.722632    ,    0.767552    ,    0.834063    ,    0.903762    ,    0.967219    ,    1.02643    ,    1.05605    ,    1.04724    ,    1.01533    ,    0.968597    ,    0.908183    ,    0.843591    ,    0.803239    ,    0.787445    ,    0.782314    ,    0.788552    ,    0.797034    ,    0.80503    ,    0.820017    ,    0.855309    ,    0.90469    ,    0.959825    ,    0.998581    ,    0.98081    ,    0.870028    ,    0.774422    ,    0.730099    ,    0.748777    ,    0.719392    ,    0.539257    ,    0.46928    ,    0.416539    ,    0.356461    ,    0.317221    ,    0.290696    ,    0.303807    ,    0.315905    ,    0.327333    ,    0.345063    ,    0.463379    ,    0.561234    ,    0.470734    ,    0.409234    ,    0.36333    ,    0.34998    ,    0.340959    ,    0.335663    ,    0.342748    ,    0.354596    ,    0.37097    ,    0.38352    ,    0.392037    ,    0.39607    ,    0.397809    ,    0.384854    ,    0.370415    ,    0.385692    ,    0.362245    ,    0.358679    ,    0.35736    ,    0.35763    ,    0.363971    ,    0.375382    ,    0.373115    ,    0.376472    ,    0.384196    ,    0.422948    ,    0.43234    ,    0.467825    ,    0.482591    ,    0.51786    ,    0.585013    ,    0.640464    ,    0.659364    ,    0.674744    ,    0.704119    ,    0.736401    ,    0.770429    },
    {    0.787737    ,    0.790845    ,    0.788308    ,    0.787032    ,    0.795969    ,    0.822685    ,    0.85846    ,    0.907009    ,    0.950501    ,    0.974223    ,    0.990524    ,    1.00342    ,    0.998693    ,    0.987319    ,    0.968556    ,    0.943306    ,    0.917357    ,    0.884393    ,    0.858003    ,    0.835119    ,    0.809667    ,    0.7824    ,    0.756327    ,    0.732195    ,    0.709054    ,    0.682842    ,    0.654026    ,    0.624246    ,    0.594835    ,    0.559916    ,    0.528375    ,    0.500085    ,    0.473285    ,    0.453805    ,    0.445261    ,    0.452199    ,    0.444982    ,    0.419166    ,    0.397021    ,    0.38555    ,    0.381483    ,    0.368137    ,    0.430624    ,    0.505521    ,    0.527335    ,    0.539417    ,    0.559635    ,    0.5813    ,    0.599506    ,    0.615075    ,    0.630901    ,    0.658753    ,    0.710578    ,    0.771168    ,    0.834111    ,    0.905151    ,    0.962894    ,    1.01585    ,    1.06564    ,    1.12583    ,    1.18289    ,    1.22434    ,    1.2238    ,    1.17409    ,    1.09128    ,    1.00146    ,    0.915728    ,    0.879442    ,    0.879323    ,    0.891338    ,    0.910561    ,    0.923904    ,    0.937136    ,    0.943159    ,    0.934231    ,    0.917678    ,    0.905918    ,    0.892527    ,    0.890607    ,    0.875496    ,    0.836253    ,    0.790021    ,    0.727542    ,    0.666936    ,    0.675119    ,    0.71612    ,    0.727137    ,    0.69643    ,    0.678194    ,    0.707755    ,    0.732472    ,    0.710164    ,    0.673245    ,    0.627484    ,    0.60054    ,    0.598466    ,    0.582227    ,    0.54168    ,    0.495328    ,    0.470436    ,    0.416775    ,    0.479307    ,    0.500372    ,    0.495061    ,    0.488444    ,    0.517901    ,    0.542918    ,    0.552806    ,    0.498036    ,    0.487133    ,    0.462982    ,    0.443616    ,    0.426362    ,    0.445352    ,    0.482499    ,    0.536401    ,    0.592806    ,    0.612081    ,    0.636927    ,    0.661741    ,    0.6931    ,    0.73893    ,    0.760105    ,    0.745464    ,    0.742834    ,    0.753193    ,    0.777255    ,    0.792453    },
    {    0.785016    ,    0.782414    ,    0.773423    ,    0.759816    ,    0.749921    ,    0.786562    ,    0.867527    ,    0.951899    ,    0.985305    ,    0.972287    ,    0.932813    ,    0.873555    ,    0.822274    ,    0.790176    ,    0.772556    ,    0.740398    ,    0.703385    ,    0.675084    ,    0.652026    ,    0.632374    ,    0.615688    ,    0.597191    ,    0.580483    ,    0.565307    ,    0.547102    ,    0.525835    ,    0.503707    ,    0.481722    ,    0.459723    ,    0.437641    ,    0.415841    ,    0.394774    ,    0.376713    ,    0.363855    ,    0.35248    ,    0.342083    ,    0.333111    ,    0.321973    ,    0.310784    ,    0.311946    ,    0.336745    ,    0.360737    ,    0.374936    ,    0.38612    ,    0.425363    ,    0.479885    ,    0.508619    ,    0.524856    ,    0.540632    ,    0.545568    ,    0.543792    ,    0.536186    ,    0.538519    ,    0.556517    ,    0.591479    ,    0.637336    ,    0.686814    ,    0.748937    ,    0.813353    ,    0.884465    ,    0.966186    ,    1.04873    ,    1.1047    ,    1.10841    ,    1.04942    ,    1.00138    ,    0.968842    ,    0.933844    ,    0.905507    ,    0.886723    ,    0.87543    ,    0.863641    ,    0.850846    ,    0.844688    ,    0.837965    ,    0.828333    ,    0.817372    ,    0.812358    ,    0.809243    ,    0.814231    ,    0.819175    ,    0.810919    ,    0.793136    ,    0.788177    ,    0.856697    ,    0.985831    ,    1.08431    ,    1.13522    ,    1.1704    ,    1.21469    ,    1.26626    ,    1.27602    ,    1.25739    ,    1.20709    ,    1.10192    ,    0.942366    ,    0.786106    ,    0.707612    ,    0.654035    ,    0.614196    ,    0.594963    ,    0.598137    ,    0.618753    ,    0.640406    ,    0.658158    ,    0.674264    ,    0.676139    ,    0.667084    ,    0.649331    ,    0.631255    ,    0.624651    ,    0.621188    ,    0.607403    ,    0.588644    ,    0.570864    ,    0.580626    ,    0.618737    ,    0.638731    ,    0.653204    ,    0.654344    ,    0.666465    ,    0.685837    ,    0.696767    ,    0.686543    ,    0.685167    ,    0.700432    ,    0.727543    ,    0.75923    },
    {    0.539827    ,    0.552655    ,    0.584583    ,    0.62938    ,    0.675019    ,    0.712585    ,    0.766523    ,    0.84335    ,    0.885834    ,    0.88267    ,    0.869811    ,    0.826776    ,    0.753996    ,    0.694943    ,    0.67591    ,    0.65898    ,    0.628495    ,    0.591076    ,    0.557011    ,    0.532579    ,    0.515296    ,    0.501287    ,    0.488329    ,    0.473157    ,    0.452045    ,    0.430051    ,    0.409379    ,    0.390218    ,    0.373117    ,    0.357509    ,    0.343901    ,    0.328129    ,    0.309667    ,    0.293549    ,    0.283527    ,    0.276323    ,    0.26916    ,    0.258643    ,    0.245814    ,    0.242238    ,    0.262142    ,    0.304454    ,    0.330074    ,    0.360769    ,    0.397782    ,    0.429043    ,    0.46465    ,    0.497329    ,    0.510762    ,    0.509285    ,    0.501078    ,    0.489121    ,    0.469233    ,    0.461473    ,    0.469645    ,    0.485882    ,    0.511524    ,    0.543714    ,    0.589514    ,    0.64098    ,    0.697254    ,    0.751742    ,    0.802966    ,    0.850133    ,    0.871391    ,    0.865848    ,    0.864684    ,    0.852812    ,    0.83205    ,    0.798632    ,    0.72646    ,    0.634816    ,    0.578437    ,    0.558781    ,    0.551587    ,    0.55639    ,    0.570583    ,    0.573498    ,    0.557337    ,    0.547601    ,    0.56052    ,    0.584484    ,    0.597734    ,    0.594164    ,    0.598656    ,    0.626486    ,    0.665537    ,    0.725642    ,    0.840693    ,    0.977929    ,    1.01992    ,    1.03011    ,    1.04909    ,    1.04102    ,    0.986253    ,    0.860068    ,    0.723859    ,    0.604998    ,    0.535454    ,    0.514065    ,    0.518636    ,    0.531894    ,    0.545069    ,    0.56368    ,    0.574611    ,    0.568709    ,    0.550323    ,    0.531654    ,    0.504912    ,    0.472286    ,    0.445578    ,    0.426585    ,    0.417663    ,    0.415469    ,    0.411797    ,    0.417132    ,    0.446951    ,    0.47278    ,    0.474496    ,    0.458834    ,    0.453938    ,    0.458083    ,    0.442985    ,    0.436553    ,    0.430205    ,    0.431738    ,    0.457827    ,    0.498559    },
    {    0.355147    ,    0.374345    ,    0.412974    ,    0.459696    ,    0.508244    ,    0.532079    ,    0.560424    ,    0.618781    ,    0.656659    ,    0.654983    ,    0.671114    ,    0.713597    ,    0.731958    ,    0.68786    ,    0.663692    ,    0.651971    ,    0.627457    ,    0.576334    ,    0.520195    ,    0.474252    ,    0.444804    ,    0.426716    ,    0.413705    ,    0.396579    ,    0.372912    ,    0.350069    ,    0.330485    ,    0.313853    ,    0.298728    ,    0.282804    ,    0.268361    ,    0.254529    ,    0.239229    ,    0.224852    ,    0.215462    ,    0.207354    ,    0.20009    ,    0.193157    ,    0.179812    ,    0.165767    ,    0.167197    ,    0.185497    ,    0.195453    ,    0.213482    ,    0.240316    ,    0.275792    ,    0.315577    ,    0.359328    ,    0.397777    ,    0.413961    ,    0.414334    ,    0.408391    ,    0.397379    ,    0.387588    ,    0.405488    ,    0.425684    ,    0.437566    ,    0.456593    ,    0.477568    ,    0.491182    ,    0.508197    ,    0.536287    ,    0.573857    ,    0.608384    ,    0.637471    ,    0.649355    ,    0.648863    ,    0.645112    ,    0.628001    ,    0.575884    ,    0.507155    ,    0.457653    ,    0.428677    ,    0.403812    ,    0.379855    ,    0.364497    ,    0.35824    ,    0.357397    ,    0.356663    ,    0.353571    ,    0.351563    ,    0.346239    ,    0.340903    ,    0.333347    ,    0.325128    ,    0.324713    ,    0.339734    ,    0.370284    ,    0.411225    ,    0.471125    ,    0.570184    ,    0.658181    ,    0.728802    ,    0.746594    ,    0.727722    ,    0.672494    ,    0.60376    ,    0.540471    ,    0.499964    ,    0.487997    ,    0.481651    ,    0.4579    ,    0.422258    ,    0.39792    ,    0.383532    ,    0.372737    ,    0.35769    ,    0.341594    ,    0.325152    ,    0.315082    ,    0.307579    ,    0.302468    ,    0.30193    ,    0.301208    ,    0.305834    ,    0.318695    ,    0.332026    ,    0.31546    ,    0.303952    ,    0.306063    ,    0.328264    ,    0.33268    ,    0.304633    ,    0.285918    ,    0.281349    ,    0.278064    ,    0.287744    ,    0.312335    },
    {    0.253298    ,    0.261038    ,    0.278977    ,    0.309328    ,    0.347735    ,    0.38778    ,    0.437314    ,    0.483288    ,    0.501893    ,    0.50867    ,    0.525308    ,    0.556249    ,    0.581957    ,    0.583874    ,    0.580238    ,    0.566099    ,    0.540781    ,    0.496816    ,    0.437915    ,    0.389301    ,    0.365932    ,    0.355567    ,    0.343221    ,    0.321022    ,    0.294086    ,    0.27391    ,    0.25876    ,    0.246462    ,    0.23485    ,    0.219653    ,    0.20567    ,    0.194036    ,    0.183539    ,    0.173433    ,    0.164788    ,    0.157781    ,    0.155063    ,    0.138385    ,    0.11632    ,    0.106206    ,    0.102296    ,    0.131252    ,    0.141463    ,    0.149161    ,    0.159664    ,    0.176298    ,    0.209568    ,    0.262179    ,    0.30111    ,    0.304535    ,    0.310769    ,    0.312613    ,    0.311056    ,    0.32906    ,    0.37419    ,    0.391697    ,    0.376624    ,    0.369983    ,    0.368339    ,    0.368128    ,    0.374342    ,    0.379782    ,    0.389168    ,    0.400205    ,    0.40552    ,    0.41241    ,    0.437692    ,    0.453298    ,    0.452659    ,    0.434662    ,    0.409216    ,    0.381231    ,    0.350794    ,    0.326337    ,    0.297888    ,    0.285102    ,    0.286137    ,    0.285785    ,    0.287966    ,    0.289994    ,    0.287169    ,    0.276257    ,    0.264545    ,    0.256645    ,    0.250276    ,    0.250283    ,    0.261693    ,    0.282009    ,    0.29827    ,    0.308528    ,    0.322093    ,    0.362824    ,    0.408857    ,    0.435123    ,    0.431262    ,    0.389789    ,    0.37741    ,    0.38671    ,    0.387726    ,    0.404037    ,    0.381537    ,    0.359753    ,    0.346978    ,    0.33103    ,    0.319191    ,    0.302033    ,    0.288805    ,    0.282361    ,    0.274229    ,    0.271023    ,    0.269065    ,    0.270232    ,    0.274533    ,    0.282662    ,    0.298043    ,    0.315218    ,    0.319402    ,    0.271014    ,    0.245302    ,    0.245616    ,    0.243432    ,    0.228193    ,    0.194446    ,    0.182525    ,    0.177504    ,    0.172741    ,    0.184077    ,    0.218242    },
    {    0.210188    ,    0.213035    ,    0.218002    ,    0.234176    ,    0.257619    ,    0.283892    ,    0.33308    ,    0.37372    ,    0.378851    ,    0.373146    ,    0.361571    ,    0.365214    ,    0.396196    ,    0.434203    ,    0.435585    ,    0.408328    ,    0.380104    ,    0.349009    ,    0.308419    ,    0.280981    ,    0.277569    ,    0.279651    ,    0.277766    ,    0.263676    ,    0.235854    ,    0.214869    ,    0.204428    ,    0.200076    ,    0.195001    ,    0.18734    ,    0.181443    ,    0.174025    ,    0.166241    ,    0.158102    ,    0.149421    ,    0.145307    ,    0.146069    ,    0.119955    ,    0.1023    ,    0.0944173    ,    0.0881258    ,    0.112427    ,    0.117016    ,    0.124555    ,    0.138304    ,    0.153668    ,    0.184474    ,    0.220245    ,    0.234864    ,    0.245356    ,    0.26187    ,    0.255873    ,    0.252755    ,    0.278071    ,    0.309664    ,    0.328836    ,    0.332816    ,    0.324356    ,    0.318869    ,    0.33254    ,    0.354078    ,    0.35057    ,    0.348359    ,    0.349616    ,    0.346819    ,    0.358867    ,    0.383364    ,    0.401405    ,    0.416919    ,    0.403841    ,    0.385712    ,    0.374858    ,    0.36963    ,    0.398194    ,    0.393685    ,    0.336301    ,    0.295902    ,    0.280763    ,    0.275009    ,    0.263271    ,    0.246259    ,    0.234135    ,    0.227411    ,    0.220948    ,    0.217905    ,    0.221454    ,    0.233933    ,    0.245231    ,    0.251518    ,    0.255437    ,    0.257071    ,    0.276264    ,    0.272873    ,    0.30373    ,    0.348917    ,    0.293936    ,    0.289019    ,    0.291849    ,    0.269629    ,    0.272704    ,    0.267735    ,    0.260693    ,    0.268296    ,    0.268275    ,    0.263633    ,    0.250712    ,    0.240577    ,    0.236534    ,    0.235598    ,    0.240476    ,    0.258926    ,    0.287986    ,    0.320229    ,    0.339319    ,    0.315248    ,    0.258365    ,    0.23522    ,    0.203773    ,    0.190238    ,    0.178728    ,    0.170634    ,    0.162721    ,    0.161037    ,    0.16296    ,    0.156155    ,    0.156896    ,    0.164982    ,    0.185109    },
    {    0.192987    ,    0.175156    ,    0.166919    ,    0.17683    ,    0.19464    ,    0.207151    ,    0.22826    ,    0.232618    ,    0.224345    ,    0.221184    ,    0.235553    ,    0.259294    ,    0.282595    ,    0.30597    ,    0.283219    ,    0.248181    ,    0.234855    ,    0.227366    ,    0.220971    ,    0.220604    ,    0.22701    ,    0.230087    ,    0.232742    ,    0.231126    ,    0.212832    ,    0.194748    ,    0.191517    ,    0.192188    ,    0.1893    ,    0.182896    ,    0.176578    ,    0.168361    ,    0.162565    ,    0.159094    ,    0.158369    ,    0.152261    ,    0.144662    ,    0.122893    ,    0.190533    ,    0.135554    ,    0.0972283    ,    0.0942189    ,    0.10961    ,    0.122357    ,    0.140137    ,    0.159085    ,    0.175678    ,    0.175317    ,    0.188184    ,    0.210666    ,    0.226122    ,    0.235561    ,    0.245352    ,    0.256137    ,    0.261688    ,    0.271952    ,    0.288761    ,    0.293154    ,    0.306308    ,    0.338219    ,    0.372955    ,    0.34717    ,    0.332839    ,    0.342883    ,    0.358867    ,    0.396097    ,    0.425251    ,    0.423058    ,    0.375145    ,    0.359727    ,    0.363034    ,    0.378769    ,    0.417702    ,    0.486977    ,    0.471373    ,    0.376856    ,    0.323339    ,    0.292751    ,    0.276966    ,    0.262095    ,    0.246115    ,    0.23598    ,    0.234851    ,    0.227079    ,    0.217402    ,    0.223755    ,    0.237655    ,    0.235589    ,    0.232134    ,    0.228589    ,    0.230187    ,    0.226519    ,    0.226288    ,    0.235003    ,    0.250693    ,    0.250968    ,    0.233965    ,    0.214623    ,    0.203681    ,    0.199216    ,    0.20246    ,    0.205388    ,    0.202551    ,    0.200561    ,    0.195367    ,    0.193518    ,    0.199771    ,    0.206366    ,    0.215313    ,    0.235294    ,    0.278477    ,    0.32936    ,    0.368901    ,    0.351359    ,    0.258112    ,    0.212363    ,    0.199575    ,    0.185324    ,    0.167188    ,    0.143852    ,    0.136748    ,    0.146166    ,    0.164189    ,    0.168611    ,    0.157992    ,    0.155168    ,    0.166737    ,    0.185971    },
    {    0.182869    ,    0.198675    ,    0.260919    ,    0.284927    ,    0.266027    ,    0.245896    ,    0.226313    ,    0.204593    ,    0.192532    ,    0.190047    ,    0.19459    ,    0.202858    ,    0.203559    ,    0.192375    ,    0.183835    ,    0.191764    ,    0.203964    ,    0.214048    ,    0.221666    ,    0.226772    ,    0.227666    ,    0.224175    ,    0.221931    ,    0.216865    ,    0.208533    ,    0.199043    ,    0.193734    ,    0.190986    ,    0.187571    ,    0.181751    ,    0.173766    ,    0.165368    ,    0.159587    ,    0.157145    ,    0.156333    ,    0.148923    ,    0.139613    ,    0.132735    ,    0.150645    ,    0.143973    ,    0.113769    ,    0.0992052    ,    0.105581    ,    0.113292    ,    0.140598    ,    0.164015    ,    0.179556    ,    0.195491    ,    0.179802    ,    0.193581    ,    0.199314    ,    0.212072    ,    0.224871    ,    0.232263    ,    0.24559    ,    0.260675    ,    0.284191    ,    0.284561    ,    0.324733    ,    0.369855    ,    0.382072    ,    0.362858    ,    0.335765    ,    0.323748    ,    0.335171    ,    0.370111    ,    0.382819    ,    0.361003    ,    0.343196    ,    0.367637    ,    0.376337    ,    0.360767    ,    0.368427    ,    0.353026    ,    0.259704    ,    0.204195    ,    0.214208    ,    0.233057    ,    0.264782    ,    0.280128    ,    0.274645    ,    0.259259    ,    0.236855    ,    0.225634    ,    0.23056    ,    0.257037    ,    0.260852    ,    0.250093    ,    0.243621    ,    0.239026    ,    0.23048    ,    0.206404    ,    0.207657    ,    0.220953    ,    0.228458    ,    0.216528    ,    0.20158    ,    0.19455    ,    0.182249    ,    0.159126    ,    0.157417    ,    0.161078    ,    0.170435    ,    0.182358    ,    0.189787    ,    0.194928    ,    0.199204    ,    0.204192    ,    0.211589    ,    0.228242    ,    0.245963    ,    0.254182    ,    0.247404    ,    0.22897    ,    0.216627    ,    0.21935    ,    0.230144    ,    0.222334    ,    0.190688    ,    0.168574    ,    0.168636    ,    0.166752    ,    0.178912    ,    0.195524    ,    0.18058    ,    0.179423    ,    0.191991    ,    0.19015    },
    {    0.265609    ,    0.280708    ,    0.298146    ,    0.292601    ,    0.250459    ,    0.207853    ,    0.1737    ,    0.156919    ,    0.15191    ,    0.155261    ,    0.161177    ,    0.163442    ,    0.167529    ,    0.174673    ,    0.184014    ,    0.194893    ,    0.207396    ,    0.216821    ,    0.219905    ,    0.221815    ,    0.223026    ,    0.22435    ,    0.224078    ,    0.217272    ,    0.208789    ,    0.199457    ,    0.190913    ,    0.188431    ,    0.188104    ,    0.189855    ,    0.186719    ,    0.175113    ,    0.16212    ,    0.15308    ,    0.158069    ,    0.159638    ,    0.137961    ,    0.132452    ,    0.133194    ,    0.154795    ,    0.124592    ,    0.114051    ,    0.110939    ,    0.14321    ,    0.190654    ,    0.244658    ,    0.243564    ,    0.237826    ,    0.213954    ,    0.21176    ,    0.211456    ,    0.216373    ,    0.216071    ,    0.224428    ,    0.24534    ,    0.264574    ,    0.286532    ,    0.318266    ,    0.375138    ,    0.394976    ,    0.314187    ,    0.277681    ,    0.268537    ,    0.271085    ,    0.275019    ,    0.266698    ,    0.271955    ,    0.282558    ,    0.292034    ,    0.298225    ,    0.268682    ,    0.227314    ,    0.219931    ,    0.228817    ,    0.212371    ,    0.191325    ,    0.208232    ,    0.212975    ,    0.256204    ,    0.282491    ,    0.281079    ,    0.258047    ,    0.220394    ,    0.215999    ,    0.302534    ,    0.349593    ,    0.350613    ,    0.343688    ,    0.331402    ,    0.300338    ,    0.267298    ,    0.247614    ,    0.247863    ,    0.246572    ,    0.23742    ,    0.227004    ,    0.223346    ,    0.22014    ,    0.201768    ,    0.193039    ,    0.19707    ,    0.19678    ,    0.199196    ,    0.199674    ,    0.197729    ,    0.19743    ,    0.198839    ,    0.206371    ,    0.216602    ,    0.214896    ,    0.204718    ,    0.197496    ,    0.195477    ,    0.199026    ,    0.195542    ,    0.189165    ,    0.172482    ,    0.227239    ,    0.25061    ,    0.205684    ,    0.195478    ,    0.210513    ,    0.210338    ,    0.213869    ,    0.223823    ,    0.204272    ,    0.197302    ,    0.207757    },
    {    0.212191    ,    0.202052    ,    0.190011    ,    0.168598    ,    0.152585    ,    0.140719    ,    0.131775    ,    0.127446    ,    0.12489    ,    0.124419    ,    0.129883    ,    0.132712    ,    0.137368    ,    0.141092    ,    0.143774    ,    0.155974    ,    0.171289    ,    0.1811    ,    0.190349    ,    0.201132    ,    0.207952    ,    0.208318    ,    0.205338    ,    0.201621    ,    0.196761    ,    0.19102    ,    0.189019    ,    0.19083    ,    0.19121    ,    0.193567    ,    0.192249    ,    0.186609    ,    0.177371    ,    0.159983    ,    0.157597    ,    0.153749    ,    0.12779    ,    0.125197    ,    0.145121    ,    0.136483    ,    0.13229    ,    0.147738    ,    0.107188    ,    0.123628    ,    0.213748    ,    0.336685    ,    0.343703    ,    0.256317    ,    0.204973    ,    0.189606    ,    0.190139    ,    0.194212    ,    0.209837    ,    0.225308    ,    0.238103    ,    0.23721    ,    0.233006    ,    0.241762    ,    0.274591    ,    0.273961    ,    0.211552    ,    0.19256    ,    0.194254    ,    0.211114    ,    0.220039    ,    0.240636    ,    0.284143    ,    0.34315    ,    0.372858    ,    0.356743    ,    0.320089    ,    0.311436    ,    0.308552    ,    0.311806    ,    0.302197    ,    0.275989    ,    0.255151    ,    0.279311    ,    0.30488    ,    0.297989    ,    0.290855    ,    0.26079    ,    0.252894    ,    0.258101    ,    0.290997    ,    0.312694    ,    0.318851    ,    0.31794    ,    0.303393    ,    0.288349    ,    0.278969    ,    0.278993    ,    0.281192    ,    0.271547    ,    0.256058    ,    0.249315    ,    0.244594    ,    0.235017    ,    0.217896    ,    0.202816    ,    0.190698    ,    0.185348    ,    0.183176    ,    0.184356    ,    0.18678    ,    0.185178    ,    0.182637    ,    0.180752    ,    0.18683    ,    0.180511    ,    0.166896    ,    0.162211    ,    0.15435    ,    0.151036    ,    0.153125    ,    0.157047    ,    0.150687    ,    0.214793    ,    0.285507    ,    0.254185    ,    0.222831    ,    0.229751    ,    0.223117    ,    0.218894    ,    0.232906    ,    0.232411    ,    0.209578    ,    0.205978    },
    {    0.177496    ,    0.172882    ,    0.165167    ,    0.158115    ,    0.156677    ,    0.152788    ,    0.147512    ,    0.141182    ,    0.1351    ,    0.129052    ,    0.124275    ,    0.120894    ,    0.120641    ,    0.12337    ,    0.122828    ,    0.128411    ,    0.138049    ,    0.151275    ,    0.167657    ,    0.178328    ,    0.18181    ,    0.183439    ,    0.18464    ,    0.184674    ,    0.183136    ,    0.184726    ,    0.189084    ,    0.191433    ,    0.187813    ,    0.17897    ,    0.178845    ,    0.19059    ,    0.1986    ,    0.17748    ,    0.14293    ,    0.138601    ,    0.123198    ,    0.115015    ,    0.136645    ,    0.181037    ,    0.182179    ,    0.187359    ,    0.172488    ,    0.129505    ,    0.366622    ,    0.504168    ,    0.408916    ,    0.358089    ,    0.330722    ,    0.309528    ,    0.293033    ,    0.2879    ,    0.276341    ,    0.255312    ,    0.227158    ,    0.202745    ,    0.180862    ,    0.158798    ,    0.162148    ,    0.179019    ,    0.190904    ,    0.172095    ,    0.182279    ,    0.196411    ,    0.214528    ,    0.254047    ,    0.284882    ,    0.305606    ,    0.330723    ,    0.325561    ,    0.291091    ,    0.257002    ,    0.242262    ,    0.239195    ,    0.244011    ,    0.244672    ,    0.220262    ,    0.208615    ,    0.219154    ,    0.230242    ,    0.239924    ,    0.241745    ,    0.240462    ,    0.244876    ,    0.251191    ,    0.253906    ,    0.259019    ,    0.260528    ,    0.26151    ,    0.260583    ,    0.267845    ,    0.276047    ,    0.277747    ,    0.256164    ,    0.237416    ,    0.232822    ,    0.226078    ,    0.20696    ,    0.185357    ,    0.170062    ,    0.156169    ,    0.151542    ,    0.1526    ,    0.15816    ,    0.157799    ,    0.154898    ,    0.15069    ,    0.152585    ,    0.150662    ,    0.147271    ,    0.135404    ,    0.124753    ,    0.127486    ,    0.125535    ,    0.125011    ,    0.130193    ,    0.131074    ,    0.146181    ,    0.266722    ,    0.273761    ,    0.237271    ,    0.231245    ,    0.22514    ,    0.221792    ,    0.228839    ,    0.232837    ,    0.219338    ,    0.197636    },
    {    0.162692    ,    0.160642    ,    0.160945    ,    0.15831    ,    0.158237    ,    0.154152    ,    0.146706    ,    0.14041    ,    0.136578    ,    0.133669    ,    0.132328    ,    0.131527    ,    0.127828    ,    0.125024    ,    0.12313    ,    0.122498    ,    0.127974    ,    0.140655    ,    0.15344    ,    0.156863    ,    0.152963    ,    0.157412    ,    0.17    ,    0.181081    ,    0.184222    ,    0.179874    ,    0.177351    ,    0.174258    ,    0.166712    ,    0.160682    ,    0.169164    ,    0.184548    ,    0.192707    ,    0.175229    ,    0.154029    ,    0.14078    ,    0.124898    ,    0.110168    ,    0.124216    ,    0.193088    ,    0.187464    ,    0.204886    ,    0.191403    ,    0.154289    ,    0.170114    ,    0.322597    ,    0.221616    ,    0.205734    ,    0.214028    ,    0.222277    ,    0.207616    ,    0.185563    ,    0.163008    ,    0.147716    ,    0.143592    ,    0.140949    ,    0.139599    ,    0.145476    ,    0.160693    ,    0.173471    ,    0.183639    ,    0.189405    ,    0.187463    ,    0.185085    ,    0.193535    ,    0.231749    ,    0.260884    ,    0.257214    ,    0.260563    ,    0.250584    ,    0.218518    ,    0.187834    ,    0.177344    ,    0.152156    ,    0.130863    ,    0.148128    ,    0.157333    ,    0.151478    ,    0.157964    ,    0.168265    ,    0.177047    ,    0.178495    ,    0.177768    ,    0.182527    ,    0.189576    ,    0.19523    ,    0.197507    ,    0.202759    ,    0.208253    ,    0.204917    ,    0.206376    ,    0.207216    ,    0.204985    ,    0.195953    ,    0.190824    ,    0.185877    ,    0.178987    ,    0.17484    ,    0.164987    ,    0.149958    ,    0.142563    ,    0.140131    ,    0.143727    ,    0.158426    ,    0.14549    ,    0.148718    ,    0.151013    ,    0.129039    ,    0.115081    ,    0.110931    ,    0.107403    ,    0.109699    ,    0.118095    ,    0.131572    ,    0.145975    ,    0.168201    ,    0.161065    ,    0.144508    ,    0.174093    ,    0.205008    ,    0.1726    ,    0.169564    ,    0.175224    ,    0.18555    ,    0.201341    ,    0.210832    ,    0.20184    ,    0.181289    },
    {    0.167366    ,    0.172155    ,    0.170115    ,    0.163732    ,    0.153223    ,    0.146114    ,    0.142312    ,    0.141901    ,    0.142738    ,    0.146579    ,    0.150209    ,    0.14972    ,    0.145975    ,    0.142287    ,    0.139314    ,    0.136712    ,    0.139979    ,    0.147621    ,    0.151409    ,    0.146363    ,    0.145026    ,    0.151636    ,    0.165128    ,    0.173825    ,    0.171154    ,    0.167727    ,    0.164846    ,    0.162302    ,    0.159258    ,    0.162466    ,    0.173819    ,    0.180738    ,    0.184523    ,    0.177993    ,    0.168925    ,    0.148274    ,    0.125421    ,    0.125926    ,    0.140903    ,    0.194054    ,    0.223865    ,    0.216874    ,    0.226061    ,    0.212029    ,    0.254717    ,    0.226714    ,    0.176781    ,    0.168485    ,    0.162047    ,    0.161049    ,    0.166859    ,    0.157224    ,    0.148481    ,    0.142917    ,    0.147465    ,    0.151099    ,    0.157219    ,    0.159376    ,    0.161261    ,    0.170792    ,    0.193271    ,    0.197034    ,    0.183183    ,    0.178185    ,    0.18476    ,    0.206919    ,    0.213213    ,    0.211753    ,    0.216394    ,    0.204025    ,    0.200951    ,    0.19884    ,    0.16597    ,    0.136606    ,    0.107654    ,    0.0882377    ,    0.0891456    ,    0.119158    ,    0.125173    ,    0.131015    ,    0.139239    ,    0.146455    ,    0.145857    ,    0.137924    ,    0.143427    ,    0.146141    ,    0.149472    ,    0.152626    ,    0.152244    ,    0.155985    ,    0.163288    ,    0.16589    ,    0.17558    ,    0.179898    ,    0.183234    ,    0.181181    ,    0.178363    ,    0.172528    ,    0.153827    ,    0.152122    ,    0.148447    ,    0.143123    ,    0.150573    ,    0.155834    ,    0.143548    ,    0.150532    ,    0.150311    ,    0.149364    ,    0.157152    ,    0.111441    ,    0.0888828    ,    0.1032    ,    0.119556    ,    0.151787    ,    0.171857    ,    0.180244    ,    0.18814    ,    0.175512    ,    0.158285    ,    0.133742    ,    0.129463    ,    0.121926    ,    0.128179    ,    0.140182    ,    0.1492    ,    0.155961    ,    0.157968    ,    0.158632    },
    {    0.180499    ,    0.180094    ,    0.176833    ,    0.166935    ,    0.155833    ,    0.152783    ,    0.155555    ,    0.159471    ,    0.162081    ,    0.163539    ,    0.164313    ,    0.163198    ,    0.159727    ,    0.152877    ,    0.149317    ,    0.152817    ,    0.159663    ,    0.163562    ,    0.157784    ,    0.15152    ,    0.155029    ,    0.159333    ,    0.161358    ,    0.161285    ,    0.158032    ,    0.156079    ,    0.152401    ,    0.154053    ,    0.157973    ,    0.166832    ,    0.178633    ,    0.190732    ,    0.193713    ,    0.189137    ,    0.176634    ,    0.150434    ,    0.126432    ,    0.128973    ,    0.14253    ,    0.211009    ,    0.292374    ,    0.292298    ,    0.301778    ,    0.319429    ,    0.363367    ,    0.298764    ,    0.181612    ,    0.14836    ,    0.153998    ,    0.157404    ,    0.159254    ,    0.154867    ,    0.152504    ,    0.1506    ,    0.157252    ,    0.167262    ,    0.172358    ,    0.172298    ,    0.171908    ,    0.173611    ,    0.1787    ,    0.17957    ,    0.172074    ,    0.167468    ,    0.172356    ,    0.174855    ,    0.1736    ,    0.178505    ,    0.183738    ,    0.197021    ,    0.191848    ,    0.230665    ,    0.294015    ,    0.255318    ,    0.161715    ,    0.0982901    ,    0.0816215    ,    0.0880745    ,    0.106159    ,    0.109992    ,    0.121141    ,    0.126744    ,    0.134797    ,    0.1358    ,    0.154487    ,    0.156445    ,    0.158295    ,    0.16822    ,    0.168969    ,    0.17489    ,    0.178261    ,    0.176946    ,    0.177113    ,    0.172215    ,    0.169825    ,    0.168918    ,    0.160779    ,    0.150824    ,    0.149877    ,    0.152761    ,    0.146804    ,    0.150413    ,    0.147031    ,    0.142229    ,    0.150842    ,    0.149008    ,    0.170479    ,    0.187953    ,    0.202651    ,    0.161864    ,    0.122999    ,    0.118291    ,    0.135421    ,    0.16104    ,    0.168923    ,    0.167623    ,    0.19506    ,    0.208822    ,    0.190672    ,    0.13642    ,    0.129329    ,    0.123298    ,    0.12502    ,    0.130382    ,    0.13687    ,    0.148516    ,    0.160026    ,    0.170897    },
    {    0.17873    ,    0.179323    ,    0.180869    ,    0.176868    ,    0.173109    ,    0.175586    ,    0.1763    ,    0.176234    ,    0.177012    ,    0.175743    ,    0.173655    ,    0.170493    ,    0.166709    ,    0.161844    ,    0.161302    ,    0.162388    ,    0.161058    ,    0.156295    ,    0.150678    ,    0.15105    ,    0.151716    ,    0.151358    ,    0.150716    ,    0.150217    ,    0.150492    ,    0.153087    ,    0.159205    ,    0.168572    ,    0.177692    ,    0.188563    ,    0.198828    ,    0.202865    ,    0.199892    ,    0.189408    ,    0.172047    ,    0.144545    ,    0.117672    ,    0.126805    ,    0.138467    ,    0.236871    ,    0.259654    ,    0.269877    ,    0.344715    ,    0.407358    ,    0.392859    ,    0.256853    ,    0.184632    ,    0.172962    ,    0.147558    ,    0.147881    ,    0.146081    ,    0.146487    ,    0.151137    ,    0.154657    ,    0.160503    ,    0.165159    ,    0.167917    ,    0.166468    ,    0.162163    ,    0.160764    ,    0.161973    ,    0.162379    ,    0.164489    ,    0.167441    ,    0.162927    ,    0.155152    ,    0.155917    ,    0.1674    ,    0.197164    ,    0.217122    ,    0.185774    ,    0.285603    ,    0.338086    ,    0.360339    ,    0.257995    ,    0.168905    ,    0.107764    ,    0.0841773    ,    0.101353    ,    0.0973796    ,    0.0949735    ,    0.0980891    ,    0.110627    ,    0.122994    ,    0.135358    ,    0.135979    ,    0.143348    ,    0.145411    ,    0.143886    ,    0.143642    ,    0.145437    ,    0.147046    ,    0.146001    ,    0.140256    ,    0.136855    ,    0.134419    ,    0.133159    ,    0.134143    ,    0.139195    ,    0.143402    ,    0.14686    ,    0.147572    ,    0.142025    ,    0.148128    ,    0.163147    ,    0.171267    ,    0.185158    ,    0.231119    ,    0.210977    ,    0.174968    ,    0.144575    ,    0.150054    ,    0.178287    ,    0.181873    ,    0.213081    ,    0.227436    ,    0.198565    ,    0.182977    ,    0.15777    ,    0.122328    ,    0.122109    ,    0.123436    ,    0.128931    ,    0.136886    ,    0.146206    ,    0.15394    ,    0.162743    ,    0.171078    },
    {    0.187576    ,    0.189796    ,    0.192294    ,    0.19537    ,    0.195219    ,    0.189177    ,    0.183549    ,    0.179624    ,    0.174713    ,    0.170364    ,    0.167337    ,    0.163165    ,    0.162003    ,    0.165257    ,    0.167392    ,    0.164669    ,    0.16042    ,    0.155359    ,    0.150876    ,    0.147552    ,    0.146057    ,    0.150471    ,    0.159047    ,    0.168343    ,    0.176378    ,    0.186396    ,    0.200506    ,    0.211414    ,    0.216908    ,    0.222492    ,    0.220684    ,    0.213817    ,    0.203574    ,    0.18836    ,    0.164412    ,    0.130544    ,    0.12652    ,    0.145563    ,    0.13744    ,    0.283092    ,    0.440099    ,    0.263184    ,    0.358953    ,    0.361721    ,    0.325847    ,    0.225403    ,    0.208825    ,    0.202404    ,    0.198742    ,    0.158341    ,    0.135563    ,    0.141659    ,    0.144796    ,    0.149992    ,    0.156163    ,    0.159676    ,    0.162192    ,    0.161396    ,    0.162868    ,    0.167423    ,    0.170473    ,    0.170134    ,    0.16447    ,    0.154634    ,    0.147811    ,    0.153258    ,    0.170017    ,    0.192034    ,    0.218924    ,    0.178087    ,    0.231153    ,    0.268368    ,    0.337282    ,    0.34682    ,    0.305809    ,    0.229645    ,    0.155892    ,    0.0817032    ,    0.105518    ,    0.114759    ,    0.159055    ,    0.107409    ,    0.0893653    ,    0.0951292    ,    0.0962187    ,    0.096757    ,    0.0996554    ,    0.10717    ,    0.113896    ,    0.115809    ,    0.118676    ,    0.115865    ,    0.114311    ,    0.116012    ,    0.123335    ,    0.130711    ,    0.13305    ,    0.132798    ,    0.134708    ,    0.135684    ,    0.139261    ,    0.140796    ,    0.148871    ,    0.16599    ,    0.181174    ,    0.19487    ,    0.229105    ,    0.226436    ,    0.208413    ,    0.226313    ,    0.250711    ,    0.302882    ,    0.209131    ,    0.215571    ,    0.230733    ,    0.211099    ,    0.213622    ,    0.210361    ,    0.131032    ,    0.132803    ,    0.138754    ,    0.142106    ,    0.145365    ,    0.156178    ,    0.167004    ,    0.168758    ,    0.174683    ,    0.181567    },
    {    0.199518    ,    0.201929    ,    0.204779    ,    0.205301    ,    0.198266    ,    0.188978    ,    0.179279    ,    0.1703    ,    0.164761    ,    0.161305    ,    0.160239    ,    0.160367    ,    0.15929    ,    0.159034    ,    0.158576    ,    0.155235    ,    0.152313    ,    0.15239    ,    0.155528    ,    0.160924    ,    0.172755    ,    0.190748    ,    0.207807    ,    0.223633    ,    0.235491    ,    0.24431    ,    0.250408    ,    0.252615    ,    0.251785    ,    0.250309    ,    0.240072    ,    0.221517    ,    0.200767    ,    0.176918    ,    0.155367    ,    0.147137    ,    0.16133    ,    0.159406    ,    0.135468    ,    0.32224    ,    0.397616    ,    0.261205    ,    0.353444    ,    0.342962    ,    0.250884    ,    0.262289    ,    0.234669    ,    0.25412    ,    0.239969    ,    0.212423    ,    0.13777    ,    0.140735    ,    0.1455    ,    0.152708    ,    0.161809    ,    0.171421    ,    0.177249    ,    0.179184    ,    0.180215    ,    0.178432    ,    0.178631    ,    0.176724    ,    0.176901    ,    0.173911    ,    0.173362    ,    0.178334    ,    0.189112    ,    0.195218    ,    0.20729    ,    0.166852    ,    0.210903    ,    0.328332    ,    0.342777    ,    0.306996    ,    0.292885    ,    0.276975    ,    0.251507    ,    0.0906558    ,    0.0987806    ,    0.121959    ,    0.178498    ,    0.111312    ,    0.0924067    ,    0.0869569    ,    0.087216    ,    0.0868056    ,    0.0911312    ,    0.0998926    ,    0.103287    ,    0.102442    ,    0.106627    ,    0.113701    ,    0.12108    ,    0.129724    ,    0.135436    ,    0.133911    ,    0.130329    ,    0.132517    ,    0.133238    ,    0.134921    ,    0.144786    ,    0.153996    ,    0.172779    ,    0.191764    ,    0.177313    ,    0.125552    ,    0.128242    ,    0.170398    ,    0.276596    ,    0.327126    ,    0.355198    ,    0.331785    ,    0.310291    ,    0.268941    ,    0.259501    ,    0.202391    ,    0.28095    ,    0.166734    ,    0.135301    ,    0.146406    ,    0.153215    ,    0.153941    ,    0.15725    ,    0.171616    ,    0.173721    ,    0.177859    ,    0.186197    ,    0.19168    },
    {    0.179035    ,    0.182674    ,    0.184479    ,    0.182378    ,    0.175119    ,    0.168452    ,    0.159363    ,    0.151841    ,    0.14952    ,    0.15034    ,    0.152308    ,    0.152959    ,    0.155645    ,    0.160515    ,    0.165379    ,    0.171637    ,    0.17991    ,    0.192035    ,    0.209528    ,    0.228323    ,    0.245228    ,    0.25965    ,    0.269222    ,    0.273985    ,    0.278821    ,    0.280727    ,    0.280313    ,    0.274443    ,    0.264955    ,    0.249647    ,    0.230781    ,    0.207476    ,    0.191404    ,    0.179411    ,    0.172388    ,    0.175932    ,    0.165406    ,    0.143705    ,    0.157456    ,    0.255224    ,    0.251252    ,    0.248655    ,    0.358604    ,    0.302808    ,    0.232225    ,    0.202799    ,    0.200978    ,    0.254014    ,    0.27421    ,    0.256765    ,    0.159606    ,    0.159506    ,    0.183539    ,    0.186477    ,    0.194161    ,    0.198062    ,    0.201189    ,    0.201026    ,    0.202854    ,    0.202559    ,    0.20076    ,    0.198273    ,    0.19594    ,    0.191906    ,    0.186866    ,    0.184146    ,    0.184327    ,    0.188224    ,    0.159604    ,    0.164776    ,    0.26932    ,    0.353908    ,    0.370215    ,    0.346629    ,    0.341015    ,    0.349233    ,    0.313293    ,    0.167975    ,    0.110299    ,    0.0963259    ,    0.147682    ,    0.143936    ,    0.0949069    ,    0.0952485    ,    0.101629    ,    0.104747    ,    0.10748    ,    0.109814    ,    0.110374    ,    0.113926    ,    0.116253    ,    0.117848    ,    0.119093    ,    0.119762    ,    0.120517    ,    0.124432    ,    0.131503    ,    0.140294    ,    0.150486    ,    0.16034    ,    0.170398    ,    0.185383    ,    0.201927    ,    0.206248    ,    0.190199    ,    0.160967    ,    0.148025    ,    0.196164    ,    0.239431    ,    0.286206    ,    0.257707    ,    0.223132    ,    0.22038    ,    0.193495    ,    0.173911    ,    0.156024    ,    0.167544    ,    0.141961    ,    0.152985    ,    0.157562    ,    0.159781    ,    0.15734    ,    0.157411    ,    0.160336    ,    0.163059    ,    0.168271    ,    0.170321    ,    0.17078    },
    {    0.140412    ,    0.141294    ,    0.140781    ,    0.137104    ,    0.134847    ,    0.135595    ,    0.136812    ,    0.137439    ,    0.141376    ,    0.14443    ,    0.150182    ,    0.157162    ,    0.165866    ,    0.174596    ,    0.186416    ,    0.199414    ,    0.21549    ,    0.232624    ,    0.244763    ,    0.256871    ,    0.264844    ,    0.269002    ,    0.274872    ,    0.278162    ,    0.276223    ,    0.270541    ,    0.264202    ,    0.253026    ,    0.23899    ,    0.223828    ,    0.213485    ,    0.20216    ,    0.19656    ,    0.191321    ,    0.186831    ,    0.182673    ,    0.157227    ,    0.153714    ,    0.268556    ,    0.817604    ,    0.218536    ,    0.264825    ,    0.261524    ,    0.230927    ,    0.201171    ,    0.183219    ,    0.180655    ,    0.193242    ,    0.210605    ,    0.180865    ,    0.175645    ,    0.185265    ,    0.198622    ,    0.206767    ,    0.216435    ,    0.216466    ,    0.21581    ,    0.214084    ,    0.212258    ,    0.210996    ,    0.210518    ,    0.207479    ,    0.202956    ,    0.196739    ,    0.188021    ,    0.178948    ,    0.172348    ,    0.159176    ,    0.129295    ,    0.240962    ,    0.282885    ,    0.324766    ,    0.321521    ,    0.326081    ,    0.359487    ,    0.378751    ,    0.343069    ,    0.293501    ,    0.156099    ,    0.0932722    ,    0.0972397    ,    0.158104    ,    0.103548    ,    0.105484    ,    0.112737    ,    0.109706    ,    0.109903    ,    0.110698    ,    0.11247    ,    0.114707    ,    0.116633    ,    0.11954    ,    0.121956    ,    0.126633    ,    0.134764    ,    0.145307    ,    0.154933    ,    0.163551    ,    0.171596    ,    0.177685    ,    0.185308    ,    0.192268    ,    0.193525    ,    0.18799    ,    0.178427    ,    0.161962    ,    0.155771    ,    0.161547    ,    0.157029    ,    0.159234    ,    0.198355    ,    0.214148    ,    0.202421    ,    0.119069    ,    0.11857    ,    0.139001    ,    0.143062    ,    0.160783    ,    0.167225    ,    0.164228    ,    0.161251    ,    0.158718    ,    0.157737    ,    0.156524    ,    0.154013    ,    0.152267    ,    0.147357    ,    0.146261    },
    {    0.112759    ,    0.116617    ,    0.120008    ,    0.124932    ,    0.131964    ,    0.135804    ,    0.138337    ,    0.141112    ,    0.145284    ,    0.146925    ,    0.152712    ,    0.159295    ,    0.164464    ,    0.172266    ,    0.181683    ,    0.18866    ,    0.19905    ,    0.207539    ,    0.21281    ,    0.221723    ,    0.22563    ,    0.230348    ,    0.22572    ,    0.222154    ,    0.220369    ,    0.217724    ,    0.215993    ,    0.21228    ,    0.20871    ,    0.201588    ,    0.195362    ,    0.191103    ,    0.189295    ,    0.18037    ,    0.182027    ,    0.17115    ,    0.132395    ,    0.172578    ,    0.262271    ,    0.238091    ,    0.236986    ,    0.24632    ,    0.246488    ,    0.244438    ,    0.282586    ,    0.207683    ,    0.203692    ,    0.227301    ,    0.21391    ,    0.183398    ,    0.18578    ,    0.187377    ,    0.193272    ,    0.192026    ,    0.201481    ,    0.207298    ,    0.211577    ,    0.214025    ,    0.214233    ,    0.214363    ,    0.212709    ,    0.20959    ,    0.202161    ,    0.191265    ,    0.179396    ,    0.165617    ,    0.147575    ,    0.135946    ,    0.109511    ,    0.212    ,    0.238108    ,    0.248851    ,    0.26179    ,    0.256448    ,    0.302957    ,    0.341119    ,    0.287361    ,    0.203934    ,    0.157088    ,    0.101438    ,    0.101938    ,    0.116928    ,    0.110509    ,    0.110261    ,    0.113034    ,    0.111371    ,    0.112399    ,    0.111793    ,    0.11378    ,    0.114619    ,    0.11498    ,    0.11629    ,    0.119246    ,    0.124868    ,    0.13282    ,    0.143124    ,    0.156185    ,    0.165886    ,    0.174618    ,    0.183641    ,    0.190179    ,    0.191271    ,    0.1916    ,    0.185903    ,    0.173889    ,    0.160945    ,    0.149234    ,    0.144442    ,    0.137749    ,    0.145211    ,    0.152179    ,    0.139326    ,    0.144106    ,    0.124654    ,    0.138252    ,    0.152229    ,    0.167157    ,    0.177726    ,    0.171664    ,    0.163127    ,    0.155388    ,    0.147515    ,    0.141823    ,    0.136125    ,    0.128233    ,    0.121807    ,    0.117257    ,    0.115563    },
    {    0.122796    ,    0.128903    ,    0.135731    ,    0.140425    ,    0.146959    ,    0.148081    ,    0.149168    ,    0.148386    ,    0.144487    ,    0.145734    ,    0.147852    ,    0.146409    ,    0.148787    ,    0.150535    ,    0.152923    ,    0.167258    ,    0.172139    ,    0.169839    ,    0.176655    ,    0.18624    ,    0.184604    ,    0.186012    ,    0.18514    ,    0.182039    ,    0.180094    ,    0.179445    ,    0.176192    ,    0.171214    ,    0.165392    ,    0.162028    ,    0.157713    ,    0.151469    ,    0.14612    ,    0.149527    ,    0.170344    ,    0.159682    ,    0.159575    ,    0.199783    ,    0.185498    ,    0.287667    ,    0.261885    ,    0.256382    ,    0.291018    ,    0.281079    ,    0.305324    ,    0.252641    ,    0.21954    ,    0.225136    ,    0.219392    ,    0.163779    ,    0.174762    ,    0.151095    ,    0.183944    ,    0.183955    ,    0.190137    ,    0.195564    ,    0.199319    ,    0.202253    ,    0.20464    ,    0.207168    ,    0.207268    ,    0.202431    ,    0.191072    ,    0.176332    ,    0.161796    ,    0.149895    ,    0.131798    ,    0.132156    ,    0.116337    ,    0.211677    ,    0.255782    ,    0.221103    ,    0.205155    ,    0.213201    ,    0.240911    ,    0.270744    ,    0.274314    ,    0.228119    ,    0.199739    ,    0.111965    ,    0.111472    ,    0.109435    ,    0.11012    ,    0.107867    ,    0.107985    ,    0.109884    ,    0.111653    ,    0.114251    ,    0.117076    ,    0.117711    ,    0.117313    ,    0.120127    ,    0.118147    ,    0.11837    ,    0.118822    ,    0.124796    ,    0.13331    ,    0.145321    ,    0.160819    ,    0.171188    ,    0.187012    ,    0.206966    ,    0.214904    ,    0.19125    ,    0.167841    ,    0.143197    ,    0.121479    ,    0.11989    ,    0.111994    ,    0.131701    ,    0.139547    ,    0.132158    ,    0.133493    ,    0.143567    ,    0.12666    ,    0.135762    ,    0.16884    ,    0.166495    ,    0.162462    ,    0.148895    ,    0.139622    ,    0.133404    ,    0.13099    ,    0.12658    ,    0.122363    ,    0.118001    ,    0.118173    ,    0.119659    },
    {    0.160365    ,    0.163736    ,    0.164591    ,    0.164979    ,    0.166435    ,    0.164565    ,    0.163128    ,    0.1612    ,    0.158239    ,    0.153004    ,    0.151105    ,    0.148156    ,    0.139859    ,    0.136624    ,    0.138469    ,    0.14001    ,    0.144314    ,    0.14783    ,    0.147662    ,    0.144997    ,    0.145935    ,    0.148477    ,    0.156357    ,    0.154001    ,    0.152921    ,    0.152453    ,    0.14431    ,    0.14044    ,    0.135734    ,    0.134226    ,    0.135218    ,    0.137102    ,    0.138446    ,    0.150238    ,    0.174582    ,    0.169859    ,    0.184867    ,    0.313179    ,    0.339831    ,    0.312895    ,    0.330474    ,    0.340686    ,    0.323188    ,    0.310893    ,    0.310223    ,    0.282724    ,    0.256707    ,    0.208254    ,    0.183194    ,    0.157494    ,    0.15041    ,    0.136411    ,    0.170786    ,    0.174343    ,    0.178602    ,    0.18263    ,    0.184278    ,    0.186991    ,    0.189692    ,    0.192775    ,    0.191743    ,    0.187623    ,    0.181939    ,    0.166846    ,    0.15135    ,    0.139559    ,    0.129412    ,    0.132103    ,    0.14307    ,    0.195766    ,    0.21694    ,    0.196193    ,    0.210103    ,    0.198197    ,    0.235936    ,    0.266921    ,    0.191372    ,    0.255839    ,    0.137793    ,    0.110434    ,    0.108923    ,    0.107675    ,    0.113417    ,    0.115787    ,    0.11492    ,    0.112209    ,    0.112329    ,    0.114775    ,    0.119245    ,    0.121607    ,    0.12338    ,    0.125685    ,    0.122635    ,    0.129715    ,    0.126863    ,    0.116238    ,    0.116981    ,    0.124165    ,    0.145982    ,    0.180511    ,    0.223238    ,    0.216923    ,    0.165754    ,    0.151252    ,    0.142116    ,    0.144662    ,    0.143715    ,    0.134218    ,    0.13462    ,    0.140691    ,    0.142803    ,    0.136675    ,    0.128503    ,    0.103713    ,    0.116199    ,    0.130086    ,    0.127684    ,    0.145723    ,    0.147565    ,    0.140432    ,    0.136424    ,    0.139959    ,    0.148348    ,    0.152894    ,    0.155865    ,    0.15672    ,    0.158125    ,    0.157055    },
    {    0.181716    ,    0.178593    ,    0.176437    ,    0.174674    ,    0.172352    ,    0.167284    ,    0.163869    ,    0.162398    ,    0.163178    ,    0.160101    ,    0.160185    ,    0.156199    ,    0.153481    ,    0.146195    ,    0.139605    ,    0.134118    ,    0.131157    ,    0.13203    ,    0.127894    ,    0.1186    ,    0.118227    ,    0.117462    ,    0.122414    ,    0.125001    ,    0.127515    ,    0.132976    ,    0.138262    ,    0.141298    ,    0.143468    ,    0.141934    ,    0.146112    ,    0.150505    ,    0.151504    ,    0.163714    ,    0.185689    ,    0.189212    ,    0.170413    ,    0.342707    ,    0.360092    ,    0.355703    ,    0.401201    ,    0.373063    ,    0.350119    ,    0.318575    ,    0.294203    ,    0.273141    ,    0.236872    ,    0.231888    ,    0.172969    ,    0.157727    ,    0.16029    ,    0.159496    ,    0.162775    ,    0.168507    ,    0.171157    ,    0.174287    ,    0.174439    ,    0.173384    ,    0.174677    ,    0.175182    ,    0.172387    ,    0.169366    ,    0.164328    ,    0.154999    ,    0.145776    ,    0.136274    ,    0.12754    ,    0.124195    ,    0.145943    ,    0.205317    ,    0.195983    ,    0.219043    ,    0.223643    ,    0.191882    ,    0.208182    ,    0.234561    ,    0.258866    ,    0.207556    ,    0.119596    ,    0.112836    ,    0.11482    ,    0.112115    ,    0.110854    ,    0.118706    ,    0.121269    ,    0.118008    ,    0.119692    ,    0.116537    ,    0.119988    ,    0.122624    ,    0.122172    ,    0.124222    ,    0.129127    ,    0.124845    ,    0.129317    ,    0.121661    ,    0.119512    ,    0.123646    ,    0.144261    ,    0.189549    ,    0.20426    ,    0.160747    ,    0.155342    ,    0.14628    ,    0.142874    ,    0.148069    ,    0.133999    ,    0.12568    ,    0.137445    ,    0.13944    ,    0.140645    ,    0.137593    ,    0.137197    ,    0.132728    ,    0.113315    ,    0.105957    ,    0.12245    ,    0.141575    ,    0.171195    ,    0.200681    ,    0.210699    ,    0.216146    ,    0.21992    ,    0.218587    ,    0.213198    ,    0.206502    ,    0.197733    ,    0.189994    },
    {    0.203685    ,    0.19299    ,    0.183218    ,    0.179957    ,    0.177065    ,    0.168964    ,    0.164353    ,    0.161819    ,    0.158964    ,    0.155005    ,    0.156208    ,    0.155583    ,    0.155752    ,    0.152263    ,    0.149588    ,    0.142266    ,    0.133541    ,    0.131279    ,    0.122816    ,    0.116962    ,    0.111369    ,    0.105072    ,    0.103126    ,    0.112607    ,    0.126158    ,    0.142002    ,    0.156877    ,    0.162971    ,    0.167194    ,    0.172195    ,    0.178573    ,    0.186766    ,    0.190855    ,    0.189873    ,    0.192589    ,    0.165115    ,    0.14852    ,    0.268523    ,    0.329983    ,    0.340583    ,    0.409169    ,    0.372382    ,    0.377816    ,    0.373928    ,    0.299074    ,    0.219407    ,    0.20847    ,    0.181486    ,    0.155689    ,    0.160349    ,    0.158961    ,    0.157279    ,    0.158312    ,    0.162727    ,    0.163648    ,    0.164163    ,    0.166175    ,    0.166658    ,    0.167697    ,    0.163102    ,    0.158931    ,    0.148628    ,    0.139427    ,    0.132263    ,    0.128055    ,    0.123536    ,    0.122791    ,    0.131209    ,    0.155933    ,    0.183642    ,    0.185541    ,    0.195126    ,    0.208889    ,    0.220364    ,    0.19994    ,    0.179637    ,    0.253926    ,    0.194045    ,    0.151938    ,    0.139896    ,    0.11437    ,    0.124982    ,    0.126687    ,    0.130432    ,    0.127487    ,    0.120908    ,    0.118422    ,    0.117334    ,    0.11933    ,    0.122329    ,    0.121189    ,    0.121342    ,    0.126237    ,    0.122674    ,    0.124185    ,    0.12241    ,    0.126449    ,    0.133369    ,    0.143724    ,    0.156011    ,    0.149301    ,    0.14211    ,    0.148092    ,    0.147262    ,    0.149585    ,    0.144599    ,    0.133333    ,    0.13504    ,    0.144701    ,    0.141645    ,    0.145374    ,    0.135339    ,    0.148252    ,    0.13748    ,    0.145656    ,    0.192765    ,    0.224589    ,    0.259699    ,    0.299752    ,    0.311303    ,    0.310796    ,    0.303198    ,    0.296302    ,    0.2851    ,    0.272242    ,    0.257015    ,    0.240347    ,    0.22261    },
    {    0.207124    ,    0.193587    ,    0.18112    ,    0.172403    ,    0.165495    ,    0.160886    ,    0.154638    ,    0.149636    ,    0.145908    ,    0.140223    ,    0.139585    ,    0.138897    ,    0.139813    ,    0.138931    ,    0.135271    ,    0.134357    ,    0.129012    ,    0.12555    ,    0.122482    ,    0.119169    ,    0.117497    ,    0.118952    ,    0.122435    ,    0.131623    ,    0.145721    ,    0.159963    ,    0.170617    ,    0.171848    ,    0.171057    ,    0.172337    ,    0.175805    ,    0.176858    ,    0.1786    ,    0.171153    ,    0.175552    ,    0.165863    ,    0.141088    ,    0.185519    ,    0.275015    ,    0.30467    ,    0.361768    ,    0.380443    ,    0.376564    ,    0.341803    ,    0.278929    ,    0.204844    ,    0.178635    ,    0.155182    ,    0.159622    ,    0.156473    ,    0.151789    ,    0.148    ,    0.149921    ,    0.152136    ,    0.151392    ,    0.148226    ,    0.151639    ,    0.15432    ,    0.152476    ,    0.151854    ,    0.148727    ,    0.145354    ,    0.14048    ,    0.136739    ,    0.132602    ,    0.125482    ,    0.125166    ,    0.126874    ,    0.184144    ,    0.168779    ,    0.199687    ,    0.185759    ,    0.191381    ,    0.191271    ,    0.175896    ,    0.280113    ,    0.401524    ,    0.280828    ,    0.247017    ,    0.163567    ,    0.128106    ,    0.129796    ,    0.134928    ,    0.135006    ,    0.132756    ,    0.132144    ,    0.131505    ,    0.133895    ,    0.133331    ,    0.132091    ,    0.129334    ,    0.128385    ,    0.130109    ,    0.129206    ,    0.131485    ,    0.132397    ,    0.137762    ,    0.132944    ,    0.133388    ,    0.117887    ,    0.141133    ,    0.113733    ,    0.139715    ,    0.187987    ,    0.139817    ,    0.152018    ,    0.142742    ,    0.140694    ,    0.147163    ,    0.149725    ,    0.140516    ,    0.12824    ,    0.149461    ,    0.168462    ,    0.191098    ,    0.221223    ,    0.253292    ,    0.28881    ,    0.308254    ,    0.31029    ,    0.308579    ,    0.302345    ,    0.296564    ,    0.290441    ,    0.280314    ,    0.267329    ,    0.249216    ,    0.228361    },
    {    0.163004    ,    0.155309    ,    0.151728    ,    0.146432    ,    0.142229    ,    0.141068    ,    0.140116    ,    0.136509    ,    0.133865    ,    0.130503    ,    0.129052    ,    0.126509    ,    0.122838    ,    0.122938    ,    0.120374    ,    0.119566    ,    0.117931    ,    0.116496    ,    0.118294    ,    0.116841    ,    0.120125    ,    0.125159    ,    0.130826    ,    0.1353    ,    0.138659    ,    0.141111    ,    0.142166    ,    0.14281    ,    0.140801    ,    0.138918    ,    0.138526    ,    0.134949    ,    0.138111    ,    0.138736    ,    0.14294    ,    0.140013    ,    0.137667    ,    0.143365    ,    0.244984    ,    0.31712    ,    0.342166    ,    0.305216    ,    0.277628    ,    0.249326    ,    0.244309    ,    0.152081    ,    0.147364    ,    0.157815    ,    0.155602    ,    0.149257    ,    0.145792    ,    0.141425    ,    0.139079    ,    0.140183    ,    0.143392    ,    0.144175    ,    0.145342    ,    0.145467    ,    0.148497    ,    0.148793    ,    0.148248    ,    0.152362    ,    0.157925    ,    0.14706    ,    0.152429    ,    0.145873    ,    0.142538    ,    0.161908    ,    0.18216    ,    0.150893    ,    0.199025    ,    0.19593    ,    0.216081    ,    0.194253    ,    0.167247    ,    0.340836    ,    0.367627    ,    0.396366    ,    0.333395    ,    0.202506    ,    0.17178    ,    0.138265    ,    0.139206    ,    0.141452    ,    0.140279    ,    0.143766    ,    0.150402    ,    0.149906    ,    0.148711    ,    0.144362    ,    0.140384    ,    0.138161    ,    0.137479    ,    0.141311    ,    0.147298    ,    0.147973    ,    0.145606    ,    0.137263    ,    0.121456    ,    0.133196    ,    0.0962616    ,    0.107356    ,    0.124481    ,    0.129413    ,    0.144752    ,    0.166085    ,    0.16427    ,    0.140885    ,    0.14616    ,    0.145141    ,    0.131219    ,    0.131432    ,    0.132475    ,    0.134628    ,    0.141353    ,    0.148398    ,    0.163572    ,    0.175965    ,    0.183949    ,    0.193351    ,    0.19981    ,    0.20011    ,    0.197505    ,    0.194877    ,    0.191144    ,    0.186677    ,    0.179782    ,    0.170504    },
    {    0.123911    ,    0.127029    ,    0.12991    ,    0.130909    ,    0.133269    ,    0.13438    ,    0.136411    ,    0.136563    ,    0.137611    ,    0.136673    ,    0.135152    ,    0.135849    ,    0.13405    ,    0.130541    ,    0.126787    ,    0.128065    ,    0.125022    ,    0.123062    ,    0.122444    ,    0.120976    ,    0.124034    ,    0.122007    ,    0.120006    ,    0.117985    ,    0.119492    ,    0.119771    ,    0.122881    ,    0.121023    ,    0.120088    ,    0.119116    ,    0.126263    ,    0.134843    ,    0.138257    ,    0.13332    ,    0.135162    ,    0.147193    ,    0.154372    ,    0.161686    ,    0.174902    ,    0.265071    ,    0.30502    ,    0.239987    ,    0.191659    ,    0.151459    ,    0.158606    ,    0.146551    ,    0.151376    ,    0.154213    ,    0.145747    ,    0.138764    ,    0.13761    ,    0.133403    ,    0.127231    ,    0.122181    ,    0.124358    ,    0.124951    ,    0.126164    ,    0.132589    ,    0.145387    ,    0.160804    ,    0.154789    ,    0.206288    ,    0.193962    ,    0.202599    ,    0.231722    ,    0.23901    ,    0.246887    ,    0.241419    ,    0.209047    ,    0.170484    ,    0.199076    ,    0.204187    ,    0.207042    ,    0.2102    ,    0.265476    ,    0.260926    ,    0.245875    ,    0.283947    ,    0.253976    ,    0.327519    ,    0.256661    ,    0.1741    ,    0.144821    ,    0.143432    ,    0.151547    ,    0.161997    ,    0.172297    ,    0.170345    ,    0.159925    ,    0.148709    ,    0.14363    ,    0.141032    ,    0.135128    ,    0.104807    ,    0.13561    ,    0.146435    ,    0.145084    ,    0.136583    ,    0.118185    ,    0.107741    ,    0.141382    ,    0.113184    ,    0.121989    ,    0.129386    ,    0.139497    ,    0.148311    ,    0.143704    ,    0.153693    ,    0.158993    ,    0.152648    ,    0.136649    ,    0.132981    ,    0.131275    ,    0.127897    ,    0.125522    ,    0.122266    ,    0.120806    ,    0.119206    ,    0.119671    ,    0.120966    ,    0.122432    ,    0.123119    ,    0.121047    ,    0.120434    ,    0.121081    ,    0.121869    ,    0.123333    ,    0.1233    },
    {    0.13143    ,    0.133916    ,    0.136788    ,    0.139993    ,    0.142436    ,    0.144396    ,    0.148317    ,    0.15182    ,    0.156222    ,    0.159139    ,    0.163202    ,    0.163884    ,    0.162598    ,    0.160216    ,    0.158837    ,    0.157914    ,    0.153588    ,    0.148469    ,    0.142834    ,    0.138827    ,    0.130076    ,    0.122978    ,    0.117363    ,    0.1116    ,    0.107312    ,    0.111654    ,    0.114746    ,    0.118383    ,    0.12254    ,    0.129302    ,    0.135741    ,    0.134669    ,    0.122721    ,    0.139114    ,    0.165571    ,    0.166147    ,    0.1806    ,    0.169393    ,    0.197379    ,    0.199228    ,    0.240696    ,    0.179548    ,    0.150617    ,    0.150824    ,    0.139828    ,    0.134753    ,    0.134803    ,    0.133883    ,    0.131403    ,    0.130788    ,    0.132143    ,    0.13234    ,    0.131075    ,    0.125665    ,    0.125014    ,    0.130257    ,    0.138739    ,    0.152116    ,    0.152458    ,    0.192871    ,    0.21779    ,    0.229525    ,    0.249145    ,    0.247872    ,    0.228152    ,    0.253637    ,    0.244061    ,    0.26218    ,    0.238839    ,    0.283285    ,    0.323646    ,    0.358669    ,    0.28862    ,    0.282069    ,    0.567262    ,    0.623516    ,    0.272949    ,    0.255741    ,    0.274027    ,    0.262438    ,    0.246822    ,    0.181528    ,    0.169223    ,    0.151925    ,    0.167489    ,    0.185377    ,    0.195749    ,    0.195481    ,    0.18351    ,    0.168407    ,    0.158034    ,    0.128558    ,    0.119915    ,    0.12854    ,    0.132863    ,    0.13147    ,    0.131164    ,    0.126189    ,    0.11108    ,    0.108156    ,    0.107174    ,    0.113533    ,    0.121019    ,    0.133568    ,    0.152898    ,    0.15201    ,    0.138957    ,    0.141361    ,    0.150406    ,    0.132979    ,    0.125234    ,    0.124748    ,    0.127492    ,    0.127943    ,    0.126157    ,    0.123536    ,    0.122607    ,    0.120243    ,    0.120196    ,    0.119893    ,    0.122889    ,    0.124821    ,    0.127225    ,    0.127873    ,    0.129524    ,    0.130306    ,    0.129978    ,    0.130276    },
    {    0.139205    ,    0.145629    ,    0.153781    ,    0.161194    ,    0.167003    ,    0.175351    ,    0.182422    ,    0.187523    ,    0.192787    ,    0.196904    ,    0.200397    ,    0.20062    ,    0.200928    ,    0.199158    ,    0.196212    ,    0.193205    ,    0.189312    ,    0.181044    ,    0.172007    ,    0.157974    ,    0.141815    ,    0.128883    ,    0.120125    ,    0.110449    ,    0.10579    ,    0.104881    ,    0.103399    ,    0.110542    ,    0.107859    ,    0.106285    ,    0.100166    ,    0.11515    ,    0.136475    ,    0.215605    ,    0.184414    ,    0.139165    ,    0.145574    ,    0.147126    ,    0.143474    ,    0.157091    ,    0.170605    ,    0.167812    ,    0.152349    ,    0.144743    ,    0.143548    ,    0.144347    ,    0.144661    ,    0.143281    ,    0.143448    ,    0.144885    ,    0.145878    ,    0.144968    ,    0.14423    ,    0.145918    ,    0.14948    ,    0.157302    ,    0.169151    ,    0.187313    ,    0.154158    ,    0.226371    ,    0.27513    ,    0.268029    ,    0.328564    ,    0.32753    ,    0.292639    ,    0.278548    ,    0.36357    ,    0.347496    ,    0.356507    ,    0.366019    ,    0.379558    ,    0.353976    ,    0.339554    ,    0.343141    ,    0.415482    ,    0.411188    ,    0.469471    ,    0.293992    ,    0.336288    ,    0.252685    ,    0.206185    ,    0.185293    ,    0.157787    ,    0.147165    ,    0.164521    ,    0.180401    ,    0.199083    ,    0.212457    ,    0.205799    ,    0.192301    ,    0.176174    ,    0.16843    ,    0.148353    ,    0.142748    ,    0.141046    ,    0.131999    ,    0.126219    ,    0.122107    ,    0.117076    ,    0.130984    ,    0.154807    ,    0.142055    ,    0.138284    ,    0.133569    ,    0.155885    ,    0.15942    ,    0.149374    ,    0.153554    ,    0.132693    ,    0.123524    ,    0.119827    ,    0.117434    ,    0.11758    ,    0.119838    ,    0.122964    ,    0.123434    ,    0.123731    ,    0.122839    ,    0.124258    ,    0.12614    ,    0.127784    ,    0.127397    ,    0.12847    ,    0.127359    ,    0.126753    ,    0.128633    ,    0.130437    ,    0.133207    },
    {    0.126601    ,    0.132848    ,    0.140751    ,    0.152592    ,    0.166083    ,    0.1806    ,    0.197323    ,    0.212482    ,    0.225898    ,    0.236622    ,    0.24363    ,    0.2472    ,    0.250933    ,    0.249979    ,    0.245035    ,    0.237852    ,    0.229595    ,    0.22104    ,    0.214355    ,    0.205645    ,    0.196881    ,    0.18316    ,    0.170784    ,    0.152713    ,    0.133292    ,    0.112441    ,    0.103465    ,    0.102966    ,    0.0984585    ,    0.124852    ,    0.122215    ,    0.14766    ,    0.226775    ,    0.206869    ,    0.161754    ,    0.140139    ,    0.147904    ,    0.144853    ,    0.141628    ,    0.142623    ,    0.140707    ,    0.137288    ,    0.13799    ,    0.139158    ,    0.143693    ,    0.146523    ,    0.147111    ,    0.144719    ,    0.143469    ,    0.144779    ,    0.146154    ,    0.1449    ,    0.144937    ,    0.151584    ,    0.164974    ,    0.179628    ,    0.187965    ,    0.205511    ,    0.213551    ,    0.188077    ,    0.259237    ,    0.276946    ,    0.286949    ,    0.283306    ,    0.287716    ,    0.307566    ,    0.321236    ,    0.282263    ,    0.277739    ,    0.304881    ,    0.301804    ,    0.302837    ,    0.351705    ,    0.298559    ,    0.284428    ,    0.284307    ,    0.318846    ,    0.363706    ,    0.309775    ,    0.256514    ,    0.309146    ,    0.242684    ,    0.197799    ,    0.166218    ,    0.163658    ,    0.173794    ,    0.189536    ,    0.203831    ,    0.209014    ,    0.198834    ,    0.149113    ,    0.18457    ,    0.218198    ,    0.146639    ,    0.149363    ,    0.148821    ,    0.139715    ,    0.12417    ,    0.114395    ,    0.191398    ,    0.181708    ,    0.16021    ,    0.14954    ,    0.126277    ,    0.154961    ,    0.160746    ,    0.16197    ,    0.159398    ,    0.139684    ,    0.131418    ,    0.123195    ,    0.11531    ,    0.114933    ,    0.115637    ,    0.119109    ,    0.122476    ,    0.127305    ,    0.126872    ,    0.1276    ,    0.126562    ,    0.125957    ,    0.125816    ,    0.123468    ,    0.120782    ,    0.119201    ,    0.118921    ,    0.120599    ,    0.122459    },
    {    0.111389    ,    0.112516    ,    0.1162    ,    0.123506    ,    0.134072    ,    0.149018    ,    0.1665    ,    0.185802    ,    0.208057    ,    0.230147    ,    0.250277    ,    0.265286    ,    0.275111    ,    0.280871    ,    0.282134    ,    0.280968    ,    0.280978    ,    0.279563    ,    0.275621    ,    0.272645    ,    0.272615    ,    0.268303    ,    0.257152    ,    0.233373    ,    0.198901    ,    0.153815    ,    0.121412    ,    0.14814    ,    0.144428    ,    0.16604    ,    0.170091    ,    0.165765    ,    0.159729    ,    0.137674    ,    0.128974    ,    0.124704    ,    0.120701    ,    0.111354    ,    0.10275    ,    0.103371    ,    0.11313    ,    0.116144    ,    0.11566    ,    0.111395    ,    0.109405    ,    0.109874    ,    0.112691    ,    0.11176    ,    0.114467    ,    0.120138    ,    0.127106    ,    0.132243    ,    0.136068    ,    0.138227    ,    0.147973    ,    0.166484    ,    0.180297    ,    0.191136    ,    0.186336    ,    0.181034    ,    0.225155    ,    0.223696    ,    0.235858    ,    0.270819    ,    0.296605    ,    0.292322    ,    0.282273    ,    0.232327    ,    0.199144    ,    0.208549    ,    0.21769    ,    0.222921    ,    0.227721    ,    0.248306    ,    0.224698    ,    0.224129    ,    0.261288    ,    0.265407    ,    0.283773    ,    0.248248    ,    0.214388    ,    0.181626    ,    0.213296    ,    0.185947    ,    0.165262    ,    0.17929    ,    0.192416    ,    0.205693    ,    0.210754    ,    0.207748    ,    0.195763    ,    0.244009    ,    0.262405    ,    0.180321    ,    0.139806    ,    0.150312    ,    0.151331    ,    0.142565    ,    0.151296    ,    0.209265    ,    0.204471    ,    0.215189    ,    0.171137    ,    0.116692    ,    0.145171    ,    0.151355    ,    0.15677    ,    0.159634    ,    0.162621    ,    0.168393    ,    0.158261    ,    0.146829    ,    0.140213    ,    0.136157    ,    0.134714    ,    0.13283    ,    0.132355    ,    0.13255    ,    0.130568    ,    0.128403    ,    0.125244    ,    0.124416    ,    0.123007    ,    0.120621    ,    0.117972    ,    0.115705    ,    0.114563    ,    0.113225    },
    {    0.112273    ,    0.108058    ,    0.105938    ,    0.10994    ,    0.118408    ,    0.131269    ,    0.145464    ,    0.163574    ,    0.189329    ,    0.215361    ,    0.236591    ,    0.254975    ,    0.271686    ,    0.281087    ,    0.285779    ,    0.290692    ,    0.295246    ,    0.298319    ,    0.298381    ,    0.300371    ,    0.310461    ,    0.324096    ,    0.323481    ,    0.297245    ,    0.248916    ,    0.185703    ,    0.154256    ,    0.241533    ,    0.257574    ,    0.246516    ,    0.1831    ,    0.154014    ,    0.135407    ,    0.124884    ,    0.125908    ,    0.134692    ,    0.123625    ,    0.109712    ,    0.123293    ,    0.135364    ,    0.126455    ,    0.116554    ,    0.111218    ,    0.10456    ,    0.099112    ,    0.0962907    ,    0.0985095    ,    0.0984938    ,    0.101347    ,    0.102134    ,    0.104546    ,    0.107795    ,    0.113627    ,    0.121288    ,    0.130919    ,    0.139626    ,    0.143509    ,    0.156387    ,    0.193694    ,    0.220468    ,    0.220914    ,    0.210099    ,    0.225412    ,    0.271383    ,    0.274229    ,    0.263463    ,    0.229071    ,    0.179534    ,    0.166913    ,    0.173336    ,    0.215241    ,    0.19218    ,    0.160573    ,    0.192421    ,    0.19855    ,    0.19967    ,    0.221759    ,    0.257387    ,    0.256317    ,    0.22552    ,    0.237319    ,    0.216469    ,    0.189426    ,    0.183688    ,    0.169682    ,    0.186974    ,    0.177633    ,    0.196098    ,    0.202    ,    0.219327    ,    0.276973    ,    0.338499    ,    0.310439    ,    0.259273    ,    0.251267    ,    0.176839    ,    0.170664    ,    0.171398    ,    0.186348    ,    0.217513    ,    0.240728    ,    0.204242    ,    0.211624    ,    0.195424    ,    0.167633    ,    0.159636    ,    0.157126    ,    0.163813    ,    0.189696    ,    0.21062    ,    0.205174    ,    0.195126    ,    0.184927    ,    0.178346    ,    0.171453    ,    0.165117    ,    0.16011    ,    0.157664    ,    0.153375    ,    0.146164    ,    0.140246    ,    0.136647    ,    0.135949    ,    0.131605    ,    0.127498    ,    0.125838    ,    0.122196    ,    0.11803    },
    {    0.148538    ,    0.145093    ,    0.139721    ,    0.131378    ,    0.127024    ,    0.12821    ,    0.135176    ,    0.149886    ,    0.175843    ,    0.208837    ,    0.240523    ,    0.260444    ,    0.272233    ,    0.278783    ,    0.282613    ,    0.282102    ,    0.282593    ,    0.286816    ,    0.29254    ,    0.300868    ,    0.313345    ,    0.333496    ,    0.342253    ,    0.319172    ,    0.26134    ,    0.161548    ,    0.159385    ,    0.230343    ,    0.329519    ,    0.286579    ,    0.183606    ,    0.169312    ,    0.156582    ,    0.14502    ,    0.148433    ,    0.150682    ,    0.135297    ,    0.121886    ,    0.12931    ,    0.131433    ,    0.128954    ,    0.135995    ,    0.141953    ,    0.142477    ,    0.140357    ,    0.138697    ,    0.140428    ,    0.139275    ,    0.137359    ,    0.135686    ,    0.135099    ,    0.12786    ,    0.115509    ,    0.111753    ,    0.116948    ,    0.126658    ,    0.127445    ,    0.121298    ,    0.129849    ,    0.195356    ,    0.176239    ,    0.183322    ,    0.203342    ,    0.225982    ,    0.24265    ,    0.249004    ,    0.219052    ,    0.184085    ,    0.17108    ,    0.187784    ,    0.205468    ,    0.196198    ,    0.159855    ,    0.219578    ,    0.218121    ,    0.208004    ,    0.208707    ,    0.301814    ,    0.243434    ,    0.221853    ,    0.208331    ,    0.225731    ,    0.235831    ,    0.217856    ,    0.201354    ,    0.2029    ,    0.194772    ,    0.20166    ,    0.208697    ,    0.25798    ,    0.301783    ,    0.304939    ,    0.386416    ,    0.312721    ,    0.298598    ,    0.315922    ,    0.327648    ,    0.299123    ,    0.266421    ,    0.209343    ,    0.201076    ,    0.175443    ,    0.174691    ,    0.208994    ,    0.201909    ,    0.201212    ,    0.19951    ,    0.196369    ,    0.181285    ,    0.202497    ,    0.210999    ,    0.211147    ,    0.213661    ,    0.207804    ,    0.202012    ,    0.200502    ,    0.200714    ,    0.198682    ,    0.194102    ,    0.189072    ,    0.180925    ,    0.173209    ,    0.170713    ,    0.168416    ,    0.16588    ,    0.163186    ,    0.158479    ,    0.153742    },
    {    0.205896    ,    0.211476    ,    0.214823    ,    0.212159    ,    0.206676    ,    0.196744    ,    0.181791    ,    0.169021    ,    0.168897    ,    0.186529    ,    0.215594    ,    0.249096    ,    0.275845    ,    0.287214    ,    0.28857    ,    0.286072    ,    0.286244    ,    0.283266    ,    0.285784    ,    0.305026    ,    0.326225    ,    0.333579    ,    0.346411    ,    0.330545    ,    0.215    ,    0.239699    ,    0.245354    ,    0.33689    ,    0.389151    ,    0.307648    ,    0.209533    ,    0.205108    ,    0.199713    ,    0.185391    ,    0.173589    ,    0.193448    ,    0.127661    ,    0.132881    ,    0.1578    ,    0.150203    ,    0.137225    ,    0.141264    ,    0.141408    ,    0.147018    ,    0.156569    ,    0.161026    ,    0.166572    ,    0.175641    ,    0.182904    ,    0.188686    ,    0.189327    ,    0.18378    ,    0.167086    ,    0.150391    ,    0.137398    ,    0.129452    ,    0.131169    ,    0.127677    ,    0.133725    ,    0.143493    ,    0.170742    ,    0.167349    ,    0.175255    ,    0.15699    ,    0.178867    ,    0.195909    ,    0.214446    ,    0.259705    ,    0.234473    ,    0.217811    ,    0.211231    ,    0.196362    ,    0.165298    ,    0.171185    ,    0.173745    ,    0.195241    ,    0.200987    ,    0.238715    ,    0.246177    ,    0.215178    ,    0.20456    ,    0.186891    ,    0.237293    ,    0.24025    ,    0.227126    ,    0.186804    ,    0.238472    ,    0.272389    ,    0.267971    ,    0.199189    ,    0.215641    ,    0.274469    ,    0.314003    ,    0.296432    ,    0.241578    ,    0.240286    ,    0.22052    ,    0.233792    ,    0.232185    ,    0.19021    ,    0.196422    ,    0.219075    ,    0.195023    ,    0.218831    ,    0.237358    ,    0.238194    ,    0.22464    ,    0.20467    ,    0.166951    ,    0.179362    ,    0.194235    ,    0.201048    ,    0.209213    ,    0.216797    ,    0.221072    ,    0.221931    ,    0.224738    ,    0.226401    ,    0.222612    ,    0.21563    ,    0.209587    ,    0.204375    ,    0.201129    ,    0.199661    ,    0.196922    ,    0.194264    ,    0.194754    ,    0.195882    },
    {    0.218183    ,    0.233117    ,    0.252261    ,    0.266028    ,    0.277576    ,    0.283692    ,    0.273695    ,    0.257062    ,    0.239364    ,    0.225061    ,    0.219719    ,    0.225866    ,    0.251228    ,    0.276894    ,    0.285564    ,    0.283615    ,    0.281845    ,    0.278584    ,    0.28733    ,    0.312786    ,    0.33859    ,    0.336148    ,    0.334124    ,    0.314078    ,    0.309608    ,    0.276662    ,    0.401707    ,    0.423226    ,    0.336018    ,    0.322905    ,    0.23744    ,    0.275723    ,    0.265838    ,    0.24625    ,    0.239237    ,    0.223325    ,    0.15336    ,    0.147823    ,    0.170929    ,    0.159735    ,    0.150358    ,    0.150756    ,    0.143167    ,    0.132414    ,    0.128839    ,    0.122452    ,    0.12066    ,    0.126596    ,    0.14408    ,    0.1671    ,    0.20007    ,    0.240813    ,    0.267968    ,    0.24592    ,    0.200868    ,    0.160393    ,    0.145701    ,    0.140226    ,    0.138424    ,    0.151361    ,    0.156288    ,    0.185135    ,    0.155825    ,    0.172624    ,    0.212856    ,    0.174287    ,    0.183663    ,    0.188868    ,    0.190577    ,    0.201301    ,    0.181674    ,    0.165365    ,    0.152377    ,    0.1769    ,    0.170007    ,    0.183192    ,    0.198762    ,    0.214892    ,    0.216549    ,    0.217205    ,    0.230895    ,    0.277847    ,    0.281842    ,    0.21057    ,    0.212048    ,    0.222903    ,    0.241034    ,    0.239668    ,    0.267064    ,    0.240214    ,    0.224111    ,    0.240689    ,    0.283129    ,    0.260188    ,    0.380228    ,    0.435539    ,    0.349198    ,    0.304929    ,    0.289272    ,    0.3044    ,    0.223175    ,    0.222294    ,    0.305873    ,    0.232213    ,    0.240466    ,    0.22367    ,    0.201146    ,    0.192694    ,    0.167487    ,    0.161997    ,    0.171946    ,    0.187076    ,    0.191081    ,    0.197852    ,    0.213182    ,    0.224365    ,    0.226325    ,    0.225002    ,    0.222646    ,    0.223085    ,    0.224081    ,    0.225566    ,    0.229522    ,    0.223646    ,    0.210806    ,    0.200135    ,    0.198058    ,    0.201669    },
    {    0.22907    ,    0.242733    ,    0.259107    ,    0.276888    ,    0.29608    ,    0.309794    ,    0.316213    ,    0.31378    ,    0.305284    ,    0.289735    ,    0.268899    ,    0.250947    ,    0.248549    ,    0.265587    ,    0.281453    ,    0.29083    ,    0.291574    ,    0.292733    ,    0.30147    ,    0.322801    ,    0.33682    ,    0.325227    ,    0.305734    ,    0.28528    ,    0.287149    ,    0.287123    ,    0.431479    ,    0.322639    ,    0.358013    ,    0.346934    ,    0.336458    ,    0.348724    ,    0.316672    ,    0.315457    ,    0.306256    ,    0.274914    ,    0.213634    ,    0.179616    ,    0.1753    ,    0.17436    ,    0.169217    ,    0.1658    ,    0.168172    ,    0.171428    ,    0.169908    ,    0.162937    ,    0.153528    ,    0.139165    ,    0.12613    ,    0.117094    ,    0.119958    ,    0.141625    ,    0.183628    ,    0.237819    ,    0.267303    ,    0.238201    ,    0.18887    ,    0.150248    ,    0.141982    ,    0.154807    ,    0.17583    ,    0.17869    ,    0.190936    ,    0.156568    ,    0.157922    ,    0.184629    ,    0.194834    ,    0.208089    ,    0.200855    ,    0.195254    ,    0.166313    ,    0.144091    ,    0.152868    ,    0.161598    ,    0.174376    ,    0.201605    ,    0.222432    ,    0.225926    ,    0.217528    ,    0.223779    ,    0.249573    ,    0.271565    ,    0.262491    ,    0.250245    ,    0.213593    ,    0.229718    ,    0.229816    ,    0.222248    ,    0.259237    ,    0.233113    ,    0.336373    ,    0.269126    ,    0.353562    ,    0.67655    ,    0.518458    ,    0.493371    ,    0.424419    ,    0.409819    ,    0.349184    ,    0.356441    ,    0.287894    ,    0.197437    ,    0.243696    ,    0.22977    ,    0.249197    ,    0.231311    ,    0.192158    ,    0.199687    ,    0.187177    ,    0.173955    ,    0.166558    ,    0.168934    ,    0.183168    ,    0.192244    ,    0.199907    ,    0.2121    ,    0.218742    ,    0.222278    ,    0.230285    ,    0.236005    ,    0.239822    ,    0.243198    ,    0.246735    ,    0.244976    ,    0.238379    ,    0.231281    ,    0.221185    ,    0.216568    },
    {    0.281977    ,    0.291181    ,    0.299131    ,    0.306848    ,    0.316139    ,    0.325127    ,    0.336864    ,    0.344624    ,    0.347504    ,    0.340602    ,    0.327207    ,    0.305041    ,    0.297067    ,    0.299885    ,    0.297682    ,    0.304251    ,    0.310748    ,    0.31619    ,    0.324072    ,    0.340515    ,    0.33929    ,    0.299768    ,    0.229104    ,    0.288823    ,    0.302142    ,    0.296736    ,    0.318267    ,    0.327484    ,    0.395737    ,    0.406536    ,    0.361766    ,    0.362331    ,    0.3655    ,    0.330615    ,    0.320272    ,    0.3126    ,    0.321182    ,    0.24568    ,    0.194084    ,    0.181959    ,    0.18562    ,    0.20099    ,    0.20067    ,    0.197399    ,    0.200192    ,    0.210028    ,    0.208986    ,    0.196425    ,    0.192083    ,    0.188569    ,    0.179979    ,    0.176817    ,    0.185111    ,    0.201131    ,    0.233378    ,    0.266951    ,    0.265036    ,    0.211862    ,    0.178489    ,    0.192391    ,    0.200412    ,    0.201189    ,    0.208956    ,    0.184674    ,    0.171113    ,    0.208372    ,    0.218528    ,    0.219803    ,    0.200587    ,    0.184648    ,    0.160838    ,    0.142297    ,    0.150768    ,    0.184361    ,    0.199437    ,    0.230182    ,    0.262505    ,    0.245725    ,    0.254568    ,    0.263486    ,    0.292231    ,    0.249482    ,    0.253154    ,    0.244095    ,    0.223046    ,    0.229624    ,    0.185758    ,    0.181513    ,    0.266701    ,    0.370972    ,    0.303581    ,    0.306845    ,    0.413853    ,    0.344737    ,    0.326161    ,    0.359072    ,    0.379769    ,    0.459345    ,    0.425937    ,    0.314444    ,    0.275481    ,    0.24652    ,    0.222257    ,    0.258069    ,    0.261464    ,    0.300784    ,    0.30536    ,    0.251052    ,    0.220167    ,    0.183695    ,    0.178608    ,    0.174416    ,    0.18295    ,    0.18146    ,    0.19381    ,    0.212219    ,    0.226799    ,    0.234864    ,    0.243061    ,    0.25408    ,    0.264023    ,    0.270776    ,    0.275054    ,    0.280256    ,    0.282552    ,    0.282733    ,    0.279154    ,    0.27768    },
    {    0.35329    ,    0.355297    ,    0.357123    ,    0.361139    ,    0.365668    ,    0.370382    ,    0.376763    ,    0.379317    ,    0.378804    ,    0.373281    ,    0.360984    ,    0.345406    ,    0.340954    ,    0.339949    ,    0.328333    ,    0.32762    ,    0.337482    ,    0.339752    ,    0.335646    ,    0.351671    ,    0.355458    ,    0.271773    ,    0.357668    ,    0.308851    ,    0.299794    ,    0.288522    ,    0.267168    ,    0.286551    ,    0.346858    ,    0.326082    ,    0.350239    ,    0.33638    ,    0.341542    ,    0.342198    ,    0.352664    ,    0.339732    ,    0.322056    ,    0.276968    ,    0.252962    ,    0.225715    ,    0.224382    ,    0.229271    ,    0.22675    ,    0.220926    ,    0.215192    ,    0.217837    ,    0.226286    ,    0.231088    ,    0.238102    ,    0.252243    ,    0.265094    ,    0.273945    ,    0.28003    ,    0.293434    ,    0.30349    ,    0.29368    ,    0.285252    ,    0.277629    ,    0.221804    ,    0.231168    ,    0.239844    ,    0.222623    ,    0.187514    ,    0.166683    ,    0.177319    ,    0.161251    ,    0.161842    ,    0.160212    ,    0.173007    ,    0.170264    ,    0.174493    ,    0.166541    ,    0.180939    ,    0.22625    ,    0.223134    ,    0.211692    ,    0.215425    ,    0.237675    ,    0.251017    ,    0.261282    ,    0.251826    ,    0.245592    ,    0.361163    ,    0.289209    ,    0.239365    ,    0.209759    ,    0.221313    ,    0.224379    ,    0.205559    ,    0.67968    ,    0.789256    ,    0.275413    ,    0.222413    ,    0.201823    ,    0.197031    ,    0.221677    ,    0.232838    ,    0.230332    ,    0.242326    ,    0.232382    ,    0.236483    ,    0.325745    ,    0.311088    ,    0.312385    ,    0.322279    ,    0.335373    ,    0.308828    ,    0.250744    ,    0.240126    ,    0.213451    ,    0.216118    ,    0.227859    ,    0.216586    ,    0.216026    ,    0.213082    ,    0.25916    ,    0.30113    ,    0.31789    ,    0.325733    ,    0.32798    ,    0.333025    ,    0.341207    ,    0.344653    ,    0.344472    ,    0.344108    ,    0.346552    ,    0.35033    ,    0.351701    },
    {    0.422293    ,    0.423939    ,    0.421332    ,    0.417897    ,    0.41427    ,    0.407847    ,    0.402856    ,    0.396717    ,    0.389017    ,    0.384467    ,    0.380147    ,    0.377984    ,    0.382014    ,    0.382452    ,    0.37571    ,    0.371142    ,    0.373848    ,    0.36709    ,    0.34648    ,    0.347926    ,    0.3526    ,    0.407951    ,    0.451857    ,    0.371142    ,    0.361033    ,    0.290198    ,    0.256561    ,    0.253273    ,    0.28802    ,    0.330216    ,    0.337719    ,    0.347971    ,    0.396363    ,    0.325737    ,    0.32443    ,    0.328448    ,    0.325743    ,    0.307048    ,    0.274539    ,    0.274872    ,    0.295382    ,    0.341431    ,    0.326061    ,    0.293199    ,    0.299114    ,    0.321625    ,    0.339515    ,    0.387207    ,    0.423156    ,    0.440711    ,    0.435352    ,    0.429856    ,    0.410045    ,    0.398483    ,    0.404874    ,    0.373251    ,    0.322964    ,    0.311662    ,    0.297285    ,    0.267103    ,    0.281754    ,    0.230385    ,    0.164005    ,    0.21132    ,    0.167128    ,    0.183336    ,    0.192786    ,    0.185559    ,    0.184513    ,    0.160699    ,    0.174945    ,    0.168873    ,    0.19912    ,    0.221211    ,    0.265202    ,    0.246985    ,    0.27427    ,    0.240884    ,    0.230663    ,    0.209208    ,    0.222907    ,    0.27774    ,    0.389216    ,    0.325362    ,    0.261039    ,    0.257677    ,    0.275949    ,    0.21345    ,    0.183758    ,    0.22271    ,    0.207437    ,    0.20481    ,    0.240925    ,    0.241316    ,    0.244281    ,    0.247744    ,    0.254002    ,    0.274132    ,    0.25354    ,    0.26416    ,    0.314871    ,    0.342476    ,    0.342617    ,    0.332881    ,    0.305159    ,    0.262822    ,    0.24214    ,    0.255618    ,    0.23548    ,    0.247736    ,    0.255455    ,    0.272601    ,    0.233274    ,    0.225105    ,    0.225499    ,    0.34556    ,    0.431639    ,    0.433598    ,    0.440446    ,    0.435538    ,    0.443042    ,    0.454928    ,    0.454179    ,    0.445244    ,    0.43768    ,    0.434916    ,    0.436151    ,    0.431346    },
    {    0.443642    ,    0.440307    ,    0.438072    ,    0.433358    ,    0.42694    ,    0.414552    ,    0.399778    ,    0.389093    ,    0.384746    ,    0.386958    ,    0.390206    ,    0.400052    ,    0.412347    ,    0.415193    ,    0.411662    ,    0.406875    ,    0.407613    ,    0.401221    ,    0.365879    ,    0.348541    ,    0.364551    ,    0.391023    ,    0.385114    ,    0.391525    ,    0.369733    ,    0.329206    ,    0.290087    ,    0.283024    ,    0.305078    ,    0.367487    ,    0.380851    ,    0.436449    ,    0.370994    ,    0.320176    ,    0.309527    ,    0.32492    ,    0.319445    ,    0.313618    ,    0.30851    ,    0.305579    ,    0.317354    ,    0.372914    ,    0.426071    ,    0.457429    ,    0.490629    ,    0.48146    ,    0.48026    ,    0.622567    ,    0.717696    ,    0.707814    ,    0.714584    ,    0.679303    ,    0.631461    ,    0.58166    ,    0.537416    ,    0.483291    ,    0.406489    ,    0.34232    ,    0.331997    ,    0.313684    ,    0.29667    ,    0.264337    ,    0.229074    ,    0.223031    ,    0.219413    ,    0.236501    ,    0.193242    ,    0.191185    ,    0.185124    ,    0.20249    ,    0.185952    ,    0.192842    ,    0.209617    ,    0.244703    ,    0.26067    ,    0.275139    ,    0.277433    ,    0.281634    ,    0.267027    ,    0.261325    ,    0.247578    ,    0.272102    ,    0.33422    ,    0.294139    ,    0.283013    ,    0.295795    ,    0.287473    ,    0.295085    ,    0.26771    ,    0.222186    ,    0.244144    ,    0.24247    ,    0.258545    ,    0.274324    ,    0.309032    ,    0.320915    ,    0.33731    ,    0.348007    ,    0.347595    ,    0.349418    ,    0.335118    ,    0.332261    ,    0.323544    ,    0.298198    ,    0.273341    ,    0.266353    ,    0.253374    ,    0.289262    ,    0.301589    ,    0.325228    ,    0.316713    ,    0.286475    ,    0.282983    ,    0.265458    ,    0.283969    ,    0.305521    ,    0.33707    ,    0.352743    ,    0.408027    ,    0.441964    ,    0.454292    ,    0.460541    ,    0.466134    ,    0.475274    ,    0.476943    ,    0.46974    ,    0.465891    ,    0.45761    },
    {    0.391155    ,    0.39128    ,    0.394562    ,    0.39819    ,    0.405938    ,    0.40577    ,    0.398397    ,    0.397461    ,    0.404835    ,    0.413218    ,    0.422077    ,    0.434503    ,    0.440813    ,    0.435298    ,    0.431755    ,    0.42934    ,    0.434741    ,    0.425385    ,    0.382098    ,    0.359131    ,    0.3971    ,    0.43911    ,    0.439888    ,    0.380237    ,    0.329331    ,    0.313201    ,    0.295609    ,    0.299418    ,    0.349072    ,    0.382293    ,    0.389148    ,    0.374146    ,    0.336781    ,    0.335474    ,    0.344548    ,    0.344142    ,    0.345878    ,    0.348681    ,    0.361677    ,    0.328106    ,    0.292358    ,    0.292832    ,    0.34417    ,    0.415752    ,    0.453452    ,    0.440477    ,    0.472085    ,    0.539661    ,    0.556921    ,    0.580054    ,    0.75282    ,    0.848349    ,    0.827542    ,    0.749569    ,    0.683045    ,    0.598787    ,    0.524989    ,    0.444011    ,    0.388877    ,    0.358893    ,    0.328207    ,    0.295038    ,    0.288621    ,    0.25732    ,    0.219463    ,    0.228844    ,    0.231831    ,    0.220599    ,    0.216336    ,    0.210415    ,    0.247177    ,    0.280157    ,    0.275564    ,    0.290239    ,    0.299246    ,    0.305667    ,    0.307505    ,    0.319393    ,    0.334138    ,    0.336208    ,    0.342231    ,    0.332462    ,    0.339401    ,    0.344329    ,    0.331708    ,    0.334856    ,    0.355524    ,    0.367222    ,    0.352219    ,    0.335995    ,    0.310795    ,    0.30923    ,    0.312178    ,    0.321434    ,    0.365576    ,    0.354366    ,    0.326662    ,    0.349878    ,    0.382454    ,    0.313043    ,    0.286852    ,    0.284806    ,    0.290748    ,    0.278027    ,    0.302195    ,    0.312714    ,    0.320943    ,    0.303392    ,    0.331949    ,    0.339977    ,    0.334907    ,    0.331589    ,    0.346644    ,    0.348712    ,    0.389288    ,    0.402616    ,    0.456033    ,    0.442367    ,    0.448396    ,    0.454901    ,    0.458336    ,    0.462147    ,    0.46286    ,    0.460802    ,    0.448833    ,    0.428489    ,    0.411734    ,    0.401166    },
    {    0.418878    ,    0.41825    ,    0.419534    ,    0.423818    ,    0.434125    ,    0.441207    ,    0.448753    ,    0.455849    ,    0.464087    ,    0.473744    ,    0.484366    ,    0.489339    ,    0.48496    ,    0.474398    ,    0.466924    ,    0.458346    ,    0.453538    ,    0.428026    ,    0.387484    ,    0.390598    ,    0.452325    ,    0.429145    ,    0.411821    ,    0.362797    ,    0.372675    ,    0.354256    ,    0.349119    ,    0.365973    ,    0.366343    ,    0.390871    ,    0.377228    ,    0.352572    ,    0.344332    ,    0.368993    ,    0.395422    ,    0.363062    ,    0.382259    ,    0.396455    ,    0.419674    ,    0.397721    ,    0.383018    ,    0.372291    ,    0.402676    ,    0.425936    ,    0.431584    ,    0.536443    ,    0.619701    ,    0.64951    ,    0.604008    ,    0.751619    ,    1.04562    ,    1.12891    ,    1.06136    ,    0.874025    ,    0.738671    ,    0.655322    ,    0.59591    ,    0.522706    ,    0.430255    ,    0.360776    ,    0.331168    ,    0.316253    ,    0.303319    ,    0.263701    ,    0.229433    ,    0.217701    ,    0.224673    ,    0.225745    ,    0.226849    ,    0.25971    ,    0.331104    ,    0.343348    ,    0.348786    ,    0.364822    ,    0.394655    ,    0.385655    ,    0.382092    ,    0.365842    ,    0.355699    ,    0.362832    ,    0.392764    ,    0.389697    ,    0.389142    ,    0.388114    ,    0.404604    ,    0.392211    ,    0.407572    ,    0.404717    ,    0.398706    ,    0.399176    ,    0.393073    ,    0.398497    ,    0.388336    ,    0.402807    ,    0.366252    ,    0.334419    ,    0.336269    ,    0.421333    ,    0.337638    ,    0.297722    ,    0.292276    ,    0.311477    ,    0.313719    ,    0.324113    ,    0.34696    ,    0.36909    ,    0.352664    ,    0.328513    ,    0.382493    ,    0.367019    ,    0.367661    ,    0.418531    ,    0.428332    ,    0.400733    ,    0.416214    ,    0.492565    ,    0.620643    ,    0.595503    ,    0.565917    ,    0.561422    ,    0.58868    ,    0.605476    ,    0.577543    ,    0.531235    ,    0.484427    ,    0.450312    ,    0.431318    ,    0.423443    },
    {    0.547513    ,    0.542888    ,    0.532491    ,    0.522801    ,    0.514838    ,    0.509633    ,    0.517526    ,    0.528504    ,    0.537906    ,    0.548724    ,    0.554398    ,    0.549103    ,    0.535969    ,    0.516401    ,    0.498864    ,    0.480841    ,    0.458124    ,    0.431846    ,    0.43033    ,    0.39864    ,    0.429751    ,    0.407986    ,    0.41021    ,    0.41956    ,    0.395045    ,    0.403288    ,    0.418495    ,    0.421303    ,    0.406807    ,    0.397307    ,    0.392414    ,    0.392903    ,    0.383646    ,    0.382675    ,    0.427641    ,    0.444558    ,    0.450422    ,    0.439593    ,    0.394388    ,    0.376364    ,    0.40814    ,    0.452041    ,    0.460661    ,    0.510217    ,    0.661066    ,    0.797587    ,    0.865272    ,    0.85011    ,    0.843309    ,    1.06382    ,    1.14804    ,    1.02696    ,    0.959563    ,    0.873807    ,    0.760644    ,    0.665568    ,    0.587157    ,    0.527763    ,    0.438957    ,    0.357816    ,    0.333006    ,    0.308671    ,    0.290519    ,    0.250006    ,    0.23861    ,    0.241945    ,    0.232112    ,    0.23963    ,    0.248119    ,    0.279617    ,    0.308051    ,    0.339773    ,    0.383632    ,    0.427401    ,    0.447724    ,    0.453852    ,    0.4758    ,    0.464301    ,    0.453981    ,    0.455176    ,    0.463111    ,    0.447419    ,    0.460144    ,    0.448893    ,    0.432296    ,    0.458187    ,    0.437788    ,    0.431011    ,    0.448748    ,    0.461321    ,    0.466071    ,    0.476343    ,    0.461495    ,    0.456272    ,    0.38046    ,    0.341672    ,    0.324027    ,    0.330389    ,    0.311457    ,    0.289622    ,    0.288298    ,    0.300129    ,    0.330159    ,    0.350057    ,    0.375451    ,    0.398374    ,    0.367843    ,    0.354596    ,    0.356975    ,    0.396711    ,    0.431502    ,    0.434352    ,    0.43935    ,    0.471548    ,    0.492691    ,    0.592231    ,    0.752769    ,    0.697911    ,    0.625448    ,    0.581847    ,    0.59825    ,    0.68117    ,    0.6874    ,    0.637453    ,    0.594631    ,    0.560823    ,    0.546485    ,    0.550682    },
    {    0.74152    ,    0.723674    ,    0.689897    ,    0.659749    ,    0.639341    ,    0.627139    ,    0.605591    ,    0.595573    ,    0.592709    ,    0.590913    ,    0.58748    ,    0.575196    ,    0.553923    ,    0.530774    ,    0.512337    ,    0.494593    ,    0.471567    ,    0.475842    ,    0.478347    ,    0.486254    ,    0.474646    ,    0.455399    ,    0.43473    ,    0.426809    ,    0.44881    ,    0.444023    ,    0.445973    ,    0.447862    ,    0.437463    ,    0.466533    ,    0.45031    ,    0.44117    ,    0.4499    ,    0.462001    ,    0.517458    ,    0.586574    ,    0.553906    ,    0.438686    ,    0.41703    ,    0.433119    ,    0.459526    ,    0.484324    ,    0.529868    ,    0.708812    ,    0.948019    ,    1.04569    ,    1.10244    ,    1.07307    ,    1.06233    ,    1.13372    ,    1.12896    ,    0.973575    ,    0.8232    ,    0.741686    ,    0.700788    ,    0.632868    ,    0.547381    ,    0.484281    ,    0.418832    ,    0.363739    ,    0.339358    ,    0.303928    ,    0.296127    ,    0.255039    ,    0.252287    ,    0.265531    ,    0.272466    ,    0.263612    ,    0.278692    ,    0.2933    ,    0.313064    ,    0.356378    ,    0.416166    ,    0.455626    ,    0.475725    ,    0.491232    ,    0.474257    ,    0.484092    ,    0.484177    ,    0.499478    ,    0.517254    ,    0.514223    ,    0.500605    ,    0.483835    ,    0.453906    ,    0.44074    ,    0.439012    ,    0.458356    ,    0.464425    ,    0.461875    ,    0.490129    ,    0.482272    ,    0.465414    ,    0.44978    ,    0.44021    ,    0.399638    ,    0.395888    ,    0.371027    ,    0.336189    ,    0.326442    ,    0.34182    ,    0.353884    ,    0.346308    ,    0.353192    ,    0.377319    ,    0.386049    ,    0.383035    ,    0.383585    ,    0.399214    ,    0.438318    ,    0.450016    ,    0.442096    ,    0.456307    ,    0.593641    ,    0.769883    ,    0.840708    ,    0.882967    ,    0.773618    ,    0.661957    ,    0.590222    ,    0.583823    ,    0.636506    ,    0.778463    ,    0.836736    ,    0.828511    ,    0.812857    ,    0.795794    ,    0.775221    },
    {    0.899125    ,    0.875404    ,    0.8315    ,    0.786534    ,    0.762228    ,    0.753576    ,    0.747415    ,    0.752879    ,    0.717481    ,    0.687722    ,    0.670794    ,    0.624982    ,    0.581499    ,    0.551904    ,    0.537804    ,    0.529041    ,    0.53794    ,    0.534312    ,    0.576951    ,    0.541484    ,    0.495043    ,    0.516244    ,    0.518887    ,    0.50105    ,    0.504828    ,    0.50082    ,    0.473308    ,    0.438631    ,    0.419147    ,    0.442747    ,    0.446072    ,    0.493919    ,    0.538863    ,    0.59802    ,    0.628732    ,    0.638914    ,    0.62455    ,    0.533061    ,    0.508024    ,    0.512506    ,    0.523075    ,    0.574179    ,    0.710657    ,    1.03205    ,    1.29076    ,    1.31396    ,    1.26632    ,    1.21148    ,    1.17179    ,    1.16646    ,    1.14912    ,    1.05535    ,    0.917624    ,    0.832539    ,    0.735604    ,    0.632284    ,    0.550167    ,    0.489551    ,    0.434034    ,    0.391276    ,    0.367653    ,    0.345472    ,    0.307123    ,    0.314367    ,    0.319325    ,    0.315524    ,    0.317892    ,    0.330567    ,    0.356772    ,    0.358331    ,    0.375968    ,    0.420358    ,    0.448718    ,    0.482486    ,    0.497562    ,    0.501501    ,    0.514803    ,    0.515915    ,    0.521263    ,    0.530252    ,    0.535104    ,    0.545794    ,    0.541403    ,    0.520322    ,    0.494191    ,    0.460166    ,    0.450269    ,    0.452642    ,    0.456426    ,    0.448345    ,    0.451076    ,    0.446137    ,    0.449218    ,    0.431705    ,    0.440206    ,    0.434595    ,    0.431279    ,    0.432042    ,    0.42518    ,    0.419681    ,    0.40174    ,    0.396029    ,    0.380412    ,    0.373108    ,    0.395417    ,    0.365231    ,    0.356967    ,    0.389193    ,    0.399366    ,    0.445747    ,    0.470555    ,    0.454991    ,    0.454903    ,    0.543812    ,    0.754804    ,    0.844997    ,    0.846498    ,    0.786526    ,    0.72523    ,    0.646027    ,    0.58786    ,    0.601512    ,    0.731372    ,    0.931884    ,    0.973501    ,    0.98177    ,    0.973947    ,    0.946169    },
    {    0.888439    ,    0.852053    ,    0.779347    ,    0.733616    ,    0.72443    ,    0.735495    ,    0.752331    ,    0.768511    ,    0.786309    ,    0.788044    ,    0.772548    ,    0.705795    ,    0.671339    ,    0.624781    ,    0.604155    ,    0.66321    ,    0.638228    ,    0.603219    ,    0.6003    ,    0.603494    ,    0.603452    ,    0.572117    ,    0.581106    ,    0.555851    ,    0.555913    ,    0.502672    ,    0.48235    ,    0.457391    ,    0.454973    ,    0.453613    ,    0.483731    ,    0.570266    ,    0.61705    ,    0.660334    ,    0.686517    ,    0.685128    ,    0.669105    ,    0.626908    ,    0.611057    ,    0.612011    ,    0.654453    ,    0.715371    ,    0.874783    ,    1.28365    ,    1.46151    ,    1.32162    ,    1.15387    ,    0.979197    ,    0.870728    ,    0.969777    ,    1.04326    ,    1.10101    ,    1.06094    ,    0.953536    ,    0.844716    ,    0.695922    ,    0.580115    ,    0.523533    ,    0.491739    ,    0.469298    ,    0.440028    ,    0.416252    ,    0.407828    ,    0.419201    ,    0.41272    ,    0.393238    ,    0.382617    ,    0.373953    ,    0.359841    ,    0.386817    ,    0.428841    ,    0.492248    ,    0.511236    ,    0.516876    ,    0.511923    ,    0.519831    ,    0.528565    ,    0.536471    ,    0.548912    ,    0.562257    ,    0.562863    ,    0.568293    ,    0.566931    ,    0.557787    ,    0.520638    ,    0.498656    ,    0.493671    ,    0.498254    ,    0.502916    ,    0.482582    ,    0.447261    ,    0.426739    ,    0.419665    ,    0.41576    ,    0.415731    ,    0.42462    ,    0.407323    ,    0.4047    ,    0.415934    ,    0.420423    ,    0.422471    ,    0.42659    ,    0.427069    ,    0.419437    ,    0.394865    ,    0.384488    ,    0.391664    ,    0.398565    ,    0.419599    ,    0.420965    ,    0.42584    ,    0.448623    ,    0.46873    ,    0.550423    ,    0.475122    ,    0.510592    ,    0.53026    ,    0.55767    ,    0.577579    ,    0.611327    ,    0.572261    ,    0.520875    ,    0.635948    ,    0.713607    ,    0.742115    ,    0.838903    ,    0.913108    ,    0.915298    },
    {    0.645108    ,    0.667101    ,    0.659478    ,    0.653418    ,    0.696588    ,    0.725692    ,    0.724207    ,    0.751523    ,    0.774783    ,    0.780099    ,    0.749735    ,    0.764273    ,    0.753228    ,    0.725946    ,    0.830877    ,    0.68492    ,    0.664007    ,    0.663967    ,    0.658417    ,    0.652624    ,    0.637041    ,    0.61351    ,    0.599692    ,    0.581675    ,    0.542836    ,    0.516645    ,    0.498526    ,    0.492333    ,    0.491005    ,    0.496257    ,    0.533579    ,    0.584863    ,    0.632351    ,    0.648015    ,    0.677004    ,    0.689546    ,    0.696074    ,    0.689904    ,    0.659313    ,    0.655742    ,    0.665855    ,    0.719507    ,    0.921367    ,    1.27903    ,    1.45251    ,    1.21864    ,    0.775282    ,    0.590354    ,    0.596449    ,    0.763129    ,    0.921675    ,    0.98298    ,    1.06272    ,    1.03888    ,    0.961042    ,    0.831792    ,    0.714717    ,    0.668946    ,    0.679534    ,    0.736123    ,    0.767285    ,    0.72552    ,    0.673359    ,    0.634181    ,    0.611116    ,    0.570596    ,    0.522751    ,    0.462724    ,    0.433796    ,    0.454    ,    0.488842    ,    0.571534    ,    0.576421    ,    0.5536    ,    0.544652    ,    0.555305    ,    0.556331    ,    0.565586    ,    0.561958    ,    0.564715    ,    0.572463    ,    0.579466    ,    0.584988    ,    0.570601    ,    0.544913    ,    0.528836    ,    0.515887    ,    0.4996    ,    0.487401    ,    0.492622    ,    0.48934    ,    0.472627    ,    0.454311    ,    0.441861    ,    0.423079    ,    0.418548    ,    0.417213    ,    0.421591    ,    0.423755    ,    0.430116    ,    0.444902    ,    0.447933    ,    0.453634    ,    0.450997    ,    0.457011    ,    0.443924    ,    0.436149    ,    0.426494    ,    0.440498    ,    0.441436    ,    0.435021    ,    0.437482    ,    0.447867    ,    0.462488    ,    0.453012    ,    0.490504    ,    0.504164    ,    0.493849    ,    0.514725    ,    0.551773    ,    0.559411    ,    0.590464    ,    0.624897    ,    0.627023    ,    0.63189    ,    0.6191    ,    0.62226    ,    0.609923    },
    {    0.669385    ,    0.662675    ,    0.695386    ,    0.737685    ,    0.780013    ,    0.731393    ,    0.724734    ,    0.744745    ,    0.75765    ,    0.758052    ,    0.748354    ,    0.743142    ,    0.738423    ,    0.730124    ,    0.729268    ,    0.720209    ,    0.704508    ,    0.697957    ,    0.683318    ,    0.671959    ,    0.653419    ,    0.624135    ,    0.607619    ,    0.591477    ,    0.577366    ,    0.577595    ,    0.578525    ,    0.583261    ,    0.604844    ,    0.627043    ,    0.652699    ,    0.667959    ,    0.676222    ,    0.688732    ,    0.698798    ,    0.70147    ,    0.71229    ,    0.741813    ,    0.714689    ,    0.668877    ,    0.64085    ,    0.641498    ,    0.697173    ,    0.894861    ,    1.11637    ,    0.872159    ,    0.562742    ,    0.525227    ,    0.62385    ,    0.677309    ,    0.838013    ,    0.84811    ,    0.852137    ,    0.867985    ,    0.84674    ,    0.80105    ,    0.731053    ,    0.692345    ,    0.727488    ,    0.850647    ,    1.02677    ,    1.08435    ,    1.0859    ,    1.04829    ,    0.957459    ,    0.84509    ,    0.77107    ,    0.687463    ,    0.564886    ,    0.528986    ,    0.516603    ,    0.536195    ,    0.601642    ,    0.58523    ,    0.590246    ,    0.593735    ,    0.591078    ,    0.6214    ,    0.600232    ,    0.58947    ,    0.587334    ,    0.587798    ,    0.593983    ,    0.586414    ,    0.584093    ,    0.562233    ,    0.535711    ,    0.517415    ,    0.526925    ,    0.524757    ,    0.50503    ,    0.500225    ,    0.487136    ,    0.476926    ,    0.466835    ,    0.456542    ,    0.44928    ,    0.436526    ,    0.438389    ,    0.44841    ,    0.463139    ,    0.471635    ,    0.487481    ,    0.491111    ,    0.484242    ,    0.486332    ,    0.47609    ,    0.470785    ,    0.462808    ,    0.468088    ,    0.459507    ,    0.458054    ,    0.455433    ,    0.47819    ,    0.465362    ,    0.474194    ,    0.468036    ,    0.494418    ,    0.51939    ,    0.549499    ,    0.591119    ,    0.623926    ,    0.616319    ,    0.612852    ,    0.615919    ,    0.650016    ,    0.661899    ,    0.666055    },
    {    0.779872    ,    0.795197    ,    0.855307    ,    0.907928    ,    0.93122    ,    0.862801    ,    0.795294    ,    0.775171    ,    0.773905    ,    0.772914    ,    0.766382    ,    0.76479    ,    0.752562    ,    0.74121    ,    0.738173    ,    0.738675    ,    0.713637    ,    0.695259    ,    0.664289    ,    0.653799    ,    0.650064    ,    0.63802    ,    0.636135    ,    0.634988    ,    0.637869    ,    0.643359    ,    0.648761    ,    0.698097    ,    0.715858    ,    0.72616    ,    0.694195    ,    0.694156    ,    0.725335    ,    0.743624    ,    0.734371    ,    0.721697    ,    0.704547    ,    0.730737    ,    0.700868    ,    0.657625    ,    0.604006    ,    0.572536    ,    0.595753    ,    0.673583    ,    0.742735    ,    0.591963    ,    0.540287    ,    0.469909    ,    0.485788    ,    0.482809    ,    0.483722    ,    0.469367    ,    0.463412    ,    0.447314    ,    0.487865    ,    0.605957    ,    0.745366    ,    0.837355    ,    0.908249    ,    1.04342    ,    1.17806    ,    1.22225    ,    1.27048    ,    1.32371    ,    1.26284    ,    1.14261    ,    1.03667    ,    0.921683    ,    0.770629    ,    0.634858    ,    0.579242    ,    0.574462    ,    0.593237    ,    0.611163    ,    0.628063    ,    0.645126    ,    0.672252    ,    0.672662    ,    0.693673    ,    0.686895    ,    0.654163    ,    0.674031    ,    0.676307    ,    0.671074    ,    0.658078    ,    0.641281    ,    0.598569    ,    0.584102    ,    0.565692    ,    0.554461    ,    0.553477    ,    0.543273    ,    0.517351    ,    0.511014    ,    0.500227    ,    0.488557    ,    0.48375    ,    0.45865    ,    0.457448    ,    0.471732    ,    0.505098    ,    0.536481    ,    0.550796    ,    0.539746    ,    0.537048    ,    0.525142    ,    0.523    ,    0.514008    ,    0.506866    ,    0.520635    ,    0.489435    ,    0.497274    ,    0.516931    ,    0.514543    ,    0.490038    ,    0.501387    ,    0.528487    ,    0.547269    ,    0.556129    ,    0.60863    ,    0.624986    ,    0.634955    ,    0.651963    ,    0.661252    ,    0.678655    ,    0.700852    ,    0.720727    ,    0.748677    },
    {    0.840957    ,    0.859925    ,    0.915706    ,    0.955512    ,    0.973266    ,    0.975427    ,    0.96688    ,    0.949992    ,    0.91421    ,    0.91206    ,    0.910173    ,    0.907113    ,    0.894619    ,    0.868719    ,    0.850724    ,    0.837359    ,    0.821334    ,    0.794836    ,    0.765335    ,    0.744752    ,    0.703316    ,    0.672607    ,    0.665654    ,    0.658767    ,    0.658797    ,    0.669021    ,    0.688149    ,    0.754586    ,    0.816242    ,    0.783397    ,    0.74762    ,    0.77876    ,    0.840245    ,    0.784552    ,    0.759704    ,    0.713419    ,    0.692892    ,    0.669481    ,    0.676092    ,    0.599441    ,    0.580356    ,    0.561823    ,    0.572839    ,    0.594689    ,    0.561707    ,    0.422174    ,    0.404386    ,    0.39234    ,    0.401954    ,    0.437784    ,    0.481575    ,    0.471233    ,    0.416661    ,    0.372045    ,    0.367033    ,    0.390578    ,    0.502791    ,    0.729425    ,    0.898579    ,    1.05735    ,    1.20796    ,    1.21425    ,    1.24191    ,    1.28733    ,    1.27529    ,    1.20637    ,    1.13369    ,    1.0327    ,    0.895004    ,    0.807041    ,    0.785215    ,    0.831815    ,    0.936378    ,    1.04627    ,    1.05029    ,    1.03051    ,    1.0514    ,    1.01934    ,    0.975819    ,    0.931228    ,    0.895578    ,    0.88796    ,    0.854825    ,    0.842168    ,    0.888366    ,    0.929503    ,    0.948732    ,    0.89987    ,    0.764782    ,    0.646034    ,    0.607661    ,    0.592826    ,    0.586504    ,    0.584474    ,    0.579465    ,    0.577247    ,    0.59191    ,    0.577896    ,    0.580032    ,    0.585208    ,    0.585776    ,    0.575663    ,    0.559643    ,    0.554791    ,    0.559223    ,    0.548358    ,    0.541286    ,    0.53076    ,    0.530231    ,    0.533163    ,    0.50012    ,    0.524593    ,    0.529279    ,    0.557186    ,    0.554082    ,    0.572371    ,    0.582871    ,    0.604104    ,    0.643202    ,    0.675632    ,    0.707463    ,    0.729136    ,    0.751278    ,    0.764853    ,    0.777212    ,    0.792618    ,    0.81311    ,    0.825265    },
    {    0.809569    ,    0.816473    ,    0.824481    ,    0.835767    ,    0.855118    ,    0.872932    ,    0.879315    ,    0.874953    ,    0.862126    ,    0.84529    ,    0.832553    ,    0.815007    ,    0.790616    ,    0.756464    ,    0.720974    ,    0.695131    ,    0.683613    ,    0.687931    ,    0.709377    ,    0.739253    ,    0.728316    ,    0.710048    ,    0.720104    ,    0.711949    ,    0.715726    ,    0.711811    ,    0.745452    ,    0.768276    ,    0.796599    ,    0.797694    ,    0.792628    ,    0.794771    ,    0.792725    ,    0.733604    ,    0.705651    ,    0.638687    ,    0.619031    ,    0.594162    ,    0.590513    ,    0.586239    ,    0.576252    ,    0.570849    ,    0.588777    ,    0.583423    ,    0.490459    ,    0.377612    ,    0.384752    ,    0.39688    ,    0.409741    ,    0.415778    ,    0.414549    ,    0.396521    ,    0.354729    ,    0.325775    ,    0.316369    ,    0.344516    ,    0.389896    ,    0.497426    ,    0.646078    ,    0.832075    ,    1.04142    ,    1.17813    ,    1.23613    ,    1.29324    ,    1.33585    ,    1.30242    ,    1.22364    ,    1.13986    ,    1.05199    ,    1.0064    ,    1.02989    ,    1.0742    ,    1.14904    ,    1.21459    ,    1.21065    ,    1.16657    ,    1.13374    ,    1.13492    ,    1.147    ,    1.15195    ,    1.15894    ,    1.11519    ,    1.02536    ,    0.970265    ,    1.0081    ,    1.01604    ,    0.991997    ,    0.953609    ,    0.889369    ,    0.830246    ,    0.793518    ,    0.76999    ,    0.756221    ,    0.721385    ,    0.685428    ,    0.652846    ,    0.641548    ,    0.644811    ,    0.649602    ,    0.659759    ,    0.670479    ,    0.659969    ,    0.650922    ,    0.632447    ,    0.620561    ,    0.617057    ,    0.619387    ,    0.615132    ,    0.616125    ,    0.600117    ,    0.5908    ,    0.594062    ,    0.587125    ,    0.582634    ,    0.605028    ,    0.596343    ,    0.633211    ,    0.65771    ,    0.677152    ,    0.692027    ,    0.704257    ,    0.721453    ,    0.733998    ,    0.7418    ,    0.755719    ,    0.766143    ,    0.77752    ,    0.791363    },
    {    0.752197    ,    0.754921    ,    0.762196    ,    0.770295    ,    0.779173    ,    0.785934    ,    0.787602    ,    0.781973    ,    0.76997    ,    0.75332    ,    0.737563    ,    0.721069    ,    0.702617    ,    0.684669    ,    0.665814    ,    0.6479    ,    0.636854    ,    0.629737    ,    0.629484    ,    0.637426    ,    0.65133    ,    0.649877    ,    0.661861    ,    0.705807    ,    0.715571    ,    0.716605    ,    0.755841    ,    0.730194    ,    0.719805    ,    0.702272    ,    0.685996    ,    0.67232    ,    0.681397    ,    0.690028    ,    0.575855    ,    0.554636    ,    0.572827    ,    0.52147    ,    0.52103    ,    0.527642    ,    0.510199    ,    0.458976    ,    0.429855    ,    0.400826    ,    0.37444    ,    0.357062    ,    0.361219    ,    0.36053    ,    0.349761    ,    0.363708    ,    0.366385    ,    0.340832    ,    0.334212    ,    0.305432    ,    0.27972    ,    0.285418    ,    0.309405    ,    0.450586    ,    0.57976    ,    0.679636    ,    0.778838    ,    0.911535    ,    1.06646    ,    1.22474    ,    1.36917    ,    1.47428    ,    1.47098    ,    1.34253    ,    1.17807    ,    1.05828    ,    1.06022    ,    1.10096    ,    1.08681    ,    1.11464    ,    1.17216    ,    1.21778    ,    1.24012    ,    1.274    ,    1.32625    ,    1.39703    ,    1.44546    ,    1.45014    ,    1.42293    ,    1.38581    ,    1.37422    ,    1.37947    ,    1.37359    ,    1.34881    ,    1.24293    ,    1.15727    ,    1.0552    ,    0.952027    ,    0.883112    ,    0.836202    ,    0.802126    ,    0.770086    ,    0.726829    ,    0.703775    ,    0.693685    ,    0.693175    ,    0.709565    ,    0.710461    ,    0.695109    ,    0.686895    ,    0.667356    ,    0.668441    ,    0.666857    ,    0.661564    ,    0.651796    ,    0.646068    ,    0.637764    ,    0.630735    ,    0.622223    ,    0.612088    ,    0.61722    ,    0.630665    ,    0.639465    ,    0.647691    ,    0.654162    ,    0.660336    ,    0.667673    ,    0.679885    ,    0.692375    ,    0.701205    ,    0.713467    ,    0.725741    ,    0.733466    ,    0.741946    },
    {    0.755921    ,    0.75843    ,    0.760195    ,    0.761296    ,    0.760519    ,    0.76074    ,    0.759373    ,    0.756956    ,    0.754198    ,    0.7465    ,    0.736861    ,    0.726724    ,    0.717995    ,    0.705132    ,    0.695036    ,    0.684299    ,    0.676073    ,    0.667524    ,    0.660515    ,    0.653404    ,    0.647525    ,    0.643296    ,    0.64436    ,    0.644559    ,    0.642928    ,    0.652163    ,    0.654609    ,    0.678121    ,    0.68017    ,    0.66487    ,    0.639677    ,    0.602343    ,    0.558171    ,    0.510679    ,    0.51245    ,    0.461991    ,    0.447749    ,    0.455584    ,    0.465949    ,    0.484165    ,    0.423303    ,    0.399963    ,    0.357519    ,    0.342267    ,    0.326361    ,    0.31042    ,    0.306251    ,    0.291946    ,    0.294145    ,    0.306455    ,    0.303168    ,    0.297453    ,    0.285547    ,    0.274629    ,    0.280306    ,    0.289192    ,    0.30262    ,    0.409709    ,    0.515306    ,    0.567249    ,    0.602958    ,    0.635294    ,    0.677164    ,    0.73274    ,    0.829938    ,    0.966987    ,    1.03511    ,    0.987781    ,    0.930941    ,    0.863023    ,    0.851477    ,    0.883915    ,    0.960885    ,    0.972466    ,    1.00154    ,    1.02761    ,    1.06421    ,    1.11126    ,    1.1757    ,    1.23527    ,    1.28867    ,    1.31765    ,    1.33734    ,    1.3397    ,    1.34292    ,    1.34732    ,    1.34724    ,    1.34024    ,    1.32808    ,    1.30235    ,    1.25247    ,    1.16919    ,    1.0939    ,    1.05013    ,    1.02912    ,    0.979587    ,    0.889831    ,    0.875365    ,    0.864689    ,    0.892618    ,    0.889638    ,    0.845122    ,    0.823315    ,    0.80825    ,    0.787445    ,    0.766233    ,    0.746528    ,    0.730231    ,    0.717598    ,    0.703972    ,    0.690577    ,    0.676429    ,    0.664264    ,    0.652829    ,    0.645862    ,    0.644273    ,    0.642278    ,    0.646861    ,    0.649744    ,    0.65554    ,    0.664234    ,    0.67189    ,    0.680776    ,    0.690507    ,    0.702136    ,    0.717063    ,    0.730421    ,    0.741951    },
    {    0.751902    ,    0.752461    ,    0.754502    ,    0.754048    ,    0.753014    ,    0.754247    ,    0.75363    ,    0.753393    ,    0.750506    ,    0.747021    ,    0.741517    ,    0.735548    ,    0.729225    ,    0.722405    ,    0.714019    ,    0.707212    ,    0.701131    ,    0.693941    ,    0.687898    ,    0.681919    ,    0.678739    ,    0.678326    ,    0.67636    ,    0.671829    ,    0.666834    ,    0.661347    ,    0.658485    ,    0.657248    ,    0.655464    ,    0.652244    ,    0.65515    ,    0.640329    ,    0.614623    ,    0.602632    ,    0.588008    ,    0.586577    ,    0.584072    ,    0.562049    ,    0.522248    ,    0.508709    ,    0.49561    ,    0.490155    ,    0.495634    ,    0.509797    ,    0.529029    ,    0.49405    ,    0.460099    ,    0.442105    ,    0.417562    ,    0.412459    ,    0.407047    ,    0.416284    ,    0.443513    ,    0.450534    ,    0.468214    ,    0.511168    ,    0.564222    ,    0.573864    ,    0.581167    ,    0.605644    ,    0.608144    ,    0.621131    ,    0.632498    ,    0.643386    ,    0.653008    ,    0.663464    ,    0.673755    ,    0.697122    ,    0.729876    ,    0.754785    ,    0.782891    ,    0.815124    ,    0.847578    ,    0.883589    ,    0.918653    ,    0.946141    ,    0.966481    ,    0.992578    ,    1.01734    ,    1.03738    ,    1.05812    ,    1.06923    ,    1.07977    ,    1.08663    ,    1.10645    ,    1.12356    ,    1.13738    ,    1.15024    ,    1.16361    ,    1.1572    ,    1.15191    ,    1.13307    ,    1.10989    ,    1.08344    ,    1.05822    ,    1.02845    ,    0.988876    ,    0.957589    ,    0.929582    ,    0.916054    ,    0.902534    ,    0.889792    ,    0.877278    ,    0.86863    ,    0.85826    ,    0.839849    ,    0.82116    ,    0.806155    ,    0.790389    ,    0.777496    ,    0.764625    ,    0.754211    ,    0.743205    ,    0.732424    ,    0.722709    ,    0.715621    ,    0.711113    ,    0.707107    ,    0.704668    ,    0.704619    ,    0.706881    ,    0.70842    ,    0.714103    ,    0.719038    ,    0.7243    ,    0.730911    ,    0.738319    ,    0.744975    },
    {    0.744679    ,    0.743712    ,    0.745833    ,    0.746818    ,    0.74792    ,    0.748072    ,    0.746941    ,    0.745113    ,    0.741871    ,    0.741677    ,    0.741208    ,    0.738791    ,    0.733686    ,    0.730192    ,    0.728978    ,    0.725143    ,    0.721736    ,    0.718816    ,    0.716202    ,    0.715243    ,    0.714498    ,    0.713164    ,    0.713255    ,    0.712633    ,    0.711767    ,    0.711313    ,    0.710702    ,    0.70802    ,    0.707253    ,    0.719495    ,    0.719725    ,    0.712608    ,    0.698565    ,    0.688915    ,    0.681992    ,    0.681206    ,    0.676902    ,    0.668219    ,    0.656858    ,    0.652328    ,    0.642753    ,    0.639116    ,    0.640014    ,    0.649519    ,    0.653647    ,    0.650822    ,    0.646978    ,    0.648156    ,    0.646606    ,    0.640525    ,    0.638704    ,    0.638506    ,    0.632529    ,    0.62926    ,    0.629142    ,    0.627021    ,    0.635916    ,    0.640133    ,    0.64751    ,    0.654344    ,    0.662072    ,    0.671738    ,    0.682033    ,    0.691301    ,    0.70025    ,    0.707697    ,    0.71573    ,    0.726516    ,    0.738939    ,    0.750804    ,    0.763602    ,    0.775117    ,    0.789431    ,    0.803447    ,    0.818561    ,    0.831735    ,    0.845437    ,    0.856478    ,    0.865509    ,    0.873141    ,    0.88108    ,    0.886687    ,    0.891039    ,    0.894467    ,    0.895027    ,    0.895317    ,    0.894809    ,    0.89398    ,    0.891718    ,    0.888617    ,    0.885381    ,    0.879832    ,    0.875982    ,    0.871051    ,    0.865357    ,    0.857817    ,    0.850836    ,    0.845445    ,    0.839105    ,    0.835261    ,    0.829918    ,    0.822511    ,    0.818444    ,    0.814836    ,    0.812363    ,    0.807316    ,    0.80283    ,    0.798768    ,    0.793928    ,    0.790194    ,    0.783436    ,    0.775361    ,    0.769834    ,    0.76375    ,    0.75614    ,    0.750716    ,    0.747466    ,    0.745117    ,    0.742831    ,    0.74135    ,    0.740989    ,    0.740993    ,    0.741193    ,    0.739089    ,    0.739116    ,    0.740285    ,    0.742177    ,    0.742164    },
    {    0.722287    ,    0.722399    ,    0.72276    ,    0.723869    ,    0.724024    ,    0.7245    ,    0.724581    ,    0.725378    ,    0.726262    ,    0.726143    ,    0.724977    ,    0.723348    ,    0.72153    ,    0.719914    ,    0.718825    ,    0.717682    ,    0.71698    ,    0.716586    ,    0.716581    ,    0.71631    ,    0.716184    ,    0.715611    ,    0.71519    ,    0.715036    ,    0.71547    ,    0.714966    ,    0.715076    ,    0.715457    ,    0.717322    ,    0.729803    ,    0.731053    ,    0.730738    ,    0.730943    ,    0.72857    ,    0.727675    ,    0.726714    ,    0.724625    ,    0.720377    ,    0.719394    ,    0.71586    ,    0.714582    ,    0.713382    ,    0.713563    ,    0.713184    ,    0.709137    ,    0.707674    ,    0.707144    ,    0.704391    ,    0.703971    ,    0.703708    ,    0.703794    ,    0.702289    ,    0.70179    ,    0.700665    ,    0.70197    ,    0.701881    ,    0.701935    ,    0.703909    ,    0.707532    ,    0.711359    ,    0.712687    ,    0.711413    ,    0.714636    ,    0.717467    ,    0.721694    ,    0.723884    ,    0.726348    ,    0.729601    ,    0.73184    ,    0.735228    ,    0.73661    ,    0.738986    ,    0.741351    ,    0.74348    ,    0.744654    ,    0.746618    ,    0.74769    ,    0.748668    ,    0.74935    ,    0.749749    ,    0.749906    ,    0.750248    ,    0.750625    ,    0.751398    ,    0.750992    ,    0.750342    ,    0.750264    ,    0.749012    ,    0.748654    ,    0.748381    ,    0.748051    ,    0.747495    ,    0.747241    ,    0.748158    ,    0.747668    ,    0.746626    ,    0.745368    ,    0.745359    ,    0.74442    ,    0.742943    ,    0.741644    ,    0.740069    ,    0.740347    ,    0.739151    ,    0.738366    ,    0.737829    ,    0.736969    ,    0.735899    ,    0.735684    ,    0.734661    ,    0.733213    ,    0.732183    ,    0.731297    ,    0.730724    ,    0.730165    ,    0.728567    ,    0.728167    ,    0.727714    ,    0.727334    ,    0.72654    ,    0.724948    ,    0.724473    ,    0.723105    ,    0.721944    ,    0.720558    ,    0.72177    ,    0.722216    ,    0.722142    }
};

double LGRTC_RCP26_peak[64][128] = {    {    1.43306     ,    1.42189     ,    1.40933     ,    1.39252     ,    1.37883     ,    1.36406     ,    1.34821     ,    1.33332     ,    1.32598     ,    1.31797     ,    1.30872     ,    1.30556     ,    1.29878     ,    1.29158     ,    1.28548     ,    1.27568     ,    1.27024     ,    1.26825     ,    1.2673     ,    1.26539     ,    1.26436     ,    1.26281     ,    1.26531     ,    1.2646     ,    1.26648     ,    1.26584     ,    1.27338     ,    1.27399     ,    1.27717     ,    1.27758     ,    1.26738     ,    1.26879     ,    1.2619     ,    1.25215     ,    1.24256     ,    1.24011     ,    1.23992     ,    1.23812     ,    1.23717     ,    1.23741     ,    1.23895     ,    1.24213     ,    1.24561     ,    1.24949     ,    1.25343     ,    1.25846     ,    1.26424     ,    1.27024     ,    1.2771     ,    1.2846     ,    1.2921     ,    1.29993     ,    1.30896     ,    1.31896     ,    1.32905     ,    1.33951     ,    1.34076     ,    1.35159     ,    1.37358     ,    1.38408     ,    1.3794     ,    1.39804     ,    1.41514     ,    1.41693     ,    1.42247     ,    1.42709     ,    1.43141     ,    1.43477     ,    1.43832     ,    1.44234     ,    1.44693     ,    1.45214     ,    1.45751     ,    1.46234     ,    1.46717     ,    1.47133     ,    1.47591     ,    1.48013     ,    1.48287     ,    1.4829     ,    1.48092     ,    1.47727     ,    1.47129     ,    1.4623     ,    1.45197     ,    1.44111     ,    1.43011     ,    1.42043     ,    1.41088     ,    1.40108     ,    1.39128     ,    1.38268     ,    1.37603     ,    1.37311     ,    1.37233     ,    1.37405     ,    1.37801     ,    1.38369     ,    1.38819     ,    1.3926     ,    1.39823     ,    1.40415     ,    1.41269     ,    1.42216     ,    1.43151     ,    1.44049     ,    1.44928     ,    1.45813     ,    1.46673     ,    1.47809     ,    1.48813     ,    1.49674     ,    1.5045     ,    1.50894     ,    1.51207     ,    1.51221     ,    1.50936     ,    1.50411     ,    1.49812     ,    1.49285     ,    1.48712     ,    1.48092     ,    1.47481     ,    1.4697     ,    1.46338     ,    1.45571     ,    1.44707     ,    1.43321    },
    {    0.93901     ,    0.93392     ,    0.93111     ,    0.92599     ,    0.92445     ,    0.9262     ,    0.93081     ,    0.94342     ,    0.96016     ,    0.97616     ,    0.98993     ,    1.00644     ,    1.02332     ,    1.0404     ,    1.05747     ,    1.06969     ,    1.08104     ,    1.09457     ,    1.10837     ,    1.12109     ,    1.13426     ,    1.14723     ,    1.16347     ,    1.18343     ,    1.21813     ,    1.27435     ,    1.34778     ,    1.39954     ,    1.46005     ,    1.51662     ,    1.55046     ,    1.53109     ,    1.4939     ,    1.44355     ,    1.39361     ,    1.34759     ,    1.30808     ,    1.27832     ,    1.2509     ,    1.2275     ,    1.20882     ,    1.19572     ,    1.1852     ,    1.181     ,    1.17947     ,    1.17791     ,    1.17689     ,    1.17731     ,    1.18357     ,    1.19646     ,    1.21726     ,    1.2416     ,    1.27033     ,    1.30501     ,    1.34272     ,    1.38143     ,    1.42133     ,    1.46076     ,    1.49673     ,    1.52891     ,    1.54404     ,    1.56338     ,    1.57646     ,    1.57767     ,    1.56985     ,    1.5513     ,    1.5285     ,    1.51241     ,    1.4961     ,    1.48414     ,    1.4764     ,    1.4582     ,    1.43995     ,    1.41929     ,    1.41538     ,    1.417     ,    1.42196     ,    1.43482     ,    1.44654     ,    1.45899     ,    1.46987     ,    1.47925     ,    1.48684     ,    1.4904     ,    1.48745     ,    1.47122     ,    1.44459     ,    1.40573     ,    1.35933     ,    1.3089     ,    1.26101     ,    1.21937     ,    1.18754     ,    1.16984     ,    1.15895     ,    1.16641     ,    1.18686     ,    1.1838     ,    1.16037     ,    1.2916     ,    1.31438     ,    1.34138     ,    1.36116     ,    1.3785     ,    1.38971     ,    1.3877     ,    1.38467     ,    1.38064     ,    1.37591     ,    1.37154     ,    1.3593     ,    1.33351     ,    1.30191     ,    1.26129     ,    1.2194     ,    1.18054     ,    1.12741     ,    1.108     ,    1.06624     ,    1.04231     ,    1.0113     ,    0.98928     ,    0.97105     ,    0.95418     ,    0.94112     ,    0.94109     ,    0.94285     ,    0.94137    },
    {    0.70323     ,    0.72589     ,    0.73633     ,    0.76366     ,    0.84734     ,    0.9394     ,    0.9733     ,    1.02014     ,    1.0525     ,    1.06694     ,    1.08889     ,    1.08296     ,    1.08947     ,    1.10009     ,    1.10701     ,    1.12236     ,    1.15691     ,    1.17362     ,    1.20041     ,    1.17875     ,    1.15142     ,    1.12213     ,    1.09456     ,    1.07806     ,    1.06691     ,    1.0739     ,    1.10755     ,    1.09072     ,    1.206     ,    1.29685     ,    1.38794     ,    1.43133     ,    1.43192     ,    1.38685     ,    1.33723     ,    1.3141     ,    1.31023     ,    1.32096     ,    1.33572     ,    1.33427     ,    1.33017     ,    1.33267     ,    1.34939     ,    1.36486     ,    1.37582     ,    1.34708     ,    1.29865     ,    1.23528     ,    1.19226     ,    1.15816     ,    1.12395     ,    1.09597     ,    1.07416     ,    1.12295     ,    1.18688     ,    1.25675     ,    1.33575     ,    1.41905     ,    1.5342     ,    1.62069     ,    1.69718     ,    1.74912     ,    1.77672     ,    1.77852     ,    1.75789     ,    1.72397     ,    1.6461     ,    1.5333     ,    1.50753     ,    1.4422     ,    1.39658     ,    1.38023     ,    1.37901     ,    1.40976     ,    1.45591     ,    1.509     ,    1.56407     ,    1.62347     ,    1.68966     ,    1.73886     ,    1.76529     ,    1.75798     ,    1.72856     ,    1.6848     ,    1.63813     ,    1.60307     ,    1.58713     ,    1.57347     ,    1.54258     ,    1.50287     ,    1.45659     ,    1.41072     ,    1.37213     ,    1.34246     ,    1.32098     ,    1.30108     ,    1.27981     ,    1.26107     ,    1.24078     ,    1.21042     ,    1.1701     ,    1.144     ,    1.10044     ,    1.06256     ,    1.02949     ,    1.01104     ,    1.00348     ,    0.99545     ,    0.97981     ,    0.95009     ,    0.90787     ,    0.8603     ,    0.79044     ,    0.77576     ,    0.75336     ,    0.74689     ,    0.75316     ,    0.76866     ,    0.79133     ,    0.81466     ,    0.8414     ,    0.87316     ,    0.84599     ,    0.80238     ,    0.69279     ,    0.65164     ,    0.64911     ,    0.67824    },
    {    0.16619     ,    0.19703     ,    0.26306     ,    0.36008     ,    0.54389     ,    0.74962     ,    0.91461     ,    0.97329     ,    1.00958     ,    1.02136     ,    1.0208     ,    1.05707     ,    1.1357     ,    1.19686     ,    1.24603     ,    1.2602     ,    1.25478     ,    1.24807     ,    1.23834     ,    1.19968     ,    1.14619     ,    1.09151     ,    1.05095     ,    1.02188     ,    0.98906     ,    0.97922     ,    0.99569     ,    1.0293     ,    1.06303     ,    1.1073     ,    1.15879     ,    1.20036     ,    1.21107     ,    1.17483     ,    1.13884     ,    1.13482     ,    1.20703     ,    1.35885     ,    1.39614     ,    1.48302     ,    1.52638     ,    1.53403     ,    1.51082     ,    1.47649     ,    1.45278     ,    1.42515     ,    1.41703     ,    1.38908     ,    1.36174     ,    1.30529     ,    1.24258     ,    1.2086     ,    1.13934     ,    1.18633     ,    1.18477     ,    1.20061     ,    1.26864     ,    1.33829     ,    1.4436     ,    1.57703     ,    1.71077     ,    1.8667     ,    1.96457     ,    1.99622     ,    1.97485     ,    1.90989     ,    1.81487     ,    1.69101     ,    1.60415     ,    1.54224     ,    1.50782     ,    1.50933     ,    1.53298     ,    1.57776     ,    1.61557     ,    1.64159     ,    1.68951     ,    1.75297     ,    1.82199     ,    1.88834     ,    1.94045     ,    1.95317     ,    1.90678     ,    1.81206     ,    1.707     ,    1.62945     ,    1.57824     ,    1.54524     ,    1.50325     ,    1.45893     ,    1.43641     ,    1.44497     ,    1.47755     ,    1.51814     ,    1.52611     ,    1.52043     ,    1.51515     ,    1.51642     ,    1.49936     ,    1.41178     ,    1.23833     ,    1.11515     ,    1.02741     ,    0.99147     ,    0.99013     ,    1.02297     ,    1.0656     ,    1.0779     ,    1.06618     ,    1.02668     ,    0.98269     ,    0.93406     ,    0.87501     ,    0.81733     ,    0.7722     ,    0.74799     ,    0.7484     ,    0.77138     ,    0.81335     ,    0.85256     ,    0.86437     ,    0.92955     ,    0.78089     ,    0.71031     ,    0.55968     ,    0.36142     ,    0.18765     ,    0.16733    },
    {    0.00133     ,    0.14368     ,    0.29883     ,    0.51407     ,    0.73922     ,    0.93993     ,    1.14045     ,    1.22079     ,    1.23011     ,    1.35408     ,    1.36598     ,    1.38662     ,    1.44428     ,    1.41506     ,    1.42052     ,    1.43065     ,    1.39983     ,    1.34105     ,    1.27987     ,    1.22113     ,    1.16634     ,    1.1114     ,    1.06468     ,    1.04027     ,    1.0128     ,    0.98489     ,    0.94975     ,    0.92399     ,    0.90736     ,    0.89464     ,    0.90318     ,    0.9113     ,    0.92643     ,    0.93994     ,    0.94214     ,    0.95182     ,    0.96199     ,    0.98194     ,    1.029     ,    1.10374     ,    1.13184     ,    1.1773     ,    1.05233     ,    0.98339     ,    0.96528     ,    1.02844     ,    1.02579     ,    1.01304     ,    0.99308     ,    0.94374     ,    0.86486     ,    0.90419     ,    1.10981     ,    1.07262     ,    1.12877     ,    1.1385     ,    1.16026     ,    1.22834     ,    1.29676     ,    1.4059     ,    1.51313     ,    1.62021     ,    1.72014     ,    1.83425     ,    1.86022     ,    1.80892     ,    1.66099     ,    1.61943     ,    1.56368     ,    1.5997     ,    1.66378     ,    1.70995     ,    1.72462     ,    1.71571     ,    1.75855     ,    1.79744     ,    1.93873     ,    1.9968     ,    2.0369     ,    2.00211     ,    1.96237     ,    1.9081     ,    1.84276     ,    1.75556     ,    1.6549     ,    1.5688     ,    1.51511     ,    1.49125     ,    1.48777     ,    1.50026     ,    1.5237     ,    1.56728     ,    1.64349     ,    1.70218     ,    1.75348     ,    1.7654     ,    1.75113     ,    1.7035     ,    1.67181     ,    1.60121     ,    1.53009     ,    1.44496     ,    1.402     ,    1.39476     ,    1.386     ,    1.36039     ,    1.32028     ,    1.26407     ,    1.19946     ,    1.13373     ,    0.972     ,    0.96368     ,    0.85286     ,    0.97375     ,    1.05486     ,    1.09967     ,    1.11625     ,    1.10636     ,    1.08895     ,    1.06032     ,    0.9971     ,    0.91175     ,    0.91331     ,    0.71283     ,    0.46707     ,    0.17578     ,    -0.00098     ,    -0.04774    },
    {    0.49508     ,    0.62929     ,    0.73244     ,    0.86581     ,    1.03599     ,    1.1962     ,    1.33694     ,    1.47256     ,    1.5843     ,    1.68895     ,    1.75729     ,    1.81381     ,    1.83962     ,    1.81714     ,    1.85423     ,    1.8721     ,    1.87496     ,    1.83229     ,    1.75899     ,    1.65328     ,    1.52795     ,    1.43244     ,    1.3569     ,    1.30337     ,    1.22973     ,    1.10861     ,    1.02276     ,    0.85203     ,    0.73294     ,    0.67581     ,    0.63758     ,    0.58688     ,    0.52798     ,    0.49336     ,    0.45226     ,    0.44149     ,    0.45272     ,    0.49085     ,    0.55879     ,    0.64172     ,    0.70575     ,    0.74764     ,    0.75689     ,    0.60376     ,    0.55491     ,    0.54998     ,    0.48094     ,    0.44709     ,    0.47032     ,    0.52776     ,    0.55826     ,    0.65782     ,    0.7635     ,    0.8154     ,    0.9001     ,    1.00304     ,    1.15437     ,    1.234     ,    1.27937     ,    1.36521     ,    1.30677     ,    1.25747     ,    1.32187     ,    1.4057     ,    1.48153     ,    1.49289     ,    1.46943     ,    1.53524     ,    1.59913     ,    1.69994     ,    1.7753     ,    1.80629     ,    1.78859     ,    1.75642     ,    1.75679     ,    1.74649     ,    1.79165     ,    1.8168     ,    1.86224     ,    1.91004     ,    1.97223     ,    1.94275     ,    1.82911     ,    1.73372     ,    1.7117     ,    1.71527     ,    1.72946     ,    1.77431     ,    1.76032     ,    1.70204     ,    1.69307     ,    1.61684     ,    1.53247     ,    1.46602     ,    1.48498     ,    1.46526     ,    1.58415     ,    1.45564     ,    1.50229     ,    1.4687     ,    1.4524     ,    1.44927     ,    1.4528     ,    1.47191     ,    1.48759     ,    1.48494     ,    1.4594     ,    1.37369     ,    1.27078     ,    1.1773     ,    1.1129     ,    1.07379     ,    1.06712     ,    1.10772     ,    1.17177     ,    1.26232     ,    1.30152     ,    1.32063     ,    1.35802     ,    1.3125     ,    1.21024     ,    1.02763     ,    0.82864     ,    0.69029     ,    0.51142     ,    0.35518     ,    0.29208     ,    0.36004    },
    {    1.06797     ,    1.16045     ,    1.22993     ,    1.30756     ,    1.42588     ,    1.60174     ,    1.84395     ,    2.13242     ,    2.41701     ,    2.66059     ,    2.86303     ,    2.9996     ,    3.05687     ,    3.02308     ,    2.95473     ,    2.83142     ,    2.70901     ,    2.58208     ,    2.49931     ,    2.45015     ,    2.39215     ,    2.32064     ,    2.22941     ,    2.10115     ,    1.97857     ,    1.83106     ,    1.68866     ,    1.45703     ,    1.23754     ,    1.07189     ,    0.94961     ,    0.84967     ,    0.68553     ,    0.4897     ,    0.30682     ,    0.12996     ,    0.03246     ,    0.03043     ,    0.10081     ,    0.15868     ,    0.25137     ,    0.3357     ,    0.3231     ,    0.27609     ,    0.32275     ,    0.35877     ,    0.32755     ,    0.29637     ,    0.2688     ,    0.26824     ,    0.31443     ,    0.40996     ,    0.50555     ,    0.5716     ,    0.64733     ,    0.73294     ,    0.8458     ,    0.96739     ,    1.07947     ,    1.1934     ,    1.22459     ,    1.23075     ,    1.25881     ,    1.27287     ,    1.23781     ,    1.18503     ,    1.20172     ,    1.25997     ,    1.36712     ,    1.44458     ,    1.51882     ,    1.55504     ,    1.58006     ,    1.59273     ,    1.6089     ,    1.6076     ,    1.59487     ,    1.59562     ,    1.65634     ,    1.74681     ,    1.8693     ,    2.08486     ,    2.22913     ,    1.98908     ,    1.67428     ,    1.52826     ,    1.51503     ,    1.56527     ,    1.59266     ,    1.60376     ,    1.58559     ,    1.60457     ,    1.56606     ,    1.47539     ,    1.46402     ,    1.47122     ,    1.44781     ,    1.5366     ,    1.65397     ,    1.70894     ,    1.6186     ,    1.61899     ,    1.55238     ,    1.46206     ,    1.3991     ,    1.38482     ,    1.38963     ,    1.41996     ,    1.48761     ,    1.60792     ,    1.54234     ,    1.43938     ,    1.3133     ,    1.33957     ,    1.3115     ,    1.27156     ,    1.19913     ,    1.17092     ,    1.1946     ,    1.26404     ,    1.28461     ,    1.21515     ,    1.12492     ,    1.06497     ,    1.0166     ,    0.94253     ,    0.84964     ,    0.92488    },
    {    1.1206     ,    1.24335     ,    1.3566     ,    1.50386     ,    1.70029     ,    1.97672     ,    2.30855     ,    2.67798     ,    3.01543     ,    3.30108     ,    3.56127     ,    3.74139     ,    3.87963     ,    3.90752     ,    3.86142     ,    3.76725     ,    3.65156     ,    3.54432     ,    3.44068     ,    3.3579     ,    3.2651     ,    3.13869     ,    2.99936     ,    2.84865     ,    2.7141     ,    2.58684     ,    2.47182     ,    2.35988     ,    2.21668     ,    2.07012     ,    1.90257     ,    1.69946     ,    1.4999     ,    1.2731     ,    1.00347     ,    0.66845     ,    0.37435     ,    0.23896     ,    0.2074     ,    0.11128     ,    0.23965     ,    0.33201     ,    -0.05801     ,    -0.03984     ,    0.05381     ,    0.19248     ,    0.25805     ,    0.27533     ,    0.26199     ,    0.25698     ,    0.30295     ,    0.4001     ,    0.53464     ,    0.64409     ,    0.73695     ,    0.78389     ,    0.82564     ,    0.89199     ,    0.98829     ,    1.07762     ,    1.11075     ,    1.0718     ,    1.0106     ,    0.99247     ,    0.96876     ,    0.96475     ,    0.99902     ,    1.0602     ,    1.12056     ,    1.13484     ,    1.16304     ,    1.19419     ,    1.22811     ,    1.28115     ,    1.3158     ,    1.31816     ,    1.34608     ,    1.4369     ,    1.52378     ,    1.53997     ,    1.69147     ,    1.77323     ,    1.76042     ,    1.68466     ,    1.59362     ,    1.57521     ,    1.56924     ,    1.59436     ,    1.64036     ,    1.71151     ,    1.85889     ,    1.81877     ,    1.67915     ,    1.67991     ,    1.6941     ,    1.63975     ,    1.52637     ,    1.43495     ,    1.37796     ,    1.32993     ,    1.23621     ,    1.14936     ,    1.07047     ,    1.02025     ,    1.02851     ,    1.14306     ,    1.27587     ,    1.37039     ,    1.44155     ,    1.4512     ,    1.43135     ,    1.36878     ,    1.28378     ,    1.20999     ,    1.18417     ,    1.15159     ,    1.08279     ,    1.09786     ,    1.08537     ,    1.10125     ,    1.09062     ,    1.10916     ,    1.13365     ,    1.20345     ,    1.22009     ,    1.1158     ,    1.00809     ,    1.02992    },
    {    0.73874     ,    0.84737     ,    0.98033     ,    1.13921     ,    1.30498     ,    1.49772     ,    1.72507     ,    1.93236     ,    2.11104     ,    2.28759     ,    2.44695     ,    2.55635     ,    2.60205     ,    2.57802     ,    2.52938     ,    2.48519     ,    2.45526     ,    2.45173     ,    2.45542     ,    2.44503     ,    2.39804     ,    2.3147     ,    2.1987     ,    2.07448     ,    1.97044     ,    1.89305     ,    1.82587     ,    1.77711     ,    1.72978     ,    1.70536     ,    1.70406     ,    1.67844     ,    1.58096     ,    1.50989     ,    1.35996     ,    1.09032     ,    0.8566     ,    0.71541     ,    0.63252     ,    0.56661     ,    0.57299     ,    0.44483     ,    0.06823     ,    -0.11958     ,    -0.19149     ,    -0.02913     ,    0.09904     ,    0.20263     ,    0.24345     ,    0.24311     ,    0.26108     ,    0.35229     ,    0.49477     ,    0.65759     ,    0.75869     ,    0.79415     ,    0.80592     ,    0.81989     ,    0.82355     ,    0.85907     ,    0.92553     ,    0.9338     ,    0.89179     ,    0.85175     ,    0.84623     ,    0.93303     ,    1.01707     ,    1.09163     ,    1.15158     ,    1.19426     ,    1.23548     ,    1.30325     ,    1.39137     ,    1.46158     ,    1.51535     ,    1.52216     ,    1.49352     ,    1.48424     ,    1.47827     ,    1.48268     ,    1.5135     ,    1.48023     ,    1.39717     ,    1.46645     ,    1.4747     ,    1.66007     ,    1.81887     ,    1.93814     ,    2.03999     ,    2.20908     ,    2.35     ,    2.47235     ,    2.55487     ,    2.66514     ,    2.70292     ,    2.55689     ,    2.12039     ,    1.94807     ,    1.82377     ,    1.71101     ,    1.63435     ,    1.55012     ,    1.49132     ,    1.4376     ,    1.47427     ,    1.62108     ,    1.98303     ,    2.1013     ,    2.06502     ,    1.96093     ,    1.70473     ,    1.63155     ,    1.52419     ,    1.35821     ,    1.17333     ,    1.01067     ,    0.93628     ,    1.02481     ,    1.1591     ,    1.21964     ,    1.16961     ,    1.04181     ,    0.93154     ,    0.87493     ,    0.78572     ,    0.6933     ,    0.6445     ,    0.6762    },
    {    0.43464     ,    0.50806     ,    0.6036     ,    0.74073     ,    0.88186     ,    1.02231     ,    1.13109     ,    1.17386     ,    1.23556     ,    1.30563     ,    1.37632     ,    1.42705     ,    1.48035     ,    1.50929     ,    1.52947     ,    1.53816     ,    1.53605     ,    1.52016     ,    1.50394     ,    1.50034     ,    1.49988     ,    1.47765     ,    1.43292     ,    1.37722     ,    1.31786     ,    1.25373     ,    1.18931     ,    1.12865     ,    1.07429     ,    1.03305     ,    0.99382     ,    0.94838     ,    0.89953     ,    0.85959     ,    0.83218     ,    0.80988     ,    0.78203     ,    0.76045     ,    0.73822     ,    0.71947     ,    0.77546     ,    0.73509     ,    0.4611     ,    0.0688     ,    -0.16967     ,    -0.08553     ,    0.03184     ,    0.17954     ,    0.30264     ,    0.33483     ,    0.37423     ,    0.45125     ,    0.51481     ,    0.59132     ,    0.65765     ,    0.72687     ,    0.76569     ,    0.78672     ,    0.81158     ,    0.85187     ,    0.8759     ,    0.88883     ,    0.87304     ,    0.8547     ,    0.85206     ,    0.88313     ,    0.93463     ,    0.97938     ,    1.01924     ,    1.07576     ,    1.16826     ,    1.30238     ,    1.418     ,    1.52378     ,    1.61217     ,    1.68538     ,    1.74708     ,    1.80213     ,    1.84867     ,    1.87302     ,    1.83502     ,    1.75305     ,    1.67406     ,    1.6759     ,    1.72443     ,    1.81475     ,    1.85187     ,    1.89679     ,    1.94937     ,    2.16595     ,    2.36004     ,    2.59708     ,    2.86381     ,    3.07326     ,    3.20177     ,    3.1664     ,    2.97096     ,    2.73918     ,    2.39741     ,    2.09017     ,    1.80593     ,    1.57171     ,    1.37951     ,    1.34633     ,    1.37513     ,    1.46384     ,    1.42618     ,    1.28868     ,    1.16802     ,    1.16104     ,    1.31215     ,    1.70128     ,    2.01764     ,    2.03949     ,    1.68971     ,    1.20769     ,    0.89777     ,    0.76071     ,    0.78705     ,    0.96838     ,    1.08869     ,    1.07518     ,    1.00715     ,    0.90363     ,    0.73709     ,    0.57961     ,    0.47893     ,    0.42685    },
    {    0.50118     ,    0.51609     ,    0.55658     ,    0.6231     ,    0.69397     ,    0.75889     ,    0.81462     ,    0.84883     ,    0.85276     ,    0.84821     ,    0.87174     ,    0.94512     ,    1.0392     ,    1.11982     ,    1.18033     ,    1.21494     ,    1.22433     ,    1.21107     ,    1.20147     ,    1.2085     ,    1.21954     ,    1.21401     ,    1.18356     ,    1.13028     ,    1.07742     ,    1.03054     ,    0.98285     ,    0.93898     ,    0.89612     ,    0.85856     ,    0.82544     ,    0.79547     ,    0.7634     ,    0.7317     ,    0.70749     ,    0.69462     ,    0.68109     ,    0.66133     ,    0.6496     ,    0.65931     ,    0.67025     ,    0.64964     ,    0.62564     ,    0.63717     ,    0.67409     ,    0.57367     ,    0.39924     ,    0.32925     ,    0.34331     ,    0.44443     ,    0.56038     ,    0.63705     ,    0.68652     ,    0.72742     ,    0.79517     ,    0.85359     ,    0.91508     ,    0.98999     ,    1.06553     ,    1.10701     ,    1.13163     ,    1.14139     ,    1.12648     ,    1.09033     ,    1.05469     ,    1.03764     ,    1.03173     ,    1.01919     ,    1.00579     ,    1.01782     ,    1.07808     ,    1.16499     ,    1.18274     ,    1.13142     ,    1.01504     ,    1.00731     ,    1.06373     ,    1.18732     ,    1.31564     ,    1.47262     ,    1.59353     ,    1.63685     ,    1.70961     ,    1.82417     ,    1.92015     ,    1.91668     ,    1.88197     ,    1.87147     ,    1.85394     ,    1.87631     ,    1.98911     ,    1.90551     ,    2.05282     ,    2.21931     ,    2.31302     ,    2.1824     ,    1.85468     ,    1.5703     ,    1.33992     ,    1.16121     ,    1.00756     ,    0.88996     ,    0.81716     ,    0.78502     ,    0.7646     ,    0.73616     ,    0.70078     ,    0.66001     ,    0.63601     ,    0.67097     ,    0.76804     ,    0.91061     ,    1.04345     ,    1.16089     ,    1.30686     ,    1.48769     ,    1.42435     ,    1.12149     ,    0.86048     ,    0.82103     ,    0.91138     ,    0.95944     ,    0.92015     ,    0.82248     ,    0.71197     ,    0.61005     ,    0.53556     ,    0.50897    },
    {    0.64151     ,    0.67948     ,    0.71829     ,    0.75962     ,    0.7975     ,    0.84196     ,    0.8978     ,    0.94056     ,    0.9443     ,    0.91128     ,    0.89392     ,    0.92709     ,    0.98023     ,    1.01935     ,    1.04327     ,    1.05481     ,    1.05411     ,    1.04402     ,    1.03792     ,    1.04957     ,    1.07084     ,    1.08118     ,    1.06137     ,    1.01069     ,    0.95212     ,    0.90415     ,    0.86729     ,    0.84096     ,    0.82175     ,    0.80646     ,    0.79387     ,    0.78232     ,    0.76596     ,    0.73824     ,    0.71065     ,    0.70091     ,    0.70666     ,    0.71237     ,    0.71459     ,    0.70828     ,    0.70313     ,    0.68989     ,    0.67586     ,    0.68769     ,    0.72975     ,    0.78498     ,    0.79458     ,    0.65392     ,    0.49777     ,    0.46019     ,    0.52667     ,    0.6155     ,    0.64652     ,    0.68199     ,    0.76956     ,    0.85486     ,    0.92015     ,    0.98884     ,    1.10041     ,    1.18657     ,    1.20612     ,    1.20347     ,    1.22523     ,    1.24063     ,    1.21352     ,    1.14965     ,    1.11383     ,    1.1014     ,    1.06833     ,    1.02725     ,    1.00042     ,    0.96552     ,    0.90477     ,    0.75389     ,    0.64384     ,    0.57154     ,    0.54581     ,    0.57602     ,    0.64266     ,    0.74819     ,    0.85607     ,    0.89503     ,    0.89956     ,    0.90328     ,    0.91951     ,    0.94828     ,    1.01244     ,    1.12555     ,    1.22384     ,    1.29568     ,    1.31473     ,    1.17864     ,    1.06094     ,    1.18636     ,    1.31884     ,    1.36188     ,    1.3154     ,    1.24319     ,    1.16362     ,    1.05149     ,    0.94379     ,    0.87143     ,    0.81688     ,    0.78852     ,    0.77803     ,    0.77477     ,    0.76881     ,    0.74411     ,    0.70539     ,    0.66996     ,    0.66639     ,    0.69737     ,    0.73013     ,    0.78388     ,    0.91074     ,    1.13593     ,    1.30763     ,    1.20831     ,    1.01472     ,    0.91894     ,    0.91102     ,    0.87822     ,    0.75869     ,    0.63576     ,    0.55729     ,    0.52746     ,    0.53884     ,    0.59011    },
    {    0.5807     ,    0.6685     ,    0.7351     ,    0.77652     ,    0.81273     ,    0.86232     ,    0.92482     ,    0.97675     ,    0.99309     ,    0.96868     ,    0.94791     ,    0.95905     ,    0.97785     ,    0.983     ,    0.98355     ,    0.98415     ,    0.9789     ,    0.97168     ,    0.97307     ,    0.99084     ,    1.0171     ,    1.03533     ,    1.02219     ,    0.96122     ,    0.89691     ,    0.85485     ,    0.82617     ,    0.80749     ,    0.79159     ,    0.77959     ,    0.76837     ,    0.75453     ,    0.73772     ,    0.7219     ,    0.71407     ,    0.72082     ,    0.72686     ,    0.71242     ,    0.68794     ,    0.65775     ,    0.64964     ,    0.67472     ,    0.65886     ,    0.62579     ,    0.6375     ,    0.72471     ,    0.8304     ,    0.74266     ,    0.49914     ,    0.36502     ,    0.2939     ,    0.30929     ,    0.30487     ,    0.29753     ,    0.37444     ,    0.47531     ,    0.54555     ,    0.63603     ,    0.80402     ,    0.95942     ,    0.98706     ,    0.91885     ,    0.91768     ,    0.94116     ,    0.95349     ,    0.92243     ,    0.86962     ,    0.81915     ,    0.76431     ,    0.71237     ,    0.68529     ,    0.68487     ,    0.68614     ,    0.66527     ,    0.61763     ,    0.57177     ,    0.55414     ,    0.57498     ,    0.61249     ,    0.65459     ,    0.6957     ,    0.71709     ,    0.71366     ,    0.71384     ,    0.71052     ,    0.71129     ,    0.73124     ,    0.78315     ,    0.85321     ,    0.93073     ,    0.98361     ,    0.92589     ,    0.83383     ,    0.83118     ,    0.86222     ,    0.87059     ,    0.89427     ,    0.96426     ,    1.04384     ,    1.0698     ,    1.06373     ,    1.05037     ,    1.03194     ,    1.02095     ,    1.03641     ,    1.05602     ,    1.02185     ,    0.926     ,    0.81082     ,    0.7274     ,    0.70308     ,    0.74447     ,    0.79538     ,    0.81441     ,    0.82477     ,    0.85724     ,    0.86531     ,    0.77095     ,    0.65649     ,    0.60368     ,    0.60182     ,    0.58176     ,    0.52822     ,    0.4852     ,    0.45218     ,    0.42606     ,    0.42629     ,    0.48793    },
    {    0.50822     ,    0.61596     ,    0.70868     ,    0.74947     ,    0.77892     ,    0.83137     ,    0.89865     ,    0.96023     ,    0.98688     ,    0.96007     ,    0.92907     ,    0.93029     ,    0.93279     ,    0.91914     ,    0.91454     ,    0.92279     ,    0.93252     ,    0.94845     ,    0.9652     ,    0.97624     ,    0.98794     ,    0.99517     ,    0.97959     ,    0.93329     ,    0.88052     ,    0.83729     ,    0.80281     ,    0.76926     ,    0.74112     ,    0.73239     ,    0.73326     ,    0.72683     ,    0.71316     ,    0.70532     ,    0.70866     ,    0.71915     ,    0.72626     ,    0.70478     ,    0.66438     ,    0.63499     ,    0.63223     ,    0.64462     ,    0.64586     ,    0.61075     ,    0.63375     ,    0.71939     ,    0.79204     ,    0.66127     ,    0.46405     ,    0.32212     ,    0.23445     ,    0.21092     ,    0.21398     ,    0.233     ,    0.2826     ,    0.32351     ,    0.34493     ,    0.41055     ,    0.56613     ,    0.79942     ,    0.92846     ,    0.84885     ,    0.73365     ,    0.70969     ,    0.72023     ,    0.74674     ,    0.75592     ,    0.7088     ,    0.63514     ,    0.57656     ,    0.54366     ,    0.53107     ,    0.5212     ,    0.51122     ,    0.50713     ,    0.52001     ,    0.53905     ,    0.56839     ,    0.59133     ,    0.60468     ,    0.61329     ,    0.60808     ,    0.60678     ,    0.6227     ,    0.62406     ,    0.60965     ,    0.62629     ,    0.68789     ,    0.74715     ,    0.82062     ,    0.92579     ,    1.00295     ,    0.97912     ,    0.91274     ,    0.85017     ,    0.81556     ,    0.82916     ,    0.9176     ,    1.02776     ,    1.1055     ,    1.15991     ,    1.21744     ,    1.27914     ,    1.34881     ,    1.4209     ,    1.40651     ,    1.2753     ,    1.08386     ,    0.89656     ,    0.78615     ,    0.75112     ,    0.76017     ,    0.76319     ,    0.73136     ,    0.68411     ,    0.63345     ,    0.55299     ,    0.45385     ,    0.39222     ,    0.38882     ,    0.40626     ,    0.4166     ,    0.42449     ,    0.41173     ,    0.40708     ,    0.40122     ,    0.40043     ,    0.43616    },
    {    0.51702     ,    0.64217     ,    0.78142     ,    0.85196     ,    0.88422     ,    0.92038     ,    0.94517     ,    0.93594     ,    0.89879     ,    0.83654     ,    0.79494     ,    0.80783     ,    0.828     ,    0.84259     ,    0.86523     ,    0.88484     ,    0.90041     ,    0.92439     ,    0.94515     ,    0.95238     ,    0.95494     ,    0.94677     ,    0.92126     ,    0.88461     ,    0.84639     ,    0.80763     ,    0.76983     ,    0.73307     ,    0.70273     ,    0.69726     ,    0.70988     ,    0.7301     ,    0.74467     ,    0.74965     ,    0.75817     ,    0.76557     ,    0.75099     ,    0.7187     ,    0.68386     ,    0.67245     ,    0.66142     ,    0.63184     ,    0.61365     ,    0.63151     ,    0.6788     ,    0.73799     ,    0.73665     ,    0.65356     ,    0.57587     ,    0.51456     ,    0.44905     ,    0.39535     ,    0.38871     ,    0.42532     ,    0.45234     ,    0.46834     ,    0.50165     ,    0.56611     ,    0.67435     ,    0.8228     ,    0.96245     ,    0.97181     ,    0.87401     ,    0.78205     ,    0.73855     ,    0.73714     ,    0.72188     ,    0.65589     ,    0.5613     ,    0.4836     ,    0.44903     ,    0.44426     ,    0.43495     ,    0.42366     ,    0.44009     ,    0.5061     ,    0.5859     ,    0.62948     ,    0.64757     ,    0.64886     ,    0.64547     ,    0.63159     ,    0.63893     ,    0.65539     ,    0.65061     ,    0.59877     ,    0.57885     ,    0.6308     ,    0.67991     ,    0.70249     ,    0.76017     ,    0.88088     ,    0.98249     ,    0.9813     ,    0.92456     ,    0.86908     ,    0.86511     ,    0.94641     ,    1.06373     ,    1.14498     ,    1.1842     ,    1.20528     ,    1.21728     ,    1.22982     ,    1.23347     ,    1.15373     ,    1.0153     ,    0.87969     ,    0.76099     ,    0.69731     ,    0.69145     ,    0.69855     ,    0.66715     ,    0.59683     ,    0.52855     ,    0.44628     ,    0.34014     ,    0.28336     ,    0.28574     ,    0.33055     ,    0.37181     ,    0.38484     ,    0.36574     ,    0.33644     ,    0.33797     ,    0.38912     ,    0.42972     ,    0.47499    },
    {    0.42569     ,    0.54555     ,    0.76905     ,    0.80965     ,    0.77215     ,    0.75757     ,    0.72697     ,    0.67658     ,    0.64654     ,    0.63708     ,    0.65022     ,    0.68956     ,    0.72744     ,    0.76415     ,    0.807     ,    0.83643     ,    0.85792     ,    0.88312     ,    0.89708     ,    0.89845     ,    0.90122     ,    0.89581     ,    0.8797     ,    0.85648     ,    0.82453     ,    0.79818     ,    0.78095     ,    0.75403     ,    0.72742     ,    0.72363     ,    0.73723     ,    0.76522     ,    0.78414     ,    0.77958     ,    0.78418     ,    0.79174     ,    0.77315     ,    0.74669     ,    0.75915     ,    0.7642     ,    0.71835     ,    0.61935     ,    0.57068     ,    0.61884     ,    0.66475     ,    0.67037     ,    0.65143     ,    0.63489     ,    0.62677     ,    0.62409     ,    0.62017     ,    0.61223     ,    0.61246     ,    0.61979     ,    0.62951     ,    0.67397     ,    0.75919     ,    0.85958     ,    0.94534     ,    0.9873     ,    0.99979     ,    0.98937     ,    0.94316     ,    0.87769     ,    0.83971     ,    0.82338     ,    0.78023     ,    0.68283     ,    0.56609     ,    0.49956     ,    0.49434     ,    0.50421     ,    0.49165     ,    0.48114     ,    0.51342     ,    0.60592     ,    0.72089     ,    0.77192     ,    0.74019     ,    0.67619     ,    0.59104     ,    0.58413     ,    0.62405     ,    0.70229     ,    0.71854     ,    0.65405     ,    0.60702     ,    0.60652     ,    0.61078     ,    0.60884     ,    0.63601     ,    0.72949     ,    0.82995     ,    0.86363     ,    0.83497     ,    0.80385     ,    0.83004     ,    0.91934     ,    0.99642     ,    0.99192     ,    0.94805     ,    0.89736     ,    0.84857     ,    0.8298     ,    0.81713     ,    0.76866     ,    0.71103     ,    0.6795     ,    0.65872     ,    0.63169     ,    0.60761     ,    0.58037     ,    0.543     ,    0.50194     ,    0.44475     ,    0.36465     ,    0.288     ,    0.22222     ,    0.21917     ,    0.29569     ,    0.34513     ,    0.30268     ,    0.20601     ,    0.20357     ,    0.2498     ,    0.28696     ,    0.33393     ,    0.3788    },
    {    0.37182     ,    0.4148     ,    0.4817     ,    0.45742     ,    0.39823     ,    0.41304     ,    0.44367     ,    0.47111     ,    0.5143     ,    0.55172     ,    0.58995     ,    0.63693     ,    0.68596     ,    0.73191     ,    0.77266     ,    0.80011     ,    0.82927     ,    0.84717     ,    0.85608     ,    0.87126     ,    0.88882     ,    0.89566     ,    0.89758     ,    0.89416     ,    0.87435     ,    0.85326     ,    0.83419     ,    0.8     ,    0.77197     ,    0.76466     ,    0.76246     ,    0.7694     ,    0.76929     ,    0.75253     ,    0.75733     ,    0.77272     ,    0.76085     ,    0.75246     ,    0.79956     ,    0.80901     ,    0.7335     ,    0.62195     ,    0.56879     ,    0.58034     ,    0.60015     ,    0.6503     ,    0.69674     ,    0.70174     ,    0.69661     ,    0.69112     ,    0.70637     ,    0.7418     ,    0.77072     ,    0.80581     ,    0.86459     ,    0.94707     ,    1.02052     ,    1.09468     ,    1.17515     ,    1.17255     ,    1.10209     ,    1.04138     ,    0.99237     ,    0.94681     ,    0.92179     ,    0.89357     ,    0.84075     ,    0.75673     ,    0.66061     ,    0.59614     ,    0.57172     ,    0.572     ,    0.58546     ,    0.6341     ,    0.68741     ,    0.76081     ,    0.82915     ,    0.87906     ,    0.88958     ,    0.77505     ,    0.61525     ,    0.54034     ,    0.53772     ,    0.5743     ,    0.57381     ,    0.53469     ,    0.52857     ,    0.52975     ,    0.49091     ,    0.46884     ,    0.49551     ,    0.5351     ,    0.56638     ,    0.59137     ,    0.60299     ,    0.6201     ,    0.65104     ,    0.67889     ,    0.67168     ,    0.63156     ,    0.61408     ,    0.62819     ,    0.63654     ,    0.64286     ,    0.64981     ,    0.65137     ,    0.64141     ,    0.61316     ,    0.5953     ,    0.58839     ,    0.57369     ,    0.54491     ,    0.50889     ,    0.48156     ,    0.45214     ,    0.39681     ,    0.31919     ,    0.20189     ,    0.03571     ,    0.1111     ,    0.19529     ,    0.11095     ,    0.05921     ,    0.09405     ,    0.09637     ,    0.21582     ,    0.31479     ,    0.36462    },
    {    0.3438     ,    0.37275     ,    0.41103     ,    0.42399     ,    0.42898     ,    0.47307     ,    0.52423     ,    0.57499     ,    0.61975     ,    0.63585     ,    0.65401     ,    0.69382     ,    0.72832     ,    0.7525     ,    0.77399     ,    0.79546     ,    0.82268     ,    0.82969     ,    0.84301     ,    0.87329     ,    0.89857     ,    0.89385     ,    0.87889     ,    0.88935     ,    0.90543     ,    0.88752     ,    0.85915     ,    0.82565     ,    0.79452     ,    0.77683     ,    0.76083     ,    0.75531     ,    0.74961     ,    0.73004     ,    0.73096     ,    0.74182     ,    0.742     ,    0.74212     ,    0.75367     ,    0.78084     ,    0.7511     ,    0.72281     ,    0.66992     ,    0.63742     ,    0.62128     ,    0.91813     ,    1.05081     ,    1.03731     ,    1.04801     ,    1.06237     ,    1.07506     ,    1.07357     ,    1.04268     ,    1.01776     ,    1.01949     ,    1.01406     ,    0.98479     ,    1.01192     ,    1.07988     ,    1.05889     ,    0.99082     ,    0.96447     ,    0.96074     ,    0.96037     ,    0.95988     ,    0.9359     ,    0.89031     ,    0.80455     ,    0.68056     ,    0.59396     ,    0.58911     ,    0.60038     ,    0.61296     ,    0.62338     ,    0.63037     ,    0.67642     ,    0.76167     ,    0.80852     ,    0.78368     ,    0.65955     ,    0.50306     ,    0.40277     ,    0.35361     ,    0.33871     ,    0.33034     ,    0.31512     ,    0.31075     ,    0.30404     ,    0.2792     ,    0.27886     ,    0.31016     ,    0.33265     ,    0.35103     ,    0.38274     ,    0.41643     ,    0.44943     ,    0.46564     ,    0.47081     ,    0.49096     ,    0.52622     ,    0.577     ,    0.61842     ,    0.64191     ,    0.6656     ,    0.65936     ,    0.63639     ,    0.628     ,    0.61254     ,    0.6008     ,    0.57075     ,    0.53606     ,    0.51948     ,    0.47826     ,    0.45789     ,    0.4434     ,    0.38634     ,    0.33134     ,    0.22995     ,    -0.00176     ,    0.007     ,    0.12541     ,    0.05204     ,    0.03431     ,    0.09557     ,    0.03948     ,    0.03564     ,    0.19444     ,    0.35553    },
    {    0.37045     ,    0.47502     ,    0.5669     ,    0.58271     ,    0.6132     ,    0.67227     ,    0.72036     ,    0.7557     ,    0.77245     ,    0.75713     ,    0.73981     ,    0.74163     ,    0.74687     ,    0.76961     ,    0.79084     ,    0.80258     ,    0.80956     ,    0.81029     ,    0.83066     ,    0.85054     ,    0.86523     ,    0.86531     ,    0.84339     ,    0.84399     ,    0.85844     ,    0.84233     ,    0.82825     ,    0.81589     ,    0.79552     ,    0.78588     ,    0.76064     ,    0.7247     ,    0.71445     ,    0.71111     ,    0.71599     ,    0.73523     ,    0.74946     ,    0.75568     ,    0.68455     ,    0.71766     ,    0.79284     ,    0.817     ,    0.86929     ,    0.79566     ,    0.68584     ,    0.82907     ,    0.94777     ,    0.87028     ,    0.88066     ,    0.93124     ,    0.943     ,    0.89908     ,    0.8523     ,    0.81993     ,    0.79494     ,    0.79448     ,    0.80045     ,    0.83557     ,    0.89064     ,    0.90596     ,    0.9208     ,    0.96053     ,    0.98733     ,    0.99913     ,    0.99506     ,    0.97722     ,    0.92158     ,    0.79909     ,    0.70405     ,    0.67651     ,    0.65453     ,    0.63331     ,    0.60408     ,    0.57723     ,    0.56011     ,    0.51087     ,    0.46334     ,    0.46698     ,    0.45329     ,    0.38299     ,    0.29872     ,    0.2531     ,    0.22658     ,    0.19012     ,    0.1635     ,    0.15603     ,    0.14424     ,    0.14075     ,    0.16858     ,    0.21345     ,    0.25044     ,    0.27734     ,    0.31462     ,    0.36184     ,    0.3968     ,    0.43332     ,    0.45926     ,    0.48175     ,    0.52336     ,    0.57049     ,    0.6324     ,    0.67161     ,    0.67916     ,    0.68198     ,    0.63839     ,    0.61694     ,    0.60759     ,    0.5933     ,    0.6016     ,    0.58359     ,    0.55381     ,    0.54599     ,    0.48951     ,    0.47614     ,    0.51027     ,    0.46462     ,    0.61125     ,    0.58688     ,    0.09371     ,    -0.03125     ,    0.10669     ,    0.09971     ,    0.05136     ,    0.10747     ,    0.06775     ,    0.04306     ,    0.18549     ,    0.34823    },
    {    0.47614     ,    0.61995     ,    0.71048     ,    0.71215     ,    0.74008     ,    0.78328     ,    0.80668     ,    0.82139     ,    0.81837     ,    0.78712     ,    0.749     ,    0.72868     ,    0.72295     ,    0.73066     ,    0.73808     ,    0.74957     ,    0.74665     ,    0.74923     ,    0.77196     ,    0.78029     ,    0.77098     ,    0.74205     ,    0.70924     ,    0.71803     ,    0.74045     ,    0.75086     ,    0.7623     ,    0.75395     ,    0.7454     ,    0.75688     ,    0.73499     ,    0.69566     ,    0.68196     ,    0.67221     ,    0.68348     ,    0.71541     ,    0.74007     ,    0.77683     ,    0.71006     ,    0.78132     ,    0.94188     ,    0.94788     ,    0.96876     ,    0.88738     ,    0.73725     ,    0.81075     ,    0.92591     ,    0.93371     ,    0.84489     ,    0.76775     ,    0.73934     ,    0.73022     ,    0.74038     ,    0.74039     ,    0.72059     ,    0.73179     ,    0.73784     ,    0.77917     ,    0.87036     ,    0.93826     ,    0.99527     ,    1.03979     ,    1.05936     ,    1.0789     ,    1.07284     ,    1.03471     ,    0.96965     ,    0.8653     ,    0.79099     ,    0.71759     ,    0.6533     ,    0.6224     ,    0.57397     ,    0.56104     ,    0.62007     ,    0.6087     ,    0.48563     ,    0.4087     ,    0.33665     ,    0.28234     ,    0.21469     ,    0.1487     ,    0.13099     ,    0.12706     ,    0.10202     ,    0.10813     ,    0.13872     ,    0.18439     ,    0.24302     ,    0.29789     ,    0.34552     ,    0.38363     ,    0.42363     ,    0.48781     ,    0.53956     ,    0.56357     ,    0.57867     ,    0.60763     ,    0.64555     ,    0.66621     ,    0.71053     ,    0.73961     ,    0.72974     ,    0.65328     ,    0.59712     ,    0.72311     ,    0.8106     ,    0.65753     ,    0.64442     ,    0.62676     ,    0.59537     ,    0.57995     ,    0.52562     ,    0.54781     ,    0.63522     ,    0.75705     ,    1.05412     ,    1.18682     ,    0.70225     ,    0.24487     ,    0.33266     ,    0.31557     ,    0.19171     ,    0.17693     ,    0.17892     ,    0.18357     ,    0.26208     ,    0.36091    },
    {    0.6615     ,    0.71243     ,    0.71363     ,    0.70653     ,    0.71763     ,    0.72099     ,    0.71415     ,    0.73352     ,    0.72829     ,    0.69598     ,    0.66737     ,    0.65557     ,    0.65088     ,    0.63655     ,    0.64031     ,    0.65349     ,    0.64915     ,    0.64493     ,    0.62699     ,    0.59866     ,    0.58066     ,    0.55753     ,    0.56451     ,    0.59898     ,    0.6317     ,    0.65357     ,    0.64743     ,    0.64264     ,    0.65398     ,    0.67243     ,    0.67191     ,    0.6597     ,    0.66581     ,    0.67325     ,    0.68209     ,    0.69764     ,    0.73404     ,    0.84038     ,    0.86454     ,    0.92481     ,    1.13979     ,    1.12194     ,    1.04639     ,    0.9211     ,    0.82082     ,    0.8943     ,    0.97171     ,    1.00166     ,    0.95916     ,    0.84172     ,    0.78735     ,    0.79793     ,    0.808     ,    0.78314     ,    0.75593     ,    0.79185     ,    0.83144     ,    0.89007     ,    0.97293     ,    1.05094     ,    1.11421     ,    1.13624     ,    1.1394     ,    1.13484     ,    1.09919     ,    1.05741     ,    0.9971     ,    0.91342     ,    0.82936     ,    0.71089     ,    0.66378     ,    0.84249     ,    0.99797     ,    1.11668     ,    1.00635     ,    0.78614     ,    0.71344     ,    0.51158     ,    0.38838     ,    0.38197     ,    0.3288     ,    0.26584     ,    0.2373     ,    0.23703     ,    0.22961     ,    0.26813     ,    0.35684     ,    0.4415     ,    0.51074     ,    0.56623     ,    0.60521     ,    0.64819     ,    0.68625     ,    0.74947     ,    0.78708     ,    0.79296     ,    0.80108     ,    0.78414     ,    0.77058     ,    0.7734     ,    0.77778     ,    0.73718     ,    0.66901     ,    0.63471     ,    0.65709     ,    0.85929     ,    1.09006     ,    1.0563     ,    0.96785     ,    0.82422     ,    0.75761     ,    0.72647     ,    0.76581     ,    0.83162     ,    0.89096     ,    1.04977     ,    1.21265     ,    1.324     ,    1.21727     ,    0.67065     ,    0.58331     ,    0.59295     ,    0.50732     ,    0.44995     ,    0.4385     ,    0.43581     ,    0.47046     ,    0.56409    },
    {    0.56097     ,    0.54871     ,    0.54877     ,    0.56619     ,    0.5735     ,    0.57716     ,    0.59652     ,    0.62879     ,    0.6358     ,    0.62224     ,    0.61199     ,    0.58279     ,    0.56106     ,    0.55526     ,    0.54829     ,    0.53258     ,    0.515     ,    0.50292     ,    0.48212     ,    0.47018     ,    0.4696     ,    0.46465     ,    0.47997     ,    0.49881     ,    0.51972     ,    0.54214     ,    0.54389     ,    0.57912     ,    0.61862     ,    0.63162     ,    0.64843     ,    0.65815     ,    0.69448     ,    0.71339     ,    0.69453     ,    0.71346     ,    0.76811     ,    0.8834     ,    1.04314     ,    1.08352     ,    1.31321     ,    1.26265     ,    1.26928     ,    1.21984     ,    1.12422     ,    1.13475     ,    1.07401     ,    1.03806     ,    1.02361     ,    0.9528     ,    0.9078     ,    0.89668     ,    0.90848     ,    0.89078     ,    0.88342     ,    0.9342     ,    0.99339     ,    1.04522     ,    1.09792     ,    1.1619     ,    1.18765     ,    1.18647     ,    1.18643     ,    1.16104     ,    1.11027     ,    1.0501     ,    0.96685     ,    0.87449     ,    0.75121     ,    0.65663     ,    0.81975     ,    1.11766     ,    1.44387     ,    1.74153     ,    1.56193     ,    1.17928     ,    0.86799     ,    0.73103     ,    0.61565     ,    0.53318     ,    0.40971     ,    0.35569     ,    0.36902     ,    0.43723     ,    0.4608     ,    0.49648     ,    0.58401     ,    0.6547     ,    0.72119     ,    0.79949     ,    0.83554     ,    0.88324     ,    0.94056     ,    0.98362     ,    1.00734     ,    1.01128     ,    0.99267     ,    0.95088     ,    0.9091     ,    0.8815     ,    0.84158     ,    0.74658     ,    0.68192     ,    0.6416     ,    0.62844     ,    0.87162     ,    1.24281     ,    1.3952     ,    1.40219     ,    1.23963     ,    1.12716     ,    1.04593     ,    0.9827     ,    0.95418     ,    0.88724     ,    1.33808     ,    1.44052     ,    1.46338     ,    1.43194     ,    0.94967     ,    0.61587     ,    0.61016     ,    0.53085     ,    0.48591     ,    0.49431     ,    0.50843     ,    0.5384     ,    0.5629    },
    {    0.41015     ,    0.46837     ,    0.51611     ,    0.55869     ,    0.58017     ,    0.59197     ,    0.61932     ,    0.6426     ,    0.63454     ,    0.61075     ,    0.59735     ,    0.5637     ,    0.53671     ,    0.51142     ,    0.47452     ,    0.45091     ,    0.43925     ,    0.41583     ,    0.39491     ,    0.38453     ,    0.37233     ,    0.37427     ,    0.40788     ,    0.46722     ,    0.52389     ,    0.56106     ,    0.60766     ,    0.66229     ,    0.68938     ,    0.698     ,    0.71417     ,    0.7425     ,    0.76956     ,    0.76323     ,    0.75875     ,    0.81445     ,    0.82704     ,    0.90239     ,    1.00809     ,    1.30515     ,    1.51277     ,    1.40636     ,    1.44339     ,    1.42103     ,    1.36977     ,    1.34637     ,    1.18513     ,    1.11903     ,    1.10724     ,    1.05225     ,    1.02062     ,    1.00627     ,    1.02136     ,    1.04783     ,    1.08474     ,    1.12033     ,    1.16389     ,    1.19983     ,    1.22867     ,    1.24541     ,    1.24947     ,    1.2501     ,    1.22306     ,    1.17048     ,    1.09121     ,    0.99161     ,    0.89315     ,    0.79721     ,    0.69438     ,    0.77207     ,    1.18414     ,    1.484     ,    1.5988     ,    1.68242     ,    1.67758     ,    1.34966     ,    0.90859     ,    0.79851     ,    0.72651     ,    0.69641     ,    0.74485     ,    0.5697     ,    0.48673     ,    0.57446     ,    0.60842     ,    0.6437     ,    0.72315     ,    0.80844     ,    0.89313     ,    0.97288     ,    1.0342     ,    1.10504     ,    1.17152     ,    1.19855     ,    1.2034     ,    1.17075     ,    1.11211     ,    1.07174     ,    1.03349     ,    0.98908     ,    0.91119     ,    0.77991     ,    0.65199     ,    0.55846     ,    0.56554     ,    0.96467     ,    1.32233     ,    1.41253     ,    1.48477     ,    1.35801     ,    1.19885     ,    1.0877     ,    1.04931     ,    1.10323     ,    1.1216     ,    1.517     ,    1.62677     ,    1.50866     ,    1.17581     ,    0.63203     ,    0.50089     ,    0.43119     ,    0.34844     ,    0.32112     ,    0.3122     ,    0.32376     ,    0.35675     ,    0.37811    },
    {    0.50896     ,    0.5736     ,    0.61445     ,    0.64259     ,    0.65853     ,    0.67352     ,    0.67635     ,    0.65808     ,    0.62657     ,    0.58004     ,    0.5425     ,    0.49893     ,    0.44827     ,    0.39015     ,    0.34799     ,    0.31064     ,    0.27873     ,    0.25676     ,    0.26039     ,    0.30565     ,    0.34515     ,    0.42898     ,    0.53161     ,    0.60464     ,    0.66939     ,    0.71501     ,    0.75644     ,    0.77211     ,    0.77128     ,    0.79996     ,    0.81264     ,    0.83845     ,    0.852     ,    0.88058     ,    0.89386     ,    0.89056     ,    0.85312     ,    0.87937     ,    1.03715     ,    1.50613     ,    1.67481     ,    1.52241     ,    1.48321     ,    1.48016     ,    1.61185     ,    1.62161     ,    1.41598     ,    1.36243     ,    1.30457     ,    1.13447     ,    1.07839     ,    1.10524     ,    1.16793     ,    1.21878     ,    1.29022     ,    1.30262     ,    1.31209     ,    1.32184     ,    1.3239     ,    1.30619     ,    1.29124     ,    1.24826     ,    1.17866     ,    1.11366     ,    1.02957     ,    0.94419     ,    0.88414     ,    0.79372     ,    0.7071     ,    1.01027     ,    1.54065     ,    1.7122     ,    1.65187     ,    1.53382     ,    1.57529     ,    1.44845     ,    1.0677     ,    0.79172     ,    0.71021     ,    0.95976     ,    1.33993     ,    0.97212     ,    0.66282     ,    0.68126     ,    0.67107     ,    0.75173     ,    0.83364     ,    0.90572     ,    0.97778     ,    1.01626     ,    1.07287     ,    1.12847     ,    1.17315     ,    1.19909     ,    1.20514     ,    1.20564     ,    1.19957     ,    1.15649     ,    1.08568     ,    1.01577     ,    0.89685     ,    0.72119     ,    0.54125     ,    0.40761     ,    0.45839     ,    1.05033     ,    1.532     ,    1.5376     ,    1.50094     ,    1.37633     ,    1.22879     ,    1.19301     ,    1.21488     ,    1.27408     ,    1.30557     ,    1.50118     ,    1.48125     ,    1.16478     ,    0.67591     ,    0.49197     ,    0.41333     ,    0.31958     ,    0.29604     ,    0.3075     ,    0.32156     ,    0.35636     ,    0.39803     ,    0.44952    },
    {    0.63995     ,    0.67277     ,    0.70402     ,    0.71701     ,    0.72411     ,    0.72383     ,    0.70104     ,    0.66841     ,    0.63741     ,    0.58454     ,    0.52081     ,    0.45591     ,    0.39797     ,    0.33642     ,    0.30658     ,    0.281     ,    0.25496     ,    0.28086     ,    0.36279     ,    0.48246     ,    0.5813     ,    0.68152     ,    0.77384     ,    0.81724     ,    0.84792     ,    0.85716     ,    0.86467     ,    0.88361     ,    0.89886     ,    0.93491     ,    0.94948     ,    0.9615     ,    0.95349     ,    0.95083     ,    0.89212     ,    0.90498     ,    0.93994     ,    0.86388     ,    1.03821     ,    1.57995     ,    1.68621     ,    1.57451     ,    1.6774     ,    1.6676     ,    1.76032     ,    1.77952     ,    1.73046     ,    1.62974     ,    1.46701     ,    1.29777     ,    1.05292     ,    1.15156     ,    1.2939     ,    1.33706     ,    1.37457     ,    1.36112     ,    1.35744     ,    1.34495     ,    1.32397     ,    1.29275     ,    1.24755     ,    1.17956     ,    1.12619     ,    1.06301     ,    0.98971     ,    0.90177     ,    0.82345     ,    0.72473     ,    0.73073     ,    1.21827     ,    1.64255     ,    1.55394     ,    1.52514     ,    1.42211     ,    1.49206     ,    1.42794     ,    1.13145     ,    0.81381     ,    0.77898     ,    1.01858     ,    1.35025     ,    1.10121     ,    0.77276     ,    0.75925     ,    0.76808     ,    0.85659     ,    0.90881     ,    0.93723     ,    0.98541     ,    1.02343     ,    1.06832     ,    1.10944     ,    1.13674     ,    1.14969     ,    1.15888     ,    1.13243     ,    1.07645     ,    0.98953     ,    0.89643     ,    0.78375     ,    0.63692     ,    0.51133     ,    0.39968     ,    0.31993     ,    0.36287     ,    0.75717     ,    1.23674     ,    1.3267     ,    1.42769     ,    1.44959     ,    1.45096     ,    1.42743     ,    1.50052     ,    1.48148     ,    1.38471     ,    1.38294     ,    1.10914     ,    0.66909     ,    0.4405     ,    0.39426     ,    0.34907     ,    0.33479     ,    0.36087     ,    0.39775     ,    0.44993     ,    0.50156     ,    0.54837     ,    0.59888    },
    {    0.72289     ,    0.76709     ,    0.80553     ,    0.81354     ,    0.80915     ,    0.79858     ,    0.78596     ,    0.74575     ,    0.68637     ,    0.62173     ,    0.58488     ,    0.55494     ,    0.54285     ,    0.52945     ,    0.53503     ,    0.56469     ,    0.61565     ,    0.69624     ,    0.8017     ,    0.88664     ,    0.9556     ,    0.99816     ,    1.01525     ,    0.99696     ,    0.9809     ,    0.99341     ,    0.99637     ,    1.02322     ,    1.0275     ,    1.0321     ,    1.01975     ,    0.99132     ,    0.96169     ,    0.92727     ,    0.9225     ,    0.95552     ,    0.90544     ,    0.84698     ,    1.30117     ,    1.63621     ,    1.56318     ,    1.71238     ,    1.93894     ,    1.85834     ,    1.79264     ,    1.76702     ,    1.75015     ,    1.63967     ,    1.54284     ,    1.33774     ,    1.00997     ,    1.10808     ,    1.18806     ,    1.20179     ,    1.23346     ,    1.21197     ,    1.20134     ,    1.17438     ,    1.14198     ,    1.10414     ,    1.06355     ,    1.02306     ,    0.98364     ,    0.90486     ,    0.82551     ,    0.75906     ,    0.76729     ,    0.75621     ,    0.87606     ,    1.51212     ,    1.63266     ,    1.50797     ,    1.44205     ,    1.27688     ,    1.3316     ,    1.27184     ,    1.19407     ,    0.98235     ,    0.84265     ,    0.92458     ,    1.18667     ,    1.10838     ,    0.86797     ,    0.87484     ,    0.92785     ,    0.96752     ,    0.99237     ,    1.02322     ,    1.0395     ,    1.06082     ,    1.05804     ,    1.04912     ,    1.02235     ,    0.96412     ,    0.89568     ,    0.80376     ,    0.7155     ,    0.64073     ,    0.57019     ,    0.48674     ,    0.41589     ,    0.36157     ,    0.32462     ,    0.3585     ,    0.40701     ,    0.52037     ,    0.6048     ,    0.74162     ,    1.27633     ,    1.61691     ,    1.69268     ,    1.56165     ,    1.4899     ,    1.35021     ,    1.31593     ,    1.14972     ,    0.61315     ,    0.45779     ,    0.42655     ,    0.39544     ,    0.38113     ,    0.41934     ,    0.4601     ,    0.5278     ,    0.5935     ,    0.6307     ,    0.66462     ,    0.6902    },
    {    0.79737     ,    0.84328     ,    0.87923     ,    0.88587     ,    0.8785     ,    0.87066     ,    0.87445     ,    0.84318     ,    0.80952     ,    0.7811     ,    0.78382     ,    0.79264     ,    0.80444     ,    0.834     ,    0.88075     ,    0.9311     ,    0.98506     ,    1.03955     ,    1.09131     ,    1.11699     ,    1.14109     ,    1.13038     ,    1.09912     ,    1.07879     ,    1.06593     ,    1.07555     ,    1.05356     ,    1.04416     ,    1.02781     ,    1.0211     ,    1.00095     ,    0.96559     ,    0.95029     ,    0.94929     ,    0.99358     ,    0.94242     ,    0.88929     ,    1.20732     ,    1.60131     ,    1.60773     ,    1.6632     ,    1.84713     ,    1.83836     ,    1.81793     ,    1.84565     ,    1.77822     ,    1.68819     ,    1.5826     ,    1.52429     ,    1.29028     ,    1.02246     ,    0.98661     ,    0.85371     ,    0.88989     ,    0.93881     ,    0.93991     ,    0.93249     ,    0.91288     ,    0.90025     ,    0.87515     ,    0.8597     ,    0.82438     ,    0.78907     ,    0.75179     ,    0.73307     ,    0.77678     ,    0.86689     ,    0.83535     ,    0.97717     ,    1.69851     ,    1.61707     ,    1.51021     ,    1.52203     ,    1.33442     ,    1.39339     ,    1.31919     ,    1.39038     ,    1.29842     ,    1.14288     ,    0.97682     ,    0.94081     ,    1.00757     ,    0.9432     ,    0.99272     ,    1.03443     ,    1.05559     ,    1.04735     ,    1.04558     ,    1.02269     ,    0.99163     ,    0.93885     ,    0.88162     ,    0.80668     ,    0.72703     ,    0.63834     ,    0.56671     ,    0.50916     ,    0.46366     ,    0.43357     ,    0.39639     ,    0.34655     ,    0.32293     ,    0.36378     ,    0.47157     ,    0.57267     ,    0.67126     ,    0.7131     ,    0.70094     ,    0.87127     ,    1.25089     ,    1.28531     ,    1.35987     ,    1.25712     ,    0.80562     ,    0.76055     ,    0.85732     ,    0.44224     ,    0.43933     ,    0.46336     ,    0.48899     ,    0.50465     ,    0.54502     ,    0.60015     ,    0.66038     ,    0.69303     ,    0.71888     ,    0.74853     ,    0.76029    },
    {    0.92119     ,    0.93753     ,    0.96776     ,    0.9891     ,    1.0083     ,    1.01051     ,    1.02204     ,    1.01789     ,    1.03336     ,    1.02151     ,    1.00233     ,    1.00147     ,    1.01919     ,    1.05034     ,    1.09117     ,    1.1035     ,    1.10753     ,    1.10703     ,    1.09594     ,    1.08965     ,    1.07385     ,    1.05011     ,    1.0276     ,    1.01476     ,    0.99804     ,    0.99014     ,    0.97765     ,    0.99143     ,    0.99625     ,    0.98471     ,    0.98251     ,    0.97652     ,    0.98334     ,    1.02006     ,    1.0204     ,    0.96242     ,    1.22472     ,    1.56636     ,    1.53164     ,    1.63484     ,    1.88925     ,    1.89214     ,    1.79828     ,    1.86001     ,    1.87172     ,    1.7733     ,    1.70419     ,    1.57402     ,    1.45731     ,    1.24265     ,    1.01475     ,    0.93199     ,    0.82322     ,    0.81684     ,    0.81949     ,    0.82861     ,    0.84018     ,    0.83221     ,    0.8325     ,    0.82655     ,    0.82078     ,    0.79773     ,    0.78959     ,    0.79014     ,    0.83047     ,    0.87482     ,    0.90585     ,    0.88199     ,    0.91808     ,    1.72145     ,    1.9263     ,    1.62371     ,    1.62605     ,    1.47506     ,    1.36822     ,    1.3428     ,    1.37153     ,    1.24351     ,    1.10353     ,    1.01073     ,    1.00451     ,    0.97669     ,    1.01303     ,    1.04123     ,    1.05792     ,    1.09596     ,    1.08343     ,    1.06415     ,    1.03442     ,    0.98451     ,    0.92858     ,    0.86917     ,    0.79245     ,    0.73111     ,    0.66131     ,    0.60332     ,    0.52497     ,    0.47941     ,    0.43907     ,    0.40766     ,    0.42273     ,    0.46626     ,    0.46354     ,    0.46502     ,    0.50101     ,    0.55198     ,    0.69346     ,    0.73917     ,    0.78325     ,    0.8181     ,    0.79377     ,    1.05597     ,    1.06708     ,    0.55293     ,    0.50456     ,    0.45072     ,    0.37139     ,    0.46125     ,    0.50888     ,    0.60608     ,    0.64544     ,    0.68511     ,    0.7358     ,    0.76111     ,    0.79636     ,    0.82516     ,    0.85589     ,    0.88839    },
    {    1.09332     ,    1.10313     ,    1.12527     ,    1.13591     ,    1.15662     ,    1.15361     ,    1.15903     ,    1.16108     ,    1.15656     ,    1.13802     ,    1.1086     ,    1.08385     ,    1.08599     ,    1.09064     ,    1.09162     ,    1.08209     ,    1.07776     ,    1.07475     ,    1.06503     ,    1.0454     ,    1.01699     ,    1.00288     ,    0.98735     ,    0.97883     ,    0.9842     ,    0.99214     ,    1.01049     ,    1.01781     ,    1.01436     ,    1.00725     ,    1.00775     ,    1.02402     ,    1.04371     ,    1.0752     ,    1.06836     ,    1.11418     ,    1.43736     ,    1.59791     ,    1.67808     ,    1.88464     ,    1.96783     ,    1.89268     ,    1.8785     ,    1.90697     ,    1.80828     ,    1.78926     ,    1.74372     ,    1.58035     ,    1.53124     ,    1.44677     ,    1.21193     ,    1.03389     ,    0.89247     ,    0.86094     ,    0.85588     ,    0.85444     ,    0.858     ,    0.85064     ,    0.84574     ,    0.84431     ,    0.83997     ,    0.83427     ,    0.83575     ,    0.86457     ,    0.90475     ,    0.89569     ,    0.9346     ,    0.94601     ,    0.80204     ,    1.41769     ,    2.10458     ,    1.65797     ,    1.57524     ,    1.57095     ,    1.49594     ,    1.42022     ,    1.31046     ,    1.21953     ,    1.14937     ,    1.08136     ,    1.09268     ,    1.08096     ,    1.06645     ,    1.03267     ,    1.0645     ,    1.08076     ,    1.08691     ,    1.08294     ,    1.05865     ,    1.03949     ,    1.01505     ,    0.97933     ,    0.94376     ,    0.89042     ,    0.81268     ,    0.74678     ,    0.67679     ,    0.5989     ,    0.47582     ,    0.4227     ,    0.34981     ,    0.23242     ,    0.16954     ,    0.36923     ,    0.64126     ,    0.67619     ,    0.77556     ,    0.77256     ,    0.74798     ,    0.78359     ,    0.7034     ,    0.73864     ,    0.59336     ,    0.47666     ,    0.53561     ,    0.47126     ,    0.58948     ,    0.76054     ,    0.7008     ,    0.80614     ,    0.84654     ,    0.8866     ,    0.92141     ,    0.95212     ,    0.99267     ,    1.019     ,    1.05002     ,    1.06953    },
    {    1.38124     ,    1.36501     ,    1.34974     ,    1.33683     ,    1.33232     ,    1.32125     ,    1.31488     ,    1.3134     ,    1.2975     ,    1.27052     ,    1.24233     ,    1.22148     ,    1.20122     ,    1.17619     ,    1.15411     ,    1.13117     ,    1.11577     ,    1.1008     ,    1.08906     ,    1.07326     ,    1.06233     ,    1.04514     ,    1.03882     ,    1.03969     ,    1.04916     ,    1.04507     ,    1.05675     ,    1.06484     ,    1.0619     ,    1.05245     ,    1.04958     ,    1.08691     ,    1.12885     ,    1.15836     ,    1.21175     ,    1.3552     ,    1.55364     ,    1.73863     ,    1.85753     ,    1.87615     ,    1.9301     ,    1.97935     ,    1.98595     ,    1.97054     ,    1.80484     ,    1.78991     ,    1.63254     ,    1.50787     ,    1.53526     ,    1.57024     ,    1.38286     ,    1.11637     ,    0.91901     ,    0.91622     ,    0.91375     ,    0.9063     ,    0.90445     ,    0.88726     ,    0.87877     ,    0.87702     ,    0.89127     ,    0.90844     ,    0.92026     ,    0.9619     ,    0.96565     ,    0.97518     ,    1.03833     ,    0.99455     ,    0.95159     ,    1.482     ,    1.76286     ,    1.56716     ,    1.70158     ,    1.71552     ,    1.67082     ,    1.52021     ,    1.30432     ,    1.23531     ,    1.15081     ,    1.11339     ,    1.10605     ,    1.10835     ,    1.10032     ,    1.07718     ,    1.10511     ,    1.09435     ,    1.10883     ,    1.11064     ,    1.11342     ,    1.10981     ,    1.10362     ,    1.07251     ,    1.0373     ,    0.99352     ,    0.9395     ,    0.86249     ,    0.77035     ,    0.62702     ,    0.47747     ,    0.30942     ,    0.06555     ,    0.15422     ,    0.88969     ,    1.08839     ,    1.0796     ,    1.0353     ,    0.9759     ,    0.83628     ,    0.73105     ,    0.7789     ,    0.689     ,    0.59419     ,    0.52478     ,    0.78349     ,    1.09227     ,    1.01879     ,    1.11814     ,    1.02873     ,    0.96903     ,    1.08241     ,    1.14997     ,    1.20102     ,    1.25827     ,    1.31945     ,    1.35074     ,    1.3784     ,    1.38951     ,    1.37551    },
    {    1.67889     ,    1.63807     ,    1.59971     ,    1.56011     ,    1.52896     ,    1.5068     ,    1.48393     ,    1.46341     ,    1.44471     ,    1.4214     ,    1.38911     ,    1.37491     ,    1.36162     ,    1.35112     ,    1.34403     ,    1.30901     ,    1.27791     ,    1.24989     ,    1.22071     ,    1.19476     ,    1.18273     ,    1.15766     ,    1.14855     ,    1.12329     ,    1.11888     ,    1.12944     ,    1.13642     ,    1.13896     ,    1.13629     ,    1.13139     ,    1.14761     ,    1.16262     ,    1.18569     ,    1.21355     ,    1.26667     ,    1.35193     ,    1.46274     ,    1.75178     ,    1.88762     ,    1.81912     ,    1.93288     ,    1.9816     ,    1.98307     ,    1.91589     ,    1.61228     ,    1.50037     ,    1.45059     ,    1.39995     ,    1.37538     ,    1.31885     ,    1.18288     ,    1.01479     ,    0.95234     ,    0.98544     ,    0.96889     ,    0.95859     ,    0.94242     ,    0.93338     ,    0.94573     ,    0.94514     ,    0.95756     ,    0.96678     ,    0.9818     ,    0.99304     ,    0.97916     ,    1.03144     ,    1.07042     ,    1.02817     ,    1.26949     ,    1.57209     ,    1.4613     ,    1.71163     ,    1.93183     ,    1.82218     ,    1.66491     ,    1.54769     ,    1.30523     ,    1.15233     ,    1.13903     ,    1.12316     ,    1.17002     ,    1.17818     ,    1.14837     ,    1.14657     ,    1.15379     ,    1.16466     ,    1.16854     ,    1.15855     ,    1.17892     ,    1.15878     ,    1.14223     ,    1.12778     ,    1.0924     ,    1.03036     ,    0.95895     ,    0.87339     ,    0.77398     ,    0.61752     ,    0.48305     ,    0.26886     ,    0.25697     ,    0.97945     ,    1.09973     ,    0.93524     ,    0.81512     ,    0.90787     ,    0.65774     ,    0.73276     ,    0.75651     ,    0.80211     ,    0.80016     ,    0.69588     ,    0.7713     ,    1.06067     ,    0.93896     ,    1.12619     ,    1.25062     ,    1.2069     ,    1.43925     ,    1.65308     ,    1.73553     ,    1.77878     ,    1.8292     ,    1.84812     ,    1.83855     ,    1.81329     ,    1.77193     ,    1.72695    },
    {    1.95447     ,    1.88188     ,    1.81182     ,    1.75145     ,    1.69967     ,    1.65545     ,    1.61046     ,    1.5591     ,    1.50404     ,    1.4685     ,    1.43439     ,    1.41242     ,    1.4082     ,    1.40452     ,    1.39999     ,    1.38842     ,    1.38338     ,    1.3731     ,    1.35561     ,    1.33597     ,    1.31789     ,    1.2814     ,    1.2519     ,    1.22559     ,    1.21812     ,    1.20827     ,    1.20563     ,    1.20883     ,    1.22839     ,    1.24977     ,    1.26467     ,    1.25056     ,    1.2665     ,    1.27924     ,    1.29394     ,    1.31417     ,    1.26537     ,    1.48968     ,    1.87566     ,    1.81997     ,    1.80902     ,    1.96437     ,    2.02807     ,    1.83159     ,    1.50864     ,    1.47876     ,    1.4418     ,    1.30352     ,    1.13417     ,    1.0845     ,    1.06168     ,    1.06249     ,    1.07595     ,    1.07175     ,    1.05618     ,    1.05212     ,    1.03223     ,    1.03696     ,    1.02495     ,    1.02558     ,    1.0399     ,    1.0441     ,    1.07472     ,    1.0704     ,    1.06721     ,    1.09972     ,    1.15882     ,    1.16937     ,    1.30538     ,    1.39231     ,    1.6286     ,    1.81369     ,    1.84163     ,    1.91101     ,    1.79235     ,    1.71446     ,    1.51869     ,    1.12862     ,    1.22517     ,    1.16735     ,    1.1195     ,    1.17219     ,    1.15392     ,    1.16735     ,    1.18116     ,    1.19794     ,    1.18123     ,    1.1483     ,    1.12203     ,    1.09117     ,    1.06953     ,    1.03833     ,    1.0096     ,    0.95158     ,    0.8834     ,    0.8108     ,    0.73087     ,    0.63278     ,    0.56334     ,    0.4813     ,    0.86031     ,    1.18761     ,    0.87302     ,    1.00754     ,    1.34424     ,    1.38954     ,    0.72184     ,    0.96053     ,    0.87207     ,    0.73128     ,    0.80406     ,    0.89121     ,    1.25941     ,    1.33929     ,    1.3461     ,    1.61607     ,    1.75641     ,    1.91617     ,    2.21961     ,    2.36733     ,    2.40836     ,    2.41739     ,    2.39785     ,    2.37361     ,    2.34081     ,    2.27124     ,    2.18537     ,    2.0691    },
    {    2.0286     ,    1.93955     ,    1.85602     ,    1.77873     ,    1.70437     ,    1.64922     ,    1.59789     ,    1.5432     ,    1.4917     ,    1.45647     ,    1.42908     ,    1.40711     ,    1.38903     ,    1.36857     ,    1.35596     ,    1.35906     ,    1.35984     ,    1.35631     ,    1.35068     ,    1.34372     ,    1.33372     ,    1.31066     ,    1.28687     ,    1.28561     ,    1.27939     ,    1.26959     ,    1.27596     ,    1.27162     ,    1.28344     ,    1.29213     ,    1.28133     ,    1.23871     ,    1.22275     ,    1.22694     ,    1.2454     ,    1.2598     ,    1.14577     ,    1.20238     ,    1.81953     ,    1.95986     ,    1.8133     ,    1.90824     ,    1.91244     ,    1.62867     ,    1.39931     ,    1.35694     ,    1.24693     ,    1.1196     ,    1.11041     ,    1.11838     ,    1.1166     ,    1.106     ,    1.09928     ,    1.09489     ,    1.08105     ,    1.07965     ,    1.08556     ,    1.0993     ,    1.09457     ,    1.11049     ,    1.11716     ,    1.11911     ,    1.14239     ,    1.14468     ,    1.17257     ,    1.18217     ,    1.22129     ,    1.25257     ,    1.33578     ,    1.43409     ,    1.70009     ,    1.82753     ,    1.96322     ,    1.98601     ,    1.68028     ,    1.57779     ,    1.5866     ,    1.13818     ,    1.14571     ,    1.27804     ,    1.15059     ,    1.13765     ,    1.1229     ,    1.11095     ,    1.13524     ,    1.15064     ,    1.1373     ,    1.12602     ,    1.1044     ,    1.08908     ,    1.0699     ,    1.05535     ,    1.03116     ,    0.99053     ,    0.94605     ,    0.89941     ,    0.83704     ,    0.80949     ,    0.78569     ,    0.84621     ,    1.05556     ,    0.9955     ,    0.78631     ,    1.09958     ,    1.142     ,    1.07832     ,    0.75033     ,    0.83311     ,    1.07925     ,    0.7464     ,    0.87002     ,    1.09231     ,    1.43051     ,    1.5846     ,    1.67059     ,    1.76035     ,    1.88088     ,    2.1165     ,    2.31728     ,    2.38381     ,    2.43348     ,    2.44526     ,    2.40909     ,    2.39351     ,    2.3626     ,    2.30491     ,    2.22445     ,    2.11591    },
    {    1.6869     ,    1.654     ,    1.63646     ,    1.59676     ,    1.55584     ,    1.52322     ,    1.49619     ,    1.46472     ,    1.44174     ,    1.42711     ,    1.41116     ,    1.40203     ,    1.39045     ,    1.38311     ,    1.37429     ,    1.371     ,    1.36528     ,    1.35755     ,    1.33891     ,    1.33034     ,    1.32482     ,    1.31935     ,    1.306     ,    1.29777     ,    1.29085     ,    1.29721     ,    1.2745     ,    1.25394     ,    1.24452     ,    1.23378     ,    1.21137     ,    1.15686     ,    1.13345     ,    1.13774     ,    1.14607     ,    1.15212     ,    1.14223     ,    1.15824     ,    1.62338     ,    1.9321     ,    1.89426     ,    1.71141     ,    1.59378     ,    1.44495     ,    1.30724     ,    1.23772     ,    1.15157     ,    1.13047     ,    1.14264     ,    1.1158     ,    1.0946     ,    1.06948     ,    1.06424     ,    1.07593     ,    1.07527     ,    1.08716     ,    1.10931     ,    1.12876     ,    1.15833     ,    1.17703     ,    1.16971     ,    1.21236     ,    1.19815     ,    1.18756     ,    1.17333     ,    1.17258     ,    1.21929     ,    1.31353     ,    1.46206     ,    1.5847     ,    1.70368     ,    1.92215     ,    2.20914     ,    1.96033     ,    1.8201     ,    1.74236     ,    1.70054     ,    1.35351     ,    1.13878     ,    1.32702     ,    1.28845     ,    1.1707     ,    1.09416     ,    1.09551     ,    1.09213     ,    1.08044     ,    1.08709     ,    1.08974     ,    1.0873     ,    1.10172     ,    1.10562     ,    1.13629     ,    1.12924     ,    1.12219     ,    1.07026     ,    1.04142     ,    0.97879     ,    0.97005     ,    0.95275     ,    1.01296     ,    1.09531     ,    0.94975     ,    0.83266     ,    0.94476     ,    0.89436     ,    1.02803     ,    1.06589     ,    0.81539     ,    0.87115     ,    0.81978     ,    0.85493     ,    0.99946     ,    1.11264     ,    1.1637     ,    1.2139     ,    1.25658     ,    1.36706     ,    1.47651     ,    1.53638     ,    1.60067     ,    1.6715     ,    1.7187     ,    1.74253     ,    1.74534     ,    1.72755     ,    1.71959     ,    1.71515     ,    1.70013    },
    {    1.2338     ,    1.2444     ,    1.25937     ,    1.26456     ,    1.27691     ,    1.28742     ,    1.30213     ,    1.31343     ,    1.32564     ,    1.33844     ,    1.34735     ,    1.35752     ,    1.35436     ,    1.34696     ,    1.33526     ,    1.33407     ,    1.32558     ,    1.31455     ,    1.30396     ,    1.30326     ,    1.30132     ,    1.28132     ,    1.25214     ,    1.22963     ,    1.19863     ,    1.18681     ,    1.16872     ,    1.1681     ,    1.1395     ,    1.12682     ,    1.11811     ,    1.11214     ,    1.10753     ,    1.11112     ,    1.13273     ,    1.16496     ,    1.19065     ,    1.28788     ,    1.50115     ,    1.65742     ,    1.69637     ,    1.58992     ,    1.39926     ,    1.28339     ,    1.11549     ,    1.05489     ,    1.16428     ,    1.1464     ,    1.12938     ,    1.10847     ,    1.11543     ,    1.11522     ,    1.12347     ,    1.12087     ,    1.1236     ,    1.13317     ,    1.15399     ,    1.18127     ,    1.17649     ,    1.17022     ,    1.22256     ,    1.30749     ,    1.42325     ,    1.47771     ,    1.42195     ,    1.35172     ,    1.3527     ,    1.40269     ,    1.46894     ,    1.64002     ,    1.78605     ,    1.84562     ,    1.83715     ,    1.64722     ,    1.74294     ,    1.76081     ,    1.68489     ,    1.50278     ,    1.40908     ,    1.30852     ,    1.33635     ,    1.32035     ,    1.10661     ,    1.09188     ,    1.0958     ,    1.09351     ,    1.09463     ,    1.08408     ,    1.06058     ,    1.0893     ,    1.10563     ,    1.14256     ,    1.17513     ,    1.17135     ,    1.03994     ,    1.04259     ,    1.02675     ,    1.03717     ,    1.03193     ,    1.13959     ,    1.12048     ,    0.95456     ,    0.88115     ,    0.87172     ,    0.93308     ,    0.96583     ,    0.94098     ,    0.92563     ,    1.10593     ,    1.05246     ,    0.84536     ,    0.83536     ,    0.812     ,    0.84148     ,    0.87369     ,    0.90535     ,    0.94246     ,    0.95797     ,    0.98474     ,    1.01912     ,    1.05773     ,    1.08465     ,    1.11068     ,    1.13642     ,    1.15922     ,    1.16891     ,    1.19294     ,    1.21418    },
    {    1.15815     ,    1.17249     ,    1.19283     ,    1.22229     ,    1.24585     ,    1.28565     ,    1.31417     ,    1.33035     ,    1.36074     ,    1.37871     ,    1.39     ,    1.39338     ,    1.39081     ,    1.39619     ,    1.39299     ,    1.37852     ,    1.34715     ,    1.31217     ,    1.27575     ,    1.22723     ,    1.17575     ,    1.1143     ,    1.06191     ,    1.04078     ,    1.03208     ,    1.04752     ,    1.05787     ,    1.05438     ,    1.05916     ,    1.06588     ,    1.04239     ,    1.1052     ,    1.10476     ,    1.14273     ,    1.21514     ,    1.0389     ,    1.03743     ,    1.25057     ,    1.50068     ,    1.46629     ,    1.44502     ,    1.31047     ,    1.11861     ,    1.08018     ,    1.08297     ,    1.09147     ,    1.10474     ,    1.09692     ,    1.08924     ,    1.09162     ,    1.09632     ,    1.09017     ,    1.10541     ,    1.10927     ,    1.10667     ,    1.1067     ,    1.12435     ,    1.1194     ,    1.09554     ,    1.19325     ,    1.32364     ,    1.31817     ,    1.383     ,    1.46969     ,    1.49805     ,    1.47417     ,    1.46425     ,    1.46149     ,    1.51839     ,    1.74311     ,    1.84471     ,    1.69355     ,    1.65412     ,    1.6967     ,    1.63112     ,    1.61414     ,    1.60848     ,    1.44044     ,    1.69744     ,    1.36398     ,    1.29588     ,    1.36933     ,    1.20478     ,    1.14718     ,    1.14694     ,    1.14672     ,    1.12162     ,    1.10125     ,    1.05401     ,    1.06808     ,    1.02173     ,    1.10588     ,    1.24357     ,    1.14276     ,    1.02706     ,    1.00276     ,    1.0286     ,    1.09482     ,    1.08956     ,    1.29209     ,    1.24864     ,    1.20916     ,    1.24119     ,    0.92387     ,    0.92398     ,    0.96773     ,    0.99541     ,    1.07658     ,    1.25024     ,    0.97911     ,    0.86702     ,    0.83578     ,    0.81581     ,    0.80644     ,    0.78304     ,    0.79526     ,    0.82159     ,    0.85752     ,    0.89482     ,    0.92236     ,    0.96255     ,    0.98998     ,    1.01537     ,    1.03991     ,    1.06687     ,    1.0841     ,    1.11218     ,    1.13292    },
    {    1.23248     ,    1.27228     ,    1.31669     ,    1.35846     ,    1.39687     ,    1.44987     ,    1.49668     ,    1.54448     ,    1.58218     ,    1.5985     ,    1.6059     ,    1.59792     ,    1.58026     ,    1.54571     ,    1.49398     ,    1.43269     ,    1.36934     ,    1.28065     ,    1.19667     ,    1.13203     ,    1.08478     ,    1.04671     ,    1.01877     ,    0.99529     ,    0.98738     ,    0.99066     ,    0.97271     ,    0.99665     ,    1.04219     ,    1.06656     ,    1.04642     ,    1.10133     ,    1.09731     ,    1.25641     ,    1.3051     ,    0.90499     ,    0.86038     ,    0.89675     ,    0.89123     ,    0.89106     ,    0.93104     ,    0.94198     ,    0.99946     ,    1.04315     ,    1.05293     ,    1.04365     ,    1.04724     ,    1.05851     ,    1.07846     ,    1.0923     ,    1.09337     ,    1.09668     ,    1.09403     ,    1.08612     ,    1.06628     ,    1.07489     ,    1.11391     ,    1.10482     ,    1.11813     ,    1.28515     ,    1.36025     ,    1.38593     ,    1.39264     ,    1.43883     ,    1.48946     ,    1.49848     ,    1.54477     ,    1.50849     ,    1.60978     ,    1.68146     ,    1.59865     ,    1.63258     ,    1.54819     ,    1.45453     ,    1.4377     ,    1.45994     ,    1.63483     ,    1.56928     ,    1.42688     ,    1.06146     ,    1.11922     ,    1.09732     ,    1.07812     ,    1.14881     ,    1.16108     ,    1.18416     ,    1.18613     ,    1.15928     ,    1.08857     ,    1.07085     ,    1.0241     ,    1.29261     ,    1.3668     ,    1.06902     ,    1.04175     ,    1.02646     ,    1.03958     ,    1.11444     ,    1.12679     ,    1.37419     ,    1.50288     ,    1.53816     ,    1.49142     ,    1.11975     ,    0.95314     ,    1.00272     ,    1.0836     ,    1.18575     ,    1.00226     ,    0.93296     ,    0.95655     ,    0.9053     ,    0.88643     ,    0.85638     ,    0.84147     ,    0.85538     ,    0.88795     ,    0.90974     ,    0.93524     ,    0.97674     ,    1.007     ,    1.03137     ,    1.05837     ,    1.07857     ,    1.10461     ,    1.14049     ,    1.16996     ,    1.19559    },
    {    1.23766     ,    1.29275     ,    1.35845     ,    1.43109     ,    1.51766     ,    1.59714     ,    1.67897     ,    1.75502     ,    1.8047     ,    1.84471     ,    1.85658     ,    1.8377     ,    1.80934     ,    1.76217     ,    1.70394     ,    1.61929     ,    1.52646     ,    1.42075     ,    1.31834     ,    1.21626     ,    1.1297     ,    1.05229     ,    0.99923     ,    0.96882     ,    0.96247     ,    0.96325     ,    0.97526     ,    1.04862     ,    1.05847     ,    1.16039     ,    1.21512     ,    1.18569     ,    1.19724     ,    1.28792     ,    1.10097     ,    0.91812     ,    0.92946     ,    0.9553     ,    0.94172     ,    0.93965     ,    0.94499     ,    0.96353     ,    0.98948     ,    1.01047     ,    1.02836     ,    1.03696     ,    1.04638     ,    1.04695     ,    1.05524     ,    1.05506     ,    1.05322     ,    1.04993     ,    1.03371     ,    1.03978     ,    1.05089     ,    1.07537     ,    1.11982     ,    1.12988     ,    1.18977     ,    1.44003     ,    1.60135     ,    1.67196     ,    1.67076     ,    1.73307     ,    1.73935     ,    1.59663     ,    1.56185     ,    1.50247     ,    1.42325     ,    1.43256     ,    1.42968     ,    1.51029     ,    1.38207     ,    1.13428     ,    1.14989     ,    1.23084     ,    1.3142     ,    1.42616     ,    1.06939     ,    0.91958     ,    1.51194     ,    1.33972     ,    1.21804     ,    1.10627     ,    1.13235     ,    1.17909     ,    1.20431     ,    1.21898     ,    1.20444     ,    1.14627     ,    1.11779     ,    1.40994     ,    1.31345     ,    1.05267     ,    0.97556     ,    1.00819     ,    1.05971     ,    1.13459     ,    1.24031     ,    1.51026     ,    1.71582     ,    1.80703     ,    1.59307     ,    1.19347     ,    1.01801     ,    0.9935     ,    1.09039     ,    1.23336     ,    1.08232     ,    1.03814     ,    1.05218     ,    1.01443     ,    1.02523     ,    0.99196     ,    0.99837     ,    0.9927     ,    1.00671     ,    1.02143     ,    1.01665     ,    1.02021     ,    1.02632     ,    1.04727     ,    1.06391     ,    1.08916     ,    1.12341     ,    1.15643     ,    1.18257     ,    1.21234    },
    {    1.22297     ,    1.24565     ,    1.28836     ,    1.34037     ,    1.39095     ,    1.4487     ,    1.52788     ,    1.61425     ,    1.70181     ,    1.78241     ,    1.86071     ,    1.92121     ,    1.97504     ,    2.01761     ,    2.03635     ,    2.02573     ,    1.98593     ,    1.90292     ,    1.79215     ,    1.64042     ,    1.47241     ,    1.27051     ,    1.10928     ,    1.02671     ,    0.96631     ,    0.9236     ,    0.91805     ,    1.08312     ,    1.19532     ,    1.27412     ,    1.21286     ,    1.0369     ,    1.09692     ,    0.98724     ,    0.9039     ,    0.93492     ,    0.97433     ,    0.96432     ,    0.97557     ,    0.92802     ,    0.8653     ,    0.81468     ,    0.80252     ,    0.79751     ,    0.79557     ,    0.80729     ,    0.83236     ,    0.85559     ,    0.88377     ,    0.92753     ,    0.95089     ,    0.94366     ,    0.95521     ,    0.97582     ,    0.98492     ,    1.01931     ,    1.05723     ,    1.0441     ,    1.11842     ,    1.51043     ,    1.73844     ,    1.76727     ,    1.83759     ,    1.89025     ,    1.83285     ,    1.66935     ,    1.50964     ,    1.40215     ,    1.209     ,    1.12611     ,    1.17273     ,    1.15586     ,    1.08684     ,    0.9946     ,    1.03913     ,    1.06628     ,    1.01488     ,    0.92997     ,    0.6408     ,    0.92932     ,    1.42807     ,    1.38577     ,    1.3352     ,    1.31787     ,    1.18656     ,    1.15855     ,    1.18515     ,    1.21637     ,    1.23711     ,    1.16859     ,    1.22018     ,    1.37851     ,    1.36137     ,    1.41004     ,    1.1318     ,    1.00173     ,    1.00887     ,    1.17286     ,    1.48491     ,    1.62086     ,    1.78925     ,    1.955     ,    1.58923     ,    1.3003     ,    1.10983     ,    1.0839     ,    1.10861     ,    1.19162     ,    1.17099     ,    1.18953     ,    1.20726     ,    1.19463     ,    1.20559     ,    1.17609     ,    1.1802     ,    1.1778     ,    1.17666     ,    1.17073     ,    1.14173     ,    1.11278     ,    1.09514     ,    1.08663     ,    1.09339     ,    1.11427     ,    1.14632     ,    1.19462     ,    1.21959     ,    1.22943    },
    {    1.24181     ,    1.26498     ,    1.30973     ,    1.37795     ,    1.42384     ,    1.44315     ,    1.45876     ,    1.48709     ,    1.52557     ,    1.56353     ,    1.63772     ,    1.72321     ,    1.81415     ,    1.90289     ,    1.98007     ,    2.04949     ,    2.06705     ,    2.01317     ,    1.96529     ,    1.90026     ,    1.77747     ,    1.60382     ,    1.44282     ,    1.22837     ,    0.95219     ,    0.83015     ,    0.85425     ,    1.05673     ,    1.12178     ,    1.06288     ,    0.86346     ,    0.75055     ,    0.80848     ,    0.90753     ,    0.94829     ,    1.01333     ,    1.08107     ,    1.00756     ,    0.95028     ,    1.00404     ,    1.00372     ,    0.94125     ,    0.91604     ,    0.87716     ,    0.81694     ,    0.79112     ,    0.7462     ,    0.67879     ,    0.62758     ,    0.62475     ,    0.63749     ,    0.688     ,    0.74335     ,    0.80715     ,    0.86151     ,    0.88307     ,    0.90458     ,    0.88163     ,    0.94984     ,    1.5029     ,    1.79196     ,    1.79898     ,    1.87406     ,    1.91602     ,    1.86766     ,    1.76383     ,    1.55764     ,    1.37625     ,    1.20154     ,    1.04198     ,    0.96511     ,    0.88174     ,    0.87833     ,    0.85306     ,    0.84577     ,    0.84764     ,    0.83805     ,    0.70462     ,    0.83784     ,    0.97234     ,    1.08883     ,    1.17881     ,    1.2522     ,    1.25287     ,    1.23361     ,    1.15536     ,    1.15779     ,    1.16111     ,    1.20052     ,    1.29151     ,    1.46064     ,    1.52325     ,    1.63643     ,    1.69689     ,    1.59679     ,    1.32371     ,    1.0966     ,    1.27594     ,    1.51137     ,    1.52538     ,    1.6719     ,    1.91212     ,    1.94762     ,    1.76933     ,    1.43789     ,    1.27124     ,    1.17238     ,    1.15783     ,    1.21288     ,    1.32756     ,    1.3871     ,    1.34595     ,    1.31697     ,    1.33657     ,    1.35131     ,    1.35904     ,    1.35989     ,    1.33651     ,    1.30928     ,    1.26875     ,    1.23058     ,    1.19729     ,    1.19149     ,    1.19056     ,    1.1952     ,    1.21341     ,    1.21507     ,    1.22769    },
    {    1.26282     ,    1.27858     ,    1.31653     ,    1.37803     ,    1.43657     ,    1.48293     ,    1.51313     ,    1.52455     ,    1.53282     ,    1.53606     ,    1.54519     ,    1.57185     ,    1.62533     ,    1.67993     ,    1.73721     ,    1.78371     ,    1.8181     ,    1.85315     ,    1.86863     ,    1.85432     ,    1.78789     ,    1.70195     ,    1.58845     ,    1.30312     ,    0.92625     ,    0.85165     ,    1.03683     ,    1.03153     ,    0.94292     ,    0.90147     ,    0.82357     ,    0.76464     ,    0.78724     ,    0.90362     ,    0.95504     ,    0.9108     ,    0.96123     ,    0.9607     ,    0.90915     ,    0.91674     ,    1.16664     ,    1.33243     ,    1.41988     ,    1.45729     ,    1.46475     ,    1.47617     ,    1.45778     ,    1.3569     ,    1.19764     ,    1.04278     ,    0.92425     ,    0.81706     ,    0.71315     ,    0.67359     ,    0.69201     ,    0.70502     ,    0.7449     ,    0.72503     ,    0.75848     ,    1.22491     ,    1.6292     ,    1.6784     ,    1.68321     ,    1.80528     ,    1.84774     ,    1.80916     ,    1.76683     ,    1.44835     ,    1.20775     ,    1.03805     ,    0.89018     ,    0.7957     ,    0.77386     ,    0.88596     ,    0.89329     ,    0.82437     ,    0.83897     ,    0.64997     ,    0.91416     ,    0.93021     ,    0.93103     ,    1.06421     ,    1.1076     ,    1.0383     ,    1.00841     ,    0.9973     ,    1.04195     ,    1.09299     ,    1.21368     ,    1.46422     ,    1.50801     ,    1.47512     ,    1.53924     ,    1.55755     ,    1.73955     ,    1.74505     ,    1.42505     ,    1.38093     ,    1.40471     ,    1.34181     ,    1.37337     ,    1.58941     ,    1.84303     ,    1.90219     ,    1.80241     ,    1.58281     ,    1.3201     ,    1.16626     ,    1.14748     ,    1.24391     ,    1.35901     ,    1.39819     ,    1.37686     ,    1.38997     ,    1.42459     ,    1.47783     ,    1.5176     ,    1.51268     ,    1.49253     ,    1.45223     ,    1.40644     ,    1.36604     ,    1.3412     ,    1.32033     ,    1.31185     ,    1.2906     ,    1.27192     ,    1.27326    },
    {    1.30624     ,    1.33615     ,    1.35397     ,    1.37685     ,    1.40973     ,    1.43204     ,    1.44824     ,    1.44804     ,    1.43914     ,    1.44451     ,    1.45537     ,    1.4915     ,    1.53614     ,    1.54613     ,    1.57687     ,    1.6044     ,    1.64643     ,    1.69923     ,    1.71851     ,    1.73905     ,    1.72429     ,    1.64861     ,    1.53602     ,    1.31793     ,    1.19147     ,    1.17637     ,    1.10158     ,    0.93407     ,    0.86711     ,    0.86816     ,    0.83973     ,    0.83196     ,    0.83182     ,    0.85916     ,    0.91418     ,    0.97988     ,    0.91924     ,    0.94277     ,    0.95217     ,    0.93948     ,    1.13656     ,    1.36902     ,    1.43429     ,    1.48831     ,    1.56731     ,    1.63196     ,    1.68078     ,    1.68962     ,    1.65475     ,    1.63039     ,    1.56446     ,    1.43207     ,    1.25999     ,    1.0478     ,    0.85353     ,    0.71203     ,    0.64604     ,    0.64428     ,    0.66313     ,    0.8631     ,    1.24537     ,    1.47714     ,    1.48714     ,    1.60326     ,    1.75277     ,    1.71776     ,    1.6153     ,    1.47585     ,    1.28047     ,    1.08156     ,    0.94251     ,    0.83862     ,    0.78075     ,    0.80578     ,    0.82336     ,    0.82026     ,    0.89197     ,    0.8184     ,    0.89026     ,    0.93347     ,    0.94565     ,    1.02008     ,    0.83498     ,    0.8599     ,    0.94187     ,    0.97994     ,    1.0545     ,    1.2092     ,    1.32619     ,    1.39065     ,    1.45425     ,    1.43423     ,    1.48395     ,    1.48298     ,    1.62059     ,    1.52347     ,    1.20712     ,    1.1757     ,    1.19232     ,    1.1162     ,    1.06865     ,    1.55004     ,    1.8165     ,    1.70054     ,    1.67718     ,    1.61056     ,    1.47956     ,    1.26864     ,    1.17433     ,    1.2039     ,    1.23448     ,    1.35134     ,    1.4258     ,    1.44418     ,    1.47407     ,    1.50146     ,    1.5342     ,    1.58229     ,    1.61834     ,    1.61666     ,    1.57485     ,    1.51271     ,    1.47088     ,    1.44073     ,    1.4041     ,    1.34539     ,    1.30688     ,    1.30055    },
    {    1.22825     ,    1.2461     ,    1.26212     ,    1.29534     ,    1.31937     ,    1.32677     ,    1.3285     ,    1.31435     ,    1.28103     ,    1.2611     ,    1.26942     ,    1.30887     ,    1.36023     ,    1.38931     ,    1.42001     ,    1.44968     ,    1.49106     ,    1.55838     ,    1.60129     ,    1.61373     ,    1.59507     ,    1.55364     ,    1.4739     ,    1.4056     ,    1.36714     ,    1.32661     ,    1.16512     ,    1.0212     ,    1.04188     ,    1.00945     ,    0.85     ,    0.83958     ,    0.85661     ,    0.84662     ,    0.92442     ,    1.1425     ,    1.04789     ,    0.946     ,    1.00002     ,    0.9876     ,    1.02277     ,    1.05799     ,    1.038     ,    1.05105     ,    1.07837     ,    1.08615     ,    1.09203     ,    1.11887     ,    1.16409     ,    1.23729     ,    1.32241     ,    1.38411     ,    1.3854     ,    1.28206     ,    1.10196     ,    0.87256     ,    0.68048     ,    0.60309     ,    0.57619     ,    0.58456     ,    0.66448     ,    1.34023     ,    1.57325     ,    1.44577     ,    1.49489     ,    1.50459     ,    1.40571     ,    1.38379     ,    1.26315     ,    1.03074     ,    0.90578     ,    0.83311     ,    0.80007     ,    0.75462     ,    0.75561     ,    0.80003     ,    0.89618     ,    0.96008     ,    0.94403     ,    1.01662     ,    1.04453     ,    0.90217     ,    0.78883     ,    0.97166     ,    1.06685     ,    1.01677     ,    0.99219     ,    1.06182     ,    1.0909     ,    1.24131     ,    1.35827     ,    1.36117     ,    1.34453     ,    1.31562     ,    1.30159     ,    1.07976     ,    0.95     ,    1.05621     ,    1.11484     ,    0.89195     ,    0.98536     ,    1.76586     ,    2.01494     ,    1.68928     ,    1.63377     ,    1.60476     ,    1.50506     ,    1.30864     ,    1.15713     ,    1.26308     ,    1.28613     ,    1.29274     ,    1.3541     ,    1.42986     ,    1.4784     ,    1.50288     ,    1.51458     ,    1.5299     ,    1.54446     ,    1.55541     ,    1.55071     ,    1.52556     ,    1.49182     ,    1.43943     ,    1.36606     ,    1.27675     ,    1.21015     ,    1.19982    },
    {    0.99317     ,    1.01594     ,    1.05357     ,    1.08991     ,    1.12044     ,    1.15003     ,    1.1667     ,    1.15699     ,    1.1336     ,    1.11654     ,    1.12841     ,    1.16783     ,    1.21689     ,    1.26428     ,    1.30229     ,    1.33505     ,    1.37951     ,    1.45248     ,    1.50146     ,    1.48862     ,    1.46763     ,    1.43329     ,    1.34844     ,    1.28353     ,    1.21754     ,    1.12406     ,    1.08423     ,    1.07926     ,    1.12688     ,    1.12331     ,    1.02456     ,    0.90984     ,    0.87632     ,    0.84437     ,    0.8336     ,    1.00949     ,    1.20193     ,    0.99019     ,    0.89294     ,    0.9022     ,    0.92402     ,    0.94421     ,    0.94949     ,    0.92453     ,    0.88158     ,    0.83713     ,    0.77573     ,    0.72556     ,    0.6839     ,    0.6453     ,    0.64895     ,    0.7174     ,    0.77668     ,    0.80135     ,    0.79785     ,    0.7315     ,    0.64817     ,    0.59722     ,    0.53541     ,    0.45534     ,    0.4214     ,    0.92348     ,    1.53472     ,    1.59511     ,    1.57335     ,    1.52798     ,    1.29656     ,    1.15687     ,    1.04849     ,    1.00447     ,    0.99692     ,    0.94416     ,    0.84694     ,    0.72488     ,    0.64351     ,    0.67468     ,    0.71516     ,    0.82163     ,    0.94061     ,    0.9979     ,    1.0327     ,    1.01235     ,    1.01031     ,    1.07111     ,    1.0352     ,    0.99073     ,    1.00232     ,    1.02286     ,    0.97191     ,    1.14556     ,    1.25806     ,    1.33721     ,    1.19238     ,    0.94502     ,    0.96349     ,    1.08736     ,    1.0221     ,    1.04752     ,    1.11756     ,    0.95797     ,    1.11424     ,    1.59043     ,    1.78104     ,    1.56864     ,    1.61841     ,    1.62776     ,    1.50035     ,    1.33622     ,    1.22413     ,    1.31682     ,    1.34553     ,    1.29152     ,    1.30555     ,    1.3682     ,    1.40926     ,    1.45042     ,    1.45709     ,    1.42944     ,    1.42136     ,    1.40215     ,    1.38603     ,    1.37513     ,    1.34437     ,    1.2987     ,    1.23591     ,    1.14208     ,    1.04703     ,    0.99757    },
    {    0.73136     ,    0.74135     ,    0.77448     ,    0.80959     ,    0.85503     ,    0.89826     ,    0.914     ,    0.92046     ,    0.93608     ,    0.94633     ,    0.97713     ,    1.03665     ,    1.09492     ,    1.14788     ,    1.19626     ,    1.23942     ,    1.27285     ,    1.35474     ,    1.42681     ,    1.40369     ,    1.34648     ,    1.30065     ,    1.22394     ,    1.15368     ,    1.08448     ,    1.05923     ,    1.07074     ,    0.96054     ,    0.86136     ,    0.92195     ,    0.97668     ,    0.88932     ,    0.81645     ,    0.81169     ,    0.82232     ,    0.88184     ,    0.99763     ,    1.05869     ,    0.87246     ,    0.8084     ,    0.79445     ,    0.78933     ,    0.7928     ,    0.77013     ,    0.73109     ,    0.70388     ,    0.64602     ,    0.55812     ,    0.47238     ,    0.39961     ,    0.33388     ,    0.27601     ,    0.24248     ,    0.24991     ,    0.28017     ,    0.29718     ,    0.3293     ,    0.39715     ,    0.47721     ,    0.43511     ,    0.30931     ,    0.43094     ,    0.91501     ,    1.10257     ,    1.28574     ,    1.47541     ,    1.35993     ,    1.12885     ,    0.89108     ,    0.93828     ,    0.96854     ,    0.97043     ,    0.86639     ,    0.66414     ,    0.57202     ,    0.63824     ,    0.64834     ,    0.74472     ,    0.90212     ,    0.9612     ,    1.02703     ,    1.03162     ,    1.0066     ,    1.0442     ,    1.03551     ,    0.9889     ,    1.00202     ,    1.01705     ,    1.04921     ,    1.05674     ,    1.10415     ,    1.02899     ,    0.79596     ,    1.0419     ,    0.96303     ,    1.02593     ,    1.0559     ,    1.00482     ,    1.02947     ,    1.10476     ,    0.99475     ,    1.26077     ,    1.40413     ,    1.39286     ,    1.5542     ,    1.61124     ,    1.53526     ,    1.40068     ,    1.3284     ,    1.36237     ,    1.39335     ,    1.38294     ,    1.36386     ,    1.31839     ,    1.32561     ,    1.39512     ,    1.39632     ,    1.33564     ,    1.29907     ,    1.25279     ,    1.20987     ,    1.18116     ,    1.13668     ,    1.08255     ,    1.01108     ,    0.9126     ,    0.81778     ,    0.75884    },
    {    0.48982     ,    0.50956     ,    0.54958     ,    0.57686     ,    0.58987     ,    0.60907     ,    0.62767     ,    0.66564     ,    0.71862     ,    0.7734     ,    0.83834     ,    0.9041     ,    0.9717     ,    1.04448     ,    1.10689     ,    1.14875     ,    1.20115     ,    1.28914     ,    1.37505     ,    1.354     ,    1.2827     ,    1.24087     ,    1.2363     ,    1.1081     ,    1.0254     ,    1.04913     ,    1.00663     ,    0.8024     ,    0.76756     ,    0.85234     ,    0.87031     ,    0.82281     ,    0.81063     ,    0.8425     ,    0.87084     ,    0.88449     ,    0.92238     ,    1.06684     ,    0.81658     ,    0.63471     ,    0.57468     ,    0.54435     ,    0.49734     ,    0.4635     ,    0.41597     ,    0.35298     ,    0.30462     ,    0.26953     ,    0.21476     ,    0.13423     ,    0.04476     ,    -0.04334     ,    -0.10775     ,    -0.1437     ,    -0.17616     ,    -0.17079     ,    -0.09054     ,    0.06392     ,    0.24429     ,    0.30765     ,    0.25654     ,    0.43092     ,    0.74576     ,    0.77831     ,    0.72301     ,    0.91534     ,    0.95542     ,    0.87383     ,    0.72812     ,    0.77073     ,    0.91388     ,    0.98813     ,    0.91866     ,    0.6315     ,    0.65633     ,    0.83556     ,    0.85336     ,    0.88188     ,    0.96925     ,    0.96421     ,    0.98321     ,    0.93368     ,    0.74139     ,    0.85065     ,    1.00474     ,    0.99562     ,    0.96397     ,    0.98101     ,    0.98045     ,    0.95347     ,    0.8993     ,    1.02134     ,    1.1138     ,    1.16271     ,    1.15871     ,    1.10339     ,    1.20019     ,    1.23011     ,    1.09717     ,    1.12093     ,    1.1154     ,    1.16026     ,    1.1897     ,    1.26347     ,    1.45912     ,    1.55738     ,    1.48479     ,    1.44743     ,    1.42084     ,    1.44348     ,    1.57852     ,    1.49539     ,    1.32839     ,    1.1621     ,    1.14151     ,    1.40474     ,    1.45972     ,    1.32931     ,    1.20367     ,    1.11663     ,    1.02265     ,    0.94462     ,    0.8807     ,    0.81303     ,    0.73499     ,    0.64994     ,    0.5623     ,    0.50865    },
    {    0.3123     ,    0.29445     ,    0.30055     ,    0.3213     ,    0.34554     ,    0.37055     ,    0.38924     ,    0.428     ,    0.49557     ,    0.57962     ,    0.67148     ,    0.76549     ,    0.86951     ,    0.97778     ,    1.0653     ,    1.10671     ,    1.1389     ,    1.19659     ,    1.30182     ,    1.37707     ,    1.32497     ,    1.28343     ,    1.27626     ,    1.11603     ,    1.03197     ,    1.02615     ,    0.93401     ,    0.92235     ,    1.03847     ,    1.00784     ,    0.93032     ,    0.97741     ,    1.0351     ,    1.01501     ,    1.04116     ,    1.09966     ,    1.12236     ,    1.11865     ,    0.91037     ,    0.48346     ,    0.27598     ,    0.13319     ,    -0.04286     ,    -0.14459     ,    -0.25137     ,    -0.48124     ,    -0.64478     ,    -0.58028     ,    -0.47256     ,    -0.41868     ,    -0.4509     ,    -0.54558     ,    -0.59292     ,    -0.61471     ,    -0.62469     ,    -0.6008     ,    -0.55999     ,    -0.40594     ,    -0.16969     ,    0.03071     ,    0.15493     ,    0.4162     ,    0.8279     ,    0.96469     ,    0.87659     ,    0.77049     ,    0.70424     ,    0.67213     ,    0.64089     ,    0.78551     ,    1.05844     ,    0.9998     ,    1.06594     ,    0.71067     ,    0.57654     ,    0.64828     ,    0.65044     ,    0.75037     ,    0.90315     ,    0.98521     ,    0.97211     ,    0.85098     ,    0.60537     ,    0.77356     ,    1.003     ,    1.00277     ,    1.01805     ,    1.02753     ,    0.97234     ,    0.94796     ,    0.94705     ,    1.05917     ,    1.07792     ,    1.15132     ,    1.13603     ,    1.09414     ,    1.12911     ,    1.19832     ,    1.14899     ,    1.12089     ,    1.11526     ,    1.0894     ,    1.08481     ,    1.18788     ,    1.32448     ,    1.27573     ,    1.24574     ,    1.23867     ,    1.20064     ,    1.20186     ,    1.46746     ,    1.4828     ,    1.28046     ,    1.11824     ,    1.03641     ,    1.31857     ,    1.48413     ,    1.34117     ,    1.14989     ,    0.97953     ,    0.79244     ,    0.66901     ,    0.59597     ,    0.5308     ,    0.48107     ,    0.45024     ,    0.3962     ,    0.34774    },
    {    0.23217     ,    0.20039     ,    0.18457     ,    0.18599     ,    0.19672     ,    0.21395     ,    0.23559     ,    0.28906     ,    0.36593     ,    0.44725     ,    0.54163     ,    0.65682     ,    0.76398     ,    0.85562     ,    0.94451     ,    1.00824     ,    1.05474     ,    1.14579     ,    1.29561     ,    1.4045     ,    1.39155     ,    1.33839     ,    1.28403     ,    1.21492     ,    1.08181     ,    0.98535     ,    0.96034     ,    1.02036     ,    1.08762     ,    1.09287     ,    1.14533     ,    1.31114     ,    1.39543     ,    1.3743     ,    1.2949     ,    1.33535     ,    1.29928     ,    1.40668     ,    1.44091     ,    1.20175     ,    0.83011     ,    0.37202     ,    -0.18237     ,    -0.54617     ,    -0.7563     ,    -0.8147     ,    -0.96483     ,    -1.76629     ,    -2.06693     ,    -1.9495     ,    -1.65261     ,    -1.5373     ,    -1.48662     ,    -1.43882     ,    -1.29066     ,    -1.05736     ,    -0.93762     ,    -0.80892     ,    -0.55239     ,    -0.21189     ,    0.08411     ,    0.23392     ,    0.41079     ,    0.62483     ,    0.90577     ,    0.93492     ,    0.91568     ,    0.93943     ,    1.01449     ,    1.06489     ,    1.00101     ,    1.11784     ,    1.21263     ,    1.12634     ,    0.83315     ,    0.55091     ,    0.45981     ,    0.51301     ,    0.74418     ,    0.91086     ,    0.9581     ,    0.79453     ,    0.64044     ,    0.86693     ,    0.95974     ,    0.94743     ,    0.98626     ,    1.01453     ,    0.99019     ,    1.0477     ,    1.10685     ,    1.04339     ,    1.09026     ,    1.03995     ,    0.92158     ,    0.87811     ,    1.02744     ,    1.12888     ,    1.10537     ,    1.0052     ,    0.92528     ,    1.02351     ,    1.07482     ,    1.10674     ,    1.11997     ,    0.98875     ,    0.96366     ,    1.02637     ,    0.97508     ,    0.96442     ,    1.07968     ,    1.09894     ,    1.01574     ,    0.99211     ,    1.04018     ,    0.98097     ,    1.13118     ,    0.91979     ,    0.76304     ,    0.6453     ,    0.52118     ,    0.4792     ,    0.46161     ,    0.39796     ,    0.36472     ,    0.35426     ,    0.30348     ,    0.2583    },
    {    0.24439     ,    0.20926     ,    0.18972     ,    0.17489     ,    0.15912     ,    0.16475     ,    0.19722     ,    0.25418     ,    0.31967     ,    0.38563     ,    0.47596     ,    0.58399     ,    0.66339     ,    0.73184     ,    0.82671     ,    0.92377     ,    1.01345     ,    1.16703     ,    1.34709     ,    1.43505     ,    1.45736     ,    1.46566     ,    1.41668     ,    1.27016     ,    1.14637     ,    1.15667     ,    1.20542     ,    1.2243     ,    1.22074     ,    1.25841     ,    1.294     ,    1.37047     ,    1.37515     ,    1.30563     ,    1.26867     ,    1.25349     ,    1.33217     ,    1.42602     ,    1.47845     ,    1.2927     ,    1.19715     ,    1.05577     ,    0.76686     ,    0.32373     ,    -0.08846     ,    -0.40964     ,    -0.79102     ,    -1.63388     ,    -2.80649     ,    -3.63069     ,    -3.75103     ,    -3.31547     ,    -2.88393     ,    -2.53486     ,    -2.05124     ,    -1.53466     ,    -1.21037     ,    -0.97     ,    -0.67943     ,    -0.30037     ,    0.08026     ,    0.27753     ,    0.40735     ,    0.59981     ,    0.9276     ,    1.0362     ,    1.11206     ,    1.17015     ,    1.24816     ,    1.27483     ,    1.27306     ,    1.27755     ,    1.31151     ,    1.30944     ,    1.15934     ,    0.70863     ,    0.68376     ,    0.60835     ,    0.82878     ,    0.90273     ,    0.88908     ,    0.67214     ,    0.65715     ,    0.76335     ,    0.87668     ,    0.88677     ,    0.93572     ,    0.96903     ,    0.97008     ,    1.02712     ,    1.07067     ,    1.1278     ,    1.16355     ,    1.14576     ,    1.05521     ,    0.93508     ,    0.90937     ,    0.94055     ,    0.85242     ,    0.79296     ,    0.89533     ,    1.02588     ,    1.07872     ,    1.06454     ,    1.05192     ,    0.89355     ,    0.85745     ,    0.89977     ,    0.89128     ,    0.8294     ,    0.74635     ,    0.67718     ,    0.70422     ,    0.87586     ,    1.06229     ,    1.0607     ,    0.85053     ,    0.64831     ,    0.55808     ,    0.49207     ,    0.4518     ,    0.46081     ,    0.44805     ,    0.40883     ,    0.3885     ,    0.35556     ,    0.29823     ,    0.26518    },
    {    0.41049     ,    0.36931     ,    0.35126     ,    0.35178     ,    0.3555     ,    0.36619     ,    0.37694     ,    0.40866     ,    0.46254     ,    0.51585     ,    0.5728     ,    0.63802     ,    0.69781     ,    0.76409     ,    0.85423     ,    0.94713     ,    1.0593     ,    1.2506     ,    1.42431     ,    1.48373     ,    1.4931     ,    1.53196     ,    1.50477     ,    1.36325     ,    1.36992     ,    1.36138     ,    1.35149     ,    1.34636     ,    1.32332     ,    1.26661     ,    1.27444     ,    1.28732     ,    1.28432     ,    1.29441     ,    1.32895     ,    1.34503     ,    1.32605     ,    1.2676     ,    1.2375     ,    1.18589     ,    1.08341     ,    1.10066     ,    0.8701     ,    0.64078     ,    0.18051     ,    -0.83647     ,    -1.12949     ,    -1.17928     ,    -1.73071     ,    -3.17748     ,    -4.38132     ,    -4.50389     ,    -4.03928     ,    -3.33959     ,    -2.50864     ,    -1.72811     ,    -1.21845     ,    -0.85903     ,    -0.51672     ,    -0.14374     ,    0.19146     ,    0.3823     ,    0.53988     ,    0.71874     ,    0.97883     ,    1.03917     ,    1.09352     ,    1.19006     ,    1.25952     ,    1.33898     ,    1.43498     ,    1.42321     ,    1.33714     ,    1.30357     ,    1.22586     ,    1.02897     ,    0.85151     ,    0.84831     ,    0.89758     ,    0.88233     ,    0.8252     ,    0.76469     ,    0.72914     ,    0.82121     ,    0.89576     ,    0.9342     ,    0.95239     ,    0.98284     ,    1.00429     ,    1.03001     ,    1.05923     ,    1.07755     ,    1.08821     ,    1.12622     ,    0.98813     ,    0.91104     ,    0.95933     ,    1.01401     ,    0.95119     ,    0.93582     ,    0.99042     ,    1.06236     ,    0.97074     ,    0.9163     ,    0.78446     ,    0.76751     ,    0.73908     ,    0.68026     ,    0.54939     ,    0.45611     ,    0.38124     ,    0.40078     ,    0.56041     ,    0.6661     ,    0.84141     ,    0.58241     ,    0.53706     ,    0.59621     ,    0.58099     ,    0.59482     ,    0.65565     ,    0.68309     ,    0.65815     ,    0.58838     ,    0.53105     ,    0.48018     ,    0.44463     ,    0.43199    },
    {    0.57629     ,    0.55623     ,    0.56711     ,    0.62054     ,    0.69989     ,    0.73395     ,    0.72016     ,    0.6998     ,    0.69887     ,    0.71371     ,    0.74186     ,    0.78985     ,    0.83983     ,    0.8838     ,    0.95891     ,    1.10176     ,    1.28416     ,    1.43677     ,    1.50555     ,    1.56921     ,    1.60912     ,    1.52871     ,    1.46807     ,    1.45521     ,    1.44106     ,    1.35476     ,    1.26244     ,    1.24476     ,    1.15633     ,    1.08923     ,    1.09543     ,    1.1753     ,    1.22599     ,    1.3016     ,    1.39554     ,    1.39957     ,    1.33436     ,    1.28547     ,    1.21824     ,    1.03532     ,    0.80293     ,    0.68367     ,    0.61047     ,    0.29854     ,    -0.55495     ,    -1.3738     ,    -0.89998     ,    -0.50227     ,    -0.99673     ,    -2.40823     ,    -3.58461     ,    -3.91677     ,    -3.7534     ,    -3.12175     ,    -2.27146     ,    -1.47319     ,    -0.89796     ,    -0.58196     ,    -0.30438     ,    -0.00283     ,    0.27374     ,    0.47297     ,    0.61641     ,    0.84928     ,    0.956     ,    1.05804     ,    1.0943     ,    1.19877     ,    1.27603     ,    1.29896     ,    1.33331     ,    1.30802     ,    1.21231     ,    1.15353     ,    1.10847     ,    1.05307     ,    0.99007     ,    0.93292     ,    0.91327     ,    0.86027     ,    0.78888     ,    0.74459     ,    0.74936     ,    0.84876     ,    0.96174     ,    0.97067     ,    0.97765     ,    0.98098     ,    0.98853     ,    0.94922     ,    0.98112     ,    1.02306     ,    1.10375     ,    1.11288     ,    0.98724     ,    0.88743     ,    0.86501     ,    0.89234     ,    0.87753     ,    0.81416     ,    0.86642     ,    0.88658     ,    0.87391     ,    0.83627     ,    0.7678     ,    0.74237     ,    0.79905     ,    0.65732     ,    0.43004     ,    0.18554     ,    -0.02507     ,    0.10307     ,    0.33985     ,    0.37144     ,    0.27084     ,    0.2084     ,    0.45238     ,    0.83799     ,    0.88409     ,    0.83192     ,    0.77818     ,    0.84397     ,    0.80329     ,    0.73136     ,    0.67034     ,    0.65051     ,    0.64251     ,    0.61611    },
    {    0.79521     ,    0.82888     ,    0.87659     ,    0.94006     ,    0.99742     ,    1.02644     ,    1.02717     ,    0.9958     ,    0.97799     ,    0.99727     ,    1.04197     ,    1.09352     ,    1.13626     ,    1.18241     ,    1.26805     ,    1.42545     ,    1.56739     ,    1.64694     ,    1.69008     ,    1.7322     ,    1.68015     ,    1.59751     ,    1.56646     ,    1.562     ,    1.48951     ,    1.41238     ,    1.3938     ,    1.35729     ,    1.24377     ,    1.15826     ,    1.10914     ,    1.12281     ,    1.14323     ,    1.2174     ,    1.31546     ,    1.4219     ,    1.40373     ,    1.3281     ,    1.2186     ,    1.00407     ,    0.87573     ,    0.83369     ,    0.47332     ,    -0.641     ,    -1.96838     ,    -1.73286     ,    -1.00322     ,    -0.92296     ,    -1.53609     ,    -2.41122     ,    -2.85997     ,    -2.72623     ,    -2.33483     ,    -1.72596     ,    -1.05289     ,    -0.59388     ,    -0.36683     ,    -0.20209     ,    -0.02614     ,    0.12697     ,    0.27446     ,    0.4827     ,    0.68442     ,    0.85444     ,    0.94368     ,    1.02181     ,    1.0709     ,    1.12362     ,    1.13639     ,    1.09285     ,    1.03088     ,    1.01201     ,    1.00256     ,    1.0059     ,    1.01587     ,    1.01264     ,    0.95756     ,    0.8772     ,    0.82042     ,    0.73314     ,    0.6331     ,    0.5668     ,    0.53194     ,    0.62294     ,    0.777     ,    0.81397     ,    0.8141     ,    0.80034     ,    0.80215     ,    0.83856     ,    0.87266     ,    0.9356     ,    0.99136     ,    1.03249     ,    1.04526     ,    0.9868     ,    0.9124     ,    0.8689     ,    0.88164     ,    0.88151     ,    0.85657     ,    0.83174     ,    0.79099     ,    0.75665     ,    0.68046     ,    0.57014     ,    0.55312     ,    0.49168     ,    0.33458     ,    0.05537     ,    -0.15503     ,    -0.05692     ,    0.05827     ,    -0.0992     ,    -0.30427     ,    -0.1437     ,    0.32912     ,    0.88271     ,    1.1049     ,    1.04651     ,    0.70145     ,    0.75668     ,    1.14847     ,    0.86827     ,    0.82384     ,    0.80299     ,    0.79642     ,    0.78972    },
    {    0.97511     ,    0.99402     ,    1.0703     ,    1.17456     ,    1.25805     ,    1.29567     ,    1.32119     ,    1.36291     ,    1.40875     ,    1.41551     ,    1.39792     ,    1.42017     ,    1.47725     ,    1.56665     ,    1.67852     ,    1.76292     ,    1.81076     ,    1.9632     ,    1.94536     ,    1.77048     ,    1.61342     ,    1.63436     ,    1.69946     ,    1.73365     ,    1.65053     ,    1.48712     ,    1.40685     ,    1.34611     ,    1.28562     ,    1.2159     ,    1.18838     ,    1.25892     ,    1.37869     ,    1.42913     ,    1.49229     ,    1.48951     ,    1.48762     ,    1.33318     ,    1.1955     ,    1.12341     ,    1.16259     ,    0.88524     ,    -0.0114     ,    -1.97226     ,    -3.3433     ,    -3.21229     ,    -2.83485     ,    -2.82237     ,    -2.89811     ,    -2.66521     ,    -2.3739     ,    -2.02206     ,    -1.30577     ,    -0.41759     ,    0.15522     ,    0.17886     ,    0.06768     ,    0.07219     ,    0.20322     ,    0.32376     ,    0.36951     ,    0.44369     ,    0.60017     ,    0.75613     ,    0.90478     ,    1.0188     ,    1.07512     ,    1.10941     ,    1.13971     ,    1.08309     ,    0.98991     ,    0.91092     ,    0.86417     ,    0.88421     ,    0.93671     ,    0.92826     ,    0.83039     ,    0.7179     ,    0.64656     ,    0.56021     ,    0.47256     ,    0.40471     ,    0.38317     ,    0.45507     ,    0.57699     ,    0.68135     ,    0.73921     ,    0.75297     ,    0.77203     ,    0.82658     ,    0.90658     ,    0.98636     ,    1.06261     ,    1.12549     ,    1.16523     ,    1.14065     ,    1.06203     ,    0.95751     ,    0.93513     ,    0.92717     ,    0.85828     ,    0.75228     ,    0.66001     ,    0.60301     ,    0.51337     ,    0.4281     ,    0.30989     ,    0.23713     ,    0.18238     ,    0.11972     ,    0.06516     ,    0.01609     ,    0.02371     ,    -0.1285     ,    -0.33916     ,    -0.2055     ,    0.0012     ,    0.27387     ,    0.50182     ,    0.60525     ,    0.54593     ,    0.70543     ,    1.52377     ,    1.37411     ,    1.2649     ,    1.19916     ,    1.1155     ,    1.02818    },
    {    1.64527     ,    1.64709     ,    1.73999     ,    1.85972     ,    1.90357     ,    1.88545     ,    1.87517     ,    1.93158     ,    1.98841     ,    1.99503     ,    1.92841     ,    1.89413     ,    1.93722     ,    1.94474     ,    1.85482     ,    1.82631     ,    1.83538     ,    1.89949     ,    1.90276     ,    1.72136     ,    1.65943     ,    1.74887     ,    1.87234     ,    1.88236     ,    1.74967     ,    1.56711     ,    1.4652     ,    1.45015     ,    1.43743     ,    1.4679     ,    1.5787     ,    1.57902     ,    1.6513     ,    1.67148     ,    1.67709     ,    1.70658     ,    1.69172     ,    1.48549     ,    1.31445     ,    1.28503     ,    1.2463     ,    0.37888     ,    -1.21989     ,    -3.71542     ,    -5.26003     ,    -5.02463     ,    -4.23982     ,    -3.39222     ,    -2.23259     ,    -1.6857     ,    -1.30548     ,    -1.18902     ,    -0.78592     ,    -0.22517     ,    -0.13158     ,    -0.18236     ,    -0.08338     ,    0.12228     ,    0.32125     ,    0.4116     ,    0.433     ,    0.47947     ,    0.55899     ,    0.65569     ,    0.75894     ,    0.87298     ,    1.04038     ,    1.22298     ,    1.26151     ,    1.1902     ,    1.10785     ,    1.06511     ,    0.93165     ,    0.88355     ,    0.91093     ,    0.88582     ,    0.80278     ,    0.69862     ,    0.62793     ,    0.58416     ,    0.54032     ,    0.52124     ,    0.50355     ,    0.50918     ,    0.55182     ,    0.63111     ,    0.72398     ,    0.78611     ,    0.84035     ,    0.88511     ,    0.93644     ,    0.99223     ,    1.08614     ,    1.11725     ,    1.10929     ,    1.08023     ,    1.02803     ,    0.95234     ,    0.8788     ,    0.79956     ,    0.70748     ,    0.61509     ,    0.55321     ,    0.54165     ,    0.52314     ,    0.48998     ,    0.43986     ,    0.42391     ,    0.3873     ,    0.36059     ,    0.27317     ,    0.17644     ,    0.10304     ,    0.11468     ,    0.13594     ,    0.08255     ,    0.05368     ,    0.09281     ,    0.18882     ,    0.36522     ,    0.51406     ,    0.68711     ,    1.00651     ,    1.14081     ,    1.17949     ,    1.31746     ,    1.51937     ,    1.60724    },
    {    1.89442     ,    2.11491     ,    2.28558     ,    2.39768     ,    2.42388     ,    2.35129     ,    2.3349     ,    2.20134     ,    2.26319     ,    2.26672     ,    2.11818     ,    1.98452     ,    1.89258     ,    1.83539     ,    1.7613     ,    1.68037     ,    1.6605     ,    1.6189     ,    1.64868     ,    1.65707     ,    1.71474     ,    1.88858     ,    2.00482     ,    1.97053     ,    1.86269     ,    1.77592     ,    1.74673     ,    1.71646     ,    1.66374     ,    1.67409     ,    1.73251     ,    1.78151     ,    1.77375     ,    1.78567     ,    1.82438     ,    1.87358     ,    1.77422     ,    1.5797     ,    1.39895     ,    1.27033     ,    0.78036     ,    -0.26246     ,    -1.88149     ,    -4.40052     ,    -5.66077     ,    -5.00395     ,    -2.45002     ,    -1.16387     ,    -0.51655     ,    -0.45969     ,    -0.40916     ,    -0.37438     ,    -0.40784     ,    -0.40606     ,    -0.74727     ,    -0.66661     ,    -0.4035     ,    -0.08643     ,    0.12607     ,    0.20748     ,    0.24946     ,    0.33113     ,    0.4443     ,    0.54439     ,    0.62449     ,    0.69785     ,    0.77503     ,    0.85794     ,    0.83457     ,    0.7872     ,    0.75324     ,    0.84701     ,    0.75418     ,    0.77801     ,    0.80617     ,    0.80632     ,    0.78832     ,    0.76982     ,    0.75114     ,    0.72343     ,    0.70354     ,    0.69495     ,    0.69856     ,    0.67294     ,    0.67102     ,    0.69554     ,    0.73165     ,    0.77408     ,    0.83958     ,    0.88731     ,    0.94394     ,    0.98841     ,    1.01472     ,    0.98686     ,    0.93888     ,    0.88338     ,    0.8438     ,    0.75733     ,    0.68509     ,    0.61396     ,    0.55534     ,    0.5565     ,    0.57409     ,    0.59587     ,    0.62709     ,    0.67566     ,    0.68086     ,    0.65946     ,    0.62723     ,    0.556     ,    0.45763     ,    0.28609     ,    0.17619     ,    0.1796     ,    0.22309     ,    0.20351     ,    0.14406     ,    0.13544     ,    0.25506     ,    0.44853     ,    0.5997     ,    0.69076     ,    0.72323     ,    0.65921     ,    0.62592     ,    0.77476     ,    1.03404     ,    1.42796    },
    {    1.43585     ,    1.7401     ,    2.00141     ,    2.13985     ,    2.21749     ,    2.23971     ,    2.30228     ,    2.18973     ,    2.13591     ,    2.10172     ,    1.987     ,    1.85511     ,    1.82637     ,    1.91303     ,    1.98814     ,    2.0377     ,    2.07774     ,    2.02257     ,    1.89529     ,    1.86038     ,    1.95787     ,    2.10487     ,    2.18403     ,    2.15906     ,    2.11992     ,    2.09824     ,    2.05925     ,    1.93128     ,    1.83808     ,    1.8264     ,    1.76702     ,    1.79938     ,    1.93275     ,    2.09797     ,    2.22302     ,    1.9938     ,    1.64132     ,    1.28372     ,    0.9976     ,    1.06927     ,    1.1049     ,    0.51843     ,    -0.45852     ,    -2.16136     ,    -3.59243     ,    -2.50085     ,    -0.99419     ,    -0.19813     ,    0.15444     ,    0.04672     ,    -0.14359     ,    -0.14098     ,    -0.06832     ,    -0.22958     ,    -0.36628     ,    -0.32194     ,    -0.30618     ,    -0.22483     ,    -0.14178     ,    -0.18246     ,    -0.17181     ,    -0.03129     ,    0.16173     ,    0.35037     ,    0.49197     ,    0.57483     ,    0.63358     ,    0.65991     ,    0.67773     ,    0.73284     ,    0.73156     ,    0.76373     ,    0.8035     ,    0.82857     ,    0.87747     ,    0.90103     ,    0.96031     ,    1.02181     ,    1.04329     ,    1.00382     ,    0.95992     ,    0.90854     ,    0.87931     ,    0.86869     ,    0.87394     ,    0.87908     ,    0.87391     ,    0.87667     ,    0.91005     ,    0.97983     ,    1.03983     ,    1.03699     ,    1.00922     ,    0.94925     ,    0.90295     ,    0.84319     ,    0.78124     ,    0.7504     ,    0.74053     ,    0.74815     ,    0.73924     ,    0.72948     ,    0.72491     ,    0.73779     ,    0.77634     ,    0.81977     ,    0.8422     ,    0.7962     ,    0.71989     ,    0.60417     ,    0.48926     ,    0.36126     ,    0.28116     ,    0.20191     ,    0.16642     ,    0.10677     ,    0.15268     ,    0.24042     ,    0.3839     ,    0.52302     ,    0.63325     ,    0.69375     ,    0.7328     ,    0.68463     ,    0.59268     ,    0.62059     ,    0.79426     ,    1.09063    },
    {    1.62338     ,    1.93385     ,    2.21555     ,    2.33036     ,    2.38378     ,    2.33606     ,    2.3225     ,    2.33647     ,    2.38636     ,    2.37253     ,    2.3133     ,    2.19551     ,    2.0886     ,    2.14031     ,    2.36383     ,    2.53691     ,    2.66011     ,    2.65706     ,    2.50041     ,    2.3863     ,    2.38187     ,    2.36383     ,    2.35778     ,    2.32272     ,    2.25049     ,    2.09695     ,    1.95829     ,    1.91007     ,    2.11046     ,    2.13928     ,    2.18013     ,    2.47339     ,    2.75473     ,    2.78934     ,    2.26565     ,    1.69138     ,    1.13398     ,    0.80255     ,    0.75902     ,    1.06287     ,    1.33026     ,    1.39221     ,    1.0978     ,    -0.02222     ,    -1.0482     ,    -0.83469     ,    -0.46968     ,    -0.13561     ,    0.14658     ,    0.23811     ,    0.18863     ,    0.19072     ,    0.23239     ,    0.17983     ,    0.07389     ,    0.043     ,    -0.11828     ,    -0.26932     ,    -0.38983     ,    -0.38155     ,    -0.30007     ,    -0.16263     ,    0.06752     ,    0.29551     ,    0.46253     ,    0.55246     ,    0.58932     ,    0.61227     ,    0.64232     ,    0.69544     ,    0.75781     ,    0.83053     ,    0.88522     ,    0.93901     ,    1.0094     ,    1.07716     ,    1.17492     ,    1.22998     ,    1.22482     ,    1.17545     ,    1.13198     ,    1.11893     ,    1.08622     ,    1.10178     ,    1.14877     ,    1.1825     ,    1.20452     ,    1.18886     ,    1.18267     ,    1.16694     ,    1.14782     ,    1.11717     ,    1.07233     ,    0.99771     ,    0.9347     ,    0.87669     ,    0.83562     ,    0.83735     ,    0.8402     ,    0.8213     ,    0.78563     ,    0.73569     ,    0.71402     ,    0.75847     ,    0.82741     ,    0.90856     ,    0.95543     ,    0.94608     ,    0.89324     ,    0.80777     ,    0.72603     ,    0.65707     ,    0.6127     ,    0.5787     ,    0.56133     ,    0.57544     ,    0.63895     ,    0.74874     ,    0.85539     ,    0.89973     ,    0.87965     ,    0.81174     ,    0.7592     ,    0.77244     ,    0.81512     ,    0.91579     ,    1.1241     ,    1.35857    },
    {    1.85489     ,    2.01104     ,    2.18097     ,    2.28288     ,    2.32416     ,    2.34155     ,    2.30208     ,    2.39274     ,    2.40287     ,    2.36405     ,    2.35914     ,    2.41458     ,    2.50793     ,    2.64762     ,    2.94255     ,    3.21454     ,    3.27251     ,    3.24317     ,    3.09256     ,    2.92631     ,    2.56434     ,    2.48311     ,    2.31949     ,    2.16254     ,    2.03475     ,    1.93062     ,    1.86677     ,    2.15777     ,    2.73997     ,    2.25536     ,    2.36578     ,    3.30104     ,    4.21892     ,    2.93807     ,    2.14154     ,    1.6598     ,    1.2608     ,    1.06294     ,    1.08698     ,    1.33669     ,    1.50541     ,    1.4847     ,    1.15331     ,    0.43741     ,    -0.23744     ,    -0.11584     ,    -0.15077     ,    -0.0343     ,    0.32287     ,    0.48292     ,    0.45953     ,    0.29888     ,    0.33259     ,    0.30286     ,    0.29142     ,    0.30347     ,    0.15173     ,    0.00774     ,    -0.17126     ,    -0.35069     ,    -0.33797     ,    -0.10101     ,    0.20866     ,    0.44443     ,    0.58998     ,    0.63499     ,    0.61849     ,    0.57414     ,    0.55143     ,    0.57462     ,    0.61817     ,    0.59223     ,    0.79615     ,    1.29456     ,    1.30003     ,    1.26933     ,    1.31312     ,    1.31214     ,    1.28822     ,    1.23482     ,    1.23032     ,    1.31547     ,    1.40031     ,    1.48725     ,    1.57785     ,    1.5541     ,    1.53372     ,    1.45948     ,    1.33806     ,    1.27635     ,    1.24446     ,    1.21703     ,    1.19833     ,    1.18897     ,    1.1668     ,    1.13135     ,    1.0969     ,    1.10224     ,    1.07807     ,    1.0077     ,    0.94499     ,    0.91603     ,    0.90564     ,    0.9824     ,    1.05728     ,    1.13006     ,    1.16942     ,    1.17497     ,    1.15937     ,    1.12582     ,    1.14165     ,    1.12377     ,    1.13319     ,    1.08433     ,    1.05038     ,    1.05763     ,    1.00694     ,    1.05475     ,    1.10282     ,    1.14072     ,    1.16188     ,    1.15507     ,    1.17097     ,    1.23891     ,    1.32584     ,    1.44394     ,    1.58189     ,    1.71422    },
    {    1.7188     ,    1.793     ,    1.84049     ,    1.88034     ,    1.92153     ,    1.94365     ,    1.95558     ,    1.93092     ,    1.88646     ,    1.8693     ,    1.91057     ,    2.01509     ,    2.16497     ,    2.33288     ,    2.50796     ,    2.64629     ,    2.73698     ,    2.75473     ,    2.72899     ,    2.7558     ,    2.1282     ,    2.01733     ,    1.8969     ,    2.54205     ,    3.32751     ,    3.28187     ,    3.23173     ,    2.74686     ,    2.96399     ,    1.92672     ,    1.90055     ,    2.80544     ,    3.69022     ,    2.82483     ,    3.03514     ,    2.43392     ,    1.95247     ,    2.05811     ,    2.08052     ,    2.05392     ,    1.91767     ,    1.65237     ,    1.08544     ,    0.54916     ,    0.26167     ,    0.06756     ,    -0.03019     ,    0.10823     ,    0.32525     ,    0.52482     ,    0.54247     ,    0.2761     ,    0.1435     ,    0.29015     ,    0.48874     ,    0.70647     ,    0.73223     ,    0.52685     ,    0.51232     ,    0.41318     ,    0.3054     ,    0.24911     ,    0.19504     ,    0.28634     ,    0.35897     ,    0.37312     ,    0.35044     ,    0.35926     ,    0.32031     ,    0.63711     ,    1.12006     ,    1.28288     ,    1.29959     ,    1.45123     ,    1.51713     ,    1.55405     ,    1.56388     ,    1.53778     ,    1.5196     ,    1.48311     ,    1.48569     ,    1.50956     ,    1.51621     ,    1.4098     ,    1.28537     ,    1.55918     ,    1.55014     ,    1.60036     ,    1.57325     ,    1.50634     ,    1.50003     ,    1.44431     ,    1.37903     ,    1.32242     ,    1.28237     ,    1.26015     ,    1.2817     ,    1.2944     ,    1.29363     ,    1.26977     ,    1.24461     ,    1.24442     ,    1.24824     ,    1.25732     ,    1.26973     ,    1.23534     ,    1.34265     ,    1.34703     ,    1.27642     ,    1.20402     ,    1.13536     ,    1.10781     ,    1.10474     ,    1.07915     ,    1.06182     ,    1.04932     ,    1.01472     ,    1.01031     ,    1.01523     ,    1.06787     ,    1.14563     ,    1.24979     ,    1.35466     ,    1.44401     ,    1.50772     ,    1.5503     ,    1.58935     ,    1.64028    },
    {    1.48478     ,    1.47731     ,    1.4653     ,    1.44726     ,    1.42729     ,    1.40123     ,    1.36582     ,    1.29627     ,    1.22091     ,    1.16202     ,    1.14055     ,    1.16366     ,    1.22933     ,    1.32522     ,    1.43907     ,    1.53835     ,    1.63302     ,    1.69005     ,    1.74844     ,    1.85012     ,    1.82023     ,    1.64196     ,    1.47202     ,    1.12718     ,    1.0704     ,    1.19832     ,    1.3179     ,    2.2055     ,    2.68246     ,    1.39291     ,    1.43214     ,    1.431     ,    1.38971     ,    1.36751     ,    1.35986     ,    1.37837     ,    1.52716     ,    1.73884     ,    1.90307     ,    1.68432     ,    1.42832     ,    1.19641     ,    0.79657     ,    0.52027     ,    0.25267     ,    0.12783     ,    0.074     ,    0.12972     ,    0.21594     ,    0.21308     ,    0.0643     ,    -0.01898     ,    0.03673     ,    0.20524     ,    0.39978     ,    0.65402     ,    0.83549     ,    0.86509     ,    0.85167     ,    0.92299     ,    0.89978     ,    0.85574     ,    0.81767     ,    0.79728     ,    0.82811     ,    0.92485     ,    0.78559     ,    0.99364     ,    0.92471     ,    1.06329     ,    1.08693     ,    1.22339     ,    1.2896     ,    1.34454     ,    1.44629     ,    1.55086     ,    1.6159     ,    1.66409     ,    1.68418     ,    1.66963     ,    1.6515     ,    1.6678     ,    1.67967     ,    1.64706     ,    1.58018     ,    1.57995     ,    1.5739     ,    1.55491     ,    1.59804     ,    1.62475     ,    1.64825     ,    1.64012     ,    1.60753     ,    1.60937     ,    1.5873     ,    1.46022     ,    1.55482     ,    1.50568     ,    1.43197     ,    1.35671     ,    1.28513     ,    1.27876     ,    1.30494     ,    1.31974     ,    1.12292     ,    1.18196     ,    1.19418     ,    1.1467     ,    1.08312     ,    1.03572     ,    1.0044     ,    1.00415     ,    1.02025     ,    1.00708     ,    0.99349     ,    1.01011     ,    1.02008     ,    1.02967     ,    1.04248     ,    1.07498     ,    1.14685     ,    1.25249     ,    1.37823     ,    1.45     ,    1.44611     ,    1.46494     ,    1.50573     ,    1.50205    },
    {    1.23194     ,    1.17065     ,    1.12622     ,    1.08592     ,    1.04302     ,    1.00311     ,    0.95944     ,    0.9093     ,    0.85966     ,    0.82102     ,    0.79361     ,    0.79025     ,    0.80681     ,    0.83608     ,    0.86924     ,    0.91459     ,    0.95041     ,    0.94459     ,    0.92504     ,    0.94482     ,    1.00957     ,    0.81089     ,    0.58028     ,    0.54974     ,    0.65417     ,    0.83888     ,    1.07838     ,    0.95394     ,    1.03783     ,    1.12775     ,    1.2231     ,    1.18115     ,    1.23371     ,    1.14999     ,    1.15483     ,    1.10406     ,    1.20151     ,    1.16179     ,    1.12234     ,    0.92453     ,    0.9534     ,    0.89154     ,    0.78246     ,    0.60345     ,    0.4206     ,    0.32432     ,    0.29576     ,    0.27095     ,    0.26976     ,    0.20072     ,    0.13305     ,    0.13014     ,    0.23627     ,    0.39114     ,    0.60852     ,    0.72965     ,    0.79346     ,    0.83898     ,    0.75118     ,    0.90481     ,    1.03556     ,    1.1266     ,    1.16883     ,    1.20507     ,    1.24213     ,    1.32398     ,    1.33339     ,    1.28462     ,    1.1608     ,    1.0085     ,    0.98433     ,    0.98978     ,    1.01284     ,    0.99916     ,    1.23236     ,    1.41261     ,    1.50841     ,    1.59058     ,    1.66189     ,    1.68229     ,    1.69425     ,    1.69305     ,    1.67271     ,    1.67266     ,    1.67795     ,    1.68078     ,    1.68498     ,    1.68455     ,    1.68629     ,    1.71347     ,    1.7342     ,    1.75108     ,    1.8058     ,    1.84725     ,    1.8483     ,    1.80801     ,    1.74792     ,    1.6951     ,    1.58954     ,    1.48576     ,    1.38218     ,    1.28368     ,    1.17504     ,    1.08703     ,    1.03347     ,    1.04692     ,    1.06534     ,    1.04369     ,    1.02178     ,    1.00886     ,    1.01103     ,    1.0204     ,    1.03097     ,    1.03018     ,    1.00574     ,    0.98462     ,    0.96245     ,    0.946     ,    0.9407     ,    0.97325     ,    1.04293     ,    1.12536     ,    1.20221     ,    1.24926     ,    1.26617     ,    1.29868     ,    1.31588     ,    1.2854    },
    {    0.95196     ,    0.93622     ,    0.91819     ,    0.90638     ,    0.89748     ,    0.88811     ,    0.87681     ,    0.87302     ,    0.86218     ,    0.84834     ,    0.83855     ,    0.83645     ,    0.83994     ,    0.85194     ,    0.86308     ,    0.88014     ,    0.90667     ,    0.91487     ,    0.90457     ,    0.89571     ,    0.88197     ,    0.85168     ,    0.83089     ,    0.83091     ,    0.88247     ,    0.97441     ,    1.08053     ,    1.1609     ,    1.26445     ,    1.43523     ,    1.61947     ,    1.32868     ,    1.58551     ,    1.4289     ,    1.4702     ,    1.49503     ,    1.57814     ,    1.36404     ,    1.13316     ,    1.08488     ,    0.79386     ,    0.89342     ,    0.96081     ,    1.15744     ,    1.20103     ,    0.79419     ,    0.7416     ,    0.6708     ,    0.62336     ,    0.60663     ,    0.61776     ,    0.63573     ,    0.68896     ,    0.75419     ,    0.80729     ,    0.77599     ,    0.79867     ,    0.84043     ,    0.83379     ,    0.75533     ,    0.88665     ,    1.00104     ,    1.05853     ,    1.0823     ,    1.0845     ,    1.08862     ,    1.09957     ,    1.0998     ,    1.10242     ,    1.128     ,    1.0712     ,    1.01391     ,    0.97176     ,    0.97692     ,    1.08419     ,    1.17743     ,    1.24264     ,    1.30814     ,    1.37229     ,    1.41676     ,    1.45868     ,    1.48993     ,    1.50652     ,    1.51529     ,    1.5178     ,    1.51912     ,    1.52859     ,    1.53323     ,    1.52451     ,    1.51136     ,    1.49153     ,    1.45757     ,    1.43413     ,    1.41382     ,    1.38733     ,    1.35411     ,    1.30161     ,    1.23435     ,    1.15086     ,    1.08138     ,    1.02662     ,    0.95233     ,    0.88056     ,    0.83087     ,    0.80803     ,    0.82214     ,    0.85037     ,    0.88179     ,    0.8962     ,    0.90071     ,    0.89847     ,    0.89426     ,    0.89002     ,    0.8758     ,    0.85943     ,    0.85053     ,    0.84442     ,    0.83466     ,    0.83191     ,    0.84416     ,    0.87714     ,    0.91903     ,    0.95313     ,    0.96625     ,    0.96877     ,    0.97124     ,    0.96958     ,    0.96466    },
    {    0.78687     ,    0.80166     ,    0.81247     ,    0.82212     ,    0.82641     ,    0.82859     ,    0.83238     ,    0.83603     ,    0.83711     ,    0.83741     ,    0.83774     ,    0.83419     ,    0.83812     ,    0.85011     ,    0.86409     ,    0.88004     ,    0.90058     ,    0.91163     ,    0.91303     ,    0.90808     ,    0.90945     ,    0.91091     ,    0.91688     ,    0.93864     ,    0.97538     ,    1.02265     ,    1.07905     ,    1.13101     ,    1.18695     ,    1.2482     ,    1.28627     ,    1.31603     ,    1.33869     ,    1.34542     ,    1.33588     ,    1.33737     ,    1.3281     ,    1.10942     ,    0.96171     ,    0.93801     ,    0.93184     ,    0.96305     ,    0.97422     ,    1.06658     ,    1.03173     ,    0.88595     ,    0.87683     ,    0.87909     ,    0.88865     ,    0.89756     ,    0.90856     ,    0.9134     ,    0.91778     ,    0.93065     ,    0.94661     ,    0.96205     ,    1.02824     ,    1.07753     ,    1.10219     ,    1.10914     ,    1.13094     ,    1.1438     ,    1.13761     ,    1.12487     ,    1.11435     ,    1.1038     ,    1.09732     ,    1.08565     ,    1.08505     ,    1.08616     ,    1.07464     ,    1.06819     ,    1.06957     ,    1.08219     ,    1.11861     ,    1.15824     ,    1.18959     ,    1.21533     ,    1.23371     ,    1.25024     ,    1.26163     ,    1.26467     ,    1.26459     ,    1.25687     ,    1.25257     ,    1.24347     ,    1.23192     ,    1.22411     ,    1.21816     ,    1.20893     ,    1.18977     ,    1.16334     ,    1.14507     ,    1.12922     ,    1.1094     ,    1.08525     ,    1.05516     ,    1.01741     ,    0.97762     ,    0.94096     ,    0.90138     ,    0.86336     ,    0.8248     ,    0.79923     ,    0.78268     ,    0.77407     ,    0.77289     ,    0.78143     ,    0.7871     ,    0.78255     ,    0.77173     ,    0.76317     ,    0.75454     ,    0.74704     ,    0.74411     ,    0.75181     ,    0.76251     ,    0.76619     ,    0.76957     ,    0.77794     ,    0.78276     ,    0.7858     ,    0.78503     ,    0.77723     ,    0.77154     ,    0.76839     ,    0.76848     ,    0.77537    },
    {    0.86988     ,    0.86648     ,    0.86354     ,    0.85828     ,    0.85463     ,    0.85053     ,    0.8492     ,    0.84741     ,    0.84954     ,    0.8535     ,    0.8585     ,    0.86446     ,    0.87147     ,    0.87625     ,    0.87939     ,    0.88078     ,    0.87939     ,    0.87675     ,    0.87043     ,    0.86004     ,    0.85203     ,    0.84461     ,    0.84348     ,    0.84676     ,    0.85449     ,    0.86583     ,    0.87856     ,    0.89174     ,    0.90419     ,    0.91247     ,    0.9183     ,    0.92142     ,    0.92323     ,    0.922     ,    0.91892     ,    0.91806     ,    0.91597     ,    0.91588     ,    0.917     ,    0.92032     ,    0.92603     ,    0.93547     ,    0.94666     ,    0.9555     ,    0.96052     ,    0.96431     ,    0.96925     ,    0.97486     ,    0.97993     ,    0.98457     ,    0.98721     ,    0.98842     ,    0.98993     ,    0.99369     ,    1.00013     ,    1.0094     ,    1.02066     ,    1.03068     ,    1.03691     ,    1.0349     ,    1.02962     ,    1.01836     ,    1.00559     ,    0.9923     ,    0.98084     ,    0.97144     ,    0.96341     ,    0.95557     ,    0.95157     ,    0.94848     ,    0.94395     ,    0.94198     ,    0.9456     ,    0.95635     ,    0.96897     ,    0.98342     ,    1.00014     ,    1.01309     ,    1.02129     ,    1.02566     ,    1.02884     ,    1.02891     ,    1.02819     ,    1.02307     ,    1.01761     ,    1.01029     ,    1.00386     ,    1.00179     ,    1.00017     ,    0.99889     ,    0.99579     ,    0.98836     ,    0.98316     ,    0.97479     ,    0.96617     ,    0.95797     ,    0.95111     ,    0.94491     ,    0.93393     ,    0.92369     ,    0.91419     ,    0.90585     ,    0.89699     ,    0.89211     ,    0.89378     ,    0.89807     ,    0.9023     ,    0.90507     ,    0.907     ,    0.91026     ,    0.91024     ,    0.91177     ,    0.91192     ,    0.91299     ,    0.91262     ,    0.91436     ,    0.91503     ,    0.91496     ,    0.91504     ,    0.91093     ,    0.90529     ,    0.89928     ,    0.89433     ,    0.88959     ,    0.88654     ,    0.88297     ,    0.87811     ,    0.87308    }
};

double Stdev_LGRTC_RCP26_peak[64][128] = {    {    0.61221     ,    0.61342     ,    0.62227     ,    0.62945     ,    0.61014     ,    0.62897     ,    0.62835     ,    0.63390     ,    0.62913     ,    0.62268     ,    0.61826     ,    0.62628     ,    0.62591     ,    0.62601     ,    0.62846     ,    0.63075     ,    0.63607     ,    0.62700     ,    0.63441     ,    0.64101     ,    0.64275     ,    0.63918     ,    0.63836     ,    0.63995     ,    0.64649     ,    0.64719     ,    0.66908     ,    0.65751     ,    0.66965     ,    0.67873     ,    0.69153     ,    0.70500     ,    0.70350     ,    0.72644     ,    0.73110     ,    0.75140     ,    0.76010     ,    0.76858     ,    0.77849     ,    0.78543     ,    0.79874     ,    0.80226     ,    0.80669     ,    0.81448     ,    0.82403     ,    0.82671     ,    0.83294     ,    0.82948     ,    0.83574     ,    0.83487     ,    0.83612     ,    0.84179     ,    0.84243     ,    0.84753     ,    0.84732     ,    0.84472     ,    0.86645     ,    0.85965     ,    0.86149     ,    0.86917     ,    0.86694     ,    0.86448     ,    0.87497     ,    0.87617     ,    0.86764     ,    0.86815     ,    0.88801     ,    0.91843     ,    0.89144     ,    0.86329     ,    0.87827     ,    0.88566     ,    0.87486     ,    0.87780     ,    0.86852     ,    0.87190     ,    0.85674     ,    0.85814     ,    0.87976     ,    0.85770     ,    0.85618     ,    0.88945     ,    0.94568     ,    0.96759     ,    0.86231     ,    0.91116     ,    0.87572     ,    0.88641     ,    0.94341     ,    0.88909     ,    0.89197     ,    0.90306     ,    0.89413     ,    0.88516     ,    0.86975     ,    0.87067     ,    0.88449     ,    0.83008     ,    0.84371     ,    0.84180     ,    0.83242     ,    0.81524     ,    0.79271     ,    0.76374     ,    0.75861     ,    0.74787     ,    0.71962     ,    0.72484     ,    0.71996     ,    0.74033     ,    0.69157     ,    0.63499     ,    0.61255     ,    0.62034     ,    0.60943     ,    0.60913     ,    0.63533     ,    0.58861     ,    0.59183     ,    0.59512     ,    0.59602     ,    0.58327     ,    0.58407     ,    0.56548     ,    0.59127     ,    0.59834     ,    0.58733     ,    0.59039    },
    {    0.96765     ,    1.00084     ,    1.01975     ,    1.04446     ,    1.09238     ,    1.12619     ,    1.17954     ,    1.20423     ,    1.23223     ,    1.27626     ,    1.32541     ,    1.36473     ,    1.39203     ,    1.40992     ,    1.42485     ,    1.43377     ,    1.43988     ,    1.44805     ,    1.45067     ,    1.44647     ,    1.43586     ,    1.41306     ,    1.38833     ,    1.36120     ,    1.31603     ,    1.22639     ,    1.13417     ,    1.06484     ,    0.98299     ,    0.89822     ,    0.79327     ,    0.72416     ,    0.72640     ,    0.71741     ,    0.73132     ,    0.79586     ,    0.82442     ,    0.85858     ,    0.86430     ,    0.87179     ,    0.89142     ,    0.89578     ,    0.90483     ,    0.91135     ,    0.90704     ,    0.90555     ,    0.92133     ,    0.93887     ,    0.94362     ,    0.92667     ,    0.92751     ,    0.92493     ,    0.92768     ,    0.92398     ,    0.92216     ,    0.92930     ,    0.93994     ,    0.92109     ,    0.89925     ,    0.88958     ,    0.90828     ,    0.92469     ,    0.93160     ,    0.92349     ,    0.91821     ,    0.90359     ,    0.90206     ,    0.85013     ,    0.83955     ,    0.81519     ,    0.83776     ,    0.86981     ,    0.89512     ,    0.86478     ,    0.85627     ,    0.85600     ,    0.85112     ,    0.85014     ,    0.90806     ,    0.94142     ,    0.96632     ,    0.88738     ,    0.83923     ,    0.82789     ,    0.86399     ,    0.89057     ,    0.89283     ,    0.87748     ,    0.99958     ,    1.08186     ,    1.13832     ,    1.01891     ,    1.05635     ,    1.05634     ,    1.00633     ,    0.95570     ,    0.99512     ,    0.97111     ,    0.86910     ,    1.13138     ,    1.19414     ,    0.90498     ,    0.98892     ,    0.83713     ,    0.81505     ,    0.76093     ,    0.87081     ,    0.74937     ,    0.77942     ,    0.74311     ,    0.71729     ,    0.66062     ,    0.63524     ,    0.63728     ,    0.62826     ,    0.63491     ,    0.69166     ,    0.69327     ,    0.79874     ,    0.78982     ,    0.78812     ,    0.82608     ,    0.86336     ,    0.87580     ,    0.88851     ,    0.93934     ,    0.92806     ,    0.95009    },
    {    1.39448     ,    1.44639     ,    1.55307     ,    1.65697     ,    1.74586     ,    1.82175     ,    1.90683     ,    1.99179     ,    2.07417     ,    2.18793     ,    2.26830     ,    2.28352     ,    2.26784     ,    2.22667     ,    2.17545     ,    2.11565     ,    2.06018     ,    2.01527     ,    1.97487     ,    1.94936     ,    1.94877     ,    1.95057     ,    1.95158     ,    1.96045     ,    1.97370     ,    1.96626     ,    1.93938     ,    1.91980     ,    1.84831     ,    1.69240     ,    1.51368     ,    1.30137     ,    1.05367     ,    0.83036     ,    0.69587     ,    0.66124     ,    0.72052     ,    0.77010     ,    0.86518     ,    0.93729     ,    0.97250     ,    1.03603     ,    1.11628     ,    1.17156     ,    1.19714     ,    1.23672     ,    1.25119     ,    1.21949     ,    1.25379     ,    1.21606     ,    1.19272     ,    1.17657     ,    1.17846     ,    1.16106     ,    1.13299     ,    1.10772     ,    1.07962     ,    1.04066     ,    1.01505     ,    1.00427     ,    0.96991     ,    0.91501     ,    0.89627     ,    0.87245     ,    0.87351     ,    0.84964     ,    0.89343     ,    0.85388     ,    0.88687     ,    0.85161     ,    0.83589     ,    0.85010     ,    0.84737     ,    0.84052     ,    0.86042     ,    0.88268     ,    0.92584     ,    0.93886     ,    0.98026     ,    0.97490     ,    0.97330     ,    1.20470     ,    1.14069     ,    1.05677     ,    1.22911     ,    1.16227     ,    1.00414     ,    0.88121     ,    0.92892     ,    0.88081     ,    0.81625     ,    0.93226     ,    0.87862     ,    0.87533     ,    0.75500     ,    0.93822     ,    0.89169     ,    0.82912     ,    0.79454     ,    0.74579     ,    0.77319     ,    0.80110     ,    0.83037     ,    0.89477     ,    0.93112     ,    0.82266     ,    0.73684     ,    0.65797     ,    0.64842     ,    0.64705     ,    0.64177     ,    0.65439     ,    0.65297     ,    0.67691     ,    0.69232     ,    0.72275     ,    0.71551     ,    0.77568     ,    0.78803     ,    0.84692     ,    0.89716     ,    0.91841     ,    0.97283     ,    1.00843     ,    1.03412     ,    1.08459     ,    1.16845     ,    1.26414    },
    {    1.70712     ,    1.78400     ,    1.94955     ,    2.09942     ,    2.23583     ,    2.36840     ,    2.49810     ,    2.59402     ,    2.70124     ,    2.80648     ,    2.85621     ,    2.82482     ,    2.74125     ,    2.62767     ,    2.52224     ,    2.40958     ,    2.32751     ,    2.29226     ,    2.25309     ,    2.19673     ,    2.17019     ,    2.14711     ,    2.12880     ,    2.08425     ,    2.03212     ,    1.99139     ,    1.94144     ,    1.92295     ,    1.89176     ,    1.83464     ,    1.74541     ,    1.61112     ,    1.42632     ,    1.18739     ,    0.96710     ,    0.78938     ,    0.74702     ,    0.79437     ,    0.89208     ,    0.95006     ,    1.03697     ,    1.08965     ,    1.17577     ,    1.27074     ,    1.40731     ,    1.51623     ,    1.51554     ,    1.49586     ,    1.49723     ,    1.43367     ,    1.38230     ,    1.34041     ,    1.32371     ,    1.31202     ,    1.23416     ,    1.14881     ,    1.10168     ,    1.01680     ,    0.94898     ,    0.88223     ,    0.82229     ,    0.79878     ,    0.85426     ,    0.85610     ,    0.88706     ,    0.87777     ,    0.86785     ,    0.84905     ,    0.85313     ,    0.87516     ,    0.91325     ,    1.02741     ,    1.21869     ,    1.01909     ,    1.13576     ,    1.24722     ,    1.20685     ,    1.84974     ,    1.02776     ,    1.04440     ,    1.30397     ,    1.03639     ,    0.91849     ,    0.89481     ,    0.79055     ,    0.79371     ,    0.82380     ,    0.74457     ,    0.77403     ,    0.77161     ,    0.79362     ,    0.85399     ,    0.98467     ,    0.98190     ,    0.92554     ,    0.92914     ,    0.85965     ,    1.08396     ,    0.73695     ,    0.67937     ,    0.70648     ,    0.75042     ,    0.62867     ,    0.55107     ,    0.53580     ,    0.59293     ,    0.72854     ,    0.78348     ,    0.75138     ,    0.73480     ,    0.75069     ,    0.74985     ,    0.75082     ,    0.77143     ,    0.79813     ,    0.84424     ,    0.86706     ,    0.87470     ,    0.86010     ,    0.86370     ,    0.88006     ,    0.91764     ,    0.93205     ,    0.95976     ,    1.05397     ,    1.15641     ,    1.35187     ,    1.52315    },
    {    1.74963     ,    1.90876     ,    2.09442     ,    2.24278     ,    2.40951     ,    2.58575     ,    2.75061     ,    2.88769     ,    2.98751     ,    3.09410     ,    3.12643     ,    3.06811     ,    2.88546     ,    2.59199     ,    2.43292     ,    2.33970     ,    2.25421     ,    2.19420     ,    2.16734     ,    2.12638     ,    2.08738     ,    2.01969     ,    1.97573     ,    1.89222     ,    1.80351     ,    1.72573     ,    1.65229     ,    1.58642     ,    1.52951     ,    1.47739     ,    1.43822     ,    1.38367     ,    1.31327     ,    1.21519     ,    1.08903     ,    0.97122     ,    0.88323     ,    0.84036     ,    0.80224     ,    0.79273     ,    0.81096     ,    0.86184     ,    0.94346     ,    1.07741     ,    1.13545     ,    1.19516     ,    1.21498     ,    1.20326     ,    1.19556     ,    1.21320     ,    1.23223     ,    1.23642     ,    1.21661     ,    1.17883     ,    1.14638     ,    1.11293     ,    1.05008     ,    0.94707     ,    0.81093     ,    0.79847     ,    0.78270     ,    0.75877     ,    0.77742     ,    0.83706     ,    0.90818     ,    0.85849     ,    1.14484     ,    0.95009     ,    0.96127     ,    0.99080     ,    1.05461     ,    1.15609     ,    1.01204     ,    1.13196     ,    0.96331     ,    1.02090     ,    1.04901     ,    1.34384     ,    1.00546     ,    1.04387     ,    0.99711     ,    0.89047     ,    0.80935     ,    0.79853     ,    0.78022     ,    0.78174     ,    0.85204     ,    0.94092     ,    1.01707     ,    1.04811     ,    1.04833     ,    1.06850     ,    1.08948     ,    1.12934     ,    1.18624     ,    1.21344     ,    1.16583     ,    1.10591     ,    1.15689     ,    0.89104     ,    0.92274     ,    0.80821     ,    0.65596     ,    0.58758     ,    0.82009     ,    0.64370     ,    0.78442     ,    0.86808     ,    0.84380     ,    0.86468     ,    0.82217     ,    0.82455     ,    0.82148     ,    0.81900     ,    0.82986     ,    0.83252     ,    0.87614     ,    0.90536     ,    0.94097     ,    0.95372     ,    0.94799     ,    0.93903     ,    1.19322     ,    1.04577     ,    1.03091     ,    1.07845     ,    1.25985     ,    1.47866    },
    {    1.85178     ,    2.06993     ,    2.30645     ,    2.48443     ,    2.67734     ,    2.89350     ,    3.07363     ,    3.20796     ,    3.31412     ,    3.39741     ,    3.46632     ,    3.50309     ,    3.50194     ,    3.42981     ,    3.23472     ,    3.00044     ,    2.76474     ,    2.54084     ,    2.35445     ,    2.22716     ,    2.14167     ,    2.09476     ,    2.04043     ,    1.94766     ,    1.84066     ,    1.76034     ,    1.64808     ,    1.54982     ,    1.56941     ,    1.48644     ,    1.43264     ,    1.35577     ,    1.23895     ,    1.17183     ,    1.09155     ,    1.02715     ,    0.98433     ,    0.95258     ,    0.92179     ,    0.90366     ,    0.87425     ,    0.83484     ,    0.86327     ,    0.96390     ,    1.01392     ,    1.04715     ,    1.09712     ,    1.15527     ,    1.20163     ,    1.23785     ,    1.29931     ,    1.38416     ,    1.47988     ,    1.53708     ,    1.54753     ,    1.59358     ,    1.63769     ,    1.52483     ,    1.22177     ,    0.91455     ,    0.79233     ,    0.75569     ,    0.80231     ,    0.82450     ,    0.81302     ,    0.76555     ,    0.78727     ,    0.83007     ,    0.79929     ,    0.81623     ,    0.82756     ,    0.82459     ,    0.82820     ,    0.84770     ,    0.85493     ,    0.84604     ,    0.84101     ,    0.85024     ,    0.89516     ,    0.92364     ,    0.94343     ,    0.93091     ,    0.92540     ,    0.91068     ,    0.88643     ,    0.91286     ,    0.98840     ,    1.10212     ,    1.20187     ,    1.24576     ,    1.18536     ,    1.18234     ,    1.21114     ,    1.23112     ,    1.26635     ,    1.28337     ,    1.31739     ,    1.18564     ,    1.10861     ,    1.11875     ,    1.01834     ,    1.00104     ,    1.00245     ,    1.01243     ,    1.01655     ,    1.16649     ,    1.22424     ,    1.17311     ,    0.99263     ,    0.96216     ,    0.96019     ,    0.97898     ,    0.97848     ,    1.00143     ,    0.99066     ,    0.98979     ,    1.01253     ,    1.00128     ,    0.97882     ,    0.96174     ,    0.94053     ,    0.90718     ,    0.85917     ,    0.89034     ,    0.91270     ,    1.01627     ,    1.18342     ,    1.45605    },
    {    2.01733     ,    2.17408     ,    2.37792     ,    2.55405     ,    2.76984     ,    2.94524     ,    3.06778     ,    3.15374     ,    3.19718     ,    3.21633     ,    3.25227     ,    3.29469     ,    3.33602     ,    3.37433     ,    3.38036     ,    3.34964     ,    3.29463     ,    3.18510     ,    3.05754     ,    2.94310     ,    2.84302     ,    2.73550     ,    2.60815     ,    2.46477     ,    2.31458     ,    2.17070     ,    2.03494     ,    1.88183     ,    1.76988     ,    1.66518     ,    1.60808     ,    1.50687     ,    1.37355     ,    1.25054     ,    1.16230     ,    1.10115     ,    1.08405     ,    1.06189     ,    1.07860     ,    0.97446     ,    0.92079     ,    0.87960     ,    0.89426     ,    1.02179     ,    1.10210     ,    1.16121     ,    1.25475     ,    1.32455     ,    1.39270     ,    1.49987     ,    1.63317     ,    1.79580     ,    1.96025     ,    2.06936     ,    2.11613     ,    2.15787     ,    2.21149     ,    2.19702     ,    2.07810     ,    1.84321     ,    1.51123     ,    1.16159     ,    0.99596     ,    0.91455     ,    0.85442     ,    0.79275     ,    0.76977     ,    0.77345     ,    0.79926     ,    0.83268     ,    0.87661     ,    0.92317     ,    0.93427     ,    0.92353     ,    0.89930     ,    0.90044     ,    0.89111     ,    0.90836     ,    0.92957     ,    0.96775     ,    1.00373     ,    1.06272     ,    1.15439     ,    1.16949     ,    1.06470     ,    1.01397     ,    1.06327     ,    1.12305     ,    1.26362     ,    1.45617     ,    1.36868     ,    1.22302     ,    1.16977     ,    1.19022     ,    1.22458     ,    1.25705     ,    1.28819     ,    1.29108     ,    1.28068     ,    1.27069     ,    1.25265     ,    1.26492     ,    1.24310     ,    1.21796     ,    1.21268     ,    1.20474     ,    1.13011     ,    1.10062     ,    1.09721     ,    1.09499     ,    1.05252     ,    1.03496     ,    1.01091     ,    1.00656     ,    0.97991     ,    0.94889     ,    0.90722     ,    0.85657     ,    0.81601     ,    0.76955     ,    0.76785     ,    0.78274     ,    0.82056     ,    0.85656     ,    0.94966     ,    1.10624     ,    1.33909     ,    1.61351    },
    {    1.70609     ,    1.84038     ,    2.06612     ,    2.24817     ,    2.41647     ,    2.55996     ,    2.69229     ,    2.79403     ,    2.86350     ,    2.91133     ,    2.95163     ,    2.95545     ,    2.93979     ,    2.95335     ,    2.97619     ,    2.98851     ,    3.01921     ,    3.02656     ,    2.96091     ,    2.86850     ,    2.75752     ,    2.62744     ,    2.49052     ,    2.35809     ,    2.22575     ,    2.09354     ,    1.96896     ,    1.84516     ,    1.71377     ,    1.56070     ,    1.44086     ,    1.37263     ,    1.32154     ,    1.25070     ,    1.21099     ,    1.18677     ,    1.15755     ,    1.11661     ,    1.04651     ,    0.95896     ,    0.92072     ,    0.84826     ,    0.88539     ,    0.92669     ,    0.96967     ,    1.07677     ,    1.24146     ,    1.41430     ,    1.56374     ,    1.71979     ,    1.88697     ,    2.05141     ,    2.13352     ,    2.13870     ,    2.18441     ,    2.34271     ,    2.54286     ,    2.72144     ,    2.87281     ,    2.95229     ,    2.92083     ,    2.77363     ,    2.59862     ,    2.40684     ,    2.19736     ,    1.97715     ,    1.74656     ,    1.53831     ,    1.42632     ,    1.43025     ,    1.50066     ,    1.59516     ,    1.64302     ,    1.57712     ,    1.41888     ,    1.34749     ,    1.32821     ,    1.41882     ,    1.42678     ,    1.15651     ,    1.04058     ,    1.04723     ,    0.99834     ,    0.98785     ,    1.00307     ,    1.05395     ,    1.08655     ,    1.13920     ,    1.24777     ,    1.46328     ,    1.65542     ,    1.48923     ,    1.21026     ,    1.10007     ,    1.12342     ,    1.16699     ,    1.19455     ,    1.23079     ,    1.24804     ,    1.24782     ,    1.25496     ,    1.25877     ,    1.24645     ,    1.25824     ,    1.21847     ,    1.20394     ,    1.15723     ,    1.17830     ,    1.17224     ,    1.14349     ,    1.08929     ,    1.04595     ,    0.99864     ,    0.94961     ,    0.90543     ,    0.87074     ,    0.92668     ,    1.03291     ,    0.97201     ,    1.00625     ,    1.04593     ,    1.05273     ,    1.05854     ,    1.04332     ,    1.05460     ,    1.08915     ,    1.18093     ,    1.36817    },
    {    1.47634     ,    1.53006     ,    1.64618     ,    1.76111     ,    1.92167     ,    2.12906     ,    2.35555     ,    2.52622     ,    2.65535     ,    2.72797     ,    2.71719     ,    2.58018     ,    2.37592     ,    2.26586     ,    2.23760     ,    2.20017     ,    2.17216     ,    2.17563     ,    2.15920     ,    2.09895     ,    2.00970     ,    1.90524     ,    1.83433     ,    1.75667     ,    1.66719     ,    1.56509     ,    1.46826     ,    1.39325     ,    1.31861     ,    1.22765     ,    1.15586     ,    1.12710     ,    1.13833     ,    1.13500     ,    1.12278     ,    1.11732     ,    1.12598     ,    1.10552     ,    1.04574     ,    0.96765     ,    0.93866     ,    0.90195     ,    0.91142     ,    0.95593     ,    0.98227     ,    1.06748     ,    1.19152     ,    1.31046     ,    1.43124     ,    1.54278     ,    1.64398     ,    1.72123     ,    1.77997     ,    1.79719     ,    1.81011     ,    1.86839     ,    1.98049     ,    2.15110     ,    2.38807     ,    2.68625     ,    3.02704     ,    3.29866     ,    3.45286     ,    3.48601     ,    3.39346     ,    3.21250     ,    2.98408     ,    2.67947     ,    2.36534     ,    2.12080     ,    2.00371     ,    1.93974     ,    1.85208     ,    1.77257     ,    1.71066     ,    1.69780     ,    1.71889     ,    1.77896     ,    1.78861     ,    1.71581     ,    1.64780     ,    1.58672     ,    1.52132     ,    1.42909     ,    1.40475     ,    1.44899     ,    1.50452     ,    1.54317     ,    1.64163     ,    1.79444     ,    1.91673     ,    1.89181     ,    1.77572     ,    1.62117     ,    1.56990     ,    1.62345     ,    1.59813     ,    1.50934     ,    1.47660     ,    1.62192     ,    1.50555     ,    1.58206     ,    1.53797     ,    1.52074     ,    1.58005     ,    1.62076     ,    1.58546     ,    1.53542     ,    1.42591     ,    1.43651     ,    1.37800     ,    1.35648     ,    1.31066     ,    1.33780     ,    1.37006     ,    1.41135     ,    1.46700     ,    1.44747     ,    1.42702     ,    1.44367     ,    1.46666     ,    1.45310     ,    1.35369     ,    1.23650     ,    1.20880     ,    1.23008     ,    1.24942     ,    1.28765    },
    {    1.36174     ,    1.38560     ,    1.42964     ,    1.44506     ,    1.51196     ,    1.71886     ,    2.13848     ,    2.55386     ,    2.69903     ,    2.66887     ,    2.57110     ,    2.32278     ,    2.00697     ,    1.75320     ,    1.62686     ,    1.51522     ,    1.40028     ,    1.31629     ,    1.27708     ,    1.26259     ,    1.26091     ,    1.24331     ,    1.22569     ,    1.20852     ,    1.18645     ,    1.15419     ,    1.10713     ,    1.05482     ,    1.00480     ,    0.96515     ,    0.93988     ,    0.92167     ,    0.91875     ,    0.92544     ,    0.93093     ,    0.92380     ,    0.92033     ,    0.91590     ,    0.88972     ,    0.85129     ,    0.84165     ,    0.85069     ,    0.94867     ,    0.92724     ,    0.96587     ,    1.10247     ,    1.21111     ,    1.27207     ,    1.28966     ,    1.28248     ,    1.29245     ,    1.30688     ,    1.34484     ,    1.38489     ,    1.42398     ,    1.47378     ,    1.53204     ,    1.61149     ,    1.74880     ,    1.91800     ,    2.14519     ,    2.46867     ,    2.72906     ,    2.87193     ,    2.92581     ,    2.89383     ,    2.79739     ,    2.69524     ,    2.58432     ,    2.48547     ,    2.38671     ,    2.30965     ,    2.24994     ,    2.17221     ,    2.07977     ,    1.99451     ,    1.95362     ,    1.94113     ,    1.91772     ,    1.90787     ,    1.90205     ,    1.81320     ,    1.68470     ,    1.55899     ,    1.55441     ,    1.90950     ,    2.26145     ,    2.40964     ,    2.47704     ,    2.61363     ,    2.75472     ,    2.84455     ,    2.84672     ,    2.71469     ,    2.48838     ,    2.19080     ,    1.86844     ,    1.74061     ,    1.72541     ,    1.75863     ,    1.78713     ,    1.86510     ,    1.94241     ,    1.97159     ,    1.96945     ,    1.93864     ,    1.86264     ,    1.73954     ,    1.56738     ,    1.40829     ,    1.33889     ,    1.33708     ,    1.38593     ,    1.43749     ,    1.43910     ,    1.44934     ,    1.50014     ,    1.49029     ,    1.43746     ,    1.36860     ,    1.31729     ,    1.31001     ,    1.31906     ,    1.26550     ,    1.20372     ,    1.16458     ,    1.16801     ,    1.22331    },
    {    0.96196     ,    0.97193     ,    1.03735     ,    1.15755     ,    1.33898     ,    1.48550     ,    1.69840     ,    2.05585     ,    2.21588     ,    2.08704     ,    2.03449     ,    2.02044     ,    1.84807     ,    1.51304     ,    1.30910     ,    1.19912     ,    1.12231     ,    1.05936     ,    1.01746     ,    1.00051     ,    0.99662     ,    0.99310     ,    0.98453     ,    0.97283     ,    0.95556     ,    0.93593     ,    0.90228     ,    0.86323     ,    0.83188     ,    0.81091     ,    0.79898     ,    0.78936     ,    0.77670     ,    0.76097     ,    0.74892     ,    0.73613     ,    0.71707     ,    0.69207     ,    0.67254     ,    0.64706     ,    0.63661     ,    0.61852     ,    0.59717     ,    0.66241     ,    0.76083     ,    0.94649     ,    1.12211     ,    1.19322     ,    1.17531     ,    1.20018     ,    1.21736     ,    1.21795     ,    1.22583     ,    1.23858     ,    1.23362     ,    1.22453     ,    1.23041     ,    1.24035     ,    1.27920     ,    1.34369     ,    1.42295     ,    1.55265     ,    1.72454     ,    1.88081     ,    1.98008     ,    2.07118     ,    2.12709     ,    2.09026     ,    2.02036     ,    1.97293     ,    1.93943     ,    1.88522     ,    1.78184     ,    1.62584     ,    1.48353     ,    1.38825     ,    1.32509     ,    1.27780     ,    1.24025     ,    1.20981     ,    1.21151     ,    1.21995     ,    1.20298     ,    1.13179     ,    1.05041     ,    1.01680     ,    1.05989     ,    1.20056     ,    1.53670     ,    1.91494     ,    2.09244     ,    2.17909     ,    2.27240     ,    2.30714     ,    2.19911     ,    1.85973     ,    1.52476     ,    1.37738     ,    1.34488     ,    1.35829     ,    1.35301     ,    1.34648     ,    1.38771     ,    1.42885     ,    1.40627     ,    1.33631     ,    1.25688     ,    1.17828     ,    1.07715     ,    0.97545     ,    0.90079     ,    0.84922     ,    0.83223     ,    0.83995     ,    0.82607     ,    0.82312     ,    0.89598     ,    1.02521     ,    1.08782     ,    1.08205     ,    1.04016     ,    0.99454     ,    0.93818     ,    0.89313     ,    0.86902     ,    0.86716     ,    0.87214     ,    0.89914    },
    {    0.71839     ,    0.72381     ,    0.78382     ,    0.90525     ,    1.08600     ,    1.16693     ,    1.20570     ,    1.42235     ,    1.51644     ,    1.26837     ,    1.14887     ,    1.25652     ,    1.39719     ,    1.30521     ,    1.19557     ,    1.14551     ,    1.09913     ,    1.00757     ,    0.92668     ,    0.87941     ,    0.85597     ,    0.84802     ,    0.84759     ,    0.84570     ,    0.82202     ,    0.78352     ,    0.73943     ,    0.70707     ,    0.68563     ,    0.67623     ,    0.65980     ,    0.65026     ,    0.63727     ,    0.62417     ,    0.59461     ,    0.57701     ,    0.54394     ,    0.50655     ,    0.47463     ,    0.43783     ,    0.41190     ,    0.41572     ,    0.40835     ,    0.43172     ,    0.50316     ,    0.63005     ,    0.71199     ,    0.83097     ,    0.94096     ,    1.00891     ,    1.01671     ,    0.99794     ,    1.01521     ,    0.99081     ,    0.98537     ,    1.04425     ,    1.09534     ,    1.12003     ,    1.13629     ,    1.14008     ,    1.15883     ,    1.21800     ,    1.27938     ,    1.32510     ,    1.36473     ,    1.39094     ,    1.41327     ,    1.42949     ,    1.41762     ,    1.33730     ,    1.18530     ,    1.08267     ,    1.04343     ,    1.01129     ,    0.93898     ,    0.86000     ,    0.80652     ,    0.77950     ,    0.77131     ,    0.75505     ,    0.73802     ,    0.72543     ,    0.71897     ,    0.70651     ,    0.68853     ,    0.67385     ,    0.65966     ,    0.65905     ,    0.70158     ,    0.82376     ,    1.08380     ,    1.30523     ,    1.48002     ,    1.55848     ,    1.52052     ,    1.37295     ,    1.20332     ,    1.08255     ,    1.06937     ,    1.08675     ,    1.06082     ,    0.99730     ,    0.93894     ,    0.87076     ,    0.81690     ,    0.78898     ,    0.75934     ,    0.72847     ,    0.70549     ,    0.68631     ,    0.66976     ,    0.65726     ,    0.65846     ,    0.66085     ,    0.66422     ,    0.69550     ,    0.78861     ,    0.86483     ,    0.82960     ,    0.82161     ,    0.86527     ,    0.84751     ,    0.76386     ,    0.71029     ,    0.69805     ,    0.70902     ,    0.70869     ,    0.69533    },
    {    0.57736     ,    0.57367     ,    0.58541     ,    0.63468     ,    0.73369     ,    0.86244     ,    0.99088     ,    1.14002     ,    1.11149     ,    0.91194     ,    0.85129     ,    0.86957     ,    0.91415     ,    0.97591     ,    1.00317     ,    1.00951     ,    0.99140     ,    0.91217     ,    0.81459     ,    0.75873     ,    0.75000     ,    0.75378     ,    0.75825     ,    0.73807     ,    0.67942     ,    0.61812     ,    0.57242     ,    0.55082     ,    0.53997     ,    0.53274     ,    0.52323     ,    0.52193     ,    0.51074     ,    0.49719     ,    0.47267     ,    0.43999     ,    0.42157     ,    0.38456     ,    0.35660     ,    0.34476     ,    0.33155     ,    0.33479     ,    0.33710     ,    0.34721     ,    0.35694     ,    0.38931     ,    0.46645     ,    0.63519     ,    0.74991     ,    0.73156     ,    0.69341     ,    0.69486     ,    0.70790     ,    0.72886     ,    0.86123     ,    0.99860     ,    1.07182     ,    1.10575     ,    1.12339     ,    1.11891     ,    1.12724     ,    1.16944     ,    1.13444     ,    1.06507     ,    1.01297     ,    0.97699     ,    0.98959     ,    1.02466     ,    1.03983     ,    0.97770     ,    0.88602     ,    0.81358     ,    0.75593     ,    0.70268     ,    0.65084     ,    0.62551     ,    0.61815     ,    0.60903     ,    0.60601     ,    0.60334     ,    0.60001     ,    0.59974     ,    0.60122     ,    0.59893     ,    0.60665     ,    0.62753     ,    0.64186     ,    0.63145     ,    0.61147     ,    0.59702     ,    0.59760     ,    0.65090     ,    0.73697     ,    0.82847     ,    0.84818     ,    0.76047     ,    0.70791     ,    0.71871     ,    0.72495     ,    0.77928     ,    0.78294     ,    0.77939     ,    0.78041     ,    0.75720     ,    0.72799     ,    0.69211     ,    0.65489     ,    0.63374     ,    0.62033     ,    0.61940     ,    0.62610     ,    0.65041     ,    0.68209     ,    0.70315     ,    0.69963     ,    0.70821     ,    0.75273     ,    0.68566     ,    0.61924     ,    0.62253     ,    0.65951     ,    0.64236     ,    0.56774     ,    0.50917     ,    0.48639     ,    0.48526     ,    0.50649     ,    0.54471    },
    {    0.47772     ,    0.47874     ,    0.49197     ,    0.51559     ,    0.57295     ,    0.68500     ,    0.79244     ,    0.83993     ,    0.79588     ,    0.73061     ,    0.71503     ,    0.73944     ,    0.78874     ,    0.84121     ,    0.79855     ,    0.73798     ,    0.71574     ,    0.68166     ,    0.64122     ,    0.62054     ,    0.61419     ,    0.62712     ,    0.63999     ,    0.58506     ,    0.47831     ,    0.41441     ,    0.39755     ,    0.40087     ,    0.41284     ,    0.41360     ,    0.40356     ,    0.39790     ,    0.38859     ,    0.37411     ,    0.36745     ,    0.37778     ,    0.39918     ,    0.34882     ,    0.36149     ,    0.36836     ,    0.33648     ,    0.32262     ,    0.30765     ,    0.31030     ,    0.31163     ,    0.33104     ,    0.44606     ,    0.55945     ,    0.52671     ,    0.48218     ,    0.52944     ,    0.56837     ,    0.58392     ,    0.64277     ,    0.73526     ,    0.85434     ,    0.94796     ,    1.00789     ,    1.08328     ,    1.20421     ,    1.35112     ,    1.39153     ,    1.23606     ,    1.15021     ,    1.10588     ,    1.12317     ,    1.16568     ,    1.20641     ,    1.24992     ,    1.20990     ,    1.13365     ,    1.01338     ,    0.89260     ,    0.83020     ,    0.75715     ,    0.64252     ,    0.55690     ,    0.53032     ,    0.53019     ,    0.50948     ,    0.49626     ,    0.51879     ,    0.54534     ,    0.55351     ,    0.58328     ,    0.66143     ,    0.70934     ,    0.70013     ,    0.66998     ,    0.63654     ,    0.57611     ,    0.55228     ,    0.51051     ,    0.56617     ,    0.68879     ,    0.62449     ,    0.59314     ,    0.60299     ,    0.54426     ,    0.51235     ,    0.52149     ,    0.52997     ,    0.56564     ,    0.59854     ,    0.62014     ,    0.62322     ,    0.61827     ,    0.60259     ,    0.58756     ,    0.63522     ,    0.71334     ,    0.79410     ,    0.85107     ,    0.86483     ,    0.79050     ,    0.66206     ,    0.59933     ,    0.54795     ,    0.51794     ,    0.54266     ,    0.58552     ,    0.53931     ,    0.49354     ,    0.44729     ,    0.42015     ,    0.41602     ,    0.44607     ,    0.47413    },
    {    0.42641     ,    0.42657     ,    0.44916     ,    0.47780     ,    0.49927     ,    0.51764     ,    0.52501     ,    0.53370     ,    0.53510     ,    0.54535     ,    0.55146     ,    0.55320     ,    0.59312     ,    0.62618     ,    0.55523     ,    0.53928     ,    0.54830     ,    0.54448     ,    0.54462     ,    0.54691     ,    0.55628     ,    0.58095     ,    0.58110     ,    0.52849     ,    0.41666     ,    0.35538     ,    0.36702     ,    0.38816     ,    0.39697     ,    0.39871     ,    0.38778     ,    0.37599     ,    0.36311     ,    0.36696     ,    0.35827     ,    0.40119     ,    0.38098     ,    0.32173     ,    0.36831     ,    0.39008     ,    0.37192     ,    0.34890     ,    0.32069     ,    0.31574     ,    0.32350     ,    0.41742     ,    0.54452     ,    0.50599     ,    0.45082     ,    0.47117     ,    0.51765     ,    0.55218     ,    0.58903     ,    0.63102     ,    0.66889     ,    0.73736     ,    0.79268     ,    0.82358     ,    0.88058     ,    1.02876     ,    1.27742     ,    1.42657     ,    1.51202     ,    1.48529     ,    1.39198     ,    1.44357     ,    1.46910     ,    1.42918     ,    1.26420     ,    1.08920     ,    0.98634     ,    0.93465     ,    0.87740     ,    0.80425     ,    0.67310     ,    0.57253     ,    0.53913     ,    0.50004     ,    0.47144     ,    0.45145     ,    0.43515     ,    0.44349     ,    0.46917     ,    0.48582     ,    0.53038     ,    0.65084     ,    0.73044     ,    0.70608     ,    0.63187     ,    0.60588     ,    0.57844     ,    0.59054     ,    0.65814     ,    0.74413     ,    0.77864     ,    0.75253     ,    0.71421     ,    0.68396     ,    0.64611     ,    0.59073     ,    0.57342     ,    0.56732     ,    0.54581     ,    0.53023     ,    0.51204     ,    0.51116     ,    0.55628     ,    0.62572     ,    0.66735     ,    0.74184     ,    0.84095     ,    0.92341     ,    0.96731     ,    0.91404     ,    0.76569     ,    0.69098     ,    0.66308     ,    0.63709     ,    0.57269     ,    0.52803     ,    0.49394     ,    0.45660     ,    0.43184     ,    0.41973     ,    0.40629     ,    0.41146     ,    0.43083     ,    0.45216    },
    {    0.40929     ,    0.45064     ,    0.56514     ,    0.58220     ,    0.52014     ,    0.49944     ,    0.47797     ,    0.43489     ,    0.38524     ,    0.35854     ,    0.34493     ,    0.35164     ,    0.36366     ,    0.37604     ,    0.43419     ,    0.47465     ,    0.48580     ,    0.49369     ,    0.50601     ,    0.52249     ,    0.55284     ,    0.56212     ,    0.53550     ,    0.48479     ,    0.43156     ,    0.39819     ,    0.40062     ,    0.41793     ,    0.42813     ,    0.43100     ,    0.42133     ,    0.41757     ,    0.42789     ,    0.44305     ,    0.37895     ,    0.41666     ,    0.36526     ,    0.31691     ,    0.34429     ,    0.35959     ,    0.35318     ,    0.34338     ,    0.33274     ,    0.35901     ,    0.50814     ,    0.58316     ,    0.56892     ,    0.53383     ,    0.53943     ,    0.55078     ,    0.56303     ,    0.58410     ,    0.57994     ,    0.59278     ,    0.62548     ,    0.67247     ,    0.71535     ,    0.74699     ,    0.84874     ,    1.14928     ,    1.24910     ,    1.21230     ,    1.17541     ,    1.05075     ,    0.93955     ,    0.93659     ,    0.91796     ,    0.79503     ,    0.63505     ,    0.58201     ,    0.56385     ,    0.54112     ,    0.49554     ,    0.40344     ,    0.38197     ,    0.38008     ,    0.41547     ,    0.41905     ,    0.42282     ,    0.45099     ,    0.45917     ,    0.45666     ,    0.44142     ,    0.44442     ,    0.43662     ,    0.47521     ,    0.56867     ,    0.63825     ,    0.65689     ,    0.65679     ,    0.64101     ,    0.64979     ,    0.67856     ,    0.70555     ,    0.71543     ,    0.67941     ,    0.64917     ,    0.65394     ,    0.62895     ,    0.55266     ,    0.51164     ,    0.50802     ,    0.49427     ,    0.47440     ,    0.46043     ,    0.47365     ,    0.51588     ,    0.55863     ,    0.59611     ,    0.65491     ,    0.70330     ,    0.70430     ,    0.67466     ,    0.63451     ,    0.66075     ,    0.66703     ,    0.63638     ,    0.60021     ,    0.53093     ,    0.44134     ,    0.41336     ,    0.37728     ,    0.36605     ,    0.40970     ,    0.37756     ,    0.39629     ,    0.43250     ,    0.42612    },
    {    0.57990     ,    0.58688     ,    0.65814     ,    0.65626     ,    0.55974     ,    0.45591     ,    0.37021     ,    0.30618     ,    0.27884     ,    0.27744     ,    0.29345     ,    0.31206     ,    0.32729     ,    0.36183     ,    0.38768     ,    0.40926     ,    0.43398     ,    0.46019     ,    0.47474     ,    0.48805     ,    0.50648     ,    0.50461     ,    0.50157     ,    0.48626     ,    0.47424     ,    0.47431     ,    0.46638     ,    0.45854     ,    0.46079     ,    0.45226     ,    0.43710     ,    0.45735     ,    0.47960     ,    0.46636     ,    0.37665     ,    0.38917     ,    0.34119     ,    0.31918     ,    0.34949     ,    0.33673     ,    0.30515     ,    0.29599     ,    0.32841     ,    0.55910     ,    0.85948     ,    0.74357     ,    0.64270     ,    0.67672     ,    0.68719     ,    0.67990     ,    0.68483     ,    0.69290     ,    0.68870     ,    0.69992     ,    0.74635     ,    0.77903     ,    0.78681     ,    0.84070     ,    1.09778     ,    1.25709     ,    0.76123     ,    0.51373     ,    0.45277     ,    0.46049     ,    0.45422     ,    0.42631     ,    0.42528     ,    0.44567     ,    0.44114     ,    0.42462     ,    0.39030     ,    0.37106     ,    0.38889     ,    0.45663     ,    0.43774     ,    0.38733     ,    0.46499     ,    0.43515     ,    0.46150     ,    0.47736     ,    0.47391     ,    0.45527     ,    0.40726     ,    0.41141     ,    0.55706     ,    0.59186     ,    0.55406     ,    0.49103     ,    0.55092     ,    0.57216     ,    0.58099     ,    0.60131     ,    0.62022     ,    0.62348     ,    0.62121     ,    0.61302     ,    0.61663     ,    0.63150     ,    0.56910     ,    0.45680     ,    0.40957     ,    0.40507     ,    0.39226     ,    0.38671     ,    0.39418     ,    0.40370     ,    0.41286     ,    0.43108     ,    0.46394     ,    0.49266     ,    0.50788     ,    0.52439     ,    0.55598     ,    0.58369     ,    0.56668     ,    0.50329     ,    0.46126     ,    0.52886     ,    0.53140     ,    0.44229     ,    0.41224     ,    0.39318     ,    0.42010     ,    0.44895     ,    0.44194     ,    0.40058     ,    0.52115     ,    0.50253    },
    {    0.57009     ,    0.55024     ,    0.50383     ,    0.45694     ,    0.40921     ,    0.37001     ,    0.34430     ,    0.31442     ,    0.29913     ,    0.30054     ,    0.31606     ,    0.33059     ,    0.34787     ,    0.36367     ,    0.36700     ,    0.37177     ,    0.37926     ,    0.39880     ,    0.42120     ,    0.44243     ,    0.48110     ,    0.49887     ,    0.49844     ,    0.48413     ,    0.48472     ,    0.48133     ,    0.46503     ,    0.44824     ,    0.43593     ,    0.41414     ,    0.40557     ,    0.41861     ,    0.44859     ,    0.46986     ,    0.44107     ,    0.35989     ,    0.30540     ,    0.30636     ,    0.34726     ,    0.30697     ,    0.26626     ,    0.26966     ,    0.21985     ,    0.45374     ,    0.71060     ,    0.85562     ,    0.91933     ,    0.73321     ,    0.63018     ,    0.62521     ,    0.63796     ,    0.64916     ,    0.64345     ,    0.66660     ,    0.70711     ,    0.70576     ,    0.70136     ,    0.81800     ,    0.96366     ,    0.76303     ,    0.43242     ,    0.37336     ,    0.37419     ,    0.40993     ,    0.40241     ,    0.39647     ,    0.42821     ,    0.54275     ,    0.65882     ,    0.69507     ,    0.55270     ,    0.47073     ,    0.49074     ,    0.47725     ,    0.47183     ,    0.50757     ,    0.51734     ,    0.55494     ,    0.59566     ,    0.58132     ,    0.53659     ,    0.42926     ,    0.37843     ,    0.37659     ,    0.43115     ,    0.45782     ,    0.44792     ,    0.43412     ,    0.42987     ,    0.42059     ,    0.42453     ,    0.45452     ,    0.49633     ,    0.49803     ,    0.48269     ,    0.48534     ,    0.48513     ,    0.45497     ,    0.38886     ,    0.34052     ,    0.32088     ,    0.31844     ,    0.32853     ,    0.34348     ,    0.35911     ,    0.36297     ,    0.38168     ,    0.40311     ,    0.43270     ,    0.46013     ,    0.47684     ,    0.50241     ,    0.52472     ,    0.50242     ,    0.42142     ,    0.38416     ,    0.41493     ,    0.45995     ,    0.51121     ,    0.45473     ,    0.39152     ,    0.39583     ,    0.43295     ,    0.45703     ,    0.47566     ,    0.47313     ,    0.49235     ,    0.50827    },
    {    0.55967     ,    0.51631     ,    0.46700     ,    0.41980     ,    0.37880     ,    0.35658     ,    0.34383     ,    0.32881     ,    0.31628     ,    0.30299     ,    0.30547     ,    0.31958     ,    0.33974     ,    0.36123     ,    0.37218     ,    0.36267     ,    0.35333     ,    0.35834     ,    0.37584     ,    0.40881     ,    0.44476     ,    0.45562     ,    0.45812     ,    0.46082     ,    0.47026     ,    0.47377     ,    0.45425     ,    0.42387     ,    0.39752     ,    0.36762     ,    0.36951     ,    0.36607     ,    0.41237     ,    0.45051     ,    0.46624     ,    0.35990     ,    0.30682     ,    0.30467     ,    0.33022     ,    0.29799     ,    0.35152     ,    0.49710     ,    0.43791     ,    0.39922     ,    0.67802     ,    1.05825     ,    1.13216     ,    0.95513     ,    0.78570     ,    0.72481     ,    0.70831     ,    0.69456     ,    0.69776     ,    0.69040     ,    0.65304     ,    0.61054     ,    0.58086     ,    0.57999     ,    0.57693     ,    0.45978     ,    0.41415     ,    0.37585     ,    0.37520     ,    0.39178     ,    0.37361     ,    0.41754     ,    0.47901     ,    0.53073     ,    0.54358     ,    0.51548     ,    0.43941     ,    0.35611     ,    0.33003     ,    0.32963     ,    0.33263     ,    0.36255     ,    0.36064     ,    0.36105     ,    0.37351     ,    0.38339     ,    0.36521     ,    0.32790     ,    0.30179     ,    0.29526     ,    0.30727     ,    0.32316     ,    0.33674     ,    0.35036     ,    0.36237     ,    0.37429     ,    0.37969     ,    0.39634     ,    0.40196     ,    0.37326     ,    0.35933     ,    0.35206     ,    0.33186     ,    0.29558     ,    0.26505     ,    0.26244     ,    0.28311     ,    0.30336     ,    0.32646     ,    0.33908     ,    0.34716     ,    0.36284     ,    0.37939     ,    0.37866     ,    0.39833     ,    0.41224     ,    0.42874     ,    0.42306     ,    0.40601     ,    0.39948     ,    0.38994     ,    0.41063     ,    0.38217     ,    0.35239     ,    0.41212     ,    0.40435     ,    0.34360     ,    0.36395     ,    0.42852     ,    0.46333     ,    0.47900     ,    0.50683     ,    0.51178     ,    0.54163    },
    {    0.46880     ,    0.45345     ,    0.42170     ,    0.38533     ,    0.35078     ,    0.32962     ,    0.31098     ,    0.30685     ,    0.30703     ,    0.30098     ,    0.30095     ,    0.31648     ,    0.32911     ,    0.34307     ,    0.35542     ,    0.35506     ,    0.35631     ,    0.35717     ,    0.38231     ,    0.40923     ,    0.41734     ,    0.42656     ,    0.44192     ,    0.45947     ,    0.47527     ,    0.47141     ,    0.43241     ,    0.40906     ,    0.39029     ,    0.36289     ,    0.37255     ,    0.39288     ,    0.40859     ,    0.39589     ,    0.38180     ,    0.33446     ,    0.31267     ,    0.26793     ,    0.24007     ,    0.34856     ,    0.36224     ,    0.52744     ,    0.67508     ,    0.66462     ,    0.45061     ,    0.46958     ,    0.48517     ,    0.43229     ,    0.47059     ,    0.51107     ,    0.49205     ,    0.48510     ,    0.49093     ,    0.47512     ,    0.45908     ,    0.44138     ,    0.42809     ,    0.39282     ,    0.34501     ,    0.37022     ,    0.37776     ,    0.32760     ,    0.30557     ,    0.31165     ,    0.35367     ,    0.42528     ,    0.47519     ,    0.44309     ,    0.40142     ,    0.35061     ,    0.26181     ,    0.20620     ,    0.19367     ,    0.21015     ,    0.22909     ,    0.22433     ,    0.21786     ,    0.22104     ,    0.26811     ,    0.27827     ,    0.27779     ,    0.28274     ,    0.27537     ,    0.27168     ,    0.28006     ,    0.29670     ,    0.32176     ,    0.36040     ,    0.38536     ,    0.39736     ,    0.40070     ,    0.39710     ,    0.39382     ,    0.38119     ,    0.37863     ,    0.34612     ,    0.33289     ,    0.33040     ,    0.32529     ,    0.32741     ,    0.32587     ,    0.30597     ,    0.31481     ,    0.32130     ,    0.32399     ,    0.32703     ,    0.33863     ,    0.35850     ,    0.37189     ,    0.38251     ,    0.39967     ,    0.39935     ,    0.39332     ,    0.36374     ,    0.31682     ,    0.43841     ,    0.62214     ,    0.48507     ,    0.33499     ,    0.33691     ,    0.30953     ,    0.33217     ,    0.40235     ,    0.45707     ,    0.48958     ,    0.49734     ,    0.49222     ,    0.53907    },
    {    0.44295     ,    0.43954     ,    0.41667     ,    0.39658     ,    0.36875     ,    0.34281     ,    0.32862     ,    0.32036     ,    0.31633     ,    0.30472     ,    0.29999     ,    0.29653     ,    0.29397     ,    0.30882     ,    0.33379     ,    0.35067     ,    0.36132     ,    0.38798     ,    0.42857     ,    0.45090     ,    0.44650     ,    0.45236     ,    0.47039     ,    0.48351     ,    0.47567     ,    0.43460     ,    0.40130     ,    0.41817     ,    0.43132     ,    0.42487     ,    0.44790     ,    0.44068     ,    0.40494     ,    0.38967     ,    0.38258     ,    0.36518     ,    0.31271     ,    0.24086     ,    0.23462     ,    0.38784     ,    0.50155     ,    0.69335     ,    0.84623     ,    0.77274     ,    0.48503     ,    0.37464     ,    0.26280     ,    0.32622     ,    0.38656     ,    0.38192     ,    0.35558     ,    0.35097     ,    0.34787     ,    0.31736     ,    0.32408     ,    0.33096     ,    0.33476     ,    0.32826     ,    0.30476     ,    0.30220     ,    0.31627     ,    0.29606     ,    0.27612     ,    0.29528     ,    0.34038     ,    0.40781     ,    0.40479     ,    0.34806     ,    0.31847     ,    0.27440     ,    0.21468     ,    0.40773     ,    0.48518     ,    0.33747     ,    0.27950     ,    0.23205     ,    0.23214     ,    0.24203     ,    0.29396     ,    0.30114     ,    0.28761     ,    0.30781     ,    0.32415     ,    0.32806     ,    0.35188     ,    0.38784     ,    0.42551     ,    0.46847     ,    0.49493     ,    0.50613     ,    0.51007     ,    0.50529     ,    0.47831     ,    0.44928     ,    0.42235     ,    0.41865     ,    0.40252     ,    0.37886     ,    0.34761     ,    0.34172     ,    0.33002     ,    0.29265     ,    0.32198     ,    0.34296     ,    0.32124     ,    0.33595     ,    0.42019     ,    0.53413     ,    0.59055     ,    0.57862     ,    0.48728     ,    0.38360     ,    0.30984     ,    0.35547     ,    0.41478     ,    0.50039     ,    0.52576     ,    0.52850     ,    0.41517     ,    0.30702     ,    0.30749     ,    0.34222     ,    0.39898     ,    0.44878     ,    0.48129     ,    0.48786     ,    0.48118     ,    0.47536    },
    {    0.50199     ,    0.47006     ,    0.41911     ,    0.38661     ,    0.36721     ,    0.36026     ,    0.35890     ,    0.35960     ,    0.35053     ,    0.32833     ,    0.31194     ,    0.29476     ,    0.29106     ,    0.31070     ,    0.34938     ,    0.35689     ,    0.35723     ,    0.38246     ,    0.41629     ,    0.42381     ,    0.42293     ,    0.43420     ,    0.45336     ,    0.43392     ,    0.40247     ,    0.39197     ,    0.42293     ,    0.47302     ,    0.50012     ,    0.49772     ,    0.47938     ,    0.44290     ,    0.40844     ,    0.40950     ,    0.42039     ,    0.38109     ,    0.29752     ,    0.25667     ,    0.29660     ,    0.45653     ,    0.66360     ,    0.84540     ,    0.88925     ,    0.69652     ,    0.48930     ,    0.34174     ,    0.34272     ,    0.27729     ,    0.28666     ,    0.29232     ,    0.27785     ,    0.25686     ,    0.25927     ,    0.27019     ,    0.29763     ,    0.32000     ,    0.32622     ,    0.33109     ,    0.32864     ,    0.32267     ,    0.31368     ,    0.30151     ,    0.29975     ,    0.32180     ,    0.34828     ,    0.34980     ,    0.31508     ,    0.28041     ,    0.24755     ,    0.23174     ,    0.27806     ,    0.70754     ,    0.86793     ,    0.89377     ,    0.31154     ,    0.25999     ,    0.26148     ,    0.29751     ,    0.31429     ,    0.33316     ,    0.34997     ,    0.38478     ,    0.41959     ,    0.42671     ,    0.43549     ,    0.45976     ,    0.48488     ,    0.50866     ,    0.54074     ,    0.55006     ,    0.53871     ,    0.49252     ,    0.44680     ,    0.44851     ,    0.44557     ,    0.41354     ,    0.36884     ,    0.33816     ,    0.31954     ,    0.32363     ,    0.32860     ,    0.33702     ,    0.32323     ,    0.31465     ,    0.30819     ,    0.30943     ,    0.38465     ,    0.53474     ,    0.56835     ,    0.55369     ,    0.38967     ,    0.42341     ,    0.60100     ,    0.62699     ,    0.66333     ,    0.72925     ,    0.71257     ,    0.65636     ,    0.69553     ,    0.37404     ,    0.35586     ,    0.40436     ,    0.43942     ,    0.48426     ,    0.50132     ,    0.49210     ,    0.49843     ,    0.51816    },
    {    0.50437     ,    0.49307     ,    0.47260     ,    0.45730     ,    0.45074     ,    0.45095     ,    0.43517     ,    0.40654     ,    0.36642     ,    0.33605     ,    0.30767     ,    0.28387     ,    0.27017     ,    0.27688     ,    0.30050     ,    0.30415     ,    0.30615     ,    0.30918     ,    0.32685     ,    0.33762     ,    0.34418     ,    0.35117     ,    0.35009     ,    0.33150     ,    0.33647     ,    0.37674     ,    0.41939     ,    0.45456     ,    0.47557     ,    0.48209     ,    0.47597     ,    0.46079     ,    0.45379     ,    0.44641     ,    0.42007     ,    0.32855     ,    0.28750     ,    0.27193     ,    0.27906     ,    0.39477     ,    0.54337     ,    0.51375     ,    0.61555     ,    0.62414     ,    0.67267     ,    0.32190     ,    0.27178     ,    0.27965     ,    0.25981     ,    0.23967     ,    0.24630     ,    0.25517     ,    0.28019     ,    0.30380     ,    0.31879     ,    0.33282     ,    0.34107     ,    0.34478     ,    0.33474     ,    0.31339     ,    0.30803     ,    0.30917     ,    0.32109     ,    0.32580     ,    0.30737     ,    0.28183     ,    0.26293     ,    0.23792     ,    0.25064     ,    0.26830     ,    0.54895     ,    0.78407     ,    0.83287     ,    0.71398     ,    0.53696     ,    0.46169     ,    0.39177     ,    0.27854     ,    0.31744     ,    0.32104     ,    0.34923     ,    0.36829     ,    0.40348     ,    0.40094     ,    0.41684     ,    0.44400     ,    0.43639     ,    0.44717     ,    0.46339     ,    0.44970     ,    0.42737     ,    0.37012     ,    0.34845     ,    0.33589     ,    0.32534     ,    0.30977     ,    0.31018     ,    0.33073     ,    0.34627     ,    0.35737     ,    0.36536     ,    0.33117     ,    0.30167     ,    0.29477     ,    0.33003     ,    0.58816     ,    0.52701     ,    0.50922     ,    0.55824     ,    0.54543     ,    0.56280     ,    0.75213     ,    0.97466     ,    0.98929     ,    1.06957     ,    0.75286     ,    0.57946     ,    0.65145     ,    0.56557     ,    0.38947     ,    0.40604     ,    0.42929     ,    0.45096     ,    0.45535     ,    0.46253     ,    0.45536     ,    0.46291     ,    0.48341    },
    {    0.56343     ,    0.55410     ,    0.55162     ,    0.56345     ,    0.56438     ,    0.53431     ,    0.48315     ,    0.43642     ,    0.38787     ,    0.34077     ,    0.30213     ,    0.27940     ,    0.24906     ,    0.25110     ,    0.27216     ,    0.28994     ,    0.30026     ,    0.30175     ,    0.31201     ,    0.30938     ,    0.31145     ,    0.32013     ,    0.31824     ,    0.31774     ,    0.33590     ,    0.38468     ,    0.43953     ,    0.48047     ,    0.48754     ,    0.49844     ,    0.50835     ,    0.50446     ,    0.48085     ,    0.42700     ,    0.36653     ,    0.28417     ,    0.28430     ,    0.24381     ,    0.24977     ,    0.63208     ,    0.79082     ,    0.36165     ,    0.68129     ,    0.85044     ,    0.88207     ,    0.63901     ,    0.57959     ,    0.64667     ,    0.55352     ,    0.31470     ,    0.24537     ,    0.26803     ,    0.28883     ,    0.31067     ,    0.33048     ,    0.33630     ,    0.32942     ,    0.31214     ,    0.29581     ,    0.28543     ,    0.29503     ,    0.29992     ,    0.29674     ,    0.28789     ,    0.26386     ,    0.24840     ,    0.23487     ,    0.23092     ,    0.30471     ,    0.20234     ,    0.66103     ,    0.89659     ,    1.06424     ,    0.87501     ,    0.62543     ,    0.67728     ,    0.71140     ,    0.28021     ,    0.28702     ,    0.31087     ,    0.48568     ,    0.36656     ,    0.31276     ,    0.31121     ,    0.35623     ,    0.38201     ,    0.38980     ,    0.38713     ,    0.38092     ,    0.37191     ,    0.33993     ,    0.31055     ,    0.29666     ,    0.29314     ,    0.29871     ,    0.30903     ,    0.32741     ,    0.33434     ,    0.33694     ,    0.34378     ,    0.32981     ,    0.29228     ,    0.28554     ,    0.28359     ,    0.41387     ,    0.65117     ,    0.68419     ,    0.64674     ,    0.74207     ,    0.75612     ,    0.93535     ,    1.15477     ,    1.16276     ,    1.27389     ,    1.19942     ,    0.72204     ,    0.87421     ,    0.69418     ,    0.40792     ,    0.33269     ,    0.34934     ,    0.36325     ,    0.37355     ,    0.37825     ,    0.39832     ,    0.41316     ,    0.44589     ,    0.49612    },
    {    0.55083     ,    0.55384     ,    0.57186     ,    0.58076     ,    0.56656     ,    0.51180     ,    0.45145     ,    0.40053     ,    0.35950     ,    0.32507     ,    0.30550     ,    0.29392     ,    0.28796     ,    0.30865     ,    0.32686     ,    0.34466     ,    0.34310     ,    0.33723     ,    0.33315     ,    0.34219     ,    0.34876     ,    0.35555     ,    0.36573     ,    0.38934     ,    0.42377     ,    0.47254     ,    0.51193     ,    0.52692     ,    0.51164     ,    0.50862     ,    0.49026     ,    0.44906     ,    0.39345     ,    0.33113     ,    0.26790     ,    0.26042     ,    0.26716     ,    0.19941     ,    0.19884     ,    0.64217     ,    1.03873     ,    0.40596     ,    0.57093     ,    0.75939     ,    0.74869     ,    0.80398     ,    0.70540     ,    0.73024     ,    0.71049     ,    0.46032     ,    0.26461     ,    0.27852     ,    0.31083     ,    0.31417     ,    0.30969     ,    0.30007     ,    0.28664     ,    0.27247     ,    0.26221     ,    0.25766     ,    0.25911     ,    0.26059     ,    0.26221     ,    0.25558     ,    0.24322     ,    0.25212     ,    0.26329     ,    0.26192     ,    0.31182     ,    0.21132     ,    0.65042     ,    0.91864     ,    1.11903     ,    0.98466     ,    0.72309     ,    0.75027     ,    0.56157     ,    0.23650     ,    0.23688     ,    0.27423     ,    0.43109     ,    0.34619     ,    0.26194     ,    0.29099     ,    0.33738     ,    0.34503     ,    0.35322     ,    0.34649     ,    0.34209     ,    0.31146     ,    0.28292     ,    0.26985     ,    0.25674     ,    0.26052     ,    0.27085     ,    0.27732     ,    0.28358     ,    0.29772     ,    0.31124     ,    0.30999     ,    0.30181     ,    0.30059     ,    0.29051     ,    0.28224     ,    0.29428     ,    0.34792     ,    0.48272     ,    0.57951     ,    0.66936     ,    0.81542     ,    0.97399     ,    1.24625     ,    1.07355     ,    1.06649     ,    0.80546     ,    0.94248     ,    0.88535     ,    0.53617     ,    0.27206     ,    0.30416     ,    0.33519     ,    0.34940     ,    0.37261     ,    0.36765     ,    0.38819     ,    0.42346     ,    0.47430     ,    0.52599    },
    {    0.55188     ,    0.54343     ,    0.53507     ,    0.50998     ,    0.46999     ,    0.40256     ,    0.35349     ,    0.33536     ,    0.33235     ,    0.33684     ,    0.33940     ,    0.34278     ,    0.35250     ,    0.35756     ,    0.37226     ,    0.36812     ,    0.35975     ,    0.35770     ,    0.36521     ,    0.37469     ,    0.38456     ,    0.39886     ,    0.41250     ,    0.44036     ,    0.46814     ,    0.48860     ,    0.49537     ,    0.48190     ,    0.46505     ,    0.43639     ,    0.39545     ,    0.34705     ,    0.30824     ,    0.26533     ,    0.23713     ,    0.23677     ,    0.20978     ,    0.15686     ,    0.21379     ,    0.42160     ,    0.45199     ,    0.34691     ,    0.52463     ,    0.76072     ,    0.74084     ,    0.70303     ,    0.74848     ,    0.82846     ,    0.86861     ,    0.58230     ,    0.35555     ,    0.26534     ,    0.28017     ,    0.28427     ,    0.28584     ,    0.27441     ,    0.27511     ,    0.26973     ,    0.26669     ,    0.27491     ,    0.28663     ,    0.29482     ,    0.28957     ,    0.27481     ,    0.25608     ,    0.24510     ,    0.23464     ,    0.26113     ,    0.27496     ,    0.48257     ,    0.74861     ,    0.76684     ,    0.77924     ,    0.72330     ,    0.57546     ,    0.62934     ,    0.63188     ,    0.53053     ,    0.21578     ,    0.20098     ,    0.33117     ,    0.35726     ,    0.23985     ,    0.23875     ,    0.26099     ,    0.28521     ,    0.28463     ,    0.28427     ,    0.26367     ,    0.23721     ,    0.22101     ,    0.21790     ,    0.21692     ,    0.21637     ,    0.22251     ,    0.23651     ,    0.26393     ,    0.29157     ,    0.31025     ,    0.32567     ,    0.34166     ,    0.32361     ,    0.31376     ,    0.31542     ,    0.29555     ,    0.28333     ,    0.30786     ,    0.34905     ,    0.54452     ,    0.66889     ,    0.71529     ,    0.70604     ,    0.61802     ,    0.59099     ,    0.60962     ,    0.68036     ,    0.38328     ,    0.22628     ,    0.27352     ,    0.31623     ,    0.33589     ,    0.33432     ,    0.35072     ,    0.36940     ,    0.39498     ,    0.43461     ,    0.48182     ,    0.50852    },
    {    0.44753     ,    0.42673     ,    0.39934     ,    0.37136     ,    0.34181     ,    0.33457     ,    0.32542     ,    0.32912     ,    0.33378     ,    0.33336     ,    0.34052     ,    0.34973     ,    0.35150     ,    0.35144     ,    0.34519     ,    0.33615     ,    0.33714     ,    0.33603     ,    0.33881     ,    0.35337     ,    0.37376     ,    0.37128     ,    0.37848     ,    0.38789     ,    0.38722     ,    0.38215     ,    0.37088     ,    0.35832     ,    0.34035     ,    0.32149     ,    0.29894     ,    0.27139     ,    0.25551     ,    0.24129     ,    0.23851     ,    0.23086     ,    0.20892     ,    0.20690     ,    0.25486     ,    1.86007     ,    0.35537     ,    0.64314     ,    0.72142     ,    0.66024     ,    0.58413     ,    0.65452     ,    0.58350     ,    0.67308     ,    0.82578     ,    0.65349     ,    0.33452     ,    0.25159     ,    0.25873     ,    0.26895     ,    0.28255     ,    0.28519     ,    0.29152     ,    0.30241     ,    0.31221     ,    0.32128     ,    0.32141     ,    0.31727     ,    0.30401     ,    0.28226     ,    0.26190     ,    0.24370     ,    0.21740     ,    0.24048     ,    0.22860     ,    0.52975     ,    0.60464     ,    0.59822     ,    0.63541     ,    0.65473     ,    0.61312     ,    0.54437     ,    0.63492     ,    0.70239     ,    0.43448     ,    0.16517     ,    0.19498     ,    0.27770     ,    0.20520     ,    0.21320     ,    0.22189     ,    0.23925     ,    0.22187     ,    0.21105     ,    0.19703     ,    0.19181     ,    0.18524     ,    0.18984     ,    0.18486     ,    0.18459     ,    0.20159     ,    0.22328     ,    0.22613     ,    0.24048     ,    0.26724     ,    0.30295     ,    0.32302     ,    0.29726     ,    0.30300     ,    0.27711     ,    0.24312     ,    0.24035     ,    0.25374     ,    0.26626     ,    0.29286     ,    0.53117     ,    0.42564     ,    0.51568     ,    0.48894     ,    0.28592     ,    0.29013     ,    0.53416     ,    0.21067     ,    0.24562     ,    0.29128     ,    0.32520     ,    0.33693     ,    0.34568     ,    0.36606     ,    0.37917     ,    0.39435     ,    0.42093     ,    0.46129     ,    0.47317    },
    {    0.32395     ,    0.32268     ,    0.32163     ,    0.31652     ,    0.33039     ,    0.33325     ,    0.32670     ,    0.32474     ,    0.32177     ,    0.32082     ,    0.31893     ,    0.31284     ,    0.30038     ,    0.29738     ,    0.28113     ,    0.26963     ,    0.27256     ,    0.26794     ,    0.27292     ,    0.28093     ,    0.28612     ,    0.27469     ,    0.28988     ,    0.27477     ,    0.26474     ,    0.25798     ,    0.25168     ,    0.24768     ,    0.24592     ,    0.24692     ,    0.24571     ,    0.23718     ,    0.23357     ,    0.23231     ,    0.23881     ,    0.22674     ,    0.24640     ,    0.27745     ,    0.27617     ,    0.35615     ,    0.55364     ,    0.54358     ,    0.53605     ,    0.61730     ,    0.54948     ,    0.48633     ,    0.60282     ,    0.49878     ,    0.62130     ,    0.49194     ,    0.33329     ,    0.26740     ,    0.25753     ,    0.25013     ,    0.25907     ,    0.26471     ,    0.26971     ,    0.27579     ,    0.28218     ,    0.28197     ,    0.28746     ,    0.29236     ,    0.28729     ,    0.28693     ,    0.27512     ,    0.24467     ,    0.19955     ,    0.19460     ,    0.21993     ,    0.39263     ,    0.37528     ,    0.44267     ,    0.55731     ,    0.57652     ,    0.58044     ,    0.58926     ,    0.49281     ,    0.57795     ,    0.43411     ,    0.15834     ,    0.15299     ,    0.18378     ,    0.19400     ,    0.19594     ,    0.19317     ,    0.20097     ,    0.19916     ,    0.19454     ,    0.18419     ,    0.16991     ,    0.15646     ,    0.15404     ,    0.15673     ,    0.15281     ,    0.17330     ,    0.18042     ,    0.17516     ,    0.17932     ,    0.19515     ,    0.23251     ,    0.25220     ,    0.22732     ,    0.22182     ,    0.19656     ,    0.18053     ,    0.16805     ,    0.16733     ,    0.18251     ,    0.19714     ,    0.20630     ,    0.19339     ,    0.24715     ,    0.31427     ,    0.14986     ,    0.16602     ,    0.24902     ,    0.21373     ,    0.20637     ,    0.22073     ,    0.24965     ,    0.27997     ,    0.30741     ,    0.32979     ,    0.33729     ,    0.34149     ,    0.34581     ,    0.33986     ,    0.33712    },
    {    0.28948     ,    0.29128     ,    0.29441     ,    0.30123     ,    0.31132     ,    0.31102     ,    0.30864     ,    0.30506     ,    0.30460     ,    0.29218     ,    0.28938     ,    0.28601     ,    0.27639     ,    0.27287     ,    0.25585     ,    0.23779     ,    0.22977     ,    0.22279     ,    0.22963     ,    0.22273     ,    0.22101     ,    0.21378     ,    0.21646     ,    0.19871     ,    0.20112     ,    0.19721     ,    0.19543     ,    0.19404     ,    0.20152     ,    0.21330     ,    0.22495     ,    0.23726     ,    0.24162     ,    0.24628     ,    0.25171     ,    0.26474     ,    0.29544     ,    0.29596     ,    0.32706     ,    0.48857     ,    0.48969     ,    0.49514     ,    0.42426     ,    0.60710     ,    0.63949     ,    0.41574     ,    0.40279     ,    0.40659     ,    0.48474     ,    0.45286     ,    0.50240     ,    0.28912     ,    0.24158     ,    0.23231     ,    0.24047     ,    0.25055     ,    0.26416     ,    0.27517     ,    0.28561     ,    0.29105     ,    0.29553     ,    0.30275     ,    0.30502     ,    0.29475     ,    0.26210     ,    0.21822     ,    0.17780     ,    0.17490     ,    0.19019     ,    0.45956     ,    0.57651     ,    0.33060     ,    0.49429     ,    0.54833     ,    0.56763     ,    0.51110     ,    0.51999     ,    0.48783     ,    0.31532     ,    0.16904     ,    0.17721     ,    0.17836     ,    0.18610     ,    0.19946     ,    0.20499     ,    0.20814     ,    0.20086     ,    0.19433     ,    0.18562     ,    0.17784     ,    0.16742     ,    0.16447     ,    0.15909     ,    0.16308     ,    0.14775     ,    0.12881     ,    0.12954     ,    0.12329     ,    0.15312     ,    0.17157     ,    0.17453     ,    0.18946     ,    0.18654     ,    0.15734     ,    0.13246     ,    0.13517     ,    0.13301     ,    0.13844     ,    0.15691     ,    0.15751     ,    0.15231     ,    0.14587     ,    0.14424     ,    0.15449     ,    0.16024     ,    0.18736     ,    0.18838     ,    0.16712     ,    0.16370     ,    0.18654     ,    0.22187     ,    0.25070     ,    0.27410     ,    0.28532     ,    0.29495     ,    0.29502     ,    0.29166     ,    0.28651    },
    {    0.27703     ,    0.27968     ,    0.28049     ,    0.28788     ,    0.29398     ,    0.30319     ,    0.31580     ,    0.29878     ,    0.28634     ,    0.28047     ,    0.27824     ,    0.26071     ,    0.26380     ,    0.26599     ,    0.25489     ,    0.22755     ,    0.22135     ,    0.21763     ,    0.21683     ,    0.21416     ,    0.22069     ,    0.21599     ,    0.20304     ,    0.20446     ,    0.20194     ,    0.20424     ,    0.20600     ,    0.20345     ,    0.20490     ,    0.22027     ,    0.23601     ,    0.24961     ,    0.25896     ,    0.27003     ,    0.28942     ,    0.30729     ,    0.31827     ,    0.41478     ,    0.53118     ,    0.52991     ,    0.54162     ,    0.49878     ,    0.48888     ,    0.42242     ,    0.41792     ,    0.41311     ,    0.29257     ,    0.30346     ,    0.36677     ,    0.39279     ,    0.45364     ,    0.31690     ,    0.23648     ,    0.23074     ,    0.23110     ,    0.24801     ,    0.27597     ,    0.29989     ,    0.31223     ,    0.31482     ,    0.30395     ,    0.30471     ,    0.28784     ,    0.25949     ,    0.21790     ,    0.18411     ,    0.16945     ,    0.16397     ,    0.18571     ,    0.39970     ,    0.35858     ,    0.31993     ,    0.47261     ,    0.61089     ,    0.51761     ,    0.35866     ,    0.30387     ,    0.63138     ,    0.28103     ,    0.20098     ,    0.20166     ,    0.20388     ,    0.20421     ,    0.20882     ,    0.22356     ,    0.20540     ,    0.21170     ,    0.21810     ,    0.22823     ,    0.21678     ,    0.20925     ,    0.19669     ,    0.17780     ,    0.15535     ,    0.14252     ,    0.13336     ,    0.12780     ,    0.12374     ,    0.13141     ,    0.15525     ,    0.20721     ,    0.20853     ,    0.23795     ,    0.23845     ,    0.21624     ,    0.17480     ,    0.15175     ,    0.14406     ,    0.13304     ,    0.14408     ,    0.14013     ,    0.14606     ,    0.14694     ,    0.13745     ,    0.20687     ,    0.28024     ,    0.31611     ,    0.19219     ,    0.17147     ,    0.20933     ,    0.23338     ,    0.25191     ,    0.26360     ,    0.28596     ,    0.29261     ,    0.29917     ,    0.28524     ,    0.27532    },
    {    0.32121     ,    0.31961     ,    0.31524     ,    0.31531     ,    0.32029     ,    0.31281     ,    0.31635     ,    0.32159     ,    0.32016     ,    0.31101     ,    0.31440     ,    0.30683     ,    0.29918     ,    0.28536     ,    0.28959     ,    0.27976     ,    0.28558     ,    0.28232     ,    0.28413     ,    0.28166     ,    0.27879     ,    0.28182     ,    0.28090     ,    0.27777     ,    0.27492     ,    0.26955     ,    0.26589     ,    0.26510     ,    0.26441     ,    0.26409     ,    0.27528     ,    0.28148     ,    0.28430     ,    0.29273     ,    0.31932     ,    0.27603     ,    0.36583     ,    0.39348     ,    0.57967     ,    0.63518     ,    0.58969     ,    0.53104     ,    0.45707     ,    0.41278     ,    0.37919     ,    0.34221     ,    0.37617     ,    0.41788     ,    0.30173     ,    0.40363     ,    0.38124     ,    0.25955     ,    0.23096     ,    0.22447     ,    0.22739     ,    0.23857     ,    0.25551     ,    0.26516     ,    0.26464     ,    0.25395     ,    0.24509     ,    0.22800     ,    0.21582     ,    0.20606     ,    0.18642     ,    0.17441     ,    0.17760     ,    0.17922     ,    0.25494     ,    0.33857     ,    0.39390     ,    0.41327     ,    0.45902     ,    0.49190     ,    0.52128     ,    0.45113     ,    0.38190     ,    0.63992     ,    0.25757     ,    0.21217     ,    0.21933     ,    0.22723     ,    0.22172     ,    0.22476     ,    0.21239     ,    0.20626     ,    0.20069     ,    0.20404     ,    0.22455     ,    0.22813     ,    0.22516     ,    0.20999     ,    0.20323     ,    0.20332     ,    0.19057     ,    0.18228     ,    0.17831     ,    0.17512     ,    0.17882     ,    0.22947     ,    0.24360     ,    0.25575     ,    0.24169     ,    0.21619     ,    0.21644     ,    0.18645     ,    0.16607     ,    0.16919     ,    0.14696     ,    0.14089     ,    0.14321     ,    0.14030     ,    0.18204     ,    0.15462     ,    0.17062     ,    0.18333     ,    0.27683     ,    0.24772     ,    0.26664     ,    0.32934     ,    0.35000     ,    0.35356     ,    0.35243     ,    0.34431     ,    0.33699     ,    0.34293     ,    0.33458     ,    0.34015    },
    {    0.37813     ,    0.36930     ,    0.36332     ,    0.35895     ,    0.35887     ,    0.34482     ,    0.35927     ,    0.35467     ,    0.36180     ,    0.36052     ,    0.36224     ,    0.33931     ,    0.34714     ,    0.33296     ,    0.35357     ,    0.34778     ,    0.34922     ,    0.34315     ,    0.34860     ,    0.35124     ,    0.34820     ,    0.35482     ,    0.35151     ,    0.35281     ,    0.35032     ,    0.35181     ,    0.34332     ,    0.34126     ,    0.35600     ,    0.35777     ,    0.36454     ,    0.36662     ,    0.35226     ,    0.32337     ,    0.31636     ,    0.25804     ,    0.17309     ,    0.31938     ,    0.41829     ,    0.51771     ,    0.55595     ,    0.54370     ,    0.55395     ,    0.40737     ,    0.35229     ,    0.34025     ,    0.37616     ,    0.26803     ,    0.24768     ,    0.26583     ,    0.25697     ,    0.24753     ,    0.23219     ,    0.23613     ,    0.23770     ,    0.24284     ,    0.24944     ,    0.24584     ,    0.23294     ,    0.21837     ,    0.21775     ,    0.21352     ,    0.21206     ,    0.21602     ,    0.21185     ,    0.20551     ,    0.20635     ,    0.28392     ,    0.30073     ,    0.29925     ,    0.40418     ,    0.47017     ,    0.48720     ,    0.47924     ,    0.43938     ,    0.45914     ,    0.52533     ,    0.66720     ,    0.38012     ,    0.21070     ,    0.21201     ,    0.23663     ,    0.23667     ,    0.22562     ,    0.23419     ,    0.21984     ,    0.20977     ,    0.21017     ,    0.22373     ,    0.22318     ,    0.22411     ,    0.20776     ,    0.19179     ,    0.19600     ,    0.21042     ,    0.21370     ,    0.21219     ,    0.20570     ,    0.19644     ,    0.21000     ,    0.16111     ,    0.20602     ,    0.23860     ,    0.23997     ,    0.25233     ,    0.26267     ,    0.18504     ,    0.19788     ,    0.16052     ,    0.16670     ,    0.15479     ,    0.17017     ,    0.23178     ,    0.26761     ,    0.29085     ,    0.33252     ,    0.35639     ,    0.39068     ,    0.46244     ,    0.52171     ,    0.54137     ,    0.54319     ,    0.52984     ,    0.50477     ,    0.50117     ,    0.50413     ,    0.46728     ,    0.43777    },
    {    0.41162     ,    0.39221     ,    0.37089     ,    0.35488     ,    0.33905     ,    0.33064     ,    0.33751     ,    0.33795     ,    0.34657     ,    0.34682     ,    0.34185     ,    0.31774     ,    0.32582     ,    0.32436     ,    0.32773     ,    0.33615     ,    0.33615     ,    0.33653     ,    0.34949     ,    0.35694     ,    0.36918     ,    0.36853     ,    0.36405     ,    0.36306     ,    0.36805     ,    0.37254     ,    0.36859     ,    0.37711     ,    0.37216     ,    0.37563     ,    0.36708     ,    0.34121     ,    0.30731     ,    0.28264     ,    0.25562     ,    0.22967     ,    0.18872     ,    0.26213     ,    0.29191     ,    0.37616     ,    0.45776     ,    0.46546     ,    0.42575     ,    0.39740     ,    0.31940     ,    0.39414     ,    0.31234     ,    0.24909     ,    0.26517     ,    0.26548     ,    0.25443     ,    0.23601     ,    0.22724     ,    0.22706     ,    0.22295     ,    0.22168     ,    0.22076     ,    0.22466     ,    0.22180     ,    0.21665     ,    0.22304     ,    0.22912     ,    0.23005     ,    0.23133     ,    0.23674     ,    0.23456     ,    0.24459     ,    0.28060     ,    0.29760     ,    0.35402     ,    0.43142     ,    0.50477     ,    0.49481     ,    0.58820     ,    0.51882     ,    0.76402     ,    0.94886     ,    0.50777     ,    0.38250     ,    0.32108     ,    0.19387     ,    0.23941     ,    0.23190     ,    0.20236     ,    0.21251     ,    0.21633     ,    0.21067     ,    0.21352     ,    0.21578     ,    0.21983     ,    0.20524     ,    0.20119     ,    0.18882     ,    0.19659     ,    0.19453     ,    0.19322     ,    0.19959     ,    0.19679     ,    0.18250     ,    0.18135     ,    0.19819     ,    0.22103     ,    0.26023     ,    0.23823     ,    0.20610     ,    0.20834     ,    0.16738     ,    0.15573     ,    0.17285     ,    0.17881     ,    0.18693     ,    0.25854     ,    0.30448     ,    0.32032     ,    0.34497     ,    0.38086     ,    0.42211     ,    0.46971     ,    0.51602     ,    0.55093     ,    0.57600     ,    0.59037     ,    0.60453     ,    0.58890     ,    0.59083     ,    0.57294     ,    0.53237     ,    0.48133    },
    {    0.34311     ,    0.34101     ,    0.33567     ,    0.32189     ,    0.31241     ,    0.30906     ,    0.30395     ,    0.29168     ,    0.28746     ,    0.28010     ,    0.26884     ,    0.25775     ,    0.25350     ,    0.25442     ,    0.25101     ,    0.24527     ,    0.24309     ,    0.24884     ,    0.25159     ,    0.24772     ,    0.24261     ,    0.25635     ,    0.25941     ,    0.27120     ,    0.28555     ,    0.28955     ,    0.30764     ,    0.30600     ,    0.31519     ,    0.27936     ,    0.26127     ,    0.21331     ,    0.18187     ,    0.16979     ,    0.16135     ,    0.18640     ,    0.22721     ,    0.24883     ,    0.39117     ,    0.38316     ,    0.38583     ,    0.39469     ,    0.36115     ,    0.40490     ,    0.41142     ,    0.44491     ,    0.24667     ,    0.24357     ,    0.25840     ,    0.26446     ,    0.25591     ,    0.23487     ,    0.22499     ,    0.23485     ,    0.22631     ,    0.23740     ,    0.22792     ,    0.22295     ,    0.22734     ,    0.23954     ,    0.22816     ,    0.23321     ,    0.21407     ,    0.23994     ,    0.25467     ,    0.26185     ,    0.24108     ,    0.28151     ,    0.34379     ,    0.40070     ,    0.43056     ,    0.51377     ,    0.57821     ,    0.51900     ,    0.43701     ,    0.92656     ,    0.68508     ,    0.51457     ,    0.41131     ,    0.41270     ,    0.47982     ,    0.15965     ,    0.20135     ,    0.19502     ,    0.17873     ,    0.17903     ,    0.17721     ,    0.18410     ,    0.19570     ,    0.20930     ,    0.21139     ,    0.21563     ,    0.21065     ,    0.19433     ,    0.18521     ,    0.18387     ,    0.17731     ,    0.17513     ,    0.15760     ,    0.17531     ,    0.19861     ,    0.22097     ,    0.24009     ,    0.22834     ,    0.19920     ,    0.23295     ,    0.23549     ,    0.15320     ,    0.14819     ,    0.18045     ,    0.22257     ,    0.24113     ,    0.25129     ,    0.26373     ,    0.28988     ,    0.30771     ,    0.35835     ,    0.36632     ,    0.38884     ,    0.41218     ,    0.41451     ,    0.43838     ,    0.43431     ,    0.42211     ,    0.42292     ,    0.38684     ,    0.36989     ,    0.3542    },
    {    0.26335     ,    0.25949     ,    0.26040     ,    0.25453     ,    0.25711     ,    0.25817     ,    0.25878     ,    0.24714     ,    0.24432     ,    0.23872     ,    0.23210     ,    0.23530     ,    0.21406     ,    0.21543     ,    0.21585     ,    0.21425     ,    0.19713     ,    0.19459     ,    0.18918     ,    0.20284     ,    0.20031     ,    0.21354     ,    0.21487     ,    0.22646     ,    0.24493     ,    0.26719     ,    0.26451     ,    0.25130     ,    0.26443     ,    0.24200     ,    0.23059     ,    0.20144     ,    0.19222     ,    0.18471     ,    0.18346     ,    0.19340     ,    0.21983     ,    0.38588     ,    0.36615     ,    0.36893     ,    0.48845     ,    0.48995     ,    0.38582     ,    0.36042     ,    0.24942     ,    0.23604     ,    0.23275     ,    0.24448     ,    0.25441     ,    0.25571     ,    0.25226     ,    0.23699     ,    0.23379     ,    0.24970     ,    0.23757     ,    0.24182     ,    0.25855     ,    0.24104     ,    0.24167     ,    0.24297     ,    0.33070     ,    0.43235     ,    0.34713     ,    0.31696     ,    0.36301     ,    0.29247     ,    0.28901     ,    0.34512     ,    0.39488     ,    0.62214     ,    0.67098     ,    0.58935     ,    0.51520     ,    0.43620     ,    0.42214     ,    0.53636     ,    0.62554     ,    0.53392     ,    0.44970     ,    0.54915     ,    0.42130     ,    0.23293     ,    0.17279     ,    0.19377     ,    0.16422     ,    0.16374     ,    0.16317     ,    0.16180     ,    0.16139     ,    0.17651     ,    0.17856     ,    0.20202     ,    0.21707     ,    0.19214     ,    0.18104     ,    0.17203     ,    0.16046     ,    0.15294     ,    0.13755     ,    0.16817     ,    0.21978     ,    0.22721     ,    0.25352     ,    0.22484     ,    0.19372     ,    0.17979     ,    0.17676     ,    0.18001     ,    0.16988     ,    0.18853     ,    0.20209     ,    0.19599     ,    0.18580     ,    0.18513     ,    0.21066     ,    0.22379     ,    0.22184     ,    0.24053     ,    0.26133     ,    0.28424     ,    0.29068     ,    0.29129     ,    0.29008     ,    0.27103     ,    0.29324     ,    0.28532     ,    0.27132     ,    0.27284    },
    {    0.34126     ,    0.33651     ,    0.32533     ,    0.31928     ,    0.29962     ,    0.28996     ,    0.27924     ,    0.27278     ,    0.27753     ,    0.28655     ,    0.29281     ,    0.29518     ,    0.29283     ,    0.28433     ,    0.27607     ,    0.25857     ,    0.24091     ,    0.22670     ,    0.21779     ,    0.19939     ,    0.20592     ,    0.18084     ,    0.19348     ,    0.20245     ,    0.22450     ,    0.24829     ,    0.26888     ,    0.24429     ,    0.24390     ,    0.25117     ,    0.23644     ,    0.22628     ,    0.20092     ,    0.20093     ,    0.24294     ,    0.22580     ,    0.23697     ,    0.24836     ,    0.31081     ,    0.44031     ,    0.38315     ,    0.23168     ,    0.23615     ,    0.20657     ,    0.20947     ,    0.21635     ,    0.22803     ,    0.24091     ,    0.23908     ,    0.22430     ,    0.22255     ,    0.23313     ,    0.23967     ,    0.24809     ,    0.25336     ,    0.24684     ,    0.23248     ,    0.23083     ,    0.24907     ,    0.31390     ,    0.41187     ,    0.50593     ,    0.53536     ,    0.55323     ,    0.40204     ,    0.44673     ,    0.43986     ,    0.35854     ,    0.51881     ,    0.59376     ,    0.68094     ,    0.80162     ,    0.60580     ,    0.43658     ,    0.49525     ,    0.54320     ,    0.43789     ,    0.51147     ,    0.40219     ,    0.58738     ,    0.31750     ,    0.24459     ,    0.18679     ,    0.20632     ,    0.20073     ,    0.19462     ,    0.17625     ,    0.16619     ,    0.15857     ,    0.16808     ,    0.15668     ,    0.32346     ,    0.30037     ,    0.24928     ,    0.19538     ,    0.16794     ,    0.16401     ,    0.16276     ,    0.13981     ,    0.19428     ,    0.19561     ,    0.16781     ,    0.18129     ,    0.24482     ,    0.20895     ,    0.18113     ,    0.17243     ,    0.17680     ,    0.20451     ,    0.17905     ,    0.17771     ,    0.16030     ,    0.16076     ,    0.16472     ,    0.17636     ,    0.18906     ,    0.18740     ,    0.17404     ,    0.18527     ,    0.20424     ,    0.24225     ,    0.26700     ,    0.28049     ,    0.28829     ,    0.30698     ,    0.32190     ,    0.33553     ,    0.34542    },
    {    0.34219     ,    0.34878     ,    0.35611     ,    0.35742     ,    0.34695     ,    0.33782     ,    0.32468     ,    0.32043     ,    0.33158     ,    0.35392     ,    0.37675     ,    0.39955     ,    0.41616     ,    0.41790     ,    0.40836     ,    0.39389     ,    0.36418     ,    0.32958     ,    0.29185     ,    0.24815     ,    0.20580     ,    0.17050     ,    0.16036     ,    0.17009     ,    0.18558     ,    0.19969     ,    0.19443     ,    0.19212     ,    0.22028     ,    0.25497     ,    0.22683     ,    0.19536     ,    0.19721     ,    0.26481     ,    0.34882     ,    0.24608     ,    0.26828     ,    0.26692     ,    0.28524     ,    0.27625     ,    0.26546     ,    0.24570     ,    0.21185     ,    0.22339     ,    0.23584     ,    0.24313     ,    0.25477     ,    0.25819     ,    0.25749     ,    0.25878     ,    0.26135     ,    0.26925     ,    0.27819     ,    0.29630     ,    0.30890     ,    0.29964     ,    0.26596     ,    0.23340     ,    0.31775     ,    0.49300     ,    0.44004     ,    0.46550     ,    0.57224     ,    0.56310     ,    0.49191     ,    0.47140     ,    0.46109     ,    0.42590     ,    0.50205     ,    0.46903     ,    0.60944     ,    0.49860     ,    0.41737     ,    0.45747     ,    0.46809     ,    0.43075     ,    0.54807     ,    0.62784     ,    0.65716     ,    0.36635     ,    0.24095     ,    0.23177     ,    0.21458     ,    0.21078     ,    0.22432     ,    0.21669     ,    0.20988     ,    0.19736     ,    0.17632     ,    0.16511     ,    0.14404     ,    0.32548     ,    0.59775     ,    0.29017     ,    0.23535     ,    0.18894     ,    0.17379     ,    0.18465     ,    0.19270     ,    0.21662     ,    0.31111     ,    0.26931     ,    0.22929     ,    0.18664     ,    0.24896     ,    0.21256     ,    0.19134     ,    0.21810     ,    0.20428     ,    0.20455     ,    0.17453     ,    0.16302     ,    0.16223     ,    0.16139     ,    0.17159     ,    0.18826     ,    0.20092     ,    0.17581     ,    0.19072     ,    0.20584     ,    0.22081     ,    0.23845     ,    0.25303     ,    0.27082     ,    0.29505     ,    0.30963     ,    0.31744     ,    0.32546    },
    {    0.30285     ,    0.31017     ,    0.32400     ,    0.34019     ,    0.35130     ,    0.34624     ,    0.35284     ,    0.36580     ,    0.39048     ,    0.42663     ,    0.46111     ,    0.49654     ,    0.53539     ,    0.56571     ,    0.58651     ,    0.59216     ,    0.57828     ,    0.55434     ,    0.51561     ,    0.47563     ,    0.41319     ,    0.34165     ,    0.28126     ,    0.22872     ,    0.20580     ,    0.18528     ,    0.16742     ,    0.18740     ,    0.20269     ,    0.25564     ,    0.25367     ,    0.30845     ,    0.48921     ,    0.36393     ,    0.26974     ,    0.23676     ,    0.23026     ,    0.22985     ,    0.22906     ,    0.22762     ,    0.22198     ,    0.21970     ,    0.22875     ,    0.24538     ,    0.25480     ,    0.26680     ,    0.27214     ,    0.28026     ,    0.28697     ,    0.29739     ,    0.30358     ,    0.30646     ,    0.32689     ,    0.33009     ,    0.33679     ,    0.34768     ,    0.33166     ,    0.30606     ,    0.35436     ,    0.37419     ,    0.45774     ,    0.55521     ,    0.56146     ,    0.61883     ,    0.54961     ,    0.45872     ,    0.51307     ,    0.48217     ,    0.59731     ,    0.53062     ,    0.56780     ,    0.54772     ,    0.56228     ,    0.56965     ,    0.49515     ,    0.40537     ,    0.39871     ,    0.44093     ,    0.34983     ,    0.30926     ,    0.46912     ,    0.41951     ,    0.32424     ,    0.22366     ,    0.23093     ,    0.22748     ,    0.22292     ,    0.21164     ,    0.20786     ,    0.19728     ,    0.15176     ,    0.43193     ,    0.59846     ,    0.32918     ,    0.27210     ,    0.23691     ,    0.23205     ,    0.24228     ,    0.27192     ,    0.32922     ,    0.34249     ,    0.34181     ,    0.27905     ,    0.25097     ,    0.26581     ,    0.24094     ,    0.22994     ,    0.36603     ,    0.26959     ,    0.24740     ,    0.22352     ,    0.20213     ,    0.20259     ,    0.19360     ,    0.18816     ,    0.20319     ,    0.21812     ,    0.21974     ,    0.23269     ,    0.24474     ,    0.25584     ,    0.25969     ,    0.27253     ,    0.28072     ,    0.29453     ,    0.29124     ,    0.30032     ,    0.30034    },
    {    0.33089     ,    0.32214     ,    0.31815     ,    0.32444     ,    0.33421     ,    0.33467     ,    0.35469     ,    0.37700     ,    0.41223     ,    0.46685     ,    0.51428     ,    0.57041     ,    0.62536     ,    0.68760     ,    0.74151     ,    0.77427     ,    0.78350     ,    0.77301     ,    0.75732     ,    0.72677     ,    0.67826     ,    0.61583     ,    0.51983     ,    0.39518     ,    0.28111     ,    0.19255     ,    0.16952     ,    0.19560     ,    0.17086     ,    0.28073     ,    0.32926     ,    0.37842     ,    0.39053     ,    0.22090     ,    0.19499     ,    0.20510     ,    0.20469     ,    0.20854     ,    0.22849     ,    0.24760     ,    0.23894     ,    0.23800     ,    0.23423     ,    0.23744     ,    0.24772     ,    0.25865     ,    0.27890     ,    0.30176     ,    0.31785     ,    0.32581     ,    0.33257     ,    0.34822     ,    0.34065     ,    0.33260     ,    0.33790     ,    0.36017     ,    0.35766     ,    0.35806     ,    0.47136     ,    0.59672     ,    0.78072     ,    0.73245     ,    0.69593     ,    0.70187     ,    0.65523     ,    0.55540     ,    0.58212     ,    0.53125     ,    0.61813     ,    0.57174     ,    0.61486     ,    0.60988     ,    0.60783     ,    0.62820     ,    0.63255     ,    0.49199     ,    0.36060     ,    0.34038     ,    0.26507     ,    0.37071     ,    0.36444     ,    0.32108     ,    0.30675     ,    0.33560     ,    0.27857     ,    0.23059     ,    0.24943     ,    0.24046     ,    0.23722     ,    0.26956     ,    0.35170     ,    0.31417     ,    0.34538     ,    0.34253     ,    0.30007     ,    0.25975     ,    0.24647     ,    0.27886     ,    0.30882     ,    0.38257     ,    0.49522     ,    0.55065     ,    0.43510     ,    0.33885     ,    0.27178     ,    0.27364     ,    0.26304     ,    0.24530     ,    0.29216     ,    0.27650     ,    0.26626     ,    0.26977     ,    0.28054     ,    0.27731     ,    0.26608     ,    0.26515     ,    0.25578     ,    0.25636     ,    0.25498     ,    0.25752     ,    0.27345     ,    0.28667     ,    0.29591     ,    0.31209     ,    0.32397     ,    0.32838     ,    0.33183     ,    0.33371    },
    {    0.39555     ,    0.40068     ,    0.40409     ,    0.40016     ,    0.40745     ,    0.40298     ,    0.40350     ,    0.40262     ,    0.40842     ,    0.43433     ,    0.47412     ,    0.53700     ,    0.60778     ,    0.67212     ,    0.73624     ,    0.78719     ,    0.82070     ,    0.82197     ,    0.80740     ,    0.77522     ,    0.75038     ,    0.72636     ,    0.66735     ,    0.52504     ,    0.37674     ,    0.27178     ,    0.24692     ,    0.41023     ,    0.49821     ,    0.25906     ,    0.26509     ,    0.22195     ,    0.25221     ,    0.22697     ,    0.19861     ,    0.21559     ,    0.21854     ,    0.22196     ,    0.27689     ,    0.30374     ,    0.27599     ,    0.24069     ,    0.22320     ,    0.21986     ,    0.22377     ,    0.23553     ,    0.24403     ,    0.26268     ,    0.28121     ,    0.29079     ,    0.30350     ,    0.32237     ,    0.33494     ,    0.34471     ,    0.35925     ,    0.36450     ,    0.36524     ,    0.39186     ,    0.64659     ,    1.00493     ,    1.09881     ,    0.86814     ,    0.73233     ,    0.66683     ,    0.54303     ,    0.54776     ,    0.56750     ,    0.63884     ,    0.69636     ,    0.69780     ,    0.73516     ,    0.76028     ,    0.76735     ,    0.74495     ,    0.73873     ,    0.65994     ,    0.57203     ,    0.49685     ,    0.39313     ,    0.49071     ,    0.48804     ,    0.42427     ,    0.33659     ,    0.35579     ,    0.39268     ,    0.31130     ,    0.27136     ,    0.26623     ,    0.25769     ,    0.34208     ,    0.61945     ,    0.52545     ,    0.42730     ,    0.46940     ,    0.48892     ,    0.43071     ,    0.31288     ,    0.38639     ,    0.47571     ,    0.93462     ,    0.58403     ,    0.60619     ,    0.61877     ,    0.49250     ,    0.36113     ,    0.27689     ,    0.26325     ,    0.27336     ,    0.30952     ,    0.30991     ,    0.29488     ,    0.30596     ,    0.33434     ,    0.35181     ,    0.34164     ,    0.31115     ,    0.30039     ,    0.29555     ,    0.29808     ,    0.28949     ,    0.29360     ,    0.30101     ,    0.31260     ,    0.32963     ,    0.34888     ,    0.36014     ,    0.36223     ,    0.37465    },
    {    0.39714     ,    0.41117     ,    0.43661     ,    0.44655     ,    0.45391     ,    0.46103     ,    0.47441     ,    0.49093     ,    0.49999     ,    0.49276     ,    0.46839     ,    0.46532     ,    0.51995     ,    0.56950     ,    0.60667     ,    0.64585     ,    0.67025     ,    0.69601     ,    0.73255     ,    0.72955     ,    0.69677     ,    0.72783     ,    0.70879     ,    0.58020     ,    0.47065     ,    0.38201     ,    0.36692     ,    0.49303     ,    0.37503     ,    0.42145     ,    0.24918     ,    0.21297     ,    0.20652     ,    0.19580     ,    0.18599     ,    0.18682     ,    0.21130     ,    0.27593     ,    0.32113     ,    0.34219     ,    0.31316     ,    0.28984     ,    0.27765     ,    0.26163     ,    0.25364     ,    0.26118     ,    0.27266     ,    0.28455     ,    0.28920     ,    0.28144     ,    0.28400     ,    0.29152     ,    0.28952     ,    0.30189     ,    0.32747     ,    0.35797     ,    0.37587     ,    0.39807     ,    0.47680     ,    0.82797     ,    0.88573     ,    0.81081     ,    0.59378     ,    0.54812     ,    0.53929     ,    0.53308     ,    0.55765     ,    0.64780     ,    0.73837     ,    0.78083     ,    0.82499     ,    0.87141     ,    0.86989     ,    0.83443     ,    0.79544     ,    0.76145     ,    0.62732     ,    0.52632     ,    0.54145     ,    0.65874     ,    0.68519     ,    0.60907     ,    0.53584     ,    0.38570     ,    0.38782     ,    0.33403     ,    0.32130     ,    0.31266     ,    0.29425     ,    0.57161     ,    0.52743     ,    0.48412     ,    0.60745     ,    0.50155     ,    0.44779     ,    0.64633     ,    0.83991     ,    0.69867     ,    0.57586     ,    0.44047     ,    0.50716     ,    0.60228     ,    0.64598     ,    0.62182     ,    0.54157     ,    0.46501     ,    0.37550     ,    0.34843     ,    0.30815     ,    0.33139     ,    0.34077     ,    0.34469     ,    0.35118     ,    0.36282     ,    0.38473     ,    0.38041     ,    0.36651     ,    0.37475     ,    0.37949     ,    0.36454     ,    0.34128     ,    0.33405     ,    0.33316     ,    0.33357     ,    0.33852     ,    0.34247     ,    0.34844     ,    0.3665    },
    {    0.51018     ,    0.50778     ,    0.50136     ,    0.47707     ,    0.45219     ,    0.45018     ,    0.48187     ,    0.54465     ,    0.58795     ,    0.62210     ,    0.61454     ,    0.58626     ,    0.55001     ,    0.52740     ,    0.51955     ,    0.52398     ,    0.53538     ,    0.57914     ,    0.64970     ,    0.67863     ,    0.65497     ,    0.69083     ,    0.68674     ,    0.64446     ,    0.60855     ,    0.61676     ,    0.54702     ,    0.67523     ,    0.95311     ,    0.67179     ,    0.26961     ,    0.24501     ,    0.21709     ,    0.21264     ,    0.25626     ,    0.41129     ,    0.22991     ,    0.25952     ,    0.28254     ,    0.33994     ,    0.35485     ,    0.35128     ,    0.35653     ,    0.36541     ,    0.36849     ,    0.37979     ,    0.39077     ,    0.37602     ,    0.34636     ,    0.32578     ,    0.31128     ,    0.31346     ,    0.31551     ,    0.31586     ,    0.30002     ,    0.29816     ,    0.31282     ,    0.34179     ,    0.40380     ,    0.50281     ,    0.70890     ,    0.59223     ,    0.56745     ,    0.48543     ,    0.47993     ,    0.49652     ,    0.50182     ,    0.53724     ,    0.65803     ,    0.70174     ,    0.77872     ,    0.80960     ,    0.83041     ,    0.84200     ,    0.82719     ,    0.76504     ,    0.65006     ,    0.61767     ,    0.69727     ,    0.72357     ,    0.75330     ,    0.67023     ,    0.49567     ,    0.44247     ,    0.44332     ,    0.47820     ,    0.46103     ,    0.35569     ,    0.25536     ,    0.52021     ,    0.83309     ,    0.74357     ,    0.49808     ,    0.47316     ,    0.37105     ,    0.33898     ,    0.36599     ,    0.47306     ,    0.51201     ,    0.29691     ,    0.32261     ,    0.55404     ,    0.57202     ,    0.55851     ,    0.54495     ,    0.54993     ,    0.54234     ,    0.44467     ,    0.34241     ,    0.34613     ,    0.38386     ,    0.42625     ,    0.42681     ,    0.44193     ,    0.47820     ,    0.50489     ,    0.47980     ,    0.45670     ,    0.45736     ,    0.44702     ,    0.43820     ,    0.43598     ,    0.41150     ,    0.39695     ,    0.40879     ,    0.42228     ,    0.43335     ,    0.45997    },
    {    0.60738     ,    0.60869     ,    0.58659     ,    0.53879     ,    0.50843     ,    0.44362     ,    0.38557     ,    0.41541     ,    0.48284     ,    0.56396     ,    0.62593     ,    0.61149     ,    0.56757     ,    0.51487     ,    0.49305     ,    0.47253     ,    0.44556     ,    0.49111     ,    0.59350     ,    0.65432     ,    0.60444     ,    0.62482     ,    0.61100     ,    0.57678     ,    0.84944     ,    0.76928     ,    0.83257     ,    0.80127     ,    0.66297     ,    1.01747     ,    0.52924     ,    0.32478     ,    0.24817     ,    0.22500     ,    0.36257     ,    0.54031     ,    0.27048     ,    0.26599     ,    0.25539     ,    0.28995     ,    0.27840     ,    0.25298     ,    0.23569     ,    0.25177     ,    0.24641     ,    0.25481     ,    0.29231     ,    0.32975     ,    0.34916     ,    0.37702     ,    0.40114     ,    0.41173     ,    0.38890     ,    0.37650     ,    0.38488     ,    0.38493     ,    0.33728     ,    0.28457     ,    0.29400     ,    0.39967     ,    0.54779     ,    0.55858     ,    0.50853     ,    0.48119     ,    0.43620     ,    0.39632     ,    0.43541     ,    0.47446     ,    0.53400     ,    0.60457     ,    0.67877     ,    0.72682     ,    0.78525     ,    0.76845     ,    0.76187     ,    0.78599     ,    0.75149     ,    0.69237     ,    0.62783     ,    0.61533     ,    0.66601     ,    0.64323     ,    0.62675     ,    0.71862     ,    0.70947     ,    0.69565     ,    0.63607     ,    0.56159     ,    0.41220     ,    0.29830     ,    0.76271     ,    0.89381     ,    0.34750     ,    0.36780     ,    0.75583     ,    0.71800     ,    0.67492     ,    0.56809     ,    0.43190     ,    0.49415     ,    0.39394     ,    0.54484     ,    0.72925     ,    0.46312     ,    0.48204     ,    0.52766     ,    0.55300     ,    0.52904     ,    0.39861     ,    0.36782     ,    0.39985     ,    0.47746     ,    0.54414     ,    0.57732     ,    0.61846     ,    0.64833     ,    0.62401     ,    0.58016     ,    0.56032     ,    0.54321     ,    0.53713     ,    0.54519     ,    0.53027     ,    0.51463     ,    0.49586     ,    0.52141     ,    0.55237     ,    0.57291    },
    {    0.57648     ,    0.59045     ,    0.61551     ,    0.60575     ,    0.55814     ,    0.49889     ,    0.43939     ,    0.42991     ,    0.44609     ,    0.48458     ,    0.54836     ,    0.57796     ,    0.56325     ,    0.51591     ,    0.45884     ,    0.42796     ,    0.39567     ,    0.42941     ,    0.51702     ,    0.58193     ,    0.55377     ,    0.55775     ,    0.45702     ,    0.47595     ,    0.66958     ,    0.68765     ,    0.71235     ,    0.82384     ,    0.95993     ,    0.99543     ,    0.75520     ,    0.55656     ,    0.44758     ,    0.56960     ,    0.71185     ,    0.65966     ,    0.35858     ,    0.26853     ,    0.24031     ,    0.26221     ,    0.27166     ,    0.28382     ,    0.27656     ,    0.26772     ,    0.26873     ,    0.26781     ,    0.26565     ,    0.26592     ,    0.25688     ,    0.23493     ,    0.23104     ,    0.25540     ,    0.32384     ,    0.40850     ,    0.43408     ,    0.40373     ,    0.39324     ,    0.33182     ,    0.27376     ,    0.32843     ,    0.42289     ,    0.62294     ,    0.77155     ,    0.70843     ,    0.58077     ,    0.49212     ,    0.46488     ,    0.45735     ,    0.45334     ,    0.50163     ,    0.53654     ,    0.58284     ,    0.62954     ,    0.62221     ,    0.63268     ,    0.63220     ,    0.67797     ,    0.77146     ,    0.69450     ,    0.61659     ,    0.60168     ,    0.70850     ,    0.76308     ,    0.80211     ,    0.80724     ,    0.65683     ,    0.54500     ,    0.64115     ,    0.67824     ,    0.46403     ,    0.51464     ,    0.38198     ,    0.95557     ,    0.93492     ,    0.89281     ,    0.80657     ,    0.68466     ,    0.71520     ,    0.66457     ,    0.56202     ,    0.55330     ,    0.42066     ,    0.60205     ,    0.48023     ,    0.49812     ,    0.55226     ,    0.55971     ,    0.55608     ,    0.45646     ,    0.41893     ,    0.40302     ,    0.45729     ,    0.51998     ,    0.54335     ,    0.60625     ,    0.67068     ,    0.66588     ,    0.66605     ,    0.65368     ,    0.61593     ,    0.57687     ,    0.54827     ,    0.52363     ,    0.50749     ,    0.48399     ,    0.49130     ,    0.51079     ,    0.52903    },
    {    0.60121     ,    0.60578     ,    0.61615     ,    0.61969     ,    0.60744     ,    0.56804     ,    0.53288     ,    0.50825     ,    0.50034     ,    0.51813     ,    0.56636     ,    0.58661     ,    0.57145     ,    0.52837     ,    0.46813     ,    0.42379     ,    0.40835     ,    0.44086     ,    0.50681     ,    0.55727     ,    0.51379     ,    0.41384     ,    0.37361     ,    0.45942     ,    0.55157     ,    0.64249     ,    0.72419     ,    0.80913     ,    0.86104     ,    0.82427     ,    0.69902     ,    0.49918     ,    0.44740     ,    0.48823     ,    0.52628     ,    0.52106     ,    0.45994     ,    0.37229     ,    0.30047     ,    0.31031     ,    0.31690     ,    0.35232     ,    0.38000     ,    0.39918     ,    0.42298     ,    0.45536     ,    0.47892     ,    0.49054     ,    0.48271     ,    0.47179     ,    0.47465     ,    0.46708     ,    0.45637     ,    0.44574     ,    0.45287     ,    0.45584     ,    0.41300     ,    0.35778     ,    0.29083     ,    0.32259     ,    0.38565     ,    0.50733     ,    0.58680     ,    0.63218     ,    0.68328     ,    0.82051     ,    0.70357     ,    0.44826     ,    0.35192     ,    0.43755     ,    0.48680     ,    0.49380     ,    0.52173     ,    0.58233     ,    0.57967     ,    0.56048     ,    0.65335     ,    0.72941     ,    0.73417     ,    0.71078     ,    0.73256     ,    0.78432     ,    0.82299     ,    0.81195     ,    0.75940     ,    0.61409     ,    0.59615     ,    0.66114     ,    0.74172     ,    0.71008     ,    0.62206     ,    1.02824     ,    0.90432     ,    0.73976     ,    0.55751     ,    0.60166     ,    0.59371     ,    0.65773     ,    0.97419     ,    0.53504     ,    0.48476     ,    0.44360     ,    0.66300     ,    0.53900     ,    0.56789     ,    0.63585     ,    0.70591     ,    0.61971     ,    0.53260     ,    0.50608     ,    0.47417     ,    0.45102     ,    0.48445     ,    0.52142     ,    0.55990     ,    0.64717     ,    0.67834     ,    0.68213     ,    0.67503     ,    0.64915     ,    0.61729     ,    0.58830     ,    0.56184     ,    0.54145     ,    0.52736     ,    0.52707     ,    0.54060     ,    0.56009    },
    {    0.67817     ,    0.67438     ,    0.68075     ,    0.67760     ,    0.65258     ,    0.62466     ,    0.59612     ,    0.57235     ,    0.55209     ,    0.54548     ,    0.56026     ,    0.57209     ,    0.56219     ,    0.53150     ,    0.49331     ,    0.46580     ,    0.46703     ,    0.49116     ,    0.54887     ,    0.54929     ,    0.46339     ,    0.49842     ,    0.68412     ,    0.56223     ,    0.53087     ,    0.54243     ,    0.61267     ,    0.85180     ,    0.90591     ,    0.69794     ,    0.64724     ,    0.59475     ,    0.62208     ,    0.54978     ,    0.50875     ,    0.50167     ,    0.48464     ,    0.45358     ,    0.42963     ,    0.42653     ,    0.44167     ,    0.45051     ,    0.45701     ,    0.45729     ,    0.46529     ,    0.52608     ,    0.61659     ,    0.64386     ,    0.65036     ,    0.62450     ,    0.61682     ,    0.59860     ,    0.63922     ,    0.65323     ,    0.61945     ,    0.56585     ,    0.49477     ,    0.45337     ,    0.37422     ,    0.38864     ,    0.43353     ,    0.47132     ,    0.51785     ,    0.52119     ,    0.47861     ,    0.48410     ,    0.44805     ,    0.38531     ,    0.33821     ,    0.41109     ,    0.41931     ,    0.46322     ,    0.54250     ,    0.55490     ,    0.66302     ,    0.70897     ,    0.82342     ,    0.79063     ,    0.89514     ,    0.88876     ,    0.74811     ,    0.73110     ,    0.56772     ,    0.51383     ,    0.67126     ,    0.67396     ,    0.66186     ,    0.69065     ,    0.81724     ,    0.68163     ,    2.26139     ,    0.39943     ,    0.65893     ,    0.87247     ,    0.89780     ,    0.60833     ,    0.63883     ,    0.62055     ,    0.53851     ,    0.54424     ,    0.55911     ,    0.40754     ,    0.63401     ,    0.58080     ,    0.47730     ,    0.49667     ,    0.56332     ,    0.54932     ,    0.57522     ,    0.47987     ,    0.49800     ,    0.48393     ,    0.48164     ,    0.46337     ,    0.51118     ,    0.65172     ,    0.70696     ,    0.70357     ,    0.69324     ,    0.67343     ,    0.67033     ,    0.66914     ,    0.68442     ,    0.67925     ,    0.65366     ,    0.64403     ,    0.63926     ,    0.65355    },
    {    0.73468     ,    0.71970     ,    0.71311     ,    0.71660     ,    0.70129     ,    0.69162     ,    0.67531     ,    0.65339     ,    0.63464     ,    0.61870     ,    0.60343     ,    0.59765     ,    0.56913     ,    0.54380     ,    0.52406     ,    0.50130     ,    0.52053     ,    0.57671     ,    0.61872     ,    0.57570     ,    0.43417     ,    0.51817     ,    0.70122     ,    0.72441     ,    0.62926     ,    0.55777     ,    0.48410     ,    0.56239     ,    0.73309     ,    0.72540     ,    0.75819     ,    0.74193     ,    0.70800     ,    0.66048     ,    0.61113     ,    0.58728     ,    0.59529     ,    0.58386     ,    0.56289     ,    0.61145     ,    0.63071     ,    0.66284     ,    0.68207     ,    0.65333     ,    0.67990     ,    0.73972     ,    0.87126     ,    0.93325     ,    0.87618     ,    0.76168     ,    0.72951     ,    0.67785     ,    0.73484     ,    0.87025     ,    0.85102     ,    0.78656     ,    0.70440     ,    0.64388     ,    0.54264     ,    0.48004     ,    0.48916     ,    0.52221     ,    0.60271     ,    0.64805     ,    0.56426     ,    0.42729     ,    0.36300     ,    0.37758     ,    0.34734     ,    0.39325     ,    0.50407     ,    0.55418     ,    0.62612     ,    0.55942     ,    0.68114     ,    0.66402     ,    0.68274     ,    0.62058     ,    0.57766     ,    0.55499     ,    0.52748     ,    0.72452     ,    0.58195     ,    0.56641     ,    0.61953     ,    0.70690     ,    0.82718     ,    0.77307     ,    0.78719     ,    0.90017     ,    0.83620     ,    0.86191     ,    0.98419     ,    1.12920     ,    0.95043     ,    0.89918     ,    0.87295     ,    0.80836     ,    0.74952     ,    0.76015     ,    0.79890     ,    0.78248     ,    0.77200     ,    0.71840     ,    0.54937     ,    0.48317     ,    0.51922     ,    0.51594     ,    0.56249     ,    0.52576     ,    0.54778     ,    0.59032     ,    0.56354     ,    0.50430     ,    0.54320     ,    0.88812     ,    1.06971     ,    1.04865     ,    0.97849     ,    0.93161     ,    0.94289     ,    0.96155     ,    0.95474     ,    0.91011     ,    0.86865     ,    0.82533     ,    0.77209     ,    0.75677    },
    {    0.68927     ,    0.69087     ,    0.70114     ,    0.71249     ,    0.71526     ,    0.70873     ,    0.69076     ,    0.66551     ,    0.64127     ,    0.61491     ,    0.58816     ,    0.56039     ,    0.54748     ,    0.54709     ,    0.54596     ,    0.53381     ,    0.56851     ,    0.64943     ,    0.68643     ,    0.69197     ,    0.44415     ,    0.54043     ,    0.62227     ,    0.66552     ,    0.60798     ,    0.61265     ,    0.63662     ,    0.64373     ,    0.70061     ,    0.77800     ,    0.76012     ,    0.71655     ,    0.71780     ,    0.66302     ,    0.59704     ,    0.59788     ,    0.63037     ,    0.65084     ,    0.65039     ,    0.64043     ,    0.67146     ,    0.76488     ,    0.87668     ,    0.95139     ,    1.00879     ,    0.96347     ,    0.90917     ,    1.35663     ,    1.49281     ,    1.11735     ,    0.97935     ,    0.92841     ,    0.91132     ,    0.95178     ,    0.89813     ,    0.83085     ,    0.82480     ,    0.86935     ,    0.78996     ,    0.58757     ,    0.47129     ,    0.48314     ,    0.54541     ,    0.51942     ,    0.59763     ,    0.57944     ,    0.52470     ,    0.44412     ,    0.47381     ,    0.46676     ,    0.46899     ,    0.54567     ,    0.60574     ,    0.64677     ,    0.57746     ,    0.55883     ,    0.58196     ,    0.51646     ,    0.43753     ,    0.41935     ,    0.42527     ,    0.44750     ,    0.54852     ,    0.62030     ,    0.71845     ,    0.81087     ,    0.93665     ,    0.98634     ,    0.86087     ,    0.85221     ,    0.97900     ,    1.23276     ,    1.25310     ,    1.19055     ,    1.10518     ,    1.10187     ,    1.05502     ,    1.00934     ,    0.95586     ,    0.96084     ,    0.92255     ,    0.85377     ,    0.81378     ,    0.83206     ,    0.74648     ,    0.67754     ,    0.64284     ,    0.66673     ,    0.61570     ,    0.61045     ,    0.57096     ,    0.58008     ,    0.58361     ,    0.54828     ,    0.60833     ,    0.70025     ,    0.89907     ,    0.96539     ,    0.94236     ,    0.94768     ,    1.02163     ,    1.11026     ,    1.07505     ,    0.95419     ,    0.89842     ,    0.84792     ,    0.75580     ,    0.70099    },
    {    0.68942     ,    0.68925     ,    0.67919     ,    0.66701     ,    0.66342     ,    0.65321     ,    0.63883     ,    0.63281     ,    0.64118     ,    0.65148     ,    0.65538     ,    0.64265     ,    0.63614     ,    0.64835     ,    0.64790     ,    0.65014     ,    0.69149     ,    0.75588     ,    0.78258     ,    0.77460     ,    0.60454     ,    0.64083     ,    0.77625     ,    0.80841     ,    0.65023     ,    0.62815     ,    0.67601     ,    0.64208     ,    0.72064     ,    0.80141     ,    0.78461     ,    0.81245     ,    0.71289     ,    0.57334     ,    0.55447     ,    0.60752     ,    0.65129     ,    0.68963     ,    0.71291     ,    0.65272     ,    0.58769     ,    0.62827     ,    0.74108     ,    0.81509     ,    0.82235     ,    0.77204     ,    0.70286     ,    0.96748     ,    1.33717     ,    1.32720     ,    1.34743     ,    1.40354     ,    1.19751     ,    1.00962     ,    0.95408     ,    0.91249     ,    0.97199     ,    1.03988     ,    0.97658     ,    0.67816     ,    0.50408     ,    0.45894     ,    0.42880     ,    0.43828     ,    0.56617     ,    0.64467     ,    0.68731     ,    0.49628     ,    0.45852     ,    0.48754     ,    0.56405     ,    0.49784     ,    0.60158     ,    0.71696     ,    0.59614     ,    0.55451     ,    0.54692     ,    0.53902     ,    0.51475     ,    0.52695     ,    0.55854     ,    0.60797     ,    0.62211     ,    0.68536     ,    0.73402     ,    0.82988     ,    0.93812     ,    1.02036     ,    1.05371     ,    1.10935     ,    1.17729     ,    1.24733     ,    1.34194     ,    1.19897     ,    1.19731     ,    1.18819     ,    1.12085     ,    1.06364     ,    1.06712     ,    0.97550     ,    0.94070     ,    0.88443     ,    0.87657     ,    0.91356     ,    0.92947     ,    0.79612     ,    0.75472     ,    0.67503     ,    0.66617     ,    0.57104     ,    0.57667     ,    0.58868     ,    0.59801     ,    0.56261     ,    0.60813     ,    0.57546     ,    0.62988     ,    0.66505     ,    0.70814     ,    0.71548     ,    0.77737     ,    0.90820     ,    0.97692     ,    0.95556     ,    0.93308     ,    0.88893     ,    0.81038     ,    0.73299    },
    {    1.00589     ,    0.99510     ,    0.95741     ,    0.95503     ,    0.97401     ,    0.96743     ,    0.94154     ,    0.94210     ,    0.97035     ,    0.98075     ,    0.96928     ,    0.92460     ,    0.87780     ,    0.86678     ,    0.86578     ,    0.88953     ,    0.92601     ,    0.91785     ,    0.89664     ,    0.78766     ,    0.77498     ,    0.81361     ,    0.78223     ,    0.66679     ,    0.70157     ,    0.67428     ,    0.70207     ,    0.75075     ,    0.80402     ,    0.84813     ,    0.84352     ,    0.71536     ,    0.66704     ,    0.65219     ,    0.63697     ,    0.65013     ,    0.82350     ,    0.82345     ,    0.72756     ,    0.70490     ,    0.72357     ,    0.70567     ,    0.75562     ,    0.69961     ,    0.56064     ,    0.70123     ,    0.75708     ,    0.78777     ,    0.92192     ,    1.36577     ,    2.00437     ,    2.30054     ,    2.06180     ,    1.38696     ,    1.07109     ,    0.98920     ,    0.99993     ,    0.91568     ,    0.73781     ,    0.55564     ,    0.50823     ,    0.45635     ,    0.45270     ,    0.53041     ,    0.57854     ,    0.57777     ,    0.58485     ,    0.55770     ,    0.53452     ,    0.55084     ,    0.64597     ,    0.63569     ,    0.63512     ,    0.65965     ,    0.60990     ,    0.57064     ,    0.58085     ,    0.60892     ,    0.60636     ,    0.61817     ,    0.65976     ,    0.70975     ,    0.75711     ,    0.75984     ,    0.85594     ,    0.94154     ,    1.02149     ,    1.06027     ,    1.13644     ,    1.21709     ,    1.34105     ,    1.45888     ,    1.57469     ,    1.54682     ,    1.36302     ,    1.17974     ,    1.03140     ,    1.00842     ,    1.11226     ,    0.97787     ,    0.95245     ,    0.93836     ,    0.94401     ,    0.88495     ,    0.84284     ,    0.76481     ,    0.72573     ,    0.64554     ,    0.70518     ,    0.63045     ,    0.55913     ,    0.58624     ,    0.52961     ,    0.57030     ,    0.61947     ,    0.69091     ,    0.86111     ,    0.92880     ,    0.88048     ,    0.84783     ,    0.95573     ,    1.10992     ,    1.19004     ,    1.22480     ,    1.22720     ,    1.18848     ,    1.11247     ,    1.03463    },
    {    1.48157     ,    1.44977     ,    1.41628     ,    1.42436     ,    1.45104     ,    1.48410     ,    1.49213     ,    1.50206     ,    1.49910     ,    1.45494     ,    1.38324     ,    1.29852     ,    1.23015     ,    1.17852     ,    1.16275     ,    1.15931     ,    1.11479     ,    1.02772     ,    0.97637     ,    0.92046     ,    0.83474     ,    0.80087     ,    0.77869     ,    0.73883     ,    0.71315     ,    0.73036     ,    0.75647     ,    0.77745     ,    0.74801     ,    0.80235     ,    0.72595     ,    0.65694     ,    0.57038     ,    0.58911     ,    0.61103     ,    0.75882     ,    0.76860     ,    0.69050     ,    0.64850     ,    0.64407     ,    0.63700     ,    0.63999     ,    0.64042     ,    0.66583     ,    0.73390     ,    0.89544     ,    0.99626     ,    1.09988     ,    1.56294     ,    2.21035     ,    2.37470     ,    2.22752     ,    2.09708     ,    1.56310     ,    1.10717     ,    0.92120     ,    0.89032     ,    0.90068     ,    0.80374     ,    0.64464     ,    0.54275     ,    0.51480     ,    0.53472     ,    0.57213     ,    0.58771     ,    0.59550     ,    0.57805     ,    0.60183     ,    0.66329     ,    0.71941     ,    0.68713     ,    0.68318     ,    0.69333     ,    0.65027     ,    0.63686     ,    0.68454     ,    0.72911     ,    0.83244     ,    0.87081     ,    0.85863     ,    0.89719     ,    0.87839     ,    0.85724     ,    0.85209     ,    0.89735     ,    0.97535     ,    1.02371     ,    1.07484     ,    1.16998     ,    1.23749     ,    1.33356     ,    1.40960     ,    1.51861     ,    1.56546     ,    1.48898     ,    1.25803     ,    1.19976     ,    1.13626     ,    1.05632     ,    1.00605     ,    1.01754     ,    1.02886     ,    1.05156     ,    0.94519     ,    0.85142     ,    0.79485     ,    0.70892     ,    0.68353     ,    0.62285     ,    0.59025     ,    0.64081     ,    0.65655     ,    0.64940     ,    0.69212     ,    0.78273     ,    0.89375     ,    1.09617     ,    1.23250     ,    1.12765     ,    1.04054     ,    1.07169     ,    1.33288     ,    1.53601     ,    1.62796     ,    1.66367     ,    1.60590     ,    1.55617     ,    1.54897    },
    {    2.20284     ,    2.13101     ,    2.01535     ,    1.95713     ,    1.93918     ,    1.94863     ,    1.89034     ,    1.85944     ,    1.82414     ,    1.75464     ,    1.67993     ,    1.58725     ,    1.51504     ,    1.45809     ,    1.40400     ,    1.31859     ,    1.17477     ,    1.09926     ,    1.03377     ,    0.93189     ,    0.79293     ,    0.70572     ,    0.65877     ,    0.63965     ,    0.68461     ,    0.68737     ,    0.69506     ,    0.68634     ,    0.62593     ,    0.62832     ,    0.57577     ,    0.56493     ,    0.56733     ,    0.62688     ,    0.72222     ,    0.80763     ,    0.86798     ,    0.76986     ,    0.75588     ,    0.77628     ,    0.78217     ,    0.76363     ,    0.76203     ,    0.88094     ,    1.10114     ,    1.22197     ,    1.44156     ,    1.64949     ,    2.00974     ,    2.43586     ,    2.52248     ,    2.29791     ,    1.99784     ,    1.52492     ,    1.17470     ,    0.99722     ,    0.92624     ,    0.93882     ,    0.88504     ,    0.77848     ,    0.66551     ,    0.57871     ,    0.58135     ,    0.60660     ,    0.58660     ,    0.60578     ,    0.62048     ,    0.59988     ,    0.60426     ,    0.62733     ,    0.65229     ,    0.70434     ,    0.75570     ,    0.75060     ,    0.77149     ,    0.86015     ,    0.89795     ,    0.94711     ,    1.00300     ,    1.00639     ,    1.08784     ,    1.12805     ,    1.13612     ,    1.09086     ,    0.99012     ,    0.96603     ,    0.99920     ,    1.04803     ,    1.12827     ,    1.19620     ,    1.22578     ,    1.28582     ,    1.35984     ,    1.43005     ,    1.54156     ,    1.56490     ,    1.57047     ,    1.49708     ,    1.32340     ,    1.21112     ,    1.21550     ,    1.20425     ,    1.13139     ,    1.05854     ,    0.95232     ,    0.83797     ,    0.75302     ,    0.69860     ,    0.66291     ,    0.65044     ,    0.65222     ,    0.65558     ,    0.69067     ,    0.88298     ,    1.08373     ,    1.21370     ,    1.39526     ,    1.41801     ,    1.26347     ,    1.15635     ,    1.06088     ,    1.21701     ,    1.63635     ,    2.00663     ,    2.15655     ,    2.21855     ,    2.27839     ,    2.31135    },
    {    2.82965     ,    2.73620     ,    2.56132     ,    2.42000     ,    2.35734     ,    2.31535     ,    2.25038     ,    2.18527     ,    2.08155     ,    1.95415     ,    1.92350     ,    1.81676     ,    1.70716     ,    1.61953     ,    1.51401     ,    1.36982     ,    1.21062     ,    1.14636     ,    1.05443     ,    0.96987     ,    0.87633     ,    0.77730     ,    0.76810     ,    0.79114     ,    0.83390     ,    0.74770     ,    0.70068     ,    0.63856     ,    0.60880     ,    0.59409     ,    0.65393     ,    0.68149     ,    0.70087     ,    0.75109     ,    0.84331     ,    0.95073     ,    1.04082     ,    0.99077     ,    0.93804     ,    0.87524     ,    0.83204     ,    0.84825     ,    0.94658     ,    1.28102     ,    1.63158     ,    1.83176     ,    1.98247     ,    2.18533     ,    2.39175     ,    2.53568     ,    2.65512     ,    2.63590     ,    2.36979     ,    1.95061     ,    1.53161     ,    1.23343     ,    1.04636     ,    0.97818     ,    0.91845     ,    0.85977     ,    0.78841     ,    0.69839     ,    0.64503     ,    0.63838     ,    0.61242     ,    0.60937     ,    0.60737     ,    0.61781     ,    0.63164     ,    0.65620     ,    0.73573     ,    0.81533     ,    0.84392     ,    0.88900     ,    0.96840     ,    1.04478     ,    1.13242     ,    1.18340     ,    1.21944     ,    1.22826     ,    1.30383     ,    1.34775     ,    1.32216     ,    1.24316     ,    1.08379     ,    1.05908     ,    1.05440     ,    0.99376     ,    1.03603     ,    1.06503     ,    1.09170     ,    1.14190     ,    1.19919     ,    1.23037     ,    1.39614     ,    1.52910     ,    1.42525     ,    1.40548     ,    1.38096     ,    1.33309     ,    1.29432     ,    1.27545     ,    1.18695     ,    1.03726     ,    0.97432     ,    0.89360     ,    0.81565     ,    0.77918     ,    0.74407     ,    0.66029     ,    0.65526     ,    0.65785     ,    0.73669     ,    0.91270     ,    1.13864     ,    1.32365     ,    1.39825     ,    1.39288     ,    1.33152     ,    1.24501     ,    1.10140     ,    1.08434     ,    1.40842     ,    2.06740     ,    2.42456     ,    2.66088     ,    2.85600     ,    2.93787    },
    {    2.57364     ,    2.51510     ,    2.33701     ,    2.14777     ,    2.10512     ,    2.08401     ,    2.02566     ,    2.00144     ,    2.00512     ,    1.98624     ,    2.00401     ,    1.99807     ,    1.85492     ,    1.66137     ,    1.49983     ,    1.36593     ,    1.33964     ,    1.26422     ,    1.16941     ,    1.11307     ,    1.04130     ,    1.04063     ,    1.00698     ,    0.91368     ,    0.86392     ,    0.83877     ,    0.80696     ,    0.78993     ,    0.76490     ,    0.77491     ,    0.77228     ,    0.79302     ,    0.79350     ,    0.82258     ,    0.92462     ,    1.04456     ,    1.14682     ,    1.12689     ,    1.06247     ,    1.02387     ,    0.99629     ,    1.06764     ,    1.30076     ,    1.79876     ,    2.12342     ,    2.25059     ,    2.31295     ,    2.29132     ,    2.26815     ,    2.57122     ,    2.82503     ,    3.08055     ,    2.93574     ,    2.61429     ,    2.28324     ,    1.76095     ,    1.31176     ,    1.12488     ,    1.05372     ,    1.01315     ,    0.96112     ,    0.86868     ,    0.78554     ,    0.73814     ,    0.69210     ,    0.63138     ,    0.61954     ,    0.64473     ,    0.67548     ,    0.77720     ,    0.84346     ,    0.97177     ,    1.05137     ,    1.15131     ,    1.20068     ,    1.26523     ,    1.34963     ,    1.35918     ,    1.39788     ,    1.42941     ,    1.43460     ,    1.43429     ,    1.37978     ,    1.29569     ,    1.20806     ,    1.12983     ,    1.06282     ,    1.00680     ,    1.02233     ,    1.02627     ,    1.06738     ,    1.09819     ,    1.14049     ,    1.14779     ,    1.17592     ,    1.18705     ,    1.18296     ,    1.18443     ,    1.21102     ,    1.16902     ,    1.18546     ,    1.18454     ,    1.10686     ,    1.01412     ,    0.88685     ,    0.84139     ,    0.76473     ,    0.72996     ,    0.68216     ,    0.63167     ,    0.66811     ,    0.75546     ,    0.85223     ,    1.01745     ,    0.99827     ,    1.06259     ,    1.12470     ,    1.15999     ,    1.19416     ,    1.20774     ,    1.11418     ,    1.02934     ,    1.19705     ,    1.51116     ,    1.74611     ,    2.03956     ,    2.38583     ,    2.56392    },
    {    1.99251     ,    2.02695     ,    1.99596     ,    1.92412     ,    1.94631     ,    2.03078     ,    1.96666     ,    1.98094     ,    1.99151     ,    1.99871     ,    2.00933     ,    1.94472     ,    1.81320     ,    1.74749     ,    1.63267     ,    1.57328     ,    1.44775     ,    1.39540     ,    1.31906     ,    1.26556     ,    1.16149     ,    1.08949     ,    1.06791     ,    1.05382     ,    1.00510     ,    0.97082     ,    0.92770     ,    0.90348     ,    0.88712     ,    0.91516     ,    0.94424     ,    0.94199     ,    0.92535     ,    0.94647     ,    1.05346     ,    1.11601     ,    1.22301     ,    1.31345     ,    1.22871     ,    1.19799     ,    1.19119     ,    1.24427     ,    1.48279     ,    1.97226     ,    2.27670     ,    2.22530     ,    1.79471     ,    1.49606     ,    1.58739     ,    2.09273     ,    2.52941     ,    2.80644     ,    3.11653     ,    3.11469     ,    2.78390     ,    2.17889     ,    1.62793     ,    1.39248     ,    1.35061     ,    1.51604     ,    1.64760     ,    1.51783     ,    1.30071     ,    1.07790     ,    0.88733     ,    0.74183     ,    0.70586     ,    0.77034     ,    0.88281     ,    0.96075     ,    1.08475     ,    1.25826     ,    1.30640     ,    1.36463     ,    1.46322     ,    1.57545     ,    1.69060     ,    1.79368     ,    1.76841     ,    1.74445     ,    1.70839     ,    1.64469     ,    1.55246     ,    1.43641     ,    1.32100     ,    1.24041     ,    1.19601     ,    1.14854     ,    1.14489     ,    1.16787     ,    1.15025     ,    1.16023     ,    1.14890     ,    1.14887     ,    1.14918     ,    1.11454     ,    1.09932     ,    1.11763     ,    1.10143     ,    1.09625     ,    1.10435     ,    1.08838     ,    1.04314     ,    1.00360     ,    0.92479     ,    0.86208     ,    0.83293     ,    0.82442     ,    0.83432     ,    0.84386     ,    0.85803     ,    0.93851     ,    1.03656     ,    1.12024     ,    1.05629     ,    1.12181     ,    1.19538     ,    1.16016     ,    1.15449     ,    1.20170     ,    1.18229     ,    1.20785     ,    1.25003     ,    1.26204     ,    1.34371     ,    1.45915     ,    1.65996     ,    1.8004    },
    {    1.73067     ,    1.76906     ,    1.82659     ,    1.78129     ,    1.79078     ,    1.85879     ,    1.88323     ,    1.83324     ,    1.81981     ,    1.81459     ,    1.79601     ,    1.77123     ,    1.70554     ,    1.64573     ,    1.60018     ,    1.54280     ,    1.47055     ,    1.41679     ,    1.34724     ,    1.25844     ,    1.20607     ,    1.34896     ,    1.21804     ,    1.14769     ,    1.09784     ,    1.01633     ,    0.94955     ,    0.92929     ,    0.97671     ,    1.02981     ,    1.06831     ,    1.08307     ,    1.12434     ,    1.20555     ,    1.23776     ,    1.16458     ,    1.26500     ,    1.38727     ,    1.42857     ,    1.39264     ,    1.32903     ,    1.35411     ,    1.35361     ,    1.53666     ,    1.89310     ,    1.79057     ,    1.47135     ,    1.28306     ,    1.43732     ,    1.58939     ,    2.01778     ,    2.06791     ,    2.29864     ,    2.57455     ,    2.56155     ,    2.32570     ,    2.01910     ,    1.84959     ,    1.76673     ,    1.97998     ,    2.35409     ,    2.48260     ,    2.42540     ,    2.05293     ,    1.49345     ,    1.04675     ,    0.85013     ,    0.83578     ,    0.90992     ,    1.06539     ,    1.15605     ,    1.27247     ,    1.49005     ,    1.56699     ,    1.64626     ,    1.77368     ,    1.95735     ,    2.23301     ,    2.22627     ,    2.15894     ,    2.06135     ,    1.97581     ,    1.87346     ,    1.74806     ,    1.64359     ,    1.52043     ,    1.40939     ,    1.32351     ,    1.31336     ,    1.25825     ,    1.16738     ,    1.13939     ,    1.13062     ,    1.11892     ,    1.07542     ,    1.02485     ,    1.03205     ,    1.02475     ,    1.04303     ,    1.08501     ,    1.05514     ,    1.05602     ,    1.02588     ,    0.99448     ,    1.02084     ,    0.98317     ,    0.93697     ,    0.86395     ,    0.88467     ,    1.01241     ,    0.96476     ,    1.00212     ,    1.06598     ,    1.13717     ,    0.99447     ,    1.04649     ,    1.02441     ,    1.05805     ,    1.06405     ,    1.15856     ,    1.16275     ,    1.20860     ,    1.20867     ,    1.23841     ,    1.32147     ,    1.42603     ,    1.49770     ,    1.56471    },
    {    1.57275     ,    1.67276     ,    1.71089     ,    1.66863     ,    1.68212     ,    1.65603     ,    1.60222     ,    1.56109     ,    1.51767     ,    1.49242     ,    1.48890     ,    1.46373     ,    1.43835     ,    1.42829     ,    1.45468     ,    1.48233     ,    1.47887     ,    1.46309     ,    1.40167     ,    1.34156     ,    1.28447     ,    1.24516     ,    1.18453     ,    1.11869     ,    1.04486     ,    1.00809     ,    1.01152     ,    1.04059     ,    1.10149     ,    1.18198     ,    1.18604     ,    1.19883     ,    1.32004     ,    1.46164     ,    1.43986     ,    1.41060     ,    1.46347     ,    1.53257     ,    1.55594     ,    1.55980     ,    1.50198     ,    1.45693     ,    1.42044     ,    1.40540     ,    1.47750     ,    1.47026     ,    1.38918     ,    1.23012     ,    1.24129     ,    1.13050     ,    1.03202     ,    1.01839     ,    1.10273     ,    1.22253     ,    1.31092     ,    1.54120     ,    1.88986     ,    2.19598     ,    2.36815     ,    2.64867     ,    2.90994     ,    2.84116     ,    2.72544     ,    2.48492     ,    2.02937     ,    1.55571     ,    1.19729     ,    1.00628     ,    0.98666     ,    1.06092     ,    1.23476     ,    1.43529     ,    1.57862     ,    1.73311     ,    1.92092     ,    2.09721     ,    2.33088     ,    2.49178     ,    2.59733     ,    2.60339     ,    2.48766     ,    2.36292     ,    2.23429     ,    2.06989     ,    1.88497     ,    1.75455     ,    1.66076     ,    1.57520     ,    1.48110     ,    1.36838     ,    1.25445     ,    1.23985     ,    1.19496     ,    1.06636     ,    1.01516     ,    0.98711     ,    0.99268     ,    1.01137     ,    1.06739     ,    1.09970     ,    1.10549     ,    1.05298     ,    1.05851     ,    1.07437     ,    1.12716     ,    1.03592     ,    0.93627     ,    0.86109     ,    0.82778     ,    0.96914     ,    0.98102     ,    0.93389     ,    0.95459     ,    0.99544     ,    0.93205     ,    0.91021     ,    0.92113     ,    1.01305     ,    1.05462     ,    1.18214     ,    1.25589     ,    1.29251     ,    1.34458     ,    1.35238     ,    1.39842     ,    1.40553     ,    1.42949     ,    1.4943    },
    {    1.57419     ,    1.61552     ,    1.65247     ,    1.68721     ,    1.69796     ,    1.63580     ,    1.58436     ,    1.54091     ,    1.50589     ,    1.47828     ,    1.45488     ,    1.44897     ,    1.46860     ,    1.48918     ,    1.52877     ,    1.55985     ,    1.59492     ,    1.59969     ,    1.56439     ,    1.50032     ,    1.32569     ,    1.20930     ,    1.10406     ,    1.02826     ,    0.99360     ,    0.97851     ,    1.00975     ,    1.06660     ,    1.24280     ,    1.28804     ,    1.34797     ,    1.49417     ,    1.70446     ,    1.67796     ,    1.62596     ,    1.59162     ,    1.62611     ,    1.69462     ,    1.69774     ,    1.65948     ,    1.61092     ,    1.52447     ,    1.48846     ,    1.40995     ,    1.29471     ,    1.30506     ,    1.18252     ,    1.16828     ,    1.18736     ,    1.16802     ,    1.11146     ,    1.07769     ,    0.97076     ,    0.95233     ,    0.98328     ,    1.06914     ,    1.31071     ,    1.85632     ,    2.30487     ,    2.62509     ,    2.88524     ,    2.69285     ,    2.48721     ,    2.40998     ,    2.28694     ,    1.88014     ,    1.48224     ,    1.20553     ,    1.16142     ,    1.27991     ,    1.54414     ,    1.94886     ,    2.54719     ,    2.88590     ,    2.67499     ,    2.75041     ,    3.12896     ,    3.28242     ,    3.23994     ,    3.05958     ,    2.79278     ,    2.52789     ,    2.31698     ,    2.21281     ,    2.17715     ,    2.09744     ,    1.96821     ,    1.78042     ,    1.61105     ,    1.46756     ,    1.29761     ,    1.30584     ,    1.26441     ,    1.19574     ,    1.08917     ,    1.05050     ,    1.02991     ,    1.04790     ,    1.05027     ,    1.02617     ,    1.00300     ,    0.96002     ,    0.96699     ,    1.04676     ,    0.99901     ,    0.89610     ,    0.83934     ,    0.76288     ,    0.72723     ,    0.80218     ,    0.86449     ,    0.91890     ,    0.95006     ,    0.96168     ,    0.92594     ,    0.97192     ,    1.03185     ,    1.11273     ,    1.15619     ,    1.22629     ,    1.28823     ,    1.35826     ,    1.42380     ,    1.46677     ,    1.49160     ,    1.49467     ,    1.49686     ,    1.54915    },
    {    1.50130     ,    1.50026     ,    1.49775     ,    1.49189     ,    1.49733     ,    1.48223     ,    1.46753     ,    1.45056     ,    1.41796     ,    1.36819     ,    1.32639     ,    1.29112     ,    1.25909     ,    1.22905     ,    1.20446     ,    1.19453     ,    1.18634     ,    1.18896     ,    1.18740     ,    1.15263     ,    1.02928     ,    0.97093     ,    0.94850     ,    0.95298     ,    1.02038     ,    1.06099     ,    1.13062     ,    1.18110     ,    1.29868     ,    1.44628     ,    1.56082     ,    1.67626     ,    1.73301     ,    1.66809     ,    1.58715     ,    1.58743     ,    1.54172     ,    1.48302     ,    1.51319     ,    1.53614     ,    1.47476     ,    1.42255     ,    1.37319     ,    1.28807     ,    1.17336     ,    1.16512     ,    1.13698     ,    1.16952     ,    1.17015     ,    1.21479     ,    1.07603     ,    1.04365     ,    0.97041     ,    0.93502     ,    0.95548     ,    0.95686     ,    1.03838     ,    1.25645     ,    1.64136     ,    2.11079     ,    2.53026     ,    2.64807     ,    2.48938     ,    2.32730     ,    2.21888     ,    2.04478     ,    1.78048     ,    1.59994     ,    1.73952     ,    2.10709     ,    2.53088     ,    2.89457     ,    3.10603     ,    3.18542     ,    3.23362     ,    3.40199     ,    3.59625     ,    3.62023     ,    3.43793     ,    3.17200     ,    2.90488     ,    2.66098     ,    2.40768     ,    2.25491     ,    2.26204     ,    2.15454     ,    2.03447     ,    1.85431     ,    1.68655     ,    1.57383     ,    1.50554     ,    1.46233     ,    1.42577     ,    1.37908     ,    1.33099     ,    1.27921     ,    1.22506     ,    1.18652     ,    1.17269     ,    1.17764     ,    1.08367     ,    1.02629     ,    1.01175     ,    1.06753     ,    1.02195     ,    1.03258     ,    1.02893     ,    1.02784     ,    0.98370     ,    0.98008     ,    0.99577     ,    1.04335     ,    1.07093     ,    1.06966     ,    1.05506     ,    1.12183     ,    1.21684     ,    1.23071     ,    1.22001     ,    1.22888     ,    1.26650     ,    1.30095     ,    1.32820     ,    1.36311     ,    1.39011     ,    1.42538     ,    1.45323     ,    1.47661    },
    {    1.25671     ,    1.25767     ,    1.25475     ,    1.24298     ,    1.21833     ,    1.19032     ,    1.16794     ,    1.14912     ,    1.12356     ,    1.09732     ,    1.07433     ,    1.05747     ,    1.03665     ,    1.02025     ,    1.00737     ,    1.00232     ,    0.99758     ,    0.98055     ,    0.94880     ,    0.89991     ,    0.84941     ,    0.86284     ,    0.91345     ,    0.99056     ,    1.08914     ,    1.12694     ,    1.20845     ,    1.25480     ,    1.36214     ,    1.48310     ,    1.44996     ,    1.44654     ,    1.39243     ,    1.34058     ,    1.23799     ,    1.22125     ,    1.31997     ,    1.31487     ,    1.26926     ,    1.23034     ,    1.16347     ,    1.09766     ,    1.05276     ,    1.03093     ,    1.12010     ,    1.07257     ,    1.16403     ,    1.21595     ,    1.28454     ,    1.24260     ,    1.14008     ,    1.07057     ,    0.98660     ,    0.93877     ,    0.90290     ,    0.86651     ,    0.87402     ,    1.05704     ,    1.24330     ,    1.47694     ,    1.77507     ,    2.09546     ,    2.38717     ,    2.56546     ,    2.64788     ,    2.68606     ,    2.63048     ,    2.58434     ,    2.50367     ,    2.46291     ,    2.70816     ,    2.90341     ,    2.94570     ,    3.09802     ,    3.23646     ,    3.28743     ,    3.20009     ,    3.07903     ,    2.95356     ,    2.83611     ,    2.69998     ,    2.55616     ,    2.40325     ,    2.23642     ,    2.11065     ,    2.07753     ,    2.03967     ,    1.98475     ,    1.89918     ,    1.80019     ,    1.68639     ,    1.60872     ,    1.55784     ,    1.50171     ,    1.46177     ,    1.42817     ,    1.36659     ,    1.31972     ,    1.29997     ,    1.25779     ,    1.22460     ,    1.24502     ,    1.24571     ,    1.24862     ,    1.23248     ,    1.18948     ,    1.16749     ,    1.14099     ,    1.12225     ,    1.09611     ,    1.08211     ,    1.06163     ,    1.04484     ,    1.04637     ,    1.04874     ,    1.07319     ,    1.08537     ,    1.09500     ,    1.10168     ,    1.11345     ,    1.13643     ,    1.17955     ,    1.22972     ,    1.27097     ,    1.29315     ,    1.29280     ,    1.28042     ,    1.26824    },
    {    1.14297     ,    1.13226     ,    1.11485     ,    1.09075     ,    1.06242     ,    1.04039     ,    1.02063     ,    1.00866     ,    1.00188     ,    0.99706     ,    0.99142     ,    0.98582     ,    0.97506     ,    0.96541     ,    0.96200     ,    0.96078     ,    0.95825     ,    0.95787     ,    0.96289     ,    0.96714     ,    0.97328     ,    1.00820     ,    1.05154     ,    1.08142     ,    1.10500     ,    1.11559     ,    1.13171     ,    1.16674     ,    1.18063     ,    1.25731     ,    1.27587     ,    1.14749     ,    1.12537     ,    1.11009     ,    1.10288     ,    1.11211     ,    1.14537     ,    1.18905     ,    1.12789     ,    1.09898     ,    1.00471     ,    0.98537     ,    0.96499     ,    0.94434     ,    0.96621     ,    1.02648     ,    0.98662     ,    0.99737     ,    1.00108     ,    1.01273     ,    0.98184     ,    0.91652     ,    0.87862     ,    0.85483     ,    0.79806     ,    0.74462     ,    0.76312     ,    0.86069     ,    0.98542     ,    1.08746     ,    1.25480     ,    1.43885     ,    1.57083     ,    1.67333     ,    1.83655     ,    2.02968     ,    2.12743     ,    2.06148     ,    1.89960     ,    1.97140     ,    2.06183     ,    2.16587     ,    2.27100     ,    2.33015     ,    2.38800     ,    2.39958     ,    2.39343     ,    2.38417     ,    2.36217     ,    2.32005     ,    2.26390     ,    2.18898     ,    2.12080     ,    2.03571     ,    1.98755     ,    1.97582     ,    1.91549     ,    1.89140     ,    1.86765     ,    1.83542     ,    1.79295     ,    1.71421     ,    1.63767     ,    1.58461     ,    1.54980     ,    1.48668     ,    1.41771     ,    1.35842     ,    1.34045     ,    1.36586     ,    1.33708     ,    1.37833     ,    1.36095     ,    1.32953     ,    1.28795     ,    1.23690     ,    1.20080     ,    1.18752     ,    1.16318     ,    1.14904     ,    1.13238     ,    1.10123     ,    1.07783     ,    1.06156     ,    1.06112     ,    1.06471     ,    1.07815     ,    1.07849     ,    1.08392     ,    1.08620     ,    1.09455     ,    1.10595     ,    1.12164     ,    1.12760     ,    1.13798     ,    1.14374     ,    1.15308     ,    1.15382    },
    {    1.05385     ,    1.04197     ,    1.02597     ,    1.01033     ,    0.99227     ,    0.97125     ,    0.94730     ,    0.92824     ,    0.91343     ,    0.91087     ,    0.90589     ,    0.90159     ,    0.89433     ,    0.88891     ,    0.88807     ,    0.88300     ,    0.88047     ,    0.88587     ,    0.89513     ,    0.91292     ,    0.93399     ,    0.96683     ,    0.99126     ,    1.01483     ,    1.02936     ,    1.04192     ,    1.05654     ,    1.06755     ,    1.09042     ,    1.11084     ,    1.07436     ,    1.04036     ,    0.97061     ,    0.95048     ,    0.93463     ,    0.93035     ,    0.90563     ,    0.88326     ,    0.85265     ,    0.85013     ,    0.85498     ,    0.85213     ,    0.90334     ,    0.91808     ,    0.88260     ,    0.89259     ,    0.92126     ,    0.92399     ,    0.90103     ,    0.90607     ,    0.89391     ,    0.87579     ,    0.85609     ,    0.84953     ,    0.80860     ,    0.83983     ,    0.90406     ,    0.91652     ,    0.99528     ,    1.06459     ,    1.12631     ,    1.18804     ,    1.23338     ,    1.27150     ,    1.29301     ,    1.32793     ,    1.36571     ,    1.40728     ,    1.43985     ,    1.47300     ,    1.52959     ,    1.58652     ,    1.62074     ,    1.65208     ,    1.67680     ,    1.67497     ,    1.67720     ,    1.68064     ,    1.68896     ,    1.69044     ,    1.68257     ,    1.66886     ,    1.65378     ,    1.62904     ,    1.62914     ,    1.62881     ,    1.60537     ,    1.60867     ,    1.60946     ,    1.59287     ,    1.56922     ,    1.54142     ,    1.49938     ,    1.45215     ,    1.40808     ,    1.35975     ,    1.32468     ,    1.32002     ,    1.31192     ,    1.30019     ,    1.29494     ,    1.28557     ,    1.27387     ,    1.26062     ,    1.22900     ,    1.19660     ,    1.17628     ,    1.15004     ,    1.12448     ,    1.10153     ,    1.09159     ,    1.08004     ,    1.06949     ,    1.06477     ,    1.06783     ,    1.07624     ,    1.09320     ,    1.09620     ,    1.09813     ,    1.10006     ,    1.09944     ,    1.09605     ,    1.09753     ,    1.09563     ,    1.09377     ,    1.09197     ,    1.08341     ,    1.07026    },
    {    0.97534     ,    0.96401     ,    0.95240     ,    0.93280     ,    0.92069     ,    0.90273     ,    0.88809     ,    0.87649     ,    0.86822     ,    0.86324     ,    0.85923     ,    0.85628     ,    0.85311     ,    0.84889     ,    0.84654     ,    0.84424     ,    0.84332     ,    0.84608     ,    0.85406     ,    0.86367     ,    0.88328     ,    0.90543     ,    0.93048     ,    0.94756     ,    0.97272     ,    1.00136     ,    1.05391     ,    1.11384     ,    1.15503     ,    1.31368     ,    1.30115     ,    1.21860     ,    1.14917     ,    1.10321     ,    1.07035     ,    1.03554     ,    1.02981     ,    1.02853     ,    1.03380     ,    1.03171     ,    1.04223     ,    1.04211     ,    1.04132     ,    1.03998     ,    1.06432     ,    1.09038     ,    1.11669     ,    1.15110     ,    1.12519     ,    1.10870     ,    1.10110     ,    1.07211     ,    1.06712     ,    1.06522     ,    1.05997     ,    1.05815     ,    1.05140     ,    1.04842     ,    1.04819     ,    1.06001     ,    1.07206     ,    1.08573     ,    1.10125     ,    1.11934     ,    1.13181     ,    1.14831     ,    1.16165     ,    1.17954     ,    1.19250     ,    1.20269     ,    1.21086     ,    1.21665     ,    1.22127     ,    1.22347     ,    1.22525     ,    1.22955     ,    1.23366     ,    1.23545     ,    1.23609     ,    1.23858     ,    1.24527     ,    1.24798     ,    1.24757     ,    1.25407     ,    1.25253     ,    1.25687     ,    1.25225     ,    1.24680     ,    1.24403     ,    1.23618     ,    1.23326     ,    1.22859     ,    1.21260     ,    1.19120     ,    1.17772     ,    1.17250     ,    1.16327     ,    1.14934     ,    1.13417     ,    1.12783     ,    1.12748     ,    1.12220     ,    1.11730     ,    1.09962     ,    1.08204     ,    1.06677     ,    1.05497     ,    1.04295     ,    1.03398     ,    1.02124     ,    1.00899     ,    1.00161     ,    0.99291     ,    0.99366     ,    0.98837     ,    0.98952     ,    0.99398     ,    0.99295     ,    0.99542     ,    0.99759     ,    1.00064     ,    1.00405     ,    1.00005     ,    1.00121     ,    0.99949     ,    0.99645     ,    0.99031     ,    0.98325    },
    {    0.90789     ,    0.90551     ,    0.90365     ,    0.90178     ,    0.89927     ,    0.89580     ,    0.89399     ,    0.89145     ,    0.89059     ,    0.88988     ,    0.88963     ,    0.88996     ,    0.88943     ,    0.89115     ,    0.89184     ,    0.89352     ,    0.89670     ,    0.90012     ,    0.90450     ,    0.91049     ,    0.91551     ,    0.92336     ,    0.93181     ,    0.93658     ,    0.94144     ,    0.94580     ,    0.94966     ,    0.95652     ,    0.97171     ,    1.05610     ,    1.06152     ,    1.06247     ,    1.06226     ,    1.06145     ,    1.05962     ,    1.05377     ,    1.04838     ,    1.04880     ,    1.03881     ,    1.03033     ,    1.02798     ,    1.02335     ,    1.02375     ,    1.02418     ,    1.02407     ,    1.02812     ,    1.02618     ,    1.02419     ,    1.02421     ,    1.02528     ,    1.02935     ,    1.02987     ,    1.03248     ,    1.03562     ,    1.03516     ,    1.03537     ,    1.03762     ,    1.03161     ,    1.03095     ,    1.03434     ,    1.03679     ,    1.04226     ,    1.04694     ,    1.05248     ,    1.05769     ,    1.05993     ,    1.05931     ,    1.05769     ,    1.05515     ,    1.05341     ,    1.04985     ,    1.04630     ,    1.04190     ,    1.03660     ,    1.03274     ,    1.02988     ,    1.02664     ,    1.02241     ,    1.01966     ,    1.01521     ,    1.01405     ,    1.00986     ,    1.00738     ,    1.00545     ,    1.00196     ,    0.99920     ,    0.99735     ,    0.99684     ,    0.99449     ,    0.99279     ,    0.99087     ,    0.98950     ,    0.98599     ,    0.96786     ,    0.96708     ,    0.96596     ,    0.96423     ,    0.96354     ,    0.96211     ,    0.95887     ,    0.95323     ,    0.95108     ,    0.94703     ,    0.94052     ,    0.93554     ,    0.93060     ,    0.92583     ,    0.92167     ,    0.91779     ,    0.91540     ,    0.91171     ,    0.90871     ,    0.90479     ,    0.90377     ,    0.90095     ,    0.89989     ,    0.89782     ,    0.89765     ,    0.89692     ,    0.89690     ,    0.89886     ,    0.90240     ,    0.90665     ,    0.91080     ,    0.91136     ,    0.91091     ,    0.91021     ,    0.9099    }
};
